#include "scriptinterface.h"

#include "Fed_Special1Text.h"
#include "Fed_Special1Maps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
static  int32 AveragePlayerBPV = 100;
//***                   ***

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

static const int32 kDeepScanDistancePlanet = 30;
static const int32 kDeepScanDistanceAnomaly = 30;
static const int32 kDeepScanDistanceShip = 15;

static const int32 kAnomalyLocX = 8;
static const int32 kAnomalyLocY = 12;

/////////////////////
// tMissionState
//

static class tMissionState
{
public:
	int32 fChatCounter;
	int32 fBorderWarning;
	int32 fPlanetScanCount;
	int32 fAnomalyScanCount;
	int32 fEnemyShipScanCount;
	int32 fEnemyDestroyCount;
public:
	tMissionState() { mInit(); }
	void mInit();
}gMissionState;

void tMissionState::mInit()
{
	fChatCounter = 0;
	fBorderWarning = 0;
	fPlanetScanCount = 0;
	fAnomalyScanCount = 0;
	fEnemyShipScanCount = 0;
	fEnemyDestroyCount = 0;
}

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kPlanetTeam,
	kAnomalyTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 6;
	Specs->fMaxDate = 24;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// Pick a random enemy race...	
/*	switch ( fMission.mRandomInt32( 1, 6 ) )
	{
	case 1:
		info.fEnemyRace = ( info.fPlayerRace == kFederation ) ? kKlingon : kFederation;
		break;
	case 2:
		info.fEnemyRace = ( info.fPlayerRace == kKlingon ) ? kRomulan : kKlingon;
		break;
	case 3:
		info.fEnemyRace = ( info.fPlayerRace == kRomulan ) ? kGorn : kRomulan;
		break;
	case 4:
		info.fEnemyRace = ( info.fPlayerRace == kGorn ) ? kHydran : kGorn;
		break;
	case 5:
		info.fEnemyRace = ( info.fPlayerRace == kHydran ) ? kLyran : kHydran;
		break;
	case 6:
		info.fEnemyRace = ( info.fPlayerRace == kLyran ) ? kFederation : kLyran;
		break;
	}
*/	

	fMission.mSetPrestigePoints (150, 120, 80, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);
	fMission.mSetTeamRace (kAnomalyTeam, kNeutralRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAnomalyTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAnomalyTeam, kEnemyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // use the player's ship(s)
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create the planets
	PlanetTeam->mCreateShip("PL3", kStartPosition_Y );
	PlanetTeam->mCreateShip("PL3", kStartPosition_Z );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	gMissionState.mInit();

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tShipInfo* Ship;
	
	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// configure Organian dead worlds 
	PlanetTeam->mGetFirstShip()->mChangeName( kOrganianPlanet1 );
	PlanetTeam->mGetNextShip()->mChangeName( kOrganianPlanet2 );

	fMission.mDisplayMessage( kCaptain_msg, kMissionScriptMessage );

	// timer to set initial team relations
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );

	// set the on screen goal indicator
	// LNF should I keep this in here?
	//fMission.mSetGoalIndicator( LpTeam->mGetFirstShip() );

	// timer to check win conditions
	fMission.mSetTimedInterval( kTimerLimitSet_2, 5 );

	// timer to for random chatter --> 'Cut the chatter red 2'
	fMission.mSetTimedInterval( kTimerLimitSet_6, 20 );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan ListeningPost from within kDeepScanDistance for 1 turn
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, 2);
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(2), kDeepScanDistancePlanet, 2);
	fMission.mSetGoalIndicator( PlanetTeam->mGetThisShip(1) );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if ( Ship->mGetTeam() == kEnemyTeam )
	{
		DeepScanTracker.mSetTarget ( Ship, kDeepScanDistanceShip, 1 );
		Ship->mSetCommButton( 1, kCommButtonHail_msg );
	}
}

//---------------------------------------------------------------------------------------------
// Spawn alien ships
static void spawnAlienShips( tMissionInfo& mission )
{
	const int32 kMaxShips = 6;
	tTeamInfo* EnemyTeam = mission.mGetTeamHandle (kEnemyTeam);

	int32 count = EnemyTeam->mGetShipsInPlayCount();

	if ( count >= kMaxShips )
	{
		return;
	}

	int32 j = 0;
	while ( count++ < kMaxShips && j < 4 )
	{
		static const int32 kPositions[4] = { kStartPosition_K, kStartPosition_L, kStartPosition_M, kStartPosition_N };
		static int32 i = 0;
		EnemyTeam->mCreateShip( AveragePlayerBPV, ( ( LargestEnemyShipClass > kClassFrigate ) ? 
			(eClassTypes) ( LargestEnemyShipClass - 1 ) : LargestEnemyShipClass ), LargestEnemyShipClass, 0, kPositions[i] );
		i = ( i + 1 ) & 3;
		j++;
	}

	mission.mDisplayMessage( kScience11_msg, kScienceScriptMessage );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);


	switch (Event)
	{
		case kNotifyAtDestination:
			break;
				
		case kNotifyDamage:
			{
			}
			break;
	
		case kNotifyOnCapture:
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				gMissionState.fEnemyDestroyCount++;
			}
			break;

		case kNotifyDisengaged:
			fMission.mDisplayMessage( kScience13_msg, kScienceScriptMessage );
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

// Timers...
		case kNotifyAtTimedInterval:
			switch ( fEventID )
			{
			case kTimerLimitSet_3:
				{
					// test if the player is near the edge of the map
					int32 ShipLocX = 0;
					int32 ShipLocY = 0;
					int32 MapWidth = 0;
					int32 MapLength = 0;

					const int32 kClose = 1;
					fMission.mGetMapDimensions( &MapWidth, &MapLength );
					for (tShipInfo* pShip = PlayerTeam->mGetFirstShip(); pShip != NULL; pShip = PlayerTeam->mGetNextShip())
					{
						pShip->mGetGridPosition( &ShipLocX, &ShipLocY );

						if (( ShipLocX <= kClose )
							|| ( ShipLocX >= MapWidth - kClose )
							|| ( ShipLocY <= kClose ) 
							|| ( ShipLocY >= MapLength - kClose ))
						{
							if ( !gMissionState.fBorderWarning )
							{
								fMission.mDisplayMessage( kScience12_msg, kScienceScriptMessage );
								gMissionState.fBorderWarning = 10;
							}
							else
							{
								gMissionState.fBorderWarning--;
							}
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
					}
				}
				break;
			case kTimerLimitSet_4:
				// We are being scanned by the energy field...
				fMission.mDisplayMessage( kScience10_msg, kScienceScriptMessage );
				fMission.mSetTimedInterval( kTimerLimitSet_5, 40 );
				break;
			case kTimerLimitSet_5:
				// Spawn enemy vessels
				spawnAlienShips( fMission );
				fMission.mSetTimedInterval( kTimerLimitSet_5, 40 );
				break;
			case kTimerLimitSet_6:
				{
					// Chatter
					if ( gMissionState.fChatCounter + kHelm1_msg <= kNavigator2_msg )
					{
						if ( !gMissionState.fPlanetScanCount )
						{
							fMission.mDisplayMessage( gMissionState.fChatCounter++ + kHelm1_msg, kScienceScriptMessage );
							fMission.mSetTimedInterval( kTimerLimitSet_6, 20 );
						}
					}
					break;
				}
			case kTimerLimitSet_10:
				{
					eDeepScanFlags flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					switch (flag)
					{
						case kOutOfRange_ScanOff:
							// player is not within any objects range and deep scan is off (Object == NULL)
							break;
						case kOutOfRange_ScanOn:
							// player is not within any objects range and deep scan is on (Object == NULL)
							break;
						case kEnteringRange_ScanOff:
							// player has just entered range and scan is off (Object points to entity)
							// notify player to turn scan on
							if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
							{
								fMission.mDisplayMessage (kPlanetEnteringRange_ScanOff_msg, kScienceScriptMessage);
							}
							else if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
							{
								fMission.mDisplayMessage (kEnemyEnteringRange_ScanOff_msg, kScienceScriptMessage);
							}
							else if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOff_msg, kScienceScriptMessage);
							}
							
							break;
						case kEnteringRange_ScanOn_TimerStarted:
							// player has just entered range and scan is on (Object points to entity)
							// notify player that timer has begun
							if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
							{
								fMission.mDisplayMessage (kPlanetEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							else if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
							{
								fMission.mDisplayMessage (kEnemyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							else if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							break;
						case kInRange_ScanOff_TimerStopped:
							// player is in range and scan is off (Object points to entity)
							// notify player that timer has been turned off
							fMission.mDisplayMessage (kInRange_ScanOff_TimerStopped_msg, kScienceScriptMessage);
							break;
						case kInRange_ScanOff:
							// player is still in range and scan is still off (Object points to entity)
							break;
						case kInRange_ScanOn_TimerStarted:
							// player is in range and scan is on (Object points to entity)
							// notify player that timer has begun
							if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
							{
								fMission.mDisplayMessage (kPlanetEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							else if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
							{
								fMission.mDisplayMessage (kEnemyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							else if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							break;
						case kInRange_ScanOn:
							// player is still in range and scan is still on (Object points to entity)
							break;
						case kLeavingRange_ScanOn_TimerStopped:
							// player has just left the range and scan is on (Object points to entity)
							// notify player that timer is now off
							fMission.mDisplayMessage (kLeavingRange_ScanOn_TimerStopped_msg, kScienceScriptMessage);
							break;
						case kLeavingRange_ScanOff:
							// player has just left the range and scan is off (Object points to entity)
							break;
					}
				}
				break;

			case kTimerCountdownSet:
				{
					eDeepScanFlags flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					if (flag == kInRange_ScanOn)
					{
						// player has successfully scanned the object
						if ( ( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ) )
						{
							if ( !gMissionState.fPlanetScanCount++ )
							{
								fMission.mDisplayMessage( kScience2_msg, kScienceScriptMessage );
								fMission.mDisplayMessage( kScience5_msg, kScienceScriptMessage );
								if ( Object == PlanetTeam->mGetThisShip(2) )
								{
									fMission.mSetGoalIndicator( PlanetTeam->mGetThisShip(1) );
								}
								else
								{
									fMission.mSetGoalIndicator( PlanetTeam->mGetThisShip(2) );
								}
							}
							else
							{
								fMission.mDisplayMessage( kScience3_msg, kScienceScriptMessage );
								fMission.mDisplayMessage( kCaptain2_msg, kScienceScriptMessage );
								fMission.mDisplayMessage( kScience4_msg, kScienceScriptMessage );
								// Create the strange energy field using the space fissure...
								fMission.mCreateMapObject( 'e', kAnomalyLocX, kAnomalyLocY );
								fMission.mSetGoalIndicator( kTrue, kAnomalyLocX, kAnomalyLocY );
								DeepScanTracker.mSetLocation( kAnomalyLocX, kAnomalyLocY, kDeepScanDistanceAnomaly, 1 );
								// Set a timer to allow the anomaly to start scanning in 20 seconds
								fMission.mSetTimedInterval( kTimerLimitSet_4, 20 );

							}
						}
						else if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
						{
							gMissionState.fEnemyShipScanCount++;
							fMission.mDisplayMessage( kScience9_msg, kScienceScriptMessage );
						}
						else if ( Object == NULL )
						{
							gMissionState.fAnomalyScanCount++;
							fMission.mSetGoalIndicator( kFalse, kAnomalyLocX, kAnomalyLocY );
							fMission.mDisplayMessage( kScience8_msg, kScienceScriptMessage );
							fMission.mDisplayMessage( kSecurity1_msg, kScienceScriptMessage );
							DeepScanTracker.mRemoveLocation( MapPoint.x, MapPoint.y );
							// timer to check if reaching the edge of space
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
						if ( Object )
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
				}
				break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kCommButtonHail_msg:
					// if hailing the Enemy team
					if (Ship->mGetTeam() == kEnemyTeam)
					{
						fMission.mDisplayMessage (fMission.mRandomInt32 (kEnemy1_msg, kEnemy4_msg), kMissionScriptMessage);
						Ship->mSetCommButton (1, kCommButtonHail_msg);
					}
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	// LNF: Add Dynaverse rating adjustments
	if( gMissionState.fPlanetScanCount > 1 && gMissionState.fAnomalyScanCount && gMissionState.fEnemyShipScanCount &&
		gMissionState.fEnemyDestroyCount )
	{	// Excellent work Commander!
		return kAstoundingVictory;
	}
	else if ( gMissionState.fPlanetScanCount > 1 && gMissionState.fAnomalyScanCount )
	{	// Good work Commander!
		return kVictory;
	}
	else if ( gMissionState.fPlanetScanCount > 1 )
	{	// You need to put a little more effort into these missions Commander
		return kDraw;
	}
	else if ( gMissionState.fPlanetScanCount )
	{	// You just suck Commander!
		return kDefeat;
	}
	else 
	{	// Just what did you do to convince us to put you in command???
		return kDevastatingDefeat;
	}

	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	switch ( VictoryLevel )
	{
	case kAstoundingVictory:
		fMission.mAddToDebriefingList( kAstoundingVictory_msg );
		break;
	case kVictory:
		fMission.mAddToDebriefingList( kVictory_msg );
		break;
	case kDraw:
		fMission.mAddToDebriefingList( kDraw_msg );
		break;
	case kDefeat:
		fMission.mAddToDebriefingList( kDefeat_msg );
		break;
	case kDevastatingDefeat:
		fMission.mAddToDebriefingList( kDevastatingDefeat_msg );
		break;
	default:
		fMission.mAddToDebriefingList( kDevastatingDefeat_msg );
		break;
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		// this determines the near maximum number of ships a player
		// should have to face if he were fighting frigates
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassWarDestroyer)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	// just adding some randomness into the final number of ships
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(-1, 1);
	}
	else
	{
		//NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}

