#include "scriptinterface.h"

#include "Fed_Special2Text.h"
#include "Fed_Special2Maps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
static  int32 AveragePlayerBPV = 100;
//***                   ***

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

static const int32 kDeepScanDistancePlanet = 30;
static const int32 kDeepScanDistanceAnomaly = 30;
static const int32 kDeepScanDistanceShip = 15;
static const int32 PatrolAttackRadius = 15.0f;
static const int32 kAnomalyLocX = 16;
static const int32 kAnomalyLocY = 11;
static const int32 kEnemySpawnTime = 40;

static void debugMessage( tMissionInfo& mission, const char *string, ... )
{
    static char msg[512];
    va_list v;

    va_start(v, string);
    vsprintf(msg, string, v);
    va_end(v);

	Messages[kDebug_msg] = msg;
	mission.mDisplayMessage( kDebug_msg, kScienceScriptMessage );
}

/////////////////////
// tMissionState
//

static class tMissionState
{
public:
	class tShipState
	{
	public:
		iTruth fLoaded;
		iTruth fEscaped;
		iTruth fDestroyed;	// may not be needed
		tShipInfo* fShip;
		tShipState() : fLoaded(kFalse), fEscaped(kFalse), fDestroyed(kFalse), fShip(NULL) {}
	};
	enum { kNumFreighters = 5 };

	int32 fChatCounter;
	int32 fEnemyShipCount;
	iTruth fShipsEntered;
	int32 fBorderWarning;
	int32 fAnomalyScanCount;
	tShipState fFreighters[kNumFreighters];
	tShipState fSoulShip;
public:
	tMissionState() { mInit(); }
	void mInit();
	int32 mGetNumEscapedFreighters() const;
	int32 mGetNumDestroyedFreighters() const;
	int32 mGetNumLoadedFreighters() const;
	int32 mGetFreighter( tShipInfo* pShip ) const;
}gMissionState;

void tMissionState::mInit()
{
	fShipsEntered = kFalse;
	fBorderWarning = 0;
	fAnomalyScanCount = 0;
	fEnemyShipCount = 0;
}

int32 tMissionState::mGetNumEscapedFreighters() const
{
	int32 count = 0;
	for ( int32 i = 0; i < tMissionState::kNumFreighters; i++ )
	{
		if ( fFreighters[i].fEscaped )
		{
			count++;
		}
	}

	return count;
}

int32 tMissionState::mGetNumDestroyedFreighters() const
{
	int32 count = 0;
	for ( int32 i = 0; i < tMissionState::kNumFreighters; i++ )
	{
		if ( fFreighters[i].fDestroyed )
		{
			count++;
		}
	}

	return count;
}

int32 tMissionState::mGetNumLoadedFreighters() const
{
	int32 count = 0;
	for ( int32 i = 0; i < tMissionState::kNumFreighters; i++ )
	{
		if ( fFreighters[i].fLoaded )
		{
			count++;
		}
	}

	return count;
}

int32 tMissionState::mGetFreighter( tShipInfo* pShip ) const
{
	for ( int32 i = 0; i < tMissionState::kNumFreighters && fFreighters[i].fShip != pShip; i++ )
	{
	}

	if ( i >= tMissionState::kNumFreighters )
	{
		return -1;
	}
	else
	{
		return i;
	}
}

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kPlanetTeam,
	kAnomalyTeam,
	kAlliedTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// Pick a random enemy race...	
/*	switch ( fMission.mRandomInt32( 1, 6 ) )
	{
	case 1:
		info.fEnemyRace = ( info.fPlayerRace == kFederation ) ? kKlingon : kFederation;
		break;
	case 2:
		info.fEnemyRace = ( info.fPlayerRace == kKlingon ) ? kRomulan : kKlingon;
		break;
	case 3:
		info.fEnemyRace = ( info.fPlayerRace == kRomulan ) ? kGorn : kRomulan;
		break;
	case 4:
		info.fEnemyRace = ( info.fPlayerRace == kGorn ) ? kHydran : kGorn;
		break;
	case 5:
		info.fEnemyRace = ( info.fPlayerRace == kHydran ) ? kLyran : kHydran;
		break;
	case 6:
		info.fEnemyRace = ( info.fPlayerRace == kLyran ) ? kFederation : kLyran;
		break;
	}
*/	
	info.fEnemyRace = kKlingon;

	fMission.mSetPrestigePoints (150, 120, 80, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);
	fMission.mSetTeamRace (kAnomalyTeam, kNeutralRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kAlliedTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kAlliedTeam, kPlanetTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlanetTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlanetTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kEnemyTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // use the player's ship(s)
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.250f;

				// set LargestEnemyShipClass to two SMALLER than the player
				if ( PlayerShipClass > kClassFrigate + 2 )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 3);
				}
				else
				{
					LargestEnemyShipClass = kClassFrigate;
				}
			}
			break;
		case 1:
			{
				TotalBPV *= 0.50f;
				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate + 1 )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 2);
				}
				else
				{
					LargestEnemyShipClass = kClassFrigate;
				}
			}
			break;
		case 2:
			{
				TotalBPV *= 0.75f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
				else
				{
					LargestEnemyShipClass = kClassFrigate;
				}
			}
			break;
	}

	AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create the planets and ships
	AlliedTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 100, kStartPosition_Y );	// Soul Ship
	// Freighters...
	AlliedTeam->mCreateFleet (tMissionState::kNumFreighters, AveragePlayerBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_Z | kGroupAround );
	PlanetTeam->mCreateShip("PL1", kStartPosition_N );	// Skorr homeworld
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	gMissionState.mInit();

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tShipInfo* Ship;
	
	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// configure allied ship(s)
	int32 planetX = 0, planetY = 0;
	PlanetTeam->mGetFirstShip()->mGetGridPosition( &planetX, &planetY );

	// configure allied ship(s)
	int32 freighterCount = 0;
	for ( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
	{		
		if ( Ship->mGetClassType() == kClassFreighter )
		{
			gMissionState.fFreighters[freighterCount].fShip = Ship;
			Ship->mChangeName( kFreighterName1_msg + freighterCount++ );
		}
		else // must be the Soul ship
		{
			Ship->mChangeName( kSidorovName_msg );
			gMissionState.fSoulShip.fShip = Ship;
		}
		Ship->mSetFinalDestination( planetX + 3, planetY - 3 );
	}
	// Set ship max speeds
	for (Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip())
	{
		Ship->mSetShipMaxSpeed(20);
	}

	// configure Skorr world
	PlanetTeam->mGetFirstShip()->mChangeName( kPlanetName_msg );

	fMission.mDisplayMessage( kCaptainsLog_msg, kMissionScriptMessage );

	// timer to set initial team relations
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );

	// set the on screen goal indicator
	// LNF should I keep this in here?
	//fMission.mSetGoalIndicator( LpTeam->mGetFirstShip() );

	// timer to check win conditions
	fMission.mSetTimedInterval( kTimerLimitSet_2, 5 );

	// timer to for random chatter --> 'Cut the chatter red 2'
	fMission.mSetTimedInterval( kTimerLimitSet_6, 20 );

	// timer to spawn enemy ships
	fMission.mSetTimedInterval( kTimerLimitSet_5, kEnemySpawnTime );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan ListeningPost from within kDeepScanDistance for 1 turn
	DeepScanTracker.mSetLocation( kAnomalyLocX, kAnomalyLocY, kDeepScanDistanceAnomaly, 1 );
}

static const int32 kNumPoints = 8;
static tMapPoint	kPatrolPoints[kNumPoints] = 
	{ { 19, 21 }, { 16, 19 }, { 6, 14 }, { 6, 11 }, { 16, 6 }, { 19, 6 }, { 24, 11 }, { 29, 14 } };

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if ( Ship->mGetTeam() == kEnemyTeam )
	{
//		Ship->mPatrol( PatrolAttackRadius, kNumPoints, &kPatrolPoints[0], kMediumPriority );
		Ship->mSetCommButton( 1, kCommButtonHail_msg );
	}
}

//---------------------------------------------------------------------------------------------
// Spawn alien ships
static void spawnShips( tMissionInfo& mission )
{
	const int32 kMaxShipsInPlay = 4;
	const int32 kMaxShips = 7;
	tTeamInfo* EnemyTeam = mission.mGetTeamHandle (kEnemyTeam);

	int32 count = EnemyTeam->mGetShipsInPlayCount();

	if ( count >= kMaxShipsInPlay || gMissionState.fEnemyShipCount >= kMaxShips )
	{
		return;
	}

	static int32 i = 0;
	EnemyTeam->mCreateShip( AveragePlayerBPV, ( ( LargestEnemyShipClass == kClassFrigate ) ? 
		LargestEnemyShipClass : (eClassTypes) ( LargestEnemyShipClass - 1 ) ), LargestEnemyShipClass, 0, 
		kStartPosition_Specified, kPatrolPoints[i].x, kPatrolPoints[i].y );
	gMissionState.fEnemyShipCount++;
	if ( ++i > kNumPoints )
	{
		i = 0;
	}
}

static void shipReachedPlanet( tMissionInfo& mission, tShipInfo* Ship )
{
	if ( Ship->mGetTeam() != kAlliedTeam )
	{
		return;
	}

	Ship->mSetShipThrust(0);
	Ship->mDisengage( kByMapEdge, kAbsolutePriority );
	if ( Ship->mGetClassType() == kClassFreighter )
	{
		mission.mDisplayMessage( kFreighterReachPlanet_msg, kScienceScriptMessage, Ship );
		const int32 kFreighter = gMissionState.mGetFreighter( Ship );
		if ( kFreighter >= 0 )
		{
			gMissionState.fFreighters[kFreighter].fLoaded = kTrue;
		}
	}
	else 
	{
		mission.mDisplayMessage( kSidorovReachesPlanet_msg, kScienceScriptMessage );
		gMissionState.fSoulShip.fLoaded = kTrue;
	}
	Ship->mSetShipMaxSpeed(31);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);


	switch (Event)
	{
		case kNotifyAtDestination:
			shipReachedPlanet( fMission, Ship );
			break;
				
		case kNotifyDamage:
			{
				if ( Ship->mGetTeam() == kAlliedTeam && Ship->mGetClassType() == kClassLightCruiser )
				{
					float shipDamage = Ship->mGetDamagePercent();
					if ( shipDamage <= 0.5f )
					{
						if ( gMissionState.fSoulShip.fLoaded )
						{
							fMission.mDisplayMessage( kSidorovDistressSoul_msg, kScienceScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( kSidorovDistress_msg, kScienceScriptMessage );
						}
					}
					else if ( shipDamage <= 0.2f )
					{
						if ( gMissionState.fSoulShip.fLoaded )
						{
							fMission.mDisplayMessage( kSidorovDeepDistressSoul_msg, kScienceScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( kSidorovDeepDistress_msg, kScienceScriptMessage );
						}
					}
				}
			}
			break;
	
		case kNotifyOnCapture:
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetClassType() == kClassFreighter && Ship->mGetTeam() == kAlliedTeam )
			{
				fMission.mDisplayMessage( kFreighterDestroyed_msg, kScienceScriptMessage, Ship );
				const int32 kFreighter = gMissionState.mGetFreighter( Ship );
				if ( kFreighter >= 0 )
				{
					gMissionState.fFreighters[kFreighter].fDestroyed = kTrue;
				}
			}
			else if ( Ship->mGetTeam() == kAlliedTeam )
			{
				fMission.mDisplayMessage( kSidorovDestroyed_msg, kScienceScriptMessage );
				gMissionState.fSoulShip.fDestroyed = kTrue;
				if ( gMissionState.fSoulShip.fLoaded )
				{
					fMission.mDisplayMessage( kSidorovDestroyedSoul_msg, kScienceScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kSidorovDestroyedNoSoul_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyDisengaged:
			{
				// Check the bad guys and make sure they are not targeting this ship anymore
				fMission.mRemoveShipAsTarget(Ship);

				if ( Ship->mGetClassType() == kClassFreighter && Ship->mGetTeam() == kAlliedTeam )
				{
					fMission.mDisplayMessage( kFreighterEscapes_msg, kScienceScriptMessage, Ship );
					const int32 kFreighter = gMissionState.mGetFreighter( Ship );
					if ( kFreighter >= 0 )
					{
						gMissionState.fFreighters[kFreighter].fEscaped = kTrue;
					}
				}
				else if ( Ship->mGetTeam() == kAlliedTeam )
				{
					fMission.mDisplayMessage( kSidorovEscapes_msg, kScienceScriptMessage );
					gMissionState.fSoulShip.fEscaped = kTrue;
				}
				else if ( Ship->mGetTeam() == kEnemyTeam )
				{
					debugMessage( fMission, "Klingon ship leaving the system!\n" );
				}
				else
				{
					fMission.mDisplayMessage( kDisengaged_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

// Timers...
		case kNotifyAtTimedInterval:
			switch ( fEventID )
			{
			case kTimerLimitSet_3:
				{
					// test if the player is near the edge of the map
					int32 ShipLocX = 0;
					int32 ShipLocY = 0;
					int32 MapWidth = 0;
					int32 MapLength = 0;

					const int32 kClose = 1;
					fMission.mGetMapDimensions( &MapWidth, &MapLength );
					for (tShipInfo* pShip = PlayerTeam->mGetFirstShip(); pShip != NULL; pShip = PlayerTeam->mGetNextShip())
					{
						pShip->mGetGridPosition( &ShipLocX, &ShipLocY );

						if (( ShipLocX <= kClose )
							|| ( ShipLocX >= MapWidth - kClose )
							|| ( ShipLocY <= kClose ) 
							|| ( ShipLocY >= MapLength - kClose ))
						{
							if ( !gMissionState.fBorderWarning )
							{
								fMission.mDisplayMessage( kApproachingEdge_msg, kScienceScriptMessage );
								gMissionState.fBorderWarning = 10;
							}
							else
							{
								gMissionState.fBorderWarning--;
							}
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
					}
				}
				break;
			case kTimerLimitSet_4:
				break;
			case kTimerLimitSet_5:
				// Spawn enemy vessels
				spawnShips( fMission );
				spawnShips( fMission );
				fMission.mSetTimedInterval( kTimerLimitSet_5, kEnemySpawnTime );
				break;
			case kTimerLimitSet_6:
				{
					// Chatter
					if ( gMissionState.fChatCounter + kChatter1_msg <= kChatter6_msg )
					{
						const int32 chat = gMissionState.fChatCounter++ + kChatter1_msg;
						if ( gMissionState.fShipsEntered && chat == kChatter6_msg )
						{
							fMission.mDisplayMessage( chat+1, kScienceScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( chat, kScienceScriptMessage );
						}
						fMission.mSetTimedInterval( kTimerLimitSet_6, 20 );
					}
					break;
				}
			case kTimerLimitSet_10:
				{
					eDeepScanFlags flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					switch (flag)
					{
						case kOutOfRange_ScanOff:
							// player is not within any objects range and deep scan is off (Object == NULL)
							break;
						case kOutOfRange_ScanOn:
							// player is not within any objects range and deep scan is on (Object == NULL)
							break;
						case kEnteringRange_ScanOff:
							// player has just entered range and scan is off (Object points to entity)
							// notify player to turn scan on
							if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOff_msg, kScienceScriptMessage);
							}
							
							break;
						case kEnteringRange_ScanOn_TimerStarted:
							// player has just entered range and scan is on (Object points to entity)
							// notify player that timer has begun
							if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							break;
						case kInRange_ScanOff_TimerStopped:
							// player is in range and scan is off (Object points to entity)
							// notify player that timer has been turned off
							fMission.mDisplayMessage (kInRange_ScanOff_TimerStopped_msg, kScienceScriptMessage);
							break;
						case kInRange_ScanOff:
							// player is still in range and scan is still off (Object points to entity)
							break;
						case kInRange_ScanOn_TimerStarted:
							// player is in range and scan is on (Object points to entity)
							// notify player that timer has begun
							if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							break;
						case kInRange_ScanOn:
							// player is still in range and scan is still on (Object points to entity)
							break;
						case kLeavingRange_ScanOn_TimerStopped:
							// player has just left the range and scan is on (Object points to entity)
							// notify player that timer is now off
							fMission.mDisplayMessage (kLeavingRange_ScanOn_TimerStopped_msg, kScienceScriptMessage);
							break;
						case kLeavingRange_ScanOff:
							// player has just left the range and scan is off (Object points to entity)
							break;
					}
				}
				break;

			case kTimerCountdownSet:
				{
					eDeepScanFlags flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					if (flag == kInRange_ScanOn)
					{
						// player has successfully scanned the object
						if ( Object == NULL )
						{
							gMissionState.fAnomalyScanCount++;
							fMission.mDisplayMessage( kAnomalyScanned_msg, kScienceScriptMessage );
							DeepScanTracker.mRemoveLocation( MapPoint.x, MapPoint.y );
							// timer to check if reaching the edge of space
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
						if ( Object )
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
				}
				break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kCommButtonHail_msg:
					// if hailing the Enemy team
					if (Ship->mGetTeam() == kEnemyTeam)
					{
						fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonHail1_msg, kKlingonHail4_msg), kMissionScriptMessage);
						Ship->mSetCommButton (1, kCommButtonHail_msg);
					}
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	const int32 kEscapedFreighters = gMissionState.mGetNumEscapedFreighters();
	const int32 kDestroyedFreighters = gMissionState.mGetNumDestroyedFreighters();
	const int32 kLoadedFreighters = gMissionState.mGetNumLoadedFreighters();

	// LNF: Add Dynaverse rating adjustments
	if( gMissionState.fSoulShip.fEscaped && kEscapedFreighters == tMissionState::kNumFreighters )
	{	// Excellent work Commander!
		return kAstoundingVictory;
	}
	else if ( gMissionState.fSoulShip.fEscaped && kEscapedFreighters )
	{	// Good work Commander!
		return kVictory;
	}
	else if ( gMissionState.fSoulShip.fEscaped )
	{	// You need to put a little more effort into these missions Commander
		return kDraw;
	}
	else if ( gMissionState.fSoulShip.fLoaded || kLoadedFreighters && gMissionState.fAnomalyScanCount )
	{	// You just suck Commander!
		return kDefeat;
	}
	else 
	{	// Just what did you do to convince us to put you in command???
		return kDevastatingDefeat;
	}

	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	switch ( VictoryLevel )
	{
	case kAstoundingVictory:
		fMission.mAddToDebriefingList( kAstoundingVictory_msg );
		break;
	case kVictory:
		fMission.mAddToDebriefingList( kVictory_msg );
		break;
	case kDraw:
		fMission.mAddToDebriefingList( kDraw_msg );
		break;
	case kDefeat:
		fMission.mAddToDebriefingList( kDefeat_msg );
		break;
	case kDevastatingDefeat:
		fMission.mAddToDebriefingList( kDevastatingDefeat_msg );
		break;
	default:
		fMission.mAddToDebriefingList( kDevastatingDefeat_msg );
		break;
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		// this determines the near maximum number of ships a player
		// should have to face if he were fighting frigates
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassWarDestroyer)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	// just adding some randomness into the final number of ships
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(-1, 1);
	}
	else
	{
		//NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}

