#include "scriptinterface.h"

#include "Fed_Special3Text.h"
#include "Fed_Special3Maps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
static  int32 AveragePlayerBPV = 100;
//***                   ***

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

static const int32 kDeepScanDistancePlanet = 30;
static const int32 kDeepScanDistanceAnomaly = 30;
static const int32 kDeepScanDistanceShip = 15;
static const int32 PatrolAttackRadius = 15.0f;
static const int32 kAnomalyLocX = 9;
static const int32 kAnomalyLocY = 14;
static const int32 kBlackHoleLocX = 24;
static const int32 kBlackHoleLocY = 5;
static const int32 kMonsterLocX = 16;
static const int32 kMonsterLocY = 16;
static const int32 kEnemySpawnTime = 40;
static const int32 kMonsterSpawnTime = 80;

// Monster stuff
enum eMonsterType
{
	kSpaceshell = 0,
	kMeater,
	kSunglider,
	kAstrominer,
	kPlanet,
};

enum eDiffLevel
{
	kDiffEasy = 0,
	kDiffMedium,
	kDiffHard,
};

const tString MonsterName[5][3] =
{
	{ "M-SSS", "M-SSM", "M-SSL" },
	{ "M-MTS", "M-MTM", "M-MTL" },
	{ "M-SGS", "M-SGM", "M-SGL" },
	{ "M-AMS", "M-AMM", "M-AML" },
	{ "PL1", "PL3", "PL0" },
};

template <class Type>
static Type Sqr( Type a )
{
	return a * a;
}

static void debugMessage( tMissionInfo& mission, const char *string, ... )
{
    static char msg[512];
    va_list v;

    va_start(v, string);
    vsprintf(msg, string, v);
    va_end(v);

	Messages[kDebug_msg] = msg;
	mission.mDisplayMessage( kDebug_msg, kScienceScriptMessage );
}

/////////////////////
// tMissionState
//

static class tMissionState
{
public:
	iTruth fAnomalyDestroyed;
	iTruth fFreighterLoaded;
	int32 fChatCounter;
	int32 fEnemyShipCount;
	int32 fMonsterCount;
	iTruth fShipsEntered;
	int32 fBorderWarning;
	int32 fAnomalyScanCount;
public:
	tMissionState() { mInit(); }
	void mInit();
}gMissionState;

void tMissionState::mInit()
{
	fAnomalyDestroyed = kFalse;
	fFreighterLoaded = kFalse;
	fShipsEntered = kFalse;
	fBorderWarning = 0;
	fAnomalyScanCount = 0;
	fEnemyShipCount = 0;
	fMonsterCount = 0;
}


//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kAnomalyTeam,
	kAlliedTeam,
	kMonsterTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// Pick a random enemy race...	
/*	switch ( fMission.mRandomInt32( 1, 6 ) )
	{
	case 1:
		info.fEnemyRace = ( info.fPlayerRace == kFederation ) ? kKlingon : kFederation;
		break;
	case 2:
		info.fEnemyRace = ( info.fPlayerRace == kKlingon ) ? kRomulan : kKlingon;
		break;
	case 3:
		info.fEnemyRace = ( info.fPlayerRace == kRomulan ) ? kGorn : kRomulan;
		break;
	case 4:
		info.fEnemyRace = ( info.fPlayerRace == kGorn ) ? kHydran : kGorn;
		break;
	case 5:
		info.fEnemyRace = ( info.fPlayerRace == kHydran ) ? kLyran : kHydran;
		break;
	case 6:
		info.fEnemyRace = ( info.fPlayerRace == kLyran ) ? kFederation : kLyran;
		break;
	}
*/	
	info.fEnemyRace = kKlingon;

	fMission.mSetPrestigePoints (200, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kAnomalyTeam, kNeutralRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kAlliedTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMonsterTeam, kMonster );

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kAlliedTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedTeam, kMonsterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kEnemyTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kMonsterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kAnomalyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // use the player's ship(s)
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.250f;

				// set LargestEnemyShipClass to two SMALLER than the player
				if ( PlayerShipClass > kClassFrigate + 2 )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 3);
				}
				else
				{
					LargestEnemyShipClass = kClassFrigate;
				}
			}
			break;
		case 1:
			{
				TotalBPV *= 0.50f;
				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate + 1 )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 2);
				}
				else
				{
					LargestEnemyShipClass = kClassFrigate;
				}
			}
			break;
		case 2:
			{
				TotalBPV *= 0.75f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
				else
				{
					LargestEnemyShipClass = kClassFrigate;
				}
			}
			break;
	}

	AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create the planets and ships
	AlliedTeam->mCreateShip( kMaxBPV, kClassFreighter, 100, kStartPosition_J );	// Contempt freighter
	EnemyTeam->mCreateShip( AveragePlayerBPV, ( ( LargestEnemyShipClass == kClassFrigate ) ? 
		LargestEnemyShipClass : (eClassTypes) ( LargestEnemyShipClass - 1 ) ), LargestEnemyShipClass, 0, 
		kStartPosition_K );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	gMissionState.mInit();

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;
	
	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// configure allied ship(s)
	// Set ship max speeds
	for (Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip())
	{
		if ( Ship->mGetClassType() == kClassFreighter )
		{
			Ship->mChangeName( kFreighterName_msg );
		}
		Ship->mSetShipMaxSpeed(20);
		Ship->mSetFinalDestination( kAnomalyLocX, kAnomalyLocY );
		Ship->mSetCommButton( 1, kCommButtonHail_msg );
	}

	// Configure bad guys ships
	for ( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kCommButtonHail_msg );
	}

	fMission.mDisplayMessage( kCaptainsLog_msg, kMissionScriptMessage );
	fMission.mDisplayMessage( kWeapon1_msg, kScienceScriptMessage );
	fMission.mSetTimedInterval( kTimerLimitSet_7, 5 );

	// timer to set initial team relations
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );

	// set the on screen goal indicator
	// LNF should I keep this in here?
	//fMission.mSetGoalIndicator( LpTeam->mGetFirstShip() );

	// timer to check win conditions
	fMission.mSetTimedInterval( kTimerLimitSet_2, 5 );

	// timer to for random chatter --> 'Cut the chatter red 2'
//	fMission.mSetTimedInterval( kTimerLimitSet_6, 20 );

	// timer to spawn enemy monsters
	fMission.mSetTimedInterval( kTimerLimitSet_4, kMonsterSpawnTime );
	// timer to spawn enemy ships
	fMission.mSetTimedInterval( kTimerLimitSet_5, kEnemySpawnTime );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan ListeningPost from within kDeepScanDistance for 1 turn
	DeepScanTracker.mSetLocation( kAnomalyLocX, kAnomalyLocY, kDeepScanDistanceAnomaly, 1 );
}

static const int32 kNumPoints = 6;
static tMapPoint	kPatrolPoints[kNumPoints] = 
	{ { 2, 17 }, { 4, 8 }, { 11, 4 }, { 18, 2 }, { 27, 19 }, { 22, 14 } };

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if ( Ship->mGetTeam() == kEnemyTeam || Ship->mGetTeam() == kMonsterTeam || Ship->mGetTeam() == kAlliedTeam )
	{
//		Ship->mPatrol( PatrolAttackRadius, kNumPoints, &kPatrolPoints[0], kMediumPriority );
		Ship->mSetCommButton( 1, kCommButtonHail_msg );
	}
}

//---------------------------------------------------------------------------------------------
// Spawn monsters
static void spawnMonsters( tMissionInfo& mission )
{
	const int32 kMaxMonstersInPlay = 2;
	const int32 kMaxMonsters = 7;
	tTeamInfo* MonsterTeam = mission.mGetTeamHandle (kMonsterTeam);

	int32 count = MonsterTeam->mGetShipsInPlayCount();

	if ( count >= kMaxMonstersInPlay || gMissionState.fMonsterCount >= kMaxMonsters )
	{
		return;
	}

	static int32 i = 0;
	tString monster = MonsterName[kSpaceshell][mission.mGetDifficultyLevel()];
	MonsterTeam->mCreateShip( monster.mCharStr(), kStartPosition_Specified, kMonsterLocX, kMonsterLocY );
	gMissionState.fMonsterCount++;
	if ( ++i > kNumPoints )
	{
		i = 0;
	}
	mission.mDisplayMessage( kMonsterEnter_msg, kScienceScriptMessage );
}

//---------------------------------------------------------------------------------------------
// Spawn alien ships
static void spawnShips( tMissionInfo& mission )
{
	const int32 kMaxShipsInPlay = 3;
	const int32 kMaxShips = 7;
	tTeamInfo* EnemyTeam = mission.mGetTeamHandle (kEnemyTeam);

	int32 count = EnemyTeam->mGetShipsInPlayCount();

	if ( count >= kMaxShipsInPlay || gMissionState.fEnemyShipCount >= kMaxShips )
	{
		return;
	}

	static int32 i = 0;
	EnemyTeam->mCreateShip( AveragePlayerBPV, ( ( LargestEnemyShipClass == kClassFrigate ) ? 
		LargestEnemyShipClass : (eClassTypes) ( LargestEnemyShipClass - 1 ) ), LargestEnemyShipClass, 0, 
		kStartPosition_Specified, kPatrolPoints[i].x, kPatrolPoints[i].y );
	gMissionState.fEnemyShipCount++;
	if ( ++i > kNumPoints )
	{
		i = 0;
	}
	mission.mDisplayMessage( kKlingonEnter_msg, kScienceScriptMessage );
}

static void shipReachedDestination( tMissionInfo& mission, tShipInfo* Ship )
{
	if ( Ship->mGetTeam() != kAlliedTeam )
	{
		return;
	}

	Ship->mSetShipThrust(0);
	if ( Ship->mGetClassType() == kClassFreighter && !gMissionState.fFreighterLoaded )
	{
		gMissionState.fFreighterLoaded = kTrue;
		Ship->mSetFinalDestination( kBlackHoleLocX - 2, kBlackHoleLocY );
		Ship->mSetShipMaxSpeed(20);
		mission.mDisplayMessage( kFreighterReachesPhenomenon_msg, kScienceScriptMessage );
	}
	else
	{
		Ship->mSetShipMaxSpeed(0);
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	switch (Event)
	{
		case kNotifyAtDestination:
			shipReachedDestination( fMission, Ship );
			break;
				
		case kNotifyDamage:
			{
			}
			break;
	
		case kNotifyOnCapture:
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kAlliedTeam && Ship->mGetClassType() == kClassFreighter )
			{
				int32 shipLocX, shipLocY;
				Ship->mGetGridPosition( &shipLocX, &shipLocY );
				int32 squareDist = Sqr( shipLocX - kBlackHoleLocX ) + Sqr( shipLocY - kBlackHoleLocY );
				if ( !gMissionState.fFreighterLoaded || squareDist > 2 )
				{
					fMission.mDisplayMessage( kFreighterDestroyedEscaped_msg, kScienceScriptMessage );
					return kPlayerLoses;
				}
				else
				{
					gMissionState.fAnomalyDestroyed = kTrue;
					fMission.mDisplayMessage( kFreighterDestroyedBlackHole_msg, kScienceScriptMessage );
					return kEnemyDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			{
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

// Timers...
		case kNotifyAtTimedInterval:
			switch ( fEventID )
			{
			case kTimerLimitSet_3:
				{
					// test if the player is near the edge of the map
					int32 ShipLocX = 0;
					int32 ShipLocY = 0;
					int32 MapWidth = 0;
					int32 MapLength = 0;

					const int32 kClose = 1;
					fMission.mGetMapDimensions( &MapWidth, &MapLength );
					for (tShipInfo* pShip = PlayerTeam->mGetFirstShip(); pShip != NULL; pShip = PlayerTeam->mGetNextShip())
					{
						pShip->mGetGridPosition( &ShipLocX, &ShipLocY );

						if (( ShipLocX <= kClose )
							|| ( ShipLocX >= MapWidth - kClose )
							|| ( ShipLocY <= kClose ) 
							|| ( ShipLocY >= MapLength - kClose ))
						{
							if ( !gMissionState.fBorderWarning )
							{
								fMission.mDisplayMessage( kApproachingEdge_msg, kScienceScriptMessage );
								gMissionState.fBorderWarning = 10;
							}
							else
							{
								gMissionState.fBorderWarning--;
							}
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
					}
				}
				break;
			case kTimerLimitSet_4:
				// Spawn monsters!
				if ( !gMissionState.fFreighterLoaded )
				{
					spawnMonsters( fMission );
				}
				fMission.mSetTimedInterval( kTimerLimitSet_4, kMonsterSpawnTime );
				break;
			case kTimerLimitSet_5:
				// Spawn enemy vessels
				if ( !gMissionState.fFreighterLoaded )
				{
					spawnShips( fMission );
					spawnShips( fMission );
				}
				fMission.mSetTimedInterval( kTimerLimitSet_5, kEnemySpawnTime );
				break;
			case kTimerLimitSet_6:
				{
					// Chatter
					if ( gMissionState.fChatCounter + kChatter1_msg <= kChatter6_msg )
					{
						const int32 chat = gMissionState.fChatCounter++ + kChatter1_msg;
						if ( gMissionState.fShipsEntered && chat == kChatter6_msg )
						{
							fMission.mDisplayMessage( chat+1, kScienceScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( chat, kScienceScriptMessage );
						}
						fMission.mSetTimedInterval( kTimerLimitSet_6, 20 );
					}
					break;
				}
			case kTimerLimitSet_7:
				fMission.mDisplayMessage( kScience1_msg, kScienceScriptMessage );
				fMission.mDisplayMessage( kWeapon2_msg, kScienceScriptMessage );
				fMission.mDisplayMessage( kCaptain1_msg, kMissionScriptMessage );
				break;
			case kTimerLimitSet_10:
				{
					eDeepScanFlags flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					switch (flag)
					{
						case kOutOfRange_ScanOff:
							// player is not within any objects range and deep scan is off (Object == NULL)
							break;
						case kOutOfRange_ScanOn:
							// player is not within any objects range and deep scan is on (Object == NULL)
							break;
						case kEnteringRange_ScanOff:
							// player has just entered range and scan is off (Object points to entity)
							// notify player to turn scan on
							if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
							{
								fMission.mDisplayMessage (kEnemyEnteringRange_ScanOff_msg, kScienceScriptMessage);
							}
							else if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOff_msg, kScienceScriptMessage);
							}
							
							break;
						case kEnteringRange_ScanOn_TimerStarted:
							// player has just entered range and scan is on (Object points to entity)
							// notify player that timer has begun
							if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
							{
								fMission.mDisplayMessage (kEnemyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							else if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							break;
						case kInRange_ScanOff_TimerStopped:
							// player is in range and scan is off (Object points to entity)
							// notify player that timer has been turned off
							fMission.mDisplayMessage (kInRange_ScanOff_TimerStopped_msg, kScienceScriptMessage);
							break;
						case kInRange_ScanOff:
							// player is still in range and scan is still off (Object points to entity)
							break;
						case kInRange_ScanOn_TimerStarted:
							// player is in range and scan is on (Object points to entity)
							// notify player that timer has begun
							if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
							{
								fMission.mDisplayMessage (kEnemyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							else if ( Object == NULL )
							{
								fMission.mDisplayMessage (kAnomalyEnteringRange_ScanOn_TimerStarted_msg, kScienceScriptMessage);
							}
							break;
						case kInRange_ScanOn:
							// player is still in range and scan is still on (Object points to entity)
							break;
						case kLeavingRange_ScanOn_TimerStopped:
							// player has just left the range and scan is on (Object points to entity)
							// notify player that timer is now off
							fMission.mDisplayMessage (kLeavingRange_ScanOn_TimerStopped_msg, kScienceScriptMessage);
							break;
						case kLeavingRange_ScanOff:
							// player has just left the range and scan is off (Object points to entity)
							break;
					}
				}
				break;

			case kTimerCountdownSet:
				{
					eDeepScanFlags flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					if (flag == kInRange_ScanOn)
					{
						// player has successfully scanned the object
						if ( ( Object != NULL ) && ( Object->mGetTeam() == kEnemyTeam ) )
						{
							fMission.mDisplayMessage( kEnemyScanned_msg, kScienceScriptMessage );
						}
						else if ( Object == NULL )
						{
							gMissionState.fAnomalyScanCount++;
							fMission.mDisplayMessage( kAnomalyScanned_msg, kScienceScriptMessage );
							DeepScanTracker.mRemoveLocation( MapPoint.x, MapPoint.y );
						}
						if ( Object )
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
				}
				break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kCommButtonHail_msg:
					switch ( Ship->mGetTeam() )
					{
					case kEnemyTeam:
						fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonHail1_msg, kKlingonHail4_msg), kMissionScriptMessage);
					break;
					case kMonsterTeam:
						fMission.mDisplayMessage ( kMonster_msg, kMissionScriptMessage);
					break;
					case kAlliedTeam:
						fMission.mDisplayMessage ( kFreighter_msg, kMissionScriptMessage);
					break;
					}
					Ship->mSetCommButton (1, kCommButtonHail_msg);
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	// LNF: Add Dynaverse rating adjustments
	if( gMissionState.fAnomalyDestroyed )
	{	// Excellent work Commander!
		return kAstoundingVictory;
	}
	else 
	{	// Just what did you do to convince us to put you in command???
		return kDevastatingDefeat;
	}

	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	switch ( VictoryLevel )
	{
	case kAstoundingVictory:
		fMission.mAddToDebriefingList( kAstoundingVictory_msg );
		break;
	default:
		fMission.mAddToDebriefingList( kDevastatingDefeat_msg );
		break;
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		// this determines the near maximum number of ships a player
		// should have to face if he were fighting frigates
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassWarDestroyer)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	// just adding some randomness into the final number of ships
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(-1, 1);
	}
	else
	{
		//NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}

