#include "scriptinterface.h"

#include "Fed_1MissionText.h"
#include "Fed_1MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 PlayerCaptured = 0;
int32 EnemyCaptured = 0;
int32 PlayerDestroyed = 0;
int32 EnemyDestroyed = 0;
int32 AtWarWithKlingon = 0;
int32 PlanetScanned = 0;
int32 PodAttack = 0; // pods begin attack
int32 PodsDestroyed = 0;
int32 PodsScanned = 0; // flag if all pods have been scanned
int32 Unprovoked = 0;
int32 PodScannedNumber[4] = {0}; // holds the flag when the pod is scanned
int32 StartScanPodNumber = 0; // the pod number that is currently being scanned
int32 PodScanned = 0; // total number of pods scanned
int32 KlingonsOut = 0; // if the Klingon team is out of commission
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);
int32 Mode = 0; // generally controls when ships appear in-game

const float kDeepScanDistanceProbe = 10.0;
const float kDeepScanDistancePlanet = 30.0;
bool AllPodsScanned = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void OrbitPlanet(tTeamInfo* OrbitTeam); // make the Klingon orbit the planet
void ListeningPostScanned( tScript& tScript, tShipInfo* Object, int32 StartScanPodNumber );
void CheckWinCondition( tScript& tScript );

enum eMissionMode
{
	kStart = 0,
	kKlingonArrive
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kLpTeam,
	kPlanetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kKlingMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kKlingon;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, kKlingon);
	fMission.mSetTeamRace (kLpTeam, kFederation);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kLpTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kEnemyTeam, kLpTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kLpTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // use the player's ship(s)
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		// create fed listening posts
		LpTeam->mCreateShip ("F-LP", kStartPosition_J);
		LpTeam->mCreateShip ("F-LP", kStartPosition_K);
		LpTeam->mCreateShip ("F-LP", kStartPosition_L);

		// create planet
		PlanetTeam->mCreateShip("PL1", kStartPosition_Specified, 20, 20);
	}
	else // Mode == kKlingonArrive
	{
		// enemy ships
		EnemyTeam->mCreateFleet (NumberOfPlayersShips, NumberOfPlayersShips * AveragePlayerBPV, kClassFrigate, PlayerShipClass, 100, kStartPosition_Specified, 15, 16);
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tShipInfo* Ship;
	
	int32 LpCount = 1;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// configure Organia (the planet)
	PlanetTeam->mGetFirstShip()->mChangeName ( kPlanetOrgania_name );

	// configure Listening Post team
	for (Ship = LpTeam->mGetFirstShip(); Ship != NULL; Ship = LpTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents(kNotifyDamage);
		if ( LpCount == 1 )
		{
			Ship->mChangeName(kListeningPost1_name);
		}
		else if ( LpCount == 2 )
		{
			Ship->mChangeName(kListeningPost2_name);
		}
		else if ( LpCount == 3 )
		{
			Ship->mChangeName(kListeningPost3_name);
		}
		LpCount++;
	}


	fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );

	// timer to set initial team relations
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );

	// set the on screen goal indicator
	// LNF should I keep this in here?
	//fMission.mSetGoalIndicator( LpTeam->mGetFirstShip() );

	// timer to check win conditions
	fMission.mSetTimedInterval( kTimerLimitSet_2, 5 );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan ListeningPost from within kDeepScanDistance for 1 turn
	DeepScanTracker.mSetTarget (LpTeam->mGetThisShip(1), kDeepScanDistanceProbe, 1);
	DeepScanTracker.mSetTarget (LpTeam->mGetThisShip(2), kDeepScanDistanceProbe, 1);
	DeepScanTracker.mSetTarget (LpTeam->mGetThisShip(3), kDeepScanDistanceProbe, 1);
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, 3);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// configure Klingon ships
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetFinalDestination(23, 23);
	Ship->mSetShipCallbackEvents(kNotifyDamage);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);


	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);
	int32 WeaponsRemaining = 0;


	switch (Event)
	{
		case kNotifyAtDestination:
			if ((Unprovoked == 1) || (AtWarWithKlingon == 1))
			{
				fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			}
			else // else make up some kind of random movement thing, maybe a patrol route
			{
				OrbitPlanet(EnemyTeam);
			}
			break;
				
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);
				// if player fires on Klingon, set AtWar
				// if player fires on listening post
				//   listening posts attack ?all?
				if ( HostileShip != NULL )
				{
					if (((HostileShip->mGetTeam() == kEnemyTeam) && (PlanetScanned == 0)) &&
						(Unprovoked == 0) && (AtWarWithKlingon == 0))
					{
						fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mDisplayMessage (kEvent5Responding_msg, kMissionScriptMessage);
						Unprovoked = 1;
						AtWarWithKlingon = 1;
					}
					else if (( HostileShip->mGetTeam() == kPlayerTeam ) &&
							 ( Ship->mGetTeam() == kLpTeam ))
					{
						if ( Ship == LpTeam->mGetThisShip(1) )
						{
							if ( PodScannedNumber[1] == 0 )
							{
								fMission.mDisplayMessage( kWarnToScanPod_msg, kMissionScriptMessage );
							}
						}
						else if ( Ship == LpTeam->mGetThisShip(2) )
						{
							if ( PodScannedNumber[2] == 0 )
							{
								fMission.mDisplayMessage( kWarnToScanPod_msg, kMissionScriptMessage );
							}
						}
						else if ( Ship == LpTeam->mGetThisShip(3) )
						{
							if ( PodScannedNumber[3] == 0 )
							{
								fMission.mDisplayMessage( kWarnToScanPod_msg, kMissionScriptMessage );
							}
						}
					}
					else
					{
						Ship->mSetShipCallbackEvents(kNotifyDamage);
					}
				}
			}
			break;
	
		case kNotifyOnCapture:
			if ( Ship->mGetOriginalTeam() == kLpTeam )
			{
				fMission.mDisplayMessage( kLpCaptured_msg, kMissionScriptMessage );
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 1023 );
				fMission.mSetTimedInterval( kTimerLimitSet_4, 1 ); // science message
			}
			else if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerCaptured++;
				// LNF if it does/will change the 1 to 0
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerCaptured;
				}
			}
			else if (Ship->mGetOriginalTeam() == kEnemyTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				EnemyCaptured++;
				EnemyDestroyed++;
				fMission.mDisplayMessage (kBoarded_msg, kMissionScriptMessage);
				if (EnemyTeam->mGetShipsInPlayCount() == 0)
				{
					KlingonsOut = 1;
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kEnemyTeam)
			{
				EnemyDestroyed++;
				if (EnemyTeam->mGetShipsInPlayCount() == 0)
				{
					// LNF need to modify this to take into account disabled/captured ships?
					KlingonsOut = 1;
				}
			}
			else if ( Ship->mGetTeam() == kLpTeam )
			{
				PodsDestroyed++;
				// if the pods are destroyed before they are scanned, player loses
				if (( Ship == LpTeam->mGetThisShip(1) ) && ( PodScannedNumber[1] == 0 ))
				{
					fMission.mDisplayMessage (kShouldaScanned_msg, kMissionScriptMessage);
					// LNF do we return a mission failed here or just not give the player points?
					return kPlayerLoses;
				}
				else if (( Ship == LpTeam->mGetThisShip(2) ) && ( PodScannedNumber[2] == 0 ))
				{
					fMission.mDisplayMessage (kShouldaScanned_msg, kMissionScriptMessage);
					// LNF do we return a mission failed here or just not give the player points?
					return kPlayerLoses;
				}
				else if (( Ship == LpTeam->mGetThisShip(3) ) && ( PodScannedNumber[3] == 0 ))
				{
					fMission.mDisplayMessage (kShouldaScanned_msg, kMissionScriptMessage);
					// LNF do we return a mission failed here or just not give the player points?
					return kPlayerLoses;
				}
				fMission.mSetTeamRelations (kPlayerTeam, kLpTeam, kTeamNeutral, kGoalDestroy, 1.0);
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kLpTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kEnemyTeam, kLpTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					// start scenario message
					fMission.mDisplayMessage( kStartScenario_1_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kStartScenario_2_msg, kMissionScriptMessage );

					Mode = kKlingonArrive;
					mCreateMissionShips( kKlingonArrive );
					break;

				case kTimerLimitSet_2:
					// win conditions
					if ((PodScanned == 3) && (PlanetScanned == 1) && (KlingonsOut == 1))
					{
						return kPlayerWins;
					}
					fMission.mSetTimedInterval( kTimerLimitSet_2, 3 );
					break;

				case kTimerLimitSet_3: // answer klingon hail
					EnemyTeam->mClearCommands();
					fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					AtWarWithKlingon = 1;
					break;

				case kTimerLimitSet_4: // science message explaining capture of lp
					fMission.mDisplayMessage( kLpCaptureExplain_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// notify player to turn scan on
								if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
								{
									fMission.mDisplayMessage (kScannerOffInRangePlanet_msg, kScienceScriptMessage);
								}
								else if (( Object != NULL ) && ( Object->mGetTeam() == kLpTeam ))
								{
									fMission.mDisplayMessage (kScannerOffInRangeLp_msg, kScienceScriptMessage);
								}
								
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// notify player that timer has begun
								if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
								{
									fMission.mDisplayMessage (kEvent4BeginScanningPlanet_msg, kScienceScriptMessage);
								}
								else if (( Object != NULL ) && ( Object->mGetTeam() == kLpTeam ))
								{
									fMission.mDisplayMessage (kEvent1A1_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// notify player that timer has begun
								if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
								{
									fMission.mDisplayMessage (kEvent4BeginScanningPlanet_msg, kScienceScriptMessage);
								}
								else if (( Object != NULL ) && ( Object->mGetTeam() == kLpTeam ))
								{
									fMission.mDisplayMessage (kEvent1A1_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// notify player that timer is now off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
							{
								if ((Unprovoked == 0) && (AtWarWithKlingon == 0))
								{
									fMission.mDisplayMessage( kPlanetScanned_msg, kScienceScriptMessage );

									// if the Klingons are somehow killed, don't display this message
									if ( KlingonsOut == 0 )
									{
										fMission.mDisplayMessage (kEvent4CommHail_msg, kMissionScriptMessage);
									}

									for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
									{
										Ship->mSetCommButton (1, kButtonGoAhead_text);
										Ship->mSetCommButton (2, kButtonIgnore_text);
									}
									// LNF should I keep this in here?
									//EnemyTeam->mGetFirstShip()->mSetCommButton (1, kButtonGoAhead_text, kTrue);
								}
								else
								{
									fMission.mDisplayMessage( kPlanetScanned_msg, kScienceScriptMessage );
								}
								PlanetScanned = 1;
							}
							else if (( Object != NULL ) && ( Object == LpTeam->mGetThisShip(1) ))
							{
								ListeningPostScanned( *this, Object, 1 );
							}
							else if (( Object != NULL ) && ( Object == LpTeam->mGetThisShip(2) ))
							{
								ListeningPostScanned( *this, Object, 2 );
							}
							else if (( Object != NULL ) && ( Object == LpTeam->mGetThisShip(3) ))
							{
								ListeningPostScanned( *this, Object, 3 );
							}
	// LNF I have to check the win condition here too.
	// LNF planet scanned too
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
					break;

				case kTimerTurnsSet_10:
					// set the Enemy Team (Klingon) comm buttons back to HAIL
					for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kButtonGoAhead_text:
					for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
					{
						Ship->mClearCommButton( 1 );
						Ship->mClearCommButton( 2 );
					}
					fMission.mDisplayMessage (kEvent4KlingonHail1_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent4KlingonHail2_msg, kMissionScriptMessage);
					fMission.mSetTimedInterval( kTimerLimitSet_3, 3 ); // wait a few seconds, then make Klingon attack
					fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
					break;

				case kButtonIgnore_text:
					for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
					{
						Ship->mClearCommButton( 1 );
						Ship->mClearCommButton( 2 );
					}
					fMission.mDisplayMessage (kEvent4Science_msg, kScienceScriptMessage);
					fMission.mSetTimedInterval( kTimerLimitSet_3, 3 ); // wait a few seconds, then make Klingon attack
					fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
					break;

				case kButtonHail_text:
					if (Ship->mGetTeam() == kLpTeam)
					{
						fMission.mDisplayMessage (kLpHail_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonHail_text);
					}

					// LNF if hailing the Enemy team
					if (Ship->mGetTeam() == kEnemyTeam)
					{
						if (AtWarWithKlingon == 0)
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonNotHostile1_msg, kKlingonNotHostile3_msg), kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonHostile1_msg, kKlingonHostile2_msg), kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
					}
					break;

				case kButtonSurrender_text:
					if (AtWarWithKlingon == 0)
					{
						fMission.mDisplayMessage (kKlingonEveryRight_msg, kMissionScriptMessage);
					}
					else
					{
						fMission.mDisplayMessage (kKlingonHumor_msg, kMissionScriptMessage);
					}
					Ship->mSetCommButton (2, kButtonSurrender_text);
					fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
					break;

				case kButtonTaunt_text:
					if (AtWarWithKlingon == 0)
					{
						fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonTauntA_msg, kKlingonTauntD_msg), kMissionScriptMessage);
					}
					else
					{
						fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonTauntE_msg, kKlingonTauntH_msg), kMissionScriptMessage);
					}
					Ship->mSetCommButton (1, kButtonTaunt_text);
					fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF there used to be a number of Klingons still alive that game points...dunno if that's relevant anymore

	VictoryPoints = (EnemyDestroyed * 3) + (PlanetScanned * 40) + (PodsDestroyed * 5) +
					(EnemyCaptured * 5) + ( PodScannedNumber[1] * 10 ) + ( PodScannedNumber[2] * 10 ) +
					( PodScannedNumber[3] * 10 );

	if (Unprovoked == 1)
	{
		// LNF need team relationship from Dynaverse to 
		// set the proper text and set the proper victory points
		eTeamRelationship blah=fMission.mGetTeamRelations(kTeam1, kTeam7);
						
		// LNF maybe some explanatory text in the debriefing if the player attacks the Klingons
		// LNF unprovokedly

		// LNF: check for current Dynaverse status of Klingons/Feds
		// -40 if allied, -20 if neutral, 0 if at war
		VictoryPoints -= 40;
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if ( Unprovoked == 1 )
		{
			fMission.mAddToDebriefingList (kUnprovokedDebrief_msg);
		}
		else if ((AtWarWithKlingon == 1) && (EnemyDestroyed == 0))
		{
			fMission.mAddToDebriefingList (kDefendYourself_msg);
		}

		fMission.mAddToDebriefingList (kDismissed_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( EnemyDestroyed != 0 ) ? fMission.mAddToDebriefingList( kEnemyDestroyedTrue_msg ) : SuccessCount++;
	( PlanetScanned != 0 ) ? fMission.mAddToDebriefingList( kPlanetScannedTrue_msg ) : SuccessCount++;
	( PodScanned == 3 ) ? fMission.mAddToDebriefingList( kPodsScannedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( EnemyDestroyed == 0 ) ? fMission.mAddToDebriefingList( kEnemyDestroyedFalse_msg ) : FailureCount++;
	( PlanetScanned == 0 ) ? fMission.mAddToDebriefingList( kPlanetScannedFalse_msg ) : FailureCount++;
	( PodScanned < 3 ) ? fMission.mAddToDebriefingList( kPodsScannedFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		// this determines the near maximum number of ships a player
		// should have to face if he were fighting frigates
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassWarDestroyer)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	// just adding some randomness into the final number of ships
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(-1, 1);
	}
	else
	{
		//NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrbitPlanet(tTeamInfo* OrbitTeam)
{
	tShipInfo* Ship;

	tMapPoint waypoints[5]=
	{
		18, 23,
		22, 23,
		23, 19,
		20, 17,
		17, 19
	};

	tMapPoint PatrolPoints[5];

	for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
	{
		PatrolPoints[0] = waypoints[0];
		PatrolPoints[1] = waypoints[1];
		PatrolPoints[2] = waypoints[2];
		PatrolPoints[3] = waypoints[3];
		PatrolPoints[4] = waypoints[4];
		Ship->mPatrol (1.0, 5, &PatrolPoints[0], kHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
// test if Listening Post is the object scanned
void ListeningPostScanned( tScript& tScript, tShipInfo* Object, int32 StartScanPodNumber )
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* LpTeam = tScript.fMission.mGetTeamHandle (kLpTeam);

	if ( PodScanned == 0 ) // this is the first pod scanned`
	{
		PodAttack = 1;
		tScript.fMission.mDisplayMessage (kEvent1A2_msg, kScienceScriptMessage);
		tScript.fMission.mDisplayMessage (kEvent1B1_msg, kScienceScriptMessage);
		tScript.fMission.mDisplayMessage (kEvent1C1_msg, kScienceScriptMessage);
		tScript.fMission.mDisplayMessage (kEvent1D1_msg, kScienceScriptMessage);
		tScript.fMission.mDisplayMessage (kEvent1E1_msg, kScienceScriptMessage);
		LpTeam->mGetThisShip(StartScanPodNumber)->mSetShipTarget( tScript.fPlayerShip, kAbsolutePriority );
		//tScript.fMission.mSetTeamRelations (kLpTeam, tScript.kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
		//tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kLpTeam, kTeamHostile, kGoalDestroy, 1.0);
	}
	else // one pod has been scanned before
	{
		tScript.fMission.mDisplayMessage (kEvent1AlreadyScanned_msg, kScienceScriptMessage);
		if ( PodsDestroyed > 0 )
		{
			tScript.fMission.mSetTeamRelations (kLpTeam, tScript.kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
			tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kLpTeam, kTeamNeutral, kGoalDestroy, 1.0);
		}
		else
		{
			LpTeam->mGetThisShip(StartScanPodNumber)->mSetShipTarget( tScript.fPlayerShip, kAbsolutePriority );
		}
	}
	PodScanned++;
	PodScannedNumber[StartScanPodNumber] = 1;
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	int32 NumberOfPodsScanned = 0;
/*
	for ( int32 count = 3; count > 0; count-- )
	{
		if ( PodScannedNumber[count] == 1 )
		{
			NumberOfPodsScanned++;
			if ( NumberOfPodsScanned == 3 )
			{
				AllPodsScanned = true;
			}
		}
	}

	if (
	{
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
	}*/
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}

