#include "scriptinterface.h"

#include "Fed_2MissionText.h"
#include "Fed_2MissionMaps.h"

/***********************************
LNF if the player has tractor activated, and the first orion isn't destroyed
LNF the orion should attack the freighter

LNF I also need to balance the freighter against the player's ship
***********************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 PlayerDestroyed = 0;
int32 EnemyDestroyed = 0;
int32 BostonDestroyed = 0;
int32 BostonCaptured = 0;
int32 BostonArrived = 0; // Boston appears in the game
int32 SesDestroyed = 0;
int32 SesOrbiting = 0; // SES is close enough to sun
int32 Unprovoked = 0; // Boston has been fired upon before Betrayal
int32 Betrayal = 0; // Boston has tractored SES
int32 FirstOrionInRange = 0;
int32 BattleBegun = 0; // Boston is an Enemy
int32 BostonMovingToIntercept = 0;
int32 BostonAtSes = 0;
int32 TractoringSes = 0; // Same thing as Betrayal or just a state?
int32 Mode = 0;
int32 StartMission = 0;
int32 SesActivated = 0;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool OrionReinforcementsArrived = false; // have the orion reinforcements arrived
bool SesIsFollowingBoston = false;

const int32 kTotalNumberOfOrions = 4;
const float kSesOrbitingDistance = 80.0;
const int32 kSesExperimentTimer = 8;

eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );

enum eMissionMode
{
	kStart = 0,
	kOrionArrival,
	kBostonArrival,
	kOrionReinforcements
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBostonTeam,
	kSesTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission; // LNF should this be only Fed?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, kOrion);
	fMission.mSetTeamRace (kBostonTeam, kMirror);
	fMission.mSetTeamRace (kSesTeam, kFederation);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBostonTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kSesTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kBostonTeam, kTeamNeutral, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kSesTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kBostonTeam, kEnemyTeam, kTeamNeutral, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kBostonTeam, kSesTeam, kTeamNeutral, kGoalCapture, 1.0);
	
	fMission.mSetTeamRelations (kSesTeam, kBostonTeam, kTeamNeutral, kGoalIgnore, 1.0);
	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BostonTeam = fMission.mGetTeamHandle (kBostonTeam);
	tTeamInfo* SesTeam = fMission.mGetTeamHandle (kSesTeam);
	tShipInfo* Ship;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if (Mode == kStart)
	{
		// create SES - Small Freighter with Phasers
		SesTeam->mCreateShip ("F-FQS", kStartPosition_J);
	}
	else if ( Mode == kOrionArrival )
	{
		// create Orion team
		EnemyTeam->mCreateShip (AveragePlayerBPV, kClassFrigate, kClassLightCruiser, 30, kStartPosition_W);
	}
	else if (Mode == kBostonArrival)
	{
		Ship = fPlayerShip;
		int32 OldX = 0;
		int32 OldY = 0;
		Ship->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + fMission.mRandomInt32 (1, 2);
		int32 NewY = OldY + fMission.mRandomInt32 (5, 10);
		// LNF: Change race relations of the Boston to attack the Orions??

		// LNF turn off tractor officer
		//Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );

		fMission.mSetTeamRelations (kEnemyTeam, kBostonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		// create Boston ship
		BostonTeam->mCreateShip( kMaxBPV, kClassDestroyer, 0, kStartPosition_Specified, NewX, NewY);
	}
	else if (Mode == kOrionReinforcements)
	{
		Ship = fPlayerShip;
		int32 OldX = 0;
		int32 OldY = 0;
		Ship->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + fMission.mRandomInt32 (-2, 2);
		int32 NewY = OldY + fMission.mRandomInt32 (8, 10);
		// create more pirate ships
		EnemyTeam->mCreateFleet( (NumberOfPlayersShips + 1), TotalBPV, kClassFrigate, LargestEnemyShipClass, 100, kStartPosition_Specified, NewX, NewY );

		fMission.mDisplayMessage( kMoreOrions_msg, kMissionScriptMessage );
		fMission.mDisplayMessage( kBostonTallyho_msg, kMissionScriptMessage );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* SesTeam = fMission.mGetTeamHandle (kSesTeam);
	tShipInfo* Ship = NULL;
	tShipInfo* SesShip = NULL;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}

	// configure SES ship
	SesShip = SesTeam->mGetThisShip(1);
	SesShip->mTrackTacticalDistance( kSesOrbitingDistance, 20, 20 ); // Set tracking distance to the star
	SesShip->mSetCommButton (1, kButtonHail_text);
	SesShip->mChangeName(kDukain_name);
	SesShip->mSetShipCallbackEvents(kNotifyDamage);
	// turn off ship's AI
	SesShip->mSetShipAIControlled( kFalse );
	
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// use a timer to check for the win condition
	fMission.mSetTimedInterval( kTimerLimitSet_3, 4 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* BostonTeam = fMission.mGetTeamHandle (kBostonTeam);
	tTeamInfo* SesTeam = fMission.mGetTeamHandle (kSesTeam);

	tShipInfo* BostonShip = BostonTeam->mGetFirstShip();
	
	if ( Mode == kOrionArrival )
	{
		// configure Orion ship
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents(kNotifyDamage);
	}
	else if ( Mode == kBostonArrival )
	{
		// configure Boston ship
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mChangeName(kBoston_name);
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		BostonShip->mDefendEntity(SesTeam->mGetFirstShip(), 70.0, 70.0, kAbsolutePriority);
	}
	else if ( Mode == kOrionReinforcements )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BostonTeam = fMission.mGetTeamHandle (kBostonTeam);
	tTeamInfo* SesTeam = fMission.mGetTeamHandle (kSesTeam);
	tShipInfo* SesShip;
	tShipInfo* BostonShip;

	SesShip = SesTeam->mGetFirstShip();
	BostonShip = BostonTeam->mGetFirstShip();
	switch (Event)
	{
		case kNotifyBeginTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID (fEventID); // what the Ship is tractoring

				if ( TractorTarget != NULL )
				{
					if (( Ship == fPlayerShip ) && ( TractorTarget == SesTeam->mGetFirstShip() ))
					{
						if ( SesOrbiting == 0 )
						{
							fMission.mDisplayMessage( kTractoringSes_msg, kScienceScriptMessage );
						}
						// if the SES is orbiting, check to make sure the player doesn't move it from orbit
						else if ( SesOrbiting == 1 )
						{
							fMission.mDisplayMessage( kDoNotDisturb_msg, kScienceScriptMessage );
							// set a counter to check the distance from sun
							fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
						}
					}
					else if (( Ship->mGetTeam() == kBostonTeam ) && ( TractorTarget == SesTeam->mGetFirstShip() ))
					{
						// LNF is this where the Betrayal goes?

						Betrayal = 1;
						//Ship->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kBostonTeam, kTeamHostile, kGoalCapture, 1.0);
						fMission.mSetTeamRelations (kBostonTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);
						fMission.mDisplayMessage (kEvent6a_msg, kScienceScriptMessage);
						//fMission.mDisplayMessage (kEvent6aHACK_msg, kScienceScriptMessage);
						fMission.mDisplayMessage (kEvent6b_msg, kScienceScriptMessage);
						//Ship->mSetFinalDestination( 40, 40 );
						//SesTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
						//SesTeam->mGetFirstShip()->mFollowEntity( Ship, kVeryHighPriority );
						SesIsFollowingBoston = true; // make the freighter leave the map on its own when the boston comes under fire
						// reset the ship max speed because it's stuck at 8 from the tractor?
						//Ship->mSetShipMaxSpeed( 25 );
					}
				}
				Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
			}
			break;

		case kNotifyAtDistance:
			// checks to see if the SES is in range of the sun to begin the experiment
			if (( Ship->mGetTeam() == kSesTeam ) &&
				( Ship->mGetTrackingDistance() == kSesOrbitingDistance ))
			{
				SesOrbiting = 1;

				// if the first Orion is destroyed, then bring in the Orion reinforcements
				if ( EnemyDestroyed == 1 )
				{
					fMission.mDisplayMessage (kEvent4SesOrbiting_msg, kScienceScriptMessage);
		
					if ( BostonArrived == 0 )
					{
						// use timer to create Boston
						BostonArrived = 1;
						fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					}

					// countdown timer for experiment
					// LNF maybe I should have another timer that will make the boston
					// LNF try to tractor the SES even if more than 1 orion are alive
					fMission.mSetTimedInterval( kTimerCountdownSet, kSesExperimentTimer );

					// if the Boston has arrived, then bring on the Orions!
					if (( BostonArrived == 1 ) && ( OrionReinforcementsArrived == false ))
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, fMission.mRandomInt32( 4, 9 ) );
					}
				}
				else
				{
					fMission.mDisplayMessage (kEvent4SesOrbitingOrionAlive_msg, kScienceScriptMessage);

					// LNF some kind of reminder to tell the player to kill the Orion
					fMission.mDisplayMessage( kFirstOrionStillAlive_msg, kScienceScriptMessage );
				}
			}
			break;
		
		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					if (StartMission == 0)
					{
						fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kPlayerTeam, kBostonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kSesTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kEnemyTeam, kBostonTeam, kTeamHostile, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kEnemyTeam, kSesTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						
						fMission.mSetTeamRelations (kBostonTeam, kEnemyTeam, kTeamHostile, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kBostonTeam, kSesTeam, kTeamNeutral, kGoalIgnore, 1.0);
						
						fMission.mSetTeamRelations (kSesTeam, kBostonTeam, kTeamNeutral, kGoalIgnore, 1.0);

						fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

						StartMission = 1;

						// the first Orion appears
						Mode = kOrionArrival;
						mCreateMissionShips( kOrionArrival );
					}

					if ((fPlayerShip->mGetTacticalDistance (EnemyTeam->mGetFirstShip()) < 55.0) && (FirstOrionInRange == 0))
					{
						fMission.mDisplayMessage (kEvent1OrionInRange_msg, kScienceScriptMessage);
						fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						FirstOrionInRange = 1;
						// turn on the SES's shields
						SesTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 0 );
						fMission.mDisplayMessage( kSesPowerUpShields_msg, kScienceScriptMessage );
					}
					else
					{
						fMission.mSetTimedInterval (kTimerLimitSet, 1);
					}

					break;

				case kTimerLimitSet_2:
					// create Orions after the SES obtains orbit -AND- the Boston has already arrived.

					// LNF should I make the Boston clear commands and attack Orions?
					Mode = kOrionReinforcements;
					mCreateMissionShips (kOrionReinforcements);
					OrionReinforcementsArrived = true;
					break;

				case kTimerLimitSet_3:
					// checks for win condition
					if ((EnemyTeam->mGetShipsInPlayCount() == 0) && (PlayerTeam->mGetShipsInPlayCount() > 0) && 
						(BostonDestroyed == 1))
					{
						return kPlayerWins;
					}
					fMission.mSetTimedInterval( kTimerLimitSet_3, 4 );
					break;

				case kTimerLimitSet_4:
					if ( SesShip->mGetTacticalDistance( 7 ) > kSesOrbitingDistance )
					{
						fMission.mSetTimedInterval( kTimerCountdownSet, 0 );
						fMission.mDisplayMessage( kSesPulledFromOrbit_msg, kScienceScriptMessage );
						SesShip->mTrackTacticalDistance( kSesOrbitingDistance, 20, 20 ); // Set tracking distance to the star
						SesOrbiting = 0;
						break;
					}
					fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
					break;

				case kTimerCountdownSet:
					// countdown time until Dukain sends message
					// LNF temp message
					fMission.mDisplayMessage( kDukainSuccessful_msg, kScienceScriptMessage );
					SesActivated = 1;
					break;

				case kTimerTurnsSet:
					// Boston arrives
					Mode = kBostonArrival;
					mCreateMissionShips (kBostonArrival);
					fMission.mDisplayMessage (kEvent2BostonArrive_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent2PC1_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent2BostonResponse_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent2PC2_msg, kMissionScriptMessage);
					BostonArrived = 1;

					if (( SesOrbiting == 1 ) && ( OrionReinforcementsArrived == false ))
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, fMission.mRandomInt32( 4, 9 ) );
					}
					break;

				case kTimerTurnsSet_9:
					// set the Enemy Team comm buttons back to HAIL
					for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Boston Team comm buttons back to HAIL
					for (Ship = BostonTeam->mGetFirstShip(); Ship != NULL; Ship = BostonTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if (( Ship->mGetTeam() == kBostonTeam ) && ( HostileShip->mGetTeam() == kPlayerTeam ) &&
						( SesIsFollowingBoston == false ))
					{
						if (Unprovoked == 0)
						{
							fMission.mDisplayMessage (kEvent5BostonUnprovoked_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kBostonTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kBostonTeam, kTeamAtWar, kGoalCapture, 1.0);
							Unprovoked = 1;
							//Ship->mSetShipCallbackEvents(kNotifyDamage);
						}
						else
						{
							Ship->mSetShipCallbackEvents(kNotifyDamage);
						}
					}
					else if (( Ship->mGetTeam() == kBostonTeam ) && ( HostileShip->mGetTeam() == kPlayerTeam ) &&
							 ( SesIsFollowingBoston == true ))
					{
						// if the SES is following the boston, clear its commands and have it attack the player
						SesTeam->mGetFirstShip()->mClearCommands();
						SesTeam->mGetFirstShip()->mSetShipAIControlled( kFalse );
						Ship->mClearCommands();
						fMission.mSetTeamRelations (kBostonTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kPlayerTeam, kBostonTeam, kTeamAtWar, kGoalCapture, 1.0);
					}
					else
					{
						Ship->mSetShipCallbackEvents(kNotifyDamage);
					}
				}
			}
			break;

		case kNotifyOnCapture:
			// if the Boston is captured without Betraying player, player needs to repeat?

			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				//EnemyDestroyed++;
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			else if (Ship->mGetOriginalTeam () == kBostonTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				fMission.mDisplayMessage( kBostonCaptured_msg, kMissionScriptMessage );

				BostonCaptured = 1;
				BostonDestroyed = 1;// LNF this is just for victory points, maybe I should do this differently
				// LNF maybe make captured and destroyed different point amounts?

				// LNF I should have to wait for SES?
				return kEnemyCaptured;
			}
			break;

		case kNotifyAtDeath:
			switch (Ship->mGetTeam())
			{
				case kEnemyTeam:

					EnemyDestroyed++;

					if ( EnemyDestroyed == 1 )
					{
						SesTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 100 );
					}

					if (BostonArrived == 0)
					{
						// all clear message & tractor SES reminder
						fMission.mDisplayMessage( kSystemIsClear_msg, kScienceScriptMessage );

						if ( SesOrbiting == 0 )
						{
							fMission.mDisplayMessage( kTractorSesToPosition_msg, kScienceScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( kDukainStartExperiment_msg, kScienceScriptMessage );
							fMission.mSetTimedInterval( kTimerCountdownSet, kSesExperimentTimer );
						}

						// timer to make the Boston arrive
						fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					}

					// if there is only one orion left, the Boston will try to tractor the Dukain
//					if ( EnemyDestroyed == (kTotalNumberOfOrions - 1) )
					if (( EnemyTeam->mGetShipsInPlayCount() == 1 ) && ( Mode == kOrionReinforcements ))
					{
						// tell the Boston to Steal the SES
						BostonShip->mClearCommands();
						fMission.mSetTeamRelations (kBostonTeam, kEnemyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						BostonShip->mTractorEntity(SesTeam->mGetFirstShip(), 39, 20, kVeryHighPriority);

						//** the tractor entity doesn't work, so in the TractorBegin notify, clear the 
						//** Boston's commands again and make the freighter follow the Boston 
					}

					// all clear message
					//if (EnemyDestroyed == kTotalNumberOfOrions)
					if (( EnemyTeam->mGetShipsInPlayCount() == 0 ) && ( Mode == kOrionReinforcements ))
					{
						fMission.mDisplayMessage (kEvent3OrionDead_msg, kMissionScriptMessage);
						fMission.mDisplayMessage (kEvent3AllOrionDead_msg, kMissionScriptMessage);
					}
					break;

				case kBostonTeam:
					BostonDestroyed = 1;
					break;

				case kSesTeam:
					SesDestroyed = 1;
					fMission.mDisplayMessage (kSesDestroyed_msg, kMissionScriptMessage);
					return kPlayerLoses;
					break;
			} // end of switch

			// LNF: why can't I use kPlayerTeam in the switch?

			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				//fMission.mDisplayMessage (kLNF_msg, kMissionScriptMessage);
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kButtonHail_text:
					if (Ship->mGetTeam() == kBostonTeam)
					{
						if ((BattleBegun == 0) && (Betrayal == 0))
						{
							fMission.mDisplayMessage (kBostonNotEnemy_msg, kMissionScriptMessage);
						}				
						else if ((BattleBegun == 0) && (TractoringSes == 1))
						{
							fMission.mDisplayMessage (kBostonNeTractoring1_msg, kMissionScriptMessage);
							fMission.mDisplayMessage (kBostonNeTractoring2_msg, kMissionScriptMessage);
							fMission.mDisplayMessage (kBostonNeTractoring3_msg, kMissionScriptMessage);
						}
						// Battle started (Boston is an enemy) Unprovoked too?
						else if (Unprovoked == 0) // player initiated combat w/o Boston tractoring Ses
						{
							fMission.mDisplayMessage (kBostonEnemyUnprovoked_msg, kMissionScriptMessage);
						}
						else // Boston has tractored the Ses
						{
						}
						if (BattleBegun || Betrayal)
						{
							Ship->mSetCommButton (1, kButtonTaunt_text);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							// If the Taunt or Surrender buttons aren't used in X turns
							// return the buttons to Hail and blank.
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
						else
						{
							Ship->mSetCommButton (1, kButtonHail_text);
						}
					}

					// Orion Hail not retreating/disabled
					if (Ship->mGetTeam() == kEnemyTeam)
					{
						fMission.mDisplayMessage (kOrionHail_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
					}
					// LNF: Orion Hail retreating, disabled, boarded

					// SES Hail
					if (Ship->mGetTeam() == kSesTeam)
					{
						fMission.mDisplayMessage (kSesHail_msg, kScienceScriptMessage);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kButtonSurrender_text:
					if (BattleBegun == 0)
					{
						if (Ship->mGetTeam() == kBostonTeam)
						{
							fMission.mDisplayMessage (kBostonSurrender_msg, kMissionScriptMessage);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
					}

					// Orion Surrender if not badly damaged
					if (Ship->mGetTeam() == kEnemyTeam)
					{
						fMission.mDisplayMessage (kOrionSurrender_msg, kMissionScriptMessage);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
					}
					break;

				case kButtonTaunt_text:
					if (BattleBegun == 0)
					{
						if (Ship->mGetTeam() == kBostonTeam)
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kBostonTauntA_msg, kBostonTauntD_msg), kMissionScriptMessage);
							Ship->mSetCommButton (1, kButtonTaunt_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
					}

					// Orion Taunts
					if (Ship->mGetTeam() == kEnemyTeam)
					{
						fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTauntA_msg, kOrionTauntD_msg), kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
					}
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
	
	VictoryPoints = (SesOrbiting * 70) + (EnemyDestroyed * 3) + (BostonDestroyed * 5) -
					(Unprovoked * 50) + (BostonCaptured * 20);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (BostonCaptured == 1)
		{
			fMission.mAddToDebriefingList (kVictoryBostonCaptured1_msg);
		}
		
		if (VictoryLevel < kDraw)
		{
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	( BostonCaptured != 0 ) ? fMission.mAddToDebriefingList( kBostonCapturedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	( BostonCaptured == 0 ) ? fMission.mAddToDebriefingList( kBostonCapturedFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// figure out what ship class to use
	if ( RandomNum > 50 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 50 ) && ( RandomNum >= 25 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 25 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 3 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 4 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
}


