#include "scriptinterface.h"

#include "Fed_4MissionText.h"
#include "Fed_4MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 ImperialAtWar = 0;
int32 ImperialDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 CounterpartDestroyed = 0; // counterpart either destroyed or captured
int32 OrionBaseDestroyed = 0;
int32 CounterpartCaptured = 0;
int32 CounterpartBlownUp = 0; // counterpart actually blown up

bool AtWarWithCounterpart = false;
bool AtWarWithNeutral = false;
bool TractoringCounterpart = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool TractoringCounterpartBeforeCapture = false;
bool CounterpartArrived = false;
bool CounterpartConfigured = false;

int32 SetRelations = 0;
int32 NeutralReinforcement = 0;
int32 Mode = 0;
int32 CaptiveTransferred = 0;

const float kDamageThreshold = 60.0;
const float kPatrolAttackRadius = 1.0;

// variables that hold the players supplies to add to the cloned counterpart ship
int32 CounterpartMissileType = 0;
int32 CounterpartMissileSpeed = 0;
int32 CounterpartNumberOfReloads = 0;
int32 CounterpartNumberLoaded = 0;
int32 CounterpartShuttles = 0;
int32 CounterpartSuicides = 0;
int32 CounterpartWildWeasel = 0;
int32 CounterpartScatterPack = 0;
int32 CounterpartSpareParts = 0;
int32 CounterpartMarines = 0;
int32 CounterpartMines = 0;


tShipInfo* PlayersBiggestShip = NULL;

enum eMissionMode
{
	kStart = 0,
	kAllImperialDead,
	kNeutralReinforcement,
};

void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void RomulanCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void GornCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void ImperialPatrol(tTeamInfo* OrbitTeam);
eGameStatus CheckWinCondition (tScript& tScript);

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kImperialTeam, 
	kNeutralTeam, 
	kCounterpartTeam,
	kOrionTeam,
	kOrionBaseTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	// The player's COUNTERPART ship cannot be destroyed, or the mission is over.
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kKlingMission | kRomulanMission | kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	info.fAlliedRace = (fMission.mRandomInt32 (1, 100) < 33) ? kKlingon
					  : (fMission.mRandomInt32 (1, 100) < 66) ? kRomulan : kGorn;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kImperialTeam, kMirror);
	fMission.mSetTeamRace (kCounterpartTeam, kMirror);

	// LNF: The Neutral team should be the same race from Fed_3
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	fMission.mSetTeamRace (kNeutralTeam, info.fAlliedRace);
	fMission.mSetTeamRace (kOrionBaseTeam, kOrion);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kCounterpartTeam, kTeamHostile, kGoalCapture, 1.0);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	// LNF: The Neutral team's disposition might be different
	fMission.mSetTeamRelations (kPlayerTeam, kNeutralTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kImperialTeam, kCounterpartTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperialTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperialTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperialTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kCounterpartTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kCounterpartTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kCounterpartTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kCounterpartTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kOrionTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kNeutralTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* CounterpartTeam = fMission.mGetTeamHandle (kCounterpartTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* NeutralTeam = fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);

	tShipInfo* Ship;
	int32 CurrentX, CurrentY, NewX, NewY;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
			PlayersBiggestShip = Ship;
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	switch (Mode)
	{
		case kStart:
			// create Enemy team
			ImperialTeam->mCreateFleet( 2, (AveragePlayerBPV * 2), kClassFrigate, LargestEnemyShipClass, 0, kStartPosition_U );

			// create Neutral ship
			NeutralTeam->mCreateShip( AveragePlayerBPV, kClassDestroyer, kClassHeavyCruiser, 20, kStartPosition_W );

			// create Orion Base
			OrionBaseTeam->mCreateShip( kMaxBPV, kClassBaseStation, kClassBaseStation, 0, kStartPosition_T );

			// create Orion Defense platforms
			OrionTeam->mCreateShip( "O-DEF", kStartPosition_R );
			OrionTeam->mCreateShip( "O-DEF", kStartPosition_S );
			break;

		case kAllImperialDead:
			// Place the Counterpart ship near the player
			if ( CounterpartArrived == false )
			{
				CounterpartArrived = true;

				Ship = fPlayerShip;
				CurrentX = 0;
				CurrentY = 0;
				Ship->mGetGridPosition (&CurrentX, &CurrentY);
				NewX = CurrentX + fMission.mRandomInt32 (1, 2);
				NewY = CurrentY + fMission.mRandomInt32 (5, 10);

				fMission.mDisplayMessage( kCounterpartArrives_msg, kMissionScriptMessage );
				fMission.mSetTeamRelations (kCounterpartTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
				fMission.mSetTeamRelations (kPlayerTeam, kCounterpartTeam, kTeamAtWar, kGoalCapture, 1.0);
				// create Counterpart ship
				// LNF somehow choose the biggest ship in the player's fleet?
				
				CounterpartTeam->mDuplicateShip( PlayersBiggestShip, kStartPosition_Specified, NewX, NewY );
			}
			break;

		case kNeutralReinforcement:
			// create Neutral reinforcement team
			NeutralTeam->mCreateFleet (3, kMaxBPV, kClassFrigate, kClassDestroyer, 20, kStartPosition_X);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}
	
	// save off the supply info of the player's ship
	
	CounterpartMissileType = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyMissileType );
	CounterpartMissileSpeed = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyMissileSpeed );
	CounterpartNumberOfReloads = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyNumberOfReloads );
	CounterpartNumberLoaded = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyNumberLoaded );
	CounterpartShuttles = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyShuttles );
	CounterpartSuicides = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplySuicides );
	CounterpartWildWeasel = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyWildWeasel );
	CounterpartScatterPack = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyScatterPack );
	CounterpartSpareParts = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplySpareParts );
	CounterpartMarines = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyMarines );
	CounterpartMines = PlayersBiggestShip->mAccessSupplies( kGetSupplyCount, kSupplyMines );

	// configure Imperial ships
	for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents(kNotifyDamage);
		ImperialPatrol(ImperialTeam);
	}

	// configure Neutral ships

	for (Ship = NeutralTeam->mGetFirstShip(); Ship != NULL; Ship = NeutralTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents(kNotifyDamage);
		//Ship->mSetFinalDestination();
	}

	// configure Orion platforms
	for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyDamage);
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* CounterpartTeam = fMission.mGetTeamHandle (kCounterpartTeam);

	if (Mode == kAllImperialDead)
	{
		if ( CounterpartConfigured == false )
		{
			CounterpartConfigured = true;

			// configure Counterpart ship
			Ship = CounterpartTeam->mGetFirstShip();
			Ship->mSetCommButton (1, kButtonHail_text);
			Ship->mChangeName(kCounterpart_name);

			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMissileType, CounterpartMissileType );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMissileSpeed, CounterpartMissileSpeed );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyNumberOfReloads, CounterpartNumberOfReloads );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyNumberLoaded, CounterpartNumberLoaded );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, CounterpartShuttles );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, CounterpartSuicides );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, CounterpartWildWeasel );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, CounterpartScatterPack );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, CounterpartSpareParts );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, CounterpartMarines );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, CounterpartMines );
		}
	}
	else if (Mode == kNeutralReinforcement)// configure Neutral Team reinforcements
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		// LNF: set a patrol route?  If this is called, I think the 
		// Neutrals will be at war.
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* CounterpartTeam = fMission.mGetTeamHandle (kCounterpartTeam);
	tTeamInfo* NeutralTeam = fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);

	tShipInfo* target = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			{
				if ( target != NULL )
				{
					if ((target == CounterpartTeam->mGetThisShip(1))
						&& ( Ship == fPlayerShip ))
					{
						// LNF only if the counterpart ship is captured?  OR should there just be a distance check to neutrals?

						fMission.mDisplayMessage (fMission.mRandomInt32 (kTractoringCounterpart1_msg, kTractoringCounterpart2_msg), kScienceScriptMessage);

						if ( CounterpartCaptured == 1 )
						{
							// LNF do I need to do something with the Neutrals here?
							TractoringCounterpart = true;
						}
						else
						{
							TractoringCounterpartBeforeCapture = true;
						}

						fPlayerShip->mSetShipCallbackEvents(kNotifyEndTractor);
					}
					else
					{
						fPlayerShip->mSetShipCallbackEvents(kNotifyBeginTractor);
					}
				}
			}
			break;

		case kNotifyEndTractor:

			TractoringCounterpartBeforeCapture = false;

			if ( TractoringCounterpart == true )
			{
				// LNF do I need to do something with the Neutrals here?
				fMission.mDisplayMessage (fMission.mRandomInt32 (kNotTractoringCounterpart1_msg, kNotTractoringCounterpart2_msg), kScienceScriptMessage);
				TractoringCounterpart = false;
			}
			fPlayerShip->mSetShipCallbackEvents(kNotifyBeginTractor);
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kImperialTeam )
						{
							fMission.mSetTeamRelations (kImperialTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							ImperialAtWar = 1;
						}
						else if ( Ship->mGetTeam() == kNeutralTeam )
						{
							fMission.mSetTeamRelations (kNeutralTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							AtWarWithNeutral = true;
							if ( NeutralReinforcement == 0 )
							{
								NeutralReinforcement = 1;
								mCreateMissionShips( kNeutralReinforcement );
							}
						}
						else if ( Ship->mGetTeam() == kOrionTeam )
						{
							fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
						else if ( Ship->mGetTeam() == kCounterpartTeam )
						{
							AtWarWithCounterpart = true;
						}
					}
				}
			}
			break;

	
/* LNF
if the Neutral is attacked or Hailed or encountered? without tractor, set at war
*/

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					if (SetRelations == 0)
					{
						fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalDestroy, 1.0); //***
						fMission.mSetTeamRelations (kPlayerTeam, kCounterpartTeam, kTeamNeutral, kGoalCapture, 1.0);
						fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						// LNF: The Neutral team's disposition might be different
						fMission.mSetTeamRelations (kPlayerTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						
						fMission.mSetTeamRelations (kImperialTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
						fMission.mSetTeamRelations (kImperialTeam, kCounterpartTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kImperialTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kImperialTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kImperialTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kCounterpartTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kCounterpartTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kCounterpartTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kCounterpartTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						
						fMission.mSetTeamRelations (kOrionTeam, kNeutralTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kOrionTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kNeutralTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	
						fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
						// Neutral Team reinforcements arrive
						fMission.mSetTimedInterval( kTimerTurnsSet, 20 );

						SetRelations = 1;
					} 
					break;

				case kTimerLimitSet_2:
					// win condition
					if ( CaptiveTransferred == 1 )
					{
						return kPlayerWins;
					}
					break;

				case kTimerTurnsSet:
					// neutral team reinforcement
					if ( NeutralReinforcement == 0 )
					{
						Mode = kNeutralReinforcement;
						mCreateMissionShips(kNeutralReinforcement);
						NeutralReinforcement = 1;
					}
				break;

				case kTimerTurnsSet_8:
					// set the Neutral Team comm buttons back to HAIL
					for (Ship = NeutralTeam->mGetFirstShip(); Ship != NULL; Ship = NeutralTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Counterpart Team comm buttons back to HAIL
					for (Ship = CounterpartTeam->mGetFirstShip(); Ship != NULL; Ship = CounterpartTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Imperial Team comm buttons back to HAIL
					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyOnCapture:
			// LNF
			// If the Counterpart is captured AND the Alien race is contacted
			// then the player wins.
			fMission.mDisplayMessage (kVesselBoarded_msg, kMissionScriptMessage);

			if (Ship->mGetTeam() == kImperialTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				ImperialDestroyed++;
				if (ImperialTeam->mGetShipsInPlayCount() == 0)
				{
					Mode = kAllImperialDead;
					mCreateMissionShips(kAllImperialDead);
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kOrionBaseTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				OrionBaseDestroyed++;
				if (OrionBaseTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kCounterpartTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				CounterpartDestroyed++;
				CounterpartCaptured++;
				Ship->mClearCommands();
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetShipThrust(0);
				// LNF play a message about tractoring ship?

				if ( TractoringCounterpartBeforeCapture == true )
				{
					TractoringCounterpart = true;
					TractoringCounterpartBeforeCapture = false;
				}

				if (CounterpartTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if ( Ship->mGetTeam() == kOrionTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kImperialTeam)
			{
				ImperialDestroyed++;
				if (ImperialTeam->mGetShipsInPlayCount() == 0)
				{
					Mode = kAllImperialDead;
					mCreateMissionShips(kAllImperialDead);
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionBaseTeam)
			{
				OrionBaseDestroyed++;
				if (OrionBaseTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kCounterpartTeam)
			{
				CounterpartDestroyed++;
				CounterpartBlownUp = 1;
				if (CounterpartTeam->mGetShipsInPlayCount() == 0)
				{
					if ( AtWarWithNeutral == false )
					{
						if ( TractoringCounterpart == false )
						{
							fMission.mSetTeamRelations (kNeutralTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kNeutralTeam, kTeamHostile, kGoalDestroy, 1.0);
						}
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								if (ImperialAtWar == 0)
								{
									fMission.mDisplayMessage (kImperialNotAtWar1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kImperialNotAtWar2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kImperialNotAtWar3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kImperialNotAtWar4_msg, kMissionScriptMessage);
									ImperialAtWar = 1;
									ImperialTeam->mClearCommands();
									fMission.mSetTeamRelations (kImperialTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								else // at war
								{
									// if not RETREATING or DISABLED
									fMission.mDisplayMessage (kImperialOK_msg, kMissionScriptMessage);
									// if RETREATING
									//fMission.mDisplayMessage (kImperialRetreating_msg, kMissionScriptMessage);
									// if DISABLED
									//fMission.mDisplayMessage (kImperialDisabled_msg, kMissionScriptMessage);
									// if BOARDED
									//fMission.mDisplayMessage (kVesselBoarded_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
								
							case kCounterpartTeam:
								if ( AtWarWithCounterpart == false )
								{
									fMission.mDisplayMessage (kCounterpartHail1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kCounterpartHail2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kCounterpartTeam, kTeamAtWar, kGoalCapture, 1.0);
									fMission.mSetTeamRelations (kCounterpartTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);

									AtWarWithCounterpart = true;
								}
								else // Counterpart at war
								{
									// if not Retreating or Disabled
									fMission.mDisplayMessage (kCounterpartOK_msg, kMissionScriptMessage);
									// if RETREATING
									//fMission.mDisplayMessage (kCounterpartRetreating_msg, kMissionScriptMessage);
									// if DISABLED
									//fMission.mDisplayMessage (kCounterpartDisabled_msg, kMissionScriptMessage);
									// if BOARDED
									//fMission.mDisplayMessage (kVesselBoarded_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
								
							case kNeutralTeam:
								switch (NeutralTeam->mGetRace())
								{
									case kKlingon:
										KlingonCommMessages(*this, fEventID, Ship);
										break;
									case kRomulan:
										RomulanCommMessages(*this, fEventID, Ship);
										break;
									case kGorn:
										GornCommMessages(*this, fEventID, Ship);
										break;
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
								
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								if ( Ship->mGetDamagePercent() <= kDamageThreshold )
								{
									fMission.mDisplayMessage (kImperialSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kImperialSurrender_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kCounterpartTeam:
								if ( Ship->mGetDamagePercent() <= kDamageThreshold )
								{
									fMission.mDisplayMessage (kCounterpartSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kCounterpartSurrender_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kNeutralTeam:
								switch (NeutralTeam->mGetRace())
								{
									case kKlingon:
										KlingonCommMessages(*this, fEventID, Ship);
										break;
									case kRomulan:
										RomulanCommMessages(*this, fEventID, Ship);
										break;
									case kGorn:
										GornCommMessages(*this, fEventID, Ship);
										break;
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kImperialTaunt1_msg, kImperialTaunt4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kCounterpartTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kCounterpartTaunt1_msg, kCounterpartTaunt4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kNeutralTeam:
								switch (NeutralTeam->mGetRace())
								{
									case kKlingon:
										KlingonCommMessages(*this, fEventID, Ship);
										break;
									case kRomulan:
										RomulanCommMessages(*this, fEventID, Ship);
										break;
									case kGorn:
										GornCommMessages(*this, fEventID, Ship);
										break;
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	//return kGameInProgress;
	//return fMission.mGetGameStatus();  //checks leave map, destroyed, captured

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF should I subtract points for killing the 'other' race
	// LNF maybe add bonus points with the addtobonusprestige thing

	VictoryPoints = (CaptiveTransferred * 80) + (CounterpartBlownUp * 49) +
					(ImperialDestroyed * 10);
	
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (VictoryLevel == kAstoundingVictory)
		{		
				fMission.mAddToDebriefingList (kVictory2_msg);
				fMission.mAddToDebriefingList (kVictory3_msg);
		}
		else if ( VictoryLevel >= kDefeat )
		{
			fMission.mAddToDebriefingList( kExpelled_msg );
			fMission.mAddToDebriefingList (kDismissedShort_msg);
			// LNF try this out:  adding results of primary objectives
			fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

			( CaptiveTransferred != 0 ) ? fMission.mAddToDebriefingList( kCaptiveTransferredTrue_msg ) : SuccessCount++;
			//( OrionBaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOrionBaseDestroyedTrue_msg ) : SuccessCount++;
			//( CounterpartDestroyed != 0 ) ? fMission.mAddToDebriefingList( kCounterpartDestroyedTrue_msg ) : SuccessCount++;
			( ImperialDestroyed != 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedTrue_msg ) : SuccessCount++;
			if ( SuccessCount == 2 )
			{
				fMission.mAddToDebriefingList( kNone_msg );
			}

			fMission.mAddToDebriefingList( kFailedObjectives_msg );

			( CaptiveTransferred == 0 ) ? fMission.mAddToDebriefingList( kCaptiveTransferredFalse_msg ) : FailureCount++;
			//( OrionBaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOrionBaseDestroyedFalse_msg ) : FailureCount++;
			//( CounterpartDestroyed == 0 ) ? fMission.mAddToDebriefingList( kCounterpartDestroyedFalse_msg ) : FailureCount++;
			( ImperialDestroyed == 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedFalse_msg ) : FailureCount++;
			if ( FailureCount == 2 )
			{
				fMission.mAddToDebriefingList( kNone_msg );
			}
			return;
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( CaptiveTransferred != 0 ) ? fMission.mAddToDebriefingList( kCaptiveTransferredTrue_msg ) : SuccessCount++;
	//( OrionBaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOrionBaseDestroyedTrue_msg ) : SuccessCount++;
	//( CounterpartDestroyed != 0 ) ? fMission.mAddToDebriefingList( kCounterpartDestroyedTrue_msg ) : SuccessCount++;
	( ImperialDestroyed != 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( CaptiveTransferred == 0 ) ? fMission.mAddToDebriefingList( kCaptiveTransferredFalse_msg ) : FailureCount++;
	//( OrionBaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOrionBaseDestroyedFalse_msg ) : FailureCount++;
	//( CounterpartDestroyed == 0 ) ? fMission.mAddToDebriefingList( kCounterpartDestroyedFalse_msg ) : FailureCount++;
	( ImperialDestroyed == 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Klingon
void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ImperialTeam = tScript.fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* CounterpartTeam = tScript.fMission.mGetTeamHandle (kCounterpartTeam);
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* NeutralTeam = tScript.fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* OrionBaseTeam = tScript.fMission.mGetTeamHandle (kOrionBaseTeam);

	if ( fEventID == kButtonHail_text )
	{
		if ( AtWarWithNeutral == false )
		{
			if ( TractoringCounterpart == true )
			{
				tScript.fMission.mDisplayMessage( kKlingonNeGoodTowing_msg, kMissionScriptMessage );
				CaptiveTransferred = 1;
				tScript.fMission.mSetTimedInterval( kTimerLimitSet_2, 0 );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kKlingonNeGoodNotTowing_msg, kMissionScriptMessage );
			}
		}
		else
		{
			tScript.fMission.mDisplayMessage( kKlingonOK_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage(kKlingonSurrenderNotEnemy_msg, kMissionScriptMessage);
			tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			AtWarWithNeutral = true;
		}
		else if ( AtWarWithNeutral == true )
		{
			if ( Ship->mGetDamagePercent() <= kDamageThreshold )
			{
				tScript.fMission.mDisplayMessage(kKlingonSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kKlingonSurrender_msg, kMissionScriptMessage);
			}
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kKlingonNePoorNotTowing1_msg, kMissionScriptMessage );
			tScript.fMission.mDisplayMessage( kKlingonNePoorNotTowing2_msg, kMissionScriptMessage );
			tScript.fMission.mDisplayMessage( kKlingonNePoorNotTowing3_msg, kMissionScriptMessage );
			tScript.fMission.mSetTeamRelations ( tScript.kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			AtWarWithNeutral = true;
		}
		else
		{
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKlingonTaunt1_msg, kKlingonTaunt3_msg), kMissionScriptMessage);
		}
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Romulan
void RomulanCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ImperialTeam = tScript.fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* CounterpartTeam = tScript.fMission.mGetTeamHandle (kCounterpartTeam);
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* NeutralTeam = tScript.fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* OrionBaseTeam = tScript.fMission.mGetTeamHandle (kOrionBaseTeam);


	if ( fEventID == kButtonHail_text )
	{
		if ( AtWarWithNeutral == false )
		{
			if ( TractoringCounterpart == true )
			{
				tScript.fMission.mDisplayMessage( kRomulanNeGoodTowing_msg, kMissionScriptMessage );
				CaptiveTransferred = 1;
				tScript.fMission.mSetTimedInterval( kTimerLimitSet_2, 0 );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kRomulanRunningOutOfTime_msg, kMissionScriptMessage );
			}
		}
		else
		{
			tScript.fMission.mDisplayMessage( kRomulanNePoorNotTowing_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kRomulanNeGoodNotTowing_msg, kMissionScriptMessage );
			tScript.fMission.mSetTeamRelations ( tScript.kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			AtWarWithNeutral = true;
		}
		else
		{
			tScript.fMission.mDisplayMessage( kRomulanSurrender_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kRomulanNeGoodNotTowing_msg, kMissionScriptMessage );
			tScript.fMission.mSetTeamRelations ( tScript.kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			AtWarWithNeutral = true;
		}
		else
		{
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kRomulanTaunt1_msg, kRomulanTaunt4_msg), kMissionScriptMessage);
		}
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Gorn
void GornCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ImperialTeam = tScript.fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* CounterpartTeam = tScript.fMission.mGetTeamHandle (kCounterpartTeam);
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* NeutralTeam = tScript.fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* OrionBaseTeam = tScript.fMission.mGetTeamHandle (kOrionBaseTeam);

	if ( fEventID == kButtonHail_text )
	{
		if ( AtWarWithNeutral == false )
		{
			if ( TractoringCounterpart == true )
			{
				tScript.fMission.mDisplayMessage( kGornNeGoodTowing_msg, kMissionScriptMessage );
				CaptiveTransferred = 1;
				tScript.fMission.mSetTimedInterval( kTimerLimitSet_2, 0 );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kGornNeTaunt_msg, kMissionScriptMessage );
			}
		}
		else
		{
			tScript.fMission.mDisplayMessage( kGornOK_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kGornNePoor_msg, kMissionScriptMessage );
			tScript.fMission.mSetTeamRelations ( tScript.kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			AtWarWithNeutral = true;
		}
		else
		{
			tScript.fMission.mDisplayMessage( kGornSurrender_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kGornNePoor_msg, kMissionScriptMessage );
			tScript.fMission.mSetTeamRelations ( tScript.kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			AtWarWithNeutral = true;
		}
		else
		{
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kGornTaunt1_msg, kGornTaunt4_msg), kMissionScriptMessage);
		}
	}
}

//---------------------------------------------------------------------------------------------
// Imperial Patrol routes
void ImperialPatrol(tTeamInfo* PatrolTeam)
{
	//tShipInfo* Ship;

	tMapPoint waypoints_1[8]=
	{
		8, 15,
		10, 10,
		15, 8,
		20, 10,
		22, 15,
		19, 21,
		15, 23,
		10, 21
	};

	tMapPoint waypoints_2[8]=
	{
		23, 15,
		20, 19,
		15, 20,
		12, 17,
		11, 13,
		14, 10,
		18, 9,
		21, 11
	};

	tMapPoint PatrolPoints[2][8];

	PatrolPoints[0][0] = waypoints_1[0];
	PatrolPoints[0][1] = waypoints_1[1];
	PatrolPoints[0][2] = waypoints_1[2];
	PatrolPoints[0][3] = waypoints_1[3];
	PatrolPoints[0][4] = waypoints_1[4];
	PatrolPoints[0][5] = waypoints_1[5];
	PatrolPoints[0][6] = waypoints_1[6];
	PatrolPoints[0][7] = waypoints_1[7];
	PatrolTeam->mGetThisShip(1)->mPatrol (kPatrolAttackRadius, 8, &PatrolPoints[0][0], kHighPriority);

	PatrolPoints[1][0] = waypoints_2[0];
	PatrolPoints[1][1] = waypoints_2[1];
	PatrolPoints[1][2] = waypoints_2[2];
	PatrolPoints[1][3] = waypoints_2[3];
	PatrolPoints[1][4] = waypoints_2[4];
	PatrolPoints[1][5] = waypoints_2[5];
	PatrolPoints[1][6] = waypoints_2[6];
	PatrolPoints[1][7] = waypoints_2[7];
	PatrolTeam->mGetThisShip(2)->mPatrol (kPatrolAttackRadius, 8, &PatrolPoints[1][0], kHighPriority);
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else// if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
