#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Fed_5MissionText.h"
#include "Fed_5MissionMaps.h"

/*********************************************

LNF is it possible to use a variable in here to display in the text messages?
LNF need to put in team relations at start of mission

*********************************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 ImperialDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 OrionDestroyed = 0;
int32 PirateBattleOver = 0;
int32 PirateBattleStart = 0;
int32 JigsUp = 0;
int32 OrionsDisengaged = 0;
int32 StolenData = 0;
int32 Mode = 0;
int32 RetrieveData = 0;

tShipInfo* PlayersBiggestShip = NULL;

bool LpWarning = false;
bool InfiltrationDetected = false;
bool OrionEngagePlayer = false;
bool RetrieveInfiltrationTeam = false;
bool ImperialReinforcementsLaunch = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool RevealedBase = false;
bool InfiltrationTeamFailed = false; // the infiltration team is detected onboard the base ( player's sec officer < base sec officer )

const float kDistanceFromBase = 50.0;
const float kDistanceFromBaseOrionsAttackingWarning = 30.0;
const float kDistanceFromBaseOrionsAttacking = 10.0;
const float kDistanceFromOrions = 60.0;
const float kListeningPostWarning = 45.0;
const float kListeningPostJigsUp = 25.0;
const int32 kBaseX = 13;
const int32 kBaseY = 8;
const int32 kNumberOfOrions = 3;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kCountdownTimer = 2;
enum eMissionMode
{
	kStart = 0,
	kPCInOrbit,
	kImperialReinforcements
};

void ImperialPatrol(tTeamInfo* OrbitTeam);
const float PatrolAttackRadius = 1.0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kImperialTeam,
	kOrionTeam,
	kOrionBaseTeam,
	kListeningPostTeam,
	kReinforcementTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	//info.fEnemyRace = info.fPlayerRace;
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 33) ? kKlingon
					  : (fMission.mRandomInt32 (1, 100) < 66) ? kRomulan : kGorn;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kImperialTeam, kMirror);
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	fMission.mSetTeamRace (kOrionBaseTeam, kOrion);
	fMission.mSetTeamRace (kListeningPostTeam, kFederation);
	fMission.mSetTeamRace (kReinforcementTeam, kMirror);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kListeningPostTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kImperialTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperialTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperialTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionBaseTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* ListeningPostTeam = fMission.mGetTeamHandle (kListeningPostTeam);
	tTeamInfo* ReinforcementTeam = fMission.mGetTeamHandle (kReinforcementTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
			PlayersBiggestShip = Ship;
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if (Mode == kStart)
	{
		// create Imperial team
		ImperialTeam->mCreateFleet (2, (4 * (TotalBPV/NumberOfPlayersShips)), kClassHeavyCruiser, kClassHeavyBattlecruiser, 100, kStartPosition_W);

		// create Orion base
		OrionBaseTeam->mCreateShip (kMaxBPV, kClassStarBase, kClassStarBase, 100, kStartPosition_V);

		// create Imperial listening posts @ T and U
		// LNF should these be federation posts?
		ListeningPostTeam->mCreateShip ("O-LP", kStartPosition_T);
		ListeningPostTeam->mCreateShip ("O-LP", kStartPosition_U);
	}
	else if (Mode == kPCInOrbit)
	{
		// create Orion team
		// LNF
		//fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		ImperialTeam->mClearCommands();
		fMission.mSetTeamRelations( kOrionTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		fMission.mSetTeamRelations( kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
		fMission.mSetTeamRelations( kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0 );
		fMission.mSetTeamRelations( kOrionTeam, kOrionBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		OrionTeam->mCreateFleet( kNumberOfOrions, (5 * (TotalBPV/NumberOfPlayersShips)), kClassHeavyCruiser, kClassHeavyBattlecruiser, 100, kStartPosition_O );
	}
	else if ( Mode == kImperialReinforcements )
	{
		fMission.mSetTeamRelations( kReinforcementTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
		fMission.mSetTeamRelations( kReinforcementTeam, kImperialTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
		fMission.mSetTeamRelations( kReinforcementTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
		fMission.mSetTeamRelations( kReinforcementTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
		fMission.mSetTeamRelations( kPlayerTeam, kReinforcementTeam, kTeamHostile, kGoalDestroy, 1.0 );
		ReinforcementTeam->mCreateShip("F-BCJ", kStartPosition_Specified, kBaseX, kBaseY );
		ReinforcementTeam->mCreateShip("F-CB", kStartPosition_Specified, kBaseX, kBaseY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* ListeningPostTeam = fMission.mGetTeamHandle (kListeningPostTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* ReinforcementTeam = fMission.mGetTeamHandle (kReinforcementTeam);
	tShipInfo* Ship = NULL;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mTrackTacticalDistance(kDistanceFromBase, kBaseX, kBaseY); // Transporter range to Orion Comm Post
		Ship->mSetShipThrust (5);
		Ship->mSetShipCallbackEvents (kNotifyTransportedItemTo);
		// LNF change kTransInfiltrator to Infiltrator Team
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransInfiltrator, 1 );
	}

	// configure Imperial ships
	for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		ImperialPatrol( ImperialTeam );
	}

	// configure Listening Post Team
	for (Ship = ListeningPostTeam->mGetFirstShip(); Ship != NULL; Ship = ListeningPostTeam->mGetNextShip())
	{
		//Ship->mSetCommButton ( 1, kButtonHail_text );
	}

	// configure Orion Base
	Ship = OrionBaseTeam->mGetThisShip(1);
	Ship->mSetCommButton ( 1, kButtonHail_text );
	Ship->mSetShipCallbackEvents(kNotifyDamage);
	Ship->mDamageSystem(kDamageMaxShields, 100);
	Ship->mSetShipCallbackEvents (kNotifyTransportedItemTo);

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to set team relations
	fMission.mSetTimedInterval( kTimerLimitSet_4, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// configure Orion ships
	if ( Mode != kImperialReinforcements )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* ListeningPostTeam = fMission.mGetTeamHandle (kListeningPostTeam);
	tTeamInfo* ReinforcementTeam = fMission.mGetTeamHandle (kReinforcementTeam);
	tShipInfo* OrionShip;

	OrionShip = OrionTeam->mGetFirstShip();

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);
	
	// int32 CountdownTimer = 2;

	switch (Event)
	{
		case kNotifyTransportedItemTo:	
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);
				// LNF doesn't work for Transporter, but it works for Capture
				//if ((PlayersTarget == OrionBaseTeam->mGetFirstShip()) && (OrionsDisengaged == 1))
				if (( TransportTarget != NULL ) && ( TransportTarget == OrionBaseTeam->mGetFirstShip() ))
				{
					// LNF change kTransInfiltrator to InfiltratorTeam
					if ( fEventItem1 == kTransInfiltrator )
					{
						// timer to take time to get data
						fMission.mDisplayMessage(kEvent5BeamInfiltrationTeam_msg, kMissionScriptMessage);
						//fMission.mSetTimedInterval (kTimerTurnsSet, CountdownTimer);

						if ( StolenData == 0 )
						{
							fMission.mSetTimedInterval (kTimerCountdownSet, kCountdownTimer);
						}
						/*
						if ((fPlayerShip->mGetOfficerRank(kSecurityOfficer)) >= (OrionBaseTeam->mGetFirstShip()->mGetOfficerRank(kSecurityOfficer)))
						{
							fMission.mDisplayMessage(kEvent5BeamInfiltrationTeam_msg, kMissionScriptMessage);
							// LNF set a timer for the player to leave the area before Imperials attack?
							CountdownTimer = (fPlayerShip->mGetOfficerRank(kSecurityOfficer)) * 5;
							fMission.mSetTimedInterval (kTimerTurnsSet, CountdownTimer);
						}
						*/
						Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
					}
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				// retrieve infiltration team
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				if ( TransportTarget != NULL )
				{
					if ( TransportTarget == OrionBaseTeam->mGetFirstShip() )
					{
						// if the data has already been stolen (the countdown timer has expired)
						if ( StolenData == 1 )
						{
							if ( fEventItem1 == kTransInfiltrator )
							{
								fMission.mDisplayMessage(kEvent5RetrieveInfiltrationTeam_msg, kMissionScriptMessage);
								RetrieveInfiltrationTeam = true;
								RetrieveData = 1;
								fMission.mDisplayMessage( kMissionComplete_msg, kMissionScriptMessage );
								FindClosestMapEdge( *this, fPlayerShip, kMapEdgeX, kMapEdgeY );
							}
						}
						else
						{
							if ( fEventItem1 == kTransInfiltrator )
							{
								if ( InfiltrationTeamFailed == true )
								{
									fMission.mDisplayMessage( kEvent5_RetrieveAfterFail_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage(kEvent5_RetrieveTooEarly_msg, kMissionScriptMessage);
								}
								RetrieveInfiltrationTeam = false;
								fMission.mSetTimedInterval( kTimerCountdownSet, 0 );
							}
						}
					}
				}
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
			}
			break;
	
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if (( Ship->mGetTeam() == kOrionBaseTeam ) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						// NotifyDamage currently only for Orion Base
						fMission.mDisplayMessage(kEvent1_JigsUp_msg, kScienceScriptMessage);
						JigsUp = 1;
						fMission.mSetTeamRelations (kImperialTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					}
					else if (( HostileShip->mGetTeam() == kPlayerTeam ) && ( Ship->mGetTeam() == kOrionTeam ))
					{
						if ( OrionEngagePlayer == false )
						{
							OrionEngagePlayer = true;
							Ship->mSetShipTarget( fPlayerShip, kHighPriority );
						}
					}
				}
			}
			break;

		case kNotifyAtDistance:
			if (( Ship == fPlayerShip ) && ( Ship->mGetTrackingDistance() == kDistanceFromBase )) // Notify when player nears Orion Base
			{
				if (PirateBattleStart == 0)
				{
					fMission.mDisplayMessage (kEvent2A_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent2B_msg, kMissionScriptMessage);
					Mode = kPCInOrbit;
					mCreateMissionShips (kPCInOrbit);
					PirateBattleStart = 1;
					// timer to turn on goal indicator
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					// timer to turn off goal indicator when in range of orions
					fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					// check distance closer to base.
					Ship->mTrackTacticalDistance(kDistanceFromBaseOrionsAttackingWarning, kBaseX, kBaseY);
				}
			}
			else if (( Ship == fPlayerShip ) &&
					( Ship->mGetTrackingDistance() == kDistanceFromBaseOrionsAttackingWarning ))
			{
				if ( PirateBattleStart == 1 )
				{
					fMission.mDisplayMessage( kBaseWarning_msg, kMissionScriptMessage );
					OrionBaseTeam->mGetFirstShip()->mDamageSystem(kDamageMaxShields, 0);
				}
				Ship->mTrackTacticalDistance(kDistanceFromBaseOrionsAttacking, kBaseX, kBaseY);
			}
			else if (( Ship == fPlayerShip ) &&
					( Ship->mGetTrackingDistance() == kDistanceFromBaseOrionsAttacking ))
			{
				if (( PirateBattleStart == 1 ) && ( ImperialReinforcementsLaunch == false ))
				{
					ImperialReinforcementsLaunch = true;
					Mode = kImperialReinforcements;
					mCreateMissionShips( kImperialReinforcements );
					fMission.mDisplayMessage( kBaseAttacksPlayer_msg, kMissionScriptMessage );
					fMission.mSetTeamRelations (kOrionBaseTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamHostile, kGoalDestroy, 1.0);
					OrionBaseTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 0 );
				}
			}
			break;
	

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// check distance from listening posts
					{
						// if the person is leaving (already has data) then I don't need this part
						if ( InfiltrationDetected == false )
						{
							for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
							{
								if ((( Ship->mGetTacticalDistance(ListeningPostTeam->mGetThisShip(1)) < kListeningPostJigsUp ) ||
									( Ship->mGetTacticalDistance(ListeningPostTeam->mGetThisShip(2)) < kListeningPostJigsUp )) &&
									( JigsUp == 0 ))
								{
									if ( Ship == fPlayerShip )
									{
										fMission.mDisplayMessage( kEvent1_JigsUp_msg, kScienceScriptMessage );
									}
									else
									{
										fMission.mDisplayMessage( kEvent1Team_msg, kScienceScriptMessage );
									}
									JigsUp = 1;
									fMission.mSetTeamRelations (kImperialTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								else if ((( Ship->mGetTacticalDistance(ListeningPostTeam->mGetThisShip(1)) < kListeningPostWarning ) ||
										( Ship->mGetTacticalDistance(ListeningPostTeam->mGetThisShip(2)) < kListeningPostWarning) ) &&
										( LpWarning == false ))
								{
									if ( Ship == fPlayerShip )
									{
										fMission.mDisplayMessage( kPlayerLpWarning_msg, kScienceScriptMessage );
									}
									else
									{
										fMission.mDisplayMessage( kLpWarning_msg, kScienceScriptMessage );
									}

									if ( RevealedBase == true )
									{
										fMission.mDisplayMessage( kRevealedBase_msg, kScienceScriptMessage );
									}
									LpWarning = true;
								}
							}
							fMission.mSetTimedInterval (kTimerLimitSet, 1);
						} // InfiltrationDetected == false
					}
					break;

				case kTimerLimitSet_2:
					// turn on goal indicator to point to the orion reinforcements
					fMission.mSetGoalIndicator( OrionTeam->mGetThisShip(1) );
					break;

				case kTimerLimitSet_3:
					{
						// turn off goal indicator when the player gets close to the orions
						for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
						{
							if ( fPlayerShip->mGetTacticalDistance(Ship) < kDistanceFromOrions )
							{
								fMission.mSetGoalIndicator( kFalse, 0, 0 );
								break;
							}
							else
							{
								fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
							}
						}
					}
					break;

				case kTimerLimitSet_4:
					// set relations
					fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kListeningPostTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					
					fMission.mSetTeamRelations (kImperialTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kImperialTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kImperialTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionBaseTeam, kListeningPostTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					// set timer to check distance to orion base
					fMission.mSetTimedInterval( kTimerLimitSet_5, 1 );
					break;

				case kTimerLimitSet_5:
					// check distance from Orion starbase
					{
						for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
						{
							if (( Ship->mGetTacticalDistance( OrionBaseTeam->mGetFirstShip() ) < 100.0 ) &&
								( RevealedBase == false ))
							{
								RevealedBase = true;
								break;
							}
						}
						fMission.mSetTimedInterval (kTimerLimitSet_5, 1);
					}
					break;

				case kTimerCountdownSet:
					// LNF should I randomize the officer rank aboard the Orion base?  that would be neat
					if (((fPlayerShip->mGetOfficerRank( kSecurityOfficer )) >= (OrionBaseTeam->mGetFirstShip()->mGetOfficerRank( kSecurityOfficer )))
						&& (( InfiltrationDetected == false ) || ( StolenData == 0 )))
					{
						fMission.mDisplayMessage(kEvent5InfiltrationTeamSuccess_msg, kMissionScriptMessage);
						// LNF set a timer for the player to leave the area before Imperials attack?
						//CountdownTimer = (fPlayerShip->mGetOfficerRank(kSecurityOfficer)) * 5;
						//CountdownTimer = 2;
						//fMission.mSetTimedInterval (kTimerTurnsSet, kCountdownTimer);

						// set timer that alerts orions that the data was stolen
						fMission.mSetTimedInterval (kTimerTurnsSet, (fPlayerShip->mGetOfficerRank( kSecurityOfficer )));
						StolenData = 1;
					}
					else
					{
						// LNF figure out security officer rank, and use that for the timer
						fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						fMission.mDisplayMessage( kInfiltrationFailedSomethingsWrong_msg, kMissionScriptMessage );
						InfiltrationTeamFailed = true;

						//fMission.mDisplayMessage(kEvent6InfiltrationDetected_msg, kMissionScriptMessage);
						//fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						//fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						InfiltrationDetected = true;
					}
					break;

				case kTimerTurnsSet:
					// alerts orions that data was stolen

					fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

					// raies the base's shields
					OrionBaseTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 0 );

					// LNF should I send out reinforcements if not already sent out?
					if ( InfiltrationTeamFailed == true )
					{
						fMission.mDisplayMessage( kEvent6InfiltrationTeamFailed_msg, kMissionScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage(kEvent6InfiltrationDetected_msg, kMissionScriptMessage);
					}
					InfiltrationDetected = true;
					break;

				case kTimerTurnsSet_8:
					// set the Orion Team comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Orion Base Team comm buttons back to HAIL
					for (Ship = OrionBaseTeam->mGetFirstShip(); Ship != NULL; Ship = OrionBaseTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Imperial Team comm buttons back to HAIL
					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kImperialTeam)
			{
				ImperialDestroyed++;
				// LNF need to find out who kills the imperial ship
				if ( ImperialDestroyed == 1 )
				{
					fMission.mDisplayMessage( kEvent9_ImperialShipDestroyed_msg, kScienceScriptMessage );
				}

				if ((ImperialTeam->mGetShipsInPlayCount() == 0) && (OrionBaseTeam->mGetShipsInPlayCount() > 0))
				{
					fMission.mDisplayMessage(kEvent4_msg, kMissionScriptMessage);
					// LNF set a unique timer here and then return kPlayerLoses
					return kPlayerLoses;
				}

				if (ImperialTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					//return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kOrionTeam)
			{
				OrionDestroyed++;
				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
					if ( JigsUp == 0 )
					{
						// player has killed orions and not revealed his intentions
						fMission.mDisplayMessage( kOrionsDestroyed_msg, kMissionScriptMessage );
						// take down the base's shields
						OrionBaseTeam->mGetFirstShip()->mDamageSystem(kDamageMaxShields, 100);
					}
					else
					{
						fMission.mDisplayMessage( kOrionsDestroyedJigsUp_msg, kMissionScriptMessage );
					}
					PirateBattleStart = 0;
					ImperialTeam->mClearCommands();
					ImperialTeam->mDefendEntity( OrionBaseTeam->mGetFirstShip(), 50.0, 50.0, kVeryHighPriority );
				}
			}
			else if (Ship->mGetTeam() == kOrionBaseTeam)
			{
				if ( StolenData == 0 )
				{
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship->mGetTeam() == kOrionTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
				break;
			}

			fMission.mDisplayMessage( kCapture_msg, kMissionScriptMessage );

			break;

		case kNotifyDisengaged:
			fMission.mSetGoalIndicator( kFalse, 0, 0 );
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								if (PirateBattleStart == 1)
								{
									fMission.mDisplayMessage (kImperialHailIc_msg, kMissionScriptMessage);
								}
								else if ((PirateBattleStart == 0) && (JigsUp == 0))
								{
									fMission.mDisplayMessage (kImperialHailNic1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kImperialHailNic2_msg, kMissionScriptMessage);
								}

								else if (JigsUp == 1)
								{
									fMission.mDisplayMessage (kImperialHailJigT_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionBaseTeam:
								if ((PirateBattleStart == 0) && (JigsUp == 0))
								{
									fMission.mDisplayMessage (kStationNicJigF_msg, kMissionScriptMessage);
									OrionBaseTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 100 );
								}
								else if (JigsUp == 1)
								{
									fMission.mDisplayMessage (kStationJigT_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage (kOrionOK_msg, kMissionScriptMessage);
								// if retreating, disabled, boarded
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								// LNF need to add text?
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionBaseTeam:
								if (JigsUp == 1)
								{
									fMission.mDisplayMessage (kStationJigT_msg, kMissionScriptMessage);
								}
								else if (JigsUp == 0)
								{
									fMission.mDisplayMessage (kStationJigF_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage (kOrionNoDamage_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								// LNF need to add text?
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionBaseTeam:
								if (JigsUp == 1)
								{
									fMission.mDisplayMessage (kStationJigT_msg, kMissionScriptMessage);
								}
								else if (JigsUp == 0)
								{
									fMission.mDisplayMessage (kStationJigF_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt1_msg, kOrionTaunt3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	//return kGameInProgress;
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (RetrieveData * 100);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints == 0)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}
	
	// remove infiltration team from the player's ship
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransInfiltrator, 0 );
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{	
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( RetrieveData != 0 ) ? fMission.mAddToDebriefingList( kRetrieveDataTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( RetrieveData == 0 ) ? fMission.mAddToDebriefingList( kRetrieveDataFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// Imperial Patrol routes
void ImperialPatrol(tTeamInfo* PatrolTeam)
{
	//tShipInfo* Ship;

	tMapPoint waypoints_1[7]= // ship W
	{
		8, 8,
		10, 11,
		13, 12,
		17, 10,
		18, 6,
		14, 4,
		10, 5
	};

	tMapPoint waypoints_2[7]= // ship X
	{
		13, 12,
		17, 10,
		18, 6,
		14, 4,
		10, 5,
		8, 8,
		10, 11
	};

	tMapPoint waypoints_3[9]= // ship Y
	{
		20, 8,
		19, 4,
		15, 3,
		10, 4,
		7, 6,
		8, 11,
		11, 14,
		16, 14,
		19, 11
	};

	tMapPoint waypoints_4[9]= // ship Z
	{
		13, 3,
		9, 5,
		7, 9,
		9, 13,
		13, 14,
		17, 13,
		19, 9,
		19, 6,
		16, 4
	};

	tMapPoint PatrolPoints[4][10];

	PatrolPoints[0][0] = waypoints_1[0];
	PatrolPoints[0][1] = waypoints_1[1];
	PatrolPoints[0][2] = waypoints_1[2];
	PatrolPoints[0][3] = waypoints_1[3];
	PatrolPoints[0][4] = waypoints_1[4];
	PatrolPoints[0][5] = waypoints_1[5];
	PatrolPoints[0][6] = waypoints_1[6];
	PatrolTeam->mGetThisShip(1)->mPatrol( PatrolAttackRadius, 7, &PatrolPoints[0][0], kHighPriority );
/*
	PatrolPoints[1][0] = waypoints_2[0];
	PatrolPoints[1][1] = waypoints_2[1];
	PatrolPoints[1][2] = waypoints_2[2];
	PatrolPoints[1][3] = waypoints_2[3];
	PatrolPoints[1][4] = waypoints_2[4];
	PatrolPoints[1][5] = waypoints_2[5];
	PatrolPoints[1][6] = waypoints_2[6];
	PatrolTeam->mGetThisShip(2)->mPatrol( PatrolAttackRadius, 7, &PatrolPoints[1][0], kHighPriority );

	PatrolPoints[2][0] = waypoints_3[0];
	PatrolPoints[2][1] = waypoints_3[1];
	PatrolPoints[2][2] = waypoints_3[2];
	PatrolPoints[2][3] = waypoints_3[3];
	PatrolPoints[2][4] = waypoints_3[4];
	PatrolPoints[2][5] = waypoints_3[5];
	PatrolPoints[2][6] = waypoints_3[6];
	PatrolPoints[2][7] = waypoints_3[7];
	PatrolPoints[2][8] = waypoints_3[8];
	PatrolTeam->mGetThisShip(3)->mPatrol( PatrolAttackRadius, 9, &PatrolPoints[2][0], kHighPriority );

	PatrolPoints[3][0] = waypoints_4[0];
	PatrolPoints[3][1] = waypoints_4[1];
	PatrolPoints[3][2] = waypoints_4[2];
	PatrolPoints[3][3] = waypoints_4[3];
	PatrolPoints[3][4] = waypoints_4[4];
	PatrolPoints[3][5] = waypoints_4[5];
	PatrolPoints[3][6] = waypoints_4[6];
	PatrolPoints[3][7] = waypoints_4[7];
	PatrolPoints[3][8] = waypoints_4[8];
	PatrolTeam->mGetThisShip(4)->mPatrol( PatrolAttackRadius, 9, &PatrolPoints[3][0], kHighPriority );
*/
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
