#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Fed_7MissionText.h"
#include "Fed_7MissionMaps.h"
/**********************
LNF wormhole warning
LNF check out if defend command works, make the 6 Imperial ships defend individual
LNF planetkillers, with absolute priority so they can be picked off one by one.
**********************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 ImperialDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 PkDestroyed = 0;
int32 StartMission = 0;

bool Suggestion = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

const float kDeepScanDistancePk = 20.0;
const int32 kWormholeX = 6;
const int32 kWormholeY = 25;
const float kWormholeWarningDistance = 30.0;
const float kWormholeDistance = 5.0;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
eVictoryLevels MyVicLevel = kDevastatingDefeat;

void CheckWinCondition( tScript& tScript );

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kImperialTeam,
	kPkTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kImperialTeam, kMirror);

	// LNF: The Neutral team can vary....might not be Klingon
	fMission.mSetTeamRace (kPkTeam, kMonster);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kImperialTeam, kPkTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* PkTeam = fMission.mGetTeamHandle (kPkTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create dead Planetkillers 
	PkTeam->mCreateShip("M-DML", kStartPosition_Specified, 30, 10, 0, 10); // pk1  W
	PkTeam->mCreateShip("M-DMS", kStartPosition_Specified, 19, 11, 22, 40); // pk2	X
	PkTeam->mCreateShip("M-DMM", kStartPosition_Specified, 9, 14, 1, 10);	// pk3	Y
	PkTeam->mCreateShip("M-DMM", kStartPosition_Specified, 31, 17); // pk4	U, V
	PkTeam->mCreateShip("M-DMS", kStartPosition_Specified, 21, 20);	// pk5
	PkTeam->mCreateShip("M-DMS", kStartPosition_Specified, 31, 34);	// pk6
	PkTeam->mCreateShip("M-DML", kStartPosition_Specified, 6, 33);	// pk7
	PkTeam->mCreateShip("M-DMM", kStartPosition_Specified, 3, 31);	// pk8

	// create Imperial team
	ImperialTeam->mCreateShip (AveragePlayerBPV, kClassFrigate, kClassHeavyCruiser, 40, kStartPosition_T);
	ImperialTeam->mCreateShip (AveragePlayerBPV, kClassHeavyCruiser, kClassHeavyCruiser, 40, kStartPosition_U);
	ImperialTeam->mCreateFleet (4, (TotalBPV * 2), kClassFrigate, kClassHeavyCruiser, 40, kStartPosition_V);

	// create Live Planetkiller
	PkTeam->mCreateShip ("M-DML", kStartPosition_Z); // pk9  Z
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* PkTeam = fMission.mGetTeamHandle (kPkTeam);
	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mTrackTacticalDistance( kWormholeWarningDistance, kWormholeX, kWormholeY );
	}
	// configure Imperial ships
	for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// tell each imperial ship to defend their respective planet killer
	ImperialTeam->mGetThisShip(1)->mDefendEntity( PkTeam->mGetThisShip(9), 20.0, 20.0, kVeryHighPriority ); // T
	ImperialTeam->mGetThisShip(2)->mDefendEntity( PkTeam->mGetThisShip(4), 20.0, 20.0, kVeryHighPriority ); // U
	ImperialTeam->mGetThisShip(3)->mDefendEntity( PkTeam->mGetThisShip(4), 20.0, 20.0, kVeryHighPriority ); // V
	ImperialTeam->mGetThisShip(4)->mDefendEntity( PkTeam->mGetThisShip(1), 20.0, 20.0, kVeryHighPriority ); // W
	ImperialTeam->mGetThisShip(5)->mDefendEntity( PkTeam->mGetThisShip(2), 20.0, 20.0, kVeryHighPriority ); // X
	ImperialTeam->mGetThisShip(6)->mDefendEntity( PkTeam->mGetThisShip(3), 20.0, 20.0, kVeryHighPriority ); // Y

	// configure Dead Planetkillers
	for (Ship = PkTeam->mGetFirstShip(); Ship != NULL; Ship = PkTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetShipAIControlled( kFalse );
	}

	// configure Live Planetkiller

	
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 5);

	// timer to check 'weapons' range to planet killer
	fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );

	
	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan XX from within distance 30 for Y turns
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(1), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(2), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(3), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(4), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(5), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(6), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(7), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(8), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (PkTeam->mGetThisShip(9), kDeepScanDistancePk, 1);
	// location deep scan tracker example
	//DeepScanTracker.mSetLocation (20, 8, DeepScanDistance, 3); // track Moon
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* PkTeam = fMission.mGetTeamHandle (kPkTeam);

	tShipInfo* PkShip = NULL;

	switch (Event)
	{
		case kNotifyAtDistance:
			// fEventID True - inside, False - outside?
			if ( Ship->mGetTrackingDistance() == kWormholeWarningDistance )	
			{
				fMission.mDisplayMessage( kWormholeWarning_msg, kMissionScriptMessage );
				Ship->mTrackTacticalDistance( kWormholeDistance, kWormholeX, kWormholeY );
			}
			else if ( Ship->mGetTrackingDistance() == kWormholeDistance )	
			{
				fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
				return kPlayerDisengaged;
			}
			break;

		case kNotifyDamage:
			if ( Ship->mGetTeam() == kPkTeam )
			{
				Ship->mDamageSystem( kDamageWeaponsAndSystems, 0 );
			}
			Ship->mSetShipCallbackEvents( kNotifyDamage );
			break;
	

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{

				case kTimerLimitSet:
					if (StartMission == 0)
					{
						fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0); //***

						fMission.mSetTeamRelations (kImperialTeam, kPkTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0); //***

						fMission.mDisplayMessage (kEvent1A_msg, kMissionScriptMessage);
						fMission.mDisplayMessage (kEvent1B_msg, kMissionScriptMessage);
						fMission.mDisplayMessage (kEvent1C_msg, kMissionScriptMessage);
						StartMission = 1;
					}
					break;

				case kTimerLimitSet_2:

					// death-distance-check-timer for planet killer
					for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
					{
						if ( PkTeam->mGetThisShip(9)->mGetTacticalDistance(Ship) <= 0.0 )
						{
							if ( Ship->mGetClassType() >= kClassHeavyCruiser )
							{
								Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
								PkTeam->mGetThisShip(9)->mDamageSystem( kDamageExcessDamageSystem, 100 );
								Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
								PkTeam->mGetThisShip(9)->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
								break;
							}
							else
							{
								Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
								Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							}
						}
					}
					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						if ( PkTeam->mGetThisShip(9)->mGetTacticalDistance(Ship) <= 0.0 )
						{
							if ( Ship->mGetClassType() >= kClassHeavyCruiser )
							{
								Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
								PkTeam->mGetThisShip(9)->mDamageSystem( kDamageExcessDamageSystem, 100 );
								Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
								PkTeam->mGetThisShip(9)->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
								break;
							}
							else
							{
								Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
								Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							}
						}
					}
					//fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					break;

				case kTimerLimitSet_3:
					// check the distance between player and pk
					// to give the suggestion
					for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
					{
						for (PkShip = PkTeam->mGetFirstShip(); PkShip != NULL; PkShip = PkTeam->mGetNextShip())
						{
							if (( PkShip->mGetTacticalDistance( Ship ) <= 50.0 ) && ( Suggestion == false ))
							{
								fMission.mDisplayMessage( kEvent4_Suggestion_msg, kMissionScriptMessage );
								Suggestion = true;
								break;
							}
						}
					}
					fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					break;

				case kTimerLimitSet_10:
				{
					eDeepScanFlags	flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
					switch (flag)
					{
						case kOutOfRange_ScanOff:
							// player is not within any objects range and deep scan is off (Object == NULL)
							break;
						case kOutOfRange_ScanOn:
							// player is not within any objects range and deep scan is on (Object == NULL)
							break;
						case kEnteringRange_ScanOff:
							// player has just entered range and scan is off (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							// notify player to turn scan on
							fMission.mDisplayMessage (kScanOffInRangePk_msg, kScienceScriptMessage);
							break;
						case kEnteringRange_ScanOn_TimerStarted:
							// player has just entered range and scan is on (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							// notify player that timer has begun
							fMission.mDisplayMessage (kStartingScanPk_msg, kScienceScriptMessage);
							break;
						case kInRange_ScanOff_TimerStopped:
							// player is in range and scan is off (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							// notify player that timer has been turned off
							fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
							break;
						case kInRange_ScanOff:
							// player is still in range and scan is still off (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							break;
						case kInRange_ScanOn_TimerStarted:
							// player is in range and scan is on (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							// notify player that timer has begun
							fMission.mDisplayMessage (kStartingScanPk_msg, kScienceScriptMessage);
							break;
						case kInRange_ScanOn:
							// player is still in range and scan is still on (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							break;
						case kLeavingRange_ScanOn_TimerStopped:
							// player has just left the range and scan is on (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							// notify player that timer is now off
/*								if ( Object == MostRecentlyDestroyed )
							{
								fMission.mDisplayMessage( kScanTargetDestroyed_msg, kMissionScriptMessage );
							}
*/								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
							break;
						case kLeavingRange_ScanOff:
							// player has just left the range and scan is off (Object points to entity)
							// (Object points to entity or if NULL then MapPoint has location)
							break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if ( Object == PkTeam->mGetThisShip(9) )
							{
								fMission.mDisplayMessage( kSuccessfulScanLivePk_msg, kScienceScriptMessage );
								Object->mChangeName( kLivePk_name );

								// turn on Pk AI
								Object->mSetShipAIControlled( kTrue );
								Object->mSetShipMaxSpeed( 6 );
								//fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							else
							{
								Object->mChangeName( kDeadPk_name );
								fMission.mDisplayMessage( kSuccessfulScanDeadPk_msg, kScienceScriptMessage );
							}
						}	
						// (Object points to entity or if NULL then MapPoint has location)
						if (Object)
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
						else
						{
							DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
						}
					}
					break;

				case kTimerTurnsSet_10:
					// set the Imperial Team comm buttons back to HAIL
					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kImperialTeam)
			{
				ImperialDestroyed++;
				if (ImperialTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if ( Ship->mGetTeam() == kPkTeam )
			{
				PkDestroyed++;

				Ship->mChangeName( kDeadPk_name );

				if ( Ship == PkTeam->mGetThisShip(9) )
				{
					CheckWinCondition( *this );
				}
				else if (PkTeam->mGetShipsInPlayCount() == 0)
				{
					
				}
			}
			break;

		case kNotifyOnCapture:
			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (Ship->mGetTeam () == kImperialTeam)
			{
				Ship->mClearCommands();
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetShipThrust(0);
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								fMission.mDisplayMessage (kImpHailOK_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPkTeam:
								fMission.mDisplayMessage (kPkHail_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonHail_text);
								break;
						}
						break;

					case kButtonSurrender_text:
						// only Imperial ships
						if ((Ship->mGetDamagePercent() >= 0.0) && (Ship->mGetDamagePercent() < 50.0))
						{
							fMission.mDisplayMessage (kImpHailOK_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kImpSurrenderDamaged_msg, kMissionScriptMessage);
						}
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						// only Imperial ships
						fMission.mDisplayMessage (kImpTaunt_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0; //LNF

	VictoryPoints = (PkDestroyed * 70) + (ImperialDestroyed * 10);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	MyVicLevel = VictoryLevel;
	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if ((VictoryLevel == kDefeat) || (VictoryLevel == kDevastatingDefeat))
		{
			fMission.mAddToDebriefingList (kDefeat_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( PkDestroyed != 0 ) ? fMission.mAddToDebriefingList( kPkDestroyedTrue_msg ) : SuccessCount++;
	( ImperialDestroyed != 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( PkDestroyed == 0 ) ? fMission.mAddToDebriefingList( kPkDestroyedFalse_msg ) : FailureCount++;
	( ImperialDestroyed == 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_4;
	}
	else
	{
		return 0;
	}
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	if ( PkDestroyed == 1 )
	{
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
	}
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
