#include "scriptinterface.h"

#include "Fed_8MissionText.h"
#include "Fed_8MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Imperial_1_Destroyed;
int32 Imperial_2_Destroyed;
int32 PlayerDestroyed;
int32 Warp = 0;
int32 StarTapOrbiting = 0;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kImperial_1_Team,
	kImperial_2_Team,
	kStarTapTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// set up races
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kImperial_1_Team, kMirror);
	fMission.mSetTeamRace (kImperial_2_Team, kMirror);
	fMission.mSetTeamRace (kStarTapTeam, kFederation);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kImperial_1_Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kImperial_2_Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kStarTapTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kImperial_1_Team, kImperial_2_Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperial_1_Team, kStarTapTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kImperial_2_Team, kStarTapTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* Imperial_1_Team = fMission.mGetTeamHandle (kImperial_1_Team);
	tTeamInfo* Imperial_2_Team = fMission.mGetTeamHandle (kImperial_2_Team);
	tTeamInfo* StarTapTeam = fMission.mGetTeamHandle (kStarTapTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	int32 NewNumberOfEnemyShips = 0;
	int32 NewEnemyBPV = 0;
	if ( NumberOfPlayersShips == 1 )
	{
		NewNumberOfEnemyShips = 1;
		NewEnemyBPV = NewNumberOfEnemyShips * AveragePlayerBPV;
	}
	else if ( NumberOfPlayersShips == 2 )
	{
		NewNumberOfEnemyShips = 2;
		NewEnemyBPV = NewNumberOfEnemyShips * AveragePlayerBPV;
	}
	else
	{
		NewNumberOfEnemyShips = 2;
		NewEnemyBPV = NewNumberOfEnemyShips * ( AveragePlayerBPV * 1.3 );
	}

	// create Imperial_1_Team
	Imperial_1_Team->mCreateFleet (NewNumberOfEnemyShips, NewEnemyBPV, kClassFrigate, LargestEnemyShipClass, 60, kStartPosition_U);

	// create Imperial_2_Team
	Imperial_2_Team->mCreateFleet (NewNumberOfEnemyShips, NewEnemyBPV, kClassFrigate, LargestEnemyShipClass, 60, kStartPosition_X);

	// create StarTap team
	StarTapTeam->mCreateShip (kMaxBPV, kClassFreighter, kClassFreighter, 60, kStartPosition_J);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* Imperial_1_Team = fMission.mGetTeamHandle (kImperial_1_Team);
	tTeamInfo* Imperial_2_Team = fMission.mGetTeamHandle (kImperial_2_Team);
	tTeamInfo* StarTapTeam = fMission.mGetTeamHandle (kStarTapTeam);
	tShipInfo* Ship;
	tShipInfo* StarTapShip;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}

	// configure Imperial_1_ ships
	for (Ship = Imperial_1_Team->mGetFirstShip(); Ship != NULL; Ship = Imperial_1_Team->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// configure Imperial_2_ ships
	for (Ship = Imperial_2_Team->mGetFirstShip(); Ship != NULL; Ship = Imperial_2_Team->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// configure StarTap ship
	StarTapShip = StarTapTeam->mGetFirstShip();
	StarTapShip->mChangeName( kStarTap_name );
	StarTapShip->mSetFinalDestination( 23, 25 );
	// Set tracking distance to the star, when Imperials attack
	StarTapShip->mTrackTacticalDistance(180.0, 24, 26); 
	//StarTapShip->mDamageSystem(kDamageLeftWarpEngine, 100);
	//StarTapShip->mDamageSystem(kDamageRightWarpEngine, 100);
	
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet_2, 2);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* Imperial_1_Team = fMission.mGetTeamHandle (kImperial_1_Team);
	tTeamInfo* Imperial_2_Team = fMission.mGetTeamHandle (kImperial_2_Team);
	tTeamInfo* StarTapTeam = fMission.mGetTeamHandle (kStarTapTeam);

	tShipInfo* target = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyAtDestination:
			// startap gets to destination
			if ( Ship == StarTapTeam->mGetFirstShip() )
			{
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetShipThrust(0);
			}
			break;

		case kNotifyBeginTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
					if ( TractorTarget == StarTapTeam->mGetFirstShip() )
					{
						fMission.mDisplayMessage (kTractoringStarTap_msg, kMissionScriptMessage);
					}
					if ((TractorTarget == StarTapTeam->mGetFirstShip()) && (StarTapOrbiting == 1))
					{
						fMission.mDisplayMessage (kEvent3InterruptStarTap_msg, kMissionScriptMessage);
						StarTapOrbiting = 0;
						Ship->mTrackTacticalDistance(80.0, 24, 26);
					}
					fPlayerShip->mSetShipCallbackEvents(kNotifyBeginTractor);
				}
			}
			break;

		// LNF do I need an EndTractor?

		case kNotifyAtDistance:
			// if the StarTap ship is near the star, Imperials attack
			if (Ship->mGetTrackingDistance() == 180.0)
			{
				//LNF make the Imperial more aggressive toward StarTap??
				fMission.mSetTeamRelations (kImperial_2_Team, kStarTapTeam, kTeamAtWar, kGoalDestroy, 1.0);
				fMission.mSetTeamRelations (kStarTapTeam, kImperial_2_Team, kTeamNeutral, kGoalIgnore, 1.0);

				fMission.mSetTeamRelations (kImperial_1_Team, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

				Ship->mTrackTacticalDistance(80.0, 24, 26);
			}
			else if (Ship->mGetTrackingDistance() == 80.0)
			{
				fMission.mDisplayMessage (kEvent1Orbit_msg, kScienceScriptMessage);
				StarTapOrbiting = 1;
				// 3 turns until Rift opens
				fMission.mSetTimedInterval (kTimerCountdownSet, 3);
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetShipThrust(0);
			}
			break;
	

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					if (StarTapOrbiting == 1)
					{
						Warp = 1;
						return kPlayerWins;
					}
					break;

				case kTimerLimitSet_2: // set relations
					fMission.mSetTeamRelations (kPlayerTeam, kImperial_1_Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kImperial_2_Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kStarTapTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					
					fMission.mSetTeamRelations (kImperial_1_Team, kImperial_2_Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kImperial_1_Team, kStarTapTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kImperial_2_Team, kStarTapTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerTurnsSet_10:
					// set the Imperial Team comm buttons back to HAIL
					for (Ship = Imperial_1_Team->mGetFirstShip(); Ship != NULL; Ship = Imperial_1_Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}

					for (Ship = Imperial_2_Team->mGetFirstShip(); Ship != NULL; Ship = Imperial_2_Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerCountdownSet: // rift opens
					for (Ship = Imperial_1_Team->mGetFirstShip(); Ship != NULL; Ship = Imperial_1_Team->mGetNextShip())
					{
						Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
						Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
					}

					for (Ship = Imperial_2_Team->mGetFirstShip(); Ship != NULL; Ship = Imperial_2_Team->mGetNextShip())
					{
						Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
						Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
					}

					Warp = 1;
					// LNF need to make this better
					// LNF maybe some text after all ships blow up
					return kPlayerWins;

					break;
			}
			break;

		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kImperial_1_Team)
			{
				Imperial_1_Destroyed++;
				if (Imperial_1_Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kImperial_2_Team)
			{
				Imperial_2_Destroyed++;
				if (Imperial_2_Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return eGameStatus ( kPlayerDestroyed | kCampaignCompleted );
				}
			}
			else if (Ship->mGetTeam() == kStarTapTeam)
			{
				fMission.mDisplayMessage (kEvent2StarTapDestroyed_msg, kScienceScriptMessage);
				Warp = 0;
				return eGameStatus( kPlayerLoses | kCampaignCompleted );
			}

			break;

		case kNotifyOnCapture:
			if ( Ship->mGetTeam() != kPlayerTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						if ((Ship->mGetTeam() == kImperial_1_Team) || (Ship->mGetTeam() == kImperial_2_Team))
						{
							fMission.mDisplayMessage (kImpOK_msg, kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonSurrender_text:
						fMission.mDisplayMessage (kImpSurrender_msg, kMissionScriptMessage);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						fMission.mDisplayMessage (kImpTaunt_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;
				}
			}
			break;

	}

	//return fMission.mGetGameStatus();
	//return kGameInProgress;

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kCampaignCompleted);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
	

	VictoryPoints = (Warp * 100); // LNF

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if ( VictoryLevel >= kDefeat )
		{
			fMission.mAddToDebriefingList ( kEndCampaignDefeat_msg );

			// LNF try this out:  adding results of primary objectives
			fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

			( Warp != 0 ) ? fMission.mAddToDebriefingList( kWarpTrue_msg ) : SuccessCount++;
			if ( SuccessCount == 1 )
			{
				fMission.mAddToDebriefingList( kNone_msg );
			}

			fMission.mAddToDebriefingList( kFailedObjectives_msg );

			( Warp == 0 ) ? fMission.mAddToDebriefingList( kWarpFalse_msg ) : FailureCount++;
			if ( FailureCount == 1 )
			{
				fMission.mAddToDebriefingList( kNone_msg );
			}

			return;
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);

	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( Warp != 0 ) ? fMission.mAddToDebriefingList( kWarpTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( Warp == 0 ) ? fMission.mAddToDebriefingList( kWarpFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
