#include "scriptinterface.h"

#include "Fed_9MissionText.h"
#include "Fed_9MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 StartMessage = 0;
int32 PlayerDestroyed = 0;
int32 DeckerDestroyed = 0;
int32 DeckerCaptured = 0;
int32 PkDestroyed = 0;
int32 FederationDestroyed = 0;
int32 FedBaseDestroyed = 0;
int32 FedPlatformDestroyed = 0;
int32 AllPkDestroyed = 0;
int32 Mode = 0; // when certain ships appear
int32 PlanetkillerWaveCount = 1; // the count of how many waves of planetkillers have arrived
int32 PkEntryPointTracker[12] = {0}; // remember the map point where the planekiller arrives, and don't use that point again
int32 PkCounter = 0; // counts how many planetkillers enter the game

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool EarthDestroyed = false;
bool PkWarning = false; // warn if planetkillers are approaching earth

const float kDamageToEarth = 80.0;
const float kPkDistanceToEarth = 30.0;
const float kPkWarningDistanceToEarth = 75.0;
const int32 kMaxPlanetKillers = 8;
const int32 kPlanetkillerWaveTimer = 4; // number of turns between planetkiller waves
const int32 kEarthX = 20;
const int32 kEarthY = 20;
const int32 kPkMaxSpeed = 6; // maximum speed the planetkiller will travel
enum eMissionMode
{
	kStart = 0,
	kPlanetkillersArrive
};

struct NovaMineStruct
{
	tShipInfo* Ship;
	int32 NumberOfMines;
};

NovaMineStruct NovaMineCount[3];

eGameStatus CheckWinCondition( tScript& tScript );

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kDeckerTeam, 
	kPkPlanetTeam,
	kFederationTeam,
	kFedBaseTeam,
	kPkFedFleetTeam,
	kFedPlatformTeam,
	kPlanetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerCapital; // LNF does this work?  list found in missionspec.h
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kDeckerTeam, kMirror);
	fMission.mSetTeamRace (kPkPlanetTeam, kMonster);
	fMission.mSetTeamRace (kFederationTeam, kFederation);
	fMission.mSetTeamRace (kFedBaseTeam, kFederation);
	fMission.mSetTeamRace (kPkFedFleetTeam, kMonster);
	fMission.mSetTeamRace (kFedPlatformTeam, kFederation);
	fMission.mSetTeamRace (kPlanetTeam, kFederation);


	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kDeckerTeam, kTeamAtWar, kGoalCapture, 1.0, kTrue );
	fMission.mSetTeamRelations (kPlayerTeam, kPkPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFederationTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFedBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDeckerTeam, kPkPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeckerTeam, kFederationTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeckerTeam, kFedBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeckerTeam, kPkFedFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeckerTeam, kFedPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeckerTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kPkPlanetTeam, kFederationTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPkPlanetTeam, kFedBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPkPlanetTeam, kPkFedFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPkPlanetTeam, kFedPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPkPlanetTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kFederationTeam, kFedBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFederationTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kFederationTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFederationTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kFedBaseTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedBaseTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedBaseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPkFedFleetTeam, kFedPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPkFedFleetTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedPlatformTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DeckerTeam = fMission.mGetTeamHandle (kDeckerTeam);
	tTeamInfo* PkPlanetTeam = fMission.mGetTeamHandle (kPkPlanetTeam);
	tTeamInfo* FederationTeam = fMission.mGetTeamHandle (kFederationTeam);
	tTeamInfo* FedBaseTeam = fMission.mGetTeamHandle (kFedBaseTeam);
	tTeamInfo* PkFedFleetTeam = fMission.mGetTeamHandle (kPkFedFleetTeam);
	tTeamInfo* FedPlatformTeam = fMission.mGetTeamHandle (kFedPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;


	if ( Mode == kStart )
	{
		// create Decker ship
		DeckerTeam->mCreateShip ("F-DNH", kStartPosition_Z);

		// create Federation team
		FederationTeam->mCreateFleet (3, TotalBPV, kClassFrigate, kClassLightCruiser, 100, kStartPosition_J);

		// create Starbase
		FedBaseTeam->mCreateShip ("F-SBR", kStartPosition_M);

		// create Federation listening posts
		// LNF use coords instead of letters
		FedPlatformTeam->mCreateShip( "F-LP", kStartPosition_Specified, 8, 9 );
		FedPlatformTeam->mCreateShip( "F-LP", kStartPosition_Specified, 31, 9 );
		FedPlatformTeam->mCreateShip( "F-LP", kStartPosition_Specified, 31, 31 );
		FedPlatformTeam->mCreateShip( "F-LP", kStartPosition_Specified, 9, 31 );

		// create Planet team
		PlanetTeam->mCreateShip( "PLPh1", kStartPosition_Specified, kEarthX, kEarthY );
	}
	else // Mode == kPlanetkillersArrive
	{
		int32 DropPoint = fMission.mRandomInt32( kStartPosition_N, kStartPosition_Y );

		// check to see if a planetkiller has already arrived at a particular drop point
		for ( int32 count = 0; count <= PkCounter; count++ )
		{
			// if the randomly selected drop point HASN'T been used
			if ( PkEntryPointTracker[ count ] != DropPoint )
			{
				// use that point and store that value in PkEntryPointTracker
				//PkEntryPointTracker[ PkCounter ] = DropPoint;
			}
			else // if it has been used, re-randomize
			{
				DropPoint = fMission.mRandomInt32( kStartPosition_N, kStartPosition_Y );
				// search the array again from the start
				count = 0;
			}
		}
	
		PkEntryPointTracker[ PkCounter ] = DropPoint;
		PkCounter++;

		fMission.mSetTeamRelations (kPlayerTeam, kPkPlanetTeam, kTeamAtWar, kGoalIgnore, 1.0);
		fMission.mSetTeamRelations (kPkPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
		fMission.mSetTeamRelations (kDeckerTeam, kPkPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kPkPlanetTeam, kFederationTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);//***
		fMission.mSetTeamRelations (kPkPlanetTeam, kFedBaseTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);//***
		fMission.mSetTeamRelations (kPkPlanetTeam, kPkFedFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kPkPlanetTeam, kFedPlatformTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);//***
		fMission.mSetTeamRelations (kPkPlanetTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

		PkPlanetTeam->mCreateShip( "M-DML", DropPoint );
		//PkPlanetTeam->mCreateShip( "M-DML", DropPoint + 5 );
	}

	// make all planetkillers go after the planet, but be AtWar with and Ignore the players?
	// also, make the planetkillers arrive two at a time from semi-random places on the map?
/*
	// create PlanetKiller team against Planet
	PkPlanetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 2, 31 );
	PkPlanetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 3, 32 );
	PkPlanetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 3, 33 );
	PkPlanetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 4, 34 );
	PkPlanetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 5, 36 );
	PkPlanetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 5, 38 );

	// create PlanetKiller team against Federation ships
	PkFedFleetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 3, 29 );
	PkFedFleetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 4, 30 );
	PkFedFleetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 5, 32 );
	PkFedFleetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 6, 34 );
	PkFedFleetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 7, 36 );
	PkFedFleetTeam->mCreateShip( "M-DML", kStartPosition_Specified, 8, 38 );
*/
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DeckerTeam = fMission.mGetTeamHandle (kDeckerTeam);
	tTeamInfo* PkPlanetTeam = fMission.mGetTeamHandle (kPkPlanetTeam);
	tTeamInfo* FederationTeam = fMission.mGetTeamHandle (kFederationTeam);
	tTeamInfo* FedBaseTeam = fMission.mGetTeamHandle (kFedBaseTeam);
	tTeamInfo* PkFedFleetTeam = fMission.mGetTeamHandle (kPkFedFleetTeam);
	tTeamInfo* FedPlatformTeam = fMission.mGetTeamHandle (kFedPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	int32 count = 0;
	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransNovaMine, 1 );
		NovaMineCount[count].Ship = Ship;
		NovaMineCount[count].NumberOfMines = 3;
		count++;
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}

	// configure Decker's ship
	DeckerTeam->mGetFirstShip()->mSetCommButton(1, kButtonHail_text);
/*
	// configure PlanetKillers against Planet
	for (Ship = PkPlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PkPlanetTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipMaxSpeed(7);
	}

	// configure PlanetKillers against Federation fleet
	for (Ship = PkFedFleetTeam->mGetFirstShip(); Ship != NULL; Ship = PkFedFleetTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipMaxSpeed(7);
	}
*/
	// configure Federation ships
	for (Ship = FederationTeam->mGetFirstShip(); Ship != NULL; Ship = FederationTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}
	
	// configure Federation Base
	FedBaseTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);

	// LNF configure Federation Defense platforms

	// configure Planet
	PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
	PlanetTeam->mGetFirstShip()->mChangeName( kEarth_name );


	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 2);

	fMission.mSetTimedInterval (kTimerLimitSet_2, 10);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mSetShipMaxSpeed( kPkMaxSpeed );
	Ship->mSetFinalDestination( kEarthX, kEarthY );
	Ship->mTrackTacticalDistance( kPkWarningDistanceToEarth, kEarthX, kEarthY );
	Ship->mChangeAIOfficerState( kSeekingWeaponDefenseOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DeckerTeam = fMission.mGetTeamHandle (kDeckerTeam);
	tTeamInfo* PkPlanetTeam = fMission.mGetTeamHandle (kPkPlanetTeam);
	tTeamInfo* FederationTeam = fMission.mGetTeamHandle (kFederationTeam);
	tTeamInfo* FedBaseTeam = fMission.mGetTeamHandle (kFedBaseTeam);
	tTeamInfo* PkFedFleetTeam = fMission.mGetTeamHandle (kPkFedFleetTeam);
	tTeamInfo* FedPlatformTeam = fMission.mGetTeamHandle (kFedPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	switch (Event)
	{
		case kNotifyAtDistance:
			{
				// this is for the planetkillers for now
				// Earth gets destroyed if the planetkillers make it this far
				if ( Ship->mGetTeam() == kPkPlanetTeam )
				{
					if ( Ship->mGetTrackingDistance() == kPkWarningDistanceToEarth )
					{
						if ( PkWarning == false )
						{
							fMission.mDisplayMessage( kPkWarning_msg, kMissionScriptMessage );
							PkWarning = true;
							for (tShipInfo* PkShip = PkPlanetTeam->mGetFirstShip(); PkShip != NULL; PkShip = PkPlanetTeam->mGetNextShip())
							{
								PkShip->mTrackTacticalDistance( kPkDistanceToEarth, kEarthX, kEarthY );
							}
						}
					}
					else if ( Ship->mGetTrackingDistance() == kPkDistanceToEarth )
					{
						if ( EarthDestroyed == false )
						{
							fMission.mDisplayMessage( kEarthIsDead_msg, kMissionScriptMessage );
							EarthDestroyed = true;
							return eGameStatus( kPlayerLoses | kCampaignCompleted);
						}
					}
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if (( Ship->mGetTeam() == kPlanetTeam ) && ( Ship->mGetDamagePercent() > kDamageToEarth ))
					{
						return eGameStatus( kPlayerLoses | kCampaignCompleted );
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ((( TransportTarget->mGetTeam() == kPkPlanetTeam ) ||
						 ( TransportTarget->mGetTeam() == kPkFedFleetTeam )) &&
						 ( fEventItem1 == kTransNovaMine ))
					{

						fMission.mSetGoalIndicator( kFalse, 0, 0 );

						TransportTarget->mSetShipAIControlled( kFalse );
						TransportTarget->mSetShipMaxSpeed( 0 );
						fMission.mDisplayMessage( kPlantedNovaMine_msg, kMissionScriptMessage );
						TransportTarget->mChangeName( kDeadPlanetkiller_name );
						TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransNovaMine, 0 );

						PkDestroyed++;

						if ( PkDestroyed == 4 )
						{
							//fMission.mDisplayMessage( PkDestroyed4_msg, kMissionScriptMessage );
						}
						else if ( PkDestroyed == 6 )
						{
							//fMission.mDisplayMessage( PkDestroyed6_msg, kMissionScriptMessage );
						}
						else if ( PkDestroyed == 7 )
						{
							//fMission.mDisplayMessage( PkDestroyed7_msg, kMissionScriptMessage );
						}
						else if ( PkDestroyed == 8 )
						{
							//fMission.mDisplayMessage( PkDestroyed8_msg, kMissionScriptMessage );
						}

						// decrement the players Nova mine count
						for ( int32 count = 2; count >= 0; count--)
						{
							if ( NovaMineCount[count].Ship == Ship )
							{
								if ( NovaMineCount[count].NumberOfMines > 1 )
								{
									Ship->mAccessSupplies( kSetTransporterItemCount, kTransNovaMine, 1 );
									NovaMineCount[count].NumberOfMines--;

									if ( NovaMineCount[count].NumberOfMines == 2 )
									{
										fMission.mDisplayMessage( kTwoNovaMinesLeft_msg, kMissionScriptMessage );
									}
									else if ( NovaMineCount[count].NumberOfMines == 1 )
									{
										fMission.mDisplayMessage( kOneNovaMineLeft_msg, kMissionScriptMessage );
									}
								}
								else
								{
									fMission.mDisplayMessage( kOutOfMines_msg, kMissionScriptMessage );
								}
							}
						}

						Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
						Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
						// check for the win condition
						return CheckWinCondition( *this );
					}
					else // don't transport mine
					{
						Ship->mAccessSupplies( kSetTransporterItemCount, kTransNovaMine, 1 );
						TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransNovaMine, 0 );
						fMission.mDisplayMessage( kUselessMines_msg, kScienceScriptMessage );
					}
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if (( TransportTarget->mGetTeam() == kPlanetTeam ) &&
						( fEventItem1 == kTransNovaMine ))
					{
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// set relations
					fMission.mSetTeamRelations (kPlayerTeam, kDeckerTeam, kTeamAtWar, kGoalCapture, 1.0 );//***
					fMission.mSetTeamRelations (kPlayerTeam, kPkPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kFederationTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kFedBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDeckerTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations (kDeckerTeam, kPkPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDeckerTeam, kFederationTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kDeckerTeam, kFedBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kDeckerTeam, kPkFedFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDeckerTeam, kFedPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kDeckerTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kPkPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPkPlanetTeam, kFederationTeam, kTeamAtWar, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPkPlanetTeam, kFedBaseTeam, kTeamAtWar, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPkPlanetTeam, kPkFedFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPkPlanetTeam, kFedPlatformTeam, kTeamAtWar, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPkPlanetTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kFederationTeam, kDeckerTeam, kTeamAtWar, kGoalCapture, 1.0);//***
					fMission.mSetTeamRelations (kFederationTeam, kPkPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kFederationTeam, kFedBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFederationTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kFederationTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFederationTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					
					fMission.mSetTeamRelations (kFedBaseTeam, kPkPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kFedBaseTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedBaseTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedBaseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPkFedFleetTeam, kFedPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPkFedFleetTeam, kPlanetTeam, kTeamAtWar, kGoalIgnore, 1.0);//***

					fMission.mSetTeamRelations (kFedPlatformTeam, kPkPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kFedPlatformTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlanetTeam, kPkFedFleetTeam, kTeamAtWar, kGoalDestroy, 1.0);//***

					// displays the scenario starting message
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					// start the planetkiller wave timer
					fMission.mSetTimedInterval( kTimerTurnsSet, kPlanetkillerWaveTimer );

					//fMission.mSetGoalIndicator( DeckerTeam->mGetFirstShip() );
					break;

				case kTimerLimitSet_2:
					if (StartMessage == 0)
					{
						fMission.mDisplayMessage (kEvent1A_msg, kMissionScriptMessage);
						fMission.mDisplayMessage (kEvent1B_msg, kMissionScriptMessage);
						fMission.mDisplayMessage (kEvent1C_msg, kMissionScriptMessage);
						fMission.mDisplayMessage (kEvent1D_msg, kMissionScriptMessage);
						StartMessage = 1;
					}
					break;

				case kTimerTurnsSet:
					// this calls mCreateMissionShips to create the waves of planetkillers
					if ( PlanetkillerWaveCount < 9 ) // 1 based - so there are 8 waves
					{
						mCreateMissionShips( kPlanetkillersArrive );
						fMission.mSetTimedInterval( kTimerTurnsSet, kPlanetkillerWaveTimer );

						if ( PlanetkillerWaveCount == 1 )
						{
							fMission.mDisplayMessage( kPkWave1_msg, kMissionScriptMessage );
						}
						else if ( PlanetkillerWaveCount == 4 )
						{
							fMission.mDisplayMessage( kPkWave4_msg, kMissionScriptMessage );
						}
						else if ( PlanetkillerWaveCount == 8 )
						{
							fMission.mDisplayMessage( kPkWave8_msg, kMissionScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( fMission.mRandomInt32( kPkWaveRandom_1_msg, kPkWaveRandom_4_msg ), kMissionScriptMessage );
						}

						PlanetkillerWaveCount++;
					}
					break;

				case kTimerTurnsSet_8:
					// set the Federation Base Team comm buttons back to HAIL
					for (Ship = FedBaseTeam->mGetFirstShip(); Ship != NULL; Ship = FedBaseTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Federation Team comm buttons back to HAIL
					for (Ship = FederationTeam->mGetFirstShip(); Ship != NULL; Ship = FederationTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Decker Team comm buttons back to HAIL
					for (Ship = DeckerTeam->mGetFirstShip(); Ship != NULL; Ship = DeckerTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kDeckerTeam)
			{
				DeckerDestroyed++;

				if ( DeckerTeam->mGetShipsInPlayCount() == 0 )
				{
					// make the Federation team defend the starbase
					FederationTeam->mDefendEntity( FedBaseTeam->mGetFirstShip(), 45.0, 45.0, kVeryHighPriority );
				}
				return CheckWinCondition( *this );
			}
			else if (Ship->mGetTeam() == kPkPlanetTeam)
			{
				PkDestroyed++;
				Ship->mChangeName( kDeadPlanetkiller_name );

				if ((PkPlanetTeam->mGetShipsInPlayCount() == 0) &&
					(PkFedFleetTeam->mGetShipsInPlayCount() == 0))
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetTeam() == kPkFedFleetTeam)
			{
				PkDestroyed++;
				if ((PkPlanetTeam->mGetShipsInPlayCount() == 0) &&
					(PkFedFleetTeam->mGetShipsInPlayCount() == 0))
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetTeam() == kFederationTeam)
			{
				FederationDestroyed++;
				if (FederationTeam->mGetShipsInPlayCount() == 0)
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetTeam() == kFedBaseTeam)
			{
				FedBaseDestroyed++;
				if (FedBaseTeam->mGetShipsInPlayCount() == 0)
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetTeam() == kFedPlatformTeam)
			{
				FedPlatformDestroyed++;
				if (FedPlatformTeam->mGetShipsInPlayCount() == 0)
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else
			{
			}
			break;

		case kNotifyOnCapture:
			if (Ship->mGetTeam () == kDeckerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				DeckerCaptured++;
				DeckerDestroyed++;

				Ship->mClearCommands();
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetShipThrust(0);

				if ( DeckerTeam->mGetShipsInPlayCount() == 0 )
				{
					// make the Federation team defend the starbase
					FederationTeam->mDefendEntity( FedBaseTeam->mGetFirstShip(), 45.0, 45.0, kVeryHighPriority );
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						if (Ship->mGetTeam() == kDeckerTeam)
						{
							if ((Ship->mGetDamagePercent() >= 0.0) && (Ship->mGetDamagePercent() < 50.0))
							{
								fMission.mDisplayMessage (kDeckerOK_msg, kMissionScriptMessage);
							}	
							else if (Ship->mGetDamagePercent() >= 50.0)
							{
								//fMission.mDisplayMessage (kDeckerDamaged50_msg, kMissionScriptMessage);
								fMission.mDisplayMessage (kDeckerDamaged_msg, kMissionScriptMessage);
							}
							Ship->mSetCommButton (1, kButtonTaunt_text);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							// If the Taunt or Surrender buttons aren't used in X turns
							// return the buttons to Hail and blank.
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
						else if ((Ship->mGetTeam() == kFederationTeam) || (Ship->mGetTeam() == kFedBaseTeam))
						{
							if (Ship->mGetDamagePercent() < 50.0)
							{
								if (fPlayerShip->mGetDamagePercent() > 50.0)
								{
									fMission.mDisplayMessage (kFedHailYouDamaged_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kFedHailOK_msg, kMissionScriptMessage);
								}
							}
							else if (Ship->mGetDamagePercent() >= 50.0)
							{
								fMission.mDisplayMessage (kFedHailDamaged_msg, kMissionScriptMessage);
							}

							if ( Ship->mGetTeam() == kFederationTeam )
							{
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
							}
							else
							{
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
							}
						}
						else if (Ship->mGetTeam() == kPkPlanetTeam)
						{
							fMission.mDisplayMessage (kPkHail_msg, kMissionScriptMessage);
							Ship->mSetCommButton(1, kButtonHail_text);	
						}
						else if ( Ship->mGetTeam() == kPlanetTeam )
						{
							if ( fPlayerShip->mGetTacticalDistance(PlanetTeam->mGetFirstShip()) <= 5.0 )
							{
								for ( int32 count = 2; count >= 0; count--)
								{
									if ( NovaMineCount[count].Ship == fPlayerShip )
									{
										if ( NovaMineCount[count].NumberOfMines < 3 )
										{
											fMission.mDisplayMessage( kReplenishMines_msg, kMissionScriptMessage );
											NovaMineCount[count].NumberOfMines = 3;
											fPlayerShip->mAccessSupplies( kSetTransporterItemCount, kTransNovaMine, 1 );
										}
										else
										{
											fMission.mDisplayMessage( kEarthHail_msg, kMissionScriptMessage );
										}
									}
								}
							}
							else
							{
								// not in range for nova mines
								fMission.mDisplayMessage( kEarthHail_msg, kMissionScriptMessage );
							}
							Ship->mSetCommButton(1, kButtonHail_text);	
						}
						
						break;

					case kButtonSurrender_text:
						if (Ship->mGetTeam() == kDeckerTeam)
						{
							fMission.mDisplayMessage (kDeckerOK_msg, kMissionScriptMessage);
							//fMission.mDisplayMessage (kDeckerDamaged_msg, kMissionScriptMessage);
							//fMission.mDisplayMessage (kDeckerDamaged50_msg, kMissionScriptMessage);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
						else if ( Ship->mGetTeam() == kFederationTeam )
						{
							fMission.mDisplayMessage (kFedTaunt_msg, kMissionScriptMessage);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
						}
						else if ( Ship->mGetTeam() == kFedBaseTeam )
						{
							fMission.mDisplayMessage (kFedTaunt_msg, kMissionScriptMessage);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
						}
						break;

					case kButtonTaunt_text:
						if (Ship->mGetTeam() == kDeckerTeam)
						{
							fMission.mDisplayMessage (kDeckerTaunt_msg, kMissionScriptMessage);
							//fMission.mDisplayMessage (kDeckerTaunt50_msg, kMissionScriptMessage);
							Ship->mSetCommButton (1, kButtonTaunt_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
						else if ( Ship->mGetTeam() == kFederationTeam)
						{
							fMission.mDisplayMessage (kFedTaunt_msg, kMissionScriptMessage);
							Ship->mSetCommButton (1, kButtonTaunt_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
						}
						else if (Ship->mGetTeam() == kFedBaseTeam)
						{
							fMission.mDisplayMessage (kFedTaunt_msg, kMissionScriptMessage);
							Ship->mSetCommButton (1, kButtonTaunt_text);
							fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
						}
						break;
				}
			}
			break;
	}

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus(status | kCampaignCompleted);
	}
	return status;
	//return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0; // LNF

	VictoryPoints = (DeckerDestroyed * 50) + (AllPkDestroyed * 50);
	

	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if ( VictoryLevel == kAstoundingVictory )
		{
			fMission.mAddToDebriefingList ( kEndCampaignVictory_msg );
		}
		else if ( VictoryLevel == kDraw )
		{
			// LNF I should change this if the planet killers were destroyed
			// LNF to something like Decker got away
			if ( DeckerDestroyed == 0 )
			{
				fMission.mAddToDebriefingList ( kEndCampaignDeckerGetsAway_msg );
			}
			else
			{
				fMission.mAddToDebriefingList ( kEndCampaignDefeat_msg );
			}
		}
		else // VictoryLevel == kDevastatingDefeat
		{
			fMission.mAddToDebriefingList ( kEndCampaignDefeat_msg );
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( DeckerDestroyed != 0 ) ? fMission.mAddToDebriefingList( kDeckerDestroyedTrue_msg ) : SuccessCount++;
	( AllPkDestroyed != 0 ) ? fMission.mAddToDebriefingList( kAllPkDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( DeckerDestroyed == 0 ) ? fMission.mAddToDebriefingList( kDeckerDestroyedFalse_msg ) : FailureCount++;
	( AllPkDestroyed == 0 ) ? fMission.mAddToDebriefingList( kAllPkDestroyedFalse_msg ) : FailureCount++;
	// LNF put in something about Earth destroyed?
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// win condition
eGameStatus CheckWinCondition( tScript& tScript )
{
	if (( DeckerDestroyed == 1 ) && ( PkDestroyed == kMaxPlanetKillers ))
	{
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		AllPkDestroyed = 1;
		return eGameStatus( kPlayerWins | kCampaignCompleted );
	}
	else
	{
		// test to end the campaign
		eGameStatus status = tScript.fMission.mGetGameStatus();
		if (status != kGameInProgress)
		{
			status = eGameStatus(status | kCampaignCompleted);
		}
		return status;
	}
}


//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
