#include "scriptinterface.h"

#include "Gen_BaseAssaultText.h"
#include "Gen_BaseAssaultMaps.h"

int32 PlayerKilled = 0;
int32 EnemyKilled = 0;
int32 EnemyCaptured = 0;
bool BaseDestroyed = false;
bool Reinforced = false;
int32 MyVicLevel = kDefeat;

enum eMissionModes
{
	kStartMode				= 0,
	kReinforcements
};

enum eMissionActs
{
	kCallingForHelp					= 0,
	kHelpArrives,
	kEnemyDisengaging,
};
eMissionActs MissionAct = kCallingForHelp;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kBaseTeam, kEnemyTeam, kFreighterTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kWarzone | kEnemyBorder | kEnemyCapital | kEnemyColonial;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecTwoShip | MissSpecThreeShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 20;
	Specs->fMaxDate = 50;
	Specs->fWait = 3;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 200, 100, 50, 0, -50 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kBaseTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kFreighterTeam, info.fEnemyRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalCripple, 1.0 );
	fMission.mSetTeamRelations( kPlayerTeam, kFreighterTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kBaseTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kBaseTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kEnemyTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return( fMission.mRandomInt32( 1, TotalMaps ));
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 TotalShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DefenderBPV = 0;
	int32 BaseBPV = PlayerBPV * .75;
	int32 AlliedBPV = 0;
	int32 DiffLevel = fMission.mGetDifficultyLevel();	// 0 = easy, 1 = medium, 2 = hard

	if( DiffLevel == 1 )
	{
		DefenderBPV = PlayerBPV * .5;
		BaseBPV = PlayerBPV * 1.5;
	}

	if( DiffLevel == 2 )
	{
		DefenderBPV = PlayerBPV * .75;
		BaseBPV = PlayerBPV * 2;
	}

	if( Mode == kStartMode )
	{
		// create enemy forces
		FreighterTeam->mCreateFleet( 2, kMaxBPV, kClassFreighter, kClassFreighter, 1, kStartPosition_N );
		EnemyTeam->mCreateShip( DefenderBPV, kClassFrigate, kClassLightCruiser, 20, kStartPosition_X );
		BaseTeam->mCreateShip( BaseBPV, kClassBaseStation, kClassBattleStation, 0, kStartPosition_M );

		// create additional player ships
		AlliedBPV = EnemyTeam->mGetCombatBPV() + BaseTeam->mGetCombatBPV() - PlayerBPV;
		PlayerTeam->mCreateFleet(( 4 - TotalShips ), AlliedBPV, kClassFrigate, kClassHeavyCruiser, 20, kStartPosition_J );

		// based on difficulty level, finish the enemy fleet
		if( DiffLevel == 1 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, 1, kStartPosition_Y );
		}

		if( DiffLevel == 2 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, 1, kStartPosition_Y );
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_Z );
		}
	}
	
	if( Mode == kReinforcements )
	{
		EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyBattlecruiser, 1, kStartPosition_R );
		EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 1, kStartPosition_S );
		EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_T );

		if( DiffLevel == 2 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_U );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );
	tShipInfo* Ship;

	// configure player ships
	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 10 );
	}

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure enemy and base ships
	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	for( Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	BaseTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );

	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );

	int32 RandomTurns = 0;

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					if( MissionAct == kCallingForHelp )
					{
						MissionAct = kHelpArrives;
						fMission.mDisplayMessage( kBaseScream_msg, kScienceScriptMessage );
						RandomTurns = fMission.mRandomInt32( 8, 16 );
						fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
						FreighterTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
					}
					else if( MissionAct == kHelpArrives )
					{
						fMission.mDisplayMessage( kEnemyReinforcements_msg, kScienceScriptMessage );
						mCreateMissionShips( kReinforcements );
						Reinforced = true;
					}
					break;

				case kTimerLimitSet:
					Ship = BaseTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage, Ship );
					}

					fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalCripple, 1.0 );
					fMission.mSetTeamRelations( kPlayerTeam, kFreighterTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kBaseTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kBaseTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
					fMission.mSetTeamRelations( kEnemyTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerKilled++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}

			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyKilled++;

				if(( EnemyTeam->mGetShipsInPlayCount() == 0 ) && ( BaseDestroyed == true ))
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetTeam() == kBaseTeam )
			{
				BaseDestroyed = true;

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerWins;
				}

				if( EnemyTeam->mGetShipsInPlayCount() >= 2 )
				{
					fMission.mDisplayMessage( kMultipleDisengaging_msg, kScienceScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kSingleDisengaging_msg, kScienceScriptMessage );
				}

				EnemyTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
				MissionAct = kEnemyDisengaging;
			}

			if( Ship->mGetTeam() == kFreighterTeam )
			{
				fMission.mAddToBonusPrestige( 20 );
			}
			break;

		case kNotifyCommButtonPress:
			if( Ship->mGetTeam() == kBaseTeam )
			{
				fMission.mDisplayMessage( kBaseResponse_msg, kMissionScriptMessage );
			}
				
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( fMission.mRandomInt32( kEnemyNoResponse1_msg, kEnemyNoResponse3_msg) , kMissionScriptMessage, Ship );
			}

			if( Ship->mGetTeam() == kFreighterTeam )
			{
				fMission.mDisplayMessage( kFreighterResponse_msg, kMissionScriptMessage );
			}
			break;
			
		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;
			}

			if( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				fMission.mAddToBonusPrestige( 200 );

				if( EnemyTeam->mGetShipsInPlayCount() >= 2 )
				{
					fMission.mDisplayMessage( kMultipleDisengaging_msg, kScienceScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kSingleDisengaging_msg, kScienceScriptMessage );
				}

				EnemyTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
				MissionAct = kEnemyDisengaging;
			}

			if( Ship->mGetTeam() == kFreighterTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				return kPlayerDisengaged;
			}

			if(( Ship->mGetOriginalTeam() == kEnemyTeam ) && ( EnemyTeam->mGetShipsInPlayCount() == 0 ))
			{
				return kPlayerWins;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );

	MyVicLevel = kDefeat;

	if( BaseDestroyed == true )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
		MyVicLevel = kVictory;
	}
 
	if( EnemyKilled >= 2 )
	{
		MyVicLevel--;
	}

	if( PlayerKilled >= 1 )
	{
		MyVicLevel++;
	}

	if( EnemyCaptured >= 1 )
	{
		MyVicLevel--;
	}

	if( MyVicLevel <= kAstoundingVictory )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
		MyVicLevel = kAstoundingVictory;
	}

	if( MyVicLevel >= kDevastatingDefeat )
	{
		MyVicLevel = kDevastatingDefeat;
	}

	return (eVictoryLevels(MyVicLevel));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;

	if( BaseDestroyed == false )
	{
		fMission.mAddToDebriefingList( kBaseNotDestroyed_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kBaseDestroyed_msg );
	}

	if( EnemyCaptured == 1 )
	{
		fMission.mAddToDebriefingList( kEnemyCapturedSingle_msg );
	}
	else if( EnemyCaptured >= 2 )
	{
		fMission.mAddToDebriefingList( kEnemyCapturedMultiple_msg );
	}

	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_2;
	}
	else
	{
		return 0;
	}
}
