#include "scriptinterface.h"

#include "Gen_BaseDefenseText.h"
#include "Gen_BaseDefenseMaps.h"

int32 EnemyCaptured = 0;
int32 EnemyDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 AlliedDestroyed = 0;
int32 DiffLevel = 0;
bool ReinforcementsCalled = false;
bool BaseDestroyed = false;
int32 MyVicLevel = kDefeat;

enum eMissionModes
{
	kStart = 0,
	kReinforcements,
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kBaseTeam, kAlliedTeam, kFreighterTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kWarzone | kEnemyBorder;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 15;
	Specs->fMaxDate = 50;
	Specs->fWait = 5;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 300, 200, 100, -50, -100 );
	DiffLevel = fMission.mGetDifficultyLevel();

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kBaseTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );

	// randomize the race of the freighter team
	if( fMission.mRandomInt32( 1, 2 ) == 1 )
	{
		fMission.mSetTeamRace( kFreighterTeam, info.fAlliedRace );
	}
	else
	{
		fMission.mSetTeamRace( kFreighterTeam, info.fPlayerRace );
	}

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kPlayerTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalCripple, 1.0 );
	fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations( kEnemyTeam, kFreighterTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kAlliedTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kAlliedTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kBaseTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return fMission.mRandomInt32( 1, TotalMaps );
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );

	if( Mode == kStart )
	{
		// create base, allied and freighters
		if( fMission.mRandomInt32( 1, 2 ) == 1 )
		{
			BaseTeam->mCreateShip( kMaxBPV, kClassBaseStation, 100, kStartPosition_M );
			AlliedTeam->mCreateShip( kMaxBPV, kClassDestroyer, 20, kStartPosition_J );
		}
		else
		{
			BaseTeam->mCreateShip( kMaxBPV, kClassBattleStation, 100, kStartPosition_M );
			AlliedTeam->mCreateFleet( 2, kMaxBPV, kClassFrigate, kClassFrigate, 20, kStartPosition_J );
		}

		FreighterTeam->mCreateFleet( 2, kMaxBPV, kClassFreighter, kClassFreighter, 10, kStartPosition_N );
		
		// create the attacking enemy forces (max 7 ships)
		int32 TotalBPV = 100 + AlliedTeam->mGetCombatBPV() + PlayerTeam->mGetCombatBPV() + BaseTeam->mGetCombatBPV();
		int32 TotalShips = AlliedTeam->mGetShipsInPlayCount() + PlayerTeam->mGetShipsInPlayCount();

		if( DiffLevel == 0 )
		{
			TotalBPV = TotalBPV * .75;
			TotalShips = TotalShips - 2;
		}

		if( DiffLevel == 2 )
		{
			TotalBPV = TotalBPV * 1.33;
			TotalShips++;
		}

		if( TotalShips < 2 )
		{
			TotalShips = 2;
		}

		EnemyTeam->mCreateShip( TotalBPV, kClassHeavyCruiser, kClassDreadnought, 20, kStartPosition_T );
		TotalBPV = TotalBPV - EnemyTeam->mGetCombatBPV();
		EnemyTeam->mCreateFleet( TotalShips, TotalBPV, kClassFrigate, kClassHeavyCruiser, 33, kStartPosition_U );
	}

	if( Mode == kReinforcements )
	{
		AlliedTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 1, kStartPosition_P );
		AlliedTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_Q );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( fMission.mRandomInt32( kBriefingA_msg, kBriefingC_msg ));
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );
	tShipInfo* Ship;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure enemy ship(s)
	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	// configure allied base
	BaseTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );

	for( Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );

	switch( Event )
	{
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kBaseTeam )
			{
				fMission.mDisplayMessage( kBaseDeath_msg );
				BaseDestroyed = true;
				EnemyTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
			}

			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerDestroyed++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					BaseDestroyed = true;
					return kPlayerDestroyed;
				}
			}

			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetTeam() == kFreighterTeam )
			{
				fMission.mAddToBonusPrestige( -20 );
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerWins;
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					BaseDestroyed = true;
					return kPlayerWins;
				}
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					if( Ship->mGetTeam() == kBaseTeam )
					{
						fMission.mDisplayMessage (kBaseHail_msg, kFleetScriptMessage);
						Ship->mSetCommButton( 1, kButtonRequestHelp_text );
					}

					if( Ship->mGetTeam() == kEnemyTeam )
					{
						fMission.mDisplayMessage( kEnemyStatic_msg, kMissionScriptMessage );

						for( tShipInfo* EnemyShip = EnemyTeam->mGetFirstShip(); EnemyShip != NULL; EnemyShip = EnemyTeam->mGetNextShip() )
						{
							EnemyShip->mSetCommButton( 1, kButtonThreaten_text );
						}
					}

					if( Ship->mGetTeam() == kFreighterTeam )
					{
						fMission.mDisplayMessage( kFreighterHail_msg, kFleetScriptMessage, Ship );

						for( Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip() )
						{
							Ship->mSetCommButton( 1, kButtonDisengage_text );
						}
					}
					break;

				case kButtonRequestHelp_text:
					fMission.mDisplayMessage( kRequestedHelp_msg, kFleetScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 8 + DiffLevel, 12 + DiffLevel ));
					ReinforcementsCalled = true;
					break;

				case kButtonThreaten_text:
					fMission.mDisplayMessage( kStaticLouder_msg, kMissionScriptMessage );

					for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
					{
						Ship->mClearCommButton( 1 );
					}
					break;

				case kButtonDisengage_text:
					fMission.mDisplayMessage( kAyeSir_msg, kFleetScriptMessage, Ship );
					FreighterTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
					break;
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kHelpArrives_msg, kMissionScriptMessage );
					mCreateMissionShips( kReinforcements );
					break;

			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );

	MyVicLevel = kDefeat;

	if( BaseDestroyed == false )
	{
		MyVicLevel = kVictory;
	}
 
	if( EnemyDestroyed >= 3 )
	{
		MyVicLevel--;
	}

	if( PlayerDestroyed >= 1 )
	{
		MyVicLevel++;
	}

	if( EnemyCaptured >= 1 )
	{
		MyVicLevel--;
	}

	if( MyVicLevel <= kAstoundingVictory )
	{
		MyVicLevel = kAstoundingVictory;
	}

	if( MyVicLevel >= kDevastatingDefeat )
	{
		MyVicLevel = kDevastatingDefeat;
	}

	return( eVictoryLevels( MyVicLevel ));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;

	if( BaseDestroyed == false )
	{
		fMission.mAddToDebriefingList( kBaseLives_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kBaseDied_msg );
	}

	if( EnemyCaptured == 1 )
	{
		fMission.mAddToDebriefingList( kCapturedSingleEnemy_msg );
	}

	if( EnemyCaptured >= 2 )
	{
		fMission.mAddToDebriefingList( kCapturedMultipleEnemies_msg );
	}

	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_2;
	}
	else
	{
		return 0;
	}
}
