#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 gEnemyKilled = 0;
iTruth gMadeIt = kFalse;
int32 MayDays = 0;
int32 gAct = 0;
iTruth PlayerDisengaged = kFalse;
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBaseTeam, // good guys
	kRunningTeam,
	kFreighterTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}



//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kWarzone | kPlayerColonial;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kKlingMission | kLyranMission | kGornMission | kHydranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 5;
	Specs->fMaxDate = 40;
	Specs->fWait = 20;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 100, 40, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kRunningTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFreighterTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
    fMission.mSetTeamRelations (kPlayerTeam, kRunningTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

    fMission.mSetTeamRelations (kBaseTeam, kRunningTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kRunningTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kFreighterTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
    fMission.mSetTeamRelations (kFreighterTeam, kRunningTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
    fMission.mSetTeamRelations (kFreighterTeam, kPlayerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
    fMission.mSetTeamRelations (kFreighterTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition += kStartPosition_H - kStartPosition_G;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* RunningTeam = fMission.mGetTeamHandle (kRunningTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	// Get the total BPV of the player and modify by 70% to 120%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * 1.1;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.5;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV = TotalBPV * 0.6f;
		}
		break;
	case 1:
		{
			// do nothing
		}
		break;
	case 2:
		{
			TotalBPV = TotalBPV * 1.5f;
		}			
		break;
	}

	if (Mode == 0)
	{

		BaseTeam->mCreateShip (kMaxBPV, kClassBaseStation, kClassStarBase, 50, kStartPosition_G);

		RunningTeam->mCreateShip (TotalBPV * 1.3, kClassDestroyer, kClassBattleship, 0, kStartPosition_I);

		FreighterTeam->mCreateShip( kMaxBPV, kClassFreighter, kClassFreighter, 50, kStartPosition_J);
		//FreighterTeam->mCreateShip( "H-HN", kStartPosition_J); // ENF the freighter causes horrible performance problems...
		// small escorts....
		EnemyTeam->mCreateShip ( TotalBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_Y);
		EnemyTeam->mCreateShip ( TotalBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_Z);
	}

	if ( Mode == 1 )
	{
		EnemyTeam->mCreateShip ( TotalBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_Y);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	int32 MessageNumber = fMission.mRandomInt32 (kBriefingA_msg, kBriefingB_msg);
	return (MessageNumber);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
//	char msg[200];

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* RunningTeam = fMission.mGetTeamHandle (kRunningTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tShipInfo* Ship;
	tShipInfo* RunningShip;
	tShipInfo* Base;
	tShipInfo* Freighter;

	Freighter = FreighterTeam->mGetFirstShip();
	Base = BaseTeam->mGetFirstShip();
	RunningShip = RunningTeam->mGetFirstShip();

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget (RunningShip, kVeryHighPriority);
		Ship->mSetShipThrust (10);
	}

	// Slow the running ship down...
	Ship = RunningTeam->mGetFirstShip();
	Ship->mDamageSystem( kDamageLeftWarpEngine, 70 );
	Ship->mDamageSystem( kDamageCenterWarpEngine, 50 );
	Ship->mDamageSystem( kDamageRightWarpEngine, 50 );
	Ship->mSetShipCallbackEvents( kNotifyDamage );  // It will issue distress calls when under attack.

	Ship->mSetFinalDestination( 16, 31);
	Ship->mTrackTacticalDistance (10.0, 16, 31);

	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget (RunningShip, kVeryHighPriority);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// on Admiral this will summon reinforcements in 2 turns
	fMission.mSetTimedInterval (kTimerTurnsSet, 5);

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* RunningTeam = fMission.mGetTeamHandle (kRunningTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch (Event)
	{
		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				if ( gMadeIt )
				{
					return kPlayerWins;
				}
				else
				{
					return kPlayerLoses;
				}
			}
		break;

		case kNotifyAtDistance:
		{
			// the ship made it home, player wins
			gMadeIt = kTrue;
			return kPlayerWins;
			break;
		}

		case kNotifyAtTimedInterval:
		{
			if ( fMission.mGetDifficultyLevel() == 2 )
			{
				if ( gAct == 0 )
				{
					gAct = 1;
					mCreateMissionShips( gAct );				
				}
			}
			return kGameInProgress;
			break;
		}

		case kNotifyDamage:
		{
			if (Ship->mGetTeam() == kRunningTeam)
			{
				float RunningShipHullIntegrity =  100.0 - Ship->mGetDamagePercent();

				if ( RunningShipHullIntegrity > 80.0 )
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );  // It will issue distress calls when under attack.
					return kGameInProgress;
				}
				if ( ( RunningShipHullIntegrity > 65.0 ) && ( MayDays == 0 ) )
				{

					fMission.mDisplayMessage (kMayday1_msg, kFleetScriptMessage);
					Ship->mSetShipCallbackEvents( kNotifyDamage );  // It will issue distress calls when under attack.
					MayDays++;
					return kGameInProgress;
				}
				if ( ( RunningShipHullIntegrity < 65.0 ) && ( RunningShipHullIntegrity > 45.0 ) && ( MayDays == 1 ) )
				{

					fMission.mDisplayMessage (kMayday2_msg, kFleetScriptMessage);
					Ship->mSetShipCallbackEvents( kNotifyDamage );  // It will issue distress calls when under attack.
					MayDays++;
					return kGameInProgress;
				}
				if ( ( RunningShipHullIntegrity < 45.0 ) && ( MayDays == 2 ) )
				{

					fMission.mDisplayMessage (kMayday3_msg, kFleetScriptMessage);
					MayDays++;
					return kGameInProgress;
				}
			}
			return kGameInProgress;
		}

		case kNotifyOnCapture:
			{
				if (Ship->mGetTeam () == kRunningTeam)
				{
					gMadeIt = kFalse;
					return kPlayerLoses;
				}

				if (Ship->mGetTeam () == kPlayerTeam)
				{
					// LNF if it does/will change the 1 to 0
					if( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						return kPlayerCaptured;
					}
				}

				if (Ship->mGetTeam () == kEnemyTeam)
				{
					gEnemyKilled++;
					if ( EnemyTeam->mGetShipsInPlayCount() == 0 ) 
					{
						fMission.mDisplayMessage (kBringEmHome_msg, kFleetScriptMessage);
					}				
					return kGameInProgress;
				}
			}
		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam () == kRunningTeam)
				{
					gMadeIt = kFalse;
					return kPlayerLoses;
				}

				if (Ship->mGetTeam () == kPlayerTeam)
				{
					if( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						return kPlayerDestroyed;
					}
				}

				if (Ship->mGetTeam () == kEnemyTeam)
				{
					gEnemyKilled++;
					if ( EnemyTeam->mGetShipsInPlayCount() == 0 ) // trick to catch the third ship for admiral
					{
						fMission.mDisplayMessage (kBringEmHome_msg, kFleetScriptMessage);
					}				
					return kGameInProgress;
				}
			}

	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* RunningTeam = fMission.mGetTeamHandle (kRunningTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	// determine who gets the combat BPV
	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 PlayerDamage = PlayerBPV - PlayerTeam->mAssessDamages(EnemyTeam);

	float playerShipHullIntegrity =  (float) PlayerDamage / (float) PlayerBPV;

	if ( PlayerDisengaged )
	{
		return kDefeat;
	}

	if ( ( gMadeIt ) && ( gEnemyKilled == 2 ) )
	{
		// kAstoundingVictory,  if the two attackers are destroyed and the ship made it..
		return kAstoundingVictory;
	}

	if ( ( gMadeIt ) && ( gEnemyKilled == 1 ) )
	{
		// kVictory, 1 attacker & ship made it
		return kVictory;
	}

	if ( ( gMadeIt ) && ( gEnemyKilled == 0 ) )
	{
		// kDraw, 2 attackers left & ship made it
		return kDraw;
	}

	if ( ( !gMadeIt ) && ( playerShipHullIntegrity < 0.5 ) )
	{
		// kDevastatingDefeat, ship did not make it, you are heavily damaged
		return kDevastatingDefeat;
	}

	if ( !gMadeIt )
	{
		// kDefeat ship did not make it
		return kDefeat;
	}

	// ENF should never get here.
	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

