#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 AlliedCaptured = 0;
int32 AlliedDestroyed = 0;
int32 EnemyCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
float AlliedHealth = 0.0;
iTruth PlayerDisengaged = kFalse;
int32 FreightersKilled = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBaseTeam,
	kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kWarzone | kAlliedBorder | kOccupiedCapital | kOccupiedColonial;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = 150;
	Specs->fMinDate = 3;
	Specs->fMaxDate = 50;
	Specs->fWait = 2;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 150, 75, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kAlliedTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations (kBaseTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.5);
	fMission.mSetTeamRelations (kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	int32 map = 1;
	// pick a random map
	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
		}
		break;
	case 1:
		{
		}
		break;
	case 2:
		{
			map = 2;
		}
		break;
	}

	return (map);
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition += kStartPosition_P - kStartPosition_G;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	// Get the total BPV of the player and modify by 80% to 130%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .8;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// pick a random number of ships
	int32 NumberEnemyShips = fMission.mRandomInt32 (PlayerTeam->mGetShipsInPlayCount() + 1, PlayerTeam->mGetShipsInPlayCount() + 2 );

	int32 NumLQShips = 1;
	int32 NumMQShips = 2;

	eClassTypes BaseType = kClassBaseStation;

	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.70f;
			NumMQShips++;
			BaseType = kClassStarBase;
			NumberEnemyShips--;
		}
		break;
	case 1:
		{
			// do nothing
			BaseType = kClassBattleStation;
		}
		break;
	case 2:
		{			
			BaseType = kClassBaseStation;
			//NumberEnemyShips++;
			TotalBPV *= 1.2f;
			NumLQShips--;
		}
		break;
	}


	// Any size ship within the appropriate BPV and a 33% chance of a variant
	EnemyTeam->mCreateFleet(NumberEnemyShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 100, kStartPosition_X | kGroupAround );

	BaseTeam->mCreateShip (kMaxBPV, BaseType, BaseType, 0, kStartPosition_S);

	// create allied ship(s)
	AlliedTeam->mCreateFleet (2, 35, kClassFreighter, kClassFreighter, 0, kStartPosition_T | kGroupAround );
	AlliedTeam->mCreateFleet (NumMQShips, 45, kClassFreighter, kClassFreighter, 0, kStartPosition_V | kGroupAround );
	AlliedTeam->mCreateFleet (NumLQShips, 70, kClassFreighter, kClassFreighter, 0, kStartPosition_U | kGroupAround );

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* Ship;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipThrust (15);
	}

	Ship = BaseTeam->mGetFirstShip();

	int32 x = 0;
	int32 y = 0;
	Ship->mGetGridPosition( &x, &y );

	// configure allied ship(s)
	for (Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
	{
		Ship->mSetFinalDestination( x, y );
		Ship->mSetShipMaxSpeed( 12 );
	}

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure Enemy ship(s)
	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget( AlliedTeam->mGetFirstShip(), kHighPriority );
	}

	// display startup message
	fMission.mDisplayMessage (kMissionWarning);

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	AlliedHealth = 100.0f - AlliedTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			fMission.mSetTeamRelations (kBaseTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
			fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
			fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
			fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.5);
			fMission.mSetTeamRelations (kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
			fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
		break;

		/* ENF See if this is necessary
		case kNotifyAtDestination:
			Ship->mSetShipThrust (0);
			if (AlliedTeam->mAllInPosition())
			{
				EnemyTeam->mDisengageShips(kByMapEdge, kAbsolutePriority);
			}
			break;
		*/

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerDestroyed++;
			}
			else if (Ship->mGetOriginalTeam () == kAlliedTeam)
			{
				AlliedDestroyed++;
			}
			else
			{
				// see if this is all over.
				if ( Enemies == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerCaptured++;
			}
			else if (Ship->mGetOriginalTeam () == kAlliedTeam)
			{
				AlliedCaptured++;
			}
			else
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;
				
				if ( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	if ( PlayerDisengaged )
	{
		if ( ( PlayerHealth > EnemyHealth ) && ( AlliedHealth > 90 ) )
		{
			return kVictory;
		}
		else if ( ( PlayerHealth < ( EnemyHealth / 2 ) ) || ( AlliedHealth < 30 ) )
		{
			fMission.mAdjustDynaverse( kAdjustEconomy, -4, PlayerTeam->mGetRace() );
			return kDevastatingDefeat;
		}
		else 
		{
			fMission.mAdjustDynaverse( kAdjustEconomy, -2, PlayerTeam->mGetRace() );
			return kDefeat;
		}
	}


	if ( ( AlliedHealth > 80 ) &&  ( ( PlayerHealth > 90.0 ) || ( EnemyCaptured ) ) )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
		fMission.mAdjustDynaverse( kAdjustEconomy, +3, PlayerTeam->mGetRace() );
		return kAstoundingVictory;
	}

	if ( AlliedHealth < 25.0 )
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -2, PlayerTeam->mGetRace() );
		return kDefeat;
	}

	if ( PlayerHealth < 25.0 ) 
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -1, PlayerTeam->mGetRace() );
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	if (AlliedCaptured == 1)
	{
		fMission.mAddToDebriefingList (kAlliedShipCaptured_msg);
	}

	if (AlliedDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kAlliedShipDestroyed_msg);
	}

	if (AlliedCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kAlliedShipsCaptured_msg);
	}

	if (AlliedDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kAlliedShipsDestroyed_msg);
	}

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
