#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 AlliedCaptured = 0;
int32 AlliedDestroyed = 0;
int32 PirateCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float PirateHealth = 0.0;
float AlliedHealth = 0.0;
iTruth PlayerDisengaged = kFalse;
int32 FreightersKilled = 0;
int32 AsteroidAmbush = 0;		// 0 == no asteroids, 1 == asteroids
int32 HailedPirates = 0;		// 0 == no hail, 1 == hailed

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kPirateTeam,
	kBaseTeam,
	kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPirateTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kOrionMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = 150;
	Specs->fMinDate = 2;
	Specs->fMaxDate = 50;
	Specs->fWait = 3;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 80, 40, 20, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kPirateTeam, kOrion );
	fMission.mSetTeamRace (kAlliedTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kBaseTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPirateTeam, kAlliedTeam, kTeamAtWar, kGoalCapture, 1.0);
	fMission.mSetTeamRelations (kAlliedTeam, kPirateTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kPirateTeam, kBaseTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kBaseTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0);

	int32 map = 1;
	// pick a random map
	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
		}
		break;
	case 1:
		{
			AsteroidAmbush = 1;
			map = 2;
		}
		break;
	case 2:
		{
			AsteroidAmbush = 1;
			map = 2;
		}
		break;
	}

	return (map);
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* Ship;


	int32 NumLQShips = 1;
	int32 NumMQShips = 2;
	int32 NumberPirateShips = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount());
	
	int32 TotalBPV = PlayerTeam->mGetCombatBPV();
	int32 DifLev = fMission.mGetDifficultyLevel();

	eClassTypes BaseType = kClassBaseStation;

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.70f;
			NumMQShips++;
			BaseType = kClassStarBase;
		}
		break;
	case 1:
		{
			// do nothing
			BaseType = kClassBattleStation;
		}
		break;
	case 2:
		{			
			BaseType = kClassBaseStation;
			NumberPirateShips++;
			TotalBPV *= 1.2f;
			NumLQShips--;
		}
		break;
	}


	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	if (Mode == 0)
	{
		// create freighters and destination base
		BaseTeam->mCreateShip (kMaxBPV, BaseType, BaseType, 25, kStartPosition_L);

		// create allied ship(s)
		AlliedTeam->mCreateFleet (2, 35, kClassFreighter, kClassFreighter, 0, kStartPosition_H | kGroupAround );
		AlliedTeam->mCreateFleet (NumMQShips, 45, kClassFreighter, kClassFreighter, 0, kStartPosition_I | kGroupAround );
		AlliedTeam->mCreateFleet (NumLQShips, 45, kClassFreighter, kClassFreighter, 0, kStartPosition_J  | kGroupAround );

		// create Da Pirates
		PirateTeam->mCreateFleet(NumberPirateShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_X | kGroupAround );
	}
	else if (Mode == 1)
	{
		// create Da Pirate Reinforcements
		if (AsteroidAmbush == 0)
		{
			Ship = PlayerTeam->mGetFirstShip();
			Ship->mGetGridPosition (&OldX, &OldY);
			NewX = OldX + fMission.mRandomInt32 (4, 8);
			NewY = OldY + fMission.mRandomInt32 (-2, 2);
			PirateTeam->mCreateShip (TotalBPV, kClassFrigate, kClassDreadnought, 0, kStartPosition_Specified, NewX, NewY);
			PirateTeam->mCreateShip (TotalBPV, kClassFrigate, kClassDreadnought, 25, kStartPosition_Specified, NewX, NewY);
		}
		else
		{
			PirateTeam->mCreateShip (TotalBPV, kClassDestroyer, kClassDreadnought, 0, kStartPosition_Z);
			PirateTeam->mCreateShip (TotalBPV, kClassFrigate, kClassDreadnought, 25, kStartPosition_Z);
		}
	}
	else if (Mode == 2)
	{
		AlliedTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_M);		
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = PirateTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipThrust (10);
	}

	// configure allied ship
	for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
	{
		if (Ship->mGetClassType() == kClassFreighter)
		{
			Ship->mSetFinalDestination();
			Ship->mSetShipThrust (10);
			Ship->mSetShipMaxSpeed( 12 );
			Ship->mSetCommButton (1, kButtonHail_text);
		}
	}

	// configure pirate ships
	for (Ship = PirateTeam->mGetFirstShip (); Ship != NULL; Ship = PirateTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	fMission.mDisplayMessage (kMissionWarning, kScienceScriptMessage);
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	fMission.mSetTimedInterval (kTimerTurnsSet, 15);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);

	if (Ship->mGetTeam() == kPirateTeam)
	{
		if (HailedPirates == 0)
		{
			// configure pirate ships
			for (Ship = PirateTeam->mGetFirstShip (); Ship != NULL; Ship = PirateTeam->mGetNextShip ())
			{
				Ship->mSetCommButton (1, kButtonHail_text);
			}
		}
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);

	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	AlliedHealth = 100.0f - AlliedTeam->mGetTotalDamagePercent();
	PirateHealth = 100.0f - PirateTeam->mGetTotalDamagePercent();

	int32 Pirates = PirateTeam->mGetShipsInPlayCount();
	switch (Event)
	{
		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				return kPlayerWins;
			}
		break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					mCreateMissionShips (1);
					fMission.mDisplayMessage (kPirateReinforcements_msg, kScienceScriptMessage);
					break;

				case kTimerLimitSet:
					fMission.mSetTeamRelations (kBaseTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPirateTeam, kAlliedTeam, kTeamAtWar, kGoalCapture, 1.0);
					fMission.mSetTeamRelations (kAlliedTeam, kPirateTeam, kTeamAtWar, kGoalIgnore, 1.0);
					fMission.mSetTeamRelations (kPirateTeam, kBaseTeam, kTeamAtWar, kGoalIgnore, 1.0);
					fMission.mSetTeamRelations (kBaseTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0);
					break;
			}
			break;

		case kNotifyCommButtonPress:
			if (Ship->mGetTeam() == kPirateTeam)
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						HailedPirates = 1;
						fMission.mDisplayMessage (kHailPirates_msg, kMissionScriptMessage);
						for (Ship = PirateTeam->mGetFirstShip (); Ship != NULL; Ship = PirateTeam->mGetNextShip ())
						{
							Ship->mSetCommButton (1, kButtonThreaten_text);
						}
						break;

					case kButtonThreaten_text:
						if (PirateTeam->mGetShipsInPlayCount() > 1)
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kThreatenPiratesNoA_msg, kThreatenPiratesNoC_msg), kMissionScriptMessage);
							Ship->mSetCommButton (1, kButtonThreaten_text);
						}
						else // CNF: if the player's ship is in better shape than the last pirate
						{
							fMission.mDisplayMessage (kThreatenPiratesYes_msg, kMissionScriptMessage);
							PirateTeam->mDisengageShips(kByMapEdge, kAbsolutePriority);
						}
						break;
				}
			}
			else if (Ship->mGetClassType() == kClassFreighter)
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						fMission.mDisplayMessage (kHailFreighters_msg, kMissionScriptMessage);
						for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
						{
							if (Ship->mGetClassType() == kClassFreighter)
							{
								Ship->mSetCommButton (1, kButtonStop_text);
								Ship->mSetCommButton (2, kButtonBreak_text);
							}
						}
						break;

					case kButtonStop_text:
						// CNF: make freighters group up and stop
						for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
						{
							if (Ship->mGetClassType() == kClassFreighter)
							{
								Ship->mSetCommButton (2, kButtonHome_text);
								Ship->mSetShipMaxSpeed( 0 );
							}
						}
						break;

					case kButtonBreak_text:
						for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
						{
							if (Ship->mGetClassType() == kClassFreighter)
							{
								Ship->mSetCommButton (1, kButtonHome_text);
								Ship->mDisengage( kByMapEdge, kAbsolutePriority );
								Ship->mSetShipMaxSpeed( 12 );
							}
						}
						break;

					case kButtonHome_text:
						for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
						{
							if (Ship->mGetClassType() == kClassFreighter)
							{
								Ship->mSetCommButton (1, kButtonStop_text);
								Ship->mSetCommButton (2, kButtonBreak_text);
								Ship->mSetFinalDestination();
								Ship->mSetShipMaxSpeed( 12 );
							}
						}
						break;
				}
			}
			else if (Ship->mGetClassType() == kClassBaseStation)
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						fMission.mDisplayMessage (kHailBase_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonHelp_text);
						break;

					case kButtonHelp_text:
						fMission.mDisplayMessage (kBaseRequestHelp_msg, kMissionScriptMessage);
						mCreateMissionShips (2);
						break;
				}
			}

			break;

			/* ENF See if this is needed - replace with pirate take damage & run..
		case kNotifyAtDestination:
			Ship->mSetShipThrust (0);
			if (AlliedTeam->mAllInPosition())
			{
				PirateTeam->mDisengageShips(kByMapEdge, kAbsolutePriority);
			}
			break;
			*/

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerDestroyed++;
			}
			else if (Ship->mGetOriginalTeam () == kAlliedTeam)
			{
				AlliedDestroyed++;
			}
			else
			{
				// see if this is all over.
				if (  PirateTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerCaptured++;
			}
			else if(( Ship->mGetOriginalTeam() == kAlliedTeam ) && ( Ship->mGetTeam() != kPirateTeam ))
			{
				AlliedCaptured++;
				Ship->mDisengage( kByMapEdge, kAbsolutePriority );
			}
			else if(( Ship->mGetOriginalTeam() == kAlliedTeam ) && ( Ship->mGetTeam() == kPirateTeam ))
			{
				Ship->mSetFinalDestination();
				AlliedCaptured--;

				if( AlliedCaptured <= 0 )
				{
					AlliedCaptured = 0;
				}
			}
			else
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				PirateCaptured++;
				
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
				
/*				if (  PirateTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
*/			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	if ( PlayerDisengaged )
	{
		if ( ( PlayerHealth > PirateHealth ) && ( AlliedHealth > 90 ) )
		{
			return kVictory;
		}
		else if ( ( PlayerHealth < ( PirateHealth / 2 ) ) || ( AlliedHealth < 30 ) )
		{
			fMission.mAdjustDynaverse( kAdjustEconomy, -4, PlayerTeam->mGetRace() );
			return kDevastatingDefeat;
		}
		else 
		{
			fMission.mAdjustDynaverse( kAdjustEconomy, -2, PlayerTeam->mGetRace() );
			return kDefeat;
		}
	}


	if ( ( AlliedHealth > 80 ) &&  ( ( PlayerHealth > 90.0 ) || ( PirateCaptured ) ) )
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, +3, PlayerTeam->mGetRace() );
		return kAstoundingVictory;
	}

	if ( AlliedHealth < 25.0 )
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -2, PlayerTeam->mGetRace() );
		return kDefeat;
	}

	if ( PlayerHealth < 25.0 ) 
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -1, PlayerTeam->mGetRace() );
		return kDraw;
	}

	// By default
	fMission.mAdjustDynaverse( kAdjustEconomy, +1, PlayerTeam->mGetRace() );
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	if (AlliedCaptured == 1)
	{
		fMission.mAddToDebriefingList (kAlliedShipCaptured_msg);
	}

	if (AlliedDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kAlliedShipDestroyed_msg);
	}

	if (AlliedCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kAlliedShipsCaptured_msg);
	}

	if (AlliedDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kAlliedShipsDestroyed_msg);
	}

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (PirateCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (PirateCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
