#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

//---------------------------------------------------------------------------------------------
// initialize mission specific counters
int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 EnemyDestroyed = 0;
int32 FreightersCaptured = 0;
int32 FreightersDestroyed = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
float FreighterHealth = 0.0;
iTruth EnemyResponded = kFalse;
iTruth PlayerDisengaged = kFalse;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBaseTeam,
	kFreighterTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kEnemyBorder | kWarzone | kEnemyColonial | kEnemyCapital;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecTwoShip | MissSpecThreeShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 11;
	Specs->fMaxDate = 50;
	Specs->fWait = 4;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 100, 50, 25, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kBaseTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kFreighterTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations (kEnemyTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	int32 map = 2;
	// pick a random map
	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
		}
		break;
	case 1:
		{
			map = 1;
		}
		break;
	case 2:
		{
			map = 1;
		}
		break;
	}

	return (map);
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	//Ship.fShipPosition = kStartPosition_G;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// Get the total BPV of the player and modify by 80% to 130%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * 1.1;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.8;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// pick a random number of ships
	int32 NumberEnemyShips = fMission.mRandomInt32 (1, PlayerTeam->mGetShipsInPlayCount());
	int32 NumberFreightersShips = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount()*2+1);
	int32 DifLev = fMission.mGetDifficultyLevel();

	eClassTypes BaseType = kClassBaseStation;

	switch ( DifLev )
	{
	case 0:
		{
			NumberFreightersShips--;
			TotalBPV *= 0.70f;
			BaseType = kClassBaseStation;
		}
		break;
	case 1:
		{
			if ( NumberEnemyShips > 1 )
			{
				NumberEnemyShips--;
			}
			// do nothing
			BaseType = kClassBattleStation;
		}
		break;
	case 2:
		{
			NumberFreightersShips++;
			TotalBPV *= 1.3f;
			BaseType = kClassStarBase;
		}
		break;
	}

	if (Mode == 0)
	{
		int32 Small = 0;
		int32 Medium = 0;
		int32 Large = 0;

		BaseTeam->mCreateShip (kMaxBPV, BaseType, BaseType, 0, kStartPosition_R);

		// create the convoy
		if ( NumberFreightersShips < 3 )
		{
			Small = NumberFreightersShips;
			// 1 or 2 small freighters
			FreighterTeam->mCreateFleet ( Small, 35, kClassFreighter, kClassFreighter, 0, kStartPosition_U | kGroupAround );
		}
		else if ( NumberFreightersShips < 5 )
		{
			// 1 or 2 small freighters
			// plus 1 or 2 medium freighters
			Small = NumberFreightersShips / 2;
			Medium = NumberFreightersShips - Small;
			FreighterTeam->mCreateFleet ( Small, 35, kClassFreighter, kClassFreighter, 0, kStartPosition_U | kGroupAround );
			FreighterTeam->mCreateFleet ( Medium, 45, kClassFreighter, kClassFreighter, 0, kStartPosition_Z | kGroupAround );
		}
		else
		{
			// 1 or 2 small freighters
			// plus 1 or 2 medium freighters
			// plus 1 or 2 or 3 large freighters
			Small = NumberFreightersShips / 3;
			Medium = Small;
			Large = NumberFreightersShips - Small - Medium;
			FreighterTeam->mCreateFleet ( Small, 35, kClassFreighter, kClassFreighter, 0, kStartPosition_U | kGroupAround );
			FreighterTeam->mCreateFleet ( Medium, 45, kClassFreighter, kClassFreighter, 0, kStartPosition_Z | kGroupAround );
			FreighterTeam->mCreateFleet ( Large, 70, kClassFreighter, kClassFreighter, 0, kStartPosition_Y | kGroupAround );
		}

		EnemyTeam->mCreateFleet ( NumberEnemyShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_V  | kGroupAround );
		return;
	}


	// create enemy reinforcements
	if (Mode == 1)
	{
		EnemyTeam->mCreateShip (HighBPV, kClassLightCruiser, kClassDreadnought, 20, kStartPosition_S);
		if (fMission.mRandomInt32 (1, 100) >= 50)
		{
			EnemyTeam->mCreateShip (HighBPV, kClassLightCruiser, kClassDreadnought, 20, kStartPosition_T);
		}

		// re balance after start
		int32 PBPV = PlayerTeam->mGetCombatBPV();
		int32 EBPV = EnemyTeam->mGetCombatBPV();
		BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;
		return;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (fMission.mRandomInt32 (kBriefingA_msg, kBriefingC_msg));
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;


	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip ())
			{
				Ship->mDamageSystem( kDamageWeaponsAndSystems, 50 );
				Ship->mSetShipMaxSpeed( 8 );
			}
		}
		break;
	case 1:
		{
			for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip ())
			{
				Ship->mDamageSystem( kDamageWeaponsAndSystems, 20 );
				Ship->mSetShipMaxSpeed( 12 );
			}
		}
		break;
	case 2:
		{
			for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip ())
			{
				Ship->mSetShipMaxSpeed( 16 );
			}
		}
		break;
	}

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipThrust (10);
	}

	Ship = BaseTeam->mGetFirstShip();

	int32 x = 0;
	int32 y = 0;
	Ship->mGetGridPosition( &x, &y );

	// configure Freighter ship(s)
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip ())
	{
		Ship->mSetFinalDestination( x, y );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetFinalDestination( x, y );
	}


	fMission.mSetTimedInterval (kTimerTurnsSet, 10);	
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();
	FreighterHealth = 100.0f - FreighterTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();
	int32 Freighters = FreighterTeam->mGetShipsInPlayCount();

	switch (Event)
	{
		case kNotifyDamage:
		{
			if ( ( Ship->mGetTeam () == kFreighterTeam ) && ( EnemyResponded == kFalse ) )
			{
				EnemyResponded = kTrue;
				for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
				{
					Ship->mSetShipTarget( PlayerTeam->mGetFirstShipInPlay(), kAbsolutePriority );
				}
			}
		}

		case kNotifyAtTimedInterval:
			{
				fMission.mDisplayMessage (kReinforcements_msg);
				mCreateMissionShips (1);
			}
			break;

		case kNotifyCommButtonPress:
			{
				if (Ship->mGetTeam() == kFreighterTeam)
				{
					fMission.mDisplayMessage (kConvoyResponse_msg, kMissionScriptMessage);
				}
				else if (EnemyTeam->mGetShipsInPlayCount() > 1)
				{
					fMission.mDisplayMessage (kEnemyThreat_msg, kMissionScriptMessage);
				}
				else
				{
					fMission.mDisplayMessage (kEnemyWarning_msg, kMissionScriptMessage);
				}
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				fMission.mAddToBonusPrestige( 25 );
				return kPlayerWins;
			}
		break;

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerCaptured++;
			}
			else if (Ship->mGetOriginalTeam () == kFreighterTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus / 2 );
				FreightersCaptured++;
				FreightersDestroyed++;
				if ( ( FreighterTeam->mGetShipsInPlayCount() == 0 ) && ( Enemies == 0 ) )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			else
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;
				
				if ( ( EnemyTeam->mGetShipsInPlayCount() == 0 ) && ( Freighters == 0 ) )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetOriginalTeam () == kPlayerTeam)
				{
					PlayerDestroyed++;
					if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						return kPlayerDestroyed;
					}
				}
				else if (Ship->mGetOriginalTeam () == kFreighterTeam)
				{
					FreightersDestroyed++;
					fMission.mAddToBonusPrestige( 20 );
				}
				else
				{
				}

				if ( ( Enemies == 0 ) && ( Freighters == 0 ) )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}

			}
			break;

	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if ( FreighterHealth == 0.0 )
	{
		fMission.mAddToBonusPrestige( 30 );
	}

	if ( PlayerDisengaged )
	{
		if ( ( PlayerHealth > 10.0 ) && ( FreightersDestroyed > 1 ) )
		{
			fMission.mAdjustDynaverse( kAdjustEconomy, -2, EnemyTeam->mGetRace() );
			return kVictory;
		}

		if ( ( PlayerHealth > 20.0 ) && ( FreightersDestroyed == 1 ) )
		{
			return kDraw;
		}

		if ( ( PlayerHealth < 15.0 ) || ( FreightersDestroyed == 0 ) )
		{
			return kDevastatingDefeat;
		}

		// By default
		return kDefeat;
	}

	if ( ( PlayerHealth > 50.0 ) && ( FreightersDestroyed > 1 ) )
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -4, EnemyTeam->mGetRace() );
		return kAstoundingVictory;
	}

	if ( ( PlayerHealth > 15.0 ) && ( FreightersDestroyed > 1 ) )
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -2, EnemyTeam->mGetRace() );
		return kVictory;
	}

	if ( ( PlayerHealth > 25.0 ) && ( FreightersDestroyed == 1 ) )
	{
		fMission.mAdjustDynaverse( kAdjustEconomy, -2, EnemyTeam->mGetRace() );
		return kVictory;
	}

	if ( PlayerHealth < 15.0  )
	{
		return kDefeat;
	}

	if ( ( PlayerHealth < 15.0 ) && ( FreightersDestroyed == 0 ) )
	{
		return kDevastatingDefeat;
	}

	// By default
	return kDraw;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}

	if (FreightersCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleFreighter_msg);
	}

	if (FreightersCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleFreighters_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
