#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 MissionType = 0;				// A == vital plans, B == important parts, C == sensitive cargo 
int32 MissionAct = 0;
int32 PlayerCaptured = 0;
int32 EnemyCaptured = 0;
int32 EnemyDestroyed = 0;
bool BaseDestroyed = false;
bool RequestHelp = false;
bool PirateAttack = false;
int32 DifLev = 1;
iTruth PlayerDisengaged = kFalse;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kAlliedTeam, kPirateTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kWarzone;
	Specs->fPlayerRace = kFedMission | kRomulanMission | kLyranMission | kHydranMission;
	Specs->fEnemyRace = kKlingMission | kLyranMission | kGornMission | kHydranMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = 150;
	Specs->fMinDate = 0;
	Specs->fMaxDate = 30;
	Specs->fWait = 6;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 80, 40, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kAlliedTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kPirateTeam, kOrion);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPirateTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPirateTeam, kEnemyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPirateTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	// pick a map
	MissionType = fMission.mRandomInt32 (1, 3);	// 3 different missions, each mission uses specific map
	return MissionType;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tShipInfo* Ship;

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV ();
	int32 ShipX = 0;
	int32 ShipY = 0;
	int32 BaseX = 0;
	int32 BaseY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	DifLev = fMission.mGetDifficultyLevel();


	if( Mode == 0 )		// initial forces
	{
		// Create three ships equal to the player's ship
		EnemyTeam->mCreateShip( PlayerBPV * 1.3f, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_Z );
		if( PlayerTeam->mGetRace() != kRomulan )
		{
			EnemyTeam->mCreateShip( PlayerBPV, kClassDestroyer, kClassLightCruiser, 0, kStartPosition_Y );
			EnemyTeam->mCreateShip( PlayerBPV, kClassLightCruiser, kClassHeavyCruiser, 0, kStartPosition_X );
		}

		// create the allied station
		AlliedTeam->mCreateShip( kMaxBPV, kClassBattleStation, kClassBattleStation, 0, kStartPosition_H );
		AlliedTeam->mCreateFleet( 2, kMaxBPV, kClassLightCruiser, kClassLightCruiser, 0, kStartPosition_I );
	}
	else if( Mode == 1 )	// enemy reinforcements
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &ShipX, &ShipY );
		NewX = ShipX + fMission.mRandomInt32( -2, 2 );
		NewY = ShipY + fMission.mRandomInt32( 14, 18 );
		if( PlayerTeam->mGetRace() != kRomulan )
		{
			EnemyTeam->mCreateShip( PlayerBPV, kClassFrigate, kClassHeavyBattlecruiser, 0, kStartPosition_Specified, NewX, NewY );
		}
		EnemyTeam->mCreateShip( PlayerBPV, kClassFrigate, kClassLightCruiser, 0, kStartPosition_Specified, NewX, NewY );
	}
	else if( Mode == 2 )	// pirate attack on the station if the player calls for reinforcements
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &ShipX, &ShipY );
		Ship = AlliedTeam->mGetFirstShip();
		Ship->mGetGridPosition( &BaseX, &BaseY );
		NewX = BaseX + fMission.mRandomInt32( 0, 4 );

		if( BaseY < ShipY )
		{
			NewY = BaseY + fMission.mRandomInt32( -18, -14 );
		}
		else
		{
			NewY = ShipY + fMission.mRandomInt32( -18, -14 );
		}

		PirateTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		PirateTeam->mCreateShip( kMaxBPV, kClassLightCruiser, kClassLightCruiser, 0, kStartPosition_Specified, NewX, NewY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// use the same briefing as the map number
	return MissionType;
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tShipInfo* Ship;

	int32 RandomShip = 0;
	int32 BaseX = 0;
	int32 BaseY = 0;

	// first find out the location of the base
	Ship = AlliedTeam->mGetFirstShip ();
	Ship->mGetGridPosition (&BaseX, &BaseY);

	// configure player ship
	Ship = PlayerTeam->mGetFirstShip ();
	Ship->mSetShipPlayerControlled ();
	Ship->mSetShipThrust (20);
	Ship->mTrackTacticalDistance (20.0, BaseX, BaseY);

	// configure enemy ships
	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	fMission.mSetTimedInterval (kTimerLimitSet, 3);	// enemy reinforcements
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);

	if (Ship->mGetTeam () != kEnemyTeam)
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tShipInfo* PlayerShip;

	int32 RandomShip = 0;

	switch (Event)
	{
		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				return kPlayerWins;
			}
		break;

		case kNotifyAtTimedInterval:
			if (MissionAct == 0)
			{
				if (MissionType == 1)
				{
					fMission.mDisplayMessage (kBaseWarning_msg, kMissionScriptMessage);
				}
				else
				{
					fMission.mDisplayMessage (kBaseWarningPirates_msg, kMissionScriptMessage);
				}

				Ship = AlliedTeam->mGetFirstShip();
				Ship->mSetCommButton (1, kButtonRequestHelp_text);
				MissionAct++;
				fMission.mSetTimedInterval (kTimerTurnsSet, 1);	
			}
			else if (MissionAct == 1)
			{
				fMission.mDisplayMessage (kEngineTrouble_msg, kScienceScriptMessage);
				MissionAct++;
				fMission.mSetTimedInterval (kTimerTurnsSet, 1);	

				PlayerShip = PlayerTeam->mGetFirstShip();
				PlayerShip->mDamageSystem( kDamageLeftWarpEngine, 5 * DifLev );
				PlayerShip->mDamageSystem( kDamageCenterWarpEngine, 5 * DifLev );
			}
			else if (MissionAct == 2)
			{
				fMission.mDisplayMessage (kEngineTrouble2_msg, kScienceScriptMessage);
				MissionAct++;
				fMission.mSetTimedInterval (kTimerTurnsSet, 1);	

				PlayerShip = PlayerTeam->mGetFirstShip();
				PlayerShip->mDamageSystem( kDamageLeftWarpEngine, 15 * DifLev );
				PlayerShip->mDamageSystem( kDamageCenterWarpEngine, 15 * DifLev );
			}
			else if (MissionAct == 3)
			{
				fMission.mDisplayMessage (kEngineTrouble3_msg, kScienceScriptMessage);
				MissionAct++;
				fMission.mSetTimedInterval (kTimerTurnsSet, 3);	// enemy reinforcements

				PlayerShip = PlayerTeam->mGetFirstShip();
				PlayerShip->mDamageSystem( kDamageLeftWarpEngine, 25 * DifLev );
				PlayerShip->mDamageSystem( kDamageCenterWarpEngine, 25 * DifLev );
			}
			else if (MissionAct == 4)
			{
				fMission.mDisplayMessage (kEnemyReinforcements_msg, kScienceScriptMessage);
				MissionAct++;
				mCreateMissionShips (1);

				for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
				{
					Ship->mClearCommButton (1);
				}
			}
			break;

		case kNotifyAtDistance:
			EnemyTeam->mDisengageShips(kByMapEdge, kAbsolutePriority);
			fMission.mDisplayMessage (kEnemyRetreating_msg, kScienceScriptMessage);
			Ship = AlliedTeam->mGetFirstShip ();
			Ship->mSetCommButton (2, kButtonBeamCargo_text);
			break;

		case kNotifyAtDeath:
			if( Ship->mGetClassType () == kClassBattleStation )
			{
				fMission.mDisplayMessage( kBaseDestroyed_msg, kMissionScriptMessage );
				BaseDestroyed = true;
				return kPlayerLoses;
			}

			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;
			}
			break;

		case kNotifyOnCapture:
			if( ( Ship->mGetOriginalTeam() == kPlayerTeam ) || ( Ship->mGetOriginalTeam() == kAlliedTeam ) )
			{
				PlayerCaptured++;
			}
			else
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;
			}
			break;

		case kNotifyCommButtonPress:
			if (Ship->mGetTeam() == kAlliedTeam)
			{
				switch (fEventID)
				{
					case kButtonRequestHelp_text:
						fMission.mDisplayMessage (kHelpOnTheWay_msg, kMissionScriptMessage);
						RequestHelp = true;

						fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.5, kTrue);

						if ((MissionType == 3) || ((MissionType == 2) && fMission.mRandomInt32 (1, 2) == 2))
						{
							mCreateMissionShips( 2 );	// pirate attack!
							PirateAttack = true;
						}
						break;

					case kButtonBeamCargo_text:
						if (PirateTeam->mGetShipsInPlayCount == 0)
						{
							if (Ship->mGetTacticalDistance (PlayerTeam->mGetFirstShip()) < 3)
							{
								fMission.mDisplayMessage (kBeamCargo_msg, kMissionScriptMessage);
								return kPlayerWins;
							}
							else
							{
								fMission.mDisplayMessage (kNoBeamDistance_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonBeamCargo_text);
							}
						}
						else
						{
							fMission.mDisplayMessage (kNoBeamPirates_msg, kMissionScriptMessage);
							Ship->mSetCommButton (2, kButtonBeamCargo_text);
						}
						break;
				}
			}
			else if (Ship->mGetTeam() == kEnemyTeam)
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						fMission.mDisplayMessage (kEnemyThreat_msg, kMissionScriptMessage, Ship);
						Ship->mSetCommButton (1, kButtonDelay_text);
						Ship->mSetCommButton (2, kButtonThreaten_text);
						break;

					case kButtonDelay_text:
						fMission.mDisplayMessage (kSendEnemyDelay_msg, kNonCommScriptMessage);
						fMission.mDisplayMessage (kEnemyDelayed_msg, kMissionScriptMessage);
						fMission.mSetTimedInterval (kTimerTurnsSet, 5);	// enemy reinforcements

						for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
						{
							Ship->mClearCommButton (1);
							Ship->mClearCommButton (2);
						}
						break;

					case kButtonThreaten_text:
						fMission.mDisplayMessage (kSendEnemyThreat_msg, kNonCommScriptMessage);

						if (EnemyTeam->mGetShipsInPlayCount() == 1)
						{
							fMission.mDisplayMessage (kEnemyThreatened_msg, kMissionScriptMessage);
							EnemyTeam->mDisengageShips(kByMapEdge, kAbsolutePriority);
							Ship = AlliedTeam->mGetFirstShip ();
							Ship->mSetCommButton (2, kButtonBeamCargo_text);
						}
						else
						{
							fMission.mDisplayMessage (kEnemyNotThreatened_msg, kMissionScriptMessage);
						}

						for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
						{
							Ship->mClearCommButton (1);
							Ship->mClearCommButton (2);
						}
						break;
				}
			}
			else if (Ship->mGetTeam() == kPirateTeam)
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						fMission.mDisplayMessage (kPiratesNoResponse_msg, kMissionScriptMessage);
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	// determine who gets the combat BPV
	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 EnemyBPV = EnemyTeam->mGetCombatBPV();

	if ( PlayerDisengaged )
	{
		return kDevastatingDefeat;
	}

	if( PirateAttack == true )
	{
		PlayerBPV = PlayerBPV + 50;
	}
	if( RequestHelp == true )
	{
		PlayerBPV = PlayerBPV - 50;
	}

	if( BaseDestroyed == true )
	{
		PlayerBPV = 0;
	}

	if (PlayerBPV > EnemyBPV)
	{
		PlayerBPV = EnemyBPV - PlayerBPV;
		EnemyBPV = 0;
	}
	else
	{
		EnemyBPV = PlayerBPV - EnemyBPV;
		PlayerBPV = 0;
	}

	// add in assesed damage from the opposing team
	PlayerBPV += EnemyTeam->mAssessDamages(PlayerTeam);
	EnemyBPV += PlayerTeam->mAssessDamages(EnemyTeam);

	return (fMission.mGetVictoryLevel (PlayerBPV, EnemyBPV));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
