#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 FreightersDestroyed = 0;
int32 FreightersCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
float RepairHealth = 0.0;
iTruth PlayerDisengaged = kFalse;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBackupTeam,
	kRepairTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kEnemyBorder | kWarzone | kEnemyColonial | kEnemyCapital;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecOneShip | MissSpecTwoShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 8;
	Specs->fMaxDate = 40;
	Specs->fWait = 3;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 80, 40, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kBackupTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kRepairTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kRepairTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kRepairTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBackupTeam, kRepairTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	int32 map = 1;
	// pick a random map
	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
		}
		break;
	case 1:
		{
		}
		break;
	case 2:
		{
			map = 2;
		}
		break;
	}

	return (map);
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = fMission.mRandomInt32 (kStartPosition_G, kStartPosition_J);
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BackupTeam = fMission.mGetTeamHandle (kBackupTeam);
	tTeamInfo* RepairTeam = fMission.mGetTeamHandle (kRepairTeam);
	tShipInfo* Ship;

	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 TotalBPV = fMission.mRandomInt32 (PlayerBPV * 1.8, PlayerBPV* 2.5);
	int32 NumRepairFreighters = 2;
	int32 FreighterBPV = 45;
	int32 NumReinforcements = 3;

	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.70f;
			NumReinforcements = 1;
			FreighterBPV = 35;
		}
		break;
	case 1:
		{
			// do nothing
		}
		break;
	case 2:
		{
			TotalBPV *= 1.25f;
			NumRepairFreighters++;
			NumReinforcements++;
			FreighterBPV = 70;
		}
		break;
	}

	if (Mode == 0)
	{
		// Any size ship within the appropriate BPV and a 33% chance of a variant
		EnemyTeam->mCreateShip (TotalBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_Z);
		RepairTeam->mCreateFleet (NumRepairFreighters, FreighterBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_X | kGroupAround);
	}
	else if (Mode == 1)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&OldX, &OldY);
		NewX = OldX + fMission.mRandomInt32 (10, 15);
		NewY = OldY + fMission.mRandomInt32 (-3, 3);
		BackupTeam->mCreateFleet (NumReinforcements, TotalBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_Specified, NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	int32 MessageNumber = fMission.mRandomInt32 (kBriefingA_msg, kBriefingC_msg);
	return (MessageNumber);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* RepairTeam = fMission.mGetTeamHandle (kRepairTeam);
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;


	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipThrust (5);
	}

	int32 DifLev = fMission.mGetDifficultyLevel();

	float Damage = 70.0;
	switch ( DifLev )
	{
	case 0:
		{
			Damage = 85.0;
		}
		break;
	case 1:
		{
		}
		break;
	case 2:
		{
			Damage = 55.0;
		}
		break;
	}

	Ship = EnemyTeam->mGetFirstShip();
	Ship->mDamageSystem (kDamageMaxWeapons, Damage);
	Ship->mDamageSystem (kDamageMaxSystems, Damage);
	Ship->mDamageSystem (kDamageLeftWarpEngine, Damage);
	Ship->mDamageSystem (kDamageCenterWarpEngine, Damage);
	Ship->mDamageSystem (kDamageMaxShields, Damage/2);
	Ship->mSetCommButton (1, kButtonHail_text);

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);
	fMission.mSetTimedInterval (kTimerTurnsSet, fMission.mRandomInt32 (8, 12));
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BackupTeam = fMission.mGetTeamHandle (kBackupTeam);
	tTeamInfo* RepairTeam = fMission.mGetTeamHandle (kRepairTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();
	RepairHealth = 100.0f - RepairTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();
	int32 Freighters = RepairTeam->mGetShipsInPlayCount();
	int32 PlayerFleet = PlayerTeam->mGetShipsInPlayCount();

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			fMission.mDisplayMessage (kEnemyReinforcements_msg, kScienceScriptMessage);
			mCreateMissionShips (1);
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage (kEnemyNoResponse_msg, kMissionScriptMessage);
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
		break;

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerCaptured++;
			}
			else if (Ship->mGetOriginalTeam () == kRepairTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus / 2 );
				FreightersCaptured++;
				Freighters--;
			}
			else if (Ship->mGetOriginalTeam () == kEnemyTeam)
			{
				EnemyCaptured++;
				Enemies--;
				
			}
			else if (Ship->mGetOriginalTeam () == kBackupTeam)
			{
				// do nothing for now...
				fMission.mAddToBonusPrestige( kCaptureBonus / 2 );
			}
			if ( (Enemies == 0) && (Freighters == 0) )
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetOriginalTeam () == kPlayerTeam)
				{
					PlayerDestroyed++;
					if ( PlayerFleet ==  0 )
					{
						return kPlayerDestroyed;
					}
				}
				else if (Ship->mGetOriginalTeam () == kRepairTeam)
				{
					FreightersDestroyed++;
				}
					// see if this is all over.
				if ( (Enemies == 0)  && ( Freighters == 0 ) )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
				return kGameInProgress;
				break;
			}
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if ( PlayerDisengaged )
	{
		if ( EnemyHealth == 0.0 )
		{
			if ( ( PlayerHealth > 90.0 ) || ( FreightersCaptured > 0 ) || ( FreightersDestroyed > 1 ) )
			{
				return kAstoundingVictory;
			}
			
			if ( ( PlayerHealth > 60.0 ) || ( FreightersDestroyed > 0 ) )
			{
				return kVictory;
			}

			if ( PlayerHealth > 40.0 )
			{
				return kDraw;
			}

			return kDefeat;			
		}
		else
		{
			return kDevastatingDefeat;
		}
	}


	if ( ( PlayerHealth > 90.0 ) || ( EnemyCaptured ) || ( FreightersCaptured > 0 ) || ( FreightersDestroyed > 1 ) )
	{
		return kAstoundingVictory;
	}

	if ( ( PlayerHealth > 60.0 ) || ( FreightersDestroyed > 0 ) )
	{
		return kVictory;
	}

	if ( PlayerHealth < 15.0 ) 
	{
		return kDevastatingDefeat;
	}

	if ( PlayerHealth < 25.0 ) 
	{
		return kDefeat;
	}

	// By default
	return kDraw;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
