#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 PlayerCaptured = 0;
int32 CourierCaptured = 0;
int32 gStartPosition = 0;
int32 gAct = 0;
int32 gEnemiesTakenCareOf = 0;
iTruth gCourierTakenCareOf = kFalse;
iTruth gCourierOuterRing = kTrue;
int32 MissionSegment = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (Courier, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kCourierTeam,
	kBaseTeam,
	kFreighterTeam,
	kLineManTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary Courier
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kEnemyBorder | kAlliedBorder | kWarzone;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 3;
	Specs->fMaxDate = 30;
	Specs->fWait = 10;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		Courier race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 100, 50, 5, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFreighterTeam, info.fPlayerRace);

	fMission.mSetTeamRace (kCourierTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kBaseTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kLineManTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kCourierTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kLineManTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kCourierTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kLineManTeam, kCourierTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLineManTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLineManTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLineManTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kEnemyTeam, kCourierTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kLineManTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	gStartPosition = fMission.mRandomInt32 (kStartPosition_I, kStartPosition_J);

	Ship.fShipPosition = gStartPosition;

	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* CourierTeam = fMission.mGetTeamHandle (kCourierTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* LineManTeam = fMission.mGetTeamHandle (kLineManTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .7;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.2;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
			// a little smaller BPV
			TotalBPV = TotalBPV * 0.7f;
		}
		break;
	case 1:
		{
			// do nothing for medium
		}
		break;
	case 2:
		{
			// a little more BPV and a LineMan to block for the courier
			TotalBPV = TotalBPV * 1.2f;	
			LineManTeam->mCreateShip( LowBPV, kClassFrigate, kClassBattleship, 50, kStartPosition_Z);
		}
		break;
	}

	if ( Mode == 0 )
	{
		int32 FreighterPosition = 0;
		if ( gStartPosition == kStartPosition_I )
		{
			FreighterPosition = kStartPosition_G;
		}
		else
		{
			FreighterPosition = kStartPosition_H;
		}
		FreighterTeam->mCreateShip(kMaxBPV, kClassFreighter, kClassFreighter, 50, FreighterPosition);
	
		BaseTeam->mCreateShip(kMaxBPV, kClassBaseStation, kClassStarBase, 50, kStartPosition_Y);
		EnemyTeam->mCreateShip( LowBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_X);

		CourierTeam->mCreateShip(TotalBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_Z);
	}

	if ( Mode ==  1 )
	{
		EnemyTeam->mCreateShip( LowBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_X);
		EnemyTeam->mCreateShip( HighBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_W);
		fMission.mDisplayMessage ( kDefendersAway_msg, kScienceScriptMessage );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	int32 MessageNumber = fMission.mRandomInt32 (kBriefingA_msg, kBriefingB_msg);

	return (MessageNumber);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* CourierTeam = fMission.mGetTeamHandle (kCourierTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* LineManTeam = fMission.mGetTeamHandle (kLineManTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tShipInfo* Ship;

	int32 DifLev = fMission.mGetDifficultyLevel();
	fMission.mSetTimedInterval (kTimerTurnsSet, 1);

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mTrackTacticalDistance (150.0, 38, 38); // if the player gets too close to the base it will launch 2 Enemys
		Ship->mSetShipThrust (5);
	}

	Ship =  EnemyTeam->mGetFirstShip();
	Ship->mSetShipTarget( PlayerTeam->mGetFirstShip(), kAbsolutePriority );

	for (Ship = FreighterTeam->mGetFirstShip (); Ship != NULL; Ship = FreighterTeam->mGetNextShip ())
	{
		Ship->mDisengage( kByMapEdge, kMediumPriority );
	}

	Ship = CourierTeam->mGetFirstShip();
	// Run for the base
	Ship->mChangeName( kCourier_msg );
	Ship->mSetFinalDestination( 38, 38 );
	Ship->mTrackTacticalDistance (40.0, 38, 38); // if the Courier makes it here the player loses.
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	switch ( DifLev )
	{
	case 0:
		{
			Ship->mSetShipMaxSpeed( 20 );
		}
		break;
	case 1:
		{
			Ship->mSetShipMaxSpeed( 23 );
		}
		break;
	case 2:
		{
			Ship->mSetShipMaxSpeed( 26 );
		}
		break;
	}


	if ( DifLev == 2 )
	{
		Ship = LineManTeam->mGetFirstShip();
		Ship->mSetShipTarget ( PlayerTeam->mGetFirstShip(), kAbsolutePriority );
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* PlayerShip = PlayerTeam->mGetFirstShip();

	switch ( gAct )
	{
	case 0:
		{
			// no ships should be created in act 0
			break;
		}
	case 1:
		{
			for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
			{
				// Run for the base
				Ship->mSetShipTarget (PlayerShip, kVeryHighPriority);
			}
			break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tShipInfo* PlayerShip = PlayerTeam->mGetFirstShip();

	int32 PlayerFleet = PlayerTeam->mGetShipsInPlayCount();

	switch (Event)
	{
		case kNotifyAtTimedInterval:
		{
			switch( MissionSegment )
			{
				case 0:
				{
					tString RendezvousMsg;
					fMission.mDisplayMessage( kLocation_msg, kNonCommScriptMessage );
					fMission.mDisplayMessage( kDeepSupply_msg, kNonCommScriptMessage );
					fMission.mSetTimedInterval (kTimerTurnsSet, 2);
					MissionSegment++;
					break;
				}
			case 1:
				{
					// only timer is at beginning of mission
					fMission.mDisplayMessage ( kGoal_msg, kScienceScriptMessage );
					break;
				}
			}
			break;
		}

		case kNotifyDamage:
		{
			if (Ship->mGetTeam () == kCourierTeam)
			{
				Ship->mSetShipMaxSpeed( 31 );
				if ( gAct == 0 ) // uh-oh the player awakened the base
				{
					gAct = 1;
					mCreateMissionShips( gAct );    // Now the base will launch attackers
					// time for the player to head for a corner and get out of here
					if ( gStartPosition == kStartPosition_I )
					{
						PlayerShip->mTrackTacticalDistance (50.0, 39, 1); // the player must disengage upper - right
					}
					else
					{
						PlayerShip->mTrackTacticalDistance (50.0, 1, 39); // the player must disengage lower - left
					}
					return kGameInProgress;
				}
			}
		};

		case kNotifyDisengaged:
		{
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				if ( gCourierTakenCareOf )
				{
					return kPlayerWins;
				}
				else
				{
					return kPlayerLoses;
				}
			}
		}

		case kNotifyAtDistance:
		{
			if (Ship->mGetTeam () == kPlayerTeam)
			{
				if ( gAct == 0 ) // uh-oh the player awakened the base
				{
					gAct = 1;
					mCreateMissionShips( gAct );    // Now the base will launch attackers
					// time for the player to head for a corner and get out of here
					return kGameInProgress;
				}
			}
			if (Ship->mGetTeam () == kCourierTeam)
			{
				if ( gCourierOuterRing )
				{
					gCourierOuterRing = kFalse;
					Ship->mTrackTacticalDistance (20.0, 38, 38); // if the Courier makes it here the player loses.
					fMission.mDisplayMessage( kCourierOuterRing_msg, kMissionScriptMessage );
				}
				else
				{
					// The courier made it.
					fMission.mDisplayMessage( kCourierMadeIt_msg, kMissionScriptMessage );
					if ( gStartPosition == kStartPosition_I )
					{
						fMission.mSetGoalIndicator( kTrue, 40, 0 );
					}
					else
					{
						fMission.mSetGoalIndicator( kTrue, 0, 40 );
					}

				}
				return kGameInProgress;
			}
			break;
		}

		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam () == kBaseTeam)
				{
					fMission.mDisplayMessage( kBaseBlownUp_msg, kMissionScriptMessage );
					return kGameInProgress;
				}

				if (Ship->mGetTeam () == kPlayerTeam)
				{
					if ( PlayerFleet ==  0 )
					{
						return kPlayerDestroyed;
					}
				}

				if (Ship->mGetTeam () == kEnemyTeam)
				{
					gEnemiesTakenCareOf++;
					if ( ( gCourierTakenCareOf ) && ( gEnemiesTakenCareOf == 2 ) )
					{
						fMission.mDisplayMessage( kDefendersDestroyed_msg, kMissionScriptMessage );
						return kGameInProgress;
					}
					return kGameInProgress;
				}

				if (Ship->mGetTeam () == kCourierTeam)
				{
					gCourierTakenCareOf = kTrue;						
					fMission.mDisplayMessage( kCourierHasBeenTakenCareOf_msg, kMissionScriptMessage );
					if ( gAct == 0 )
					{
						gAct = 1;
					}
					// time for the player to head for a corner and get out of here
					mCreateMissionShips( gAct );    // Now the base will launch attackers
					if ( gStartPosition == kStartPosition_I )
					{
						fMission.mSetGoalIndicator( kTrue, 40, 0 );
					}
					else
					{
						fMission.mSetGoalIndicator( kTrue, 0, 40 );
					}
					return kGameInProgress;
				}
			}

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				// LNF if it does/will change the 1 to 0
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerCaptured;
				}
			}

			if (Ship->mGetOriginalTeam () == kEnemyTeam)
			{
				gEnemiesTakenCareOf++;
				if ( ( gCourierTakenCareOf ) && ( gEnemiesTakenCareOf == 2 ) )
				{
					fMission.mDisplayMessage( kDefendersDestroyed_msg, kMissionScriptMessage );
					return kGameInProgress;
				}
				return kGameInProgress;
			}

			if (Ship->mGetOriginalTeam () == kCourierTeam)
			{
				gCourierTakenCareOf = kTrue;
				fMission.mDisplayMessage( kShipCaptured_msg, kMissionScriptMessage );
				if ( gAct == 0 )
				{
					gAct = 1;
				}
				// send it to the border
				Ship->mDisengage( kByMapEdge, kMediumPriority );

				// time for the player to head for a corner and get out of here
				mCreateMissionShips( gAct );    // Now the base will launch attackers
				if ( gStartPosition == kStartPosition_I )
				{
					fMission.mSetGoalIndicator( kTrue, 40, 0 );
				}
				else
				{
					fMission.mSetGoalIndicator( kTrue, 0, 40 );
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* CourierTeam = fMission.mGetTeamHandle (kCourierTeam);

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	float playerShipHullIntegrity =  (float) PlayerBPV / (float) PlayerBPV;

	if ( ( gCourierTakenCareOf ) && ( gEnemiesTakenCareOf > 0 ) )
	{
		// kAstoundingVictory, the courier was stopped and 1 or more Enemys were taken care of 
		fMission.mScheduleMission( kLinkedMission_name );
		return kAstoundingVictory;
	}

	if ( ( gCourierTakenCareOf ) && ( playerShipHullIntegrity > 0.75 ) )
	{
		// kVictory, the courier was stopped and the player m,ade it out relatively heathly
		return kVictory;
	}

	if ( ( gCourierTakenCareOf )  && ( playerShipHullIntegrity > 0.05 ) )
	{
		// kDraw, the courier was stopped but the player's ship is pretty banged up.
		return kDraw;
	}

	if ( ( !gCourierTakenCareOf ) && ( playerShipHullIntegrity < 0.50 ) )
	{
		// kDevastatingDefeat, the courier was not stopped and the player's ship is pretty banged up.
		return kDevastatingDefeat;
	}

	if ( !gCourierTakenCareOf ) 
	{
		// kDefeat, the courier was not stopped.
		return kDefeat;
	}
	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
