#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
float AlliedHealth = 0.0;
float TroubleHealth = 0.0;
iTruth PlayerDisengaged = kFalse;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kAlliedTeam, kTroubleTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerCapital | kPlayerColonial | kWarzone;
	Specs->fPlayerRace = kFedMission | kKlingMission | kLyranMission | kGornMission | kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 2;
	Specs->fMaxDate = 50;
	Specs->fWait = 2;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 120, 80, 20, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kAlliedTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kTroubleTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, kOrion);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kTroubleTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kTroubleTeam, kTeamAtWar, kGoalCapture, 2.0);
	fMission.mSetTeamRelations (kAlliedTeam, kEnemyTeam, kTeamHostile, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kTroubleTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kTroubleTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// pick a random map
	return (fMission.mRandomInt32 (1, TotalMaps));
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* TroubleTeam = fMission.mGetTeamHandle (kTroubleTeam);

	// Get the total BPV of the player and modify by 75% to 150%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * 0.9;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// pick a random number of ships
	int32 NumberEnemyShips = PlayerTeam->mGetShipsInPlayCount() + 2;

	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.90f;
		}
		break;
	case 1:
		{
			NumberEnemyShips++;
			TotalBPV *= 1.25f;
		}
		break;
	case 2:
		{
			NumberEnemyShips += 2;
			TotalBPV *= 1.8f;
		}
		break;
	}

	// Any size ship within the appropriate BPV and a 33% chance of a variant
	EnemyTeam->mCreateFleet(NumberEnemyShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_X);

	// create allied ships
	AlliedTeam->mCreateShip (kMaxBPV, kClassBaseStation, kClassBaseStation, 0, kStartPosition_H);
	AlliedTeam->mCreateFleet (2, kMaxBPV, kClassFreighter, kClassFreighter, 100, kStartPosition_I);

	// create freighters in trouble
	TroubleTeam->mCreateFleet (2, kMaxBPV, kClassFreighter, kClassFreighter, 100, kStartPosition_K);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	int32 MessageNumber = fMission.mRandomInt32 (kBriefingA_msg, kBriefingC_msg);
	return (MessageNumber);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* TroubleTeam = fMission.mGetTeamHandle (kTroubleTeam);
	tShipInfo* Ship;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure pirate ships
	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// configure base and allied ships
	for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// configure freighters in trouble
	for (Ship = TroubleTeam->mGetFirstShip (); Ship != NULL; Ship = TroubleTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetFinalDestination();
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* TroubleTeam = fMission.mGetTeamHandle (kTroubleTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();
	TroubleHealth = 100.0f - TroubleTeam->mGetTotalDamagePercent();
	AlliedHealth = 100.0f - AlliedTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch (Event)
	{
		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				return kPlayerWins;
			}
		break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerDestroyed++;
			}
			else
			{
				// see if this is all over.
				if ( Enemies == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				if ( Ship->mGetTeam() == kAlliedTeam )
				{
					fMission.mDisplayMessage (kGladToSeeYou_msg, kMissionScriptMessage);
					for (Ship = AlliedTeam->mGetFirstShip (); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
					{
						Ship->mClearCommButton (1);
					}
				}
				else if ( Ship->mGetTeam() == kTroubleTeam )
				{
					fMission.mDisplayMessage (kWeAppreciateYourPatrol_msg, kMissionScriptMessage);
					for (Ship = TroubleTeam->mGetFirstShip (); Ship != NULL; Ship = TroubleTeam->mGetNextShip ())
					{
						Ship->mClearCommButton (1);
					}
				}
				else if ( Ship->mGetTeam() == kEnemyTeam )
				{
					fMission.mDisplayMessage (kStatic_msg, kMissionScriptMessage);
					for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
					{
						Ship->mClearCommButton (1);
					}
				}
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* TroubleTeam = fMission.mGetTeamHandle (kTroubleTeam);

	if ( PlayerDisengaged )
	{
		if ( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if ( PlayerHealth < ( EnemyHealth / 2 ) )
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}


	if ( ( PlayerHealth > 90.0 ) || ( EnemyCaptured ) )
	{
		if ( ( AlliedHealth > 40.0 ) && ( TroubleHealth > 40.0 ) )
		{
			return kAstoundingVictory;
		}
		else if ( ( AlliedHealth > 40.0 ) || ( TroubleHealth > 40.0 ) )
		{
			return kVictory;
		}
		else if ( ( AlliedHealth > 10.0 ) || ( TroubleHealth > 10.0 ) )
		{
			return kDraw;
		}
		else
		{
			return kDefeat;
		}
		
	}


	if ( PlayerHealth < 25.0 ) 
	{
		if ( ( AlliedHealth > 40.0 ) && ( TroubleHealth > 40.0 ) )
		{
			return kVictory;
		}
		else if ( ( AlliedHealth > 40.0 ) || ( TroubleHealth > 40.0 ) )
		{
			return kDraw;
		}
		else if ( ( AlliedHealth > 10.0 ) || ( TroubleHealth > 10.0 ) )
		{
			return kDefeat;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}

	// By default
	return kVictory;

}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
