#include "scriptinterface.h"

#include "Gen_SurveyMonsterText.h"
#include "Gen_SurveyMonsterMaps.h"

int32 ScanX = 0;
int32 ScanY = 0;
int32 AsteroidsLeft = 0;
int32 NumMonsters = 0;
int32 RandomMonster = 0;
int32 DiffLevel = 0;
int32 BonusBalancePoints = 0;
int32 MapNumber = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
bool AsteroidMap = false;
bool TargetScanned = false;
bool MonsterScanned = false;
bool Scanning = false;
bool PlayerDisengaged = false;
bool PETA = false;
bool PETAKiller = false;

const int32 AsteroidCoordinates[3][2] =
{
	{ 10, 19 },	// Map 3 Asteroid X, Y Coordinates
	{ 15, 23 },
	{ 20, 27 },
};

enum eMonsterType
{
	kSpaceshell = 1,
	kMeater,
	kSunglider,
	kAstrominer,
};

enum eDiffLevel
{
	kDiffEasy = 0,
	kDiffMedium,
	kDiffHard,
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kScanTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kUnexplored;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 3;
	Specs->fMaxDate = 50;
	Specs->fWait = 2;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 90, 60, 30, 0, 0 );

	DiffLevel = fMission.mGetDifficultyLevel();

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kMonster );
	fMission.mSetTeamRace( kScanTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kScanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kScanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	MapNumber = fMission.mRandomInt32( 1, TotalMaps );

	if( MapNumber == 3 )
	{
		AsteroidMap = true;
	}

	if(( info.fPlayerRace == kFederation ) || ( info.fPlayerRace == kHydran ))
	{
		PETA = true;
	}

	// pick a random map
	return( MapNumber );
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* ScanTeam = fMission.mGetTeamHandle( kScanTeam );

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 Position = fMission.mRandomInt32( kStartPosition_M, kStartPosition_X );

	RandomMonster = fMission.mRandomInt32( kSpaceshell, kAstrominer );

	if(( RandomMonster == kAstrominer ) && ( AsteroidMap == false ))
	{
		RandomMonster = kSunglider;		// astrominers can only appear on the asteroid map
	}									// sungliders appear more often than other monsters

	switch( RandomMonster )
	{
		case kSpaceshell:
			switch( DiffLevel )
			{
				case kDiffEasy:
					EnemyTeam->mCreateShip( "M-SSS", Position );
					EnemyTeam->mCreateShip( "M-SSS", Position + 1 );
					break;

				case kDiffMedium:
					EnemyTeam->mCreateShip( "M-SSS", Position );
					EnemyTeam->mCreateShip( "M-SSM", Position + 1 );
					EnemyTeam->mCreateShip( "M-SSM", Position + 2 );
					break;

				case kDiffHard:
					EnemyTeam->mCreateShip( "M-SSS", Position );
					EnemyTeam->mCreateShip( "M-SSM", Position + 1 );
					EnemyTeam->mCreateShip( "M-SSM", Position + 2 );
					EnemyTeam->mCreateShip( "M-SSL", Position + 3 );
					break;
			}
			break;

		case kMeater:
			switch( DiffLevel )
			{
				case kDiffEasy:
					EnemyTeam->mCreateShip( "M-MTS", Position );
					break;

				case kDiffMedium:
					EnemyTeam->mCreateShip( "M-MTM", Position );
					break;

				case kDiffHard:
					EnemyTeam->mCreateShip( "M-MTL", Position );
					break;
			}
			break;

		case kSunglider:
			switch( DiffLevel )
			{
				case kDiffEasy:
					EnemyTeam->mCreateShip( "M-SGS", Position );
					break;

				case kDiffMedium:
					EnemyTeam->mCreateShip( "M-SGM", Position );
					
					if( PlayerBPV > EnemyTeam->mGetCombatBPV() )
					{
						EnemyTeam->mCreateShip( "M-SGS", Position + 1 );
					}
					break;

				case kDiffHard:
					EnemyTeam->mCreateShip( "M-SGL", Position );

					if( PlayerBPV > EnemyTeam->mGetCombatBPV() )
					{
						EnemyTeam->mCreateShip( "M-SGM", Position + 1 );
					}

					if( PlayerBPV > EnemyTeam->mGetCombatBPV() )
					{
						EnemyTeam->mCreateShip( "M-SGS", Position + 2 );
					}
					break;
			}
			break;

		case kAstrominer:
			switch( DiffLevel )
			{
				case kDiffEasy:
					EnemyTeam->mCreateShip( "M-AMS", Position );
					break;

				case kDiffMedium:
					EnemyTeam->mCreateShip( "M-AMM", Position );
					
					if( PlayerBPV > EnemyTeam->mGetCombatBPV() )
					{
						EnemyTeam->mCreateShip( "M-AMS", Position + 1 );
					}
					break;

				case kDiffHard:
					EnemyTeam->mCreateShip( "M-AML", Position );

					if( PlayerBPV > EnemyTeam->mGetCombatBPV() )
					{
						EnemyTeam->mCreateShip( "M-AMM", Position + 1 );
					}

					if( PlayerBPV > EnemyTeam->mGetCombatBPV() )
					{
						EnemyTeam->mCreateShip( "M-AMS", Position + 2 );
					}
					break;
			}
			break;
	}

	if( MapNumber == 1 )
	{
		ScanTeam->mCreateShip( "PL3", kStartPosition_Specified, 15, 15 );
	}
	else if( MapNumber == 2 )
	{
		ScanTeam->mCreateShip( "PL2", kStartPosition_Specified, 22, 21 );
	}
	else
	{
		for( int32 Loop = 0; Loop <= DiffLevel; Loop++ )
		{
			ScanX = AsteroidCoordinates[ Loop ][0];
			ScanY = AsteroidCoordinates[ Loop ][1];
			ScanTeam->mCreateShip( "N-AST", kStartPosition_Specified, ScanX, ScanY );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefingA_msg + MapNumber - 1 );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* ScanTeam = fMission.mGetTeamHandle( kScanTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 5 );

	if( MapNumber == 1 )
	{
		Ship->mTrackTacticalDistance( 30.0, 15, 15 );	// tracking planet-3
	}
	else if( MapNumber == 2 )
	{
		Ship->mTrackTacticalDistance( 30.0, 22, 21 );	// tracking planet-2
	}
	else if( MapNumber == 3 )	// Asteroids Map
	{
		AsteroidsLeft = DiffLevel;
		ScanX = AsteroidCoordinates[ AsteroidsLeft ][0];
		ScanY = AsteroidCoordinates[ AsteroidsLeft ][1];
		fMission.mSetGoalIndicator( kTrue, ScanX, ScanY );
		Ship->mTrackTacticalDistance( 15.0, ScanX, ScanY );
	}

	for( Ship = ScanTeam->mGetFirstShip(); Ship != NULL; Ship = ScanTeam->mGetNextShip() )
	{
		Ship->mSetShipAIControlled( kFalse );
	}

	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	NumMonsters = EnemyTeam->mGetShipsInPlayCount();

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ScanTeam = fMission.mGetTeamHandle( kScanTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else
			{
				if(( PETA == true ) && ( PETAKiller == false ))
				{
					fMission.mAddToBonusPrestige( -25 );
					PETAKiller = true;
				}

				if(( Enemies == 0 ) && ( TargetScanned == true ))
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kEnemyDestroyed;
				}
				else if( Enemies == 0 )
				{
					fMission.mDisplayMessage( kNoMoreMonster_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = true;
				fMission.mAddToBonusPrestige( BonusBalancePoints );

				if(( PETA == true ) && ( MonsterScanned == true ) && ( PETAKiller == false ))
				{
					fMission.mAddToBonusPrestige( 25 );
				}
				return kPlayerWins;
			}
			break;

		case kNotifyAtDistance:
			if( fIsDeepScanOn == kTrue )
			{
				Scanning = true;

				if( AsteroidMap == false )
				{
					fMission.mDisplayMessage( kScanningPlanet_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
				}
				else
				{
					fMission.mDisplayMessage( kScanningAsteroid_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
				}
			}
			else
			{
				fMission.mDisplayMessage( kCloseEnough_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage( kMonsterHail_msg, kMissionScriptMessage );
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kScanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kScanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;

				case kTimerTurnsSet:
					if( AsteroidMap == false )
					{
						fMission.mDisplayMessage( kScienceBriefPlanet_msg, kScienceScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kScienceBriefAsteroid_msg, kScienceScriptMessage );
					}
					break;

				case kTimerTurnsSet_2:	// scanning some asteroids
					if( fIsDeepScanOn == kTrue )
					{
						if( fPlayerShip->mGetTacticalDistance( ScanX, ScanY ) <= 15.0 )
						{
							fMission.mDisplayMessage (kAsteroidScanned_msg, kScienceScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kFarButGotIt_msg, kScienceScriptMessage);
						}

						AsteroidsLeft--;

						if( AsteroidsLeft < 0 )
						{
							fMission.mDisplayMessage( kAllScanned_msg, kScienceScriptMessage );
							TargetScanned = true;
							fMission.mAddToBonusPrestige( 60 );

							if( EnemyTeam->mGetShipsInPlayCount() == 0 )
							{
								fMission.mAddToBonusPrestige( BonusBalancePoints );
								return kEnemyDestroyed;
							}
						}
						else
						{
							fMission.mDisplayMessage( kNextAsteroid1_msg + AsteroidsLeft, kScienceScriptMessage );
							ScanX = AsteroidCoordinates[ AsteroidsLeft ][0];
							ScanY = AsteroidCoordinates[ AsteroidsLeft ][1];
							fMission.mSetGoalIndicator( kTrue, ScanX, ScanY );
							fPlayerShip->mTrackTacticalDistance( 15.0, ScanX, ScanY );
						}
					}
					break;

				case kTimerTurnsSet_3:	// scanning a planet
					if( fIsDeepScanOn == kTrue )
					{
						if( fPlayerShip->mGetTacticalDistance( ScanX, ScanY ) <= 30.0 )
						{
							fMission.mDisplayMessage (kPlanetScanned_msg, kScienceScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kFarButGotIt_msg, kScienceScriptMessage);
						}

						TargetScanned = true;
						fMission.mAddToBonusPrestige( 60 );

						if( EnemyTeam->mGetShipsInPlayCount() == 0 )
						{
							fMission.mAddToBonusPrestige( BonusBalancePoints );
							return kEnemyDestroyed;
						}
					}
					break;
			}
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				tShipInfo* Target = fMission.mGetShipTarget( fPlayerShip );

				if( Target->mGetTeam() == kScanTeam )	// scanning a planet or asteroid
				{
					if(( Scanning == false ) && ( TargetScanned == false ))
					{
						if(( AsteroidMap == true ) && ( fPlayerShip->mGetTacticalDistance( ScanX, ScanY ) <= 15.0 ))
						{
							fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
							fMission.mDisplayMessage( kScanningAsteroid_msg, kScienceScriptMessage );
							Scanning = true;
						}
						else if(( AsteroidMap == false ) && ( fPlayerShip->mGetTacticalDistance( ScanX, ScanY ) <= 30.0 ))
						{
							fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
							fMission.mDisplayMessage( kScanningPlanet_msg, kScienceScriptMessage );
							Scanning = true;
						}
						else
						{
							fMission.mDisplayMessage( kTooFar_msg, kScienceScriptMessage );
						}
					}
				}
				else	// scanning a monster
				{
					if( MonsterScanned == false )
					{
						Ship = fPlayerShip;
						tShipInfo* Monster;

						for( Monster = EnemyTeam->mGetFirstShip(); Monster != NULL; Monster = EnemyTeam->mGetNextShip() )
						{
							if(( Ship->mGetTacticalDistance( Monster ) < 10 ) && ( MonsterScanned == false ))
							{
								MonsterScanned = true;
								fMission.mDisplayMessage( kMonsterScanned_msg, kScienceScriptMessage, Monster );
							}
						}
					}
				}
			}
			else if(( Scanning == true ) && ( TargetScanned == false ))
			{
				Scanning = false;

				if(( AsteroidMap == true ) && ( fPlayerShip->mGetTacticalDistance( ScanX, ScanY ) <= 15.0 ))
				{
					fMission.mDisplayMessage( kDidNotScan_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet_2, 0 );
				}

				if(( AsteroidMap == false ) && ( fPlayerShip->mGetTacticalDistance( ScanX, ScanY ) <= 30.0 ))
				{
					fMission.mDisplayMessage( kDidNotScan_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet_3, 0 );
				}
			}
			break;

		case kNotifyProbeLaunch:
			tShipInfo* Target = fMission.mGetShipTarget( fPlayerShip );

			if( Target != NULL )
			{
				if(( Target->mGetTeam() == kEnemyTeam ) && ( MonsterScanned == false ))
				{
					MonsterScanned = true;
					fMission.mDisplayMessage( kMonsterProbed_msg, kScienceScriptMessage, Target );
				}
				else
				{
					fMission.mDisplayMessage( kPlanetProbed_msg, kScienceScriptMessage );
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( PETA == true )
	{
		if(( PETAKiller == false ) && ( TargetScanned == true ))
		{
			return kVictory;
		}
		else
		{
			return kDefeat;
		}
	}
	else
	{
		if(( PlayerDisengaged == true ) && ( TargetScanned == false ))
		{
			if( PlayerHealth > EnemyHealth )
			{
				return kVictory;
			}
			else if( PlayerHealth < ( EnemyHealth / 2 ))
			{
				return kDevastatingDefeat;
			}
			else 
			{
				return kDefeat;
			}
		}

		if(( PlayerHealth > 90.0 ) && ( TargetScanned == true ))
		{
			return kAstoundingVictory;
		}

		if(( PlayerHealth < 25.0 ) || ( TargetScanned == false ))
		{
			return kDraw;
		}
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if( PETA == true )
	{
		if( PETAKiller == false )
		{
			fMission.mAddToDebriefingList( kDebriefMonsterKilled_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kDebriefMonsterAlive_msg );
		}
	}
	else
	{
		if( AsteroidMap == false )
		{
			if( TargetScanned == true )
			{
				fMission.mAddToDebriefingList( kDebriefPlanetScan_msg );
			}
			else
			{
				fMission.mAddToDebriefingList( kDebriefPlanetNoScan_msg );
			}
		}
		else
		{
			if( TargetScanned == true )
			{
				fMission.mAddToDebriefingList( kDebriefAsteroidScan_msg );
			}
			else
			{
				fMission.mAddToDebriefingList( kDebriefAsteroidNoScan_msg );
			}
		}
	}

	if( MonsterScanned == false )
	{
		fMission.mAddToDebriefingList( kDebriefMonsterNoScan_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kDebriefMonsterScan_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
