#include "scriptinterface.h"

#include "Gen_SurveyPirateText.h"
#include "Gen_SurveyPirateMaps.h"

int32 BonusBalancePoints = 0;
int32 AsteroidsLeft = 0;
int32 MapNumber = 0;
int32 AstX = 0;
int32 AstY = 0;
int32 DiffLevel = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
bool PlayerDisengaged = false;
bool Scanning = false;
bool ScannedAll = false;

const int32 AsteroidCoordinates[9][2] =
{
	{ 25, 25 },	// Map 1 Asteroid X, Y Coordinates
	{ 13, 20 },
	{ 13, 29 },
	{ 20, 20 },	// Map 2
	{ 26, 13 },
	{ 29, 28 },
	{ 29, 18 },	// Map 3
	{ 16, 29 },
	{ 11, 19 },
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kAsteroidTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kUnexplored;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 3;
	Specs->fMaxDate = 50;
	Specs->fWait = 2;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 75, 50, 25, 0, 0 );

	DiffLevel = fMission.mGetDifficultyLevel();

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kOrion );
	fMission.mSetTeamRace( kAsteroidTeam, kMonster );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a random map
	MapNumber = fMission.mRandomInt32( 1, TotalMaps );

	return( MapNumber );
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );

	// Get the total BPV of the player and modify by 80% to 120%
	int32 TotalBPV = PlayerTeam->mGetCombatBPV();
	int32 NumShips = 2;
	DiffLevel = fMission.mGetDifficultyLevel();	// 0 = easy, 1 = medium, 2 = hard

	if( DiffLevel == 0 )
	{
		TotalBPV = TotalBPV * .75;
		NumShips--;
	}

	if( DiffLevel == 2 )
	{
		TotalBPV = TotalBPV * 1.33;
	}

	EnemyTeam->mCreateFleet( NumShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 50, kStartPosition_M + fMission.mRandomInt32( 0, 12 ));

	// create the asteroids that need to be scanned
	AsteroidsLeft = 1 + DiffLevel;	// total number of asteroids that must be scanned

	for( int32 Loop = AsteroidsLeft; Loop != 0 ; Loop-- )
	{
		AstX = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( Loop - 1 ) ][0];
		AstY = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( Loop - 1 ) ][1];

		AsteroidTeam->mCreateShip( "N-AST", kStartPosition_Specified, AstX, AstY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefingA_msg + DiffLevel );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// setup mission goals
	AstX = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( AsteroidsLeft - 1 ) ][0];
	AstY = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( AsteroidsLeft - 1 ) ][1];

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mTrackTacticalDistance( 15.0, AstX, AstY );
	Ship->mSetShipThrust( 5 );
	fMission.mSetGoalIndicator( kTrue, AstX, AstY );

	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	for( Ship = AsteroidTeam->mGetFirstShip(); Ship != NULL; Ship = AsteroidTeam->mGetNextShip() )
	{
		Ship->mChangeName( kAsteroid_name );
		Ship->mSetShipAIControlled( kFalse );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else
			{
				if(( Enemies == 0 ) && ( ScannedAll == true ))
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kEnemyDestroyed;
				}
				else if( Enemies == 0 )
				{
					fMission.mDisplayMessage( kNoMorePirates_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyOnCapture:
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
			Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = true;
				fMission.mAddToBonusPrestige( BonusBalancePoints );

				if( ScannedAll == true )
				{
					return kPlayerWins;
				}
				else
				{
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyAtDistance:
			if( fIsDeepScanOn == kTrue )
			{
				fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
				fMission.mDisplayMessage( kScanningAsteroid_msg, kScienceScriptMessage );
				Scanning = true;
			}
			else
			{
				fMission.mDisplayMessage( kCloseEnough_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage( fMission.mRandomInt32( kPirateHail1_msg, kPirateHail3_msg ), kFleetScriptMessage, Ship );

			for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
			{
				Ship->mClearCommButton( 1 );
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kScienceBrief_msg, kScienceScriptMessage );
					break;

				case kTimerTurnsSet_2:
					if( fIsDeepScanOn == kTrue )
					{
						if( fPlayerShip->mGetTacticalDistance( AstX, AstY ) <= 15.0 )
						{
							fMission.mDisplayMessage (kAsteroidScanned_msg, kScienceScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kFarButGotIt_msg, kScienceScriptMessage);
						}

						AsteroidsLeft--;

						if( AsteroidsLeft == 0 )
						{
							fMission.mDisplayMessage( kAllScanned_msg, kScienceScriptMessage );
							ScannedAll = true;
							fMission.mAddToBonusPrestige( 60 );
							fMission.mSetGoalIndicator( kFalse, 0, 0 );

							if( EnemyTeam->mGetShipsInPlayCount() == 0 )
							{
								fMission.mAddToBonusPrestige( BonusBalancePoints );
								return kEnemyDestroyed;
							}
						}
						else
						{
							fMission.mDisplayMessage( kNextAsteroid1_msg + ( AsteroidsLeft - 1 ), kScienceScriptMessage );
							AstX = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( AsteroidsLeft - 1 ) ][0];
							AstY = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( AsteroidsLeft - 1 ) ][1];
							fMission.mSetGoalIndicator( kTrue, AstX, AstY );
							fPlayerShip->mTrackTacticalDistance( 15.0, AstX, AstY );
						}
					}
					break;
			}
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				if( Scanning == false )
				{
					if( fPlayerShip->mGetTacticalDistance( AstX, AstY ) <= 15.0 )
					{
						fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
						fMission.mDisplayMessage( kScanningAsteroid_msg, kScienceScriptMessage );
						Scanning = true;
					}
					else
					{
						fMission.mDisplayMessage( kTooFar_msg, kScienceScriptMessage );
					}
				}
			}
			else if( Scanning == true )
			{
				fMission.mDisplayMessage (kDidNotScan_msg, kScienceScriptMessage);
				Scanning = false;
				fMission.mSetTimedInterval( kTimerTurnsSet_2, 0 );
			}
			break;

		case kNotifyProbeLaunch:
			Ship = AsteroidTeam->mGetThisShip( AsteroidsLeft );

			if(( ScannedAll == false ) && ( fMission.mGetShipTarget( fPlayerShip ) == Ship ))
			{
					fMission.mDisplayMessage( kAsteroidScanned_msg, kScienceScriptMessage );
					AsteroidsLeft--;

					if( AsteroidsLeft == 0 )
					{
						fMission.mDisplayMessage( kAllScanned_msg, kScienceScriptMessage );
						ScannedAll = true;
						fMission.mAddToBonusPrestige( 60 );
						fMission.mSetGoalIndicator( kFalse, 0, 0 );

						if( EnemyTeam->mGetShipsInPlayCount() == 0 )
						{
							fMission.mAddToBonusPrestige( BonusBalancePoints );
							return kEnemyDestroyed;
						}
					}
					else
					{
						fMission.mDisplayMessage( kNextAsteroid1_msg + ( AsteroidsLeft - 1 ), kScienceScriptMessage );
						AstX = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( AsteroidsLeft - 1 ) ][0];
						AstY = AsteroidCoordinates[ (( MapNumber - 1 ) * 3 ) + ( AsteroidsLeft - 1 ) ][1];
						fMission.mSetGoalIndicator( kTrue, AstX, AstY );
						fPlayerShip->mTrackTacticalDistance( 15.0, AstX, AstY );
					}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if(( PlayerDisengaged == true ) && ( ScannedAll == false ))
	{
		if( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if( PlayerHealth < ( EnemyHealth / 2 ))
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}

	if(( PlayerHealth > 90.0 ) && ( ScannedAll == true ))
	{
		return kAstoundingVictory;
	}

	if(( PlayerHealth < 25.0 ) || ( ScannedAll == false ))
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );

	if( ScannedAll == true )
	{ 
		fMission.mAddToDebriefingList( kDebriefScanned_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kDebriefNoScan_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
