#include "scriptinterface.h"
#include <math.h>

#include "Gor_10MissionText.h"
#include "Gor_10MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 RomulanDestroyed = 0;
int32 MonsterDestroyed = 0;
int32 FatherDestroyed = 0;
int32 OrionDestroyed = 0;
int32 EggsRetrieved = 0;

bool FreighterCaptured = false;
bool FatherWarningDelivered = false;
bool PlanetScanned = false;
bool MoonScanned = false;
bool StationDestroyed = false;
bool FatherAppeared = false;
bool MarinesRetrieved = false;
bool AtWarWithRomulan = false;
bool GetOuttaHere = false;
bool FatherScanned = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

const float kDamageThreshold = 60.0f;
const int32 kPlanetX = 20;
const int32 kPlanetY = 20;
const int32 kMoonX = 26;
const int32 kMoonY = 13;
const float kFatherWarningDistance = 40.0f;
const float kDeepScanDistancePlanet = 30.0f;
const float kDeepScanDistanceMoon = 30.0f;
const int32 kDeepScanTurns = 2;
const float kDamageStationDestroyed = 10.0f;
const float kDeepScanDistanceFather = 15.0f;
const int32 kDeepScanTurnsFather = 1;

void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam);

enum eMarineMode
{
	kMarinesReady = 0,
	kMarinesDeployed,
	kMarinesRecovered
};
eMarineMode MarineMode = kMarinesReady;

enum eMissionMode
{
	kStart = 0,
	kFatherArrives,
	kSpawnMonster
};
eMissionMode Mode = kStart;

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMonsterTeam, 
	kRomulanTeam,
	kOrionTeam,
	kFatherTeam,
	kPlanetTeam,
	kMoonTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kMonsterTeam, kGorn );
	fMission.mSetTeamRace( kRomulanTeam, kRomulan );
	fMission.mSetTeamRace( kOrionTeam, kOrion );
	fMission.mSetTeamRace( kFatherTeam, kGorn );
	fMission.mSetTeamRace( kPlanetTeam, kGorn );
	fMission.mSetTeamRace( kMoonTeam, kGorn );

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
	fMission.mSetTeamRelations (kPlayerTeam, kRomulanTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFatherTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMoonTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kMonsterTeam, kRomulanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kMonsterTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kMonsterTeam, kFatherTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kMonsterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kMonsterTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRomulanTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kRomulanTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomulanTeam, kFatherTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomulanTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomulanTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionTeam, kFatherTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFatherTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kFatherTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFatherTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlanetTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FatherTeam = fMission.mGetTeamHandle (kFatherTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* MoonTeam = fMission.mGetTeamHandle (kMoonTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	switch ( Mode )
	{
		case kStart:
			// create Monster team
			MonsterTeam->mCreateShip( "M-SSS", kStartPosition_R );
			MonsterTeam->mCreateShip( "M-SSS", kStartPosition_S );

			// create Romulan team
			RomulanTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassDreadnought, 1, kStartPosition_T );

			// create Orion team

			// create Planet team
			PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPlanetX, kPlanetY );

			// create Moon team
			MoonTeam->mCreateShip( "PL0", kStartPosition_Specified, kMoonX, kMoonY );
			break;

		case kFatherArrives:
			// create Father team
			FatherTeam->mCreateShip( "M-SSL", kStartPosition_Q );
			break;

		case kSpawnMonster:
			{
				if ( MonsterTeam->mGetShipsInPlayCount() <= 6 )
				{
					int32 randomInt = fMission.mRandomInt32( -3141, 3141 );
					float theta = (float)randomInt / 1000.0f;

					int32 NewX = 32 + (1.0f * cos(theta));
					int32 NewY = 29 + (1.0f * sin(theta));

					MonsterTeam->mCreateShip( "M-SSS", kStartPosition_Specified, NewX, NewY );

					randomInt = fMission.mRandomInt32( -3141, 3141 );
					theta = (float)randomInt / 1000.0f;

					NewX = 32 + (1.0f * cos(theta));
					NewY = 29 + (1.0f * sin(theta));
					MonsterTeam->mCreateShip( "M-SSS", kStartPosition_Specified, NewX, NewY );
				}
			}
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// there is only one briefing for this mission
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FatherTeam = fMission.mGetTeamHandle (kFatherTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* MoonTeam = fMission.mGetTeamHandle (kMoonTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for ( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents (kNotifyAtDistance);
		Ship->mTrackTacticalDistance( kFatherWarningDistance, kPlanetX, kPlanetY );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}
	PlayerTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );

	// configure Monster Team
	for ( Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip() )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		//Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Romulan team
	for ( Ship = RomulanTeam->mGetFirstShip(); Ship != NULL; Ship = RomulanTeam->mGetNextShip() )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Orion team

	// configure Planet team
	for ( Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mChangeName( kFargaross_name );
	}

	// configure Moon team
	for ( Ship = MoonTeam->mGetFirstShip(); Ship != NULL; Ship = MoonTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mChangeName( kMoon_name );
	}


	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
	DeepScanTracker.mSetTarget (MoonTeam->mGetThisShip(1), kDeepScanDistanceMoon, kDeepScanTurns);
	// DeepScanTracker.mSetLocation (fractureNumber[1].x, fractureNumber[1].y, kDeepScanDistance, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* FatherTeam = fMission.mGetTeamHandle (kFatherTeam);

	switch ( Mode )
	{
		case kFatherArrives:
			// configure Father team
			Ship->mSetCommButton (1, kButtonHail_text);
			Ship->mSetShipCallbackEvents( kNotifyDamage );
			Ship->mChangeName( kQezl_name );

			DeepScanTracker.mSetTarget (FatherTeam->mGetThisShip(1), kDeepScanDistanceFather, kDeepScanTurnsFather);
			break;

		case kSpawnMonster:
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FatherTeam = fMission.mGetTeamHandle (kFatherTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* MoonTeam = fMission.mGetTeamHandle (kMoonTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ( TransportTarget->mGetTeam() == kPlanetTeam )
					{
						if ( fEventItem1 == kTransAwayTeamItem )
						{
							if ( GetOuttaHere == false )
							{
								if ( StationDestroyed == true )
								{
									fMission.mDisplayMessage( kBeamLandingParty_msg, kMissionScriptMessage );
									// set time to reveal eggs
									fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
									MarineMode = kMarinesDeployed;
								}
								else
								{
									// don't let the player beam down the landing party
									fMission.mDisplayMessage( kEvent9BeamDown_msg, kMissionScriptMessage );
									Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );
									TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
								}
							}
							else
							{
								Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );
								TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
							}
						}
						else if ( fEventItem1 == kTransGornEgg )
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
						}
					}
					else if ( TransportTarget->mGetTeam() != kPlayerTeam )
					{
						if ( fEventItem1 == kTransAwayTeamItem )
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
						}
						else if ( fEventItem1 == kTransGornEgg )
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ( GetOuttaHere == false )
					{
						if ( fEventItem1 == kTransGornEgg )
						{
							EggsRetrieved = 1;
						}

						if ( MarineMode == kMarinesDeployed )
						{
							if ( fEventItem1 == kTransAwayTeamItem )
							{
								MarinesRetrieved = true;
								MarineMode = kMarinesRecovered;
							}
						}
						else if ( MarineMode == kMarinesRecovered )
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
						}

						if (( MarinesRetrieved == true ) && ( EggsRetrieved == 1 ))
						{
							fMission.mDisplayMessage( kGetOuttaHere_msg, kMissionScriptMessage );
							GetOuttaHere = true;
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kPlanetTeam )
						{
							if ( FatherAppeared == false )
							{
								FatherAppeared = true;
								Mode = kFatherArrives;
								mCreateMissionShips( kFatherArrives );
							}
						}
						else if ( Ship->mGetTeam() == kMoonTeam )
						{
							if (( MoonTeam->mGetFirstShip()->mGetDamagePercent() >= kDamageStationDestroyed )
								&& ( StationDestroyed == false ))
							{
								if ( MoonScanned == true )
								{
									StationDestroyed = true;
									fMission.mDisplayMessage( kEvent3WeakenQezl_msg, kScienceScriptMessage );
								}
							}
						}
						else if ( Ship->mGetTeam() == kFatherTeam )
						{
							if ( StationDestroyed == false )
							{
								// LNF maybe the Father should be able to be destroyed if the 
								// LNF station is NOT destroyed?
								Ship->mDamageSystem( kDamageWeaponsAndSystems, 0 );
								break;
							}
						}
						else if ( Ship->mGetTeam() == kRomulanTeam )
						{
							if ( AtWarWithRomulan == false )
							{
								AtWarWithRomulan = true;
							}
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			if ( Ship->mGetTeam() == kPlayerTeam )
			{
				if (( Ship->mGetTrackingDistance() == kFatherWarningDistance )
					&& ( FatherWarningDelivered == false ))
				{
					FatherWarningDelivered = true;
					fMission.mDisplayMessage( kEvent1Qezl_1_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kEvent1Qezl_2_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kEvent1Qezl_3_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kRomulanTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kFatherTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kMoonTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonsterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kMonsterTeam, kRomulanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kMonsterTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kMonsterTeam, kFatherTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kMonsterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kMonsterTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kRomulanTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kRomulanTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kRomulanTeam, kFatherTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kRomulanTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kRomulanTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionTeam, kFatherTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kFatherTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kFatherTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFatherTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlanetTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					break;

				case kTimerLimitSet_2:
					break;
					
				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;

							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;

							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;

							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								if ( PlanetScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else if ( MoonScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanMoon_msg, kScienceScriptMessage);
								}
								else if ( FatherScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanFather_msg, kScienceScriptMessage);
								}

								// LNF make the Father appear if either the planet or moon is scanned or attacked
								if ( FatherAppeared == false )
								{
									FatherAppeared = true;
									Mode = kFatherArrives;
									mCreateMissionShips( kFatherArrives );
								}
								break;

							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								if ( PlanetScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else if ( MoonScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanMoon_msg, kScienceScriptMessage);
								}
								else if ( FatherScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanFather_msg, kScienceScriptMessage);
								}

								// LNF make the Father appear if either the planet or moon is scanned or attacked
								if ( FatherAppeared == false )
								{
									FatherAppeared = true;
									Mode = kFatherArrives;
									mCreateMissionShips( kFatherArrives );
								}
								break;

							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;

							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object or location

							if (( Object->mGetTeam() == kPlanetTeam )
								&& ( PlanetScanned == false ))
							{
								PlanetScanned = true;

								if ( StationDestroyed == false )
								{
									fMission.mDisplayMessage( kEvent8ScanPlanet_msg, kScienceScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kEvent8ShieldsDown_msg, kScienceScriptMessage );
								}
							}
							else if (( Object->mGetTeam() == kMoonTeam )
									&& ( MoonScanned == false ))
							{
								MoonScanned = true;
								fMission.mDisplayMessage( kEvent2ScanMoon_msg, kScienceScriptMessage );
							}
							else if ( Object->mGetTeam() == kFatherTeam )
							{
								FatherScanned = true;
								
								if ( StationDestroyed == false )
								{
									fMission.mDisplayMessage( kEvent10ScanQezl_1_msg, kScienceScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kEvent10ScanQezl_2_msg, kScienceScriptMessage );
								}
							}

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_2:
					// reveal eggs on planet
					fMission.mDisplayMessage( kEvent9FoundEggs_msg, kMissionScriptMessage );
					PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );
					break;

				case kTimerTurnsSet_8:
					// set Father comm buttons back to HAIL
					for (Ship = FatherTeam->mGetFirstShip(); Ship != NULL; Ship = FatherTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Romulan comm buttons back to HAIL
					for (Ship = RomulanTeam->mGetFirstShip(); Ship != NULL; Ship = RomulanTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kRomulanTeam)
			{
				RomulanDestroyed++;
				if (RomulanTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;

				fMission.mAddToBonusPrestige( 3 );

				// if the station is not destroyed, create 2 NEW monsters
				// for every one that has been destroyed
				if ( StationDestroyed == false )
				{
					Mode = kSpawnMonster;
					mCreateMissionShips( kSpawnMonster );
				}

				if (MonsterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kFatherTeam)
			{
				FatherDestroyed++;
				fMission.mDisplayMessage( kEvent4QezlDies_msg, kMissionScriptMessage );

				if (FatherTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kRomulanTeam:
								if ( AtWarWithRomulan == false )
								{
									fMission.mDisplayMessage( kRomHailNic_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kRomHailCombat_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage( kMonsterHail_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kFatherTeam:
								fMission.mDisplayMessage( kFatherHail_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kOrionTeam:
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kRomulanTeam:
								if ( AtWarWithRomulan == false )
								{
									fMission.mDisplayMessage( kRomSurrenderFriendly_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kRomSurrender_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage( kMonsterSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kFatherTeam:
								fMission.mDisplayMessage( kFatherSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kOrionTeam:
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kRomulanTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kRomTaunt_1_msg, kRomTaunt_4_msg), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage( kMonsterSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kFatherTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kFatherTaunt_1_msg, kFatherTaunt_4_msg), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kOrionTeam:
								break;
						}
						break;
				
				}
			}
			break;

	}

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus(status | kCampaignCompleted);
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = 0;

	VictoryPoints += (EggsRetrieved * 50) + (FatherDestroyed * 50);

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		// LNF need to add condition about victory points
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( EggsRetrieved != 0 ) ? fMission.mAddToDebriefingList( kEggsRetrievedTrue_msg ) : SuccessCount++;
	( FatherDestroyed != 0 ) ? fMission.mAddToDebriefingList( kFatherDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( EggsRetrieved == 0 ) ? fMission.mAddToDebriefingList( kEggsRetrievedFalse_msg ) : FailureCount++;
	( FatherDestroyed == 0 ) ? fMission.mAddToDebriefingList( kFatherDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	// LNF Final text?
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam)
{
	tTeamInfo* MonsterTeam = tScript.fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* RomulanTeam = tScript.fMission.mGetTeamHandle (kRomulanTeam);
	tShipInfo* Ship;

	tMapPoint waypoints_1[8]=
	{
		30, 20,
		26, 20,
		22, 22,
		20, 27,
		21, 31,
		26, 31,
		30, 27,
		31, 23
	};

	tMapPoint waypoints_2[2]=
	{
		10, 14,
		17, 14
	};

	tMapPoint PatrolPoints[8];

	if ( OrbitTeam == RomulanTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_1[0];
			PatrolPoints[1] = waypoints_1[1];
			PatrolPoints[2] = waypoints_1[2];
			PatrolPoints[3] = waypoints_1[3];
			PatrolPoints[4] = waypoints_1[4];
			PatrolPoints[5] = waypoints_1[5];
			PatrolPoints[6] = waypoints_1[6];
			PatrolPoints[7] = waypoints_1[7];
			Ship->mPatrol( 60.0, 8, &PatrolPoints[0], kHighPriority );
		}
	}
	else if ( OrbitTeam == MonsterTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_2[0];
			PatrolPoints[1] = waypoints_2[1];
			Ship->mPatrol( 50.0, 2, &PatrolPoints[0], kHighPriority );
		}
	}
}
