#include "scriptinterface.h"

#include "Gor_1MissionText.h"
#include "Gor_1MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Mode = 0;
int32 PlayerDestroyed = 0;
int32 TraitorLoneDestroyed = 0;
int32 TraitorLoneCaptured = 0;
int32 TraitorPairDestroyed = 0;
int32 TraitorPairDisengaged = 0;
int32 OrionDestroyed = 0;
int32 OrionDisengaged = 0;
int32 SuppliesDelivered = 0; // win condition
int32 ChallengeStatus = 0;
int32 DuelWon = 0; // win condition
int32 IgorashAttackedByPlayer = 0; // win condition
int32 GolashCaptured = 0; // TraitorLone == Golash
int32 DuelWonMessage = 0; // flag for the message printed
int32 TraitorLoneDestroyedMessage = 0; // flag for the message printed

float DistanceToExpireArrow = 0.0;
tShipInfo* ShipArrowIsPointingTo = NULL;

const float kGolashDistanceFromIgorash2 = 25.0; // distance Golash uses to declare arrival at Igorash II
const float kGolashDistToIgorash = 30.0; // distance from Golash to Igorash
const float kIgorashDestroyedDamage = 80.0; // amt of damage to 'destroy' Igorash
const float kOrionRetreatDamagePercent = 50.0;
const int32 kPlanet1X = 24;
const int32 kPlanet1Y = 13;
const int32 kPlanet2X = 10;
const int32 kPlanet2Y = 21;
const int32 kGolashWaitForIgorash = 0; // number of turns Golash will wait near Igorash for player's comm

void PointToNearestOfTeam(tScript& tScript, tTeamInfo* TeamToCheck, float DistanceToCheck);
eGameStatus WinCondition();

bool OutpostThreatened = false;
bool GoalIndicatorInUse = false; // flag to check if something in the mission is currently using the goal indicator
bool AtWarWithTraitor = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool FiredOnGolash = false;
bool PlayerAtIgorash2 = false;
bool OrionRetreating = false;
bool MantharFireOnPlanet = false;

int32 Elite = 0;
enum eOfferStatus
{
	kNoOffer = 0,

	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
	kAtPlanet,
	kCompletedOffer
};

enum eMissionMode
{
	kStart = 0,
	kOrionEnter,
	kTraitorEnter
};

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// for placement of ships in certain quadrants relative to the player
enum ePlacementEnum
{
	kFore = 1,
	kAft,
	kPort,
	kStarboard,
	kCatty
};
//int32 NewXCoord(tScript& tScript, int32 DirectionX, int32 DirectionY, int32 PlacementDirection);

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kTraitorPairTeam,
	kTraitorLoneTeam,
	kOrionTeam,
	kPlanetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface -
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kTraitorPairTeam, kGorn);
	fMission.mSetTeamRace (kTraitorLoneTeam, kGorn);
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	// this mission starts without any enemy ships on screen
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

/*
	fMission.mSetTeamRelations (kPlayerTeam, kTraitorPairTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kTraitorLoneTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kTraitorPairTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kTraitorPairTeam, kTraitorLoneTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionTeam, kTraitorLoneTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
*/
	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
		return 1; // map 1
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
		return 2; // map 2
	}
	else
	{
		Elite = kErrant;
		return 3; // map 3
	}
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TraitorPairTeam = fMission.mGetTeamHandle (kTraitorPairTeam);
	tTeamInfo* TraitorLoneTeam = fMission.mGetTeamHandle (kTraitorLoneTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		// create planets
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPlanet1X, kPlanet1Y );
		PlanetTeam->mCreateShip( "PL2", kStartPosition_Specified, kPlanet2X, kPlanet2Y );
	}
	else if ( Mode == kOrionEnter )
	{
		// create Orion team
		int32 NumberOfOrions = 0;
		int32 OldX = 0;
		int32 OldY = 0;
		fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + fMission.mRandomInt32 (8, 9);
		int32 NewY = OldY + fMission.mRandomInt32 (-8, 8);

		fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlanetTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		if ( NumberOfPlayersShips >= 2 )
		{
			NumberOfOrions = 2;
		}
		else
		{
			NumberOfOrions = 1;
		}

		if ( PlayerShipClass > kClassFrigate )
		{
			eClassTypes LargestOrionShipClass = (eClassTypes) ((int32)PlayerShipClass - 2);

			if ( LargestOrionShipClass < kClassFrigate )
			{
				LargestOrionShipClass = kClassFrigate;
			}

			OrionTeam->mCreateFleet (NumberOfOrions, (AveragePlayerBPV * NumberOfOrions), kClassFrigate, LargestOrionShipClass, 1, kStartPosition_Specified, NewX, NewY);
		}
		else
		{
			OrionTeam->mCreateFleet (NumberOfOrions, (AveragePlayerBPV * NumberOfOrions), kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, NewX, NewY);
		}

		fMission.mDisplayMessage( kOrionsArrive_msg, kMissionScriptMessage );
	}
	else if ( Mode == kTraitorEnter )
	{
		// create Traitor team
		// LNF change ship type for balance?
		int32 OldX = 0;
		int32 OldY = 0;
		fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + fMission.mRandomInt32 (-4, 4);
		int32 NewY = OldY + fMission.mRandomInt32 (9, 12);

		fMission.mSetTeamRelations (kTraitorPairTeam, kTraitorLoneTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kTraitorPairTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		// LNF changing this to make Golash fire on the player while he is pounding the planet
		fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		TraitorPairTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, NewX + 1, NewY );

		if ( PlayerTeam->mGetCombatBPV() > 140 ) // 70 is old value
		{
			TraitorPairTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, NewX - 1, NewY );
		}

		TraitorLoneTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 100, kStartPosition_Specified, NewX, NewY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if ( Elite == kKingsGuard )
	{
		return( kKingsGuardBriefing_msg );
	}
	else if ( Elite == kEggDefender )
	{
		return( kEggDefenderBriefing_msg );
	}
	else
	{
		return( kErrantBriefing_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TraitorPairTeam = fMission.mGetTeamHandle (kTraitorPairTeam);
	tTeamInfo* TraitorLoneTeam = fMission.mGetTeamHandle (kTraitorLoneTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}
	fPlayerShip->mAccessSupplies( kSetTransporterItemCount, kTransMedicalSupplies, 1 );
	fPlayerShip->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 1 );

	// configure the planets
	PlanetTeam->mGetThisShip( 1 )->mChangeName( kPlanetIgorash_name );
	PlanetTeam->mGetThisShip( 1 )->mSetCommButton (1, kButtonHail_text);
	PlanetTeam->mGetThisShip( 1 )->mSetShipCallbackEvents( kNotifyDamage );

	PlanetTeam->mGetThisShip( 2 )->mChangeName( kPlanetIgorash2_name );
	PlanetTeam->mGetThisShip( 2 )->mSetCommButton (1, kButtonHail_text);

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	//fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to bring in Orions
	fMission.mSetTimedInterval (kTimerTurnsSet, 4);

	// start message
	fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
	fMission.mDisplayMessage( kStartScenario_2_msg, kMissionScriptMessage );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* TraitorPairTeam = fMission.mGetTeamHandle (kTraitorPairTeam);
	tTeamInfo* TraitorLoneTeam = fMission.mGetTeamHandle (kTraitorLoneTeam);

	if ( Mode == kOrionEnter )
	{
		// configure Orion team
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		PointToNearestOfTeam( *this, OrionTeam, 70.0);
	}
	else if ( Mode == kTraitorEnter )
	{
		Ship->mSetShipThrust (10);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		
		Ship->mSetFinalDestination(kPlanet1X, kPlanet1Y);

		// maybe put in a timer that checks when Golash's ship gets near ( 30.0 ) of the planet
		// and then he starts bombarding it after a few turns?
		fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );

		if ( Ship->mGetTeam() == kTraitorLoneTeam )
		{
			PointToNearestOfTeam( *this, TraitorLoneTeam, 70.0);
			//Ship->mChangeName( kGolash_name );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TraitorPairTeam = fMission.mGetTeamHandle (kTraitorPairTeam);
	tTeamInfo* TraitorLoneTeam = fMission.mGetTeamHandle (kTraitorLoneTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				if ( TransportTarget != NULL )
				{
					if ( fEventItem1 == kTransMedicalSupplies )
					{
						if (( Ship->mGetTeam() == kPlayerTeam ) &&
							( TransportTarget == PlanetTeam->mGetThisShip( 1 ) ))
						{
							// win condition
							fMission.mDisplayMessage( kEvent3BeamSupplies_msg, kScienceScriptMessage );

							SuppliesDelivered = 1;
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
							return WinCondition();
						}
						else
						{
							// make it so the player can't transport outside of own team
							if ( TransportTarget->mGetOriginalTeam() != kPlayerTeam )
							{
								fMission.mDisplayMessage( kMedicalSuppliesNotUseful_msg, kMissionScriptMessage );

								Ship->mAccessSupplies( kSetTransporterItemCount, kTransMedicalSupplies, 1 );
								TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransMedicalSupplies, 0 );
							}
						}
					}
					else if ( fEventItem1 == kTransGornCaptain )
					{
						if ( DuelWon == 0 )
						{
							if ( TransportTarget == PlanetTeam->mGetThisShip( 2 ) )
							{
								PlayerAtIgorash2 = true;

								if ( ChallengeStatus == kAtPlanet )
								{
									// win condition
									DuelWon = 1;
									TraitorLoneDestroyed++;
									DuelWonMessage = 1; // duel won
									TraitorLoneDestroyedMessage = 2;// traitorlone is not destroyed, but duel won

									fMission.mDisplayMessage( kEvent6PlayerBeamsDown_msg, kMissionScriptMessage );
									Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
									ChallengeStatus = kCompletedOffer;

									// set player wins duel timer
									fMission.mSetTimedInterval( kTimerTurnsSet_4, 1 );
								}
								else
								{
									fMission.mDisplayMessage( kWaitForGolash_msg, kMissionScriptMessage );
									Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 1 );
									TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 0 );
								}
							}
							else
							{
								// make it so the player can't transport *outside* of own team
								if ( TransportTarget->mGetOriginalTeam() != kPlayerTeam )
								{
									fMission.mDisplayMessage( kBeamDownToIgorash2_msg, kMissionScriptMessage );

									Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 1 );
									TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 0 );
								}
							}
						}
						else // DuelWon == 1
						{
							// message to deny transport
							fMission.mDisplayMessage( kAwayTeamAlreadyUsed_msg, kMissionScriptMessage );

							Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 0 );
						}
					}
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				if ( TransportTarget != NULL )
				{
					// don't allow the player to transport the medical supplies once they are on the planet
					if ( fEventItem1 == kTransMedicalSupplies )
					{
						if (( Ship->mGetTeam() == kPlayerTeam ) &&
							( SuppliesDelivered == 1 ))
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransMedicalSupplies, 0 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransMedicalSupplies, 1 );
						}
					}
					else if ( fEventItem1 == kTransGornCaptain )
					{
						if (( Ship->mGetTeam() == kPlayerTeam ) &&
							( DuelWon == 1 ))
						{
							fMission.mDisplayMessage( kTeamRecovered_msg, kMissionScriptMessage );
						}
					}
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					// if the Orion are damaged more than 50%, make them disengage
					if ( Ship->mGetTeam() == kOrionTeam )
					{
						if (( Ship->mGetDamagePercent() >= kOrionRetreatDamagePercent )
							&& ( OrionRetreating == false ))
						{
							fMission.mDisplayMessage( kEvent7OrionRetreating_msg, kMissionScriptMessage );
							Ship->mDisengage( kByMapEdge, kVeryHighPriority );
							OrionRetreating = true;
						}
						else
						{
							Ship->mSetShipCallbackEvents( kNotifyDamage );
						}
					}
					else if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship == PlanetTeam->mGetFirstShip() )
						{
							if ( IgorashAttackedByPlayer == 0 )
							{
								fMission.mDisplayMessage( kMantharBombed_msg, kMissionScriptMessage );

								IgorashAttackedByPlayer = 1;
							}

							if ( PlanetTeam->mGetFirstShip()->mGetDamagePercent() >= kIgorashDestroyedDamage )
							{
								fMission.mDisplayMessage( kIgorashDestroyed_msg, kMissionScriptMessage );
							}
						}
						else if (( Ship->mGetTeam() == kTraitorPairTeam ) && ( AtWarWithTraitor == false ))
						{
							TraitorPairTeam->mClearCommands();
							fMission.mSetTeamRelations (kTraitorPairTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							AtWarWithTraitor = true;
						}
						else if ( Ship->mGetTeam() == kTraitorLoneTeam )
						{
							if ( AtWarWithTraitor == false )
							{
								AtWarWithTraitor = true;

								if ( ChallengeStatus == kNoOffer )
								{
									TraitorPairTeam->mClearCommands();
									fMission.mSetTeamRelations (kTraitorPairTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
									
									// LNF if the player shoots at Golash, he shouldn't give the challenge message?
									// LNF and if the message is already up, it should be taken down.?
									// LNF but what if Golash is already attacking the planet?  Then the player should have to 
									// LNF fire on Golash to make him stop, then be given the challenge?
									// LNF but the challenge is automatically given when the traitorPair team is destroyed?
									// LNF so this would only happen if the player is getting pounded on by the traitorPair?
									// LNF team and the player is attacking the traitorLone team.  that would be weird.?
								}
								else if ( ChallengeStatus == kMadeOffer )
								{
									// kMadeOffer isn't set until the TraitorPair team is destroyed.
									FiredOnGolash = true;
									TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
									TraitorLoneTeam->mGetFirstShip()->mClearCommButton( 2 );
									fMission.mDisplayMessage( kRejectOffer_msg, kMissionScriptMessage );
									ChallengeStatus = kRejectedOffer;
									fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								else if (( ChallengeStatus == kAcceptedOffer ) || ( ChallengeStatus == kAtPlanet ))
								{
									FiredOnGolash = true;
									ChallengeStatus = kRejectedOffer;
									fMission.mDisplayMessage( kEvent4FireOnGolash_msg, kMissionScriptMessage );
									fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								else if ( ChallengeStatus == kCompletedOffer )
								{
									// LNF need a message here?
									fMission.mDisplayMessage( kFireAfterWinDuel_msg, kMissionScriptMessage );


									FiredOnGolash = true;
									fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
							}
							else // AtWarWithTraitor == true
							{
								if ( FiredOnGolash == false )
								{
									FiredOnGolash = true;

									if ( ChallengeStatus == kMadeOffer )
									{
										// kMadeOffer isn't set until the TraitorPair team is destroyed.
										TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
										TraitorLoneTeam->mGetFirstShip()->mClearCommButton( 2 );
										fMission.mDisplayMessage( kRejectOffer_msg, kMissionScriptMessage );
										ChallengeStatus = kRejectedOffer;
										fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
									}
									else if (( ChallengeStatus == kAcceptedOffer ) || ( ChallengeStatus == kAtPlanet ))
									{
										ChallengeStatus = kRejectedOffer;
										fMission.mDisplayMessage( kEvent4FireOnGolash_msg, kMissionScriptMessage );
										fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
									}
									else if ( ChallengeStatus == kCompletedOffer )
									{
										// LNF need a message here?
										fMission.mDisplayMessage( kFireAfterWinDuel_msg, kMissionScriptMessage );

										fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
									}
								}
							}
						}
					}
					else if (( HostileShip->mGetTeam() == kTraitorLoneTeam ) && ( Ship == PlanetTeam->mGetFirstShip() ))
					{
						if ( Ship->mGetDamagePercent() >= kIgorashDestroyedDamage )
						{
							// LNF Igorash destroyed message
							fMission.mSetTeamRelations (kTraitorLoneTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
							TraitorLoneTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
							
							if ( TraitorPairTeam->mGetShipsInPlayCount() > 0 )
							{
								TraitorPairTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
							}
							break;
						}
						else if ( MantharFireOnPlanet == false )
						{
							MantharFireOnPlanet = true;

							fMission.mDisplayMessage( kMantharFireOnPlanetWarning_msg, kMissionScriptMessage );
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			if (( Ship->mGetTeam() == kTraitorLoneTeam ) &&
				( Ship->mGetTrackingDistance() == kGolashDistanceFromIgorash2 ))
			{
				if ( ChallengeStatus == kAcceptedOffer )
				{
					ChallengeStatus = kAtPlanet;

					// LNF set the speed to 12, but if the player shoots at ship, reset to max 31
					//Ship->mSetShipMaxSpeed( 12 );

					if ( PlayerAtIgorash2 == true )
					{
						fMission.mDisplayMessage( kEvent6PlayerBeamsDown_msg, kMissionScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_4, 1 );
					}
					else
					{
						fMission.mDisplayMessage( kEvent6GolashArrives_msg, kMissionScriptMessage );
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // Traitor team arrives
					Mode = kTraitorEnter;
					mCreateMissionShips( kTraitorEnter );
					fMission.mDisplayMessage( kTraitorArrive_msg, kScienceScriptMessage );

					// display intercept communications message
					fMission.mSetTimedInterval( kTimerLimitSet_5, 4 );
					//fMission.mSetTimedInterval (kTimerTurnsSet_2, 1);
					break;

				case kTimerLimitSet_2: // show intercepted message
					{
						fMission.mDisplayMessage( kEvent2Message_msg, kMissionScriptMessage );
						fMission.mDisplayMessage( kEvent2Response_msg, kMissionScriptMessage );
						OutpostThreatened = true;

						TraitorLoneTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
						for (tShipInfo* tempShip = TraitorPairTeam->mGetFirstShip(); tempShip != NULL; tempShip = TraitorPairTeam->mGetNextShip())
						{
							tempShip->mSetCommButton (1, kButtonHail_text);
						}
						// LNF if the player hails either the Outpost or the Traitors,
						// LNF two traitor ships will attack
					}
					break;

				case kTimerLimitSet_3:
					{
						// check distance to expire goal indicator
						for (tShipInfo* PlayerShip = PlayerTeam->mGetFirstShip(); PlayerShip != NULL; PlayerShip = PlayerTeam->mGetNextShip())
						{
							if ( PlayerShip->mGetTacticalDistance( ShipArrowIsPointingTo ) < DistanceToExpireArrow )
							{
								fMission.mSetGoalIndicator( kFalse, 0, 0 );
								ShipArrowIsPointingTo = NULL;
								DistanceToExpireArrow = 9999.0;
								GoalIndicatorInUse = false;
								break;
							}
						}

						fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					}
					break;

				case kTimerLimitSet_4:
					// check to see if Golash (TraitorLoneTeam) get's close to Igorash
					if ( TraitorLoneTeam->mGetFirstShip()->mGetTacticalDistance( PlanetTeam->mGetFirstShip() ) <= kGolashDistToIgorash )
					{
						// set a timer that will check how long he is 'orbiting' and make Golash attack planet if this timer expires
						fMission.mSetTimedInterval( kTimerTurnsSet_3, kGolashWaitForIgorash );

						// LNF maybe a message here saying Golash is close to attacking the planet?
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
					}
					break;

				case kTimerLimitSet_5: // Intercept Traitor message
					fMission.mDisplayMessage( kInterceptMessage_msg, kMissionScriptMessage, TraitorLoneTeam->mGetFirstShip() );

					// display traitor threaten message
					fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
					break;

				case kTimerTurnsSet: // Orions Enter
					Mode = kOrionEnter;
					mCreateMissionShips( kOrionEnter );
					break;

				case kTimerTurnsSet_2:
					break;

				case kTimerTurnsSet_3:
					if ( GolashCaptured == 0 )
					{
						// Golash starts attacking Igorash
						fMission.mSetTeamRelations (kTraitorLoneTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0 );
						fMission.mSetTeamRelations (kPlanetTeam, kTraitorLoneTeam, kTeamNeutral, kGoalIgnore, 1.0 );
						TraitorLoneTeam->mGetFirstShip()->mSetShipTarget( PlanetTeam->mGetFirstShip(), kVeryLowPriority );
					}
					break;

				case kTimerTurnsSet_4:
					// Golash and player duel
					fMission.mDisplayMessage( kPlayerWinsDuel_msg, kMissionScriptMessage );
					return WinCondition();
					break;

				case kTimerTurnsSet_8:
					// set Orion ship's comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Traitor Lone ship's comm buttons back to HAIL
					for (Ship = TraitorLoneTeam->mGetFirstShip(); Ship != NULL; Ship = TraitorLoneTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Traitor Pair Team comm buttons back to HAIL
					for (Ship = TraitorPairTeam->mGetFirstShip(); Ship != NULL; Ship = TraitorPairTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship->mGetTeam() == kOrionTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			else if ( Ship->mGetTeam() == kTraitorPairTeam )
			{
				TraitorPairDestroyed++;

				if (TraitorPairTeam->mGetShipsInPlayCount() == 0)
				{
					if ( TraitorLoneTeam->mGetShipsInPlayCount() > 0 )
					{
						fMission.mDisplayMessage( kRenegadeChallenge_msg, kMissionScriptMessage );
						ChallengeStatus = kMadeOffer;

						if ( FiredOnGolash == false )
						{
							TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonAccept_text, kTrue );
							TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 2, kButtonReject_text );
						}


						// LNF challenge to duel on Igorash II
						// if Gorn ship is fired upon, no duel
						// otherwise, player goes to planet and beams down(what?) to automatically win
					}
				}
			}
			else if ( Ship->mGetTeam() == kTraitorLoneTeam )
			{
				// LNF more bonus points, and maybe a special message in the debriefing?
				fMission.mAddToBonusPrestige( kCaptureBonus );

				TraitorLoneDestroyed++; // need this to get the victory points/message

				GolashCaptured++;
				DuelWon = 1;
				TraitorLoneDestroyed++;
				TraitorLoneDestroyedMessage = 2;// golash not destroyed, but captured
				DuelWonMessage = 2; // duel not won, golash captured
				return WinCondition();
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kTraitorLoneTeam)
			{
				TraitorLoneDestroyed++;
				TraitorLoneDestroyedMessage = 1;
				return WinCondition();
			}
			else if (Ship->mGetOriginalTeam() == kTraitorPairTeam)
			{
				TraitorPairDestroyed++;

				if (TraitorPairTeam->mGetShipsInPlayCount() == 0)
				{
					if ( TraitorLoneTeam->mGetShipsInPlayCount() > 0 )
					{
						fMission.mDisplayMessage( kRenegadeChallenge_msg, kMissionScriptMessage );
						ChallengeStatus = kMadeOffer;

						if ( FiredOnGolash == false )
						{
							TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonAccept_text, kTrue );
							TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 2, kButtonReject_text );
						}


						// LNF challenge to duel on Igorash II
						// if Gorn ship is fired upon, no duel
						// otherwise, player goes to planet and beams down(what?) to automatically win
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				OrionDestroyed++;

				// LNF temporary line
				GoalIndicatorInUse = false;

				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
					// the Traitor team arrives
					fMission.mSetTimedInterval (kTimerLimitSet, fMission.mRandomInt32( 8, 16 ));
					// LNF maybe play a message SCIENCE: ships arriving, intercept transmission
					// LNF they make some kind of threat the player has to respond to
					//fMission.mSetTimedInterval (kTimerLimitSet, 4);
				}

				return WinCondition();
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			// LNF if the Orions disengage, the traitor team arrives
			if ( Ship->mGetOriginalTeam() == kOrionTeam )
			{
				OrionDisengaged++;

				fMission.mDisplayMessage( kLNFOrionDisengaged_msg );
				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
					// traitor team arrive
					fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				}
			}
			else if ( Ship->mGetOriginalTeam() == kTraitorPairTeam )
			{
				TraitorPairDisengaged++;

				if (TraitorPairTeam->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kRenegadeChallenge_msg, kMissionScriptMessage );
					ChallengeStatus = kMadeOffer;

					if ( FiredOnGolash == false )
					{
						TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonAccept_text, kTrue );
						TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 2, kButtonReject_text );
					}


					// LNF challenge to duel on Igorash II
					// if Gorn ship is fired upon, no duel
					// otherwise, player goes to planet and beams down(what?) to automatically win
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonAccept_text:
						ChallengeStatus = kAcceptedOffer;
						fMission.mDisplayMessage( kAcceptedOffer_msg, kMissionScriptMessage );
						//fMission.mSetTeamRelations (kTraitorLoneTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

						TraitorLoneTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kTraitorLoneTeam, kTraitorPairTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kTraitorLoneTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

						TraitorLoneTeam->mGetFirstShip()->mSetFinalDestination( kPlanet2X, kPlanet2Y );
						TraitorLoneTeam->mGetFirstShip()->mTrackTacticalDistance( kGolashDistanceFromIgorash2, kPlanet2X, kPlanet2Y );

						TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
						TraitorLoneTeam->mGetFirstShip()->mClearCommButton( 2 );
						// LNF the gorn ship should go the planetteam( 2 )
						break;

					case kButtonReject_text:
						ChallengeStatus = kRejectedOffer;
						fMission.mDisplayMessage( kRejectOffer_msg, kMissionScriptMessage );
						TraitorLoneTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kTraitorLoneTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						TraitorLoneTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
						break;

					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kTraitorLoneTeam:

								if ( ChallengeStatus == kAtPlanet )
								{
									fMission.mDisplayMessage( kGolashOnPlanet_msg, kMissionScriptMessage );
									Ship->mSetCommButton( 1, kButtonHail_text );
									break;
								}
								else
								{
									if ( AtWarWithTraitor == false )
									{
										fMission.mDisplayMessage( kRenegadeHailNic_msg, kMissionScriptMessage );
									}
									else
									{
										if ( ChallengeStatus == kNoOffer )
										{
											fMission.mDisplayMessage( kRenegadeHailCombat_msg, kMissionScriptMessage );
										}
										else if ( ChallengeStatus == kAcceptedOffer )
										{
											fMission.mDisplayMessage( kGolashHailAcceptOffer_msg, kMissionScriptMessage );
										}
										else if ( ChallengeStatus == kRejectedOffer )
										{
											fMission.mDisplayMessage( kRejectOffer_msg, kMissionScriptMessage );
										}
										else if ( ChallengeStatus = kCompletedOffer )
										{
											fMission.mDisplayMessage( kHailAfterWinDuel_msg, kMissionScriptMessage );
											break;
										}

									}
								}
								Ship->mSetCommButton( 1, kButtonHail_text );
								
/*
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
*/
								break;

							case kTraitorPairTeam:
								if ( OutpostThreatened == true )
								{
									TraitorPairTeam->mClearCommands();
									fMission.mSetTeamRelations (kTraitorPairTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								else
								{
/*
									Ship->mSetCommButton (1, kButtonTaunt_text);
									Ship->mSetCommButton (2, kButtonSurrender_text);
									// If the Taunt or Surrender buttons aren't used in X turns
									// return the buttons to Hail and blank.
									fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
*/
								}

								fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
								break;

							case kPlanetTeam:
								if ( Ship == PlanetTeam->mGetFirstShip() )
								{
									if (( TraitorPairTeam->mGetShipsInPlayCount() == 0 )
										&& ( TraitorLoneTeam->mGetShipsInPlayCount() == 0 ))
									{
										fMission.mDisplayMessage( kMantharSaved_msg, kMissionScriptMessage );
									}
									else
									{
										fMission.mDisplayMessage( kMantharHelp_msg, kMissionScriptMessage );
									}

									if ( OutpostThreatened == true )
									{
										// Golash delivers ultimatum
										fMission.mDisplayMessage( kPairAttacks_msg, kMissionScriptMessage );

										TraitorPairTeam->mClearCommands();
										fMission.mSetTeamRelations (kTraitorPairTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
										fMission.mSetTeamRelations (kTraitorPairTeam, kTraitorLoneTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
									}
								}
								else
								{
									fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
								}
								break;

							case kOrionTeam:
								fMission.mDisplayMessage( kOrionHail_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kTraitorLoneTeam:
								break;

							case kTraitorPairTeam:
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPlanetTeam:
								break;

							case kOrionTeam:
								fMission.mDisplayMessage( kOrionSurrenderOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
                                fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kTraitorLoneTeam:
								break;

							case kTraitorPairTeam:
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPlanetTeam:
								break;

							case kOrionTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage);

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (SuppliesDelivered * 50) - (IgorashAttackedByPlayer * 100);


	// LNF destroying AND winning the duel should give the player bonus points?
	// LNF maybe not

	if ( Elite == kKingsGuard )
	{
		VictoryPoints += (OrionDestroyed * 14) + (DuelWon * 11) + (TraitorLoneDestroyed * 25); 
	}
	else if ( Elite == kEggDefender )
	{
		VictoryPoints += (OrionDestroyed * 14) + (DuelWon * 21) + (TraitorLoneDestroyed * 15);
	}
	else // Elite == kErrant
	{
		VictoryPoints += (OrionDestroyed * 10) + (DuelWon * 30) + (TraitorLoneDestroyed * 10);
	}


	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDefeat;
	}

	// remove transporter items from the player's ships
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransMedicalSupplies, 0 );
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornCaptain, 0 );
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kKingsGuard)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kKingsGuardAstoundingVictory_msg);
		}
		else if (fMission.mGetEliteOrganization() == kEggDefender)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kEggDefenderAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kErrantAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SuppliesDelivered != 0 ) ? fMission.mAddToDebriefingList( kSuppliesDeliveredTrue_msg ) : SuccessCount++;
	( OrionDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOrionDestroyedTrue_msg ) : SuccessCount++;

	if ( GolashCaptured != 0 )
	{
		fMission.mAddToDebriefingList( kTraitorLoneCapturedTrue_msg );
	}
	else if ( TraitorLoneDestroyedMessage == 1 )
	{
		fMission.mAddToDebriefingList( kTraitorLoneDestroyedTrue_msg );
	}
	else
	{
		SuccessCount++;
	}

	if ( DuelWonMessage == 2 )
	{
		SuccessCount++;
	}
	else if ( DuelWonMessage == 1 )
	{
		fMission.mAddToDebriefingList( kDuelWonTrue_msg );
	}
	else
	{
		SuccessCount++;
	}

	if ( SuccessCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SuppliesDelivered == 0 ) ? fMission.mAddToDebriefingList( kSuppliesDeliveredFalse_msg ) : FailureCount++;
	( OrionDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOrionDestroyedFalse_msg ) : FailureCount++;

	if ( TraitorLoneDestroyedMessage == 0 )
	{
		fMission.mAddToDebriefingList( kTraitorLoneDestroyedFalse_msg );
	}
	else if ( TraitorLoneDestroyedMessage == 2 )
	{
		FailureCount++;
	}
	else
	{
		FailureCount++;
	}

	if ( DuelWonMessage == 0 )
	{
		fMission.mAddToDebriefingList( kDuelWonFalse_msg );
	}
	else
	{
		FailureCount++;
	}

	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;

		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


//---------------------------------------------------------------------------------------------
// return quadrant specific coordinates relative to the player
/*
int32 NewXCoord(tScript& tScript, int32 DirectionX, int32 DirectionY, int32 PlacementDirection)
{
	int32 PlayerX;
	int32 PlayerY;

	tScript.fPlayerShip->mGetGridPosition (&PlayerX, &PlayerY);

	int32 DeltaX = PlayerX - DirectionX;
	int32 DeltaY = PlayerY - DirectionY;

	int32 OffsetX;
	switch ( PlacementDirection )
	{
		case kFore:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kAft:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kPort:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kStarboard:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kCatty:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;
	}
}
*/

//---------------------------------------------------------------------------------------------
// determine the distance between the player's ship and the nearest ship of another team
// and set the goal indicator to point at that ship
void PointToNearestOfTeam(tScript& tScript, tTeamInfo* TeamToCheck, float DistanceToCheck)
{
	float ClosestDistance = 9999.0;
	tShipInfo* ClosestShip = NULL;

	if ( GoalIndicatorInUse == false )
	{
		for (tShipInfo* Ship = TeamToCheck->mGetFirstShip(); Ship != NULL; Ship = TeamToCheck->mGetNextShip())
		{
			if ( tScript.fPlayerShip->mGetTacticalDistance( Ship ) < ClosestDistance )
			{
				ClosestDistance = tScript.fPlayerShip->mGetTacticalDistance( Ship );
				ClosestShip = Ship;
			}
		}

		if ( ClosestDistance > DistanceToCheck )
		{
			tScript.fMission.mSetGoalIndicator( ClosestShip );
			GoalIndicatorInUse = true;

			// check the distance to the ship and expire the goal indicator when within that distance
			DistanceToExpireArrow = DistanceToCheck;
			ShipArrowIsPointingTo = ClosestShip;
			tScript.fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
		}
	}
}

//---------------------------------------------------------------------------------------------
// determine the distance between the player's ship and the nearest ship of another team
// and set the goal indicator to point at that ship
eGameStatus WinCondition()
{
	if (( SuppliesDelivered == 1 ) && ( OrionDestroyed > 0 ) &&
		(( TraitorLoneDestroyed >= 1 ) || ( DuelWon == 1 )))
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}