#include "scriptinterface.h"

#include "Gor_2MissionText.h"
#include "Gor_2MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Mode = 0;
int32 PlayerDestroyed = 0;
int32 MantharDestroyed = 0;
int32 MantharCaptured = 0;
int32 BaseDestroyed = 0;
int32 DefenseDestroyed = 0;
int32 OrionDestroyed = 0;
int32 OrionDisengaged = 0;
int32 CacheRecovered = 0;
int32 CacheDestroyed = 0;
int32 BaseSecured = 0;
int32 NumberOfPlayersShips = 0;
int32 CommandoShipDestroyed = 0;

const float kOrionRetreatDamagePercent = 50.0;
const float kOrionSurrenderDamagePercent = 80.0;
const float kMantharSurrenderDamagePercent = 70.0;
const float kDeepScanDistance = 15.0;
const int32 kMaxCache = 2; // the number of caches (asteroid bases) the player has to capture
const float kActivateMantharDistance = 100.0;

bool OutpostThreatened = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key


eGameStatus CheckWinCondition( tScript& tScript );

int32 Elite = 0;
enum eOfferStatus
{
	kNoOffer = 0,

	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
};

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eMissionMode
{
	kStart = 0,
	kPlaceShips,
	kOrionEnter
};

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// for placement of ships in certain quadrants relative to the player
enum ePlacementEnum 
{
	kFore = 1,
	kAft,
	kPort,
	kStarboard,
	kCatty
};

//int32 NewXCoord(tScript& tScript, int32 DirectionX, int32 DirectionY, int32 PlacementDirection);

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMantharTeam,
	kCacheTeam,
	kBaseTeam,
	kDefenseTeam,
	kOrionTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMantharTeam, kGorn);
	fMission.mSetTeamRace (kCacheTeam, kGorn);
	fMission.mSetTeamRace (kBaseTeam, kGorn);
	fMission.mSetTeamRace (kDefenseTeam, kGorn);
	fMission.mSetTeamRace (kOrionTeam, kOrion);

	// Set team relations to show up on Tactical map
	// this mission starts without any enemy ships on screen


	fMission.mSetTeamRelations (kPlayerTeam, kMantharTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kCacheTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

  	fMission.mSetTeamRelations (kMantharTeam, kCacheTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
  	fMission.mSetTeamRelations (kMantharTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
  	fMission.mSetTeamRelations (kMantharTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
  	fMission.mSetTeamRelations (kMantharTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

   	fMission.mSetTeamRelations (kCacheTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
   	fMission.mSetTeamRelations (kCacheTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
   	fMission.mSetTeamRelations (kCacheTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

   	fMission.mSetTeamRelations (kBaseTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
   	fMission.mSetTeamRelations (kBaseTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

   	fMission.mSetTeamRelations (kDefenseTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MantharTeam = fMission.mGetTeamHandle (kMantharTeam);
	tTeamInfo* CacheTeam = fMission.mGetTeamHandle (kCacheTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	// this is not the standard balance statements...NumberOfPlayersShips is global here
	NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;

		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;

		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		// going with the G-COM because it's available in all time periods
		PlayerTeam->mCreateShip( "G-COM", kStartPosition_J );
	}	
	else if ( Mode == kPlaceShips )
	{
		// place the ships in the game that shouldn't appear on the briefing tactical map

		// LNF create orion ships based on the player's BPV?
		// LNF maybe randomly place the ship(s) near the cache like they are raiding them
		// LNF maybe the Manthar ship should be chasing the orion ship?

		// LNF this will need a lot of tweaking since the player will probably still be very weak

		// create Manthar ship(s)
		MantharTeam->mCreateShip( AveragePlayerBPV, kClassFreighter, LargestEnemyShipClass, 1, kStartPosition_R );

		// create Defense
		DefenseTeam->mCreateShip( "G-DEF", kStartPosition_S );
		DefenseTeam->mCreateShip( "G-DEF", kStartPosition_T );
		DefenseTeam->mCreateShip( "G-DEF", kStartPosition_U );
		DefenseTeam->mCreateShip( "G-DEF", kStartPosition_V );
		DefenseTeam->mCreateShip( "G-DEF", kStartPosition_W );

		// create Cache
		CacheTeam->mCreateShip( "N-ABPh", kStartPosition_X );
		CacheTeam->mCreateShip( "N-ABPh", kStartPosition_Y );

		// create Base
		BaseTeam->mCreateShip( "G-BS", kStartPosition_Z );

		// create Orion
		if ( fMission.mRandomInt32( 1, 2 ) == 1 )
		{
			OrionTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_P );
		}
		else
		{
			OrionTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Q );
		}
	}
	else if ( Mode == kOrionEnter )
	{
		// create Orion team
		int32 OldX = 0;
		int32 OldY = 0;
		fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + fMission.mRandomInt32 (7, 8);
		int32 NewY = OldY + fMission.mRandomInt32 (-8, 8);

		//OrionTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
		OrionTeam->mCreateFleet (2, (TotalBPV/2.0), kClassFrigate, kClassFrigate, 50, kStartPosition_Specified, NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if ( Elite == kKingsGuard )
	{
		return( kKingsGuardBriefing_msg );
	}
	else if ( Elite == kEggDefender )
	{
		return( kEggDefenderBriefing_msg );
	}
	else
	{
		return( kErrantBriefing_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MantharTeam = fMission.mGetTeamHandle (kMantharTeam);
	tTeamInfo* CacheTeam = fMission.mGetTeamHandle (kCacheTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}
	PlayerTeam->mGetThisShip( NumberOfPlayersShips + 1 )->mChangeName( kEviscerator_name );
	PlayerTeam->mGetThisShip( NumberOfPlayersShips + 1 )->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 42 );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to place ships that shouldn't appear on briefing tactical map and set relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to check player's distance to Manthar ship
	fMission.mSetTimedInterval (kTimerLimitSet_2, 1);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	Ship->mSetShipCallbackEvents( kNotifyDamage );

	if ( Mode == kPlaceShips )
	{
		// LNF might have to damage the base a little

		if ( Ship->mGetTeam() == kOrionTeam )
		{
			Ship->mSetShipThrust( 10 );
			Ship->mSetFinalDestination();
		}
		else if ( Ship->mGetTeam() == kCacheTeam )
		{
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 8 );
		}
		else if ( Ship->mGetTeam() == kBaseTeam )
		{
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 16 );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
			Ship->mDamageSystem( kDamageWeapon11, 100.0f );
			Ship->mDamageSystem( kDamageWeapon15, 100.0f );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MantharTeam = fMission.mGetTeamHandle (kMantharTeam);
	tTeamInfo* CacheTeam = fMission.mGetTeamHandle (kCacheTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					// if the Orion are damaged more than 50%, make them disengage
					if ( Ship->mGetOriginalTeam() == kOrion )
					{
						if ( Ship->mGetDamagePercent() >= kOrionRetreatDamagePercent )
						{
							// LNF need a message here?
							Ship->mDisengage( kByMapEdge, kVeryHighPriority );
						}
					}
					else if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kDefenseTeam )
						{
							fMission.mSetTeamRelations (kPlayerTeam, kCacheTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kBaseTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0);
						}
					}
				}				
			}
			break;

		case kNotifyAtDestination:
			if ( Ship->mGetTeam() == kOrionTeam )
			{
				Ship->mDisengage( kByMapEdge, kAbsolutePriority );
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// place ships that shouldn't appear on debriefing tactical map and set relations

					fMission.mSetTeamRelations (kPlayerTeam, kMantharTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kCacheTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0);//***

  					fMission.mSetTeamRelations (kMantharTeam, kCacheTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
  					fMission.mSetTeamRelations (kMantharTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
  					fMission.mSetTeamRelations (kMantharTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
  					fMission.mSetTeamRelations (kMantharTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

   					fMission.mSetTeamRelations (kCacheTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
   					fMission.mSetTeamRelations (kCacheTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
   					fMission.mSetTeamRelations (kCacheTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

   					fMission.mSetTeamRelations (kBaseTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
   					fMission.mSetTeamRelations (kBaseTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDefenseTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
   					fMission.mSetTeamRelations (kDefenseTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***

					Mode = kPlaceShips;
					mCreateMissionShips( kPlaceShips );

					// deep scan base from within kDeepScanDistance for 1 turn
					DeepScanTracker.mSetTarget (CacheTeam->mGetThisShip(1), kDeepScanDistance, 1);
					DeepScanTracker.mSetTarget (CacheTeam->mGetThisShip(2), kDeepScanDistance, 1);
					break;

				case kTimerLimitSet_2:
					{
						// checks distance from player to Manthar ship
						for (tShipInfo* CurrentShip = PlayerTeam->mGetFirstShip(); CurrentShip != NULL; CurrentShip = PlayerTeam->mGetNextShip())
						{
							if ( CurrentShip->mGetTacticalDistance( MantharTeam->mGetFirstShip() ) < kActivateMantharDistance )
							{
								fMission.mSetTeamRelations (kMantharTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
								fMission.mSetTeamRelations (kPlayerTeam, kMantharTeam, kTeamHostile, kGoalDestroy, 1.0);//***
								break;
							}					
						}

						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					// set Orion ship's comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Manthar ship's comm buttons back to HAIL
					for (Ship = MantharTeam->mGetFirstShip(); Ship != NULL; Ship = MantharTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set BaseTeam comm buttons back to HAIL
					for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			{
				// LNF add the bonus for capturing
				if ( Ship->mGetTeam() == kOrionTeam )
				{
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
					Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
				}
				else if ( Ship->mGetTeam() == kCacheTeam )
				{
					fMission.mAddToBonusPrestige( kCaptureBonus );
					
					CacheRecovered++;

					if ( CacheRecovered == kMaxCache )
					{
						fMission.mDisplayMessage( kAllCachesCaptured_msg, kScienceScriptMessage );

						if (( BaseSecured == 1 ) || ( BaseTeam->mGetShipsInPlayCount == 0 ))
						{
							fMission.mDisplayMessage( kPrimaryMissionComplete_msg, kScienceScriptMessage );
							return kPlayerWins;
						}
						else
						{
							fMission.mDisplayMessage( kFindBase_msg, kScienceScriptMessage );
						}
					}
					else
					{
						fMission.mDisplayMessage( kWeaponsSeized_msg, kScienceScriptMessage );
					}
				}
				else if ( Ship->mGetTeam() == kBaseTeam )
				{
					fMission.mAddToBonusPrestige( kCaptureBonus );
					
					BaseSecured = 1;

					if (( CacheRecovered == kMaxCache ) || ( CacheTeam->mGetShipsInPlayCount() == 0 ))
					{
						fMission.mDisplayMessage( kPrimaryMissionComplete_msg, kScienceScriptMessage );
						return kPlayerWins;
					}
					else
					{
						fMission.mDisplayMessage( kFindCache_msg, kScienceScriptMessage );
					}
				}
				else if ( Ship->mGetTeam() == kDefenseTeam )
				{
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
					Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 23 );
				}
				else if ( Ship->mGetTeam() == kMantharTeam )
				{
					fMission.mAddToBonusPrestige( kCaptureBonus );
					
					MantharCaptured++;
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kMantharTeam)
			{
				MantharDestroyed++;
			}
			else if (Ship->mGetOriginalTeam() == kBaseTeam)
			{
				BaseDestroyed++;
				BaseSecured = 0;
				fMission.mDisplayMessage( kBaseDestroyed_msg, kMissionScriptMessage );

				if (BaseTeam->mGetShipsInPlayCount() == 0)
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetOriginalTeam() == kCacheTeam)
			{
				CacheDestroyed++;
				fMission.mDisplayMessage( kCacheDestroyed_msg, kMissionScriptMessage );

				if (CacheTeam->mGetShipsInPlayCount() == 0)
				{
					return CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetOriginalTeam() == kDefenseTeam)
			{
				DefenseDestroyed++;

				if (DefenseTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				OrionDestroyed++;

				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if ( Ship == PlayerTeam->mGetThisShip( NumberOfPlayersShips + 1 ) )
				{
					CommandoShipDestroyed++;
				}

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			break;

		case kNotifyDisengaged:
			if ( Ship->mGetTeam() == kOrionTeam )
			{
				OrionDisengaged++;
				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kBaseTeam:
								fMission.mDisplayMessage( kMantharBase_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMantharTeam:
								fMission.mDisplayMessage( kRenegadeHailOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionTeam:
								// LNF can I do some kind of check to see if the Orion ship is disengaging?
								if ( Ship->mGetDamagePercent() >= kOrionRetreatDamagePercent )
								{
									fMission.mDisplayMessage( kOrionHailRetreating_msg, kMissionScriptMessage );
								}
								else 
								{
									fMission.mDisplayMessage( kOrionHailOK_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kBaseTeam:
								fMission.mDisplayMessage( kMantharBaseSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMantharTeam:
								if ( Ship->mGetDamagePercent() < kMantharSurrenderDamagePercent )
								{
									fMission.mDisplayMessage( kRenegadeHailDamaged_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kRenegadeSurrender_msg, kMissionScriptMessage );
									break;
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionTeam:
								if ( Ship->mGetDamagePercent() < kOrionSurrenderDamagePercent )
								{
									fMission.mDisplayMessage( kOrionSurrenderOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kOrionSurrenderDamaged_msg, kMissionScriptMessage );

									// LNF making the ship disengage opens a whole bunch of other things
									// LNF have to make a state if the ship gets fired upon after it surrenders
									// LNF maybe I'll cheese it and switch it to an ally?
									fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
									break;
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kBaseTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kMantharBaseTaunt_1_msg, kMantharBaseTaunt_4_msg), kMissionScriptMessage);

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMantharTeam:
								if ( Ship->mGetDamagePercent() < kMantharSurrenderDamagePercent )
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kRenegadeTaunt_1_msg, kRenegadeTaunt_4_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage( kRenegadeSurrender_msg, kMissionScriptMessage );
									break;
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionTeam:
								if ( Ship->mGetDamagePercent() < kOrionSurrenderDamagePercent )
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage( kOrionSurrenderDamaged_msg, kMissionScriptMessage );

									// LNF making the ship disengage opens a whole bunch of other things
									// LNF have to make a state if the ship gets fired upon after it surrenders
									// LNF maybe I'll cheese it and switch it to an ally?
									fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
									break;
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = 0 - (CommandoShipDestroyed * 20);

	// LNF need points for manthar captured?  adding bonus things and such..maybe I'll reduce
	// LNF total victory points for the mission

	if ( Elite == kKingsGuard )
	{
		VictoryPoints += (OrionDestroyed * 10) + (CacheRecovered * 20) + (MantharDestroyed * 20)
						 + (BaseSecured * 30);
	}
	else if ( Elite == kEggDefender )
	{
		VictoryPoints += (OrionDestroyed * 20) + (CacheRecovered * 20) + (MantharDestroyed * 10)
						 + (BaseSecured * 30);
	}
	else // Elite == kErrant
	{
		VictoryPoints += (OrionDestroyed * 10) + (CacheRecovered * 15) + (MantharDestroyed * 20)
						 + (BaseSecured * 40);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 20))
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kKingsGuard)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kKingsGuardAstoundingVictory_msg);
		}
		else if (fMission.mGetEliteOrganization() == kEggDefender)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kEggDefenderAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kErrantAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( BaseSecured != 0 ) ? fMission.mAddToDebriefingList( kBaseSecuredTrue_msg ) : SuccessCount++;
	( CacheRecovered == kMaxCache ) ? fMission.mAddToDebriefingList( kCacheRecoveredTrue_msg ) : SuccessCount++;
	( OrionDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOrionDestroyedTrue_msg ) : SuccessCount++;

	if ( MantharDestroyed != 0 )
	{
		fMission.mAddToDebriefingList( kMantharDestroyedTrue_msg );
	}
	else if ( MantharCaptured != 0 )
	{
		fMission.mAddToDebriefingList( kMantharCapturedTrue_msg );
	}
	else
	{
		SuccessCount++;
	}

	if ( SuccessCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( BaseSecured == 0 ) ? fMission.mAddToDebriefingList( kBaseSecuredFalse_msg ) : FailureCount++;
	( CacheRecovered != kMaxCache ) ? fMission.mAddToDebriefingList( kCacheRecoveredFalse_msg ) : FailureCount++;
	( OrionDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOrionDestroyedFalse_msg ) : FailureCount++;

	if(( MantharDestroyed == 0 ) && ( MantharCaptured == 0 ))
	{
		if ( MantharDestroyed == 0)
		{
			fMission.mAddToDebriefingList( kMantharDestroyedFalse_msg );
		}
		else if ( MantharCaptured == 0 )
		{
			fMission.mAddToDebriefingList( kMantharCapturedFalse_msg );
		}
	}
	else
	{
		FailureCount++;
	}

	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// check for the win conditions
eGameStatus CheckWinCondition( tScript& tScript )
{
	tTeamInfo* MantharTeam = tScript.fMission.mGetTeamHandle (kMantharTeam);
	tTeamInfo* CacheTeam = tScript.fMission.mGetTeamHandle (kCacheTeam);
	tTeamInfo* BaseTeam = tScript.fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* DefenseTeam = tScript.fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);

	if ((( BaseSecured == 1 ) || ( BaseTeam->mGetShipsInPlayCount() == 0 ))
		&& ( CacheTeam->mGetShipsInPlayCount() == 0 ))
	{
		return kPlayerWins;
	}
	else if ((( CacheRecovered == kMaxCache ) || ( CacheTeam->mGetShipsInPlayCount() == 0 ))
			&& ( BaseTeam->mGetShipsInPlayCount() == 0 ))
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}

//---------------------------------------------------------------------------------------------
// return quadrant specific coordinates relative to the player
/*
int32 NewXCoord(tScript& tScript, int32 DirectionX, int32 DirectionY, int32 PlacementDirection)
{
	int32 PlayerX;
	int32 PlayerY;

	tScript.fPlayerShip->mGetGridPosition (&PlayerX, &PlayerY);

	int32 DeltaX = PlayerX - DirectionX;
	int32 DeltaY = PlayerY - DirectionY;

	int32 OffsetX;
	switch ( PlacementDirection )
	{
		case kFore:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kAft:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kPort:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kStarboard:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;

		case kCatty:
			if (( DeltaX > 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY > 0 ))
			{
				return( PlayerX - OffsetX );
			}
			else if (( DeltaX < 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else if (( DeltaX > 0 ) && ( DeltaY < 0 ))
			{
				return( PlayerX + OffsetX );
			}
			else
			{
				// LNF check the 0 condition
				return( OffsetX );
			}
			break;
	}
}
*/
