#include "scriptinterface.h"
#include "findClosestMapEdge.h"

#include "Gor_4MissionText.h"
#include "Gor_4MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 Mode = 0;
int32 PlayerDestroyed = 0;
int32 FreighterDestroyed = 0;
int32 OrionDestroyed = 0;
int32 DoomedDestroyed = 0;
int32 DoomedShipStatus = 0;
int32 FreighterTractored = 0;
int32 EggsDelivered = 0; // win condition
int32 NumberOfOrions = 2; // total number of Orion ships

bool FreighterCapturedOnce = false;
bool FreighterCaptured = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool FirstDoomedShipExplodes = false;


const float kDamageThreshold = 60.0f;
const int32 kPlanetX = 5;
const int32 kPlanetY = 26;
const float kDistanceToPlanet = 20.0f;
const float kDistanceFromPlanet = 21.0f;
const int32 kMapEdgeX = 30;
const int32 kMapEdgeY = 30;

void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam);

enum eDoomedShipStatus
{
	kAOK = 0,
	kDamagedFirst,
	kDamagedBoth,
	kDamagedBothWaitForFirst,
	kDestroyedFirst,
	kDestroyedFirstWaitForSecond,
	kDestroyedBoth
};

enum eMissionMode
{
	kStart = 0,
	kOrionArrive
};

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kDoomedTeam, 
	kFreighterTeam,
	kOrionTeam,
	kPlanetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kDoomedTeam, kGorn );
	fMission.mSetTeamRace( kFreighterTeam, kGorn );
	fMission.mSetTeamRace( kOrionTeam, kOrion );
	fMission.mSetTeamRace( kPlanetTeam, kGorn );

	// Set team relations to show up on Tactical map
	// this mission starts without any enemy ships on screen
	fMission.mSetTeamRelations (kPlayerTeam, kDoomedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDoomedTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDoomedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFreighterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
	// LNF maybe change the location of the player's starting position based on faction?
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* DoomedTeam = fMission.mGetTeamHandle( kDoomedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle( kOrionTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;

		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;

		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == 0 )
	{
		// create Doomed team
		DoomedTeam->mCreateShip( "G-BB", kStartPosition_K );
		DoomedTeam->mCreateShip( "G-BB", kStartPosition_L );

		// create Freighter team
		FreighterTeam->mCreateShip( "G-FSPL", kStartPosition_J );

		// create Planet
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPlanetX, kPlanetY );
	}
	else if ( Mode == kOrionArrive )
	{
		// create Orion team
		// LNF change the ship type and location
		fMission.mSetTeamRelations (kOrionTeam, kFreighterTeam, kTeamAtWar, kGoalCapture, 1.0);

		// LNF mission worked okay before I added this line.  The Orions would ignore the player
		// LNF ships unless they were fired on, so this line should make them shoot at the player
		// LNF but the Orion's primary target should be the freighter
		fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);

		OrionTeam->mCreateShip( "O-DW1", kStartPosition_Specified, 5, 5 );
		OrionTeam->mCreateShip( "O-DW2", kStartPosition_Specified, 5, 6 );

		NumberOfOrions = 2;
		// NumberOfOrions is set at the top
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// there is only one briefing for this mission
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* DoomedTeam = fMission.mGetTeamHandle( kDoomedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle( kOrionTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyBeginTractor);
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
		Ship->mTrackTacticalDistance (kDistanceToPlanet, kPlanetX, kPlanetY, kTrue);
	}

	// configure Doomed team
	for ( Ship = DoomedTeam->mGetFirstShip(); Ship != NULL; Ship = DoomedTeam->mGetNextShip() )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		//Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetFinalDestination();
	}
	DoomedTeam->mGetThisShip( 1 )->mChangeName( kBBMasticator_name );
	DoomedTeam->mGetThisShip( 2 )->mChangeName( kBBLockedFang_name );

	// configure Freighter team
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );
		Ship->mFollowEntity( DoomedTeam->mGetFirstShip(), kVeryLowPriority );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
		Ship->mSetShipCallbackEvents (kNotifyBeginTractor);
	}

	// configure Planet team
	PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer for the destruction of the doomed ships
	fMission.mSetTimedInterval (kTimerTurnsSet, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// configure Orion ships
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents( kNotifyDamage );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* DoomedTeam = fMission.mGetTeamHandle( kDoomedTeam );
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle( kFreighterTeam );
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle( kOrionTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );

	switch (Event)
	{
		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransporterTarget = fMission.mFindShipID( fEventID );

				if ( TransporterTarget != NULL )
				{
					if ( fEventItem1 == kTransGornEgg )
					{
						// LNF how do I handle transport of egg?
						// LNF obviously the player is supposed to want to transport the egg
						// LNF because the transporter is damaged at the start
						// LNF but should they be able to, and if they transport the eggs from
						// LNF the freighter, should they have to save the freighter?
						// LNF and if they do, what is the reason that they need to save the freighter?

						fMission.mDisplayMessage( kGornEggsCantBeTransported_msg, kScienceScriptMessage );

						TransporterTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );
						Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
					}

					// LNF I should only need one of these
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyBeginTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
					if ( Ship->mGetTeam() == kPlayerTeam )
					{
						if ( TractorTarget->mGetOriginalTeam() == kFreighterTeam )
						{
							FreighterTractored = 1;
						}
					}
				}
				Ship->mSetShipCallbackEvents (kNotifyEndTractor);
			}
			break;

		case kNotifyEndTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
					if ( Ship->mGetTeam() == kPlayerTeam )
					{
						if ( TractorTarget->mGetOriginalTeam() == kFreighterTeam )
						{
							FreighterTractored = 0;
						}
					}
				}
				Ship->mSetShipCallbackEvents (kNotifyBeginTractor);
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kDoomedTeam )
						{
							FreighterTeam->mClearCommands();
							fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
					}
				}
			}
			break;

		case kNotifyAtDistance:
			// LNF check player's distance to planet if he is tractoring the freighter
			if ( Ship->mGetTrackingDistance() == kDistanceToPlanet )
			{
				if ( FreighterTractored == 1 )
				{
					if (( FreighterTeam->mGetFirstShip()->mGetTeam() == kPlayerTeam )
						|| ( FreighterTeam->mGetFirstShip()->mGetTeam() == kFreighterTeam ))
					{
						// win condition
						EggsDelivered = 1;
						FreighterTeam->mGetFirstShip()->mSetShipAIControlled( kFalse );
						FreighterTeam->mGetFirstShip()->mSetShipMaxSpeed( 0 );

						if (OrionTeam->mGetShipsInPlayCount() == 0)
						{
							fMission.mDisplayMessage( kPrimaryMissionComplete_msg, kScienceScriptMessage );
							FindClosestMapEdge( *this, fPlayerShip, kMapEdgeX, kMapEdgeY);
						}
						else
						{
							fMission.mDisplayMessage( kStopOrions_msg, kScienceScriptMessage );
						}
					}
				}
				else
				{
					//fMission.mDisplayMessage( kLNFFreighterCaptured_msg );
				}
				Ship->mTrackTacticalDistance (kDistanceFromPlanet, kPlanetX, kPlanetY, kTrue);
			}
			else if ( Ship->mGetTrackingDistance() == kDistanceFromPlanet )
			{
				//fMission.mDisplayMessage( kImproperlyExitedMission_msg );
				Ship->mTrackTacticalDistance (kDistanceToPlanet, kPlanetX, kPlanetY, kTrue);
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kDoomedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDoomedTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					break;

				case kTimerLimitSet_2: // doomed ships explode
					if ( DoomedShipStatus == kDamagedBothWaitForFirst )
					{
						DoomedTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text, kTrue );
						DoomedShipStatus = kDamagedBoth;
						fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
					}
					else if ( DoomedShipStatus == kDamagedBoth )
					{
						fMission.mDisplayMessage( kLNFBlowUp_msg, kMissionScriptMessage );
						DoomedTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100 );
						DoomedTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPoints, 1023 );
						DoomedShipStatus = kDestroyedFirstWaitForSecond;
						fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
					}
					else if ( DoomedShipStatus == kDestroyedFirstWaitForSecond )
					{
						DoomedTeam->mGetThisShip(2)->mSetCommButton( 1, kButtonHail_text, kTrue );
						fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
						DoomedShipStatus = kDestroyedFirst;
					}
					else if ( DoomedShipStatus == kDestroyedFirst )
					{
						DoomedTeam->mGetThisShip(2)->mDamageSystem( kDamageExcessDamageSystem, 100 );
						DoomedTeam->mGetThisShip(2)->mDamageSystem( kDamageInternalsWithPoints, 1023 );

						fMission.mDisplayMessage( kLNFBlowUp_msg, kMissionScriptMessage );

						for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
						{
							Ship->mDamageSystem( kDamageInternalsWithPoints, 15 );
							Ship->mDamageSystem( kDamageTractorSystem, 100 );
							Ship->mDamageSystem( kDamageTransporterSystem, 100 );
							Ship->mDamageSystem( kDamageTractorGroupSystem, 100 );
						}

						FreighterTeam->mGetFirstShip()->mSetFinalDestination( 5, 5 );
						// LNF reset the speed to 31 somewhere?
						FreighterTeam->mGetFirstShip()->mSetShipMaxSpeed( 12 );
						// LNF should I turn off the Freighter Helm AI?

						Mode = kOrionArrive;
						mCreateMissionShips( kOrionArrive );

						fMission.mDisplayMessage( kDamageReport_msg, kMissionScriptMessage );
						//fMission.mDisplayMessage( kEvent1_C_msg, kScienceScriptMessage );
						fMission.mSetTimedInterval( kTimerLimitSet_3, 6 );
					}
					break;

				case kTimerLimitSet_3:
					fMission.mDisplayMessage( kEvent2DamageReport_1_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent2DamageReport_2_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent2DamageReport_3_msg, kMissionScriptMessage );
					break;
					
				case kTimerTurnsSet: // doomed ships take damage
					fMission.mDisplayMessage( kWarning_msg, kScienceScriptMessage );

					DoomedTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text, kTrue );
					// LNF maybe print a message from this ship?

					for (Ship = DoomedTeam->mGetFirstShip(); Ship != NULL; Ship = DoomedTeam->mGetNextShip())
					{
						Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 25 );
						Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 25 );
						Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 25 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
					}

					DoomedTeam->mGetThisShip(2)->mSetCommButton( 1, kButtonHail_text, kTrue );
					// LNF maybe print a message from this ship?

					DoomedShipStatus = kDamagedBothWaitForFirst;
					fMission.mSetTimedInterval (kTimerLimitSet_2, 2);					
					break;

				case kTimerTurnsSet_2:
					break;

				case kTimerTurnsSet_9:
					// set Orion comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Freighter comm buttons back to HAIL
					for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				// LNF need a way to find out what team is doing the capturing

				if ( Ship == FreighterTeam->mGetFirstShip() )
				{
					if ( FreighterCapturedOnce == false )
					{
						FreighterCapturedOnce = true;
						Ship->mClearCommands();
						Ship->mSetShipAIControlled( kFalse );
						Ship->mSetShipThrust( 0 );
					}

					if ( fEventID == kPlayerTeam )
					{
						fMission.mDisplayMessage( kEvent5Retake_msg, kMissionScriptMessage );
						FreighterCaptured = false;
					}
					else if ( fEventID == kOrionTeam )
					{
						fMission.mDisplayMessage( kEvent2Boarding_msg, kMissionScriptMessage );
						FreighterCaptured = true;
					}
				}
				else if ( Ship->mGetTeam() == kOrionTeam )
				{
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
					Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 23 );
				}
			}		
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;

				if (FreighterTeam->mGetShipsInPlayCount() == 0)
				{
					if ( DoomedTeam->mGetShipsInPlayCount() == 0 )
					{
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kDoomedTeam)
			{
				DoomedDestroyed++;

				if (DoomedTeam->mGetShipsInPlayCount() == 0)
				{
					if (FreighterTeam->mGetShipsInPlayCount() == 0)
					{
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				OrionDestroyed++;

				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
					if ( EggsDelivered == 1 )
					{
						fMission.mDisplayMessage( kPrimaryMissionComplete_msg, kScienceScriptMessage );
						FindClosestMapEdge( *this, fPlayerShip, kMapEdgeX, kMapEdgeY);
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								if ( DoomedShipStatus == kAOK )
								{
									fMission.mDisplayMessage( kSteadyAsSheGoes_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kFreighterHail_msg, kMissionScriptMessage );
								}
								
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage( kOrionHailOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPlanetTeam:
								fMission.mDisplayMessage( kHatchHoldHail_msg, kScienceScriptMessage );
								Ship->mSetCommButton (1, kButtonHail_text);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								fMission.mDisplayMessage( kFreighterSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage( kOrionSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								fMission.mDisplayMessage( kFreighterSurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kOrionTaunt_1_msg, kOrionTaunt_3_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = 0 - (FreighterDestroyed * 100);

	if ( Elite == kKingsGuard )
	{
		if ( EggsDelivered )
		{
			VictoryPoints += 90;
		}

		VictoryPoints += (OrionDestroyed * 5);
	}
	else if ( Elite == kEggDefender )
	{
		if ( EggsDelivered )
		{
			VictoryPoints += 100;
		}

		VictoryPoints += (OrionDestroyed * 0);
	}
	else // Elite == kErrant
	{
		if ( EggsDelivered )
		{
			VictoryPoints += 80;
		}

		VictoryPoints += (OrionDestroyed * 10);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 80))
	{
		MyVictoryLevel = kVictory;
	}
/*
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
*/
	else if (VictoryPoints < 80)
	{
		MyVictoryLevel = kDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kKingsGuard)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kKingsGuardAstoundingVictory_msg);
		}
		else if (fMission.mGetEliteOrganization() == kEggDefender)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kEggDefenderAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kErrantAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( EggsDelivered != 0 ) ? fMission.mAddToDebriefingList( kEggsDeliveredTrue_msg ) : SuccessCount++;
	( OrionDestroyed == NumberOfOrions ) ? fMission.mAddToDebriefingList( kOrionDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( EggsDelivered == 0 ) ? fMission.mAddToDebriefingList( kEggsDeliveredFalse_msg ) : FailureCount++;
	( OrionDestroyed != NumberOfOrions ) ? fMission.mAddToDebriefingList( kOrionDestroyedFalse_msg ) : FailureCount++;
	( FreighterDestroyed == 1 ) ? fMission.mAddToDebriefingList( kFreighterDestroyedTrue_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam)
{
	tTeamInfo* DoomedTeam = tScript.fMission.mGetTeamHandle (kDoomedTeam);
	tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
	tShipInfo* Ship;

	tMapPoint waypoints_1[8]=
	{
		30, 20,
		26, 20,
		22, 22,
		20, 27,
		21, 31,
		26, 31,
		30, 27,
		31, 23
	};

	tMapPoint waypoints_2[2]=
	{
		10, 14,
		17, 14
	};

	tMapPoint PatrolPoints[8];

	if ( OrbitTeam == FreighterTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_1[0];
			PatrolPoints[1] = waypoints_1[1];
			PatrolPoints[2] = waypoints_1[2];
			PatrolPoints[3] = waypoints_1[3];
			PatrolPoints[4] = waypoints_1[4];
			PatrolPoints[5] = waypoints_1[5];
			PatrolPoints[6] = waypoints_1[6];
			PatrolPoints[7] = waypoints_1[7];
			Ship->mPatrol( 60.0, 8, &PatrolPoints[0], kHighPriority );
		}
	}
	else if ( OrbitTeam == DoomedTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_2[0];
			PatrolPoints[1] = waypoints_2[1];
			Ship->mPatrol( 50.0, 2, &PatrolPoints[0], kHighPriority );
		}
	}
}
