#include "scriptinterface.h"

#include "Gor_5MissionText.h"
#include "Gor_5MissionMaps.h"
#include "Gor_5FactionScript.h"
#include "Gor_5KingsGuard.h"
#include "Gor_5EggDefender.h"
#include "Gor_5Errant.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 Gorn_1Destroyed = 0;
int32 Gorn_2Destroyed = 0;
int32 OrionDestroyed = 0;
int32 GornDestroyed = 0;
int32 OrionPatrolDestroyed = 0;
int32 OrionCommandDestroyed = 0;
int32 OrionMeetDestroyed = 0;
int32 HolyMother = 0;
int32 HolyMotherRescued = -1; // win condition
int32 HolyMotherKilled = 0; // win condition
int32 ListeningPostActivated = 0; // win condition
int32 MistressTreachery = 0; // win condition
int32 MaxOrions = 0; // win condition
int32 HolyMotherRescuedM = -1; // for debrief messages -1 no message, 0 not rescued, 1 rescued

bool UpgradeOffered = false;
bool EggDetection = false;
bool OrionCommandSurrenders = false;
bool CaptureHolyMother = false;
bool EggFireProbe = false;
bool GornTreachery = false;
bool FoundTheHolyMother = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

const int32 kOrionCaptureMotherTimer = 10; // LNF maybe make this longer than 10?
const float kOrionMeetRadius = 1.0f;
const float kOrionFollowRadius = 30.0f;
const float kCommandThreatRadius = 20.0f;
const float kCommandFollowRadius = 30.0f;
const int32 kPlanetX = 17;
const int32 kPlanetY = 17;
const float kPlayerEggDistance = 80.0f;
const float kOrionCommandDamage = 50.0f;
const float kCommandPlanetDistance = 30.0f;
const int32 kNumberOfListeningPosts = 3; // win condition
const float kPlanetTransporterDistance = 5.0f;
const float kDeepScanDistance = 15.0f;
const float kDeepScanDistancePlanet = 30.0f;
const int32 kDeepScanTurnsPlanet = 2;
struct asteroidStruct
{
	int32 x;
	int32 y;
};
asteroidStruct asteroidLocation[3];

eMissionMode MissionMode = kStart;
void WinCondition( tScript& tScript );

tGornFactionScript* gCurGornFaction = NULL;

void OrionPatrolOrbit(tScript& tScript, tTeamInfo* OrbitTeam);

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
// LNF is it okay to move this to Gor_5FactionScript.h???
/*enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kOrionMeetTeam, 
	kGorn_1Team,
	kGorn_2Team,
	kOrionPatrolTeam,
};
*/
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kOrionMeetTeam, kOrion);
	fMission.mSetTeamRace (kGorn_1Team, kGorn);
	fMission.mSetTeamRace (kGorn_2Team, kGorn);
	fMission.mSetTeamRace (kOrionPatrolTeam, kOrion);
	fMission.mSetTeamRace (kOrionCommandTeam, kOrion);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kOrionMeetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kGorn_1Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kGorn_2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionPatrolTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionCommandTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionMeetTeam, kGorn_1Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionMeetTeam, kGorn_2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionMeetTeam, kOrionPatrolTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionMeetTeam, kOrionCommandTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kGorn_1Team, kGorn_2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_1Team, kOrionPatrolTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_1Team, kOrionCommandTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kGorn_2Team, kOrionPatrolTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_2Team, kOrionCommandTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionPatrolTeam, kOrionCommandTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
		gCurGornFaction = new tKingsGuard( kGorn_1Team, kGorn_2Team, kPlayerTeam );
		return 1; // KingsGuard map
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
		gCurGornFaction = new tEggDefender( kGorn_1Team, kGorn_2Team, kPlayerTeam );
		return 2; // Egg Defender map
	}
	else
	{
		Elite = kErrant;
		gCurGornFaction = new tErrant( kGorn_1Team, kGorn_2Team, kPlayerTeam );
		return 3; // Errant map
	}
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
	// LNF maybe change the location of the player's starting position based on faction?
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionMeetTeam = fMission.mGetTeamHandle (kOrionMeetTeam);
	tTeamInfo* Gorn_1Team = fMission.mGetTeamHandle (kGorn_1Team);
	tTeamInfo* Gorn_2Team = fMission.mGetTeamHandle (kGorn_2Team);
	tTeamInfo* OrionPatrolTeam = fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionCommandTeam = fMission.mGetTeamHandle (kOrionCommandTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( MissionMode == kStart )
	{
	}
	else if ( MissionMode == kInitialShips )
	{
		// LNF should there be only one ship per faction?  
		// or maybe dependent on how many the player has?
		gCurGornFaction->mCreateShips( Gorn_1Team, Gorn_2Team, AveragePlayerBPV, LargestEnemyShipClass );

		// create OrionMeet team
		OrionMeetTeam->mCreateFleet( 2, (2 * AveragePlayerBPV), kClassDestroyer, kClassDestroyer, 0, kStartPosition_X );
		MaxOrions += 2;

		// create OrionPatrol team
		OrionPatrolTeam->mCreateFleet( 2, (2 * AveragePlayerBPV), kClassDestroyer, kClassDestroyer, 0, kStartPosition_U );
		MaxOrions += 2;

		// create OrionCommand team
		OrionCommandTeam->mCreateShip( "O-CA3", kStartPosition_Z );
		MaxOrions += 1;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return gCurGornFaction->mInitializeFinished();
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionMeetTeam = fMission.mGetTeamHandle (kOrionMeetTeam);
	tTeamInfo* Gorn_1Team = fMission.mGetTeamHandle (kGorn_1Team);
	tTeamInfo* Gorn_2Team = fMission.mGetTeamHandle (kGorn_2Team);
	tTeamInfo* OrionPatrolTeam = fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionCommandTeam = fMission.mGetTeamHandle (kOrionCommandTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{		
		//Ship->mAccessSupplies( kSetTransporterItemCount, kTransWeaponSchematic, 1 );
		if ( Elite == kKingsGuard )
		{
			Ship->mTrackTacticalDistance( kPlayerEggDistance, kPlanetX, kPlanetY );
		}
	}

	// set up asteroid coordinates
	asteroidLocation[0].x = 11;
	asteroidLocation[0].y = 17;
	asteroidLocation[1].x = 13;
	asteroidLocation[1].y = 13;
	asteroidLocation[2].x = 17;
	asteroidLocation[2].y = 11;

	if ( Elite == kKingsGuard )
	{
		fMission.mDisplayMessage (kStartScenarioKing_msg, kMissionScriptMessage);
	}
	else if ( Elite == kEggDefender )
	{
		fMission.mDisplayMessage (kStartScenarioDefender_msg, kMissionScriptMessage);
	}
	else
	{
		fMission.mDisplayMessage (kStartScenarioErrant_msg, kMissionScriptMessage);
	}


	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// LNF timer to countdown when the Orions will kidnap the Holy Mother
	fMission.mSetTimedInterval (kTimerTurnsSet, kOrionCaptureMotherTimer);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn
	
	if ( Elite == kErrant )
	{
		DeepScanTracker.mSetLocation (asteroidLocation[0].x, asteroidLocation[0].y, kDeepScanDistance, 1);
		DeepScanTracker.mSetLocation (asteroidLocation[1].x, asteroidLocation[1].y, kDeepScanDistance, 1);
		DeepScanTracker.mSetLocation (asteroidLocation[2].x, asteroidLocation[2].y, kDeepScanDistance, 1);
	}
	else if ( Elite == kKingsGuard )
	{
		DeepScanTracker.mSetLocation (kPlanetX, kPlanetY, kDeepScanDistancePlanet, kDeepScanTurnsPlanet);
		//DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurnsPlanet);
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if ( Ship->mGetTeam() == kGorn_1Team )
	{
		gCurGornFaction->mSetUpShips( *this, Ship );
	}
	else if ( Ship->mGetTeam() == kGorn_2Team )
	{
		gCurGornFaction->mSetUpShips( *this, Ship );
	}
	else if ( Ship->mGetTeam() == kOrionMeetTeam )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}
	else if ( Ship->mGetTeam() == kOrionPatrolTeam )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}
	else if ( Ship->mGetTeam() == kOrionCommandTeam )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mChangeName( kOrionCommand_name );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionMeetTeam = fMission.mGetTeamHandle (kOrionMeetTeam);
	tTeamInfo* Gorn_1Team = fMission.mGetTeamHandle (kGorn_1Team);
	tTeamInfo* Gorn_2Team = fMission.mGetTeamHandle (kGorn_2Team);
	tTeamInfo* OrionPatrolTeam = fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionCommandTeam = fMission.mGetTeamHandle (kOrionCommandTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			// LNF when the KingsGuard attack the Orion, the Egg Defender should come to the
			// Orion's aid

			// LNF how will the Errant be caught planting listening devices?
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if (( Ship->mGetTeam() == kGorn_1Team )
							&& ( gCurGornFaction->fAtWarWith[ kAWGorn_1Team ] == false ))
						{
							if ( HostileShip->mGetShipTargetID() == Ship->mGetShipID() )
							{
								gCurGornFaction->fAtWarWith[ kAWGorn_1Team ] = true;
							}
						}
						else if (( Ship->mGetTeam() == kGorn_2Team )
							&& ( gCurGornFaction->fAtWarWith[ kAWGorn_2Team ] == false ))
						{
							if ( HostileShip->mGetShipTargetID() == Ship->mGetShipID() )
							{
								gCurGornFaction->fAtWarWith[ kAWGorn_2Team ] = true;
							}
						}
						else if ( Ship->mGetTeam() == kOrionCommandTeam )
						{
							if (( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								&& ( GornTreachery == false ))
							{
								GornTreachery = true;
								fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
							}
							else if( ( Ship->mGetDamagePercent() >= kOrionCommandDamage )
								&& ( OrionCommandSurrenders == false )
								&& ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand) )
							{
								// LNF maybe this should be any Gorn faction and not
								// LNF just the player's
								OrionCommandSurrenders = true;

								fMission.mDisplayMessage( kOrionCommandSurrendersPlayer_msg, kMissionScriptMessage );
								fMission.mDisplayMessage( kOrionReturnsMother_msg, kMissionScriptMessage );

								OrionCommandTeam->mClearCommands();
								OrionCommandTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );

								gCurGornFaction->fHolyMotherStatus = kOnKingsGuard;
								HolyMotherRescued = 1;
								HolyMotherRescuedM = 1;

								//LNF OrionMeetTeam->mClearCommands();
								//LNF OrionMeetTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
							}
						}

						if ( Elite == kEggDefender )
						{
							if ((( Ship->mGetTeam() == kOrionCommandTeam )
								|| ( Ship->mGetTeam() == kOrionMeetTeam )
								|| ( Ship->mGetTeam() == kOrionPatrolTeam ))
								&& (HolyMotherRescued == 0 ))
							{
								MistressTreachery = 1;
							}
						}
					}
					else
					{
						if (( Ship->mGetDamagePercent() >= kOrionCommandDamage )
							&& ( OrionCommandSurrenders == false ))
						{
							OrionCommandSurrenders = true;

							fMission.mDisplayMessage( kOrionCommandSurrenders_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kOrionReturnsMother_msg, kMissionScriptMessage );

							OrionCommandTeam->mClearCommands();
							OrionCommandTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );

							gCurGornFaction->fHolyMotherStatus = kOnKingsGuard;
							HolyMotherRescued = 1;
							HolyMotherRescuedM = 1;
						}
					}
				}

				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyAtDistance:
			if (( Ship->mGetTeam() == kPlayerTeam ) && ( EggDetection == false ))
			{
				EggDetection = true;

				// the player has gotten close to the planet and has been warned that the 
				// Egg Defenders know of their presence
				fMission.mDisplayMessage( kHereTheyCome_msg, kMissionScriptMessage );
				gCurGornFaction->mEggDetection( *this );
			}
			else if (( Elite == kKingsGuard ) && ( Ship == fPlayerShip )
					&& ( gCurGornFaction->fHolyMotherStatus == kOnPlanet ))
			{
				gCurGornFaction->fHolyMotherStatus = kOnKingsGuard;

				HolyMotherRescued = 1; // LNF do I need this?
				HolyMotherRescuedM = 1;
				fMission.mDisplayMessage( kHolyMotherCaptured_msg );

				fMission.mSetTimedInterval( kTimerTurnsSet, 0 );

				// LNF What should the Orions do since the player 'rescued' the Holy Mother??
				// LNF have to make the Errant attack the Orion? and the egg defenders will
				// LNF still attack the player, I guess
			}
			break;

		case kNotifyAtDestination:
			if ( Ship->mGetTeam() == kOrionCommandTeam )
			{
				OrionCommandTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
				OrionMeetTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					gCurGornFaction->mSetAllTeamRelations( fMission );

					MissionMode = kInitialShips;
					mCreateMissionShips( kInitialShips );

					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					break;

				case kTimerLimitSet_2:
					gCurGornFaction->mErrantPatrol( *this );
					gCurGornFaction->mEggDefenderPatrol( *this );
					gCurGornFaction->mKingsGuardPatrol( *this );
					OrionPatrolOrbit( *this, OrionPatrolTeam);

					OrionMeetTeam->mDefendEntity( OrionCommandTeam->mGetFirstShip(), 15.0, 15.0, kVeryHighPriority );
					break;
					
				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;

							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;

							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on

								if ( Elite == kErrant )
								{
									fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								}
								else if ( Elite == kKingsGuard )
								{
									fMission.mDisplayMessage (kScannerOffInRangePlanet_msg, kScienceScriptMessage);
								}
								break;

							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								if ( Elite == kErrant )
								{
									fMission.mDisplayMessage (kStartingScanAsteroid_msg, kScienceScriptMessage);
								}
								else if ( Elite == kKingsGuard )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								break;

							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off

								if ( Elite == kErrant )
								{
									fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								}
								else if ( Elite == kKingsGuard )
								{
									fMission.mDisplayMessage (kTurnedOffScanInRangePlanet_msg, kScienceScriptMessage);
								}
								break;

							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								if ( Elite == kErrant )
								{
									fMission.mDisplayMessage (kStartingScanAsteroid_msg, kScienceScriptMessage);
								}
								else if ( Elite == kKingsGuard )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								break;

							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off

								if ( Elite == kErrant )
								{
									fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								}
								else if ( Elite == kKingsGuard )
								{
									fMission.mDisplayMessage (kScanOnMoveOutOfRangePlanet_msg, kScienceScriptMessage);
								}
								break;

							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object or location

							if ( Elite == kErrant )
							{
								fMission.mDisplayMessage( kListeningPostActivated_msg, kScienceScriptMessage );
								ListeningPostActivated++;

								if ( EggFireProbe == false )
								{
									EggFireProbe = true;
									Gorn_1Team->mGetFirstShip()->mFireProbe( fPlayerShip, kVeryHighPriority );

									fMission.mDisplayMessage( kErrantDetected_msg, kMissionScriptMessage );
								}


								if ( ListeningPostActivated == kNumberOfListeningPosts )
								{
									fMission.mDisplayMessage( kAllListeningPostActivated_msg, kScienceScriptMessage );
								}
							}
							else if ( Elite == kKingsGuard )
							{
								if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								{
									if ( FoundTheHolyMother == false )
									{
										FoundTheHolyMother = true;
										fMission.mDisplayMessage( kFoundDaHolyMother_msg, kScienceScriptMessage );

										// reset the player's tactical tracking to track the distance to transport
										// the Holy Mother from the planet
										for (tShipInfo* tempShip = PlayerTeam->mGetFirstShip(); tempShip != NULL; tempShip = PlayerTeam->mGetNextShip())
										{		
											tempShip->mTrackTacticalDistance( kPlanetTransporterDistance, kPlanetX, kPlanetY );
										}
									}
								}
							}


							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					{
						// Orions capture Holy Mother
						if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
						{
							gCurGornFaction->fHolyMotherStatus = kOnOrionCommand;

							HolyMotherRescued = 0;
							HolyMotherRescuedM = 0;
							fMission.mDisplayMessage( kEvent2Treachery_msg, kMissionScriptMessage );

							fMission.mSetTeamRelations( kOrionMeetTeam, kOrionPatrolTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
							fMission.mSetTeamRelations( kOrionMeetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
							fMission.mSetTeamRelations( kOrionMeetTeam, kGorn_1Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
							fMission.mSetTeamRelations( kOrionMeetTeam, kGorn_2Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

							fMission.mSetTeamRelations( kOrionCommandTeam, kGorn_1Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
							fMission.mSetTeamRelations( kOrionCommandTeam, kGorn_2Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

							// Orion command ship move to edge of map, then disengage
							OrionCommandTeam->mGetFirstShip()->mSetShipMaxSpeed( 23 );
							OrionCommandTeam->mGetFirstShip()->mSetFinalDestination( 24, 4 );
							OrionMeetTeam->mDefendEntity( OrionCommandTeam->mGetFirstShip(), kCommandThreatRadius, kCommandFollowRadius, kVeryHighPriority );

							for (tShipInfo* tempShip = Gorn_1Team->mGetFirstShip(); tempShip != NULL; tempShip = Gorn_1Team->mGetNextShip())
							{
								tempShip->mSetShipMaxSpeed( 31 );
							}

							for (tempShip = Gorn_2Team->mGetFirstShip(); tempShip != NULL; tempShip = Gorn_2Team->mGetNextShip())
							{
								tempShip->mSetShipMaxSpeed( 31 );
							}

							gCurGornFaction->mCaptureHolyMother( *this, Gorn_1Team, Gorn_2Team );
						}
						else
						{
							OrionCommandTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
							fMission.mSetTeamRelations (kOrionMeetTeam, kOrionPatrolTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
							fMission.mSetTeamRelations (kOrionMeetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kOrionMeetTeam, kGorn_1Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kOrionMeetTeam, kGorn_2Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}

						// OrionPatrolTeam attacks all factions
						OrionPatrolTeam->mClearCommands();
						fMission.mSetTeamRelations (kOrionPatrolTeam, kOrionMeetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kOrionPatrolTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kOrionPatrolTeam, kGorn_1Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kOrionPatrolTeam, kGorn_2Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					}
					break;

				case kTimerTurnsSet_2:
					break;

				case kTimerTurnsSet_5:
					// set OrionCommand comm buttons back to HAIL
					for (Ship = OrionCommandTeam->mGetFirstShip(); Ship != NULL; Ship = OrionCommandTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_6:
					// set Gorn_1 comm buttons back to HAIL
					for (Ship = Gorn_1Team->mGetFirstShip(); Ship != NULL; Ship = Gorn_1Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_7:
					// set Gorn_2 comm buttons back to HAIL
					for (Ship = Gorn_2Team->mGetFirstShip(); Ship != NULL; Ship = Gorn_2Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8:
					// set OrionMeet comm buttons back to HAIL
					for (Ship = OrionMeetTeam->mGetFirstShip(); Ship != NULL; Ship = OrionMeetTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set OrionPatrol Team comm buttons back to HAIL
					for (Ship = OrionPatrolTeam->mGetFirstShip(); Ship != NULL; Ship = OrionPatrolTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (( Ship->mGetTeam() == kOrionCommandTeam )
				|| ( Ship->mGetTeam() == kOrionMeetTeam )
				|| ( Ship->mGetTeam() == kOrionPatrolTeam ))
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 23 );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kGorn_1Team)
			{
				Gorn_1Destroyed++;
				GornDestroyed++;

				if (Gorn_1Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kGorn_2Team)
			{
				Gorn_2Destroyed++;
				GornDestroyed++;

				if (Gorn_2Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionMeetTeam)
			{
				OrionMeetDestroyed++;
				OrionDestroyed++;

				if (OrionMeetTeam->mGetShipsInPlayCount() == 0)
				{
					WinCondition( *this );
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionPatrolTeam)
			{
				OrionPatrolDestroyed++;
				OrionDestroyed++;

				if (OrionPatrolTeam->mGetShipsInPlayCount() == 0)
				{
					WinCondition( *this );
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionCommandTeam)
			{
				OrionCommandDestroyed++;
				OrionDestroyed++;

				if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
				{
					gCurGornFaction->fHolyMotherStatus = kDead;
					HolyMotherKilled = 1;
					HolyMotherRescued = 0;
					HolyMotherRescuedM = -1;

					fMission.mDisplayMessage( kOrionCommandDestroyed_1_msg, kScienceScriptMessage );
				}

				if (OrionCommandTeam->mGetShipsInPlayCount() == 0)
				{
					WinCondition( *this );
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			if ( Ship->mGetTeam() == kOrionCommandTeam )
			{
				if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
				{
					// LNF adjust win conditions and this message per faction
					fMission.mDisplayMessage( kEvent15OrionsEscape_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kGorn_1Team:
								gCurGornFaction->mHailMessage( *this, Gorn_1Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
								break;

							case kGorn_2Team:
								gCurGornFaction->mHailMessage( *this, Gorn_2Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;

							case kOrionMeetTeam:
								if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								{
									fMission.mDisplayMessage( kOrionMeetHailNoTreachery_msg, kMissionScriptMessage );
								}
								else if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
								{
									fMission.mDisplayMessage( kOrionMeetHail_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kOrionPatrolTeam:
								if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								{
									fMission.mDisplayMessage( kOrionHailNoTreachery_msg, kMissionScriptMessage );
								}
								else if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
								{
									fMission.mDisplayMessage( kOrionHailTreachery_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionCommandTeam:
								if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								{
									fMission.mDisplayMessage( kOrionMeetHailNoTreachery_msg, kMissionScriptMessage );
								}
								else if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
								{
									fMission.mDisplayMessage( kOrionMeetHail_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_5, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kGorn_1Team:
								gCurGornFaction->mSurrenderMessage( *this, Gorn_1Team );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
								break;

							case kGorn_2Team:
								gCurGornFaction->mSurrenderMessage( *this, Gorn_2Team );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;

							case kOrionMeetTeam:
								if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								{
									fMission.mDisplayMessage( kOrionHailNoTreachery_msg, kMissionScriptMessage );
								}
								else if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
								{
									fMission.mDisplayMessage( kOrionMeetSurrender_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kOrionPatrolTeam:
								fMission.mDisplayMessage( kOrionSurrenderOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionCommandTeam:
								if ( gCurGornFaction->fHolyMotherStatus == kOnPlanet )
								{
									fMission.mDisplayMessage( kOrionHailNoTreachery_msg, kMissionScriptMessage );
								}
								else if ( gCurGornFaction->fHolyMotherStatus == kOnOrionCommand )
								{
									fMission.mDisplayMessage( kOrionMeetSurrender_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_5, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kGorn_1Team:
								gCurGornFaction->mTauntMessage( *this, Gorn_1Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
								break;

							case kGorn_2Team:
								gCurGornFaction->mTauntMessage( *this, Gorn_2Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;

							case kOrionMeetTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kOrionPatrolTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kOrionCommandTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_5, 2);
								break;
						}
						break;				
				}
			}
			break;
	}

	//return fMission.mGetGameStatus();
	//return kGameInProgress;

	// LNF need to check if a return is made somewhere else
	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = GornDestroyed * (-10);

	if ( Elite == kKingsGuard )
	{
		VictoryPoints += (OrionDestroyed * 12) + (HolyMotherRescued * 28) - (HolyMotherKilled * 30);
	}
	else if ( Elite == kEggDefender )
	{
		VictoryPoints += (OrionDestroyed * 10) + (HolyMotherRescued * 40) - (HolyMotherKilled * 100)
						- (MistressTreachery * 100);
	}
	else // Elite == kErrant
	{
		VictoryPoints += (OrionDestroyed * 10) + (HolyMotherRescued * 10) - (HolyMotherKilled * 20)
						+ (ListeningPostActivated * 10);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 60))
	{
		MyVictoryLevel = kVictory;
	}
/*
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
*/
	else if (VictoryPoints <= 59)
	{
		MyVictoryLevel = kDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kKingsGuard)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kKingsGuardAstoundingVictory_msg);
		}
		else if (fMission.mGetEliteOrganization() == kEggDefender)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kEggDefenderAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kErrantAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( OrionDestroyed == MaxOrions ) ? fMission.mAddToDebriefingList( kOrionDestroyedTrue_msg ) : SuccessCount++;
	( HolyMotherRescuedM == 1 ) ? fMission.mAddToDebriefingList( kHolyMotherRescuedTrue_msg ) : SuccessCount++;
	( HolyMotherKilled == 0 ) ? SuccessCount++ : SuccessCount++;
	( GornDestroyed == 0 ) ? SuccessCount++ : SuccessCount++;

	if ( Elite == kEggDefender )
	{
		( MistressTreachery == 0 ) ? fMission.mAddToDebriefingList( kMistressTreacheryTrue_msg ) : SuccessCount++;

		if ( SuccessCount == 5 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else if ( Elite == kErrant )
	{
		( ListeningPostActivated == kNumberOfListeningPosts ) ? fMission.mAddToDebriefingList( kListeningPostActivatedTrue_msg ) : SuccessCount++;

		if ( SuccessCount == 5 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else
	{
		if ( SuccessCount == 4 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( OrionDestroyed != MaxOrions ) ? fMission.mAddToDebriefingList( kOrionDestroyedFalse_msg ) : FailureCount++;
	( HolyMotherRescuedM <= 0 ) ? fMission.mAddToDebriefingList( kHolyMotherRescuedFalse_msg ) : FailureCount++;
	( HolyMotherKilled != 0 ) ? fMission.mAddToDebriefingList( kHolyMotherKilledFalse_msg ) : FailureCount++;
	( GornDestroyed != 0 ) ? fMission.mAddToDebriefingList( kGornDestroyedFalse_msg ) : FailureCount++;

	if ( Elite == kEggDefender )
	{
		( MistressTreachery != 0 ) ? fMission.mAddToDebriefingList( kMistressTreacheryFalse_msg ) : FailureCount++;

		if ( FailureCount == 5 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else if ( Elite == kErrant )
	{
		( ListeningPostActivated < kNumberOfListeningPosts ) ? fMission.mAddToDebriefingList( kListeningPostActivatedFalse_msg ) : FailureCount++;

		if ( FailureCount == 5 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else
	{
		if ( FailureCount == 4 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}

	if( gCurGornFaction )
	{
		delete gCurGornFaction;
		gCurGornFaction = NULL;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrionPatrolOrbit(tScript& tScript, tTeamInfo* OrbitTeam)
{
	tTeamInfo* OrionPatrolTeam = tScript.fMission.mGetTeamHandle (kOrionMeetTeam);
	tTeamInfo* Gorn_1Team = tScript.fMission.mGetTeamHandle (kGorn_1Team);
	tShipInfo* Ship;

	tMapPoint waypoints_1[8]=
	{
		30, 20,
		26, 20,
		22, 22,
		20, 27,
		21, 31,
		26, 31,
		30, 27,
		31, 23
	};

	tMapPoint waypoints_2[3]=
	{
		24, 3,
		27, 3,
		20, 3
	};

	tMapPoint PatrolPoints[8];

	if ( OrbitTeam == Gorn_1Team )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_1[0];
			PatrolPoints[1] = waypoints_1[1];
			PatrolPoints[2] = waypoints_1[2];
			PatrolPoints[3] = waypoints_1[3];
			PatrolPoints[4] = waypoints_1[4];
			PatrolPoints[5] = waypoints_1[5];
			PatrolPoints[6] = waypoints_1[6];
			PatrolPoints[7] = waypoints_1[7];
			Ship->mPatrol( 60.0, 8, &PatrolPoints[0], kHighPriority );
		}
	}
	else if ( OrbitTeam == OrionPatrolTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_2[0];
			PatrolPoints[1] = waypoints_2[1];
			PatrolPoints[2] = waypoints_2[2];
			Ship->mPatrol( 1.0, 3, &PatrolPoints[0], kHighPriority );
		}
	}
}

//---------------------------------------------------------------------------------------------
// Check the win conditions
void WinCondition( tScript& tScript )
{
	if ( OrionDestroyed == MaxOrions )
	{
		tScript.fMission.mDisplayMessage( kAllOrionsDestroyed_msg, kScienceScriptMessage );
	}
}
