#include "Gor_5KingsGuard.h"

// LNF when the player gets within a certain distance of the planet (100.0), the egg defenders should intercept

tKingsGuard::tKingsGuard( eTeamTypes errant, eTeamTypes eggDefender, eTeamID playerTeam )
{
	fErrantTeam = errant;
	fEggDefenderTeam = eggDefender;
	fPlayerTeam = playerTeam;
}

void tKingsGuard::mCreateShips( tTeamInfo* errantTeam, tTeamInfo* eggDefenderTeam, int32 shipBPV, eClassTypes LargestEnemyShipClass )
{
	// LNF should I make this dependent on how many ships the player has?

	errantTeam->mCreateShip( shipBPV, kClassFrigate, LargestEnemyShipClass, 0, kStartPosition_J );
	errantTeam->mCreateShip( shipBPV, kClassFrigate, LargestEnemyShipClass, 0, kStartPosition_K );

	eggDefenderTeam->mCreateShip( shipBPV, kClassDestroyer, LargestEnemyShipClass, 0, kStartPosition_M );
	eggDefenderTeam->mCreateShip( shipBPV, kClassDestroyer, LargestEnemyShipClass, 0, kStartPosition_N );
}

void tKingsGuard::mSetUpShips( tScript& tScript, tShipInfo* ship )
{
	ship->mSetCommButton (1, kButtonHail_text);
	ship->mSetShipCallbackEvents( kNotifyDamage );
	if ( ship->mGetTeam() == fErrantTeam )
	{
		ship->mChangeName( kErrant_name );
	}
	else
	{
		ship->mChangeName( kEggDefender_name );
	}
}

int32 tKingsGuard::mInitializeFinished()
{
	return kKingsGuardBriefing_msg;
}

void tKingsGuard::mSetAllTeamRelations( tMissionInfo& mission )
{
	mission.mSetTeamRelations( fPlayerTeam, kOrionMeetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, fErrantTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, fEggDefenderTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, kOrionPatrolTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, kOrionCommandTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

	mission.mSetTeamRelations( kOrionMeetTeam, fErrantTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( kOrionMeetTeam, fEggDefenderTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( kOrionMeetTeam, kOrionPatrolTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( kOrionMeetTeam, kOrionCommandTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( fErrantTeam, fEggDefenderTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fErrantTeam, kOrionPatrolTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fErrantTeam, kOrionCommandTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

	mission.mSetTeamRelations( fEggDefenderTeam, kOrionPatrolTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( fEggDefenderTeam, kOrionCommandTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( kOrionPatrolTeam, kOrionCommandTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
}

void tKingsGuard::mCaptureHolyMother( tScript& tScript, tTeamInfo* ErrantTeam, tTeamInfo* EggDefenderTeam )
{
	// tScript.fMission.mDisplayMessage( kLNFEliminateOrion_msg, kMissionScriptMessage );
	// LNF Orions set to AtWar
	// LNF EggDefender faction?
	tScript.fMission.mSetTeamRelations( tScript.kPlayerTeam, fEggDefenderTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue );
}

void tKingsGuard::mHailMessage( tScript& tScript, tTeamInfo* hailedTeam )
{
	if ( hailedTeam->mGetTeam() == fErrantTeam )
	{
		if ( fOrionAttemptCapture == false )
		{
			if ( fAtWarWith[ kAWGorn_1Team ] == false )
			{
				tScript.fMission.mDisplayMessage( kErrantHailNoTreachery_msg, kMissionScriptMessage );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kErrantHailEnemy_msg, kMissionScriptMessage );
			}
		}
		else
		{
			if ( fAtWarWith[ kAWGorn_1Team ] == false )
			{
				tScript.fMission.mDisplayMessage( kErrantHailTreachery_msg, kMissionScriptMessage );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kErrantHailEnemy_msg, kMissionScriptMessage );
			}
		}
	}
	else // eggDefender
	{
		if ( fOrionAttemptCapture == false )
		{
			if ( fAtWarWith[ kAWGorn_2Team ] == false )
			{
				tScript.fMission.mDisplayMessage( kEggHailNoTreachery_msg, kMissionScriptMessage );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kEggHailEnemy_msg, kMissionScriptMessage );
			}
		}
		else
		{
			if ( fAtWarWith[ kAWGorn_2Team ] == false )
			{
				tScript.fMission.mDisplayMessage( kEggHailTreachery_msg, kMissionScriptMessage );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kEggHailEnemy_msg, kMissionScriptMessage );
			}
		}
	}
}

void tKingsGuard::mTauntMessage( tScript& tScript, tTeamInfo* hailedTeam )
{
	if ( hailedTeam->mGetTeam() == fErrantTeam )
	{
		tScript.fMission.mDisplayMessage( tScript.fMission.mRandomInt32( kErrantTaunt_1_msg, kErrantTaunt_4_msg ), kMissionScriptMessage );
	}
	else
	{
		tScript.fMission.mDisplayMessage( tScript.fMission.mRandomInt32( kEggTaunt_1_msg, kEggTaunt_4_msg ), kMissionScriptMessage );
	}
}

void tKingsGuard::mSurrenderMessage( tScript& tScript, tTeamInfo* hailedTeam )
{
	if ( hailedTeam->mGetTeam() == fErrantTeam )
	{
		if ( fAtWarWith[ kAWGorn_1Team ] == false )
		{
			tScript.fMission.mDisplayMessage( kErrantSurrenderNotEnemy_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kErrantSurrenderOK_msg, kMissionScriptMessage );
		}
	}
	else // eggDefender
	{
		if ( fAtWarWith[ kAWGorn_2Team ] == false )
		{
			tScript.fMission.mDisplayMessage( kEggSurrenderNotEnemy_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kEggSurrenderOK_msg, kMissionScriptMessage );
		}
	}
}

void tKingsGuard::mEggDetection( tScript& tScript  )
{
	tScript.fMission.mSetTeamRelations( tScript.kPlayerTeam, fEggDefenderTeam, kTeamHostile, kGoalDestroy, 1.0 );
	tScript.fMission.mSetTeamRelations( fEggDefenderTeam, tScript.kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
}

void tKingsGuard::mErrantPatrol( tScript& tScript )
{
	const float kPatrolAttackRadius = 20.0f;

	tMapPoint waypoints_1[7]=
	{
		12, 5,
		17, 6,
		17, 10,
		13, 12,
		11, 16,
		8, 11,
		8, 6
	};

	tMapPoint PatrolPoints[10];

	for (tShipInfo* Ship = tScript.fMission.mGetTeamHandle(fErrantTeam)->mGetFirstShip(); Ship != NULL; Ship = tScript.fMission.mGetTeamHandle(fErrantTeam)->mGetNextShip())
	{
		PatrolPoints[0] = waypoints_1[0];
		PatrolPoints[1] = waypoints_1[1];
		PatrolPoints[2] = waypoints_1[2];
		PatrolPoints[3] = waypoints_1[3];
		PatrolPoints[4] = waypoints_1[4];
		PatrolPoints[5] = waypoints_1[5];
		PatrolPoints[6] = waypoints_1[6];
		Ship->mPatrol( kPatrolAttackRadius, 7, &PatrolPoints[0], kVeryHighPriority );

		Ship->mSetShipMaxSpeed( 10 );
	}
}

void tKingsGuard::mDamageOrionCommand( tScript& tScript, eTeamTypes HostileTeam )
{
}

void tKingsGuard::mEggDefenderPatrol( tScript& tScript )
{
	const float kPatrolAttackRadius = 1.0f;
	const int32 kShipMaxSpeed = 12;

	tMapPoint waypoints_1[5]=
	{
		17, 14,
		14, 16,
		15, 12,
		20, 14,
		23, 14
	};

	tMapPoint PatrolPoints[10];

	for (tShipInfo* Ship = tScript.fMission.mGetTeamHandle(fEggDefenderTeam)->mGetFirstShip(); Ship != NULL; Ship = tScript.fMission.mGetTeamHandle(fEggDefenderTeam)->mGetNextShip())
	{
		PatrolPoints[0] = waypoints_1[0];
		PatrolPoints[1] = waypoints_1[1];
		PatrolPoints[2] = waypoints_1[2];
		PatrolPoints[3] = waypoints_1[3];
		PatrolPoints[4] = waypoints_1[4];
		Ship->mPatrol( kPatrolAttackRadius, 5, &PatrolPoints[0], kVeryHighPriority );

		Ship->mSetShipMaxSpeed( kShipMaxSpeed );
	}
}

void tKingsGuard::mKingsGuardPatrol( tScript& tScript )
{
}

void tKingsGuard::mAtWarWithFaction( tScript& tScript )
{
}
