#include "Gor_6KingsGuard.h"

tKingsGuard::tKingsGuard( eTeamTypes errant, eTeamTypes eggDefender, eTeamID playerTeam )
{
	fErrantTeam = errant;
	fEggDefenderTeam = eggDefender;
	fPlayerTeam = playerTeam;
}

void tKingsGuard::mCreateShips( tTeamInfo* errantTeam, tTeamInfo* eggDefenderTeam, int32 lowBPV )
{
	errantTeam->mCreateShip( lowBPV, kClassFrigate, kClassDreadnought, 100, kStartPosition_X );

	eggDefenderTeam->mCreateShip( lowBPV, kClassDestroyer, kClassDreadnought, 100, kStartPosition_U );
}

int32 tKingsGuard::mInitializeFinished()
{
	return kKingsGuardBriefing_msg;
}

void tKingsGuard::mSetUpShips( tScript& tScript, tShipInfo* ship )
{
	ship->mSetCommButton (1, kButtonHail_text);
	ship->mSetShipCallbackEvents( kNotifyDamage );

	if ( ship->mGetTeam() == fErrantTeam )
	{
		ship->mChangeName( kErrant_name );
	}
	else
	{
		ship->mChangeName( kEggDefender_name );
	}
}

void tKingsGuard::mSetAllTeamRelations( tMissionInfo& mission )
{
	mission.mSetTeamRelations( fPlayerTeam, kShipyardTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, fErrantTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, fEggDefenderTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, kRepairingTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, kMysteryTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( kShipyardTeam, fErrantTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( kShipyardTeam, fEggDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( kShipyardTeam, kRepairingTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( kShipyardTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( kShipyardTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( fErrantTeam, fEggDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fErrantTeam, kRepairingTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( fErrantTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( fErrantTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( fEggDefenderTeam, kRepairingTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	mission.mSetTeamRelations( fEggDefenderTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( fEggDefenderTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( kRepairingTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	mission.mSetTeamRelations( kRepairingTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	mission.mSetTeamRelations( kMysteryTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	mission.mDisplayMessage( kEvent6StartKings_msg, kMissionScriptMessage );
}

void tKingsGuard::mHailMessage( tScript& tScript, tTeamInfo* hailedTeam )
{
	if ( hailedTeam->mGetTeam() == fErrantTeam )
	{
		tScript.fMission.mDisplayMessage( kErrantHailOK_msg, kMissionScriptMessage );
	}
	else // fEggDefenderTeam
	{
		tScript.fMission.mDisplayMessage( kEggHailOK_msg, kMissionScriptMessage );
	}
}

void tKingsGuard::mTauntMessage( tScript& tScript, tTeamInfo* hailedTeam )
{
	if ( hailedTeam->mGetTeam() == fErrantTeam )
	{
		tScript.fMission.mDisplayMessage( tScript.fMission.mRandomInt32( kErrantTaunt_1_msg, kErrantTaunt_4_msg ), kMissionScriptMessage );
	}
	else // fEggDefenderTeam
	{
		tScript.fMission.mDisplayMessage( tScript.fMission.mRandomInt32( kEggTaunt_1_msg, kEggTaunt_4_msg ), kMissionScriptMessage );
	}
}

void tKingsGuard::mSurrenderMessage( tScript& tScript, tTeamInfo* hailedTeam )
{
	if ( hailedTeam->mGetTeam() == fErrantTeam )
	{
		tScript.fMission.mDisplayMessage( kErrantSurrenderOK_msg, kMissionScriptMessage );
	}
	else // fEggDefenderTeam
	{
		tScript.fMission.mDisplayMessage( kEggSurrenderOK_msg, kMissionScriptMessage );
	}
}

void tKingsGuard::mSetFactionWar( tScript& tScript )
{
}

void tKingsGuard::mCheckWinCondition( tScript& tScript, tTeamInfo* teamXInfo )
{
}
