#include "scriptinterface.h"

#include "Gor_6MissionText.h"
#include "Gor_6MissionMaps.h"
#include "Gor_6FactionScript.h"
#include "Gor_6KingsGuard.h"
#include "Gor_6EggDefender.h"
#include "Gor_6Errant.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 Gorn_1Destroyed = 0;
int32 Gorn_2Destroyed = 0;
int32 ShipyardDestroyed = 0;
int32 RepairingDestroyed = 0;
int32 MonsterEncountered = 0; // win condition
int32 MysteryDestroyed = 0;
int32 AllGorn_1Destroyed = 0;
int32 AllGorn_2Destroyed = 0;
int32 ShipyardDestroyedOrCaptured = 0;

bool UpgradeOffered = false;
bool AllShipyardsDestroyed = false;
bool AllRepairingDestroyed = false;
bool MonsterAppears = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

const int32 kMaxShipyards = 4;
const int32 kPlanetX = 26;
const int32 kPlanetY = 26;
const int32 kMonsterAppearsTimer = 16;

eMissionMode MissionMode = kStart;

tGornFactionScript* gCurGornFaction = NULL;

void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam);
void SetAllTeamRelations(tScript& tScript, eTeamID kPlayerTeam);
void CheckWinCondition( tScript& tScript );
void KingsGuardDebriefing( tScript& tScript );
void EggDefenderDebriefing( tScript& tScript );
void ErrantDebriefing( tScript& tScript );

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)

// Team enums are in Gor_XFactionScript.h

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kShipyardTeam, kGorn);
	fMission.mSetTeamRace (kGorn_1Team, kGorn);
	fMission.mSetTeamRace (kGorn_2Team, kGorn);
	fMission.mSetTeamRace (kRepairingTeam, kGorn);
	fMission.mSetTeamRace (kMysteryTeam, kGorn);
	fMission.mSetTeamRace (kPlanetTeam, kGorn);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kShipyardTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kGorn_1Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kGorn_2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kRepairingTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kShipyardTeam, kGorn_1Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kShipyardTeam, kGorn_2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kShipyardTeam, kRepairingTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kShipyardTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kShipyardTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kGorn_1Team, kGorn_2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_1Team, kRepairingTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_1Team, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_1Team, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kGorn_2Team, kRepairingTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_2Team, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kGorn_2Team, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRepairingTeam, kMysteryTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRepairingTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kMysteryTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
		gCurGornFaction = new tKingsGuard( kGorn_1Team, kGorn_2Team, kPlayerTeam );
		return 1; // return KingsGuard map
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
		gCurGornFaction = new tEggDefender( kGorn_1Team, kGorn_2Team, kPlayerTeam );
		return 2; // Egg Defender map
	}
	else
	{
		Elite = kErrant;
		gCurGornFaction = new tErrant( kGorn_1Team, kGorn_2Team, kPlayerTeam );
		return 3; // Errant map
	}
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
	// LNF maybe change the location of the player's starting position based on faction?
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* Gorn_1Team = fMission.mGetTeamHandle (kGorn_1Team);
	tTeamInfo* Gorn_2Team = fMission.mGetTeamHandle (kGorn_2Team);
	tTeamInfo* RepairingTeam = fMission.mGetTeamHandle (kRepairingTeam);
	tTeamInfo* MysteryTeam = fMission.mGetTeamHandle (kMysteryTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( MissionMode == kStart )
	{
		// create Shipyard team
		ShipyardTeam->mCreateShip( "G-FRD", kStartPosition_J );
		ShipyardTeam->mCreateShip( "G-FRD", kStartPosition_L );
		ShipyardTeam->mCreateShip( "G-FRD", kStartPosition_M );
		ShipyardTeam->mCreateShip( "G-FRD", kStartPosition_N );

		// create the ships in the shipyard = repairing team
		RepairingTeam->mCreateShip( "G-CC", kStartPosition_J );
		RepairingTeam->mCreateShip( "G-CC", kStartPosition_L );
		RepairingTeam->mCreateShip( "G-CC", kStartPosition_M );
		RepairingTeam->mCreateShip( "G-CC", kStartPosition_N );

		// create planet
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPlanetX, kPlanetY );
	}
	else if ( MissionMode == kInitialShips )
	{
		// create the other faction's ships
		gCurGornFaction->mCreateShips( Gorn_1Team, Gorn_2Team, TotalBPV );
	}
	else if ( MissionMode == kMonsterAppears )
	{
		MysteryTeam->mCreateShip( "M-SSL", kStartPosition_Q );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return gCurGornFaction->mInitializeFinished();
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* Gorn_1Team = fMission.mGetTeamHandle (kGorn_1Team);
	tTeamInfo* Gorn_2Team = fMission.mGetTeamHandle (kGorn_2Team);
	tTeamInfo* RepairingTeam = fMission.mGetTeamHandle (kRepairingTeam);
	tTeamInfo* MysteryTeam = fMission.mGetTeamHandle (kMysteryTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{		
	}

	// configure Shipyard team
	for (Ship = ShipyardTeam->mGetFirstShip(); Ship != NULL; Ship = ShipyardTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Repairing team
	for (Ship = RepairingTeam->mGetFirstShip(); Ship != NULL; Ship = RepairingTeam->mGetNextShip())
	{
		Ship->mSetShipAIControlled( kFalse );
		Ship->mDamageSystem( kDamageMaxWeapons, 100.0f );
	}

	// configure Planet 
	for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
	{
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	switch ( MissionMode )
	{
		case kInitialShips:
			gCurGornFaction->mSetUpShips( *this, Ship );
			break;

		case kMonsterAppears:
			Ship->mChangeName( kMystery_name );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* Gorn_1Team = fMission.mGetTeamHandle (kGorn_1Team);
	tTeamInfo* Gorn_2Team = fMission.mGetTeamHandle (kGorn_2Team);
	tTeamInfo* RepairingTeam = fMission.mGetTeamHandle (kRepairingTeam);
	tTeamInfo* MysteryTeam = fMission.mGetTeamHandle (kMysteryTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyAtDestination:
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if (( Ship->mGetTeam() == kShipyardTeam )
						|| ( Ship->mGetTeam() == kRepairingTeam ))
					{
						// start a timer that will make the monster appear
						//fMission.mSetTimedInterval( kTimerTurnsSet, kMonsterAppearsTimer );
						//gCurGornFaction->mSetFactionWar( *this, HostileShip );
					}
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					gCurGornFaction->mSetAllTeamRelations( fMission );

					MissionMode = kInitialShips;
					mCreateMissionShips( kInitialShips );
					break;

				case kTimerLimitSet_2:
					fMission.mSetGoalIndicator( MysteryTeam->mGetFirstShip() );
					fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					break;

				case kTimerLimitSet_3:
					{
						for (tShipInfo* tempShip = PlayerTeam->mGetFirstShip(); tempShip != NULL; tempShip = PlayerTeam->mGetNextShip())
						{		
							if ( tempShip->mGetTacticalDistance( MysteryTeam->mGetFirstShip() ) <= 80.0f )
							{
								fMission.mSetGoalIndicator( kFalse, 0, 0 );
								break;
							}
							else
							{
								fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
								break;
							}
						}
					}
					break;
					
				case kTimerTurnsSet:
					// monster appears
					if ( MonsterAppears == false )
					{
						MonsterAppears = true;

						fMission.mSetTeamRelations( kPlayerTeam, kMysteryTeam, kTeamHostile, kGoalDestroy, 1.0 );
						fMission.mSetTeamRelations( kMysteryTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );

						MonsterEncountered = 1;
						MissionMode = kMonsterAppears;
						mCreateMissionShips( kMonsterAppears );

						fMission.mDisplayMessage( kMessageFromDustCloud_msg, kMissionScriptMessage );

						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					break;

				case kTimerTurnsSet_2:
					break;

				case kTimerTurnsSet_8:
					// set Gorn_1 comm buttons back to HAIL
					for (Ship = Gorn_1Team->mGetFirstShip(); Ship != NULL; Ship = Gorn_1Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Gorn_2 comm buttons back to HAIL
					for (Ship = Gorn_2Team->mGetFirstShip(); Ship != NULL; Ship = Gorn_2Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Mystery Team comm buttons back to HAIL
					for (Ship = MysteryTeam->mGetFirstShip(); Ship != NULL; Ship = MysteryTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
			}
			break;
		
		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyOnCapture:
			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship->mGetTeam() == kRepairingTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 23 );
			}
			else if ( Ship->mGetTeam() == kShipyardTeam )
			{
				ShipyardDestroyedOrCaptured++;

				if (ShipyardTeam->mGetShipsInPlayCount() == 0)
				{
					AllShipyardsDestroyed = true;

					if ( AllRepairingDestroyed == true )
					{
						if ( Elite == kKingsGuard )
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
						}
						else if ( Elite == kEggDefender )
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyedErrant_msg, kMissionScriptMessage );

							// LNF add expell from elite here?
							return (eGameStatus)( kPlayerLoses | kExpellFromElite );
						}

						// monster appears
						fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
					}
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kGorn_1Team)
			{
				Gorn_1Destroyed++;

				if (Gorn_1Team->mGetShipsInPlayCount() == 0)
				{
					AllGorn_1Destroyed = 1;

					if ( Elite == kKingsGuard )
					{
						fMission.mDisplayMessage( kAllErrantShipsDestroyed_msg, kScienceScriptMessage );
					}
					else if ( Elite == kEggDefender )
					{
					}
					else
					{
						if ( Gorn_2Team->mGetShipsInPlayCount() == 0 )
						{
							if ( (kMaxShipyards - ShipyardDestroyed) == 4 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_4_msg, kScienceScriptMessage );
							}
							else if ( (kMaxShipyards - ShipyardDestroyed) == 3 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_3_msg, kScienceScriptMessage );
							}
							else if ( (kMaxShipyards - ShipyardDestroyed) == 2 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_2_msg, kScienceScriptMessage );
							}
							else if ( (kMaxShipyards - ShipyardDestroyed) == 1 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_1_msg, kScienceScriptMessage );
							}

							// monster appears
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kGorn_2Team)
			{
				Gorn_2Destroyed++;

				if (Gorn_2Team->mGetShipsInPlayCount() == 0)
				{
					AllGorn_2Destroyed = 1;

					if ( Elite == kKingsGuard )
					{
					}
					else if ( Elite == kEggDefender )
					{
					}
					else
					{
						if ( Gorn_1Team->mGetShipsInPlayCount() == 0 )
						{
							if ( (kMaxShipyards - ShipyardDestroyed) == 4 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_4_msg, kScienceScriptMessage );
							}
							else if ( (kMaxShipyards - ShipyardDestroyed) == 3 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_3_msg, kScienceScriptMessage );
							}
							else if ( (kMaxShipyards - ShipyardDestroyed) == 2 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_2_msg, kScienceScriptMessage );
							}
							else if ( (kMaxShipyards - ShipyardDestroyed) == 1 )
							{
								fMission.mDisplayMessage( kEvent4AttackersRepelled_1_msg, kScienceScriptMessage );
							}

							// monster appears
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kShipyardTeam)
			{
				ShipyardDestroyed++;
				ShipyardDestroyedOrCaptured++;

				if (ShipyardTeam->mGetShipsInPlayCount() == 0)
				{
					AllShipyardsDestroyed = true;

					if ( AllRepairingDestroyed == true )
					{
						if ( Elite == kKingsGuard )
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
						}
						else if ( Elite == kEggDefender )
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyedErrant_msg, kMissionScriptMessage );

							return (eGameStatus)(kPlayerLoses | kExpellFromElite);
						}

						// monster appears
						fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kRepairingTeam)
			{
				RepairingDestroyed++;

				if (RepairingTeam->mGetShipsInPlayCount() == 0)
				{
					AllRepairingDestroyed = true;

					if ( AllShipyardsDestroyed == true )
					{
						if ( Elite == kKingsGuard )
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
						}
						else if ( Elite == kEggDefender )
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
						}
						else
						{
							fMission.mDisplayMessage( kAllShipyardsDestroyedErrant_msg, kMissionScriptMessage );

							return (eGameStatus)(kPlayerLoses | kExpellFromElite);
						}

						// monster appears
						fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kMysteryTeam)
			{
				MysteryDestroyed++;

				if (MysteryTeam->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kEvent7MysteryDestroyed_msg, kScienceScriptMessage );

					if ( Elite == kKingsGuard )
					{
						if ( Gorn_1Team->mGetShipsInPlayCount() > 0 )
						{
							fMission.mDisplayMessage( kTakeOutErrant_msg, kMissionScriptMessage );
						}
						else
						{
							return kPlayerWins;
							// LNF can leave system, or end mission by script
						}
					}
					else
					{
						// LNF can leave system, or end mission by script
						return kPlayerWins;
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kGorn_1Team:
								gCurGornFaction->mHailMessage( *this, Gorn_1Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kGorn_2Team:
								gCurGornFaction->mHailMessage( *this, Gorn_2Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMysteryTeam:
								fMission.mDisplayMessage( kMysteryHailOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kGorn_1Team:
								gCurGornFaction->mSurrenderMessage( *this, Gorn_1Team );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kGorn_2Team:
								gCurGornFaction->mSurrenderMessage( *this, Gorn_2Team );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMysteryTeam:
								fMission.mDisplayMessage( kMysterySurrender_msg, kMissionScriptMessage );

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kGorn_1Team:
								gCurGornFaction->mTauntMessage( *this, Gorn_1Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kGorn_2Team:
								gCurGornFaction->mTauntMessage( *this, Gorn_2Team );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMysteryTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kMysteryTaunt_1_msg, kMysteryTaunt_4_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;				
				}
			}
			break;
	}

	//return fMission.mGetGameStatus();
	//return kGameInProgress;

	// LNF check other return statements
	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints += (MonsterEncountered * 10) + (MysteryDestroyed * 20);

	if ( Elite == kKingsGuard )
	{
		VictoryPoints += (Gorn_1Destroyed * 5) + (ShipyardDestroyed * 15);
	}
	else if ( Elite == kEggDefender )
	{
		VictoryPoints += (ShipyardDestroyed * 15);
	}
	else
	{
		VictoryPoints += ((kMaxShipyards - ShipyardDestroyed) * 15) + (Gorn_1Destroyed * 5)
						+ (Gorn_2Destroyed * 5);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kKingsGuard)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kKingsGuardAstoundingVictory_msg);
		}
		else if (fMission.mGetEliteOrganization() == kEggDefender)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kEggDefenderAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kErrantAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	if ( Elite == kKingsGuard )
	{
		KingsGuardDebriefing( *this );
	}
	else if ( Elite == kEggDefender )
	{
		EggDefenderDebriefing( *this );
	}
	else
	{
		ErrantDebriefing( *this );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}

	if( gCurGornFaction )
	{
		delete gCurGornFaction;
		gCurGornFaction = NULL;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam)
{
	tTeamInfo* ShipyardTeam = tScript.fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* Gorn_1Team = tScript.fMission.mGetTeamHandle (kGorn_1Team);
	tShipInfo* Ship;

	tMapPoint waypoints_1[8]=
	{
		30, 20,
		26, 20,
		22, 22,
		20, 27,
		21, 31,
		26, 31,
		30, 27,
		31, 23
	};

	tMapPoint waypoints_2[2]=
	{
		10, 14,
		17, 14
	};

	tMapPoint PatrolPoints[8];

	if ( OrbitTeam == Gorn_1Team )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_1[0];
			PatrolPoints[1] = waypoints_1[1];
			PatrolPoints[2] = waypoints_1[2];
			PatrolPoints[3] = waypoints_1[3];
			PatrolPoints[4] = waypoints_1[4];
			PatrolPoints[5] = waypoints_1[5];
			PatrolPoints[6] = waypoints_1[6];
			PatrolPoints[7] = waypoints_1[7];
			Ship->mPatrol( 60.0, 8, &PatrolPoints[0], kHighPriority );
		}
	}
	else if ( OrbitTeam == ShipyardTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_2[0];
			PatrolPoints[1] = waypoints_2[1];
			Ship->mPatrol( 50.0, 2, &PatrolPoints[0], kHighPriority );
		}
	}
}

void CheckWinCondition( tScript& tScript )
{
}

void KingsGuardDebriefing( tScript& tScript )
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	// LNF try this out:  adding results of primary objectives
	tScript.fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( ShipyardDestroyed == kMaxShipyards ) ? tScript.fMission.mAddToDebriefingList( kShipyardDestroyedTrue_msg ) : SuccessCount++;
	( AllGorn_1Destroyed == 1 ) ? tScript.fMission.mAddToDebriefingList( kAllErrantDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		tScript.fMission.mAddToDebriefingList( kNone_msg );
	}

	tScript.fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( ShipyardDestroyed < kMaxShipyards ) ? tScript.fMission.mAddToDebriefingList( kShipyardDestroyedFalse_msg ) : FailureCount++;
	( AllGorn_1Destroyed == 0 ) ? tScript.fMission.mAddToDebriefingList( kAllErrantDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		tScript.fMission.mAddToDebriefingList( kNone_msg );
	}
}

void EggDefenderDebriefing( tScript& tScript )
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	// LNF try this out:  adding results of primary objectives
	tScript.fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( ShipyardDestroyed == kMaxShipyards ) ? tScript.fMission.mAddToDebriefingList( kShipyardDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		tScript.fMission.mAddToDebriefingList( kNone_msg );
	}

	tScript.fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( ShipyardDestroyed < kMaxShipyards ) ? tScript.fMission.mAddToDebriefingList( kShipyardDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		tScript.fMission.mAddToDebriefingList( kNone_msg );
	}
}

void ErrantDebriefing( tScript& tScript )
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	// LNF try this out:  adding results of primary objectives
	tScript.fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( ShipyardDestroyed < kMaxShipyards ) ? tScript.fMission.mAddToDebriefingList( kShipyardErrantTrue_msg ) : SuccessCount++;
	( AllGorn_1Destroyed == 1 ) ? tScript.fMission.mAddToDebriefingList( kAllEggDefenderDestroyedTrue_msg ) : SuccessCount++;
	( AllGorn_2Destroyed == 1 ) ? tScript.fMission.mAddToDebriefingList( kAllKingsGuardDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		tScript.fMission.mAddToDebriefingList( kNone_msg );
	}

	tScript.fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( ShipyardDestroyed == kMaxShipyards ) ? tScript.fMission.mAddToDebriefingList( kShipyardErrantFalse_msg ) : FailureCount++;
	( AllGorn_1Destroyed == 0 ) ? tScript.fMission.mAddToDebriefingList( kAllEggDefenderDestroyedFalse_msg ) : FailureCount++;
	( AllGorn_2Destroyed == 0 ) ? tScript.fMission.mAddToDebriefingList( kAllKingsGuardDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		tScript.fMission.mAddToDebriefingList( kNone_msg );
	}
}