#include "scriptinterface.h"

#include "Gor_7MissionText.h"
#include "Gor_7MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 Mode = 0;
int32 PlayerDestroyed = 0;
int32 OrionBaseDestroyed = 0;
int32 OrionPatrolDestroyed = 0;
int32 OrionHeavyDestroyed = 0;
int32 OrionGuardDestroyed = 0;
int32 GornFleetDestroyed = 0;
int32 BaseScanned = 0; // win condition

const float kDeepScanDistanceBase = 15.0f; // shorter scan range because of nebula?
const int32 kDeepScanTurns = 2;
const int32 kGornFleetX = 23;
const int32 kGornFleetY = 36;
const float kGornFleetDistance = 15.0f;

bool OrionBaseExtinct = false;
bool OrionPatrolExtinct = false;
bool OrionHeavyExtinct = false;
bool OrionGuardExtinct = false;
bool FleetRendezvous = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void setPatrolRoute(tScript& tScript, tTeamInfo* patrolRouteTeam);

eGameStatus checkWinCondition();

enum eMissionMode
{
	kStart = 0,
	kMakeBase
};
eMissionMode MissionMode = kStart;

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kOrionBaseTeam,	 
	kOrionPatrolTeam,
	kOrionHeavyTeam,
	kOrionGuardTeam,
	kGornFleetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial;  // LNF takes place in Gorn space kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF Gorn is the enemy
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kOrionBaseTeam, kOrion );
	fMission.mSetTeamRace( kOrionPatrolTeam, kOrion );
	fMission.mSetTeamRace( kOrionHeavyTeam, kOrion );
	fMission.mSetTeamRace( kOrionGuardTeam, kOrion );
	fMission.mSetTeamRace( kGornFleetTeam, kGorn );

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kGornFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);


	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
	// LNF maybe change the location of the player's starting position based on faction?
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* OrionPatrolTeam = fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionHeavyTeam = fMission.mGetTeamHandle (kOrionHeavyTeam);
	tTeamInfo* OrionGuardTeam = fMission.mGetTeamHandle (kOrionGuardTeam);
	tTeamInfo* GornFleetTeam = fMission.mGetTeamHandle (kGornFleetTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;

		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;

		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( MissionMode == kStart )
	{
		// create Orion Patrol team
		OrionPatrolTeam->mCreateFleet( 2, (2 * AveragePlayerBPV), kClassDestroyer, kClassLightCruiser, 0, kStartPosition_V );

		// create Orion Heavy team
		OrionHeavyTeam->mCreateFleet( 2, (2 * AveragePlayerBPV), kClassLightCruiser, kClassLightCruiser, 0, kStartPosition_R );

		// create Orion Guard team
		OrionGuardTeam->mCreateFleet( 2, (2 * AveragePlayerBPV), kClassDestroyer, kClassDestroyer, 0, kStartPosition_T );

		// create Gorn Reinforcement team
		GornFleetTeam->mCreateFleet( 4, (4 * AveragePlayerBPV), kClassHeavyCruiser, kClassDreadnought, 0, kStartPosition_J );
	}
	else if ( MissionMode == kMakeBase )
	{
		// create Orion Base
		OrionBaseTeam->mCreateShip( "O-BS", kStartPosition_Z );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// there is only one briefing for this mission
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* OrionPatrolTeam = fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionHeavyTeam = fMission.mGetTeamHandle (kOrionHeavyTeam);
	tTeamInfo* OrionGuardTeam = fMission.mGetTeamHandle (kOrionGuardTeam);
	tTeamInfo* GornFleetTeam = fMission.mGetTeamHandle (kGornFleetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		// LNF need a notify distance thing for the Gorn fleet
		// LNF should I use a ship instead of a point in space?
		Ship->mTrackTacticalDistance( kGornFleetDistance, kGornFleetX, kGornFleetY );
	}

	// configure Orion Patrol team
	for (Ship = OrionPatrolTeam->mGetFirstShip(); Ship != NULL; Ship = OrionPatrolTeam->mGetNextShip())
	{
		//Ship->mSetCommButton (1, kButtonHail_text);
	}
	setPatrolRoute( *this, OrionPatrolTeam );

	// configure Orion Heavy team
	for (Ship = OrionHeavyTeam->mGetFirstShip(); Ship != NULL; Ship = OrionHeavyTeam->mGetNextShip())
	{
		//Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipMaxSpeed( 12 );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Orion Guard team
	for (Ship = OrionGuardTeam->mGetFirstShip(); Ship != NULL; Ship = OrionGuardTeam->mGetNextShip())
	{
		//Ship->mSetCommButton (1, kButtonHail_text);
	}
	setPatrolRoute( *this, OrionGuardTeam );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn
	//DeepScanTracker.mSetTarget( OrionBaseTeam->mGetFirstShip(), kDeepScanDistanceBase, kDeepScanTurns );
	// DeepScanTracker.mSetLocation (fractureNumber[1].x, fractureNumber[1].y, kDeepScanDistance, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if ( Ship->mGetTeam() == kOrionBaseTeam )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		DeepScanTracker.mSetTarget( Ship, kDeepScanDistanceBase, kDeepScanTurns );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* OrionPatrolTeam = fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionHeavyTeam = fMission.mGetTeamHandle (kOrionHeavyTeam);
	tTeamInfo* OrionGuardTeam = fMission.mGetTeamHandle (kOrionGuardTeam);
	tTeamInfo* GornFleetTeam = fMission.mGetTeamHandle (kGornFleetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kOrionHeavyTeam )
						{
							Ship->mSetShipMaxSpeed( 31 );
						}
						else if ( Ship->mGetTeam() == kOrionBaseTeam )
						{
							for (tShipInfo* tempShip = OrionHeavyTeam->mGetFirstShip(); tempShip != NULL; tempShip = OrionHeavyTeam->mGetNextShip())
							{
								tempShip->mSetShipMaxSpeed( 31 );
							}
						}
					}
				}
			}
			break;

		case kNotifyAtDistance:
			// LNF check distance to Gorn Fleet only after BaseScanned = 1
			// LNF make them attack the Orion base

			if (( BaseScanned == 1 ) && ( FleetRendezvous == false ))
			{
				FleetRendezvous = true;
				fMission.mDisplayMessage( kFoundFleet_msg, kMissionScriptMessage );
				fMission.mSetGoalIndicator( kFalse, kGornFleetX, kGornFleetY );

				fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				fMission.mSetTeamRelations (kPlayerTeam, kOrionPatrolTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				fMission.mSetTeamRelations (kPlayerTeam, kOrionHeavyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				fMission.mSetTeamRelations (kPlayerTeam, kOrionGuardTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				fMission.mSetTeamRelations (kPlayerTeam, kGornFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

				fMission.mSetTeamRelations (kOrionBaseTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***

				fMission.mSetTeamRelations (kOrionPatrolTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***

				fMission.mSetTeamRelations (kOrionHeavyTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***

				fMission.mSetTeamRelations (kOrionGuardTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***

				fMission.mSetTeamRelations (kGornFleetTeam, kOrionBaseTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
				fMission.mSetTeamRelations (kGornFleetTeam, kOrionPatrolTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
				fMission.mSetTeamRelations (kGornFleetTeam, kOrionHeavyTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
				fMission.mSetTeamRelations (kGornFleetTeam, kOrionGuardTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
			}
			else
			{
				Ship->mTrackTacticalDistance( kGornFleetDistance, kGornFleetX, kGornFleetY );
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kOrionBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionPatrolTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionHeavyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionGuardTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kGornFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionBaseTeam, kOrionPatrolTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionBaseTeam, kOrionHeavyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionBaseTeam, kOrionGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionBaseTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionPatrolTeam, kOrionHeavyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionPatrolTeam, kOrionGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionPatrolTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionHeavyTeam, kOrionGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionHeavyTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionGuardTeam, kGornFleetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					MissionMode = kMakeBase;
					mCreateMissionShips( kMakeBase );

					fMission.mSetTimedInterval( kTimerLimitSet_2, 12 );
					break;

				case kTimerLimitSet_2:
					// timer for warning message
					fMission.mDisplayMessage( kEvent1Start_1_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kEvent1Start_2_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kEvent1Start_3_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kEvent1Start_4_msg, kScienceScriptMessage );
					break;
					
				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;

							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;

							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;

							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								fMission.mDisplayMessage (kStartingScanBase_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								fMission.mDisplayMessage (kStartingScanBase_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;

							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object or location
							BaseScanned = 1;

							fMission.mDisplayMessage( kEvent4EnoughData_msg, kScienceScriptMessage );

							fMission.mSetGoalIndicator( kTrue, kGornFleetX, kGornFleetY );

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_2:
					break;

				case kTimerTurnsSet_9:
					// set OrionHeavy buttons back to HAIL
					for (Ship = OrionHeavyTeam->mGetFirstShip(); Ship != NULL; Ship = OrionHeavyTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Orion Patrol buttons back to HAIL
					for (Ship = OrionPatrolTeam->mGetFirstShip(); Ship != NULL; Ship = OrionPatrolTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (( Ship->mGetTeam() == kOrionBaseTeam )
				|| ( Ship->mGetTeam() == kOrionPatrolTeam )
				|| ( Ship->mGetTeam() == kOrionHeavyTeam )
				|| ( Ship->mGetTeam() == kOrionGuardTeam ))
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageWeaponsAndSystems, 1023 );
			}

			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kOrionBaseTeam)
			{
				OrionBaseDestroyed++;
				if (OrionBaseTeam->mGetShipsInPlayCount() == 0)
				{
					OrionBaseExtinct = true;
					return checkWinCondition();
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionPatrolTeam)
			{
				OrionPatrolDestroyed++;
				if (OrionPatrolTeam->mGetShipsInPlayCount() == 0)
				{
					OrionPatrolExtinct = true;
					return checkWinCondition();
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionHeavyTeam)
			{
				OrionHeavyDestroyed++;
				if (OrionHeavyTeam->mGetShipsInPlayCount() == 0)
				{
					OrionHeavyExtinct = true;
					return checkWinCondition();
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionGuardTeam)
			{
				OrionGuardDestroyed++;
				if (OrionGuardTeam->mGetShipsInPlayCount() == 0)
				{
					OrionGuardExtinct = true; 
					return checkWinCondition();
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					// LNF need a button to request reinforcements
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kOrionPatrolTeam:

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionBaseTeam:
								break;

							case kOrionHeavyTeam:

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kOrionPatrolTeam:
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionBaseTeam:
								break;

							case kOrionHeavyTeam:
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kOrionPatrolTeam:
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionBaseTeam:
								break;

							case kOrionHeavyTeam:
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints += (BaseScanned * 40) + (OrionBaseDestroyed * 60);

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( BaseScanned != 0 ) ? fMission.mAddToDebriefingList( kOrionBaseScannedTrue_msg ) : SuccessCount++;
	( OrionBaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOrionBaseDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( BaseScanned == 0 ) ? fMission.mAddToDebriefingList( kOrionBaseScannedFalse_msg ) : FailureCount++;
	( OrionBaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOrionBaseDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships patrol
eGameStatus checkWinCondition()
{
	if (( OrionBaseExtinct == true )
		&& ( OrionHeavyExtinct == true )
		&& ( OrionGuardExtinct == true )
		&& ( OrionPatrolExtinct == true ))
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}

//---------------------------------------------------------------------------------------------
// make the ships patrol
void setPatrolRoute(tScript& tScript, tTeamInfo* patrolRouteTeam)
{
	tTeamInfo* OrionPatrolTeam = tScript.fMission.mGetTeamHandle (kOrionPatrolTeam);
	tTeamInfo* OrionGuardTeam = tScript.fMission.mGetTeamHandle (kOrionGuardTeam);

	const float kGuardRadius = 10.0f;
	const float kPatrolRadius = 10.0f;
	
	tMapPoint waypoints_1[7]= // ship V
	{
		14, 22,
		16, 23,
		20, 24,
		21, 20,
		18, 19,
		14, 18,
		11, 20
	};

	tMapPoint waypoints_2[7]= // ship W
	{
		18, 19,
		14, 18,
		11, 20,
		14, 22,
		16, 23,
		20, 24,
		21, 20
	};

	tMapPoint waypoints_3[6]= // ship X
	{
		12, 15,
		17, 15,
		18, 11,
		17, 7,
		12, 8,
		11, 12
	};

	tMapPoint waypoints_4[6]= // ship Y
	{
		17, 7,
		12, 8,
		11, 12,
		12, 15,
		17, 15,
		18, 11
	};

	tMapPoint waypoints_5[6]= // ship T
	{
		26, 17,
		27, 20,
		29, 20,
		30, 17,
		29, 14,
		27, 14
	};

	tMapPoint waypoints_6[6]= // ship U
	{
		30, 17,
		29, 14,
		27, 14,
		26, 17,
		27, 20,
		29, 20
	};


	tMapPoint PatrolPoints[10];

	if ( patrolRouteTeam == OrionPatrolTeam )
	{
		PatrolPoints[0] = waypoints_1[0];
		PatrolPoints[1] = waypoints_1[1];
		PatrolPoints[2] = waypoints_1[2];
		PatrolPoints[3] = waypoints_1[3];
		PatrolPoints[4] = waypoints_1[4];
		PatrolPoints[5] = waypoints_1[5];
		PatrolPoints[6] = waypoints_1[6];
		OrionPatrolTeam->mGetThisShip( 1 )->mPatrol( kPatrolRadius, 7, &PatrolPoints[0], kVeryHighPriority );
		PatrolPoints[0] = waypoints_2[0];
		PatrolPoints[1] = waypoints_2[1];
		PatrolPoints[2] = waypoints_2[2];
		PatrolPoints[3] = waypoints_2[3];
		PatrolPoints[4] = waypoints_2[4];
		PatrolPoints[5] = waypoints_2[5];
		PatrolPoints[6] = waypoints_2[6];
		OrionPatrolTeam->mGetThisShip( 2 )->mPatrol( kPatrolRadius, 7, &PatrolPoints[0], kVeryHighPriority );
/*
		PatrolPoints[0] = waypoints_3[0];
		PatrolPoints[1] = waypoints_3[1];
		PatrolPoints[2] = waypoints_3[2];
		PatrolPoints[3] = waypoints_3[3];
		PatrolPoints[4] = waypoints_3[4];
		PatrolPoints[5] = waypoints_3[5];
		OrionPatrolTeam->mGetThisShip( 3 )->mPatrol( 45.0, 6, &PatrolPoints[0], kHighPriority );
		PatrolPoints[0] = waypoints_4[0];
		PatrolPoints[1] = waypoints_4[1];
		PatrolPoints[2] = waypoints_4[2];
		PatrolPoints[3] = waypoints_4[3];
		PatrolPoints[4] = waypoints_4[4];
		PatrolPoints[5] = waypoints_4[5];
		OrionPatrolTeam->mGetThisShip( 4 )->mPatrol( 45.0, 6, &PatrolPoints[0], kHighPriority );
*/
	}
	else if ( patrolRouteTeam == OrionGuardTeam )
	{
		PatrolPoints[0] = waypoints_5[0];
		PatrolPoints[1] = waypoints_5[1];
		PatrolPoints[2] = waypoints_5[2];
		PatrolPoints[3] = waypoints_5[3];
		PatrolPoints[4] = waypoints_5[4];
		PatrolPoints[5] = waypoints_5[5];
		OrionGuardTeam->mGetThisShip( 1 )->mPatrol( kGuardRadius, 6, &PatrolPoints[0], kVeryHighPriority );
		PatrolPoints[0] = waypoints_6[0];
		PatrolPoints[1] = waypoints_6[1];
		PatrolPoints[2] = waypoints_6[2];
		PatrolPoints[3] = waypoints_6[3];
		PatrolPoints[4] = waypoints_6[4];
		PatrolPoints[5] = waypoints_6[5];
		OrionGuardTeam->mGetThisShip( 2 )->mPatrol( kGuardRadius, 6, &PatrolPoints[0], kVeryHighPriority );
	}
}
