#include "scriptinterface.h"

#include "Gor_8MissionText.h"
#include "Gor_8MissionMaps.h"

// LNF check the text messages to use Vanguard Team instead of security

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 Mode = 0;
int32 PlayerDestroyed = 0;
int32 FedLegendaryDestroyed = 0;
int32 FedLegendaryCaptured = 0;
int32 FedPatrolDestroyed = 0;
int32 FedGuardDestroyed = 0;
int32 MuddRescued = 0; // win condition
int32 PlanetDamageReport1 = 0;
int32 PlanetShieldDropped = 0;
int32 BaseDestroyed = 0;

const float kFedPatrolRadius = 45.0f;
const float kFedGuardRadius = 40.0f;
const float kFedLegendaryRadius = 2.0f;
const int32 kPlanetX = 20;
const int32 kPlanetY = 20;
const float kPlanetDetectionDistance = 30.0f;
const float kDeepScanDistancePlanet = 30.0f;
const int32 kDeepScanTurns = 2;
const float kFedSurrenderDamage = 60.0f;
const float kPlanetDamageReport = 10.0f;
const float kPlanetDamageDropShield = 20.0f;

bool currentlyScanning = false;
bool orbiting = false;
bool PlanetScanned = false;
bool PlanetWeaknessFound = false;
bool MuddFound = false;
bool ShieldDetected = false;
bool FailedScan = false; // the player failed to successfully scan the planet to find the shield weakness
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void setPatrolRoute(tScript& tScript, tTeamInfo* patrolRouteTeam, int32 NumberOfShips = 1);

enum eMissionMode
{
	kStart = 0,
	kLegendaryArrive
};

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFedPatrolTeam, 
	kFedLegendaryTeam,
	kFedGuardTeam,
	kFedPlatformTeam,
	kPlanetTeam,
	kBaseTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kFedPatrolTeam, kFederation );
	fMission.mSetTeamRace( kFedLegendaryTeam, kFederation );
	fMission.mSetTeamRace( kFedGuardTeam, kFederation );
	fMission.mSetTeamRace( kFedPlatformTeam, kFederation );
	fMission.mSetTeamRace( kPlanetTeam, kFederation );
	fMission.mSetTeamRace( kBaseTeam, kFederation );

	// Set team relations to show up on Tactical map
	// this mission starts without any enemy ships on screen
	fMission.mSetTeamRelations (kPlayerTeam, kFedPatrolTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFedLegendaryTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFedGuardTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFedPlatformTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedPatrolTeam, kFedLegendaryTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedPatrolTeam, kFedGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedPatrolTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedPatrolTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedPatrolTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedLegendaryTeam, kFedGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedLegendaryTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedLegendaryTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedLegendaryTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedGuardTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedGuardTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedGuardTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedPlatformTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedPlatformTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlanetTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
	// LNF maybe change the location of the player's starting position based on faction?
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedPatrolTeam = fMission.mGetTeamHandle (kFedPatrolTeam);
	tTeamInfo* FedLegendaryTeam = fMission.mGetTeamHandle (kFedLegendaryTeam);
	tTeamInfo* FedGuardTeam = fMission.mGetTeamHandle (kFedGuardTeam);
	tTeamInfo* FedPlatformTeam = fMission.mGetTeamHandle (kFedPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		// LNF why did I create a ship here?
		//PlayerTeam->mCreateShip( "G-FF", kStartPosition_J );

		int32 NumberOfFedPatrolShips = 0;
		int32 NumberOfFedGuardShips = 0;

		if (( NumberOfPlayersShips == 1 ) || ( NumberOfPlayersShips == 2 ))
		{
			NumberOfFedPatrolShips = 1;
			NumberOfFedGuardShips = 1;
		}
		else
		{
			NumberOfFedPatrolShips = 1;
			NumberOfFedGuardShips = 2;
		}

		// create Federation Patrol
		FedPatrolTeam->mCreateFleet( NumberOfFedPatrolShips, (NumberOfFedPatrolShips * AveragePlayerBPV),
									 kClassFrigate, kClassLightCruiser, 0, kStartPosition_P );

		// create Federation Guard team
		FedGuardTeam->mCreateFleet( NumberOfFedGuardShips, (NumberOfFedGuardShips * AveragePlayerBPV),
									kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_S );

		// create Federation Defense Platforms
		FedPlatformTeam->mCreateShip( "F-DEF", kStartPosition_U );
		//FedPlatformTeam->mCreateShip( "F-DEF", kStartPosition_V );
		FedPlatformTeam->mCreateShip( "F-DEF", kStartPosition_W );
		//FedPlatformTeam->mCreateShip( "F-DEF", kStartPosition_X );
		FedPlatformTeam->mCreateShip( "F-DEF", kStartPosition_Y );
		//FedPlatformTeam->mCreateShip( "F-DEF", kStartPosition_Z );

		// create Planet
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPlanetX, kPlanetY );

		// create Base
		BaseTeam->mCreateShip( "F-BS+", kStartPosition_N );
	}
	else if ( Mode == kLegendaryArrive )
	{
		// Place the Legendary ship near the player

		// LNF I want to be able to tell where the player's ship is, and make the Legendary ship appear
		// LNF on the other side of the planet.
		// LNF since this is called right after the player transports Harry, they are <= 5 from the planet
		int32 CurrentX = 0;
		int32 CurrentY = 0;
		fPlayerShip->mGetGridPosition (&CurrentX, &CurrentY);
		int32 NewX = CurrentX + fMission.mRandomInt32 (1, 8);
		int32 NewY = CurrentY + fMission.mRandomInt32 (5, 10);

		// LNF do I need team relations here?
		FedLegendaryTeam->mCreateShip( "F-CA", kStartPosition_Specified, NewX, NewY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// there is only one briefing for this mission
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedPatrolTeam = fMission.mGetTeamHandle (kFedPatrolTeam);
	tTeamInfo* FedLegendaryTeam = fMission.mGetTeamHandle (kFedLegendaryTeam);
	tTeamInfo* FedGuardTeam = fMission.mGetTeamHandle (kFedGuardTeam);
	tTeamInfo* FedPlatformTeam = fMission.mGetTeamHandle (kFedPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyBeginOrbit );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mTrackTacticalDistance(kPlanetDetectionDistance, kPlanetX, kPlanetY); // LNF I forgot how the true/false flag worked
	}

	// configure Federation Patrol Team
	int32 FedShipCount = 0;

	for ( Ship = FedPatrolTeam->mGetFirstShip(); Ship != NULL; Ship = FedPatrolTeam->mGetNextShip() )
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		FedShipCount++;
	}
	setPatrolRoute( *this, FedPatrolTeam, FedShipCount );

	// configure Federation Guard team
	for (Ship = FedGuardTeam->mGetFirstShip(); Ship != NULL; Ship = FedGuardTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mDefendEntity( PlanetTeam->mGetFirstShip(), kFedGuardRadius, kFedGuardRadius, kVeryHighPriority );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Federation Defense Platforms
	for (Ship = FedPlatformTeam->mGetFirstShip(); Ship != NULL; Ship = FedPlatformTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Planet
	PlanetTeam->mGetFirstShip()->mChangeName( kPlanetNewBrisbane_name );
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 0);

	// timer for StartScenario message
	fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
	// DeepScanTracker.mSetLocation (fractureNumber[1].x, fractureNumber[1].y, kDeepScanDistance, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if ( Mode == kLegendaryArrive )
	{
		// configure Legendary Fed ship
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetOfficerRank( kMaxOfficers, kLegendary );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	//Ship->mSetShipCallbackEvents( kNotifyDamage );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedPatrolTeam = fMission.mGetTeamHandle (kFedPatrolTeam);
	tTeamInfo* FedLegendaryTeam = fMission.mGetTeamHandle (kFedLegendaryTeam);
	tTeamInfo* FedGuardTeam = fMission.mGetTeamHandle (kFedGuardTeam);
	tTeamInfo* FedPlatformTeam = fMission.mGetTeamHandle (kFedPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ( TransportTarget->mGetOriginalTeam() != kPlayerTeam )
					{
						if ( fEventItem1 == kTransHarryMudd )
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransHarryMudd, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransHarryMudd, 0 );
						}
					}
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if (( fEventItem1 == kTransHarryMudd )
						&& ( MuddRescued == 0 ))
					{
						MuddRescued = 1;
						fMission.mDisplayMessage( kEvent5CapturedMudd_msg, kMissionScriptMessage );
						Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );

						Mode = kLegendaryArrive;
						mCreateMissionShips( kLegendaryArrive );
						fMission.mSetTimedInterval( kTimerLimitSet_2, 7 );
					}
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				Ship->mSetShipCallbackEvents( kNotifyDamage );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if (( Ship->mGetTeam() == kFedPatrolTeam ) || ( Ship->mGetTeam() == kFedPlatformTeam ))
						{
							FedPatrolTeam->mClearCommands();

							// LNF if this doesn't work, maybe I can call the timerlimitset again to reset all relations?
							fMission.mSetTeamRelations (kFedPatrolTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
						}
						else if ( Ship->mGetTeam() == kPlanetTeam )
						{
							if ( PlanetWeaknessFound == false )
							{
								// if the planet hasn't been scanned to find the weakness, regenerate the planet
								Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 0 );
							}
							else
							{
								if (( Ship->mGetDamagePercent() >= kPlanetDamageReport )
									&& ( Ship->mGetDamagePercent() < kPlanetDamageDropShield )
									&& ( PlanetDamageReport1 == 0 )
									&& ( PlanetShieldDropped == 0 ))
								{
									PlanetDamageReport1 = 1;
									fMission.mDisplayMessage( kPlanetShieldTenPercent_msg, kScienceScriptMessage );
								}
								else if (( Ship->mGetDamagePercent() >= kPlanetDamageDropShield )
										&& ( PlanetShieldDropped == 0 ))
								{
									PlanetShieldDropped = 1;
									fMission.mDisplayMessage( kPlanetShieldDropped_msg, kScienceScriptMessage );
									fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
									//DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
									// LNF set the shield down flag here
									break;
								}
							}
						}
						else if( Ship->mGetTeam() == kFedLegendaryTeam )
						{
							Ship->mClearCommands();
							fMission.mSetTeamRelations( kPlayerTeam, kFedLegendaryTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			// LNF what about the true/false parameter?
			if (( Ship->mGetTrackingDistance() == kPlanetDetectionDistance )
				&& ( ShieldDetected == false ) && ( PlanetWeaknessFound == false ))
			{
				if (( fIsDeepScanOn == kFalse ) || ( PlanetScanned == true ))
				{
					fMission.mDisplayMessage( kEvent1ScanPlanet_msg, kScienceScriptMessage );
				}
				ShieldDetected = true;
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					fMission.mSetTeamRelations( kPlayerTeam, kFedPatrolTeam, kTeamHostile, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations( kPlayerTeam, kFedLegendaryTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kFedGuardTeam, kTeamHostile, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations( kPlayerTeam, kFedPlatformTeam, kTeamHostile, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations( kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamHostile, kGoalIgnore, 1.0 );//***

					fMission.mSetTeamRelations( kFedPatrolTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations( kFedPatrolTeam, kFedLegendaryTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedPatrolTeam, kFedGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedPatrolTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedPatrolTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedPatrolTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kFedLegendaryTeam, kFedGuardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedLegendaryTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedLegendaryTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedLegendaryTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kFedGuardTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations( kFedGuardTeam, kFedPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedGuardTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedGuardTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kFedPlatformTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );//***
					fMission.mSetTeamRelations( kFedPlatformTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kFedPlatformTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kPlanetTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kBaseTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0 );//***
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kEvent6Challenge_1_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent6Challenge_2_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent6Challenge_3_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent6Challenge_4_msg, kMissionScriptMessage );

					// set a patrol route for the Fed Ship
					setPatrolRoute( *this, FedLegendaryTeam );

					//FedLegendaryTeam->mGetFirstShip()->mSetFinalDestination( kPlanetX, kPlanetY );
					fMission.mSetTimedInterval( kTimerLimitSet_3, 8 );
					break;

				case kTimerLimitSet_3:
					fMission.mDisplayMessage( kEvent6Challenge_5_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent6Challenge_6_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kEvent6Challenge_7_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_4:
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_5:
					// timer to end the mission, player wins
					return kPlayerWins;
					break;
					
				case kTimerLimitSet_6:
					// timer to end mission, player loses
					return (eGameStatus)( kPlayerLoses | kExpellFromElite );
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;

							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;

							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;

							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								if ( PlanetScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStartingScanMudd_msg, kScienceScriptMessage);
								}
								break;

							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								if ( PlanetScanned == false )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStartingScanMudd_msg, kScienceScriptMessage);
								}
								break;

							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;

							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object or location

							if ( PlanetScanned == false )
							{
								PlanetScanned = true;
								fMission.mDisplayMessage( kPlanetScanned_msg, kScienceScriptMessage );

								if ((fPlayerShip->mGetOfficerRank( kScienceOfficer ))
									>= (PlanetTeam->mGetFirstShip()->mGetOfficerRank( kScienceOfficer )))
								{
									fMission.mDisplayMessage( kEvent2Weakness_msg, kScienceScriptMessage );
									PlanetWeaknessFound = true;
								}
								else
								{
									if ( BaseDestroyed == 0 )
									{
										fMission.mDisplayMessage( kScannersNotPowerfulEnough_msg, kScienceScriptMessage );
										FailedScan = true;
									}
									else
									{
										// if the player doesn't even scan the planet, and goes straight for the base and destroys
										// it, but needed it to complete the mission (because of his science officer rank),
										// make the mission end once they scan the planet and find out they needed the base station
										fMission.mDisplayMessage( kBaseDestroyedLoseMissionScan_msg, kScienceScriptMessage );

										// timer to end the mission
										fMission.mSetTimedInterval( kTimerLimitSet_6, 4 );
									}
								}
							}
							else // PlanetScanned == true
							{
								MuddFound = true;
								PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransHarryMudd, 1 );
								fMission.mDisplayMessage( kEvent4FoundMudd_1_msg, kScienceScriptMessage );
								fMission.mDisplayMessage( kEvent4FoundMudd_3_msg, kScienceScriptMessage );
							}

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					// reset the DeepScanTracker to look for Harry Mudd
					if ( fIsDeepScanOn == kFalse )
					{
						fMission.mDisplayMessage( kReadyToScanForMudd_msg, kScienceScriptMessage );
					}
					DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
					break;

				case kTimerTurnsSet_8:
					// set FedPatrol comm buttons back to HAIL
					for (Ship = FedPatrolTeam->mGetFirstShip(); Ship != NULL; Ship = FedPatrolTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set FedGuard comm buttons back to HAIL
					for (Ship = FedGuardTeam->mGetFirstShip(); Ship != NULL; Ship = FedGuardTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set FedLegendary comm buttons back to HAIL
					for (Ship = FedLegendaryTeam->mGetFirstShip(); Ship != NULL; Ship = FedLegendaryTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship == BaseTeam->mGetFirstShip() )
			{
				if ( PlanetScanned == true )
				{
					if ( PlanetWeaknessFound == true )
					{
						// just plain ol' captured the base
						fMission.mDisplayMessage( kCapturedBase_msg, kScienceScriptMessage );
					}
					else // PlanetWeaknessFound == false
					{
						// capture the base, and will bring down the shield from the planet so the
						// player can scan for Harry Mudd
						PlanetWeaknessFound = true;

						// set PlanetShieldDropped = 1 so the player doesn't get messages about
						// damaging the planet
						PlanetShieldDropped = 1;
						fMission.mDisplayMessage( kCapturedBaseWeaknessFound_msg, kScienceScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
						//DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
					}
				}
				else // PlanetScanned == false
				{
					if ((fPlayerShip->mGetOfficerRank( kScienceOfficer ))
						>= (PlanetTeam->mGetFirstShip()->mGetOfficerRank( kScienceOfficer )))
					{
						fMission.mDisplayMessage( kPlayerShouldScanPlanet_msg, kScienceScriptMessage );
					}
					else
					{
						// capture the base, and will bring down the shield from the planet so the
						// player can scan for Harry Mudd
						PlanetWeaknessFound = true;

						// set PlanetShieldDropped = 1 so the player doesn't get messages about
						// damaging the planet
						PlanetShieldDropped = 1;
						fMission.mDisplayMessage( kCapturedBaseWeaknessFound_msg, kScienceScriptMessage );

						PlanetScanned = true;
						fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
						//DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
					}
				}
			}
			else if ( Ship->mGetTeam() == kFedLegendaryTeam )
			{
				FedLegendaryCaptured = 1;

				fMission.mAddToBonusPrestige( kCaptureBonus );

				if (FedLegendaryTeam->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kEvent7LegendCapture_msg, kMissionScriptMessage );
					// LNF maybe there should be some mention of this in the victory messages?

					// set a timer to end the mission
					fMission.mSetTimedInterval( kTimerLimitSet_5, 4 );
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kFedLegendaryTeam)
			{
				FedLegendaryDestroyed = 1;

				if (FedLegendaryTeam->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kEvent7LegendDefeat_msg, kMissionScriptMessage );
					// LNF maybe there should be some mention of this in the victory messages?

					// set a timer to end the mission
					fMission.mSetTimedInterval( kTimerLimitSet_5, 4 );
				}
			}
			else if ( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				BaseDestroyed = 1;

				// check to see if the player has failed the scan on the planet and is _supposed_ to
				// capture the base, but destroys it instead.  Then there should be a message
				// and if not, then if the player destroys the base after transporting mudd, there should
				// be no message
				if (( FailedScan == true )
					&& ( MuddFound == false ))
				{
					fMission.mDisplayMessage( kBaseDestroyedLoseMission_msg, kScienceScriptMessage );

					// timer to end the mission
					fMission.mSetTimedInterval( kTimerLimitSet_6, 4 );
				}
			}
			else if (Ship->mGetOriginalTeam() == kFedPatrolTeam)
			{
				FedPatrolDestroyed++;
				if (FedPatrolTeam->mGetShipsInPlayCount() == 0)
				{
					if (FedLegendaryTeam->mGetShipsInPlayCount() == 0)
					{
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kFedGuardTeam)
			{
				FedGuardDestroyed++;
				if (FedGuardTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					// LNF add expell from elite?
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetOriginalTeam() == kFedLegendaryTeam)
			{
				FedLegendaryDestroyed++;
				if (FedLegendaryTeam->mGetShipsInPlayCount() == 0)
				{
					if ( FedPatrolTeam->mGetShipsInPlayCount() == 0 )
					{
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kFedPatrolTeam)
			{
				FedPatrolDestroyed++;
				if (FedPatrolTeam->mGetShipsInPlayCount() == 0)
				{
					if (FedLegendaryTeam->mGetShipsInPlayCount() == 0)
					{
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kFedGuardTeam)
			{
				FedGuardDestroyed++;
				if (FedGuardTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFedLegendaryTeam:
								fMission.mDisplayMessage( kLegendHailOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFedPatrolTeam:
								fMission.mDisplayMessage( kFedHailOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kFedGuardTeam:
								fMission.mDisplayMessage( kFedHailOK_msg, kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFedLegendaryTeam:
								if ( Ship->mGetDamagePercent() <= kFedSurrenderDamage )
								{
									fMission.mDisplayMessage( kLegendSurrenderOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kLegendSurrenderDamaged_msg, kMissionScriptMessage );
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFedPatrolTeam:
								if ( Ship->mGetDamagePercent() <= kFedSurrenderDamage )
								{
									fMission.mDisplayMessage( kFedSurrenderOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kFedSurrenderDamaged_msg, kMissionScriptMessage );
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kFedGuardTeam:
								if ( Ship->mGetDamagePercent() <= kFedSurrenderDamage )
								{
									fMission.mDisplayMessage( kFedSurrenderOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kFedSurrenderDamaged_msg, kMissionScriptMessage );
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFedLegendaryTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kLegendTaunt_1_msg, kLegendTaunt_4_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFedPatrolTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kFedTaunt_1_msg, kFedTaunt_4_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kFedGuardTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kFedTaunt_1_msg, kFedTaunt_4_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

						}
						break;
				
				}
			}
			break;

	}

	//return fMission.mGetGameStatus();
	//return kGameInProgress;

	// LNF check other returns
	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = 0;

	if ( Elite == kKingsGuard )
	{
		( MuddRescued == 1 ) ? VictoryPoints += 100 : VictoryPoints += 0;
	}
	else if ( Elite == kEggDefender )
	{
		( MuddRescued == 1 ) ? VictoryPoints += 100 : VictoryPoints += 0;
	}
	else // Elite == kErrant
	{
		( MuddRescued == 1 ) ? VictoryPoints += 80 : VictoryPoints += 0;

		if (( FedLegendaryDestroyed == 1 )
			|| ( FedLegendaryCaptured == 1 ))
		{
			VictoryPoints += 20;
		}
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDefeat;
	}

	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransHarryMudd, 0 );
	}
	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if ( FedLegendaryDestroyed == 1 )
		{
			fMission.mAddToDebriefingList ( kVictoryLegendaryDestroyed_msg );
		}
		else if ( FedLegendaryCaptured == 1 )
		{
			fMission.mAddToDebriefingList ( kVictoryLegendaryCaptured_msg );
		}

		fMission.mAddToDebriefingList ( kDismissedShort_msg );
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( MuddRescued != 0 ) ? fMission.mAddToDebriefingList( kMuddRescuedTrue_msg ) : SuccessCount++;

	if ( Elite == kErrant )
	{
		( FedLegendaryDestroyed != 0 ) ? fMission.mAddToDebriefingList( kFedLegendaryDestroyedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else
	{
		if ( SuccessCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( MuddRescued == 0 ) ? fMission.mAddToDebriefingList( kMuddRescuedFalse_msg ) : FailureCount++;

	if ( Elite == kErrant )
	{
		( FedLegendaryDestroyed == 0 ) ? fMission.mAddToDebriefingList( kFedLegendaryDestroyedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else
	{
		if ( FailureCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships patrol
void setPatrolRoute(tScript& tScript, tTeamInfo* patrolRouteTeam, int32 NumberOfShips)
{
	tTeamInfo* FedPatrolTeam = tScript.fMission.mGetTeamHandle( kFedPatrolTeam );
	tTeamInfo* FedGuardTeam = tScript.fMission.mGetTeamHandle( kFedGuardTeam );
	tTeamInfo* FedLegendaryTeam = tScript.fMission.mGetTeamHandle( kFedLegendaryTeam );

	tMapPoint waypoints_1[3]= // Ship P
	{
		12, 20,
		12, 26,
		12, 11
	};

	tMapPoint waypoints_2[3]= // Ship Q
	{
		21, 10,
		29, 10,
		13, 10
	};

	tMapPoint waypoints_3[3]= // Ship R
	{
		30, 20,
		30, 11,
		30, 26
	};

	tMapPoint waypoints_4[5]= // Ship S
	{
		17, 20,
		17, 15,
		25, 15,
		25, 25,
		17, 25
	};

	tMapPoint waypoints_5[5]= // Ship T
	{
		25, 20,
		25, 25,
		17, 25,
		17, 15,
		25, 15
	};

	tMapPoint waypoints_6[6]= // Legendary Ship
	{
		24, 21,
		22, 16,
		18, 16,
		16, 19,
		17, 23,
		20, 25
	};


	tMapPoint PatrolPoints[10];

	if ( patrolRouteTeam == FedPatrolTeam )
	{
		for ( int32 i = 1; i <= NumberOfShips; i++ )
		{
			if ( i == 1 )
			{
				PatrolPoints[0] = waypoints_1[0];
				PatrolPoints[1] = waypoints_1[1];
				PatrolPoints[2] = waypoints_1[2];
				FedPatrolTeam->mGetThisShip( 1 )->mPatrol( kFedPatrolRadius, 3, &PatrolPoints[0], kHighPriority );
			}
			else if ( i == 2 )
			{
				PatrolPoints[0] = waypoints_2[0];
				PatrolPoints[1] = waypoints_2[1];
				PatrolPoints[2] = waypoints_2[2];
				FedPatrolTeam->mGetThisShip( 2 )->mPatrol( kFedPatrolRadius, 3, &PatrolPoints[0], kHighPriority );
			}
			else
			{
				PatrolPoints[0] = waypoints_3[0];
				PatrolPoints[1] = waypoints_3[1];
				PatrolPoints[2] = waypoints_3[2];
				FedPatrolTeam->mGetThisShip( 3 )->mPatrol( kFedPatrolRadius, 3, &PatrolPoints[0], kHighPriority );
			}
		}
	}
	else if ( patrolRouteTeam == FedGuardTeam )
	{
		PatrolPoints[0] = waypoints_4[0];
		PatrolPoints[1] = waypoints_4[1];
		PatrolPoints[2] = waypoints_4[2];
		PatrolPoints[3] = waypoints_4[3];
		PatrolPoints[4] = waypoints_4[4];
		FedGuardTeam->mGetThisShip( 1 )->mPatrol( kFedGuardRadius, 5, &PatrolPoints[0], kHighPriority );
		PatrolPoints[0] = waypoints_5[0];
		PatrolPoints[1] = waypoints_5[1];
		PatrolPoints[2] = waypoints_5[2];
		PatrolPoints[3] = waypoints_5[3];
		PatrolPoints[4] = waypoints_5[4];
		FedGuardTeam->mGetThisShip( 2 )->mPatrol( kFedGuardRadius, 5, &PatrolPoints[0], kHighPriority );
	}
	else if( patrolRouteTeam == FedLegendaryTeam )
	{
		PatrolPoints[0] = waypoints_6[0];
		PatrolPoints[1] = waypoints_6[1];
		PatrolPoints[2] = waypoints_6[2];
		PatrolPoints[3] = waypoints_6[3];
		PatrolPoints[4] = waypoints_6[4];
		PatrolPoints[4] = waypoints_6[5];
		FedLegendaryTeam->mGetThisShip( 1 )->mPatrol( kFedLegendaryRadius, 6, &PatrolPoints[0], kHighPriority );
	}
}
