#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Gor_9MissionText.h"
#include "Gor_9MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 Mode = 0;
int32 PlayerDestroyed = 0;
int32 FreighterDestroyed = 0;
int32 OrionDestroyed = 0;
int32 LpDestroyed = 0;
int32 RomulanDestroyed = 0;
int32 OrionEnforcerDestroyed = 0;
int32 NumberOfPlayersShips = 0;
int32 AveragePlayerBPV = 0;
int32 LowBPV = 0;
int32 HighBPV = 0;
int32 TotalBPV = 0;
int32 EggsRescued = 0; // win condition
int32 FreighterPositionInFleet = 0; // the ordinal number for the freighter

const int32 kFreighterDestinationX = 18;
const int32 kFreighterDestinationY = 18;
const float kFreighterDamage = 30.0f;
const float kLpWarningDistance = 130.0f;
const float kLpShipDetectionDistance = 100.0f;
const float kLpFreighterDetectionDistance = 50.0f;
const float kPlanetThreat = 100.0f;
const float kPlanetFollow = 100.0f;
const float kPlanetScanDistance = 30.0f;
const int32 kDeepScanTurns = 2;
const int32 kTurnsToDiscovery = 2;
const float kPatrolDetectionRange = 40.0f;
const float kOrionSurrenderDamage = 80.0f;
const int32 kOrionDestroyedBonus = 15;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kEnforcerWaveTurns = 12; // number of turns between enforcer waves

bool GreatFatherMessage = false;
bool OperationGetStuff = false;
bool OrionAlert = false;
bool SetRomulanPatrol = false;
bool AtWarWithRomulan = false;
bool EggsRecovered = false;
bool MarinesReturned = false;
bool PlanetScanned = false;
bool FoundClosestMapEdge = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

tShipInfo* GoalIndicatorShip = NULL; // the ship that the goal indicator is pointing to
tShipInfo* GornEggShip = NULL; // ship that is holding the Gorn Eggs

eVictoryLevels MyVicLevel = kDevastatingDefeat;

void SetPatrolRoute(tScript& tScript, tTeamInfo* PatrolTeam);

enum eWhichShip
{
	kPlayerShip1 = 0,
	kPlayerShip2,
	kPlayerShip3,
	kPlayerFreighter,
	kNotFreighter
};
eWhichShip CurrentPlayerShip = kPlayerShip1;

enum eFreighterState
{
	kFreighterTraitor = 0,
	kFreighterCaptured,
	kFreighterGoingHome
};
eFreighterState FreighterState = kFreighterTraitor;

enum eMissionState
{
	kStart = 0,
	kPlaceLps,
	kOrionEnforcers
};
eMissionState MissionState = kStart;

enum eChallengeState
{
	kNoChallenge = 0,
	kChallenge,
	kWaitForAcknowledge,
	kAcknowledgeSuccessful,
	kAcknowledgeFailure
};
eChallengeState ChallengeState = kNoChallenge;

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kLpTeam, 
	kFreighterTeam,
	kOrionTeam,
	kRomulanTeam,
	kPlanetTeam,
	kOrionEnforcerTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kLpTeam, kOrion );
	fMission.mSetTeamRace( kFreighterTeam, kGorn );
	fMission.mSetTeamRace( kOrionTeam, kOrion );
	fMission.mSetTeamRace( kRomulanTeam, kRomulan );
	fMission.mSetTeamRace( kPlanetTeam, kOrion );
	fMission.mSetTeamRace( kOrionEnforcerTeam, kOrion );

	// Set team relations to show up on Tactical map
	// this mission starts without any enemy ships on screen
	fMission.mSetTeamRelations (kPlayerTeam, kLpTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kRomulanTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionEnforcerTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kLpTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLpTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLpTeam, kRomulanTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLpTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLpTeam, kOrionEnforcerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFreighterTeam, kRomulanTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFreighterTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFreighterTeam, kOrionEnforcerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionTeam, kRomulanTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kOrionTeam, kOrionEnforcerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRomulanTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomulanTeam, kOrionEnforcerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlanetTeam, kOrionEnforcerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
	// LNF maybe change the location of the player's starting position based on faction?
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* OrionEnforcerTeam = fMission.mGetTeamHandle (kOrionEnforcerTeam);

	// Get the total BPV of the player and modify by a low and high percentage

	// balance
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;

		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;

		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	if ( MissionState == kStart )
	{
		// create Player's freighter
		PlayerTeam->mCreateShip( "G-FQS", kStartPosition_J );

		// balance
		NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
		AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;
		LowBPV = PlayerTeam->mGetCombatBPV() * .80;
		HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
		TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

		// create Freighter team
		FreighterTeam->mCreateShip( "G-FQS", kStartPosition_M );

		// create Planet team
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Z );
	}
	else if ( MissionState == kPlaceLps )
	{
		// create Orion team
		OrionTeam->mCreateFleet( 2, (2 * AveragePlayerBPV), kClassLightCruiser, kClassHeavyCruiser, 50, kStartPosition_R );

		// create Romulan team
		RomulanTeam->mCreateShip( AveragePlayerBPV, kClassLightCruiser, kClassHeavyCruiser, 50, kStartPosition_T );

		// create Listening Posts
		LpTeam->mCreateShip( "O-LP", kStartPosition_V );
		LpTeam->mCreateShip( "O-LP", kStartPosition_W );
		LpTeam->mCreateShip( "O-LP", kStartPosition_X );
		LpTeam->mCreateShip( "O-LP", kStartPosition_Y );
	}
	else
	{
		// create Orion Enforcer team
		OrionEnforcerTeam->mCreateShip( "O-BCH1", kStartPosition_Specified, 18, 20 );
		OrionEnforcerTeam->mCreateShip( "O-BCH2", kStartPosition_Specified, 18, 19 );
		OrionEnforcerTeam->mCreateShip( "O-BCH3", kStartPosition_Specified, 22, 20 );
		OrionEnforcerTeam->mCreateShip( "O-BCH2", kStartPosition_Specified, 22, 19 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// there is only one briefing for this mission
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* OrionEnforcerTeam = fMission.mGetTeamHandle (kOrionEnforcerTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyDamage );

		FreighterPositionInFleet++;
	}
	PlayerTeam->mGetThisShip( NumberOfPlayersShips )->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );

	// configure Freighter team
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		// LNF need to put boarding parties?  Maybe a landing party in transporter?
		// LNF also some kind of timer or something so the player can't just hold onto the
		// LNF freighter with a tractor, but has to capture the ship
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mSetShipThrust( 16 );
		Ship->mSetShipMaxSpeed( 16 );
		Ship->mSetFinalDestination( kFreighterDestinationX, kFreighterDestinationY );
	}

	// configure Planet
	//PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// Start a timer to check the distance between player's ships and listening posts
	fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );

	// timer to place Lps
	fMission.mSetTimedInterval( kTimerLimitSet_5, 2 );

	// timer to check for enforcer waves
	fMission.mSetTimedInterval( kTimerTurnsSet_5, kEnforcerWaveTurns );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn

	DeepScanTracker.mSetTarget (PlanetTeam->mGetFirstShip(), kPlanetScanDistance, kDeepScanTurns);

	// DeepScanTracker.mSetLocation (fractureNumber[1].x, fractureNumber[1].y, kDeepScanDistance, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* OrionEnforcerTeam = fMission.mGetTeamHandle (kOrionEnforcerTeam);

	// LNF will need to add OrionEnforcerTeam here
	if ( MissionState == kPlaceLps )
	{
		// configure Romulan team
		if ( Ship->mGetTeam() == kRomulanTeam )
		{
			Ship->mSetCommButton (1, kButtonHail_text);
			Ship->mSetShipCallbackEvents( kNotifyDamage );

			if ( SetRomulanPatrol == false )
			{
				SetRomulanPatrol = true;
				SetPatrolRoute( *this, RomulanTeam);
			}
		}
		// configure Listening Post
		else if ( Ship->mGetTeam() == kLpTeam )
		{
			// LNF some way to make the listening posts detect the player's ships and send a message
			Ship->mSetShipCallbackEvents( kNotifyDamage );
		}
		else if ( Ship->mGetTeam() == kOrionTeam )
		{
			//Ship->mSetCommButton (1, kButtonHail_text);
			Ship->mSetShipCallbackEvents( kNotifyDamage );
			Ship->mDefendEntity( PlanetTeam->mGetFirstShip(), kPlanetThreat, kPlanetFollow, kVeryHighPriority );
		}
	}
	else // MissionState == kOrionEnforcers
	{
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 2 );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 2 );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, 8 );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LpTeam = fMission.mGetTeamHandle (kLpTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle (kRomulanTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* OrionEnforcerTeam = fMission.mGetTeamHandle (kOrionEnforcerTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kRomulanTeam )
					{
						if ( Ship->mGetTeam() == kPlayerTeam )
						{
							AtWarWithRomulan = true;
						}
					}
					else if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kRomulanTeam )
						{
							AtWarWithRomulan = true;
						}
					}
				}
			}
			break;

		case kNotifyChangingShip:
			{
				tShipInfo* NewShip = fMission.mFindShipID( fEventID ); // ship switched to

				if ( NewShip != NULL )
				{
					if ( NewShip->mGetClassType() == kClassFreighter )
					{
						CurrentPlayerShip = kPlayerFreighter;

						// Freighter Comm buttons
						if ( FreighterState == kFreighterTraitor )
						{
							FreighterTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
						}
						else if ( FreighterState == kFreighterCaptured )
						{
							FreighterTeam->mGetFirstShip()->mSetCommButton( 1, kButtonReturnHome_text );
							FreighterTeam->mGetFirstShip()->mSetCommButton( 2, kButtonWaitHere_text );
						}
						else
						{
						}

						// Orion Comm buttons
						for (tShipInfo* Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
						{
							Ship->mSetCommButton( 1, kButtonHail_text );
						}

						// Romulan Comm buttons
						for (Ship = RomulanTeam->mGetFirstShip(); Ship != NULL; Ship = RomulanTeam->mGetNextShip())
						{
							Ship->mSetCommButton( 1, kButtonHail_text );
						}

						// Planet Comm buttons
						if (( ChallengeState > kNoChallenge ) && ( ChallengeState < kAcknowledgeSuccessful ))
						{
							PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonClap_text );
							PlanetTeam->mGetFirstShip()->mSetCommButton( 2, kButtonHuh_text );
						}
						else
						{
							PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
						}

						// Orion Enforcer team
						// no messages, yet

					}
					else
					{
						CurrentPlayerShip = kNotFreighter;

						// Orion Comm buttons
						for (tShipInfo* Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
						{
							Ship->mClearCommButton( 1 );
							Ship->mClearCommButton( 2 );
						}

						// Planet Comm buttons
						PlanetTeam->mGetFirstShip()->mClearCommButton( 1 );
						PlanetTeam->mGetFirstShip()->mClearCommButton( 2 );
					}
				}
			}
			break;

		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if (( fEventItem1 == kTransGornEgg )
						&& ( TransportTarget->mGetOriginalTeam() != kPlayerTeam ))
					{
						Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );
						TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
					}
					else
					{
						GornEggShip = TransportTarget;
					}

					if ( TransportTarget == PlanetTeam->mGetFirstShip() )
					{
						if ( fEventItem1 == kTransAwayTeamItem )
						{
							if ( PlanetScanned == true )
							{
								if ( EggsRecovered == false )
								{
									EggsRecovered = true;
									fMission.mDisplayMessage( kEvent5EggCapture_msg, kMissionScriptMessage );
									PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 1 );

									// countdown to discovery
									fMission.mSetTimedInterval( kTimerTurnsSet_3, kTurnsToDiscovery );
								}
								else
								{
									Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );
									PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
								}
							}
							else
							{
								fMission.mDisplayMessage( kEvent5NoData_msg, kMissionScriptMessage );
								Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 1 );
								PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
							}
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ( TransportTarget == PlanetTeam->mGetFirstShip() )
					{
						if ( fEventItem1 == kTransGornEgg )
						{
							EggsRescued = 1;
							GornEggShip = Ship;
						}
						else if ( fEventItem1 == kTransAwayTeamItem )
						{
							MarinesReturned = true;
						}

						if (( EggsRescued == 1 ) && ( MarinesReturned == true ))
						{
							OperationGetStuff = true;
							fMission.mDisplayMessage( kEvent7Returned_msg, kMissionScriptMessage );

							GoalIndicatorShip = PlayerTeam->mGetFirstShipInPlay();
							fMission.mSetGoalIndicator( GoalIndicatorShip );

							// timer to check if the player gets close to the FirstShipInPlay
							// or if a certain amount of turns goes by
							fMission.mSetTimedInterval( kTimerLimitSet_7, 1 );
							fMission.mSetTimedInterval( kTimerTurnsSet_4, 5 );

							// timer to check when the player gets to the edge of the map
							fMission.mSetTimedInterval( kTimerLimitSet_6, 1 );
						}
					}
					else if( TransportTarget->mGetTeam() == kPlayerTeam )
					{
						if( fEventItem1 == kTransGornEgg )
						{
							GornEggShip = Ship;
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				}
			}
			break;

		case kNotifyAtDestination:
			if ( Ship == FreighterTeam->mGetFirstShip() )
			{
				fMission.mDisplayMessage( kFreighterNotStopped_msg, kMissionScriptMessage );
				return kPlayerLoses;
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kLpTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamHostile, kGoalCapture, 1.0 );
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kRomulanTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionEnforcerTeam, kTeamHostile, kGoalDestroy, 1.0);//***

					fMission.mSetTeamRelations (kLpTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLpTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLpTeam, kRomulanTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLpTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLpTeam, kOrionEnforcerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kFreighterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0 );
					fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFreighterTeam, kRomulanTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFreighterTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFreighterTeam, kOrionEnforcerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionTeam, kRomulanTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kOrionTeam, kOrionEnforcerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kRomulanTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kRomulanTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kRomulanTeam, kOrionEnforcerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlanetTeam, kOrionEnforcerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionEnforcerTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					break;

				case kTimerLimitSet_2:
					// set freighter comm buttons after it's captured or surrendered
					FreighterTeam->mGetFirstShip()->mSetCommButton( 1, kButtonReturnHome_text );
					FreighterTeam->mGetFirstShip()->mSetCommButton( 2, kButtonWaitHere_text );
					break;

				case kTimerLimitSet_3:
					{
						// check distance between LP and player's ships
						for (tShipInfo* iShip = LpTeam->mGetFirstShip(); iShip != NULL; iShip = LpTeam->mGetNextShip())
						{
							for (tShipInfo* jShip = PlayerTeam->mGetFirstShip(); jShip != NULL; jShip = PlayerTeam->mGetNextShip())
							{
								if ( jShip->mGetTacticalDistance( iShip ) <= kLpWarningDistance )
								{
									fMission.mDisplayMessage( kLpWarning_msg, kScienceScriptMessage );

									// set timer for the REAL detection of player
									fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
									return kGameInProgress;
								}
							}
						}

						fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					}
					break;

				case kTimerLimitSet_4:
					{
						// check distance between LP and player's ships
						for (tShipInfo* iShip = LpTeam->mGetFirstShip(); iShip != NULL; iShip = LpTeam->mGetNextShip())
						{
							for (tShipInfo* jShip = PlayerTeam->mGetFirstShip(); jShip != NULL; jShip = PlayerTeam->mGetNextShip())
							{
								if ( jShip->mGetTacticalDistance( iShip ) <= kLpShipDetectionDistance )
								{
									if (( jShip->mGetClassType() == kClassFreighter )
										&& ( jShip->mGetOriginalTeam() == kPlayerTeam ))
									{
										if ( jShip->mGetTacticalDistance( iShip ) <= kLpFreighterDetectionDistance )
										{
											if( FreighterState >= kFreighterCaptured )
											{
												ChallengeState = kChallenge;
												fMission.mDisplayMessage( kChallenge_msg, kMissionScriptMessage );
												PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonClap_text, kTrue );
												PlanetTeam->mGetFirstShip()->mSetCommButton( 2, kButtonHuh_text );
												fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
												return kGameInProgress;
											}
											else
											{
												fMission.mDisplayMessage( kEvent2Scanned_1_msg, kScienceScriptMessage );
												fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
												MissionState = kOrionEnforcers;
												mCreateMissionShips( kOrionEnforcers );
												return kGameInProgress;
											}
										}
									}
									else
									{
										fMission.mDisplayMessage( kEvent2Scanned_1_msg, kScienceScriptMessage );
										fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
										MissionState = kOrionEnforcers;
										mCreateMissionShips( kOrionEnforcers );
										return kGameInProgress;
									}
								}
							}
						}

						fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
					}
					break;

				case kTimerLimitSet_5:
					MissionState = kPlaceLps;
					mCreateMissionShips( kPlaceLps );
					break;

				case kTimerLimitSet_6:
					{
						// test if the player is near the edge of the map
						int32 ShipLocX = 0;
						int32 ShipLocY = 0;

						for (tShipInfo* pShip = PlayerTeam->mGetFirstShip(); pShip != NULL; pShip = PlayerTeam->mGetNextShip())
						{
							pShip->mGetGridPosition( &ShipLocX, &ShipLocY );

							if( ( ShipLocX <= 3 )
								|| ( ShipLocX >= 38 )
								|| ( ShipLocY <= 3 ) 
								|| ( ShipLocY >= 38 ) )
							{
								if( ( GornEggShip->mTestShipStatus( kShipCaptured ) == kFalse )
									&& ( GornEggShip->mTestShipStatus( kShipDestroyed ) == kFalse )
									&& ( GreatFatherMessage == false ) )
								{
									if( pShip == GornEggShip )
									{
										GreatFatherMessage = true;

										// remove eggs from player's inventory
										for (tShipInfo* tempShip = PlayerTeam->mGetFirstShip(); tempShip != NULL; tempShip = PlayerTeam->mGetNextShip())
										{
											tempShip->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
										}

										fMission.mDisplayMessage( kEvent13GreatFather_1_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent13GreatFather_2_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent13GreatFather_3_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent13GreatFather_4_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent13GreatFather_5_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent13GreatFather_6_msg, kMissionScriptMessage );
									}
								}
/*
								if (( pShip->mAccessSupplies( kGetTransporterItemCount, kTransGornEgg ) == 1 )
									&& ( GreatFatherMessage == false ))
								{
									GreatFatherMessage = true;

									// remove eggs from player's inventory
									for (tShipInfo* tempShip = PlayerTeam->mGetFirstShip(); tempShip != NULL; tempShip = PlayerTeam->mGetNextShip())
									{
										tempShip->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
									}

									fMission.mDisplayMessage( kEvent13GreatFather_1_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kEvent13GreatFather_2_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kEvent13GreatFather_3_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kEvent13GreatFather_4_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kEvent13GreatFather_5_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kEvent13GreatFather_6_msg, kMissionScriptMessage );
								}
*/
							}
							else
							{
								fMission.mSetTimedInterval( kTimerLimitSet_6, 1 );
							}
						}

						fMission.mSetTimedInterval( kTimerLimitSet_6, 1 );
					}
					break;
					
				case kTimerLimitSet_7:
					// checks if the player is close to the FirstShipInPlay and points goal indicator to map edge
					if ( PlayerTeam->mGetThisShip( FreighterPositionInFleet )->mGetTacticalDistance( GoalIndicatorShip ) <= 80.0f )
					{
						if ( FoundClosestMapEdge == false )
						{
							FoundClosestMapEdge = true;
							FindClosestMapEdge( *this, PlayerTeam->mGetThisShip( FreighterPositionInFleet ), kMapEdgeX, kMapEdgeY );
						}
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_7, 1 );
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;

							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;

							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on

								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;

							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off

								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off

								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;

							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

						if (flag == kInRange_ScanOn)
						{
							if ( ChallengeState != kAcknowledgeFailure )
							{
								// player has successfully scanned the object or location
								fMission.mDisplayMessage( kEvent3ScanPlanet_msg, kScienceScriptMessage );
								PlanetScanned = true;
							}

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					if ( ChallengeState == kChallenge )
					{
						fMission.mDisplayMessage( kAcknowledgeChallenge_msg, kMissionScriptMessage, PlayerTeam->mGetThisShip( NumberOfPlayersShips ));
						ChallengeState = kWaitForAcknowledge;
						fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
					}
					break;

				case kTimerTurnsSet_2:
					// final challenge warning
					if ( ChallengeState == kAcknowledgeSuccessful )
					{
						break;
					}
					else
					{
						// challenge failed, send out enforcers
						ChallengeState = kAcknowledgeFailure;

						fMission.mDisplayMessage( kFailedAcknowledge_msg, kMissionScriptMessage );
						MissionState = kOrionEnforcers;
						mCreateMissionShips( kOrionEnforcers );
					}
					break;

				case kTimerTurnsSet_3:
					// Eggs discovered missing

					if ( OperationGetStuff == false )
					{
						fMission.mDisplayMessage( kEvent6Discovered_msg, kMissionScriptMessage );
					}

					fMission.mDisplayMessage( kHereTheyCome_msg, kScienceScriptMessage );
					OrionAlert = true;
					OrionTeam->mClearCommands();
					fMission.mSetTeamRelations( kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

					MissionState = kOrionEnforcers;
					mCreateMissionShips( kOrionEnforcers );
					break;

				case kTimerTurnsSet_4:
					// points goal indicator to nearest map edge
					if ( FoundClosestMapEdge == false )
					{
						FoundClosestMapEdge = true;
						FindClosestMapEdge( *this, PlayerTeam->mGetThisShip( FreighterPositionInFleet ), kMapEdgeX, kMapEdgeY );
					}
					break;

				case kTimerTurnsSet_5:
					// checks to see if MissionState == kOrionEnforcers;
					// and generates another wave every kEnforcerWaveTurns turns
					if( MissionState == kOrionEnforcers )
					{
						mCreateMissionShips( kOrionEnforcers );
					}

					fMission.mSetTimedInterval( kTimerTurnsSet_5, kEnforcerWaveTurns );

					break;

				case kTimerTurnsSet_6:
					break;

				case kTimerTurnsSet_7:
					// set Planet comm buttons back to HAIL
					for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8:
					// set Romulan comm buttons back to HAIL
					for (Ship = RomulanTeam->mGetFirstShip(); Ship != NULL; Ship = RomulanTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Orion comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Freighter comm buttons back to HAIL
					for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyOnCapture:
			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship == FreighterTeam->mGetFirstShip() )
			{
				fMission.mDisplayMessage( kEvent1CaptureFreighter_msg, kMissionScriptMessage );
				FreighterState = kFreighterCaptured;

				Ship->mClearCommands();
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetShipThrust( 0 );

				fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				// use timer to set Freighter comm buttons
				fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
			}
			else if (( Ship->mGetTeam() == kOrionTeam ) || ( Ship->mGetTeam() == kOrionEnforcerTeam ))
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
				if (FreighterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kLpTeam)
			{
				LpDestroyed++;
				if (LpTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				OrionDestroyed++;
				fMission.mAddToBonusPrestige( kOrionDestroyedBonus );

				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kRomulanTeam)
			{
				RomulanDestroyed++;
				if (RomulanTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kOrionEnforcerTeam)
			{
				OrionEnforcerDestroyed++;
				fMission.mAddToBonusPrestige( kOrionDestroyedBonus );

				if (OrionEnforcerTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			// if the ship that disengages has the eggs, the player wins, else the player loses.

			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( Ship != GornEggShip )
				{
					EggsRescued = 0;
				}

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mSetGoalIndicator( kFalse, 0, 0 );
				}
				else
				{
					return kGameInProgress;
				}
			}

			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonClap_text:
						Ship->mClearCommButton(2);
						ChallengeState = kAcknowledgeSuccessful;
						fMission.mDisplayMessage( kClearForApproach_msg, kMissionScriptMessage, PlayerTeam->mGetThisShip( NumberOfPlayersShips ));
						break;

					case kButtonHuh_text:
						if ( ChallengeState == kChallenge )
						{
							Ship->mSetCommButton( 2, kButtonHuh_text );
							fMission.mDisplayMessage( kAcknowledgeChallenge_msg, kMissionScriptMessage, PlayerTeam->mGetThisShip( NumberOfPlayersShips ));
							ChallengeState = kWaitForAcknowledge;
							fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
						}
						else
						{
							Ship->mClearCommButton(1);
							fMission.mSetTimedInterval( kTimerTurnsSet_2, 0 );
						}
						break;

					case kButtonReturnHome_text:
						fMission.mDisplayMessage( kFreighterDepartingForHome_msg, kMissionScriptMessage );
						FreighterState = kFreighterGoingHome;

						Ship->mClearCommButton(2);
						Ship->mSetShipAIControlled( kTrue );
						Ship->mSetShipMaxSpeed(31);
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						break;

					case kButtonWaitHere_text:
						fMission.mDisplayMessage( kAcknowledged_msg, kMissionScriptMessage );
						Ship->mClearCommButton(1);
						break;

					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								if ( FreighterState == kFreighterTraitor )
								{
									fMission.mDisplayMessage( kFreighterHailOK_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonTaunt_text);
									Ship->mSetCommButton (2, kButtonSurrender_text);
									// If the Taunt or Surrender buttons aren't used in X turns
									// return the buttons to Hail and blank.
									fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								}
								else if ( FreighterState == kFreighterCaptured )
								{
									fMission.mDisplayMessage( kBoarded_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else
								{
									fMission.mDisplayMessage( kFreighterDepartingForHome_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonHail_text);
								}

								break;

							case kOrionTeam:
								if ( OrionAlert == false )
								{
									fMission.mDisplayMessage( kOrionHailFreighterOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kOrionHailOK_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kRomulanTeam:
								if ( AtWarWithRomulan == false )
								{
									fMission.mDisplayMessage( kRomHailNic_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kRomHailCombat_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kPlanetTeam:
								if ( OrionAlert == false )
								{
									fMission.mDisplayMessage( kPlanetHailFreighterOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kPlanetHailFreighterExposed_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								if ( FreighterState == kFreighterTraitor )
								{
									if ( FreighterTeam->mGetFirstShip()->mGetDamagePercent() >= kFreighterDamage )
									{
										fMission.mDisplayMessage( kFreighterSurrender_msg, kMissionScriptMessage );

										Ship->mSetShipAIControlled( kFalse );
										Ship->mSetShipThrust( 0 );
										
										fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

										// set the comm buttons to return home or wait here
										fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
									}
									else
									{
										fMission.mDisplayMessage( fMission.mRandomInt32( kFreighterTaunt_1_msg, kFreighterTaunt_4_msg ), kMissionScriptMessage );
										Ship->mSetCommButton (2, kButtonSurrender_text);
										fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
									}
								}
								else if ( FreighterState == kFreighterCaptured )
								{
									fMission.mDisplayMessage( kBoarded_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else
								{
									fMission.mDisplayMessage( kFreighterDepartingForHome_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonHail_text);
								}

								break;

							case kOrionTeam:
								if ( OrionAlert == false )
								{
									fMission.mDisplayMessage( kOrionSurrenderFreighterOK_msg, kMissionScriptMessage );
								}
								else
								{
									if ( Ship->mGetDamagePercent() >= kOrionSurrenderDamage )
									{
										fMission.mDisplayMessage( kOrionSurrenderOK_msg, kMissionScriptMessage );
									}
									else
									{
										// LNF any penalty for shooting at/destroying a ship that has surrendered?
										fMission.mDisplayMessage( kOrionSurrenderDamaged_msg, kMissionScriptMessage );
										Ship->mDisengage( kByMapEdge, kAbsolutePriority );
									}
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kRomulanTeam:
								if ( AtWarWithRomulan == false )
								{
									fMission.mDisplayMessage( kRomSurrenderFriendly_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kRomSurrenderOK_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kPlanetTeam:
								if ( OrionAlert == false )
								{
									fMission.mDisplayMessage( kPlanetSurrenderFreighterOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kPlanetSurrenderFreighterExposed_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								if ( FreighterState == kFreighterTraitor )
								{
									fMission.mDisplayMessage( fMission.mRandomInt32( kFreighterTaunt_1_msg, kFreighterTaunt_4_msg ), kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonTaunt_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								}
								else if ( FreighterState == kFreighterCaptured )
								{
									fMission.mDisplayMessage( kBoarded_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else
								{
									fMission.mDisplayMessage( kFreighterDepartingForHome_msg, kMissionScriptMessage );
									Ship->mSetCommButton (1, kButtonHail_text);
								}

								break;

							case kOrionTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kOrionTaunt_1_msg, kOrionTaunt_4_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kRomulanTeam:
								fMission.mDisplayMessage( fMission.mRandomInt32( kRomTaunt_1_msg, kRomTaunt_4_msg ), kMissionScriptMessage );

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kPlanetTeam:
								if ( OrionAlert == false )
								{
									fMission.mDisplayMessage( kPlanetSurrenderFreighterOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kPlanetSurrenderFreighterExposed_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = 0;

	( EggsRescued == 1 ) ? VictoryPoints += 100 : VictoryPoints += 0;

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransGornEgg, 0 );
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransAwayTeamItem, 0 );
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	MyVicLevel = VictoryLevel;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// fMission.mAddToDebriefingList (kDismissedShort_msg);


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( EggsRescued != 0 ) ? fMission.mAddToDebriefingList( kEggsRescuedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( EggsRescued == 0 ) ? fMission.mAddToDebriefingList( kEggsRescuedFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_4;
	}
	else
	{
		return 0;
	}
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void SetPatrolRoute(tScript& tScript, tTeamInfo* PatrolTeam)
{
	tTeamInfo* RomulanTeam = tScript.fMission.mGetTeamHandle (kRomulanTeam);
	tShipInfo* Ship;

	tMapPoint waypoints_1[2]=
	{
		6, 4,
		34, 4,
	};

	tMapPoint waypoints_2[2]=
	{
		10, 14,
		17, 14
	};

	tMapPoint PatrolPoints[10];

	if ( PatrolTeam == RomulanTeam )
	{
		for (Ship = PatrolTeam->mGetFirstShip(); Ship != NULL; Ship = PatrolTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_1[0];
			PatrolPoints[1] = waypoints_1[1];
			Ship->mPatrol( kPatrolDetectionRange, 2, &PatrolPoints[0], kVeryHighPriority );
		}
	}
}
