#include "scriptinterface.h"

#include "His_AsteroidText.h"
#include "His_AsteroidMaps.h"

int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 PlanetX = 0;
int32 PlanetY = 0;
bool AsteroidOnCourse = true;
bool PlayerDisengaged = false;
bool AsteroidDestroyed = false;
bool FiredAtEnemy = false;
bool EnemyDisengaging = false;
bool TractoringAsteroid = false;
bool AsteroidImpacted = false;
bool DeepScanned = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kAsteroidTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// set enemy race
	if( info.fPlayerRace == kFederation )
	{
		info.fEnemyRace = kKlingon;
	}
	else if( info.fPlayerRace == kKlingon )
	{
		info.fEnemyRace = kHydran;
	}
	else if( info.fPlayerRace == kRomulan )
	{
		info.fEnemyRace = kGorn;
	}
	else if( info.fPlayerRace == kGorn )
	{
		info.fEnemyRace = kRomulan;
	}
	else if( info.fPlayerRace == kLyran )
	{
		info.fEnemyRace = kHydran;
	}
	else if( info.fPlayerRace == kHydran )
	{
		info.fEnemyRace = kLyran;
	}
	else
	{
		info.fEnemyRace = kKlingon;
	}

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kAsteroidTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAsteroidTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	
	// pick a random map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );

	int32 DiffLevel = fMission.mGetDifficultyLevel();

	switch( DiffLevel )
	{
		case 0:
			PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 100, kStartPosition_Z );
			break;

		case 1:
			PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
			EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 100, kStartPosition_Z );
			break;

		case 2:
			PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
			EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 100, kStartPosition_Z );
			break;
	}

	// create the Asteroid!
	AsteroidTeam->mCreateShip( "N-AST", kStartPosition_M );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	
	fMission.mGetGridPosition( '3', &PlanetX, &PlanetY );

	// configure player ships
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 5 );
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor | kNotifyEndTractor );
	}

	// configure the asteroid
	Ship = AsteroidTeam->mGetFirstShip();
	Ship->mSetShipThrust( 6 );
	Ship->mSetShipMaxSpeed( 6 );
	Ship->mChangeName( kAsteroid_name );
	Ship->mSetFinalDestination( PlanetX, PlanetY );
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mSetShipCallbackEvents( kNotifyAtDestination );

	// configure Enemy ship
	Ship = EnemyTeam->mGetFirstShip();
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mSetShipCallbackEvents( kNotifyDamage | kNotifyBeginTractor | kNotifyEndTractor );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	fMission.mSetTimedInterval( kTimerLimitSet_2, 60 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );

	tShipInfo* PlayerTarget = fMission.mGetShipTarget( fPlayerShip );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kAsteroidTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					fMission.mSetTimedInterval( kTimerLimitSet_9, 3 );

					Ship = AsteroidTeam->mGetFirstShipInPlay();
					Ship->mSetShipAIControlled( kFalse );

					Ship = EnemyTeam->mGetFirstShipInPlay();
					Ship->mDefendEntity( AsteroidTeam->mGetFirstShipInPlay(), 10, 10, kAbsolutePriority );
					break;

				case kTimerLimitSet_2:	// check to see if the enemy ship needs to correct the asteroid
					if( EnemyTeam->mGetShipsInPlayCount() > 0 )
					{
						if( EnemyDisengaging == false )
						{
							if( AsteroidOnCourse == true )
							{
								fMission.mSetTimedInterval( kTimerLimitSet_2, 6 );
							}
							else
							{
								Ship = EnemyTeam->mGetFirstShipInPlay();
								tShipInfo* Asteroid = AsteroidTeam->mGetFirstShipInPlay();

								if(( Ship != NULL ) && ( Asteroid != NULL ))
								{
									Ship->mTractorEntity( Asteroid, PlanetX, PlanetY, kAbsolutePriority );
								}
							}
						}
					}
					break;

				case kTimerLimitSet_9:
					fMission.mDisplayMessage( kOutOfBelt_msg, kScienceScriptMessage );
					Ship = AsteroidTeam->mGetFirstShipInPlay();
					fMission.mSetGoalIndicator( Ship );
					fMission.mSetTimedInterval( kTimerTurnsSet_5, 1 );
					break;

				case kTimerTurnsSet:
					if( FiredAtEnemy == false )
					{
						Ship = EnemyTeam->mGetFirstShipInPlay();

						if( Ship != NULL )
						{
							fMission.mDisplayMessage( kOnScienceMission_msg, kFleetScriptMessage, Ship );
						}
					}
					break;

				case kTimerTurnsSet_2:
					if( TractoringAsteroid == true )
					{
						Ship = AsteroidTeam->mGetFirstShipInPlay();

						if( Ship != NULL )
						{
							fMission.mDisplayMessage( kMovedOffCourse_msg, kScienceScriptMessage );
							AsteroidOnCourse = false;

							Ship->mClearCommands();
							Ship->mSetShipAIControlled( kTrue );
							Ship->mSetFinalDestination( PlanetX + 4, PlanetY + 4 );
							Ship->mSetShipAIControlled( kFalse );

							if( EnemyTeam->mGetShipsInPlayCount() == 0 )
							{
								fMission.mDisplayMessage( kPlanetSaved_msg, kScienceScriptMessage );
								return kPlayerWins;
							}

							fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
						}
					}
					break;

				case kTimerTurnsSet_3:
					if( TractoringAsteroid == true )
					{
						fMission.mDisplayMessage( kTractorOverload_msg, kEngineerScriptMessage );
						fPlayerShip->mDamageSystem( kDamageTractorGroupSystem, 100.0 );
					}
					break;

				case kTimerTurnsSet_4:
					if(( EnemyTeam->mGetShipsInPlayCount() > 0 ) && ( AsteroidOnCourse == false ))
					{
						if( AsteroidTeam->mGetShipsInPlayCount() > 0 )
						{
							AsteroidOnCourse = true;
							Ship = AsteroidTeam->mGetFirstShipInPlay();
							Ship->mClearCommands();
							Ship->mSetShipAIControlled( kTrue );
							Ship->mSetFinalDestination( PlanetX, PlanetY );
							Ship->mSetShipAIControlled( kFalse );
							EnemyTeam->mGetFirstShipInPlay()->mClearCommands();
							fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
						}
					}
					break;

				case kTimerTurnsSet_5:
					fMission.mDisplayMessage( kScienceSuggestion_msg, kScienceScriptMessage );
					break;
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = true;

				if(( EnemyTeam->mGetShipsInPlayCount() > 0 ) && ( AsteroidDestroyed == false ))
				{
					AsteroidImpacted = true;
					return kPlayerLoses;
				}
				else if(( EnemyTeam->mGetShipsInPlayCount() == 0 ) && ( AsteroidOnCourse == true ))
				{
					AsteroidImpacted = true;
					return kPlayerLoses;
				}
				else
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetTeam() == kAsteroidTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerDisengaged = true;

			if(( EnemyTeam->mGetShipsInPlayCount() > 0 ) && ( AsteroidDestroyed == false ))
			{
				AsteroidImpacted = true;
				return kPlayerLoses;
			}
			else if(( EnemyTeam->mGetShipsInPlayCount() == 0 ) && ( AsteroidOnCourse == true ))
			{
				AsteroidImpacted = true;
				return kPlayerLoses;
			}
			else
			{
				return kPlayerWins;
			}
			break;

		case kNotifyAtDestination:	// asteroid goes boom!
			Ship = AsteroidTeam->mGetFirstShipInPlay();

			if( Ship != NULL )
			{
				if( Ship->mGetTacticalDistance( PlanetX, PlanetY ) < 10 )
				{
					fMission.mDisplayMessage( kAsteroidImpact_msg );
					AsteroidImpacted = true;
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			fMission.mAddToBonusPrestige( kCaptureBonus );
			EnemyCaptured++;
				
			if( AsteroidOnCourse == false )
			{
				fMission.mDisplayMessage( kPlanetSaved_msg, kScienceScriptMessage );
				return kPlayerWins;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				if( AsteroidOnCourse == false )
				{
					fMission.mDisplayMessage( kPlanetSaved_msg, kScienceScriptMessage );
					return kPlayerWins;
				}
			}

			if( Ship->mGetOriginalTeam() == kAsteroidTeam )
			{
				AsteroidDestroyed = true;
				AsteroidOnCourse = false;

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mDisplayMessage( kPlanetSaved_msg, kScienceScriptMessage );
					return kPlayerWins;
				}
				else
				{
					fMission.mDisplayMessage( kEnemyAttackWarning_msg, kScienceScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
				}
			}

			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				PlayerDestroyed++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					if(( AsteroidOnCourse == true ) && ( EnemyTeam->mGetShipsInPlayCount() > 0 ))
					{
						AsteroidImpacted = true;
					}

					return kPlayerLoses;
				}
			}
			break;

		case kNotifyBeginTractor:
			if(( PlayerTarget->mGetTeam() == kAsteroidTeam ) && ( Ship == fPlayerShip ))
			{
				TractoringAsteroid = true;
				fMission.mDisplayMessage( kTakesTime_msg, kScienceScriptMessage );
				fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );	// asteroid off course in 1 turn
			}

			if(( PlayerTarget->mGetTeam() == kEnemyTeam ) && ( Ship == fPlayerShip ))
			{
				fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			}

			if( Ship->mGetTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( kEnemyTractoring_msg, kScienceScriptMessage, Ship );
				fMission.mSetTimedInterval( kTimerTurnsSet_4, 1 );	// asteroid on course in 1 turn
			}

			Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
			break;

		case kNotifyEndTractor:
			TractoringAsteroid = false;

			if( AsteroidOnCourse == true )
			{
				fMission.mDisplayMessage( kAsteroidWillHit_msg, kScienceScriptMessage );
			}

			Ship->mSetShipCallbackEvents( kNotifyEndTractor );
			break;

		case kNotifyCommButtonPress:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( FiredAtEnemy == true )
				{
					fMission.mDisplayMessage( kStatic_msg, kFleetScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kNoReally_msg, kFleetScriptMessage, Ship );
				}
			}

			if( Ship->mGetTeam() == kAsteroidTeam )
			{
				fMission.mDisplayMessage( kItsARockSir_msg, kFleetScriptMessage );
			}
			break;

		case kNotifyDamage:
			if( FiredAtEnemy == false )
			{
				FiredAtEnemy = true;
			}
			else
			{
				Ship = EnemyTeam->mGetFirstShipInPlay();

				if( Ship != NULL )
				{
					if( Ship->mGetDamagePercent() > 60.0 )
					{
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						EnemyDisengaging = true;
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		case kNotifyDeepScan:
			if( DeepScanned == false )
			{
				DeepScanned = true;
				
				if( EnemyTeam->mGetShipsInPlayCount() > 0 )
				{
					fMission.mDisplayMessage( kDeepScan_msg, kScienceScriptMessage );
				}
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle( kAsteroidTeam );

	float PlayerDamage = PlayerTeam->mGetTotalDamagePercent();

	if( AsteroidImpacted == true )
	{
		return kDevastatingDefeat;
	}

	if( EnemyCaptured > 0 )
	{
		return kAstoundingVictory;
	}

	if( PlayerDamage > 80.0 )
	{
		return kDefeat;
	}

	if( PlayerDamage > 50.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( PlayerDestroyed > 0 )
	{
		fMission.mAddToDebriefingList( kPlayerDestroyed_msg );
	}

	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	fMission.mAddToDebriefingList( kYouScored_msg );
	fMission.mAddToDebriefingList( VictoryLevel + kVicLevelAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
