#include "scriptinterface.h"

#include "His_BlackHoleManiaText.h"
#include "His_BlackHoleManiaMaps.h"

void mSetDefaultMenuItems (tDynaverseScriptInfo& info);

int32 MapSize = 0;
int32 PrimaryEnemyTeam;
int32 EnemyDestroyed = 1;
int32 EnemyCaptured = 0;
int32 TotalBlackHoles = 0;
int32 DiffLevel = 0;
bool NewBlackHole = false;
bool PlayerDisengaged = false;
bool StartingEnemies = false;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return PrimaryEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kEnableSupplyDock;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// check the first AI teams ship count - if its 0 then we need to fill in the default values before proceeding
	if (info.fMemberShipCounts[kTeam2] == 0)
	{
		mSetDefaultMenuItems (info);
	}

	// must setup the races and relations of the reserved teams in the script
	fMission.mSetSkirmishTeams (info);

	if (fMission.fTotalEnemyCPUTeams)
	{
		tTeamInfo* Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, 0);
		PrimaryEnemyTeam = Team->mGetTeam();
	}
	else
	{
		PrimaryEnemyTeam = kPlayerTeam;
	}

	MapSize = 40;

	if( info.fPreferedMap == 1 )
	{
		MapSize = 25;
	}

	if( info.fPreferedMap == 2 )
	{
		MapSize = 15;
	}

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = kStartPosition_G | kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tShipInfo* Ship;
	tTeamInfo* Team;

	int32 position = kStartPosition_H;
	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	DiffLevel = fMission.mGetDifficultyLevel();

	if( Mode == 0 )
	{
		// create friendly AI ships first
		for (int32 i = 0; i < fMission.fTotalAlliedCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_AlliedTeams, i);
			Team->mCreateCPUFleet (20, position | kGroupAround);
			position++;
		}

		position = kStartPosition_U;

		// create enemy AI ships last
		for (i = 0; i < fMission.fTotalEnemyCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, i);
			Team->mCreateCPUFleet (20, position | kGroupAround);
			position++;
		}
	}

	// Let's make a new black hole!  Yea!
	if( Mode == 1 )
	{
		NewBlackHole = true;
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&OldX, &OldY);
		NewX = OldX + fMission.mRandomInt32( -3, 3 );
		NewY = OldY + fMission.mRandomInt32( -3, 3 );

		if(( NewX <= 1 ) || ( NewX >= MapSize ))
		{
			NewBlackHole = false;
		}

		if(( NewY <= 1 ) || ( NewY >= MapSize ))
		{
			NewBlackHole = false;
		}

		if(( NewX == OldX ) && ( NewY == OldY ))
		{
			NewBlackHole = false;
		}
		
		if( NewBlackHole == true )
		{
			fMission.mCreateMapObject( 'B', NewX, NewY );
		}
	}

	if( fMission.fTotalEnemyCPUTeams > 0 )
	{
		StartingEnemies = true;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 RandomTurns = 0;

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// Start the initial timer
	RandomTurns = fMission.mRandomInt32( 1, 2 );
	fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* Team;

	float AvgPlayerDamage = PlayerTeam->mGetTotalDamagePercent();
	float AvgEnemyDamage = 0;
	int32 i = 0;

	for( i = 0; i < fMission.fTotalAlliedCPUTeams; i++ )
	{
		Team = fMission.mGetTeamHandle( kCPU_AlliedTeams, i );
		AvgPlayerDamage = AvgPlayerDamage + Team->mGetTotalDamagePercent();
	}

	AvgPlayerDamage = AvgPlayerDamage / ( 1 + fMission.fTotalAlliedCPUTeams );

	for( i = 0; i < fMission.fTotalEnemyCPUTeams; i++ )
	{
		Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, i);
		AvgEnemyDamage = AvgEnemyDamage + Team->mGetTotalDamagePercent();
	}

	AvgEnemyDamage = AvgEnemyDamage / fMission.fTotalEnemyCPUTeams;

	PlayerHealth = 100.0f - AvgPlayerDamage;
	EnemyHealth = 100.0f - AvgEnemyDamage;

	int32 RandomTurns = 0;
	int32 RandomMessage = 0;

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			if( TotalBlackHoles == 0 )
			{
				fMission.mDisplayMessage( kScienceStarting_msg, kScienceScriptMessage );
				RandomTurns = fMission.mRandomInt32( 2, ( 6 - DiffLevel ));
				fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
				TotalBlackHoles++;
			}
			else if( TotalBlackHoles < 8 )
			{
				mCreateMissionShips( 1 );

				if( NewBlackHole == true )
				{
					RandomMessage = fMission.mRandomInt32( kNewSingularity1_msg, kNewSingularity3_msg );
					fMission.mDisplayMessage( RandomMessage, kScienceScriptMessage );
					TotalBlackHoles++;
				}

				RandomTurns = fMission.mRandomInt32( 2, ( 6 - DiffLevel ));
				fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}
			else
			{
				if( StartingEnemies == true )
				{
					bool EnemyAlive = false;

					for( i = 0; i < fMission.fTotalEnemyCPUTeams; i++ )
					{
						Team = fMission.mGetTeamHandle( kCPU_EnemyTeams, i );

						if( Team->mGetShipsInPlayCount() > 0 )
						{
							EnemyAlive = true;
						}

						if( EnemyAlive == false )
						{
							return kPlayerWins;
						}
					}
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}
			else
			{
				EnemyCaptured++;

				if( StartingEnemies == true )
				{
					bool EnemyAlive = false;

					for( i = 0; i < fMission.fTotalEnemyCPUTeams; i++ )
					{
						Team = fMission.mGetTeamHandle( kCPU_EnemyTeams, i );

						if( Team->mGetShipsInPlayCount() > 0 )
						{
							EnemyAlive = true;
						}

						if( EnemyAlive == false )
						{
							return kPlayerWins;
						}
					}
				}
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					PlayerDisengaged = true;
					return kPlayerWins;
				}
			}
			break;
	}
	
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if( PlayerTeam->mGetShipsInPlayCount() == 0 )
	{
		return kDevastatingDefeat;
	}

	if( PlayerDisengaged == true )
	{
		if( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if( PlayerHealth < ( EnemyHealth / 2 ))
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}


	if(( PlayerHealth > 75.0 ) && ( EnemyCaptured > 0 ))
	{
		return kAstoundingVictory;
	}

	if( PlayerHealth > 90.0 )
	{
		return kAstoundingVictory;
	}

	if( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( kYouScored_msg );
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
//
void mSetDefaultMenuItems (tDynaverseScriptInfo& info)
{
	// default player settings:
	info.fMemberShipCounts[kTeam1]	= 1;				// must be 1-3
	info.fMemberBPVLimits[kTeam1]	= 400;
	info.fMemberSmallHulls[kTeam1]	= kClassLightCruiser;
	info.fMemberLargeHulls[kTeam1]	= kClassLightCruiser;

	// default AI settings:
	//	ships:		- 1
	//	small hull:	- kClassFrigate
	//	large hull:	- kClassHeavyCruiser
	//	BPV:		- 400
	//
	//	team 2:		- Klingon (or Romulan if player is Klingon)
	//	teams 3-6:	- disabled

	for (int32 i = kTeam2; i <= kTeam6; i++)
	{
		info.fMembersInGame[i]		= kPlayerTypeNoOne;
		info.fMemberTeams[i]		= eTeamID(i);
		info.fMemberRaces[i]		= kNoRaceType;
		info.fMemberShipCounts[i]	= 1;				// must be 1-3
		info.fMemberBPVLimits[i]	= 400;
		info.fMemberSmallHulls[i]	= kClassFrigate;
		info.fMemberLargeHulls[i]	= kClassHeavyCruiser;
	}
	info.fMembersInGame[kTeam2]		= kPlayerTypeAI;
	info.fMemberRaces[kTeam2]		= (info.fPlayerRace != kKlingon) ? kKlingon : kRomulan;
}
