#include "scriptinterface.h"

#include "His_MightyHoodText.h"
#include "His_MightyHoodMaps.h"

bool HoodLeaving = false;
bool HoodAttack = false;
bool HoodCaptured = false;
int32 MissionChapter = 0;
int32 ChallengerID = 0;
int32 BloodshedderID = 0;
int32 DesolationID = 0;
int32 VictoryPoints = 2;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kFederation );

	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	PlayerTeam->mCreateShip( "K-D7B", kStartPosition_G );
	PlayerTeam->mCreateShip( "K-D6B", kStartPosition_H );
	PlayerTeam->mCreateShip( "K-D6B", kStartPosition_I );

	EnemyTeam->mCreateShip( "F-CA+", kStartPosition_Z );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	Ship = PlayerTeam->mGetThisShip( 1 );
	Ship->mChangeName( kChallenger_name );
	Ship->mSetShipPlayerControlled();
	Ship->mSetCommButton( 1, kButtonAttack_text );
	ChallengerID = Ship->mGetShipID();

	Ship = PlayerTeam->mGetThisShip( 2 );
	Ship->mChangeName( kBloodshedder_name );
	Ship->mSetCommButton( 1, kButtonAttack_text );
	BloodshedderID = Ship->mGetShipID();

	Ship = PlayerTeam->mGetThisShip( 3 );
	Ship->mChangeName( kDesolation_name );
	Ship->mSetCommButton( 1, kButtonAttack_text );
	DesolationID = Ship->mGetShipID();

	Ship = EnemyTeam->mGetFirstShip();
	Ship->mChangeName( kHood_name );
	Ship->mSetFinalDestination();
	Ship->mSetShipCallbackEvents( kNotifyDamage );

	fMission.mSetTimedInterval( kTimerLimitSet, 4 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					switch( MissionChapter )
					{
						case 0:
							fMission.mDisplayMessage( kScienceReport_msg, kScienceScriptMessage );
							MissionChapter++;
							fMission.mSetTimedInterval( kTimerLimitSet, 8 );
							break;

						case 1:
							if( HoodAttack == false )
							{
								fMission.mDisplayMessage( kOrders_msg );
								MissionChapter++;
							}
							break;
					}
					break;

				case kTimerTurnsSet:
					if(( HoodLeaving == false ) && ( HoodAttack == false ))
					{
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						fMission.mDisplayMessage( kHoodRunning_msg, kScienceScriptMessage );
						HoodLeaving = true;
					}
					break;
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( Ship->mGetDamagePercent() >= 20 )
				{
					Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					fMission.mDisplayMessage( kHoodRunning_msg, kScienceScriptMessage );
					HoodLeaving = true;
				}
				else
				{
					HoodAttack = true;
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}

			Ship->mSetShipCallbackEvents( kNotifyDamage );
			break;

		case kNotifyAtDestination:
			if( ( HoodLeaving == false ) && ( HoodAttack == false ) )
			{
				fMission.mDisplayMessage( kHoodSlowing_msg, kScienceScriptMessage );
				fMission.mSetTimedInterval( kTimerTurnsSet, 10 );	// the hood is done
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( kHoodDestroyed_msg, kScienceScriptMessage );
				VictoryPoints++;
				return kPlayerWins;
			}
			else
			{
				VictoryPoints--;

				if( PlayerTeam->mGetShipsInPlayCount() != 0 )
				{
					if( Ship->mGetShipID() == ChallengerID )
					{
						fMission.mDisplayMessage( kChallengerDestroyed_msg, kScienceScriptMessage );
					}
					else if( Ship->mGetShipID() == BloodshedderID )
					{
						fMission.mDisplayMessage( kBloodshedderDestroyed_msg, kScienceScriptMessage );
					}
					else if( Ship->mGetShipID() == DesolationID )
					{
						fMission.mDisplayMessage( kDesolationDestroyed_msg, kScienceScriptMessage );
					}
				}
				else
				{
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( kHoodCaptured_msg, kScienceScriptMessage );
				VictoryPoints = VictoryPoints + 2;
				HoodCaptured = true;
				return kPlayerWins;
			}
			else
			{
				VictoryPoints = VictoryPoints - 2;

				if( PlayerTeam->mGetShipsInPlayCount != 0 )
				{
					if( Ship->mGetShipID() == ChallengerID )
					{
						fMission.mDisplayMessage( kChallengerCaptured_msg, kScienceScriptMessage );
					}
					else if( Ship->mGetShipID() == BloodshedderID )
					{
						fMission.mDisplayMessage( kBloodshedderCaptured_msg, kScienceScriptMessage );
					}
					else if( Ship->mGetShipID() == DesolationID )
					{
						fMission.mDisplayMessage( kDesolationCaptured_msg, kScienceScriptMessage );
					}
				}
				else
				{
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage( kAttacking_msg );
			fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			HoodAttack = true;

			for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
			{
				Ship->mClearCommButton( 1 );
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				VictoryPoints = 0;
				return kPlayerLoses;
			}

			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					VictoryPoints = 0;
					return kPlayerLoses;
				}
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( VictoryPoints >= 4 )
	{
		return( kAstoundingVictory );
	}
	else if( VictoryPoints == 3 )
	{
		return( kVictory );
	}
	else if( VictoryPoints == 2 )
	{
		return( kDraw );
	}
	else if( VictoryPoints == 1 )
	{
		return( kDefeat );
	}

	return( kDevastatingDefeat );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( VictoryPoints >= 3 )
	{
		fMission.mAddToDebriefingList( kGoodDebrief_msg );
	}

	if( VictoryPoints <= 1 )
	{
		fMission.mAddToDebriefingList( kBadDebrief_msg );
	}

	if( HoodCaptured == true )
	{
		fMission.mAddToDebriefingList( kHoodCaptureDebrief_msg );
	}
	
	fMission.mAddToDebriefingList( kYouScored_msg );
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

