#include "scriptinterface.h"

#include "His_RepairRendezvousText.h"
#include "His_RepairRendezvousMaps.h"

int32 NumberParts = 3;
int32 DiffLevel = 0;
bool FedPlayer = false;
bool EnemyDestroyed = false;
bool HospitalDestroyed = false;
bool CruiserDestroyed = false;
bool EscortDestroyed = false;
bool RepairDestroyed = false;
bool EnemyClose = false;
bool PressedESC = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kAlliedTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// set enemy race
	if( info.fPlayerRace == kFederation )
	{
		FedPlayer = true;
		info.fEnemyRace = kKlingon;
	}
	else if( info.fPlayerRace == kKlingon )
	{
		info.fEnemyRace = kFederation;
	}
	else if( info.fPlayerRace == kRomulan )
	{
		info.fEnemyRace = kGorn;
	}
	else if( info.fPlayerRace == kGorn )
	{
		info.fEnemyRace = kRomulan;
	}
	else if( info.fPlayerRace == kLyran )
	{
		info.fEnemyRace = kHydran;
	}
	else if( info.fPlayerRace == kHydran )
	{
		info.fEnemyRace = kLyran;
	}
	else
	{
		info.fEnemyRace = kKlingon;
	}

	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0 );

	DiffLevel = fMission.mGetDifficultyLevel();	// 0 = easy, 1 = medium, 2 = hard

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if( PlayerTeam->mGetRace() == kFederation )
	{
		PlayerTeam->mCreateShip( "F-CAR", kStartPosition_G );
		PlayerTeam->mCreateShip( "F-FFG", kStartPosition_I );
		AlliedTeam->mCreateShip( "F-FLPh", kStartPosition_R );
		AlliedTeam->mCreateShip( "F-CLH", kStartPosition_H );
		EnemyTeam->mCreateShip( "K-D6", kStartPosition_Z );
	}
	else if( PlayerTeam->mGetRace() == kKlingon )
	{
		PlayerTeam->mCreateShip( "K-D7K", kStartPosition_G );
		PlayerTeam->mCreateShip( "K-E4B", kStartPosition_I );
		AlliedTeam->mCreateShip( "K-FLPh", kStartPosition_R );
		EnemyTeam->mCreateShip( "F-NCL", kStartPosition_Z );
	}
	else if( PlayerTeam->mGetRace() == kRomulan )
	{
		PlayerTeam->mCreateShip( "R-SPA+", kStartPosition_G );
		PlayerTeam->mCreateShip( "R-K4RB", kStartPosition_I );
		AlliedTeam->mCreateShip( "R-FLPh", kStartPosition_R );
		EnemyTeam->mCreateShip( "G-CL", kStartPosition_Z );
	}
	else if( PlayerTeam->mGetRace() == kGorn )
	{
		PlayerTeam->mCreateShip( "G-BC", kStartPosition_G );
		PlayerTeam->mCreateShip( "G-DD+", kStartPosition_I );
		AlliedTeam->mCreateShip( "G-FLPh", kStartPosition_R );
		EnemyTeam->mCreateShip( "R-KR", kStartPosition_Z );
	}
	else if( PlayerTeam->mGetRace() == kLyran )
	{
		PlayerTeam->mCreateShip( "L-CAP", kStartPosition_G );
		PlayerTeam->mCreateShip( "L-FF+", kStartPosition_I );
		AlliedTeam->mCreateShip( "L-FLPh", kStartPosition_R );
		EnemyTeam->mCreateShip( "H-MNG", kStartPosition_Z );
	}
	else if( PlayerTeam->mGetRace() == kHydran )
	{
		PlayerTeam->mCreateShip( "H-RN+", kStartPosition_G );
		PlayerTeam->mCreateShip( "H-HN", kStartPosition_I );
		AlliedTeam->mCreateShip( "H-FLPh", kStartPosition_R );
		EnemyTeam->mCreateShip( "L-CL", kStartPosition_Z );
	}
	else
	{
		PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
		PlayerTeam->mCreateShip( kMaxBPV, kClassFrigate, 0, kStartPosition_I );
		AlliedTeam->mCreateShip( kMaxBPV, kClassFreighter, 0, kStartPosition_R );
		EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 0, kStartPosition_Z );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	if( FedPlayer == true )
	{
		return( kFedBriefing_msg );
	}

	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	NumberParts = NumberParts - DiffLevel;	// reduce the number of spare parts based on diff level
	int32 StartingDamage = 90 + ( DiffLevel * 15 );	// easy 90pts, medium 105, hard 120

	Ship = PlayerTeam->mGetFirstShip();	// the damaged cruiser
	Ship->mSetShipPlayerControlled();
	Ship->mDamageSystem( kDamageInternalsWithPointsSilently, StartingDamage );
	Ship->mDamageSystem( kDamageShield_1_Fore, fMission.mRandomInt32( 50 + ( DiffLevel * 25 ), 100 ) );
	Ship->mDamageSystem( kDamageShield_2_BottomFore, fMission.mRandomInt32( 50 + ( DiffLevel * 25 ), 100 ) );
	Ship->mDamageSystem( kDamageShield_3_BottomAft, fMission.mRandomInt32( 50 + ( DiffLevel * 25 ), 100 ) );
	Ship->mDamageSystem( kDamageShield_4_Aft, fMission.mRandomInt32( 50 + ( DiffLevel * 25 ), 100 ) );
	Ship->mDamageSystem( kDamageShield_5_TopAft, fMission.mRandomInt32( 50 + ( DiffLevel * 25 ), 100 ) );
	Ship->mDamageSystem( kDamageShield_6_TopFore, fMission.mRandomInt32( 50 + ( DiffLevel * 25 ), 100 ) );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyNumberOfReloads, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyNumberLoaded, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );

	if( PlayerTeam->mGetRace() == kHydran )
	{
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay1Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay2Count, 0 );
	}

	Ship = PlayerTeam->mGetNextShip();	// escort
	Ship->mSetFinalDestination();

	for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton( 1, kButtonDisengage_text );

		if( Ship->mGetClassType() == kClassFreighter )
		{
			Ship->mSetCommButton( 2, kButtonRepair_text );
		}
		else
		{
			Ship->mSetCommButton( 2, kButtonTransferWounded_text );
		}
	}

	Ship = EnemyTeam->mGetFirstShip();
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mTrackTacticalDistance( 60.0, 20, 21 );

	fMission.mSetTimedInterval( kTimerLimitSet, 4 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	switch( Event )
	{
		case kNotifyAtDistance:
			EnemyClose = true;
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					if( FedPlayer == true )
					{
						fMission.mDisplayMessage( kFedStarting_msg, kNonCommScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					}

					fMission.mSetTimedInterval( kTimerLimitSet_2, 10 );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0 );
					fMission.mSetTeamRelations( kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0 );
					
					Ship = EnemyTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						Ship->mSetShipTarget( PlayerTeam->mGetThisShip( 2 ), kMediumPriority );	// target the escort
					}
					break;

				case kTimerLimitSet_2:
					if( fPlayerShip->mGetClassType() != kClassHeavyCruiser )
					{
						fMission.mDisplayMessage( kCommReport_msg, kFleetScriptMessage );
					}
					break;

				case kTimerTurnsSet:	// transferring parts
					Ship = AlliedTeam->mGetFirstShip();	// should be the repair ship
					if(( Ship != NULL ) && ( strcmp( Ship->mGetClassName(), "F-CLH" ) != 0 ))
					{
						if( Ship->mGetTacticalDistance( fPlayerShip ) >= 3 )
						{
							fMission.mDisplayMessage( kRepairTransferIncomplete_msg, kFleetScriptMessage );
							Ship->mSetCommButton( 1, kButtonRepair_text );
						}
						else
						{
							NumberParts--;
							if( NumberParts <= 0 )
							{
								fMission.mDisplayMessage( kRepairOutOfGoods_msg, kFleetScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( kRepairTransferComplete_msg, kFleetScriptMessage );
								Ship->mSetCommButton( 2, kButtonRepair_text );
							}

							Ship = fPlayerShip;
							int32 NumParts = Ship->mAccessSupplies( kGetSupplyCount, kSupplySpareParts );
							Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, NumParts + 4 );
						}
					}
					break;

				case kTimerTurnsSet_2:	// transferring wounded
					Ship = AlliedTeam->mGetThisShip( 2 );	// should be the fed hospital ship
					if(( Ship != NULL ) && ( strcmp( Ship->mGetClassName(), "F-CLH" ) == 0 ))
					{
						if( Ship->mGetTacticalDistance( fPlayerShip ) >= 5 )
						{
							fMission.mDisplayMessage( kHospitalTransferIncomplete_msg, kFleetScriptMessage );
							Ship->mSetCommButton( 2, kButtonTransferWounded_text );
						}
						else
						{
							fMission.mDisplayMessage( kHospitalTransferComplete_msg, kFleetScriptMessage );
							Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						}
					}
					break;
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( Ship->mGetDamagePercent() >= 75 )
				{
					fMission.mDisplayMessage( kEnemyRetreat_msg, kScienceScriptMessage );
					Ship->mDisengage( kByMapEdge, kAbsolutePriority );
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			else
			{
				if( Ship->mGetDamagePercent() >= 30 )
				{
					if( Ship->mGetClassType() == kClassFreighter )
					{
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						fMission.mDisplayMessage( kRepairLeaving_msg, kFleetScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kHospitalLeaving_msg, kFleetScriptMessage );
					}
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PressedESC = true;
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				return kPlayerWins;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyDestroyed = true;
				return kPlayerWins;
			}
			else if( Ship->mGetOriginalTeam() == kAlliedTeam )
			{
				if( strcmp( Ship->mGetClassName(), "F-CLH" ) == 0 )
				{
					fMission.mDisplayMessage( kHospitalDestroyed_msg, kScienceScriptMessage );
					HospitalDestroyed = true;
				}
				else
				{
					fMission.mDisplayMessage( kRepairDestroyed_msg, kScienceScriptMessage );
					RepairDestroyed = true;
				}
			}
			else if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() > 0 )
				{
					if( Ship->mGetClassType() == kClassHeavyCruiser )
					{
						fMission.mDisplayMessage( kCruiserDestroyed_msg, kScienceScriptMessage );
						CruiserDestroyed = true;
					}
					else
					{
						fMission.mDisplayMessage( kEscortDestroyed_msg, kScienceScriptMessage );
						EscortDestroyed = true;
					}
				}
				else
				{
					CruiserDestroyed = true;
					EscortDestroyed = true;
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyDestroyed = true;
				return kPlayerWins;
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					fMission.mDisplayMessage( kEnemyResponse_msg, kFleetScriptMessage );
					break;

				case kButtonDisengage_text:
					if( strcmp( Ship->mGetClassName(), "F-CLH" ) == 0 )
					{
						fMission.mDisplayMessage( kHospitalDisengaging_msg, kFleetScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kRepairDisengaging_msg, kFleetScriptMessage );
					}

					Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					break;

				case kButtonRepair_text:
					if( Ship->mGetTacticalDistance( fPlayerShip ) >= 3 )
					{
						fMission.mDisplayMessage( kRepairTooFar_msg, kFleetScriptMessage );
						Ship->mSetCommButton( 2, kButtonRepair_text );
					}
					else
					{
						fMission.mDisplayMessage( kRepairStarting_msg, kFleetScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					}
					break;

				case kButtonTransferWounded_text:
					if( Ship->mGetTacticalDistance( fPlayerShip ) >= 5 )
					{
						fMission.mDisplayMessage( kHospitalTooFar_msg, kFleetScriptMessage );
						Ship->mSetCommButton( 2, kButtonTransferWounded_text );
					}
					else
					{
						fMission.mDisplayMessage( kHospitalStarting_msg, kFleetScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
					}
					break;
			}
			break;

			case kNotifyChangingShip:
				if( EnemyClose == false )
				{
					for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
					{
						Ship->mSetFinalDestination();
					}
				}
				break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( PressedESC == true )
	{
		return( kDevastatingDefeat );
	}

	if( CruiserDestroyed == true )
	{
		if(( HospitalDestroyed == true ) || ( EscortDestroyed == true )
			|| ( RepairDestroyed == true ))
		{
			return( kDevastatingDefeat );
		}
		else
		{
			return( kDefeat );
		}
	}

	if(( HospitalDestroyed == true ) || ( EscortDestroyed == true )
		|| ( RepairDestroyed == true ))
	{
		return( kDraw );
	}

	if( EnemyDestroyed == true )
	{
		return( kAstoundingVictory );
	}

	return( kVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if(( CruiserDestroyed == true ) && ( PressedESC == false ))
	{
		fMission.mAddToDebriefingList( kLostCruiser_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kCruiserEscaped_msg );

		if( EscortDestroyed == true )
		{
			fMission.mAddToDebriefingList( kLostEscort_msg );
		}

		if( RepairDestroyed == true )
		{
			fMission.mAddToDebriefingList( kLostRepair_msg );
		}

		if( HospitalDestroyed == true )
		{
			fMission.mAddToDebriefingList( kLostHospital_msg );
		}
	}

	fMission.mAddToDebriefingList( kYouScored_msg );
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

