#include "scriptinterface.h"

#include "His_StarbaseAssaultText.h"
#include "His_StarbaseAssaultMaps.h"

int32 MyVicLevel = kDefeat;
int32 PlayerKilled = 0;
int32 EnemyKilled = 0;
int32 EnemyCaptured = 0;
bool BaseDestroyed = false;

void mSetDefaultMenuItems (tDynaverseScriptInfo& info);

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kBaseTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kBaseTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kEnableSupplyDock | kDisableTeamC | kDisableTeamD | kDisableTeamE | kDisableTeamF;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	int32 RandomRace = 0;

	// check the first AI teams ship count - if its 0 then we need to fill in the default values before proceeding
	if (info.fMemberShipCounts[kTeam2] == 0)
	{
		mSetDefaultMenuItems (info);
	}

	// must setup the races and relations of the reserved teams in the script
	fMission.mSetSkirmishTeams (info);

	if (fMission.fTotalEnemyCPUTeams)
	{
		tTeamInfo* Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, 0);
		info.fEnemyRace = Team->mGetRace();
	}
	else
	{
		// set enemy race since there are no other enemy teams
		if( info.fPlayerRace == kFederation )
		{
			RandomRace = fMission.mRandomInt32( 1, 4 );

			switch( RandomRace )
			{
				case 1:
					info.fEnemyRace = kRomulan;
					break;

				case 2:
					info.fEnemyRace = kKlingon;
					break;

				case 3:
					info.fEnemyRace = kLyran;
					break;

				case 4:
					info.fEnemyRace = kGorn;
					break;
			}
		}
		else if( info.fPlayerRace == kKlingon )
		{
			RandomRace = fMission.mRandomInt32( 1, 4 );

			switch( RandomRace )
			{
				case 1:
					info.fEnemyRace = kRomulan;
					break;

				case 2:
					info.fEnemyRace = kFederation;
					break;

				case 3:
					info.fEnemyRace = kLyran;
					break;

				case 4:
					info.fEnemyRace = kHydran;
					break;
			}
		}
		else if( info.fPlayerRace == kRomulan )
		{
			RandomRace = fMission.mRandomInt32( 1, 3 );

			switch( RandomRace )
			{
				case 1:
					info.fEnemyRace = kFederation;
					break;

				case 2:
					info.fEnemyRace = kKlingon;
					break;

				case 3:
					info.fEnemyRace = kGorn;
					break;
			}
		}
		else if( info.fPlayerRace == kGorn )
		{
			RandomRace = fMission.mRandomInt32( 1, 4 );

			if( RandomRace == 1 )
			{
				info.fEnemyRace = kFederation;
			}
			else
			{
				info.fEnemyRace = kRomulan;
			}
		}
		else if( info.fPlayerRace == kLyran )
		{
			RandomRace = fMission.mRandomInt32( 1, 3 );

			switch( RandomRace )
			{
				case 1:
					info.fEnemyRace = kFederation;
					break;

				case 2:
					info.fEnemyRace = kKlingon;
					break;

				case 3:
					info.fEnemyRace = kHydran;
					break;
			}
		}
		else if( info.fPlayerRace == kHydran )
		{
			RandomRace = fMission.mRandomInt32( 1, 2 );

			if( RandomRace == 1 )
			{
				info.fEnemyRace = kKlingon;
			}
			else
			{
				info.fEnemyRace = kLyran;
			}
		}
	}

	fMission.mSetTeamRace( kBaseTeam, info.fEnemyRace );

	fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = kStartPosition_G | kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* Team;

	int32 position = kStartPosition_H;
	int32 BaseBPV = PlayerTeam->mGetCombatBPV();
	int32 DiffLevel = fMission.mGetDifficultyLevel();

	// create friendly AI ships first
	for (int32 i = 0; i < fMission.fTotalAlliedCPUTeams; i++)
	{
		Team = fMission.mGetTeamHandle (kCPU_AlliedTeams, i);
		Team->mCreateCPUFleet (20, position | kGroupAround);
		BaseBPV = BaseBPV + Team->mGetCombatBPV();
		fMission.mSetTeamRelations( Team->mGetTeam(), kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		position++;
	}

	if( DiffLevel == 0 )	// easy
	{
		BaseBPV = BaseBPV * .75;
	}

	if( DiffLevel == 2 )	// hard
	{
		BaseBPV = BaseBPV * 1.5;
	}

	position = kStartPosition_U;

	// create enemy AI ships last
	for (i = 0; i < fMission.fTotalEnemyCPUTeams; i++)
	{
		Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, i);
		Team->mCreateCPUFleet (20, position | kGroupAround);
		fMission.mSetTeamRelations( Team->mGetTeam(), kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
		position++;
	}

	BaseTeam->mCreateShip( BaseBPV, kClassBaseStation, kClassStarBase, 100, kStartPosition_M );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	switch( Event )
	{
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerKilled++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}

			if( Ship->mGetTeam() == kBaseTeam )
			{
				BaseDestroyed = true;

				return kPlayerWins;
			}

			if( Ship->mGetTeam() != kPlayerTeam )
			{
				EnemyKilled++;
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kBaseTeam )
			{
				BaseDestroyed = true;

				return kPlayerWins;
			}

			if( Ship->mGetTeam() != kPlayerTeam )
			{
				EnemyCaptured++;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	MyVicLevel = kDefeat;

	if( BaseDestroyed == true )
	{
		MyVicLevel = kVictory;
	}
 
	if( EnemyKilled >= 2 )
	{
		MyVicLevel--;
	}

	if( PlayerKilled == 0 )
	{
		MyVicLevel--;
	}

	if( PlayerKilled >= 2 )
	{
		MyVicLevel++;
	}

	if( EnemyCaptured >= 1 )
	{
		MyVicLevel--;
	}

	if( MyVicLevel <= kAstoundingVictory )
	{
		MyVicLevel = kAstoundingVictory;
	}

	if( MyVicLevel >= kDevastatingDefeat )
	{
		MyVicLevel = kDevastatingDefeat;
	}

	return( eVictoryLevels( MyVicLevel ));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( kYourScore_msg );
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
//
void mSetDefaultMenuItems (tDynaverseScriptInfo& info)
{
	// default player settings:
	info.fMemberShipCounts[kTeam1]	= 3;				// must be 1-3
	info.fMemberBPVLimits[kTeam1]	= 400;
	info.fMemberSmallHulls[kTeam1]	= kClassLightCruiser;
	info.fMemberLargeHulls[kTeam1]	= kClassHeavyCruiser;

	// default AI settings:
	//	ships:		- 1
	//	small hull:	- kClassFrigate
	//	large hull:	- kClassHeavyCruiser
	//	BPV:		- 400
	//
	//	team 2:		- Klingon (or Romulan if player is Klingon)
	//	teams 3-6:	- disabled

	for (int32 i = kTeam2; i <= kTeam6; i++)
	{
		info.fMembersInGame[i]		= kPlayerTypeNoOne;
		info.fMemberTeams[i]		= eTeamID(kTeam2);
		info.fMemberRaces[i]		= kNoRaceType;
		info.fMemberShipCounts[i]	= 3;				// must be 1-3
		info.fMemberBPVLimits[i]	= 250;
		info.fMemberSmallHulls[i]	= kClassFrigate;
		info.fMemberLargeHulls[i]	= kClassLightCruiser;
	}
	info.fMembersInGame[kTeam2]		= kPlayerTypeAI;
	info.fMemberRaces[kTeam2]		= (info.fPlayerRace != kKlingon) ? kKlingon : kRomulan;
}
