#include "scriptinterface.h"

#include "His_SurpriseReversedText.h"
#include "His_SurpriseReversedMaps.h"

int32 EnemyDestroyed = 0;
bool PlayerDestroyed = false;
bool EnemyShipActive[5] = {false, false, false, false, false};
bool EnemyAlive = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission | kKlingMission | kRomulanMission | kLyranMission | kHydranMission | kGornMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	int32 RandomRace = 0;

	// set enemy race
	// CNF: perhaps this should be setable from the customize menus?
	if( info.fPlayerRace == kFederation )
	{
		RandomRace = fMission.mRandomInt32( 1, 4 );

		switch( RandomRace )
		{
			case 1:
				info.fEnemyRace = kRomulan;
				break;

			case 2:
				info.fEnemyRace = kKlingon;
				break;

			case 3:
				info.fEnemyRace = kLyran;
				break;

			case 4:
				info.fEnemyRace = kGorn;
				break;
		}
	}
	else if( info.fPlayerRace == kKlingon )
	{
		RandomRace = fMission.mRandomInt32( 1, 4 );

		switch( RandomRace )
		{
			case 1:
				info.fEnemyRace = kRomulan;
				break;

			case 2:
				info.fEnemyRace = kFederation;
				break;

			case 3:
				info.fEnemyRace = kLyran;
				break;

			case 4:
				info.fEnemyRace = kHydran;
				break;
		}
	}
	else if( info.fPlayerRace == kRomulan )
	{
		RandomRace = fMission.mRandomInt32( 1, 3 );

		switch( RandomRace )
		{
			case 1:
				info.fEnemyRace = kFederation;
				break;

			case 2:
				info.fEnemyRace = kKlingon;
				break;

			case 3:
				info.fEnemyRace = kGorn;
				break;
		}
	}
	else if( info.fPlayerRace == kGorn )
	{
		RandomRace = fMission.mRandomInt32( 1, 4 );

		if( RandomRace == 1 )
		{
			info.fEnemyRace = kFederation;
		}
		else
		{
			info.fEnemyRace = kRomulan;
		}
	}
	else if( info.fPlayerRace == kLyran )
	{
		RandomRace = fMission.mRandomInt32( 1, 3 );

		switch( RandomRace )
		{
			case 1:
				info.fEnemyRace = kFederation;
				break;

			case 2:
				info.fEnemyRace = kKlingon;
				break;

			case 3:
				info.fEnemyRace = kHydran;
				break;
		}
	}
	else if( info.fPlayerRace == kHydran )
	{
		RandomRace = fMission.mRandomInt32( 1, 2 );

		if( RandomRace == 1 )
		{
			info.fEnemyRace = kKlingon;
		}
		else
		{
			info.fEnemyRace = kLyran;
		}
	}

	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	// set team relations
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	if( EnemyTeam->mGetRace() == kRomulan )
	{
		EnemyTeam->mCreateShip ("R-KR", kStartPosition_V);
		EnemyTeam->mCreateShip ("R-WE", kStartPosition_W);
		EnemyTeam->mCreateShip ("R-K5R", kStartPosition_X);
		EnemyTeam->mCreateShip ("R-SNB", kStartPosition_Y);
		EnemyTeam->mCreateShip ("R-SNB", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kKlingon )
	{
		EnemyTeam->mCreateShip ("K-D7", kStartPosition_V);
		EnemyTeam->mCreateShip ("K-D6", kStartPosition_W);
		EnemyTeam->mCreateShip ("K-D6", kStartPosition_X);
		EnemyTeam->mCreateShip ("K-F5", kStartPosition_Y);
		EnemyTeam->mCreateShip ("K-F5", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kFederation )
	{
		EnemyTeam->mCreateShip ("F-DN", kStartPosition_V);
		EnemyTeam->mCreateShip ("F-CA", kStartPosition_W);
		EnemyTeam->mCreateShip ("F-NCL", kStartPosition_X);
		EnemyTeam->mCreateShip ("F-DD", kStartPosition_Y);
		EnemyTeam->mCreateShip ("F-FF", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kGorn )
	{
		EnemyTeam->mCreateShip ("G-CA", kStartPosition_V);
		EnemyTeam->mCreateShip ("G-CL", kStartPosition_W);
		EnemyTeam->mCreateShip ("G-CL", kStartPosition_X);
		EnemyTeam->mCreateShip ("G-DD", kStartPosition_Y);
		EnemyTeam->mCreateShip ("G-DD", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kLyran )
	{
		EnemyTeam->mCreateShip ("L-DN", kStartPosition_V);
		EnemyTeam->mCreateShip ("L-CA", kStartPosition_W);
		EnemyTeam->mCreateShip ("L-CW", kStartPosition_X);
		EnemyTeam->mCreateShip ("L-DD", kStartPosition_Y);
		EnemyTeam->mCreateShip ("L-DD", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kHydran )
	{
		EnemyTeam->mCreateShip ("H-PAL", kStartPosition_V);
		EnemyTeam->mCreateShip ("H-RN", kStartPosition_W);
		EnemyTeam->mCreateShip ("H-LN", kStartPosition_X);
		EnemyTeam->mCreateShip ("H-HN", kStartPosition_Y);
		EnemyTeam->mCreateShip ("H-HN", kStartPosition_Z);
	}

	if( PlayerTeam->mGetRace() == kFederation )
	{
		PlayerTeam->mCreateShip ("F-CC+", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kKlingon )
	{
		PlayerTeam->mCreateShip ("K-D7K", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kRomulan )
	{
		PlayerTeam->mCreateShip ("R-K7RB", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kGorn )
	{
		PlayerTeam->mCreateShip ("G-CC+", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kLyran )
	{
		PlayerTeam->mCreateShip ("L-CC+", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kHydran )
	{
		PlayerTeam->mCreateShip ("H-LM", kStartPosition_G);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	PlayerTeam->mGetFirstShip()->mTrackTacticalDistance (40.0, 21, 19);

	for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
	{
		Ship->mSetShipAIControlled( kFalse );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	int32 ShipCounter = 1;
	int32 ActiveShipsThisTurn = 0;
	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					for( ShipCounter = 1; ShipCounter <= 5; ShipCounter++ )
					{
						if( ( fMission.mRandomInt32( 1, 4 ) == 1 ) && ( EnemyShipActive[ ShipCounter ] == false ) )
						{
							EnemyShipActive[ ShipCounter ] = true;
							Ship = EnemyTeam->mGetThisShip( ShipCounter );

							if( Ship->mTestShipStatus( kShipDestroyed ) == kFalse )
							{
								Ship->mSetShipAIControlled( kTrue );
								ActiveShipsThisTurn++;
							}
						}
					}

					if( ActiveShipsThisTurn > 0 )
					{
						fMission.mDisplayMessage (kDamageAlive_msg + ActiveShipsThisTurn, kScienceScriptMessage);
						ActiveShipsThisTurn = 0;
					}

					// set the delay before the next chance for ships to wake up
					fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, 4 ) );
					break;

				case kTimerLimitSet:
					fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);
					break;
			}
			break;

		case kNotifyAtDistance:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage (kCrossed_msg, kScienceScriptMessage);
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 3, 5 ) );
				EnemyAlive = true;
			}
			break;

		case kNotifyDamage:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage (kDamageAlive_msg, kScienceScriptMessage);
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 3, 5 ) );
				EnemyAlive = true;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				PlayerDestroyed = true;
				return kPlayerLoses;
			}
			else
			{
				EnemyDestroyed++;
			}

			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage (kDamageAlive_msg, kScienceScriptMessage);
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 3, 5 ) );
				EnemyAlive = true;
			}

			if( Enemies == 0 )
			{
				return kPlayerWins;
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage, Ship );
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				return kPlayerWins;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( PlayerDestroyed == true )
	{
		return( kDevastatingDefeat );
	}

	switch( EnemyDestroyed )
	{
		case 0:
			return( kDevastatingDefeat );

		case 1:
			return( kDefeat );

		case 2:
			return( kDraw );

		case 3:
			return( kVictory );
	}

	return( kAstoundingVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( PlayerDestroyed == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList( kPlayerDestroyed_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
