#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

// void mSetDefaultMenuItems (tDynaverseScriptInfo& info);
tString PlayerShipClass;
eClassTypes PlayerClass = kClassShuttle;
bool ShipMap = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kFedTeam, kKlingonTeam, kRomulanTeam, kLyranTeam, kHydranTeam, kGornTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kEnableSupplyDock;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	if( info.fPreferedMap >= 8 )
	{
		ShipMap = true;
	}

	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kFedTeam, kFederation );
	fMission.mSetTeamRace( kKlingonTeam, kKlingon );
	fMission.mSetTeamRace( kRomulanTeam, kRomulan );
	fMission.mSetTeamRace( kLyranTeam, kLyran );
	fMission.mSetTeamRace( kHydranTeam, kHydran );
	fMission.mSetTeamRace( kGornTeam, kGorn );

	fMission.mSetTeamRelations( kPlayerTeam, kFedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kKlingonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kRomulanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kFedTeam, kKlingonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kFedTeam, kRomulanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kFedTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kFedTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kFedTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kKlingonTeam, kRomulanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kKlingonTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kKlingonTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kKlingonTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kRomulanTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kRomulanTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kRomulanTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kLyranTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kLyranTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kHydranTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = kStartPosition_G | kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* FedTeam = fMission.mGetTeamHandle( kFedTeam );
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle( kKlingonTeam );
	tTeamInfo* RomulanTeam = fMission.mGetTeamHandle( kRomulanTeam );
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle( kLyranTeam );
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle( kHydranTeam );
	tTeamInfo* GornTeam = fMission.mGetTeamHandle( kGornTeam );
	tShipInfo* Ship;

	if( ShipMap == true )
	{
		switch( Mode )
		{
			case 0:
				// determine the biggest ship class the player has
				for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
				{
					if (PlayerClass < Ship->mGetClassType())
					{
						PlayerClass = Ship->mGetClassType();
					}
				}

				FedTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_U );
				FedTeam->mCreateShip( "F-CA", kStartPosition_T );
				KlingonTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_V );
				KlingonTeam->mCreateShip( "K-D7", kStartPosition_S );
				RomulanTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_W );
				LyranTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_X );
				HydranTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_Y );
				GornTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_Z );

				KlingonTeam->mCreateShip( "N-PUCK", kStartPosition_P );
				break;

			case 1:
				FedTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_U );
				break;

			case 2:
				KlingonTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_V );
				break;

			case 3:
				RomulanTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_W );
				break;

			case 4:
				LyranTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_X );
				break;

			case 5:
				HydranTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_Y );
				break;

			case 6:
				GornTeam->mCreateShip( kMaxBPV, kClassFrigate, PlayerClass, 0, kStartPosition_Z );
				break;

		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	PlayerShipClass = Ship->mGetClassName();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (0);
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 4 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if( Ship->mGetTeam() == kPlayerTeam )
	{
		Ship->mSetShipPlayerControlled();
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			fMission.mDisplayMessage( kStarting_msg );
			fMission.mSetTeamRelations( kPlayerTeam, kFedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kKlingonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kRomulanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kGornTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kFedTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kFedTeam, kRomulanTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kFedTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kFedTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kFedTeam, kGornTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kKlingonTeam, kRomulanTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kKlingonTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kKlingonTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kKlingonTeam, kGornTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kRomulanTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kRomulanTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kRomulanTeam, kGornTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kLyranTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kLyranTeam, kGornTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kHydranTeam, kGornTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerTeam->mCreateShip( PlayerShipClass.mCharStr(), kStartPosition_G );
			}

			if( ShipMap == true )
			{
				if( Ship->mGetTeam() == kFedTeam )
				{
					mCreateMissionShips( 1 );
				}
				else if( Ship->mGetTeam() == kKlingonTeam )
				{
					mCreateMissionShips( 2 );
				}
				else if( Ship->mGetTeam() == kRomulanTeam )
				{
					mCreateMissionShips( 3 );
				}
				else if( Ship->mGetTeam() == kLyranTeam )
				{
					mCreateMissionShips( 4 );
				}
				else if( Ship->mGetTeam() == kHydranTeam )
				{
					mCreateMissionShips( 5 );
				}
				else if( Ship->mGetTeam() == kGornTeam )
				{
					mCreateMissionShips( 6 );
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			else
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (fMission.mGetVictoryLevel (0, 0));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (kEnding_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
//
/*
void mSetDefaultMenuItems (tDynaverseScriptInfo& info)
{
	// default player settings:
	info.fMemberShipCounts[kTeam1]	= 1;				// must be 1-3
	info.fMemberBPVLimits[kTeam1]	= 2000;
	info.fMemberSmallHulls[kTeam1]	= kClassDreadnought;
	info.fMemberLargeHulls[kTeam1]	= kClassDreadnought;

	// default AI settings:
	//	ships:		- 1
	//	small hull:	- kClassFrigate
	//	large hull:	- kClassHeavyCruiser
	//	BPV:		- 400
	//
	//	team 2:		- Klingon (or Romulan if player is Klingon)
	//	teams 3-6:	- disabled

	for (int32 i = kTeam2; i <= kTeam6; i++)
	{
		info.fMembersInGame[i]		= kPlayerTypeNoOne;
		info.fMemberTeams[i]		= eTeamID(i);
		info.fMemberRaces[i]		= kNoRaceType;
		info.fMemberShipCounts[i]	= 1;				// must be 1-3
		info.fMemberBPVLimits[i]	= 400;
		info.fMemberSmallHulls[i]	= kClassFrigate;
		info.fMemberLargeHulls[i]	= kClassFrigate;
	}
//	info.fMembersInGame[kTeam2]		= kPlayerTypeAI;
//	info.fMemberRaces[kTeam2]		= (info.fPlayerRace != kKlingon) ? kKlingon : kRomulan;
}
*/