#include "scriptinterface.h"

#include "His_TheDuelText.h"
#include "His_TheDuelMaps.h"

bool EnemyDestroyed = false;
bool EnemyCaptured = false;
bool PlayerDestroyed = false;
bool Monster = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	if( fMission.mGetINIFileInt( kINIFileHeader_msg, kINIFileLabel_msg ) == 1 )
	{
		Monster = true;
	}

	// set enemy race
	if( info.fPlayerRace == kFederation )
	{
		info.fEnemyRace = kKlingon;
	}
	else if( info.fPlayerRace == kKlingon )
	{
		info.fEnemyRace = kFederation;
	}
	else if( info.fPlayerRace == kRomulan )
	{
		info.fEnemyRace = kGorn;
	}
	else if( info.fPlayerRace == kGorn )
	{
		info.fEnemyRace = kRomulan;
	}
	else if( info.fPlayerRace == kLyran )
	{
		info.fEnemyRace = kHydran;
	}
	else if( info.fPlayerRace == kHydran )
	{
		info.fEnemyRace = kLyran;
	}
	else
	{
		info.fEnemyRace = kKlingon;
	}

	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// pick a map
	return fMission.mRandomInt32( 1, TotalMaps );
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	int32 DiffLevel = fMission.mGetDifficultyLevel();	// 0 = Easy, 1 = Medium, 2 = Hard

	if( Monster == true )
	{
		PlayerTeam->mCreateShip( "M-SGL", kStartPosition_G );
		if( EnemyTeam->mGetRace() == kFederation )
		{
			EnemyTeam->mCreateShip( "F-DN+", kStartPosition_Z );
		}
		else if( EnemyTeam->mGetRace() == kKlingon )
		{
			EnemyTeam->mCreateShip( "K-C8", kStartPosition_Z );
		}
		else if( EnemyTeam->mGetRace() == kRomulan )
		{
			EnemyTeam->mCreateShip( "R-CON", kStartPosition_Z );
		}
		else if( EnemyTeam->mGetRace() == kGorn )
		{
			EnemyTeam->mCreateShip( "G-DN", kStartPosition_Z );
		}
		else if( EnemyTeam->mGetRace() == kLyran )
		{
			EnemyTeam->mCreateShip( "L-DN", kStartPosition_Z );
		}
		else if( EnemyTeam->mGetRace() == kHydran )
		{
			EnemyTeam->mCreateShip( "H-PAL", kStartPosition_Z );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, 0, kStartPosition_Z );
		}

		return;
	}

	switch( DiffLevel )
	{
		case 0:
			if( PlayerTeam->mGetRace() == kFederation )
			{
				PlayerTeam->mCreateShip( "F-CC+", kStartPosition_G );
				EnemyTeam->mCreateShip( "K-D6", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kKlingon )
			{
				PlayerTeam->mCreateShip( "K-D7L", kStartPosition_G );
				EnemyTeam->mCreateShip( "F-CA", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kRomulan )
			{
				PlayerTeam->mCreateShip( "R-K7R", kStartPosition_G );
				EnemyTeam->mCreateShip( "G-CL", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kGorn )
			{
				PlayerTeam->mCreateShip( "G-BC", kStartPosition_G );
				EnemyTeam->mCreateShip( "R-WE", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kLyran )
			{
				PlayerTeam->mCreateShip( "L-CC+", kStartPosition_G );
				EnemyTeam->mCreateShip( "H-TR", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kHydran )
			{
				PlayerTeam->mCreateShip( "H-LM", kStartPosition_G );
				EnemyTeam->mCreateShip( "L-CL", kStartPosition_Z );
			}
			else
			{
				PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
				EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 0, kStartPosition_Z );
			}
			break;

		case 1:
			if( PlayerTeam->mGetRace() == kFederation )
			{
				PlayerTeam->mCreateShip( "F-CA", kStartPosition_G );
				EnemyTeam->mCreateShip( "K-D7", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kKlingon )
			{
				PlayerTeam->mCreateShip( "K-D7", kStartPosition_G );
				EnemyTeam->mCreateShip( "F-CA", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kRomulan )
			{
				PlayerTeam->mCreateShip( "R-KR", kStartPosition_G );
				EnemyTeam->mCreateShip( "G-CA", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kGorn )
			{
				PlayerTeam->mCreateShip( "G-CA", kStartPosition_G );
				EnemyTeam->mCreateShip( "R-KR", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kLyran )
			{
				PlayerTeam->mCreateShip( "L-CA", kStartPosition_G );
				EnemyTeam->mCreateShip( "H-RN", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kHydran )
			{
				PlayerTeam->mCreateShip( "H-RN", kStartPosition_G );
				EnemyTeam->mCreateShip( "L-CA", kStartPosition_Z );
			}
			else
			{
				PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
				EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_Z );
			}
			break;

		case 2:
			if( PlayerTeam->mGetRace() == kFederation )
			{
				PlayerTeam->mCreateShip( "F-CA", kStartPosition_G );
				EnemyTeam->mCreateShip( "K-D7L", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kKlingon )
			{
				PlayerTeam->mCreateShip( "K-D6", kStartPosition_G );
				EnemyTeam->mCreateShip( "F-CC+", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kRomulan )
			{
				PlayerTeam->mCreateShip( "R-WE", kStartPosition_G );
				EnemyTeam->mCreateShip( "G-BC", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kGorn )
			{
				PlayerTeam->mCreateShip( "G-CL", kStartPosition_G );
				EnemyTeam->mCreateShip( "R-K7R", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kLyran )
			{
				PlayerTeam->mCreateShip( "L-CL", kStartPosition_G );
				EnemyTeam->mCreateShip( "H-LM", kStartPosition_Z );
			}
			else if( PlayerTeam->mGetRace() == kHydran )
			{
				PlayerTeam->mCreateShip( "H-TR", kStartPosition_G );
				EnemyTeam->mCreateShip( "L-CC+", kStartPosition_Z );
			}
			else
			{
				PlayerTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 0, kStartPosition_G );
				EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_Z );
			}
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	Ship = PlayerTeam->mGetFirstShip();	// the damaged cruiser
	Ship->mSetShipPlayerControlled();

	Ship = EnemyTeam->mGetFirstShip();
	if( Monster == false )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 4 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( Ship->mGetDamagePercent() >= 80 )
				{
					fMission.mDisplayMessage( kEnemyRetreat_msg, kScienceScriptMessage );
					Ship->mDisengage( kByMapEdge, kAbsolutePriority );
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				return kPlayerWins;
			}
			else
			{
				if (( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
				{
					return kPlayerDisengaged;
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyDestroyed = true;
				return kPlayerWins;
			}
			else
			{
				PlayerDestroyed = true;
				return kPlayerLoses;
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyCaptured = true;
				return kPlayerWins;
			}
			else
			{
				return kPlayerLoses;
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage( kEnemyResponse_msg, kFleetScriptMessage );
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if( PlayerDestroyed == true )
	{
		return( kDefeat );
	}

	if( EnemyDestroyed == true )
	{
		return( kVictory );
	}

	if( EnemyCaptured == true )
	{
		return( kAstoundingVictory );
	}

	return( kDraw );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( kYouScored_msg );
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

