// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
//  NOTE:	This is a complete production mission. For an explaination of the functions used refer to
//			ScriptInterface.h, ScriptStructs.h and Tst_Examples.cpp. Also see Gen_Template.cpp, 
//			His_Template.cpp, and Mul_Template.cpp for simple template missions.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







#include "scriptinterface.h"

#include "His_WrathOfKhanText.h"
#include "His_WrathOfKhanMaps.h"

bool ReliantScanable = false;
bool EnterpriseDestroyed = false;
bool ReliantDestroyed = false;
bool KhanTaunted = false;

int32 Chapter = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kKhanTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kKhanTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetTeamRelations( kPlayerTeam, kKhanTeam, kTeamAtWar, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kKhanTeam, kPlayerTeam, kTeamHostile, kGoalIgnore, 1.0 );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* KhanTeam = fMission.mGetTeamHandle (kKhanTeam);

	PlayerTeam->mCreateShip( "F-CA+", kStartPosition_G );

	KhanTeam->mCreateShip( "F-NCL", kStartPosition_Z );

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* KhanTeam = fMission.mGetTeamHandle (kKhanTeam);
	tShipInfo* Ship;

	// configure the Enterprise
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust (10);
	Ship->mChangeName( kEnterprise_name );
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 20.0 );
	Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 40 );
	Ship->mDamageSystem( kDamageLeftWarpEngine, 50.0 );
	Ship->mDamageSystem( kDamageHeavyWeapon1, 0.0 );
	Ship->mDamageSystem( kDamageHeavyWeapon2, 100.0 );
	Ship->mDamageSystem( kDamageHeavyWeapon7, 100.0 );
	Ship->mSetOfficerName( kEngineeringOfficer, kScotty_name );
	Ship->mSetOfficerRank( kEngineeringOfficer, kLegendary );
	Ship->mSetOfficerName( kScienceOfficer, kSpock_name );
	Ship->mSetOfficerRank( kScienceOfficer, kLegendary );
	Ship->mSetOfficerName( kHelmOfficer, kSulu_name );
	Ship->mSetOfficerRank( kHelmOfficer, kLegendary );
	Ship->mSetOfficerName( kWeaponsOfficer, kChekov_name );
	Ship->mSetOfficerRank( kWeaponsOfficer, kVeteran );
	Ship->mSetOfficerName( kSecurityOfficer, kRedShirt_name );
	Ship->mSetOfficerRank( kSecurityOfficer, kRookie );

	// configure the Reliant
	Ship = KhanTeam->mGetFirstShip();
	Ship->mChangeName( kReliant_name );
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 25.0 );
	Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 30 );
	Ship->mDamageSystem( kDamageHeavyWeapon7, 100.0 );
	Ship->mSetCommButton( 1, kButtonTaunt_text );
	Ship->mSetCommButton( 2, kButtonPlead_text );
	Ship->mSetShipCallbackEvents( kNotifyDamage );

	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
	fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 3, 4 ));
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* KhanTeam = fMission.mGetTeamHandle (kKhanTeam);

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					switch( Chapter )
					{
						case 0:	// the player waited too long to taunt Khan and he has repaired his ship
							if( KhanTaunted == false )
							{
								Ship = KhanTeam->mGetFirstShip();
								KhanTeam->mGetFirstShip()->mClearCommButton( 1 );
								KhanTeam->mGetFirstShip()->mClearCommButton( 2 );
								Ship->mDamageSystem( kDamageWeaponsAndSystems, 0.0 );
								fMission.mSetTeamRelations( kKhanTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 2.0 );
								fMission.mDisplayMessage( kReliantPursuing_msg, kScienceScriptMessage );
							}
							break;

						case 1:	// genesis device is going to explode
							Ship = KhanTeam->mGetFirstShip();
							Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
							Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
							fMission.mDisplayMessage( kGenesisExplodes_msg, kNonCommScriptMessage );
							ReliantDestroyed = true;

							if( Ship->mGetTacticalDistance( PlayerTeam->mGetFirstShip() ) > 60 )
							{
								fMission.mDisplayMessage( kEnterpriseEscapes_msg, kNonCommScriptMessage );
								return kPlayerWins;
							}
							else	// enterprise destroyed
							{
								fMission.mDisplayMessage( kEnterpriseDestroyed_msg, kNonCommScriptMessage );
								Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
								PlayerTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPoints, 5000 );
								EnterpriseDestroyed = true;
								return kPlayerLoses;
							}
							break;
					}
					break;

				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mDisplayMessage( kSlowingDown_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_7:
					fMission.mDisplayMessage( kKhanFinal1_msg );
					fMission.mSetTimedInterval( kTimerLimitSet_8, 3 );
					break;

				case kTimerLimitSet_8:
					fMission.mDisplayMessage( kKhanFinal2_msg );
					fMission.mSetTimedInterval( kTimerLimitSet_9, 4 );
					break;

				case kTimerLimitSet_9:
					fMission.mDisplayMessage( kKhanFinal3_msg );
					break;

				case kTimerLimitSet_10:
					fMission.mDisplayMessage( kEnergySource_msg, kScienceScriptMessage );
					if(( fIsDeepScanOn == kTrue ) && ( ReliantScanable == true ))
					{
						fMission.mDisplayMessage( kReliantScanned_msg, kScienceScriptMessage );
						fMission.mSetTimedInterval( kTimerLimitSet_7, 5 );
						ReliantScanable = false;
					}
					fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, 4 ));
					Chapter = 1;
					break;

			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam)
			{
				EnterpriseDestroyed = true;
				return kPlayerLoses;
			}
			else
			{
				ReliantDestroyed = true;
				return kPlayerWins;
			}
			break;

		case kNotifyOnCapture:
			fMission.mDisplayMessage( kEnergySource_msg, kScienceScriptMessage );
			fMission.mSetTimedInterval( kTimerTurnsSet, 4 );
			Chapter = 1;
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kKhanTeam )
			{
				if( Ship->mGetDamagePercent() > 80.0 )
				{
					fMission.mDisplayMessage( kReliantDisabled_msg, kScienceScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kKhanTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTimedInterval( kTimerLimitSet_10, 8 );
					Ship->mSetCommButton( 1, kButtonSurrender_text );
					ReliantScanable = true;
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonTaunt_text:
					fMission.mDisplayMessage( kKirkTaunt_msg );
					fMission.mDisplayMessage( kKhanTaunted_msg );
					fMission.mSetTeamRelations( kKhanTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 2.0 );
					fMission.mDisplayMessage( kReliantPursuing_msg, kScienceScriptMessage );
					KhanTaunted = true;
					KhanTeam->mGetFirstShip()->mClearCommButton( 2 );
					break;

				case kButtonPlead_text:
					fMission.mDisplayMessage( kKirkPlead_msg );
					fMission.mDisplayMessage( kKhanPleaded_msg );
					KhanTeam->mGetFirstShip()->mClearCommButton( 1 );
					break;

				case kButtonSurrender_text:
					fMission.mDisplayMessage( kKirkSurrender_msg );
					break;
			}
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				if( ReliantScanable == true )
				{
					fMission.mDisplayMessage( kReliantScanned_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_7, 5 );
					ReliantScanable = false;
				}
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				return kPlayerWins;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (fMission.mGetVictoryLevel (0, 0));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( EnterpriseDestroyed == true )
	{
		if( ReliantDestroyed == true )
		{
			fMission.mAddToDebriefingList( kDraw_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kDefeat_msg );
		}
	}
	else
	{
		if( ReliantDestroyed == true )
		{
			fMission.mAddToDebriefingList( kVictory_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kDisengageDefeat_msg );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

