#include "scriptinterface.h"

#include "Hyd_10MissionText.h"
#include "Hyd_10MissionMaps.h"


int32 PlayerDestroyed = 0;
int32 MonsterDestroyed = 0;
int32 MonsterEscaped = 0;
int32 MonstersSubdued = 0;
int32 WorshiperDestroyed = 0;
int32 HydranAllyDestroyed = 0;

int32 ActiveBeasts = 0;
int32 ActiveEnemy = 0;

const int kTotalMonsters = 5;
const float DamageThreshold = 30.0;
int32 Loyalty = 0;
int32 SwitchedLoyalty = 0;
int32 Anarchy = 0;
int32 TalkedToWorshiper = 0;

enum eLoyalty
{
	kHydranLoyalty = 0,
	kMonsterLoyalty
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kMonsterTeam, kWorshiperTeam, kHydranAllyTeam
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 300, 200, 100, 50, 0 );

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kMonsterTeam, kMonster );
	fMission.mSetTeamRace( kWorshiperTeam, kHydran );
	fMission.mSetTeamRace( kHydranAllyTeam, kHydran );

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0 );
	fMission.mSetTeamRelations( kPlayerTeam, kWorshiperTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kHydranAllyTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

	fMission.mSetTeamRelations( kMonsterTeam, kWorshiperTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kMonsterTeam, kHydranAllyTeam, kTeamHostile, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kMonsterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0 );

	fMission.mSetTeamRelations( kHydranAllyTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0 );

	//fMission.mSetTeamRelations (kWorshiperTeam, kHydranAllyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );
	tTeamInfo* HydranAllyTeam = fMission.mGetTeamHandle( kHydranAllyTeam );

	int32 DiffLevel = fMission.mGetDifficultyLevel();

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32( LowBPV, HighBPV );

	switch( DiffLevel )
	{
		case 0:	// easy
			// create Worshiper team
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, kClassHeavyBattlecruiser, 50, kStartPosition_K );
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 50, kStartPosition_L );
			WorshiperTeam->mCreateShip( kMaxBPV, kClassDestroyer, kClassWarDestroyer, 25, kStartPosition_M );

			// create HydranAlly
			HydranAllyTeam->mCreateShip( kMaxBPV, kClassDreadnought, 1, kStartPosition_U );
			break;

		case 1:	// medium


			// create Worshiper team
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, kClassHeavyBattlecruiser, 50, kStartPosition_K );
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 75, kStartPosition_L );
			WorshiperTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 50, kStartPosition_M );

			// create HydranAlly
			HydranAllyTeam->mCreateShip( kMaxBPV, kClassDreadnought, 50, kStartPosition_U );
			HydranAllyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 50, kStartPosition_R );
			break;

		case 2:	// hard

			// create Worshiper team
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, kClassHeavyBattlecruiser, 0, kStartPosition_K );
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 100, kStartPosition_L );
			WorshiperTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_M );

			// create HydranAlly
			HydranAllyTeam->mCreateShip( kMaxBPV, kClassDreadnought, 0, kStartPosition_R );
			HydranAllyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_U );
			break;
	}

	// create the evil galaxy scourging Monsters (er, maybe not)
	MonsterTeam->mCreateShip( "M-AML", kStartPosition_V ); // Scovian, Ancient
	MonsterTeam->mCreateShip( "M-SGL", kStartPosition_W); // Yantani
	MonsterTeam->mCreateShip( "M-LCL", kStartPosition_X); // Iturahama
	MonsterTeam->mCreateShip( "M-MTL", kStartPosition_Y); // Zolati
	MonsterTeam->mCreateShip( "M-SSL", kStartPosition_Z); // Hanyurii


}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );
	tTeamInfo* HydranAllyTeam = fMission.mGetTeamHandle( kHydranAllyTeam );

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay1Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay2Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay3Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay4Count, 0 );
	}

	int32 MonsterNumber = 0;

	// configure Monster
	for( Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip() )
	{
		Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mChangeName( kMonster1_name + MonsterNumber );
		MonsterNumber++;
		Ship->mSetShipMaxSpeed( 12 );
	}

	// configure Worshiper
	for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay1Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay2Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay3Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay4Count, 0 );
	}

	// configure HydranAlly
	for (Ship = HydranAllyTeam->mGetFirstShip(); Ship != NULL; Ship = HydranAllyTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay1Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay2Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay3Count, 0 );
		Ship->mAccessSupplies( kSetFighterItem, kFighterBay4Count, 0 );
	}

	// Start a timer
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	fMission.mSetTimedInterval( kTimerTurnsSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* HydranAllyTeam = fMission.mGetTeamHandle( kHydranAllyTeam );
}


//---------------------------------------------------------------------------------------------
iTruth  IsTheGameOver(tMissionInfo* Mission)
{
	// Over because of thses reasons
	// 1. All Monsters dead or escaped
	// 2. All Enemies Dead
	tTeamInfo* WorshiperTeam = Mission->mGetTeamHandle( kWorshiperTeam );
	tTeamInfo* HydranAllyTeam = Mission->mGetTeamHandle( kHydranAllyTeam );
	tTeamInfo* MonsterTeam = Mission->mGetTeamHandle( kMonsterTeam );

	if(MonsterEscaped+MonsterDestroyed>=kTotalMonsters)
	{
		return kTrue;
	}

	if ( Loyalty == kMonsterLoyalty )
	{
		int AreTheyAllDead = 1;
		tShipInfo* Ship;
		for (Ship = HydranAllyTeam->mGetFirstShip(); Ship != NULL; Ship = HydranAllyTeam->mGetNextShip())
		{
			if(Ship->mInPlay())
			{
				AreTheyAllDead = 0;
			}
		}

		if( AreTheyAllDead )
		{
			return kTrue;
		}
	}else
	{
		// If you are a monster hater, you can't stop until you subdue all of the monsters that are left in the game.
		int AreTheyAllDead = 1;
		tShipInfo* Ship;
		for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
		{
			if(Ship->mInPlay())
			{
				AreTheyAllDead = 0;
			}
		}
		int AllMonstersSubdued = 0;

		if(MonstersSubdued > kTotalMonsters - MonsterDestroyed - MonsterEscaped)
		{
			AllMonstersSubdued = 1;
		}


		if( AreTheyAllDead && AllMonstersSubdued)
		{
			return kTrue;
		}

	}
	return kFalse;

}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );
	tTeamInfo* HydranAllyTeam = fMission.mGetTeamHandle( kHydranAllyTeam );

	tShipInfo* PlayersTarget = fMission.mGetShipTarget( fPlayerShip );

	switch( Event )
	{
		case kNotifyAtDestination:
			Ship->mDisengage(kByMapEdge,kAbsolutePriority);
			break;
		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				fMission.mDisplayMessage( kEvent3Cluster_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if(HostileShip && Ship && HostileShip->mGetTeam() == kPlayerTeam)
				{
					if(( Ship->mGetTeam() == kMonsterTeam ) ||
					   ( Ship->mGetTeam() == kWorshiperTeam ))
					{
						if( Loyalty == kMonsterLoyalty )
						{
							Loyalty = kHydranLoyalty;
							SwitchedLoyalty++;
							fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamAtWar, kGoalDestroy, 1.0 );
							fMission.mSetTeamRelations( kPlayerTeam, kWorshiperTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
							fMission.mSetTeamRelations( kPlayerTeam, kHydranAllyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
							fMission.mSetTeamRelations( kMonsterTeam, kHydranAllyTeam, kTeamAtWar, kGoalDestroy, 1.0 );
						}
					}
					else if( Ship->mGetTeam() == kHydranAllyTeam )
					{
						if( Loyalty == kHydranLoyalty )
						{
							Loyalty = kMonsterLoyalty;
							SwitchedLoyalty++;
							fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0 );
							fMission.mSetTeamRelations( kPlayerTeam, kWorshiperTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
							fMission.mSetTeamRelations( kPlayerTeam, kHydranAllyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
							fMission.mSetTeamRelations( kMonsterTeam, kHydranAllyTeam, kTeamAtWar, kGoalDestroy, 1.0 );
						}
					}

					if(( SwitchedLoyalty > 3 ) && ( Anarchy == false ))
					{
						// LNF what else does anarchy do?
						Anarchy = true;
					}
				}

				if(Ship->mGetTeam() == kMonsterTeam && Ship->mGetDamagePercent() > 50)
				{
					// Say the Monster is subdued.
					Ship->mSetShipAIControlled(kFalse);
					MonstersSubdued++;
					fMission.mDisplayMessage( kMonsterSubdued_msg, kScienceScriptMessage );
				}

				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyOnCapture:
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );

					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0);
					fMission.mSetTeamRelations (kPlayerTeam, kWorshiperTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kHydranAllyTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonsterTeam, kWorshiperTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kMonsterTeam, kHydranAllyTeam, kTeamHostile, kGoalDestroy, 1.0);
					fMission.mSetTeamRelations (kMonsterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);

					fMission.mSetTeamRelations (kHydranAllyTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0);

					fMission.mSetTimedInterval( kTimerLimitSet_2, 8 );
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kEvent4BlackHole_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_3:
					break;

				case kTimerTurnsSet:
					fMission.mSetTeamRelations( kHydranAllyTeam, kMonsterTeam, kTeamAtWar, kGoalDestroy, 1.0 );
					break;

				case kTimerTurnsSet_8:
					break;

				case kTimerTurnsSet_9:
					// set the Worshiper comm buttons back to HAIL
					for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return eGameStatus(kPlayerDestroyed | kCampaignCompleted);
				}
			}

			if (Ship->mGetTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;
			}

			if (Ship->mGetTeam() == kWorshiperTeam)
			{
				WorshiperDestroyed++;
				if (WorshiperTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kHydranAllyTeam)
			{
				HydranAllyDestroyed++;
				if (HydranAllyTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			if(IsTheGameOver(&fMission))
			{
				if(mCalcVictoryStatus() <= kDraw)
				{
					return eGameStatus(kPlayerWins| kCampaignCompleted);
				}else
				{
					return eGameStatus(kPlayerLoses| kCampaignCompleted);
				}
			}

			break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kMonsterTeam)
			{
				MonsterEscaped++;
			}
			if(IsTheGameOver(&fMission))
			{
				if(mCalcVictoryStatus() <= kDraw)
				{
					return eGameStatus(kPlayerWins| kCampaignCompleted);
				}else
				{
					return eGameStatus(kPlayerLoses| kCampaignCompleted);
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailAll_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kWorshiperTeam:
								if (( TalkedToWorshiper == 0 ) && ( Loyalty != kMonsterLoyalty ) &&
									( Anarchy == 0 ))
								{
									fMission.mDisplayMessage(kWorshiperHailNoTalk_msg, kMissionScriptMessage);
									TalkedToWorshiper = 1;
								}
								else if (( TalkedToWorshiper == 0 ) && ( Loyalty == kMonsterLoyalty ) &&
									( Anarchy == 0 ))
								{
									fMission.mDisplayMessage(kWorshiperHailNoTalkMonster_msg, kMissionScriptMessage);
									TalkedToWorshiper = 1;
								}
								else if (( TalkedToWorshiper == 1 ) && ( Loyalty != kMonsterLoyalty ) &&
									( Anarchy == 0 ))
								{
									fMission.mDisplayMessage(kWorshiperHailTalked_msg, kMissionScriptMessage);
								}
								else if (( TalkedToWorshiper == 1 ) && ( Loyalty == kMonsterLoyalty ) &&
									( Anarchy == 0 ))
								{
									fMission.mDisplayMessage(kWorshiperHailTalkedMonster_msg, kMissionScriptMessage);
								}
								else if ( Anarchy == 1 )
								{
									fMission.mDisplayMessage(kWorshiperHailAnarchy_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kHydranAllyTeam:
								if (( Loyalty != kMonsterLoyalty ) && ( Anarchy == 0 ))
								{
									if (Ship->mGetDamagePercent() < DamageThreshold )
									{
										fMission.mDisplayMessage(kHydranHailOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kHydranHailDamaged_msg, kMissionScriptMessage);
									}
								}
								else if (( Loyalty == kMonsterLoyalty ) || ( Anarchy == 1 ))
								{
									fMission.mDisplayMessage(kHydranHailAnarchy_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailAll_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kWorshiperTeam:
								if (( Loyalty == kMonsterLoyalty ) && ( Anarchy == 0 ))
								{
									fMission.mDisplayMessage(kWorshiperSurrenderMonster_msg, kMissionScriptMessage);
								}
								else if (( Loyalty != kMonsterLoyalty ) || ( Anarchy == 1 ))
								{
									if ( Ship->mGetDamagePercent() < DamageThreshold )
									{
										fMission.mDisplayMessage(kWorshiperSurrenderAnarchyOK_msg, kMissionScriptMessage);
									}
									else
									{
										//fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
									}
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kHydranAllyTeam:
								if (( Loyalty != kMonsterLoyalty ) && ( Anarchy == 0 ))
								{
									fMission.mDisplayMessage(kHydranSurrender_msg, kMissionScriptMessage);
								}
								else if (( Loyalty == kMonsterLoyalty ) || ( Anarchy == 1 ))
								{
									if ( Ship->mGetDamagePercent() < DamageThreshold )
									{
										fMission.mDisplayMessage(kHydranSurrenderAnarchyOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kHydranSurrenderAnarchy_msg, kMissionScriptMessage);
									}
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailAll_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kWorshiperTeam:
								if (( Loyalty == kMonsterLoyalty ) && ( Anarchy == 0 ))
								{
									//fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kWorshiperTaunt_1_msg, kWorshiperTaunt_3_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kHydranAllyTeam:
								if (( Loyalty == kMonsterLoyalty ) || ( Anarchy == 1 ))
								{
									if ( Ship->mGetDamagePercent() < DamageThreshold )
									{
										//fMission.mDisplayMessage(kOrionHail_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage (fMission.mRandomInt32 (kHydranTaunt_1_msg, kHydranTaunt_3_msg), kMissionScriptMessage);										
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus(status | kCampaignCompleted);
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );
	tTeamInfo* HydranAllyTeam = fMission.mGetTeamHandle( kHydranAllyTeam );
	

	if ( Loyalty == kMonsterLoyalty )
	{
	// check if the Hydran team doesn't exist
		int AreTheyAllDead = 1;
		tShipInfo* Ship;
		for (Ship = HydranAllyTeam->mGetFirstShip(); Ship != NULL; Ship = HydranAllyTeam->mGetNextShip())
		{
			if(Ship->mInPlay())
			{
				AreTheyAllDead = 0;
			}
		}

		if( AreTheyAllDead )
		{
			VictoryPoints = 100;
		}
	}
	else
	{
		// check if the monsters don't exist
		int AreTheyAllDead = 1;
		tShipInfo* Ship;
		for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
		{
			if(Ship->mInPlay())
			{
				AreTheyAllDead = 0;
			}
		}

		if( AreTheyAllDead )
		{
			VictoryPoints = 100;
		}

		VictoryPoints -= MonsterEscaped * 15;
	}

	if(MonsterDestroyed>=kTotalMonsters)
	{
		VictoryPoints -= 100;
	}else
	{
		VictoryPoints -= 3*MonsterDestroyed;
	}

	if(Anarchy == kTrue)
	{
		VictoryPoints -= 20;
	}

	if(VictoryPoints<0)
	{
		VictoryPoints = 0;
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	fMission.mAddToDebriefingList (kDismissedShort_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

