#include "scriptinterface.h"

#include "Hyd_1MissionText.h"
#include "Hyd_1MissionMaps.h"
#include "sfcAssert.h"


int32 PlayerDestroyed = 0;
int32 MonsterDestroyed = 0;
const float DamageThreshold = 30.0;
int32 AtWarWithPoacher = 0;
int32 MonsterKidnapped = 0;
int32 MonsterDeposited = 0;
int32 BetrayedTug = 0;
int32 PoacherSurrender = 0;

int32 kMineralFieldX = 6;
int32 kMineralFieldY = 34;
float kDepositeRange = 70.0f;
float kRemoveRange = 75.0f;
iTruth TugsRouted = kFalse;



enum eEntityIndex
{
	kInvalidIndex = -1,
	
	kMonster1 = 0,
	kMonster2,
	kMonster3,

	kMaxMonsterCount,

	kPoacher1 = 0,
	kPoacher2,
	kPoacher3,
	kPoacher4,

	kMaxPoacherCount
};

enum eMonsterState
{
	kDoingNothing,
	kHome,
	kDeadMonster,
	kTaken,
};

eMonsterState gMonsterState[kMaxMonsterCount];
int32 gMonsterIDs[kMaxMonsterCount];

struct tPoacherState
{
	tPoacherState()
		: fID(-1), fState(kDefending), fPoacherTarget(kInvalidIndex), fGotMonster(kFalse) 
	{}

	enum ePoacherState
	{
		kDefending,
		kPoaching,
		kGotMonster,
		kEscaped,
		kPoacherDead,
		kSurrendered,
	};

	iTruth mStillActive()
	{
		if( fState == kDefending || 
			fState == kPoaching || 
			fState == kGotMonster )
		{
			return kTrue;
		}
		else
		{
			return kFalse;
		}
	}

	int fID;
	ePoacherState fState;
	eEntityIndex fPoacherTarget;
	iTruth fGotMonster;

	
};
	

tPoacherState gPoacherState[kMaxPoacherCount];

int32 gAvailableToPoachCount = 0;
eEntityIndex gAvailableToPoach[kMaxMonsterCount];


eClassTypes PlayerShipClass= kClassShuttle;
int32 MonsterBeingTakenByPlayer = -1;

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

eEntityIndex GetMonsterID( tScript* script, int32 id );
eEntityIndex GetMonsterID( tScript* script, tShipInfo* ship );
eEntityIndex GetPoacherID( tScript* script, tShipInfo* ship );

void AddPoachTarget( eEntityIndex monster );
bool NewPoacherMission( tScript* script, tShipInfo* ship, int32 count, eEntityIndex* targetList );
void RemovePoachTarget( eEntityIndex monster );
eEntityIndex FindClosestTarget( tScript* script, tShipInfo* ship, int32 count = -1, eEntityIndex* targetList = NULL );
int32 PoacherTeamCount( tScript* script );

void PoachTarget( tScript* script, eEntityIndex poacher, eEntityIndex target );
void DefendTarget( tScript* script, eEntityIndex poacher, eEntityIndex target );

eGameStatus CheckMissionEndState( tScript* script );

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMonsterTeam, 
	kPoacherTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 160, 80, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMonsterTeam, kMonster);
	fMission.mSetTeamRace (kPoacherTeam, kFederation);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPoacherTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kPoacherTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	for( int i = 0; i < kMaxMonsterCount; i++ )
	{
		gMonsterState[i] = kDoingNothing;
		gMonsterIDs[i] = -1;
	} 

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* PoacherTeam = fMission.mGetTeamHandle (kPoacherTeam);
	
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.60f;
			// LNF should I put mCreate in here?
		}
		break;
	case 1:
		{
			// do nothing
			// LNF should I put mCreate in here?
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
			// LNF should I put mCreate in here?
		}
		break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Monster team
	// LNF is this the right monster?
	MonsterTeam->mCreateShip ("M-SSM", kStartPosition_T);
	MonsterTeam->mCreateShip ("M-SSM", kStartPosition_U);
	MonsterTeam->mCreateShip ("M-SSM", kStartPosition_V);

	// create Poacher team at least 4? separate teams?
	PoacherTeam->mCreateShip ("H-FQS",kStartPosition_W);
	PoacherTeam->mCreateShip ("H-FQS",kStartPosition_X);
	PoacherTeam->mCreateShip ("H-FQS",kStartPosition_Y);
	PoacherTeam->mCreateShip ("H-FQS",kStartPosition_Z);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* PoacherTeam = fMission.mGetTeamHandle (kPoacherTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	tShipInfo* playersShip = PlayerTeam->mGetFirstShip();;
	playersShip->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}

	// configure Monster
	for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
	{
		//Ship->mSetShipCallbackEvents (kNotifyDamage);
		eEntityIndex monster = GetMonsterID( this, Ship );
		if( monster == kInvalidIndex )
		{
			continue;
		}

		gMonsterIDs[monster] = Ship->mGetShipID();
		Ship->mTrackTacticalDistance( kDepositeRange, kMineralFieldX, kMineralFieldY, kTrue );
		Ship->mSetCommButton(1, kButtonHail_text);
		AddPoachTarget( monster );
	}

	// configure Poacher
	for (Ship = PoacherTeam->mGetFirstShip(); Ship != NULL; Ship = PoacherTeam->mGetNextShip())
	{
		eEntityIndex poacher = GetPoacherID( this, Ship );
		Assert( poacher != kInvalidIndex );
		
		gPoacherState[poacher].fID = Ship->mGetShipID();
		NewPoacherMission( this, Ship, gAvailableToPoachCount, gAvailableToPoach );
		
		if( poacher == kPoacher1 )
		{
			Ship->mSetCommButton(1, kButtonReceiveHail_text, kTrue);
		}
		else
		{
			Ship->mSetCommButton(1, kButtonHail_text);
		}

		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* PoacherTeam = fMission.mGetTeamHandle (kPoacherTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			Ship->mSetShipCallbackEvents(kNotifyEndTractor);
			if (Ship->mGetTeam() == kPoacherTeam)
			{
				eEntityIndex poacher = GetPoacherID( this, Ship );
				if( poacher == kInvalidIndex )
				{
					break;
				}

				gPoacherState[poacher].fState = tPoacherState::kGotMonster;

				fMission.mDisplayMessage( kEventKidnapped_msg, kScienceScriptMessage );
			}
			else if( Ship->mGetTeam() == kPlayerTeam && PlayersTarget->mGetTeam() == kMonsterTeam )
			{
				eEntityIndex monster = GetMonsterID( this, PlayersTarget );
				if( monster == kInvalidIndex )
				{
					break;
				}

				if( gMonsterState[monster] != kHome )
				{
					MonsterBeingTakenByPlayer = PlayersTarget->mGetShipID();
					fMission.mDisplayMessage( kEventTractoredScience_msg, kScienceScriptMessage );
					fMission.mSetGoalIndicator( kTrue, kMineralFieldX, kMineralFieldY ); 
					fMission.mDisplayMessage( kEventTractoredHelm_msg, kMissionScriptMessage );
				}
			}
			break;

		case kNotifyEndTractor:
			Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
			if (Ship->mGetTeam() == kPoacherTeam)
			{
				eEntityIndex poacher = GetPoacherID( this, Ship );
				if( poacher == kInvalidIndex )
				{
					break;
				}
				
				if( gPoacherState[poacher].fState == tPoacherState::kGotMonster )
				{
					
					gPoacherState[poacher].fState = tPoacherState::kPoaching;
				}
			}
			else if( Ship->mGetTeam() == kPlayerTeam )
			{
				MonsterBeingTakenByPlayer = -1;
				fMission.mSetGoalIndicator( kFalse, kMineralFieldX, kMineralFieldY );
			}
			break;
		

		case kNotifyAtDistance:
		{
			eEntityIndex monster = GetMonsterID( this, Ship );
			if( monster == kInvalidIndex )
			{
				break;
			}

			if( fEventID ) //Enter field 
			{
				gMonsterState[monster] = kHome;
			}
			else
			{
				gMonsterState[monster] = kDoingNothing;
			}

			if( gMonsterState[monster] == kHome )
			{
				if( Ship->mGetShipID() == MonsterBeingTakenByPlayer )
				{
					fMission.mSetGoalIndicator( kFalse, kMineralFieldX, kMineralFieldY );
				}

				fMission.mDisplayMessage( kEventDeposit_msg, kScienceScriptMessage );
				Ship->mTrackTacticalDistance( kRemoveRange, kMineralFieldX, kMineralFieldY, kTrue );
				MonsterDeposited++;
			}
			else
			{
				fMission.mDisplayMessage( kEventRemoved_msg, kScienceScriptMessage );
				Ship->mTrackTacticalDistance( kDepositeRange, kMineralFieldX, kMineralFieldY, kTrue );
				MonsterDeposited--;
			}
		}
		break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kPoacherTeam)
			{
				eEntityIndex poacher = GetPoacherID( this, Ship );
				if( poacher == kInvalidIndex )
				{
					break;
				}
				
				if( gPoacherState[poacher].fState == tPoacherState::kGotMonster )
				{ 
					gMonsterState[ gPoacherState[poacher].fPoacherTarget ] = kTaken;
					MonsterKidnapped++;
				}

				gPoacherState[poacher].fState = tPoacherState::kEscaped; 
			}

			// LNF if all poachers are disengaged, check win condition
			break;

		case kNotifyDamage:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mDisplayMessage( kStarting_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kStarting_1_msg, kMissionScriptMessage );

					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPoacherTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonsterTeam, kPoacherTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					break;

				case kTimerLimitSet_2:
					break;

				case kTimerLimitSet_3:
					// the win condition
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					break;

				case kTimerTurnsSet_9:
					// set the Poacher comm buttons back to HAIL
					for (Ship = PoacherTeam->mGetFirstShip(); Ship != NULL; Ship = PoacherTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kMonsterTeam )
			{
				eEntityIndex monster = GetMonsterID( this, Ship );
				if( monster == kInvalidIndex )
				{
					break;
				}

				MonsterDestroyed++;
				gMonsterState[monster] = kDeadMonster;

				//Clear the monster from the poachers list
				for( int poacher = 0; poacher < kMaxPoacherCount; poacher++ )
				{
					if( gPoacherState[poacher].fPoacherTarget == monster )
					{
						if( NewPoacherMission( this, Ship, gAvailableToPoachCount, gAvailableToPoach ) == false )
						{
							//No targets.. let's get out of here
							Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						}
					}
				}

				fMission.mDisplayMessage(kEventKilled_msg, kScienceScriptMessage);
			}
			else if (Ship->mGetTeam() == kPoacherTeam)
			{
				eEntityIndex deadPoacher = GetPoacherID( this, Ship );
				if( deadPoacher == kInvalidIndex )
				{
					break;
				}

				if( gPoacherState[deadPoacher].fState == tPoacherState::kPoaching || 
					gPoacherState[deadPoacher].fState == tPoacherState::kGotMonster )
				{
					iTruth reasignedTarget = kFalse;
					for( int poacher = 0; poacher < kMaxPoacherCount; poacher++ )
					{
						if( gPoacherState[poacher].fState == tPoacherState::kDefending )
						{
							PoachTarget( this, (eEntityIndex)poacher, gPoacherState[deadPoacher].fPoacherTarget );
							reasignedTarget = kTrue;
						}
					}

					if( !reasignedTarget )
					{
						AddPoachTarget( gPoacherState[deadPoacher].fPoacherTarget );
					}
				}

				gPoacherState[deadPoacher].fState = tPoacherState::kPoacherDead; 
			}

			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonReceiveHail_text:
						fMission.mDisplayMessage(kTugHailNoTalk_1_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kTugHailNoTalk_2_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kTugHailNoTalk_3_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kTugHailNoTalk_4_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonHail_text);
						break;

					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterAll_msg, kMissionScriptMessage);
								break;

							case kPoacherTeam:
								if (AtWarWithPoacher == 0)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kTugHailTalked_1_msg, kTugHailTalked_3_msg), kMissionScriptMessage);
								}
								else if ( BetrayedTug == 1 )
								{
									fMission.mDisplayMessage(kTugHailBetrayed_msg, kMissionScriptMessage);
								}
								else // ( AtWarWithPoacher == 1 )
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kTugHailCombat_1_msg, kTugHailCombat_3_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kPoacherTeam:
								if (Ship->mGetDamagePercent() < DamageThreshold )
								{
									fMission.mDisplayMessage(kTugSurrenderOK_msg, kMissionScriptMessage);
									Ship->mSetCommButton (2, kButtonSurrender_text);
								}
								else
								{
									eEntityIndex poacher = GetPoacherID( this, Ship );
									if( poacher == kInvalidIndex )
									{
										break;
									}

									fMission.mDisplayMessage(kTugSurrenderDamaged_msg, kMissionScriptMessage);
									
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
									PoacherSurrender++;
									gPoacherState[poacher].fState = tPoacherState::kSurrendered;
									Ship->mClearCommButton(1);
								}
								
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kPoacherTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kTugTaunt_1_msg, kTugTaunt_3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

						}
						break;
				}
			}
			break;
	}

	eGameStatus gameStatus = CheckMissionEndState( this );
	if( !TugsRouted && PoacherTeamCount( this ) == 0 )
	{
		TugsRouted = kTrue;
		
		fMission.mDisplayMessage( kAllTugsRouted_1, kMissionScriptMessage);
		if( gameStatus != kPlayerWins )
		{
			fMission.mDisplayMessage( kAllTugsRouted_2, kScienceScriptMessage);
		}
	}

	return gameStatus;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (MonsterDeposited * 30);
	VictoryPoints += PoacherSurrender * 5;

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if( VictoryPoints >= 90 )
	{
		MyVictoryLevel = kVictory;
	}
	else if( VictoryPoints >= 60 )
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	fMission.mAddToDebriefingList (kDismissedShort_msg);


	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	int32 FailureCount = 0;
	int32 SuccessCount = 0;

	( MonsterDeposited == 3 ) ? fMission.mAddToDebriefingList( kSavedAllMonsters ) : SuccessCount++;
	( MonsterDeposited > 0 && MonsterDeposited < 3 ) ? fMission.mAddToDebriefingList( kSavedSomeMonsters ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( MonsterDeposited == 0 ) ? fMission.mAddToDebriefingList( kLostedAllMonsters ) : FailureCount++;
	( MonsterDeposited > 0 && MonsterDeposited < 3 ) ? fMission.mAddToDebriefingList( kLostedSomeMonsters ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

eEntityIndex GetMonsterID( tScript* script, int32 id )
{
	return GetMonsterID( script, script->fMission.mFindShipID( id ) );
}

eEntityIndex GetMonsterID( tScript* script, tShipInfo* ship )
{
	tTeamInfo* MonsterTeam = script->fMission.mGetTeamHandle (kMonsterTeam);
	if( ship == MonsterTeam->mGetThisShip(1) )
	{
		return kMonster1;
	}
	else if( ship == MonsterTeam->mGetThisShip(2) )
	{
		return kMonster2;
	}
	else if( ship == MonsterTeam->mGetThisShip(3) )
	{
		return kMonster3;
	}

	Assert(kFalse);
	return kInvalidIndex;
}

eEntityIndex GetPoacherID( tScript* script, tShipInfo* ship )
{
	tTeamInfo* PoacherTeam = script->fMission.mGetTeamHandle( kPoacherTeam );
	if( ship == PoacherTeam->mGetThisShip(1) )
	{
		return kPoacher1;
	}
	else if( ship == PoacherTeam->mGetThisShip(2) )
	{
		return kPoacher2;
	}
	else if( ship == PoacherTeam->mGetThisShip(3) )
	{
		return kPoacher3;
	}
	else if( ship == PoacherTeam->mGetThisShip(4) )
	{
		return kPoacher4;
	}

	Assert(kFalse);
	return kInvalidIndex;
}

int32 PoacherTeamCount( tScript* script )
{
	int32 count = 0;
	for( int poacher = 0; poacher < kMaxPoacherCount; poacher++ )
	{
		if( gPoacherState[ poacher ].mStillActive() )
		{
			count++;
		}
	}

	return count;
}

eGameStatus CheckMissionEndState( tScript* script )
{
	tTeamInfo* PlayerTeam = script->fMission.mGetTeamHandle ( script->kPlayerTeam);
	tTeamInfo* MonsterTeam = script->fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* PoacherTeam = script->fMission.mGetTeamHandle (kPoacherTeam);
	
	if( MonsterTeam->mGetShipsInPlayCount() == 0 )
	{
		return kPlayerLoses;
	}
	else if( PoacherTeamCount( script ) == 0 )
	{
		for( int monster = 0; monster < kMaxMonsterCount; monster++ )
		{
			if( gMonsterState[ monster ] == kDoingNothing )
			{
				return kGameInProgress;
			}
		}

		return kPlayerWins;
	}

	return script->fMission.mGetGameStatus();
}

void AddPoachTarget( eEntityIndex monster )
{
	Assert( gAvailableToPoachCount < kMaxMonsterCount );
	if( gAvailableToPoachCount < kMaxMonsterCount )
	{
		gAvailableToPoach[gAvailableToPoachCount++] = monster;
	}
}

void RemovePoachTarget( eEntityIndex monster )
{
	for( int erase = 0; erase < gAvailableToPoachCount; erase++ )
	{
		if( gAvailableToPoach[erase] == monster )
		{
			//Found it
			break;			
		}
	}

	//Ship the id
	
	for( int i = erase + 1; i < gAvailableToPoachCount; i++ )
	{
		gAvailableToPoach[i-1] = gAvailableToPoach[i];
	}

	gAvailableToPoachCount--;
}

eEntityIndex FindClosestTarget( tScript* script, tShipInfo* ship, int32 count, eEntityIndex* targetList )
{
	int closestTarget = -1;
	float closestRange = -1.0f;

	if( targetList == NULL )
	{
		count = kMaxMonsterCount;
	}
	
	for( int target = 0; target < count; target++ )
	{
		int index;
		if( targetList )
		{
			index = targetList[target];
		}
		else
		{
			index = target;
		}

		if( gMonsterState[index] != kTaken && gMonsterState[index] != kDeadMonster )
		{
			int id = gMonsterIDs[ index ];
			tShipInfo* targetInfo = script->fMission.mFindShipID( id );
			float range = ship->mGetTacticalDistance( targetInfo );
			if( closestTarget == -1 || range < closestRange )
			{
				closestTarget = id;
				closestRange = range;
			}
		}
	}

	if( closestTarget != -1 )
	{
		return GetMonsterID( script, closestTarget );
	} 
	else
	{
		return kInvalidIndex;
	}
}

bool NewPoacherMission( tScript* script, tShipInfo* ship, int32 count, eEntityIndex* targetList )
{
	eEntityIndex poacher = GetPoacherID( script, ship );
	Assert( poacher != kInvalidIndex );
	if( poacher == kInvalidIndex ) 
	{
		return false;
	}

	eEntityIndex target = FindClosestTarget( script, ship, gAvailableToPoachCount, gAvailableToPoach );  
	if( target != kInvalidIndex )
	{
		PoachTarget( script, poacher, target );
	}
	else
	{
		eEntityIndex target = FindClosestTarget( script, ship );  
		if( target != kInvalidIndex )
		{
			DefendTarget( script, poacher, target );
		}
		else
		{
			return false;
		}

	}	

	return true;
}

void PoachTarget( tScript* script, eEntityIndex poacher, eEntityIndex target )
{
	gPoacherState[poacher].fPoacherTarget = target;
	gPoacherState[poacher].fState = tPoacherState::kPoaching;
	int32 id = gMonsterIDs[ target ];
	
	tShipInfo* ship = script->fMission.mFindShipID( gPoacherState[poacher].fID );
	Assert( ship );
	ship->mTractorEntity( script->fMission.mFindShipID( id ) ,41, 20, kAbsolutePriority );		
	RemovePoachTarget( target );
}

void DefendTarget( tScript* script, eEntityIndex poacher, eEntityIndex target )
{
	gPoacherState[poacher].fPoacherTarget = target;
	gPoacherState[poacher].fState = tPoacherState::kDefending;
	int32 id = gMonsterIDs[ target ];
	
	tShipInfo* ship = script->fMission.mFindShipID( gPoacherState[poacher].fID );
	Assert( ship );
	ship->mDefendEntity( script->fMission.mFindShipID( id ), 25.0f, 30.0f, kAbsolutePriority );
}

