#include "scriptinterface.h"

#include "Hyd_2MissionText.h"
#include "Hyd_2MissionMaps.h"

int32 PlayerDestroyed = 0;
int32 MuriyzDestroyed = 0;
int32 CurrentGenerator = 0;
int32 NumGeneratorsDestroyed = 0;
float DamageThreshold = 50.0;
bool GeneratorRepaired[3] = { false, false, false };
bool GeneratorDestroyed[3] = { false, false, false };
bool MonsterDestroyed = false;
bool BetrayedMuriyz = false;
bool FiredOnSpaceMonster = false;
bool TalkedWithMuriyz = false;
bool AtWarWithMuriyz = false;
bool AllyDestroyed = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kMuriyzTeam,
	kGeneratorTeam,
	kAlliedTeam,
	kMonsterTeam
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAlliedTeam, kHydran );
	fMission.mSetTeamRace( kMuriyzTeam, kHydran );
	fMission.mSetTeamRace( kGeneratorTeam, kHydran );
	fMission.mSetTeamRace( kMonsterTeam, kMonster );

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kMuriyzTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kGeneratorTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0 );

	fMission.mSetTeamRelations( kAlliedTeam, kMuriyzTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kAlliedTeam, kGeneratorTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kAlliedTeam, kMonsterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	fMission.mSetTeamRelations( kMuriyzTeam, kGeneratorTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kMuriyzTeam, kMonsterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	fMission.mSetTeamRelations( kGeneratorTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kMonsterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0 );

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* MuriyzTeam = fMission.mGetTeamHandle( kMuriyzTeam );
	tTeamInfo* GeneratorTeam = fMission.mGetTeamHandle( kGeneratorTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	
	// balance
	int32 DiffLev = fMission.mGetDifficultyLevel();

	switch ( DiffLev )
	{
		case 0:
			{
				MuriyzTeam->mCreateShip( kMaxBPV, kClassFrigate, 1, kStartPosition_M );
				MuriyzTeam->mCreateShip( "H-ST16", kStartPosition_N );
			}
			break;

		case 1:
			{
				MuriyzTeam->mCreateShip( kMaxBPV, kClassFrigate, 1, kStartPosition_M );
				MuriyzTeam->mCreateShip( "H-ST16", kStartPosition_N );
				MuriyzTeam->mCreateShip( "H-ST16", kStartPosition_O );
			}
			break;

		case 2:
			{
				MuriyzTeam->mCreateShip( kMaxBPV, kClassFrigate, 0, kStartPosition_M );
				MuriyzTeam->mCreateShip( "H-ST16", kStartPosition_N );
				MuriyzTeam->mCreateShip( "H-ST16", kStartPosition_O );
				MuriyzTeam->mCreateShip( kMaxBPV, kClassFrigate, 50, kStartPosition_P );
			}
			break;
	}

	// create Generator
	GeneratorTeam->mCreateShip( "H-LP", kStartPosition_U );
	GeneratorTeam->mCreateShip( "H-LP", kStartPosition_V );
	GeneratorTeam->mCreateShip( "H-LP", kStartPosition_W );

	// create Monster
	MonsterTeam->mCreateShip( "M-MTM", kStartPosition_T);

	// create Ally
	AlliedTeam->mCreateShip( kMaxBPV, kClassDestroyer, 10, kStartPosition_J );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* MuriyzTeam = fMission.mGetTeamHandle( kMuriyzTeam );
	tTeamInfo* GeneratorTeam = fMission.mGetTeamHandle( kGeneratorTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransScientists, 1 );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo | kNotifyTransportedItemFrom );
	}

	// configure Allied Ship
	Ship = AlliedTeam->mGetFirstShip();
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mSetCommButton( 1, kButtonHail_text );

	// configure Muriyz
	for( Ship = MuriyzTeam->mGetFirstShip(); Ship != NULL; Ship = MuriyzTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mSetShipMaxSpeed( 20 );
	}

	int32 TempNum = 0;

	// configure Generator
	for( Ship = GeneratorTeam->mGetFirstShip(); Ship != NULL; Ship = GeneratorTeam->mGetNextShip() )
	{
		Ship->mSetShipAIControlled( kFalse );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mChangeName( kSubspace1_name + TempNum );
		TempNum++;
	}

	// configure Monster
	Ship = MonsterTeam->mGetFirstShip();
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mChangeName( kMonster_name );

	// Start a timer
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* MuriyzTeam = fMission.mGetTeamHandle( kMuriyzTeam );
	tTeamInfo* GeneratorTeam = fMission.mGetTeamHandle( kGeneratorTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );

	tShipInfo* PlayersTarget = fMission.mGetShipTarget( fPlayerShip );

	switch( Event )
	{
		case kNotifyTransportedItemTo:
			if(( PlayersTarget->mGetTeam() == kGeneratorTeam ) && ( fEventItem1 == kTransScientists ))
			{
				CurrentGenerator = PlayersTarget->mGetStartPositionIndex() - kStartPosition_U;

				if( GeneratorRepaired[ CurrentGenerator ] == false )
				{
					fMission.mDisplayMessage( kEventLanded_msg, kMissionScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
				}
				else
				{
					fMission.mDisplayMessage( kEventRepaired_msg, kMissionScriptMessage );
				}
			}

			Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
			break;

		case kNotifyTransportedItemFrom:
			if(( PlayersTarget->mGetTeam() == kGeneratorTeam ) && ( fEventItem1 == kTransScientists ))
			{
				fMission.mDisplayMessage( kEventRetrieved_msg, kMissionScriptMessage );
			}

			Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kMonsterTeam )
			{
				FiredOnSpaceMonster = true;
				fMission.mDisplayMessage( kEventAttackedMonster_msg, kMissionScriptMessage );
				fMission.mSetTeamRelations( kMonsterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
				fMission.mSetTeamRelations( kMonsterTeam, kMuriyzTeam, kTeamAtWar, kGoalDestroy, 1.0 );
			}

			if( Ship->mGetTeam() == kMuriyzTeam )
			{
				AtWarWithMuriyz = true;
				fMission.mSetTeamRelations( kPlayerTeam, kMuriyzTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
				fMission.mSetTeamRelations( kAlliedTeam, kMuriyzTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kMuriyzTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kScienceScriptMessage );

					// Set team relations
					fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kMuriyzTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kGeneratorTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0 );

					fMission.mSetTeamRelations( kAlliedTeam, kMuriyzTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kAlliedTeam, kGeneratorTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kAlliedTeam, kMonsterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kMuriyzTeam, kGeneratorTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kMuriyzTeam, kMonsterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kGeneratorTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kMonsterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0 );
					break;

				case kTimerTurnsSet:
					GeneratorRepaired[ CurrentGenerator ] = true;
					fMission.mDisplayMessage( kRepairedGenerator1_msg + CurrentGenerator, kScienceScriptMessage );
					break;

				case kTimerTurnsSet_9:
					// set the Monster comm buttons back to HAIL
					for( Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip() )
					{
						Ship->mClearCommButton( 2 );
						Ship->mSetCommButton( 1, kButtonHail_text );
					}
					break;

				case kTimerTurnsSet_10:
					// set the Muriyz comm buttons back to HAIL
					for( Ship = MuriyzTeam->mGetFirstShip(); Ship != NULL; Ship = MuriyzTeam->mGetNextShip() )
					{
						Ship->mClearCommButton( 2 );
						Ship->mSetCommButton( 1, kButtonHail_text );
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerDestroyed++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}

			if( Ship->mGetTeam() == kMuriyzTeam )
			{
				MuriyzDestroyed++;

				if( MuriyzTeam->mGetShipsInPlayCount() == 0 )
				{
					bool Winner = true;
					int32 TempNum = 0;

					for( TempNum; TempNum > 2; TempNum++ )
					{
						if( GeneratorDestroyed[ TempNum ] == true )
						{
							Winner = false;
						}

						if(( GeneratorRepaired[ TempNum ] == false ) && ( GeneratorDestroyed[ TempNum ] == false ))
						{
							GeneratorRepaired[ TempNum ] = true;
						}
					}

					if( Winner == true  )
					{
						return kPlayerWins;
					}
					else
					{
						fMission.mDisplayMessage( kMuriyzDead_msg, kScienceScriptMessage );
					}
				}
			}

			if( Ship->mGetTeam() == kGeneratorTeam )
			{
				int32 DestroyedGenerator = Ship->mGetStartPositionIndex() - kStartPosition_U;

				GeneratorDestroyed[ DestroyedGenerator ] = true;
				GeneratorRepaired[ DestroyedGenerator ] = false;
				
				NumGeneratorsDestroyed++;

				fMission.mDisplayMessage( kEventDestroyed_msg, kScienceScriptMessage, Ship );

				if( Ship->mAccessSupplies( kGetTransporterItemCount, kTransScientists ) > 0 )
				{
					fMission.mDisplayMessage( kEventLostTeam_msg, kScienceScriptMessage );
				}

				if( GeneratorTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}

			if( Ship->mGetTeam() == kMonsterTeam )
			{
				MonsterDestroyed = true;
				fMission.mDisplayMessage( kEventKilledMonster_msg, kMissionScriptMessage );
			}

			if( Ship->mGetTeam() == kAlliedTeam )
			{
				AllyDestroyed = true;
			}
			break;

		case kNotifyPlayerLeaving:
			Ship = fPlayerShip;
			// case falls through
		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					bool Winner = true;
					int32 TempNum = 0;

					for( TempNum; TempNum > 2; TempNum++ )
					{
						if( GeneratorDestroyed[ TempNum ] == true )
						{
							Winner = false;
						}

						if( GeneratorRepaired[ TempNum ] == false )
						{
							Winner = false;
						}
					}

					if( MonsterDestroyed == true )
					{
						Winner = false;
					}

					if( Winner == false )
					{
						return kPlayerLoses;
					}
					else
					{
						return kPlayerWins;
					}
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch( fEventID )
				{
					case kButtonDisengage_text:
						fMission.mDisplayMessage( kAllyDisengage_msg, kFleetScriptMessage, Ship );
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
						break;

					case kButtonAttack_text:
						if( MuriyzTeam->mGetShipsInPlayCount() == 0 )
						{
							fMission.mDisplayMessage( kAllyNoAttack_msg, kFleetScriptMessage, Ship );
						}
						else
						{
							fMission.mDisplayMessage( kAllyAttacking_msg, kFleetScriptMessage, Ship );
							fMission.mSetTeamRelations( kAlliedTeam, kMuriyzTeam, kTeamAtWar, kGoalDestroy, 1.0 );
						}
						break;

					case kButtonHail_text:
						switch( Ship->mGetTeam() )
						{
							case kAlliedTeam:
								fMission.mDisplayMessage( kAllyHail_msg, kFleetScriptMessage, Ship );
								Ship->mSetCommButton( 1, kButtonDisengage_text );
								Ship->mSetCommButton( 2, kButtonAttack_text );
								break;

							case kMuriyzTeam:
								if(( AtWarWithMuriyz == false ) && ( TalkedWithMuriyz == false ))
								{
									fMission.mDisplayMessage( kMuriyzHailNic_1_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kMuriyzHailNic_2_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kMuriyzHailNic_3_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kMuriyzHailNic_4_msg, kMissionScriptMessage );
									fMission.mDisplayMessage( kMuriyzHailNic_5_msg, kMissionScriptMessage );
									TalkedWithMuriyz = true;
								}
								else if(( AtWarWithMuriyz == false ) && ( TalkedWithMuriyz == true ))
								{
									fMission.mDisplayMessage( fMission.mRandomInt32( kMuriyzHailTalked_1_msg, kMuriyzHailTalked_3_msg ), kMissionScriptMessage );
								}
								else if( BetrayedMuriyz == true )
								{
									fMission.mDisplayMessage( kMuriyzHailBetrayed_msg, kMissionScriptMessage );
								}
								else if( AtWarWithMuriyz == true )
								{
									fMission.mDisplayMessage( fMission.mRandomInt32( kMuriyzHailCombat_1_msg, kMuriyzHailCombat_3_msg ), kMissionScriptMessage );
								}

								Ship->mSetCommButton( 1, kButtonTaunt_text );
								Ship->mSetCommButton( 2, kButtonSurrender_text );

								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval( kTimerTurnsSet_10, 2 );
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage( kMonsterAll_msg, kMissionScriptMessage );
								Ship->mSetCommButton( 1, kButtonTaunt_text );
								Ship->mSetCommButton( 2, kButtonSurrender_text );

								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval( kTimerTurnsSet_9, 2 );
								break;
						}
						break;

					case kButtonSurrender_text:
						switch( Ship->mGetTeam() )
						{
							case kMuriyzTeam:
								if( Ship->mGetDamagePercent() < DamageThreshold )
								{
									fMission.mDisplayMessage( kMuriyzSurrenderOK_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kMuriyzSurrenderDamaged_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton( 2, kButtonSurrender_text );
								fMission.mSetTimedInterval( kTimerTurnsSet_10, 2 );
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage( kMonsterAll_msg, kMissionScriptMessage );
								Ship->mSetCommButton( 2, kButtonSurrender_text );
								fMission.mSetTimedInterval( kTimerTurnsSet_9, 2 );
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kMuriyzTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kMuriyzTaunt_1_msg, kMuriyzTaunt_3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterAll_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 100 - ( NumGeneratorsDestroyed * 23 );
	int32 TempNum = 0;

	if( MonsterDestroyed == true )
	{
		VictoryPoints -= 31;
	}

	if( AllyDestroyed == true )
	{
		VictoryPoints -= 31;
	}

	MyVictoryLevel = kDraw;

	if( VictoryPoints == 100 )
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if(( VictoryPoints <= 99 ) && ( VictoryPoints >= 75 ))
	{
		MyVictoryLevel = kVictory;
	}
	else if( VictoryPoints <= 24 )
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	return( MyVictoryLevel );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );

	fMission.mAddToDebriefingList( kDBGeneratorDestroyed_0_msg + NumGeneratorsDestroyed );

	if( MonsterDestroyed == true )
	{
		fMission.mAddToDebriefingList( kKilledMonster_msg );
	}

	fMission.mAddToDebriefingList( kMessage2_msg );
	fMission.mAddToDebriefingList( kDismissedShort_msg );

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
*/

	fMission.mScheduleMission( kNextMission_msg );

	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch( NumberOfSpacerMissions )
	{
		case 0:
			fMission.mScheduleMission( kRandomMission_msg );
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
