#include "scriptinterface.h"

#include "Hyd_3MissionText.h"
#include "Hyd_3MissionMaps.h"


int32 PlayerDestroyed = 0;
int32 SpyDestroyed = 0;
int32 LpDestroyed = 0;
int32 DiffLevel = 0;
const float DamageThreshold = 30.0;
const int32 SelfDestruct = 3;	// in turns
bool AtWarWithSpy = false;
bool DataRetrieved = false;
bool PlayerLoses = false;

enum eEnemyRaces
{
	kRace_Lyran	= 0,
	kRace_Klingon,
};

eEnemyRaces EnemyRace = kRace_Lyran;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kSpyTeam, 
	kLpTeam,
	kDefSatTeam,
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kKlingMission | kLyranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 200, 160, 100, 40, 0 );

	// must set up the races of the teams in the script
	if( info.fEnemyRace == kKlingon )
	{
		EnemyRace = kRace_Klingon;
	}
	else
	{
		info.fEnemyRace = kLyran;
	}

	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kSpyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kLpTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kDefSatTeam, info.fEnemyRace );

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations( kPlayerTeam, kSpyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kLpTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kDefSatTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

	fMission.mSetTeamRelations( kSpyTeam, kLpTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kSpyTeam, kDefSatTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	fMission.mSetTeamRelations( kLpTeam, kDefSatTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* SpyTeam = fMission.mGetTeamHandle( kSpyTeam );
	tTeamInfo* LpTeam = fMission.mGetTeamHandle( kLpTeam );
	tTeamInfo* DefSatTeam = fMission.mGetTeamHandle( kDefSatTeam );
	
	// balance
	DiffLevel = fMission.mGetDifficultyLevel();

	if( Mode == 0 )
	{
		if( EnemyRace == kRace_Lyran )
		{
			LpTeam->mCreateShip( "L-LP", kStartPosition_M );
			DefSatTeam->mCreateShip( "L-DEF", kStartPosition_N );
			DefSatTeam->mCreateShip( "L-DEF", kStartPosition_O );
		}
		else
		{
			LpTeam->mCreateShip( "K-LP", kStartPosition_M );
			DefSatTeam->mCreateShip( "K-DEF", kStartPosition_N );
			DefSatTeam->mCreateShip( "K-DEF", kStartPosition_O );
		}

		SpyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_Z );

		switch( DiffLevel )
		{
			case 0:
				{
					SpyTeam->mCreateShip( kMaxBPV, kClassFrigate, 1, kStartPosition_Y );
				}
				break;

			case 1:
				{
					SpyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 0, kStartPosition_Y );
				}
				break;

			case 2:
				{
					SpyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 1, kStartPosition_Y );
				}
				break;
		}
	}
	else	// reinforcements
	{
		switch( DiffLevel )
		{
			case 0:
				SpyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 50, kStartPosition_X );
				break;

			case 1:
				SpyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 25, kStartPosition_X );
				break;

			case 2:
				SpyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_X );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* SpyTeam = fMission.mGetTeamHandle( kSpyTeam );
	tTeamInfo* LpTeam = fMission.mGetTeamHandle( kLpTeam );
	tTeamInfo* DefSatTeam = fMission.mGetTeamHandle( kDefSatTeam );

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure Spy Team
	for( Ship = SpyTeam->mGetFirstShip(); Ship != NULL; Ship = SpyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	// configure Comm Station
	Ship = LpTeam->mGetFirstShip();
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mChangeName( kComm_name );
 
	// Start a timer
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// initial message/relationships
	fMission.mSetTimedInterval( kTimerTurnsSet, 15 );	// enemy reinforcements
	fMission.mSetTimedInterval( kTimerTurnsSet_4, 6 - DiffLevel );
}	

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mSetCommButton( 1, kButtonHail_text );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* SpyTeam = fMission.mGetTeamHandle( kSpyTeam );
	tTeamInfo* LpTeam = fMission.mGetTeamHandle( kLpTeam );
	tTeamInfo* DefSatTeam = fMission.mGetTeamHandle( kDefSatTeam );

	tShipInfo* PlayersTarget = fMission.mGetShipTarget( fPlayerShip );

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			break;

		case kNotifyTransportedItemFrom:
			break;

		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDeepScan:
			if (fIsDeepScanOn == kTrue)
			{
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kLpTeam )
			{
				int32 DiffLevel = fMission.mGetDifficultyLevel();
				float BaseThreshold = 0.0;

				switch( DiffLevel )
				{
					case 0:
						BaseThreshold = 90.0;
						break;

					case 1:
						BaseThreshold = 80.0;
						break;

					case 2:
						BaseThreshold = 70.0;
						break;
				}

				if( Ship->mGetDamagePercent() > BaseThreshold )
				{
					fMission.mSetTimedInterval( kTimerCountdownSet, 5 );
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kLpTeam )
			{
				fMission.mDisplayMessage( kEvent1Control_msg, kMissionScriptMessage );
				fMission.mSetTimedInterval( kTimerTurnsSet_3, 2 ); // time it takes to crack computer
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );

					// Set team relations
					fMission.mSetTeamRelations( kPlayerTeam, kSpyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kLpTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kDefSatTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

					fMission.mSetTeamRelations( kSpyTeam, kLpTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kSpyTeam, kDefSatTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kLpTeam, kDefSatTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;

				case kTimerTurnsSet:	// enemy reinforcements
					if( SpyTeam->mGetShipsInPlayCount() > 1 )
					{
						fMission.mSetTimedInterval( kTimerTurnsSet, 2 );
					}
					else
					{
						mCreateMissionShips( 1 );
						fMission.mDisplayMessage( kEnemyReinforcements_msg, kScienceScriptMessage );
						int32 RandomTurns = fMission.mRandomInt32( 8, 12 );
						fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
					}
					break;

				case kTimerTurnsSet_3:
					if( LpDestroyed == 0 )
					{
						// retrieved the data
						fMission.mDisplayMessage( kEvent2Retrieved_msg, kMissionScriptMessage );
						DataRetrieved = true;
					}
					break;

				case kTimerTurnsSet_4:
					{
						int32 RandomDamage = fMission.mRandomInt32( 1, 4 + DiffLevel );
						int32 RandomMessage = fMission.mRandomInt32( 0, 2 );
						int32 RandomDelay = fMission.mRandomInt32( 2, 6 );
						fPlayerShip->mDamageSystem( kDamageInternalsWithPoints, RandomDamage );
						fMission.mDisplayMessage( kDischarge_1_msg + RandomMessage, kScienceScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_4, RandomDelay );
					}
					break;

				case kTimerCountdownSet:
					// self destruct timer blow up listening post
					Ship = LpTeam->mGetFirstShipInPlay();

					if( Ship != NULL )
					{
						Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
						fMission.mDisplayMessage( kBaseGoBoom_msg, kScienceScriptMessage );
					}
					break;

				case kTimerTurnsSet_10:
					// set the Spy comm buttons back to HAIL
					for (Ship = SpyTeam->mGetFirstShip(); Ship != NULL; Ship = SpyTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerDestroyed++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}

			if( Ship->mGetTeam() == kSpyTeam )
			{
				SpyDestroyed++;

				if( SpyTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mDisplayMessage( kOnceWeGet_msg, kScienceScriptMessage );
				}
			}

			if( Ship->mGetTeam() == kLpTeam )
			{
				LpDestroyed++;

				if( DataRetrieved == false )
				{
					return( kPlayerLoses );
				}
			}

			if( Ship->mGetTeam() == kDefSatTeam )
			{
				if( DefSatTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mDisplayMessage( kLastDefSat_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					if( DataRetrieved == false )
					{
						PlayerLoses = true;
						return kPlayerLoses;
					}
					else
					{
						return kPlayerWins;
					}
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerLoses = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch( fEventID )
				{
					case kButtonHail_text:
						if( EnemyRace == kRace_Klingon )
						{
							if( AtWarWithSpy == false )
							{
								fMission.mDisplayMessage( kKliHailNic_msg, kFleetScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( fMission.mRandomInt32( kKliHailCombat_1_msg, kKliHailCombat_3_msg ), kFleetScriptMessage );
							}
						}
						else	// Lyran
						{
							if( AtWarWithSpy == false )
							{
								fMission.mDisplayMessage( kLyrHailNic_msg, kFleetScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( fMission.mRandomInt32( kLyrHailCombat_1_msg, kLyrHailCombat_3_msg ), kFleetScriptMessage );
							}
						}
						Ship->mSetCommButton( 1, kButtonTaunt_text );
						Ship->mSetCommButton( 2, kButtonSurrender_text );
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval( kTimerTurnsSet_10, 2 );
						break;

					case kButtonSurrender_text:
						if( EnemyRace == kRace_Klingon )
						{
							if( Ship->mGetDamagePercent() < DamageThreshold )
							{
								fMission.mDisplayMessage( kKliSurrenderOK_msg, kMissionScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( kKliSurrenderDamaged_msg, kMissionScriptMessage );
							}
						}
						else // kLyran
						{
							if( Ship->mGetDamagePercent() < DamageThreshold )
							{
								fMission.mDisplayMessage( kLyrSurrenderOK_msg, kMissionScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( kLyrSurrenderDamaged_msg, kMissionScriptMessage );
							}
						}
						Ship->mSetCommButton( 2, kButtonSurrender_text );
						fMission.mSetTimedInterval( kTimerTurnsSet_10, 2 );
						break;

					case kButtonTaunt_text:
						if( EnemyRace == kRace_Klingon )
						{
							fMission.mDisplayMessage( kKliTaunt_msg, kMissionScriptMessage );
						}
						else 
						{
							fMission.mDisplayMessage( kLyrTaunt_msg, kMissionScriptMessage );
						}
						Ship->mSetCommButton( 1, kButtonTaunt_text );
						fMission.mSetTimedInterval( kTimerTurnsSet_10, 2 );
						break;
				}
			}
			break;
	}

	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if(( DataRetrieved == true ) && ( PlayerLoses == false ))
	{
		VictoryPoints = 100;
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return( MyVictoryLevel );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	fMission.mScheduleMission( kNextMission_msg );

	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );

	switch (NumberOfSpacerMissions)
	{
		case 0:
			fMission.mScheduleMission( kRandomMission_msg );
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

