#include "scriptinterface.h"

#include "Hyd_4MissionText.h"
#include "Hyd_4MissionMaps.h"


int32 PlayerDestroyed = 0;
int32 MonsterDestroyed = 0;
int32 PoacherDestroyed = 0;
int32 WorshiperDestroyed = 0;
const float DamageThreshold = 30.0;
bool AtWarWithWorshiper = false;
bool AtWarWithPoacher = false;
bool TalkedToWorshiper = false; // Worshiper == Muriyz
bool Plunged = false;
bool PlayerLoses = false;
int32 CallOfMonster = 0;
int32 AttackedAlly = 0;
int32 InTooDeep = 0;
int32 FinalWord = 0;
int32 AttackedMonster = 0;
int32 NoWorshiper = 0;
int32 SunX = 0;
int32 SunY = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMonsterTeam, 
	kPoacherTeam,
	kWorshiperTeam
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 200, 160, 100, 40, 0 );

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kMonsterTeam, kMonster );
	fMission.mSetTeamRace( kPoacherTeam, kHydran );
	fMission.mSetTeamRace( kWorshiperTeam, kHydran );

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kPoacherTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kWorshiperTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

	fMission.mSetTeamRelations( kMonsterTeam, kPoacherTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kMonsterTeam, kWorshiperTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	fMission.mSetTeamRelations( kPoacherTeam, kWorshiperTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	tTeamInfo* PoacherTeam = fMission.mGetTeamHandle( kPoacherTeam );
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );
	
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .9;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.4;
	int32 TotalBPV = fMission.mRandomInt32( LowBPV, HighBPV );
	int32 AverageBPV = TotalBPV / PlayerTeam->mGetShipsInPlayCount();
	int32 EnemyBPV = 0;

	// balance
	int32 DiffLevel = fMission.mGetDifficultyLevel();

	switch ( DiffLevel )
	{
	case 0:
		{
			EnemyBPV = AverageBPV * 2;
			PoacherTeam->mCreateFleet( 2, EnemyBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_W);
			WorshiperTeam->mCreateFleet( 2, EnemyBPV * 0.8, kClassFrigate, kClassFrigate, 50, kStartPosition_T);
		}
		break;
	case 1:
		{
			EnemyBPV = AverageBPV * 3;
			PoacherTeam->mCreateFleet( 3, EnemyBPV, kClassDestroyer, kClassLightCruiser, 50, kStartPosition_W);
			WorshiperTeam->mCreateFleet( 3, EnemyBPV * 0.8, kClassFrigate, kClassFrigate, 50, kStartPosition_T);
		}
		break;
	case 2:
		{
			EnemyBPV = AverageBPV * 3;
			PoacherTeam->mCreateFleet( 3, EnemyBPV, kClassLightCruiser, kClassHeavyCruiser, 50, kStartPosition_W);
			WorshiperTeam->mCreateFleet( 3, EnemyBPV * 0.8, kClassFrigate, kClassDestroyer, 50, kStartPosition_T);
		}
		break;
	}

	// create Monster
	MonsterTeam->mCreateShip( "M-SGL", kStartPosition_Z );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	tTeamInfo* PoacherTeam = fMission.mGetTeamHandle( kPoacherTeam );
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );

	tShipInfo* Ship;

	fMission.mGetGridPosition( '7', &SunX, &SunY );

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}

	// configure Monster
	for( Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mTrackTacticalDistance( 50.0, SunX, SunY );
		Ship->mSetFinalDestination( SunX, SunY );
		Ship->mSetShipThrust( 5 );
		Ship->mSetShipMaxSpeed( 8 );
		Ship->mChangeName( kCtalhru_name );
	}

	// configure Poacher
	for( Ship = PoacherTeam->mGetFirstShip(); Ship != NULL; Ship = PoacherTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	// configure Worshiper
	for( Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip() )
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	// Start a timer
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle( kMonsterTeam );
	tTeamInfo* PoacherTeam = fMission.mGetTeamHandle( kPoacherTeam );
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle( kWorshiperTeam );

	tShipInfo* PlayersTarget = fMission.mGetShipTarget( fPlayerShip );

	switch( Event )
	{
		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
			}
			break;

		case kNotifyDamage:
			if( PlayersTarget != NULL )
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if(HostileShip && Ship && HostileShip->mGetTeam() == kPlayerTeam)
				{
					if(( PlayersTarget->mGetTeam() == kWorshiperTeam) && ( CallOfMonster == 1 ))
					{
						AtWarWithWorshiper = true;
						AttackedAlly = 1;
					}
					else if( PlayersTarget->mGetTeam() == kMonsterTeam )
					{
						AttackedMonster = 1;
					}
					else if( PlayersTarget->mGetTeam() == kPoacherTeam )
					{
						AtWarWithPoacher = true;

						if( TalkedToWorshiper == true )
						{
							fMission.mDisplayMessage( kWorshiperHailBetrayal_msg );
						}
					}
				}
			}
			break;

		case kNotifyAtDistance:
			if( Ship->mGetTrackingDistance() == 50.0 )
			{
				fMission.mDisplayMessage( kEvent3N2Deep_msg, kMissionScriptMessage );
				InTooDeep = 1;
				Ship->mTrackTacticalDistance( 20.0, SunX, SunY );
				PoacherTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
			}
			else if( Ship->mGetTrackingDistance() == 20.0 )
			{
				Plunged = true;
				fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
				Ship->mDamageSystem( kDamageWeaponsAndSystems, 98.0 );
			}
			break;

		case kNotifyOnCapture:
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

					// Set team relations
					fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kPoacherTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kWorshiperTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue );

					fMission.mSetTeamRelations( kMonsterTeam, kPoacherTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kMonsterTeam, kWorshiperTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

					fMission.mSetTeamRelations( kPoacherTeam, kWorshiperTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					break;

				case kTimerLimitSet_2:
					Ship = MonsterTeam->mGetFirstShipInPlay();
					if( Ship != NULL )
					{
						Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
					}
					break;

				case kTimerLimitSet_3:
					return kPlayerWins;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					// set the Poacher comm buttons back to HAIL
					for (Ship = PoacherTeam->mGetFirstShip(); Ship != NULL; Ship = PoacherTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Worshiper comm buttons back to HAIL
					for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			if (Ship->mGetTeam() == kMonsterTeam)
			{
				if( Plunged == true )
				{
					fMission.mDisplayMessage( kEvent4Plunged_msg, kMissionScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_3, 4 );
				}
				else
				{
					PlayerLoses = true;
					return kPlayerLoses;
				}
			}

			if (Ship->mGetTeam() == kPoacherTeam)
			{
				PoacherDestroyed++;
				if (PoacherTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kWorshiperTeam)
			{
				WorshiperDestroyed++;
				if (WorshiperTeam->mGetShipsInPlayCount() == 0)
				{
					NoWorshiper = 1;
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					if( Plunged == false )
					{
						PlayerLoses = true;
						return kPlayerLoses;
					}
					else
					{
						return kPlayerWins;
					}
				}
			}
			break;

		case kNotifyPlayerLeaving:
			if( Plunged == false )
			{
				PlayerLoses = true;
				return kPlayerLoses;
			}
			else
			{
				return kPlayerWins;
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								if ((( TalkedToWorshiper == false ) || (CallOfMonster == 1)) &&
									(InTooDeep == 0))
								{
									fMission.mDisplayMessage(kMonsterHail_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHail_2_msg, kMissionScriptMessage);
								}
								else if(( TalkedToWorshiper == true ) && (CallOfMonster == 0) &&
										(InTooDeep == 0))
								{
									fMission.mDisplayMessage(kMonsterHailCall_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailCall_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailCall_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailCall_4_msg, kMissionScriptMessage);
									CallOfMonster = 1;
								}
								else if ((InTooDeep == 1) && (FinalWord == 0))
								{
									fMission.mDisplayMessage(kMonsterHailN2Deep_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailN2Deep_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailN2Deep_3_msg, kMissionScriptMessage);
									FinalWord = 1;
								}
								else if (FinalWord == 1)
								{
									fMission.mDisplayMessage(kMonsterHailFinalWord_msg, kMissionScriptMessage);
								}
								else if (AttackedMonster == 1)
								{
									fMission.mDisplayMessage(kMonsterHailAttacked_msg, kMissionScriptMessage);
								}

								
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kWorshiperTeam:
								if(( TalkedToWorshiper == false ) && ( AtWarWithWorshiper == false ) && ( Plunged == false ))
								{
									fMission.mDisplayMessage(kWorshiperHailNic_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailNic_2_msg, kMissionScriptMessage);
								}
								else if(( TalkedToWorshiper == false ) && ( AtWarWithWorshiper == true ) && (Plunged == false))
								{
									fMission.mDisplayMessage(kWorshiperHailCombat_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailCombat_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailCombat_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailCombat_4_msg, kMissionScriptMessage);
									TalkedToWorshiper = true;
								}
								else if(( TalkedToWorshiper == true ) && (CallOfMonster == 0) && (Plunged == false) &&
										(AttackedAlly == 0))
								{
									fMission.mDisplayMessage(kWorshiperHailNoCall_msg, kMissionScriptMessage);
								}
								else if ((CallOfMonster == 1) && (Plunged == false) && (AttackedAlly == 0))
								{
									fMission.mDisplayMessage(kWorshiperHailCall_msg, kMissionScriptMessage);
								}
								else if ((Plunged == true) && (AttackedAlly == 0))
								{
									fMission.mDisplayMessage(kWorshiperHailPlunged_msg, kMissionScriptMessage);
								}
								else if (AttackedAlly == 1)
								{
									fMission.mDisplayMessage(kWorshiperHailBetrayal_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPoacherTeam:
								if( AtWarWithPoacher == false )
								{
									fMission.mDisplayMessage(kHunterHailNic_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kHunterHailCombat_msg, kMissionScriptMessage);
								}								
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kWorshiperTeam:
								if (Ship->mGetDamagePercent() < DamageThreshold)
								{
									fMission.mDisplayMessage(kWorshiperSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kWorshiperSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPoacherTeam:
								if (Ship->mGetDamagePercent() < DamageThreshold)
								{
									fMission.mDisplayMessage(kHunterSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kHunterSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kWorshiperTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kWorshiperTaunt_1_msg, kWorshiperTaunt_3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPoacherTeam:
								if (Ship->mGetDamagePercent() < DamageThreshold)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kHunterTauntOK_1_msg, kHunterTauntOK_3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kHunterTauntDamaged_1_msg, kHunterTauntDamaged_3_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	if( PlayerLoses == true )
	{
		return kDevastatingDefeat;
	}
	int32 VictoryPoints = 0;

	if( Plunged == true )
	{
		VictoryPoints = 80;
	}

	VictoryPoints += (CallOfMonster * 10) + (NoWorshiper * 10) -
					 (AttackedMonster * 100) - (MonsterDestroyed * 100);

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// CNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel >= kDefeat )
	{
		if ( AttackedMonster == 0 )
		{
			fMission.mAddToDebriefingList( kDidNotAttackMonster_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kAttackedMonster_msg );
		}
	}

	if (( VictoryLevel == kAstoundingVictory ) ||
		( VictoryLevel == kVictory ))
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
		fMission.mAddToDebriefingList( kVictory2_msg );
		fMission.mAddToDebriefingList ( kDismissedLong_msg );
	}
	else
	{
		fMission.mAddToDebriefingList (kDevastatingDefeat_msg);
		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			fMission.mScheduleMission( kRandomMission_msg );
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

