#include "scriptinterface.h"

#include "Hyd_6aMissionText.h"
#include "Hyd_6aMissionMaps.h"
/**********************************
LNF need to figure out how bad of a defeat
LNF will keep the player from part 2 of this mission
*********************************/

int32 PlayerDestroyed = 0;
int32 HunterDestroyed = 0;
int32 MonsterDestroyed = 0;
const float DamageThreshold = 30.0;
int32 AtWarWithHunter = 0;
int32 Number5Aggressive = 0;
int32 Number5Tagged = 0;
int32 TalkedWithNumber5 = 0;
int32 MonsterTaggedCount = 0;
int32 MonsterAlliance = 0;

struct MonsterStats
{
	int32 ScannedCount;
	int32 Tagged;
	int32 Aggressive;
};

MonsterStats MonsterMember[6] = {0}; // there are only 5 monsters, but I'm going to igonore [0]

eClassTypes PlayerShipClass= kClassShuttle;

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kHunterTeam, 
	kMonsterTeam,
	kMonster5Team
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kHunterTeam, kHydran);
	fMission.mSetTeamRace (kMonsterTeam, kMonster);
	fMission.mSetTeamRace (kMonster5Team, kMonster);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMonster5Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kHunterTeam, kMonsterTeam, kTeamNeutral, kGoalCapture, 1.0, kTrue);
	fMission.mSetTeamRelations (kHunterTeam, kMonster5Team, kTeamNeutral, kGoalCapture, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kMonster5Team, kTeamAllied, kGoalCapture, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* Monster5Team = fMission.mGetTeamHandle (kMonster5Team);
	
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.60f;
			// LNF should I put mCreate in here?
		}
		break;
	case 1:
		{
			// do nothing
			// LNF should I put mCreate in here?
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
			// LNF should I put mCreate in here?
		}
		break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Hunter
	//HunterTeam->mCreateFleet (3, kMaxBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_S);

	// create Monster
	//MonsterTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	MonsterTeam->mCreateFleet (4, kMaxBPV, kClassMonster, kClassMonster, 50, kStartPosition_V);

	// create Monster
	//MonsterTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	Monster5Team->mCreateFleet (1, kMaxBPV, kClassMonster, kClassMonster, 50, kStartPosition_Z);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* Monster5Team = fMission.mGetTeamHandle (kMonster5Team);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		//Ship->mSetShipCallbackEvents(kNotifyTransportedItemTo);
	}

	// configure Hunter
	for (Ship = HunterTeam->mGetFirstShip(); Ship != NULL; Ship = HunterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Monster
	for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Monster5
	for (Ship = Monster5Team->mGetFirstShip(); Ship != NULL; Ship = Monster5Team->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// Timer to check distance from any of the monsters?
	// what about the other ships getting too close?
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* Monster5Team = fMission.mGetTeamHandle (kMonster5Team);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDeepScan:
			if (fIsDeepScanOn == kTrue)
			{
				if (fPlayerShip->mGetTacticalDistance(PlayersTarget) <= 15.0)
					// LNF need to be able to check shield up or down
				{
					if(PlayersTarget == MonsterTeam->mGetThisShip(1))
					{
						MonsterMember[1].Aggressive = 1;
						MonsterMember[1].ScannedCount++;
						fMission.mDisplayMessage( kEventShield_msg, kScienceScriptMessage );
						// LNF find the closest ship, attack
						// set a timer? and look for the closest ship, attack that ship
					}
					else if(PlayersTarget == MonsterTeam->mGetThisShip(2))
					{
						MonsterMember[2].Aggressive = 1;
						MonsterMember[2].ScannedCount++;
						fMission.mDisplayMessage( kEventShield_msg, kScienceScriptMessage );
						// LNF find the closest ship, attack
						// set a timer? and look for the closest ship, attack that ship
					}
					else if(PlayersTarget == MonsterTeam->mGetThisShip(3))
					{
						MonsterMember[3].Aggressive = 1;
						MonsterMember[3].ScannedCount++;
						fMission.mDisplayMessage( kEventShield_msg, kScienceScriptMessage );
						// LNF find the closest ship, attack
						// set a timer? and look for the closest ship, attack that ship
					}
					else if(PlayersTarget == MonsterTeam->mGetThisShip(4))
					{
						MonsterMember[4].Aggressive = 1;
						MonsterMember[4].ScannedCount++;
						fMission.mDisplayMessage( kEventShield_msg, kScienceScriptMessage );
						// LNF find the closest ship, attack
						// set a timer? and look for the closest ship, attack that ship
					}
					else if(PlayersTarget == MonsterTeam->mGetThisShip(5))
					{
						MonsterMember[5].Aggressive = 1;
						MonsterMember[5].ScannedCount++;
						fMission.mDisplayMessage( kEventShield_msg, kScienceScriptMessage );
						// LNF find the closest ship, attack
						// set a timer? and look for the closest ship, attack that ship
					}
				}
				else if ((PlayersTarget == MonsterTeam->mGetThisShip(1)) && (MonsterMember[1].ScannedCount > 2))
				{
					MonsterMember[1].Aggressive = 1;
					fMission.mDisplayMessage( kEventScan_msg, kScienceScriptMessage );
					// LNF find the closest ship, attack
					// set a timer? and look for the closest ship, attack that ship
				}
				else if ((PlayersTarget == MonsterTeam->mGetThisShip(2)) && (MonsterMember[2].ScannedCount > 2))
				{
					MonsterMember[2].Aggressive = 1;
					fMission.mDisplayMessage( kEventScan_msg, kScienceScriptMessage );
					// LNF find the closest ship, attack
					// set a timer? and look for the closest ship, attack that ship
				}
				else if ((PlayersTarget == MonsterTeam->mGetThisShip(3)) && (MonsterMember[3].ScannedCount > 2))
				{
					MonsterMember[3].Aggressive = 1;
					fMission.mDisplayMessage( kEventScan_msg, kScienceScriptMessage );
					// LNF find the closest ship, attack
					// set a timer? and look for the closest ship, attack that ship
				}
				else if ((PlayersTarget == MonsterTeam->mGetThisShip(4)) && (MonsterMember[4].ScannedCount > 2))
				{
					MonsterMember[4].Aggressive = 1;
					fMission.mDisplayMessage( kEventScan_msg, kScienceScriptMessage );
					// LNF find the closest ship, attack
					// set a timer? and look for the closest ship, attack that ship
				}
				else if ((PlayersTarget == MonsterTeam->mGetThisShip(5)) && (MonsterMember[5].ScannedCount > 2))
				{
					MonsterMember[5].Aggressive = 1;
					fMission.mDisplayMessage( kEventScan_msg, kScienceScriptMessage );
					// LNF find the closest ship, attack
					// set a timer? and look for the closest ship, attack that ship
				}

			}
			break;

		case kNotifyDamage:
			//LNF VERY IMPORTANT -- I need to know either who the shooter is so I can set the 
			// AtWarWith... flag properly, or know when a Hostile person
			// takes damage and starts shooting back
			if(PlayersTarget == MonsterTeam->mGetThisShip(1))
			{
				MonsterMember[1].Aggressive = 1;
				// LNF find the closest ship, attack
				// set a timer? and look for the closest ship, attack that ship
			}
			else if(PlayersTarget == MonsterTeam->mGetThisShip(2))
			{
				MonsterMember[2].Aggressive = 1;
				// LNF find the closest ship, attack
				// set a timer? and look for the closest ship, attack that ship
			}
			else if(PlayersTarget == MonsterTeam->mGetThisShip(3))
			{
				MonsterMember[3].Aggressive = 1;
				// LNF find the closest ship, attack
				// set a timer? and look for the closest ship, attack that ship
			}
			else if(PlayersTarget == MonsterTeam->mGetThisShip(4))
			{
				MonsterMember[4].Aggressive = 1;
				// LNF find the closest ship, attack
				// set a timer? and look for the closest ship, attack that ship
			}
			else if(PlayersTarget == MonsterTeam->mGetThisShip(5))
			{
				MonsterMember[5].Aggressive = 1;
				// LNF find the closest ship, attack
				// set a timer? and look for the closest ship, attack that ship
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyOnCapture:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kMonster5Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kHunterTeam, kMonsterTeam, kTeamNeutral, kGoalCapture, 1.0, kTrue);
					fMission.mSetTeamRelations (kHunterTeam, kMonster5Team, kTeamNeutral, kGoalCapture, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonsterTeam, kMonster5Team, kTeamAllied, kGoalCapture, 1.0, kTrue);
					break;

				case kTimerLimitSet_2:
					break;

				case kTimerLimitSet_3:
					// the win condition
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					// set the Monster5 comm buttons back to HAIL
					for (Ship = Monster5Team->mGetFirstShip(); Ship != NULL; Ship = Monster5Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Hunter comm buttons back to HAIL
					for (Ship = HunterTeam->mGetFirstShip(); Ship != NULL; Ship = HunterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			if (Ship->mGetTeam() == kHunterTeam)
			{
				HunterDestroyed++;
			}

			if (Ship->mGetTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;
				if (MonsterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kMonster5Team)
			{
				MonsterDestroyed++;
				if (Monster5Team->mGetShipsInPlayCount() == 0)
				{
				}
			}

			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailMute_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonster5Team:
								if ((Number5Aggressive == 0) && (MonsterAlliance == 0))
								{
									fMission.mDisplayMessage(kMonsterHailNoAlliance_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoAlliance_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoAlliance_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoAlliance_4_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoAlliance_5_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoAlliance_6_msg, kMissionScriptMessage);
									TalkedWithNumber5 = 1;
								}
								else if ((Number5Aggressive == 0) && (MonsterAlliance == 1))
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kMonsterHailAlliance_1_msg, kMonsterHailAlliance_3_msg), kMissionScriptMessage);
									TalkedWithNumber5 = 1;
								}
								else if (Number5Aggressive == 1)
								{
									fMission.mDisplayMessage(kMonsterHailAggressive_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailMute_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMonster5Team:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailMute_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMonster5Team:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// no victory points since this is part I

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	// LNF need to make a 'continuation' debriefing
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	fMission.mAddToDebriefingList (kDismissedShort_msg);

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
*/

	fMission.mScheduleMission( kNextMission_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

