#include "scriptinterface.h"

#include "Hyd_6bMissionText.h"
#include "Hyd_6bMissionMaps.h"


int32 PlayerDestroyed = 0;
int32 AggressorDestroyed = 0;
int32 BaseDestroyed = 0;
int32 MonsterDestroyed = 0;
const float DamageThreshold = 30.0; // damage a ship takes for surrender message
const float MonsterDamageThreshold = 30.0; // damage monster takes before beserking
int32 AtWarWithAggressor = 0;
int32 OutOfControl = 0; // Monster is out of control - too much damage
int32 AttackedMonster = 0;
eVictoryLevels MyVicLevel = kDevastatingDefeat;

void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void LyranCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);

eClassTypes PlayerShipClass= kClassShuttle;

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kAggressorTeam, 
	kMonsterTeam,
	kBaseTeam
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kKlingMission | kLyranMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 50) ? kKlingon : kLyran;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kAggressorTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kMonsterTeam, kMonster);
	fMission.mSetTeamRace (kBaseTeam, kHydran);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kAggressorTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kAggressorTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.60f;
			// LNF should I put mCreate in here?
		}
		break;
	case 1:
		{
			// do nothing
			// LNF should I put mCreate in here?
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
			// LNF should I put mCreate in here?
		}
		break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Aggressor
	AggressorTeam->mCreateFleet (6, kMaxBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_U);

	// create Monster
	//MonsterTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	MonsterTeam->mCreateShip (kMaxBPV, kClassMonster, kClassMonster, 50, kStartPosition_J);

	// create Base
	BaseTeam->mCreateShip("H-SB", kStartPosition_K);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}

	// configure Aggressor
	for (Ship = AggressorTeam->mGetFirstShip(); Ship != NULL; Ship = AggressorTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Monster
	for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Base
	for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		//Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		//Ship->mDamageSystem(kDamageRightWarpEngine, 100);
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDeepScan:
			if (fIsDeepScanOn == kTrue)
			{
			}
			break;

		case kNotifyDamage:
			//LNF VERY IMPORTANT -- I need to know either who the shooter is so I can set the 
			// AtWarWith... flag properly, or know when a Hostile person
			// takes damage and starts shooting back
			if (PlayersTarget == MonsterTeam->mGetFirstShip())
			{
				// LNF check when the monster gets shot
				//fMission.mDisplayMessage(kIFAllAttacked_msg, kMissionScriptMessage);
				OutOfControl = 1;
				fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			}

			if (MonsterTeam->mGetFirstShip()->mGetDamagePercent() >= MonsterDamageThreshold)
			{
				fMission.mDisplayMessage (kEventMonsterMad_msg, kMissionScriptMessage);
				fMission.mSetTeamRelations (kMonsterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				fMission.mSetTeamRelations (kMonsterTeam, kAggressorTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			}

			// LNF the beast is supposed to get happy if it repairs itself 10% from where it went berserk
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyOnCapture:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kAggressorTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kAggressorTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonsterTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					break;

				case kTimerLimitSet_2:
					break;

				case kTimerLimitSet_3:
					// the win condition
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					break;

				case kTimerTurnsSet_9:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Aggressor comm buttons back to HAIL
					for (Ship = AggressorTeam->mGetFirstShip(); Ship != NULL; Ship = AggressorTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			if (Ship->mGetTeam() == kAggressorTeam)
			{
				AggressorDestroyed++;
			}

			if (Ship->mGetTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;
				fMission.mDisplayMessage (kEventMonsterKilled_msg, kMissionScriptMessage);
				if (MonsterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kBaseTeam)
			{
				BaseDestroyed++;
				fMission.mDisplayMessage (kEventBaseDestroyed_msg, kMissionScriptMessage);
				if (BaseTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kAggressorTeam:
								// LNF how do I check the race of info.fEnemyRace?
/*								if (info.fEnemyRace == kKlingon)
								{
									KlingonCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
								else
								{
									LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
*/								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								if ((OutOfControl == 0) && (AtWarWithAggressor == 0))
								{
									fMission.mDisplayMessage(kMonsterHailNic_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (fMission.mRandomInt32 (kMonsterHailNic_2_msg, kMonsterHailNic_4_msg), kMissionScriptMessage);
								}
								else if ((OutOfControl == 0) && (AtWarWithAggressor == 1))
								{
									fMission.mDisplayMessage(kMonsterHailCombat_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (fMission.mRandomInt32 (kMonsterHailCombat_2_msg, kMonsterHailCombat_4_msg), kMissionScriptMessage);
								}
								else if (OutOfControl == 1)
								{
									fMission.mDisplayMessage(kMonsterHailOutOfControl_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailOutOfControl_2_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kAggressorTeam:
/*								if (info.fEnemyRace == kKlingon)
								{
									KlingonCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
								else
								{
									LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
*/								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kAggressorTeam:
/*								if (info.fEnemyRace == kKlingon)
								{
									KlingonCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
								else
								{
									LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
*/								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = ((1-BaseDestroyed) * 100) - (BaseDestroyed * 100) - (MonsterDestroyed * 30);
	// LNF check if there are still enemy on the screen not disabled

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;
	if ( VictoryLevel >= kDefeat )
	{
		if ( BaseDestroyed == 1 )
		{
			fMission.mAddToDebriefingList( kDefeatBaseLost_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kDefeatBaseNotLost_msg );
		}
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}

	if ( VictoryLevel <= kVictory )
	{
		fMission.mAddToDebriefingList( kVictory2_msg );
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
*/

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_4;
	}
	else
	{
		return 0;
	}
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// Klingon Comm Messages
void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (AtWarWithAggressor == 0)
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliHailNic_1_msg, kKliHailNic_3_msg), kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliHailCombat_1_msg, kKliHailCombat_3_msg), kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() <= DamageThreshold)
			{
				tScript.fMission.mDisplayMessage(kKliSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kKliSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliTaunt_1_msg, kKliTaunt_3_msg), kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Lyran Comm Messages
void LyranCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (AtWarWithAggressor == 0)
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kLyrHailNic_1_msg, kLyrHailNic_3_msg), kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kLyrHailCombat_1_msg, kLyrHailCombat_3_msg), kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() <= DamageThreshold)
			{
				tScript.fMission.mDisplayMessage(kLyrSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kLyrSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kLyrTaunt_1_msg, kLyrTaunt_3_msg), kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;
	}
}

