#include "scriptinterface.h"

#include "Hyd_7MissionText.h"
#include "Hyd_7MissionMaps.h"


int32 PlayerDestroyed = 0;
int32 MonsterDestroyed = 0;
int32 AggressorDestroyed = 0;
int32 WorshiperDestroyed = 0;
int32 BaseDestroyed = 0;
int32 AsteroidDestroyed = 0;
const float DamageThreshold = 30.0;
const int32 TimeUntilAggressorArrives = 600; // LNF adjust this value
const float TruceDamagePercent = 25.0; // Amount of damage that Worshiper will offer truce
										// if hailed or asked to surrender
int32 AtWarWithMonster = 0; // player attacked the monster
int32 AtWarWithAggressor = 0;
int32 AtWarWithWorshiper = 0;
int32 MissionMode = 0;
int32 MonsterPermission = 0; // Monster gives permission to use monster as weapon?
int32 BetrayedWorshiper = 0; // player attacks worshipers after TRUCE
int32 PeacefulSolution = 0;
int32 Truce = 0;
int32 AggressorArrival = 0; // flag for aggressor arrival
int32 BaseTaken = 0; // the player takes the base
int32 PlanetTaken = 0;

eClassTypes PlayerShipClass= kClassShuttle;

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void LyranCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);

enum eMissionMode
{
	kStart = 0,
	kMonsterArrives,
	kAggressorArrives
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMonsterTeam, 
	kAggressorTeam,
	kWorshiperTeam,
	kBaseTeam,
	kAsteroidTeam
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kKlingMission | kLyranMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 50) ? kKlingon : kLyran;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMonsterTeam, kMonster);
	fMission.mSetTeamRace (kAggressorTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kWorshiperTeam, kHydran);
	fMission.mSetTeamRace (kBaseTeam, kHydran);
	//fMission.mSetTeamRace (kAsteroidTeam, kMonster); // LNF I don't know what to make the asteroid

	// Set team relations to show up on Tactical map
	//fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kWorshiperTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kPlayerTeam, kAsteroidTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	//fMission.mSetTeamRelations (kMonsterTeam, kAggressorTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	//fMission.mSetTeamRelations (kMonsterTeam, kWorshiperTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kMonsterTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kMonsterTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	//fMission.mSetTeamRelations (kAggressorTeam, kWorshiperTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kAggressorTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	//fMission.mSetTeamRelations (kAggressorTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kWorshiperTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kWorshiperTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	//fMission.mSetTeamRelations (kBaseTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle (kWorshiperTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle (kAsteroidTeam);
	

	if (MissionMode == kStart)
	{
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.60f;
			// LNF should I put mCreate in here?
		}
		break;
	case 1:
		{
			// do nothing
			// LNF should I put mCreate in here?
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
			// LNF should I put mCreate in here?
		}
		break;
	}

	/***
		LNF pick a random number of ships depending on how many the player has
		check the class of player's ships
		have some kind of table that determines balance for ship classes vs number
		< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
		choose whether to have fewer powerful ships or many smaller ships
	***/
		
		// create Worshipers
		WorshiperTeam->mCreateFleet (4, kMaxBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 25, 10);

		// create Base
		BaseTeam->mCreateShip (kMaxBPV, kClassStarBase, kClassStarBase, 50, kStartPosition_K);

		// create Asteroid or whatever
		//AsteroidTeam->mCreateShip("L-CAP", kStartPosition_Y);
	}
	else if (MissionMode == kMonsterArrives)
	{
		fMission.mSetTeamRelations (kMonsterTeam, kPlayerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kMonsterTeam, kWorshiperTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kMonsterTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kMonsterTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		// create Monster
		MonsterTeam->mCreateShip (kMaxBPV, kClassMonster, kClassMonster, 50, kStartPosition_Specified, 33, 33);
	}
	else if (MissionMode == kAggressorArrives)
	{
		fMission.mSetTeamRelations (kAggressorTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kAggressorTeam, kMonsterTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kAggressorTeam, kWorshiperTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kAggressorTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kAggressorTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		// create Aggressor
		//AggressorTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
		AggressorTeam->mCreateFleet (4, kMaxBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 20, 2);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle (kWorshiperTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle (kAsteroidTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}

	// configure Worshiper
	for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Base
	for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to countdown arrival of Aggressor fleet
	fMission.mSetTimedInterval (kTimerLimitSet_2, TimeUntilAggressorArrives);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (MissionMode == kMonsterArrives)
	{
		// configure Monster
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		// LNF need a way to permanently set a ship's speed
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
	}
	else if (MissionMode == kAggressorArrives)
	{
		// configure Aggressors
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* WorshiperTeam = fMission.mGetTeamHandle (kWorshiperTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* AsteroidTeam = fMission.mGetTeamHandle (kAsteroidTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			// LNF temporary for now....if the player launches a probe close to the 'monster home'
			// the monster will appear.  deep scan? or something else?

			if ((fPlayerShip->mGetTacticalDistance(33, 33) <= 30.0) && (Truce == 1))
			{
				MissionMode = kMonsterArrives;
				mCreateMissionShips(kMonsterArrives);
			}
			break;

		case kNotifyDeepScan:
			if (fIsDeepScanOn == kTrue)
			{
			}
			break;

		case kNotifyDamage:
			//LNF VERY IMPORTANT -- I need to know either who the shooter is so I can set the 
			// AtWarWith... flag properly, or know when a Hostile person
			// takes damage and starts shooting back
			if (Ship->mGetTeam() == kMonsterTeam)
			{
				AtWarWithMonster = 1;
			}
			else if ((Ship->mGetTeam() == kWorshiperTeam) && (Truce == 1))
			{
				BetrayedWorshiper = 1;
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyOnCapture:
			// LNF
/*			if (Ship->mGetOriginalTeam() == kBaseTeam)
			{
				fMission.mDisplayMessage( kEvent5BaseTaken_msg, kMissionScriptMessage );
				BaseTaken = 1;
			}
			else if (Ship->mGetOriginalTeam() == kPlanetTeam
			{
				fMission.mDisplayMessage( kEvent7PlanetTaken_msg, kMissionScriptMessage );
				PlanetTaken = 1;
			}
			// LNF there are lines for the base and planet NOT taken, where do those go?
*/			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					//fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kWorshiperTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kPlayerTeam, kAsteroidTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					//fMission.mSetTeamRelations (kMonsterTeam, kAggressorTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					//fMission.mSetTeamRelations (kMonsterTeam, kWorshiperTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kMonsterTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kMonsterTeam, kAsteroidTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					//fMission.mSetTeamRelations (kAggressorTeam, kWorshiperTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kAggressorTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					//fMission.mSetTeamRelations (kAggressorTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kWorshiperTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kWorshiperTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					//fMission.mSetTeamRelations (kBaseTeam, kAsteroidTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					break;

				case kTimerLimitSet_2: // aggressor arrival timer
					// LNF if the player takes the planet and the base and/or gets the peacefulsolution
					// bring in the Aggressor team instead of waiting for the timer
					fMission.mDisplayMessage (kEvent3Arrival_msg, kScienceScriptMessage);
					MissionMode = kAggressorArrives;
					mCreateMissionShips(kAggressorArrives);
					break;

				case kTimerLimitSet_3:
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_7:
					// set the Worshiper comm buttons back to HAIL
					for (Ship = WorshiperTeam->mGetFirstShip(); Ship != NULL; Ship = WorshiperTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8:
					// set the Aggressor comm buttons back to HAIL
					for (Ship = AggressorTeam->mGetFirstShip(); Ship != NULL; Ship = AggressorTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Base comm buttons back to HAIL
					for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			if (Ship->mGetTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;
			}

			if (Ship->mGetTeam() == kAggressorTeam)
			{
				AggressorDestroyed++;
				if (AggressorTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kWorshiperTeam)
			{
				WorshiperDestroyed++;
				if (WorshiperTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kBaseTeam)
			{
				BaseDestroyed++;
				if (BaseTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kAsteroidTeam)
			{
				AsteroidDestroyed++;
				if (AsteroidTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								if (AtWarWithMonster == 0)
								{
									if (MonsterPermission == 0)
									{
										fMission.mDisplayMessage(kMonsterHailNoPermission_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoPermission_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoPermission_3_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoPermission_4_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoPermission_5_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoPermission_6_msg, kMissionScriptMessage);
										MonsterPermission = 1;
									}
									else
									{
										fMission.mDisplayMessage(kMonsterHailPermission_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailPermission_2_msg, kMissionScriptMessage);
										// LNF move the player's ship to a random location...is this possible?
									}
								}
								else // AtWarWithMonster == 1
								{
									// LNF is ths supposed to be monster attacked?
									fMission.mDisplayMessage(kMonsterHailAttacked_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kAggressorTeam:
/*								if (info.fEnemyRace == kKlingon)
								{
									KlingonCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
								else
								{
									LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
*/								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kWorshiperTeam:
								if ((BetrayedWorshiper == 1) || (AtWarWithMonster == 1))
								{
									fMission.mDisplayMessage(kWorshiperHailNoResponse_msg, kMissionScriptMessage);
								}
								else if ((PeacefulSolution == 0) && (Truce == 0) && (AtWarWithWorshiper == 0))
								{
									fMission.mDisplayMessage(kWorshiperHailNic_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailNic_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailNic_3_msg, kMissionScriptMessage);
								}
								else if ((PeacefulSolution == 0) && (Truce == 0) && (AtWarWithWorshiper == 1))
								{
									if (Ship->mGetDamagePercent() >= TruceDamagePercent)
									{
										fMission.mDisplayMessage(kWorshiperHailCombatOK_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kWorshiperHailCombatOK_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kWorshiperHailCombatOK_3_msg, kMissionScriptMessage);
									}
								}
								else if ((MonsterPermission == 0) && (Truce == 0) && (AtWarWithWorshiper == 1))
								{
									if (Ship->mGetDamagePercent() <= TruceDamagePercent)
									{
										fMission.mDisplayMessage(kWorshiperHailCombatDamaged_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kWorshiperHailCombatDamaged_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kWorshiperHailCombatDamaged_3_msg, kMissionScriptMessage);
										Truce = 1;
									}
								}
								else if ((MonsterPermission == 0) && (Truce == 1))
								{
									fMission.mDisplayMessage(kWorshiperHailTruce_msg, kMissionScriptMessage);
								}
								else if ((MonsterPermission == 1) && (PeacefulSolution == 0))
								{
									fMission.mDisplayMessage(kWorshiperHailPermission_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailPermission_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailPermission_3_msg, kMissionScriptMessage);
									PeacefulSolution = 1;
								}
								else if ((PeacefulSolution == 1) && (AggressorArrival == 0))
								{
									fMission.mDisplayMessage(kWorshiperHailPeaceful_msg, kMissionScriptMessage);
								}
								else if ((PeacefulSolution == 1) && (AggressorArrival == 1))
								{
									fMission.mDisplayMessage(kWorshiperHailArrival_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kWorshiperHailArrival_2_msg, kMissionScriptMessage);									
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;

							case kBaseTeam:
								if ((BaseTaken == 0) && (PeacefulSolution == 0) && (AggressorArrival == 0))
								{
									fMission.mDisplayMessage(kBaseHail_msg, kMissionScriptMessage);
								}
								else if (((BaseTaken == 1) || (PeacefulSolution == 1)) && (AggressorArrival == 0))
								{
									fMission.mDisplayMessage(kBaseHailNothing_msg, kMissionScriptMessage);
								}
								else if (AggressorArrival == 1)
								{
									if ( BaseTeam->mGetThisShip(1)->mGetDamagePercent() < 10.0 )
									{
										fMission.mDisplayMessage(kBaseHailArrival_1_msg, kMissionScriptMessage);
									}
									else if (( BaseTeam->mGetThisShip(1)->mGetDamagePercent() >= 10.0 ) &&
											( BaseTeam->mGetThisShip(1)->mGetDamagePercent() <= 60.0 ))
									{
										fMission.mDisplayMessage(kBaseHailArrival_2_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kBaseHailArrival_3_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kAsteroidTeam:
								//fMission.mDisplayMessage(kMWHailIF_msg, kMissionScriptMessage);
/*								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
*/								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kAggressorTeam:
/*								if (info.fEnemyRace == kKlingon)
								{
									KlingonCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
								else
								{
									LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
*/								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kWorshiperTeam:
								fMission.mDisplayMessage(kWorshiperSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;

							case kBaseTeam:
								if ((BaseTaken == 0) && (PeacefulSolution == 0) && (AggressorArrival == 0))
								{
									fMission.mDisplayMessage(kBaseSurrender_msg, kMissionScriptMessage);
								}
								else if (((BaseTaken == 1) || (PeacefulSolution == 1)) && (AggressorArrival == 0))
								{
									fMission.mDisplayMessage(kBaseSurrenderTaken_msg, kMissionScriptMessage);
								}
								else if (AggressorArrival == 1)
								{
									fMission.mDisplayMessage(kBaseSurrenderArrival_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kAsteroidTeam:
/*								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
*/								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kAggressorTeam:
/*								if (info.fEnemyRace == kKlingon)
								{
									KlingonCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
								else
								{
									LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
								}
*/								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kWorshiperTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kWorshiperTaunt_1_msg, kWorshiperTaunt_3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
								break;

							case kBaseTeam:
								if ((BaseTaken == 0) && (PeacefulSolution == 0) && (AggressorArrival == 0))
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kBaseTaunt_1_msg, kBaseTaunt_3_msg), kMissionScriptMessage);
								}
								else if ((BaseTaken == 1) || (PeacefulSolution == 1))
								{
									fMission.mDisplayMessage(kBaseTauntTaken_msg, kMissionScriptMessage);
								}								
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kAsteroidTeam:
/*								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
*/								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (AggressorDestroyed * 5) + (PlanetTaken * 30) + (BaseTaken * 30) - (BaseDestroyed * 50);
	// LNF need to figure out discrepancies between base taken and destroyed, and 
	// add points if player leaves when there are still Aggressor ships.

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	fMission.mAddToDebriefingList (kDismissedShort_msg);

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
*/

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// Klingon Comm Messages
void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (AtWarWithAggressor == 0)
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliHailNic_1_msg, kKliHailNic_3_msg), kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliHailCombat_1_msg, kKliHailCombat_3_msg), kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() <= DamageThreshold)
			{
				tScript.fMission.mDisplayMessage(kKliSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kKliSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliTaunt_1_msg, kKliTaunt_2_msg), kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Lyran Comm Messages
void LyranCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (AtWarWithAggressor == 0)
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kLyrHailNic_1_msg, kLyrHailNic_3_msg), kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kLyrHailCombat_2_msg, kLyrHailCombat_3_msg), kMissionScriptMessage);
			}
			// LNF need to put something about Lyran Betrayal
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() <= DamageThreshold)
			{
				tScript.fMission.mDisplayMessage(kLyrSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kLyrSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kLyrTaunt_1_msg, kLyrTaunt_3_msg), kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;
	}
}

