#include "scriptinterface.h"

#include "Hyd_9MissionText.h"
#include "Hyd_9MissionMaps.h"


int32 PlayerDestroyed = 0;
int32 MonsterDestroyed = 0;
int32 HunterDestroyed = 0;
int32 HunterCaptured = 0;
int32 TransformedDestroyed = 0;

int32 PlayerDisengaged = 0;
int32 MonsterDisengaged = 0;
int32 HunterDisengaged = 0;
int32 TransformedDisengaged = 0;

int32 HunterEvolved = 0; // the count of Hunter ships that evolve
int32 AtWarWithMonster = 0;
int32 ScannedCluster = 0;
int32 Communications = 0;
int32 HunterExplanation = 0;
int32 HunterTransformX = 0;// X location of Hunter ship when it transforms
int32 HunterTransformY = 0;// Y location of Hunter ship when it transforms
int32 AllHunterShipsDestroyed = 0;

// the amount of damage the Hunter team sustains before a chance to transform to monster
const float kDamageThresholdTransformation = 60.0f;
const int32 StarX = 25;
const int32 StarY = 25;
const float kDeepScanDistance = 100.0f;

bool InRangeOfStarFirstTime = false;// flag if the player gets in range of star for first time

eClassTypes PlayerShipClass= kClassShuttle;

enum eMissionMode
{
	kStart = 0,
	kTransformation,
};
eMissionMode MissionMode = kStart;

eGameStatus CheckWinCondition( tScript& tScript );

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMonsterTeam, 
	kHunterTeam,
	kTransformedTeam,
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMonsterTeam, kMonster);
	fMission.mSetTeamRace (kHunterTeam, kHydran);
	fMission.mSetTeamRace (kTransformedTeam, kMonster);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	//fMission.mSetTeamRelations (kPlayerTeam, kTransformedTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kHunterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kMonsterTeam, kTransformedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	//fMission.mSetTeamRelations (kHunterTeam, kTransformedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* TransformedTeam = fMission.mGetTeamHandle (kTransformedTeam);
	
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 DifLev = fMission.mGetDifficultyLevel();
	int32 AverageBPV = 0;
	int32 EnemyShipCount = 0;

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;
				AverageBPV = TotalBPV / PlayerTeam->mGetShipsInPlayCount();
				EnemyShipCount = 2;
			}
			break;
		case 1:
			{
				// do nothing
				AverageBPV = TotalBPV / PlayerTeam->mGetShipsInPlayCount();
				EnemyShipCount = 2;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;
				AverageBPV = TotalBPV / PlayerTeam->mGetShipsInPlayCount();
				EnemyShipCount = 3;
			}
			break;
	}

	if ( MissionMode == kStart )
	{
		// create Monster team
		// these are Hanyurii
		//MonsterTeam->mCreateShip ( "M-SSM", kStartPosition_U );
		MonsterTeam->mCreateShip ( "M-SSL", kStartPosition_V );
		MonsterTeam->mCreateShip ( "M-SSL", kStartPosition_W );
		//MonsterTeam->mCreateShip ( "M-SSM", kStartPosition_X );
		MonsterTeam->mCreateShip ( "M-SSM", kStartPosition_Y );
		//MonsterTeam->mCreateShip ( "M-SSL", kStartPosition_Z );

		// create Hunter team
		HunterTeam->mCreateFleet (EnemyShipCount, (EnemyShipCount * AverageBPV), kClassLightCruiser, kClassHeavyCruiser, 50, kStartPosition_Q);

		// create Transformed
		// LNF need MissionModes, and some way to make these transformed monsters appear one at a time
		//TransformedTeam->mCreateShip("0-DN2", kStartPosition_Z);
	}
	else if ( MissionMode == kTransformation )
	{
		// LNF should I put the transformed ships on their own team, or add to Monster team?
		TransformedTeam->mCreateShip( "M-SSS", kStartPosition_Specified, HunterTransformX, HunterTransformY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* TransformedTeam = fMission.mGetTeamHandle (kTransformedTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyBeginTractor);
	}

	// configure Monster
	for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Hunter
	for (Ship = HunterTeam->mGetFirstShip(); Ship != NULL; Ship = HunterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Transformed
/*	for (Ship = TransformedTeam->mGetFirstShip(); Ship != NULL; Ship = TransformedTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
	}
*/

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to check the distance of the player from the Monster
	fMission.mSetTimedInterval( kTimerLimitSet_2, 2);

	// timer to make the comm buttons diappear because of the dimensional star cluster
	// LNF this should be turns
	fMission.mSetTimedInterval( kTimerLimitSet_3, 200);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn
	// DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurns);
	DeepScanTracker.mSetLocation( StarX, StarY, kDeepScanDistance, 2);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// transformed ship stuff here
	if ( MissionMode == kTransformation )
	{
		Ship->mFollowEntity( fPlayerShip, kAbsolutePriority );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* TransformedTeam = fMission.mGetTeamHandle (kTransformedTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kMonsterTeam )
					{
						// if a monster damages a Hunter ship after the Hunter ship has been damaged
						// kDamageThresholdTransformation, then 5% chance it will transform into a monster
						if ( Ship->mGetTeam() == kHunterTeam )
						{
							if ( Ship->mGetDamagePercent() >= kDamageThresholdTransformation )
							{
								int32 ChanceToTransform = fMission.mRandomInt32( 1, 20 );

								// LNF might have to change the percentage
								if ( ChanceToTransform == 20 )
								{
									HunterEvolved++;

									fMission.mDisplayMessage( kTransformation_msg, kScienceScriptMessage );

									Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
									Ship->mDamageSystem( kDamageInternalsWithPoints, 1023 );

									Ship->mGetGridPosition (&HunterTransformX, &HunterTransformY);

									MissionMode = kTransformation;
									mCreateMissionShips( kTransformation );
								}
							}
						}
					}
					else if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kMonsterTeam )
						{
							for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
							{
								if (( Ship == tempShip )
									&& ( AtWarWithMonster != 1 ))
								{
									AtWarWithMonster = 1;
									break;
								}
								else
								{
									break;
								}
							}
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kTransformedTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonsterTeam, kHunterTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kMonsterTeam, kTransformedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kHunterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kHunterTeam, kMonsterTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kHunterTeam, kTransformedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					break;

				case kTimerLimitSet_2: // check distance from player to monster
					// LNF if any of the players ships are close to the monster
					// the monster will attack the player but not pursue until the 
					// player moves out of range
					break;

				case kTimerLimitSet_3: // make comm buttons disappear
					// LNF periodically the comm buttons will disappear
					break;

				case kTimerLimitSet_4:
					// check the win conditions

					return CheckWinCondition( *this );

					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;

							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;

							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on

								if ( InRangeOfStarFirstTime == false )
								{
									InRangeOfStarFirstTime = true;
									fMission.mDisplayMessage (kScannerOffInRangeFirstTime_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								}
								break;

							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								fMission.mDisplayMessage (kStartingScanStar_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off

								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun

								fMission.mDisplayMessage (kStartingScanStar_msg, kScienceScriptMessage);
								break;

							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;

							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off

								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;

							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object or location
							ScannedCluster = 1;

							fMission.mDisplayMessage( kEvent18Scanning_1_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kEvent18Scanning_2_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kEvent18Scanning_3_msg, kMissionScriptMessage );

							// check the win conditions
							fMission.mSetTimedInterval( kTimerLimitSet_4, 5 );

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					break;

				case kTimerTurnsSet_9:
					// set the Hunter comm buttons back to HAIL
					for (Ship = HunterTeam->mGetFirstShip(); Ship != NULL; Ship = HunterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;

				if ( MonsterTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}
			else if (Ship->mGetOriginalTeam() == kHunterTeam)
			{
				HunterDestroyed++;

				if (HunterTeam->mGetShipsInPlayCount() == 0)
				{
					AllHunterShipsDestroyed = 1;

					fMission.mDisplayMessage( kAllHunterShipsDestroyed_msg, kScienceScriptMessage );

					if ( InRangeOfStarFirstTime == false )
					{
						InRangeOfStarFirstTime = true;
						fMission.mDisplayMessage (kScannerOffInRangeFirstTime_msg, kScienceScriptMessage);
					}

					for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
					{
						tempShip->mFollowEntity( fPlayerShip, kAbsolutePriority );
					}

					// check the win conditions
					fMission.mSetTimedInterval( kTimerLimitSet_4, 5 );
				}
			}
			else if (Ship->mGetOriginalTeam() == kTransformedTeam)
			{
				TransformedDestroyed++;
				
				if (TransformedTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (Ship->mGetOriginalTeam() == kHunterTeam)
			{
				HunterCaptured++;

				if (HunterTeam->mGetShipsInPlayCount() == 0)
				{
					AllHunterShipsDestroyed = 1;

					fMission.mDisplayMessage( kAllHunterShipsDestroyed_msg, kScienceScriptMessage );

					if ( InRangeOfStarFirstTime == false )
					{
						InRangeOfStarFirstTime = true;
						fMission.mDisplayMessage (kScannerOffInRangeFirstTime_msg, kScienceScriptMessage);
					}

					for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
					{
						tempShip->mFollowEntity( fPlayerShip, kAbsolutePriority );
					}

					// check the win conditions
					fMission.mSetTimedInterval( kTimerLimitSet_4, 5 );
				}
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetOriginalTeam() == kMonsterTeam)
			{
				MonsterDisengaged++;

				if ( MonsterTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}
			}
			else if (Ship->mGetOriginalTeam() == kHunterTeam)
			{
				HunterDisengaged++;

				if (HunterTeam->mGetShipsInPlayCount() == 0)
				{
					AllHunterShipsDestroyed = 1;

					fMission.mDisplayMessage( kAllHunterShipsDestroyed_msg, kScienceScriptMessage );

					if ( InRangeOfStarFirstTime == false )
					{
						InRangeOfStarFirstTime = true;
						fMission.mDisplayMessage (kScannerOffInRangeFirstTime_msg, kScienceScriptMessage);
					}

					for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
					{
						tempShip->mFollowEntity( fPlayerShip, kAbsolutePriority );
					}

					// check the win conditions
					fMission.mSetTimedInterval( kTimerLimitSet_4, 5 );
				}
			}
			else if (Ship->mGetOriginalTeam() == kTransformedTeam)
			{
				TransformedDisengaged++;
			
				if (TransformedTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDisengaged++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								if (( AtWarWithMonster == 0 ) && ( ScannedCluster == 0 ))
								{
									fMission.mDisplayMessage(kMonsterHail_msg, kMissionScriptMessage);
								}
								else if (( AtWarWithMonster == 0 ) && ( ScannedCluster == 1 ))
								{
									fMission.mDisplayMessage(kMonsterHailOdyssey_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailOdyssey_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailOdyssey_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailOdyssey_4_msg, kMissionScriptMessage);
								}
								else if ( AtWarWithMonster == 1 )
								{
									fMission.mDisplayMessage(kMonsterHailAttacked_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kHunterTeam:
								if ( Communications == 0 )
								{
									if ( ScannedCluster == 0 )
									{
										fMission.mDisplayMessage(kHunterHailNoComm_1_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kHunterHailNoComm_2_msg, kMissionScriptMessage);
									}
								}
								else // Communications == 1
								{
									if ( HunterExplanation == 0 )
									{
										fMission.mDisplayMessage(kHunterHailNoExplanation_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kHunterHailNoExplanation_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kHunterHailNoExplanation_3_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kHunterHailNoExplanation_4_msg, kMissionScriptMessage);
										HunterExplanation = 1;
									}
									else // HunterExplanation == 1
									{
										fMission.mDisplayMessage (fMission.mRandomInt32 (kHunterHailExplanation_1_msg, kHunterHailExplanation_3_msg), kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kHunterTeam:
								fMission.mDisplayMessage(kHunterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kHunterTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kHunterTaunt_1_msg, kHunterTaunt_3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (ScannedCluster * 50) - (MonsterDestroyed * 30)
					+ (AllHunterShipsDestroyed * 50) - (HunterEvolved * 10);

	// LNF need to subtract points of any Hunters are left
	// and subtract the number of Hunters that transform

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	fMission.mAddToDebriefingList (kDismissedShort_msg);

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
*/


	// LNF do I need Final text?
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// check the win conditions
eGameStatus CheckWinCondition( tScript& tScript )
{
	if (( ScannedCluster == 1 )
		&& ( AllHunterShipsDestroyed == 1 ))
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}
