#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Kli_Special1Text.h"
#include "Kli_Special1Maps.h"

// sequence of events
// 1.  The player sees three ships that are non functioning  ahead of him
// 2.  If they scan, most of the ships have been stripped down (one is fine)
// 3.  Three Lyran(Wyn) ships approach.
// 4.  The science officer suggests that a bug could be placed in the ships via a capture or hit and run option.
// 5.  At 35% damage the WYN ships bail.

int32 VoicedWarOpionin = 0;
int32 VoicedSuspicion = 0;
int32 VoicedKnowledge = 0;

int32 gTotalWYNShipsdeparted = 0;
int32 AllWYNDestroyed = 0;
int32 HulksDestroyed = 0;
int32 WYNDestroyed = 0;

int32 ShipOneScanned = 0;
int32 ShipTwoScanned = 0;
int32 ShipThirdScanned = 0;

int32 TrackingSuccess =0;
int32 WYNScanned = 0;

int32 WYN_ESCAPES[3] = {0,0,0};
int32 WYN_TRACKED[3] = {0,0,0};


int32 HulkScanned[4] = {0,0,0,0};

const float kWYNDamageThreshhold = 10.0f;
const float kDeepScanDistancePk = 10.0f;
const float kDistanceFromWYNForWARDeclaration = 100.0f;
const float kDistanceFromWYNForWeirdness = 60.0f;
const float kDistanceFromWYNForKnowledge = 15.0f;

eClassTypes PlayerShipClass= kClassShuttle;
int32 Elite = 0;
int32 PlayLevel = 0;

enum eOfferStatus
{
	kNoOffer = 0,
	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
};

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);


// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kDerelictTeam, 
	kWYNTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission; //XXXXXX Huh?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 6;
	Specs->fMaxDate = 24;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);   // XXXXXXXX Huh?

	// must set up the races of the teams in the script
	info.fEnemyRace = kLyran; // WYN
	info.fAlliedRace = kKlingon;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kDerelictTeam, info.fAlliedRace);
	fMission.mSetTeamRace (kWYNTeam, info.fEnemyRace);


	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DerelictTeam = fMission.mGetTeamHandle (kDerelictTeam);
	tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);


	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

		switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * 1.0;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.4;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.7;
			break;
	}
	
	// create Orion team
	//WYNTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	int32 WYNShipCount = 3; // PlayerTeam->mGetShipsInPlayCount();
	WYNTeam->mCreateFleet (1, (TotalBPV/3), kClassFrigate,  kClassHeavyCruiser, 50, kStartPosition_J);
	WYNTeam->mCreateFleet (1, (TotalBPV/3), kClassFrigate,  kClassHeavyCruiser, 50, kStartPosition_K);
	WYNTeam->mCreateFleet (1, (TotalBPV/3), kClassFrigate,  kClassHeavyCruiser, 50, kStartPosition_L);


	// create Freighter team
	DerelictTeam->mCreateShip ("K-F5C", kStartPosition_M);
	DerelictTeam->mCreateShip ("K-D7W", kStartPosition_N);
	DerelictTeam->mCreateShip ("K-D6", kStartPosition_O);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kMissionBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DerelictTeam = fMission.mGetTeamHandle (kDerelictTeam);
	tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);


	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure Orion team
	for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetFinalDestination();
		//Ship->mDamageSystem(kDamageLeftWarpEngine, 80);
		//Ship->mDamageSystem(kDamageRightWarpEngine, 80);
	}

	// configure Freighter Team
	for (Ship = DerelictTeam->mGetFirstShip(); Ship != NULL; Ship = DerelictTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipThrust (10);
		//Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
		Ship->mDamageSystem(kDamageMaxWeapons, 100);
		Ship->mDamageSystem(kDamageMaxWeapons, 100);
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 0 );
		Ship->mSetShipAIControlled(kFalse);
		Ship->mSetShipMaxSpeed(0.0f);
	}


	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer


	DeepScanTracker.mSetTarget (DerelictTeam->mGetThisShip(1), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (DerelictTeam->mGetThisShip(2), kDeepScanDistancePk, 1);
	DeepScanTracker.mSetTarget (DerelictTeam->mGetThisShip(3), kDeepScanDistancePk, 1);
	
	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point to freighter to scan
//	fMission.mSetGoalIndicator (DerelictTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DerelictTeam = fMission.mGetTeamHandle (kDerelictTeam);
	tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);


	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);


				if ( HostileShip != NULL )
				{
					if ((Ship->mGetTeam() == kDerelictTeam) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						fMission.mDisplayMessage( kDumbShit, kMissionScriptMessage );
					}
					else if ((Ship->mGetTeam() == kWYNTeam) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						switch (fEventItem1)
						{
							case kDamageWeaponByMarines:
							case kDamageSystemByMarines:
								if( Ship->mGetShipID() == (WYNTeam->mGetThisShip(1))->mGetShipID() && WYN_TRACKED[0] == 0)
								{
									fMission.mDisplayMessage( kHitRaidSuc, kMissionScriptMessage );
									WYN_TRACKED[0] = 1;
								}else 
								if( Ship->mGetShipID() == (WYNTeam->mGetThisShip(2))->mGetShipID() && WYN_TRACKED[1] == 0)
								{
									fMission.mDisplayMessage( kHitRaidSuc, kMissionScriptMessage );
									WYN_TRACKED[1] = 1;
								}else
								if( Ship->mGetShipID() == (WYNTeam->mGetThisShip(3))->mGetShipID() && WYN_TRACKED[2] == 0)
								{
									fMission.mDisplayMessage( kHitRaidSuc, kMissionScriptMessage );
									WYN_TRACKED[2] = 1;
								}
								break;
							default:
								// Nothing happends;
								;
						}
					}
					if(Ship->mGetTeam() == kWYNTeam)
					{
						if(Ship->mGetDamagePercent() > kWYNDamageThreshhold)
						{
							//fMission.mDisplayMessage( kButtonSurrender_text, kMissionScriptMessage );
							Ship->mClearCommands();
							Ship->mDisengage(kByMapEdge, kAbsolutePriority);
						}
					}
				}
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						int distanceFromEnemy = 0;
						for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
						{

							distanceFromEnemy = fPlayerShip->mGetTacticalDistance(Ship);

							if(  VoicedWarOpionin == 0 && distanceFromEnemy <= kDistanceFromWYNForWARDeclaration )
							{
								VoicedWarOpionin = 1;
							//	fMission.mDisplayMessage(kScanHulkOne, kScienceScriptMessage);
							}
							if( VoicedSuspicion == 0 && distanceFromEnemy <= kDistanceFromWYNForWeirdness)
							{
								VoicedSuspicion = 1;
								fMission.mDisplayMessage(kComingWithinWYNRange, kScienceScriptMessage);
							}
							if( VoicedKnowledge == 0 && distanceFromEnemy <= kDistanceFromWYNForKnowledge)
							{
								VoicedKnowledge  = 1;
								fMission.mDisplayMessage(kWYNScanned, kScienceScriptMessage);
							}
						}

						switch (flag)
						{
							case kEnteringRange_ScanOff:
								{
									for (int EnterScanRangeOffLoop = 0;EnterScanRangeOffLoop<4;EnterScanRangeOffLoop++)
									{
										if ( Object == DerelictTeam->mGetThisShip(EnterScanRangeOffLoop+1) && HulkScanned[EnterScanRangeOffLoop] == 0)
										{
											HulkScanned[EnterScanRangeOffLoop] = 1;
											fMission.mDisplayMessage(kScanHulkOne, kScienceScriptMessage);
										}
									}
								}
								break;
							case kEnteringRange_ScanOn_TimerStarted:
							case kInRange_ScanOn_TimerStarted:
								{
									for (int EnterScanRangeOffLoop = 0;EnterScanRangeOffLoop<4;EnterScanRangeOffLoop++)
									{
										if ( Object == DerelictTeam->mGetThisShip(EnterScanRangeOffLoop+1) && HulkScanned[EnterScanRangeOffLoop] == 0)
										{
											fMission.mDisplayMessage(kScanningSir, kScienceScriptMessage);
										}
									}
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								fMission.mDisplayMessage (kScanningStopped, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								fMission.mDisplayMessage (kScanningStoppedOOR, kScienceScriptMessage);
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							for (int EnterScanRangeOffLoop = 0;EnterScanRangeOffLoop<4;EnterScanRangeOffLoop++)
							{
								if ( Object == DerelictTeam->mGetThisShip(EnterScanRangeOffLoop+1) && HulkScanned[EnterScanRangeOffLoop] == 0)
								{
									HulkScanned[EnterScanRangeOffLoop] = 1;
									fMission.mDisplayMessage(kScanHulkOne+EnterScanRangeOffLoop, kScienceScriptMessage);
								}
								if(Object == WYNTeam->mGetThisShip(EnterScanRangeOffLoop+1) && WYNScanned == 0)
								{
									WYNScanned = 1;
									fMission.mDisplayMessage(kWYNScanned, kScienceScriptMessage);
								}
							}
							// player has successfully scanned the object
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_9:
					// set ALL the ship's comm buttons back to HAIL
					for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set ALL the ship's comm buttons back to HAIL
					for (Ship = DerelictTeam->mGetFirstShip(); Ship != NULL; Ship = DerelictTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			if (Ship->mGetTeam() == kWYNTeam)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
				// we don't test objectives now - only when ship blows up
				break;
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kDerelictTeam)
			{
				HulksDestroyed++;
			}
			else if (Ship->mGetTeam() == kWYNTeam)
			{
				WYNDestroyed++;
				TestObjectives (*this);
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kWYNTeam)
			{
				tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);
				gTotalWYNShipsdeparted++;
				for(int shipIndex = 0;shipIndex<3;shipIndex++)
				{
					if(Ship == WYNTeam->mGetThisShip(shipIndex+1))
					{
						if(WYN_TRACKED[shipIndex])
						{
							fMission.mDisplayMessage(kWYNEscapedGood, kMissionScriptMessage);
							WYN_ESCAPES[shipIndex] = 0;
						}
						else
						{
							fMission.mDisplayMessage(kWYNEscapedBad, kMissionScriptMessage);
							WYN_ESCAPES[shipIndex] = 1;
						}
						
					}
				}

				if(WYNDestroyed+gTotalWYNShipsdeparted>=3)
				{
					if(WYN_ESCAPES[0]+WYN_ESCAPES[1]+WYN_ESCAPES[2])
					{
						return kPlayerLoses;
					}else
					{
						return kPlayerWins;
					}
				}

				TestObjectives (*this);
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kDerelictTeam:
								fMission.mDisplayMessage(kDerelictNot_msg, kMissionScriptMessage);
								break;
							case kWYNTeam:
								fMission.mDisplayMessage(kWYN_msg, kMissionScriptMessage);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kDerelictTeam:
								fMission.mDisplayMessage(kDerelictNot_msg, kMissionScriptMessage);
								break;

							case kWYNTeam:
								fMission.mDisplayMessage(kWYN_msg, kMissionScriptMessage);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kDerelictTeam:
								fMission.mDisplayMessage(kDerelictNot_msg, kMissionScriptMessage);
								break;

							case kWYNTeam:
								fMission.mDisplayMessage(kWYN_msg, kMissionScriptMessage);
								break;
						}
						break;
				
				}

			}
			break;
	}

	eGameStatus  gamestatus = fMission.mGetGameStatus();


	return gamestatus;
	
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	int totalEscapes = WYN_ESCAPES[0]+WYN_ESCAPES[1]+WYN_ESCAPES[2];
	int totalTracks = WYN_TRACKED[0]+WYN_TRACKED[1]+WYN_TRACKED[2];

	VictoryPoints = (HulksDestroyed * -35) + (WYNDestroyed * 20);
	VictoryPoints += (totalTracks * 40) - (totalEscapes * 10);
	// Penalty for not getting any tracks
	if(totalTracks < 1)
	{
		VictoryPoints -= 50;
	}
	if(VictoryPoints<0)
	{
		VictoryPoints = 0;
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints >= 50)
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints >= 35)
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList ((kMaxVictoryLevels-VictoryLevel) + kDebrief_1 -1 );

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& Script, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += Script.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += Script.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += Script.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kDerelictTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kWYNTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);


	tScript.fMission.mSetTeamRelations (kDerelictTeam, kWYNTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
}

