#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Kli_Special2Text.h"
#include "Kli_Special2Maps.h"

#define kScannerTimer kTimerLimitSet_10
#define kThreeMinuteBonusPoints 20
#define kDamageTimer 30

#define kDist_Understanding 175.0f
#define kDist_SCAN 20.0f
#define kDist_NON_SCAN 10.0f

#define kClue0_Understanding 0
#define kClue1_Jammer 1
#define kClue2_General 2
#define kClue3_Tight 3

// sequence of events
// 1.  The player sees three ships that are non functioning  ahead of him
// 2.  If they scan, most of the ships have been stripped down (one is fine)
// 3.  Three Lyran(Wyn) ships approach.
// 4.  The science officer suggests that a bug could be placed in the ships via a capture or hit and run option.
// 5.  At 35% damage the WYN ships bail.

int32 FinalMessage = 0;
int32 NoticedSomeOddHarmonic = 0;
int32 RecordedHarmonicAtHighRes = 0;


int32 AllAstrosDestroyed = 0;
int32 AstrosDestroyed = 0;

int32 damageTimer = 0;
int32 secondsIntoTheMission = 0;

int32 buggersInRange = 0;
int32 buggerXReporting[3] = {0,0,0};

int32 CluesGiven[4] = {0,0,0,0};

const float kNoticedHarmonicDist = 30.0f;
const float kRecordedHarmonicDist = 10.0f;

const float kSensorRange = 100.0f;

eClassTypes PlayerShipClass= kClassShuttle;
int32 Elite = 0;
int32 PlayLevel = 0;

enum eOfferStatus
{
	kNoOffer = 0,
	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
};

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);


// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kPlanetTeam,
	kAstroTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission; //XXXXXX Huh?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);   // XXXXXXXX Huh?

	// must set up the races of the teams in the script
	info.fEnemyRace = kHydran; // WYN
	info.fAlliedRace = kKlingon;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kPlanetTeam, info.fAlliedRace);
	fMission.mSetTeamRace (kAstroTeam, info.fEnemyRace);


	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	//return kTrue;  // use player's ship
	return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* AstroTeam = fMission.mGetTeamHandle (kAstroTeam);


	
	PlanetTeam->mCreateShip ("PL4", kStartPosition_N);
	PlanetTeam->mCreateShip ("PL4", kStartPosition_M);
	PlanetTeam->mCreateShip ("PL4", kStartPosition_O);	


	PlayerTeam->mCreateShip ("K-D6X", kStartPosition_G);

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kMissionBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* AstroTeam = fMission.mGetTeamHandle (kAstroTeam);


	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure Orion team
	for (Ship = AstroTeam->mGetFirstShip(); Ship != NULL; Ship = AstroTeam->mGetNextShip())
	{
//			Ship->mSetFinalDestination();
		//Ship->mDamageSystem(kDamageLeftWarpEngine, 80);
		//Ship->mDamageSystem(kDamageRightWarpEngine, 80);
	}

	// configure Freighter Team
	for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (10);
		//Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetShipAIControlled(kFalse);
		Ship->mSetShipMaxSpeed(0.0f);
	}


	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mDamageSystem(kDamageMaxShields,70.0f);
	}



	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kScannerTimer, 1);	// will use kScannerTimer as polling timer


	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point to freighter to scan
//	fMission.mSetGoalIndicator (PlanetTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* AstroTeam = fMission.mGetTeamHandle (kAstroTeam);


	switch (Event)
	{
		case kNotifyDamage:
			{
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					{
						AstroTeam->mCreateShip("M-SSS", kStartPosition_K);
						AstroTeam->mCreateShip("M-SSS", kStartPosition_L);
						

						switch( PlayLevel )
						{
							case 0:		// easy
								break;
							case 1:		// medium
								AstroTeam->mCreateShip("M-SSS", kStartPosition_P);
								AstroTeam->mCreateShip("M-SSL", kStartPosition_J);
								break;

							case 2:		// hard
								AstroTeam->mCreateShip("M-SSS", kStartPosition_P);
								AstroTeam->mCreateShip("M-SSL", kStartPosition_J);
								AstroTeam->mCreateShip("M-SSL", kStartPosition_Q);
								break;
						}
						// hack to Set team relations
						SetAllRelations (*this);
						fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					}
					break;

					// Set every one second?
				case kScannerTimer:
					{
						// Do old stuff for deep scan.  Mostly I just set up the distance messages
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* LoopShip = NULL; 
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						int distanceFromEnemy = 0;
						Ship = PlanetTeam->mGetFirstShip();
						int shipIndex = 0;
						int scanDistance = kDist_NON_SCAN;
						if(fIsDeepScanOn)
						{
							scanDistance = kDist_SCAN;
						}
						for (LoopShip = PlanetTeam->mGetFirstShip(); LoopShip != NULL; LoopShip = PlanetTeam->mGetNextShip())
						{
							distanceFromEnemy = fPlayerShip->mGetTacticalDistance(LoopShip);
							switch(shipIndex)						
							{
								case 0: // Main Planet 
									{
										if(CluesGiven[kClue0_Understanding] == 0&& distanceFromEnemy< kDist_Understanding)
										{
											CluesGiven[kClue0_Understanding] = 1;
											// Say lets get close
											fMission.mDisplayMessage(kUnderstandingAndClue, kScienceScriptMessage );
										}
										
										if( CluesGiven[kClue3_Tight] == 0 && distanceFromEnemy< scanDistance)
										{
											CluesGiven[kClue3_Tight] = 1;
											// Describe Tight beam
											fMission.mDisplayMessage(kTightBeam,kScienceScriptMessage );
											
										}
										break;
									}
								case 1:
									{
										if( CluesGiven[kClue1_Jammer] == 0 && distanceFromEnemy< scanDistance)
										{
											CluesGiven[kClue1_Jammer] = 1;
											// Describe Jamming 
											fMission.mDisplayMessage(kJammerFound,kScienceScriptMessage );

										}
										break;
									}
								case 2:
									{
										if( CluesGiven[kClue2_General] == 0 && distanceFromEnemy< scanDistance)
										{
											CluesGiven[kClue2_General] = 1;
											// Describe General signal
											fMission.mDisplayMessage(kGeneralBecon,kScienceScriptMessage );
										}
										break;
									}	
							}
							shipIndex++;
						}
						if(FinalMessage == 0 && CluesGiven[kClue0_Understanding] + 	CluesGiven[kClue3_Tight] + CluesGiven[kClue1_Jammer] + CluesGiven[kClue2_General] > 3)
						{
							FinalMessage = 1;
							fMission.mDisplayMessage(kLetsGetTheHellOutOfHere, kScienceScriptMessage);
							fMission.mSetGoalIndicator(1,25,50);
						}

						if(buggersInRange<3)
						{
							LoopShip = NULL;
							int BuggerIndex = 0;

							for (LoopShip = AstroTeam->mGetFirstShip(); LoopShip != NULL; LoopShip = AstroTeam->mGetNextShip())
							{
								distanceFromEnemy = fPlayerShip->mGetTacticalDistance(LoopShip);
								if(distanceFromEnemy<kSensorRange && buggerXReporting[BuggerIndex] == 0)
								{
									buggerXReporting[BuggerIndex] = 1;
									switch(buggersInRange)
									{
									case 0:
										// XX Alien Life FOrm appoaching sir
										fMission.mDisplayMessage(kFirstMonster,kScienceScriptMessage );
										buggersInRange++;
										break;
									case 1:
										// XX Another Alien Life  form Commander
										fMission.mDisplayMessage(kSecondMonster,kScienceScriptMessage );
										buggersInRange++;
									case 2:
										// XX Sir I suggest we leave as quickly as possible sir.
										fMission.mDisplayMessage(kThirdMonster,kScienceScriptMessage );
										buggersInRange++;
									default:
										NULL;
									}
									buggersInRange++;
								}
								BuggerIndex++;
							}
						}

						// Keep track of elapsed time.
						secondsIntoTheMission++;
						damageTimer++;
						// Damage the ship at kDamageTimer intervals
						if(damageTimer>kDamageTimer)
						{
							// Do mean shit.
							damageTimer = 0;
						}
					}
					break;
				case kNotifyProbeLaunch:
					break;

				case kNotifyPlayerLeaving:
					// make the player lose
					// PlayerQuit = true;
					break;

				case kTimerCountdownSet:
					{
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_9:
					break;

				case kTimerTurnsSet_10:
					break;
			}
			break;
		
		case kNotifyOnCapture:
			break;
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kAstroTeam)
			{
				AstrosDestroyed++;
				TestObjectives (*this);
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				TestObjectives (*this);
			}
			break;

		case kNotifyCommButtonPress:
			{
			}
			break;
	}

	eGameStatus  gamestatus = fMission.mGetGameStatus();

	return gamestatus;
	
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
/*
	0-24 : Utter Failure
	25-49: Mostly Failure
	50   : Draw
	51-76: Partial Success
	77-101: Total success
*/

	if(CluesGiven[kClue3_Tight])
	{
		// Needed for Victory
		VictoryPoints+=51;
	}

	if(CluesGiven[kClue1_Jammer])
	{
		VictoryPoints+=25;
	}
	if(CluesGiven[kClue2_General])
	{
		VictoryPoints+=25;
	}

	if (VictoryPoints >= 77)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 51)
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints == 50)
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints >= 25)
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList ((kMaxVictoryLevels-VictoryLevel) + kDebrief_1 - 1 );


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			fMission.mScheduleMission( kRandomMission_msg );
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& Script, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += Script.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += Script.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += Script.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kAstroTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlanetTeam, kAstroTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{

	;
}

