#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Kli_Special3Text.h"
#include "Kli_Special3Maps.h"




#define kSecondTimer kTimerLimitSet_10
eClassTypes PlayerShipClass= kClassShuttle;
int32 Elite = 0;
int32 PlayLevel = 0;

int32 reinforcementcounter = 300;

// =========== Score keeping =================
const float FRDReward = 100;

int32 FRDsDestroyed = 0;

// ===========================================

// ========= Death Notice Keeping =========
int32 ShipIDs[3] = { -1,-1,-1 };

enum eOfferStatus
{
	kNoOffer = 0,
	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
};

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);


// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kWYNTeam,
	kOrionTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission; //XXXXXX Huh?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 0, 0);   // XXXXXXXX Huh?

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion; 
	info.fAlliedRace = kKlingon;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kWYNTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kOrionTeam, info.fEnemyRace);

	


	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	//return kTrue;  // use player's ship
	return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	


	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	
	WYNTeam->mCreateShip ("O-FRD", kStartPosition_N);
	WYNTeam->mCreateShip ("O-FRD", kStartPosition_M);
	
	PlayerTeam->mCreateShip("K-C9", kStartPosition_H);
	PlayerTeam->mCreateShip("K-D7", kStartPosition_I);
	PlayerTeam->mCreateShip("K-D7", kStartPosition_G);

	tShipInfo* Ship;

	for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
	{
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
		Ship->mDamageSystem(kDamageMaxWeapons, 100);
		Ship->mSetShipMaxSpeed(0.0f);
		Ship->mSetShipAIControlled(kFalse);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kMissionBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);


	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure Orion team
	for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetFinalDestination();
	}

	for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetFinalDestination();
	}



	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);
	fMission.mSetTimedInterval (kSecondTimer, 1);

	// point to First DEF platform to destroy
	fMission.mSetGoalIndicator (WYNTeam->mGetFirstShip());
}

void CheckFRDDestructionProgress(tScript& Script)
{
	if(FRDsDestroyed == 1)
	{
		Script.fMission.mDisplayMessage( kOneFRDsDown_msg, kScienceScriptMessage );
	}
	else
	{
		Script.fMission.mDisplayMessage( kTwoFRDsDown_msg, kScienceScriptMessage );
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* WYNTeam = fMission.mGetTeamHandle (kWYNTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	int RandomNumber = 0;

	switch (Event)
	{
		case kNotifyDamage:
			if(Ship->mGetTeam() == kPlayerTeam)
			{
				int32 ShipID = Ship->mGetShipID();
				if(Ship->mGetDamagePercent() < 35.0f )
				{
					for(int ShipIDLoop = 0;ShipIDLoop<3;ShipIDLoop++)
					{
						if(ShipIDs[ShipIDLoop] == ShipID)
						{
							break; // Already processed this so we won't write the message again
						}
						// New location so we
						if(ShipIDs[ShipIDLoop] == -1)
						{
							fMission.mDisplayMessage(kWeAreDoomed_msg, kMissionScriptMessage);
							ShipIDs[ShipIDLoop] = ShipID;
							break; // We have displayed the message so we stop.
						}
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:

					for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
					{
						Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
						Ship->mDamageSystem(kDamageRightWarpEngine, 100);
						Ship->mDamageSystem(kDamageMaxWeapons, 100);
						Ship->mSetShipMaxSpeed(0.0f);
						Ship->mSetShipAIControlled(kFalse);
					}
					for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
					{
						Ship->mDamageSystem(kDamageLeftWarpEngine, 30);
						Ship->mDamageSystem(kDamageRightWarpEngine, 30);
						Ship->mDamageSystem(kDamageMaxWeapons, 15);
						Ship->mDamageSystem(kDamageMaxShields,40);
					}
					// hack to Set team relations
					OrionTeam->mCreateShip ("O-LR1", kStartPosition_Q);
					OrionTeam->mCreateShip ("O-LR1", kStartPosition_L);
					OrionTeam->mCreateShip ("O-LR1", kStartPosition_K);
					switch(PlayLevel)
					{
					case 0:
						break;
					case 1:
						OrionTeam->mCreateShip ("O-LR1", kStartPosition_O);
						break;
					case 2:
						OrionTeam->mCreateShip ("O-LR1", kStartPosition_O);
						OrionTeam->mCreateShip ("O-LR1", kStartPosition_J);
						break;
					}

					SetAllRelations (*this);
					for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
					{
						Ship->mSetCommButton (1, kButtonHail_text);
						Ship->mSetFinalDestination();
					}

					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mSetCommButton (1, kButtonHail_text);
						Ship->mSetFinalDestination();
					}

					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kSecondTimer:
					{
						reinforcementcounter++;
						if(reinforcementcounter>300)
						{
							int randomShip = fMission.mRandomInt32( 0, 2 );
							switch(randomShip)
							{
							case 0:
								WYNTeam->mCreateShip ("H-LC", kStartPosition_J);
								break;
							case 1:
								WYNTeam->mCreateShip ("K-D7", kStartPosition_J);
								break;
							case 2:
								WYNTeam->mCreateShip ("F-CC", kStartPosition_J);
								break;
							}
							reinforcementcounter = 0;
						}
						fMission.mSetTimedInterval (kSecondTimer, 1);
					}
					break;

				case kTimerCountdownSet:
					break;
				case kTimerTurnsSet:
					break;
				case kTimerTurnsSet_9:
					// set ALL the ship's comm buttons back to HAIL
					for (Ship = WYNTeam->mGetFirstShip(); Ship != NULL; Ship = WYNTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
				case kTimerTurnsSet_10:
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (Ship->mGetTeam() == kWYNTeam || Ship->mGetTeam() == kOrionTeam)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );

				if(strcmp( Ship->mGetClassName(), "O-FRD" ) == 0 )
				{
					FRDsDestroyed ++;
					CheckFRDDestructionProgress(*this);
				}
				// we don't test objectives now - only when ship blows up
			}
			break;
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kWYNTeam)
			{
				TestObjectives (*this);
				if(strcmp( Ship->mGetClassName(), "O-FRD" ) == 0 )
				{
					FRDsDestroyed ++;
					CheckFRDDestructionProgress(*this);
				}
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				// Check if it too damaged to survive
				if(Ship->mGetDamagePercent() > 50.0f)
				{
					return kPlayerDestroyed	;
				}
				TestObjectives (*this);
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kWYNTeam:
								RandomNumber = fMission.mRandomInt32(0,3);
								switch(RandomNumber)
								{
									case 0:
										fMission.mDisplayMessage(kWYN_HAIL0_msg, kMissionScriptMessage);
										break;
									case 1:
										fMission.mDisplayMessage(kWYN_HAIL1_msg, kMissionScriptMessage);
										break;
									case 2:
										fMission.mDisplayMessage(kWYN_HAIL2_msg, kMissionScriptMessage);
										break;
									case 3:
										fMission.mDisplayMessage(kWYN_HAIL3_msg, kMissionScriptMessage);
										break;
								}
								break;
							case kOrionTeam:
								RandomNumber = fMission.mRandomInt32(0,3);
								switch(RandomNumber)
								{
									case 0:
										fMission.mDisplayMessage(kOrion_HAIL0_msg, kMissionScriptMessage);
										break;
									case 1:
										fMission.mDisplayMessage(kOrion_HAIL1_msg, kMissionScriptMessage);
										break;
									case 2:
										fMission.mDisplayMessage(kOrion_HAIL2_msg, kMissionScriptMessage);
										break;
									case 3:
										fMission.mDisplayMessage(kOrion_HAIL3_msg, kMissionScriptMessage);
										break;
								}
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{

							case kWYNTeam:
								fMission.mDisplayMessage(kWYN_SURRENDER_msg, kMissionScriptMessage);
								break;
							case kOrionTeam:
								fMission.mDisplayMessage(kOrion_SURRENDER_msg, kMissionScriptMessage);
								break;
						}
						break;
				}

			}
			break;
	
	}

	eGameStatus  gamestatus = fMission.mGetGameStatus();



	return gamestatus;
	
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = FRDsDestroyed * FRDReward;
	// Penalty for not getting any tracks
	if (VictoryPoints >= 200)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList ((kMaxVictoryLevels-VictoryLevel) + kDebrief_1 -1);

//	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
//	int32 SuccessCount = 0;
//	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

//	( TalkedToFreighter ) ? fMission.mAddToDebriefingList( kTalkedToFreighterTrue_msg ) : SuccessCount++;
//	( AllWYNDestroyed ) ? fMission.mAddToDebriefingList( kAllWYNDestroyedTrue_msg ) : SuccessCount++;
//	if ( SuccessCount == 3 )
//	{
//		fMission.mAddToDebriefingList( kNone_msg );
//	}
/*
	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( TalkedToFreighter == 0 ) ? fMission.mAddToDebriefingList( kTalkedToFreighterFalse_msg ) : FailureCount++;
	( AllWYNDestroyed == false ) ? fMission.mAddToDebriefingList( kAllWYNDestroyedFalse_msg ) : FailureCount++;
	(AttackedFreighter) ? fMission.mAddToDebriefingList( kFreighterAttacked_msg ) : FailureCount++;
	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
	*/
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& Script, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += Script.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += Script.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += Script.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kWYNTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOrionTeam, kWYNTeam,  kTeamAllied, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
}

