#include "scriptinterface.h"

#include "Kli_1AlternateText.h"
#include "Kli_1AlternateMaps.h"

bool Scanned = false;
bool Probed = false;
bool AlienAttack = false;
bool AttackerIsLyran = false;
bool GotDataFromFed = false;
bool SentDataToFed = false;
bool FedCaptured = false;
bool AttackerCaptured = false;
bool FedDestroyed = false;
bool AttackerDestroyed = false;

eClassTypes PlayerShipClass= kClassShuttle;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFedTeam, kAggressorTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;	// kLyranMission | kHydranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 0;	// LNF: temp for E3/Beta only
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (100, 80, 50, 20, 0);

	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFedTeam, kFederation);

	if( fMission.mRandomInt32( 1, 3 ) == 1 )
	{
		AttackerIsLyran = true;
		info.fEnemyRace = kLyran;	// 33% of Lyran ship
	}
	else
	{
		info.fEnemyRace = kHydran;
	}
	fMission.mSetTeamRace (kAggressorTeam, info.fEnemyRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kAggressorTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	tShipInfo* Ship;

	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}
	
	if (Mode == 0)
	{
		FedTeam->mCreateShip( TotalBPV, kClassFrigate, PlayerShipClass, 20, kStartPosition_X );
	}
	else
	{
		fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		AggressorTeam->mCreateShip (TotalBPV, kClassFrigate, PlayerShipClass, 20, kStartPosition_Specified, 20, 20);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		Ship->mSetShipCallbackEvents( kNotifyProbeLaunch );
		Ship->mTrackTacticalDistance(30.0, 20, 20); // distance to special fissure
	}

	// configure Federation ship
	Ship = FedTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mClearCommButton( 2 );
	fMission.mSetTeamRelations( kPlayerTeam, kAggressorTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyProbeLaunch:
			if( Scanned == true )
			{
				if( Probed == false )
				{
					fMission.mDisplayMessage( kProbeLaunch_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet, 5 );
					Probed = true;
				}
				else
				{
					fMission.mDisplayMessage( kProbeLaunch_msg, kScienceScriptMessage );
				}
			}
			else
			{
				fMission.mDisplayMessage( kMustScanFirst_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				if( Scanned == false )
				{
					if( fPlayerShip->mGetTacticalDistance( 20, 20 ) <= 30.0 )
					{
						fMission.mDisplayMessage( kEventScanPhenom_msg, kScienceScriptMessage );
						fMission.mSetTimedInterval( kTimerLimitSet_4, 5 );
						Scanned = true;
					}
					else
					{
						fMission.mDisplayMessage( kTooFarToScan_msg, kScienceScriptMessage );
					}
				}
			}
			else if( Scanned == false )
			{
				fMission.mDisplayMessage( kStoppedScan_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyAtDistance:
			if( Scanned == false )
			{
				if (fIsDeepScanOn == kTrue)
				{
					fMission.mDisplayMessage( kEventScanPhenom_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_4, 5 );
					Scanned = true;
				}
				else
				{
					Ship->mTrackTacticalDistance(30.0, 20, 20); // distance to special fissure
				}
			}
			break;

		case kNotifyDamage:
			Ship->mSetCommButton( 1, kButtonTaunt_text );
			Ship->mSetCommButton( 2, kButtonSurrender_text );
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kEvent6Strange_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_2, 5 );
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kEvent6AlienShip_msg, kNonCommScriptMessage );
					mCreateMissionShips( 1 );
					if( FedDestroyed == false )
					{
						FedTeam->mGetFirstShip()->mDisengage( kByMapEdge, kAbsolutePriority );
					}
					break;

				case kTimerLimitSet_3:
					fMission.mDisplayMessage( kScienceWarning_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_4:
					fMission.mDisplayMessage( kLaunchAProbe_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_10:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mDisplayMessage( kReadyToScan_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kFedShipAlert_msg, kScienceScriptMessage );
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kFedHailInfo1_msg );
					break;
			}
			break;

		
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else if( Ship->mGetTeam() == kAggressorTeam )
			{
				AttackerDestroyed = true;
				return kPlayerWins;
			}
			else if( Ship->mGetTeam() == kFedTeam )
			{
				FedDestroyed = true;
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				// LNF if it does/will change the 1 to 0
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerCaptured;
				}
			}
			else if( Ship->mGetOriginalTeam() == kAggressorTeam )
			{
				AttackerCaptured = true;
				return kPlayerWins;
			}
			else if( Ship->mGetOriginalTeam() == kFedTeam )
			{
				FedCaptured = true;
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					if( Ship->mGetTeam() == kFedTeam )
					{
						fMission.mDisplayMessage( kFedHailNoInfo1_msg );
						Ship->mSetCommButton( 1, kButtonRequest_text );
						Ship->mSetCommButton( 2, kButtonThreaten_text );
					}
					else
					{
						if( AttackerIsLyran == false )
						{
							fMission.mDisplayMessage( kHydranHailEnemy_msg );
						}
						else
						{
							fMission.mDisplayMessage( kLyranHailEnemy_msg );
						}
						Ship->mSetCommButton( 1, kButtonTaunt_text );
						Ship->mSetCommButton( 2, kButtonSurrender_text );
					}
					break;

				case kButtonTaunt_text:
					if( Ship->mGetTeam() == kFedTeam )
					{
						fMission.mDisplayMessage( fMission.mRandomInt32( kFedTaunt1_msg, kFedTaunt3_msg ) );
					}
					else
					{
						if( AttackerIsLyran == false )
						{
							fMission.mDisplayMessage( fMission.mRandomInt32( kHydranTaunt1_msg, kHydranTaunt4_msg ) );
						}
						else
						{
							fMission.mDisplayMessage( fMission.mRandomInt32( kLyranTaunt1_msg, kLyranTaunt3_msg ) );
						}
					}
					Ship->mSetCommButton( 1, kButtonTaunt_text );
					break;

				case kButtonSurrender_text:
					if( Ship->mGetTeam() == kFedTeam )
					{
						if( Ship->mGetDamagePercent() >= 70 )
						{
							fMission.mDisplayMessage( kFedSurrenderDamaged_msg );
							fMission.mSetTeamRelations( kPlayerTeam, kFedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
							Ship->mClearCommButton( 1 );
							FedCaptured = true;
						}
						else
						{
							fMission.mDisplayMessage( kFedSurrenderOK_msg );
							Ship->mSetCommButton( 2, kButtonSurrender_text );
						}
					}
					else
					{
						if( AttackerIsLyran == true )
						{
							if( Ship->mGetDamagePercent() >= 80 )
							{
								fMission.mDisplayMessage( kLyranSurrenderDamaged_msg );
								Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
								return kPlayerWins;
							}
							else
							{
								fMission.mDisplayMessage( kLyranSurrenderOK_msg );
								Ship->mSetCommButton( 2, kButtonSurrender_text );
							}
						}
						else
						{
							if( Ship->mGetDamagePercent() >= 66 )
							{
								fMission.mDisplayMessage( kHydranSurrenderDamaged_msg );
								AggressorTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
							}
							else
							{
								fMission.mDisplayMessage( kHydranSurrenderOK_msg );
								Ship->mSetCommButton( 2, kButtonSurrender_text );
							}
						}
					}
					break;

				case kButtonRequest_text:
					fMission.mDisplayMessage( kFedHailNoInfo2_msg );
					GotDataFromFed = true;
					Ship->mSetCommButton( 1, kButtonSend_text );
					Ship->mSetCommButton( 2, kButtonSayNo_text );
					fMission.mSetTimedInterval( kTimerTurnsSet, 2 );
					fMission.mSetTimedInterval( kTimerLimitSet_3, 5 );
					break;

				case kButtonThreaten_text:
					fMission.mDisplayMessage( kThreatenFed_msg );
					Ship->mSetCommButton( 1, kButtonTaunt_text );
					Ship->mSetCommButton( 2, kButtonSurrender_text );
					break;

				case kButtonSend_text:
					if( Scanned == true )
					{
						fMission.mDisplayMessage( kDataSent_msg );
						SentDataToFed = true;
						Ship->mClearCommButton( 2 );
					}
					else
					{
						fMission.mDisplayMessage( kNoDataToSend_msg );
						Ship->mSetCommButton( 1, kButtonSend_text );
					}
					break;

				case kButtonSayNo_text:
					fMission.mDisplayMessage( kJustSayNo_msg );
					Ship->mClearCommButton( 1 );
					fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = ( Scanned * 30 ) + ( Probed * 20 ) + ( GotDataFromFed * 30 )
		+ ( FedCaptured * 20 ) + ( FedDestroyed * 10 ) + ( AttackerCaptured * 30 )
		+ ( AttackerDestroyed * 30 ) - ( SentDataToFed * 40 );

	if( PlayerTeam->mGetFirstShip()->mGetDamagePercent() >= 50 )
	{
		VictoryPoints = VictoryPoints - 25;
	}

	if( PlayerTeam->mGetFirstShip()->mGetDamagePercent() >= 75 )
	{
		VictoryPoints = VictoryPoints - 25;
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
	fMission.mAddToDebriefingList( kDebrief_msg );

	if( VictoryLevel > kVictory )
	{
		fMission.mAddToDebriefingList( kDismissedShort_msg );
	}
	else
	{
		fMission.mAddToDebriefingList ( kDismissedLong_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	fMission.mScheduleMission( kRandomMission_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

