#include "scriptinterface.h"

#include "Kli_2bMissionText.h"
#include "Kli_2bMissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 HydranDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 HydranBaseDestroyed = 0;
int32 HydranCaptured = 0;
int32 PlayerCaptured = 0;
int32 HydranBaseCaptured = 0;
int32 StartMission = 0;
int32 AtWarWithHydran = 0;
int32 TotalBPV = 0;
const float kBombDetonationDistance = 2.0; // distance from base when bomb explodes
const float kBombDetectionDistance = 80.0; // distance from base when bomb is detected
const float kBombInterceptionDistance = 60.0; // distance from base when bomb is intercepted
const float kAttackFreighterRange = 15.0; // distance that the Hydran will attack freighter or player
const int32 kWormholeX = 37; // X coord of wormhole
const int32 kWormholeY = 30; // Y coord of wormhole
const float kWormholeWarningDistance = 30.0; // distance to warn player of wormhole
const float kWormholeDistance = 0.0; // distance to disengage player
const int32 kMaxNumberOfHydrans = 5; // max number of hydan in initial wave including the base
bool BombDetected = false;
bool BombIntercepted = false;
bool WinMessageGiven = false;
int32 Mode = 0;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus, tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

enum eMissionMode
{
	kStart = 0,
	kHydranReinforcement
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,
 
	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kHydranTeam,
	kLyranFreighterTeam,
	kHydranReinforcementTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kHydranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, kLyran);
	fMission.mSetTeamRace (kHydranTeam, kHydran);
	fMission.mSetTeamRace (kLyranFreighterTeam, kLyran);
	fMission.mSetTeamRace (kHydranReinforcementTeam, kHydran);

	// LNF check the relationships from Dynaverse first?
	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kLyranFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kHydranTeam, kLyranFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse; // use script given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranFreighterTeam = fMission.mGetTeamHandle (kLyranFreighterTeam);
	tTeamInfo* HydranReinforcementTeam = fMission.mGetTeamHandle (kHydranReinforcementTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by 80% to 120%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .85;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.15;
	TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		// create Player team  LNF this ship should be the same as the ship captured in part 1
		// LNF also will have a lyran destroyer without ESG
		PlayerTeam->mCreateShip("K-LCL", kStartPosition_G);

		// create Hydran team
		//HydranTeam->mCreateFleet (4, TotalBPV,  kClassFrigate, kClassLightCruiser, 50, kStartPosition_V);
		HydranTeam->mCreateShip ("H-HN", kStartPosition_V);
		HydranTeam->mCreateShip ("H-CU", kStartPosition_W);
		HydranTeam->mCreateShip ("H-LN", kStartPosition_X);
		HydranTeam->mCreateShip ("H-HR", kStartPosition_Y);
		// the base must be the last hydran created
		HydranTeam->mCreateShip ("H-BS", kStartPosition_Z); // base station 

		// create LyranFreighter
		LyranFreighterTeam->mCreateShip ("L-FQS", kStartPosition_J);
	}
	else if ( Mode == kHydranReinforcement )
	{
		fMission.mSetTeamRelations (kPlayerTeam, kHydranReinforcementTeam, kTeamNeutral, kGoalIgnore, 1.0);
		fMission.mSetTeamRelations (kHydranReinforcementTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
		fMission.mSetTeamRelations (kLyranFreighterTeam, kHydranReinforcementTeam, kTeamNeutral, kGoalIgnore, 1.0);
		fMission.mSetTeamRelations (kHydranReinforcementTeam, kLyranFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0);
		fMission.mSetTeamRelations (kHydranReinforcementTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		int32 OldX = 0;
		int32 OldY = 0;
		fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + fMission.mRandomInt32 (-10, 10);
		int32 NewY = OldY + fMission.mRandomInt32 (10, 12);
		HydranReinforcementTeam->mCreateFleet( 3, (TotalBPV * 2), kClassDestroyer, kClassHeavyCruiser, 100, kStartPosition_Specified, NewX, NewY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranFreighterTeam = fMission.mGetTeamHandle (kLyranFreighterTeam);
	tShipInfo* Ship;

	int32 ShipNumber = 0; // ship number to set patrol route

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		Ship->mTrackTacticalDistance(kWormholeWarningDistance, kWormholeX, kWormholeY); // distance to wormhole
		if ( fMission.mRandomInt32( 1, 2 ) == 1 )
		{
			Ship->mSetOfficerRank( kMaxOfficers, kVeteran );
		}
		else
		{
			Ship->mSetOfficerRank( kMaxOfficers, kSenior );
		}
	}

	// configure Hydran ships
	for (Ship = HydranTeam->mGetFirstShip(); Ship != NULL; Ship = HydranTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		if (ShipNumber > 0)
		{
			PatrolRoute(Ship, ShipNumber);
		}
		ShipNumber++;
	}
	// LNF this may change depending on the Hydran fleet size and ease of killing freighter
	HydranTeam->mGetThisShip( kMaxNumberOfHydrans )->mDamageSystem( kDamageMaxWeapons,  70 );

	// configure Lyran Freighter
	Ship = LyranFreighterTeam->mGetThisShip(1);
	Ship->mSetFinalDestination(20, 20);
	Ship->mSetShipThrust(10);	// LNF set the ships MAX thrust to 10, this will affect the distance
								// that the Hydrans engage the freighter
								// the freighter should be destroyed unless the player distracts some
								// of the Hydran patrol vessels
								// LNF maybe make the Hydran ships bigger, and destroy some of them when
								// the starbase explodes, giving the player a chance to escape?
	Ship->mSetShipMaxSpeed( 22 );
	Ship->mTrackTacticalDistance(kBombDetonationDistance, 20, 20);  // distance to base station - kaboom
	//Ship->mSetShipCallbackEvents (kNotifyDamage);

	// display starting message
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// start a timer to check the distance of ANY Hydran to ANY of your ships
	fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );

	// make the Hydrans attack the freighter or player
	fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranFreighterTeam = fMission.mGetTeamHandle (kLyranFreighterTeam);

	tShipInfo* HydranBase = HydranTeam->mGetThisShip(5);
	tShipInfo* Bomb = LyranFreighterTeam->mGetThisShip(1);
	tShipInfo* LyranShip = NULL;
	tShipInfo* HydranShip = NULL;


	switch (Event)
	{
		case kNotifyAtDistance:
			if ( Ship == fPlayerShip )
			{
				if (fPlayerShip->mGetTrackingDistance() == kWormholeWarningDistance)
				{
					//warn player approaching wormhole
					fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
					if ( HydranBaseDestroyed == 0 )
					{
						// warn player to destroy base first
						fMission.mDisplayMessage( kEvent7_ApproachWormholeNoKill_msg, kMissionScriptMessage );
						fPlayerShip->mTrackTacticalDistance(kWormholeDistance, kWormholeX, kWormholeY);
					}
					else
					{
						fPlayerShip->mTrackTacticalDistance(kWormholeDistance, kWormholeX, kWormholeY);
					}					
				}
				else if (fPlayerShip->mGetTrackingDistance() == kWormholeDistance)
				{
					fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
					fMission.mSetGoalIndicator( kFalse, 0, 0 );
					return kPlayerDisengaged;
				}
			}
			else if (HydranBase->mGetTacticalDistance(Bomb) <= kBombDetonationDistance)
			{
				HydranBaseDestroyed = 1;
				LyranFreighterTeam->mGetThisShip(1)->mDamageSystem(kDamageExcessDamageSystem, 100);
				LyranFreighterTeam->mGetThisShip(1)->mDamageSystem(kDamageInternalsWithPoints, 10000);
				HydranTeam->mGetThisShip(5)->mDamageSystem(kDamageExcessDamageSystem, 100);
				HydranTeam->mGetThisShip(5)->mDamageSystem(kDamageInternalsWithPoints, 10000);
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if (( HostileShip->mGetTeam() == kPlayerTeam ) &&
						( Ship->mGetTeam() == kHydranTeam ))
					{
						if (AtWarWithHydran == 0)
						{
							fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							AtWarWithHydran = 1;
						}
					}
				}
			}
			break;

		case kNotifyDisengaged:
			if ( Ship->mGetTeam() == kPlayerTeam )
			{
				if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDisengaged;
				}
				else // the player still has ships
				{
				}
			}
			else
			{
				CheckTeamDestruction(*this, fMission.mGetTeamHandle (Ship->mGetOriginalTeam()), kEventShipDisengaged, Ship);
			}
			break;

		case kNotifyOnCapture:
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				tShipInfo* CapturingShip = fMission.mFindShipID (fEventID);

				if ( Ship->mGetTeam() == kPlayerTeam )
				{
					if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						return kPlayerCaptured;
					}
					else  // the player still has ships
					{
					}
				}
				else
				{
					CheckTeamDestruction(*this, fMission.mGetTeamHandle (Ship->mGetOriginalTeam()), kEventShipCaptured, Ship, CapturingShip);
				}
			}
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kPlayerTeam )
			{
				if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
				else // the player still has ships
				{
				}
			}
			else
			{
				CheckTeamDestruction(*this, fMission.mGetTeamHandle (Ship->mGetOriginalTeam()), kEventShipDestroyed, Ship);
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					if (StartMission == 0)
					{
						fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kLyranFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kHydranTeam, kLyranFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kLyranFreighterTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0);

						StartMission = 1;

						fMission.mDisplayMessage( kEvent1_StartScenario1_msg, kMissionScriptMessage );
						fMission.mDisplayMessage( kEvent1_StartScenario2_msg, kMissionScriptMessage );
					}
					break;

				case kTimerLimitSet_2:// check distance between kPlayerTeam and kHydranTeam
					{
						for (LyranShip = PlayerTeam->mGetFirstShip(); LyranShip != NULL; LyranShip = PlayerTeam->mGetNextShip())
						{
							for (HydranShip = HydranTeam->mGetFirstShip(); HydranShip != NULL; HydranShip = HydranTeam->mGetNextShip())
							{
								if (( LyranShip->mGetTacticalDistance(HydranShip) <= kBombDetectionDistance ) &&
									( BombDetected == false ) && ( BombIntercepted == false ))
								{
									// first warning to Lyran from Hydran
									fMission.mDisplayMessage( kHydranHailNic_msg, kMissionScriptMessage );
									BombDetected = true;
								}
								else if (( LyranShip->mGetTacticalDistance(HydranShip) <= kBombInterceptionDistance ) &&
										( BombIntercepted == false ))
								{
									// the player get's closer than the DetectionDistance and is Intercepted
									fMission.mDisplayMessage( kHydransClosingToAttack_msg, kMissionScriptMessage );
									// LNF do I have to clearCommands here?
									fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
									fMission.mSetTeamRelations (kHydranTeam, kLyranFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0);
									fMission.mSetTeamRelations (kLyranFreighterTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0);
									BombIntercepted = true;
								}
								else if ( BombIntercepted == false )
								{
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
								}
							}
						}
					}
					break;

				case kTimerLimitSet_3:
					// check to see if the distance from the player to the freighter is <= 20.0
					// if it is <= 20.0, make the Hydrans attack the player.  If > 20.0 attack freighter
					if ( HydranBaseDestroyed == 0 )
					{
						if ( fPlayerShip->mGetTacticalDistance(LyranFreighterTeam->mGetFirstShip()) <= kAttackFreighterRange )
						{
							fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kHydranTeam, kLyranFreighterTeam, kTeamNeutral, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kLyranFreighterTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0);
						}
						else
						{
							fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
							fMission.mSetTeamRelations (kHydranTeam, kLyranFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kLyranFreighterTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0);
						}
						fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					}
					else
					{
						fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0);
						fMission.mSetTeamRelations (kHydranTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
					}

					break;

				case kTimerTurnsSet:
					// Hydran Reinforcements arrive
					fMission.mDisplayMessage( kEvent3_HydranReinforcement_msg, kMissionScriptMessage );
					Mode = kHydranReinforcement;
					mCreateMissionShips ( Mode );
					break;
					
				case kTimerTurnsSet_10:
					// set the Hydran comm buttons back to HAIL
					for (Ship = HydranTeam->mGetFirstShip(); Ship != NULL; Ship = HydranTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyCommButtonPress:
			{

				switch (fEventID)
				{
					case kButtonHail_text:
						if (AtWarWithHydran == 0)
						{
							fMission.mDisplayMessage (kHydranHailNic_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kHydranHail_msg, kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonSurrender_text:
						if (Ship->mGetDamagePercent() < 50.0)
						{
							fMission.mDisplayMessage (kHydranSurrenderOK_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kHydranSurrenderDamaged_msg, kMissionScriptMessage);
						}
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						if (AtWarWithHydran == 0)
						{
							fMission.mDisplayMessage (kHydranTauntNic_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kHydranTaunt1_msg, kHydranTaunt4_msg), kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

				}
			}
			break;

	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
	
	// LNF check the victory points for accuracy and balance
	VictoryPoints = (HydranBaseDestroyed * 80) + (HydranDestroyed * 5);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (VictoryLevel > kVictory)
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( HydranBaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kHydranBaseDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( HydranBaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kHydranBaseDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount ==  1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
	const float PatrolAttackRadius = 1.0;
		
	tMapPoint waypoints1[6]= // ship at W
	{
		19, 22,
		16, 20,
		17, 15,
		21, 14,
		24, 18,
		21, 22
	};

	tMapPoint waypoints2[6]= // ship at X
	{
		20, 22,
		18, 21,
		17, 18,
		20, 16,
		23, 18,
		22, 21
	};

	tMapPoint waypoints3[6]= // ship at Y
	{
		20, 16,
		18, 18,
		18, 21,
		20, 22,
		22, 20,
		22, 18
	};

	tMapPoint PatrolPoints[3][10];

	switch (ShipNumber)
	{
		case 0:
			PatrolPoints[0][0] = waypoints1[0];
			PatrolPoints[0][1] = waypoints1[1];
			PatrolPoints[0][2] = waypoints1[2];
			PatrolPoints[0][3] = waypoints1[3];
			PatrolPoints[0][4] = waypoints1[4];
			PatrolPoints[0][5] = waypoints1[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[0][0], kHighPriority);
			break;
		case 1:
			PatrolPoints[1][0] = waypoints2[0];
			PatrolPoints[1][1] = waypoints2[1];
			PatrolPoints[1][2] = waypoints2[2];
			PatrolPoints[1][3] = waypoints2[3];
 			PatrolPoints[1][4] = waypoints2[4];
			PatrolPoints[1][5] = waypoints2[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[1][0], kHighPriority);
			break;
		case 2:
			PatrolPoints[2][0] = waypoints3[0];
			PatrolPoints[2][1] = waypoints3[1];
			PatrolPoints[2][2] = waypoints3[2];
			PatrolPoints[2][3] = waypoints3[3];
			PatrolPoints[2][4] = waypoints3[4];
			PatrolPoints[2][5] = waypoints3[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[2][0], kHighPriority);
			break;
		}
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus, tShipInfo* Ship, tShipInfo* CapturingShip)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* HydranTeam = tScript.fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranFreighterTeam = tScript.fMission.mGetTeamHandle (kLyranFreighterTeam);

	// if all ships on a team are gone
	if ( EventTeam->mGetShipsInPlayCount() == 0 )
	{
		if ( EventTeam == HydranTeam )
		{
			tScript.fMission.mDisplayMessage( kHydranTeamDestroyed_msg, kMissionScriptMessage );
			switch (EventShipStatus)
			{
				case kEventShipDestroyed:
					break;

				case kEventShipCaptured:
					// LNF if the base is captured, the player should not get a win message
					break;

				case kEventShipDisengaged:
					break;
			}
		}
		else if ( EventTeam == LyranFreighterTeam )
		{
			switch (EventShipStatus)
			{
				case kEventShipDestroyed:
					if ( HydranBaseDestroyed == 0 )
					{
						// freighter gets destroyed before it delivers bomb
						tScript.fMission.mDisplayMessage( kLyranFreighterDestroyed_msg, kMissionScriptMessage );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					}
					else
					{
						// freighter delivers bomb
						tScript.fMission.mDisplayMessage( kEvent3_StationDestroyed_msg, kMissionScriptMessage );
						tScript.fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					}
					break;

				case kEventShipCaptured:
					// check if the player is capturing an allied ship
					if ( CapturingShip->mGetTeam() == tScript.kPlayerTeam )
					{
						tScript.fMission.mDisplayMessage( kCaptureAlliedShip_msg, kMissionScriptMessage );
					}
					else
					{
						tScript.fMission.mDisplayMessage( kLyranFreighterCaptured_msg, kMissionScriptMessage );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					}
					break;

				case kEventShipDisengaged:
					tScript.fMission.mDisplayMessage( kLyranFreighterDestroyed_msg, kMissionScriptMessage );
					tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					break;
			}
		}
	}
	else // there are some ships on a team remaining
	{
		if ( EventTeam == HydranTeam )
		{
			switch (EventShipStatus)
			{
				case kEventShipDestroyed:
					if (( Ship == EventTeam->mGetThisShip(5) ) && ( HydranBaseDestroyed == 0 ))
					{
						tScript.fMission.mDisplayMessage( kEvent3_StationDestroyed_msg, kMissionScriptMessage );
						tScript.fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
						HydranBaseDestroyed = 1;
					}
					break;

				case kEventShipCaptured:
					break;

				case kEventShipDisengaged:
					break;
			}
		}
		else if ( EventTeam == LyranFreighterTeam )
		{
			// shouldn't even get here because there is only one ship on the LyranFreighterTeam
			switch (EventShipStatus)
			{
				case kEventShipDestroyed:
					if ( HydranBaseDestroyed == 0 )
					{
						// freighter gets destroyed before it delivers bomb
						tScript.fMission.mDisplayMessage( kLyranFreighterDestroyed_msg, kMissionScriptMessage );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					}
					else
					{
						// freighter delivers bomb
						tScript.fMission.mDisplayMessage( kEvent3_StationDestroyed_msg, kMissionScriptMessage );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					}
					break;

				case kEventShipCaptured:
					// check if the player is capturing an allied ship
					if ( CapturingShip->mGetTeam() == tScript.kPlayerTeam )
					{
						tScript.fMission.mDisplayMessage( kCaptureAlliedShip_msg, kMissionScriptMessage );
					}
					else
					{
						tScript.fMission.mDisplayMessage( kLyranFreighterCaptured_msg, kMissionScriptMessage );
						tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					}
					break;

				case kEventShipDisengaged:
					tScript.fMission.mDisplayMessage( kLyranFreighterDestroyed_msg, kMissionScriptMessage );
					tScript.fMission.mSetGoalIndicator( kTrue, 37, 30 );
					break;
			}
		}
	}
}
