#include "scriptinterface.h"

#include "Kli_3MissionText.h"
#include "Kli_3MissionMaps.h"
/***********************************************
LNF things to do

scan planet
capture
***********************************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 PlayerDestroyed = 0;
int32 WeakDestroyed = 0;
int32 ColonyDestroyed = 0;
int32 DefenseDestroyed = 0;
int32 KlingonDestroyed = 0; // counter for all dead Klingon ships
int32 WeakCaptured = 0;
int32 KlingonCaptured = 0;
int32 PlanetScanned = 0;
int32 AtWarWithDeserter = 0;
int32 DeserterCaptured = 0;
int32 DefenseCaptured = 0;
const float kColonyDamagePercent = 50.0; // amount of damage to destroy colony
const float kPatrolAttackRadius = 1.0;
const float kDeepScanDistancePlanet = 30.0;
const float kDeepScanDistanceWeak = 20.0;
const int32 kPlanetX = 17;
const int32 kPlanetY = 12;
const int32 kWormholeX = 4;
const int32 kWormholeY = 3;
const float kWormholeWarningDistance = 30.0;
const float kWormholeDistance = 0.0;
int32 KlingonDeclaration = 0;
bool WarnAboutNoTarget = false;
bool AllKlingonsDestroyed = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

tShipInfo* ScuttleThisShip = NULL; // blow up this ship

void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kDeserterTeam, 
	kWeakTeam,
	kDefenseTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kHydranMission | kLyranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kDeserterTeam, kKlingon	);
	fMission.mSetTeamRace (kWeakTeam, kKlingon);
	fMission.mSetTeamRace (kDefenseTeam, kKlingon);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kDeserterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kWeakTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDeserterTeam, kWeakTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeserterTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDeserterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDefenseTeam, kWeakTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDefenseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kWeakTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DeserterTeam = fMission.mGetTeamHandle (kDeserterTeam);
	tTeamInfo* WeakTeam = fMission.mGetTeamHandle (kWeakTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by 80% to 120%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create Deserter team
	DeserterTeam->mCreateFleet (2, TotalBPV, kClassFrigate, kClassFrigate, 20, kStartPosition_X);

	// create Weak ship
	WeakTeam->mCreateShip (LowBPV, kClassFrigate, kClassFrigate, 20, kStartPosition_W);

	// create Defense team
	DefenseTeam->mCreateShip( "K-DEF", kStartPosition_U );
	DefenseTeam->mCreateShip( "K-DEF", kStartPosition_V );

	// create Planet
	PlanetTeam->mCreateShip( "PLDr1", kStartPosition_Specified, kPlanetX, kPlanetY );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DeserterTeam = fMission.mGetTeamHandle (kDeserterTeam);
	tTeamInfo* WeakTeam = fMission.mGetTeamHandle (kWeakTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;
	
	int32 ShipNumber = 0;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		// set distance from wormhole. This will determine if the player disengages
		Ship->mTrackTacticalDistance(kWormholeWarningDistance, kWormholeX, kWormholeY);
	}

	// configure Deserter Team ships
	for (Ship = DeserterTeam->mGetFirstShip(); Ship != NULL; Ship = DeserterTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		if (ShipNumber < 2)
		{
			PatrolRoute(Ship, ShipNumber);
		}
		ShipNumber++;
	}

	// configure Weak ship
	Ship = WeakTeam->mGetThisShip(1);
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mDamageSystem(kDamageMaxSystems, 50);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyMarines, 0);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplySpareParts, 0);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	if (ShipNumber == 2)
	{
		PatrolRoute(Ship, ShipNumber);
	}
	ShipNumber++;

	// configure Defense team
	for (Ship = DefenseTeam->mGetFirstShip(); Ship != NULL; Ship = DefenseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Planet
	PlanetTeam->mGetFirstShip()->mChangeName( kPlanet_name );
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// LNF win timer
	fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan planet from within distance 30 for 2 turns
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, 2);
	// deep scan weak ship within 20.0 for 1 turn
	DeepScanTracker.mSetTarget (WeakTeam->mGetThisShip(1), kDeepScanDistanceWeak, 1);

	// location deep scan tracker example
	//DeepScanTracker.mSetLocation (20, 8, DeepScanDistance, 3); // track Moon
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DeserterTeam = fMission.mGetTeamHandle (kDeserterTeam);
	tTeamInfo* WeakTeam = fMission.mGetTeamHandle (kWeakTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if (( Ship->mGetTeam() == kPlanetTeam ) &&
						( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						if (( PlanetScanned == 1 ) && ( Ship->mGetDamagePercent() > kColonyDamagePercent ) &&
							( ColonyDestroyed == 0 ))
						{
							fMission.mDisplayMessage(kEvent3_msg, kScienceScriptMessage);
							ColonyDestroyed = 1;
						}
						else if ( PlanetScanned == 0 )
						{
							// the player shouldn't be able to damage the planet.
							Ship->mDamageSystem( kDamageMaxShields, 0 );
							if ( WarnAboutNoTarget == false )
							{
								fMission.mDisplayMessage( kEvent3_BombardPlanetNoScan_msg, kMissionScriptMessage );
								WarnAboutNoTarget = true;
								// timer to reset warning
								fMission.mSetTimedInterval( kTimerTurnsSet, 2 );
							}
						}
					}

					if ((( Ship->mGetTeam() == kDeserterTeam ) || ( Ship->mGetTeam() == kWeakTeam ) ||
						( Ship->mGetTeam() == kDefenseTeam )) && ( AtWarWithDeserter == 0 ))
					{
						fMission.mSetTeamRelations (kDeserterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kWeakTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kDefenseTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AtWarWithDeserter = 1;
					}
				}
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyAtDistance:
			if (fPlayerShip->mGetTrackingDistance() == kWormholeWarningDistance)
			{
				// warn player approaching wormhole
				fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
				fPlayerShip->mTrackTacticalDistance(kWormholeDistance, kWormholeX, kWormholeY);
			}
			else if (fPlayerShip->mGetTrackingDistance() == kWormholeDistance)
			{
				fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
				return kPlayerDisengaged;
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kDeserterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kWeakTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDeserterTeam, kWeakTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDeserterTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDeserterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDefenseTeam, kWeakTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDefenseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kWeakTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					// start message
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					// win conditions
					// make the KlingonDeclaration a "secondary" objective, or at least not important
					if (( ColonyDestroyed == 1 ) && ( AllKlingonsDestroyed == true ))
					{
						// put go to wormhole message and goal
						fMission.mDisplayMessage( kMissionAccomplished_msg, kMissionScriptMessage );
						fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY );
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );
					}
					break;

				case kTimerLimitSet_3:
					// self destruct the traitor ships

					ScuttleThisShip->mDamageSystem( kDamageExcessDamageSystem, 100 );
					ScuttleThisShip->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								if ( Object->mGetTeam() == kPlanetTeam )
								{
									fMission.mDisplayMessage (kScannerOffInRangePlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kScannerOffInRangeWeak_msg, kScienceScriptMessage);
								}
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								if ( Object->mGetTeam() == kPlanetTeam )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStartingScanWeak_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								if ( Object->mGetTeam() == kPlanetTeam )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStartingScanWeak_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							if ( Object->mGetTeam() == kWeakTeam )
							{
								if ( WeakCaptured == 0 )
								{
									// player has successfully scanned the object
									// change the name of the weak ship
									Object->mChangeName( kWeak_name );
									fMission.mDisplayMessage(kEvent2_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage( kEvent2_NothingNew_msg, kScienceScriptMessage );
								}
							}
							else
							{
								fMission.mDisplayMessage(kEvent1A_msg, kScienceScriptMessage);
								fMission.mDisplayMessage(kEvent1B_msg, kScienceScriptMessage);
								fMission.mDisplayMessage(kEvent1C_msg, kScienceScriptMessage);
								fMission.mDisplayMessage(kEvent1D_msg, kScienceScriptMessage);
								PlanetScanned = 1;
							}
						}
						// (Object points to entity or if NULL then MapPoint has location)
						if (Object)
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
						else
						{
							DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
						}
					}
					break;

				case kTimerTurnsSet:
					// timer to reset No Target Warning
					WarnAboutNoTarget = false;
					break;

				case kTimerTurnsSet_10:
					// set the Deserter comm buttons back to HAIL
					for (Ship = DeserterTeam->mGetFirstShip(); Ship != NULL; Ship = DeserterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

//************************
// LNF capture thing bleah
		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			switch (Ship->mGetTeam())
			{
				// LNF should I make it impossible to capture?  Blow everything up?
				case kDeserterTeam:
					fMission.mDisplayMessage (kVesselBoardedScuttle_msg, kMissionScriptMessage);
					KlingonCaptured++;

					// blow up ship on timer
					ScuttleThisShip = Ship;
					fMission.mSetTimedInterval( kTimerLimitSet_3, 5 );
					break;

				case kWeakTeam:
					DeepScanTracker.mRemoveTarget( Ship );
					fMission.mDisplayMessage (kEvent5_VesselBoardedWeak_msg, kMissionScriptMessage);
					WeakCaptured = 1;
					KlingonCaptured++;

					// blow up ship on timer
					ScuttleThisShip = Ship;
					fMission.mSetTimedInterval( kTimerLimitSet_3, 5 );
					break;

				case kDefenseTeam:
					// shouldn't be able to capture this, blow it up
					fMission.mDisplayMessage (kVesselBoardedBlowUp_msg, kMissionScriptMessage);
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
					Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
					DefenseCaptured++;
					break;
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				fMission.mDisplayMessage(kPlayerMovesOffMap_msg, kMissionScriptMessage);
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kDeserterTeam)
			{
				KlingonDestroyed++;
				if ( KlingonDestroyed == 1 )
				{
					fMission.mDisplayMessage( kEvent3_DestroyOthers_msg, kMissionScriptMessage );
				}

				if ((DeserterTeam->mGetShipsInPlayCount() == 0) &&
					(WeakTeam->mGetShipsInPlayCount() == 0) &&
					(DefenseTeam->mGetShipsInPlayCount() == 0))
				{
					AllKlingonsDestroyed = true;
				}

				if (( AllKlingonsDestroyed == true ) && ( PlanetScanned == 0 ))
				{
					fMission.mDisplayMessage( kEvent8_AdviseScanPlanet_msg, kMissionScriptMessage );
				}
				else if ( AllKlingonsDestroyed == true )
				{
					fMission.mDisplayMessage( kEvent8_AllKlingonsDestroyed_msg, kMissionScriptMessage );
				}
			}
			else if (Ship->mGetOriginalTeam() == kWeakTeam)
			{
				DeepScanTracker.mRemoveTarget( Ship );
				if (WeakCaptured == 0)
				{
					KlingonDestroyed++;
				}

				if ( KlingonDestroyed == 1 )
				{
					fMission.mDisplayMessage( kEvent3_DestroyOthers_msg, kMissionScriptMessage );
				}

				if ((DeserterTeam->mGetShipsInPlayCount() == 0) &&
					(WeakTeam->mGetShipsInPlayCount() == 0) &&
					(DefenseTeam->mGetShipsInPlayCount() == 0))
				{
					AllKlingonsDestroyed = true;
				}

				if (( AllKlingonsDestroyed == true ) && ( PlanetScanned == 0 ))
				{
					fMission.mDisplayMessage( kEvent8_AdviseScanPlanet_msg, kMissionScriptMessage );
				}
				else if ( AllKlingonsDestroyed == true )
				{
					fMission.mDisplayMessage( kEvent8_AllKlingonsDestroyed_msg, kMissionScriptMessage );
				}
			}
			else if (Ship->mGetOriginalTeam() == kDefenseTeam)
			{
				KlingonDestroyed++;
				DefenseDestroyed++;

				if ( KlingonDestroyed == 1 )
				{
					fMission.mDisplayMessage( kEvent3_DestroyOthers_msg, kMissionScriptMessage );
				}

				if ((DeserterTeam->mGetShipsInPlayCount() == 0) &&
					(WeakTeam->mGetShipsInPlayCount() == 0) &&
					(DefenseTeam->mGetShipsInPlayCount() == 0))
				{
					AllKlingonsDestroyed = true;
				}

				if (( AllKlingonsDestroyed == true ) && ( PlanetScanned == 0 ))
				{
					fMission.mDisplayMessage( kEvent8_AdviseScanPlanet_msg, kMissionScriptMessage );
				}
				else if ( AllKlingonsDestroyed == true )
				{
					fMission.mDisplayMessage( kEvent8_AllKlingonsDestroyed_msg, kMissionScriptMessage );
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						if ( KlingonDeclaration == 0 )
						{
							fMission.mDisplayMessage( kKliHailDeclaration_1_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kKliHailDeclaration_2_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kKliHailDeclaration_3_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kKliHailDeclaration_4_msg, kMissionScriptMessage );
							KlingonDeclaration = 1;
						}
						else if (AtWarWithDeserter == 0)
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kKliHailNic1_msg, kKliHailNic3_msg), kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kKliHail1_msg, kMissionScriptMessage);
							fMission.mDisplayMessage (kKliHail2_msg, kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonSurrender_text:
						fMission.mDisplayMessage(kKliSurrender1_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kKliSurrender2_msg, kMissionScriptMessage);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						if (AtWarWithDeserter == 0)
						{
							fMission.mDisplayMessage(kKliTauntNic_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kKliTaunt1_msg, kKliTaunt4_msg), kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF verify accuracy/balance
	VictoryPoints = (ColonyDestroyed * 20) + (KlingonDestroyed * 20) + (KlingonDeclaration * 20) +
					(DefenseCaptured * 10) + (DefenseDestroyed * 5);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (VictoryLevel > kVictory)
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( ColonyDestroyed != 0 ) ? fMission.mAddToDebriefingList( kColonyDestroyedTrue_msg ) : SuccessCount++;
	( AllKlingonsDestroyed == true ) ? fMission.mAddToDebriefingList( kAllKlingonsDestroyedTrue_msg ) : SuccessCount++;
	( KlingonDeclaration != 0 ) ? fMission.mAddToDebriefingList( kKlingonDeclarationTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( ColonyDestroyed == 0 ) ? fMission.mAddToDebriefingList( kColonyDestroyedFalse_msg ) : FailureCount++;
	( AllKlingonsDestroyed == false ) ? fMission.mAddToDebriefingList( kAllKlingonsDestroyedFalse_msg ) : FailureCount++;
	( KlingonDeclaration == 0 ) ? fMission.mAddToDebriefingList( kKlingonDeclarationFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
		
	tMapPoint waypoints1[4]= // ship at X
	{
		14, 11,
		12, 13,
		13, 16,
		16, 16
	};

	tMapPoint waypoints2[6]= // ship at Y
	{
		20, 8,
		18, 7,
		17, 9,
		21, 11,
		24, 10,
		23, 7
	};

	tMapPoint waypoints3[9]= // ship at W
	{
		18, 16,
		16, 18,
		13, 18,
		12, 16,
		13, 13,
		15, 7,
		20, 6,
		24, 11,
		22, 16
	};


	tMapPoint PatrolPoints[3][10];

	switch (ShipNumber)
	{
		case 0:
			PatrolPoints[0][0] = waypoints1[0];
			PatrolPoints[0][1] = waypoints1[1];
			PatrolPoints[0][2] = waypoints1[2];
			PatrolPoints[0][3] = waypoints1[3];
			Ship->mPatrol (kPatrolAttackRadius, 4, &PatrolPoints[0][0], kHighPriority);
			break;

		case 1:
			PatrolPoints[1][0] = waypoints2[0];
			PatrolPoints[1][1] = waypoints2[1];
			PatrolPoints[1][2] = waypoints2[2];
			PatrolPoints[1][3] = waypoints2[3];
			PatrolPoints[1][4] = waypoints2[4];
			PatrolPoints[1][5] = waypoints2[5];
			Ship->mPatrol (kPatrolAttackRadius, 6, &PatrolPoints[1][0], kHighPriority);
			break;

		case 2:
			PatrolPoints[2][0] = waypoints3[0];
			PatrolPoints[2][1] = waypoints3[1];
			PatrolPoints[2][2] = waypoints3[2];
			PatrolPoints[2][3] = waypoints3[3];
			PatrolPoints[2][4] = waypoints3[4];
			PatrolPoints[2][5] = waypoints3[5];
			PatrolPoints[2][6] = waypoints3[6];
			PatrolPoints[2][7] = waypoints3[7];
			PatrolPoints[2][8] = waypoints3[8];
			Ship->mPatrol (kPatrolAttackRadius, 9, &PatrolPoints[2][0], kHighPriority);
			break;
		}
}
