#include "scriptinterface.h"

#include "Kli_4MissionText.h"
#include "Kli_4MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 FreighterDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 AggressorDestroyed = 0;
int32 FreighterScanned = 0;
int32 AtWarWithAggressor = 0;
int32 AtWarWithCrazy = 0;
int32 CrazyAnnounce = 0;
int32 CrazyDestroyed = 0;
int32 CrazyCaptured = 0;
int32 TotalBPV = 0;
int32 CrazyBlownUp = 0; // the traitor dude is acutally BLOWN UP. NOT captured

const int32 kKlingonGoesCrazyTurns = 4; // how many turns from beginning the Klingon goes crazy
const float kPatrolAttackRadius = 1.0;
const int32 kWormholeX = 2;
const int32 kWormholeY = 27;
const float kWormholeWarningDistance = 30.0;
const float kWormholeDistance = 0.0;
const float kDeepScanDistanceFreighter = 30.0;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool PrematureAttack = false;

void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kAggressorTeam, // Enemy of Klingon - Fed/Hydran/Lyran
	kFreighterTeam, // anti-cloaking device
	kCrazyBoyTeam, // KrazyKlingon wingman
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kHydranMission | kLyranMission | kFedMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 50) ? kHydran
					  : (fMission.mRandomInt32 (1, 100) < 50) ? kLyran : kFederation;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kAggressorTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kFreighterTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kCrazyBoyTeam, kKlingon);

	// Set team relations to show up on Tactical map
	// LNF this might change from dynaverse...
	fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kCrazyBoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kAggressorTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAggressorTeam, kCrazyBoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFreighterTeam, kCrazyBoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // use the player's ship(s)
	//return kFalse; // use a script given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* CrazyBoyTeam = fMission.mGetTeamHandle (kCrazyBoyTeam);

	// Get the total BPV of the player and modify by 80% to 120%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	tShipInfo* Ship = NULL;

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create Aggressor team
	// LNF temp fed DN make this something else.
/*	if ( AggressorTeam->mGetRace() == kFederation )
	{
		AggressorTeam->mCreateShip ("F-DN", kStartPosition_U);
	}
	else if ( AggressorTeam->mGetRace() == kLyran )
	{
		AggressorTeam->mCreateShip ("F-DN", kStartPosition_U);
	}
	else if ( AggressorTeam->mGetRace() == kHydran )
	{
		AggressorTeam->mCreateShip ("F-DN", kStartPosition_U);
	}
*/	//AggressorTeam->mCreateShip ("F-DN", kStartPosition_U);
	AggressorTeam->mCreateFleet (NumberOfPlayersShips, TotalBPV, kClassFrigate, LargestEnemyShipClass, 20, kStartPosition_V);

	// create Freighter team
	FreighterTeam->mCreateShip (kMaxBPV, kClassFreighter, kClassFreighter, 20, kStartPosition_Z);

	// create Crazy Klingon ship
	// LNF specific ship?
	CrazyBoyTeam->mCreateShip (LowBPV, kClassDestroyer, kClassDestroyer, 0, kStartPosition_J);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* CrazyBoyTeam = fMission.mGetTeamHandle (kCrazyBoyTeam);

	tShipInfo* Ship;
	int32 ShipNumber = 0;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mTrackTacticalDistance(kWormholeWarningDistance, kWormholeX, kWormholeY); // track to wormhole
		//Ship->mTrackTacticalDistance(50.0, 20, 20); // track to Freighter
		Ship->mSetShipThrust (5);
	}

	// configure Aggressor team
	for (Ship = AggressorTeam->mGetFirstShip(); Ship != NULL; Ship = AggressorTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		if (ShipNumber < 3)
		{
			PatrolRoute(Ship, ShipNumber);
		}
		ShipNumber++;
	}
	AggressorTeam->mGetFirstShip()->mDefendEntity( FreighterTeam->mGetFirstShip(), 20.0, 20.0, kAbsolutePriority );

	// configure Freighter
	Ship = FreighterTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
	Ship->mDamageSystem(kDamageRightWarpEngine, 100);

	// configure Crazy Klingon
	Ship = CrazyBoyTeam->mGetFirstShip();
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mSetCommButton (1, kButtonHail_text);

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// timer when Klingon 'buddy' goes crazy
	fMission.mSetTimedInterval (kTimerTurnsSet, kKlingonGoesCrazyTurns);

	// win timer
	fMission.mSetTimedInterval( kTimerLimitSet_4, 4 );

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan freighter from within distance 30 for 2 turns
	DeepScanTracker.mSetTarget (FreighterTeam->mGetThisShip(1), kDeepScanDistanceFreighter, 2);
	// location deep scan tracker example
	//DeepScanTracker.mSetLocation (20, 8, DeepScanDistance, 3); // track Moon
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* AggressorTeam = fMission.mGetTeamHandle (kAggressorTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* CrazyBoyTeam = fMission.mGetTeamHandle (kCrazyBoyTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if (( Ship == CrazyBoyTeam->mGetFirstShip() ) && ( HostileShip->mGetTeam() == kPlayerTeam ) &&
						( PrematureAttack == false ))
					{
						PrematureAttack = true;
						AtWarWithCrazy = 1;
					}
					else if ((AtWarWithAggressor == 0) &&
						(( Ship->mGetTeam() == kAggressorTeam ) || ( Ship->mGetTeam() == kFreighterTeam )) &&
						( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						// if an Aggressor ship is shot at, set AtWar with player
						fMission.mSetTeamRelations (kAggressorTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AtWarWithAggressor = 1;
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		case kNotifyAtDistance:
			if (fPlayerShip->mGetTrackingDistance() == kWormholeWarningDistance)
			{
				// warn player approaching wormhole
				fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
				fPlayerShip->mTrackTacticalDistance(kWormholeDistance, kWormholeX, kWormholeY);
			}
			else if (fPlayerShip->mGetTrackingDistance() == kWormholeDistance)
			{
				fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
				return kPlayerDisengaged;
			}
			/*
			else if ((fPlayerShip->mGetTrackingDistance() == 50.0) &&
					(CrazyAnnounce == 0))
			{
				// LNF Klingon wingman goes crazy
				fMission.mDisplayMessage(kEvent2A_msg, kMissionScriptMessage);
				fMission.mDisplayMessage(kEvent2B_msg, kMissionScriptMessage);
				fMission.mDisplayMessage(kEvent2C_msg, kMissionScriptMessage);
				fMission.mDisplayMessage(kEvent2D_msg, kMissionScriptMessage);
				CrazyAnnounce = 1;
				fMission.mSetTimedInterval (kTimerLimitSet_2, 14);
			}*/
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kAggressorTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kCrazyBoyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kAggressorTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kAggressorTeam, kCrazyBoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kFreighterTeam, kCrazyBoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					// start scenario text
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					fMission.mSetTeamRelations (kCrazyBoyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
					fMission.mSetTimedInterval (kTimerLimitSet_3, 5);
					break;
				
				case kTimerLimitSet_3:
					fMission.mSetTeamRelations (kPlayerTeam, kCrazyBoyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kAggressorTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					AggressorTeam->mGetFirstShip()->mClearCommands();
					AggressorTeam->mGetFirstShip()->mDefendEntity( FreighterTeam->mGetFirstShip(), 20.0, 20.0, kAbsolutePriority );
					break;

				case kTimerLimitSet_4:
					// win timer
					// LNF freighter destroyed - primary

					// LNF freighter scanned/captured
					// LNF crazy klingon captured
					// LNF crazy klingon destroyed

					if ( FreighterDestroyed == 1 )
					{
						fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
						fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY );
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_4, 4 );
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							fMission.mDisplayMessage(kEvent7_ScanFreighter_msg, kScienceScriptMessage);
							FreighterScanned = 1;
						}
						// (Object points to entity or if NULL then MapPoint has location)
						if (Object)
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
						else
						{
							DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
						}
					}
					break;

				case kTimerTurnsSet:
					if (CrazyAnnounce == 0)
					{
						// LNF Klingon wingman goes crazy
						fMission.mDisplayMessage(kEvent2A_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kEvent2B_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kEvent2C_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kEvent2D_msg, kMissionScriptMessage);
						CrazyAnnounce = 1;
						fMission.mSetTimedInterval (kTimerLimitSet_2, 14);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Crazy Boy Team buttons back to HAIL
					for (Ship = CrazyBoyTeam->mGetFirstShip(); Ship != NULL; Ship = CrazyBoyTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Aggressor comm buttons back to HAIL
					for (Ship = AggressorTeam->mGetFirstShip(); Ship != NULL; Ship = AggressorTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
				if (FreighterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kAggressorTeam)
			{
				AggressorDestroyed++;
			}
			else if (Ship->mGetTeam() == kCrazyBoyTeam)
			{
				CrazyDestroyed = 1;
				CrazyBlownUp = 1;
			}

			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship == FreighterTeam->mGetFirstShip() )
			{
				// blow up freighter
				fMission.mDisplayMessage( kEvent5_BoardingFreighter_msg, kMissionScriptMessage );
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			if (Ship->mGetOriginalTeam() == kCrazyBoyTeam)
			{
				fMission.mDisplayMessage(kVesselCaptured_msg, kMissionScriptMessage);
				CrazyCaptured = 1;
				CrazyDestroyed = 1;
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kAggressorTeam:
								switch (AggressorTeam->mGetRace())
								{
									case kFederation:
										if (AtWarWithAggressor == 0)
										{
											fMission.mDisplayMessage (kFedHailNeNic1_msg, kMissionScriptMessage);
											fMission.mDisplayMessage (kFedHailNeNic2_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (kFedHail_msg, kMissionScriptMessage);
										}
										break;

									case kLyran:
										if (AtWarWithAggressor == 0)
										{
											fMission.mDisplayMessage (kLyrHailNeNic_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (kLyrHail_msg, kMissionScriptMessage);
										}
										break;

									case kHydran:
										if (AtWarWithAggressor == 0)
										{
											fMission.mDisplayMessage (kHydHailNeNic_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (kHydHail_msg, kMissionScriptMessage);
										}
										break;
									if (AtWarWithAggressor)
									{
										Ship->mSetCommButton (1, kButtonTaunt_text);
										Ship->mSetCommButton (2, kButtonSurrender_text);
										// If the Taunt or Surrender buttons aren't used in X turns
										// return the buttons to Hail and blank.
										fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
									}
									else
									{
										Ship->mSetCommButton (1, kButtonHail_text);
									}
								}
								break;

							case kCrazyBoyTeam:
								if (CrazyAnnounce == 0)
								{
									fMission.mDisplayMessage (kKliHailOK_msg, kMissionScriptMessage);
								}
								else if (AtWarWithCrazy == 0)
								{
									fMission.mDisplayMessage (kKliHailCrazyNic1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kKliHailCrazyNic2_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kKliHail_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kAggressorTeam:
								switch (AggressorTeam->mGetRace())
								{
									case kFederation:
										if (Ship->mGetDamagePercent() < 50.0)
										{
											fMission.mDisplayMessage (kFedSurrenderOK_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (kFedSurrenderDamaged_msg, kMissionScriptMessage);
										}
										break;

									case kLyran:
										if (Ship->mGetDamagePercent() < 50.0)
										{
											fMission.mDisplayMessage (kLyrSurrenderOK_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (kLyrSurrenderDamaged_msg, kMissionScriptMessage);
										}
										break;

									case kHydran:
										if (Ship->mGetDamagePercent() < 50.0)
										{
											fMission.mDisplayMessage (kHydSurrenderOK_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (kHydSurrenderDamaged_msg, kMissionScriptMessage);
										}
										break;
									Ship->mSetCommButton (2, kButtonSurrender_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								}
								break;

							case kCrazyBoyTeam:
								if (CrazyAnnounce == 0)
								{
									fMission.mDisplayMessage(kKliSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kKliSurrenderCrazy_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kAggressorTeam:
								switch (AggressorTeam->mGetRace())
								{
									case kFederation:
										fMission.mDisplayMessage (fMission.mRandomInt32 (kFedTaunt1_msg, kFedTaunt4_msg), kMissionScriptMessage);
										break;

									case kLyran:
										if (AtWarWithAggressor == 0)
										{
											fMission.mDisplayMessage (kLyrTauntNic_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (fMission.mRandomInt32 (kLyrTaunt1_msg, kLyrTaunt4_msg), kMissionScriptMessage);
										}
										break;

									case kHydran:
										if (AtWarWithAggressor == 0)
										{
											fMission.mDisplayMessage (kHydTauntNic_msg, kMissionScriptMessage);
										}
										else 
										{
											fMission.mDisplayMessage (fMission.mRandomInt32 (kHydTaunt1_msg, kHydTaunt4_msg), kMissionScriptMessage);
										}
										break;
									Ship->mSetCommButton (1, kButtonTaunt_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								}
								break;

							case kCrazyBoyTeam:
								if (CrazyAnnounce == 0)
								{
									fMission.mDisplayMessage(kKliSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kKliTaunt1_msg, kKliTaunt4_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF confirm for accuracy/balance
	VictoryPoints = (FreighterDestroyed * 70) + (AggressorDestroyed * 3) + 
					(CrazyCaptured * 20) + (CrazyBlownUp * 10);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 69)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (VictoryLevel > kVictory)
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
		else
		{
			if ( CrazyCaptured == 0 )
			{
				fMission.mAddToDebriefingList (kEscortCaptured_msg);
			}
			fMission.mAddToDebriefingList (kVictory2_msg);
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( FreighterDestroyed != 0 ) ? fMission.mAddToDebriefingList( kFreighterDestroyedTrue_msg ) : SuccessCount++;
	( AggressorDestroyed != 0 ) ? fMission.mAddToDebriefingList( kAggressorDestroyedTrue_msg ) : SuccessCount++;
	( CrazyDestroyed != 0 ) ? fMission.mAddToDebriefingList( kCrazyTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( FreighterDestroyed == 0 ) ? fMission.mAddToDebriefingList( kFreighterDestroyedFalse_msg ) : FailureCount++;
	( AggressorDestroyed == 0 ) ? fMission.mAddToDebriefingList( kAggressorDestroyedFalse_msg ) : FailureCount++;
	( CrazyDestroyed == 0 ) ? fMission.mAddToDebriefingList( kCrazyFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
		
	tMapPoint waypoints1[6]= // ship at U
	{
		30, 30,
		32, 26,
		30, 23,
		25, 24,
		23, 28,
		26, 30
	};

	tMapPoint waypoints2[6]= // ship at V
	{
		24, 25,
		19, 24,
		17, 21,
		18, 17,
		22, 16,
		24, 20
	};

	tMapPoint waypoints3[6]= // ship at W
	{
		16, 13,
		21, 14,
		24, 18,
		23, 22,
		19, 23,
		16, 20
	};

	tMapPoint PatrolPoints[3][10];

	switch (ShipNumber)
	{
		case 0:
			PatrolPoints[0][0] = waypoints1[0];
			PatrolPoints[0][1] = waypoints1[1];
			PatrolPoints[0][2] = waypoints1[2];
			PatrolPoints[0][3] = waypoints1[3];
			PatrolPoints[0][4] = waypoints1[4];
			PatrolPoints[0][5] = waypoints1[5];
			Ship->mPatrol (kPatrolAttackRadius, 6, &PatrolPoints[0][0], kHighPriority);
			break;

		case 1:
			PatrolPoints[1][0] = waypoints2[0];
			PatrolPoints[1][1] = waypoints2[1];
			PatrolPoints[1][2] = waypoints2[2];
			PatrolPoints[1][3] = waypoints2[3];
			PatrolPoints[1][4] = waypoints2[4];
			PatrolPoints[1][5] = waypoints2[5];
			Ship->mPatrol (kPatrolAttackRadius, 6, &PatrolPoints[1][0], kHighPriority);
			break;

		case 2:
			PatrolPoints[2][0] = waypoints2[0];
			PatrolPoints[2][1] = waypoints2[1];
			PatrolPoints[2][2] = waypoints2[2];
			PatrolPoints[2][3] = waypoints2[3];
			PatrolPoints[2][4] = waypoints2[4];
			PatrolPoints[2][5] = waypoints2[5];
			Ship->mPatrol (kPatrolAttackRadius, 6, &PatrolPoints[2][0], kHighPriority);
			break;
		}
}