#include "scriptinterface.h"

#include "Kli_5MissionText.h"
#include "Kli_5MissionMaps.h"
#include "sfcAssert.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 FedDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 TribbleCruiserDestroyed = 0;
int32 TribbleCruiserCaptured = 0;
int32 StationDestroyed = 0;
int32 AtWarWithFed = 0;
int32 TribbleHomeDestroyed = 0;
int32 StationCaptured = 0;
int32 FedCaptured = 0;
int32 TotalBPV = 0;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

tShipInfo* MostRecentlyDestroyed = NULL;

const float kDamageToKillTribbleHome = 80.0; // percent of damage to planet for it to be considered dead
const float kPatrolAttackRadius = 1.0;
const int32 kWormholeX = 4;
const int32 kWormholeY = 22;
const float kWormholeWarningDistance = 30.0;
const float kWormholeDistance = 0.0;
const float kDeepScanDistance = 30.0; // distance to do deep scan

void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

/*************************************************

LNF add this stuff

make one of the Fed ships defend the wormhole
check if fed team does hit and run damage and if so, make some tribbles
appear in the player's transporter
*************************************************/

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFedTeam, 
	kTribbleCruiserTeam,
	kStationTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kHydranMission | kLyranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kFederation;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFedTeam, kFederation);
	fMission.mSetTeamRace (kTribbleCruiserTeam, kFederation);
	fMission.mSetTeamRace (kStationTeam, kFederation);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kTribbleCruiserTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kStationTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedTeam, kTribbleCruiserTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kStationTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kStationTeam, kTribbleCruiserTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kStationTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kTribbleCruiserTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // player uses own ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* TribbleCruiserTeam = fMission.mGetTeamHandle (kTribbleCruiserTeam);
	tTeamInfo* StationTeam = fMission.mGetTeamHandle (kStationTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by 80% to 120%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .85;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.15;
	TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	FedTeam->mCreateFleet (4, TotalBPV, kClassFrigate, kClassDestroyer, 20, kStartPosition_U);

	TribbleCruiserTeam->mCreateShip (LowBPV, kClassLightCruiser, kClassLightCruiser, 20, kStartPosition_Y);

	StationTeam->mCreateShip (kMaxBPV, kClassBaseStation, kClassBaseStation, 20, kStartPosition_Z);

	PlanetTeam->mCreateShip ("PL1", kStartPosition_Specified, 25, 15);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* TribbleCruiserTeam = fMission.mGetTeamHandle (kTribbleCruiserTeam);
	tTeamInfo* StationTeam = fMission.mGetTeamHandle (kStationTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;
	int32 ShipNumber = 0;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		Ship->mTrackTacticalDistance(kWormholeWarningDistance, kWormholeX, kWormholeY); // distance to wormhole
	}

	// configure Federation ships
	for (Ship = FedTeam->mGetFirstShip(); Ship != NULL; Ship = FedTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		PatrolRoute(Ship, ShipNumber);
		ShipNumber++;
	}

	// configure Tribble activists cruiser
	Ship = TribbleCruiserTeam->mGetFirstShip();
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mChangeName(kSpct_name);
	// LNF change kTransTribbles to Tribble
	Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );

	// configure Cyrano Jones' space station
	// LNF disable weapons...maybe specify a station and leave some weak stuff?
	Ship = StationTeam->mGetFirstShip();
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	// LNF do the ships shields don't repair
	Ship->mDamageSystem(kDamageMaxShields, 100);
	Ship->mChangeName(kCyranoJones_name);
	Ship->mDamageSystem(kDamageMaxWeapons, 100);
	// LNF change kTransTribbles to Tribble
	Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );

	// configure planet
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
	// LNF change kTransTribbles to Tribble
	PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// win timer
	fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan XX from within distance 30 for Y turns
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistance, 4);
	DeepScanTracker.mSetTarget (StationTeam->mGetThisShip(1), kDeepScanDistance, 3);
	DeepScanTracker.mSetTarget (TribbleCruiserTeam->mGetThisShip(1), kDeepScanDistance, 2);
	// location deep scan tracker example
	//DeepScanTracker.mSetLocation (20, 8, DeepScanDistance, 3); // track Moon
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* TribbleCruiserTeam = fMission.mGetTeamHandle (kTribbleCruiserTeam);
	tTeamInfo* StationTeam = fMission.mGetTeamHandle (kStationTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);
	tShipInfo* Station;

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if (( HostileShip->mGetTeam() == kPlayerTeam ) && ( AtWarWithFed == 0 ))
					{
						// if a Federation ship is shot at, set AtWar with player
						fMission.mSetTeamRelations (kFedTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kTribbleCruiserTeam, kPlayerTeam, kTeamAtWar, kGoalCapture, 1.0);
						fMission.mSetTeamRelations (kStationTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						Station = StationTeam->mGetFirstShip();
						Station->mDamageSystem( kDamageMaxShields, 0 );

						AtWarWithFed = 1;
					}
					else if (( Ship->mGetTeam() == kPlanetTeam ) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						if (( Ship->mGetDamagePercent() > kDamageToKillTribbleHome  ) && ( TribbleHomeDestroyed == 0 ))
						{
							TribbleHomeDestroyed = 1;
							fMission.mDisplayMessage( kEvent2_Tribblecide_msg, kMissionScriptMessage );
							fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY );
						}
					}
					Ship->mSetShipCallbackEvents( kNotifyDamage );

					if (( Ship->mGetTeam() == kPlayerTeam ) && ( HostileShip == TribbleCruiserTeam->mGetFirstShip() ))
					{
						switch (fEventItem1)
						{
							case kDamageWeaponByMarines:
								{
									// LNF if the player's ship was hit by hit and run by tribble cruiser
									// LNF change kTransTribbles to Tribble
									Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );
									// LNF what kind of effect can happen here?
									// LNF remove repair goo?
									// LNF maybe it should do this once per turn?
									int32 GooOnShip = Ship->mAccessSupplies( kGetSupplyCount, kSupplySpareParts );
									if ( GooOnShip > 0 )
									{
										Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, (GooOnShip - 1) );
									}

									// LNF increase the change course delay time?

									if ( fEventItem3 == 0 )
									{
										//fMission.mDisplayMessage( kLNFHitNRunWeap_msg, kNonCommScriptMessage );
									}

									// fEventItem2 == which weapon was hit: kDamageHeavyWeapon1 - kDamageHeavyWeapon25
									// fEventItem3 == weapons current health; 0 == weapon destroyed
								}
								break;


							case kDamageSystemByMarines:
								{
									// LNF if the player's ship was hit by hit and run by tribble cruiser
									// LNF change kTransTribbles to Tribble
									Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );
									// LNF what kind of effect can happen here?
									// LNF remove repair goo?
									// LNF maybe it should do this once per turn?
									int32 GooOnShip = Ship->mAccessSupplies( kGetSupplyCount, kSupplySpareParts );
									if ( GooOnShip > 0 )
									{
										Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, (GooOnShip - 1) );
									}

									//fMission.mDisplayMessage( kLNFHitNRunSys_msg, kNonCommScriptMessage );
									// fEventItem2 == which system was hit: kDamageSensorSystem, 
									//										kDamageTransporterSystem,
									//										kDamageTractorSystem
									//										kDamageShuttleBaySystem
									// fEventItem3 == systems current health; 0 == system destroyed
								}
								break;

							case kDamageByWeaponOrDebris:
								// nothing extra to test
								break;
						}
					}
				}
			}
			break;

		// This is how the player finishes mission
		case kNotifyAtDistance: 
			if (fPlayerShip->mGetTrackingDistance() == kWormholeWarningDistance)
			{
				// warn player approaching wormhole
				fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
				fPlayerShip->mTrackTacticalDistance(kWormholeDistance, kWormholeX, kWormholeY);
			}
			else if (fPlayerShip->mGetTrackingDistance() == kWormholeDistance)
			{
				fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
				return kPlayerDisengaged;
			}
			break;
	

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kTribbleCruiserTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kStationTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kFedTeam, kTribbleCruiserTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kStationTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kStationTeam, kTribbleCruiserTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kStationTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					
					fMission.mSetTeamRelations (kTribbleCruiserTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					// start scenario text
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					// win condition
					if ((TribbleHomeDestroyed == 1) && (TribbleCruiserDestroyed == 1) &&
						(StationDestroyed == 1))
					{
						fMission.mDisplayMessage( kMissionAccomplished_msg, kMissionScriptMessage );
						fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY );
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );
					}
					break;

					case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								if ( Object == PlanetTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kScannerOffInRangePlanet_msg, kScienceScriptMessage);
								}
								else if ( Object == StationTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kScannerOffInRangeStation_msg, kScienceScriptMessage);
								}
								else if ( Object == TribbleCruiserTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kScannerOffInRangeShip_msg, kScienceScriptMessage);
								}
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								if ( Object == PlanetTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else if ( Object == StationTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanStation_msg, kScienceScriptMessage);
								}
								else if ( Object == TribbleCruiserTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanShip_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								if ( Object == PlanetTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else if ( Object == StationTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanStation_msg, kScienceScriptMessage);
								}
								else if ( Object == TribbleCruiserTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanShip_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								if ( Object == MostRecentlyDestroyed )
								{
									fMission.mDisplayMessage( kScanTargetDestroyed_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								}
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if ( Object == PlanetTeam->mGetFirstShip() )
							{
								if ( TribbleHomeDestroyed == 0 )
								{
									fMission.mDisplayMessage( kEvent14_PlanetScannedNotDestroyed_msg, kMissionScriptMessage );
								}
								else
								{
								}
							}
							else if ( Object == StationTeam->mGetFirstShip() )
							{
								if ( TribbleCruiserCaptured == 0 )
								{
									fMission.mDisplayMessage( kEvent13_StationScannedNotCaptured_msg, kMissionScriptMessage );
								}
								else
								{
								}
							}
							else if ( Object == TribbleCruiserTeam->mGetFirstShip() )
							{
								if ( TribbleCruiserCaptured == 0 )
								{
									fMission.mDisplayMessage( kEvent12_CruiserScannedNotCaptured_msg, kMissionScriptMessage );
								}
								else
								{
								}
							}
						}
						// (Object points to entity or if NULL then MapPoint has location)
						if (Object)
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
						else
						{
							DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
						}
					}
					break;

				case kTimerTurnsSet_8:
					// set the Station comm buttons back to HAIL
					for (Ship = StationTeam->mGetFirstShip(); Ship != NULL; Ship = StationTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Tribble Cruiser comm buttons back to HAIL
					for (Ship = TribbleCruiserTeam->mGetFirstShip(); Ship != NULL; Ship = TribbleCruiserTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Fed comm buttons back to HAIL
					for (Ship = FedTeam->mGetFirstShip(); Ship != NULL; Ship = FedTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			if (Ship->mGetTeam() == kFedTeam)
			{
				FedDestroyed++;
				if (FedTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}

			if (Ship->mGetTeam() == kTribbleCruiserTeam)
			{
				TribbleCruiserDestroyed++;
				fMission.mDisplayMessage( kEvent4_ZoologyShipDestroyed_msg, kMissionScriptMessage );
			}

			if (Ship->mGetTeam() == kStationTeam)
			{
				StationDestroyed++;
				fMission.mDisplayMessage( kEvent3_StationDestroyed_msg, kMissionScriptMessage );
				MostRecentlyDestroyed = Ship;
				// LNF generate reinforcements?
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			// LNF player can get info on capture of Fed ship...any fed ship?
			Assert( Ship->mGetTeam() != kPlayerTeam );
			fMission.mDisplayMessage( kVesselCaptured_msg, kMissionScriptMessage );
			if ( Ship->mGetTeam() == kFedTeam )
			{
				FedCaptured++;
				fMission.mDisplayMessage( kEvent7_CaptureFed_msg, kMissionScriptMessage );
			}
			else if ( Ship == StationTeam->mGetFirstShip() )
			{
				StationCaptured++;
				fMission.mDisplayMessage( kEvent9_CaptureStation_msg, kMissionScriptMessage );
				DeepScanTracker.mRemoveTarget (Ship);
			}
			else if ( Ship == TribbleCruiserTeam->mGetFirstShip() )
			{
				// LNF what if a ship is captured while in the middle of a scan? - nada, from what I can tell
				TribbleCruiserCaptured++;
				fMission.mDisplayMessage( kEvent8_CaptureZoology_msg, kMissionScriptMessage );
				DeepScanTracker.mRemoveTarget (Ship);
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if (AtWarWithFed == 0)
								{
									fMission.mDisplayMessage (kFedHailNic_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kFedHail_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kTribbleCruiserTeam:
								if (AtWarWithFed == 0)
								{
									fMission.mDisplayMessage (kZooHailNic_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kZooHail_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kStationTeam:
								if (AtWarWithFed == 0)
								{
									fMission.mDisplayMessage (kStationHailNic_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStationHail_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if (Ship->mGetDamagePercent() < 50.0)
								{
									fMission.mDisplayMessage(kFedSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kFedSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kTribbleCruiserTeam:
								fMission.mDisplayMessage(kZooSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kStationTeam:
								fMission.mDisplayMessage(kStationSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if (AtWarWithFed == 0)
								{
									fMission.mDisplayMessage(kFedHail_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kFedTaunt1_msg, kFedTaunt4_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kTribbleCruiserTeam:
								fMission.mDisplayMessage (kZooTaunt_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kStationTeam:
								fMission.mDisplayMessage (kStationSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	//return fMission.mGetGameStatus();
	//return kGameInProgress;

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF primary - Destroy Tribble homeworld
	// LNF primary - destroy space station
	// LNF primary - destroy light cruiser with tribble on board

	VictoryPoints = (TribbleHomeDestroyed * 80) + (TribbleCruiserDestroyed * 5) + (StationDestroyed * 15);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 69)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// remove tribbles from the player's ship
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 0 );
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if ( VictoryLevel >= kDefeat )
		{
			fMission.mAddToDebriefingList( kExpelled_msg );

			// LNF try this out:  adding results of primary objectives
			fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

			( TribbleHomeDestroyed != 0 ) ? fMission.mAddToDebriefingList( kTribbleHomeDestroyedTrue_msg ) : SuccessCount++;
			( TribbleCruiserDestroyed != 0 ) ? fMission.mAddToDebriefingList( kTribbleCruiserDestroyedTrue_msg ) : SuccessCount++;
			( StationDestroyed != 0 ) ? fMission.mAddToDebriefingList( kStationDestroyedTrue_msg ) : SuccessCount++;
			if ( SuccessCount == 3 )
			{
				fMission.mAddToDebriefingList( kNone_msg );
			}

			fMission.mAddToDebriefingList( kFailedObjectives_msg );

			( TribbleHomeDestroyed == 0 ) ? fMission.mAddToDebriefingList( kTribbleHomeDestroyedFalse_msg ) : FailureCount++;
			( TribbleCruiserDestroyed == 0 ) ? fMission.mAddToDebriefingList( kTribbleCruiserDestroyedFalse_msg ) : FailureCount++;
			( StationDestroyed == 0 ) ? fMission.mAddToDebriefingList( kStationDestroyedFalse_msg ) : FailureCount++;
			if ( FailureCount == 3 )
			{
				fMission.mAddToDebriefingList( kNone_msg );
			}

			return;
		}

		if (VictoryLevel > kVictory)
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( TribbleHomeDestroyed != 0 ) ? fMission.mAddToDebriefingList( kTribbleHomeDestroyedTrue_msg ) : SuccessCount++;
	( TribbleCruiserDestroyed != 0 ) ? fMission.mAddToDebriefingList( kTribbleCruiserDestroyedTrue_msg ) : SuccessCount++;
	( StationDestroyed != 0 ) ? fMission.mAddToDebriefingList( kStationDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( TribbleHomeDestroyed == 0 ) ? fMission.mAddToDebriefingList( kTribbleHomeDestroyedFalse_msg ) : FailureCount++;
	( TribbleCruiserDestroyed == 0 ) ? fMission.mAddToDebriefingList( kTribbleCruiserDestroyedFalse_msg ) : FailureCount++;
	( StationDestroyed == 0 ) ? fMission.mAddToDebriefingList( kStationDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
		
	tMapPoint waypoints1[7]= // ship at U, V
	{
		25, 27,
		20, 26,
		18, 23,
		21, 20,
		27, 20,
		32, 23,
		29, 26
	};

	tMapPoint waypoints2[7]= // ship at W, X
	{
		24, 9,
		21, 11,
		20, 15,
		24, 18,
		29, 17,
		30, 12,
		27, 10
	};

	tMapPoint PatrolPoints[2][10];

	switch (ShipNumber)
	{
		case 0:
		case 1:
			PatrolPoints[0][0] = waypoints1[0];
			PatrolPoints[0][1] = waypoints1[1];
			PatrolPoints[0][2] = waypoints1[2];
			PatrolPoints[0][3] = waypoints1[3];
			PatrolPoints[0][4] = waypoints1[4];
			PatrolPoints[0][5] = waypoints1[5];
			PatrolPoints[0][6] = waypoints1[6];
			Ship->mPatrol (kPatrolAttackRadius, 7, &PatrolPoints[0][0], kHighPriority);
			break;

		case 2:
		case 3:
			PatrolPoints[1][0] = waypoints2[0];
			PatrolPoints[1][1] = waypoints2[1];
			PatrolPoints[1][2] = waypoints2[2];
			PatrolPoints[1][3] = waypoints2[3];
			PatrolPoints[1][4] = waypoints2[4];
			PatrolPoints[1][5] = waypoints2[5];
			PatrolPoints[1][6] = waypoints2[6];
			Ship->mPatrol (kPatrolAttackRadius, 7, &PatrolPoints[1][0], kHighPriority);
			break;
		}
}