#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Kli_6MissionText.h"
#include "Kli_6MissionMaps.h"
/*********************************

LNF maybe make the planet killer go after the player's biggest ship?
LNF should a frigate be able to kill the biggest planet killer?
LNF maybe a heavy, light, frigate, for the B, M, S PK?
*********************************/


//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 FedDestroyed = 0;
int32 FedCaptured = 0;
bool AllFedDestroyed = false;
int32 PlayerDestroyed = 0;
int32 ImperialDestroyed = 0;
int32 ImperialCaptured = 0;
bool AllImperialDestroyed = false;
int32 PkDestroyed = 0;
int32 AtWarWithFed = 0;
int32 AtWarWithImperial = 0;
int32 PkUnleashed = 0;
int32 Meeting = 0;
int32 Truce = 0;
int32 SecondAttack = 0;
int32 Mode = 0;
bool FireOnPkFirstTime = false;
bool FirstContact = false;
bool SecondContact = false;
bool ThirdContact = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key


const float kDistanceToImperial = 70.0; // distance to first meeting
const float kDistanceToImperial_2 = 40.0; // distance to second meeting
const float kDistanceToImperial_3 = 30.0; // distance to third meeting
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;

void setPatrolRoute(tShipInfo* Ship, tScript& tScript);

enum eMissionMode
{
	kStart = 0,
	kFedArrival
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFedTeam, 
	kImperialTeam,
	kPkTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kHydranMission | kLyranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kFederation;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFedTeam, kFederation);
	fMission.mSetTeamRace (kImperialTeam, kMirror);
	fMission.mSetTeamRace (kPkTeam, kMonster);
	

	// Set team relations to show up on Tactical map
	//fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	//fMission.mSetTeamRelations (kFedTeam, kImperialTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	//fMission.mSetTeamRelations (kFedTeam, kPkTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPkTeam, kImperialTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // player uses own ship
//	return kFalse; // player uses script assigned ships
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* PkTeam = fMission.mGetTeamHandle (kPkTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .85;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.15;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		ImperialTeam->mCreateShip (TotalBPV, kClassHeavyCruiser, kClassHeavyCruiser, 20, kStartPosition_Z);

		PkTeam->mCreateShip ("M-DMM", kStartPosition_V);
	}
	else
	{
		fMission.mSetTeamRelations (kFedTeam, kImperialTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kFedTeam, kPkTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		FedTeam->mCreateFleet (NumberOfPlayersShips, TotalBPV, kClassDestroyer, LargestEnemyShipClass, 20, kStartPosition_W);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* PkTeam = fMission.mGetTeamHandle (kPkTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// configure Imperial ship
	Ship = ImperialTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mChangeName(kApollo_name);

	// configure planetkiller
	PkTeam->mGetFirstShip()->mSetShipAIControlled( kFalse );
	PkTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );


	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// countdown for planet killer to be released if not by any other means
	fMission.mSetTimedInterval( kTimerTurnsSet, 20 );

	// timer to check player's distance to imperial ship
	fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );

	// win condition
	fMission.mSetTimedInterval( kTimerLimitSet_4, 4 );

	// LNF this is a temp thing, hopefully
	// pk death distance - ANY ship will blow up if it gets within 0.0 of the Pk
	//fMission.mSetTimedInterval( kTimerLimitSet_5, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);

	// configure Fed ships
	fMission.mSetGoalIndicator( FedTeam->mGetFirstShip() );
	// timer to turn off goal indicator
	fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	// LNF I think they should just attack player or imperial?
	Ship->mSetShipCallbackEvents (kNotifyAtDestination);
	Ship->mSetFinalDestination();
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* PkTeam = fMission.mGetTeamHandle (kPkTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);
	switch (Event)
	{
		case kNotifyAtDestination:
			if ( Ship->mGetTeam() == kFedTeam )
			{
				Ship->mClearCommands();
				fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				setPatrolRoute(Ship, *this);
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if (( Ship->mGetTeam() == kFedTeam ) &&
						( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						if (AtWarWithFed == 0)
						{
							AtWarWithFed = 1;
							FedTeam->mClearCommands();
							fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
						else if ((AtWarWithFed == 1) && (Truce == 1))
						{
							SecondAttack = 1;
							// LNF gotta do something about player breaking the truce
						}
					}
					else if (( Ship->mGetTeam() == kImperialTeam ) &&
							( HostileShip->mGetTeam() == kPlayerTeam ) && ( AtWarWithImperial == 0 ))
					{
						PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
						AtWarWithImperial = 1;
						// LNF check the Klingon/Federation relations - determine if at war
						fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						PkUnleashed = 1;
						fMission.mDisplayMessage( kEvent3_AttackImperial_msg, kMissionScriptMessage );
						fMission.mSetTimedInterval (kTimerCountdownSet, 1); // unleash planet killer
						// kill the global unleash timer
						fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
					}
					else if (( Ship == PkTeam->mGetFirstShip() ) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						Ship->mDamageSystem( kDamageWeaponsAndSystems, 0 );
						PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
						if ( PkUnleashed == 0 )
						{
							PkUnleashed = 1;
							fMission.mDisplayMessage( kEvent3_AttackImperial_msg, kMissionScriptMessage );
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							// kill the global unleash timer
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}

						if ( FireOnPkFirstTime == false )
						{
							FireOnPkFirstTime = true;
							fMission.mDisplayMessage( kEvent15_FireOnPk_msg, kMissionScriptMessage );
						}
						// LNF the planet killer should be invulnerable, except for an attack
						// LNF from the front by ramming something down its throat or nova bomb
						// LNF or are we making the planet killer "mortal"?
						//Ship->mDamageSystem( kDamageMaxShields, 0 );
						// LNF make the pk attack player?
					}
					Ship->mSetShipCallbackEvents (kNotifyDamage);
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					//fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					//fMission.mSetTeamRelations (kFedTeam, kImperialTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					//fMission.mSetTeamRelations (kFedTeam, kPkTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPkTeam, kImperialTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					break;

				case kTimerLimitSet_2:
					// check player's distance to imperial ship
					for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
					{
						if (( Ship->mGetTacticalDistance(ImperialTeam->mGetFirstShip()) <= kDistanceToImperial ) &&
							( Ship->mGetTacticalDistance(ImperialTeam->mGetFirstShip()) > kDistanceToImperial_2 ) &&
							( FirstContact == false ))
						{
							FirstContact = true;
							Meeting = 1;
							// communicate with Imperial - Event1
							fMission.mDisplayMessage(kEvent1_A_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_B_msg, kMissionScriptMessage);
							// set goal indicator to point to fed
							// LNF turn off goal indicator when player gets close
							mCreateMissionShips( Mode = kFedArrival );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;
						}
						else if (( Ship->mGetTacticalDistance(ImperialTeam->mGetFirstShip()) <= kDistanceToImperial_2 ) &&
								( Ship->mGetTacticalDistance(ImperialTeam->mGetFirstShip()) > kDistanceToImperial_3 ) &&
								( SecondContact == false ))
						{
							SecondContact = true;
							fMission.mDisplayMessage( kEvent1_TauntPlayer_msg, kMissionScriptMessage );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;
						}
/*						else if (( Ship->mGetTacticalDistance(ImperialTeam->mGetFirstShip()) <= kDistanceToImperial_3 ) &&
								( ThirdContact == false ))
						{
							ThirdContact = true;
							AtWarWithImperial = 1;
							PkUnleashed = 1;
							// kill the global unleash timer
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
							Meeting = 1;
							fMission.mDisplayMessage(kEvent1_C_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_D_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_PkUnleashed_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
							break;
						}
*/						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
					}
					
					break;

				case kTimerLimitSet_3:
					// turn off goal indicator that points to fed ships
					{
						tShipInfo* PlayerTeamsShip = NULL;
						tShipInfo* FedTeamsShip = NULL;

						for (PlayerTeamsShip = PlayerTeam->mGetFirstShip(); PlayerTeamsShip != NULL; PlayerTeamsShip = PlayerTeam->mGetNextShip())
						{
							for (FedTeamsShip = FedTeam->mGetFirstShip(); FedTeamsShip != NULL; FedTeamsShip = FedTeam->mGetNextShip())
							{
								if ( PlayerTeamsShip->mGetTacticalDistance(FedTeamsShip) < 50.0 )
								{
									fMission.mSetGoalIndicator( kFalse, 0, 0 );
									break;
								}
								else
								{
									fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
								}
							}
						}
					}
					break;

				case kTimerLimitSet_4:
					// win timer
					if (( AllFedDestroyed == true ) && ( AllImperialDestroyed == true ) && ( PkUnleashed == 1 ))
					{
						// LNF tell the player to leave the map
						fMission.mDisplayMessage( kCompleteObjectives_msg, kMissionScriptMessage );
						FindClosestMapEdge( *this, fPlayerShip, kMapEdgeX, kMapEdgeY );
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_4, 4 );
					}
					break;

				case kTimerLimitSet_5:
					// LNF temp timer to check distance for planet killer and
					// LNF kill whatever ship is within 0.0
					for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
					{
						if ( PkTeam->mGetFirstShip()->mGetTacticalDistance(Ship) <= 0.0 )
						{
							Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
							Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							PkTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100 );
							PkTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							break;
						}
					}

					for (Ship = FedTeam->mGetFirstShip(); Ship != NULL; Ship = FedTeam->mGetNextShip())
					{
						if ( PkTeam->mGetFirstShip()->mGetTacticalDistance(Ship) <= 0.0 )
						{
							Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
							Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							PkTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100 );
							PkTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							break;
						}
					}

					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						if ( PkTeam->mGetFirstShip()->mGetTacticalDistance(Ship) <= 0.0 )
						{
							Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
							Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							PkTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100 );
							PkTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPointsSilently, 1000 );
							break;
						}
					}
					fMission.mSetTimedInterval( kTimerLimitSet_5, 1 );
					break;

				case kTimerCountdownSet: // releases the Planet Killer
					if ( PkUnleashed == 0 )
					{
						PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
						fMission.mDisplayMessage( kEvent1_PkUnleashed_msg, kScienceScriptMessage );
						fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
						PkUnleashed = 1;
					}
					break;

				case kTimerTurnsSet: // releases the Planet Killer
					PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
					if ( PkUnleashed == 0 )
					{
						fMission.mDisplayMessage( kEvent1_PkUnleashed_msg, kScienceScriptMessage );
						fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
						PkUnleashed = 1;
					}
					break;

				case kTimerTurnsSet_9:
					// set the Imperial comm buttons back to HAIL
					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Fed comm buttons back to HAIL
					for (Ship = FedTeam->mGetFirstShip(); Ship != NULL; Ship = FedTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetOriginalTeam() == kFedTeam)
				{
					FedDestroyed++;
					if (FedTeam->mGetShipsInPlayCount() == 1)
					{
						// unleash the planet killer, if not already unleashed
						// the imperial has to say something
						if (PkUnleashed == 0)
						{
							PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
							PkUnleashed = 1;
							fMission.mDisplayMessage(kEvent1_C_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_D_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_PkUnleashed_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
							// LNF make sure this works
							ThirdContact = true;
							// kill the global unleash timer
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
					else if (FedTeam->mGetShipsInPlayCount() == 0)
					{
						AllFedDestroyed = true;

						// unleash the planet killer, if not already unleashed
						// the imperial has to say something
						if (PkUnleashed == 0)
						{
							PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
							PkUnleashed = 1;
							fMission.mDisplayMessage(kEvent1_C_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_D_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_PkUnleashed_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
							// LNF make sure this works
							ThirdContact = true;
							// kill the global unleash timer
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
				}
				else if (Ship->mGetOriginalTeam() == kImperialTeam)
				{
					ImperialDestroyed++;
					fMission.mDisplayMessage( kEvent4ImperialDead_msg, kMissionScriptMessage );
					if ( ImperialTeam->mGetShipsInPlayCount() == 0 )
					{
						AllImperialDestroyed = true;
						if ( PkUnleashed == 0 )
						{
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
				}
				else if (Ship->mGetOriginalTeam() == kPkTeam)
				{
					PkDestroyed++;
					// LNF what to do here
				}
				else if (Ship->mGetOriginalTeam() == kPlayerTeam)
				{
					PlayerDestroyed++;
					if (PlayerTeam->mGetShipsInPlayCount() == 0)
					{
						return kPlayerDestroyed;
					}
				}
			}
			break;

		case kNotifyOnCapture:
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );

				if ( Ship->mGetTeam() == kFedTeam )
				{
					FedCaptured++;
					fMission.mDisplayMessage( kEvent7_BoardFedShip_msg, kMissionScriptMessage );
					if (FedTeam->mGetShipsInPlayCount() == 1)
					{
						// unleash the planet killer, if not already unleashed
						// the imperial has to say something
						if (PkUnleashed == 0)
						{
							PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
							PkUnleashed = 1;
							fMission.mDisplayMessage(kEvent1_C_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_D_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_PkUnleashed_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
							// LNF make sure this works
							ThirdContact = true;
							// kill the global unleash timer
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
					else if (FedTeam->mGetShipsInPlayCount() == 0)
					{
						AllFedDestroyed = true;

						// unleash the planet killer, if not already unleashed
						// the imperial has to say something
						if (PkUnleashed == 0)
						{
							PkTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
							PkUnleashed = 1;
							fMission.mDisplayMessage(kEvent1_C_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_D_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent1_PkUnleashed_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPkTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kPkTeam, kTeamHostile, kGoalDestroy, 1.0);
							// LNF make sure this works
							ThirdContact = true;
							// kill the global unleash timer
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
				}
				else if ( Ship->mGetTeam() == kImperialTeam )
				{
					ImperialCaptured++;
					fMission.mDisplayMessage( kEvent8_BoardImperialShip_msg, kMissionScriptMessage );
					if ( ImperialTeam->mGetShipsInPlayCount() == 0 )
					{
						AllImperialDestroyed = true;
						if ( PkUnleashed == 0 )
						{
							fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
						}
					}
				}
			}
			break;

		case kNotifyDisengaged:
			// LNF check if the fed team is gone
			if ( Ship == fPlayerShip )
			{
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if ((AtWarWithFed == 0) && (AtWarWithImperial == 0)) // LNF and ALLIES
								{
									// LNF if dynaverse allies use this and make another condition for next line
									//fMission.mDisplayMessage (kFedHailAlliesAffAif1_msg, kMissionScriptMessage);
									//fMission.mDisplayMessage (kFedHailAlliesAffAif2_msg, kMissionScriptMessage);
									//fMission.mDisplayMessage (kFedHailAlliesAffAif3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kFedHailNicAffAif1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kFedHailNicAffAif2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kFedHailNicAffAif3_msg, kMissionScriptMessage);
								}
/*								else if ((AtWarWithFed == 0) && (AtWarWithImperial == 0)) // LNF and ENEMIES
								{
									fMission.mDisplayMessage(kFedHailAffAif_msg, kMissionScriptMessage);
								}*/
								else if ((AtWarWithFed == 1) && ((PkUnleashed == 0) || (SecondAttack == 1)))
								{
									fMission.mDisplayMessage(kFedHailAft2ndt_msg, kMissionScriptMessage);
								}
								else if ((AtWarWithFed == 1) && (PkUnleashed == 0) && (SecondAttack == 0))
								{
									fMission.mDisplayMessage(kFedHailAft2ndf_msg, kMissionScriptMessage);
									Truce = 1;
									fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
								}
								else if ((AtWarWithFed == 0) && (PkUnleashed == 0) && (SecondAttack == 0))
								{
									fMission.mDisplayMessage(kFedHailAff2ndfPkuf_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kImperialTeam:
								if ((Meeting == 0) && (AtWarWithFed == 0) && (AtWarWithImperial == 0))
								{
									fMission.mDisplayMessage(kImpHailMeetf_msg, kMissionScriptMessage);
								}
								else if ((Meeting == 1) && (AtWarWithFed == 0) && (AtWarWithImperial == 0))
								{
									fMission.mDisplayMessage(kImpHailMeett_msg, kMissionScriptMessage);
								}
								else if ((AtWarWithFed == 1) && (AtWarWithImperial == 0) && (PkUnleashed == 0))
								{
									if ( AllFedDestroyed == false )
									{
										fMission.mDisplayMessage(kImpHailAft_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kImpHailAit_msg, kMissionScriptMessage);
									}
								}
								else if ((AtWarWithImperial == 1) || (PkUnleashed == 1))
								{
									fMission.mDisplayMessage(kImpHailAit_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if ((PkUnleashed == 1) && (SecondAttack == 0)) // LNF need allies here?
								{
									fMission.mDisplayMessage(kFedTauntAllies_msg, kMissionScriptMessage);
								}
								else if (Ship->mGetDamagePercent() < 50.0)
								{
									fMission.mDisplayMessage(kFedSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kFedSurrenderDamaged_msg, kMissionScriptMessage);
									// LNF Fed is disengaging here, do I have to check something?
									Ship->mDisengage( kByMapEdge, kAbsolutePriority );
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kImperialTeam:
								fMission.mDisplayMessage(kImpSurrender1_msg, kMissionScriptMessage);
								if (AtWarWithImperial == 0)
								{
									fMission.mDisplayMessage(kImpSurrender2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if ((PkUnleashed == 1) && (SecondAttack == 0)) // LNF need allies?
								{
									fMission.mDisplayMessage(kFedTauntAllies_msg, kMissionScriptMessage);
								}
								else if (AtWarWithFed == 1)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kFedTaunt1_msg, kFedTaunt4_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kImperialTeam:
								fMission.mDisplayMessage(kImpSurrender1_msg, kMissionScriptMessage);
								if (AtWarWithImperial == 0)
								{
									fMission.mDisplayMessage(kImpSurrender2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF I should invert the PKUnleashed and the PkDestroyed amounts
	VictoryPoints = (PkUnleashed * 80) + (ImperialDestroyed * 10) + (FedDestroyed * 6) + (PkDestroyed * 11);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49) // There is no Defeat debrief
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (VictoryLevel == kDevastatingDefeat)
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( PkUnleashed != 0 ) ? fMission.mAddToDebriefingList( kPkUnleashedTrue_msg ) : SuccessCount++;
	( AllImperialDestroyed == true ) ? fMission.mAddToDebriefingList( kAllImperialDestroyedTrue_msg ) : SuccessCount++;
	( AllFedDestroyed == true ) ? fMission.mAddToDebriefingList( kAllFedDestroyedTrue_msg ) : SuccessCount++;
	( PkDestroyed != 0 ) ? fMission.mAddToDebriefingList( kPkDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( PkUnleashed == 0 ) ? fMission.mAddToDebriefingList( kPkUnleashedFalse_msg ) : FailureCount++;
	( AllImperialDestroyed == false ) ? fMission.mAddToDebriefingList( kAllImperialDestroyedFalse_msg ) : FailureCount++;
	( AllFedDestroyed == false ) ? fMission.mAddToDebriefingList( kAllFedDestroyedFalse_msg ) : FailureCount++;
	( PkDestroyed == 0 ) ? fMission.mAddToDebriefingList( kPkDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void setPatrolRoute(tShipInfo* Ship, tScript& tScript)
{
	const float PatrolAttackRadius = 30.0;
	tTeamInfo* FedTeam = tScript.fMission.mGetTeamHandle (kFedTeam);		

	tMapPoint waypoints1[8]= // ship W
	{
		24, 13,
		23, 16,
		20, 18,
		16, 16,
		14, 12,
		16, 8,
		20, 7,
		23, 9
	};

	tMapPoint waypoints2[9]= // ship X
	{
		25, 12,
		24, 9,
		20, 10,
		20, 13,
		22, 16,
		25, 17,
		29, 15,
		29, 11,
		27, 9
	};

	tMapPoint waypoints3[6]= // ship Y
	{
		26, 13,
		25, 17,
		22, 17,
		21, 13,
		24, 10,
		29, 12
	};

	tMapPoint PatrolPoints[3][10];

	if ( Ship == FedTeam->mGetThisShip(1) )
	{
		PatrolPoints[0][0] = waypoints1[0];
		PatrolPoints[0][1] = waypoints1[1];
		PatrolPoints[0][2] = waypoints1[2];
		PatrolPoints[0][3] = waypoints1[3];
		PatrolPoints[0][4] = waypoints1[4];
		PatrolPoints[0][5] = waypoints1[5];
		PatrolPoints[0][6] = waypoints1[6];
		PatrolPoints[0][7] = waypoints1[7];
		Ship->mPatrol (PatrolAttackRadius, 8, &PatrolPoints[0][0], kMediumPriority);
		//tScript.fMission.mDisplayMessage( kLNFReachDestination1_msg, kMissionScriptMessage );
	}
	else if ( Ship == FedTeam->mGetThisShip(2) )
	{
		PatrolPoints[1][0] = waypoints2[0];
		PatrolPoints[1][1] = waypoints2[1];
		PatrolPoints[1][2] = waypoints2[2];
		PatrolPoints[1][3] = waypoints2[3];
		PatrolPoints[1][4] = waypoints2[4];
		PatrolPoints[1][5] = waypoints2[5];
		PatrolPoints[1][6] = waypoints2[6];
		PatrolPoints[1][7] = waypoints2[7];
		PatrolPoints[1][8] = waypoints2[8];
		Ship->mPatrol (PatrolAttackRadius, 9, &PatrolPoints[1][0], kMediumPriority);
		//tScript.fMission.mDisplayMessage( kLNFReachDestination2_msg, kMissionScriptMessage );
	}
	else if ( Ship == FedTeam->mGetThisShip(3) )
	{
		PatrolPoints[2][0] = waypoints3[0];
		PatrolPoints[2][1] = waypoints3[1];
		PatrolPoints[2][2] = waypoints3[2];
		PatrolPoints[2][3] = waypoints3[3];
		PatrolPoints[2][4] = waypoints3[4];
		PatrolPoints[2][5] = waypoints3[5];
		Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[2][0], kMediumPriority);
		//tScript.fMission.mDisplayMessage( kLNFReachDestination3_msg, kMissionScriptMessage );
	}
}