#include "scriptinterface.h"

#include "Kli_7aMissionText.h"
#include "Kli_7aMissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 OrionDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 ScanFracture = 0;
int32 EnterFracture = 0;
const float kDeepScanDistance = 30.0;
int32 theChosenOne = -1; // the fracture that ends the mission
bool OrionsDetected = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

bool DamageFromFracture = false;

struct fractures
{
	int32 x;
	int32 y;
	bool scanned;
};

fractures fractureNumber[7]; // the number of fractures...there are 6, but start at 1
int32 currentFracture = -1;

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kOrionTeam, 
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission; //LNF change this for Klingons
	Specs->fEnemyRace = kAllRaceMission; //LNF should only be Hydran or Lyran
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .85;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.15;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// player's location
	const int32  kGPosX = 33;
	const int32  kGPosY = 10;

	// use a fancy schmancy thing to drop ships in a circle around any ship
	int32 RandomX = 0;
	int32 RandomY = 0;
	const int32 kMinRadius = 4;
	const int32 kLowRandom = -9;
	const int32 kHighRandom = 9;

	while ((abs(RandomX) < kMinRadius ) && ( abs(RandomY) < kMinRadius ))
	{
		RandomX = fMission.mRandomInt32( kLowRandom, kHighRandom );
		RandomY = fMission.mRandomInt32( kLowRandom, kHighRandom );
	}
	OrionTeam->mCreateShip (LowBPV, kClassDestroyer, kClassHeavyCruiser, 20, kStartPosition_Specified, (kGPosX + RandomX), (kGPosY + RandomY));
	RandomX = 0;
	RandomY = 0;
	while ((abs(RandomX) < kMinRadius ) && ( abs(RandomY) < kMinRadius ))
	{
		RandomX = fMission.mRandomInt32( kLowRandom, kHighRandom );
		RandomY = fMission.mRandomInt32( kLowRandom, kHighRandom );
	}
	OrionTeam->mCreateShip (LowBPV, kClassDestroyer, kClassHeavyCruiser, 20, kStartPosition_Specified, (kGPosX + RandomX), (kGPosY + RandomY));
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
	}

	// configure Orion ships
	for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}


	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// timer to check distance to fracture for damage and win
	fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );


	// initialize the coordinates of fractureNumber[]
	// LNF the comparison to find out which fracture is being scanned depends on 
	// LNF the fact that no two values of X are the same
	fractureNumber[1].x = 18;
	fractureNumber[1].y = 11;
	fractureNumber[2].x = 7;
	fractureNumber[2].y = 16;
	fractureNumber[3].x = 32;
	fractureNumber[3].y = 17;
	fractureNumber[4].x = 15;
	fractureNumber[4].y = 20;
	fractureNumber[5].x = 28;
	fractureNumber[5].y = 26;
	fractureNumber[6].x = 9;
	fractureNumber[6].y = 28;
	for ( int32 count = 1; count < 7; count++ )
	{
		fractureNumber[count].scanned = false;
	}

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord or ship from within kDeepScanDistance for X turn
	DeepScanTracker.mSetLocation (fractureNumber[1].x, fractureNumber[1].y, kDeepScanDistance, 1);
	DeepScanTracker.mSetLocation (fractureNumber[2].x, fractureNumber[2].y, kDeepScanDistance, 1);
	DeepScanTracker.mSetLocation (fractureNumber[3].x, fractureNumber[3].y, kDeepScanDistance, 1);
	DeepScanTracker.mSetLocation (fractureNumber[4].x, fractureNumber[4].y, kDeepScanDistance, 1);
	DeepScanTracker.mSetLocation (fractureNumber[5].x, fractureNumber[5].y, kDeepScanDistance, 1);
	DeepScanTracker.mSetLocation (fractureNumber[6].x, fractureNumber[6].y, kDeepScanDistance, 1);

	theChosenOne = fMission.mRandomInt32( 1, 6 ); // randomly pick the fracture that ends the mission
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					//fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0);
					//fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
					// make Orion AtWar
					fMission.mSetTimedInterval (kTimerTurnsSet, 1);

					// start message
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					break;

				case kTimerLimitSet_2:
					{
						// should be the win condition here

						int32 DistanceToFracture = 0;
						// checks the distance to all fractures for damage or win
						for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
						{
							for ( int32 count = 1; count < 7; count++ )
							{
								DistanceToFracture = Ship->mGetTacticalDistance( fractureNumber[count].x, fractureNumber[count].y );
								if ( DistanceToFracture == 0.0 )
								{
									if ( ScanFracture == 0 )
									{
										if ( DamageFromFracture == false )
										{
											Ship->mDamageSystem( kDamageInternalsWithPoints, 2 );
											DamageFromFracture = true;
											fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
											if ( Ship == fPlayerShip )
											{
												fMission.mDisplayMessage( kDamageByFracture_msg, kScienceScriptMessage );
											}
											else
											{
												fMission.mDisplayMessage( kDamageByFractureOther_msg, kScienceScriptMessage );
											}
										}
									}
									else
									{
										EnterFracture = 1;
										return kPlayerWins;
									}
								}
							}
						}
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								{
									for ( int32 count = 1; count < 7; count++ )
									{
										if (( MapPoint.x == fractureNumber[count].x ) &&
											( fractureNumber[count].scanned == false ))
										{
											fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
										}
									}
								}
								
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								{
									for ( int32 count = 1; count < 7; count++ )
									{
										if (( MapPoint.x == fractureNumber[count].x ) &&
											( fractureNumber[count].scanned == false ))
										{
											fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
											currentFracture = count;
										}
									}
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								{
									for ( int32 count = 1; count < 7; count++ )
									{
										if (( MapPoint.x == fractureNumber[count].x ) &&
											( fractureNumber[count].scanned == false ))
										{
											fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
										}
									}
								}
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								{
									for ( int32 count = 1; count < 7; count++ )
									{
										if (( MapPoint.x == fractureNumber[count].x ) &&
											( fractureNumber[count].scanned == false ))
										{
											fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
											currentFracture = count;
										}
									}
								}
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								{
									for ( int32 count = 1; count < 7; count++ )
									{
										if (( MapPoint.x == fractureNumber[count].x ) &&
											( fractureNumber[count].scanned == false ))
										{
											fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
										}
									}
								}
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object or location
							if ( currentFracture == theChosenOne )
							{
								fMission.mDisplayMessage( kEvent3CorrectFracture_msg, kScienceScriptMessage );
								ScanFracture = 1;

								// LNF have to figure out how the player enters this thing.
								// LNF probably set a distance thing here and tell the player to 
								// LNF go away and come back
								// LNF something about remodulating shields to be able to enter
								// LNF otherwise take damage
							}
							else
							{
								fMission.mDisplayMessage( kEvent1WrongFracture_msg, kScienceScriptMessage );
							}
							fractureNumber[currentFracture].scanned = true;

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					if ( OrionsDetected == false )
					{
						fMission.mDisplayMessage( kOrionsDetected_msg, kMissionScriptMessage );
						fMission.mSetTimedInterval (kTimerTurnsSet, 1);
						OrionsDetected = true;
					}
					else
					{
						fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0);
						fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
					}
					//fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					break;

				case kTimerTurnsSet_2:
					// waits 1 turn before fracture damages player
					DamageFromFracture = false;
					break;
			}
			break;

		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				OrionDestroyed++;
				if (OrionTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
				fMission.mDisplayMessage(kOriCaptured_msg, kMissionScriptMessage);
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						fMission.mDisplayMessage (kOriHail_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						break;

					case kButtonSurrender_text:
						fMission.mDisplayMessage(kOriSurrender_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						break;

					case kButtonTaunt_text:
						fMission.mDisplayMessage (fMission.mRandomInt32 (kOriTaunt1_msg, kOriTaunt4_msg), kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						break;
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = EnterFracture * 100;

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 69)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( EnterFracture != 0 ) ? fMission.mAddToDebriefingList( kEnterFractureTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( EnterFracture == 0 ) ? fMission.mAddToDebriefingList( kEnterFractureFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
