#include "scriptinterface.h"

#include "Kli_7bMissionText.h"
#include "Kli_7bMissionMaps.h"

/*************************
LNF create the wormhole for player to exit
LNF does it matter that the player can exit the game
LNF by just disengaging?
*************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 DefenseDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 PlanetScanned = 0;
int32 ShipyardDestroyed = 0;
int32 Revelation = 0;
int32 ShipyardCount = 0;
eVictoryLevels MyVicLevel = kDevastatingDefeat;

int32 WormholeX = 0;
int32 WormholeY = 0;
const float kWormholeWarningDistance = 30.0;
const float kWormholeDistance = 0.0;
const float kDeepScanDistancePlanet = 30.0;
const float kCloseToPlanet = 10.0;

bool AllShipyardsDestroyed = false;
bool AllPlatformsDestroyed = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kDefenseTeam,
	kShipyardTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission; //LNF change this for Klingons
	Specs->fEnemyRace = kAllRaceMission; //LNF should only be Hydran or Lyran
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kKlingon;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kDefenseTeam, kKlingon);
	fMission.mSetTeamRace (kShipyardTeam, kKlingon);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kShipyardTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDefenseTeam, kShipyardTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDefenseTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kShipyardTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	// Get the total BPV of the player and modify by 80% to 120%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	tShipInfo* Ship = NULL;

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// LNF need more Defense platforms
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 11, 7);
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 13, 12);
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 13, 18);
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 8, 20);
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 18, 12);
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 18, 18);
	DefenseTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 25, 16);

	// create shipyards
	ShipyardTeam->mCreateShip ("K-FRD", kStartPosition_W);
	ShipyardTeam->mCreateShip ("K-FRD", kStartPosition_X);
	ShipyardTeam->mCreateShip ("K-FRD", kStartPosition_Y);
	ShipyardTeam->mCreateShip ("K-D6G", kStartPosition_W);
	ShipyardTeam->mCreateShip ("K-E4", kStartPosition_X);
	ShipyardTeam->mCreateShip ("K-D5L", kStartPosition_Y);

	// create Planet
	PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, 15, 15 );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		//Ship->mTrackTacticalDistance(kWormholeWarningDistance, WormholeX, WormholeY); // Track wormhole
	}

	// configure Defense platforms
	for (Ship = DefenseTeam->mGetFirstShip(); Ship != NULL; Ship = DefenseTeam->mGetNextShip())
	{
		Ship->mChangeName(kDefensePlatform_name);
	}

	// configure Shipyards
	for (Ship = ShipyardTeam->mGetFirstShip(); Ship != NULL; Ship = ShipyardTeam->mGetNextShip())
	{
		Ship->mChangeName(kShipyard_name);
		Ship->mSetShipAIControlled( kFalse );
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 4);

	// check distance to planet
	fMission.mSetTimedInterval (kTimerLimitSet_3, 1);

	// check win condition
	fMission.mSetTimedInterval (kTimerLimitSet_4, 4);

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan planet from within distance 30 for 2 turns
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, 2);

	// location deep scan tracker example
	//DeepScanTracker.mSetLocation (20, 8, DeepScanDistance, 3); // track Moon
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	
	switch (Event)
	{
		// This is how the player finishes mission
		case kNotifyAtDistance: 
			if (fPlayerShip->mGetTrackingDistance() == kWormholeWarningDistance)
			{
				// warn player approaching wormhole
				fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
				fPlayerShip->mTrackTacticalDistance(kWormholeDistance, WormholeX, WormholeY);
			}
			else if (fPlayerShip->mGetTrackingDistance() == kWormholeDistance)
			{
				fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
				return kPlayerDisengaged;
			}
			break;
	

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// beginning message
					fMission.mDisplayMessage (kEvent1_A_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent1_B_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent1_C_msg, kMissionScriptMessage);
					fMission.mDisplayMessage (kEvent1_D_msg, kMissionScriptMessage);

					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kShipyardTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDefenseTeam, kShipyardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDefenseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kShipyardTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					// delay for message to scroll before defense starts firing
					fMission.mSetTimedInterval (kTimerLimitSet_2, 12);
					fMission.mSetTimedInterval (kTimerLimitSet_3, 3); // checking distance to planet
					break;
					
				case kTimerLimitSet_2:
					// defense platforms start attacking
					fMission.mSetTeamRelations (kPlayerTeam, kDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					break;

				case kTimerLimitSet_3:
					// checking distance to planet
					{
						for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
						{
							if (( Ship->mGetTacticalDistance(PlanetTeam->mGetFirstShip()) <= kCloseToPlanet ) &&
								( Revelation == 0 ))
							{
								fMission.mDisplayMessage(kEvent2_A_msg, kMissionScriptMessage);
								fMission.mDisplayMessage(kEvent2_B_msg, kMissionScriptMessage);
								fMission.mDisplayMessage(kEvent2_C_msg, kMissionScriptMessage);
								fMission.mDisplayMessage(kEvent2_D_msg, kMissionScriptMessage);
								Revelation = 1;
							}
							else if ( Revelation == 0 )
							{
								fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
							}
						}
					}
					break;

				case kTimerLimitSet_4:
					{
						// win condition
						if (( Revelation == 1 ) && ( PlanetScanned == 1 ) && ( AllPlatformsDestroyed == true ) &&
							( AllShipyardsDestroyed == true ))
						{
							fMission.mDisplayMessage( kObjectivesCompleted_msg, kMissionScriptMessage );
							switch ( fMission.mRandomInt32( 1, 3 ) )
							{
								case 1:
									WormholeX = 7;
									WormholeY = 21;
									break;

								case 2:
									WormholeX = 19;
									WormholeY = 6;
									break;

								case 3:
									WormholeX = 27;
									WormholeY = 13;
									break;
							}
							fMission.mCreateMapObject( 'E', WormholeX, WormholeY );
							fMission.mSetGoalIndicator( kTrue, WormholeX, WormholeY );
							for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
							{
								Ship->mTrackTacticalDistance(kWormholeWarningDistance, WormholeX, WormholeY ); // Track wormhole
							}
						}
						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_4, 4 );
						}
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								fMission.mDisplayMessage (kScannerOffInRangePlanet_msg, kScienceScriptMessage);
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						// successful scan
						if (flag == kInRange_ScanOn)
						{
							fMission.mDisplayMessage(kEvent3_ScanPlanet_msg, kScienceScriptMessage);
							PlanetScanned = 1;
							if ( Revelation == 0 )
							{
								fMission.mDisplayMessage( kEvent3_GetCloser_msg, kMissionScriptMessage );
							}
						}
						// (Object points to entity or if NULL then MapPoint has location)
						if (Object)
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
						else
						{
							DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
						}
					}
					break;

				case kTimerTurnsSet_10:
					// set the Planet comm buttons back to HAIL
					for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetOriginalTeam() == kDefenseTeam)
			{
				DefenseDestroyed++;
				if (DefenseTeam->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kEvent4_PlatformsDestroyed_msg, kMissionScriptMessage );
					AllPlatformsDestroyed = true;
				}
			}
			else if (Ship->mGetOriginalTeam() == kShipyardTeam)
			{
				ShipyardDestroyed++;
				if (ShipyardTeam->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kAllShipyardsDestroyed_msg, kMissionScriptMessage );
					AllShipyardsDestroyed = true;
				}
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			// LNF make the ships in the shipyard blow up if captured
			// LNF or maybe put a large amount of marines aboard those ships
			if ((Ship->mGetTeam() == kDefenseTeam) || (Ship->mGetTeam() == kShipyardTeam))
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			break;

		case kNotifyDisengaged:
			if ( Ship == fPlayerShip )
			{
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
			}
			break;
	
		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						// LNF hail the planet
						if ( Revelation == 0 )
						{
							fMission.mDisplayMessage( kHailPlanetNoRevelation_msg, kMissionScriptMessage );
							Ship->mSetCommButton (1, kButtonHail_text);
						}
						else
						{
							fMission.mDisplayMessage( kHailPlanetRevelation_msg, kMissionScriptMessage );
							Ship->mSetCommButton (1, kButtonTaunt_text);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							// If the Taunt or Surrender buttons aren't used in X turns
							// return the buttons to Hail and blank.
							fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						}
						break;

					case kButtonSurrender_text:
						fMission.mDisplayMessage( kHailPlanetRevelation_msg, kMissionScriptMessage );
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						fMission.mDisplayMessage( kHailPlanetRevelation_msg, kMissionScriptMessage );
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;
				}
			}
			break;
	}
	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (Revelation * 30) + (ShipyardDestroyed * 10) + (PlanetScanned * 15) + (DefenseDestroyed * 5);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		MyVicLevel = VictoryLevel;
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if ((VictoryLevel == kAstoundingVictory) || (VictoryLevel == kVictory))
		{
			fMission.mAddToDebriefingList (kVictory2_msg);
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( Revelation != 0 ) ? fMission.mAddToDebriefingList( kRevelationTrue_msg ) : SuccessCount++;
	( ShipyardDestroyed != 0 ) ? fMission.mAddToDebriefingList( kShipyardDestroyedTrue_msg ) : SuccessCount++;
	( PlanetScanned != 0 ) ? fMission.mAddToDebriefingList( kPlanetScannedTrue_msg ) : SuccessCount++;
	( DefenseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kDefenseDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( Revelation == 0 ) ? fMission.mAddToDebriefingList( kRevelationFalse_msg ) : FailureCount++;
	( ShipyardDestroyed == 0 ) ? fMission.mAddToDebriefingList( kShipyardDestroyedFalse_msg ) : FailureCount++;
	( PlanetScanned == 0 ) ? fMission.mAddToDebriefingList( kPlanetScannedFalse_msg ) : FailureCount++;
	( DefenseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kDefenseDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_4;
	}
	else
	{
		return 0;
	}
}
