#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Kli_9MissionText.h"
#include "Kli_9MissionMaps.h"
/***********************************
LNF global variable - KraaCaptured
LNF if Kraa is captured, then ignore some of the comm messages

LNF need to store the players ammo in variables to resupply later
***********************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

// inter-script variable
int32 checkthisout2 = 0;

int32 PlayerDestroyed = 0;
int32 FedDestroyed = 0;
int32 KlingonDestroyed = 0;
int32 HydranDestroyed = 0;
int32 LyranDestroyed = 0;
int32 PraxisDefenseDestroyed = 0;
int32 QnosDefenseDestroyed = 0;
int32 Mode = 0;

bool ResupplyInProgress = false;
bool PlayerFull = true; // ammo state of the player
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool AllFedDestroyed = false;
bool AllKlingonDestroyed = false;
bool AllHydranDestroyed = false;
bool AllLyranDestroyed = false;
bool AllPraxisDestroyed = false;
bool AllQnosDestroyed = false;
bool AllEnemyDestroyed = false;

const int32 kWormholeX = 4;
const int32 kWormholeY = 28;
const float kDamagePlanetDestroyed = 80.0;
const float kDamageMoonDestroyed = 80.0;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kDesiredNumberOfPlayersShips = 3;

// victory conditions
int32 PsionicDisruptorPlaced = 0;
int32 AttackedHome = 0;
int32 QnosDestroyed = 0;
int32 AllShipsDestroyed = 0;

bool MoonCaptured = false;

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

eGameStatus CheckWinCondition( tScript& tScript );
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

void configureShips(tScript& tScript, tShipInfo* Ship, tTeamInfo* TeamName);

enum eMissionMode
{
	kStart = 0,
	kReinforcements
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFedTeam,
	kKlingonTeam,
	kHydranTeam,
	kLyranTeam,
	kPraxisTeam,
	kQnosTeam,
	kPlanetTeam,
	kMoonTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kFederation;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFedTeam, kFederation);
	fMission.mSetTeamRace (kKlingonTeam, kKlingon);
	fMission.mSetTeamRace (kHydranTeam, kHydran);
	fMission.mSetTeamRace (kLyranTeam, kLyran);
	fMission.mSetTeamRace (kPraxisTeam, kKlingon);
	fMission.mSetTeamRace (kQnosTeam, kKlingon);
	fMission.mSetTeamRace (kPlanetTeam, kKlingon);
	fMission.mSetTeamRace (kMoonTeam, kKlingon);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kKlingonTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPraxisTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kQnosTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMoonTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedTeam, kKlingonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kKlingonTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKlingonTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKlingonTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKlingonTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKlingonTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKlingonTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kHydranTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kHydranTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kHydranTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kHydranTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kHydranTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kLyranTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLyranTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLyranTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLyranTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPraxisTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPraxisTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPraxisTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kQnosTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kQnosTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlanetTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// Get global variable information here.  there are currently 16 of them
	// int32 info.fScriptValues[0];

	//checkthisout2 = info.fScriptValues[0];

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle (kLyranTeam);
	tTeamInfo* PraxisTeam = fMission.mGetTeamHandle (kPraxisTeam);
	tTeamInfo* QnosTeam = fMission.mGetTeamHandle (kQnosTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* MoonTeam = fMission.mGetTeamHandle (kMoonTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	tShipInfo* Ship = NULL;

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// LNF should give player six ships, + maybe some reinforcements?
	// LNF maybe let the player use his own ships with less enemy ships,
	// LNF when the disruptor is placed, then reinforcements arrive on both sides
	// LNF and the player gets to control 6 ships?
	// LNF and reinforce the enemy when they die until psionic disruptor is placed
	// LNF reinforce the enemy from a wormhole

	if ( Mode == kStart )
	{
		// LNF ** begin testing this stuff **
		int32 NumPlayerShips = PlayerTeam->mGetShipsInPlayCount();
		int32 PlayerCombat = PlayerTeam->mGetCombatBPV();

		int32 LowBPV = (float) PlayerCombat / (float) NumPlayerShips * 0.8f;
		int32 HighBPV = (float) PlayerCombat / (float) NumPlayerShips * 1.3f;
		int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

		int32 EstimatedEnemyBPV = 5 * TotalBPV + 200;  // 5 enemy ships and the bats
		int32 OutGunned = EstimatedEnemyBPV - PlayerCombat;
		// I want the player to have 3 ships under their command.
		int32 gNumShipsAddedToCommand = kDesiredNumberOfPlayersShips - PlayerTeam->mGetShipsInPlayCount();
		int32 EscortBPV = 0.30f * (float) OutGunned / (float) gNumShipsAddedToCommand;
		// LNF original .65

		for ( int32 j = 0; j < gNumShipsAddedToCommand; j++ )
		{
			PlayerTeam->mCreateShip (EscortBPV, kClassFrigate, kClassBattleship, 50, kStartPosition_H + j);
		}
		// LNF ** end testing this stuff **

		FedTeam->mCreateFleet (1, (AveragePlayerBPV * 1), kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, 11, 35); // Z
		KlingonTeam->mCreateFleet (1, (AveragePlayerBPV * 1), kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, 14, 26); // W
		HydranTeam->mCreateFleet (2, (AveragePlayerBPV * 1), kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, 15, 34); // Y
		LyranTeam->mCreateFleet (1, (AveragePlayerBPV * 1), kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, 17, 30); // X
		//PraxisTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 6, 27);
		//PraxisTeam->mCreateShip ("K-BATS+", kStartPosition_Specified, 7, 26);
		//PraxisTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 4, 30);
		PraxisTeam->mCreateShip ("K-BATSR", kStartPosition_Specified, 13, 30);
		//PraxisTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 6, 33);
		//PraxisTeam->mCreateShip ("K-BATS+", kStartPosition_Specified, 8, 35);
		//QnosTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 33, 9);
		//QnosTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 29, 11);
		QnosTeam->mCreateShip ("K-SBR", kStartPosition_Specified, 31, 8);
		//QnosTeam->mCreateShip ("K-BATSR", kStartPosition_Specified, 35, 19);
		//QnosTeam->mCreateShip ("K-DEF", kStartPosition_Specified, 38, 14);
		PlanetTeam->mCreateShip ("PL1", kStartPosition_Specified, 33, 13);
		MoonTeam->mCreateShip ("PL0", kStartPosition_Specified, 8, 30);
	}
	else if ( Mode == kReinforcements )
	{
/*
		// LNF player reinforcements after they place psionic disruptor
		int32 AddToFed = abs(2 - FedTeam->mGetShipsInPlayCount());
		int32 AddToKlingon = abs(2 - KlingonTeam->mGetShipsInPlayCount());
		int32 AddToHydran = abs(2 - HydranTeam->mGetShipsInPlayCount());
		int32 AddToLyran = abs(2 - LyranTeam->mGetShipsInPlayCount());
		int32 AddToPlayer = abs(6 - PlayerTeam->mGetShipsInPlayCount());

		fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlayerTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlayerTeam, kQnosTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kPlayerTeam, kMoonTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		fMission.mSetTeamRelations (kFedTeam, kKlingonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kFedTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kFedTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kFedTeam, kQnosTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kFedTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		fMission.mSetTeamRelations (kKlingonTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kQnosTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		fMission.mSetTeamRelations (kHydranTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kHydranTeam, kQnosTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kHydranTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kHydranTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		
		fMission.mSetTeamRelations (kLyranTeam, kQnosTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kLyranTeam, kPlanetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kLyranTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		fMission.mSetTeamRelations (kQnosTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kQnosTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		fMission.mSetTeamRelations (kPlanetTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

//		FedTeam->mCreateFleet (AddToFed, TotalBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX, kWormholeY);
//		KlingonTeam->mCreateFleet (AddToKlingon, TotalBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX+1, kWormholeY);
//		HydranTeam->mCreateFleet (AddToHydran, TotalBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX, kWormholeY+1);
//		LyranTeam->mCreateFleet (AddToLyran, TotalBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX+1, kWormholeY+1);
//		PlayerTeam->mCreateFleet (AddToPlayer, TotalBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_M);

		FedTeam->mCreateFleet (1, AveragePlayerBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX, kWormholeY);
		KlingonTeam->mCreateFleet (1, AveragePlayerBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX+1, kWormholeY);
		HydranTeam->mCreateFleet (1, AveragePlayerBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX, kWormholeY+1);
		LyranTeam->mCreateFleet (1, AveragePlayerBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_Specified, kWormholeX+1, kWormholeY+1);
		PlayerTeam->mCreateFleet (4, TotalBPV, kClassHeavyCruiser, kClassBattleship, 20, kStartPosition_M);
*/
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle (kLyranTeam);
	tTeamInfo* PraxisTeam = fMission.mGetTeamHandle (kPraxisTeam);
	tTeamInfo* QnosTeam = fMission.mGetTeamHandle (kQnosTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* MoonTeam = fMission.mGetTeamHandle (kMoonTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransPsionicDisruptor, 1 );
		//Ship->mTrackTacticalDistance(30.0, 11, 29);
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}

	// configure ships
	configureShips(*this, Ship, FedTeam);
	configureShips(*this, Ship, KlingonTeam);
	configureShips(*this, Ship, LyranTeam);
	//configureShips(*this, Ship, HydranTeam);

	// configure Hydrans
	for (Ship = HydranTeam->mGetFirstShip(); Ship != NULL; Ship = HydranTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// configure Defense platforms
	for (Ship = QnosTeam->mGetFirstShip(); Ship != NULL; Ship = QnosTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	// configure Planet
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
	// LNF temporary
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyTransportedItemTo );
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
	PlanetTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
	PlanetTeam->mGetFirstShip()->mChangeName( kQonos_name );

	//PlanetTeam->mGetFirstShip()->mAccessSupplies( kSetTransporterItemCount, kTransPsionicDisruptor, 1 );

	// configure Moon
	MoonTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyTransportedItemTo );
	MoonTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	MoonTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
	MoonTeam->mGetFirstShip()->mChangeName( kPraxis_name );


	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 3);


	//*****bbb
	if ( checkthisout2 == 5 )
	{
		//fMission.mDisplayMessage( kfive_msg );
	}
	else
	{
		//fMission.mDisplayMessage( kfive2_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle (kLyranTeam);
	tTeamInfo* PraxisTeam = fMission.mGetTeamHandle (kPraxisTeam);
	tTeamInfo* QnosTeam = fMission.mGetTeamHandle (kQnosTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* MoonTeam = fMission.mGetTeamHandle (kMoonTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				// LNF how about this.  If the player is able to transport the psionic
				// LNF disruptor to the moon, set a counter to 'rewire' the base
				// LNF and then do whatever effect this psi disruptor does.
				// LNF I don't know if the player will have to capture the moon base
				// LNF but probably not
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				// LNF change kPlanetTeam to kMoonTeam
				// LNF change kTransPsionicDisruptor to PsionicDisrupter

				// LNF put in helpful messages like "need to capture planet" and things like that.

				if ( TransportTarget != NULL )
				{
					if ( TransportTarget->mGetTeam() == kMoonTeam )
					{
						if ( fEventItem1 == kTransPsionicDisruptor )
						{
							if ( TransportTarget->mAccessSupplies( kGetTransporterItemCount, kTransPsionicDisruptor ) > 1 )
							{
								// LNF the player should NOT be able to transport multiple projectors
								// LNF maybe let them transport and not care if they lose it
							}
							else
							{
								// transported the psionic projector to the moon
								fMission.mDisplayMessage( kEvent3_ActivatingNeutralizer_msg, kMissionScriptMessage );
								PsionicDisruptorPlaced = 1;
								// blow up all bats
								for (tShipInfo* DefShip = PraxisTeam->mGetFirstShip(); DefShip != NULL; DefShip = PraxisTeam->mGetNextShip())
								{
									DefShip->mDamageSystem( kDamageExcessDamageSystem, 100 );
									DefShip->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
								}
								// reinforcements arrive
								//Mode = kReinforcements;
								//mCreateMissionShips( Mode );
							}
						}
						else // if it's any other item
						{
							//fMission.mDisplayMessage( kLNFWrongItem_msg, kMissionScriptMessage );
						}
					}
					else // player shouldn't be able to transport item
					{
						if ( fEventItem1 == kTransSparePart )
						{
							int32 partsOnPlayersShip = fPlayerShip->mAccessSupplies( kGetSupplyCount, kSupplySpareParts );
							int32 partsInTransporter = fPlayerShip->mAccessSupplies( kGetTransporterItemCount, kTransSparePart );
							int32 partsOnTargetShip = TransportTarget->mAccessSupplies( kGetSupplyCount, kSupplySpareParts );

							fPlayerShip->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, (partsOnPlayersShip + 1) );
							TransportTarget->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, (partsOnTargetShip - 1) );

							//fMission.mDisplayMessage( kLNFCantTransTo_msg, kMissionScriptMessage );
						}
						else
						{
							//fMission.mDisplayMessage( kLNFCantTransTo_msg, kMissionScriptMessage );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, fEventItem1, 0 );
							fPlayerShip->mAccessSupplies( kSetTransporterItemCount, fEventItem1, 1 );
						}
					}
				}

				Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				// LNF how about this.  If the player is able to transport the psionic
				// LNF disruptor to the moon, set a counter to 'rewire' the base
				// LNF and then do whatever effect this psi disruptor does.
				// LNF I don't know if the player will have to capture the moon base
				// LNF but probably not
				tShipInfo* TransportFrom = fMission.mFindShipID (fEventID);

				if ( TransportFrom != NULL )
				{
					if ( TransportFrom->mGetOriginalTeam() == kMoonTeam )
					{
						// LNF don't let the player take the projector
						// LNF replace kTransPsionicDisruptor with PsionicProjector
						if ( fEventItem1 == kTransPsionicDisruptor )
						{
							// LNF message about not being able to take egg
							//fMission.mDisplayMessage( kLNFCantTransFrom_msg, kMissionScriptMessage );
							TransportFrom->mAccessSupplies( kSetTransporterItemCount, fEventItem1, 1 );
							fPlayerShip->mAccessSupplies( kSetTransporterItemCount, fEventItem1, 0 );
						}
					}
				}
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ((( Ship->mGetTeam() == kPlanetTeam ) || ( Ship->mGetTeam() == kQnosTeam ))
							&& ( AttackedHome == 0 ))
						{
							AttackedHome = 1;
							fMission.mDisplayMessage( kEvent2_AttackedHome_msg, kMissionScriptMessage );
							fMission.mDisplayMessage( kEvent2_HomeAttacksBack_msg, kMissionScriptMessage );
							fMission.mSetTeamRelations (kQnosTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
						else if (( Ship->mGetTeam() == kFedTeam )
								|| ( Ship->mGetTeam() == kKlingonTeam )
								|| ( Ship->mGetTeam() == kLyranTeam )
								|| ( Ship->mGetTeam() == kPraxisTeam )
								|| ( Ship->mGetTeam() == kMoonTeam ))
						{
							FedTeam->mClearCommands();
							KlingonTeam->mClearCommands();
							LyranTeam->mClearCommands();
							fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kPraxisTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
					}
					else if ( Ship->mGetTeam() == kPlanetTeam )
					{
						if ( PlanetTeam->mGetFirstShip()->mGetDamagePercent() > kDamagePlanetDestroyed )
						{
							fMission.mDisplayMessage( kQnosDestroyed_msg, kMissionScriptMessage );
							QnosDestroyed = 1;
						}
						else
						{
							Ship->mSetShipCallbackEvents( kNotifyDamage );
						}
					}
					else if ( Ship->mGetTeam() == kMoonTeam )
					{
						if ( MoonTeam->mGetFirstShip()->mGetDamagePercent() > kDamageMoonDestroyed )
						{
							fMission.mDisplayMessage( kMoonDestroyed_msg, kMissionScriptMessage );
						}
						else
						{
							Ship->mSetShipCallbackEvents( kNotifyDamage );
						}
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		// This is how the player finishes mission
		case kNotifyAtDistance: 
/*			if (fPlayerShip->mGetTrackingDistance() == 30.0)
			{
				// warn player approaching wormhole
				fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
				fPlayerShip->mTrackTacticalDistance(10.0, 11, 29);
			}
			else if (fPlayerShip->mGetTrackingDistance() == 10.0)
			{
				return kPlayerDisengaged;
			}*/
			break;
	

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kHydranTeam, kTeamHostile, kGoalDestroy, 1.0);//****
					fMission.mSetTeamRelations (kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPraxisTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kQnosTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kMoonTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kFedTeam, kKlingonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kKlingonTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kKlingonTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kKlingonTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kKlingonTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kKlingonTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kKlingonTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kHydranTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//****
					fMission.mSetTeamRelations (kHydranTeam, kLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kHydranTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kHydranTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kHydranTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kHydranTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					
					fMission.mSetTeamRelations (kLyranTeam, kPraxisTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLyranTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLyranTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLyranTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPraxisTeam, kQnosTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPraxisTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPraxisTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kQnosTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kQnosTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlanetTeam, kMoonTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					// start scenario message 
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					// resupply distance timer
					if ( ResupplyInProgress == true )
					{
						if ( fPlayerShip->mGetTacticalDistance( PlanetTeam->mGetFirstShip() ) > 5.0 )
						{
							fMission.mDisplayMessage( kTooFarToTransport_msg, kMissionScriptMessage );
							ResupplyInProgress = false;
						}
						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
					}
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kEvent3_AlienInformation_msg, kMissionScriptMessage );
					// LNF I actually have to do something here now that the disruptor is working
					break;

				case kTimerCountdownSet:
					// resupply timer
					// LNF give the player his crap
					fMission.mDisplayMessage( kResupplyDone_msg, kMissionScriptMessage );
					ResupplyInProgress = false;
					break;
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			// if any ship but the moon and planet are captured, destroy it
			if (( Ship->mGetTeam() != kPlayerTeam ) && ( Ship->mGetTeam() != kPlanetTeam ) &&
				( Ship->mGetTeam() != kMoonTeam ))
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
			}
			else if ( Ship->mGetTeam() == kMoonTeam )
			{
				if ( PsionicDisruptorPlaced == 0 )
				{
					fMission.mDisplayMessage( kPlacePsionicDisruptor_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyAtDeath:
			switch ( Ship->mGetOriginalTeam() )
			{
				case kFedTeam:
					FedDestroyed++;

					if (FedTeam->mGetShipsInPlayCount() == 0)
					{
						AllFedDestroyed = true;
						return CheckWinCondition( *this );
					}
					break;

				case kKlingonTeam:
					KlingonDestroyed++;

					if (KlingonTeam->mGetShipsInPlayCount() == 0)
					{
						AllKlingonDestroyed = true;
						return CheckWinCondition( *this );
					}
					break;

				case kHydranTeam:
					HydranDestroyed++;

					if (HydranTeam->mGetShipsInPlayCount() == 0)
					{
						AllHydranDestroyed = true;
						return CheckWinCondition( *this );
					}
					break;

				case kLyranTeam:
					LyranDestroyed++;

					if (LyranTeam->mGetShipsInPlayCount() == 0)
					{
						AllLyranDestroyed = true;
						return CheckWinCondition( *this );
					}
					break;

				case kPraxisTeam:
					PraxisDefenseDestroyed++;

					if (PraxisTeam->mGetShipsInPlayCount() == 0)
					{
						AllPraxisDestroyed = true;
						return CheckWinCondition( *this );
					}
					break;

				case kQnosTeam:
					QnosDefenseDestroyed++;

					if (QnosTeam->mGetShipsInPlayCount() == 0)
					{
						AllQnosDestroyed = true;
						return CheckWinCondition( *this );
					}
					break;
			}

			if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						if ( Ship->mGetTeam() == kPlanetTeam )
						{
							if ( AttackedHome == 0 )
							{
								fMission.mDisplayMessage( kCommPlanet_msg, kMissionScriptMessage );
								Ship->mSetCommButton( 1, kButtonResupply_text );
							}
							else
							{
								fMission.mDisplayMessage( kCommPlanetEnemy_msg, kMissionScriptMessage );
								Ship->mSetCommButton( 1, kButtonHail_text );
							}
						}
						/*
						// LNF if Kraa is captured? or leaves? ships say nothing?
						fMission.mDisplayMessage(kHailKraaPresent_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						*/
						break;

					case kButtonSurrender_text:
						fMission.mDisplayMessage(kSurrenderKraaPresent_msg, kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						break;

					case kButtonTaunt_text:
						fMission.mDisplayMessage (fMission.mRandomInt32 (kTaunt1_msg, kTaunt4_msg), kMissionScriptMessage);
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						break;

					case kButtonResupply_text:
						if ( AttackedHome == 0 )
						{
							// LNF test stuff
							fPlayerShip->mAccessSupplies( kResetSupplyCounts, kSupplyNumberLoaded );
							fPlayerShip->mAccessSupplies( kResetSupplyCounts, kSupplyMarines );

							if ( fPlayerShip->mGetTacticalDistance( PlanetTeam->mGetFirstShip() ) <= 5.0 )
							{
								fMission.mDisplayMessage( kRequestResupply_msg, kMissionScriptMessage );
								// LNF check here if the player needs stuff
/*								if ( PlayerFull == false )
								{
									fMission.mDisplayMessage( kRequestResupply_msg, kMissionScriptMessage );
									ResupplyInProgress = true;
									// resupply timer
									fMission.mSetTimedInterval( kTimerCountdownSet, 2 );
									// distance timer
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
								}
								else
								{
									fMission.mDisplayMessage( kPlayerFull_msg, kMissionScriptMessage );
								}
*/
							}
							else
							{
								fMission.mDisplayMessage( kRequestResupplyOutOfRange_msg, kMissionScriptMessage );
							}
							// LNF make this go back to Hail?
							Ship->mSetCommButton( 1, kButtonResupply_text );
						}
						else
						{
							fMission.mDisplayMessage( kCommAttackedHome_msg, kMissionScriptMessage );
							Ship->mSetCommButton( 1, kButtonHail_text );
						}
						break;
				}
			}
			break;
	}
	//return fMission.mGetGameStatus();
	//return kGameInProgress;

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus(status | kCampaignCompleted);
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF maybe need enemy fleet destroyed to make up the other half of the victory points.
	VictoryPoints = (PsionicDisruptorPlaced * 50) - (AttackedHome * 100) - (QnosDestroyed * 100);
	( AllEnemyDestroyed == true ) ? VictoryPoints += 50 : VictoryPoints;

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		//fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
/*
	kAstoundingVictory,
	kVictory,
	kDraw,
	kDefeat,
	kDevastatingDefeat,
*/
		if ( VictoryLevel >= kDraw )
		{
			fMission.mAddToDebriefingList ( kEndCampaignDefeat_msg );
		}
		else
		{
			fMission.mAddToDebriefingList ( kVictory2_msg );
			fMission.mAddToDebriefingList ( kEndCampaignVictory_msg );
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	// LNF maybe need enemy fleet destroyed to make up the other half of the victory points.
	( PsionicDisruptorPlaced != 0 ) ? fMission.mAddToDebriefingList( kPsionicDisruptorPlacedTrue_msg ) : SuccessCount++;
	( AllEnemyDestroyed == true ) ? fMission.mAddToDebriefingList( kAllEnemyDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( PsionicDisruptorPlaced == 0 ) ? fMission.mAddToDebriefingList( kPsionicDisruptorPlacedFalse_msg ) : FailureCount++;
	( AllEnemyDestroyed == false ) ? fMission.mAddToDebriefingList( kAllEnemyDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// configure the fleets
void configureShips(tScript& tScript, tShipInfo* Ship, tTeamInfo* TeamName)
{
	tTeamInfo* MoonTeam = tScript.fMission.mGetTeamHandle (kMoonTeam);
	tTeamInfo* KlingonTeam = tScript.fMission.mGetTeamHandle (kKlingonTeam);

	for (Ship = TeamName->mGetFirstShip(); Ship != NULL; Ship = TeamName->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mDefendEntity( MoonTeam->mGetFirstShip(), 60.0, 100.0, kAbsolutePriority );
		Ship->mSetShipCallbackEvents( kNotifyDamage );

		if ( Ship->mGetTeam() == kKlingonTeam )
		{
			Ship->mChangeName( kPsionicKlingon_name );
		}
	}
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip)
{
/*
	// if all ships on a team are gone
	//if ( EventTeam->mGetShipsInPlayCount() == ((EventShipStatus == kEventShipCaptured) ? 1 : 0) )
	if (Ship->mGetTeam() == kTuviusTeam)
	{
		TuviusDestroyed++;
		if ( TuviusTeam->mGetShipsInPlayCount() == 0 )
		{
			AllTuviusTeamDestroyed = true;
			CheckWinCondition( tScript );
		}
	}
	else if (Ship->mGetTeam() == kFactionTeam)
	{
		FactionDestroyed++;
		if ( FactionTeam->mGetShipsInPlayCount() == 0 )
		{
			AllFactionTeamDestroyed = true;
			CheckWinCondition( tScript );
		}
	}
	else if (Ship->mGetTeam() == kShiarTeam)
	{
		ShiarDestroyed++;
		if ( ShiarTeam->mGetShipsInPlayCount() == 0 )
		{
			AllShiarTeamDestroyed = true;
			CheckWinCondition( tScript );
		}
	}
	else if (Ship->mGetTeam() == kPraiexTeam)
	{
		PraiexDestroyed++;
		if ( PraiexTeam->mGetShipsInPlayCount() == 0 )
		{
			AllPraiexTeamDestroyed = true;
			CheckWinCondition( tScript );
		}
	}
	else if (Ship->mGetTeam() == kPlanetDefenseTeam)
	{
		PlanetDefenseDestroyed++;
		if ( PlanetDefenseTeam->mGetShipsInPlayCount() == 0 )
		{
			AllPlanetDefenseTeamDestroyed = true;
			tScript.fMission.mDisplayMessage( kEvent6DefPlatsDestroyed_msg, kMissionScriptMessage );

			// LNF other ships should attack anyone

			for (Ship = TuviusTeam->mGetFirstShip(); Ship != NULL; Ship = TuviusTeam->mGetNextShip())
			{
				// LNF check this line against the configure line
				Ship->mSetShipAIControlled( kTrue );
				// LNF tell it to go to planet?
			}

			CheckWinCondition( tScript );
		}
	}*/
}

//---------------------------------------------------------------------------------------------
// win condition
eGameStatus CheckWinCondition( tScript& tScript )
{
	if (( AllFedDestroyed == true ) && ( AllKlingonDestroyed == true ) &&
		( AllHydranDestroyed == true ) && ( AllLyranDestroyed == true ) && 
		( AllPraxisDestroyed == true )) // took out checking for the psionic disruptor
	{
		AllEnemyDestroyed = true;
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		return eGameStatus( kPlayerWins | kCampaignCompleted );
		//FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
	}
	else
	{
		// test to end the campaign
		eGameStatus status = tScript.fMission.mGetGameStatus();
		if (status != kGameInProgress)
		{
			status = eGameStatus(status | kCampaignCompleted);
		}
		return status;
	}
}
