#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_1MissionText.h"
#include "Lyr_1MissionMaps.h"
/**********************************
LNF need black planet
LNF figure out DeepScanTracking to make things work with overlapping scan areas
LNF shrink the maps and make a completed goal message that says
LNF "primary goals complete", and point to nearest map edge to end mission?
**********************************/


// sequence of events:
// 1. player hails the freighter BEFORE attacking it
//    (if player attacks first then planet message does not get sent)
// 2. player scans the moon.
// 3. player scans the planet position.
// 4. player attacks Orion ships
//
// player objectives:
// 1. talk with freighter.
// 2. scan the planet
// 3. destroy all Orions



int32 AttackedFreighter = 0;
int32 AtWarWithOrion = 0;
int32 TalkedToFreighter = 0;
int32 MoonScanned = 0;
int32 PlanetScanned = 0;
int32 FreighterDestroyed = 0;
int32 OrionShipCount = 0;
int32 OrionDestroyed = 0;
bool AllOrionsDestroyed = false;
const float kDamageThreshold = 40.0;
const float kMoonDeepScanDistance = 10.0;
const float kPlanetDeepScanDistance = 20.0;
int32 MoonX = 15;
int32 MoonY = 9;

eClassTypes PlayerShipClass= kClassShuttle;
int32 Elite = 0;
int32 PlayLevel = 0;

enum eOfferStatus
{
	kNoOffer = 0,
	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
};

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFreighterTeam, 
	kOrionTeam,
	kPlanetTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	info.fAlliedRace = kLyran;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFreighterTeam, info.fAlliedRace);
	fMission.mSetTeamRace (kOrionTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	Elite = fMission.mGetEliteOrganization();
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

		switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .7;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.0;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.4;
			break;
	}
	
	// create Orion team
	//OrionTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	OrionShipCount = PlayerTeam->mGetShipsInPlayCount();
	OrionTeam->mCreateFleet (OrionShipCount, (TotalBPV/OrionShipCount), kClassFrigate, kClassFrigate, 50, kStartPosition_M);
	OrionTeam->mCreateFleet (OrionShipCount, (TotalBPV/OrionShipCount), kClassFrigate, kClassFrigate, 50, kStartPosition_N);

	// create Planet
	PlanetTeam->mCreateShip ("PLPh5", kStartPosition_P);

	// create Freighter team
	FreighterTeam->mCreateShip ("L-FLPh", kStartPosition_Z);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure Orion team
	for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageLeftWarpEngine, 80);
		Ship->mDamageSystem(kDamageRightWarpEngine, 80);
	}

	// configure Freighter Team
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipThrust (10);
		//Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
	}

	// cloak the planet
	Ship = PlanetTeam->mGetFirstShip();
	Ship->mChangeAIOfficerState( kCloakOfficer, kFalse );
	Ship->mCloak( kTrue );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord from within kDeepScanDistance for 1 turn

	// LNF this it the moon object.  Don't know if it has to be scanned, 
	DeepScanTracker.mSetTarget (PlanetTeam->mGetFirstShip(), kPlanetDeepScanDistance, 3);
	DeepScanTracker.mSetLocation (MoonX, MoonY, kMoonDeepScanDistance, 2); // track Moon

	
	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point to freighter to scan
	fMission.mSetGoalIndicator (FreighterTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ((Ship->mGetTeam() == kFreighterTeam) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						AttackedFreighter = 1;
					}
					else if ((Ship->mGetTeam() == kOrionTeam) && ( HostileShip->mGetTeam() == kPlayerTeam ))
					{
						AtWarWithOrion = 1;
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kEnteringRange_ScanOff:
								if ( Object == PlanetTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage(kEvent2ScanningRange_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								}
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								if ( Object == PlanetTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStartingScanMoon_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOn_TimerStarted:
								if ( Object == PlanetTeam->mGetFirstShip() )
								{
									fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kStartingScanMoon_msg, kScienceScriptMessage);
								}
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if ( Object == PlanetTeam->mGetFirstShip() )
							{
								fMission.mDisplayMessage(kEvent3ScanPlanet_1_msg, kScienceScriptMessage);
								fMission.mDisplayMessage(kEvent3ScanPlanet_2_msg, kScienceScriptMessage);
								fMission.mDisplayMessage(kEvent3ScanPlanet_3_msg, kScienceScriptMessage);
								PlanetScanned = 1;
								TestObjectives (*this);
							}
							else
							{
								fMission.mDisplayMessage(kEvent1ScanMoon_msg, kScienceScriptMessage);
								MoonScanned = 1;
								TestObjectives (*this);
							}

							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_9:
					// set ALL the ship's comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set ALL the ship's comm buttons back to HAIL
					for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			if (Ship->mGetTeam() == kOrionTeam)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
				// we don't test objectives now - only when ship blows up
				break;
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
			}
			else if (Ship->mGetTeam() == kOrionTeam)
			{
				OrionDestroyed++;
				TestObjectives (*this);
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								if (AtWarWithOrion == 0)
								{
									if ((AttackedFreighter == 0) && (TalkedToFreighter == 1))
									{
										fMission.mDisplayMessage(kFreighterHailTalkedTo_msg, kMissionScriptMessage);
									}
								}
								else
								{
									if ((AttackedFreighter == 0) && (TalkedToFreighter == 1) && (AllOrionsDestroyed == false))
									{
										fMission.mDisplayMessage(kFreighterHailTalkedToCombat_msg, kMissionScriptMessage);
									}
									else if ((AttackedFreighter == 0) && (TalkedToFreighter == 1))
									{
										fMission.mDisplayMessage(kGoodJobWithOrions_msg, kMissionScriptMessage);
									}
								}							   

								if ((AttackedFreighter == 0) && (TalkedToFreighter == 0))
								{
									fMission.mDisplayMessage(kFreighterHail_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFreighterHail_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFreighterHail_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFreighterHail_4_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFreighterHail_5_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFreighterHail_6_msg, kMissionScriptMessage);
									TalkedToFreighter = 1;
									TestObjectives (*this);
								}
								else if (AttackedFreighter == 1)
								{
									fMission.mDisplayMessage(kFreighterHailAttacked_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionTeam:
								if (AtWarWithOrion == 0)
								{
									fMission.mDisplayMessage(kOrionHailNic_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionHail_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								fMission.mDisplayMessage(kFreighterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionTeam:
								if (Ship->mGetDamagePercent() < kDamageThreshold)
								{
									fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
									Ship->mDisengage (kByMapEdge, kAbsolutePriority);
								}
								else
								{
									fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								fMission.mDisplayMessage(kFreighterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOrionTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (PlanetScanned * 35) + (TalkedToFreighter * 35) + (OrionDestroyed * 10) - (AttackedFreighter * 40);

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints >= 50)
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints >= 35)
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		fMission.mAddToDebriefingList (VictoryLevel + kRedClawAstoundingVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kIronFangAstoundingVictory_msg);
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( PlanetScanned ) ? fMission.mAddToDebriefingList( kPlanetScannedTrue_msg ) : SuccessCount++;
	( TalkedToFreighter ) ? fMission.mAddToDebriefingList( kTalkedToFreighterTrue_msg ) : SuccessCount++;
	( AllOrionsDestroyed ) ? fMission.mAddToDebriefingList( kAllOrionsDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( PlanetScanned == 0 ) ? fMission.mAddToDebriefingList( kPlanetScannedFalse_msg ) : FailureCount++;
	( TalkedToFreighter == 0 ) ? fMission.mAddToDebriefingList( kTalkedToFreighterFalse_msg ) : FailureCount++;
	( AllOrionsDestroyed == false ) ? fMission.mAddToDebriefingList( kAllOrionsDestroyedFalse_msg ) : FailureCount++;
	(AttackedFreighter) ? fMission.mAddToDebriefingList( kFreighterAttacked_msg ) : FailureCount++;
	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);
	if (OrionTeam->mGetShipsInPlayCount() == 0)
	{
		AllOrionsDestroyed = true;
	}

	if ((PlanetScanned) && (TalkedToFreighter) && (AllOrionsDestroyed))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}
	else if (!TalkedToFreighter)
	{
		tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
		tScript.fMission.mSetGoalIndicator (FreighterTeam->mGetFirstShip());
	}
	else if (!MoonScanned)
	{
		tScript.fMission.mSetGoalIndicator (kTrue, MoonX, MoonY);
	}
	else if (!PlanetScanned)
	{
		tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam);
		tScript.fMission.mSetGoalIndicator (PlanetTeam->mGetFirstShip());
	}
	else
	{
		tScript.fMission.mSetGoalIndicator (kFalse, 0, 0);
	}
}

