#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_2MissionText.h"
#include "Lyr_2MissionMaps.h"
/***************************************
LNF need Lyran listening post?
***************************************/

// sequence of events RED CLAW:
// 1. Hail cousin ship
//    (this will cause enemy AI to swarm to cousin)
// 2. player scans the planet
// 3. player scans the planet
// sequence of events IRON FANG:
// 1. Hail cousin ship
//    (this will cause enemy AI to swarm to cousin)
// 2. player scans the planet
// 3. player scans the planet
// 4. fly to cousin and lead to edge of map
//
// player objectives RED CLAW:
// 1. scan the station
// 2. scan the planet
// player objectives IRON FANG:
// 1. scan the station
// 2. scan the planet
// 3. rescue cousin ship


int32 AtWarWithOpposition = 0;
int32 PlatformScanned = 0;
int32 PlanetScanned = 0;
int32 DistressOrTreasonEventOccured = 0;
int32 LPDestroyed = 0;
bool  AllLPsDestroyed = false;
int32 PlanetaryDDestroyed = 0;
bool  AllPlanetaryDsDestroyed = false;
int32 CousinDestroyed = 0;
bool  AllCousinsDestroyed = false;
int32 DefenseDestroyed = 0;
bool  AllDefensesDestroyed = false;
int32 CousinRescued = 0;
int32 CousinDisabled = 0;
int32 OppositionDestroyed = 0;
bool  AllOppositionsDestroyed = false;
int32 Elite = 0;
int32 PlayLevel = 0;
int32 Go = 0;
const float DeepScanDistance = 30.0;
const float CousinDisableDamagePercent = 50.0;
const float PatrolAttackRadius = 15.0;

eClassTypes PlayerShipClass= kClassShuttle;

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);


void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kLPTeam, 
	kPlanetaryDTeam,
	kCousinTeam,
	kOppositionTeam,
	kPlanetTeam,
	kDefenseTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kLyran;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kLPTeam, kLyran);
	fMission.mSetTeamRace (kPlanetaryDTeam, kLyran);
	fMission.mSetTeamRace (kCousinTeam, kLyran);
	fMission.mSetTeamRace (kOppositionTeam, kLyran);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);
	fMission.mSetTeamRace (kDefenseTeam, kLyran);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	Elite = fMission.mGetEliteOrganization();
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LPTeam = fMission.mGetTeamHandle (kLPTeam);
	tTeamInfo* PlanetaryDTeam = fMission.mGetTeamHandle (kPlanetaryDTeam);
	tTeamInfo* CousinTeam = fMission.mGetTeamHandle (kCousinTeam);
	tTeamInfo* OppositionTeam = fMission.mGetTeamHandle (kOppositionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Planetary Defense team
	//PlanetaryDTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	PlanetaryDTeam->mCreateShip ("L-BS", kStartPosition_Z);

	// create Listening Post team
	LPTeam->mCreateShip ("L-DEF", kStartPosition_Y);

	// create Cousin Team
	CousinTeam->mCreateShip("L-CLP", kStartPosition_J);

	// create Opposition Team
	// LNF make this 'smarter'
	OppositionTeam->mCreateFleet(2, HighBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_T);

	// create Defense team - defends planet
	DefenseTeam->mCreateFleet(2, HighBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_V);

	// create Planet
	PlanetTeam->mCreateShip ("PLPh5", kStartPosition_P);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LPTeam = fMission.mGetTeamHandle (kLPTeam);
	tTeamInfo* PlanetaryDTeam = fMission.mGetTeamHandle (kPlanetaryDTeam);
	tTeamInfo* CousinTeam = fMission.mGetTeamHandle (kCousinTeam);
	tTeamInfo* OppositionTeam = fMission.mGetTeamHandle (kOppositionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}

	// configure PlanetaryDefense team
	for (Ship = PlanetaryDTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetaryDTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageMaxWeapons, 100);
	}

	// configure ListeningPost Team
	for (Ship = LPTeam->mGetFirstShip(); Ship != NULL; Ship = LPTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetShipAIControlled (kFalse);
	}

	// configure Cousin Team
	for (Ship = CousinTeam->mGetFirstShip(); Ship != NULL; Ship = CousinTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonReady_text);
		Ship->mSetShipAIControlled (kFalse);
	}
	if ( Elite == kRedClaw )
	{
		CousinTeam->mGetFirstShip()->mChangeName( kGarvarn_name );
	}
	else
	{
		CousinTeam->mGetFirstShip()->mChangeName( kAvar_name );
	}

	// configure Opposition Team
	int32 ShipNumber = 0;
	for (Ship = OppositionTeam->mGetFirstShip(); Ship != NULL; Ship = OppositionTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		PatrolRoute(Ship, ShipNumber);
		ShipNumber++;
	}

	// configure Defense team
	// LNF should I make these defend the planet instead of patrol?
	for (Ship = DefenseTeam->mGetFirstShip(); Ship != NULL; Ship = DefenseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		PatrolRoute(Ship, ShipNumber);
		ShipNumber++;
	}
	
	// cloak the planet
	Ship = PlanetTeam->mGetFirstShip();
	Ship->mChangeAIOfficerState( kCloakOfficer, kFalse );
	Ship->mCloak( kTrue );

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord from within DeepScanDistance for 1 turn
	DeepScanTracker.mSetTarget (PlanetTeam->mGetFirstShip(), DeepScanDistance, 3);
	DeepScanTracker.mSetTarget (PlanetaryDTeam->mGetFirstShip(), DeepScanDistance, 2);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point to cousins ship
	fMission.mSetGoalIndicator (CousinTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LPTeam = fMission.mGetTeamHandle (kLPTeam);
	tTeamInfo* PlanetaryDTeam = fMission.mGetTeamHandle (kPlanetaryDTeam);
	tTeamInfo* CousinTeam = fMission.mGetTeamHandle (kCousinTeam);
	tTeamInfo* OppositionTeam = fMission.mGetTeamHandle (kOppositionTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);

	switch (Event)
	{
		case kNotifyDamage:
			if (( Ship->mGetTeam() == kCousinTeam ) && ( Ship->mGetDamagePercent() >= CousinDisableDamagePercent ))
			{
				fMission.mDisplayMessage(kEvent1IronFang_msg, kMissionScriptMessage);
				CousinTeam->mGetFirstShip()->mDamageSystem( kDamageLeftWarpEngine, 100 );
				CousinTeam->mGetFirstShip()->mDamageSystem( kDamageRightWarpEngine, 100 );
				fMission.mSetTeamRelations (kCousinTeam, kLPTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				fMission.mSetTeamRelations (kCousinTeam, kOppositionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				fMission.mSetTeamRelations (kOppositionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				CousinDisabled = 1;
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerLimitSet_3: // after TREASON event...wait a few seconds
					fMission.mSetTeamRelations (kPlayerTeam, kLPTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetaryDTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOppositionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kEnteringRange_ScanOff:
								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								if ( Object != NULL )
								{
									if ( Object == PlanetaryDTeam->mGetFirstShip() )
									{
										fMission.mDisplayMessage (kStartingScanStation_msg, kScienceScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
									}
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOn_TimerStarted:
								if ( Object != NULL )
								{
									if ( Object == PlanetaryDTeam->mGetFirstShip() )
									{
										fMission.mDisplayMessage (kStartingScanStation_msg, kScienceScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage (kStartingScanPlanet_msg, kScienceScriptMessage);
									}
								}
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if ( Object == PlanetTeam->mGetFirstShip() )
							{
								fMission.mDisplayMessage(kEvent2ScanPlanet_msg, kScienceScriptMessage);
								PlanetScanned = 1;
								TestObjectives (*this);
								if (!PlatformScanned)
								{
									fMission.mDisplayMessage(kEventScanStation_msg, kScienceScriptMessage);
								}
							}
							else if ( Object == PlanetaryDTeam->mGetFirstShip() )
							{
								fMission.mDisplayMessage(kEvent3ScanPlatform_msg, kScienceScriptMessage);
								PlatformScanned = 1;
								TestObjectives (*this);
								if (!PlanetScanned)
								{
									fMission.mDisplayMessage(kEventScanPlanet_msg, kScienceScriptMessage);
								}
							}

							// (Object points to entity or if NULL then MapPoint has location)
							if (Object)
							{
								DeepScanTracker.mRemoveTarget (Object);
							}
							else
							{
								DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
							}
						}
					}
					break;

				case kTimerTurnsSet: // TREASON event
					if (Elite == kRedClaw)
					{
						fMission.mDisplayMessage(kEvent1RedClaw_1_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kEvent1RedClaw_2_msg, kMissionScriptMessage);

						// wait a few seconds before changing relations
						fMission.mSetTimedInterval (kTimerLimitSet_3, 4);
					}
					else
					{
						if (CousinDisabled == 0)
						{
							fMission.mDisplayMessage(kEvent1IronFang_msg, kMissionScriptMessage);
							CousinTeam->mGetFirstShip()->mDamageSystem( kDamageLeftWarpEngine, 100 );
							CousinTeam->mGetFirstShip()->mDamageSystem( kDamageRightWarpEngine, 100 );
							fMission.mSetTeamRelations (kCousinTeam, kLPTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
							fMission.mSetTeamRelations (kCousinTeam, kOppositionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
							fMission.mSetTeamRelations (kOppositionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							CousinDisabled = 1;
						}
						// LNF need to add some action here
						// dunno what yet.....
					}
					DistressOrTreasonEventOccured = 1;
					TestObjectives (*this);
					break;

				case kTimerTurnsSet_9:
					// set the cousin's ship's comm buttons back to HAIL
					for (Ship = CousinTeam->mGetFirstShip(); Ship != NULL; Ship = CousinTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Opposition's ship's comm buttons back to HAIL
					for (Ship = OppositionTeam->mGetFirstShip(); Ship != NULL; Ship = OppositionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kLPTeam)
			{
				LPDestroyed++;
				if (LPTeam->mGetShipsInPlayCount() == 0)
				{
					AllLPsDestroyed = true;
				}
			}

			if (Ship->mGetTeam() == kPlanetaryDTeam)
			{
				PlanetaryDDestroyed++;
				if (PlanetaryDTeam->mGetShipsInPlayCount() == 0)
				{
					AllPlanetaryDsDestroyed = true;
				}
			}

			if (Ship->mGetTeam() == kCousinTeam)
			{
				CousinDestroyed++;
				if (CousinTeam->mGetShipsInPlayCount() == 0)
				{
					AllCousinsDestroyed = true;
					TestObjectives (*this);
				}
			}

			if (Ship->mGetTeam() == kOppositionTeam)
			{
				OppositionDestroyed++;
				if (OppositionTeam->mGetShipsInPlayCount() == 0)
				{
					AllOppositionsDestroyed = true;
				}
			}

			if (Ship->mGetTeam() == kDefenseTeam)
			{
				DefenseDestroyed++;
				if (DefenseTeam->mGetShipsInPlayCount() == 0)
				{
					AllDefensesDestroyed = true;
				}
			}

			break;

		case kNotifyDisengaged:
			if (Ship->mGetTeam() == kCousinTeam)
			{
				CousinRescued = 1;
				TestObjectives (*this);
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonReady_text:
						fMission.mDisplayMessage(kCousinHailNotReady_msg, kMissionScriptMessage);
						Ship->mSetCommButton(1, kButtonGo_text);
						break;

					case kButtonGo_text:
						Ship->mSetShipAIControlled (kTrue);
						fMission.mDisplayMessage(kCousinHailReady_1_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kCousinHailReady_2_msg, kMissionScriptMessage);
						Go = 1;
						if (Elite == kRedClaw)
						{
							// Set a timer for the TREASON event
							fMission.mSetTimedInterval (kTimerTurnsSet, fMission.mRandomInt32(1, 3));
						}
						else
						{
							// make the Cousin ship attack the ListeningPost
							fMission.mSetTeamRelations (kCousinTeam, kLPTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kLPTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kCousinTeam, kOppositionTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kOppositionTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0);
							// Set a timer...if the cousin isn't disabled by then, disable in scripts
							// LNF maybe make the cousin get tractored by the enemy?
							fMission.mSetTimedInterval (kTimerTurnsSet, fMission.mRandomInt32(3, 5));
						}
						Ship->mSetCommButton(1, kButtonHail_text);
						TestObjectives (*this);
						break;

					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kCousinTeam:
								if (DistressOrTreasonEventOccured == 0)
								{
									fMission.mDisplayMessage(kCousinHailF_msg, kMissionScriptMessage);
								}
								else
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kCousinRedClaw_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kCousinHailIronFang_msg, kMissionScriptMessage);
									}
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOppositionTeam:
								fMission.mDisplayMessage(kOppositionHail_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kCousinTeam:
								if (DistressOrTreasonEventOccured == 0)
								{
									fMission.mDisplayMessage(kCousinSurrenderF_msg, kMissionScriptMessage);
								}
								else
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kCousinRedClaw_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kCousinSurrenderIronFang_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOppositionTeam:
								fMission.mDisplayMessage(kOppositionSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kCousinTeam:
								if (DistressOrTreasonEventOccured == 0)
								{
									fMission.mDisplayMessage(kCousinSurrenderF_msg, kMissionScriptMessage);
								}
								else
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kCousinRedClaw_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kCousinTauntIronFang_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOppositionTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kOppositionTaunt_1_msg, kOppositionTaunt_4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (PlanetScanned * 35) + (PlatformScanned * 35);

	if ((Elite == kRedClaw) && (Go == 1))
	{
		VictoryPoints += (CousinDestroyed * 30);
	}
	else if ((Elite == kIronFang) && (Go == 1))
	{
		// LNF need to put something about Cousin disengaging?
		VictoryPoints += (CousinRescued * 30);
	}

	if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 50)
	{
		MyVictoryLevel = kDraw;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		fMission.mAddToDebriefingList (VictoryLevel + kRedClawAstoundingVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kIronFangAstoundingVictory_msg);
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( PlanetScanned ) ? fMission.mAddToDebriefingList( kPlanetScannedTrue_msg ) : SuccessCount++;
	( PlatformScanned ) ? fMission.mAddToDebriefingList( kPlatformScannedTrue_msg ) : SuccessCount++;
	if (Go == 1)
	{
		if (Elite == kRedClaw)
		{
			( AllCousinsDestroyed ) ? fMission.mAddToDebriefingList( kAllCousinsDestroyed_msg ) : SuccessCount++;
		}
		else
		{
			( CousinRescued ) ? fMission.mAddToDebriefingList( kCousinDisengaged_msg ) : SuccessCount++;
		}
	}
	else
	{
		SuccessCount++;
	}
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( !PlanetScanned ) ? fMission.mAddToDebriefingList( kPlanetScannedFalse_msg ) : FailureCount++;
	( !PlatformScanned ) ? fMission.mAddToDebriefingList( kPlatformScannedFalse_msg ) : FailureCount++;
	if (Go == 1)
	{
		if (Elite == kRedClaw)
		{
			( !AllCousinsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllCousinsDestroyed_msg ) : FailureCount++;
		}
		else
		{
			( !CousinRescued ) ? fMission.mAddToDebriefingList( kCousinNotDisengaged_msg ) : FailureCount++;
		}
	}
	else
	{
		FailureCount++;
	}
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
		
	tMapPoint waypoints0[7]= // ship at T
	{
		10, 9,
		10, 12,
		13, 15,
		17, 14,
		19, 11,
		17, 7,
		13, 6
	};

	tMapPoint waypoints1[7]= // ship at U
	{
		23, 9,
		20, 6,
		15, 5,
		10, 9,
		10, 13,
		13, 16,
		20, 15
	};

	tMapPoint waypoints2[4]= // ship at V
	{
		26, 12,
		21, 17,
		14, 19,
		17, 13
	};

	tMapPoint waypoints3[5]= // ship at W
	{
		34, 18,
		36, 20,
		35, 24,
		31, 24,
		30, 20
	};

	tMapPoint PatrolPoints[4][10];

	switch (ShipNumber)
	{								
		case 0:							// opposition team, ship 1
			PatrolPoints[0][0] = waypoints0[0];
			PatrolPoints[0][1] = waypoints0[1];
			PatrolPoints[0][2] = waypoints0[2];
			PatrolPoints[0][3] = waypoints0[3];
			PatrolPoints[0][4] = waypoints0[4];
			PatrolPoints[0][5] = waypoints0[5];
			PatrolPoints[0][6] = waypoints0[6];
			Ship->mPatrol (PatrolAttackRadius, 7, &PatrolPoints[0][0], kHighPriority);
			break;

		case 1:							// opposition team, ship 2
			PatrolPoints[1][0] = waypoints1[0];
			PatrolPoints[1][1] = waypoints1[1];
			PatrolPoints[1][2] = waypoints1[2];
			PatrolPoints[1][3] = waypoints1[3];
			PatrolPoints[1][4] = waypoints1[4];
			PatrolPoints[1][5] = waypoints1[5];
			PatrolPoints[1][6] = waypoints1[6];
			Ship->mPatrol (PatrolAttackRadius, 7, &PatrolPoints[1][0], kHighPriority);
			break;

		case 2:							// defensive team, ship 1
			PatrolPoints[2][0] = waypoints2[0];
			PatrolPoints[2][1] = waypoints2[1];
			PatrolPoints[2][2] = waypoints2[2];
			PatrolPoints[2][3] = waypoints2[3];
			Ship->mPatrol (PatrolAttackRadius, 4, &PatrolPoints[2][0], kHighPriority);
			break;

		case 3:							// defensive team, ship 2
			PatrolPoints[3][0] = waypoints3[0];
			PatrolPoints[3][1] = waypoints3[1];
			PatrolPoints[3][2] = waypoints3[2];
			PatrolPoints[3][3] = waypoints3[3];
			PatrolPoints[3][4] = waypoints3[4];
			Ship->mPatrol (PatrolAttackRadius, 5, &PatrolPoints[3][0], kHighPriority);
			break;
		}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	// LNF this is going to depend on the Elite faction, no?
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kLPTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetaryDTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kCousinTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kOppositionTeam, kTeamHostile, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kDefenseTeam, kTeamHostile, kGoalDestroy, 1.0);//

	tScript.fMission.mSetTeamRelations (kLPTeam, kPlanetaryDTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kLPTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kLPTeam, kOppositionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kLPTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kLPTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kPlanetaryDTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (kPlanetaryDTeam, kOppositionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlanetaryDTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlanetaryDTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kCousinTeam, kPlanetaryDTeam, kTeamHostile, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (kCousinTeam, kOppositionTeam, kTeamHostile, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (kCousinTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kCousinTeam, kDefenseTeam, kTeamHostile, kGoalIgnore, 1.0);

	tScript.fMission.mSetTeamRelations (kOppositionTeam, tScript.kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (kOppositionTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (kOppositionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOppositionTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kPlanetTeam, kDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kDefenseTeam, tScript.kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//
	tScript.fMission.mSetTeamRelations (kDefenseTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0);//
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	// the win condition
	if (Elite == kRedClaw)
	{
		if ((PlanetScanned) && (PlatformScanned) && (AllCousinsDestroyed))
		{
			int32 MapEdgeX, MapEdgeY;
			tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

			// primary objectives complete
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
			FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
		}
	}
	else
	{
		// LNF should the cousin rescue be a primary goal?
		if ((PlanetScanned) && (PlatformScanned) && (CousinRescued))
		{
			if (CousinRescued == kFalse)
			{
				int32 MapEdgeX, MapEdgeY;
				tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

				// primary objectives complete
				tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
				FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
			}
		}
	}

	if (!PlatformScanned)
	{
		tTeamInfo* PlanetaryDTeam = tScript.fMission.mGetTeamHandle (kPlanetaryDTeam);
		tScript.fMission.mSetGoalIndicator (PlanetaryDTeam->mGetFirstShip());
	}
	else if (!PlanetScanned)
	{
		tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam);
		tScript.fMission.mSetGoalIndicator (PlanetTeam->mGetFirstShip());
	}
	else if ((Elite == kRedClaw && !AllCousinsDestroyed)
			 || (Elite == kIronFang && !CousinRescued))
	{
		tTeamInfo* CousinTeam = tScript.fMission.mGetTeamHandle (kCousinTeam);
		tScript.fMission.mSetGoalIndicator (CousinTeam->mGetFirstShip());
	}
}

