#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_3MissionText.h"
#include "Lyr_3MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayLevel = 0;
int32 OrionDestroyed = 0;
bool  AllOrionsDestroyed = false;
int32 FreighterDestroyed = 0;
bool  FreighterDisengaged = false;
int32 LPDestroyed = 0;
int32 KlingonDestroyed = 0;
bool  AllKlingonsDestroyed = false;
const float DamageThreshold = 30.0;
int32 RendezvousWithFreighter = 0;
bool  FreighterFollowing = false;
int32 AtWarWithKlingon = 0;

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);
const float PatrolAttackRadius = 1.0;
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kOrionTeam, 
	kFreighterTeam,
	kLPTeam,
	kKlingonTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kOrionTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kFreighterTeam, kLyran);
	fMission.mSetTeamRace (kLPTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kKlingonTeam, kKlingon);

	// determine if we are at war with the Klingons
	int32 index = 0;
	while (info.fEnemyRaces[index] != kNoRace)
	{
		if (info.fEnemyRaces[index] == kKlingon)
		{
			AtWarWithKlingon = 1;
			break;
		}
		index++;
	}

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	Elite = fMission.mGetEliteOrganization();
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* LPTeam = fMission.mGetTeamHandle (kLPTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .85;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.15;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;
	
	int32 NumberOfEnemyShips = 0;
	int32 NewEnemyBPV = 0;

	if ( NumberOfPlayersShips < 3 )
	{
		NumberOfEnemyShips = 1;
		NewEnemyBPV = AveragePlayerBPV;
	}
	else
	{
		NumberOfEnemyShips = 1;
		NewEnemyBPV = ( AveragePlayerBPV * 2 );
	}

	// create Freighter team
	//FreighterTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
	FreighterTeam->mCreateShip ("L-FLDs", kStartPosition_J);

	// create Orion team
	// LNF need to fix up all teams
	OrionTeam->mCreateFleet ( (NumberOfEnemyShips + 1), TotalBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_V );

	// create Listening Post Team
	LPTeam->mCreateShip("L-LP", kStartPosition_Z);

	// create Klingon Team
	KlingonTeam->mCreateFleet(NumberOfEnemyShips, TotalBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_R);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* LPTeam = fMission.mGetTeamHandle (kLPTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}

	// configure Freighter team
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mSetShipAIControlled (kFalse);
	}

	// configure Orion Team
	int32 ShipNumber = 4; // start at the Orion ships
	for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		PatrolRoute(Ship, ShipNumber);
		ShipNumber++;
	}

	// configure ListeningPost Team
	for (Ship = LPTeam->mGetFirstShip(); Ship != NULL; Ship = LPTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Klingon Team
	ShipNumber = 0; // reset number for Klingons
	for (Ship = KlingonTeam->mGetFirstShip(); Ship != NULL; Ship = KlingonTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		PatrolRoute(Ship, ShipNumber);
		ShipNumber++;
	}
	
	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to check distance from player ship to freighter
	fMission.mSetTimedInterval (kTimerLimitSet_2, 2);

	// point at the listening post
	fMission.mSetGoalIndicator (LPTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* LPTeam = fMission.mGetTeamHandle (kLPTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);

	switch (Event)
	{
		case kNotifyDisengaged:
			if (FreighterFollowing 
				&& (Ship->mGetTeam() == kFreighterTeam
				    || Ship->mGetTeam() == kPlayerTeam))
			{
				FreighterDisengaged = true;
				TestObjectives (*this);
			}
			break;

		case kNotifyDeepScan:
			break;

		case kNotifyDamage:
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerLimitSet_2:
					if (fPlayerShip->mGetTacticalDistance(FreighterTeam->mGetFirstShip()) <= 30.0)
					{
						if (Elite == kRedClaw)
						{
							fMission.mDisplayMessage(kEvent1RedClaw_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage(kEvent1IronFang_msg, kMissionScriptMessage);
						}
						RendezvousWithFreighter = 1;
					}
					else
					{
						fMission.mSetTimedInterval (kTimerLimitSet_2, 2);
					}
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					// set the Freighter comm buttons back to HAIL
					for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Klingon comm buttons back to HAIL
					for (Ship = KlingonTeam->mGetFirstShip(); Ship != NULL; Ship = KlingonTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Orion comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			if (Ship->mGetTeam() == kOrionTeam)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
				// we don't test objectives now - only when ship blows up
				break;
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kOrionTeam)
			{
				OrionDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
			}

			if (Ship->mGetTeam() == kLPTeam)
			{
				LPDestroyed++;
				TestObjectives (*this);
				// let the Orions attack the player
				for (Ship = OrionTeam->mGetFirstShipInPlay(); Ship != NULL; Ship = OrionTeam->mGetNextShipInPlay())
				{
					Ship->mClearCommands();
				}
				// let the Klinongs attack the player if at war
				if (AtWarWithKlingon)
				{
					for (Ship = KlingonTeam->mGetFirstShipInPlay(); Ship != NULL; Ship = KlingonTeam->mGetNextShipInPlay())
					{
						Ship->mClearCommands();
					}
				}
				break;
			}

			if (Ship->mGetTeam() == kKlingonTeam)
			{
				KlingonDestroyed++;
				TestObjectives (*this);
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kOrionTeam:
								if (LPDestroyed == 0)
								{
									fMission.mDisplayMessage(kOrionHailLpOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionHailLpByeBye_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFreighterTeam:
								{
									if (RendezvousWithFreighter == 0)
									{
										fMission.mDisplayMessage(kFreighterHailF_msg, kMissionScriptMessage);
										Ship->mSetCommButton (1, kButtonHail_text);
									}
									else
									{
										if (Ship->mGetDamagePercent() < DamageThreshold)
										{
											fMission.mDisplayMessage (fMission.mRandomInt32 (kFreighterHailTOK_1_msg, kFreighterHailTOK_3_msg), kMissionScriptMessage);
										}
										else
										{
											fMission.mDisplayMessage(kFreighterHailTDamaged_msg, kMissionScriptMessage);
										}
										Ship->mSetShipAIControlled (kTrue);
										Ship->mFollowEntity (fPlayerShip, kHighPriority);
										FreighterFollowing = true;
									}
									if (LPDestroyed && RendezvousWithFreighter)
									{
										fMission.mDisplayMessage(kFreighterFollow_msg, kMissionScriptMessage);
										int32 MapEdgeX, MapEdgeY;
										fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);
										FindClosestMapEdge(*this, fPlayerShip, MapEdgeX, MapEdgeY);
									}
	//								Ship->mSetCommButton (1, kButtonTaunt_text);
	//								Ship->mSetCommButton (2, kButtonSurrender_text);
	//								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
									break;
								}

							case kKlingonTeam:
								if (AtWarWithKlingon == 0)
								{
									fMission.mDisplayMessage(kKlingonHail_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kKlingonHailCombat_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kKlingonHailCombat_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kKlingonHailCombat_3_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								// let the Klingons attack the player
								for (Ship = KlingonTeam->mGetFirstShipInPlay(); Ship != NULL; Ship = KlingonTeam->mGetNextShipInPlay())
								{
									Ship->mClearCommands();
								}
								AtWarWithKlingon = 1;
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kOrionTeam:
								if (Ship->mGetDamagePercent() <= DamageThreshold)
								{
									fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFreighterTeam:
								fMission.mDisplayMessage(kFreighterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kKlingonTeam:
								fMission.mDisplayMessage(kKlingonSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kOrionTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFreighterTeam:
								fMission.mDisplayMessage(kFreighterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kKlingonTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kKlingonTaunt_1_msg, kKlingonTaunt_4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if (FreighterDisengaged && !FreighterDestroyed)
	{
		VictoryPoints = 80;
	}
	VictoryPoints += (OrionDestroyed * 3) + (LPDestroyed * 11) + (KlingonDestroyed * 2);


	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 75)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		fMission.mAddToDebriefingList (VictoryLevel + kRedClawAstoundingVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kIronFangAstoundingVictory_msg);
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 limit = 3;
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( FreighterDisengaged ) ? fMission.mAddToDebriefingList( kFreighterDisengaged_msg ) : SuccessCount++;
	( AllOrionsDestroyed ) ? fMission.mAddToDebriefingList( kAllOrionsDestroyed_msg ) : SuccessCount++;
	( LPDestroyed ) ? fMission.mAddToDebriefingList( kLPDestroyed_msg ) : SuccessCount++;
	if (AtWarWithKlingon)
	{
		( AllKlingonsDestroyed ) ? fMission.mAddToDebriefingList( kAllKlingonsDestroyed_msg ) : SuccessCount++;
		limit++;
	}
	if ( SuccessCount == limit )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	limit = 3;
	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( !FreighterDisengaged ) ? fMission.mAddToDebriefingList( kFreighterNotDisengaged_msg ) : FailureCount++;
	( !AllOrionsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllOrionsDestroyed_msg ) : FailureCount++;
	( !LPDestroyed ) ? fMission.mAddToDebriefingList( kLPNotDestroyed_msg ) : FailureCount++;
	if (AtWarWithKlingon)
	{
		( !AllKlingonsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllKlingonsDestroyed_msg ) : FailureCount++;
		limit++;
	}
	if ( FailureCount == limit )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
		
	tMapPoint waypoints0[6]= // ship at R
	{
		11, 22,
		6, 20,
		6, 16,
		11, 15,
		16, 17,
		16, 19
	};

	tMapPoint waypoints1[7]= // ship at S
	{
		18, 20,
		21, 19,
		23, 17,
		22, 15,
		18, 15,
		14, 17,
		15, 20
	};

	tMapPoint waypoints2[5]= // ship at T
	{
		24, 19,
		26, 16,
		29, 14,
		30, 18,
		26, 23
	};

	tMapPoint waypoints3[10]= // ship at U
	{
		29, 19,
		26, 23,
		21, 23,
		20, 20,
		22, 16,
		26, 16,
		32, 23,
		35, 23,
		36, 19,
		33, 16
	};

	tMapPoint waypoints4[6]= // ship at V
	{
		11, 11,
		9, 14,
		15, 15,
		18, 10,
		14, 7,
		9, 7
	};

	tMapPoint waypoints5[6]= // ship at W
	{
		15, 7,
		19, 8,
		22, 11,
		18, 13,
		13, 12,
		12, 8
	};

	tMapPoint waypoints6[6]= // ship at X
	{
		25, 12,
		21, 11,
		20, 8,
		24, 5,
		29, 7,
		28, 11
	};

	tMapPoint waypoints7[7]= // ship at Y
	{
		32, 7,
		29, 6,
		26, 8,
		24, 11,
		27, 13,
		32, 13,
		35, 11
	};

	tMapPoint PatrolPoints[8][10];

	switch (ShipNumber)
	{
		case 0:
			PatrolPoints[0][0] = waypoints0[0];
			PatrolPoints[0][1] = waypoints0[1];
			PatrolPoints[0][2] = waypoints0[2];
			PatrolPoints[0][3] = waypoints0[3];
			PatrolPoints[0][4] = waypoints0[4];
			PatrolPoints[0][5] = waypoints0[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[0][0], kHighPriority);
			break;

		case 1:
			PatrolPoints[1][0] = waypoints1[0];
			PatrolPoints[1][1] = waypoints1[1];
			PatrolPoints[1][2] = waypoints1[2];
			PatrolPoints[1][3] = waypoints1[3];
			PatrolPoints[1][4] = waypoints1[4];
			PatrolPoints[1][5] = waypoints1[5];
			PatrolPoints[1][6] = waypoints1[6];
			Ship->mPatrol (PatrolAttackRadius, 7, &PatrolPoints[1][0], kHighPriority);
			break;

		case 2:
			PatrolPoints[2][0] = waypoints2[0];
			PatrolPoints[2][1] = waypoints2[1];
			PatrolPoints[2][2] = waypoints2[2];
			PatrolPoints[2][3] = waypoints2[3];
			PatrolPoints[2][4] = waypoints2[4];
			Ship->mPatrol (PatrolAttackRadius, 5, &PatrolPoints[2][0], kHighPriority);
			break;

		case 3:
			PatrolPoints[3][0] = waypoints3[0];
			PatrolPoints[3][1] = waypoints3[1];
			PatrolPoints[3][2] = waypoints3[2];
			PatrolPoints[3][3] = waypoints3[3];
			PatrolPoints[3][4] = waypoints3[4];
			PatrolPoints[3][5] = waypoints3[5];
			PatrolPoints[3][6] = waypoints3[6];
			PatrolPoints[3][7] = waypoints3[7];
			PatrolPoints[3][8] = waypoints3[8];
			PatrolPoints[3][9] = waypoints3[9];
			Ship->mPatrol (PatrolAttackRadius, 10, &PatrolPoints[3][0], kHighPriority);
			break;

		case 4:
			PatrolPoints[4][0] = waypoints4[0];
			PatrolPoints[4][1] = waypoints4[1];
			PatrolPoints[4][2] = waypoints4[2];
			PatrolPoints[4][3] = waypoints4[3];
			PatrolPoints[4][4] = waypoints4[4];
			PatrolPoints[4][5] = waypoints4[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[4][0], kHighPriority);
			break;

		case 5:
			PatrolPoints[5][0] = waypoints5[0];
			PatrolPoints[5][1] = waypoints5[1];
			PatrolPoints[5][2] = waypoints5[2];
			PatrolPoints[5][3] = waypoints5[3];
			PatrolPoints[5][4] = waypoints5[4];
			PatrolPoints[5][5] = waypoints5[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[5][0], kHighPriority);
			break;

		case 6:
			PatrolPoints[6][0] = waypoints6[0];
			PatrolPoints[6][1] = waypoints6[1];
			PatrolPoints[6][2] = waypoints6[2];
			PatrolPoints[6][3] = waypoints6[3];
			PatrolPoints[6][4] = waypoints6[4];
			PatrolPoints[6][5] = waypoints6[5];
			Ship->mPatrol (PatrolAttackRadius, 6, &PatrolPoints[6][0], kHighPriority);
			break;

		case 7:
			PatrolPoints[7][0] = waypoints7[0];
			PatrolPoints[7][1] = waypoints7[1];
			PatrolPoints[7][2] = waypoints7[2];
			PatrolPoints[7][3] = waypoints7[3];
			PatrolPoints[7][4] = waypoints7[4];
			PatrolPoints[7][5] = waypoints7[5];
			PatrolPoints[7][6] = waypoints7[6];
			Ship->mPatrol (PatrolAttackRadius, 7, &PatrolPoints[7][0], kHighPriority);
			break;
		}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kLPTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kOrionTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOrionTeam, kLPTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOrionTeam, kKlingonTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kFreighterTeam, kLPTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kFreighterTeam, kKlingonTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kLPTeam, kKlingonTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	tTeamInfo* KlingonTeam = tScript.fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);

	if (KlingonTeam->mGetShipsInPlayCount() == 0)
	{
		AllKlingonsDestroyed = true;
	}
	if (OrionTeam->mGetShipsInPlayCount() == 0)
	{
		AllOrionsDestroyed = true;
	}

	if (FreighterFollowing && AllOrionsDestroyed && LPDestroyed && (!AtWarWithKlingon || AllKlingonsDestroyed))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}

	if (!LPDestroyed)
	{
		tTeamInfo* LPTeam = tScript.fMission.mGetTeamHandle (kLPTeam);
		tScript.fMission.mSetGoalIndicator (LPTeam->mGetFirstShip());
	}
	else if (!FreighterFollowing && !FreighterDestroyed)
	{
		tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
		tScript.fMission.mSetGoalIndicator (FreighterTeam->mGetFirstShip());
	}
}

