#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_4MissionText.h"
#include "Lyr_4MissionMaps.h"
/*********************************
LNF are there going to be mines to place?
*********************************/

int32 Elite = 0;
int32 PlayLevel = 0;
int32 OrionDestroyed = 0;
bool  AllOrionsDestroyed = false;
int32 OrionBaseDestroyed = 0;
bool  AllOrionBasesDestroyed = false;
int32 LPDestroyed = 0;
int32 FactionDestroyed = 0;
bool  AllFactionsDestroyed = false;
int32 MountainDestroyed = 0;
int32 ProbeDestroyed = 0;
const float DamageThreshold = 30.0;
int32 AtWarWithOrion = 0;
int32 Attacked = 0;
int32 ProbeScanned = 0;
int32 Mode = 0;
int32 DreadnoughtArrived = 0;
const float DeepScanDistance = 30.0;

eClassTypes PlayerShipClass= kClassShuttle;

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);
void PeacefulOrbit(tTeamInfo* PeacefulTeam);

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

enum eMissionMode
{
	kStart = 0,
	kOrions,
	kTaunted
};

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kOrionTeam, 
	kOrionBaseTeam,
	kFactionTeam,
	kMountainTeam,
	kProbeTeam,
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	fMission.mSetTeamRace (kOrionBaseTeam, kOrion);
	fMission.mSetTeamRace (kFactionTeam, kLyran);
	fMission.mSetTeamRace (kMountainTeam, kLyran);
	fMission.mSetTeamRace (kProbeTeam, kLyran);

	// determine if we are at war with the Orions
	int32 index = 0;
	while (info.fEnemyRaces[index] != kNoRace)
	{
		if (info.fEnemyRaces[index] == kOrion)
		{
			AtWarWithOrion = 1;
			break;
		}
		index++;
	}

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	Elite = fMission.mGetEliteOrganization();
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* ProbeTeam = fMission.mGetTeamHandle (kProbeTeam);
	

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .7;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.0;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.4;
			break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	if (Mode == kStart)
	{
		// create OrionBase team
		//OrionBaseTeam->mCreateFleet (NumberOfEnemy(*this, PlayerTeam), (TotalBPV/1.6), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 31, 18);
		OrionBaseTeam->mCreateShip ("O-BS", kStartPosition_Z);

		// create Faction Team
		// LNF make the amount of ships same as player?  +/- 1?
		int32 ships = PlayerTeam->mGetShipsInPlayCount();
		FactionTeam->mCreateFleet(ships, kMaxBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_J);

		// create Probe
		ProbeTeam->mCreateShip("O-LP", kStartPosition_M);
	}
	else if (Mode == kOrions)
	{
		Mode = kOrions;
		fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kOrionBaseTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kFactionTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kOrionTeam, kProbeTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		OrionTeam->mCreateFleet (4, TotalBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 30, 25);
	}
	else if (Mode == kTaunted)
	{
		// create Mountain team
		fMission.mSetTeamRelations (kPlayerTeam, kMountainTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kOrionTeam, kMountainTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kOrionBaseTeam, kMountainTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kFactionTeam, kMountainTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kMountainTeam, kProbeTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		MountainTeam->mCreateShip("L-CAP", kStartPosition_Y);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* ProbeTeam = fMission.mGetTeamHandle (kProbeTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}

	// configure Orion Base team
	for (Ship = OrionBaseTeam->mGetFirstShip(); Ship != NULL; Ship = OrionBaseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Faction Team
	for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
	{
		Ship->mSetFinalDestination (kStartPosition_M);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}
	
	// configure Probe
	Ship = ProbeTeam->mGetFirstShip();
	Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
	Ship->mDamageSystem(kDamageRightWarpEngine, 100);
	Ship->mDamageSystem(kDamageCenterWarpEngine, 100);
	Ship->mChangeName(kLNFProbe_name);	

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan coord from within DeepScanDistance for 1 turn
	DeepScanTracker.mSetTarget (ProbeTeam->mGetFirstShip(), DeepScanDistance, 3);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point at the probe
	fMission.mSetGoalIndicator (ProbeTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (Mode == kOrions)
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}
	else if (Mode == kTaunted)
	{
		// LNF configure Mountain Team
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* ProbeTeam = fMission.mGetTeamHandle (kProbeTeam);

	switch (Event)
	{
		case kNotifyDamage:
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerTurnsSet:
					if (DreadnoughtArrived == 0)
					{
						mCreateMissionShips(kTaunted);
						DreadnoughtArrived = 1;
					}
					break;

				case kTimerTurnsSet_8:
					// set the Mountain comm buttons back to HAIL
					for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Faction comm buttons back to HAIL
					for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_7:
					// set the Orion comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			
				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kInRange_ScanOn_TimerStarted:
							case kEnteringRange_ScanOn_TimerStarted:
								fMission.mDisplayMessage(kEvent1ScanningProbe_1_msg, kMissionScriptMessage);
								fMission.mDisplayMessage(kEvent1ScanningProbe_2_msg, kMissionScriptMessage);
								fMission.mDisplayMessage(kEvent1ScanningProbe_3_msg, kMissionScriptMessage);
								mCreateMissionShips(kOrions);
								fMission.mSetTimedInterval (kTimerTurnsSet, 1); // timer to create MW Dreadnought
								DeepScanTracker.mRemoveTarget (Object);
								ProbeScanned = 1;
								fMission.mSetGoalIndicator (kFalse, 0, 0);
								TestObjectives (*this);
								break;
						}
					}
					break;

			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			if (Ship->mGetTeam() == kOrionTeam)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
				// we don't test objectives now - only when ship blows up
				break;
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kOrionTeam)
			{
				OrionDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kOrionBaseTeam)
			{
				OrionBaseDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kFactionTeam)
			{
				FactionDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kMountainTeam)
			{
				MountainDestroyed++;
			}

			if (Ship->mGetTeam() == kProbeTeam)
			{
				ProbeDestroyed++;
			}

			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kOrionTeam:
								fMission.mDisplayMessage(kOrionHail_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFactionTeam:
								if ((Attacked == 0) && (AtWarWithOrion == 0))
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCHailNic_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFHailNic_msg, kMissionScriptMessage);
									}
								}
								else if ((Attacked == 0) && (Ship->mGetDamagePercent() <= DamageThreshold))
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCHailOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFHailOK_msg, kMissionScriptMessage);
									}
								}
								else if ((Attacked == 0) && (Ship->mGetDamagePercent() > DamageThreshold))
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCHailDamaged_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFHailDamaged_msg, kMissionScriptMessage);
									}
								}
								else if (Attacked == 1)
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCAllAttacked_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFAllAttacked_msg, kMissionScriptMessage);
									}
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMountainTeam:
								if (Elite == kRedClaw)
								{
									fMission.mDisplayMessage(kMWHailRC_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kMWHailIF_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kOrionTeam:
								if (Ship->mGetDamagePercent() < DamageThreshold)
								{
									fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFactionTeam:
								if (Attacked == 0)
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCSurrender_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFSurrender_msg, kMissionScriptMessage);
									}
								}
								else
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCAllAttacked_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFAllAttacked_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMountainTeam:
								fMission.mDisplayMessage(kMWSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kOrionTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFactionTeam:
								if (Attacked == 0)
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage (fMission.mRandomInt32 (kRCTaunt_1_msg, kRCTaunt_3_msg), kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage (fMission.mRandomInt32 (kIFTaunt_1_msg, kIFTaunt_3_msg), kMissionScriptMessage);
									}
								}
								else
								{
									if (Elite == kRedClaw)
									{
										fMission.mDisplayMessage(kRCAllAttacked_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kIFAllAttacked_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMountainTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kMWTaunt_1_msg, kMWTaunt_3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (OrionBaseDestroyed * 25) + (OrionDestroyed * 5) + (ProbeScanned * 75)
					- (FactionDestroyed * 20);


	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 50)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		fMission.mAddToDebriefingList (VictoryLevel + kRedClawAstoundingVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kIronFangAstoundingVictory_msg);
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( ProbeScanned ) ? fMission.mAddToDebriefingList( kProbeScanned_msg ) : SuccessCount++;
	( AllOrionsDestroyed ) ? fMission.mAddToDebriefingList( kAllOrionsDestroyed_msg ) : SuccessCount++;
	( AllOrionBasesDestroyed ) ? fMission.mAddToDebriefingList( kAllOrionBasesDestroyed_msg ) : SuccessCount++;
	if ( SuccessCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( !ProbeScanned ) ? fMission.mAddToDebriefingList( kNoProbeScanned_msg ) : FailureCount++;
	( !AllOrionsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllOrionsDestroyed_msg ) : FailureCount++;
	( !AllOrionBasesDestroyed ) ? fMission.mAddToDebriefingList( kNotAllOrionBasesDestroyed_msg ) : FailureCount++;
	( AllFactionsDestroyed ) ? fMission.mAddToDebriefingList( kAllFactionsDestroyed_msg ) : FailureCount++;
	if ( FailureCount == 5 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// make the peaceful team patrol the planet
void PeacefulOrbit(tTeamInfo* PeacefulTeam)
{
	tShipInfo* Ship;

	tMapPoint waypoints[6]=
	{
		17, 15,
		18, 17,
		22, 17,
		23, 15,
		22, 13,
		19, 13,
	};

	tMapPoint PatrolPoints[6];

	for (Ship = PeacefulTeam->mGetFirstShip(); Ship != NULL; Ship = PeacefulTeam->mGetNextShip())
	{
		PatrolPoints[0] = waypoints[0];
		PatrolPoints[1] = waypoints[1];
		PatrolPoints[2] = waypoints[2];
		PatrolPoints[3] = waypoints[3];
		PatrolPoints[4] = waypoints[4];
		PatrolPoints[5] = waypoints[5];
		Ship->mPatrol (1.0, 6, &PatrolPoints[0], kHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kOrionBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kProbeTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kOrionTeam, kOrionBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOrionTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOrionTeam, kProbeTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kOrionBaseTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kOrionBaseTeam, kProbeTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kFactionTeam, kProbeTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	tTeamInfo* OrionTeam = tScript.fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionBaseTeam = tScript.fMission.mGetTeamHandle (kOrionBaseTeam);
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* ProbeTeam = tScript.fMission.mGetTeamHandle (kProbeTeam);

	if (FactionTeam->mGetShipsInPlayCount() == 0)
	{
		AllFactionsDestroyed = true;
	}
	if (OrionTeam->mGetShipsInPlayCount() == 0)
	{
		AllOrionsDestroyed = true;
	}
	if (OrionBaseTeam->mGetShipsInPlayCount() == 0)
	{
		AllOrionBasesDestroyed = true;
	}
	if ((ProbeScanned) && (AllOrionsDestroyed) && (AllOrionBasesDestroyed))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}

	if (!ProbeScanned)
	{
		tScript.fMission.mSetGoalIndicator (ProbeTeam->mGetFirstShip());
	}
	else if (!AllOrionBasesDestroyed)
	{
		tScript.fMission.mSetGoalIndicator (OrionBaseTeam->mGetFirstShip());
	}
}
