#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_5MissionText.h"
#include "Lyr_5MissionMaps.h"
/*************************************
LNF need Lyran listening post
*************************************/

int32 Elite = 0;
int32 PlayLevel = 0;
int32 MinesDestroyed = 0;
bool  AllMinesDestroyed = false;
int32 MinesDestroyedByPlayer = 0;
int32 MinesDestroyedByCousin = 0;
int32 IntruderDestroyed = 0;
bool  AllIntrudersDestroyed = false;
int32 CousinDestroyed = 0;
int32 IntruderArrived = 0;
int32 AttackedCousin = 0;
int32 CousinSurrendered = 0;
const float DamageThreshold = 60.0;
bool  FirstShip = true;
int32 PointingToMineIndex = -1;

#define kTotalMines 11
int32 LastTeamToAttack[kTotalMines][3];
tMapPoint MinePositions[kTotalMines]=
{
	8,	9,
	19,	5,
	17, 11,
	29, 14,
	20, 17,
	35, 17,
	23, 19,
	32, 26,
	12, 31,
	21, 30,
	28, 35,
};

eClassTypes PlayerShipClass= kClassShuttle;

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
int32 GetMineIndex (int32 ShipID);
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);
void PointToNearestMine (tScript& tScript);

enum eMissionMode
{
	kStart = 0,
	kIntruders
};

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMineTeam, 
	kIntruderTeam,
	kCousinTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMineTeam, kLyran);
	// the race of the Intruder team depends on the Faction of the player
	Elite = fMission.mGetEliteOrganization();
	if (Elite == kRedClaw)
	{
		fMission.mSetTeamRace (kIntruderTeam, kLyran);
	}
	else
	{
		fMission.mSetTeamRace (kIntruderTeam, kOrion);
	}
	fMission.mSetTeamRace (kCousinTeam, kLyran);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// just use the players first ship
	if (FirstShip)
	{
		FirstShip = kFalse;
		return kTrue;
	}
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MineTeam = fMission.mGetTeamHandle (kMineTeam);
	tTeamInfo* IntruderTeam = fMission.mGetTeamHandle (kIntruderTeam);
	tTeamInfo* CousinTeam = fMission.mGetTeamHandle (kCousinTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .7;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.0;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.4;
			break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	

	if (Mode == kStart)
	{
		// create Listening Post team
		for (int32 i = 0; i < kTotalMines; i++)
		{
			MineTeam->mCreateShip ("L-LP", kStartPosition_Specified, MinePositions[i].x, MinePositions[i].y);
		}

		// create Cousin Team
		CousinTeam->mCreateShip("L-CLP", kStartPosition_W);
	}
	else if (Mode == kIntruders)
	{
		// create Intruder team
		fMission.mSetTeamRelations (kPlayerTeam, kIntruderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kMineTeam, kIntruderTeam, kTeamAtWar, kGoalDestroy, 1.0);
		fMission.mSetTeamRelations (kIntruderTeam, kMineTeam, kTeamNeutral, kGoalIgnore, 1.0);
		fMission.mSetTeamRelations (kIntruderTeam, kCousinTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		IntruderTeam->mCreateFleet (3, LowBPV, kClassFrigate, kClassFrigate, 50, kStartPosition_X);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MineTeam = fMission.mGetTeamHandle (kMineTeam);
	tTeamInfo* IntruderTeam = fMission.mGetTeamHandle (kIntruderTeam);
	tTeamInfo* CousinTeam = fMission.mGetTeamHandle (kCousinTeam);

	tShipInfo* Ship;

	// configure Player ship
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure Mines
	int32 index = 0;
	for (Ship = MineTeam->mGetFirstShip(); Ship != NULL; Ship = MineTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
		LastTeamToAttack[index][0] = Ship->mGetShipID();
		LastTeamToAttack[index][1] = 0;		// last team to attack
		LastTeamToAttack[index][2] = 0;		// if == 1 - mine destroyed
		index++;
	}

	// configure Cousin Team
	for (Ship = CousinTeam->mGetFirstShip(); Ship != NULL; Ship = CousinTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	PointToNearestMine (*this);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// configure Intruder team
	Ship->mSetCommButton(1, kButtonHail_text);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MineTeam = fMission.mGetTeamHandle (kMineTeam);
	tTeamInfo* IntruderTeam = fMission.mGetTeamHandle (kIntruderTeam);
	tTeamInfo* CousinTeam = fMission.mGetTeamHandle (kCousinTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDeepScan:
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);
				if ( HostileShip != NULL )
				{
					eTeamID hostile_team = HostileShip->mGetTeam();
					if (Ship->mGetTeam() == kMineTeam)
					{
						int32 index = GetMineIndex (Ship->mGetShipID());
						if (index >= 0)
						{
							LastTeamToAttack[index][1] = hostile_team;
						}
					}
					else if ((Ship->mGetTeam() == kCousinTeam) && (hostile_team == kPlayerTeam)
							 && (AttackedCousin == 0))
					{
						AttackedCousin = 1;
						break;
					}
				}
				Ship->mSetShipCallbackEvents (kNotifyDamage);
			}
			break;

		case kNotifyAtDistance:
			// LNF put in some kind of check for the cousin ship
			// if it is within 60.0 of ANY player ship AND outside the LP, don't shoot at it
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerTurnsSet_9:
					// set the cousin's ship's comm buttons back to HAIL
					for (Ship = CousinTeam->mGetFirstShip(); Ship != NULL; Ship = CousinTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Intruder's ship's comm buttons back to HAIL
					for (Ship = IntruderTeam->mGetFirstShip(); Ship != NULL; Ship = IntruderTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			if (Ship->mGetTeam() == kIntruderTeam
				&& Elite == kIronFang)
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kSelfDestruct_msg, kScienceScriptMessage );
				// we don't test objectives now - only when ship blows up
				break;
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kMineTeam)
			{
				MinesDestroyed++;
				// LNF need to know WHO killed the mine
				int32 index = GetMineIndex (Ship->mGetShipID());
				if (index >= 0)
				{
					LastTeamToAttack[index][2] = 1;
					if (LastTeamToAttack[index][1] == kPlayerTeam)
					{
						MinesDestroyedByPlayer++;
						PointToNearestMine (*this);
					}
					else
					{
						MinesDestroyedByCousin++;
						// select a new mine if one player was pointing at has been destroy by the cousin
						if (index == PointingToMineIndex)
						{
							PointToNearestMine (*this);
						}
					}
				}
				if (MinesDestroyed == 4)
				{
					mCreateMissionShips(kIntruders);
					fMission.mDisplayMessage(kEvent1Intruder_1_msg, kScienceScriptMessage);
					fMission.mDisplayMessage(kEvent1Intruder_2_msg, kScienceScriptMessage);
					IntruderArrived = 1;
				}
				else
				{
					TestObjectives (*this);
				}

			}

			if (Ship->mGetTeam() == kIntruderTeam)
			{
				IntruderDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kCousinTeam)
			{
				CousinDestroyed++;
			}

			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kCousinTeam:
								if ((IntruderArrived == 0) && (AttackedCousin == 0))
								{
									fMission.mDisplayMessage(kCousinHail_msg, kMissionScriptMessage);
								}
								else if ((IntruderArrived == 1) && (AttackedCousin == 0))
								{
									fMission.mDisplayMessage(kCousinHailIntruder_msg, kMissionScriptMessage);
								}
								else if ((AttackedCousin == 1) && (CousinSurrendered == 0))
								{
									fMission.mDisplayMessage(kCousinHailAttacked_msg, kMissionScriptMessage);
								}
								else if ((AttackedCousin == 1) && (CousinSurrendered == 1))
								{
									fMission.mDisplayMessage(kCousinHailAll_msg, kMissionScriptMessage);
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kIntruderTeam:
								if (Elite == kRedClaw)
								{
									fMission.mDisplayMessage(kLyranHail_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionHail_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kCousinTeam:
								if (CousinSurrendered == 0)
								{
									if (Ship->mGetDamagePercent() < DamageThreshold)
									{
										fMission.mDisplayMessage(kCousinSurrenderOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kCousinSurrenderDamaged_msg, kMissionScriptMessage);
										CousinSurrendered = 1;
									}
								}
								else
								{
									fMission.mDisplayMessage(kCousinHailAll_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kIntruderTeam:
								if (Elite == kRedClaw)
								{
									fMission.mDisplayMessage(kLyranHail_msg, kMissionScriptMessage);
								}
								else
								{
									if (Ship->mGetDamagePercent() < DamageThreshold)
									{
										fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kCousinTeam:
								if (CousinSurrendered == 0)
								{
									fMission.mDisplayMessage(kCousinTaunt_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kCousinHailAll_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kIntruderTeam:
								if (Elite == kRedClaw)
								{
									fMission.mDisplayMessage(kLyranTaunt_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt_1_msg, kOrionTaunt_4_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

						}
						break;
				
				}
			}
			break;

	}

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() == kDevastatingDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
	if (AllMinesDestroyed || CousinSurrendered)
	{
		if (MinesDestroyedByPlayer == MinesDestroyedByCousin)
		{
			VictoryPoints = 50;
		}
		if (MinesDestroyedByPlayer > MinesDestroyedByCousin || CousinSurrendered)
		{
			VictoryPoints = 80;
		}
	}
	VictoryPoints += (IntruderDestroyed * 10);

	if (VictoryPoints >= 80)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{

	if (VictoryLevel == kAstoundingVictory)
	{
		if (Elite == kRedClaw)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
			fMission.mAddToDebriefingList (kBothVictoryP3_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
			fMission.mAddToDebriefingList (kIronFangVictory_msg);
			fMission.mAddToDebriefingList (kBothVictoryP3_msg);
		}
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( MinesDestroyedByPlayer > MinesDestroyedByCousin ) ? fMission.mAddToDebriefingList( kDefeatedRival_msg ) : SuccessCount++;
	( AllIntrudersDestroyed ) ? fMission.mAddToDebriefingList( kDefeatedIntruders_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( MinesDestroyedByPlayer <= MinesDestroyedByCousin ) ? fMission.mAddToDebriefingList( kNotDefeatedRival_msg ) : FailureCount++;
	( !AllIntrudersDestroyed ) ? fMission.mAddToDebriefingList( kNotDefeatedIntruders_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	if (VictoryLevel != kDevastatingDefeat)
	{
		fMission.mScheduleMission( kNextMission_msg );
		// LNF modify the number of spacer missions?
		int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
		switch (NumberOfSpacerMissions)
		{
			case 0:
				break;

			case 1:
				fMission.mScheduleMission( kRandomMission_msg );
				break;
				
			case 2:
				fMission.mScheduleMission( kRandomMission_msg );
				fMission.mScheduleMission( kRandomMission_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kMineTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kCousinTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMineTeam, kCousinTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	tTeamInfo* MineTeam = tScript.fMission.mGetTeamHandle (kMineTeam);
	tTeamInfo* IntruderTeam = tScript.fMission.mGetTeamHandle (kIntruderTeam);

	if (IntruderTeam->mGetShipsInPlayCount() == 0)
	{
		AllIntrudersDestroyed = true;
	}
	if (MineTeam->mGetShipsInPlayCount() == 0)
	{
		AllMinesDestroyed = true;
	}

	if ((AllIntrudersDestroyed) && (AllMinesDestroyed))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}

	if (!AllMinesDestroyed)
	{
		PointToNearestMine (tScript);
	}
	else if (!AllIntrudersDestroyed)
	{
		tScript.fMission.mSetGoalIndicator (IntruderTeam->mGetFirstShipInPlay());
	}

}

int32 GetMineIndex (int32 ShipID)
{
	int32 i = kTotalMines;
	while (i--)
	{
		if (LastTeamToAttack[i][0] == ShipID)
		{
			return i;
		}
	}
	return -1;
}

void PointToNearestMine (tScript& tScript)
{
	int32 shipx, shipy;
	int32 dist = 9999999;
	int32 found = -1;
	int32 i = kTotalMines;

	tScript.fPlayerShip->mGetGridPosition (&shipx, &shipy);
	while (i--)
	{
		if (LastTeamToAttack[i][2] == 0)
		{
			int32 distx = shipx - MinePositions[i].x;
			int32 disty = shipy - MinePositions[i].y;
			int32 new_dist = (distx * distx) + (disty * disty);
			if (new_dist < dist)
			{
				dist = new_dist;
				found = i;
			}
		}
	}

	PointingToMineIndex = found;
	if (found != -1)
	{
		tScript.fMission.mSetGoalIndicator (kTrue, MinePositions[found].x, MinePositions[found].y);
	}
	else
	{
		TestObjectives (tScript);
	}
}
