#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_6MissionText.h"
#include "Lyr_6MissionMaps.h"


int32 Elite = 0;
int32 PlayLevel = 0;
int32 SatelliteDestroyed = 0;
int32 FreighterDestroyed = 0;
int32 BigLyranDestroyed = 0;
int32 Lyran1Destroyed = 0;
int32 Lyran2Destroyed = 0;
int32 ForcedToTalk = 0;
int32 FreighterTalk = 0;
int32 CoordinatesGive = 0;
int32 IonicTrade = 0;

// LNF is there a way to find the number of satellites and still have a global array?
#define kTotalSatellites 3
int32 SatelliteScannedList[kTotalSatellites] = {0}; // change this if satellites are added or subtracted
int32 SatelliteID[kTotalSatellites] = {0};  // the ship ID of the Satellite ship entity
tMapPoint SatellitePostions[kTotalSatellites]=
{
	16,	10,
	10, 18,
	23, 17,
};

int32 SatelliteIDNum = 0;
int32 SatelliteScanned = 0;

bool AllSatellitesScanned = false;
bool DestroyedIonicProjectorShip = false; // if the ship holding the Ionic Projector is destroyed
bool KnowsAboutProjector = false;

const float DeepScanDistance = 20.0;
const float kDamageThreshold = 50.0; // damage that the freighter takes in Iron Fang before giving up info

eClassTypes PlayerShipClass= kClassShuttle;

void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
void RedClawCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void IronFangCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void LyranCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void PointToNearestSatellite (tScript& tScript);

void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);
void PeacefulOrbit(tTeamInfo* PeacefulTeam);

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kSatelliteTeam, 
	kFreighterTeam,
	kBigLyranTeam,
	kLyran1Team,
	kLyran2Team
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kSatelliteTeam, kLyran);
	fMission.mSetTeamRace (kFreighterTeam, kLyran);
	fMission.mSetTeamRace (kBigLyranTeam, kLyran);
	fMission.mSetTeamRace (kLyran1Team, kLyran);
	fMission.mSetTeamRace (kLyran2Team, kLyran);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	Elite = fMission.mGetEliteOrganization();
	PlayLevel = fMission.mGetDifficultyLevel();

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* SatelliteTeam = fMission.mGetTeamHandle (kSatelliteTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* BigLyranTeam = fMission.mGetTeamHandle (kBigLyranTeam);
	tTeamInfo* Lyran1Team = fMission.mGetTeamHandle (kLyran1Team);
	tTeamInfo* Lyran2Team = fMission.mGetTeamHandle (kLyran2Team);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .7;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.0;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.4;
			break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Satellites
	for (int32 i = 0; i < kTotalSatellites; i++)
	{
		SatelliteTeam->mCreateShip ("L-LP", kStartPosition_Specified, SatellitePostions[i].x, SatellitePostions[i].y);
		SatelliteScannedList[i] = 0;
	}

	// create Freighter
	FreighterTeam->mCreateShip ("L-FSDs", kStartPosition_Z);

	// create Big Lyran ship Team
	BigLyranTeam->mCreateShip(LowBPV, kClassHeavyCruiser, kClassHeavyCruiser, 50, kStartPosition_Y);

	// create Lyran 1 team
	Lyran1Team->mCreateFleet(3, TotalBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_V);

	// create Lyran 2 team - This is the team to get the Ionic projector from.
	Lyran2Team->mCreateFleet(2, LowBPV, kClassFrigate, kClassLightCruiser, 50, kStartPosition_T);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* SatelliteTeam = fMission.mGetTeamHandle (kSatelliteTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* BigLyranTeam = fMission.mGetTeamHandle (kBigLyranTeam);
	tTeamInfo* Lyran1Team = fMission.mGetTeamHandle (kLyran1Team);
	tTeamInfo* Lyran2Team = fMission.mGetTeamHandle (kLyran2Team);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}

	// configure Satellites
	SatelliteIDNum = 0;
	for (Ship = SatelliteTeam->mGetFirstShip(); Ship != NULL; Ship = SatelliteTeam->mGetNextShip())
	{
		//Ship->mSetShipCallbackEvents (kNotifyDamage);
		//Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		//Ship->mDamageSystem(kDamageRightWarpEngine, 100);
		// LNF initialize the satellite array with the ship ID numbers

		SatelliteID[SatelliteIDNum] = Ship->mGetShipID();
		SatelliteIDNum++;
	}

	// configure Freighter
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Big Lyran
	for (Ship = BigLyranTeam->mGetFirstShip(); Ship != NULL; Ship = BigLyranTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Lyran 1 team
	for (Ship = Lyran1Team->mGetFirstShip(); Ship != NULL; Ship = Lyran1Team->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}
	
	// configure Lyran 2 team
	for (Ship = Lyran2Team->mGetFirstShip(); Ship != NULL; Ship = Lyran2Team->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// put the Ionic Projector on the Lyran2Team ship ONLY if the player is Red Claw
	if ( Elite == kRedClaw )
	{
		Lyran2Team->mGetThisShip(1)->mAccessSupplies( kSetTransporterItemCount, kTransIonicProjector, 1 );
	}
	
	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	for (Ship = SatelliteTeam->mGetFirstShip(); Ship != NULL; Ship = SatelliteTeam->mGetNextShip())
	{
		DeepScanTracker.mSetTarget (Ship, DeepScanDistance, 0);
	}

	PointToNearestSatellite (*this);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* SatelliteTeam = fMission.mGetTeamHandle (kSatelliteTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* BigLyranTeam = fMission.mGetTeamHandle (kBigLyranTeam);
	tTeamInfo* Lyran1Team = fMission.mGetTeamHandle (kLyran1Team);
	tTeamInfo* Lyran2Team = fMission.mGetTeamHandle (kLyran2Team);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	int32 ShipNumber = -1;
	int32 SatelliteCount = 0;

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ( fEventItem1 == kTransIonicProjector )
					{
						IonicTrade = 1; // the player has recovered the projector
						fMission.mDisplayMessage( kGotTheProjector_msg, kScienceScriptMessage );
						TransportTarget->mDamageSystem( kDamageMaxShields, 0 );
						TestObjectives (*this);
					}
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( Elite == kRedClaw )
					{
					}
					else // Elite == kIronFang
					{
						// if the Iron Fang ship is heavily damaged by player, print the message and set flags
						if (( HostileShip->mGetTeam() == kPlayerTeam ) && ( Ship->mGetDamagePercent() >= kDamageThreshold ) &&
							( ForcedToTalk == 0 ))
						{
							ForcedToTalk = 1;
							fMission.mDisplayMessage( kFreighterSurrenderDamagedIF_msg, kMissionScriptMessage );
							fMission.mDisplayMessage(kFreighterPrompt_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kFreighterHailIFTalk_2_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kFreighterHailIFTalk_3_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kFreighterHailIFTalk_4_msg, kMissionScriptMessage);

							FreighterTalk = 1;
							TestObjectives (*this);
						}

						// reset the damage callback
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		case kNotifyAtDistance:
			// LNF put in some kind of check for the cousin ship
			// if it is within 60.0 of ANY player ship AND outside the LP, don't shoot at it
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// hack to Set team relations
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerTurnsSet_7:
					// set the Freighter's comm buttons back to HAIL
					for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8:
					// set the Big Lyran comm buttons back to HAIL
					for (Ship = BigLyranTeam->mGetFirstShip(); Ship != NULL; Ship = BigLyranTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Lyran1 comm buttons back to HAIL
					for (Ship = Lyran1Team->mGetFirstShip(); Ship != NULL; Ship = Lyran1Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Lyran2 comm buttons back to HAIL
					for (Ship = Lyran2Team->mGetFirstShip(); Ship != NULL; Ship = Lyran2Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kEnteringRange_ScanOn_TimerStarted:
							case kInRange_ScanOn_TimerStarted:
								if ( Object )
								{
									int32 ShipNumber = Object->mGetShipID();
									int32 SatelliteCount = 0;
									while (ShipNumber != SatelliteID[SatelliteCount])
									{
										SatelliteCount++;
									}
									if (SatelliteScannedList[SatelliteCount] == 0)
									{
										fMission.mDisplayMessage(kEvent1Scanning_msg, kScienceScriptMessage);
										SatelliteScanned++;
										SatelliteScannedList[SatelliteCount] = 1;
										Object->mChangeName( kProjector_name );
										if (SatelliteScanned == kTotalSatellites)
										{
											AllSatellitesScanned = true;
											fMission.mDisplayMessage(kEvent1Done_msg, kScienceScriptMessage);
											TestObjectives (*this);
										}
										else
										{
											PointToNearestSatellite (*this);
										}
									}
									else
									{
										fMission.mDisplayMessage(kEvent1Repeat_msg, kScienceScriptMessage);
									}
								}
								break;
						}
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}

			if ( Ship->mGetTeam() == kSatelliteTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kSatelliteTeam)
			{
				SatelliteDestroyed++;
			}

			if (Ship->mGetOriginalTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
			}

			if (Ship->mGetOriginalTeam() == kBigLyranTeam)
			{
				BigLyranDestroyed++;
				fMission.mSetTeamRelations (kPlayerTeam, kLyran1Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			}

			if (Ship->mGetOriginalTeam() == kLyran1Team)
			{
				Lyran1Destroyed++;
			}

			if (Ship->mGetOriginalTeam() == kLyran2Team)
			{
				Lyran2Destroyed++;

				// if the ship destroyed is the one containing the ionic projector
				if ( Ship == Lyran2Team->mGetFirstShip() )
				{
					DestroyedIonicProjectorShip = true;
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (Ship->mGetTeam())
				{
				case kFreighterTeam:
					if (Elite == kRedClaw)
					{
						RedClawCommMessages(*this, fEventID, Ship);
					}
					else
					{
						IronFangCommMessages(*this, fEventID, Ship);
					}
					break;

				case kBigLyranTeam:
					LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_8);
					break;

				case kLyran1Team:
					LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_9);
					break;

				case kLyran2Team:
					LyranCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
					break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (SatelliteScanned * 25) + (FreighterTalk * 25);


	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		if (VictoryLevel == kAstoundingVictory)
		{
			fMission.mAddToDebriefingList (kAstoundingVictoryP1_msg);
			fMission.mAddToDebriefingList (kRedClawAstoundingVictory_msg);
		}
		else if (VictoryLevel == kVictory)
		{
			fMission.mAddToDebriefingList (kVictoryP1_msg);
			fMission.mAddToDebriefingList (kRedClawVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kRedClawAstoundingVictory_msg);
		}
	}
	else
	{
		if (VictoryLevel == kAstoundingVictory)
		{
			fMission.mAddToDebriefingList (kAstoundingVictoryP1_msg);
			fMission.mAddToDebriefingList (kIronFangAstoundingVictory_msg);
		}
		else if (VictoryLevel == kVictory)
		{
			fMission.mAddToDebriefingList (kVictoryP1_msg);
			fMission.mAddToDebriefingList (kIronFangVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kIronFangAstoundingVictory_msg);
		}
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( AllSatellitesScanned ) ? fMission.mAddToDebriefingList( kAllSatellitesScanned_msg ) : SuccessCount++;
	( FreighterTalk ) ? fMission.mAddToDebriefingList( kFreighterTalk_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( !AllSatellitesScanned ) ? fMission.mAddToDebriefingList( kNotAllSatellitesScanned_msg ) : FailureCount++;
	( !FreighterTalk ) ? fMission.mAddToDebriefingList( kNoFreighterTalk_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// make the peaceful team patrol the planet
void PeacefulOrbit(tTeamInfo* PeacefulTeam)
{
	tShipInfo* Ship;

	tMapPoint waypoints[6]=
	{
		17, 15,
		18, 17,
		22, 17,
		23, 15,
		22, 13,
		19, 13,
	};

	tMapPoint PatrolPoints[6];

	for (Ship = PeacefulTeam->mGetFirstShip(); Ship != NULL; Ship = PeacefulTeam->mGetNextShip())
	{
		PatrolPoints[0] = waypoints[0];
		PatrolPoints[1] = waypoints[1];
		PatrolPoints[2] = waypoints[2];
		PatrolPoints[3] = waypoints[3];
		PatrolPoints[4] = waypoints[4];
		PatrolPoints[5] = waypoints[5];
		Ship->mPatrol (1.0, 6, &PatrolPoints[0], kHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
// Red Claw Comm Messages
void RedClawCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (IonicTrade == 0)
			{
				tScript.fMission.mDisplayMessage(kFreighterHailRC_1_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailRC_2_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailRC_3_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailRC_4_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailRC_5_msg, kMissionScriptMessage);
				Ship->mSetCommButton (1, kButtonHail_text);
				KnowsAboutProjector = true;

				// if the player already destroyed those ships, then print this message...
				if ( DestroyedIonicProjectorShip == true )
				{
					tScript.fMission.mDisplayMessage( kDestroyedIonicProjectorShips_msg, kMissionScriptMessage );
				}
				else
				{
					TestObjectives (tScript);
				}
			}
			else if ((IonicTrade == 1) && (FreighterTalk == 0))
			{
				tScript.fMission.mDisplayMessage(kFreighterHailRCIonic_1_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailIFTalk_2_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailIFTalk_3_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailIFTalk_4_msg, kMissionScriptMessage);
				Ship->mSetCommButton (1, kButtonHail_text);
				FreighterTalk = 1;
				TestObjectives (tScript);
			}
			else if (FreighterTalk == 1)
			{
				tScript.fMission.mDisplayMessage(kFreighterHailRCDone_1_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kFreighterHailRCDone_2_msg, kMissionScriptMessage);

				Ship->mSetCommButton (1, kButtonTaunt_text);
				Ship->mSetCommButton (2, kButtonSurrender_text);
				// If the Taunt or Surrender buttons aren't used in X turns
				// return the buttons to Hail and blank.
				tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			}
			break;

		case kButtonSurrender_text:
			tScript.fMission.mDisplayMessage(kFreighterSurrenderRC_msg, kMissionScriptMessage);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage(kFreighterTauntRC_msg, kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Iron Fang Comm Messages
void IronFangCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonTalk_text:
			tScript.fMission.mDisplayMessage(kFreighterPrompt_msg, kMissionScriptMessage);
			tScript.fMission.mDisplayMessage(kFreighterHailIFTalk_2_msg, kMissionScriptMessage);
			tScript.fMission.mDisplayMessage(kFreighterHailIFTalk_3_msg, kMissionScriptMessage);
			tScript.fMission.mDisplayMessage(kFreighterHailIFTalk_4_msg, kMissionScriptMessage);
			FreighterTalk = 1;
			TestObjectives (tScript);
			Ship->mSetCommButton (1, kButtonHail_text);
			break;

		case kButtonHail_text:
			if (ForcedToTalk == 0)
			{
				tScript.fMission.mDisplayMessage(kFreighterHailIF_msg, kMissionScriptMessage);
			}
			else if (FreighterTalk == 1)
			{
				tScript.fMission.mDisplayMessage(kFreighterHailIFDone_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() <= kDamageThreshold)
			{
				tScript.fMission.mDisplayMessage(kFreighterSurrenderOKIF_msg, kMissionScriptMessage);
			}
			else if ((Ship->mGetDamagePercent() > kDamageThreshold) && (ForcedToTalk == 0))
			{
				tScript.fMission.mDisplayMessage(kFreighterSurrenderDamagedIF_msg, kMissionScriptMessage);
				ForcedToTalk = 1;
				Ship->mClearCommButton (2);
				Ship->mSetCommButton (1, kButtonTalk_text);
				break;
			}
			else
			{
				tScript.fMission.mDisplayMessage(kFreighterSurrenderIFDone_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;

		case kButtonTaunt_text:
			if (ForcedToTalk == 0)
			{
				tScript.fMission.mDisplayMessage(kFreighterTauntIFF_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kFreighterTauntIFT_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Lyran Comm Messages
void LyranCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	tTeamInfo* SatelliteTeam = tScript.fMission.mGetTeamHandle (kSatelliteTeam);
	tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* BigLyranTeam = tScript.fMission.mGetTeamHandle (kBigLyranTeam);
	tTeamInfo* Lyran1Team = tScript.fMission.mGetTeamHandle (kLyran1Team);
	tTeamInfo* Lyran2Team = tScript.fMission.mGetTeamHandle (kLyran2Team);

	switch (fEventID)
	{
		case kButtonProjector_text:
			tScript.fMission.mDisplayMessage( kProjectorRequest_msg, kMissionScriptMessage );

			tScript.fMission.mDisplayMessage( kComplyToProjectorRequest_msg, kMissionScriptMessage );

			Ship->mDamageSystem( kDamageMaxShields, 100 );
			Ship->mSetCommButton (1, kButtonHail_text);
			break;

		case kButtonHail_text:
			if ( Elite == kRedClaw )
			{
				if ( Ship == Lyran2Team->mGetFirstShip() )
				{
					// if the projector has already been traded
					if ( IonicTrade == 0 )
					{
						tScript.fMission.mDisplayMessage(kLyranHail_1_msg, kMissionScriptMessage);
						tScript.fMission.mDisplayMessage(kLyranHail_2_msg, kMissionScriptMessage);
						Ship->mSetCommButton( 1, kButtonProjector_text );
						Ship->mClearCommButton( 2 );
						break;
					}
					else // play this message and disengage ships
					{
						tScript.fMission.mDisplayMessage(kOnOurWay_msg, kMissionScriptMessage);
						Lyran2Team->mDisengageShips( kByMapEdge, kAbsolutePriority );
						break;
					}
				}
				else if ( Ship == Lyran2Team->mGetThisShip( 2 ) )
				{
					tScript.fMission.mDisplayMessage(kTalkToBuddy_msg, kMissionScriptMessage);
					Ship->mSetCommButton (1, kButtonHail_text);
					break;
				}
				else
				{
					tScript.fMission.mDisplayMessage(kLyranHail_1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kLyranHail_2_msg, kMissionScriptMessage);
				}
			}
			else // Elite == kIronFang
			{
				tScript.fMission.mDisplayMessage(kLyranHail_1_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kLyranHail_2_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonSurrender_text:
			if ( Ship->mGetTeam() == kLyran2Team )
			{
			}
			else
			{
				tScript.fMission.mDisplayMessage(kLyranHail_1_msg, kMissionScriptMessage);
				tScript.fMission.mDisplayMessage(kLyranHail_2_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonTaunt_text:
			if ( Ship->mGetTeam() == kLyran2Team )
			{
			}
			else
			{
				tScript.fMission.mDisplayMessage(kLyranTaunt_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kSatelliteTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFreighterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kBigLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kLyran1Team, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kLyran2Team, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kSatelliteTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kSatelliteTeam, kBigLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kSatelliteTeam, kLyran1Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kSatelliteTeam, kLyran2Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kFreighterTeam, kBigLyranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kFreighterTeam, kLyran1Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kFreighterTeam, kLyran2Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kBigLyranTeam, kLyran1Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBigLyranTeam, kLyran2Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kLyran1Team, kLyran2Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	if ((AllSatellitesScanned) && (FreighterTalk))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
		return;
	}

	// point to nearest satellite if they are not all scanned
	if (!AllSatellitesScanned)
	{
		PointToNearestSatellite (tScript);
	}
	// handle Red Claw
	else if (Elite == kRedClaw)
	{
		// point to the freighter if we don't know about the projector or we have it already
		if (!FreighterDestroyed
			&& (KnowsAboutProjector == false || IonicTrade == 1))
		{
			tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
			tScript.fMission.mSetGoalIndicator (FreighterTeam->mGetFirstShip());
		}
		// point to ship that has Ionic projector
		else if (KnowsAboutProjector)
		{
			tTeamInfo* Lyran2Team = tScript.fMission.mGetTeamHandle (kLyran2Team);
			tScript.fMission.mSetGoalIndicator (Lyran2Team->mGetFirstShip());
		}
	}
	else if (!FreighterTalk && !FreighterDestroyed)
	{
		tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
		tScript.fMission.mSetGoalIndicator (FreighterTeam->mGetFirstShip());
	}
}

void PointToNearestSatellite (tScript& tScript)
{
	int32 shipx, shipy;
	int32 dist = 9999999;
	int32 found = -1;
	int32 i = kTotalSatellites;

	tScript.fPlayerShip->mGetGridPosition (&shipx, &shipy);
	while (i--)
	{
		if (SatelliteScannedList[i] == 0)
		{
			int32 distx = shipx - SatellitePostions[i].x;
			int32 disty = shipy - SatellitePostions[i].y;
			int32 new_dist = (distx * distx) + (disty * disty);
			if (new_dist < dist)
			{
				dist = new_dist;
				found = i;
			}
		}
	}
	if (found != -1)
	{
		tScript.fMission.mSetGoalIndicator (kTrue, SatellitePostions[found].x, SatellitePostions[found].y);
	}
}
