#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_7MissionText.h"
#include "Lyr_7MissionMaps.h"


//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayLevel = 0;
int32 PlayerDestroyed = 0;
int32 MountainDestroyed = 0;
int32 BratDestroyed = 0;
int32 MWDreadnoughtDestroyed = 0;
bool  AllMWDreadnoughtsDestroyed = false;
int32 DeathHowlDestroyed = 0;
bool  AllDeathHowlsDestroyed = false;
int32 FactionDestroyed = 0;
bool  AllFactionsDestroyed = false;
int32 BattleStarted = 0;
int32 AttackedBrat = 0;
const int32 CommButtonTimeout = 2; // duration before comm buttons reset to Hail
const float DamageThreshold = 50.0;  // how much damage before surrender and other things
int32 AttackedAlly = 0;
const int32 BratAmusedCounter = 20;
const int32 DreadnoughtArrivalCounter = 20;
int32 DreadEvent = 0;
bool  BratOwnedByPlayer = kFalse;


void MountainCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void BratCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void DeathHowlCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void RedClawCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void IronFangCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);

enum eMissionMode
{
	kStart = 0,
	kDreadnoughtArrives
};

void SetAllRelations(tScript& tScript, eTeamRelationship State);
void TestObjectives (tScript& tScript);
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);
void PeacefulOrbit(tTeamInfo* PeacefulTeam);

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMountainTeam, 
	kBratTeam,
	kMWDreadnoughtTeam,
	kDeathHowlTeam,
	kFactionTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMountainTeam, kLyran);
	fMission.mSetTeamRace (kBratTeam, kLyran);
	fMission.mSetTeamRace (kMWDreadnoughtTeam, kLyran);
	fMission.mSetTeamRace (kDeathHowlTeam, kLyran);
	fMission.mSetTeamRace (kFactionTeam, kLyran);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this, kTeamHostile);

	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		Elite = kRedClaw;
		return 1; // return Red Claw map
	}
	else
	{
		Elite = kIronFang;
		return 2; // return Iron Fang map
	}
	PlayLevel = fMission.mGetDifficultyLevel();

}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BratTeam = fMission.mGetTeamHandle (kBratTeam);
	tTeamInfo* MWDreadnoughtTeam = fMission.mGetTeamHandle (kMWDreadnoughtTeam);
	tTeamInfo* DeathHowlTeam = fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .7;

			// set LargestEnemyShipClass to one SMALLER than the player
			if ( PlayerShipClass > kClassFrigate )
			{
				LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
			}
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.1;
			// set LargestEnemyShipClass to one SMALLER than the player

			LargestEnemyShipClass = PlayerShipClass;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.5;
			// set LargestEnemyShipClass to one SMALLER than the player

			// set LargestEnemyShipClass to one LARGER than the player
			if ( PlayerShipClass < kClassBattleship )
			{
				LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
			}
			break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	if (Mode == kStart)
	{
		// create Mountain Watchers
		MountainTeam->mCreateFleet (4, TotalBPV, kClassFreighter, kClassLightCruiser, 50, kStartPosition_Specified, 36, 26);

		// create Brat's ship
		BratTeam->mCreateShip ("L-NCA", kStartPosition_Z);

		// create Death Howl team
		DeathHowlTeam->mCreateFleet(3, TotalBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_Specified, 23, 30);

		// create Faction team
		FactionTeam->mCreateFleet(NumberOfPlayersShips, TotalBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_J);
	}
	else if (Mode == kDreadnoughtArrives)
	{
		// create Mountain Watcher Dreadnought
		fMission.mSetTeamRelations (kPlayerTeam, kMWDreadnoughtTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kMountainTeam, kMWDreadnoughtTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kBratTeam, kMWDreadnoughtTeam, kTeamAllied, kGoalCapture, 1.0, kTrue);
		fMission.mSetTeamRelations (kDeathHowlTeam, kMWDreadnoughtTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kFactionTeam, kMWDreadnoughtTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		MWDreadnoughtTeam->mCreateShip("L-DNPP", kStartPosition_Y);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BratTeam = fMission.mGetTeamHandle (kBratTeam);
	tTeamInfo* MWDreadnoughtTeam = fMission.mGetTeamHandle (kMWDreadnoughtTeam);
	tTeamInfo* DeathHowlTeam = fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		// track the distance to the planet - 1
		Ship->mTrackTacticalDistance(80.0, 25, 15);
	}

	// configure Mountain Watchers
	for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mChangeName( kMountainWatcherNormal_name );
	}

	// configure Brat
	for (Ship = BratTeam->mGetFirstShip(); Ship != NULL; Ship = BratTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mChangeName(kZarrAttep_name);
	}

	// configure Death Howl 
	for (Ship = DeathHowlTeam->mGetFirstShip(); Ship != NULL; Ship = DeathHowlTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mChangeName( kDeathHowl_name );
	}
	
	// configure Faction
	for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);

		if ( Elite == kIronFang )
		{
			Ship->mChangeName( kRedClaw_name );
		}
		else
		{
			Ship->mChangeName( kIronFang_name );
		}
	}
	
	// Start a timer for setting team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point at the brat
	fMission.mSetGoalIndicator (BratTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// configure Mountain Watcher Dreadnought
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mSetCommButton(1, kButtonHail_text);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BratTeam = fMission.mGetTeamHandle (kBratTeam);
	tTeamInfo* MWDreadnoughtTeam = fMission.mGetTeamHandle (kMWDreadnoughtTeam);
	tTeamInfo* DeathHowlTeam = fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				// LNF I don't really know what this does/is supposed to do
				if (BattleStarted == 0)
				{
					BattleStarted = 1;
					Ship->mSetShipCallbackEvents (kNotifyDamage);
					fMission.mDisplayMessage(kEvent2_A_msg, kMissionScriptMessage);
					fMission.mDisplayMessage(kEvent2_B_msg, kMissionScriptMessage);
					BattleStarted = 1;
					fMission.mSetTimedInterval (kTimerLimitSet_3, BratAmusedCounter);
				}

				if ( HostileShip != NULL )
				{
					if ((HostileShip->mGetTeam() == kPlayerTeam) && (Ship->mGetTeam() == kFactionTeam) &&
						(AttackedAlly == 0))
					{
						AttackedAlly = 1;
						Ship->mSetShipCallbackEvents (kNotifyDamage);
						// set counter for arrival of MW dreadnought
						fMission.mSetTimedInterval (kTimerLimitSet_3, BratAmusedCounter);
						fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					}
					else if ((HostileShip->mGetTeam() == kPlayerTeam) && (Ship->mGetTeam() == kBratTeam) &&
							(AttackedBrat == 0))
					{
						AttackedBrat = 1;
						Ship->mSetShipCallbackEvents (kNotifyDamage);
					}
				}
			}
			break;

		case kNotifyAtDistance:
			// LNF check this crap too.  again.  feh.
			if (BattleStarted == 0)
			{
				fMission.mDisplayMessage(kEvent2_A_msg, kMissionScriptMessage);
				fMission.mDisplayMessage(kEvent2_B_msg, kMissionScriptMessage);
				BattleStarted = 1;
				fMission.mSetTimedInterval (kTimerLimitSet_3, BratAmusedCounter);
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					SetAllRelations (*this, kTeamHostile);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerLimitSet_3:
					if (DreadEvent == 0)
					{
						fMission.mDisplayMessage(kEvent3_A_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kEvent3_B_msg, kMissionScriptMessage);
						DreadEvent = 1;
						fMission.mSetTimedInterval (kTimerLimitSet_3, DreadnoughtArrivalCounter);
						SetAllRelations (*this, kTeamAtWar);
					}
					else if (DreadEvent == 1)
					{
						fMission.mDisplayMessage(kEvent4_msg, kMissionScriptMessage);
						mCreateMissionShips(kDreadnoughtArrives);
					}
					break;

				case kTimerTurnsSet_6:
					// set the Mountain Watcher comm buttons back to HAIL
					for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_7:
					// set the Brat's comm buttons back to HAIL
					for (Ship = BratTeam->mGetFirstShip(); Ship != NULL; Ship = BratTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8:
					// set the Mountain Watcher Dreadnought comm buttons back to HAIL
					for (Ship = MWDreadnoughtTeam->mGetFirstShip(); Ship != NULL; Ship = MWDreadnoughtTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Death Howl comm buttons back to HAIL
					for (Ship = DeathHowlTeam->mGetFirstShip(); Ship != NULL; Ship = DeathHowlTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Faction comm buttons back to HAIL
					for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam() == kBratTeam)
			{
				if (fEventID == kPlayerTeam)
				{
					fMission.mAddToBonusPrestige( kCaptureBonus );
					BratOwnedByPlayer = kTrue;
					TestObjectives (*this);
					Ship->mFollowEntity (fPlayerShip, kHighPriority);
				}
				else
				{
					BratOwnedByPlayer = kFalse;
				}
				break;
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
			}

			if (Ship->mGetTeam() == kMountainTeam)
			{
				MountainDestroyed++;
			}

			if (Ship->mGetTeam() == kBratTeam)
			{
				BratDestroyed++;
			}

			if (Ship->mGetTeam() == kMWDreadnoughtTeam)
			{
				MWDreadnoughtDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kDeathHowlTeam)
			{
				DeathHowlDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kFactionTeam)
			{
				FactionDestroyed++;
				TestObjectives (*this);
			}

			break;

		case kNotifyDisengaged:
			if (Ship->mGetOriginalTeam() == kBratTeam)
			{
				if (BratOwnedByPlayer)
				{
					return kPlayerWins;
				}
				else
				{
					return eGameStatus(kPlayerLoses | kExpellFromElite);
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (Ship->mGetTeam())
				{
					case kMountainTeam:
						MountainCommMessages(*this, fEventID, Ship, kTimerTurnsSet_6);
						break;

					case kBratTeam:
						BratCommMessages(*this, fEventID, Ship, kTimerTurnsSet_7);
						break;

					case kMWDreadnoughtTeam:
						MountainCommMessages(*this, fEventID, Ship, kTimerTurnsSet_8);
						break;

					case kDeathHowlTeam:
						DeathHowlCommMessages(*this, fEventID, Ship, kTimerTurnsSet_9);
						break;

					case kFactionTeam:
						if (Elite == kRedClaw)
						{
							IronFangCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
						}
						else
						{
							RedClawCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
						}
						break;
				}
			}
			break;
	}

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		if (mCalcVictoryStatus() == kDevastatingDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if (BratOwnedByPlayer)
	{
		VictoryPoints = (1 - BratDestroyed) * 100;
	}
	VictoryPoints -= (PlayerDestroyed * 10);


	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (VictoryLevel == kAstoundingVictory)
	{
		fMission.mAddToDebriefingList (kAstoundingVictory_msg);
		fMission.mAddToDebriefingList (kVictoryP2_msg);
	}
	else if (VictoryLevel == kVictory)
	{
		fMission.mAddToDebriefingList (kVictory_msg);
		fMission.mAddToDebriefingList (kVictoryP2_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( BratOwnedByPlayer ) ? fMission.mAddToDebriefingList( kBratNotDestroyed_msg ) : SuccessCount++;
	( AllMWDreadnoughtsDestroyed ) ? fMission.mAddToDebriefingList( kAllMWDreadnoughtsDestroyed_msg ) : SuccessCount++;
	( AllDeathHowlsDestroyed ) ? fMission.mAddToDebriefingList( kAllDeathHowlsDestroyed_msg ) : SuccessCount++;
	( AllFactionsDestroyed ) ? fMission.mAddToDebriefingList( kAllFactionsDestroyed_msg ) : SuccessCount++;
	if ( SuccessCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( BratDestroyed ) ? fMission.mAddToDebriefingList( kBratDestroyed_msg ) : FailureCount++;
	( !AllMWDreadnoughtsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllMWDreadnoughtsDestroyed_msg ) : FailureCount++;
	( !AllDeathHowlsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllDeathHowlsDestroyed_msg ) : FailureCount++;
	( !AllFactionsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllFactionsDestroyed_msg ) : FailureCount++;
	if ( FailureCount == 4 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	if (VictoryLevel != kDevastatingDefeat)
	{
		fMission.mScheduleMission( kNextMission_msg );
		// LNF modify the number of spacer missions?
		int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
		switch (NumberOfSpacerMissions)
		{
			case 0:
				break;

			case 1:
				fMission.mScheduleMission( kRandomMission_msg );
				break;
				
			case 2:
				fMission.mScheduleMission( kRandomMission_msg );
				fMission.mScheduleMission( kRandomMission_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// make the peaceful team patrol the planet
void PeacefulOrbit(tTeamInfo* PeacefulTeam)
{
	tShipInfo* Ship;

	tMapPoint waypoints[6]=
	{
		17, 15,
		18, 17,
		22, 17,
		23, 15,
		22, 13,
		19, 13,
	};

	tMapPoint PatrolPoints[6];

	for (Ship = PeacefulTeam->mGetFirstShip(); Ship != NULL; Ship = PeacefulTeam->mGetNextShip())
	{
		PatrolPoints[0] = waypoints[0];
		PatrolPoints[1] = waypoints[1];
		PatrolPoints[2] = waypoints[2];
		PatrolPoints[3] = waypoints[3];
		PatrolPoints[4] = waypoints[4];
		PatrolPoints[5] = waypoints[5];
		Ship->mPatrol (1.0, 6, &PatrolPoints[0], kHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
// Mountain Comm Messages
void MountainCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (BattleStarted == 0)
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainHailNicIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainHailNicRC_msg, kMissionScriptMessage);
				}
			}
			else // BattleStarted == 1
			{
				tScript.fMission.mDisplayMessage(kMountainHailCombat_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in CommButtonTimeout turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() < DamageThreshold)
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainSurrenderOKIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainSurrenderOKRC_msg, kMissionScriptMessage);
				}
			}
			else // DamagePercent > DamageThreshold
			{
				tScript.fMission.mDisplayMessage(kMountainSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			if (Ship->mGetDamagePercent() < DamageThreshold)
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainTauntOKIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainTauntOKRC_msg, kMissionScriptMessage);
				}
			}
			else // DamagePercent > DamageThreshold
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainTauntDamagedIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainTauntDamagedRC_msg, kMissionScriptMessage);
				}
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Brat Comm Messages
void BratCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (BattleStarted == 0)
			{
				tScript.fMission.mDisplayMessage(kBratHailNic_msg, kMissionScriptMessage);
			}
			else if ((BattleStarted == 1) && (AttackedBrat == 0))
			{
				tScript.fMission.mDisplayMessage(kBratHailCombat_msg, kMissionScriptMessage);
			}
			else if (AttackedBrat == 1)
			{
				tScript.fMission.mDisplayMessage(kBratHailAttacked_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in CommButtonTimeout turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (BattleStarted == 0)
			{
				tScript.fMission.mDisplayMessage(kBratSurrenderNic_msg, kMissionScriptMessage);
			}
			else if (!AllMWDreadnoughtsDestroyed || !AllDeathHowlsDestroyed || !AllFactionsDestroyed)
			{
				tScript.fMission.mDisplayMessage(kBratSurrenderCombat_msg, kMissionScriptMessage);
			}
			else if (!BratOwnedByPlayer)
			{
				tScript.fMission.mDisplayMessage(kBratSurrender_msg, kMissionScriptMessage);
				BratOwnedByPlayer = kTrue;
				TestObjectives (tScript);
				tTeamInfo* BratTeam = tScript.fMission.mGetTeamHandle (kBratTeam);
				BratTeam->mGetFirstShipInPlay()->mFollowEntity (tScript.fPlayerShip, kHighPriority);
				BratTeam->mGetFirstShipInPlay()->mClearCommButton (1);
				BratTeam->mGetFirstShipInPlay()->mClearCommButton (2);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kBratTaunt_1_msg, kBratTaunt_3_msg), kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Death Howl Comm Messages
void DeathHowlCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (Elite == kIronFang)
			{
				if (BattleStarted == 0)
				{
					tScript.fMission.mDisplayMessage(kDHHailNicIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kDHHailCombatIF_msg, kMissionScriptMessage);
				}
			}
			else
			{
				if (BattleStarted == 0)
				{
					tScript.fMission.mDisplayMessage(kDHHailNicRC_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kDHHailCombatRC_msg, kMissionScriptMessage);
				}
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in CommButtonTimeout turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() < DamageThreshold)
			{
				tScript.fMission.mDisplayMessage(kDHSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kDHSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kDHTaunt_1_msg, kDHTaunt_3_msg), kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Red Claw Comm Messages
// player is Iron Fang and hailing a Red Claw ship
void RedClawCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	tTeamInfo* MountainTeam = tScript.fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BratTeam = tScript.fMission.mGetTeamHandle (kBratTeam);
	tTeamInfo* MWDreadnoughtTeam = tScript.fMission.mGetTeamHandle (kMWDreadnoughtTeam);
	tTeamInfo* DeathHowlTeam = tScript.fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam);

	int32 OtherShipsRemaining = 0;

	if ((MWDreadnoughtTeam->mGetShipsInPlayCount() > 0) ||
		(DeathHowlTeam->mGetShipsInPlayCount() > 0) ||
		(MountainTeam->mGetShipsInPlayCount() > 0))
	{
		OtherShipsRemaining = 1;
	}
	else
	{
		OtherShipsRemaining = 0;
	}

	switch (fEventID)
	{
		case kButtonHail_text:
			if (BattleStarted == 0)
			{
				tScript.fMission.mDisplayMessage(kRCHail_msg, kMissionScriptMessage);
			}
			else if ((AttackedAlly == 0) && (OtherShipsRemaining == 1))
			{
				tScript.fMission.mDisplayMessage(kFactionHail_msg, kMissionScriptMessage);
			}
			else if ((AttackedAlly == 0) && (OtherShipsRemaining == 0) && (BratDestroyed == 0))
			{
				tScript.fMission.mDisplayMessage(kFactionHailDuel_msg, kMissionScriptMessage);
			}
			else if ((AttackedAlly == 0) && (BratDestroyed == 1))
			{
				tScript.fMission.mDisplayMessage(kFactionHailBratDead_msg, kMissionScriptMessage);
			}
			else if (AttackedAlly == 1)
			{
				tScript.fMission.mDisplayMessage(kFactionHailAttacked_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonSurrender_text:
			if (BattleStarted == 0)
			{
				tScript.fMission.mDisplayMessage(kFactionSurrenderNic_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kFactionSurrenderCombat_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage(kFactionTaunt_msg, kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Iron Fang Comm Messages
// the player is Red Claw, and hailing an Iron Fang ship
void IronFangCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	tTeamInfo* MountainTeam = tScript.fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BratTeam = tScript.fMission.mGetTeamHandle (kBratTeam);
	tTeamInfo* MWDreadnoughtTeam = tScript.fMission.mGetTeamHandle (kMWDreadnoughtTeam);
	tTeamInfo* DeathHowlTeam = tScript.fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam);

	int32 OtherShipsRemaining = 0;

	if ((MWDreadnoughtTeam->mGetShipsInPlayCount() > 0) ||
		(DeathHowlTeam->mGetShipsInPlayCount() > 0) ||
		(MountainTeam->mGetShipsInPlayCount() > 0))
	{
		OtherShipsRemaining = 1;
	}
	else
	{
		OtherShipsRemaining = 0;
	}

	switch (fEventID)
	{
		case kButtonHail_text:
			if (BattleStarted == 0)
			{
				tScript.fMission.mDisplayMessage(kIFHail_msg, kMissionScriptMessage);
			}
			else if ((AttackedAlly == 0) && (OtherShipsRemaining == 1))
			{
				tScript.fMission.mDisplayMessage(kFactionHail_msg, kMissionScriptMessage);
			}
			else if ((AttackedAlly == 0) && (OtherShipsRemaining == 0) && (BratDestroyed == 0))
			{
				tScript.fMission.mDisplayMessage(kFactionHailDuel_msg, kMissionScriptMessage);
			}
			else if ((AttackedAlly == 0) && (BratDestroyed == 1))
			{
				tScript.fMission.mDisplayMessage(kFactionHailBratDead_msg, kMissionScriptMessage);
			}
			else if (AttackedAlly == 1)
			{
				tScript.fMission.mDisplayMessage(kFactionHailAttacked_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (BattleStarted == 0)
			{
				tScript.fMission.mDisplayMessage(kFactionSurrenderNic_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kFactionSurrenderCombat_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage(kFactionTaunt_msg, kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript, eTeamRelationship State)
{
	// Set team relations to show up on Tactical map
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kMountainTeam, State, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kBratTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kDeathHowlTeam, State, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, State, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kMountainTeam, kBratTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kDeathHowlTeam, State, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kFactionTeam, State, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kBratTeam, kDeathHowlTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBratTeam, kFactionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kDeathHowlTeam, kFactionTeam, State, kGoalDestroy, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	tTeamInfo* MWDreadnoughtTeam = tScript.fMission.mGetTeamHandle (kMWDreadnoughtTeam);
	tTeamInfo* DeathHowlTeam = tScript.fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam);

	if (MWDreadnoughtTeam->mGetShipsInPlayCount() == 0)
	{
		AllMWDreadnoughtsDestroyed = true;
	}
	if (DeathHowlTeam->mGetShipsInPlayCount() == 0)
	{
		AllDeathHowlsDestroyed = true;
	}
	if (FactionTeam->mGetShipsInPlayCount() == 0)
	{
		AllFactionsDestroyed = true;
	}

	if ((AllMWDreadnoughtsDestroyed) && (AllDeathHowlsDestroyed) && (AllFactionsDestroyed) && (BratOwnedByPlayer))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}
}
