#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_8MissionText.h"
#include "Lyr_8MissionMaps.h"


// sequence of events:
// 1. player must get into attack range of the base.
//    (players fleet and allies will be damaged 60-70%)
// 2. when damage to players ship is more than 70% the player receives instructions to orbit planet.
// 3. player will receive transmission at start of orbit.
//
// player objectives:
// 1. receive transmission
// 2. destroy all bases
// 3. destroy all shipyards


int32 Elite = 0;
int32 PlayLevel = 0;
int32 MountainDestroyed = 0;
int32 BaseDestroyed = 0;
bool  AllBasesDestroyed = false;
int32 ShipyardDestroyed = 0;
bool  AllShipyardsDestroyed = false;
int32 DeathHowlDestroyed = 0;
int32 FactionDestroyed = 0;
int32 FireRunnersDestroyed = 0;
int32 GreyHuntersDestroyed = 0;
int32 AttackedAlly = 0;
int32 Instructed = 0;
int32 Kaboom = 0;
int32 Transmission = 0;
int32 NumberOfAllies = 0; // number of ships in allied fleet EXcluding the player's
const float DamageThreshold = 50.0;  // how much damage before surrender and other things
const int32 CommButtonTimeout = 2; // duration before comm buttons reset to Hail
const float Event1Range = 80.0; // player's range to base when event 1 occurs
const float Event2Range = 20.0; // player's range to planet when event 2 occurs
eVictoryLevels MyVicLevel = kDevastatingDefeat;
int32 OrbitX;
int32 OrbitY;


void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
void MountainCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void AllyCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
iTruth DamageTeam (tScript& tScript, tTeamInfo* Team, iTruth DestroyShip);


eClassTypes PlayerShipClass= kClassShuttle;

void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMountainTeam, 
	kBaseTeam,
	kShipyardTeam,
	kDeathHowlTeam,
	kFactionTeam,
	kFireRunnersTeam,
	kGreyHuntersTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMountainTeam, kLyran);
	fMission.mSetTeamRace (kBaseTeam, kLyran);
	fMission.mSetTeamRace (kShipyardTeam, kLyran);
	fMission.mSetTeamRace (kDeathHowlTeam, kLyran);
	fMission.mSetTeamRace (kFactionTeam, kLyran);
	fMission.mSetTeamRace (kFireRunnersTeam, kLyran);
	fMission.mSetTeamRace (kGreyHuntersTeam, kLyran);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		Elite = kRedClaw;
		OrbitX = 10;
		OrbitY = 37;
		return 1; // return Red Claw map
	}
	else
	{
		Elite = kIronFang;
		OrbitX = 33;
		OrbitY = 33;
		return 2; // return Iron Fang map
	}
	PlayLevel = fMission.mGetDifficultyLevel();

}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* DeathHowlTeam = fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* FireRunnersTeam = fMission.mGetTeamHandle (kFireRunnersTeam);
	tTeamInfo* GreyHuntersTeam = fMission.mGetTeamHandle (kGreyHuntersTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .8;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.2;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.5;
			break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Mountain Watchers
	MountainTeam->mCreateFleet (4, TotalBPV, kClassFreighter, kClassLightCruiser, 50, kStartPosition_Specified, 29, 16);

	// create Base
	switch (PlayLevel)
	{
		case 0:
			BaseTeam->mCreateShip ("L-BS", kStartPosition_Z);
			break;
		case 1:
			BaseTeam->mCreateShip ("L-BATS", kStartPosition_Z);
			break;
		case 2:
			BaseTeam->mCreateShip ("L-SB", kStartPosition_Z);
			break;
	}

	// create Shipyards
	ShipyardTeam->mCreateShip("L-DNPP", kStartPosition_W);

	// create Death Howl team
	DeathHowlTeam->mCreateFleet(3, TotalBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_Specified, 6, 32);

	if (Elite == kIronFang)
	{
		// create Faction Iron Fang
		FactionTeam->mCreateFleet(3, kMaxBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_Specified, 9, 25);
	}
	else
	{
		// create Faction Red Claw
		FactionTeam->mCreateFleet(3, kMaxBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_Specified, 7, 17);
	}

	// create Fire Runners
	FireRunnersTeam->mCreateFleet(2, TotalBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_Specified, 6, 10);

	// create Grey Hunters
	GreyHuntersTeam->mCreateFleet(4, TotalBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_Specified, 19, 35);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* DeathHowlTeam = fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* FireRunnersTeam = fMission.mGetTeamHandle (kFireRunnersTeam);
	tTeamInfo* GreyHuntersTeam = fMission.mGetTeamHandle (kGreyHuntersTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mTrackTacticalDistance(Event1Range, 29, 16);
	}

	// configure Mountain Watchers
	for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Base
	for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Shipyard
	for (Ship = ShipyardTeam->mGetFirstShip(); Ship != NULL; Ship = ShipyardTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Death Howl 
	for (Ship = DeathHowlTeam->mGetFirstShip(); Ship != NULL; Ship = DeathHowlTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		NumberOfAllies++;
	}
	
	// configure Faction
	for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		NumberOfAllies++;
	}
	
	// configure Fire Runners
	for (Ship = FireRunnersTeam->mGetFirstShip(); Ship != NULL; Ship = FireRunnersTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		NumberOfAllies++;
	}
	
	// configure Grey Hunters
	for (Ship = GreyHuntersTeam->mGetFirstShip(); Ship != NULL; Ship = GreyHuntersTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		NumberOfAllies++;
	}
	
	// Start a timer for setting team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// point at the base
	fMission.mSetGoalIndicator (BaseTeam->mGetFirstShip());
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* ShipyardTeam = fMission.mGetTeamHandle (kShipyardTeam);
	tTeamInfo* DeathHowlTeam = fMission.mGetTeamHandle (kDeathHowlTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* FireRunnersTeam = fMission.mGetTeamHandle (kFireRunnersTeam);
	tTeamInfo* GreyHuntersTeam = fMission.mGetTeamHandle (kGreyHuntersTeam);

	switch (Event)
	{
		case kNotifyDamage:
			if (Ship == fPlayerShip
				&& Instructed == 0)
			{
				fMission.mDisplayMessage(kEvent2_A_msg, kMissionScriptMessage);
				Ship->mTrackTacticalDistance(Event2Range, OrbitX, OrbitY);
				fMission.mSetGoalIndicator (kTrue, OrbitX, OrbitY);
				Instructed = 1;
			}
			Ship->mSetShipCallbackEvents (kNotifyDamage);
			break;

		case kNotifyAtDistance:
			{
				if (Ship->mGetTrackingDistance() == Event1Range)
				{
					for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
					{
						Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, fMission.mRandomInt32 (25, 30));
					}
					fMission.mDisplayMessage(kEvent1_A_msg, kMissionScriptMessage);
					fMission.mDisplayMessage(kEvent1_B_msg, kMissionScriptMessage);
					fMission.mDisplayMessage(kEvent1_C_msg, kMissionScriptMessage);
					iTruth destroy_ship = kFalse;
					destroy_ship = DamageTeam (*this, DeathHowlTeam, destroy_ship);
					destroy_ship = DamageTeam (*this, FactionTeam, destroy_ship);
					destroy_ship = DamageTeam (*this, FireRunnersTeam, destroy_ship);
					destroy_ship = DamageTeam (*this, GreyHuntersTeam, destroy_ship);
					Kaboom = 1;
				}
				else if (Ship->mGetTrackingDistance() == Event2Range)
				{
					if (Elite == kIronFang)
					{
						if ((Kaboom == 1) && (Transmission == 0))
						{
							fMission.mDisplayMessage(kEvent3OrbitingIF_1_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingIF_2_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingIF_3_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingIF_6_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingIF_7_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingIF_8_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingIF_9_msg, kMissionScriptMessage);
							Transmission = 1;
							TestObjectives (*this);
						}					
					}
					else // Red Claw
					{
						if ((Kaboom == 1) && (Transmission == 0))
						{
							fMission.mDisplayMessage(kEvent3OrbitingRC_1_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingRC_2_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingRC_3_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingRC_4_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingRC_5_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingRC_6_msg, kMissionScriptMessage);
							fMission.mDisplayMessage(kEvent3OrbitingRC_7_msg, kMissionScriptMessage);
							Transmission = 1;
							TestObjectives (*this);
						}					
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations to show up on Tactical map
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerTurnsSet_6:
					// set the Mountain Watcher comm buttons back to HAIL
					for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_7:
					// set the Fire Runner comm buttons back to HAIL
					for (Ship = FireRunnersTeam->mGetFirstShip(); Ship != NULL; Ship = FireRunnersTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8:
					// set the Grey Hunters comm buttons back to HAIL
					for (Ship = GreyHuntersTeam->mGetFirstShip(); Ship != NULL; Ship = GreyHuntersTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Death Howl comm buttons back to HAIL
					for (Ship = DeathHowlTeam->mGetFirstShip(); Ship != NULL; Ship = DeathHowlTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Faction comm buttons back to HAIL
					for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kMountainTeam)
			{
				MountainDestroyed++;
			}

			if (Ship->mGetTeam() == kBaseTeam)
			{
				BaseDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kShipyardTeam)
			{
				ShipyardDestroyed++;
				TestObjectives (*this);
			}

			if (Ship->mGetTeam() == kDeathHowlTeam)
			{
				DeathHowlDestroyed++;
			}

			if (Ship->mGetTeam() == kFactionTeam)
			{
				FactionDestroyed++;
			}

			if (Ship->mGetTeam() == kFireRunnersTeam)
			{
				FireRunnersDestroyed++;
			}

			if (Ship->mGetTeam() == kGreyHuntersTeam)
			{
				GreyHuntersDestroyed++;
			}

			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				switch (Ship->mGetTeam())
				{
					case kMountainTeam:
						MountainCommMessages(*this, fEventID, Ship, kTimerTurnsSet_6);
						break;

					case kDeathHowlTeam:
						AllyCommMessages(*this, fEventID, Ship, kTimerTurnsSet_9);
						break;

					case kFactionTeam:
						AllyCommMessages(*this, fEventID, Ship, kTimerTurnsSet_10);
						break;

					case kFireRunnersTeam:
						AllyCommMessages(*this, fEventID, Ship, kTimerTurnsSet_7);
						break;

					case kGreyHuntersTeam:
						AllyCommMessages(*this, fEventID, Ship, kTimerTurnsSet_8);
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (Transmission * 70) + (BaseDestroyed * 85) + (ShipyardDestroyed * 3);

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;
	if (VictoryLevel <= kVictory)
	{
		fMission.mAddToDebriefingList (kAstoundingVictory_msg);
		if (Transmission == 0)
		{
			fMission.mAddToDebriefingList (kVictoryTransmissionFalse_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kVictoryTransmissionTrue_msg);
		}
	}
	else
	{
		fMission.mAddToDebriefingList (kDevastatingDefeat_msg);
	}
	fMission.mAddToDebriefingList (kDismissedShort_msg);

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	( Transmission ) ? fMission.mAddToDebriefingList( kMetWithClient_msg ) : SuccessCount++;
	( AllBasesDestroyed ) ? fMission.mAddToDebriefingList( kAllBasesDestroyed_msg ) : SuccessCount++;
	( AllShipyardsDestroyed ) ? fMission.mAddToDebriefingList( kAllShipyardsDestroyed_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( !Transmission ) ? fMission.mAddToDebriefingList( kNotMetWithClient_msg ) : FailureCount++;
	( !AllBasesDestroyed ) ? fMission.mAddToDebriefingList( kNotAllBasesDestroyed_msg ) : FailureCount++;
	( !AllShipyardsDestroyed ) ? fMission.mAddToDebriefingList( kNotAllShipyardsDestroyed_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_4;
	}
	else
	{
		return 0;
	}
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships to create
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}


//---------------------------------------------------------------------------------------------
// Mountain Comm Messages
void MountainCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (Kaboom == 0)
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainHailIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainHailRC_msg, kMissionScriptMessage);
				}
			}
			else // Kaboom == 1
			{
				tScript.fMission.mDisplayMessage(kMountainHailKaboom_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in CommButtonTimeout turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() < DamageThreshold)
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainSurrenderOKIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainSurrenderOKRC_msg, kMissionScriptMessage);
				}
			}
			else // DamagePercent > DamageThreshold
			{
				tScript.fMission.mDisplayMessage(kMountainSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			if (Elite == kIronFang)
			{
				tScript.fMission.mDisplayMessage(kMountainTauntIF_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kMountainTauntRC_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Ally Comm Messages
void AllyCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (AttackedAlly == 0)
			{
				if ((Elite == kIronFang) && (Kaboom == 0))
				{
					tScript.fMission.mDisplayMessage(kAllyHailIF_msg, kMissionScriptMessage);
				}
				else if ((Elite == kRedClaw) && (Kaboom == 0))
				{
					tScript.fMission.mDisplayMessage(kAllyHailRC_msg, kMissionScriptMessage);
				}
				else if ((Kaboom == 1) && (Ship->mGetDamagePercent() <= DamageThreshold))
				{
					tScript.fMission.mDisplayMessage(kAllyHailKaboomOK_msg, kMissionScriptMessage);
				}
				else if ((Kaboom == 1) && (Ship->mGetDamagePercent() > DamageThreshold))
				{
					tScript.fMission.mDisplayMessage(kAllyHailKaboomDamaged_msg, kMissionScriptMessage);
				}

			}
			else // if Attacked ally == 1
			{
				tScript.fMission.mDisplayMessage(kAllyAllAttacked_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in CommButtonTimeout turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (AttackedAlly == 0)
			{
				tScript.fMission.mDisplayMessage(kAllySurrender_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kAllyAllAttacked_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			if (AttackedAlly == 0)
			{
				tScript.fMission.mDisplayMessage(kAllySurrender_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kAllyAllAttacked_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kMountainTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShipyardTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kDeathHowlTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFireRunnersTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kGreyHuntersTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kMountainTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kShipyardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kDeathHowlTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kFireRunnersTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kGreyHuntersTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kBaseTeam, kShipyardTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBaseTeam, kDeathHowlTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBaseTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBaseTeam, kFireRunnersTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBaseTeam, kGreyHuntersTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kShipyardTeam, kDeathHowlTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShipyardTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShipyardTeam, kFireRunnersTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShipyardTeam, kGreyHuntersTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kDeathHowlTeam, kFactionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kDeathHowlTeam, kFireRunnersTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kDeathHowlTeam, kGreyHuntersTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kFactionTeam, kFireRunnersTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kFactionTeam, kGreyHuntersTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kFireRunnersTeam, kGreyHuntersTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// Destroy half the ships, shields down, engine power to 70%
// get half the number of ships in the fleet excluding the player's
// enumerate all ships, and destroy half...tried this in some lyran mission
// the rest of the ships get their shields taken down, or minimized
// and engines reduced to at least 70%
iTruth DamageTeam (tScript& tScript, tTeamInfo* Team, iTruth DestroyShip)
{
	tShipInfo* Ship;
	for (Ship = Team->mGetFirstShipInPlay(); Ship != NULL; Ship = Team->mGetNextShipInPlay())
	{
		if (DestroyShip)
		{
			Ship->mDamageSystem (kDamageInternalsWithPoints, 5000);
			Ship->mDamageSystem (kDamageExcessDamageSystem, 5000);
		}
		else
		{
			Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, tScript.fMission.mRandomInt32 (30, 40));
		}
		DestroyShip ^= kTrue;
	}
	return DestroyShip;
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	tTeamInfo* BaseTeam = tScript.fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* ShipyardTeam = tScript.fMission.mGetTeamHandle (kShipyardTeam);

	if (BaseTeam->mGetShipsInPlayCount() == 0)
	{
		AllBasesDestroyed = true;
	}
	if (ShipyardTeam->mGetShipsInPlayCount() == 0)
	{
		AllShipyardsDestroyed = true;
	}
	
	if ((Transmission) && (AllBasesDestroyed) && (AllShipyardsDestroyed))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}

	if ((Kaboom == 1) && (Transmission == 0))
	{
		tScript.fMission.mSetGoalIndicator (kTrue, OrbitX, OrbitY);
	}
	else if (!AllBasesDestroyed)
	{
		tScript.fMission.mSetGoalIndicator (BaseTeam->mGetFirstShip());
	}
	else if (!AllShipyardsDestroyed)
	{
		tScript.fMission.mSetGoalIndicator (ShipyardTeam->mGetFirstShipInPlay());
	}

}
