#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Lyr_9MissionText.h"
#include "Lyr_9MissionMaps.h"

// sequence of events:
// 1. player must get into attack range of the planet.
//    (players fleet and allies will be damaged 40-50%)
// 2. scan freighters looking for the one that contains the shield generator.
//    (gets message when correct freighter scanned)
// 3. destroy the freighter with the shield generator.
//    (ships returned to normal)
//
// player objectives:
// 1. destroy all freighters
// 2. destroy all defenses

int32 Elite = 0;
int32 PlayLevel = 0;
int32 MountainDestroyed = 0;
int32 DefenseDestroyed = 0;
bool  AllDefensesDestroyed = false;
int32 FreighterDestroyed = 0;
bool  AllFreightersDestroyed = false;
const float DeepScanDistance = 30.0; // minimum distance to perform a Deep Scan


// LNF is there a way to find the number of Freighter units and still have a global array? Dynamic Array?
const int32 NumberOfFreighters = 4;
int32 FreighterID[NumberOfFreighters] = {0};  // the CallbackID of the Freighter ship entity
int32 FreightersScannedList[NumberOfFreighters];
int32 FreighterIDNum = 0;
tMapPoint FreighterPostions[NumberOfFreighters]=
{
	20, 17,
	20, 23,
	17, 20,
	23, 20,
};

const int32 NumberOfPlatforms = 3;
int32 PlatformID[NumberOfPlatforms] = {0};  // the CallbackID of the Platform ship entity
int32 PlatformsDestroyedList[NumberOfPlatforms];
int32 PlatformIDNum = 0;
tMapPoint PlatformPostions[NumberOfPlatforms]=
{
	17, 17,
	23, 17,
	23, 23,
	//17, 23,
};

int32 Kaboom = 0;
int32 FreighterWithNegator = 0; // the array index number of the freighter with the negator
const float DamageThreshold = 50.0;  // how much damage before surrender and other things
const int32 CommButtonTimeout = 2; // duration before comm buttons reset to Hail
const float Event1Range = 50.0; // player's range to base when event 1 occurs


void SetAllRelations(tScript& tScript);
void TestObjectives (tScript& tScript);
void MountainCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType);
void PointToNearestFreighter (tScript& tScript);
void PointToNearestPlatform (tScript& tScript);


eClassTypes PlayerShipClass= kClassShuttle;

void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kRedClaw = 1,
	kIronFang,
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kMountainTeam, 
	kDefenseTeam,
	kFreighterTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kMountainTeam, kLyran);
	fMission.mSetTeamRace (kDefenseTeam, kLyran);
	fMission.mSetTeamRace (kFreighterTeam, kLyran);

	// Set team relations to show up on Tactical map
	SetAllRelations (*this);

	if (fMission.mGetEliteOrganization() == kRedClaw)
	{
		Elite = kRedClaw;
		return 1; // return Red Claw map
	}
	else
	{
		Elite = kIronFang;
		return 2; // return Iron Fang map
	}
	PlayLevel = fMission.mGetDifficultyLevel();

}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	switch( PlayLevel )
	{
		case 0:		// easy
			TotalBPV = TotalBPV * .8;
			break;

		case 1:		// medium
			TotalBPV = TotalBPV * 1.2;
			break;

		case 2:		// hard
			TotalBPV = TotalBPV * 1.5;
			break;
	}

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	
	// create Mountain Watchers
	MountainTeam->mCreateFleet (3, TotalBPV, kClassLightCruiser, kClassBattleship, 50, kStartPosition_Specified, 29, 16);

	// create Defense
	for (int32 i = 0; i < NumberOfPlatforms; i++)
	{
		DefenseTeam->mCreateShip ("L-DEF", kStartPosition_Specified, PlatformPostions[i].x, PlatformPostions[i].y);
	}

	// create Freighters
	for (i = 0; i < NumberOfFreighters; i++)
	{
		FreighterTeam->mCreateShip (kMaxBPV, kClassFreighter, 50, kStartPosition_Specified,
									FreighterPostions[i].x, FreighterPostions[i].y);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kRedClaw)
	{
		// LNF need to put conditional on variables from previous mission
		return (kRedClawBriefing_msg);
	}
	else
	{
		return (kIronFangBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mTrackTacticalDistance(Event1Range, 20, 20); // distance from planet for Event 1

	}

	// configure Mountain Watchers
	for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
	}

	// configure Defense
	PlatformIDNum = 0;
	for (Ship = DefenseTeam->mGetFirstShip(); Ship != NULL; Ship = DefenseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		PlatformsDestroyedList[PlatformIDNum] = 0;
		PlatformID[PlatformIDNum] = Ship->mGetShipID();
		PlatformIDNum++;
	}

	// configure Freighter
	FreighterIDNum = 0;
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mSetShipMaxSpeed(0);
		// LNF initialize the Freighter array with the CallbackID numbers
		FreightersScannedList[FreighterIDNum] = 0;
		FreighterID[FreighterIDNum] = Ship->mGetShipID();
		FreighterIDNum++;
	}

	// randomly pick the freighter that will hold the negator
	FreighterWithNegator = fMission.mRandomInt32(0, (NumberOfFreighters - 1));

	// Start a timer for setting team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		DeepScanTracker.mSetTarget (Ship, DeepScanDistance, 0);
	}

	PointToNearestFreighter (*this);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* MountainTeam = fMission.mGetTeamHandle (kMountainTeam);
	tTeamInfo* DefenseTeam = fMission.mGetTeamHandle (kDefenseTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	switch (Event)
	{
		case kNotifyDamage:
			break;

		case kNotifyAtDistance:
			{
				tShipInfo* Ship;
				for (Ship = PlayerTeam->mGetFirstShipInPlay(); Ship != NULL; Ship = PlayerTeam->mGetNextShipInPlay())
				{
					// damage the ship only if its got less than 15% damage already
					if (Ship->mGetDamagePercent() < 15)
					{
						Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, fMission.mRandomInt32 (15, 20));
						if (Kaboom == 0)
						{
							fMission.mDisplayMessage(kEvent1_msg, kScienceScriptMessage);
							Kaboom = 1;
						}
					}
				}
				Kaboom = 1;
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations to show up on Tactical map
					SetAllRelations (*this);
					fMission.mDisplayMessage (kStarting_msg, kScienceScriptMessage);
					break;

				case kTimerTurnsSet_10:
					// set the Mountain Watcher comm buttons back to HAIL
					for (Ship = MountainTeam->mGetFirstShip(); Ship != NULL; Ship = MountainTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			
				case kTimerLimitSet_10:
					{
						eDeepScanFlags	flag;
						tMapPoint		MapPoint;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);
						switch (flag)
						{
							case kEnteringRange_ScanOn_TimerStarted:
							case kInRange_ScanOn_TimerStarted:
								if (Object)
								{
									int32 ShipNumber = Object->mGetShipID();
									int32 FreighterCount = 0;
									while (ShipNumber != FreighterID[FreighterCount])
									{
										FreighterCount++;
									}
									FreightersScannedList[FreighterCount] = 1;
									if (ShipNumber == FreighterID[FreighterWithNegator])
									{
										fMission.mDisplayMessage(kEvent3Correct_msg, kScienceScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kEvent2Wrong_msg, kScienceScriptMessage);
									}
									PointToNearestFreighter (*this);
								}
								break;
						}
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			if (fEventID == kPlayerTeam)
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
			}
			// case falls thru
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kMountainTeam)
			{
				MountainDestroyed++;
			}

			if (Ship->mGetTeam() == kDefenseTeam)
			{
				DefenseDestroyed++;
				if (DefenseTeam->mGetShipsInPlayCount() == 0)
				{
					AllDefensesDestroyed = true;
					if (FreighterTeam->mGetShipsInPlayCount() == 0)
					{
						fMission.mDisplayMessage(kEvent6_1_msg, kScienceScriptMessage);
						TestObjectives (*this);
					}
					else
					{
						fMission.mDisplayMessage(kEvent6_2_msg, kScienceScriptMessage);
					}					
				}
				else
				{
					PointToNearestPlatform (*this);
				}
			}

			if (Ship->mGetTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
				int32 ShipNumber = Ship->mGetShipID();
				if (ShipNumber == FreighterID[FreighterWithNegator])
				{
					fMission.mDisplayMessage(kEvent4Destroyed_msg, kScienceScriptMessage);
					tShipInfo* Ship;
					for (Ship = PlayerTeam->mGetFirstShipInPlay(); Ship != NULL; Ship = PlayerTeam->mGetNextShipInPlay())
					{
						Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0);
					}
				}

				if (FreighterTeam->mGetShipsInPlayCount() == 0)
				{
					AllFreightersDestroyed = true;
					if (DefenseTeam->mGetShipsInPlayCount() == 0)
					{
						fMission.mDisplayMessage(kEvent5_1_msg, kScienceScriptMessage);
						TestObjectives (*this);
					}
					else
					{
						fMission.mDisplayMessage(kEvent5_2_msg, kScienceScriptMessage);
						PointToNearestPlatform (*this);
					}
				}
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyCommButtonPress:
			{
				MountainCommMessages(*this, fEventID, Ship, kTimerTurnsSet_6);
			}
			break;
	}

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus(status | kCampaignCompleted);
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (FreighterDestroyed * 20) + (DefenseDestroyed * 5);

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (VictoryLevel <= kVictory)
	{
		fMission.mAddToDebriefingList (kAstoundingVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDevastatingDefeat_msg);
	}

	// LNF try this out:  adding results of primary objectives
	int32 SuccessCount = 0;
	( AllFreightersDestroyed ) ? fMission.mAddToDebriefingList( kAllFreightersDestroyed_msg ) : SuccessCount++;
	( AllDefensesDestroyed ) ? fMission.mAddToDebriefingList( kAllDefensesDestroyed_msg ) : SuccessCount++;
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	int32 FailureCount = 0;
	fMission.mAddToDebriefingList( kFailedObjectives_msg );
	( !AllFreightersDestroyed ) ? fMission.mAddToDebriefingList( kNotAllFreightersDestroyed_msg ) : FailureCount++;
	( !AllDefensesDestroyed ) ? fMission.mAddToDebriefingList( kNotAllDefensesDestroyed_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	// LNF need Final text?
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships to create
// by checking the CLASS of each ship the player has
// randomly adding or subtracting based on how MANY ships the player has
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(-2, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 2);
	}
	else
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}


//---------------------------------------------------------------------------------------------
// Mountain Comm Messages
void MountainCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTimerIntervalTypes TimerType)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (Kaboom == 0)
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kMountainHail_1_msg, kMountainHail_3_msg)); 
			}
			else // Kaboom == 1
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kMountainHail_1_msg, kMountainHail_3_msg)); 
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in CommButtonTimeout turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() < DamageThreshold)
			{
				if (Elite == kIronFang)
				{
					tScript.fMission.mDisplayMessage(kMountainSurrenderOKIF_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kMountainSurrenderOKRC_msg, kMissionScriptMessage);
				}
			}
			else // DamagePercent > DamageThreshold
			{
				tScript.fMission.mDisplayMessage(kMountainSurrenderDamaged_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;

		case kButtonTaunt_text:
			if (Elite == kIronFang)
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kMountainTauntIF_1_msg, kMountainTauntIF_3_msg)); 
			}
			else
			{
				tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kMountainTauntRC_1_msg, kMountainTauntRC_3_msg));
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (TimerType, CommButtonTimeout);
			break;
	}
}

//---------------------------------------------------------------------------------------------
//
void SetAllRelations (tScript& tScript)
{
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kMountainTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kMountainTeam, kDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kMountainTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kDefenseTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determine if the player has met all of the required objectives
void TestObjectives (tScript& tScript)
{
	if ((AllFreightersDestroyed) && (AllDefensesDestroyed))
	{
		int32 MapEdgeX, MapEdgeY;
		tScript.fMission.mGetMapDimensions (&MapEdgeX, &MapEdgeY);

		//put this line where it checks for the *primary* victory conditions
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompleted_msg, kMissionScriptMessage );
		FindClosestMapEdge(tScript, tScript.fPlayerShip, MapEdgeX, MapEdgeY);
	}
}

//---------------------------------------------------------------------------------------------
//
void PointToNearestFreighter (tScript& tScript)
{
	int32 shipx, shipy;
	int32 dist = 9999999;
	int32 found = -1;
	int32 i = NumberOfFreighters;

	tScript.fPlayerShip->mGetGridPosition (&shipx, &shipy);
	while (i--)
	{
		if (FreightersScannedList[i] == 0)
		{
			int32 distx = shipx - FreighterPostions[i].x;
			int32 disty = shipy - FreighterPostions[i].y;
			int32 new_dist = (distx * distx) + (disty * disty);
			if (new_dist < dist)
			{
				dist = new_dist;
				found = i;
			}
		}
	}
	if (found != -1)
	{
		tScript.fMission.mSetGoalIndicator (kTrue, FreighterPostions[found].x, FreighterPostions[found].y);
	}
	else
	{
		PointToNearestPlatform (tScript);
	}
}

//---------------------------------------------------------------------------------------------
//
void PointToNearestPlatform (tScript& tScript)
{
	int32 shipx, shipy;
	int32 dist = 9999999;
	int32 found = -1;
	int32 i = NumberOfPlatforms;

	tScript.fPlayerShip->mGetGridPosition (&shipx, &shipy);
	while (i--)
	{
		if (PlatformsDestroyedList[i] == 0)
		{
			int32 distx = shipx - PlatformPostions[i].x;
			int32 disty = shipy - PlatformPostions[i].y;
			int32 new_dist = (distx * distx) + (disty * disty);
			if (new_dist < dist)
			{
				dist = new_dist;
				found = i;
			}
		}
	}
	if (found != -1)
	{
		tScript.fMission.mSetGoalIndicator (kTrue, PlatformPostions[found].x, PlatformPostions[found].y);
	}
}

