#include "scriptinterface.h"

#include "Mul_StarbaseAssaultText.h"
#include "Mul_StarbaseAssaultMaps.h"

int32 TeamAShips = 0;
int32 TeamBShips = 0;
int32 HumanTeams[tEntityMgr::kMaxNumberofPlayers];
bool BaseDestroyed = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kBaseTeam,
};

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = MissSpecAllShips;			// max number of ships per player
	Specs->fWait = 6;								// max number of players
	Specs->fEnableSupplyDock = kEnableSupplyDock | kDisableTeamC | kDisableTeamD | kDisableTeamE | kDisableTeamF;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams (info);

	// keep track of which teams the humans are on
	TeamAShips = 0;
	TeamBShips = 0;
	iTruth ATeamKlingons = kTrue;
	eRaceName BRace = kKlingon;
	for (int32 i= 0; i< tEntityMgr::kMaxNumberofPlayers; i++)
	{
		HumanTeams[i] = -1;		// -1 == empty
		if( info.fMembersInGame[i] != kPlayerTypeNoOne )	// AI teams are human, too...
		{
			HumanTeams[i] = info.fMemberTeams[i];

			if( info.fMemberTeams[i] == kTeam1 )
			{
				TeamAShips += info.fMemberShipCounts[i];
				if (info.fMemberRaces[i] != kKlingon)
				{
					ATeamKlingons = kFalse;
				}
			}
			else if( info.fMemberTeams[i] == kTeam2 )
			{
				TeamBShips += info.fMemberShipCounts[i];
				BRace = info.fMemberRaces[i];
			}
		}
	}

	// set the base race
	// if there are team B players, use the first B player race, else pick
	// Klingon unless all players on Team A are Klingon, then use Federation
	if (ATeamKlingons && TeamBShips == 0)
	{
		BRace = kFederation;
	}
	fMission.mSetTeamRace( kBaseTeam, BRace );

	for (i= 0; i< tEntityMgr::kMaxNumberofPlayers; i++)
	{
		if( info.fMemberTeams[i] == kTeam1 )
		{
			fMission.mSetTeamRelations( eTeamTypes(i), kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		}
		else
		{
			fMission.mSetTeamRelations( eTeamTypes(i), kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
		}

	}

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// Team A uses G-L, Team B uses U-Z
	if( fMission.mGetTeamHandle( kPlayerTeam )->mGetMultiplayerTeam() == kTeam2 )
	{
		Ship.fShipPosition += kStartPosition_U - kStartPosition_G;
	}

	Ship.fShipPosition |= kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	// only the host needs to create the AI ships
	if (fThisScriptIsTheHost)
	{
		tTeamInfo* Team;
		int32 position;

		// create friendly AI ships first
		for (int32 i = 0; i < fMission.fTotalAlliedCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_AlliedTeams, i);
			position = Team->mGetMultiplayerTeam() + kStartPosition_G;
			Team->mCreateCPUFleet (20, position | kGroupAround);
		}

		// create enemy AI ships last
		for (i = 0; i < fMission.fTotalEnemyCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, i);
			position = Team->mGetMultiplayerTeam() + kStartPosition_U;
			Team->mCreateCPUFleet (20, position | kGroupAround);
		}

		// Make the base
		BaseTeam->mCreateShip( kMaxBPV, kClassStarBase, 100, kStartPosition_M );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* Team;

	// game ends for everyone when the last man is standing
	switch (Event)
	{
		case kNotifyAtTimedInterval:
			if( PlayerTeam->mGetMultiplayerTeam() == kTeam1 )
			{
				Ship = BaseTeam->mGetFirstShip();
				fMission.mDisplayMessage( kStartingA_msg, kNonCommScriptMessage, Ship );
			}
			else
			{
				fMission.mDisplayMessage( kStartingB_msg, kNonCommScriptMessage );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kBaseTeam )
			{
				BaseDestroyed = true;

				if( PlayerTeam->mGetMultiplayerTeam() == kTeam1 )
				{
					return kPlayerWins;
				}
				else
				{
					return kPlayerLoses;
				}
			}
			else
			{
				Team = fMission.mGetTeamHandle( Ship->mGetTeam() );

				if( Team->mGetMultiplayerTeam() == kTeam1 )
				{
					TeamAShips--;

					if( TeamAShips == 0 )
					{
						if( PlayerTeam->mGetMultiplayerTeam() == kTeam2 )
						{
							return kPlayerWins;
						}
						else
						{
							return kPlayerLoses;
						}
					}
				}
			}

			if(( Ship->mGetTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				fMission.mDisplayMessage( kYouAreDead_msg );
			}
			break;

		case kNotifyHostChanged:
			fThisScriptIsTheHost = kTrue;
			break;

		case kNotifyDisengaged:
			fEventID = Ship->mGetTeam();

			if (Ship == fPlayerShip)
			{
				fMission.mDisplayMessage( kYouDisengaged_msg );
				fMission.mGetTeamHandle (kPlayerTeam)->mDisengageShips (kByMapEdge, kAbsolutePriority);
				return kPlayerLoses;
			}

			Team = fMission.mGetTeamHandle( Ship->mGetTeam() );

			if( Team->mGetMultiplayerTeam() == kTeam1 )
			{
				TeamAShips--;

				if( TeamAShips == 0 )
				{
					if( PlayerTeam->mGetMultiplayerTeam() == kTeam2 )
					{
						return kPlayerWins;
					}
					else
					{
						return kPlayerLoses;
					}
				}
			}
			break;

/*		case kNotifyPlayerLeaving:
			if (fEventID != kPlayerTeam)
			{
				HumanTeams[fEventID] = -1;
				int32 first_team = -1;
				for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
				{
					if (HumanTeams[i] != -1)
					{
						if (first_team == -1)
						{
							first_team = HumanTeams[i];
						}
						if (HumanTeams[i] != first_team)
						{
							return kGameInProgress;		// still have two teams in the game
						}
					}
				}
				// must be only one team left, and we're on it - we may win!
				if( PlayerTeam->mGetMultiplayerTeam() == kTeam2 )
				{
					return kPlayerWins;
				}
			}
			break;
*/	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	// Team A
	if( PlayerTeam->mGetMultiplayerTeam() == kTeam1 )
	{
		if( BaseDestroyed == true )
		{
			return kAstoundingVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}

	// Team B
	if( PlayerTeam->mGetMultiplayerTeam() == kTeam2 )
	{
		if( BaseDestroyed == false )
		{
			return kAstoundingVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}

	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	// Team A
	if( PlayerTeam->mGetMultiplayerTeam() == kTeam1 )
	{
		if( BaseDestroyed == true )
		{
			fMission.mAddToDebriefingList( kWinnerA_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kLoserA_msg );
		}
	}

	// Team B
	if( PlayerTeam->mGetMultiplayerTeam() == kTeam2 )
	{
		if( BaseDestroyed == false )
		{
			fMission.mAddToDebriefingList( kWinnerB_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kLoserB_msg );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


