#include "scriptinterface.h"

#include "Mul_BattlefestLiteText.h"
#include "Mul_BattlefestLiteMaps.h"

int32 PlayerShipClass = 0;
iTruth RespawningLocalShip = kFalse;
bool BlackHoleMap = false;
int32 StartX = 0;
int32 StartY = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
};

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
//
#define kMaxShipsToRespawn 3
#include "teamtracker.h"
tTeamTrackerClass fTeamTracker;

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = 0;				// max number of ships per player
	Specs->fWait = 6;								// max number of players
	Specs->fEnableSupplyDock = kDisableAI;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// we need to know if this is a black hole map for respawning purposes
	if( info.fPreferedMap >= FirstBlackHoleMap )
	{
		BlackHoleMap = true;
	}

	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams (info);

	// setup the team tracker class
	fTeamTracker.mInitialize (kPlayerTeam, info, kMaxShipsToRespawn);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
//	Ship.fShipPosition = (Ship.fTeam + kStartPosition_G) | kGroupAround;
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 TeamNumber = PlayerTeam->mGetMultiplayerTeam();
	int32 X = 0;
	int32 Y = 0;
	int32 BigX = 0;
	int32 BigY = 0;
	int32 SmallX = 40;
	int32 SmallY = 40;
	int32 NewX = 0;
	int32 NewY = 0;

	if( BlackHoleMap == true )
	{
		NewX = StartX;
		NewY = StartY;
	}
	else if( Mode > 0 )
	{
		// find average starting point for respawning
		int32 MapX, MapY;
		fMission.mGetMapDimensions (&MapX, &MapY);

		for( int32 i = 0; i <= kReserved; i++ )
		{
			Ship = fMission.mGetTeamHandle( kTeam1 + i )->mGetFirstShipInPlay();
			if( Ship != NULL )
			{
				Ship->mGetGridPosition( &X, &Y );
				if( X > BigX ){ BigX = X; }
				if( X < SmallX ){ SmallX = X; }
				if( Y > BigY ){ BigY = Y; }
				if( Y < SmallY ){ SmallY = Y; }
			}
		}

		NewX = ( SmallX + BigX ) / 2;
		NewY = ( SmallY + BigY ) / 2;

		if(( SmallX == BigX ) && ( SmallY == BigY ))
		{
			NewX = NewX + fMission.mRandomInt32( 3, 4 );
			NewY = NewY + fMission.mRandomInt32( 3, 4 );
		}
		NewX = max (1, min (MapX, NewX));
		NewY = max (1, min (MapY, NewY));
	}

	switch( Mode )
	{
		case 0:
			PlayerTeam->mCreateShip( kMaxBPV, kClassFrigate, 100, kStartPosition_G + TeamNumber );
			break;

		case 1:
			PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 100, kStartPosition_Specified, NewX, NewY );
			break;

		case 2:
			PlayerTeam->mCreateShip( kMaxBPV, kClassDreadnought, 100, kStartPosition_Specified, NewX, NewY );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	// configure player ship(s)
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mGetGridPosition( &StartX, &StartY );

	fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
	fMission.mSetTimedInterval( kTimerLimitSet, 5 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (RespawningLocalShip && Ship->mGetTeam() == kPlayerTeam)
	{
		Ship->mSetShipPlayerControlled();
		RespawningLocalShip = kFalse;
		Ship->mSetTargetable( kFalse );
		fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
	}
	// test for new leader
	fTeamTracker.mTrackLeader (&fMission);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kControl_msg );
					break;

				case kTimerTurnsSet:
					Ship = PlayerTeam->mGetFirstShipInPlay();

					if( Ship != NULL )
					{
						Ship->mSetTargetable( kTrue );
					}
					break;
			}
			break;

		case kNotifyDisengaged:
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
			Ship->mDamageSystem( kDamageInternalsWithPoints, 300 );
			break;

		case kNotifyAtDeath:
			fTeamTracker.mShipDied (Ship->mGetTeam());
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerShipClass++;
				if( PlayerShipClass < kMaxShipsToRespawn )
				{
					RespawningLocalShip = kTrue;
					mCreateMissionShips( PlayerShipClass );
					fMission.mSetTimedInterval( kTimerLimitSet, 5 );
				}
				else
				{
					fMission.mDisplayMessage( kYouAreDead_msg );
				}
			}
			break;

		case kNotifyPlayerLeaving:
			fTeamTracker.mTeamGone (eTeamID(fEventID), &fMission);
			break;
	}

	return fTeamTracker.mGetTeamsStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( fTeamTracker.mGetTeamsStatus() == kPlayerWins )
	{
		return kAstoundingVictory;
	}
	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( fTeamTracker.mGetTeamsStatus() == kPlayerWins )
	{
		fMission.mAddToDebriefingList( kWinner_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kLoser_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


