#include "scriptinterface.h"

#include "Mul_PassTheTribblesText.h"
#include "Mul_PassTheTribblesMaps.h"

int32 MaxTribbleCount = 16;
int32 BirthTimeDelay = 3;
int32 TribbleCount = 1;
bool TransporterStunned = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,
};

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
//

#include "teamtracker.h"
tTeamTrackerClass fTeamTracker;

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = MissSpecOneShip;				// max number of ships per player
	Specs->fWait = 6;								// max number of players
	Specs->fEnableSupplyDock = kDisableAI;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams( info );

	// setup the team tracker class
	fTeamTracker.mInitialize( kPlayerTeam, info, 0 );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	int32 position = fMission.mGetTeamHandle (kPlayerTeam)->mGetMultiplayerTeam() + kStartPosition_G;
	Ship.fShipPosition = position | kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tShipInfo* Ship;

	// configure player ship(s)
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();

	Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo | kNotifyTransportedItemFrom | kNotifyDamage );
	Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );

	fMission.mSetTimedInterval( kTimerTurnsSet, 3 );	// start the tribble babies
	fMission.mSetTimedInterval( kTimerLimitSet, 2 );	// starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	int32 RandomMessage = 0;

	// game ends for everyone when the last man is standing
	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					RandomMessage = fMission.mRandomInt32( kStarting1_msg, kStarting3_msg );
					fMission.mDisplayMessage( RandomMessage, kNonCommScriptMessage );
					break;

				case kTimerTurnsSet: // tribble babies (doubles the number of tribbles on a ship)
					if( PlayerTeam->mGetShipsInPlayCount() != 0 )
					{
						int ExtraTribbles = 0;

						if( TribbleCount > 4 )
						{
							ExtraTribbles =  fMission.mRandomInt32( 1, 4 );
						}
						else if( TribbleCount > 0 )
						{
							ExtraTribbles = fMission.mRandomInt32( 1, TribbleCount );
						}
						else
						{
							ExtraTribbles = 1;
						}
						
						for( int loop = 0;loop < ExtraTribbles;loop++ )
						{
							Ship = PlayerTeam->mGetFirstShip();
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransTribbles, 1 );
						}

						TribbleCount += ExtraTribbles;
						if( TribbleCount > MaxTribbleCount )
						{
							Ship = PlayerTeam->mGetFirstShip();
							Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
							Ship->mDamageSystem( kDamageInternalsWithPoints, 5000.0 );	// self-destruct!
						}
						else
						{
							fMission.mDisplayMessage( kTribbleDouble_msg, kScienceScriptMessage );
							fMission.mDisplayMessage( kTribbleCount0_msg + TribbleCount, kMissionScriptMessage );
							fMission.mSetTimedInterval( kTimerTurnsSet, BirthTimeDelay );
						}
					}
					break;
			}
			break;

		case kNotifyTransportedItemFrom:	// to the player ship
			Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );

			if( fEventItem1 == kTransTribbles )
			{
				fMission.mDisplayMessage( kGotTribble_msg );
				TribbleCount++;

				if( TribbleCount > MaxTribbleCount )
				{
					Ship = PlayerTeam->mGetFirstShip();
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
					Ship->mDamageSystem( kDamageInternalsWithPoints, 5000.0 );	// self-destruct!
				}
			}
			break;

		case kNotifyTransportedItemTo:		// from the player ship
			Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );

			if( fEventItem1 == kTransTribbles )
			{
				TribbleCount--;

				if( TribbleCount < 0 )
				{
					TribbleCount = 0;
				}
			}
			break;

		case kNotifyAtDeath:
			RandomMessage = fMission.mRandomInt32( kTooMany1_msg, kTooMany6_msg );
			fMission.mDisplayMessage( RandomMessage, kNonCommScriptMessage, Ship );
			fTeamTracker.mShipDied( Ship->mGetTeam() );

			if( Ship->mGetTeam() == kPlayerTeam )
			{
				fMission.mDisplayMessage( kYouAreDead_msg );
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetDamagePercent() > 50 )
			{
				Ship->mDamageSystem( kDamageWeaponsAndSystems, 0.0 );
				// Don't Change this.  If you do the counts will be off
				TribbleCount ++;
				if( TribbleCount > MaxTribbleCount )
				{
					Ship = PlayerTeam->mGetFirstShip();
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
					Ship->mDamageSystem( kDamageInternalsWithPoints, 5000.0 );	// self-destruct!
				}
				else
				{
					fMission.mDisplayMessage( kTribbleDouble_msg, kScienceScriptMessage );
					fMission.mDisplayMessage( kTribbleCount0_msg + TribbleCount, kMissionScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet, BirthTimeDelay );
				}

			}
			
			Ship->mSetShipCallbackEvents( kNotifyDamage );
			break;

		case kNotifyHostChanged:
			fThisScriptIsTheHost = kTrue;
			break;

		case kNotifyDisengaged:
			fEventID = Ship->mGetTeam();
			if (fEventID == kPlayerTeam)
			{
				fMission.mDisplayMessage( kYouDisengaged_msg );
				return kPlayerLoses;
			}
			// case falls thru
		case kNotifyPlayerLeaving:
			fTeamTracker.mTeamGone( eTeamID( fEventID ), NULL );
			break;
	}
	return fTeamTracker.mGetTeamsStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if (fTeamTracker.mGetTeamsStatus() == kPlayerWins)
	{
		return kAstoundingVictory;
	}

	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( fTeamTracker.mGetTeamsStatus() == kPlayerWins )
	{
		fMission.mAddToDebriefingList( kWinner_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kLoser_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


