#include "scriptinterface.h"				// must have this as the first include

#include "Rom_Special1Text.h"					// must have these two includes
#include "Rom_Special1Maps.h"

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
// To insert a new script project into the workspace:
//		1. create a new folder off of the \SFCScripts folder.
//		2. copy template files (.cpp and .h) into new folder.
//		3. rename copied template files the same name as new folder.
//		4. start MSVC++, load SFCScripts workspace.
//		5. click Project on main menu, select Add to Project->New.
//		6. click Project on New menu, select Win32 Dynamic-Link Library.
//		7. click Add to Current Project.
//		8. click Dependency of _RebuildAllScripts.
//		9. enter Project Name (same as new folder name), click OK.
//		10. select Empty DLL Project, click Finish. Click OK to confirm.
//		11. right click on new project in workspace window.
//		12. select Add Files to Project.
//		13. highlight all files in new folder (.cpp and .h). Click OK.
//		14. click Project on main menu, select Dependencies.
//		15. set new project dependent on _ScriptInterface. Click OK.
//		16. click Project on main menu, select Settings...
//		17. select new project and set Settings For to Release.
//		18. click C/C++ tab, select Preprocessor category.
//		19. add to Addition Include Dir: ..\scriptinterface,..\includes
//		20. click Link tab, select General category.
//		21. in Output File Name box change .dll to .scr.
//		22. add to Object/Library Modules box: scriptinterface.lib StarFleet1.lib StarFleet2.lib
//		23. select Input category.
//		24. add to Ignore Libraries box: libcd.lib,libcmt.lib
//		25. add to Additional Library Path: ..\libs,..\,..\scriptinterface
//		26. open new .cpp file and change text and map include file names.
//		27. select Build->Set Active Configuration..., set to Release.
//		28. Compile.
//		29. copy .scr file from \Release folder into \Assets\Scripts folder of SFC.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for human player teams.
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.
	kRescueTeam,
	kOrionTeam,

};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed
const float kHullDamageLevel			= 15.0;
const int32 kMaxVictoryLevel			= 140;
const int32 kHintFrequency				= 15;
const int32 kOrbitingDistance			= 80;
const int32 kCrisisTeamSendDelay		= 3;
const int32 kCrisisTeamDisarmDelay		= 5;
const int32 kMarinesFindTheBombDelay	= 3;

bool gSentCrisisTeam = false;
bool gDisarmedBomb = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

// Set to true when enemy has been destroyed, disabled, or forced to retreat
iTruth		gFreighterRescued			= kFalse;
iTruth		gFreighterCaptured			= kFalse;
// Set to true when the freighter has been scanned
iTruth		gFreighterScanned			= kFalse;
iTruth		gFreighterTractored			= kFalse;

int32		gEnemyShipDestroyed			= 0;
int32		gEnemyShipCaptured			= 0;
iTruth		gRescueTeamDestroyed		= kFalse;
int32		gVictoryLevel				= 0;
iTruth		gReachOrbit					= kFalse;

int32		BonusBalancePoints			= 0;

// -----------------------------------------------------------------------------------
// 
// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;				// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// use values from eMissSpecSect enum in missionspec.h
	Specs->fPlayerRace = kRomulanMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fEnemyRace = kOrionMission;					// use values from eMissSpecRace enum in missionspec.h
	Specs->fNumShips = MissSpecAllShips;				// use values from Bit masks in missionspec.h
	Specs->fMinBPV = 0;									// player must have at least this much BPV to play this mission
	Specs->fMaxBPV = kMaxBPV;							// player cannot have more than this much BPV to play this mission
	Specs->fMinDate = 6;								// player can play this mission starting on this year date (date starts at 0
														//   and advances 1 year at a time)
	Specs->fMaxDate = 9999;								// player will not be able to play this mission after this year date
	Specs->fWait = kUniqueMission;						// has multiple uses depending on which fMissionType was specified above:
														// for kCampaignMission:
														//		specifies how many years Dynaverse will wait before selecting 
														//		this misison again. when fWait == kUniqueMission then this mission
														//		is played only once per campaign.
														// for kTutorialMission & kTechnicalMission:
														//		specifies this missions position in the academy menu. must be 0 - 5.
														// for kHistoricalMission & kMultiplayerMission:
														//		unused - should be 0
	Specs->fEnableSupplyDock = kEnableSupplyDock;		// enables/disables the Supply dock button in the briefing screen.
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 120, 80, 20, 0);

	// can change the race of enemy or ally if needed (but NOT player)

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kRescueTeam, kRomulan);
	fMission.mSetTeamRace (kOrionTeam, kOrion);

	// must setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kRescueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRescueTeam, kOrionTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode == 0.
// also can be called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* RescueTeam = fMission.mGetTeamHandle (kRescueTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);

	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .6;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 0.8;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);
	int32 FreighterBPV = 0;

	int32 NumberEnemyShips = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount());
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.70f;
			FreighterBPV = 35;
			NumberEnemyShips--;
		}
		break;
	case 1:
		{
			FreighterBPV = 45;
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
			FreighterBPV = 70;
			NumberEnemyShips++;
		}
		break;
	}

	RescueTeam->mCreateShip (FreighterBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_M);
	OrionTeam->mCreateFleet(NumberEnemyShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_I | kGroupAround);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* RescueTeam = fMission.mGetTeamHandle (kRescueTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tShipInfo* Ship;
	tShipInfo* Target;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = OrionTeam->mGetCombatBPV() + RescueTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mTrackTacticalDistance( kOrbitingDistance, 20, 10, kTrue ); // Set tracking distance to the star

	
	// configure enemy ship(s)
	for (Ship = RescueTeam->mGetFirstShip(); Ship != NULL; Ship = RescueTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		// deep scan rescue team from within distance 20 for 4 turns
		DeepScanTracker.mSetTarget (Ship, 20.0, 2);
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 1 );
		Ship->mChangeName( kAuriga_name );
		Ship->mSetShipMaxSpeed( 12 );

		Target = OrionTeam->mGetNextShip();
	}

	for (Ship = OrionTeam->mGetFirstShip (); Ship != NULL; Ship = OrionTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	OrionTeam->mGetFirstShip()->mSetShipTarget( PlayerTeam->mGetFirstShip(), kHighPriority );

	// display the Start Mission text
	fMission.mSetTimedInterval( kTimerLimitSet_6, 4 );
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	static int32 hintMsgID = 0;

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* RescueTeam = fMission.mGetTeamHandle (kRescueTeam);

	switch (Event)
	{
		case kNotifyBeginTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
					if (( Ship->mGetTeam() == kPlayerTeam )
						&& ( TractorTarget->mGetTeam() == kRescueTeam ))
					{
						if ( gDisarmedBomb == false )
						{
							fMission.mDisplayMessage( kFreighterScanned_msg, kScienceScriptMessage );

							TractorTarget->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
							TractorTarget->mDamageSystem( kDamageInternalsWithPoints, 1023.0 );

							Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 60 );
						}
					}
				}
			}
			break;

		case kNotifyEndTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
				}
			}
			break;

		case kNotifyDamage:
		{
			tShipInfo* HostileShip = fMission.mFindShipID( fEventID );
			float damage = Ship->mGetDamagePercent();

			if ( HostileShip != NULL )
			{
				if ( HostileShip->mGetTeam() == kPlayerTeam )
				{
					if (Ship->mGetDamagePercent() > kHullDamageLevel)
					{
						OrionTeam->mClearCommands();
						OrionTeam->mDisengageShips(kByMapEdge, kVeryHighPriority);
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;
		}

		case kNotifyAtDistance:
		{
			if (Ship->mGetTrackingDistance() == kOrbitingDistance)
			{
			}
			break;
		}


		case kNotifyAtDeath:
		{
			switch (Ship->mGetOriginalTeam())
			{
				case kOrionTeam:
				{
					gEnemyShipDestroyed++;

					if (OrionTeam->mGetShipsInPlayCount() == 0)
					{
						// Display enemy destroyed message
						gFreighterRescued = kTrue;
						fMission.mDisplayMessage(kLastEnemyDestroyed_msg, kMissionScriptMessage);

						if ( RescueTeam->mGetShipsInPlayCount() > 0 )
						{
							fMission.mDisplayMessage(kEscortFreighter_msg, kMissionScriptMessage);

							if ( gReachOrbit == kFalse )
							{
								// Make the freighter fly toward player ship
								for (tShipInfo* tempShip = RescueTeam->mGetFirstShip(); tempShip != NULL; tempShip = RescueTeam->mGetNextShip())
								{
									tempShip->mFollowEntity(PlayerTeam->mGetFirstShip(), kHighPriority);
								}

								// LNF maybe put this back later
								//fMission.mSetTimedInterval( kTimerLimitSet, kHintFrequency );
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							else
							{
								return kPlayerWins;
							}
						}
						else
						{
							Ship = RescueTeam->mGetFirstShip();

							if( Ship != NULL )
							{
								if( Ship->mTestShipStatus( kCaptured ) == kTrue)
								{
									return kPlayerWins;
								}
								else
								{
									return kPlayerLoses;
								}
							}

							// LNF the freighter was destroyed
							return kPlayerLoses;
						}
					}
					break;
				}

				case kRescueTeam:
				{
					if (RescueTeam->mGetShipsInPlayCount() == 0)
					{
						if( OrionTeam->mGetShipsInPlayCount() == 0 )
						{
							return kPlayerLoses;
						}
						else
						{
							gRescueTeamDestroyed = kTrue;
							fMission.mDisplayMessage(kRescueTeamDestroyed_msg, kMissionScriptMessage);
						}
					}
					break;
				}

			}

			if ( Ship->mGetTeam() == kPlayerTeam )
			{
				if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}

			break;
		}

		case kNotifyOnCapture:
			{
				switch (Ship->mGetOriginalTeam())
				{
					case kOrionTeam:
					{
						gEnemyShipCaptured++;

						if (OrionTeam->mGetShipsInPlayCount() == 0)
						{
							// Display enemy destroyed message
							gFreighterRescued = kTrue;
							fMission.mDisplayMessage(kLastEnemyDestroyed_msg, kMissionScriptMessage);

							if ( RescueTeam->mGetShipsInPlayCount() > 0 )
							{
								fMission.mDisplayMessage(kEscortFreighter_msg, kMissionScriptMessage);
							
								if ( gReachOrbit == kFalse )
								{
									// Make the freighter fly toward player ship
									for (Ship = RescueTeam->mGetFirstShip(); Ship != NULL; Ship = RescueTeam->mGetNextShip())
									{
										Ship->mFollowEntity(PlayerTeam->mGetFirstShip(), kHighPriority);
									}

									// LNF maybe put this back later
									//fMission.mSetTimedInterval( kTimerLimitSet, kHintFrequency );
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
								}
								else
								{
									return kPlayerWins;
								}
							}
						}
						break;
					}
				
					case kRescueTeam:
					{
						gFreighterRescued = kTrue;
						gFreighterCaptured = kTrue;
						Ship->mSetCommButton (1, kButtonHail_text);
						fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);

						// keep the ship from disengaging
						Ship->mClearCommands();
						
						// Mention that we should send the crisis team
						fMission.mSetTimedInterval( kTimerLimitSet_5, kMarinesFindTheBombDelay );


						// LNF if the freighter hasn't been disarmed already...

						// Make the freighter fly toward player ship
						for (Ship = RescueTeam->mGetFirstShip(); Ship != NULL; Ship = RescueTeam->mGetNextShip())
						{
							Ship->mFollowEntity(PlayerTeam->mGetFirstShip(), kHighPriority);

							// LNF set the max speed of the freighter so it doesn't run into the player's ship too easily
							Ship->mSetShipMaxSpeed( 9 );
						}

						// LNF maybe put these back in
						//fMission.mSetTimedInterval( kTimerLimitSet, kHintFrequency );

						// check if the player gets too close to the freighter
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );

						return kGameInProgress;
						break;
					}
				}
			}	
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyAtTimedInterval:
		{
			switch (fEventID)
			{
				case kTimerLimitSet:
				{
					if (gFreighterScanned == kFalse && gFreighterTractored == kFalse)
					{
						if (hintMsgID == 0)
						{
							hintMsgID = kHint1_msg;
						}
						else
						{
							hintMsgID++;
							if (hintMsgID > kHint6_msg)
							{
								hintMsgID = kHint1_msg;
							}
						}
						fMission.mDisplayMessage(hintMsgID, kMissionScriptMessage);
						fMission.mSetTimedInterval( kTimerLimitSet, kHintFrequency );
					}
					break;
				}

				case kTimerLimitSet_2:
				{
					// if the player gets within 1.5 of the Freighter, destroy the freighter
					// and damage the player's ship

					tShipInfo* playerShip = PlayerTeam->mGetFirstShip(); 
					tShipInfo* Ship;

					if ( gDisarmedBomb == false )
					{
						for (Ship = RescueTeam->mGetFirstShip(); Ship != NULL; Ship = RescueTeam->mGetNextShip())
						{
							int32 distance = playerShip->mGetTacticalDistance(Ship);
							if (distance < 1.5)
							{
								Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
								Ship->mDamageSystem( kDamageInternalsWithPoints, 5000.0 );	// self-destruct!
								playerShip->mDamageSystem( kDamageWeaponsSystemsAndShields, 60 );

								return kGameInProgress;
							}
						}
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					break;
				}

				case kTimerLimitSet_3:
				{
					if ( gFreighterCaptured )
					{
						if ( gRescueTeamDestroyed == false )
						{
							// shall we send over crisis
							fMission.mDisplayMessage( kFreighterCrisisAway, kScienceScriptMessage );

							// set timer to check for successful disarming of bomb
							fMission.mSetTimedInterval( kTimerLimitSet_4, kCrisisTeamDisarmDelay );
						}
					}
					else
					{
						// we need to capture 
						fMission.mDisplayMessage( kFreighterCrisisNoCapture, kScienceScriptMessage );
					}
					break;
				}	

				case kTimerLimitSet_4:
				{
					if ( gRescueTeamDestroyed == false )
					{
						// disarming of bomb

						eOfficerRank scienceOfficerRank = fPlayerShip->mGetOfficerRank( kScienceOfficer );
						eOfficerRank securityOfficerRank = fPlayerShip->mGetOfficerRank( kSecurityOfficer );

						int32 combinedOfficerStrength = scienceOfficerRank + securityOfficerRank;
						int32 baseDifficulty = 10;

						int32 adjustedDifficulty = baseDifficulty - combinedOfficerStrength;

						/*
						Officer ranks vary from 1 to 5
						What do we want?
						Junior + Junior ~ 25%
						Junior + Senior ~ 40%
						Senior + Senior ~ 70%
						Senior + Veteran ~ 80%
						Veteran + Veteran ~ 90%
						Veteran + Legendary ~ 95%
						Legendary + Legendary = 100%
						*/

						int32 DifLev = fMission.mGetDifficultyLevel();

						switch ( DifLev )
						{
						case 0:
							{
								adjustedDifficulty -= 2;
								break;
							}
						case 1:
							{
								// adjustedDifficulty unchanged
								break;
							}
						case 2:
							{
								adjustedDifficulty += 2;
								break;
							}
						}

						int32 dieRoll = fMission.mRandomInt32 (1, 10);

						iTruth disarmSuccess = kFalse;

						if ( ( dieRoll - adjustedDifficulty) > 0 )
						{
							disarmSuccess = kTrue;
						}

						if ( disarmSuccess )
						{
							gDisarmedBomb = true;
							fMission.mDisplayMessage( kFreighterCrisisTeamSuccessful, kScienceScriptMessage );
							gReachOrbit = kTrue;

							// if there are no more Orions left, end the mission
							if ( OrionTeam->mGetShipsInPlayCount() == 0 )
							{
								return kPlayerWins;
							}
						}
						else
						{
							fMission.mDisplayMessage( kFreighterCrisisTeamFail, kScienceScriptMessage );
							fMission.mSetTimedInterval( kTimerTurnsSet_7, 1 );
						}
					}
					break;
				}

				case kTimerLimitSet_5:
				{
					// The marines have found the bomb
					gFreighterScanned = true;
	
					if ( gSentCrisisTeam == false )
					{
						gSentCrisisTeam = true;

						fMission.mDisplayMessage( kMarinesFoundBomb, kScienceScriptMessage );

						// Mention that we should send the crisis team
						fMission.mSetTimedInterval( kTimerLimitSet_3, kCrisisTeamSendDelay );
					}
					else
					{
						// already know about bomb, just disarm it
						fMission.mSetTimedInterval( kTimerLimitSet_4, kCrisisTeamSendDelay );
					}
					break;
				}

				case kTimerLimitSet_6:
					// display the start mission message
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					fMission.mSetTeamRelations (kPlayerTeam, kRescueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kRescueTeam, kOrionTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					break;

				case kTimerLimitSet_10:
				{
					eDeepScanFlags	flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

					if ( RescueTeam->mGetShipsInPlayCount() > 0 )
					{
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player to turn scan on
								fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on
								// (Object points to entity or if NULL then MapPoint has location)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on
								// (Object points to entity or if NULL then MapPoint has location)
								// notify player that timer is now off
								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off
								// (Object points to entity or if NULL then MapPoint has location)
								break;
						}
					}
					break;	
				}
				
				case kTimerCountdownSet:
				{
					eDeepScanFlags	flag;
					tMapPoint		MapPoint;
					tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag, &MapPoint);

					if (flag == kInRange_ScanOn)
					{
						// player has successfully scanned the object or location
						fMission.mDisplayMessage( kFreighterScanned_msg, kScienceScriptMessage );
						gFreighterScanned = kTrue;

						if ( gSentCrisisTeam == false )
						{
							gSentCrisisTeam = true;

							// Mention that we should send the crisis team
							fMission.mSetTimedInterval( kTimerLimitSet_3, kCrisisTeamSendDelay );
						}
						else
						{
							// the player already knows about the bomb, just disarm it
							fMission.mSetTimedInterval( kTimerLimitSet_4, kCrisisTeamSendDelay );
						}

						// (Object points to entity or if NULL then MapPoint has location)
						if (Object)
						{
							DeepScanTracker.mRemoveTarget (Object);
						}
						else
						{
							DeepScanTracker.mRemoveLocation (MapPoint.x, MapPoint.y);
						}
					}
					break;
				}
				
				case kTimerTurnsSet_7:
				{
					// blow up freighter and damage player's ship if they are too close
					tShipInfo* playerShip = PlayerTeam->mGetFirstShip(); 
					tShipInfo* Ship;

					for (Ship = RescueTeam->mGetFirstShip(); Ship != NULL; Ship = RescueTeam->mGetNextShip())
					{
						Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 5000.0 );	// self-destruct!

						int32 distance = playerShip->mGetTacticalDistance(Ship);
						if (distance < 1.5)
						{
							playerShip->mDamageSystem( kDamageWeaponsSystemsAndShields, 60 );
						}
					}
					return kPlayerLoses;
					break;
				}

			}
			break;
		}

		case kNotifyCommButtonPress:
		{
			switch (fEventID)
			{
				case kButtonHail_text:
				{
					if ( Ship->mGetTeam() == kPlayerTeam)
					{
						if ( Ship->mGetOriginalTeam() == kRescueTeam )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						if ( Ship->mGetOriginalTeam() == kOrionTeam )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						// Set Surrender and Taunt button
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
					}

					switch (Ship->mGetTeam())
					{
						case kRescueTeam:
							{
								if ( gFreighterRescued )
								{
									if ( gFreighterScanned )
									{
										fMission.mDisplayMessage(kRescueHailRescuedScan_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kRescueHailRescuedNoScanNoTractor_msg, kMissionScriptMessage);
									}
								}
								else
								{
									fMission.mDisplayMessage(kRescueHailRescued_msg, kMissionScriptMessage);
								}
								break;
							}
						case kOrionTeam:
							{
								int32 RanHailReply = fMission.mRandomInt32 (kEnemyHail1_msg, kEnemyHail4_msg);
								fMission.mDisplayMessage(RanHailReply, kMissionScriptMessage);
								break;
							}
					}
					
					// Set Surrender and Taunt button
					Ship->mSetCommButton (1, kButtonTaunt_text);
					Ship->mSetCommButton (2, kButtonSurrender_text);
					break;
				}

				case kButtonTaunt_text:
				{
					if ( Ship->mGetTeam() == kPlayerTeam)
					{
						if ( Ship->mGetOriginalTeam() == kRescueTeam )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
					}

					switch (Ship->mGetTeam())
					{
						case kRescueTeam:
							{
								fMission.mDisplayMessage(kRescueHailTaunt_msg, kMissionScriptMessage);
								break;
							}
						case kOrionTeam:
						{
							int32 RanHailReply = fMission.mRandomInt32 (kEnemyHailTaunt1_msg, kEnemyHailTaunt4_msg);
							fMission.mDisplayMessage(RanHailReply, kMissionScriptMessage);
							break;
						}
					}
					// Set Surrender and Taunt button
					Ship->mSetCommButton (1, kButtonTaunt_text);
					Ship->mSetCommButton (2, kButtonSurrender_text);
					break;
				}

				case kButtonSurrender_text:
				{
					if ( Ship->mGetTeam() == kPlayerTeam)
					{
						if ( Ship->mGetOriginalTeam() == kRescueTeam )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						if ( Ship->mGetOriginalTeam() == kOrionTeam )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						// Set Surrender and Taunt button
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
					}

					switch (Ship->mGetTeam())
					{
						case kRescueTeam:
							{
								fMission.mDisplayMessage(kRescueHailTaunt_msg, kMissionScriptMessage);
								break;
							}
						case kOrionTeam:
						{
							float damage = Ship->mGetDamagePercent(); 
							if (damage <= kHullDamageLevel)
							{
								fMission.mDisplayMessage(kEnemyHailSurrenderGoodHull_msg, kMissionScriptMessage);
							}
							else
							{
								fMission.mDisplayMessage(kEnemyHailSurrenderBadHull_msg, kMissionScriptMessage);
							}
							break;
						}
					}
					// Set Surrender and Taunt button
					Ship->mSetCommButton (1, kButtonTaunt_text);
					Ship->mSetCommButton (2, kButtonSurrender_text);
					break;
				}
			}
			break;
		}
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;
	int32 VictoryPoints = 0;

	VictoryPoints += gEnemyShipDestroyed * 20;
	VictoryPoints += gEnemyShipCaptured * 25;
	if (gFreighterScanned)
	{
		VictoryPoints += 15;
	}
	
	if (gRescueTeamDestroyed)
	{
		if (gFreighterScanned)
		{
			VictoryPoints += 15;
		}
		else
		{
			VictoryPoints -= 100;
		}
	}

	if (gReachOrbit)
	{
		VictoryPoints += 70;
	}


	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints >= 40)
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints >= 20)
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		fMission.mScheduleMission( kNextMission_text );
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


