#include "scriptinterface.h"				// must have this as the first include

#include "Rom_Special2Text.h"					// must have these two includes
#include "Rom_Special2Maps.h"


//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for human player teams.
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.
	kOrionTeam,
	kFreighterTeam,
	kOrionFleetTeam,
	kRomFleetTeam,

};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed

// -----------------------------------------------------------------------------------
//

int32			gOrionsDestroyed = 0;
int32			gOrionsCaptured = 0;
iTruth			gFreighterCaptured = false;
iTruth			gFreighterDestroyed = false;
iTruth			gFreighterRescued = false;
iTruth			gOrionsHuntingPlayer = false;
int32			kTurnsTillOrionFleet = 7;
int32			kTurnsTillRomFleet = 2;
int32			kSecondsTillOpening = 2;
iTruth			gOrionFleetSummoned = false;
int32			BonusBalancePoints = 0;

const float kHullDamageLevel			= 50.0;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

enum eMissionMode
{
	kStart,
	kOrionFleetArrives,
	kTalShiarArrives,
};
eMissionMode MissionMode = kStart;

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;				// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// use values from eMissSpecSect enum in missionspec.h
	Specs->fPlayerRace = kRomulanMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fEnemyRace = kOrionMission;					// use values from eMissSpecRace enum in missionspec.h
	Specs->fNumShips = MissSpecAllShips;				// use values from Bit masks in missionspec.h
	Specs->fMinBPV = 0;									// player must have at least this much BPV to play this mission
	Specs->fMaxBPV = kMaxBPV;							// player cannot have more than this much BPV to play this mission
	Specs->fMinDate = 9999;								// player can play this mission starting on this year date (date starts at 0
														//   and advances 1 year at a time)
	Specs->fMaxDate = 9999;								// player will not be able to play this mission after this year date
	Specs->fWait = kUniqueMission;						// has multiple uses depending on which fMissionType was specified above:
														// for kCampaignMission:
														//		specifies how many years Dynaverse will wait before selecting 
														//		this misison again. when fWait == kUniqueMission then this mission
														//		is played only once per campaign.
														// for kTutorialMission & kTechnicalMission:
														//		specifies this missions position in the academy menu. must be 0 - 5.
														// for kHistoricalMission & kMultiplayerMission:
														//		unused - should be 0
	Specs->fEnableSupplyDock = 0;						// enables/disables the Supply dock button in the briefing screen.
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 120, 80, 20, 0);

	// can change the race of enemy or ally if needed (but NOT player)

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFreighterTeam, kRomulan);
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	fMission.mSetTeamRace (kOrionFleetTeam, kOrion);
	fMission.mSetTeamRace (kRomFleetTeam, kRomulan);

	// must setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionFleetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kRomFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRomFleetTeam, kOrionFleetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomFleetTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomFleetTeam, kFreighterTeam, kTeamHostile, kGoalCapture, 1.0);//***

	fMission.mSetTeamRelations (kFreighterTeam, kRomFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kFreighterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFreighterTeam, kOrionFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionFleetTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode == 0.
// also can be called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionFleetTeam = fMission.mGetTeamHandle (kOrionFleetTeam);
	tTeamInfo* RomFleetTeam = fMission.mGetTeamHandle (kRomFleetTeam);

	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .6;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.0;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);
	int32 FreighterBPV = 0;

	// pick a random number of ships
	int32 NumberEnemyShips = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount());
	int32 NumberOrionFleetShips = fMission.mRandomInt32 (3, PlayerTeam->mGetShipsInPlayCount() + 2 );
	int32 NumberRomFleetShips = fMission.mRandomInt32 (3, PlayerTeam->mGetShipsInPlayCount() + 2 );

	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.70f;
				FreighterBPV = 35;
				NumberEnemyShips--;
				kTurnsTillOrionFleet += 2;
				kTurnsTillRomFleet -= 1;
			}
			break;

		case 1:
			{
				FreighterBPV = 45;
			}
			break;

		case 2:
			{
				TotalBPV *= 1.5f;
				FreighterBPV = 70;
				NumberEnemyShips++;
				kTurnsTillOrionFleet -= 1;
				kTurnsTillRomFleet += 2;
			}
			break;
	}

	if ( MissionMode == kStart )
	{
		FreighterTeam->mCreateShip (FreighterBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_Z);
		// Any size ship within the appropriate BPV and a 33% chance of a variant
		OrionTeam->mCreateFleet(NumberEnemyShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_T | kGroupAround);
		return;
	}

	if ( MissionMode == kOrionFleetArrives )
	{
		gOrionFleetSummoned = true;
		OrionFleetTeam->mCreateFleet(NumberOrionFleetShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_Y | kGroupAround);
		return;
	}

	if ( MissionMode == kTalShiarArrives )
	{
		// LNF try to put the romulans where the Orions popped in, otherwise use J
		RomFleetTeam->mCreateFleet(NumberRomFleetShips, (TotalBPV * 1.33f), kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_Y | kGroupAround);
		return;
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionFleetTeam = fMission.mGetTeamHandle (kOrionFleetTeam);
	tTeamInfo* RomFleetTeam = fMission.mGetTeamHandle (kRomFleetTeam);
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = OrionTeam->mGetCombatBPV() + FreighterTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}
	
	// configure Freighter
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetFinalDestination( 1, 2 );

		switch( fMission.mGetDifficultyLevel() )
		{
			case 0:	// easy
				Ship->mSetShipMaxSpeed( 8 );
				break;

			case 1: // medium
				Ship->mSetShipMaxSpeed( 10 );
				break;
			
			case 2:	// hard
				Ship->mSetShipMaxSpeed( 12 );
				break;
		}
	}

	// configure Orion escorts
	for (Ship = OrionTeam->mGetFirstShip (); Ship != NULL; Ship = OrionTeam->mGetNextShip ())
	{
		Ship->mDefendEntity( FreighterTeam->mGetFirstShip(), 1.0, 1.0, kHighPriority );
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	// start a timer to bring in the Orion fleet
	fMission.mSetTimedInterval( kTimerTurnsSet, kTurnsTillOrionFleet );

	// set team relations
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );

	// play opening message
	fMission.mSetTimedInterval( kTimerLimitSet_2, kSecondsTillOpening );
	
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* OrionFleetTeam = fMission.mGetTeamHandle (kOrionFleetTeam);
	tTeamInfo* RomFleetTeam = fMission.mGetTeamHandle (kRomFleetTeam);

	switch ( MissionMode )
	{
		case kStart:
			{
			}
			break;

		// Orion Fleet Arrives
		case kOrionFleetArrives:
			{
				Ship->mSetCommButton (1, kButtonHail_text);
			}
			break;

		// Rom Fleet Arrives
		case kTalShiarArrives:
			{
				Ship->mSetCommButton (1, kButtonHail_text);
			}
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	static int32 hintMsgID = 0;

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionFleetTeam = fMission.mGetTeamHandle (kOrionFleetTeam);
	tTeamInfo* RomFleetTeam = fMission.mGetTeamHandle (kRomFleetTeam);

	switch (Event)
	{
		case kNotifyAtDestination:
			if ( Ship->mGetTeam() == kFreighterTeam )
			{
				Ship->mClearCommands();
				Ship->mDisengage( kByMapEdge, kAbsolutePriority );
			}
			break;

		case kNotifyAtTimedInterval:
		{
			switch (fEventID)
			{
				case kTimerLimitSet:
					// set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kRomFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kRomFleetTeam, kOrionFleetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kRomFleetTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kRomFleetTeam, kFreighterTeam, kTeamHostile, kGoalCapture, 1.0);//***

					fMission.mSetTeamRelations (kFreighterTeam, kRomFleetTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kFreighterTeam, kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFreighterTeam, kOrionFleetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionFleetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kOrionFleetTeam, kOrionTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					break;

				case kTimerLimitSet_2:
				{	// opening statement
					fMission.mDisplayMessage( kStartScenario_1_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kStartScenario_2_msg, kMissionScriptMessage );
					break;
				}

				case kTimerTurnsSet:
				{
					// Time for the OrionFleet
					if ( !gOrionFleetSummoned )
					{
						MissionMode = kOrionFleetArrives;
						mCreateMissionShips( MissionMode );
						fMission.mDisplayMessage( kOrionFleetRevealed, kScienceScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_2, kTurnsTillRomFleet );
					}
					break;
				}

				case kTimerTurnsSet_2:
				{
					// Time for the RomFleet
					MissionMode = kTalShiarArrives;
					mCreateMissionShips( MissionMode );
					fMission.mDisplayMessage( kRomFleetRevealed, kScienceScriptMessage );
					break;
				}				
			}
			break;
		}

		case kNotifyAtDeath:
		{
			switch (Ship->mGetOriginalTeam())
			{
				case kOrionTeam:
				{
					gOrionsDestroyed++;

					if (OrionTeam->mGetShipsInPlayCount() == 0)
					{
						// The Orion Escort Ships are gone, if the Orion reinforcements are not here yet lets
						// call them in.
						if ( !gOrionFleetSummoned )
						{
							MissionMode = kOrionFleetArrives;
							mCreateMissionShips( MissionMode );
							fMission.mDisplayMessage( kOrionFleetRevealed, kScienceScriptMessage );
							fMission.mSetTimedInterval( kTimerTurnsSet_2, kTurnsTillRomFleet );
						}
						else
						{
							// totally kicked the Orions butt
							if (OrionFleetTeam->mGetShipsInPlayCount() == 0)
							{
								gFreighterRescued = true;
								return kPlayerWins;
							}
						}						
					}
					break;
				}

				case kOrionFleetTeam:
				{
					gOrionsDestroyed++;
					if (OrionFleetTeam->mGetShipsInPlayCount() == 0)
					{
						// totally kicked the Orions butt
						if (OrionTeam->mGetShipsInPlayCount() == 0)
						{
							gFreighterRescued = true;
							return kPlayerWins;
						}
					}
					break;
				}

				case kFreighterTeam:
				{
					if (FreighterTeam->mGetShipsInPlayCount() == 0)
					{
						fMission.mDisplayMessage( kFreighterDestroyed, kScienceScriptMessage );
						gFreighterDestroyed = kTrue;
					}
					break;
				}
			}

			if ( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}
			return kGameInProgress;
			break;
		}

		case kNotifyOnCapture:
			{
				if ( Ship->mGetTeam() == kPlayerTeam)
				{
					if ( Ship->mGetOriginalTeam() == kFreighterTeam )
					{
						gFreighterCaptured = false;
						fMission.mSetTeamRelations (kOrionTeam, kFreighterTeam, kTeamAtWar, kGoalCapture, 1.0);
						fMission.mSetTeamRelations (kOrionFleetTeam, kFreighterTeam, kTeamAtWar, kGoalCapture, 1.0);
					}
				}

				switch (Ship->mGetOriginalTeam())
				{
					case kOrionTeam:
					{
						gOrionsCaptured++;
						fMission.mAddToBonusPrestige( kCaptureBonus );

						Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 1023 );
						break;
					}

					case kOrionFleetTeam:
					{
						gOrionsCaptured++;
						fMission.mAddToBonusPrestige( kCaptureBonus );

						Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
						Ship->mDamageSystem( kDamageInternalsWithPoints, 1023 );
						break;
					}
									
					case kFreighterTeam:
					{
						gFreighterCaptured = true;

						Ship->mClearCommButton( 1 );
						Ship->mClearCommButton( 2 );
						FreighterTeam->mDisengageShips(kByMapEdge, kHighPriority);

						FreighterTeam->mGetFirstShip()->mSetShipMaxSpeed( 31 );
						fMission.mAddToBonusPrestige( kCaptureBonus );
						fMission.mDisplayMessage( kFreighterCaptured, kScienceScriptMessage );

						if ( !gOrionFleetSummoned )
						{
							MissionMode = kOrionFleetArrives;
							mCreateMissionShips( MissionMode );
							fMission.mDisplayMessage( kOrionFleetRevealed, kScienceScriptMessage );
							fMission.mSetTimedInterval( kTimerTurnsSet_2, kTurnsTillRomFleet );
						}
					}
					break;
				}	

				return kGameInProgress;
			}
			break;

		case kNotifyDisengaged:
			{
				if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					fMission.mDisplayMessage( kLeaveSystem, kScienceScriptMessage );
					return kPlayerWins;
				}
				if( Ship->mGetOriginalTeam() == kFreighterTeam )
				{
					if ( gFreighterCaptured )
					{
						// good the player has it, lets see if they manage to leave the map as well.
						fMission.mDisplayMessage( kFreighterEscapesGood, kScienceScriptMessage );
						gFreighterRescued = true;
					}
					else
					{
						fMission.mDisplayMessage( kFreighterEscapesBad, kScienceScriptMessage );
						return kPlayerLoses;

					}

				}

			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
		{
			switch (fEventID)
			{
				case kButtonHail_text:
				{
					if ( Ship->mGetTeam() == kPlayerTeam)
					{
						if ( Ship->mGetOriginalTeam() == kFreighterTeam )
						{
							fMission.mDisplayMessage(kOrionFreighterCaptured_msg, kMissionScriptMessage);
						}
						if ( ( Ship->mGetOriginalTeam() == kOrionTeam ) || ( Ship->mGetOriginalTeam() == kOrionFleetTeam ) )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						// Set Surrender and Taunt button
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
					}

					switch (Ship->mGetTeam())
					{
						case kRomFleetTeam:
							{
								fMission.mDisplayMessage(kRomFleetHail_msg, kMissionScriptMessage);
								// Set Surrender and Taunt button
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
							}				
							break;

						case kFreighterTeam:
							{
								fMission.mDisplayMessage(kOrionFreighterHail_msg, kMissionScriptMessage);
								// Set Surrender and Taunt button
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
							}				
							break;

						case kOrionTeam:
							{
								if ( gOrionsHuntingPlayer == false )
								{
									fMission.mDisplayMessage(kEnemyHailSeq, kMissionScriptMessage);
									gOrionsHuntingPlayer = true;

									Ship->mSetCommButton (1, kButtonHail_text);

									// LNF need to change this back to 31?
									FreighterTeam->mGetFirstShip()->mSetShipMaxSpeed( 5 );

									OrionTeam->mClearCommands();
									fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

									for (Ship = OrionTeam->mGetFirstShip (); Ship != NULL; Ship = OrionTeam->mGetNextShip ())
									{
									}
								}
								else
								{
									int32 RanHailReply = fMission.mRandomInt32 (kEnemyHail1, kEnemyHail4);
									fMission.mDisplayMessage(RanHailReply, kMissionScriptMessage);
									// Set Surrender and Taunt button
									Ship->mSetCommButton (1, kButtonTaunt_text);
									Ship->mSetCommButton (2, kButtonSurrender_text);
								}
							}
							break;
					}					
					break;
				}

				case kButtonTaunt_text:
				{
					if ( Ship->mGetTeam() == kPlayerTeam)
					{
						if ( Ship->mGetOriginalTeam() == kFreighterTeam )
						{
							fMission.mDisplayMessage(kOrionFreighterCaptured_msg, kMissionScriptMessage);
						}
						if ( ( Ship->mGetOriginalTeam() == kOrionTeam ) || ( Ship->mGetOriginalTeam() == kOrionFleetTeam ) )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						// Set Surrender and Taunt button
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
					}

					switch (Ship->mGetTeam())
					{
						case kRomFleetTeam:
							{
								fMission.mDisplayMessage(kRomFleetHail_msg, kMissionScriptMessage);
								// Set Surrender and Taunt button
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
							}				
							break;
						case kFreighterTeam:
							{
								fMission.mDisplayMessage(kOrionFreighterHail_msg, kMissionScriptMessage);
								// Set Surrender and Taunt button
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
							}				
							break;
						case kOrionTeam:
						case kOrionFleetTeam:
						{
							int32 RanHailReply = fMission.mRandomInt32 (kEnemyHailTaunt1_msg, kEnemyHailTaunt4_msg);
							fMission.mDisplayMessage(RanHailReply, kMissionScriptMessage);
							// Set Surrender and Taunt button
							Ship->mSetCommButton (1, kButtonTaunt_text);
							Ship->mSetCommButton (2, kButtonSurrender_text);
							break;
						}
					}
					break;
				}

				case kButtonSurrender_text:
				{
					if ( Ship->mGetTeam() == kPlayerTeam)
					{
						if ( Ship->mGetOriginalTeam() == kFreighterTeam )
						{
							fMission.mDisplayMessage(kOrionFreighterCaptured_msg, kMissionScriptMessage);
						}
						if ( ( Ship->mGetOriginalTeam() == kOrionTeam ) || ( Ship->mGetOriginalTeam() == kOrionFleetTeam ) )
						{
							fMission.mDisplayMessage(kEnemyHailBoarded_msg, kMissionScriptMessage);
						}
						// Set Surrender and Taunt button
						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
					}

					switch (Ship->mGetTeam())
					{
						case kRomFleetTeam:
							{
								fMission.mDisplayMessage(kRomFleetHail_msg, kMissionScriptMessage);
								// Set Surrender and Taunt button
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
							}				
							break;
						case kFreighterTeam:
							{
								fMission.mDisplayMessage(kOrionFreighterHail_msg, kMissionScriptMessage);
								// Set Surrender and Taunt button
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
							}				
							break;
						case kOrionTeam:
						case kOrionFleetTeam:
						{
							float damage = Ship->mGetDamagePercent(); 
							if (damage <= kHullDamageLevel)
							{
								fMission.mDisplayMessage(kEnemyHailSurrenderGoodHull_msg, kMissionScriptMessage);
							}
							else
							{
								fMission.mDisplayMessage(kEnemyHailSurrenderBadHull_msg, kMissionScriptMessage);
							}
							// Set Surrender and Taunt button
							Ship->mSetCommButton (1, kButtonTaunt_text);
							Ship->mSetCommButton (2, kButtonSurrender_text);
						}
						break;
					}
					break;
				}
			}
			break;
		}
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;
	int32 VictoryPoints = 0;

	VictoryPoints += gOrionsDestroyed * 5;
	VictoryPoints += gOrionsCaptured * 10;

	if (gFreighterCaptured)
	{
		VictoryPoints += 50;
	}

	if (gFreighterRescued)
	{
		VictoryPoints += 20;
	}
	
	if (gFreighterDestroyed)
	{
		VictoryPoints += 20;
	}
	

	if (VictoryPoints >= 85)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints >= 40)
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints >= 20)
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		fMission.mScheduleMission( kNextMission_text );
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


