#include "scriptinterface.h"				// must have this as the first include

#include "Rom_Special3Text.h"					// must have these two includes
#include "Rom_Special3Maps.h"

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
// To insert a new script project into the workspace:
//		1. create a new folder off of the \SFCScripts folder.
//		2. copy template files (.cpp and .h) into new folder.
//		3. rename copied template files the same name as new folder.
//		4. start MSVC++, load SFCScripts workspace.
//		5. click Project on main menu, select Add to Project->New.
//		6. click Project on New menu, select Win32 Dynamic-Link Library.
//		7. click Add to Current Project.
//		8. click Dependency of _RebuildAllScripts.
//		9. enter Project Name (same as new folder name), click OK.
//		10. select Empty DLL Project, click Finish. Click OK to confirm.
//		11. right click on new project in workspace window.
//		12. select Add Files to Project.
//		13. highlight all files in new folder (.cpp and .h). Click OK.
//		14. click Project on main menu, select Dependencies.
//		15. set new project dependent on _ScriptInterface. Click OK.
//		16. click Project on main menu, select Settings...
//		17. select new project and set Settings For to Release.
//		18. click C/C++ tab, select Preprocessor category.
//		19. add to Addition Include Dir: ..\scriptinterface,..\includes
//		20. click Link tab, select General category.
//		21. in Output File Name box change .dll to .scr.
//		22. add to Object/Library Modules box: scriptinterface.lib StarFleet1.lib StarFleet2.lib
//		23. select Input category.
//		24. add to Ignore Libraries box: libcd.lib,libcmt.lib
//		25. add to Additional Library Path: ..\libs,..\,..\scriptinterface
//		26. open new .cpp file and change text and map include file names.
//		27. select Build->Set Active Configuration..., set to Release.
//		28. Compile.
//		29. copy .scr file from \Release folder into \Assets\Scripts folder of SFC.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for human player teams.
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.
	kDivitryTeam,
	kRokyrTeam,
	
	kDivitryEscortTeam,
	kRokyrEscortTeam,

	kBombFreighterTeam,
};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed
// -----------------------------------------------------------------------------------

//iTruth	gDivitryCaptured	= kFalse;
//iTruth	gDivitryExecuted	= kFalse;
iTruth	gFreighterLaunched	= kFalse;
int32	gOrionsDestroyed	= 0;
int32	gOrionsCaptured		= 0;
int32	gFreighterDestroyed	= 0;

bool gAtWarWithDivitry = false;
bool gAtWarWithRokyr = false;
bool gAtWarWithDivitryEscort = false;
bool gDivitryCaptured = false;
bool gDivitryExecuted = false;
bool gCloseToRokyr = false;
bool gTractoringDivitry = false;
bool gAllOrionsDestroyed = false; // win condition
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

const float kRokyrThreatDistance = 15.0f;
const float kRokyrFollowDistance = 25.0f;
const float kDivitryThreatDistance = 15.0f;
const float kDivitryFollowDistance = 25.0f;
const float kDistanceFromRokyr = 40.0f;
const int32 kPlanetX = 24;
const int32 kPlanetY = 14;
const float kReleaseFreighterBomb = 20.0f;

tShipInfo* CapturedDivitry;

void divitryPatrolRoute( tScript& tScript, tShipInfo* Ship );
eGameStatus checkWinCondition( tScript& tScript );

enum eDivitryState
{
	kNoCapture = 0,
	kDivitryCaptured,
	kExchangeInProgress,
	kExchangeCompleted,
	kExchangeFailed,
	kDivitryExecuted
};
eDivitryState divitryState = kNoCapture;

enum eBombState
{
	kInOrbit = 0,
	kOnTheWay,
	kIntoPlanet,
	kNeutralized
};
eBombState bombState = kInOrbit;

// 
// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;				// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// use values from eMissSpecSect enum in missionspec.h
	Specs->fPlayerRace = kRomulanMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fEnemyRace = kOrionMission;					// use values from eMissSpecRace enum in missionspec.h
	Specs->fNumShips = MissSpecAllShips;				// use values from Bit masks in missionspec.h
	Specs->fMinBPV = 0;									// player must have at least this much BPV to play this mission
	Specs->fMaxBPV = kMaxBPV;							// player cannot have more than this much BPV to play this mission
	Specs->fMinDate = 9999;								// player can play this mission starting on this year date (date starts at 0
														//   and advances 1 year at a time)
	Specs->fMaxDate = 9999;								// player will not be able to play this mission after this year date
	Specs->fWait = kUniqueMission;						// has multiple uses depending on which fMissionType was specified above:
														// for kCampaignMission:
														//		specifies how many years Dynaverse will wait before selecting 
														//		this misison again. when fWait == kUniqueMission then this mission
														//		is played only once per campaign.
														// for kTutorialMission & kTechnicalMission:
														//		specifies this missions position in the academy menu. must be 0 - 5.
														// for kHistoricalMission & kMultiplayerMission:
														//		unused - should be 0
	Specs->fEnableSupplyDock = kEnableSupplyDock;		// enables/disables the Supply dock button in the briefing screen.
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 250, 100, 20, 0);

	// can change the race of enemy or ally if needed (but NOT player)

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBombFreighterTeam, kOrion);

	fMission.mSetTeamRace (kDivitryTeam, kOrion);
	fMission.mSetTeamRace (kDivitryEscortTeam, kOrion);

	fMission.mSetTeamRace (kRokyrTeam, kOrion);
	fMission.mSetTeamRace (kRokyrEscortTeam, kOrion);

	// must setup team relations and target goals
//	fMission.mSetTeamRelations (kPlayerTeam, kDivitryTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
//	fMission.mSetTeamRelations (kPlayerTeam, kRokyrTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
//	fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
//	fMission.mSetTeamRelations (kPlayerTeam, kRokyrEscortTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
//	fMission.mSetTeamRelations (kPlayerTeam, kBombFreighterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode == 0.
// also can be called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam		 = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DivitryTeam		 = fMission.mGetTeamHandle (kDivitryTeam);
	tTeamInfo* RokyrTeam		 = fMission.mGetTeamHandle (kRokyrTeam);
	tTeamInfo* DivitryEscortTeam = fMission.mGetTeamHandle (kDivitryEscortTeam);
	tTeamInfo* RokyrEscortTeam	 = fMission.mGetTeamHandle (kRokyrEscortTeam);
	tTeamInfo* BombFreighterTeam = fMission.mGetTeamHandle (kBombFreighterTeam);

	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .8;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.2;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	int32 DifLev = fMission.mGetDifficultyLevel();

	int32 NumDivitryEscorts = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount() + 1);
	int32 NumRokyrEscorts = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount() + 1);

	int32 FreighterBPV = 45;


	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.70f;
			FreighterBPV = 35;
			NumDivitryEscorts--;
			NumRokyrEscorts--;
		}
		break;
	case 1:
		{
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
			FreighterBPV = 75;
			NumDivitryEscorts++;
			NumRokyrEscorts++;
		}
		break;
	}

	DivitryTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_T );

	BombFreighterTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_Z );

	RokyrTeam->mCreateShip( TotalBPV * 1.3, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_J );

	switch ( NumRokyrEscorts )
	{
	case 1:
	case 2:
		{
			RokyrEscortTeam->mCreateFleet(NumRokyrEscorts, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_X | kGroupAround );
			break;
		}
	case 3:
	case 4:
		{
			//EnemyTeam->mCreateFleet(2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_S | kGroupAround );
			RokyrEscortTeam->mCreateFleet(NumRokyrEscorts - 2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_X | kGroupAround );
			break;
		}
	case 5:
	case 6:
		{
			//EnemyTeam->mCreateFleet(2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_S | kGroupAround );
			//EnemyTeam->mCreateFleet(2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_R | kGroupAround );
			RokyrEscortTeam->mCreateFleet(NumRokyrEscorts - 2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_X | kGroupAround );
			break;
		}
	default:
		{
			RokyrEscortTeam->mCreateFleet(NumRokyrEscorts, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_X | kGroupAround );
			break;
		}
	}

	switch ( NumDivitryEscorts )
	{
	case 1:
	case 2:
		{
			DivitryEscortTeam->mCreateFleet(NumDivitryEscorts, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_Q | kGroupAround );
			break;
		}
	case 3:
	case 4:
		{
			//EnemyTeam->mCreateFleet(2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_S | kGroupAround );
			DivitryEscortTeam->mCreateFleet(NumDivitryEscorts - 2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_Q | kGroupAround );
			break;
		}
	case 5:
	case 6:
		{
			//EnemyTeam->mCreateFleet(2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_S | kGroupAround );
			//EnemyTeam->mCreateFleet(2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_R | kGroupAround );
			DivitryEscortTeam->mCreateFleet(NumDivitryEscorts - 2, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_Q | kGroupAround );
			break;
		}
	default:
		{
			DivitryEscortTeam->mCreateFleet(NumDivitryEscorts, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 0, kStartPosition_Q | kGroupAround );
			break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return ( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam		 = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DivitryTeam		 = fMission.mGetTeamHandle (kDivitryTeam);
	tTeamInfo* RokyrTeam		 = fMission.mGetTeamHandle (kRokyrTeam);
	tTeamInfo* DivitryEscortTeam = fMission.mGetTeamHandle (kDivitryEscortTeam);
	tTeamInfo* RokyrEscortTeam	 = fMission.mGetTeamHandle (kRokyrEscortTeam);
	tTeamInfo* BombFreighterTeam = fMission.mGetTeamHandle (kBombFreighterTeam);

	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = DivitryTeam->mGetCombatBPV() + RokyrTeam->mGetCombatBPV() + DivitryEscortTeam->mGetCombatBPV() + RokyrEscortTeam->mGetCombatBPV();
	// LNF BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}
	
	// configure Rokyr
	for (Ship = RokyrTeam->mGetFirstShip(); Ship != NULL; Ship = RokyrTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mChangeName( kRokyr_name );
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}

	// configure RokyrEscort ships
	for (Ship = RokyrEscortTeam->mGetFirstShip(); Ship != NULL; Ship = RokyrEscortTeam->mGetNextShip())
	{
		Ship->mDefendEntity( RokyrTeam->mGetFirstShip(), kRokyrThreatDistance, kRokyrFollowDistance, kLowPriority );
		Ship->mChangeName( kRokyrEscort_name );
	}

	// configure Divitry
	for (Ship = DivitryTeam->mGetFirstShip(); Ship != NULL; Ship = DivitryTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetCommButton (1, kButtonHail_text);
		divitryPatrolRoute( *this, Ship );
		Ship->mChangeName( kDivitry_name );
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}

	// configure DivitryEscort ships
	for (Ship = DivitryEscortTeam->mGetFirstShip(); Ship != NULL; Ship = DivitryEscortTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		//divitryPatrolRoute( *this, Ship );
		Ship->mDefendEntity( DivitryTeam->mGetFirstShip(), kDivitryThreatDistance, kDivitryFollowDistance, kLowPriority );
		Ship->mChangeName( kDivitryEscort_name );
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	}

	// configure the Freighter Bomb
	for (Ship = BombFreighterTeam->mGetFirstShip(); Ship != NULL; Ship = BombFreighterTeam->mGetNextShip())
	{
		// LNF is this a good thing?  The player might shoot at the freighter or something.
		Ship->mDamageSystem( kDamageMaxShields, 0.0f );
		Ship->mChangeName( kOrionFreighter_name );
		Ship->mSetShipAIControlled( kFalse );
		Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 16 );
	}

	// set relations so they don't fire on each other
	fMission.mSetTeamRelations (kDivitryTeam, kRokyrTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDivitryTeam, kDivitryEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDivitryTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDivitryTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRokyrTeam, kDivitryEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRokyrTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRokyrTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDivitryEscortTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kDivitryEscortTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRokyrEscortTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// set a timer to reset team relations
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );

	// set a timer to check distance to Rokyr
	fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{

	tTeamInfo* PlayerTeam		 = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* DivitryTeam		 = fMission.mGetTeamHandle (kDivitryTeam);
	tTeamInfo* RokyrTeam		 = fMission.mGetTeamHandle (kRokyrTeam);
	tTeamInfo* DivitryEscortTeam = fMission.mGetTeamHandle (kDivitryEscortTeam);
	tTeamInfo* RokyrEscortTeam	 = fMission.mGetTeamHandle (kRokyrEscortTeam);
	tTeamInfo* BombFreighterTeam = fMission.mGetTeamHandle (kBombFreighterTeam);

	switch (Event)
	{
		case kNotifyBeginTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
					if ( Ship->mGetTeam() == kPlayerTeam )
					{
						if ( TractorTarget->mGetOriginalTeam() == kDivitryTeam )
						{
							if ( gAtWarWithDivitry == false )
							{
								gAtWarWithDivitry = true;

								DivitryTeam->mClearCommands();
								DivitryEscortTeam->mClearCommands();

								fMission.mSetTeamRelations (kPlayerTeam, kDivitryTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							}

							if ( divitryState == kDivitryCaptured )
							{
								gTractoringDivitry = true;
							}
						}
						else if (( TractorTarget->mGetOriginalTeam() == kBombFreighterTeam )
								|| ( TractorTarget->mGetOriginalTeam() == kRokyrTeam ))
						{
							RokyrTeam->mClearCommands();
							RokyrEscortTeam->mClearCommands();

							fMission.mSetTeamRelations (kPlayerTeam, kRokyrTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kRokyrEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyEndTractor );
				}
			}
			break;

		case kNotifyEndTractor:
			{
				tShipInfo* TractorTarget = fMission.mFindShipID( fEventID );

				if ( TractorTarget != NULL )
				{
					if ( Ship->mGetTeam() == kPlayerTeam )
					{
						if ( TractorTarget->mGetOriginalTeam() == kDivitryTeam )
						{
							if ( divitryState == kDivitryCaptured )
							{
								gTractoringDivitry = false;
							}
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
				}
			}
			break;

		case kNotifyAtDestination:
			// LNF check when Divitry makes it to Rokyr's ship
			fMission.mDisplayMessage( kDivitryTransported, kScienceScriptMessage );

			return kPlayerWins;
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ((( Ship->mGetTeam() == kDivitryTeam )
							|| ( Ship->mGetTeam() == kDivitryEscortTeam ))
							&& ( gAtWarWithDivitry == false ))
						{
							DivitryTeam->mClearCommands();
							DivitryEscortTeam->mClearCommands();

							fMission.mSetTeamRelations (kPlayerTeam, kDivitryTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

							gAtWarWithDivitry = true;
							//gAtWarWithDivitryEscort = true;
						}
						else if ( divitryState == kExchangeInProgress )
						{
							if (( Ship->mGetTeam() == kDivitryTeam )
								|| ( Ship->mGetTeam() == kDivitryEscortTeam )
								|| ( Ship->mGetTeam() == kRokyrTeam )
								|| ( Ship->mGetTeam() == kRokyrEscortTeam )
								|| ( Ship->mGetTeam() == kBombFreighterTeam ))
							{
								fMission.mSetTeamRelations (kPlayerTeam, kRokyrTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								fMission.mSetTeamRelations (kPlayerTeam, kRokyrEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								fMission.mSetTeamRelations (kPlayerTeam, kBombFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

								divitryState = kExchangeFailed;
							}
						}
					}
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyOnCapture:
			{
				if ( Ship->mGetOriginalTeam() == kDivitryTeam )
				{
					Ship->mClearCommands();
					Ship->mSetShipMaxSpeed( 0 );

					fMission.mDisplayMessage( kCapturedDivitry, kMissionScriptMessage );
					gOrionsDestroyed++;
					gDivitryCaptured = true;

					divitryState = kDivitryCaptured;
					DivitryTeam->mGetFirstShip()->mClearCommButton( 1 );
					DivitryTeam->mGetFirstShip()->mClearCommButton( 2 );

					fMission.mSetTeamRelations (kPlayerTeam, kDivitryTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDivitryTeam, kDivitryEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDivitryTeam, kRokyrTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kDivitryTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					RokyrTeam->mGetFirstShip()->mSetCommButton( 1, kButtonExchange_text );
					RokyrTeam->mGetFirstShip()->mSetCommButton( 2, kButtonExecute_text );
				}
				else if ( Ship->mGetOriginalTeam() == kDivitryEscortTeam )
				{
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
					Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 23 );
				}
				else if ( Ship->mGetOriginalTeam() == kRokyrTeam )
				{
					gOrionsDestroyed++;
				}
				else if ( Ship->mGetOriginalTeam() == kRokyrEscortTeam )
				{
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
					Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 23 );
				}
				else if ( Ship->mGetOriginalTeam() == kBombFreighterTeam )
				{
				}

				return checkWinCondition( *this );
			}
			break;

		case kNotifyAtDeath:
			{
				if ( Ship->mGetOriginalTeam() == kDivitryTeam )
				{
					gOrionsDestroyed++;
				}
				else if ( Ship->mGetOriginalTeam() == kDivitryEscortTeam )
				{
					gOrionsDestroyed++;
				}
				else if ( Ship->mGetOriginalTeam() == kRokyrTeam )
				{
					gOrionsDestroyed++;
				}
				else if ( Ship->mGetOriginalTeam() == kRokyrEscortTeam )
				{
					gOrionsDestroyed++;
				}
				else if ( Ship->mGetOriginalTeam() == kBombFreighterTeam )
				{
					// the bomb runs into the planet
					if (( bombState == kOnTheWay )
						&& ( Ship->mGetTacticalDistance( kPlanetX, kPlanetY ) <= 0.0f ))
					{
						fMission.mDisplayMessage( kTheFreighterHitPlanet, kScienceScriptMessage );
						bombState = kIntoPlanet;

						// LNF is there something more serious here?
						return kPlayerLoses;
					}
					else if ( bombState <= kOnTheWay )
					{
						// the player blew up the freighter
						fMission.mDisplayMessage( kTheFreighterDestroyed, kScienceScriptMessage );
						fMission.mDisplayMessage( kRokyrDestroyPlanet_msg, kMissionScriptMessage );
						bombState = kNeutralized;
					}
				}
				else if ( Ship->mGetOriginalTeam() == kPlayerTeam )
				{
					if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						return kPlayerDestroyed;
					}
				}

				return checkWinCondition( *this );
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyAtTimedInterval:
			{
				switch (fEventID)
				{
					case kTimerLimitSet:
						fMission.mSetTeamRelations (kPlayerTeam, kDivitryTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kRokyrTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kRokyrEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kBombFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kDivitryTeam, kRokyrTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kDivitryTeam, kDivitryEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kDivitryTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kDivitryTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kRokyrTeam, kDivitryEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kRokyrTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kRokyrTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kDivitryEscortTeam, kRokyrEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kDivitryEscortTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mSetTeamRelations (kRokyrEscortTeam, kBombFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

						// set a timer to display a message to investigate Divitry's ships
						fMission.mSetTimedInterval( kTimerLimitSet_3, 7 );
						break;

					case kTimerLimitSet_2:
						{
							for (tShipInfo* tempShip = PlayerTeam->mGetFirstShip(); tempShip != NULL; tempShip = PlayerTeam->mGetNextShip())
							{
								if (( tempShip->mGetTacticalDistance( RokyrTeam->mGetFirstShip() ) <= kReleaseFreighterBomb )
									&& ( bombState < kOnTheWay ))
								{
									if ( gTractoringDivitry == false )
									{
										fMission.mDisplayMessage( kWitnessPlanetDeath, kMissionScriptMessage );
										
										// send freighter into planet
										BombFreighterTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
										BombFreighterTeam->mGetFirstShip()->mSetFinalDestination( kPlanetX, kPlanetY );
										BombFreighterTeam->mGetFirstShip()->mSetShipThrust( 31 );
										BombFreighterTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 0.0f );
										BombFreighterTeam->mGetFirstShip()->mSetShipAIControlled( kFalse );
										bombState = kOnTheWay;

										return kGameInProgress;
									}
									else
									{
										fMission.mDisplayMessage( kGotYourSon, kMissionScriptMessage );
										fMission.mDisplayMessage( kDestroyFreighterForSon, kMissionScriptMessage );

										// set timer to destroy freighter and win the mission
										fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
									}
								}
							}

							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
						break;

					case kTimerLimitSet_3:
						// set goal pointer temporarily and tell player to invetigate Divitry
						fMission.mSetGoalIndicator( DivitryTeam->mGetFirstShip() );
						fMission.mDisplayMessage( kShipsAtEdge_msg, kMissionScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_3, 3 );
						break;

					case kTimerTurnsSet:
						{
							// destroy the freighter
							BombFreighterTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
							BombFreighterTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPoints, 23 );
							gFreighterDestroyed = 1;
							bombState = kNeutralized;

							fMission.mDisplayMessage( kFreighterDestroyed, kScienceScriptMessage );
						}
						break;

					case kTimerTurnsSet_2:
						// destroy the bomb and win the game after tractoring Divitry to 
						// Rokyr and Hailing
						fMission.mDisplayMessage( kDivitryTransported, kScienceScriptMessage );

						BombFreighterTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100.0f );
						BombFreighterTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPoints, 23 );
						gFreighterDestroyed = 1;
						bombState = kNeutralized;

						fMission.mDisplayMessage( kFreighterDestroyed, kScienceScriptMessage );

						return kPlayerWins;
						break;

					case kTimerTurnsSet_3:
						fMission.mSetGoalIndicator( kFalse, 0, 0 );
						break;
				}
			}
			break;

			
		case kNotifyCommButtonPress:
		{
			switch (fEventID)
			{
				case kButtonExchange_text:
					RokyrTeam->mGetFirstShip()->mClearCommButton( 2 );
					divitryState = kExchangeInProgress;

					fMission.mDisplayMessage( kGotYourSon, kMissionScriptMessage );
					fMission.mDisplayMessage( kBrokerSonForPlanet, kMissionScriptMessage );

					// set a timer to blow up the freighter
					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );

					// set the Orion ships to be non-hostile to the player,
					// but if the player shoots at an Orion (except Divitry) set the Orions AtWar
					fMission.mSetTeamRelations (kPlayerTeam, kRokyrTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kRokyrEscortTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBombFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					// send Divitry's freighter toward Rokyr
					int32 RokyrX, RokyrY;
					RokyrTeam->mGetFirstShip()->mGetGridPosition( &RokyrX, &RokyrY );

					// LNF do I have to check this in case Divitry never makes it exactly to where Rokyr is, 
					// LNF or maybe Rokyr might move from his current location?
					DivitryTeam->mGetFirstShip()->mClearCommands();
					DivitryTeam->mGetFirstShip()->mSetShipMaxSpeed( 31 );
					DivitryTeam->mGetFirstShip()->mSetFinalDestination( RokyrX, RokyrY );
					break;

				case kButtonExecute_text:
					if (( DivitryTeam->mGetFirstShip()->mTestShipStatus( kShipCaptured ) )
						&& !( DivitryTeam->mGetFirstShip()->mTestShipStatus( kShipDisengaged ) )
						&& !( DivitryTeam->mGetFirstShip()->mTestShipStatus( kShipDestroyed ) ))
					{
						DivitryTeam->mGetFirstShip()->mDamageSystem( kDamageExcessDamageSystem, 100 );
						DivitryTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );

						RokyrTeam->mGetFirstShip()->mClearCommButton( 1 );
						fMission.mDisplayMessage( kYouMonster, kMissionScriptMessage );
						gDivitryExecuted = true;
						divitryState = kDivitryExecuted;

						if (!( BombFreighterTeam->mGetFirstShip()->mTestShipStatus( kShipCaptured ) )
							&& !( BombFreighterTeam->mGetFirstShip()->mTestShipStatus( kShipDisengaged ) )
							&& !( BombFreighterTeam->mGetFirstShip()->mTestShipStatus( kShipDestroyed ) ))
						{
							// send the bomb into the planet
							fMission.mDisplayMessage( kBombOnTheWay_msg, kScienceScriptMessage );
							BombFreighterTeam->mGetFirstShip()->mSetShipAIControlled( kTrue );
							BombFreighterTeam->mGetFirstShip()->mSetFinalDestination( kPlanetX, kPlanetY );
							BombFreighterTeam->mGetFirstShip()->mSetShipThrust( 31 );
							BombFreighterTeam->mGetFirstShip()->mSetShipAIControlled( kFalse );
							bombState = kOnTheWay;
						}

						// make all ships attack the player
						DivitryTeam->mClearCommands();
						RokyrTeam->mClearCommands();
						RokyrEscortTeam->mClearCommands();

						fMission.mSetTeamRelations (kPlayerTeam, kRokyrTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kRokyrEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kDivitryEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					}
					else
					{
						// LNF do I need to set this back up as a hail button?
						RokyrTeam->mGetFirstShip()->mClearCommButton( 1 );
					}
					break;

				case kButtonHail_text:
					switch (Ship->mGetTeam())
					{
						case kDivitryTeam:
							fMission.mDisplayMessage( kHailDivitry, kMissionScriptMessage );
							break;

						case kRokyrTeam:
							if (( fPlayerShip->mGetTacticalDistance( RokyrTeam->mGetFirstShip() ) >= kDistanceFromRokyr )
								&& ( gCloseToRokyr == false ))
							{
								// LNF check if the player is tractoring Divitry's freighter
								gCloseToRokyr = true;

								if( BombFreighterTeam->mGetShipsInPlayCount() == 0 )
								{
									fMission.mDisplayMessage( kHailNoresponse, kMissionScriptMessage );
								}
								else if ( gTractoringDivitry == true )
								{
									fMission.mDisplayMessage( kGotYourSon, kMissionScriptMessage );
									fMission.mDisplayMessage( kDestroyFreighterForSon, kMissionScriptMessage );

									// set timer to destroy freighter and win the mission
									fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
								}
								else
								{
									fMission.mDisplayMessage( kHailFromAfar, kMissionScriptMessage );
								}
							}
							else if ( divitryState < kDivitryCaptured )
							{
								fMission.mDisplayMessage( kHailMurder, kMissionScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( kHailNoresponse, kMissionScriptMessage );
								break;
							}
							break;

						default:
							fMission.mDisplayMessage( kHailEnemyOrion, kMissionScriptMessage );
					}
					
					// Set Surrender and Taunt button
					Ship->mSetCommButton (1, kButtonTaunt_text);
					Ship->mSetCommButton (2, kButtonSurrender_text);
					break;

				case kButtonTaunt_text:
					switch (Ship->mGetTeam())
					{
						case kDivitryTeam:							
							fMission.mDisplayMessage(fMission.mRandomInt32 (kTauntDivitry1, kTauntDivitry4), kMissionScriptMessage);
							break;

						case kRokyrTeam:
							fMission.mDisplayMessage( kGenericTaunt, kMissionScriptMessage );
							break;

						default:
							fMission.mDisplayMessage(fMission.mRandomInt32 (kEnemyHailTaunt1_msg, kEnemyHailTaunt4_msg), kMissionScriptMessage);
					}

					Ship->mClearCommButton( 2 );
					Ship->mSetCommButton( 1, kButtonHail_text );
					break;

				case kButtonSurrender_text:
					switch (Ship->mGetTeam())
					{
						case kDivitryTeam:
							fMission.mDisplayMessage( kRequestSurrenderDivitry, kMissionScriptMessage );
							break;

						case kRokyrTeam:
							fMission.mDisplayMessage( kRequestGenericSurrender, kMissionScriptMessage );
							break;

						default:
							fMission.mDisplayMessage( kRequestOrionSurrender, kMissionScriptMessage );
					}

					Ship->mSetCommButton( 1, kButtonHail_text );
					break;
			}
			break;
		}
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;
	int32 VictoryPoints = 0;


	
	// LNF for victory conditions, if the bomb freighter is still alive, player loses, and if
	// LNF the bomb freighter destroys planet player loses, and if the bomb is destroyed,
	// LNF but the player runs away, he loses....check how many orions are killed?

	// LNF so is the only way to win to either trade the kid for the bomb, or kill everything?

	VictoryPoints += gOrionsDestroyed * 20;

	if (( gAllOrionsDestroyed == true )
		|| ( divitryState = kExchangeCompleted ))
	{
		VictoryPoints = 100;
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints >= 70)
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints >= 40)
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints >= 20)
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set the patrol route for Divitry
void divitryPatrolRoute( tScript& tScript, tShipInfo* Ship )
{
	tTeamInfo* DivitryTeam		 = tScript.fMission.mGetTeamHandle (kDivitryTeam);
	tTeamInfo* DivitryEscortTeam = tScript.fMission.mGetTeamHandle (kDivitryEscortTeam);

	tMapPoint waypoints1[2]=
	{
		6, 27,
		6, 6
	};

	tMapPoint waypoints2[2]=
	{
		5, 25,
		5, 4
	};

	tMapPoint PatrolPoints[2][10];

	if ( Ship->mGetTeam() == kDivitryTeam )
	{
		PatrolPoints[0][0] = waypoints1[0];
		PatrolPoints[0][1] = waypoints1[1];
		Ship->mPatrol (kDivitryThreatDistance, 2, &PatrolPoints[0][0], kLowPriority);
	}
	else
	{
		PatrolPoints[1][0] = waypoints2[0];
		PatrolPoints[1][1] = waypoints2[1];
		Ship->mPatrol (kDivitryThreatDistance, 2, &PatrolPoints[1][0], kLowPriority);
	}
}

//---------------------------------------------------------------------------------------------
// check the win condition to exit the scenario
eGameStatus checkWinCondition( tScript& tScript )
{
	tTeamInfo* PlayerTeam		 = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* DivitryTeam		 = tScript.fMission.mGetTeamHandle (kDivitryTeam);
	tTeamInfo* RokyrTeam		 = tScript.fMission.mGetTeamHandle (kRokyrTeam);
	tTeamInfo* DivitryEscortTeam = tScript.fMission.mGetTeamHandle (kDivitryEscortTeam);
	tTeamInfo* RokyrEscortTeam	 = tScript.fMission.mGetTeamHandle (kRokyrEscortTeam);
	tTeamInfo* BombFreighterTeam = tScript.fMission.mGetTeamHandle (kBombFreighterTeam);

	if (( DivitryTeam->mGetShipsInPlayCount() == 0 )
		&& ( DivitryEscortTeam->mGetShipsInPlayCount() == 0 )
		&& ( RokyrTeam->mGetShipsInPlayCount() == 0 )
		&& ( RokyrEscortTeam->mGetShipsInPlayCount() == 0 )
		&& ( BombFreighterTeam->mGetShipsInPlayCount() == 0 ))
	{
		gAllOrionsDestroyed = true;
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}

}
