#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_1MissionText.h"
#include "Rom_1MissionMaps.h"
/**************************************************
LNF stuff needed

capture
disengage - the primary way for the player to finish the mission?
victory points

also, make sure the freighter always follows the players ships, especially if the
player dies...
**************************************************/

//***     balance     ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                 ***

int32 PlayerDestroyed = 0;
int32 FreighterDestroyed = 0;
int32 OrionDestroyed = 0;
int32 EscortDestroyed = 0;
int32 AtWarWithOrion = 0;
int32 Elite = 0; // the elite faction the player belongs to
int32 Permission = 0; // permission to take freighter 'away'
int32 FreighterFollows = 0; // freighter is following Shiar player
int32 PraiexPlace = 0; // freighter at the Praiex planet -	for VictoryPoints
int32 ShiarPlace = 0; // freighter at Shiar planet		-	for VictoryPoints
int32 OrionsEliminated = 0; // no Orions remaining
int32 Mode = 0; // sets the 'mode' of the mission, when ships appear
bool SuggestPlan = false;  // for Shiar only, suggest the plan when freighter is damaged
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool FreighterIsAboutToDie = false; // flag to display heavily damaged freighter message
tShipInfo* ShipFreighterIsFollowing = NULL; // this stores which ship the freighter is following

const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kPraiexPlanetX = 31;
const int32 kPraiexPlanetY = 10;
const int32 kShiarPlanetX = 34;
const int32 kShiarPlanetY = 31;
const float kPlanetArrivalDistance = 25.0; // distance from destination planet
const float kFreighterDamageOK = 15.0; // percent damage that the freighter WON'T follow the player

int32 FactionText( tScript& tScript, tShipInfo* Ship );

void CheckWinCondition( tScript& tScript );

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar,
};

enum ePlanetNames
{
	kPraiexPlanet = 0,
	kShiarPlanet,
	kBothPlanets,
	kNoPlanet
};

int32 FreighterCurrentDestination = kNoPlanet; // current planet the freighter is moving toward - starts out as kPraiexPlanet
int32 FreighterFinalDestination = kNoPlanet; // this is the planet where the freighter ended up

enum eMissionMode
{
	kStart = 0,
	kOrionsArrive
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFreighterTeam, 
	kOrionTeam,
	kEscortTeam,
	kPlanetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFreighterTeam, kRomulan);
	fMission.mSetTeamRace (kOrionTeam, kOrion);
	fMission.mSetTeamRace (kEscortTeam, kRomulan);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
	fMission.mSetTeamRelations (kFreighterTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFreighterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kOrionTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kOrionTeam, kEscortTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
	fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kEscortTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0);//***
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if ( Mode == kStart )
	{
		// create Wedding Freighter
		FreighterTeam->mCreateShip ("R-FQL", kStartPosition_J);

		// create Escort team
		EscortTeam->mCreateFleet (2, kMaxBPV, kClassDestroyer, kClassDestroyer, 100, kStartPosition_K);

		// create planets
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPraiexPlanetX, kPraiexPlanetY );
		PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kShiarPlanetX, kShiarPlanetY );
	}
	else if ( Mode == kOrionsArrive )
	{
		// create Orion team
		// + 1 Orions for the Escort team to play with
		for ( int32 count = 0; count < (NumberOfPlayersShips + 1); count++ )
		{
			// get a random position for the ship
			int32 RandomPosition = fMission.mRandomInt32 (kStartPosition_V, kStartPosition_Z);

			// get the new ship class based on random stuff
			eClassTypes EnemyShipClass = NewEnemyShipClass( *this, LargestEnemyShipClass );

			// create the Orion
			OrionTeam->mCreateShip( AveragePlayerBPV, EnemyShipClass, EnemyShipClass, 100, RandomPosition );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
		return (kPraiexBriefing_msg);
	}
	else
	{
		Elite = kShiar;
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Wedding Freighter
	for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipMaxSpeed (12);
		// tell the freighter to go to the praiex planet
		Ship->mSetFinalDestination( kPraiexPlanetX, kPraiexPlanetY );
		FreighterCurrentDestination = kPraiexPlanet;
		// check if the freighter has 'arrived'
		//Ship->mTrackTacticalDistance( kPlanetArrivalDistance, kPraiexPlanetX, kPraiexPlanetY );
		Ship->mChangeName(kOselia_name);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Escort team
	for (Ship = EscortTeam->mGetFirstShip(); Ship != NULL; Ship = EscortTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mDefendEntity( FreighterTeam->mGetFirstShip(), 20.0, 20.0, kVeryHighPriority );
		//Ship->mSetShipThrust (10);
		//Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mChangeName( kEscort_name );
	}

	// configure planet
	PlanetTeam->mGetThisShip(1)->mChangeName( kPraiexPlanet_name );
	PlanetTeam->mGetThisShip(2)->mChangeName( kShiarPlanet_name );
	
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval( kTimerLimitSet, 1 ); // set relations

	// timer to check the freighter distance from both planets
	fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);

	// configure Orion team
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	OrionTeam->mGetThisShip(2)->mSetShipTarget( FreighterTeam->mGetFirstShip(), kAbsolutePriority );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FreighterTeam = fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* OrionTeam = fMission.mGetTeamHandle (kOrionTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			// if the Orion takes damage from the player : set AtWarWithOrion = 1
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ( AtWarWithOrion == 0 )
					{
						AtWarWithOrion = 1;

						if (( Ship->mGetTeam() == kOrionTeam ) && ( HostileShip->mGetTeam() == kEscortTeam ))
						{
							fMission.mDisplayMessage( kClosingWithOrion_msg, kMissionScriptMessage );
						}
						else if ( HostileShip->mGetTeam() == kOrionTeam )
						{
							fMission.mDisplayMessage( kOrionFirstStrike_msg, kMissionScriptMessage );
						}
					}
					
					if ( Elite == kShiar )
					{
						if ( Ship->mGetTeam() == kFreighterTeam )
						{
							// if the Freighter takes more than kFreighterDamageOK, then display a message
							// declaring the freighter is being damaged, and maybe would agree to the Shiar plan
							if (( FreighterTeam->mGetFirstShip()->mGetDamagePercent() >= kFreighterDamageOK ) &&
								( Permission == 0 ) && ( SuggestPlan == false ))
							{
								fMission.mDisplayMessage( kFreighterIsVulnerable_msg, kMissionScriptMessage );
								SuggestPlan = true;
							}
							else
							{
								Ship->mSetShipCallbackEvents( kNotifyDamage );
							}
						}
					}

					if (( FreighterTeam->mGetFirstShip()->mGetDamagePercent() > 50.0 ) &&
						( FreighterIsAboutToDie == false ))
					{
						FreighterIsAboutToDie = true;
						fMission.mDisplayMessage( kFreighterIsAboutToDie_msg, kMissionScriptMessage );
					}
				}

				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyAtDistance:
			{
/*
				// if the freighter has arrived at the planet
				if ( Ship->mGetTeam() == kFreighterTeam )
				{
					if ( Ship->mGetTrackingDistance() == kPlanetArrivalDistance )
					{
						if ( FreighterCurrentDestination == kPraiexPlanet )
						{
							fMission.mDisplayMessage( kEvent2PraiexPlace_msg, kMissionScriptMessage );
							// LNF set the ship speed to 0
							Ship->mSetShipAIControlled( kFalse );
							Ship->mSetShipThrust( 0 );
							PraiexPlace = 1;
							ShiarPlace = 0;
							FreighterFinalDestination = kPraiexPlanet;
						}
						else if ( FreighterCurrentDestination == kShiarPlanet )
						{
							fMission.mDisplayMessage( kEvent3ShiarPlace_msg, kMissionScriptMessage );
							// LNF set the ship speed to 0
							Ship->mSetShipAIControlled( kFalse );
							Ship->mSetShipThrust( 0 );
							PraiexPlace = 0;
							ShiarPlace = 1;
							FreighterFinalDestination = kShiarPlanet;
						}
						else // FreighterCurrentDestination == kBothPlanets
						{
							// check which planet you are at.
						}
					}
					CheckWinCondition( *this );
				}
*/
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kFreighterTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOrionTeam, kTeamHostile, kGoalDestroy, 1.0);//**
					fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kFreighterTeam, kOrionTeam, kTeamNeutral, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kFreighterTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFreighterTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kOrionTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kOrionTeam, kFreighterTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kOrionTeam, kEscortTeam, kTeamAtWar, kGoalIgnore, 1.0);//***
					fMission.mSetTeamRelations (kOrionTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0);//***

					fMission.mSetTeamRelations (kEscortTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0);//***

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

					// bring in the Orions!
					Mode = kOrionsArrive;
					mCreateMissionShips( Mode );
					break;

				case kTimerLimitSet_2:
					// win condition
					return kPlayerWins;
					break;

				case kTimerLimitSet_3:
					// if the Freighter is already following the player's ship, reassign it to another in the fleet
					// using this timer because I don't think fPlayer ship has been reassigned yet
					FreighterTeam->mGetFirstShip()->mFollowEntity( fPlayerShip, kAbsolutePriority );
					break;

				case kTimerLimitSet_4:
					// checks distance of Oseliea's freighter to both planets

					// LNF does this thing know what's going on with the mission?
					// LNF like if the freighter is supposed to be following the player or not?

					if ( FreighterCurrentDestination != kNoPlanet )
					{
						if ( FreighterTeam->mGetFirstShip()->mGetTacticalDistance( PlanetTeam->mGetThisShip(1) ) <= kPlanetArrivalDistance )
						{
							// player is at Praiex planet
							fMission.mDisplayMessage( kEvent2PraiexPlace_msg, kMissionScriptMessage );
							// LNF set the ship speed to 0
							//Ship->mSetShipAIControlled( kFalse );
							FreighterTeam->mGetFirstShip()->mSetShipMaxSpeed( 0 );
							PraiexPlace = 1;
							ShiarPlace = 0;
							FreighterFinalDestination = kPraiexPlanet;
							CheckWinCondition( *this );
							FreighterCurrentDestination = kNoPlanet;
						}
						else if ( FreighterTeam->mGetFirstShip()->mGetTacticalDistance( PlanetTeam->mGetThisShip(2) ) <= kPlanetArrivalDistance )
						{
							if ( Elite == kShiar )
							{
								// player is at Shiar planet
								fMission.mDisplayMessage( kEvent3ShiarPlace_msg, kMissionScriptMessage );
								// LNF set the ship speed to 0
								//Ship->mSetShipAIControlled( kFalse );
								FreighterTeam->mGetFirstShip()->mSetShipMaxSpeed( 0 );
								PraiexPlace = 0;
								ShiarPlace = 1;
								FreighterFinalDestination = kShiarPlanet;
								CheckWinCondition( *this );
								FreighterCurrentDestination = kNoPlanet;
							}
						}
					}

					fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );

					break;

				case kTimerTurnsSet_8:
					// set Escort comm buttons back to HAIL
					for (Ship = EscortTeam->mGetFirstShip(); Ship != NULL; Ship = EscortTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set freighter comm buttons back to HAIL
					for (Ship = FreighterTeam->mGetFirstShip(); Ship != NULL; Ship = FreighterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set ALL the ship's comm buttons back to HAIL
					for (Ship = OrionTeam->mGetFirstShip(); Ship != NULL; Ship = OrionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionCaptured_1_msg, kOrionCaptured_2_msg), kMissionScriptMessage);

				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
			
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}

				if ( FreighterFollows == 1 )
				{
					// the ship that the freighter was following is destroyed
					if ( Ship == ShipFreighterIsFollowing )
					{
						// reset the freighter to go to the Praiex planet and print a message
						FreighterTeam->mClearCommands();
						FreighterTeam->mGetFirstShip()->mSetFinalDestination( kPraiexPlanetX, kPraiexPlanetY );
						// check if the freighter has 'arrived'
						//FreighterTeam->mGetFirstShip()->mTrackTacticalDistance( kPlanetArrivalDistance, kPraiexPlanetX, kPraiexPlanetY );
						fMission.mDisplayMessage( kLeaderShipDestroyed_msg, kMissionScriptMessage );
						FreighterCurrentDestination = kPraiexPlanet;
						FreighterFollows = 2; // LNF does this mess things up? maybe I need another state?
					}
				}
			}
			else if (Ship->mGetTeam() == kFreighterTeam)
			{
				FreighterDestroyed++;
				return kPlayerLoses;
			}
			else if (Ship->mGetOriginalTeam() == kOrionTeam)
			{
				OrionDestroyed++;

				if ( Elite == kShiar )
				{
					if ( OrionDestroyed == 1 )
					{
						// LNF I need to do something else here other than make permission = 1
						Permission = 1;
						SuggestPlan = true;
						// LNF maybe a text message saying something here?
						FreighterTeam->mGetFirstShip()->mSetCommButton (1, kButtonFollowMe_text);
						fMission.mDisplayMessage( kOneOrionDestroyed_msg, kMissionScriptMessage );
					}
				}

				if ( OrionTeam->mGetShipsInPlayCount() == 0 )
				{
					// set elimination flag
					OrionsEliminated = 1;

					// if the player is Shiar and the escorts are still alive, tell them to defend the Freighter again
					if ( Elite == kShiar )
					{
						if ( EscortTeam->mGetShipsInPlayCount() > 0 )
						{
							EscortTeam->mDefendEntity( FreighterTeam->mGetFirstShip(), 20.0, 20.0, kVeryHighPriority );
						}
					}

					CheckWinCondition( *this );
				}
			}
			else if ( Ship->mGetTeam() == kEscortTeam )
			{
				EscortDestroyed++;
/*
				if ( Elite == kShiar )
				{
					if ( EscortDestroyed == 1 )
					{
						if ( SuggestPlan == false )
						{
							// LNF set a flag so the Freighter will follow player
							fMission.mDisplayMessage( kFreighterEscortsTakingLosses_msg, kMissionScriptMessage );
							SuggestPlan = true;
							FreighterTeam->mGetFirstShip()->mSetCommButton (1, kButtonFollowMe_text);
							Permission = 1;
						}
					}
				}
*/
				if ( EscortTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mDisplayMessage( kEscortsDestroyed_msg, kMissionScriptMessage );
				}
			}

			break;

		case kNotifyDisengaged:
			if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
			{
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFreighterTeam:
								if ( Elite == kPraiex )
								{
									if ( OrionsEliminated == 0 )
									{
										if (AtWarWithOrion == 0)
										{
											fMission.mDisplayMessage(kFreighterHailNic_msg, kMissionScriptMessage);
										}
										else
										{
											fMission.mDisplayMessage(kFreighterHailOrionCombat_msg, kMissionScriptMessage);
										}
									}
									else
									{
										fMission.mDisplayMessage( kFreighterHailOrionDestroyed_msg, kMissionScriptMessage );
									}
								}
								else // Elite == kShiar
								{
									// LNF the freighter following is only supposed to happen with the Shiar faction
									// LNF I should move the orions closer.  the freighter should get blown up if the
									// LNF player doesn't help the escorts in commodore mode.
									if (AtWarWithOrion == 0)
									{
										fMission.mDisplayMessage(kFreighterHailNic_msg, kMissionScriptMessage);
									}
									else if ( AtWarWithOrion == 1 )
									{
										// if the freighter is NOT at a planet
										if ( OrionsEliminated == 0 )
										{
											// LNF i could do some more checking if there are Orions alive...
											if ( FreighterFinalDestination == kNoPlanet )
											{
												// LNF I could use the "protect us" line, and then put up a couple of 
												// LNF buttons to ask if the freighter wants to follow....
												if ( Permission == 0 )
												{
													if ( Ship->mGetDamagePercent() < kFreighterDamageOK )
													{
														fMission.mDisplayMessage(kFreighterHailOrionCombatShiar_msg, kMissionScriptMessage);
													}
													else if (( Ship->mGetDamagePercent() >= kFreighterDamageOK ) ||
															 ( EscortTeam->mGetShipsInPlayCount() > 0 ))
													{
														// this should offer to give permission
														fMission.mDisplayMessage(kShiarHailF_msg, kMissionScriptMessage);
														Permission = 1;
														SuggestPlan = true; // set this because this message would be redundant

														// LNF set the comm button to follow me
														Ship->mSetCommButton (1, kButtonFollowMe_text);
														break;
													}
													else if (( Ship->mGetDamagePercent() >= kFreighterDamageOK ) ||
															 ( EscortTeam->mGetShipsInPlayCount() == 0 ))
													{
														fMission.mDisplayMessage(kFreighterHailPerF_msg, kMissionScriptMessage);
													}
												}
												else if ( Permission == 1 )
												{
													if (FreighterFollows == 1)
													{
														fMission.mDisplayMessage(kFreighterHailFollowing_msg, kMissionScriptMessage);
													}
													else if ( FreighterFollows == 2 )
													{
														// the ship the freighter was following has been destroyed
														fMission.mDisplayMessage( kLeaderShipDestroyed_msg, kMissionScriptMessage );
													}
												}
											}
											else // FreighterFinalDestination != kNoPlanet  this means the freighter IS at a planet
											{
												if ( FreighterFinalDestination == kPraiexPlanet )
												{
													fMission.mDisplayMessage( kEvent2PraiexPlace_msg, kMissionScriptMessage );
												}
												else if ( FreighterFinalDestination == kShiarPlanet )
												{
													fMission.mDisplayMessage( kEvent3ShiarPlace_msg, kMissionScriptMessage );
												}
												else // FreighterFinalDestination == kBoth  ??
												{
												}
											}
										}
										else // OrionsEliminated == 1
										{
											fMission.mDisplayMessage( kFreighterHailOrionDestroyed_msg, kMissionScriptMessage );
										}
									}
								}

								Ship->mSetCommButton (1, kButtonHail_text);
								break;

							case kOrionTeam:
								if (Ship->mGetDamagePercent() < 50.0)
								{
									fMission.mDisplayMessage(kOrionHailOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionHailDamaged_msg, kMissionScriptMessage);
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								if (AtWarWithOrion == 0)
								{
									fMission.mDisplayMessage(kRomHailNic_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(FactionText( *this, Ship ), kMissionScriptMessage);
								}

								Ship->mSetCommButton (1, kButtonHail_text);
								break;
						}
						break;

					case kButtonSurrender_text:
						if (Ship->mGetDamagePercent() < 50.0)
						{
							fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
						}
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						fMission.mDisplayMessage (fMission.mRandomInt32 (kOrionTaunt1_msg, kOrionTaunt4_msg), kMissionScriptMessage);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonFollowMe_text:
						// the button the player presses to tell the Freighter to follow them.
						FreighterFollows = 1;
						//FreighterCurrentDestination = kShiarPlanet;

						// the player should be able to finish mission at BOTH planets, just get less points
						// for taking to Praiex planet
						FreighterCurrentDestination = kBothPlanets;

						fMission.mDisplayMessage(kFreighterHailPerT_msg, kMissionScriptMessage);
						// LNF make the freighter follow, and somehow make the orions
						// LNF seem less interested?  What happens if the escorts are destroyed?
						// LNF maybe as the player gets close to planet II, reinforcements arrive
						// LNF and chase away the Orions?

						FreighterTeam->mClearCommands();
						//FreighterTeam->mGetFirstShip()->mTrackTacticalDistance( kPlanetArrivalDistance, kShiarPlanetX, kShiarPlanetY );
						// LNF need to do something if the player dies.  Maybe make the ship go back to the Praiex planet?
						FreighterTeam->mGetFirstShip()->mFollowEntity( fPlayerShip, kAbsolutePriority );
						ShipFreighterIsFollowing = fPlayerShip;
						EscortTeam->mClearCommands();
						fMission.mSetTeamRelations (kEscortTeam, kOrionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						FreighterTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
						break;

				}
			}
			break;

	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if (Elite == kPraiex)
	{
		VictoryPoints = (PraiexPlace * 70) + (OrionDestroyed * 30) - (EscortDestroyed * 15);
	}
	else
	{
		// LNF I have to tweak these number so the Praiex planet default is victory
		// LNF and the Shiar planet is Astounding, but I also have to make sure the orions are
		// LNF destroyed
		if ( FreighterFinalDestination == kShiarPlanet )
		{
			VictoryPoints = (ShiarPlace * 90) + (OrionDestroyed * 5) - (EscortDestroyed * 15);
		}
		else
		{
			// LNF maybe bonus points for orions destroyed?
			VictoryPoints = (PraiexPlace * 70) + (OrionDestroyed * 5) - (EscortDestroyed * 15);
		}
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kPraiexVictory_msg);
				fMission.mAddToDebriefingList(kPraiexAstoundingVictory_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
			}
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kShiarAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( OrionsEliminated != 0 ) ? fMission.mAddToDebriefingList( kOrionsEliminatedTrue_msg ) : SuccessCount++;
	( FreighterFinalDestination != kNoPlanet ) ? fMission.mAddToDebriefingList( kFreighterDestinationTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( OrionsEliminated == 0 ) ? fMission.mAddToDebriefingList( kOrionsEliminatedFalse_msg ) : FailureCount++;
	( FreighterFinalDestination == kNoPlanet ) ? fMission.mAddToDebriefingList( kFreighterDestinationFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// called to figure out what Faction's text should be displayed
int32 FactionText( tScript& tScript, tShipInfo* Ship)
{
	tTeamInfo* FreighterTeam = tScript.fMission.mGetTeamHandle (kFreighterTeam);
	tTeamInfo* EscortTeam = tScript.fMission.mGetTeamHandle (kEscortTeam);

	if (Elite == kPraiex)
	{
		if (Ship->mGetDamagePercent() < 50.0)
		{
			return kPraiexHailOK_msg;
		}
		else
		{
			return kPraiexHailDamaged_msg;
		}
	}
	else // player is Shiar
	{
		if ( FreighterFinalDestination == kNoPlanet )
		{
			if ( Permission == 0 )
			{
				if ( FreighterTeam->mGetFirstShip()->mGetDamagePercent() < kFreighterDamageOK )
				{
					return kEscortHailOrionCombatShiar_msg;
				}
				else if (( Ship->mGetDamagePercent() >= kFreighterDamageOK ) ||
						 ( EscortTeam->mGetShipsInPlayCount() > 0 ))
				{
					// this should offer to give permission
					Permission = 1;

					// LNF set the comm button to follow me
					FreighterTeam->mGetFirstShip()->mSetCommButton (1, kButtonFollowMe_text);

					SuggestPlan = true; // set flag true because this message would be redundant
					return kShiarHailF_msg;
				}
				else if (( Ship->mGetDamagePercent() >= kFreighterDamageOK ) ||
						 ( EscortTeam->mGetShipsInPlayCount() == 0 ))
				{
					return kFreighterHailPerF_msg;
				}
				else
				{
					return kFreighterHailPerF_msg;
				}
			}
			else // Permission == 1
			{
				return kShiarHailT_msg;
			}
		}
		else // FreighterFinalDestination != kNoPlanet  this means the freighter IS at a planet
		{
			if ( FreighterFinalDestination == kPraiexPlanet )
			{
				return kEvent2PraiexPlace_msg;
			}
			else if ( FreighterFinalDestination == kShiarPlanet )
			{
				return kEvent3ShiarPlace_msg;
			}
			else // FreighterFinalDestination == kBothPlanets ??
			{
				// LNF need a return value here
				return kEvent3ShiarPlace_msg;
			}
		}
	}
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	// if the freighter arrives at a planet
	if ( FreighterFinalDestination != kNoPlanet )
	{
		if ( OrionsEliminated == 1 )
		{
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			//FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
			tScript.fMission.mSetTimedInterval( kTimerLimitSet_2, 3 );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kOrionsStillAround_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
		}
	}
	else // the freighter hasn't gotten to a planet
	{
		if ( Elite == kPraiex )
		{
			// LNF this should only be called when the orions have been destroyed
			tScript.fMission.mDisplayMessage( kGetFreighterToPlanetPraiex_msg, kMissionScriptMessage );
		}
		else // Elite == kShiar
		{
			// LNF this should only be called when the orions have been destroyed
			tScript.fMission.mDisplayMessage( kGetFreighterToPlanetShiar_msg, kMissionScriptMessage );
		}
	}
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// figure out what ship class to use
	if ( RandomNum > 50 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 50 ) && ( RandomNum >= 25 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 25 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 3 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 4 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
}


