#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_2MissionText.h"
#include "Rom_2MissionMaps.h"

/*********************************************
capture
disengage

LNF the planet needs hails...what if the player actually does what the mission says?
LNF the planet will need to drop shields as well.  maybe summon reinforcements? or just be indestructible?

LNF we need to have the ability to drop shields of Allied or maybe
LNF neutral teams to be able to accept transporter?
LNF after agreeing to transport the medicine, the target ship
LNF should approach the players ship.  Maybe set that ship to tractor the
LNF fPlayerShip?  drop speed to 0?
LNF Should also set the ships to Hostile, and if the player shoots at the
LNF ship that has its shields down, the ship should put up shields and engage
LNF the player.
*********************************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 PlayerDestroyed = 0;
int32 IlvuriusDestroyed = 0;
int32 LesaniusDestroyed = 0;
int32 IlvuriusAllDestroyed = 0;
int32 LesaniusAllDestroyed = 0;
int32 PeacefulDestroyed = 0;
int32 Elite = 0;
int32 IlvuriusAlly = 0; // medicine transported to Ilvurius
int32 LesaniusAlly = 0; // medicine transported to Lesanius
int32 AtWarWithIlvurius = 0; // player fires upon Ilvurius
int32 AtWarWithLesanius = 0; // player fires upon Lesanius
int32 AtWarWithPeaceful = 0; // player fires upon Peaceful
int32 IlvOffer = 0;
int32 LesOffer = 0;
int32 PeacefulOrbiting = 0;
bool DetectIlvurius = false;
bool DetectLesanius = false;
bool LesaniusExists = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

int32 TransportedMedicine = 0;
tShipInfo* ShipWithMedicine = NULL; // the ship that the medicine is transported to
int32 GoalPointerTo = 0; // the faction that the goal pointer is pointing to
int32 TeamMedicineWasTransportedTo = 0; // the Faction name that the medicine was transported to

const int32 kMapEdgeX = 34;
const int32 kMapEdgeY = 34;

bool NeuMessage_1 = false; // flag if a message has been sent

void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer);
void PeacefulOrbit(tTeamInfo* PeacefulTeam);
tShipInfo* FindClosestShip(tScript& tScript);
void CheckWinCondition( tScript& tScript );

enum eFactionName
{
	kIlvuriusFaction = 1,
	kLesaniusFaction,
	kPlanetFaction,
	kPeacefulFaction
};

enum eOfferStatus
{
	kNoOffer = 0,
	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
	kCompleteTransaction
};

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kIlvuriusTeam, 
	kLesaniusTeam,
	kPeacefulTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kIlvuriusTeam, kRomulan);
	fMission.mSetTeamRace (kLesaniusTeam, kRomulan);
	fMission.mSetTeamRace (kPeacefulTeam, kRomulan);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kLesaniusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPeacefulTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kIlvuriusTeam, kLesaniusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kIlvuriusTeam, kPeacefulTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kIlvuriusTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kLesaniusTeam, kPeacefulTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kLesaniusTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPeacefulTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
	}
	else
	{
		Elite = kShiar;
		LesaniusExists = true;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* IlvuriusTeam = fMission.mGetTeamHandle (kIlvuriusTeam);
	tTeamInfo* LesaniusTeam = fMission.mGetTeamHandle (kLesaniusTeam);
	tTeamInfo* PeacefulTeam = fMission.mGetTeamHandle (kPeacefulTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// The Enemy teams depend on which Elite group the player belongs to
	if (Elite == kPraiex)
	{
		// create Ilvurius team
		IlvuriusTeam->mCreateFleet (NumberOfPlayersShips, TotalBPV, kClassFrigate, LargestEnemyShipClass, 100, kStartPosition_Specified, 15, 19);
	}
	else if (Elite == kShiar)
	{
		// create Ilvurius team
		IlvuriusTeam->mCreateShip (AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_Specified, 15, 19);
		IlvuriusTeam->mCreateFleet (2, (2 * AveragePlayerBPV), kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_Specified, 15, 19);

		// create Lesanius team
		LesaniusTeam->mCreateFleet (2, (2 * AveragePlayerBPV), kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_Specified, 31, 18);
	}

	// create Peaceful team
	PeacefulTeam->mCreateFleet (2, (2 * AveragePlayerBPV), kClassFreighter, kClassLightCruiser, 50, kStartPosition_Specified, 9, 10);

	// create Planet
	PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, 20, 15 );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* IlvuriusTeam = fMission.mGetTeamHandle (kIlvuriusTeam);
	tTeamInfo* LesaniusTeam = fMission.mGetTeamHandle (kLesaniusTeam);
	tTeamInfo* PeacefulTeam = fMission.mGetTeamHandle (kPeacefulTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	// LNF change kTransMedicineJar to medicine jar
	fPlayerShip->mAccessSupplies (kSetTransporterItemCount, kTransMedicineJar, 1);

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyTransportedItemTo);
	}

	// configure Ilvurius Team
	for (Ship = IlvuriusTeam->mGetFirstShip(); Ship != NULL; Ship = IlvuriusTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		// LNF I think the ships are supposed to be operating at a lower efficiency from the disease
		//Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		//Ship->mDamageSystem(kDamageRightWarpEngine, 100);
	}

	// The Lesanius team only appears when the player is Shiar
	if (Elite == kShiar)
	{
		// configure Lesanius team
		for (Ship = LesaniusTeam->mGetFirstShip(); Ship != NULL; Ship = LesaniusTeam->mGetNextShip())
		{
			Ship->mSetCommButton (1, kButtonHail_text);
			Ship->mSetShipCallbackEvents (kNotifyDamage);
			//Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
			//Ship->mDamageSystem(kDamageRightWarpEngine, 100);
		}
	}

	// configure Peaceful team
	for (Ship = PeacefulTeam->mGetFirstShip(); Ship != NULL; Ship = PeacefulTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipThrust (10);
		Ship->mSetFinalDestination(17, 15);
		// notify when near the planet and then set orbit (patrol)
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Planet
	PlanetTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );


	
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 2);
	fMission.mSetTimedInterval (kTimerLimitSet_10, 3); // for 'Orbiting' ships
	fMission.mSetTimedInterval (kTimerLimitSet_2, 3); // Event 1 scanning range
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* IlvuriusTeam = fMission.mGetTeamHandle (kIlvuriusTeam);
	tTeamInfo* LesaniusTeam = fMission.mGetTeamHandle (kLesaniusTeam);
	tTeamInfo* PeacefulTeam = fMission.mGetTeamHandle (kPeacefulTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				// LNF change kTransMedicineJar to medicine jar
				if ( fEventItem1 == kTransMedicineJar )
				{
					if (Elite == kPraiex)
					{
						if (( TransportTarget != NULL ) && ( TransportTarget == PlanetTeam->mGetFirstShip() ))
						{
							// LNF send a message that the Praiex is not pleased of that choice
							// LNF and set at war with player???
							// LNF need to make sure shield thing gets turned off
							fMission.mDisplayMessage (kEvent4_TransportToPlanet_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamHostile, kGoalDestroy, 1.0);

							if ( IlvuriusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kIlvChosePoorly_msg, kMissionScriptMessage );
							}

							TeamMedicineWasTransportedTo = kPlanetFaction;
							// LNf do I set the ally setting here?
						}
						else if (( TransportTarget != NULL ) && (TransportTarget->mGetTeam() == kIlvuriusTeam))
						{
							fMission.mDisplayMessage (kEvent2_TransportToIlvurius_msg, kMissionScriptMessage);
							//fMission.mSetTeamRelations (kPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							//fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0);
							IlvuriusAlly = 1;
							IlvOffer = kCompleteTransaction;
							TeamMedicineWasTransportedTo = kIlvuriusFaction;
							IlvuriusTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
						}
						else if (( TransportTarget != NULL ) && (TransportTarget->mGetTeam() == kPeacefulTeam))
						{
							fMission.mDisplayMessage (kTransportToNeutral_msg, kMissionScriptMessage);
							//fMission.mSetTeamRelations (kPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							//fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamHostile, kGoalDestroy, 1.0);

							if ( IlvuriusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kIlvChosePoorly_msg, kMissionScriptMessage );
							}

							TeamMedicineWasTransportedTo = kPeacefulFaction;
						}
					}
					else if (Elite == kShiar)
					{
						if (( TransportTarget != NULL ) && (TransportTarget == PlanetTeam->mGetFirstShip() ))
						{
							// LNF send a message that the Praiex is not pleased of that choice
							// and set at war with player???
							fMission.mDisplayMessage (kEvent4_TransportToPlanet_msg, kMissionScriptMessage);
							fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kLesaniusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kLesaniusTeam, kTeamHostile, kGoalDestroy, 1.0);

							if ( IlvuriusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kIlvChosePoorly_msg, kMissionScriptMessage );
							}

							if ( LesaniusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kLesChosePoorly_msg, kMissionScriptMessage );
							}

							TeamMedicineWasTransportedTo = kPlanetFaction;
						}
						else if (( TransportTarget != NULL ) && (TransportTarget->mGetTeam() == kIlvuriusTeam))
						{
							fMission.mDisplayMessage (kEvent2_TransportToIlvurius_msg, kMissionScriptMessage);
							//fMission.mSetTeamRelations (kPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							//fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kLesaniusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kLesaniusTeam, kTeamHostile, kGoalDestroy, 1.0);
							IlvuriusAlly = 1;
							IlvOffer = kCompleteTransaction;
							TeamMedicineWasTransportedTo = kIlvuriusFaction;

							if ( LesaniusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kLesChosePoorly_msg, kMissionScriptMessage );
							}

							IlvuriusTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
						}
						else if (( TransportTarget != NULL ) && (TransportTarget->mGetTeam() == kLesaniusTeam))
						{
							fMission.mDisplayMessage (kEvent3_TransportToLesanius_msg, kMissionScriptMessage);
							//fMission.mSetTeamRelations (kPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							//fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamHostile, kGoalDestroy, 1.0);
							LesaniusAlly = 1;
							LesOffer = kCompleteTransaction;
							TeamMedicineWasTransportedTo = kLesaniusFaction;

							if ( IlvuriusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kIlvChosePoorly_msg, kMissionScriptMessage );
							}

							LesaniusTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
						}
						else if (( TransportTarget != NULL ) && (TransportTarget->mGetTeam() == kPeacefulTeam))
						{
							fMission.mDisplayMessage (kTransportToNeutral_msg, kMissionScriptMessage);
							//fMission.mSetTeamRelations (kPlanetTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							//fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kLesaniusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPlayerTeam, kLesaniusTeam, kTeamHostile, kGoalDestroy, 1.0);

							if ( IlvuriusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kIlvChosePoorly_msg, kMissionScriptMessage );
							}

							if ( LesaniusAllDestroyed == 0 )
							{
								fMission.mDisplayMessage( kLesChosePoorly_msg, kMissionScriptMessage );
							}

							TeamMedicineWasTransportedTo = kPeacefulFaction;
						}
					}
					TransportedMedicine = 1;
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
					CheckWinCondition( *this );
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				if ( TransportTarget != NULL )
				{
					// LNF change kTransMedicineJar to medicine jar
					if (( TransportedMedicine == 1 ) && ( fEventItem1 == kTransMedicineJar ))
					{
						//fMission.mDisplayMessage( kCantTakeMedicineBack_msg, kMissionScriptMessage );
						fPlayerShip->mAccessSupplies (kSetTransporterItemCount, kTransMedicineJar, 0);
						TransportTarget->mAccessSupplies (kSetTransporterItemCount, kTransMedicineJar, 1);
						Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
					}
				}
			}
			break;

		case kNotifyAtDestination:
			// set the Peaceful team to orbit (patrol) the planet
			if ((Ship->mGetTeam() == kPeacefulTeam) && (PeacefulOrbiting == 0))
			{
				PeacefulOrbiting = 1;
				PeacefulOrbit(PeacefulTeam);
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if( HostileShip == NULL )
				{
					Ship->mSetShipCallbackEvents (kNotifyDamage);
					break;
				}

				if ( HostileShip->mGetTeam() == kPlayerTeam )
				{
					// if the player shoots at either team asking for medicine with their shields down
					if (( IlvOffer == kAcceptedOffer ) || ( LesOffer == kAcceptedOffer ))
					{
						// put the shields back up
						ShipWithMedicine->mDamageSystem(kDamageMaxShields, 0);
						ShipWithMedicine->mSetShipMaxSpeed( 31 );

						if ( ShipWithMedicine->mGetTeam() == kIlvuriusTeam )
						{
							AtWarWithIlvurius = 1;
							IlvOffer = kRejectedOffer;
							fMission.mSetTimedInterval( kTimerLimitSet_3, 0 );
						}
						else
						{
							AtWarWithLesanius = 1;
							LesOffer = kRejectedOffer;
							fMission.mSetTimedInterval( kTimerLimitSet_4, 0 );
						}
					}
				}
				Ship->mSetShipCallbackEvents (kNotifyDamage);
/*
				switch (Ship->mGetTeam())
				{
					case kIlvuriusTeam:
						if (IlvuriusAlly == 1)
						{
							fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}

						break;
					case kLesaniusTeam:
						break;
					case kPeacefulTeam:
						break;
				}
*/
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kIlvuriusTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kLesaniusTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPeacefulTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kIlvuriusTeam, kLesaniusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kIlvuriusTeam, kPeacefulTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kIlvuriusTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kLesaniusTeam, kPeacefulTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kLesaniusTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPeacefulTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					{
						// find the closest ship to the player on either the Ilvurius or Lesanius team
						// point the goal pointer at it
						tShipInfo* ClosestShip = FindClosestShip(*this);

						if ( ClosestShip != NULL )
						{
							eTeamID ClosestTeam = ClosestShip->mGetTeam();

							if (fPlayerShip->mGetTacticalDistance(ClosestShip) <= 106.0)
							{
								if ( ClosestTeam == kIlvuriusTeam)
								{
									if ((DetectIlvurius == false) && (DetectLesanius == false))
									{
										fMission.mDisplayMessage (kEvent1A_msg, kScienceScriptMessage);
										DetectIlvurius = true;
										fMission.mSetTimedInterval (kTimerLimitSet_2, 1); // Event 1 scanning range
									}
									else if ((DetectIlvurius == false) && (DetectLesanius == true))
									{
										fMission.mDisplayMessage (kEvent1B_msg, kScienceScriptMessage);
										DetectIlvurius = true;
									}
									GoalPointerTo = kIlvuriusFaction;
								}
								else if ( ClosestTeam == kLesaniusTeam )
								{
									if ((DetectIlvurius == false) && (DetectLesanius == false))
									{
										fMission.mDisplayMessage (kEvent1A_msg, kScienceScriptMessage);
										DetectLesanius = true;
										fMission.mSetTimedInterval (kTimerLimitSet_2, 1); // Event 1 scanning range
									}
									else if ((DetectIlvurius == true) && (DetectLesanius == false))
									{
										fMission.mDisplayMessage (kEvent1B_msg, kScienceScriptMessage);
										DetectLesanius = true;
									}
									GoalPointerTo = kLesaniusFaction;
								}
								fMission.mSetGoalIndicator( ClosestShip );
							}
							else if ((DetectIlvurius == false) || (DetectLesanius == false))
							{
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1); // Event 1 scanning range
							}
						}
					}
					break;

				case kTimerLimitSet_3:
					if (( TransportedMedicine == 0 ) && ( AtWarWithIlvurius == 0 ))
					{
						//keep turning off shield until item is tranported
						ShipWithMedicine->mDamageSystem(kDamageMaxShields, 100);
						fMission.mSetTimedInterval (kTimerLimitSet_3, 1);
					}
					else
					{
						// LNF this doesn't put up all the shields
						ShipWithMedicine->mDamageSystem(kDamageMaxShields, 0);
						ShipWithMedicine->mSetShipMaxSpeed( 31 );
					}
					break;

				case kTimerLimitSet_4:
					if (( TransportedMedicine == 0 ) && ( AtWarWithLesanius == 0 ))
					{
						//keep turning off shield until item is tranported
						ShipWithMedicine->mDamageSystem(kDamageMaxShields, 100);
						fMission.mSetTimedInterval (kTimerLimitSet_4, 1);
					}
					else
					{
						// LNF this doesn't put up all the shields
						ShipWithMedicine->mDamageSystem(kDamageMaxShields, 0);
						ShipWithMedicine->mSetShipMaxSpeed( 31 );
					}
					break;

				case kTimerLimitSet_10:
					// set the two competitors to harass the player about the medicine
					if (Elite == kPraiex)
					{
						Bother(*this, fPlayerShip, IlvuriusTeam, -2, 2);
					}
					else if (Elite == kShiar)
					{
						Bother(*this, fPlayerShip, IlvuriusTeam, -3, 3);
						Bother(*this, fPlayerShip, LesaniusTeam, -2, 2);
					}
					fMission.mSetTimedInterval (kTimerLimitSet_10, 3);
					break;

				case kTimerCountdownSet:
					// LNF use this for offers
					break;

				case kTimerTurnsSet_8:
					// set Ilvurius comm buttons back to HAIL
					for (Ship = IlvuriusTeam->mGetFirstShip(); Ship != NULL; Ship = IlvuriusTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Lesanius comm buttons back to HAIL
					for (Ship = LesaniusTeam->mGetFirstShip(); Ship != NULL; Ship = LesaniusTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Peaceful comm buttons back to HAIL
					for (Ship = PeacefulTeam->mGetFirstShip(); Ship != NULL; Ship = PeacefulTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:
			// LNF need capture message/code
			fMission.mAddToBonusPrestige( kCaptureBonus );
			break;

		case kNotifyAtDeath:
			// LNF I have to know if a team is allied or not with the player when they die
			// LNF so I can count that death for the plsyer's points.
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kIlvuriusTeam)
			{
				IlvuriusDestroyed++;
				if (IlvuriusTeam->mGetShipsInPlayCount() == 0)
				{
					IlvuriusAllDestroyed = 1;
				}
			}
			else if (Ship->mGetTeam() == kLesaniusTeam)
			{
				LesaniusDestroyed++;
				if (LesaniusTeam->mGetShipsInPlayCount() == 0)
				{
					LesaniusAllDestroyed = 1;
				}
			}
			else if (Ship->mGetTeam() == kPeacefulTeam)
			{
				PeacefulDestroyed++;
				fMission.mDisplayMessage( kPeacefulShipDied_msg, kMissionScriptMessage );
			}

			break;

		case kNotifyDisengaged:
			if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
			{
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kIlvuriusTeam:
								if ((IlvuriusAlly == 0) && (AtWarWithIlvurius == 0) &&
									(IlvOffer == kNoOffer))// && (LesOffer == kNoOffer))
								{
									fMission.mDisplayMessage(kIlvHailOffer_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonYES_text);
									Ship->mSetCommButton (2, kButtonNO_text);
									IlvOffer = kMadeOffer;
									Ship->mChangeName( kIlvurius_name );
									// turn off goal pointer
									if ( GoalPointerTo == kIlvuriusFaction )
									{
										fMission.mSetGoalIndicator( kFalse, 0, 0 );
									}
									break;
								}
								else if ((IlvuriusAlly == 0) && (AtWarWithIlvurius == 0) &&
										 (IlvOffer == kMadeOffer))// && (LesOffer == kNoOffer))
								{
									fMission.mDisplayMessage(kIlvHailMadeOffer_msg, kMissionScriptMessage);
									Ship->mChangeName( kIlvurius_name );
									if ( GoalPointerTo == kIlvuriusFaction )
									{
										fMission.mSetGoalIndicator( kFalse, 0, 0 );
									}
								}
								else if ((IlvuriusAlly == 1) && (AtWarWithIlvurius == 0))
								{
									fMission.mDisplayMessage (kIlvHailAlly_msg, kMissionScriptMessage);
								}
								else if ((IlvuriusAlly == 1) && (AtWarWithIlvurius == 1))
								{
									fMission.mDisplayMessage (kIlvAttacked_msg, kMissionScriptMessage);
								}
								else if ((IlvuriusAlly == 0) && (AtWarWithIlvurius == 1))
								{
									fMission.mDisplayMessage (kIlvHailCombat_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kLesaniusTeam:
								if ((LesaniusAlly == 0) && (AtWarWithLesanius == 0) &&
									(LesOffer == kNoOffer))// && (IlvOffer == kNoOffer))
								{
									fMission.mDisplayMessage(kLesHailOffer_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonYES_text);
									Ship->mSetCommButton (2, kButtonNO_text);
									LesOffer = kMadeOffer;
									Ship->mChangeName( kLesanius_name );
									if ( GoalPointerTo == kLesaniusFaction )
									{
										fMission.mSetGoalIndicator( kFalse, 0, 0 );
									}
									break;
								}
								else if ((LesaniusAlly == 0) && (AtWarWithLesanius == 0) &&
										 (LesOffer == kMadeOffer))// && (IlvOffer == kNoOffer))
								{
									fMission.mDisplayMessage(kLesHailMadeOffer_msg, kMissionScriptMessage);
									Ship->mChangeName( kLesanius_name );
									if ( GoalPointerTo == kLesaniusFaction )
									{
										fMission.mSetGoalIndicator( kFalse, 0, 0 );
									}
								}
								else if ((LesaniusAlly == 1) && (AtWarWithLesanius == 0))
								{
									fMission.mDisplayMessage (kLesHailAlly_msg, kMissionScriptMessage);
								}
								else if ((LesaniusAlly == 1) && (AtWarWithLesanius == 1))
								{
									fMission.mDisplayMessage (kLesAttacked_msg, kMissionScriptMessage);
								}
								else if ((LesaniusAlly == 0) && (AtWarWithLesanius == 1))
								{
									fMission.mDisplayMessage (kLesHailCombat_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPeacefulTeam:
								if (AtWarWithPeaceful == 0)
								{
									if ( NeuMessage_1 == false )
									{
										fMission.mDisplayMessage(kRomHailNic_msg, kMissionScriptMessage);
										NeuMessage_1 = true;
									}
									else
									{
										fMission.mDisplayMessage( kRomHailNic_2_msg, kMissionScriptMessage );
									}
								}
								else
								{
									fMission.mDisplayMessage(kRomHailCombat_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPlanetTeam:
								fMission.mDisplayMessage( kPlanetHail_msg, kMissionScriptMessage );
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kIlvuriusTeam:
								if ((IlvuriusAlly == 1) && (AtWarWithIlvurius == 0))
								{
									fMission.mDisplayMessage(kIlvSurrenderAlly_msg, kMissionScriptMessage);
								}
								else if (AtWarWithIlvurius == 1)
								{
									fMission.mDisplayMessage(kIlvSurrender_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kLesaniusTeam:
								if ((LesaniusAlly == 1) && (AtWarWithLesanius == 0))
								{
									fMission.mDisplayMessage(kLesSurrenderAlly_msg, kMissionScriptMessage);
								}
								else if (AtWarWithLesanius == 1)
								{
									fMission.mDisplayMessage(kLesSurrender_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPeacefulTeam:
								fMission.mDisplayMessage(kRomSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kIlvuriusTeam:
								if (AtWarWithIlvurius == 0)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kIlvTaunt1_msg, kIlvTaunt3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kLesaniusTeam:
								if (AtWarWithLesanius == 0)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kLesTaunt1_msg, kLesTaunt3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kPeacefulTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kRomTaunt1_msg, kRomTaunt3_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;
				
					case kButtonYES_text:
						{
							if (Ship->mGetTeam() == kIlvuriusTeam)
							{
								// LNF should I have a time limit to transport the medicine? - yes with countdown timer

								// LNF how about a counter offer from the planet?

								fMission.mDisplayMessage(kIlvAccept_msg, kMissionScriptMessage);
								IlvOffer = kAcceptedOffer;
								Ship->mSetCommButton (1, kButtonHail_text);
								Ship->mClearCommButton (2);
								// bring shields down, slow ship, tell ship to follow player
								Ship->mDamageSystem(kDamageMaxShields, 100);
								Ship->mSetShipMaxSpeed( 8 );
								//keep turning off shield until item is tranported
								fMission.mSetTimedInterval (kTimerLimitSet_3, 1);
							}
							else
							{
								// LNF should I have a time limit to transport the medicine? - yes with countdown timer
								fMission.mDisplayMessage(kLesAccept_msg, kMissionScriptMessage);
								LesOffer = kAcceptedOffer;
								Ship->mSetCommButton (1, kButtonHail_text);
								Ship->mClearCommButton (2);
								// bring shields down, slow ship, tell ship to follow player
								Ship->mDamageSystem(kDamageMaxShields, 100);
								Ship->mSetShipMaxSpeed( 8 );
								//keep turning off shield until item is tranported
								fMission.mSetTimedInterval (kTimerLimitSet_4, 1);
							}
							ShipWithMedicine = Ship;
						}
						break;

					case kButtonNO_text:
						{
							if (Ship->mGetTeam() == kIlvuriusTeam)
							{
								fMission.mDisplayMessage(kIlvReject_msg, kMissionScriptMessage);
								fMission.mSetTeamRelations (kIlvuriusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								AtWarWithIlvurius = 1;
								IlvOffer = kRejectedOffer;
								Ship->mSetCommButton (1, kButtonHail_text);
							}
							else
							{
								fMission.mDisplayMessage(kLesReject_msg, kMissionScriptMessage);
								fMission.mSetTeamRelations (kLesaniusTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
								AtWarWithLesanius = 1;
								LesOffer = kRejectedOffer;
								Ship->mSetCommButton (1, kButtonHail_text);
							}
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;


	// LNF the SHIAR player should not get an astounding victory unless they transport to lesanius?

	if (Elite == kPraiex)
	{
		// LNF need to put in points here
		if ( TransportedMedicine == 1 )
		{
			if ( TeamMedicineWasTransportedTo == kPlanetFaction )
			{
				VictoryPoints = 80 + (IlvuriusAllDestroyed * 20) - (PeacefulDestroyed * 50);
			}
		}
	}
	else // player is Shiar
	{
		// LNF need to put in bonus? points here
		if ( TransportedMedicine == 1 )
		{
			if ( TeamMedicineWasTransportedTo == kPlanetFaction )
			{
				VictoryPoints = 80 - (PeacefulDestroyed * 50) - (LesaniusDestroyed * 20);
			}
			else if ( TeamMedicineWasTransportedTo == kLesaniusFaction )
			{
				VictoryPoints = 100 - (PeacefulDestroyed * 50) - (LesaniusDestroyed * 20);
			}
		}
		//VictoryPoints += (LesaniusAlly * 100);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
			if (VictoryLevel == kAstoundingVictory)
			{
				//fMission.mAddToDebriefingList(kPraiexAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kPraiexVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				//fMission.mAddToDebriefingList(kPraiexVictory_msg);
				fMission.mAddToDebriefingList(kPraiexVictory2_msg);
			}
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kShiarAstoundingVictory_msg);
			if (VictoryLevel == kAstoundingVictory)
			{
				//fMission.mAddToDebriefingList(kShiarAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kShiarVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				//fMission.mAddToDebriefingList(kShiarVictory_msg);
				fMission.mAddToDebriefingList(kShiarVictory2_msg);
			}
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( TeamMedicineWasTransportedTo == kPlanetFaction ) ? fMission.mAddToDebriefingList( kPraiexTransportToPlanetTrue_msg ) : SuccessCount++;

		if ( SuccessCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( TeamMedicineWasTransportedTo != kPlanetFaction ) ? fMission.mAddToDebriefingList( kPraiexTransportToPlanetFalse_msg ) : FailureCount++;
		if ( FailureCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		if ( TeamMedicineWasTransportedTo == kPlanetFaction )
		{
			fMission.mAddToDebriefingList( kShiarTransportToPlanetTrue_msg );
		}
		else if ( TeamMedicineWasTransportedTo == kLesaniusFaction )
		{
			fMission.mAddToDebriefingList( kShiarTransportToLesaniusTrue_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		if (( TeamMedicineWasTransportedTo != kPlanetFaction ) && ( TeamMedicineWasTransportedTo != kLesaniusFaction ))
		{
			fMission.mAddToDebriefingList( kShiarTransportMedicineFalse_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make a team 'orbit' a specific ship
void Bother(tScript& tScript, tShipInfo* ShipToBother, tTeamInfo* BotherTeam, int32 inner, int32 outer)
{
	// The ship/team will harass another by checking the distance between 
	// themselves every so many seconds and trying to keep that distance
	// by telling the BotherTeam to go to the spot +/- the ShipToBother

	tShipInfo* Ship;

	for (Ship = BotherTeam->mGetFirstShip(); Ship != NULL; Ship = BotherTeam->mGetNextShip())
	{
		// LNF maybe make this 'smarter' so the player can't lure away the
		// enemy ships with a weaker ship and complete mission??
		int32 OldX = 0;
		int32 OldY = 0;
		tScript.fPlayerShip->mGetGridPosition (&OldX, &OldY);
		int32 NewX = OldX + tScript.fMission.mRandomInt32 (inner, outer);
		int32 NewY = OldY + tScript.fMission.mRandomInt32 (inner, outer);

		Ship->mSetFinalDestination(NewX, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// make the peaceful team patrol the planet
void PeacefulOrbit(tTeamInfo* PeacefulTeam)
{
	tShipInfo* Ship;

	tMapPoint waypoints[6]=
	{
		17, 15,
		18, 17,
		22, 17,
		23, 15,
		22, 13,
		19, 13,
	};

	tMapPoint PatrolPoints[6];

	for (Ship = PeacefulTeam->mGetFirstShip(); Ship != NULL; Ship = PeacefulTeam->mGetNextShip())
	{
		PatrolPoints[0] = waypoints[0];
		PatrolPoints[1] = waypoints[1];
		PatrolPoints[2] = waypoints[2];
		PatrolPoints[3] = waypoints[3];
		PatrolPoints[4] = waypoints[4];
		PatrolPoints[5] = waypoints[5];
		Ship->mPatrol (1.0, 6, &PatrolPoints[0], kHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
// Find the closest ship to the player
tShipInfo* FindClosestShip(tScript& tScript)
{
	tTeamInfo* IlvuriusTeam = tScript.fMission.mGetTeamHandle (kIlvuriusTeam);
	tTeamInfo* LesaniusTeam = tScript.fMission.mGetTeamHandle (kLesaniusTeam);
	tTeamInfo* PeacefulTeam = tScript.fMission.mGetTeamHandle (kPeacefulTeam);

	tShipInfo* Ship;
	tShipInfo* ClosestShip = NULL;
	float ClosestShipDistance = 5555555.5;

	if (DetectIlvurius == false)
	{
		for (Ship = IlvuriusTeam->mGetFirstShip(); Ship != NULL; Ship = IlvuriusTeam->mGetNextShip())
		{
			if ((tScript.fPlayerShip->mGetTacticalDistance(Ship) < ClosestShipDistance))
			{
				ClosestShip = Ship;
				ClosestShipDistance = tScript.fPlayerShip->mGetTacticalDistance(Ship);
			}
		}
	}

	if ((DetectLesanius == false) && (LesaniusExists == true))
	{
		for (Ship = LesaniusTeam->mGetFirstShip(); Ship != NULL; Ship = LesaniusTeam->mGetNextShip())
		{
			if ((tScript.fPlayerShip->mGetTacticalDistance(Ship) < ClosestShipDistance))
			{
				ClosestShip = Ship;
				ClosestShipDistance = tScript.fPlayerShip->mGetTacticalDistance(Ship);
			}
		}
	}
	return ClosestShip;
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	switch ( TeamMedicineWasTransportedTo )
	{
		// LNF should the player get the mission complete message if they transport
		// LNF medicine to the wrong group?

		// LNF maybe the player gets some prestige for giving it to anyone?
		case kIlvuriusFaction: // both elites have Ilvurius
			tScript.fMission.mDisplayMessage( kGiveMedicineToIlvurius_msg, kMissionScriptMessage );
			break;

		case kLesaniusFaction: // only the Shiar has the Lesanius faction
			// this is a good choice for the Shiar player
			// but the player can also deliver to the planet
			// the player should get a bonus for delivering to Lesanius
			break;

		case kPlanetFaction:
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesCompletePlanet_msg, kMissionScriptMessage );
			break;

		case kPeacefulFaction:
			if ( Elite == kPraiex )
			{
				tScript.fMission.mDisplayMessage( kGiveMedicineToPeacefulPraiex_msg, kMissionScriptMessage );
			}
			else // Elite == kShiar
			{
				tScript.fMission.mDisplayMessage( kGiveMedicineToPeacefulShiar_msg, kMissionScriptMessage );
			}				
			break;
	}

	if ( TransportedMedicine == 1 )
	{
		//tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
	}
	else
	{
		//tScript.fMission.mDisplayMessage( kGetFreighterToPlanet_msg, kMissionScriptMessage );
	}
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
