#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_4MissionText.h"
#include "Rom_4MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 BaseDefenderDestroyed = 0;
int32 BaseDestroyed = 0;
int32 PlatformDestroyed = 0;
int32 OriossDestroyed = 0;
int32 AtWarWithNeighbor = 0;
int32 AtWarWithOrioss = 0;
int32 BaseRaided = 0;
int32 AttackedOrioss = 0;

int32 LNFEnemyRace = 0;

const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);
void SpewMessage(tScript& tScript, TextMessages MessageNumber);
void CheckWinCondition( tScript& tScript );

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kBaseDefenderTeam, 
	kBaseTeam,
	kPlatformTeam,
	kOriossTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kFedMission | kKlingMission | kGornMission;  // LNF I hope this works
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 50) ? kFederation
					  : (fMission.mRandomInt32 (1, 100) < 50) ? kKlingon : kGorn;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseDefenderTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kBaseTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kPlatformTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kOriossTeam, kRomulan);

	// LNF
	LNFEnemyRace = info.fEnemyRace;

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kOriossTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseDefenderTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenderTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenderTeam, kOriossTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kOriossTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlatformTeam, kOriossTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
	}
	else
	{
		Elite = kShiar;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* OriossTeam = fMission.mGetTeamHandle (kOriossTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .75;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	// create Base Defender ships
	BaseDefenderTeam->mCreateFleet (3, TotalBPV, kClassDestroyer, kClassHeavyCruiser, 50, kStartPosition_W);

	// create Base
	if ( LNFEnemyRace == kFederation )
	{
		BaseTeam->mCreateShip ("F-BS", kStartPosition_Z);
	}
	else if ( LNFEnemyRace == kKlingon )
	{
		BaseTeam->mCreateShip ("K-BS", kStartPosition_Z);
	}
	else if ( LNFEnemyRace == kGorn )
	{
		BaseTeam->mCreateShip ("G-BS", kStartPosition_Z);
	}
	

	// create Platform team
	if ( LNFEnemyRace == kFederation )
	{
		PlatformTeam->mCreateShip ("F-DEF", kStartPosition_S);
		PlatformTeam->mCreateShip ("F-DEF", kStartPosition_T);
		PlatformTeam->mCreateShip ("F-DEF", kStartPosition_U);
		PlatformTeam->mCreateShip ("F-DEF", kStartPosition_V);
	}
	else if ( LNFEnemyRace == kKlingon )
	{
		PlatformTeam->mCreateShip ("K-DEF", kStartPosition_S);
		PlatformTeam->mCreateShip ("K-DEF", kStartPosition_T);
		PlatformTeam->mCreateShip ("K-DEF", kStartPosition_U);
		PlatformTeam->mCreateShip ("K-DEF", kStartPosition_V);
	}
	else if ( LNFEnemyRace == kGorn )
	{
		PlatformTeam->mCreateShip ("G-DEF", kStartPosition_S);
		PlatformTeam->mCreateShip ("G-DEF", kStartPosition_T);
		PlatformTeam->mCreateShip ("G-DEF", kStartPosition_U);
		PlatformTeam->mCreateShip ("G-DEF", kStartPosition_V);
	}
	

	// create Orioss ship, same class as player
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (Ship->mGetClassType() > PlayerShipClass)
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}
	OriossTeam->mCreateShip (LowBPV, PlayerShipClass, PlayerShipClass, 30, kStartPosition_J);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* OriossTeam = fMission.mGetTeamHandle (kOriossTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
/*	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}
*/
	// configure BaseDefender ships
	for (Ship = BaseDefenderTeam->mGetFirstShip(); Ship != NULL; Ship = BaseDefenderTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}
	BaseDefenderTeam->mDefendEntity(BaseTeam->mGetFirstShip(), 70.0, 100.0, kVeryHighPriority);

	// configure Base
	for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Platform team
	for (Ship = PlatformTeam->mGetFirstShip(); Ship != NULL; Ship = PlatformTeam->mGetNextShip())
	{
		Ship->mDamageSystem(kDamageLeftWarpEngine, 100);
		Ship->mDamageSystem(kDamageRightWarpEngine, 100);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Orioss' Ship
	Ship = OriossTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mChangeName(kOrioss_name);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mDefendEntity(fPlayerShip, 40.0, 40.0, kHighPriority);
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 2);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* OriossTeam = fMission.mGetTeamHandle (kOriossTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ((Ship->mGetTeam() == kOriossTeam) &&
						(HostileShip->mGetTeam() == kPlayerTeam) &&
						(AttackedOrioss == 0))
					{
						fMission.mDisplayMessage(kEvent1_msg, kMissionScriptMessage);
						AttackedOrioss = 1;
						Ship->mSetShipCallbackEvents (kNotifyDamage);
					}
					else if ((Ship->mGetTeam() == kOriossTeam) &&
							(HostileShip->mGetTeam() == kPlayerTeam) && (AttackedOrioss == 1))
					{
						fMission.mDisplayMessage(kEvent2_msg, kMissionScriptMessage);
						OriossTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kOriossTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AttackedOrioss = 2;
					}
					else if ((Ship->mGetTeam() == kBaseTeam) || (Ship->mGetTeam() == kBaseDefenderTeam) ||
							(Ship->mGetTeam() == kPlatformTeam))
					{
						OriossTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kOriossTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kOriossTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kOriossTeam, kPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AtWarWithNeighbor = 1;
					}
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kOriossTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kBaseDefenderTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseDefenderTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseDefenderTeam, kOriossTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kBaseTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseTeam, kOriossTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlatformTeam, kOriossTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerTurnsSet_10: // COMM Button timer
					Ship->mClearCommButton (2);
					Ship->mSetCommButton (1, kButtonHail_text);
					break;
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				BaseDestroyed++;
				// LNF need some victory condition things
				fMission.mDisplayMessage( kEvent4BaseCaptured_msg, kMissionScriptMessage );
				BaseRaided = 1;

				fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
				FindClosestMapEdge( *this, fPlayerShip, kMapEdgeX, kMapEdgeY );

				// LNF display a message telling player to leave map to finish mission?
				if (BaseTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kBaseDefenderTeam)
			{
				BaseDefenderDestroyed++;
				if (BaseDefenderTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kPlatformTeam)
			{
				PlatformDestroyed++;
				if (PlatformTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kOriossTeam)
			{
				OriossDestroyed++;
				if (OriossTeam->mGetShipsInPlayCount() == 0)
				{
				}

				if (fPlayerShip->mGetTacticalDistance(Ship) < 100.0)
				{
					fMission.mDisplayMessage(kEvent3OriossDestroyed_msg, kScienceScriptMessage);
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kBaseDefenderTeam)
			{
				BaseDefenderDestroyed++;
				if (BaseDefenderTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kBaseTeam)
			{
				BaseDestroyed++;
				if (BaseTeam->mGetShipsInPlayCount() == 0)
				{
					if ( BaseRaided == 0 )
					{
						fMission.mDisplayMessage( kBaseDestroyed_msg, kMissionScriptMessage );
						return eGameStatus(kPlayerLoses | kExpellFromElite);
					}
				}
			}
			else if (Ship->mGetTeam() == kPlatformTeam)
			{
				PlatformDestroyed++;
				if (PlatformTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kOriossTeam)
			{
				OriossDestroyed++;
				if (OriossTeam->mGetShipsInPlayCount() == 0)
				{
				}

				if (fPlayerShip->mGetTacticalDistance(Ship) < 100.0)
				{
					fMission.mDisplayMessage(kEvent3OriossDestroyed_msg, kScienceScriptMessage);
				}
			}

			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								if (AtWarWithNeighbor == 0)
								{
									SpewMessage(*this, kAnyHail_msg);
								}
								else
								{
									SpewMessage(*this, kNeighborHail_msg);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kBaseTeam:
								if (BaseRaided == 0)
								{
									fMission.mDisplayMessage(kBaseHailNotRaided_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kBaseHailRaided_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonHail_text);
								break;

							case kOriossTeam:
								if ((AtWarWithNeighbor == 0) && (AtWarWithOrioss == 0))
								{
									fMission.mDisplayMessage(kOriossHailNic_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else if ((AtWarWithNeighbor == 1) && (AtWarWithOrioss == 0))
								{
									if (Ship->mGetDamagePercent() < 50.0)
									{
										fMission.mDisplayMessage(kOriossHailOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kOriossHailDamaged_msg, kMissionScriptMessage);
									}
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else if (AtWarWithOrioss == 1)
								{
									fMission.mDisplayMessage(kOrioss_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
									Ship->mSetCommButton (2, kButtonSurrender_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								}
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								if (Ship->mGetDamagePercent() < 50.0)
								{
									SpewMessage (*this, kNeighborSurrenderOK_msg);
								}
								else
								{
									SpewMessage (*this, kNeighborSurrenderDamaged_msg);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOriossTeam:
								fMission.mDisplayMessage(kOrioss_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								SpewMessage (*this, kNeighborTaunt_msg);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kOriossTeam:
								fMission.mDisplayMessage(kOrioss_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								break;
						}
						break;
				}
			}
			break;

	}

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		// LNF change this victory thing for draw if you want.
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (BaseRaided * 80);

	if (Elite == kPraiex)
	{
		if ( OriossDestroyed == 0 )
		{
			VictoryPoints += 20;
		}
	}
	else
	{
		if (( OriossDestroyed > 0 ) && ( AttackedOrioss == 0 ))
		{
			VictoryPoints += 20;
		}
		else if (( OriossDestroyed > 0 ) && ( AttackedOrioss == 1 ))
		{
			VictoryPoints += 10;
		}
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kPraiexAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				fMission.mAddToDebriefingList(kPraiexVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
			}
		}
		else
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kShiarAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				// PraiexVictory_msg used for both factions
				fMission.mAddToDebriefingList(kShiarVictory_msg); 
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (VictoryLevel + kShiarAstoundingVictory_msg);
			}
		}

		//************  this handles the Expulsion
		if ( VictoryLevel >= kDefeat )
		{
			if ( Elite == kPraiex )
			{
				fMission.mAddToDebriefingList( kPraiexExpelled_msg );
			}
			else // Elite == kShair
			{
				fMission.mAddToDebriefingList( kShiarExpelled_msg );
			}
			// LNF try this out:  adding results of primary objectives
			fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

			if ( Elite == kPraiex )
			{
				( BaseRaided != 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedTrue_msg ) : SuccessCount++;
				( OriossDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOriossSavedTrue_msg ) : SuccessCount++;
				if ( SuccessCount == 2 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}

				fMission.mAddToDebriefingList( kFailedObjectives_msg );

				( BaseRaided == 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedFalse_msg ) : FailureCount++;
				( OriossDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOriossSavedFalse_msg ) : FailureCount++;
				if ( FailureCount == 2 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}
			}
			else // Shiar win conditions
			{
				( BaseRaided != 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedTrue_msg ) : SuccessCount++;
				( OriossDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOriossDestroyedTrue_msg ) : SuccessCount++;
				if ( SuccessCount == 2 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}

				fMission.mAddToDebriefingList( kFailedObjectives_msg );

				( BaseRaided == 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedFalse_msg ) : FailureCount++;
				( OriossDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOriossDestroyedFalse_msg ) : FailureCount++;
				if ( FailureCount == 2 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}
			}

			return;
		}
		//****************** this handles the expulsion

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( BaseRaided != 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedTrue_msg ) : SuccessCount++;
		( OriossDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOriossSavedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( BaseRaided == 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedFalse_msg ) : FailureCount++;
		( OriossDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOriossSavedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		( BaseRaided != 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedTrue_msg ) : SuccessCount++;
		( OriossDestroyed != 0 ) ? fMission.mAddToDebriefingList( kOriossDestroyedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( BaseRaided == 0 ) ? fMission.mAddToDebriefingList( kBaseRaidedFalse_msg ) : FailureCount++;
		( OriossDestroyed == 0 ) ? fMission.mAddToDebriefingList( kOriossDestroyedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(1, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// spew out the proper text message depending on Aggressor team
void SpewMessage(tScript& tScript, TextMessages MessageNumber)
{
	if (LNFEnemyRace == kFederation)
	{
		tScript.fMission.mDisplayMessage ((MessageNumber + (0 * 5)), kMissionScriptMessage);
	}
	if (LNFEnemyRace == kKlingon)
	{
		tScript.fMission.mDisplayMessage ((MessageNumber + (1 * 5)), kMissionScriptMessage);
	}
	if (LNFEnemyRace == kGorn)
	{
		tScript.fMission.mDisplayMessage ((MessageNumber + (2 * 5)), kMissionScriptMessage);
	}
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
/*
	tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
	FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
*/
}
