#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_5MissionText.h"
#include "Rom_5MissionMaps.h"
/***************************************
LNF need to put in stuff to make the ai controlled Tal Shiar
LNF ship, try to capture the Base, or if not, at least
LNF know that the base is captured, and try to transport
LNF the black box aboard the base.
LNF don't have to capture the base in this mission, maybe ride hit and run
***************************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 BaseDefenderDestroyed = 0;
int32 BaseDestroyed = 0;
int32 PlatformDestroyed = 0;
int32 EscortDestroyed = 0; // mission goal BOTH
int32 AtWarWithOrion = 0;
int32 AtWarWithEscort = 0;
int32 BaseRaided = 0;
int32 AttackedEscort = 0;
int32 BlackBoxDelivered = 0; // mission goal SHIAR
int32 DataRetrieved = 0; // mission goal BOTH
int32 BoxFound = 0; // found box by scanning mission goal PRAIEX
int32 EnemyShipVictoryValue = 0; // the number of points the enemy ships are worth for Victory Points

const float kDeepScanDistance = 30.0;
const float kDamageBaseForTransport = 30.0;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool AiShiarDeliveredBlackBox = false; // the ai controlled ship delivered the black box

void DefendPlayersWeakestShip( tScript& tScript, tTeamInfo* Defender, tTeamInfo* Defendee, float AttackRadius, float PursuitRadius );
void CheckWinCondition( tScript& tScript );

int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kBaseDefenderTeam, 
	kBaseTeam,
	kPlatformTeam,
	kEscortTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kFedMission | kKlingMission | kGornMission;  // LNF I hope this works
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseDefenderTeam, kOrion);
	fMission.mSetTeamRace (kBaseTeam, kOrion);
	fMission.mSetTeamRace (kPlatformTeam, kOrion);
	fMission.mSetTeamRace (kEscortTeam, kRomulan);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseDefenderTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenderTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenderTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlatformTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
	}
	else
	{
		Elite = kShiar;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// calculate how much each enemy ship is worth for the Victory Points
	EnemyShipVictoryValue = 12 / NumberOfPlayersShips;
	

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	// create Base Defender ships
	// there are four letters reserved.  M, N, O, P
	for ( int32 count = 0; count < NumberOfPlayersShips; count++ )
	{
		BaseDefenderTeam->mCreateShip (AveragePlayerBPV, kClassDestroyer, NewEnemyShipClass( *this, LargestEnemyShipClass ), 50, kStartPosition_M + count);
	}
	//BaseDefenderTeam->mCreateFleet (NumberOfPlayersShips, TotalBPV, kClassDestroyer, PlayerShipClass, 50, kStartPosition_W);

	// create Base
	BaseTeam->mCreateShip ("O-BS+", kStartPosition_Z);

	// create Platform team
	//PlatformTeam->mCreateFleet (4, kMaxBPV, kClassBattleship, kClassBattleship, 50, kStartPosition_S);
	PlatformTeam->mCreateShip ("O-DEF", kStartPosition_S);
	PlatformTeam->mCreateShip ("O-DEF", kStartPosition_T);
	PlatformTeam->mCreateShip ("O-DEF", kStartPosition_U);
	PlatformTeam->mCreateShip ("O-DEF", kStartPosition_V);

	// create Escort ship
	EscortTeam->mCreateShip ((TotalBPV / NumberOfPlayersShips), PlayerShipClass, PlayerShipClass, 30, kStartPosition_J);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	if ( Elite == kShiar )
	{
		// LNF change from kTransBlackBox to BlackBox
		fPlayerShip->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 1);
	}

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
	}

	// configure BaseDefender ships
	for (Ship = BaseDefenderTeam->mGetFirstShip(); Ship != NULL; Ship = BaseDefenderTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}
	BaseDefenderTeam->mDefendEntity(BaseTeam->mGetFirstShip(), 70.0, 100.0, kVeryHighPriority);

	// configure Base
	for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Platform team
	for (Ship = PlatformTeam->mGetFirstShip(); Ship != NULL; Ship = PlatformTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure the other faction's ship
	Ship = EscortTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	// LNF where does this line get turned off?
	Ship->mDefendEntity(fPlayerShip, 15.0, 15.0, kHighPriority);

	if ( Elite == kPraiex )
	{
		// LNF change from kTransBlackBox to BlackBox
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 1);
		Ship->mChangeName( kTalShiar_name );
	}
	else
	{
		Ship->mChangeName( kTalPraiex_name );
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan base from within kDeepScanDistance for 1 turn
	DeepScanTracker.mSetTarget (BaseTeam->mGetThisShip(1), kDeepScanDistance, 2);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				if ( TransportTarget != NULL )
				{
					if ( Elite == kShiar )
					{
						// if the target is the base and the item transported is the black box
						if (( TransportTarget == BaseTeam->mGetFirstShip() ) &&
							( fEventItem1 == kTransBlackBox ))
						{
							fMission.mDisplayMessage( kEvent4TransportBlackBox_msg, kScienceScriptMessage );
							BlackBoxDelivered = 1;
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );

							CheckWinCondition( *this );
						}
						else
						{
							// reset the callback
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
						}
					}
				}
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportFrom = fMission.mFindShipID( fEventID );

				if ( TransportFrom != NULL )
				{
					if ( Elite == kShiar )
					{
						// LNF change kTransBlackBox to Black Box
						if (( BlackBoxDelivered == 1 ) && ( fEventItem1 == kTransBlackBox ))
						{
							// don't let the player take back the black box
							Ship->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 0);
							TransportFrom->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 1);
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
						}
						// LNF check Praiex scenario if player tries to take gorn egg from shiar ship?
					}
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);
	
				if ( HostileShip != NULL )
				{
					// if the ESCORT is shot at by the PLAYER
					if ((Ship->mGetTeam() == kEscortTeam) &&
						(HostileShip->mGetTeam() == kPlayerTeam) &&
						(AttackedEscort == 0))
					{
						fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						//EscortTeam->mGetFirstShip()->mClearCommands();
						AttackedEscort = 1;
					}
					// if the BASE is shot at by the ESCORT
					else if (( Ship->mGetTeam() == kBaseTeam ) && ( HostileShip->mGetTeam() == kEscortTeam ) &&
							 ( Elite == kPraiex ) && ( BlackBoxDelivered == 0 ))
					{
						// LNF if the EscortTeam does any damage, then tranport black box and leave the system

						if ( Ship->mGetDamagePercent() > kDamageBaseForTransport )
						{
							HostileShip->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 0);

							fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
							fMission.mGetTeamHandle(HostileShip->mGetTeam())->mDisengageShips( kByMapEdge, kAbsolutePriority );

							fMission.mDisplayMessage( kPraiexBlackBoxDelivered_msg, kMissionScriptMessage );

							BlackBoxDelivered = 1;

							// if the Data has already been retrieved
							if ( DataRetrieved == 1 )
							{
								// reset the DeepScanTracker to scan for the Black Box
								DeepScanTracker.mSetTarget (BaseTeam->mGetThisShip(1), kDeepScanDistance, 1);
							}
							else // DataRetrieved == 0
							{
								// wait for the player to scan the base for the data, then
								// reset the Deep Scan Tracker
								fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
							}
						}
/*
						// this should be the AI Shiar ship transporting the black box onto the base
						switch (fEventItem1)
						{
							case kDamageWeaponByMarines:
							case kDamageSystemByMarines:
								// make the ship 'transport' the black box
								//Ship->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 1);
								HostileShip->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 0);

								fMission.mDisplayMessage( kPraiexBlackBoxDelivered_msg, kMissionScriptMessage );
								// if the Data has already been retrieved
								if ( DataRetrieved == 1 )
								{
									// reset the DeepScanTracker to scan for the Black Box
									DeepScanTracker.mSetTarget (BaseTeam->mGetThisShip(1), kDeepScanDistance, 3);
								}
								else // DataRetrieved == 0
								{
									// wait for the player to scan the base for the data, then
									// reset the Deep Scan Tracker
									fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
								}
								break;

							case kDamageByWeaponOrDebris:
								// nothing extra to test
								break;
						}
*/
					}
					// LNF temp conditional
/*
					else if (( Ship->mGetTeam() == kBaseTeam ) &&
							 ( EscortTeam->mGetFirstShip()->mGetTacticalDistance(BaseTeam->mGetFirstShip()) <= 5.0 ))
					{
						BlackBoxDelivered = 1;
						fMission.mDisplayMessage( kPraiexBlackBoxDelivered_msg, kMissionScriptMessage );
						// if the Data has already been retrieved
						if ( DataRetrieved == 1 )
						{
							// reset the DeepScanTracker to scan for the Black Box
							DeepScanTracker.mSetTarget (BaseTeam->mGetThisShip(1), kDeepScanDistance, 3);
						}
						else // DataRetrieved == 0
						{
							// wait for the player to scan the base for the data, then
							// reset the Deep Scan Tracker
							fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
						}
					}
					//-----------------------------------------------------------------
*/
					// if any other ship is damaged set AtWarWithOrion
					else if (((Ship->mGetTeam() == kBaseTeam) || (Ship->mGetTeam() == kBaseDefenderTeam) ||
							  (Ship->mGetTeam() == kPlatformTeam)) && ( AtWarWithOrion == 0 ))
					{
						//EscortTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kEscortTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kEscortTeam, kPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AtWarWithOrion = 1;
					}
				}
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kBaseDefenderTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kBaseDefenderTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseDefenderTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseDefenderTeam, kEscortTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					// LNF at some point, the escort needs to be told to capture the base
					// LNF maybe based on both timer and how many defenders are left?
					fMission.mSetTeamRelations (kBaseTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);//***
					fMission.mSetTeamRelations (kBaseTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamHostile, kGoalCapture, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlatformTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlatformTeam, kEscortTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					// display scenario start message
					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					// waits for the player to scan the base for the data
					// then resets Deep Scan Tracker to scan for the Black Box

					// LNF maybe some kind of 'rescan for black box' message?
					if ( DataRetrieved == 1 )
					{
						DeepScanTracker.mSetTarget (BaseTeam->mGetThisShip(1), kDeepScanDistance, 3);
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, 2 );
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// notify player to turn scan on
								if ( DataRetrieved == 0 )
								{
									fMission.mDisplayMessage (kScannerOffInRange_msg, kScienceScriptMessage);
								}
								else if ( BlackBoxDelivered == 1 )
								{
									fMission.mDisplayMessage (kScannerOffInRangeBB_msg, kScienceScriptMessage);
								}
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// notify player that timer has begun
								if ( DataRetrieved == 0 )
								{
									fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								}
								else if ( BlackBoxDelivered == 1 )
								{
									fMission.mDisplayMessage (kStartingScanBB_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// notify player that timer has begun
								if ( DataRetrieved == 0 )
								{
									fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								}
								else if ( BlackBoxDelivered == 1 )
								{
									fMission.mDisplayMessage (kStartingScanBB_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// notify player that timer is now off

								// LNF check here if the base gets destroyed while scanning
								if (( BaseDestroyed == 1 ) && ( DataRetrieved == 0 ) && ( BoxFound == 0 ))
								{
									fMission.mDisplayMessage( kBaseDestroyedWhileScanning_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								}
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							fMission.mDisplayMessage( kEvent3SuccessfulScan_msg, kScienceScriptMessage );

							DataRetrieved = 1;
							if ( Elite == kPraiex )
							{
								if ( BlackBoxDelivered == 1 )
								{
									fMission.mDisplayMessage( kEvent3ScanWithBlackBox_msg, kScienceScriptMessage );
									BoxFound = 1;
								}
								else
								{
									// The escort (shiar) ship should try to capture the station.
									// and deliver the black box
									// LNF maybe I should check the shields on the base station and deliver the black box
									// LNF is the Escort team takes down a shield and is within transporter range?
									fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamAtWar, kGoalDestroy, 1.0);
									fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0);
									// LNF try to reset the deep scan tracker when the Escort ship 'transports' black box?
								}
							}
							CheckWinCondition( *this );
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
					break;

				case kTimerTurnsSet_9: // COMM Button timer for Escort team
					for (Ship = EscortTeam->mGetFirstShip(); Ship != NULL; Ship = EscortTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10: // COMM Button timer for BaseDefender
					for (Ship = BaseDefenderTeam->mGetFirstShip(); Ship != NULL; Ship = BaseDefenderTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyOnCapture:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );
				tShipInfo* PShip = NULL;

				// LNF if the player is Praiex and the base is captured, the Escort ship (Shiar) should
				// LNF try to transport the black box aboard the captured station
				// LNF might have to ride the hit and run.  Don't have to capture a base in this mission
				fMission.mAddToBonusPrestige( kCaptureBonus );

				if ( HostileShip != NULL )
				{
					if (Ship->mGetTeam() == kBaseDefenderTeam)
					{
						// LNF maybe when all of the enemies are destroyed, or near destroyed, the
						// LNF ai ship, if the player is Praiex, should try to capture the base station?


						// LNF need to check these numbers to make sure they are not taking into account
						// LNF the +1 for the previous capture stuff
						//BaseDefenderDestroyed++;

						Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
						Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
/*						
						if (( PlatformTeam->mGetShipsInPlayCount() < 4 ) &&
							( BaseDefenderTeam->mGetShipsInPlayCount() <= 3 ))
						{
							EscortTeam->mClearCommands();
							fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0);
							fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalCapture, 1.0);
						}
*/
					}
					else if (Ship->mGetTeam() == kBaseTeam)
					{
						// base has been captured
						//BaseDestroyed++;

						Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
						Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );

						DefendPlayersWeakestShip( *this, EscortTeam, PlayerTeam, 30.0, 30.0 );

						if ( HostileShip->mGetTeam() == kEscortTeam )
						{
							// if it was captured by the escort team, then make the escort team
							// defend the weakest ship on the Player's team

							// LNF If the player is Praiex, check to see if the escort has transported
							// LNF the black box, if not, transport it.
							// LNF need to check in the death too
							DefendPlayersWeakestShip( *this, EscortTeam, PlayerTeam, 30.0, 30.0 );
						}
					}
					else if (Ship->mGetTeam() == kPlatformTeam)
					{
						//PlatformDestroyed++;

						Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
						Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );

						if (( PlatformTeam->mGetShipsInPlayCount() < 4 ) &&
							( BaseDefenderTeam->mGetShipsInPlayCount() <= 3 ))
						{
							EscortTeam->mClearCommands();
							fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0);
							fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalCapture, 1.0);
						}
					}
					else if (Ship->mGetTeam() == kEscortTeam)
					{
						EscortDestroyed++;
					}
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kBaseDefenderTeam)
			{
				// LNF maybe when all of the enemies are destroyed, or near destroyed, the
				// LNF ai ship, if the player is Praiex, should try to capture the base station?
				BaseDefenderDestroyed++;
				if (( PlatformTeam->mGetShipsInPlayCount() < 3 ) &&
					( BaseDefenderTeam->mGetShipsInPlayCount() <= 2 ))
				{
					EscortTeam->mClearCommands();
					fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0);
					fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalCapture, 1.0);
				}
			}
			else if (Ship->mGetTeam() == kBaseTeam)
			{
				BaseDestroyed++;

				if ( DataRetrieved == 0 )
				{
					fMission.mDisplayMessage( kBaseDestroyed_msg, kMissionScriptMessage );
					return kPlayerLoses;
					// LNF should check win conditions here...?
				}
			}
			else if (Ship->mGetTeam() == kPlatformTeam)
			{
				PlatformDestroyed++;
				if (( PlatformTeam->mGetShipsInPlayCount() < 3 ) &&
					( BaseDefenderTeam->mGetShipsInPlayCount() <= 2 ))
				{
					EscortTeam->mClearCommands();
					fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0);
					fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalCapture, 1.0);
				}
			}
			else if (Ship->mGetTeam() == kEscortTeam)
			{
				EscortDestroyed++;
			}

			break;

		case kNotifyDisengaged:
			if ( Ship->mGetTeam() == kPlayerTeam )
			{
				if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mSetGoalIndicator( kFalse, 0, 0 );
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								fMission.mDisplayMessage(kOrionHail_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kBaseTeam:
								if (BaseRaided == 0)
								{
									fMission.mDisplayMessage(kBaseHailNotRaided_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kBaseHailRaided_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonHail_text);
								break;

							case kEscortTeam:
								if ((AtWarWithOrion == 0) && (AtWarWithEscort == 0))
								{
									fMission.mDisplayMessage(kRomHailNic_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else if ((AtWarWithOrion == 1) && (AtWarWithEscort == 0))
								{
									if (Ship->mGetDamagePercent() < 50.0)
									{
										fMission.mDisplayMessage(kRomHailOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kRomHailDamaged_msg, kMissionScriptMessage);
									}
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else if (AtWarWithEscort == 1)
								{
									fMission.mDisplayMessage(kRomMisc_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
									Ship->mSetCommButton (2, kButtonSurrender_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								}
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								if (Ship->mGetDamagePercent() < 50.0)
								{
									fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								fMission.mDisplayMessage(kRomMisc_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								fMission.mDisplayMessage(kOrionTaunt_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								fMission.mDisplayMessage(kRomMisc_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF modify the points thing to correspond to how many enemy ships there are

	if (Elite == kPraiex)
	{
		VictoryPoints = (DataRetrieved * 80) + (BaseDefenderDestroyed * EnemyShipVictoryValue) +
						(BoxFound * 10 ) - (EscortDestroyed * 30);
	}
	else
	{
		VictoryPoints = (DataRetrieved * 70) + (BaseDefenderDestroyed * EnemyShipVictoryValue) + (BlackBoxDelivered * 20) -
						(EscortDestroyed * 30);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 69)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kPraiexAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				fMission.mAddToDebriefingList(kPraiexVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
			}
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kShiarAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( DataRetrieved != 0 ) ? fMission.mAddToDebriefingList( kDataRetrievedTrue_msg ) : SuccessCount++;
		( BoxFound != 0 ) ? fMission.mAddToDebriefingList( kBoxFoundTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( DataRetrieved == 0 ) ? fMission.mAddToDebriefingList( kDataRetrievedFalse_msg ) : FailureCount++;
		( BoxFound == 0 ) ? fMission.mAddToDebriefingList( kBoxFoundFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		( DataRetrieved != 0 ) ? fMission.mAddToDebriefingList( kDataRetrievedTrue_msg ) : SuccessCount++;
		( BlackBoxDelivered != 0 ) ? fMission.mAddToDebriefingList( kBlackBoxDeliveredTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( DataRetrieved == 0 ) ? fMission.mAddToDebriefingList( kDataRetrievedFalse_msg ) : FailureCount++;
		( BlackBoxDelivered == 0 ) ? fMission.mAddToDebriefingList( kBlackBoxDeliveredFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(1, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// Defend player's weakest ship
void DefendPlayersWeakestShip( tScript& tScript, tTeamInfo* Defender, tTeamInfo* Defendee, float AttackRadius, float PursuitRadius )
{
	float DefendeeShipDamage = -1.0;
	tShipInfo* Ship = NULL;
	tShipInfo* DefendeeShip = NULL;

	for (Ship = Defendee->mGetFirstShip(); Ship != NULL; Ship = Defendee->mGetNextShip())
	{
		if ( Ship->mGetDamagePercent() > DefendeeShipDamage )
		{
			DefendeeShipDamage = Ship->mGetDamagePercent();
			DefendeeShip = Ship;
		}
	}

	if ( DefendeeShip != NULL )
	{
		Defender->mClearCommands();
		Defender->mDefendEntity( DefendeeShip, AttackRadius, PursuitRadius, kAbsolutePriority );
	}
	else
	{
		Defender->mDefendEntity( tScript.fPlayerShip, AttackRadius, PursuitRadius, kVeryHighPriority );
	}
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	// LNF this also needs to check if the player is Shiar and if the black box was transported
	if ( Elite == kPraiex )
	{
		if ( DataRetrieved == 1 )
		{
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
		}
	}
	else // Elite == kShiar
	{
		if (( DataRetrieved == 1 ) && ( BlackBoxDelivered == 1 ))
		{
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
		}
	}
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
