#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_6MissionText.h"
#include "Rom_6MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 KlingonsArrive = 0;
int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 BaseDefenderDestroyed = 0;

int32 AllPlatformsDestroyed = 0;
int32 AllKlingonsDestroyed = 0;
int32 AllBaseDefendersDestroyed = 0;

int32 BaseDestroyed = 0; // win condition BOTH
int32 PlatformDestroyed = 0;
int32 EscortDestroyed = 0;
int32 KlingonDestroyed = 0;
int32 AtWarWithOrion = 0;
int32 AtWarWithEscort = 0;
int32 BaseRaided = 0;
int32 AttackedEscort = 0;
const float kDeepScanDistance = 10.0;
const int32 kMapEdgeX = 40.0;
const int32 kMapEdgeY = 40.0;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void CheckWinCondition( tScript& tScript );
int32 ListeningPostDestroyed = 0; // win condition PRAIEX
int32 ScannedDominatus = 0; // win condition SHIAR

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

int32 KlingonCount = 1; // number of Klingon ship created
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam);

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kBaseDefenderTeam, 
	kBaseTeam,
	kPlatformTeam,
	kEscortTeam,
	kKlingonTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kFedMission | kKlingMission | kGornMission;  // LNF I hope this works
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 50) ? kFederation
					  : (fMission.mRandomInt32 (1, 100) < 50) ? kKlingon : kGorn;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseDefenderTeam, kOrion);
	fMission.mSetTeamRace (kBaseTeam, kOrion);
	fMission.mSetTeamRace (kPlatformTeam, kOrion);
	fMission.mSetTeamRace (kEscortTeam, kRomulan);
	fMission.mSetTeamRace (kKlingonTeam, kKlingon);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseDefenderTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenderTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenderTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlatformTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
	}
	else
	{
		Elite = kShiar;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .85;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.15;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	if (Mode == 0)
	{
	/***
		LNF pick a random number of ships depending on how many the player has
		check the class of player's ships
		have some kind of table that determines balance for ship classes vs number
		< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
		choose whether to have fewer powerful ships or many smaller ships
	***/
		// create Base Defender ships
		BaseDefenderTeam->mCreateFleet (4, (2 * (TotalBPV/NumberOfPlayersShips)), kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_V);

		// create Base
		BaseTeam->mCreateShip ("O-BS", kStartPosition_Z);

		// create Platform team
		PlatformTeam->mCreateShip ("O-LP", kStartPosition_U);

		// create Escort ship
		for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
		{
			if (Ship->mGetClassType() > PlayerShipClass)
			{
				PlayerShipClass = Ship->mGetClassType();
			}
		}
		EscortTeam->mCreateShip ((LowBPV/NumberOfPlayersShips), PlayerShipClass, PlayerShipClass, 30, kStartPosition_J);
	}
	else if (Mode == 1)
	{
		fMission.mSetTeamRelations (kKlingonTeam, kPlayerTeam, kTeamAtWar, kGoalCapture, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kPlatformTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (kKlingonTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		KlingonTeam->mCreateFleet (2, 2 * (LowBPV / NumberOfPlayersShips), kClassDestroyer, kClassDestroyer, 50, kStartPosition_S);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
/*	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}
*/
	// configure BaseDefender ships
	for (Ship = BaseDefenderTeam->mGetFirstShip(); Ship != NULL; Ship = BaseDefenderTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}
	BaseDefenderTeam->mDefendEntity(BaseTeam->mGetFirstShip(), 60.0, 80.0, kVeryHighPriority);

	// configure Base
	for (Ship = BaseTeam->mGetFirstShip(); Ship != NULL; Ship = BaseTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Platform team
	for (Ship = PlatformTeam->mGetFirstShip(); Ship != NULL; Ship = PlatformTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Escort ship
	Ship = EscortTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mDefendEntity(fPlayerShip, 20.0, 20.0, kHighPriority);
	Ship->mChangeName( kDominatus_name );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 2);

	// Set a timer to bring in the Klingons
	fMission.mSetTimedInterval (kTimerLimitSet_2, 300);
	fMission.mSetTimedInterval (kTimerTurnsSet_2, 14);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	if ( Elite == kShiar )
	{
		// deep scan Escort from within kDeepScanDistance for 1 turn
		DeepScanTracker.mSetTarget (EscortTeam->mGetThisShip(1), kDeepScanDistance, 1);
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);

	DeepScanTracker.mSetTarget (KlingonTeam->mGetThisShip(KlingonCount), kDeepScanDistance, 1);
	KlingonCount++;
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ((Ship->mGetTeam() == kEscortTeam) &&
						(HostileShip->mGetTeam() == kPlayerTeam ) &&
						(AttackedEscort == 0))
					{
						EscortTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AttackedEscort = 1;
					}
					else if ((Ship->mGetTeam() == kEscortTeam) &&
							(HostileShip->mGetTeam() == kPlayerTeam ) &&
							(AttackedEscort == 1))
					{
						EscortTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AttackedEscort = 2;
					}
					else if ((Ship->mGetTeam() == kBaseTeam) || (Ship->mGetTeam() == kBaseDefenderTeam) ||
							(Ship->mGetTeam() == kPlatformTeam))
					{
						EscortTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kEscortTeam, kBaseDefenderTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kEscortTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kEscortTeam, kPlatformTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						AtWarWithOrion = 1;

						if ( Elite == kShiar )
						{
							// reset the deepscantracker so the player can scan the Escort successfully
							DeepScanTracker.mSetTarget (EscortTeam->mGetThisShip(1), kDeepScanDistance, 1);
						}
					}
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenderTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

					fMission.mSetTeamRelations (kBaseDefenderTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseDefenderTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseDefenderTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kBaseTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kBaseTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlatformTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					if (KlingonsArrive == 0)
					{
						KlingonsArrive = 1;
						fMission.mDisplayMessage(kEvent2_msg, kScienceScriptMessage);
						fMission.mCreateMapObject( 'E', 5, 22 );
						mCreateMissionShips(1);
					}
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// notify player to turn scan on
								if ( Object == EscortTeam->mGetThisShip(1) )
								{
									fMission.mDisplayMessage (kScannerOffInRangeDominatus_msg, kScienceScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kScannerOffInRangeKlingon_msg, kScienceScriptMessage);
								}
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// notify player that timer has begun
								fMission.mDisplayMessage (kStartingScan_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// notify player that timer is now off

								// LNF check if the scan target gets destroyed while scanning

								fMission.mDisplayMessage (kScanOnMoveOutOfRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if ( Object->mGetTeam() == kEscortTeam )
							{
								if (AtWarWithOrion == 0)
								{
									fMission.mDisplayMessage (kEvent3CombatFalse1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage (kEvent3CombatFalse2_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (kEvent3CombatTrue1_msg, kMissionScriptMessage);
									if (AtWarWithEscort == 0)
									{
										fMission.mDisplayMessage (kEvent3CombatTrue2_msg, kMissionScriptMessage);
										AtWarWithEscort = 1;
									}
									ScannedDominatus = 1;
									DeepScanTracker.mRemoveTarget (Object);
								}
								CheckWinCondition( *this );
							}
							else // Object->mGetTeam() == kKlingonTeam
							{
								// LNF need to make conditionals for Klingon vs. Escort
								fMission.mDisplayMessage( kEvent6ScanningKlingon_msg, kScienceScriptMessage );
								DeepScanTracker.mRemoveTarget (KlingonTeam->mGetThisShip(1));
								DeepScanTracker.mRemoveTarget (KlingonTeam->mGetThisShip(2));
							}
						}
					}
					break;

				case kTimerTurnsSet_2:
					if (KlingonsArrive == 0)
					{
						KlingonsArrive = 1;
						fMission.mDisplayMessage(kEvent2_msg, kScienceScriptMessage);
						fMission.mCreateMapObject( 'E', 5, 22 );
						mCreateMissionShips(1);
					}
					break;

				case kTimerTurnsSet_9: // COMM Button timer for Escort Team
					for (Ship = EscortTeam->mGetFirstShip(); Ship != NULL; Ship = EscortTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10: // COMM Button timer
					for (Ship = BaseDefenderTeam->mGetFirstShip(); Ship != NULL; Ship = BaseDefenderTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyOnCapture:
			{
				tShipInfo* CapturingShip = fMission.mFindShipID (fEventID);

				fMission.mAddToBonusPrestige( kCaptureBonus );

				CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipCaptured, Ship, CapturingShip );
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam() == kPlayerTeam)
				{
					PlayerDestroyed++;
					if (PlayerTeam->mGetShipsInPlayCount() == 0)
					{
						return kPlayerDestroyed;
					}
				}
				else
				{
					CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDestroyed, Ship );
				}
			}
			break;

		case kNotifyDisengaged:
			{
				CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDisengaged, Ship );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								fMission.mDisplayMessage(kOrionHail_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kBaseTeam:
								fMission.mDisplayMessage(kBaseHail_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonHail_text);
								break;

							case kEscortTeam:
								if ((AtWarWithOrion == 0) && (AtWarWithEscort == 0))
								{
									fMission.mDisplayMessage(kRomHailNic_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else if ((AtWarWithOrion == 1) && (AtWarWithEscort == 0))
								{
									if (Ship->mGetDamagePercent() < 50.0)
									{
										fMission.mDisplayMessage(kRomHailOK_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kRomHailDamaged_msg, kMissionScriptMessage);
									}
									Ship->mSetCommButton (1, kButtonHail_text);
								}
								else if (AtWarWithEscort == 1)
								{
									if (ScannedDominatus == 0)
									{
										fMission.mDisplayMessage(kRomAttackedAlly_msg, kMissionScriptMessage);
										Ship->mSetCommButton (1, kButtonTaunt_text);
										Ship->mSetCommButton (2, kButtonSurrender_text);
										fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
									}
									else
									{
										fMission.mDisplayMessage(kRomScanned_msg, kMissionScriptMessage);
										Ship->mSetCommButton (1, kButtonTaunt_text);
										Ship->mSetCommButton (2, kButtonSurrender_text);
										fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
									}
								}
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								if (Ship->mGetDamagePercent() < 50.0)
								{
									fMission.mDisplayMessage(kOrionSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kOrionSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								if (ScannedDominatus == 0)
								{
									fMission.mDisplayMessage(kRomAttackedAlly_msg, kMissionScriptMessage);
									Ship->mSetCommButton (2, kButtonSurrender_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								}
								else
								{
									fMission.mDisplayMessage(kRomScanned_msg, kMissionScriptMessage);
									Ship->mSetCommButton (2, kButtonSurrender_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								}
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kBaseDefenderTeam:
								fMission.mDisplayMessage(kOrionTaunt_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								if (ScannedDominatus == 0)
								{
									fMission.mDisplayMessage(kRomAttackedAlly_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								}
								else
								{
									fMission.mDisplayMessage(kRomScanned_msg, kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
									fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								}
								break;
						}
						break;
				}
			}
			break;

	}

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		// LNF change this victory thing for draw if you want.
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if (Elite == kPraiex)
	{
		VictoryPoints = (BaseDestroyed * 70) + (PlatformDestroyed * 20) + (BaseDefenderDestroyed * 3);
	}
	else
	{
		VictoryPoints = (BaseDestroyed * 70) + (ScannedDominatus * 15) + (EscortDestroyed * 5) + (BaseDefenderDestroyed * 3);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 69)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kShiarAstoundingVictory_msg);
		}

		//************  this handles the Expulsion
		if ( VictoryLevel >= kDefeat )
		{
			if ( Elite == kPraiex )
			{
				fMission.mAddToDebriefingList( kPraiexExpelled_msg );
			}
			else // Elite == kShair
			{
				fMission.mAddToDebriefingList( kShiarExpelled_msg );
			}
			// LNF try this out:  adding results of primary objectives
			fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

			if ( Elite == kPraiex )
			{
				( BaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedTrue_msg ) : SuccessCount++;
				if ( SuccessCount == 1 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}

				fMission.mAddToDebriefingList( kFailedObjectives_msg );

				( BaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedFalse_msg ) : FailureCount++;
				if ( FailureCount == 1 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}
			}
			else // Shiar win conditions
			{
				( BaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedTrue_msg ) : SuccessCount++;
				( ScannedDominatus != 0 ) ? fMission.mAddToDebriefingList( kScannedDominatusTrue_msg ) : SuccessCount++;
				if ( SuccessCount == 2 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}

				fMission.mAddToDebriefingList( kFailedObjectives_msg );

				( BaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedFalse_msg ) : FailureCount++;
				( ScannedDominatus == 0 ) ? fMission.mAddToDebriefingList( kScannedDominatusFalse_msg ) : FailureCount++;
				if ( FailureCount == 2 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}
			}

			return;
		}
		//****************** this handles the expulsion

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( BaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( BaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedFalse_msg ) : FailureCount++;
		if ( FailureCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		( BaseDestroyed != 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedTrue_msg ) : SuccessCount++;
		( ScannedDominatus != 0 ) ? fMission.mAddToDebriefingList( kScannedDominatusTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( BaseDestroyed == 0 ) ? fMission.mAddToDebriefingList( kBaseDestroyedFalse_msg ) : FailureCount++;
		( ScannedDominatus == 0 ) ? fMission.mAddToDebriefingList( kScannedDominatusFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}



	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// determine the number of enemy ships
int32 NumberOfEnemy(tScript& tScript, tTeamInfo* PlayerTeam)
{
	int32 NumShips = 0;
	tShipInfo* Ship;

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		PlayerShipClass = Ship->mGetClassType();
		if (PlayerShipClass <= kClassLightCruiser)
		{
			NumShips++;
		}
		else if (PlayerShipClass <= kClassHeavyCruiser)
		{
			NumShips += 2;
		}
		else 
		{
			NumShips += 3;
		}
	}
	
	if (PlayerTeam->mGetShipsInPlayCount() == 3)
	{
		NumShips += tScript.fMission.mRandomInt32(1, 2);
	}
	else if (PlayerTeam->mGetShipsInPlayCount() == 2)
	{
		NumShips += tScript.fMission.mRandomInt32(0, 1);
	}

	return NumShips;
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	if ( BaseDestroyed == 1 )
	{
		if ( Elite == kPraiex )
		{
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
		}
		else // Elite == kShiar
		{
			if ( ScannedDominatus == 1 )
			{
				tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
				FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
			}
		}
	}

	// LNF check to see if all things are destroyed
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* BaseDefenderTeam = tScript.fMission.mGetTeamHandle (kBaseDefenderTeam);
	tTeamInfo* BaseTeam = tScript.fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlatformTeam = tScript.fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* EscortTeam = tScript.fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* KlingonTeam = tScript.fMission.mGetTeamHandle (kKlingonTeam);

	// if all ships on a team are gone
	//if ( EventTeam->mGetShipsInPlayCount() == ((EventShipStatus == kEventShipCaptured) ? 1 : 0) )
	if (EventTeam == BaseDefenderTeam)
	{
		BaseDefenderDestroyed++;

		if ( EventShipStatus == kEventShipCaptured )
		{
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
			Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
		}
		else if ( BaseDefenderTeam->mGetShipsInPlayCount() == 0 )
		{
			AllBaseDefendersDestroyed = 1;
			//tScript.fMission.mDisplayMessage( kLNFBaseDefender_msg );
		}
	}
	else if (EventTeam == BaseTeam)
	{
		BaseDestroyed++;

		if ( EventShipStatus == kEventShipCaptured )
		{
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
			Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
		}
		else if ( BaseTeam->mGetShipsInPlayCount() == 0 )
		{
			CheckWinCondition( tScript );
			//tScript.fMission.mDisplayMessage( kLNFBase_msg );
		}
	}
	else if (EventTeam == PlatformTeam)
	{
		PlatformDestroyed++;

		if ( EventShipStatus == kEventShipCaptured )
		{
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
			Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 1023 );
		}
		else if ( PlatformTeam->mGetShipsInPlayCount() == 0 )
		{
			AllPlatformsDestroyed = 1;
		}
	}
	else if (EventTeam == EscortTeam)
	{
		EscortDestroyed++;

		if ( EscortTeam->mGetShipsInPlayCount() == 0 )
		{
			//tScript.fMission.mDisplayMessage( kLNFEscort_msg );
		}
	}
	else if (EventTeam == KlingonTeam)
	{
		KlingonDestroyed++;
		if ( KlingonTeam->mGetShipsInPlayCount() == 0 )
		{
			AllKlingonsDestroyed = 1;
			//tScript.fMission.mDisplayMessage( kLNFKlingon_msg );
		}
	}
}
