#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_7MissionText.h"
#include "Rom_7MissionMaps.h"

// LNF need to remove life pod dude from the player's ship at the end of the scenario

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 MissionAct = 1;
int32 KlingonsArrive = 0;
int32 ScanAlly = 0;
const int32 kMaxLifePods = 6;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kPlanetX = 15;
const int32 kPlanetY = 15;


int32 AtWarWithEscort = 0;
int32 BaseRaided = 0;
int32 AttackedEscort = 0;
tShipInfo* LastToShootAtLifePod = NULL;
void CheckWinCondition( tScript& tScript );
void PointToPod( tScript& tScript, tShipInfo* DestroyedPod = NULL );

//-----------------------------------------------
int32 FiredOnLifePods = 0;
int32 RedTalkedTo = 0;
int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 GreenPacified = 0;
int32 GreenDestroyed = 0;
int32 ShiarDestroyed = 0;
int32 ShiarPacified = 0;
int32 ShiarBetrayed = 0; // shiar ship attacked after pacified
int32 BlueDestroyed = 0;
int32 BluePacified = 0;
int32 PsionicDestroyed = 0;
int32 PsionicPacified = 0;
int32 LifePodDestroyed = 0;
int32 LifePodPacified = 0;
int32 GreenBetrayed = 0;
int32 BlueBetrayed = 0;
int32 PsionicEnemy = 0;
int32 LifePodEnemy = 0;
int32 LifePodTransported = 0;
int32 SurvivorsRemaining = kMaxLifePods;

bool ShiarAttacked = false;
bool ShiarAttackedLifePod = false;
bool AllLifePodsRescuedOrDestroyed = false;
bool AllPsionicShipsNeutralized = false;
bool AllBlueShipsNeutralized = false;
bool AllGreenShipsNeutralized = false;
bool OrderAchieved = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool PlayerAttackedLifePod = false; // player shoots at lifepod

void SetAllTeamRelationsBeginning(tScript& tScript, eTeamID kPlayerTeam);
void SetAllTeamRelations(tScript& tScript, eTeamID kPlayerTeam);
void GreenCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void ShiarCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void BlueCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void PsionicCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void LifePodCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);

struct LifePodCount
{
	tShipInfo* Ship; // ship pointer
	int32 Occupant; // is there an occupant in this pod
};

LifePodCount LifePodTransItem[kMaxLifePods];

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kGreenTeam, // attacks player
	kShiarTeam,
	kBlueTeam, // Planetary Defense
	kPsionicTeam,
	kLifePodTeam, // duh
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kFedMission | kKlingMission | kGornMission;  // LNF I hope this works
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kGreenTeam, kRomulan);
	fMission.mSetTeamRace (kShiarTeam, kRomulan);
	fMission.mSetTeamRace (kBlueTeam, kRomulan);
	fMission.mSetTeamRace (kPsionicTeam, kRomulan);
	fMission.mSetTeamRace (kLifePodTeam, kRomulan);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	SetAllTeamRelationsBeginning(*this, kPlayerTeam);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
	}
	else
	{
		Elite = kShiar;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* GreenTeam = fMission.mGetTeamHandle (kGreenTeam);
	tTeamInfo* ShiarTeam = fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* BlueTeam = fMission.mGetTeamHandle (kBlueTeam);
	tTeamInfo* PsionicTeam = fMission.mGetTeamHandle (kPsionicTeam);
	tTeamInfo* LifePodTeam = fMission.mGetTeamHandle (kLifePodTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	int32 NewNumberOfEnemyShips = 0;
	int32 NewEnemyBPV = 0;

	if ( NumberOfPlayersShips == 1 )
	{
		NewNumberOfEnemyShips = 1;
		NewEnemyBPV = AveragePlayerBPV;
	}
	else
	{
		NewNumberOfEnemyShips = 2;
		NewEnemyBPV = NewNumberOfEnemyShips * AveragePlayerBPV;
	}

	// create Green
	GreenTeam->mCreateShip (AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_P);

	// create Shiar
	ShiarTeam->mCreateFleet (NewNumberOfEnemyShips, NewEnemyBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_Y);

	// create Blue - planet defense
	BlueTeam->mCreateShip (AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_V);

	// create Psionic
	PsionicTeam->mCreateFleet (NewNumberOfEnemyShips, NewEnemyBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_W);

	// create LifePod
	LifePodTeam->mCreateShip( "N-POD", kStartPosition_Specified, fMission.mRandomInt32(8,12), fMission.mRandomInt32(13,16) ); // lp1 10, 15
	LifePodTeam->mCreateShip( "N-POD", kStartPosition_Specified, fMission.mRandomInt32(8,14), fMission.mRandomInt32(18,23) ); // lp2 11, 21
	LifePodTeam->mCreateShip( "N-POD", kStartPosition_Specified, fMission.mRandomInt32(17,20), fMission.mRandomInt32(19,23) ); // lp3 19, 23
	LifePodTeam->mCreateShip( "N-POD", kStartPosition_Specified, fMission.mRandomInt32(22,26), fMission.mRandomInt32(13,17) ); // lp4 25, 15
	LifePodTeam->mCreateShip( "N-POD", kStartPosition_Specified, fMission.mRandomInt32(20,25), fMission.mRandomInt32(6,10) );  // lp5 24, 6
	LifePodTeam->mCreateShip( "N-POD", kStartPosition_Specified, fMission.mRandomInt32(4,7), fMission.mRandomInt32(16,19) );  // lp6 4, 18

	// create Planet
	PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, kPlanetX, kPlanetY );

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// both factions use the same briefing
	return (kPraiexBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* GreenTeam = fMission.mGetTeamHandle (kGreenTeam);
	tTeamInfo* ShiarTeam = fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* BlueTeam = fMission.mGetTeamHandle (kBlueTeam);
	tTeamInfo* PsionicTeam = fMission.mGetTeamHandle (kPsionicTeam);
	tTeamInfo* LifePodTeam = fMission.mGetTeamHandle (kLifePodTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// configure Player
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}

	// configure Green
	Ship = GreenTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	//Ship->mChangeName (kGreen_name);

	// configure Shiar
	for (Ship = ShiarTeam->mGetFirstShip(); Ship != NULL; Ship = ShiarTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mChangeName (kShiar_name);
	}

	// configure Blue
	// LNF defend planet command, until surrender command (with damage %?)
	Ship = BlueTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonHail_text);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mDefendEntity( PlanetTeam->mGetFirstShip(), 50.0, 50.0, kVeryHighPriority );
	//Ship->mChangeName (kBlue_name);

	// configure Psionic
	for (Ship = PsionicTeam->mGetFirstShip(); Ship != NULL; Ship = PsionicTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mChangeName (kPsionic_name);
	}

	// configure LifePod
	int32 count = 0;
	for (Ship = LifePodTeam->mGetFirstShip(); Ship != NULL; Ship = LifePodTeam->mGetNextShip())
	{
		//Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mChangeName (kLifePod_name);
		Ship->mSetShipAIControlled( kFalse );
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 1);
		LifePodTransItem[count].Ship = Ship;
		LifePodTransItem[count].Occupant = 1;
		count++;
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 2);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* GreenTeam = fMission.mGetTeamHandle (kGreenTeam);
	tTeamInfo* ShiarTeam = fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* BlueTeam = fMission.mGetTeamHandle (kBlueTeam);
	tTeamInfo* PsionicTeam = fMission.mGetTeamHandle (kPsionicTeam);
	tTeamInfo* LifePodTeam = fMission.mGetTeamHandle (kLifePodTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				// don't let the player transport survivors anywhere
				if (( TransportTarget->mGetTeam() != kLifePodTeam ) &&
					( fEventItem1 == kTransLifePodDude ))
				{
					TransportTarget->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 0);
					Ship->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 1);
				}
				else if ( fEventItem1 == kTransLifePodDude )
				{
					for ( int32 count = 0; count < kMaxLifePods; count++ )
					{
						if (( LifePodTransItem[count].Ship == TransportTarget ) &&
							( LifePodTransItem[count].Occupant == 0 ))
						{
							TransportTarget->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 0);
							Ship->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 1);
						}
						else if (( LifePodTransItem[count].Ship == TransportTarget ) &&
								 ( LifePodTransItem[count].Occupant == 1 ))
						{
							TransportTarget->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 1);
							Ship->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 1);
						}
					}
				}				
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
			}
			break;

		case kNotifyTransportedItemFrom:
			{
				tShipInfo* TransportFrom = fMission.mFindShipID( fEventID );

				if (( TransportFrom->mGetTeam() == kLifePodTeam ) &&
					( fEventItem1 == kTransLifePodDude ))
				{
					LifePodTransported++;
					if ( LifePodTransported == 1 )
					{
						fMission.mDisplayMessage( kTransportSurvivor_msg, kMissionScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage ( fMission.mRandomInt32 (kTransportSurvivor_2_msg, kTransportSurvivor_4_msg), kMissionScriptMessage);
					}

					// determine which life pod is being transported from
					// and decrement the SurvivorsRemaining
					for ( int32 count = 0; count < kMaxLifePods; count++ )
					{
						if ( LifePodTransItem[count].Ship == TransportFrom )
						{
							LifePodTransItem[count].Occupant = 0;
							SurvivorsRemaining--;
						}
					}
					// if the SurvivorsRemaining == 0 then set the appropriate flags
					( SurvivorsRemaining == 0 ) ? AllLifePodsRescuedOrDestroyed = true : AllLifePodsRescuedOrDestroyed = false;

					// set the goal pointer to point at the next valid life pod
					PointToPod( *this );
					CheckWinCondition( *this );
				}
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
				Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ((Ship->mGetTeam() == kPsionicTeam) &&
						(HostileShip->mGetTeam() == kPlayerTeam) &&
						(AttackedEscort == 0))
					{
						fMission.mSetTeamRelations (kPlayerTeam, kPsionicTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						PsionicTeam->mGetFirstShip()->mClearCommands();
						AttackedEscort = 1;
					}
					else if ((Ship->mGetTeam() == kPsionicTeam) &&
							(HostileShip->mGetTeam() == kPlayerTeam) &&
							(AttackedEscort == 1))
					{
						fMission.mSetTeamRelations (kPlayerTeam, kPsionicTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						PsionicTeam->mGetFirstShip()->mClearCommands();
						AttackedEscort = 2;
					}
					else if (( Ship->mGetTeam() == kShiarTeam ) && ( HostileShip->mGetTeam() == kPlayerTeam ) &&
							 ( ShiarAttacked == false))
					{
						ShiarAttacked = true;
						fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kShiarTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					}
					else if (( Ship->mGetTeam() == kLifePodTeam ) && ( HostileShip->mGetTeam() == kShiarTeam ) &&
							 ( ShiarAttackedLifePod == false ))
					{
						ShiarAttackedLifePod = true;
					}
					else if (( Ship->mGetTeam() == kLifePodTeam ) && ( HostileShip->mGetTeam() == kPlayerTeam ) &&
							 ( PlayerAttackedLifePod == false ))
					{
						PlayerAttackedLifePod = true;
						fMission.mDisplayMessage( kPlayerAttackedLifePod_msg, kMissionScriptMessage );
					}
					else if ((Ship->mGetTeam() == kShiarTeam) || (Ship->mGetTeam() == kGreenTeam) ||
							(Ship->mGetTeam() == kBlueTeam))
					{
						PsionicTeam->mGetFirstShip()->mClearCommands();
						fMission.mSetTeamRelations (kPlayerTeam, kGreenTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kBlueTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPsionicTeam, kGreenTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPsionicTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPsionicTeam, kBlueTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					{
						// Set team relations
						SetAllTeamRelations(*this, kPlayerTeam);

						fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );

						PointToPod( *this );
					}
					break;

				case kTimerLimitSet_2:
					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_3: // COMM Button timer
					// Green
					for (Ship = GreenTeam->mGetFirstShip(); Ship != NULL; Ship = GreenTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
				case kTimerTurnsSet_4: // COMM Button timer
					// Shiar
					for (Ship = ShiarTeam->mGetFirstShip(); Ship != NULL; Ship = ShiarTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
				case kTimerTurnsSet_5: // COMM Button timer
					// Blue
					for (Ship = BlueTeam->mGetFirstShip(); Ship != NULL; Ship = BlueTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
				case kTimerTurnsSet_6: // COMM Button timer
					// Psionic
					for (Ship = PsionicTeam->mGetFirstShip(); Ship != NULL; Ship = PsionicTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
				case kTimerTurnsSet_7: // COMM Button timer
					break;
				case kTimerTurnsSet_8: // COMM Button timer
					break;
				case kTimerTurnsSet_9: // COMM Button timer
					break;
				case kTimerTurnsSet_10: // COMM Button timer
					// Lifepod
					for (Ship = LifePodTeam->mGetFirstShip(); Ship != NULL; Ship = LifePodTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyOnCapture:
			{
				tShipInfo* CapturingShip = fMission.mFindShipID( fEventID );

				fMission.mAddToBonusPrestige( kCaptureBonus );

				CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipCaptured, Ship, CapturingShip );
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam() == kPlayerTeam)
				{
					PlayerDestroyed++;
					if (PlayerTeam->mGetShipsInPlayCount() == 0)
					{
						return kPlayerDestroyed;
					}
				}
				else
				{
					CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDestroyed, Ship );
				}
			}
			break;

		case kNotifyDisengaged:
			{
				if ( Ship->mGetTeam() == kPlayerTeam )
				{
					if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						fMission.mSetGoalIndicator( kFalse, 0, 0 );
					}
				}
				else
				{
					CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDisengaged, Ship );
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (Ship->mGetTeam())
				{
					case kGreenTeam:
						GreenCommMessages(*this, fEventID, Ship);
						break;

					case kShiarTeam:
						ShiarCommMessages(*this, fEventID, Ship);
						break;

					case kBlueTeam:
						BlueCommMessages(*this, fEventID, Ship);
						break;

					case kPsionicTeam:
						PsionicCommMessages(*this, fEventID, Ship);
						break;

					case kLifePodTeam:
						LifePodCommMessages(*this, fEventID, Ship);
						break;
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF point values should be based on how many ships there are of a certain team
	VictoryPoints = (LifePodTransported * 15) - (LifePodDestroyed * 5) - (PlayerAttackedLifePod * 100) +
					(PsionicDestroyed * 20) - (GreenDestroyed * 20) - (BlueDestroyed * 20);

	if (Elite == kPraiex)
	{		
	}
	else
	{		
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 40))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 39) && (VictoryPoints >= 21))
	{
		MyVictoryLevel = kDefeat;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (VictoryLevel == kVictory)
		{
			fMission.mAddToDebriefingList (kPraiexVictory_msg);
			fMission.mAddToDebriefingList (kVictory2_msg);
		}
		else if (VictoryLevel == kAstoundingVictory)
		{
			fMission.mAddToDebriefingList (kPraiexAstoundingVictory_msg);
			fMission.mAddToDebriefingList (kVictory2_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( OrderAchieved == true ) ? fMission.mAddToDebriefingList( kOrderAchievedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( OrderAchieved == false ) ? fMission.mAddToDebriefingList( kOrderAchievedFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// Set all of the team relations
void SetAllTeamRelations(tScript& tScript, eTeamID kPlayerTeam)
{

	tScript.fMission.mSetTeamRelations (kPlayerTeam, kGreenTeam, kTeamHostile, kGoalDestroy, 1.0 );// ***
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kBlueTeam, kTeamHostile, kGoalDestroy, 1.0 );//***
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kPsionicTeam, kTeamHostile, kGoalDestroy, 1.0 );// ***
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kGreenTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );// ***
	tScript.fMission.mSetTeamRelations (kGreenTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kBlueTeam, kTeamNeutral, kGoalDestroy, 1.0 );//***
	tScript.fMission.mSetTeamRelations (kGreenTeam, kPsionicTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kLifePodTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kShiarTeam, kBlueTeam, kTeamNeutral, kGoalIgnore, 1.0 );// ***
	tScript.fMission.mSetTeamRelations (kShiarTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShiarTeam, kLifePodTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kBlueTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );//***
	tScript.fMission.mSetTeamRelations (kBlueTeam, kGreenTeam, kTeamAtWar, kGoalDestroy, 1.0 );//***
	tScript.fMission.mSetTeamRelations (kBlueTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0 );// ***
	tScript.fMission.mSetTeamRelations (kBlueTeam, kPsionicTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBlueTeam, kLifePodTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBlueTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kPsionicTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );// ***
	tScript.fMission.mSetTeamRelations (kPsionicTeam, kLifePodTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPsionicTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kLifePodTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// Display comm messages for GREEN team
void GreenCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (FiredOnLifePods == 0)
			{
				if (GreenBetrayed == 0)
				{
					if (GreenPacified == 0)
					{
						tScript.fMission.mDisplayMessage(kGreenHail1_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kGreenPacified_msg, kMissionScriptMessage);
					}
				}
				else
				{
					tScript.fMission.mDisplayMessage(kGreenBetrayed_msg, kMissionScriptMessage);
				}
			}
			else
			{
				tScript.fMission.mDisplayMessage(kGreenMurderer_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_3, 2);
			break;
			
		case kButtonSurrender_text:
			if ( FiredOnLifePods == 0 )
			{
				if (GreenBetrayed == 0)
				{
					if (GreenPacified == 0)
					{
						tScript.fMission.mDisplayMessage(kGreenSurrender_1_msg, kMissionScriptMessage);
						tScript.fMission.mDisplayMessage(kGreenSurrender_2_msg, kMissionScriptMessage);
						tScript.fMission.mDisplayMessage(kGreenSurrender_3_msg, kMissionScriptMessage);

						GreenPacified = 1;
						tScript.fMission.mSetTeamRelations (kGreenTeam, tScript.kPlayerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
						tScript.fMission.mSetTeamRelations (kGreenTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
						tScript.fMission.mSetTeamRelations (kGreenTeam, kBlueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
						tScript.fMission.mSetTeamRelations (kGreenTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						tScript.fMission.mSetTeamRelations (kGreenTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
						tScript.fMission.mSetTeamRelations (kGreenTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						CheckWinCondition( tScript );
					}
					else
					{
						tScript.fMission.mDisplayMessage(kGreenPacified_msg, kMissionScriptMessage);
					}
				}
				else // GreenBetrayed == 1
				{
					tScript.fMission.mDisplayMessage(kGreenBetrayed_msg, kMissionScriptMessage);
				}
			}
			else // FiredOnLifePods == 1
			{
				tScript.fMission.mDisplayMessage(kGreenMurderer_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_3, 2);
			break;

		case kButtonTaunt_text:
			if ( FiredOnLifePods == 0 )
			{
				tScript.fMission.mDisplayMessage(kTauntNoResponse_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kGreenMurderer_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_3, 2);
			break;
	}
}
	
//---------------------------------------------------------------------------------------------
// Display comm messages for Shiar team
void ShiarCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if ( Elite == kPraiex )
			{
				if ( ShiarAttacked == false )
				{
					tScript.fMission.mDisplayMessage(kShiarHailPraiexShiar_1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kShiarHailPraiex_2_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kShiarHailPraiexShiar_3_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kShiarHailPraiexShiar_4_msg, kMissionScriptMessage);
				}
				else if (( ShiarAttacked == true ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarTauntNotEnemy_msg, kMissionScriptMessage );
				}
				else if (( ShiarPacified == 1 ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarPacified_msg, kMissionScriptMessage );
				}
				else if ( ShiarBetrayed == 1 )
				{
					tScript.fMission.mDisplayMessage( kShiarBetrayed_msg, kMissionScriptMessage );
				}
			}
			else // Elite == kShiar
			{
				if ( ShiarAttacked == false )
				{
					tScript.fMission.mDisplayMessage(kShiarHailShiar_1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kShiarHailPraiexShiar_3_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kShiarHailPraiexShiar_4_msg, kMissionScriptMessage);
				}
				else if (( ShiarAttacked == true ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarTauntNotEnemy_msg, kMissionScriptMessage );
				}
				else if (( ShiarPacified == 1 ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarPacified_msg, kMissionScriptMessage );
				}
				else if ( ShiarBetrayed == 1 )
				{
					tScript.fMission.mDisplayMessage( kShiarBetrayed_msg, kMissionScriptMessage );
				}
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_4, 2);
			break;
			
		case kButtonSurrender_text:
			if ( Elite == kPraiex )
			{
				if ( ShiarPacified == 1 )
				{
					tScript.fMission.mDisplayMessage( kShiarPacifiedSurrendered_msg, kMissionScriptMessage );
				}
				else if ( ShiarAttacked == false )
				{
					tScript.fMission.mDisplayMessage( kShiarTauntNotEnemy_msg, kMissionScriptMessage );
				}
				else if ( ShiarBetrayed == 1 )
				{
					tScript.fMission.mDisplayMessage( kShiarBetrayed_msg, kMissionScriptMessage );
				}
				else if ( Ship->mGetDamagePercent() > 25.0 )
				{
					// no surrender for Praiex
					tScript.fMission.mDisplayMessage( kShiarSurrender1_msg, kMissionScriptMessage );
				}
			}
			else // Elite == kShiar
			{
				tScript.fMission.mDisplayMessage( kShiarSurrenderShiar_1_msg, kMissionScriptMessage );
				tScript.fMission.mDisplayMessage( kShiarSurrenderShiar_2_msg, kMissionScriptMessage );

				ShiarPacified = 1;
				tScript.fMission.mSetTeamRelations (kShiarTeam, tScript.kPlayerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
				tScript.fMission.mSetTeamRelations (kShiarTeam, kGreenTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
				tScript.fMission.mSetTeamRelations (kShiarTeam, kBlueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
				tScript.fMission.mSetTeamRelations (kShiarTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (kShiarTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_4, 2);
			break;

		case kButtonTaunt_text:
			if ( Elite == kPraiex )
			{
				if (( ShiarAttacked == true ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarTauntNotEnemy_msg, kMissionScriptMessage );
				}
				else if (( ShiarPacified == 1 ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarPacified_msg, kMissionScriptMessage );
				}
				else if ( ShiarBetrayed == 1 )
				{
					tScript.fMission.mDisplayMessage( kShiarBetrayed_msg, kMissionScriptMessage );
				}
				else
				{
					tScript.fMission.mDisplayMessage( kShiarNoResponse_msg, kMissionScriptMessage );
				}
			}
			else // Elite == kShiar
			{
				if (( ShiarAttacked == true ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarTauntNotEnemy_msg, kMissionScriptMessage );
				}
				else if (( ShiarPacified == 1 ) && ( ShiarBetrayed == 0 ))
				{
					tScript.fMission.mDisplayMessage( kShiarPacified_msg, kMissionScriptMessage );
				}
				else if ( ShiarBetrayed == 1 )
				{
					tScript.fMission.mDisplayMessage( kShiarBetrayed_msg, kMissionScriptMessage );
				}
				else
				{
					tScript.fMission.mDisplayMessage( kShiarNoResponse_msg, kMissionScriptMessage );
					//tScript.fMission.mDisplayMessage (fMission.mRandomInt32 (kShiarNoResponseEnemy_1_msg, kShiarNoResponseEnemy_3_msg), kMissionScriptMessage);
				}
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_4, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for BLUE team
void BlueCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (BlueBetrayed == 0)
			{
				if (BluePacified == 0)
				{
					tScript.fMission.mDisplayMessage(kBlueHail1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kBlueHail2_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kBlueHail3_msg, kMissionScriptMessage);

					BluePacified = 1;
					tScript.fMission.mSetTeamRelations (kBlueTeam, tScript.kPlayerTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					tScript.fMission.mSetTeamRelations (kBlueTeam, kGreenTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					tScript.fMission.mSetTeamRelations (kBlueTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					tScript.fMission.mSetTeamRelations (kBlueTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kBlueTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kBlueTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					CheckWinCondition( tScript );
				}
				else
				{
					tScript.fMission.mDisplayMessage(kBluePacified_msg, kMissionScriptMessage);
				}
			}
			else
			{
				tScript.fMission.mDisplayMessage(kBlueBetrayed_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_5, 2);
			break;
			
		case kButtonSurrender_text:
			if (BlueBetrayed == 0)
			{
				if (BluePacified == 0)
				{
					tScript.fMission.mDisplayMessage(kBlueHail1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kBlueHail2_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kBlueHail3_msg, kMissionScriptMessage);
					BluePacified = 1;
				}
				else
				{
					tScript.fMission.mDisplayMessage(kBluePacified_msg, kMissionScriptMessage);
				}
			}
			else
			{
				tScript.fMission.mDisplayMessage(kBlueBetrayed_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_5, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage(kTauntNoResponse_msg, kMissionScriptMessage);
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_5, 2);
			break;
	}
}
//---------------------------------------------------------------------------------------------
// Display comm messages for Psionic team
void PsionicCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (PsionicEnemy == 0)
			{
				if (PsionicPacified == 0)
				{
					tScript.fMission.mDisplayMessage(kPsionicHail1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kPsionicHail2_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kPsionicHail3_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kPsionicPacified_msg, kMissionScriptMessage);
				}
			}
			else
			{
				tScript.fMission.mDisplayMessage(kPsionicBetrayed_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
			break;
			
		case kButtonSurrender_text:
			if (Ship->mGetDamagePercent() < 50.0)
			{
				tScript.fMission.mDisplayMessage(kPsionicSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kPsionicSurrenderDamaged_msg, kMissionScriptMessage);
				PsionicPacified = 1;
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
			break;

		case kButtonTaunt_text:
			if (PsionicEnemy == 0)
			{
				if (PsionicPacified == 0)
				{
					tScript.fMission.mDisplayMessage(kPsionicPacified_msg, kMissionScriptMessage);
				}
				else
				{
					tScript.fMission.mDisplayMessage(kPsionicTaunt_msg, kMissionScriptMessage);
				}
			}
			else
			{
				tScript.fMission.mDisplayMessage(kPsionicBetrayed_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
			break;
	}
}
//---------------------------------------------------------------------------------------------
// Display comm messages for LIFE POD team
void LifePodCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
			break;
			
		case kButtonSurrender_text:
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
			break;
		case kButtonTaunt_text:
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
			break;
	}
}


//---------------------------------------------------------------------------------------------
// Set all of the team relations at the beginning
void SetAllTeamRelationsBeginning(tScript& tScript, eTeamID kPlayerTeam)
{

	tScript.fMission.mSetTeamRelations (kPlayerTeam, kGreenTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kBlueTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kPsionicTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kGreenTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kBlueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kGreenTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kShiarTeam, kBlueTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShiarTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShiarTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kBlueTeam, kPsionicTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBlueTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kBlueTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kPsionicTeam, kLifePodTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (kPsionicTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	tScript.fMission.mSetTeamRelations (kLifePodTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	if (( AllLifePodsRescuedOrDestroyed == true ) &&
		( AllPsionicShipsNeutralized == true ) &&
		(( AllBlueShipsNeutralized == true ) || ( BluePacified == 1 )) &&
		(( AllGreenShipsNeutralized == true ) || ( GreenPacified == 1)))
	{
		OrderAchieved = true;
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
	}
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* GreenTeam = tScript.fMission.mGetTeamHandle (kGreenTeam);
	tTeamInfo* ShiarTeam = tScript.fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* BlueTeam = tScript.fMission.mGetTeamHandle (kBlueTeam);
	tTeamInfo* PsionicTeam = tScript.fMission.mGetTeamHandle (kPsionicTeam);
	tTeamInfo* LifePodTeam = tScript.fMission.mGetTeamHandle (kLifePodTeam);

	// if all ships on a team are gone
	//if ( EventTeam->mGetShipsInPlayCount() == ((EventShipStatus == kEventShipCaptured) ? 1 : 0) )
	if (EventTeam == GreenTeam)
	{
		GreenDestroyed++;
		if ( EventTeam->mGetShipsInPlayCount() == 0 )
		{
			AllGreenShipsNeutralized = true;
			CheckWinCondition( tScript );
		}
	}
	else if (EventTeam == ShiarTeam)
	{
		ShiarDestroyed++;
		if ( EventTeam->mGetShipsInPlayCount() == 0 )
		{
			// LNF what about Shiar ships?
			CheckWinCondition( tScript );
		}
	}
	else if (EventTeam == BlueTeam)
	{
		BlueDestroyed++;
		if ( EventTeam->mGetShipsInPlayCount() == 0 )
		{
			AllBlueShipsNeutralized = true;
			CheckWinCondition( tScript );
		}
	}
	else if (EventTeam == PsionicTeam)
	{
		PsionicDestroyed++;
		if ( EventTeam->mGetShipsInPlayCount() == 0 )
		{
			AllPsionicShipsNeutralized = true;
			CheckWinCondition( tScript );
		}
	}
	else if (EventTeam == LifePodTeam)
	{
		LifePodDestroyed++;

		// if the life pod destroyed still has its occupant
		// subtract one from the SurvivorsRemaining count
		for ( int32 count = 0; count < kMaxLifePods; count++ )
		{
			if ( LifePodTransItem[count].Ship == Ship )
			{
				if ( LifePodTransItem[count].Occupant == 1 )
				{
					SurvivorsRemaining--;
				}
			}
		}
		// if the SurvivorsRemaining == 0 then set the appropriate flags
		( SurvivorsRemaining == 0 ) ? AllLifePodsRescuedOrDestroyed = true : AllLifePodsRescuedOrDestroyed = false;

		// point the goal pointer at the next life pod
		PointToPod( tScript, Ship );

		if ( EventTeam->mGetShipsInPlayCount() == 0 )
		{
			AllLifePodsRescuedOrDestroyed = true;
			CheckWinCondition( tScript );
		}
	}
}

//---------------------------------------------------------------------------------------------
// set the goal pointer to point at the nearest life pod with an occupant
void PointToPod( tScript& tScript, tShipInfo* DestroyedPod )
{
	float LifePodDistance = 9999.9f;

	tShipInfo* ClosestLifePod = NULL;
	for ( int32 count = 0; count < kMaxLifePods; count++ )
	{
		if ( LifePodTransItem[count].Ship != NULL )
		{
			if (( tScript.fPlayerShip->mGetTacticalDistance( LifePodTransItem[count].Ship ) < LifePodDistance ) &&
				( LifePodTransItem[count].Occupant != 0 ) && ( LifePodTransItem[count].Ship != DestroyedPod ) &&
				( LifePodTransItem[count].Ship->mInPlay() == kTrue ))
			{
				LifePodDistance = tScript.fPlayerShip->mGetTacticalDistance( LifePodTransItem[count].Ship );
				ClosestLifePod = LifePodTransItem[count].Ship;
			}
		}
	}

	if ( ClosestLifePod != NULL )
	{
		tScript.fMission.mSetGoalIndicator( ClosestLifePod );	
	}
	else
	{
		if ( LifePodDestroyed == 6 )
		{
			tScript.fMission.mDisplayMessage( kAllLifePodsDestroyed_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kAllLifePodsDealtWith_msg, kMissionScriptMessage );
		}
		tScript.fMission.mSetGoalIndicator( kFalse, 0, 0 );
	}
}
