#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_8MissionText.h"
#include "Rom_8MissionMaps.h"
/***********************************
LNF I'll probably have to make the game a little easier
LNF for the computer Shiar
***********************************/

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 PackageDelivered = 0; // win condition BOTH
int32 MessageReceived = 0; // win condition SHIAR
int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 FedDestroyed = 0;
int32 FactionDestroyed = 0; // win condition PRAIEX?
int32 PlatformDestroyed = 0;
int32 FlagshipDestroyed = 0; // win condition PRAIEX? same as FactionDestroyed?
int32 AtWarWithFed = 0;
int32 AtWarWithShiar = 0;

bool FoundShiar = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

eVictoryLevels MyVicLevel = kDevastatingDefeat;

tShipInfo* DestroyThisShip = NULL;

void CheckWinCondition( tScript& tScript );

const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kVulcanX = 16;
const int32 kVulcanY = 16;

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar
};
//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kFedTeam, 
	kFactionTeam,
	kPlatformTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kFedMission | kKlingMission | kGornMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kFederation;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kFedTeam, kFederation);
	fMission.mSetTeamRace (kFactionTeam, kRomulan);
	fMission.mSetTeamRace (kPlatformTeam, kFederation);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFactionTeam, kPlatformTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlatformTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
		// return a map depending on the player's faction
		return 1;
	}
	else
	{
		Elite = kShiar;
		// return a map depending on the player's faction
		return 2;
	}
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

/***
	LNF pick a random number of ships depending on how many the player has
	check the class of player's ships
	have some kind of table that determines balance for ship classes vs number
	< Light Cruiser = 1 ; Heavy Cruiser = 2 ; > Heavy Cruiser = 3
	choose whether to have fewer powerful ships or many smaller ships
***/
	// create Fed Defender ships
	FedTeam->mCreateFleet (2, 2 * AveragePlayerBPV, kClassDestroyer, kClassHeavyCruiser, 50, kStartPosition_W);

	// create Faction
	FactionTeam->mCreateShip (AveragePlayerBPV, kClassHeavyCruiser, kClassHeavyCruiser, 50, kStartPosition_J);
	FactionTeam->mCreateShip (AveragePlayerBPV, kClassDestroyer, kClassLightCruiser, 50, kStartPosition_K);

	// create Platform team
	PlatformTeam->mCreateShip ("F-DEF", kStartPosition_T);
	PlatformTeam->mCreateShip ("F-DEF", kStartPosition_U);
	PlatformTeam->mCreateShip ("F-DEF", kStartPosition_V);

	// create Planet(s)
	PlanetTeam->mCreateShip( "PLPh1", kStartPosition_Specified, kVulcanX, kVulcanY );
	//PlanetTeam->mCreateShip( "PL0", kStartPosition_Specified, 23, 11 );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if ( Elite == kPraiex )
		{
			// track distance to detect the AI Shiar ships
			Ship->mTrackTacticalDistance( 50.0, kVulcanX, kVulcanY );
		}
		else
		{
			// track distance for MessageReceived flag
			Ship->mTrackTacticalDistance( 35.0, kVulcanX, kVulcanY );
		}
	}
	if ( Elite == kShiar )
	{
		// LNF change from kTransDeathPlague
		fPlayerShip->mAccessSupplies (kSetTransporterItemCount, kTransDeathPlague, 1);
		fPlayerShip->mSetShipCallbackEvents( kNotifyTransportedItemTo );
	}

	// configure Fed Defender ships
	for (Ship = FedTeam->mGetFirstShip(); Ship != NULL; Ship = FedTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}
	// if the player is Praiex, have the Fed defend the planet, if not, just attack the player
	if ( Elite == kPraiex )
	{
		// Fed team defend Vulcan
		FedTeam->mDefendEntity(PlanetTeam->mGetFirstShip(), 80.0, 80.0, kVeryHighPriority);
	}

	// configure Faction
	for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}
	// give the special item to the flagship of the opposite team
	if ( Elite == kPraiex )
	{
		// LNF change kTransDeathPlague to the appropriate item
		FactionTeam->mGetFirstShip()->mAccessSupplies (kSetTransporterItemCount, kTransDeathPlague, 1);
		// make the ai Shiar team head toward the planet
		FactionTeam->mGetFirstShip()->mSetFinalDestination( kVulcanX, kVulcanY );
		// check when the ai ship is within 5.0 of the planet (transporter range)
		// do a timer thing and check if the ship is < 10? then slow it down? so it
		// doesn't run into the planet
		fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
		// other ship(s) on the Faction team defend the leader
		FactionTeam->mGetThisShip(2)->mDefendEntity( FactionTeam->mGetFirstShip(), 20.0, 35.0, kVeryHighPriority );
	}

	// configure Platform team
	for (Ship = PlatformTeam->mGetFirstShip(); Ship != NULL; Ship = PlatformTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure Planet
	PlanetTeam->mGetFirstShip()->mChangeName( kVulcan_name );


	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 2);

	// timer to check if the player(Praiex) is close to Shiar
	if ( Elite == kPraiex )
	{
		fMission.mSetTimedInterval( kTimerTurnsSet, 3 );
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* FedTeam = fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* PlatformTeam = fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID (fEventID);

				if ( TransportTarget != NULL )
				{
					// if the PLAYER is transporting onto VULCAN
					if (( TransportTarget == PlanetTeam->mGetFirstShip() ) &&
						( Ship->mGetTeam() == kPlayerTeam ))
					{
						if ( fEventItem1 == kTransDeathPlague )
						{
							// success!! this is what the player is supposed to do
							fMission.mDisplayMessage( kEvent7TransportPlague_msg, kScienceScriptMessage );
							PackageDelivered = 1;
							CheckWinCondition( *this );
						}
						else
						{
							// Vulcan doesn't want that item
							fMission.mDisplayMessage( kVulcanDoesntWantThat_msg, kScienceScriptMessage );
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
						}
					}
					// else if the player is transporting to anyone but Vulcan
					else if ( Ship->mGetTeam() == kPlayerTeam )
					{
						// the target is NOT the player's team and the item is the plague
						if (( TransportTarget->mGetTeam() != kPlayerTeam ) && ( fEventItem1 == kTransDeathPlague ))
						{
							DestroyThisShip = TransportTarget;
							fMission.mSetTimedInterval( kTimerLimitSet_3, 4 );
							fMission.mDisplayMessage( kPlagueDoesntBelong_msg, kScienceScriptMessage );
						}
						else
						{
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
						}
					}
				}
			}
			break;

		case kNotifyDamage:
			break;

		case kNotifyAtDistance:
			{
				if ( Elite == kPraiex )
				{
					// LNF put tracking distance thing in here
					if ( Ship->mGetTeam() == kPlayerTeam )
					{
						if ( FoundShiar == false )
						{
							// tell the player where Shiar is and point goal pointer
							fMission.mDisplayMessage( kFoundShiar_msg, kScienceScriptMessage );
							fMission.mSetGoalIndicator( FactionTeam->mGetThisShip(2) );
							// turn off goal pointer < 90.0
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							FoundShiar = true;
						}
					}
				}
				else // Elite == kShiar
				{
					fMission.mDisplayMessage( kPraiexHail_1_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kPraiexHail_2_msg, kMissionScriptMessage );
					fMission.mDisplayMessage( kPraiexHail_3_msg, kMissionScriptMessage );
					MessageReceived = 1;
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kFedTeam, kTeamHostile, kGoalDestroy, 1.0);// ***
					if ( Elite == kPraiex )
					{
						fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kFedTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0);// ***
						fMission.mSetTeamRelations (kFactionTeam, kFedTeam, kTeamHostile, kGoalDestroy, 1.0);// ***
					}
					else
					{
						fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kFactionTeam, kFedTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					}
					fMission.mSetTeamRelations (kPlayerTeam, kPlatformTeam, kTeamHostile, kGoalDestroy, 1.0);// ***
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kFedTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);// ***
					//fMission.mSetTeamRelations (kFedTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0);// ***
					fMission.mSetTeamRelations (kFedTeam, kPlatformTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kFedTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					//fMission.mSetTeamRelations (kFactionTeam, kFedTeam, kTeamHostile, kGoalDestroy, 1.0);// ***
					fMission.mSetTeamRelations (kFactionTeam, kPlatformTeam, kTeamHostile, kGoalDestroy, 1.0);// ***
					fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamHostile, kGoalIgnore, 1.0);// ***

					fMission.mSetTeamRelations (kPlatformTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);// ***
					fMission.mSetTeamRelations (kPlatformTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlatformTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0);// ***

					fMission.mSetTeamRelations (kPlanetTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0);// ***

					if ( Elite == kPraiex )
					{
						fMission.mDisplayMessage( kStartScenarioPraiex_msg, kMissionScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kStartScenarioShiar_msg, kMissionScriptMessage );
					}
					break;

				case kTimerLimitSet_2:
					// checks to see if the Shiar ship is < 90.0 then turns off goal pointer
					if ( fPlayerShip->mGetTacticalDistance(FactionTeam->mGetThisShip(2)) <= 90.0 )
					{
						fMission.mSetGoalIndicator( kFalse, 0, 0 );
					}
					else
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					break;

				case kTimerLimitSet_3:
					// after player transports plague anywhere but planet or player's team
					fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
					fMission.mDisplayMessage( kPlagueShipUnstable_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_4:
					{
						// checks to see if Faction ship is near Vulcan

						for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
						{
							if (( Ship->mGetTacticalDistance( PlanetTeam->mGetFirstShip()) <= 10.0 ) &&
								( PackageDelivered == 0 ) && ( Ship->mInPlay() == kTrue ))
							{
								Ship->mSetShipMaxSpeed( 5 );
								if ( Ship->mGetTacticalDistance( PlanetTeam->mGetFirstShip()) < 5.0 )
								{
									fMission.mDisplayMessage( kFactionShiarTransportObject_msg, kScienceScriptMessage );
									// LNF remove item from shiar ship.  change from kTransDeathPlague
									FactionTeam->mGetFirstShip()->mAccessSupplies (kSetTransporterItemCount, kTransDeathPlague, 0);
									PackageDelivered = 1;
									// LNF check win condition here?
									break;
								}
							}
						}
						fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
					}
					break;

				case kTimerTurnsSet:
					// checks if player(Praiex) is close to Shiar
					if ( FoundShiar == false )
					{
						// tell the player where Shiar is and point goal pointer
						fMission.mDisplayMessage( kFoundShiar_msg, kScienceScriptMessage );
						fMission.mSetGoalIndicator( FactionTeam->mGetThisShip(2) );
						// turn off goal pointer < 90.0
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						FoundShiar = true;
					}
					break;

				case kTimerTurnsSet_2:
					// blows up ship/entity that the plague was transported to
					DestroyThisShip->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
					DestroyThisShip->mDamageSystem( kDamageExcessDamageSystem, 100 );
					CheckWinCondition( *this );

					break;

				case kTimerTurnsSet_9: // Fed COMM Button timer
					for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10: // Faction COMM Button timer
					for (Ship = FedTeam->mGetFirstShip(); Ship != NULL; Ship = FedTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyOnCapture:
			{
				tShipInfo* CapturingShip = fMission.mFindShipID( fEventID );

				fMission.mAddToBonusPrestige( kCaptureBonus );

				CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipCaptured, Ship, CapturingShip );
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else
			{
				CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDestroyed, Ship );
			}
			break;

		case kNotifyDisengaged:
			{
				if ( Ship->mGetTeam() == kPlayerTeam )
				{
					if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						fMission.mSetGoalIndicator( kFalse, 0, 0 );
					}
				}
				else
				{
					CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDestroyed, Ship );
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								if ((Elite == kPraiex) && (AtWarWithFed == 0))
								{
									fMission.mDisplayMessage(kFedHailPraiexNic1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFedHailPraiexNic2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFedHailPraiexNic3_msg, kMissionScriptMessage);
								}
								else if ((Elite == kShiar) && (AtWarWithFed == 0))
								{
									fMission.mDisplayMessage(kFedHailShiarNic1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFedHailShiarNic2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFedHailShiarNic3_msg, kMissionScriptMessage);
								}
								else if (AtWarWithFed == 1)
								{
									fMission.mDisplayMessage(kFedHailCombat1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kFedHailCombat2_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFactionTeam:
								if (Elite == kPraiex)
								{
									if (AtWarWithShiar == 0)
									{
										fMission.mDisplayMessage(kShiarHailNic_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
									}
								}
								else
								{
									if (PackageDelivered == 0)
									{
										fMission.mDisplayMessage(kPraiexHail_msg, kMissionScriptMessage);
										MessageReceived = 1;
									}
									else
									{
										fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								fMission.mDisplayMessage(kFedSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFactionTeam:
								if (Elite == kPraiex)
								{
									if (AtWarWithShiar == 0)
									{
										fMission.mDisplayMessage(kShiarHailNic_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
									}
								}
								else
								{
									if (PackageDelivered == 0)
									{
										fMission.mDisplayMessage(kPraiexHail_msg, kMissionScriptMessage);
										MessageReceived = 1;
									}
									else
									{
										fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kFedTeam:
								fMission.mDisplayMessage(kFedTaunt_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kFactionTeam:
								if (Elite == kPraiex)
								{
									if (AtWarWithShiar == 0)
									{
										fMission.mDisplayMessage(kShiarHailNic_msg, kMissionScriptMessage);
									}
									else
									{
										fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
									}
								}
								else
								{
									if (PackageDelivered == 0)
									{
										fMission.mDisplayMessage(kPraiexHail_msg, kMissionScriptMessage);
										MessageReceived = 1;
									}
									else
									{
										fMission.mDisplayMessage(kNoResponse_msg, kMissionScriptMessage);
									}
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if (Elite == kPraiex)
	{
		// LNF somehow I have to assume that the package gets delivered if the
		// LNF shiar ship and the planet are still in the game when the player leaves.
		if (( PackageDelivered == 0 ) && ( FlagshipDestroyed == 1 ))
		{
			VictoryPoints = 100;
		}
		else
		{
			VictoryPoints = 0;
		}
	}
	else
	{
		if (( PackageDelivered == 1 ) || 
			(( PackageDelivered == 0 ) && ( MessageReceived == 1 )))
		{
			VictoryPoints = 100;
		}
		else
		{
			VictoryPoints = 0;
		}
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	MyVicLevel = VictoryLevel;
	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
		}
		else
		{
			if (PackageDelivered == 0)
			{
				fMission.mAddToDebriefingList (kShiarVictoryNotDelivered_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (kShiarVictoryDelivered_msg);
			}
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( PackageDelivered == 0 ) ? fMission.mAddToDebriefingList( kPackageDeliveredFalse_msg ) : SuccessCount++;
		( FlagshipDestroyed != 0 ) ? fMission.mAddToDebriefingList( kFlagshipDestroyedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( PackageDelivered != 0 ) ? fMission.mAddToDebriefingList( kPackageDeliveredTrue_msg ) : FailureCount++;
		( FlagshipDestroyed == 0 ) ? fMission.mAddToDebriefingList( kFlagshipDestroyedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		( PackageDelivered != 0 ) ? fMission.mAddToDebriefingList( kPackageDeliveredTrue_msg ) : SuccessCount++;
		( MessageReceived != 0 ) ? fMission.mAddToDebriefingList( kMessageReceivedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( PackageDelivered == 0 ) ? fMission.mAddToDebriefingList( kPackageDeliveredFalse_msg ) : FailureCount++;
		( MessageReceived == 0 ) ? fMission.mAddToDebriefingList( kMessageReceivedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_4;
	}
	else
	{
		return 0;
	}
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	if ( Elite == kPraiex )
	{
		if (( PackageDelivered == 0 ) && ( FlagshipDestroyed == 1 ))
		{
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
		}
	}
	else // Elite == kShiar
	{
		if  (( PackageDelivered == 1 ) ||
			(( PackageDelivered == 0 ) && ( MessageReceived == 1 )))
		{
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
		}
	}
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* FedTeam = tScript.fMission.mGetTeamHandle (kFedTeam);
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* PlatformTeam = tScript.fMission.mGetTeamHandle (kPlatformTeam);
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam);

	// if all ships on a team are gone
	//if ( EventTeam->mGetShipsInPlayCount() == ((EventShipStatus == kEventShipCaptured) ? 1 : 0) )
	if (EventTeam == FedTeam)
	{
		FedDestroyed++;
	}
	else if (EventTeam == FactionTeam)
	{
		FactionDestroyed++;
		if ( Ship == FactionTeam->mGetFirstShip() )
		{
			FlagshipDestroyed = 1;
		}
	}
	else if (EventTeam == PlatformTeam)
	{
		PlatformDestroyed++;
	}

	CheckWinCondition( tScript );
}
