#include "scriptinterface.h"
#include "FindClosestMapEdge.h"

#include "Rom_9MissionText.h"
#include "Rom_9MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

const float kDamageThreshold = 40.0;
const float kDistanceToRomulus = 25.0;
const int32 kRomulusX = 27;
const int32 kRomulusY = 11;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;
const int32 kCountdownToRelease = 16;// determines how many tursn until the game ends

int32 Elite = 0;
int32 Loyalty = 0; // win condition...the side the player is on
int32 PlayerDestroyed = 0;
int32 TuviusDestroyed = 0;
int32 FactionDestroyed = 0;
int32 ShiarDestroyed = 0;
int32 PraiexDestroyed = 0;
int32 PlanetDefenseDestroyed = 0;
int32 PlanetDestroyed = 0;
int32 SwitchedSides = 0;
int32 WinningSide = 0; // the side that wins...coupled with Loyalty

bool AllTuviusTeamDestroyed = false;
bool AllFactionTeamDestroyed = false;
bool AllShiarTeamDestroyed = false;
bool AllPraiexTeamDestroyed = false;
bool AllPlanetDefenseTeamDestroyed = false;
bool DefeatedTuvius = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void SetAllTeamRelations(tScript& tScript, eTeamID kPlayerTeam);
void SetAllTeamRelationsStartMission(tScript& tScript, eTeamID kPlayerTeam);
void setPatrolRoute(tShipInfo* Ship, tScript& tScript);
void activateTuvius( tScript& tScript );
void setTuviusStartingPatrol(tShipInfo* Ship, tScript& tScript);

enum eAffiliationEnum
{
	kPraiex = 1,
	kShiar,
	kAnarchy
};

int32 TeamAffiliation( tScript& tScript, tShipInfo* Ship );

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

eGameStatus CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

void TuviusCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTeamID kPlayerTeam);
void PraiexCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void ShiarCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
// LNF how about Faction comm messages?
void PlanetCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void SetSwitchToShiarTeamRelations( tScript& tScript ); // make player loyal to Shiar
void SetSwitchToPraiexTeamRelations( tScript& tScript ); // make player loyal to Praiex
void SetAnarchyTeamRelations( tScript& tScript ); // make player Anarchy
eGameStatus CheckWinCondition( tScript& tScript );

//************************
int32 Ultimatum = 0;
int32 Betrayal = 0;

int32 TimesUp = 0;


//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kTuviusTeam, 
	kFactionTeam,
	kShiarTeam,
	kPraiexTeam,
	kPlanetDefenseTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kRomulan;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kTuviusTeam, kRomulan);
	fMission.mSetTeamRace (kFactionTeam, kRomulan);
	fMission.mSetTeamRace (kShiarTeam, kRomulan);
	fMission.mSetTeamRace (kPraiexTeam, kRomulan);
	fMission.mSetTeamRace (kPlanetDefenseTeam, kRomulan);
	fMission.mSetTeamRace (kPlanetTeam, kRomulan);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
		Loyalty = kPraiex;
		// Set team relations to show up on Tactical map
		SetAllTeamRelations(*this, kPlayerTeam);

		// return a map depending on the player's faction
		return 1;
	}
	else
	{
		Elite = kShiar;
		Loyalty = kShiar;
		// Set team relations to show up on Tactical map
		SetAllTeamRelations(*this, kPlayerTeam);

		// return a map depending on the player's faction
		return 2;
	}
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TuviusTeam = fMission.mGetTeamHandle (kTuviusTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* ShiarTeam = fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* PraiexTeam = fMission.mGetTeamHandle (kPraiexTeam);
	tTeamInfo* PlanetDefenseTeam = fMission.mGetTeamHandle (kPlanetDefenseTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// LNF
	// LNF either change the ship type, reduce number of ships, or give more ships to the player.
	// LNF

	// create Tuvius
	//TuviusTeam->mCreateShip ("R-PRA", kStartPosition_Specified, 6, 8); // this is Tuvius' ship
	//TuviusTeam->mCreateShip ("R-KCN", kStartPosition_Specified, 5, 9);

	// LNF try a new tuvius ship
	TuviusTeam->mCreateShip( kMaxBPV, kClassDreadnought, kClassBattleship, 100.0, kStartPosition_Specified, 6, 8 );

	// create the Faction team - this is the player's counterpart team
	FactionTeam->mCreateFleet (NumberOfPlayersShips, TotalBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_M); // N and O

	// create Shiar
	ShiarTeam->mCreateFleet (2, 2 * AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_W); // X and Y

	// create Praiex
	PraiexTeam->mCreateFleet (2, 2 * AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_T); // U and V

	// create PlanetDefense
	PlanetDefenseTeam->mCreateShip ("R-DEF", kStartPosition_Q);
	PlanetDefenseTeam->mCreateShip ("R-DEF", kStartPosition_R);
	PlanetDefenseTeam->mCreateShip ("R-DEF", kStartPosition_S);

	// create Planet
	PlanetTeam->mCreateShip ("PLPh1", kStartPosition_Specified, kRomulusX, kRomulusY);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TuviusTeam = fMission.mGetTeamHandle (kTuviusTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* ShiarTeam = fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* PraiexTeam = fMission.mGetTeamHandle (kPraiexTeam);
	tTeamInfo* PlanetDefenseTeam = fMission.mGetTeamHandle (kPlanetDefenseTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
/*	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}
*/
	// configure Tuvius
	// Tuvius should do nothing at the beginning, until the def plats are destroyed
	for (Ship = TuviusTeam->mGetFirstShip(); Ship != NULL; Ship = TuviusTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		setTuviusStartingPatrol( Ship, *this );
	}
	TuviusTeam->mGetFirstShip()->mChangeName(kLNFTuvius_name);

	// LNF uncomment next line
	//TuviusTeam->mGetThisShip(2)->mChangeName(kTuviusWingman_name);

	// configure Faction
	for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);

		if ( Elite == kShiar )
		{
			Ship->mDefendEntity( PlanetTeam->mGetFirstShip(), 70.0, 70.0, kVeryHighPriority );
			Ship->mChangeName( kLNFPraiex_name );
		}
		else
		{
			Ship->mChangeName( kLNFShiar_name );
		}
	}

	// configure Shiar
	for (Ship = ShiarTeam->mGetFirstShip(); Ship != NULL; Ship = ShiarTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mChangeName(kLNFShiar_name);
	}

	// configure Praiex
	for (Ship = PraiexTeam->mGetFirstShip(); Ship != NULL; Ship = PraiexTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDefendEntity( PlanetTeam->mGetFirstShip(), 80.0, 80.0, kVeryHighPriority );
		Ship->mChangeName(kLNFPraiex_name);
	}

	// configure PlanetDefense
	for (Ship = PlanetDefenseTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetDefenseTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
	}

	// configure the Planet
	for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mChangeName(kRomulus_name);
	}
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 1);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TuviusTeam = fMission.mGetTeamHandle (kTuviusTeam);
	tTeamInfo* FactionTeam = fMission.mGetTeamHandle (kFactionTeam);
	tTeamInfo* ShiarTeam = fMission.mGetTeamHandle (kShiarTeam);
	tTeamInfo* PraiexTeam = fMission.mGetTeamHandle (kPraiexTeam);
	tTeamInfo* PlanetDefenseTeam = fMission.mGetTeamHandle (kPlanetDefenseTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyAtDestination:

			if ( Ship->mGetTeam() == kTuviusTeam )
			{

			}

			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						// don't switch sides if damage done by mine - this doesn't work
						if ( fEventItem1 != kDamageByMine )
						{
							// if player is loyal to PRAIEX and shoots at a PRAIEX, change affiliation to SHIAR
							if (( Loyalty == kPraiex )
								&& ( TeamAffiliation( *this, Ship ) == kPraiex )
								&& ( SwitchedSides <= 4 )
								&& ( Ship->mGetShipID() == HostileShip->mGetShipTargetID() ))
							{
								SwitchedSides++;

								// if SwitchedSides > 4, set ANARCHY
								if ( SwitchedSides <= 4 )
								{
									fMission.mDisplayMessage( kEvent2SwitchToShiar_msg, kMissionScriptMessage );

									SetSwitchToShiarTeamRelations( *this );
								}
								else
								{
									// set ANARCHY
									fMission.mDisplayMessage( kEvent2Anarchy_msg, kMissionScriptMessage );

									SetAnarchyTeamRelations( *this );
								}
							}
							// if player is loyal to SHIAR and shoots at SHIAR, change affiliation to PRAIEX
							else if (( Loyalty == kShiar )
									&& ( TeamAffiliation( *this, Ship ) == kShiar )
									&& ( SwitchedSides <= 4 )
									&& ( Ship->mGetShipID() == HostileShip->mGetShipTargetID() ))
							{
								SwitchedSides++;

								// if SwitchedSides > 4, set ANARCHY
								if ( SwitchedSides <= 4 )
								{
									fMission.mDisplayMessage( kEvent3SwitchToPraiex_msg, kMissionScriptMessage );

									SetSwitchToPraiexTeamRelations( *this );
								}
								else
								{
									// set ANARCHY here
									fMission.mDisplayMessage( kEvent2Anarchy_msg, kMissionScriptMessage );

									SetAnarchyTeamRelations( *this );
								}
							}
						}
					}
					
					if ( Ship->mGetTeam() == kTuviusTeam )
					{
						activateTuvius( *this );
					}
	
					// LNF maybe I should put a 1 turn wait between switching sides?
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			if ((Ship->mGetTrackingDistance() == kDistanceToRomulus)
				&& (Ship->mGetTeam() == kTuviusTeam)
				&& ( Ultimatum == 0 ))
			{
				fMission.mDisplayMessage(kEvent4Ultimatum1_msg, kMissionScriptMessage);
				fMission.mDisplayMessage(kEvent4Ultimatum2_msg, kMissionScriptMessage);
				fMission.mDisplayMessage(kEvent4Ultimatum3_msg, kMissionScriptMessage);
				Ultimatum = 1;

				TuviusTeam->mClearCommands();
				for (Ship = TuviusTeam->mGetFirstShip(); Ship != NULL; Ship = TuviusTeam->mGetNextShip())
				{
					setPatrolRoute(Ship, *this );
				}

				// set a countdown - DoomCounter
				fMission.mSetTimedInterval( kTimerCountdownSet, kCountdownToRelease );

				if ( Elite == kShiar )
				{
					//fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
				else // kPraiex
				{
					fMission.mSetTeamRelations ( kPraiexTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0 );
				}
/*
	kTuviusTeam, 
	kFactionTeam,
	kShiarTeam,
	kPraiexTeam,
	kPlanetDefenseTeam,
	kPlanetTeam
*/				
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					
					SetAllTeamRelationsStartMission(*this, kPlayerTeam);

					if ( Elite == kPraiex )
					{
						fMission.mDisplayMessage( kStartScenarioPraiex_msg, kMissionScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kStartScenarioShiar_msg, kMissionScriptMessage );
					}
					break;

				case kTimerCountdownSet:
					// countdown timer to the end of Romulus
					if ( DefeatedTuvius == false )
					{
						fMission.mDisplayMessage( kEvent5TuviusTransportsNewPlague_msg, kScienceScriptMessage );
						return eGameStatus(kPlayerLoses | kCampaignCompleted);
					}
					else
					{
					}
					break;

				case kTimerTurnsSet_6: // Praiex COMM Button timer
					for (Ship = PraiexTeam->mGetFirstShip(); Ship != NULL; Ship = PraiexTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_7: // Shiar COMM Button timer
					for (Ship = ShiarTeam->mGetFirstShip(); Ship != NULL; Ship = ShiarTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_8: // Tuvius COMM Button timer
					for (Ship = TuviusTeam->mGetFirstShip(); Ship != NULL; Ship = TuviusTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9: // Faction COMM Button timer
					for (Ship = FactionTeam->mGetFirstShip(); Ship != NULL; Ship = FactionTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10: // planet COMM button timer
					for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyOnCapture:
			{
				tShipInfo* CapturingShip = fMission.mFindShipID( fEventID );

				fMission.mAddToBonusPrestige( kCaptureBonus );
				return CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipCaptured, Ship, CapturingShip );
			}
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetOriginalTeam() != kPlayerTeam )
			{
				return CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDestroyed, Ship );
			}
			else
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			{
				return CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDisengaged, Ship );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (Ship->mGetTeam())
				{
					case kTuviusTeam:
						TuviusCommMessages(*this, fEventID, Ship, kPlayerTeam);
						break;

					case kFactionTeam:
						if (Elite == kPraiex)
						{
							ShiarCommMessages(*this, fEventID, Ship);
						}
						else
						{
							PraiexCommMessages(*this, fEventID, Ship);
						}
						break;

					case kShiarTeam: // switch
						if (Betrayal == 0)
						{
							ShiarCommMessages(*this, fEventID, Ship);
						}
						else
						{
							PraiexCommMessages(*this, fEventID, Ship);
						}
						break;

					case kPraiexTeam: // switch
						if (Betrayal == 0)
						{
							PraiexCommMessages(*this, fEventID, Ship);
						}
						else
						{
							ShiarCommMessages(*this, fEventID, Ship);
						}
						break;

					case kPlanetDefenseTeam:
						PraiexCommMessages(*this, fEventID, Ship);
						break;

					case kPlanetTeam:
						PraiexCommMessages(*this, fEventID, Ship);
						break;
				}
			}
			break;

	}

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus( status | kCampaignCompleted );
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF add the tuvius condition....if Tuvius doesn't give ultimatum or something.
	// LNF then I have to figure out what side is the winning side.

	if ( WinningSide == kPraiex )
	{
		if ( Loyalty == kPraiex )
		{
			VictoryPoints = 100;
		}
	}
	else if ( WinningSide == kShiar )
	{
		if ( Loyalty == kShiar )
		{
			VictoryPoints = 100;
		}
	}
	else // WinningSide == kAnarchy
	{
		if ( Loyalty == kAnarchy )
		{
			VictoryPoints = 100;
		}
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( PlayerUsedESCKey == false )
	{
		if (Loyalty == kPraiex)
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kVictoryLoyaltyPraiex_msg);
				fMission.mAddToDebriefingList(kPraiexCleaningUp_msg);
			}
			else
			{
				fMission.mAddToDebriefingList(kDefeatLoyaltyPraiex_msg);
				fMission.mAddToDebriefingList(kShiarSlowToRecognize_msg);
			}
		}
		else if (Loyalty == kShiar)
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kVictoryLoyaltyShiar_msg);
				fMission.mAddToDebriefingList(kShiarSlowToRecognize_msg);
			}
			else
			{
				fMission.mAddToDebriefingList(kDefeatLoyaltyShiar_msg);
			}
		}
		else
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kAnarchyPart1_msg);
				if (Elite == kPraiex)
				{
					fMission.mAddToDebriefingList(kPraiexCleaningUp_msg);
				}
				else
				{
					fMission.mAddToDebriefingList(kAnarchyShiar_msg);
				}
			}
			else
			{
				fMission.mAddToDebriefingList(kAnarchyPart1_msg);
				fMission.mAddToDebriefingList(kDefeatLoyaltyAnarchy_msg);
			}
		}

		if ( VictoryLevel >= kDraw )
		{
			fMission.mAddToDebriefingList ( kEndCampaignDefeat_msg );
		}
		else
		{
			fMission.mAddToDebriefingList ( kEndCampaignVictory_msg );
		}
		//fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
		( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
		( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
		( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
		( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
		if ( FailureCount == 2 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}

*/
	// LNF need Final text?
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}



//---------------------------------------------------------------------------------------------
// Set team relations when you SwitchSides if player is Praiex and switches to Shiar
void SetSwitchToShiarTeamRelations( tScript& tScript )
{
	// set the player's loyalty to Shiar
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam); // shiar
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam); // either
	tTeamInfo* ShiarTeam = tScript.fMission.mGetTeamHandle (kShiarTeam); // shiar
	tTeamInfo* PraiexTeam = tScript.fMission.mGetTeamHandle (kPraiexTeam); // praiex
	tTeamInfo* PlanetDefenseTeam = tScript.fMission.mGetTeamHandle (kPlanetDefenseTeam); // praiex
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam); // praiex

	Loyalty = kShiar;

	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kTuviusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
	if ( Elite == kPraiex )
	{
		// faction team is SHIAR
		tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	}
	else
	{
		// faction team is PRAIEX
		tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	}
}

//---------------------------------------------------------------------------------------------
// Set team relations when you SwitchSides if player is Shiar
void SetSwitchToPraiexTeamRelations( tScript& tScript )
{
	// set the Player's loyalty to Praiex
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam); // shiar
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam); // either
	tTeamInfo* ShiarTeam = tScript.fMission.mGetTeamHandle (kShiarTeam); // shiar
	tTeamInfo* PraiexTeam = tScript.fMission.mGetTeamHandle (kPraiexTeam); // praiex
	tTeamInfo* PlanetDefenseTeam = tScript.fMission.mGetTeamHandle (kPlanetDefenseTeam); // praiex
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam); // praiex

	Loyalty = kPraiex;

	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	if ( Elite == kPraiex )
	{
		// faction team is SHIAR
		tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	}
	else
	{
		// faction team is PRAIEX
		tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	}
}

//---------------------------------------------------------------------------------------------
// Set team relations when you Anarchy
void SetAnarchyTeamRelations( tScript& tScript )
{
	// set the player's loyalty to Anarchy
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam); // shiar
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam); // either
	tTeamInfo* ShiarTeam = tScript.fMission.mGetTeamHandle (kShiarTeam); // shiar
	tTeamInfo* PraiexTeam = tScript.fMission.mGetTeamHandle (kPraiexTeam); // praiex
	tTeamInfo* PlanetDefenseTeam = tScript.fMission.mGetTeamHandle (kPlanetDefenseTeam); // praiex
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam); // praiex

	Loyalty = kAnarchy;

	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
}

//---------------------------------------------------------------------------------------------
// determines the team affiliation of the ship the player shoots at
int32 TeamAffiliation( tScript& tScript, tShipInfo* Ship )
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam); // shiar
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam); // either
	tTeamInfo* ShiarTeam = tScript.fMission.mGetTeamHandle (kShiarTeam); // shiar
	tTeamInfo* PraiexTeam = tScript.fMission.mGetTeamHandle (kPraiexTeam); // praiex
	tTeamInfo* PlanetDefenseTeam = tScript.fMission.mGetTeamHandle (kPlanetDefenseTeam); // praiex
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam); // praiex

	if ( Elite == kPraiex )
	{
		// check if Praiex
		if (( Ship->mGetTeam() == kTuviusTeam ) ||
			( Ship->mGetTeam() == kFactionTeam ) ||
			( Ship->mGetTeam() == kShiarTeam ))
		{
			return kShiar;
		}
		// check if Shiar
		else if (( Ship->mGetTeam() == kPraiexTeam ) ||
				 ( Ship->mGetTeam() == kPlanetDefenseTeam ) ||
				 ( Ship->mGetTeam() == kPlanetTeam ))
		{
			return kPraiex;
		}
		return kPraiex;
	}
	else // Elite == kShiar
	{
		// check if Praiex
		if (( Ship->mGetTeam() == kFactionTeam ) ||
			( Ship->mGetTeam() == kPraiexTeam ) ||
			( Ship->mGetTeam() == kPlanetDefenseTeam ) ||
			( Ship->mGetTeam() == kPlanetTeam ))
		{
			return kPraiex;
		}
		// check if Shiar
		else if (( Ship->mGetTeam() == kTuviusTeam ) ||
				 ( Ship->mGetTeam() == kShiarTeam ))
		{
			return kShiar;
		}
		return kShiar;
	}
}

//---------------------------------------------------------------------------------------------
// Set all of the team relations to show up on tactical map
void SetAllTeamRelations(tScript& tScript, eTeamID kPlayerTeam)
{
	// LNF make all ships on Shiar side attack ONLY Defense Platforms
	// LNF when all def plats are destroyed, then go after Praiex ships.

	// set the relations to show up on the map in the right colors
	if (Elite == kPraiex)
	{
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kShiarTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPraiexTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPlanetDefenseTeam, kPlanetTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);
	}
	else // shiar relations
	{
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kTuviusTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPraiexTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kShiarTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPraiexTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetDefenseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPlanetDefenseTeam, kPlanetTeam, kTeamAllied, kGoalDestroy, 1.0, kTrue);
	}
}

//---------------------------------------------------------------------------------------------
// Set all of the team relations at mission start
void SetAllTeamRelationsStartMission(tScript& tScript, eTeamID kPlayerTeam)
{
	// set the relations at the beginning of the mission

	// the Shiar teams should try to take out defense platforms because Tuvius won't move on Romulus
	// until the def plats are destroyed

	// the Praiex team should defend the Planet? def plats?  player?
	if (Elite == kPraiex) // Faction = Shi'ar
	{
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kTuviusTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPraiexTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetDefenseTeam, kTeamHostile, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamHostile, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamHostile, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPraiexTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPlanetDefenseTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kPlanetDefenseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	}
	else // shiar relations  Faction = Prai'ex
	{
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kTuviusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPraiexTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalDestroy, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPraiexTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetDefenseTeam, kTeamHostile, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kTuviusTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetDefenseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kShiarTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamHostile, kGoalDestroy, 1.0);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPraiexTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		tScript.fMission.mSetTeamRelations (kPlanetDefenseTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
		tScript.fMission.mSetTeamRelations (kPlanetDefenseTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for TUVIUS team
void TuviusCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship, eTeamID kPlayerTeam)
{
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam);


	switch (fEventID)
	{
		if ( Ship == TuviusTeam->mGetFirstShip() )
		{
			case kButtonHail_text:
				if ((Loyalty == kShiar) && (Ultimatum == 0))
				{
					if (Ship->mGetDamagePercent() < kDamageThreshold)
					{
						tScript.fMission.mDisplayMessage(kTuviusHailOK_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kTuviusHailDamaged_msg, kMissionScriptMessage);
					}
				}
				else if ((Loyalty == kShiar) && (Ultimatum == 1))
				{
					tScript.fMission.mDisplayMessage(kTuviusHail_1_msg, kMissionScriptMessage);
					tScript.fMission.mDisplayMessage(kTuviusHail_2_msg, kMissionScriptMessage);
				}
				else if (Loyalty == kPraiex)
				{
					if (Ultimatum == 0)
					{
						tScript.fMission.mDisplayMessage(kTuviusHailPraiexF_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kTuviusHailPraiexT_1_msg, kMissionScriptMessage);
						tScript.fMission.mDisplayMessage(kTuviusHailPraiexT_2_msg, kMissionScriptMessage);
					}
				}
				else if (Loyalty == kAnarchy)
				{
					tScript.fMission.mDisplayMessage(kTuviusHailAnarchy_msg, kMissionScriptMessage);
				}
				Ship->mSetCommButton (1, kButtonTaunt_text);
				Ship->mSetCommButton (2, kButtonSurrender_text);
				// If the Taunt or Surrender buttons aren't used in X turns
				// return the buttons to Hail and blank.
				tScript.fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
				break;
				
			case kButtonSurrender_text:
				// LNF Surrender and taunt both work the same
				// if I do the button thing, make sure the surrender replaces itself
				if (Loyalty == kShiar)
				{
					if (Ultimatum == 0)
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderShiarF_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderShiarT_msg, kMissionScriptMessage);
						// LNF need to change relations to have Praiex friendly and Shiar AtWar
						SetSwitchToPraiexTeamRelations( tScript );
					}
				}
				else if (Loyalty == kPraiex)
				{
					if (Ultimatum == 0)
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderPraiexF_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderPraiexF_msg, kMissionScriptMessage);
					}
				}
				else // Loyalty == kAnarchy
				{
					// LNF put something here
				}
				Ship->mSetCommButton (2, kButtonSurrender_text);
				tScript.fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
				break;

			case kButtonTaunt_text:
				if (Loyalty == kShiar)
				{
					if (Ultimatum == 0)
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderShiarF_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderShiarT_msg, kMissionScriptMessage);
						SetSwitchToPraiexTeamRelations( tScript );
					}
				}
				else if (Loyalty == kPraiex)
				{
					if (Ultimatum == 0)
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderPraiexF_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kTuviusSurrenderPraiexF_msg, kMissionScriptMessage);
					}
				}
				else // Loyalty == kAnarchy
				{
					// LNF put something here
				}
				Ship->mSetCommButton (1, kButtonTaunt_text);
				tScript.fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for PRAIEX team
void PraiexCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (Loyalty == kShiar)
			{
				if (Ultimatum == 0)
				{
					tScript.fMission.mDisplayMessage(kPraiexHailShiarUltimatumFalse_msg, kMissionScriptMessage);
				}
				else // Ultimatum == 1
				{
					if (TimesUp == 0)
					{
						tScript.fMission.mDisplayMessage(kPraiexHailShiarTimeNotUp_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kPraiexHailShiarTimeUp_msg, kMissionScriptMessage);
					}
				}
			}
			else if (Loyalty == kPraiex)
			{
				if (Ultimatum == 0)
				{
					if (Ship->mGetDamagePercent() < kDamageThreshold)
					{
						tScript.fMission.mDisplayMessage(kPraiexHailPraiexUltimatumFalseOK_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kPraiexHailPraiexUltimatumFalseDamaged_msg, kMissionScriptMessage);
					}
				}
				else
				{
					if (TimesUp == 0)
					{
						tScript.fMission.mDisplayMessage(kPraiexHailPraiexTimeNotUp_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kPraiexHailPraiexTimeUp_msg, kMissionScriptMessage);
					}
				}
			}
			else if (Loyalty == kAnarchy)
			{
				tScript.fMission.mDisplayMessage(kPraiexHailAnarchy_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
			break;
			
		case kButtonSurrender_text:
			if (Loyalty == kPraiex)
			{
				tScript.fMission.mDisplayMessage( kPraiexSurrenderPraiex_msg, kMissionScriptMessage );
			}
			else if ( Loyalty == kShiar )
			{
				tScript.fMission.mDisplayMessage( kPraiexSurrender_msg, kMissionScriptMessage );
			}
			else // Loyalty == kAnarchy
			{
				tScript.fMission.mDisplayMessage( kPraiexHailAnarchy_msg, kMissionScriptMessage );
			}
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
			break;

		case kButtonTaunt_text:
			if ( Loyalty == kPraiex )
			{
				tScript.fMission.mDisplayMessage( kPraiexSurrenderPraiex_msg, kMissionScriptMessage );
			}
			else if ( Loyalty == kShiar )
			{
				tScript.fMission.mDisplayMessage( tScript.fMission.mRandomInt32(kPraiexTaunt_1_msg, kPraiexTaunt_4_msg), kMissionScriptMessage );
			}
			else // Loyalty == kAnarchy
			{
				tScript.fMission.mDisplayMessage( kPraiexHailAnarchy_msg, kMissionScriptMessage );
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_6, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for SHIAR team
void ShiarCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			if (Loyalty == kShiar)
			{
				if (Ultimatum == 0)
				{
					if (Ship->mGetDamagePercent() < kDamageThreshold)
					{
						tScript.fMission.mDisplayMessage(kShiarHailShiarUltimatumFalseOK_msg, kMissionScriptMessage);
					}
					else
					{
						tScript.fMission.mDisplayMessage(kShiarHailShiarUltimatumFalseDamaged_msg, kMissionScriptMessage);
					}
				}
				else // Ultimatum == 1
				{
					int32 RandomMessageNumber = tScript.fMission.mRandomInt32 (kShiarHailAll_1_msg, kShiarHailAll_4_msg);
					if (RandomMessageNumber == kShiarHailAll_4_msg)
					{
						tScript.fMission.mDisplayMessage(RandomMessageNumber, kMissionScriptMessage);
						// LNF maybe I need another message here?

						// make player loyal to Praiex
						SetSwitchToPraiexTeamRelations( tScript );
					}
					else // random message is one of the other three
					{
						tScript.fMission.mDisplayMessage(RandomMessageNumber, kMissionScriptMessage);
					}
				}
			}
			else if (Loyalty == kPraiex)
			{
				if (Ultimatum == 0)
				{
					tScript.fMission.mDisplayMessage(kShiarHailPraiexUltimatumFalse_msg, kMissionScriptMessage);
				}
				else // Ultimatum == 1
				{
					int32 RandomMessageNumber = tScript.fMission.mRandomInt32 (1, 100);
					if ((RandomMessageNumber >= 1) && (RandomMessageNumber <= 75))
					{
						tScript.fMission.mDisplayMessage(tScript.fMission.mRandomInt32(kShiarHailAll_1_msg, kShiarHailAll_3_msg), kMissionScriptMessage);
					}
					else if ((RandomMessageNumber >= 75) && (RandomMessageNumber <= 100))
					{
						tScript.fMission.mDisplayMessage(tScript.fMission.mRandomInt32(kShiarHailAll_4_msg, kShiarHailAll_6_msg), kMissionScriptMessage);
						SetSwitchToPraiexTeamRelations( tScript );						
					}
				}
			}
			else if (Loyalty == kAnarchy)
			{
				tScript.fMission.mDisplayMessage(kShiarHailAnarchy_msg, kMissionScriptMessage);
			}
			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;
			
		case kButtonSurrender_text:
			tScript.fMission.mDisplayMessage(kShiarSurrender_msg, kMissionScriptMessage);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;

		case kButtonTaunt_text:
			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_7, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for the Planet
void PlanetCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	switch (fEventID)
	{
		case kButtonHail_text:
			tScript.fMission.mDisplayMessage(kPraiexSurrenderPraiex_msg, kMissionScriptMessage);

			Ship->mSetCommButton (1, kButtonTaunt_text);
			Ship->mSetCommButton (2, kButtonSurrender_text);
			// If the Taunt or Surrender buttons aren't used in X turns
			// return the buttons to Hail and blank.
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
			break;
			
		case kButtonSurrender_text:
			tScript.fMission.mDisplayMessage(kPraiexSurrenderPraiex_msg, kMissionScriptMessage);

			Ship->mSetCommButton (2, kButtonSurrender_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
			break;

		case kButtonTaunt_text:
			tScript.fMission.mDisplayMessage(kPraiexSurrenderPraiex_msg, kMissionScriptMessage);

			Ship->mSetCommButton (1, kButtonTaunt_text);
			tScript.fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
			break;
	}
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
eGameStatus CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam); // shiar
	tTeamInfo* FactionTeam = tScript.fMission.mGetTeamHandle (kFactionTeam); // either
	tTeamInfo* ShiarTeam = tScript.fMission.mGetTeamHandle (kShiarTeam); // shiar
	tTeamInfo* PraiexTeam = tScript.fMission.mGetTeamHandle (kPraiexTeam); // praiex
	tTeamInfo* PlanetDefenseTeam = tScript.fMission.mGetTeamHandle (kPlanetDefenseTeam); // praiex
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam); // praiex

	// if all ships on a team are gone
	//if ( EventTeam->mGetShipsInPlayCount() == ((EventShipStatus == kEventShipCaptured) ? 1 : 0) )
	if (Ship->mGetOriginalTeam() == kTuviusTeam)
	{
		TuviusDestroyed++;

		if ( Ship == TuviusTeam->mGetFirstShip() )
		{
			DefeatedTuvius = true;

			if ( EventShipStatus == kEventShipCaptured )
			{
				tScript.fMission.mDisplayMessage( kTuviusCaptured_msg, kMissionScriptMessage );
			}
			else
			{
				tScript.fMission.mDisplayMessage( kTuviusDestroyed_msg, kMissionScriptMessage );
			}
		}

		if ( TuviusTeam->mGetShipsInPlayCount() == 0 )
		{
			AllTuviusTeamDestroyed = true;
			return CheckWinCondition( tScript );
		}
		else
		{
			return kGameInProgress;
		}
	}
	else if (Ship->mGetOriginalTeam() == kFactionTeam)
	{
		FactionDestroyed++;

		if ( FactionTeam->mGetShipsInPlayCount() == 0 )
		{
			AllFactionTeamDestroyed = true;
			return CheckWinCondition( tScript );
		}
		else
		{
			return kGameInProgress;
		}
	}
	else if (Ship->mGetOriginalTeam() == kShiarTeam)
	{
		ShiarDestroyed++;

		if ( ShiarTeam->mGetShipsInPlayCount() == 0 )
		{
			AllShiarTeamDestroyed = true;

			if ( AllPlanetDefenseTeamDestroyed == false )
			{
				// if the Shiar team is destroyed, unleash Tuvius
				activateTuvius( tScript );
			}
			return CheckWinCondition( tScript );
		}
		else
		{
			return kGameInProgress;
		}
	}
	else if (Ship->mGetOriginalTeam() == kPraiexTeam)
	{
		PraiexDestroyed++;

		if ( PraiexTeam->mGetShipsInPlayCount() == 0 )
		{
			AllPraiexTeamDestroyed = true;
			return CheckWinCondition( tScript );
		}
		else
		{
			return kGameInProgress;
		}
	}
	else if (Ship->mGetOriginalTeam() == kPlanetDefenseTeam)
	{
		PlanetDefenseDestroyed++;

		if (( PlanetDefenseTeam->mGetShipsInPlayCount() == 0 ) && ( AllShiarTeamDestroyed == false ))
		{
			AllPlanetDefenseTeamDestroyed = true;
			tScript.fMission.mDisplayMessage( kEvent6DefPlatsDestroyed_msg, kMissionScriptMessage );

			// make the Shi'ar ships attack any remaining Prai'ex ships
			if ( Loyalty == kPraiex )
			{
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kTuviusTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				if ( Elite == kPraiex ) // Faction = Shi'ar
				{
					tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kTuviusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
				else
				{
					tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
			}
			else if ( Loyalty == kShiar )
			{
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kTuviusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShiarTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				if ( Elite == kPraiex ) // Faction = Shi'ar
				{
					tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kTuviusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
				else
				{
					tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
			}
			else // Loyalty == kAnarchy
			{
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
				tScript.fMission.mSetTeamRelations (kTuviusTeam, tScript.kPlayerTeam, kTeamHostile, kGoalDestroy, 1.0);//***
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				tScript.fMission.mSetTeamRelations (kShiarTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				tScript.fMission.mSetTeamRelations (kShiarTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				tScript.fMission.mSetTeamRelations (kPraiexTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

				if ( Elite == kPraiex ) // Faction = Shi'ar
				{
					tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kTuviusTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
				else
				{
					tScript.fMission.mSetTeamRelations (tScript.kPlayerTeam, kFactionTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kTuviusTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					tScript.fMission.mSetTeamRelations (kTuviusTeam, kFactionTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					tScript.fMission.mSetTeamRelations (kFactionTeam, kShiarTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPraiexTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					tScript.fMission.mSetTeamRelations (kFactionTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
				}
			}

			// LNF other ships should attack anyone

			activateTuvius( tScript );

			return CheckWinCondition( tScript );
		}
		else
		{
			return kGameInProgress;
		}
	}
	else
	{
		return kGameInProgress;
	}
}

//---------------------------------------------------------------------------------------------
// win condition
eGameStatus CheckWinCondition( tScript& tScript )
{
	if ( Elite == kPraiex ) // faction == kShiar
	{
		if (( AllTuviusTeamDestroyed == true ) &&
			( AllShiarTeamDestroyed == true ) &&
			( AllFactionTeamDestroyed == true ) &&
			( AllPraiexTeamDestroyed == false ) &&
			( AllPlanetDefenseTeamDestroyed == false ))
		{
			WinningSide = kPraiex;
			tScript.fMission.mDisplayMessage( kEvent7LastShiarShipDestroyed_msg, kMissionScriptMessage );
		}
		else if (( AllTuviusTeamDestroyed == false ) &&
			( AllShiarTeamDestroyed == false ) &&
			( AllFactionTeamDestroyed == false ) &&
			( AllPraiexTeamDestroyed == true ) &&
			( AllPlanetDefenseTeamDestroyed == true ))
		{
			WinningSide = kShiar;
			tScript.fMission.mDisplayMessage( kEvent7LastPraiexShipDestroyed_msg, kMissionScriptMessage );
		}
		else if (( AllTuviusTeamDestroyed == true ) &&
				 ( AllShiarTeamDestroyed == true ) &&
  				 ( AllFactionTeamDestroyed == true ) &&
  				 ( AllPraiexTeamDestroyed == true ) &&
				 ( AllPlanetDefenseTeamDestroyed == true ))
		{
			// ANARCHY!!!
			WinningSide = kAnarchy;
			tScript.fMission.mDisplayMessage( kEvent7LastShipDestroyed_msg, kMissionScriptMessage );
		}
	}
	else // Elite == kShiar  faction == kPraiex
	{
		if (( AllTuviusTeamDestroyed == true ) &&
			( AllShiarTeamDestroyed == true ) &&
			( AllFactionTeamDestroyed == false ) &&
			( AllPraiexTeamDestroyed == false ) &&
			( AllPlanetDefenseTeamDestroyed == false ))
		{
			WinningSide = kPraiex;
			tScript.fMission.mDisplayMessage( kEvent7LastShiarShipDestroyed_msg, kMissionScriptMessage );
		}
		else if (( AllTuviusTeamDestroyed == false ) &&
			( AllShiarTeamDestroyed == false ) &&
			( AllFactionTeamDestroyed == true ) &&
			( AllPraiexTeamDestroyed == true ) &&
			( AllPlanetDefenseTeamDestroyed == true ))
		{
			WinningSide = kShiar;
			tScript.fMission.mDisplayMessage( kEvent7LastPraiexShipDestroyed_msg, kMissionScriptMessage );
		}
		else if (( AllTuviusTeamDestroyed == true ) &&
				 ( AllShiarTeamDestroyed == true ) &&
  				 ( AllFactionTeamDestroyed == true ) &&
  				 ( AllPraiexTeamDestroyed == true ) &&
				 ( AllPlanetDefenseTeamDestroyed == true ))
		{
			// ANARCHY!!!
			WinningSide = kAnarchy;
			tScript.fMission.mDisplayMessage( kEvent7LastShipDestroyed_msg, kMissionScriptMessage );
		}
	}

	if ( WinningSide != 0 )
	{
		if ( Loyalty == WinningSide )
		{
				return (eGameStatus)( kPlayerWins | kCampaignCompleted );
		}
		else
		{
				return (eGameStatus)( kPlayerLoses | kCampaignCompleted );
		}
	}
	else
	{
		return kGameInProgress;
	}
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void setPatrolRoute(tShipInfo* Ship, tScript& tScript)
{
	const float PatrolAttackRadius = 30.0;
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam);		

	tMapPoint waypoints1[2]= // ship W
	{
		24, 11,
		23, 13,
	};

	tMapPoint PatrolPoints[3][10];

	PatrolPoints[0][0] = waypoints1[0];
	PatrolPoints[0][1] = waypoints1[1];
//	PatrolPoints[0][2] = waypoints1[2];
//	PatrolPoints[0][3] = waypoints1[3];
//	PatrolPoints[0][4] = waypoints1[4];
//	PatrolPoints[0][5] = waypoints1[5];
//	PatrolPoints[0][6] = waypoints1[6];
//	PatrolPoints[0][7] = waypoints1[7];
	Ship->mPatrol (PatrolAttackRadius, 2, &PatrolPoints[0][0], kMediumPriority);
}

//---------------------------------------------------------------------------------------------
// activate the Tuvius team - make it go toward Romulus and drop the traveller's death
void activateTuvius( tScript& tScript )
{
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam);

	TuviusTeam->mGetFirstShip()->mTrackTacticalDistance(kDistanceToRomulus, kRomulusX, kRomulusY);  // dist to planet
	TuviusTeam->mGetFirstShip()->mSetFinalDestination( kRomulusX, kRomulusY ); // LNF go to planet...should be NEAR planet or it will crash into planet

	// LNF uncomment next line
	//TuviusTeam->mGetThisShip(2)->mSetFinalDestination( kRomulusX, kRomulusY ); // LNF go to planet...should be NEAR planet or it will crash into planet
	TuviusTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
}

//---------------------------------------------------------------------------------------------
// set patrol for Tuvius team
void setTuviusStartingPatrol(tShipInfo* Ship, tScript& tScript)
{
	const float PatrolAttackRadius = 1.0;
	tTeamInfo* TuviusTeam = tScript.fMission.mGetTeamHandle (kTuviusTeam);		

	tMapPoint waypoints1[3]=
	{
		10, 8,
		5, 6,
		10, 4,
	};

	tMapPoint PatrolPoints[3][10];

	PatrolPoints[0][0] = waypoints1[0];
	PatrolPoints[0][1] = waypoints1[1];
	PatrolPoints[0][2] = waypoints1[2];
	Ship->mPatrol (PatrolAttackRadius, 3, &PatrolPoints[0][0], kLowPriority);
}
