#include "scriptinterface.h"

#include "Tec_TechnicalBCHText.h"
#include "Tec_TechnicalBCHMaps.h"

int32 EnemyKilled = 0;
int32 TechnicalSuccess = 0;
int32 OldTechnicalSuccess = 0;
bool SkillsPassed = false;
bool MissionLost = false;

const tString TechnicalShip[6] = { "F-BCJ", "K-C7", "R-KRCS", "L-BCH", "H-OV", "G-BCH" };

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kTargetTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTechnicalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 4;							// fifth choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	TechnicalSuccess = info.fTechnicalSuccess;
	OldTechnicalSuccess = TechnicalSuccess;

	if( TechnicalSuccess != 0 )
	{
		SkillsPassed = true;
	}

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kTargetTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kTargetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kTargetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will be using a Battlecruiser for this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* TargetTeam = fMission.mGetTeamHandle( kTargetTeam );
	tShipInfo* Ship;

	tString CurrentPlayerShip = TechnicalShip[ PlayerTeam->mGetRace() ];

	// Create a battlecruiser for the player to use
	if (Mode == 0)
	{
		if( SkillsPassed == true )
		{
			PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_H);
		}
		else
		{
			PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_G);
			TargetTeam->mCreateShip( kMaxBPV, kClassFreighter, 0, kStartPosition_M );

			if( fMission.mRandomInt32( 1, 2 ) == 1 )
			{
				EnemyTeam->mCreateShip( "K-E4D", kStartPosition_P );
			}
			else
			{
				EnemyTeam->mCreateShip( "K-E4D", kStartPosition_R );
			}

			if( fMission.mRandomInt32( 1, 2 ) == 1 )
			{
				EnemyTeam->mCreateShip( "K-E4D", kStartPosition_Q );
			}
			else
			{
				EnemyTeam->mCreateShip( "K-E4D", kStartPosition_S );
			}
		}
	}

	// create an enemy frigate in a location away from the player's current position
	if( Mode == 1 )
	{
		Ship = PlayerTeam->mGetFirstShip();

		if( Ship->mGetTacticalDistance( 'Y' ) <= 40.0 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyBattlecruiser, 20, kStartPosition_Z );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyBattlecruiser, 20, kStartPosition_Y );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	if( SkillsPassed == false )
	{
		return( kBriefing_msg );
	}
	else
	{
		return( kBriefingReturn_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* TargetTeam = fMission.mGetTeamHandle( kTargetTeam );
	tShipInfo* Ship;

	// configure the player's battlecruiser
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipCallbackEvents( kNotifyBeginTractor );

	if( SkillsPassed == false )
	{
		// configure the target
		Ship = TargetTeam->mGetFirstShip();
		Ship->mSetShipMaxSpeed( 0 );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );
		Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );

		// configure the missile cruisers
		for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
		{
			Ship->mDamageSystem( kDamageWeapon12, 100.0 );
			Ship->mDamageSystem( kDamageWeapon13, 100.0 );
			Ship->mDamageSystem( kDamageWeapon20, 100.0 );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyMissileSpeed, 0 );
			Ship->mSetShipAIControlled( kFalse );
			Ship->mSetShipCallbackEvents( kNotifyDamage );
		}

		fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
		fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );	// toggle missile ships on and off
	}
	else
	{
		fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );	// returning to the technical
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* TargetTeam = fMission.mGetTeamHandle( kTargetTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kTargetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kTargetTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTimedInterval( kTimerCountdownSet, 12 );
					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					break;

				case kTimerCountdownSet:
					fMission.mAddToBonusPrestige( kTechnicalBonus );	// one-time prestige bonus for finishing aptitude
					fMission.mDisplayMessage( kWePassed_msg );
					SkillsPassed = true;

					fPlayerShip->mDamageSystem( kDamageWeaponsSystemsAndShields, 0.0 );
					fPlayerShip->mAccessSupplies( kResetAllItems, 0, 0 );

					Ship = TargetTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						Ship->mSetShipMaxSpeed( 31 );
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					}

					for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
					{
						Ship->mSetShipMaxSpeed( 31 );
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					}

					fMission.mSetTimedInterval( kTimerTurnsSet_2, 4 );
					break;

				case kTimerTurnsSet:
					for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
					{
						Ship->mSetShipAIControlled( kTrue );
						Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
						Ship->mSetShipTarget( TargetTeam->mGetFirstShip(), kAbsolutePriority );
						Ship->mSetShipMaxSpeed( 0 );
					}
					break;

				case kTimerTurnsSet_2:
					fMission.mDisplayMessage( kStartComsim_msg );
					mCreateMissionShips( 1 );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kTargetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;

				case kTimerTurnsSet_3:
					if( SkillsPassed == false )
					{
						for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
						{
							if( fMission.mRandomInt32( 1, 3 ) == 1 )
							{
								Ship->mSetShipAIControlled( kFalse );
							}
							else
							{
								Ship->mSetShipAIControlled( kTrue );
								Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
								Ship->mSetShipTarget( TargetTeam->mGetFirstShip(), kAbsolutePriority );
								Ship->mSetShipMaxSpeed( 0 );
							}
						}
					}
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( SkillsPassed == false )
				{
					return kPlayerLoses;
				}
				else
				{
					return kPlayerWins;
				}
			}
			
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				if( Ship->mGetClassType() == kClassHeavyBattlecruiser )
				{
					EnemyKilled++;
					fMission.mDisplayMessage( kNewEnemy_msg );
					mCreateMissionShips( 1 );
				}
				else if( SkillsPassed == false )
				{
					fMission.mDisplayMessage( kCannotAttack_msg, kNonCommScriptMessage );
					return kPlayerLoses;
				}
			}
			
			if(( Ship->mGetTeam() == kTargetTeam ) && ( SkillsPassed == false ))
			{
				fMission.mDisplayMessage( kTargetDestroyed_msg, kNonCommScriptMessage );
				return kPlayerLoses;
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				EnemyKilled++;

				if( SkillsPassed == true )
				{
					fMission.mDisplayMessage( kNewEnemy_msg );
					mCreateMissionShips( 1 );
				}
				else
				{
					return kPlayerLoses;
				}
			}

			if( Ship->mGetTeam() == kTargetTeam )
			{
				fMission.mDisplayMessage( kTargetCaptured_msg );
				return kPlayerLoses;
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( SkillsPassed == false )
				{
					fMission.mDisplayMessage( kCannotAttack_msg );
					return kPlayerLoses;
				}
			}
			
			if( Ship->mGetTeam() == kTargetTeam )
			{
				if( Ship->mGetDamagePercent() > 75 )
				{
					fMission.mDisplayMessage( kTargetCrippled_msg );
				}
				else
				{
					if( Ship != NULL )
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		case kNotifyBeginTractor:
			if( SkillsPassed == false )
			{
				fMission.mDisplayMessage( kNoTractoring_msg );
				return kPlayerLoses;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( SkillsPassed == true )
	{
		TechnicalSuccess = 1 + EnemyKilled;

		switch( EnemyKilled )
		{
			case 0:
				return( kDefeat );

			case 1:
				return( kDraw );

			case 2:
				return( kVictory );

			default:
				return( kAstoundingVictory );
		}
	}

	TechnicalSuccess = 0;
	return( kDevastatingDefeat );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if(( OldTechnicalSuccess == 0 ) && ( TechnicalSuccess > 0 ))
	{
		fMission.mAddToDebriefingList( kAptitudeSuccess_msg );
	}

	if( SkillsPassed == false )
	{
		fMission.mAddToDebriefingList( kAptitudeFailure_msg );
	}

	if( SkillsPassed == true )
	{
		switch( EnemyKilled )
		{
			case 0:
				fMission.mAddToDebriefingList( kKilled0_msg );
				break;

			case 1:
				fMission.mAddToDebriefingList( kKilled1_msg );
				break;

			case 2:
				fMission.mAddToDebriefingList( kKilled2_msg );
				break;

			default:
				fMission.mAddToDebriefingList( kKilled3_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TechnicalSuccess <= OldTechnicalSuccess )
	{
		TechnicalSuccess = OldTechnicalSuccess;
	}

	if( TechnicalSuccess >= 99 )	// yep, this is going to happen
	{
		TechnicalSuccess = 99;
	}

	return TechnicalSuccess;
}
