#include "scriptinterface.h"

#include "Tec_TechnicalCAText.h"
#include "Tec_TechnicalCAMaps.h"

int32 Checkpoint = 0;
int32 RivalCP = 0;
int32 EnemyKilled = 0;
int32 TechnicalSuccess = 0;
int32 OldTechnicalSuccess = 0;
bool SkillsPassed = false;

const tString TechnicalShip[6] = { "F-CAR", "K-D7B", "R-SPA+", "L-CA+", "H-RN+", "G-CA" };

const int32 AstXY[6][2] =
{
	{ 13, 20 },
	{ 13, 15 },
	{ 10, 11 },
	{  7, 15 },
	{  7, 20 },
	{ 10, 29 }
};

const int32 RivalXY[6][2] =
{
	{ 12, 20 },
	{ 12, 15 },
	{ 10, 12 },
	{  8, 15 },
	{  8, 20 },
	{ 10, 29 }
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTechnicalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 3;							// fourth choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	TechnicalSuccess = info.fTechnicalSuccess;
	OldTechnicalSuccess = TechnicalSuccess;

	if( TechnicalSuccess != 0 )
	{
		SkillsPassed = true;
	}

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	// setup initial team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will be using a Frigate for this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	tString CurrentPlayerShip = TechnicalShip[ PlayerTeam->mGetRace() ];

	// Create a ship for the player to use
	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_G );

		if( SkillsPassed == false )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_H );
		}
	}

	// create an enemy ship in a location away from the player's current position
	if( Mode == 1 )
	{
		if( fPlayerShip->mGetTacticalDistance( 'Y' ) <= 40.0 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 20, kStartPosition_Z );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 20, kStartPosition_Y );
		}
		fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	if( SkillsPassed == false )
	{
		return( kBriefing_msg );
	}
	else
	{
		return( kBriefingReturn_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();

	if( SkillsPassed == false )
	{
		Ship->mTrackTacticalDistance( 6.0, AstXY[ Checkpoint ][0], AstXY[ Checkpoint ][1] );
		fMission.mSetGoalIndicator( kTrue, AstXY[ Checkpoint ][0], AstXY[ Checkpoint ][1] );

		Ship = EnemyTeam->mGetFirstShip();
		Ship->mChangeName( kRacerX_name );
		Ship->mSetFinalDestination( RivalXY[ RivalCP ][0], RivalXY[ RivalCP ][1] );

		fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	}
	else
	{
		fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
	}

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch( Event )
	{
		case kNotifyAtDestination:
			RivalCP++;

			if( RivalCP < 6 )
			{
				Ship->mSetFinalDestination( RivalXY[ RivalCP ][0], RivalXY[ RivalCP ][1] );
			}
			else if( SkillsPassed == false )
			{
				fMission.mDisplayMessage( kTimeFailed_msg );
				fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
				return kPlayerLoses;
			}
			break;

		case kNotifyAtDistance:
			Checkpoint++;

			if( Checkpoint < 6 )
			{
				fMission.mDisplayMessage( kCheckpointReached_msg, kNonCommScriptMessage );
				Ship->mTrackTacticalDistance( 7.5, AstXY[ Checkpoint ][0], AstXY[ Checkpoint ][1] );
				fMission.mSetGoalIndicator( kTrue, AstXY[ Checkpoint ][0], AstXY[ Checkpoint ][1] );
			}
			else
			{
				SkillsPassed = true;
				fMission.mDisplayMessage( kWinner_msg );
				fMission.mAddToBonusPrestige( kTechnicalBonus );
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
				Ship = PlayerTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 0.0 );
				Ship->mAccessSupplies( kResetAllItems, 0, 0 );

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mDisplayMessage( kStartComsim_msg );
					mCreateMissionShips( 1 );
				}
				else
				{
					Ship = EnemyTeam->mGetFirstShip();
					Ship->mClearCommands();
					fMission.mDisplayMessage( kRivalAttack_msg );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kStartComsim_msg );
					mCreateMissionShips( 1 );
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( SkillsPassed == false )
				{
					return kPlayerLoses;
				}
				else
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				if( SkillsPassed == true )
				{
					EnemyKilled++;
					fMission.mDisplayMessage( kNewEnemy_msg );
					mCreateMissionShips( 1 );
				}
				else
				{
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mDisplayMessage( kMustFinish_msg );
				}
			}

			break;

		case kNotifyOnCapture:
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
			Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );

			if( SkillsPassed == true )
			{
				fMission.mDisplayMessage( kNewEnemy_msg );
				mCreateMissionShips( 1 );
			}
			else
			{
				fMission.mDisplayMessage( kMustFinish_msg );
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( SkillsPassed == true )
	{
		TechnicalSuccess = 1 + EnemyKilled;
		return( kVictory );
	}

	TechnicalSuccess = 0;
	return( kDefeat );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( SkillsPassed == false )
	{
		fMission.mAddToDebriefingList( kAptitudeFailure_msg );
	}

	if(( OldTechnicalSuccess == 0 ) && ( SkillsPassed == true ))
	{
		fMission.mAddToDebriefingList( kAptitudeSuccess_msg );
	}

	if( SkillsPassed == true )
	{
		switch( EnemyKilled )
		{
			case 0:
				fMission.mAddToDebriefingList( kKilled0_msg );
				break;

			case 1:
				fMission.mAddToDebriefingList( kKilled1_msg );
				break;

			case 2:
				fMission.mAddToDebriefingList( kKilled2_msg );
				break;

			default:
				fMission.mAddToDebriefingList( kKilled3_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TechnicalSuccess <= OldTechnicalSuccess )
	{
		TechnicalSuccess = OldTechnicalSuccess;
	}

	if( TechnicalSuccess >= 99 )					// yep, this is going to happen
	{
		TechnicalSuccess = 99;
	}

	return TechnicalSuccess;
}
