#include "scriptinterface.h"

#include "Tec_TechnicalDDText.h"
#include "Tec_TechnicalDDMaps.h"

int32 TugsDelivered = 0;
int32 EnemyKilled = 0;
int32 TechnicalSuccess = 0;
int32 OldTechnicalSuccess = 0;
bool SkillsPassed = false;

const tString TechnicalShip[6] = { "F-DDG", "K-FWK", "R-K5LB", "L-DW+", "H-DWH", "G-BDL+" };

void TugPatrol( tTeamInfo* PatrolTeam );

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
//( use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams( enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kTugTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID( void )
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kTechnicalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr( );
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 1;							// second choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart( tDynaverseScriptInfo& info )
{
	TechnicalSuccess = info.fTechnicalSuccess;
	OldTechnicalSuccess = TechnicalSuccess;

	if( TechnicalSuccess != 0 )
	{
		SkillsPassed = true;
	}

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kTugTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kTugTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kTugTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip( tShipInGame& Ship )
{
	// we will be using a Frigate for this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call)
void tScript::mCreateMissionShips( int32 Mode )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* TugTeam = fMission.mGetTeamHandle( kTugTeam );

	tString CurrentPlayerShip = TechnicalShip[ PlayerTeam->mGetRace() ];

	// Create a frigate for the player to use
	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_G );

		if( SkillsPassed == false )
		{
			TugTeam->mCreateShip( kMaxBPV, kClassFreighter, 0, kStartPosition_M );
			TugTeam->mCreateShip( kMaxBPV, kClassFreighter, 0, kStartPosition_N );
		}
	}

	// create an enemy frigate in a location away from the player's current position
	if( Mode == 1 )
	{
		if( fPlayerShip->mGetTacticalDistance( 'Y' ) <= 40.0 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 20, kStartPosition_Z );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 20, kStartPosition_Y );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	if( SkillsPassed == false )
	{
		return( kBriefing_msg );
	}
	else
	{
		return( kBriefingReturn_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission( void )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* TugTeam = fMission.mGetTeamHandle( kTugTeam );
	tShipInfo* Ship;

	// configure the player
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();

	if( SkillsPassed == false )
	{
		// configure the tugs
		for( Ship = TugTeam->mGetFirstShip(); Ship != NULL; Ship = TugTeam->mGetNextShip() )
		{
			Ship->mTrackTacticalDistance( 6.0, 20, 20 );
			Ship->mSetShipMaxSpeed( 8 );
		}

		TugPatrol( TugTeam );
		fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	}
	else
	{
		fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip( tShipInfo* Ship )
{
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent( tShipInfo* Ship, iNotifyEvents Event )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* TugTeam = fMission.mGetTeamHandle( kTugTeam );

	switch( Event)
	{
		case kNotifyAtDistance:
			Ship->mSetShipMaxSpeed( 0 );
			Ship->mSetShipAIControlled( kFalse );
			TugsDelivered++;

			if( TugsDelivered == 1 )
			{
				fMission.mDisplayMessage( kTugGood_msg );
			}
			else
			{
				fMission.mDisplayMessage( kTugExcellent_msg );
				SkillsPassed = true;
				fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );
				fMission.mAddToBonusPrestige( kTechnicalBonus );	// one-time prestige bonus for finishing aptitude
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mSetTimedInterval( kTimerCountdownSet, 22 );
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kStartComsim_msg );
					mCreateMissionShips( 1 );
					break;

				case kTimerCountdownSet:
					if( SkillsPassed == false )
					{
						fMission.mDisplayMessage( kTimeFailed_msg );
						fPlayerShip->mSetShipThrust( 0 );
						return kPlayerLoses;
					}
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( SkillsPassed == false )
				{
					return kPlayerLoses;
				}
				else
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyKilled++;
				fMission.mDisplayMessage( kNewEnemy_msg );
				mCreateMissionShips( 1 );
			}

			if(( Ship->mGetTeam() == kTugTeam ) && ( SkillsPassed == false ))
			{
				return kPlayerLoses;
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus( void )
{
	if( SkillsPassed == true )
	{
		TechnicalSuccess = 1 + EnemyKilled;

		switch( EnemyKilled )
		{

			case 0:
				return( kDefeat );

			case 1:
				return( kDraw );

			case 2:
				return( kVictory );

			default:
				return( kAstoundingVictory );
		}
	}

	TechnicalSuccess = 0;
	return( kDevastatingDefeat );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
	if(( OldTechnicalSuccess == 0 ) && ( SkillsPassed == true ))
	{
		fMission.mAddToDebriefingList( kAptitudeSuccess_msg );
	}

	if( SkillsPassed == false )
	{
		fMission.mAddToDebriefingList( kAptitudeFailure_msg );
	}
	else
	{
		switch( EnemyKilled )
		{
			case 0:
				fMission.mAddToDebriefingList( kKilled0_msg );
				break;

			case 1:
				fMission.mAddToDebriefingList( kKilled1_msg );
				break;

			case 2:
				fMission.mAddToDebriefingList( kKilled2_msg );
				break;

			default:
				fMission.mAddToDebriefingList( kKilled3_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	if( TechnicalSuccess <= OldTechnicalSuccess )
	{
		TechnicalSuccess = OldTechnicalSuccess;
	}

	if( TechnicalSuccess >= 99 )					// yep, this is going to happen
	{
		TechnicalSuccess = 99;
	}

	return TechnicalSuccess;
}

//---------------------------------------------------------------------------------------------
// Tug Patrol routes
void TugPatrol( tTeamInfo* PatrolTeam )
{
	tMapPoint waypoints_1[8]=
	{
		25, 22,
		27, 24,
		29, 22,
		30, 20,
		29, 18,
		27, 16,
		25, 18,
		24, 20
	};

	tMapPoint waypoints_2[8]=
	{
		14, 18,
		12, 16,
		 8, 18,
		 6, 20,
		 8, 22,
		10, 24,
		14, 24,
		16, 20
	};

	tMapPoint PatrolPoints[2][8];

	PatrolPoints[0][0] = waypoints_1[0];
	PatrolPoints[0][1] = waypoints_1[1];
	PatrolPoints[0][2] = waypoints_1[2];
	PatrolPoints[0][3] = waypoints_1[3];
	PatrolPoints[0][4] = waypoints_1[4];
	PatrolPoints[0][5] = waypoints_1[5];
	PatrolPoints[0][6] = waypoints_1[6];
	PatrolPoints[0][7] = waypoints_1[7];
	PatrolTeam->mGetThisShip(1)->mPatrol( 0, 8, &PatrolPoints[0][0], kHighPriority );

	PatrolPoints[1][0] = waypoints_2[0];
	PatrolPoints[1][1] = waypoints_2[1];
	PatrolPoints[1][2] = waypoints_2[2];
	PatrolPoints[1][3] = waypoints_2[3];
	PatrolPoints[1][4] = waypoints_2[4];
	PatrolPoints[1][5] = waypoints_2[5];
	PatrolPoints[1][6] = waypoints_2[6];
	PatrolPoints[1][7] = waypoints_2[7];
	PatrolTeam->mGetThisShip(2)->mPatrol( 0, 8, &PatrolPoints[1][0], kHighPriority );
}
