#include "scriptinterface.h"

#include "Tec_TechnicalDreadnoughtText.h"
#include "Tec_TechnicalDreadnoughtMaps.h"

int32 EnemyKilled = 0;
int32 TechnicalSuccess = 0;
int32 OldTechnicalSuccess = 0;
bool SkillsPassed = false;
bool FrigatesKilled = false;
bool DamagedPlayerDreadnought = false;

const tString TechnicalShip[6] = { "F-DNG", "K-C9K", "R-CON", "L-DNP+", "H-PAL+", "G-DNF" };

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kFrigateTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;					// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTechnicalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 5;							// last choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	TechnicalSuccess = info.fTechnicalSuccess;
	OldTechnicalSuccess = TechnicalSuccess;

	if( TechnicalSuccess > 0 )
	{
		SkillsPassed = true;
	}

	fMission.mSetPrestigePoints( 0, 0, 0, 0, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kFrigateTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kFrigateTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kFrigateTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will be using a Dreadnought for this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* FrigateTeam = fMission.mGetTeamHandle( kFrigateTeam );

	tString CurrentPlayerShip = TechnicalShip[ PlayerTeam->mGetRace() ];

	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_G );

		if( SkillsPassed == false )
		{
			FrigateTeam->mCreateFleet( 3, kMaxBPV, kClassFrigate, kClassFrigate, 20, kStartPosition_V );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, kClassDreadnought, 20, kStartPosition_Y );
		}
		return;
	}

	// create an enemy Dreadnought in a location away from the player's current position
	if( Mode == 1 )
	{
		if( fPlayerShip->mGetTacticalDistance( 'Y' ) <= 40.0 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, kClassDreadnought, 20, kStartPosition_V );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, kClassDreadnought, 20, kStartPosition_Y );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	if( TechnicalSuccess == 0 )
	{
		return( kBriefing_msg );
	}
	else
	{
		return( kBriefingReturn_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* FrigateTeam = fMission.mGetTeamHandle( kFrigateTeam );
	tShipInfo* Ship;

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	if( SkillsPassed == false )
	{
		fMission.mDisplayMessage( kStarting_msg );				// startup message
		fMission.mSetTimedInterval( kTimerCountdownSet, 36 );
		fMission.mSetTimedInterval( kTimerTurnsSet, 1 );

		for( Ship = FrigateTeam->mGetFirstShip(); Ship != NULL; Ship = FrigateTeam->mGetNextShip() )
		{
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 0 );
			Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
			Ship->mChangeAIOfficerState( kCloakOfficer, kFalse );
		}
	}
	else
	{
		fMission.mDisplayMessage( kStartComsim_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* FrigateTeam = fMission.mGetTeamHandle( kFrigateTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			if( FrigatesKilled == true )
			{
				fMission.mDisplayMessage( kRepairYourShip_msg, kMissionScriptMessage );
				Ship = PlayerTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 0.0 );
				Ship->mAccessSupplies( kResetAllItems, 0, 0 );
			}
			else if( DamagedPlayerDreadnought == false )
			{
				DamagedPlayerDreadnought = true;
				fMission.mDisplayMessage( kDamageYourShip_msg, kMissionScriptMessage );
				Ship = PlayerTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageMaxWeapons, 25.0 );
			}
			else if( SkillsPassed == false )
			{
				fMission.mDisplayMessage( kTimeFailed_msg );
				return kPlayerLoses;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( SkillsPassed == false )
				{
					return kPlayerLoses;
				}
				else
				{
					return kPlayerWins;
				}
			}
			
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyKilled++;
				fMission.mDisplayMessage( kNewEnemy_msg );
				mCreateMissionShips( 1 );
			}
			
			if( Ship->mGetOriginalTeam() == kFrigateTeam )
			{
				if( FrigateTeam->mGetShipsInPlayCount() == 0 )
				{
					FrigatesKilled = true;
					SkillsPassed = true;
					fMission.mSetTimedInterval( kTimerCountdownSet, 0 );
					fMission.mDisplayMessage( kStartComsim_msg );
					mCreateMissionShips( 1 );
					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					fMission.mAddToBonusPrestige( kTechnicalBonus );	// one-time prestige bonus for finishing aptitude
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kFrigateTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
			}

			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 5000 );
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( SkillsPassed == true )
	{
		TechnicalSuccess = EnemyKilled + 1;

		switch( EnemyKilled )
		{
			case 0:
				return( kDefeat );

			case 1:
				return( kDraw );
			
			case 2:
				return( kVictory );

			default:
				return( kAstoundingVictory );
		}
	}

	TechnicalSuccess = 0;
	return( kDevastatingDefeat );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if(( OldTechnicalSuccess == 0 ) && ( TechnicalSuccess > 0 ))
	{
		fMission.mAddToDebriefingList( kAptitudeSuccess_msg );
	}

	if( SkillsPassed == false )
	{
		fMission.mAddToDebriefingList( kAptitudeFailure_msg );
	}

	if( SkillsPassed == true )
	{
		switch( EnemyKilled )
		{
			case 0:
				fMission.mAddToDebriefingList( kKilled0_msg );
				break;

			case 1:
				fMission.mAddToDebriefingList( kKilled1_msg );
				break;

			case 2:
				fMission.mAddToDebriefingList( kKilled2_msg );
				break;

			default:
				fMission.mAddToDebriefingList( kKilled3_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TechnicalSuccess <= OldTechnicalSuccess )
	{
		TechnicalSuccess = OldTechnicalSuccess;
	}

	if( TechnicalSuccess >= 99 )					// yep, this is going to happen
	{
		TechnicalSuccess = 99;
	}

	return TechnicalSuccess;
}
