#include "scriptinterface.h"

#include "Tec_TechnicalFrigateText.h"
#include "Tec_TechnicalFrigateMaps.h"

int32 BuoyNumber = 0;
int32 EnemyKilled = 0;
int32 TechnicalSuccess = 0;
int32 OldTechnicalSuccess = 0;
int32 BuoyX = 0;
int32 BuoyY = 0;
bool SkillsPassed = false;

const tString TechnicalShip[6] = { "F-FFG", "K-F5", "R-K4RB", "L-FFP", "H-CRU", "G-DD+" };

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kBuoyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTechnicalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	TechnicalSuccess = info.fTechnicalSuccess;
	OldTechnicalSuccess = TechnicalSuccess;

	if( TechnicalSuccess != 0 )
	{
		SkillsPassed = true;
	}

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kBuoyTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kBuoyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kBuoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will be using a Frigate for this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BuoyTeam = fMission.mGetTeamHandle( kBuoyTeam );
	tShipInfo* Ship;

	tString CurrentPlayerShip = TechnicalShip[ PlayerTeam->mGetRace() ];

	// Create a frigate for the player to use
	if( Mode == 0 )
	{
		BuoyTeam->mCreateShip( "K-LP", kStartPosition_P );
		BuoyTeam->mCreateShip( "K-LP", kStartPosition_Q );
		BuoyTeam->mCreateShip( "K-LP", kStartPosition_R );
		BuoyTeam->mCreateShip( "K-LP", kStartPosition_S );

		if( SkillsPassed == true )
		{
			PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_H );
		}
		else
		{
			PlayerTeam->mCreateShip( CurrentPlayerShip.mCharStr(), kStartPosition_G );
		}
	}

	// create an enemy frigate in a location away from the player's current position
	if( Mode == 1 )
	{
		Ship = PlayerTeam->mGetFirstShip();

		if( Ship->mGetTacticalDistance ('Y') <= 40.0 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, kClassFrigate, 20, kStartPosition_Z );
		}
		else
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, kClassFrigate, 20, kStartPosition_Y );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	if( SkillsPassed == false )
	{
		return( kBriefing_msg );
	}
	else
	{
		return( kBriefingReturn_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BuoyTeam = fMission.mGetTeamHandle( kBuoyTeam );
	tShipInfo* Ship;

	// configure the player's frigate
	BuoyTeam->mGetThisShip( 1 )->mGetGridPosition( &BuoyX, &BuoyY );
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();

	if( SkillsPassed == false )
	{
		Ship->mTrackTacticalDistance( 5.0, BuoyX, BuoyY );			// set the destination for the first buoy
		fMission.mSetGoalIndicator( BuoyTeam->mGetThisShip( 1 ));
	}

	BuoyTeam->mGetThisShip(1)->mChangeName( kBuoy1_name );
	BuoyTeam->mGetThisShip(2)->mChangeName( kBuoy2_name );
	BuoyTeam->mGetThisShip(3)->mChangeName( kBuoy3_name );
	BuoyTeam->mGetThisShip(4)->mChangeName( kBuoy4_name );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BuoyTeam = fMission.mGetTeamHandle( kBuoyTeam );

	switch (Event)
	{
		case kNotifyAtDistance:
			switch( BuoyNumber )
			{
				case 0:									// arrived at the first buoy
					BuoyNumber++;
					BuoyTeam->mGetThisShip( 2 )->mGetGridPosition( &BuoyX, &BuoyY );
					Ship->mTrackTacticalDistance( 5.0, BuoyX, BuoyY );
					fMission.mDisplayMessage( kBuoyOne_msg );
					fMission.mSetGoalIndicator( BuoyTeam->mGetThisShip( 2 ));
					break;
				case 1:									// second buoy
					BuoyNumber++;
					BuoyTeam->mGetThisShip( 3 )->mGetGridPosition( &BuoyX, &BuoyY );
					Ship->mTrackTacticalDistance( 5.0, BuoyX, BuoyY );
					fMission.mDisplayMessage( kBuoyTwo_msg );
					fMission.mSetGoalIndicator( BuoyTeam->mGetThisShip( 3 ));
					break;
				case 2:									// third buoy
					BuoyNumber++;
					BuoyTeam->mGetThisShip( 4 )->mGetGridPosition( &BuoyX, &BuoyY );
					Ship->mTrackTacticalDistance( 5.0, BuoyX, BuoyY );
					fMission.mDisplayMessage( kBuoyThree_msg );
					fMission.mSetGoalIndicator( BuoyTeam->mGetThisShip( 4 ));
					break;
				case 3:									// last buoy
					BuoyNumber++;
					SkillsPassed = true;
					fMission.mDisplayMessage( kBuoyFour_msg );			// player succeeds
					fMission.mSetTimedInterval( kTimerCountdownSet, 0 );
					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					fMission.mSetGoalIndicator( kFalse, 0, 0 );
					fMission.mAddToBonusPrestige( kTechnicalBonus );	// one-time prestige bonus for finishing aptitude
					break;
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kBuoyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kBuoyTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

					if( SkillsPassed == false )
					{
						fMission.mDisplayMessage( kStarting_msg );		// startup message
						fMission.mSetTimedInterval( kTimerCountdownSet, 12 );
					}
					else
					{
						fMission.mDisplayMessage( kStartComsim_msg );	// player has already
						mCreateMissionShips( 1 );						// now blow stuff up
					}
					break;

				case kTimerCountdownSet:
					if( SkillsPassed == false )
					{
						fMission.mDisplayMessage( kTimeFailed_msg );
						Ship = PlayerTeam->mGetFirstShip();
						Ship->mTrackTacticalDistance( 0.0, 0, 0 );		// clear the distance notification
						Ship->mSetShipThrust( 0 );
						return kPlayerLoses;
					}
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kStartComsim_msg );
					mCreateMissionShips( 1 );
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( SkillsPassed == false )
				{
					return kPlayerLoses;
				}
				else
				{
					return kPlayerWins;
				}
			}
			else
			{
				EnemyKilled++;
				fMission.mDisplayMessage( kNewEnemy_msg );
				mCreateMissionShips( 1 );
			}
			break;

		case kNotifyOnCapture:
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
			Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( SkillsPassed == true )
	{
		TechnicalSuccess = 1 + EnemyKilled;

		switch( EnemyKilled )
		{
			case 0:
				return( kDefeat );

			case 1:
				return( kDraw );

			case 2:
				return( kVictory );

			default:
				return( kAstoundingVictory );
		}
	}

	TechnicalSuccess = 0;
	return( kDevastatingDefeat );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if(( OldTechnicalSuccess == 0 ) && ( TechnicalSuccess > 0 ))
	{
		fMission.mAddToDebriefingList( kAptitudeSuccess_msg );
	}

	if( SkillsPassed == false )
	{
		fMission.mAddToDebriefingList( kAptitudeFailure_msg );
	}

	if( SkillsPassed == true )
	{
		switch( EnemyKilled )
		{
			case 0:
				fMission.mAddToDebriefingList( kKilled0_msg );
				break;

			case 1:
				fMission.mAddToDebriefingList( kKilled1_msg );
				break;

			case 2:
				fMission.mAddToDebriefingList( kKilled2_msg );
				break;

			default:
				fMission.mAddToDebriefingList( kKilled3_msg );
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TechnicalSuccess <= OldTechnicalSuccess )
	{
		TechnicalSuccess = OldTechnicalSuccess;
	}

	if( TechnicalSuccess >= 99 )	// yep, this is going to happen
	{
		TechnicalSuccess = 99;
	}

	return TechnicalSuccess;
}
