#include "scriptinterface.h"

#include "tst_tractortestMissionText.h"
#include "tst_tractortestMissionMaps.h"


//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kTractorTeam,
	kTractorTargetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (100, 80, 50, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kTractorTeam, kFederation);
	fMission.mSetTeamRace (kTractorTargetTeam, kFederation);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kTractorTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kTractorTargetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kTractorTeam, kTractorTargetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue; // use the player's ship(s)
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TractorTeam = fMission.mGetTeamHandle (kTractorTeam);
	tTeamInfo* TractorTargetTeam = fMission.mGetTeamHandle (kTractorTargetTeam);

	// create player teammates
	//PlayerTeam->mCreateShip ("R-K7R", kStartPosition_H);

	// ship that does the tractor
	TractorTeam->mCreateShip( "F-DDG", kStartPosition_K );
	TractorTeam->mCreateShip( "F-NCL", kStartPosition_L );

	// create the target of the tractor
	TractorTargetTeam->mCreateShip ("F-FQS", kStartPosition_J);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TractorTeam = fMission.mGetTeamHandle (kTractorTeam);
	tTeamInfo* TractorTargetTeam = fMission.mGetTeamHandle (kTractorTargetTeam);

	tShipInfo* Ship;
	
	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	//PlayerTeam->mGetThisShip(2)->mDamageSystem( kDamageCloakSystem, 100 );
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents( kNotifyChangingShip );
	}


	// configure Tractor ships
	for (Ship = TractorTeam->mGetFirstShip(); Ship != NULL; Ship = TractorTeam->mGetNextShip())
	{
		//Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
		//Ship->mSetShipTarget( TractorTargetTeam->mGetFirstShip(), kVeryHighPriority );
	}
	//TractorTeam->mGetFirstShip()->mTractorEntity( TractorTargetTeam->mGetFirstShip(), 40, 40, kVeryHighPriority );

	// configure target
	for (Ship = TractorTargetTeam->mGetFirstShip(); Ship != NULL; Ship = TractorTargetTeam->mGetNextShip())
	{
		//Ship->mDamageSystem( kDamageLeftWarpEngine, 100 );
		//Ship->mDamageSystem( kDamageRightWarpEngine, 100 );
		

		Ship->mAccessSupplies (kSetTransporterItemCount, kTransGornEgg, 1);
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* TractorTeam = fMission.mGetTeamHandle (kTractorTeam);
	tTeamInfo* TractorTargetTeam = fMission.mGetTeamHandle (kTractorTargetTeam);


	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);


	switch (Event)
	{
		case kNotifyChangingShip:
			fMission.mDisplayMessage( kLNFChangingShip_msg, kMissionScriptMessage );
			break;

		case kNotifyBeginTractor:
			fMission.mDisplayMessage( kTractoringShip_msg, kMissionScriptMessage );
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					fMission.mSetTeamRelations (kPlayerTeam, kTractorTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kTractorTargetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kTractorTeam, kTractorTargetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					//TractorTeam->mGetFirstShip()->mClearCommands();
					//TractorTeam->mGetFirstShip()->mDefendEntity( TractorTargetTeam->mGetFirstShip(), 10.0, 10.0, kAbsolutePriority );

					//TractorTeam->mGetFirstShip()->mTractorEntity( TractorTargetTeam->mGetFirstShip(), 40, 40, kAbsolutePriority );

					break;

				case kTimerCountdownSet:
					break;

				case kTimerTurnsSet_10:
					break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kButtonHail_text:
					//TractorTargetTeam->mGetFirstShip()->mAccessSupplies (kSetTransporterItemCount, kTransGornEgg, 0);
					fPlayerShip->mDamageSystem( kDamageInternalsWithPoints, 1.0f );
					break;

				case kButtonSurrender_text:
					break;

				case kButtonTaunt_text:
					break;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


