#include "scriptinterface.h"

#include "Tut_Fed_1MissionText.h"
#include "Tut_Fed_1MissionMaps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 EnemyDestroyed = 0;
bool TutorialPassed = false;
bool WaitForSpeed = false;

const char* kA4_SPEED_BAR	=	"A4_SPEED_BAR";
const char* lastBlinkingButton	=	NULL;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a frigate for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip ("F-FF", kStartPosition_G);
	}

	if (Mode == 1)
	{
		Ship = PlayerTeam->mGetFirstShip ();
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("F-FSPh", kStartPosition_Specified, x+2, y);
		EnemyTeam->mCreateShip ("F-FSPh", kStartPosition_Specified, x, y+2);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	// configure the player's frigate
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust (0);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 4 );
	Ship->mChangeName( kNeversail_name );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 45 );
	Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 100 );
	Ship->mDamageSystem( kDamageMaxShields, 100 );
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch( Event )
	{
		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );

				if( WaitForSpeed == true )
				{
					WaitForSpeed = false;
				}
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch( fEventID )
				{
					case kTimerLimitSet:
						{
							switch( TutorialSegment )
							{
								case 0:
									fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);				// startup message
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 1:
									fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 2:
									fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 3:
									fMission.mDisplayMessage (kSulu_Welcome_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluWelcome);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 4:
									fMission.mDisplayMessage (kSulu_IHaveBeenAsked_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluIHaveBeenAsked);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
									
								case 5:
									fMission.mDisplayMessage (kSulu_QuickReview_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluQuickReview);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 6:
									fMission.mDisplayMessage (kSulu_ToggleD_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluToggleD);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 7:
									fMission.mDisplayMessage (kSulu_BeautySpace_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluBeautySpace);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;					
								case 8:
									fMission.mDisplayMessage (kSulu_SpaceDangerous_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluSpaceDangerous);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 9:
									fMission.mDisplayMessage (kSulu_RedAlert_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluRedAlert);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 10: // ** blinking speed bar **
									fMission.mDisplayMessage (kSulu_Speed_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluSpeed);
									fMission.mStartFlashingButton( kA4_SPEED_BAR );
									WaitForSpeed = true;
									fMission.mSetTimedInterval( kTimerLimitSet_3, 20 );
									break;
								case 11:
									fMission.mDisplayMessage (kSulu_Turn_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluTurn);
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
									break;
								case 12:
									fMission.mDisplayMessage (kSulu_Freighters_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluFreighters);
									mCreateMissionShips ( 1 );
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 13:
									fMission.mDisplayMessage (kSulu_TKey_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluTKey);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 14:
									fMission.mDisplayMessage (kSulu_Good_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluGood);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 15:
									fMission.mDisplayMessage (kSulu_Fire_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluFire);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 16:
									fMission.mDisplayMessage (kSulu_Destroy_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSuluDestroy);
									break;
								default:
									return kPlayerWins;
							}
						}
						break;

					case kTimerLimitSet_2:	// Loop until voice is done playing...
						if( fMission.mVoiceSampleStillPlaying() != 0 )
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
						else
						{
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
						}
						break;

					case kTimerLimitSet_3:
						if( WaitForSpeed == true )
						{
							fMission.mStopFlashingButton( kA4_SPEED_BAR );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;

				if( EnemyDestroyed == 1 )
				{
					fMission.mDisplayMessage (kSulu_OneMore_msg, kScienceScriptMessage);
					fMission.mPlayVoiceSample (kSndSuluOneMore);
				}
				else
				{
					fMission.mDisplayMessage (kSulu_Excellent_msg, kScienceScriptMessage);
					fMission.mPlayVoiceSample (kSndSuluExcellent);
					fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
				}
			}
			
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 50);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
	//return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}

	fMission.mAddToDebriefingList (kDebrief_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
