#include "scriptinterface.h"

#include "Tut_Fed_2MissionText.h"
#include "Tut_Fed_2MissionMaps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 EnemyDestroyed = 0;
int32 MissionMode = 0;
bool TutorialPassed = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 1;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a frigate for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip( "F-FFG", kStartPosition_G );
	}

	if (Mode == 1) // ENF tune the exact freighter strengths
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("F-FSPh", kStartPosition_Specified, x+2, y);
		MissionMode = 1;
	}

	if (Mode == 2)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("F-FSPh", kStartPosition_Specified, x+2, y);
		MissionMode = 2;
	}

	if (Mode == 3)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("F-FSPh", kStartPosition_Specified, x+2, y);
		MissionMode = 3;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	// configure the player's frigate
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust (0);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
	Ship->mChangeName( kNeversail_name );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 50 );
	Ship->mDamageSystem( kDamageMaxWeapons, 0);
	Ship->mDamageSystem(kDamageShuttleBaySystem, 100);
	Ship->mDamageSystem( kDamageMaxShields, 100 );

	if ( MissionMode == 1 )
	{
		Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 100 );
		
	}
	else if ( MissionMode == 2 )
	{
		Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 75 );
		
	}
	else if ( MissionMode == 3 )
	{
		Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 60 );
		
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch (Event)
	{
		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch( fEventID )
				{
					case kTimerLimitSet:
						switch( TutorialSegment )
						{
							case 0:
								fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;
							case 1:
								fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;
							case 2:
								fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;
							case 3:
								fMission.mDisplayMessage (kSulu2_Anxious_msg, kScienceScriptMessage);
								fMission.mDisplayMessage( kGoToRedAlert_msg, kNonCommScriptMessage );
								fMission.mPlayVoiceSample (kSndSulu2_Anxious);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 4:
								fMission.mDisplayMessage (kSulu2_CrippledPhasers_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_CrippledPhasers);
								fPlayerShip->mDamageSystem( kDamageHeavyWeapon7, 100.0 );
								fPlayerShip->mDamageSystem( kDamageWeapon11, 100.0 );
								fPlayerShip->mDamageSystem( kDamageWeapon12, 100.0 );
								fPlayerShip->mDamageSystem( kDamageWeapon13, 100.0 );
								fPlayerShip->mDamageSystem( kDamageWeapon14, 100.0 );
								fPlayerShip->mDamageSystem( kDamageWeapon15, 100.0 );
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
								break;
								
							case 5:
								fMission.mDisplayMessage (kSulu2_LearnPhoton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_LearnPhoton);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 6:
								fMission.mDisplayMessage (kSulu2_PhotonPowerful_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_PhotonPowerful);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 7:
								fMission.mDisplayMessage (kSulu2_PhotonDrawback_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_PhotonDrawback);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;					
							case 8: // **blinking weapon 1**
								fMission.mDisplayMessage (kSulu2_ClickOnPhotonBlinking_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "PlayerSchem.Weapon1" );
								fMission.mPlayVoiceSample (kSndSulu2_ClickOnPhotonBlinking);
								break;
							case 9:
								fMission.mDisplayMessage (kSulu2_PieChart_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_PieChart);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 10: // **blinking All Weapons**
								fMission.mDisplayMessage (kSulu2_ClickAllWeapons_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "A5_WEAPONS_SELECT" );
								fMission.mPlayVoiceSample (kSndSulu2_ClickAllWeapons);
								break;
							case 11: // **First Freighter Created here**
								fMission.mDisplayMessage (kSulu2_DestroyFreighterPhoton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_DestroyFreighterPhoton);
								mCreateMissionShips ( 1 );
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 12:
								fMission.mDisplayMessage (kSulu2_ManeuverFrigate_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_ManeuverFrigate);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 13:
								fMission.mDisplayMessage (kSulu2_GainFeel_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_GainFeel);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 14: // **wait for FIRST FREIGHTER**
								fMission.mDisplayMessage (kSulu2_DistanceDisplay1_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_DistanceDisplay1);
								break;
							case 15:
								// We should get into here from the destruction of a freighter.
								fMission.mDisplayMessage (kSulu2_DireCircum_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_DireCircum);
								
								// Restore their photon torpedoes if they were damaged.
								fPlayerShip->mDamageSystem( kDamageHeavyWeapon1, 0.0 );
								fPlayerShip->mDamageSystem( kDamageHeavyWeapon2, 0.0 );
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 16: // **blinking Weapon 1**
								fMission.mDisplayMessage (kSulu2_ClickOnPhotonBlinking_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "PlayerSchem.Weapon1" );
								fMission.mPlayVoiceSample (kSndSulu2_ClickOnPhotonBlinking);
								break;

							case 17: // **blinking Overloaded**
								fMission.mDisplayMessage (kSulu2_ThatIsOverloaded_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "A6_PHOTON_OVERLOAD" );
								fMission.mPlayVoiceSample (kSndSulu2_ThatIsOverloaded);
								break;
							case 18:
								fMission.mDisplayMessage (kSulu2_OverloadedOnlyShort_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_OverloadedOnlyShort);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 19:
								fMission.mDisplayMessage (kSulu2_OverloadedCostMoreEnergy_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_OverloadedCostMoreEnergy);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 20: // **blinking All Weapons**
								fMission.mDisplayMessage (kSulu2_ClickAllWeapons_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "A5_WEAPONS_SELECT" );
								fMission.mPlayVoiceSample (kSndSulu2_ClickAllWeapons);
								break;
							case 21: // ** SECOND FREIGHTER**
								fMission.mDisplayMessage (kSulu2_DestroyFreighterWithPhoton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_DestroyFreighterWithPhoton);
								mCreateMissionShips ( 2 );
								// The player will need to destroy the frieghter to advance the script.
								break;
							case 22: // **blinking All Weapons**
								fMission.mDisplayMessage (kSulu2_ClickAllWeapons_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "A5_WEAPONS_SELECT" );
								fMission.mPlayVoiceSample (kSndSulu2_ClickAllWeapons);
								break;
							case 23:
								fMission.mDisplayMessage (kSulu2_HoldCtrlKey_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_HoldCtrlKey);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 24:
								fMission.mDisplayMessage (kSulu2_SetAllToWeapon1_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_SetAllToWeapon1);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 25: // **blinking Weapons 1**
								fMission.mDisplayMessage (kSulu2_ClickOnPhotonBlinking_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "PlayerSchem.Weapon1" );
								fMission.mPlayVoiceSample (kSndSulu2_ClickOnPhotonBlinking);
								break;
							case 26: // **blinking Weapons 2**
								fMission.mDisplayMessage (kSulu2_ShiftClick_msg, kScienceScriptMessage);
								fMission.mStartFlashingButton( "PlayerSchem.Weapon2" );
								fMission.mPlayVoiceSample (kSndSulu2_ShiftClick);
								break;
							case 27:
								fMission.mDisplayMessage (kSulu2_SelectedTwoPhoton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_SelectedTwoPhoton);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 28:
								fMission.mDisplayMessage (kSulu2_AgainCtrlKey2_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_AgainCtrlKey2);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
								break;
							case 29:
								fMission.mDisplayMessage (kSulu2_WeaponGroup2_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_WeaponGroup2);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
								break;
							case 30:
								fMission.mDisplayMessage (kSulu2_PressBackAndForth1n2_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_PressBackAndForth1n2);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
								break;
							case 31:
								fMission.mDisplayMessage (kSulu2_NoticeSelection_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_NoticeSelection);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
								break;
							case 32:
								fMission.mDisplayMessage (kSulu2_FirePhotonsTap2_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_FirePhotonsTap2);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								break;
							case 33:
								fMission.mDisplayMessage (kSulu2_GoAheadDestroy_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_GoAheadDestroy);
								fPlayerShip->mDamageSystem( kDamageWeaponsAndSystems, 0 );
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 34:
								// The player will need to destroy the frieghter to advance the script.
								fMission.mDisplayMessage (kSulu2_BasicTactic_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu2_BasicTactic);
								mCreateMissionShips ( 3 );
								break;

							default:
								return kPlayerWins;
						}
						break;

					case kTimerLimitSet_2:	// Loop until voice is done playing...
						if( fMission.mVoiceSampleStillPlaying() != 0 )
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
						else
						{
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;

				switch( EnemyDestroyed )
				{
					case 1:
					{
						fMission.mDisplayMessage( kSulu2_WellDone_msg, kScienceScriptMessage );
						fMission.mPlayVoiceSample( kSndSulu2_WellDone );
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						break;
					}
					case 2:
					{
						fMission.mDisplayMessage (kSulu2_FineJob_msg, kScienceScriptMessage);
						fMission.mPlayVoiceSample (kSndSulu2_FineJob);
						fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
						break;
					}
					case 3:
					{
						fMission.mDisplayMessage (kSulu2_ExcellentCommander_msg, kScienceScriptMessage);
						fMission.mPlayVoiceSample (kSndSulu2_ExcellentCommander);
						fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
						break;
					}
				}
			}

			if(Ship->mGetTeam() == kPlayerTeam)
			{
				fMission.mDisplayMessage (kSulu2_Sigh_msg, kScienceScriptMessage);
				fMission.mPlayVoiceSample (kSndSulu2_Sigh);
				return kPlayerLoses;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 35);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
//	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kDebrief_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}

	//fMission.mAddToDebriefingList (kDebrief_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
