#include "scriptinterface.h"

#include "Tut_Fed_3MissionText.h"
#include "Tut_Fed_3MissionMaps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 EnemyDestroyed = 0;
int32 Mode = 0;
bool TutorialPassed = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kBaseTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 2;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a ship with mines for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip ("F-CA", kStartPosition_G);
	}

	if (Mode == 1)
	{
		Ship = PlayerTeam->mGetFirstShip ();
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("F-FSPh", kStartPosition_Specified, x+3, y);
	}

	if (Mode == 2)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, x+1, y);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, x-1, y);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, x-1, y-1);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, x-2, y-1);
		BaseTeam->mCreateShip ("F-SBR", kStartPosition_Specified, x-6, y-6);
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* Ship;

	// configure the player's frigate
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust (8);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, 20 );
	Ship->mChangeName( kNeversail_name );
	Ship->mSetOfficerRank( kHelmOfficer, kVeteran );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch ( Mode )
	{
		case 1:
			Ship->mDamageSystem( kDamageWeaponsAndSystems, 80 );
			Ship->mSetShipMaxSpeed( 6 );
			Ship->mDamageSystem(kDamageShuttleBaySystem, 100);
			fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
			Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
			Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
			Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
			Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
			Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
			Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
			break;

		case 2:
			Ship->mSetShipMaxSpeed( 12 );
			Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
			Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
			Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
			break;
	}

}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			{
				switch( fEventID )
				{
					case kTimerLimitSet:
						switch( TutorialSegment )
							{
								case 0:
									fMission.mDisplayMessage( kWhere_msg, kNonCommScriptMessage );
									TutorialSegment++;
									fMission.mSetTimedInterval( kTimerLimitSet, 1 );
									break;
								case 1:
									fMission.mDisplayMessage( kWhere2_msg, kNonCommScriptMessage );
									TutorialSegment++;
									fMission.mSetTimedInterval( kTimerLimitSet, 1 );
									break;
								case 2:
									fMission.mDisplayMessage( kWhere3_msg, kNonCommScriptMessage );
									TutorialSegment++;
									fMission.mSetTimedInterval( kTimerLimitSet, 1 );
									break;
								case 3:
									fMission.mDisplayMessage( kSulu3_DiscussMines_msg, kScienceScriptMessage );
									fMission.mPlayVoiceSample( kSndSulu3_DiscussMines );
									fMission.mDisplayMessage( kSulu3_SetRedAlert_msg, kNonCommScriptMessage );
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
									break;
								case 4: // **blinking Weapons Officer button**
									fMission.mDisplayMessage( kSulu3_OfficerPanel_msg, kScienceScriptMessage );
									fMission.mPlayVoiceSample( kSndSulu3_OfficerPanel );
									fMission.mStartFlashingButton( "A3_OFFICER_WEAPONS" );
									break;							
								case 5: // **blinking mine button**
									fMission.mDisplayMessage( kSulu3_MineDeploy_msg, kScienceScriptMessage );
									fMission.mPlayVoiceSample( kSndSulu3_MineDeploy );
									fMission.mStartFlashingButton( "A3_WEAPONS_LAYMINE" );
									break;
								case 6:
									fMission.mDisplayMessage( kSulu3_GreenRings1_msg, kScienceScriptMessage );
									fMission.mPlayVoiceSample( kSndSulu3_GreenRings1 );
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
									break;
								case 7:
									fMission.mDisplayMessage( kSulu3_GreenRings2_msg, kScienceScriptMessage );
									fMission.mPlayVoiceSample( kSndSulu3_GreenRings2 );
									fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
									break;
								case 8:
									fMission.mDisplayMessage (kSulu3_PlacedInSpace_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_PlacedInSpace);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;					
								case 9: // **blinking Transporter button**
									fMission.mDisplayMessage (kSulu3_TransporterButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_TransporterButton);
									fMission.mStartFlashingButton( "PlayerSchem.Transporter" );
									break;
								case 10: // **Blinking Transporter Mine button**
									fMission.mDisplayMessage (kSulu3_ActivateMine1_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_ActivateMine1);
									fMission.mStartFlashingButton( "TRANSPORT_BOMB" );
									break;
								case 11:
									fMission.mDisplayMessage (kSulu3_ActivateMine2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_ActivateMine2);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 12:
									fMission.mDisplayMessage (kSulu3_NoticeWeaponOfficer_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_NoticeWeaponOfficer);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 13:
									fMission.mDisplayMessage (kSulu3_CrippledFrigate_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_CrippledFrigate);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 14: // player won't progress until ships are dead
									fMission.mDisplayMessage (kSulu3_PhasersPhotonsCrippled_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_PhasersPhotonsCrippled);
									// Create first wave of ships
									Mode = 1;
									mCreateMissionShips ( Mode );
									fPlayerShip->mDamageSystem(kDamageMaxWeapons, 100);
									break;
								case 15:
									fMission.mDisplayMessage (kSulu3_WellDoneAdvancedHelm_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_WellDoneAdvancedHelm);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 16:
									fMission.mDisplayMessage (kSulu3_Speed15_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_Speed15);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 17: // **blinking Helm Officer button**
									fMission.mDisplayMessage (kSulu3_HelmOfficer_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_HelmOfficer);
									fMission.mStartFlashingButton( "A3_OFFICER_HELM" );
									break;
								case 18: // **blinking HET reverse button**
									fMission.mDisplayMessage (kSulu3_HETButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_HETButton);
									fMission.mStartFlashingButton( "A3_HELM_180_HET" );
									break;
								case 19:
									fMission.mDisplayMessage (kSulu3_ManeuverPowerful_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_ManeuverPowerful);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 20:
									fMission.mDisplayMessage (kSulu3_SimulatedRogue_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_SimulatedRogue);
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 21: // Final wave of ships
									fMission.mDisplayMessage (kSulu3_LureFrigates_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_LureFrigates);
									Mode = 2;
									mCreateMissionShips ( Mode );
									fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
									break;
								case 22:
									fMission.mDisplayMessage (kSulu3_UseHET_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndSulu3_UseHET);
									TutorialSegment++;
									break;

								default:
									return kPlayerWins;
							}
							break;

					case kTimerLimitSet_2:	// Loop until voice is done playing...
						if( fMission.mVoiceSampleStillPlaying() != 0 )
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
						else
						{
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
						}
						break;
				}
				break;

			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;

				if( Mode == 1 )
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				else
				{
					if( EnemyTeam->mGetShipsInPlayCount() == 0 )
					{
						fMission.mDisplayMessage( kSulu3_StarbasesAreImpressive_msg, kScienceScriptMessage );
						fMission.mPlayVoiceSample( kSndSulu3_StarbasesAreImpressive );
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
				}
			}
			
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				fMission.mDisplayMessage( kSulu3_Sigh_msg, kScienceScriptMessage );
				fMission.mPlayVoiceSample( kSndSulu3_Sigh );
				return kPlayerLoses;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 20);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
	// return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}

	//LNF should I add a debrief?
	//fMission.mAddToDebriefingList (kSulu3_IAmImpressed_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
