#include "scriptinterface.h"

#include "Tut_Fed_4MissionText.h"
#include "Tut_Fed_4MissionMaps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 EnemyDestroyed = 0;
int32 NumberToKill = 4;
int32 Mode = 0;
bool TutorialPassed = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 3;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a frigate for the player to use
	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( "F-FFG", kStartPosition_G );
	}

	if( Mode == 1 )
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition( &x, &y );

		for( int j = 1; j < NumberToKill; j++ )
		{
			EnemyTeam->mCreateShip (kMaxBPV, kClassFreighter, kClassFreighter, 100, kStartPosition_Specified, x+j+1, y+j+1);
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	// configure the player's Ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled ();
	Ship->mSetShipThrust (0);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 4 );
	Ship->mChangeName( kNeversail_name );
	Ship->mSetOfficerRank( kWeaponsOfficer, kLegendary );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (Mode > 0)
	{
		Ship->mDamageSystem( kDamageWeaponsAndSystems, 50 );
		Ship->mDamageSystem( kDamageShuttleBaySystem, 100.0 );
		Ship->mDamageSystem( kDamageMaxShields, 50.0 );
		Ship->mDamageSystem( kDamageExcessDamageSystem, 50.0 );
		Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
		Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
		Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
		Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
		Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
		Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch (Event)
	{
		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch( fEventID )
				{
					case kTimerLimitSet:
						switch (TutorialSegment)
						{
							case 0:
								fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);				// startup message
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;
							case 1:
								fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;
							case 2:
								fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;
							case 3:
								fMission.mDisplayMessage( kSulu4_ManyDefenses_msg, kScienceScriptMessage );
								fMission.mPlayVoiceSample( kSndSulu4_ManyDefenses );
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );

								if( fPlayerShip->mGetAlertStatus() != kAlertRed )
								{
									fMission.mDisplayMessage( kGoToRedAlert_msg, kNonCommScriptMessage );
								}
								break;
							case 4: // **blinking EW**
								fMission.mDisplayMessage (kSulu4_EWButton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_EWButton);
								fMission.mStartFlashingButton( "PlayerSchem.Sensors" );
								break;
								
							case 5:
								fMission.mDisplayMessage (kSulu4_StarshipEW_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_StarshipEW);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 6:
								fMission.mDisplayMessage (kSulu4_UseECM_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_UseECM);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 7:
								fMission.mDisplayMessage (kSulu4_ECCM_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ECCM);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;					
							case 8:
								fMission.mDisplayMessage (kSulu4_ECCMSlider_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ECCMSlider);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								break;
							case 9:
								fMission.mDisplayMessage (kSulu4_CurrentECM_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_CurrentECM);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								break;
							case 10:
								fMission.mDisplayMessage (kSulu4_Speed10_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_Speed10);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 11:
								fMission.mDisplayMessage (kSulu4_Shields_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_Shields);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 12: // **blinking Shield button**
								fMission.mDisplayMessage (kSulu4_ShieldButton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ShieldButton);
								fMission.mStartFlashingButton( "A5_SHIELD_SELECTER" );
								break;
							case 13: // **blinking Center Shield**
								fMission.mDisplayMessage (kSulu4_CenterShield_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_CenterShield);
								fMission.mStartFlashingButton( "A6_ALL_SHIELDS_SELECT" );
								break;
							case 14:
								fMission.mDisplayMessage (kSulu4_Reinforce_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_Reinforce);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 15: // **blinking Science Officer**
								fMission.mDisplayMessage (kSulu4_ScienceButton_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ScienceButton);
								fMission.mStartFlashingButton( "A3_OFFICER_SCIENCE" );
								break;
							case 16:
								fMission.mDisplayMessage (kSulu4_Science_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_Science);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 17: // **blinking Helm Officer**
								fMission.mDisplayMessage (kSulu4_FinallyClick_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_FinallyClick);
								fMission.mStartFlashingButton( "A3_OFFICER_HELM" );
								break;
							case 18: // **blinking Erratic Maneuvers**
								fMission.mDisplayMessage (kSulu4_ClickOnErratic1_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ClickOnErratic1);
								fMission.mStartFlashingButton( "A3_HELM_ERRATIC" );
								break;
							case 19:
								fMission.mDisplayMessage (kSulu4_ClickOnErratic2_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ClickOnErratic2);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 20:
								fMission.mDisplayMessage (kSulu4_BetweenEW_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_BetweenEW);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 8);
								break;
							case 21:
								fMission.mDisplayMessage (kSulu4_ScottySays_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_ScottySays);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 22:
								fMission.mDisplayMessage (kSulu4_QShips_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_QShips);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;
							case 23:
								Mode = 1;	
								mCreateMissionShips ( 1 );
								fMission.mDisplayMessage (kSulu4_Experiment_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndSulu4_Experiment);
								TutorialSegment++;
								break;
							default:
								return kPlayerWins;
						}
						break;

					case kTimerLimitSet_2:	// Loop until voice is done playing...
						if( fMission.mVoiceSampleStillPlaying() != 0 )
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
						}
						else
						{
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;

				if( EnemyDestroyed == 3 )
				{
					fMission.mDisplayMessage( kSulu4_StrongImprovement_msg, kScienceScriptMessage );
					fMission.mPlayVoiceSample( kSndSulu4_StrongImprovement );
					fMission.mSetTimedInterval( kTimerLimitSet, 4 );
				}
			}
			else if( Ship->mGetTeam() == kPlayerTeam )
			{
				fMission.mDisplayMessage( kSulu4_Sigh_msg, kScienceScriptMessage );
				return kPlayerLoses;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 35);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
//	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
