#include "scriptinterface.h"

#include "Tut_Fed_5MissionText.h"
#include "Tut_Fed_5MissionMaps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool BadStuff = false;
bool EnemyCaptured = false;
bool TutorialPassed = false;
bool marinesSent = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID( void )
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 4;							// fifth choice in the menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart( tDynaverseScriptInfo& info )
{
	fMission.mSetPrestigePoints( 0, 0, 0, 0, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kKlingon );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip( tShipInGame& Ship )
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips( int32 Mode )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	PlayerTeam->mCreateShip( "F-NCT", kStartPosition_G );
	AlliedTeam->mCreateShip( "F-BS", kStartPosition_H );
	EnemyTeam->mCreateShip( "K-E4", kStartPosition_Z );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished( void )
{
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission( void )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tShipInfo* Ship;

	// configure Player Ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	Ship->mAccessSupplies (kSetSupplyCount, kSupplySpareParts, 0);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplySuicides, 2);
	Ship->mChangeName( kPlayerShipName_msg );
	Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );

	// configure Enemy Ship
	Ship = EnemyTeam->mGetFirstShip();
	Ship->mChangeName( kEnemyShipName_msg );
	Ship->mSetFinalDestination();
	Ship->mSetShipMaxSpeed( 9 );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mDamageSystem( kDamageMaxWeapons, 75.0 );
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );

	AlliedTeam->mGetFirstShip()->mDamageSystem( kDamageMaxWeapons, 100 );
	AlliedTeam->mGetFirstShip()->mAccessSupplies (kSetSupplyCount, kSupplySpareParts, 0);

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip( tShipInfo* Ship )
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent( tShipInfo* Ship, iNotifyEvents Event )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	int32 BaseX = 0;
	int32 BaseY = 0;

	int32 WeaponsRemaining = 0;

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
				{
					switch( TutorialSegment )
					{
						case 0:		// mission start
							fMission.mDisplayMessage( kWhere_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kWhere2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kWhere3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kSuluT5Welcome_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5Welcome);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 4:
							fMission.mDisplayMessage( kSuluT5Object_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5Object);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 5:
							fMission.mDisplayMessage( kSuluT5Chase_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5Chase);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 6:
							fMission.mDisplayMessage( kSuluT5Security_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5Security);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 7:
							fMission.mDisplayMessage( kSuluT5HitAndRunRaids_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5HitAndRunRaids);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 8:		// last message until all weapons on the enemy ship are destroyed
							fMission.mDisplayMessage( kSuluT5DoHitAndRun_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5DoHitAndRun);
							break;

						case 9:		// all enemy ship weapons have been destroyed
							fMission.mDisplayMessage( kSuluT5Capture_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5Capture);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 10:
							if( EnemyCaptured == false )
							{
								fMission.mDisplayMessage( kSuluT5CaptureProblems_msg, kMissionScriptMessage );
								fMission.mPlayVoiceSample (kSndSuluT5CaptureProblems);
							}
							break;

						case 11:
							fMission.mDisplayMessage( kSuluT5SuicideShuttle_msg, kMissionScriptMessage );
							Ship = PlayerTeam->mGetFirstShip();
							Ship->mDamageSystem( kDamageShuttleBaySystem, 0.0 );
							Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 4 );
							fMission.mPlayVoiceSample (kSndSuluT5SuicideShuttle);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 12:
							fMission.mDisplayMessage( kSuluT5SuicideLaunch_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5SuicideLaunch);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 13:
							fMission.mDisplayMessage( kSuluT5RepeatSuicide_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndSuluT5RepeatSuicide);
							PlayerTeam->mGetFirstShip()->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 4 );
							fMission.mSetTimedInterval( kTimerLimitSet_4, 10 );
							break;
					}
				}
				break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;

			case kTimerLimitSet_3:
				// keep the AI ship's shield down
				Ship = EnemyTeam->mGetFirstShip();
				
				if( Ship != NULL )
				{
					Ship->mDamageSystem( kDamageMaxShields, 100.0 );
					fMission.mSetTimedInterval ( kTimerLimitSet_3, 3 );
				}
				break;

			case kTimerLimitSet_4:
				Ship = PlayerTeam->mGetFirstShip();

				if( Ship != NULL )
				{
					int32 SuicideCount = Ship->mAccessSupplies( kGetSupplyCount, kSupplySuicides );
	
					if( SuicideCount == 0 )
					{
						fMission.mDisplayMessage( kHaveMoreShuttles_msg, kNonCommScriptMessage );
						Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 2 );
					}

					fMission.mSetTimedInterval( kTimerLimitSet_4, 3 );
				}
				break;

			case kTimerLimitSet_5:
				fMission.mDisplayMessage( kSuluT5GoodWork_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndSuluT5GoodWork);
				return kPlayerWins;
			}
			break;

		case kNotifyBeginTractor:
			if( PlayersTarget->mGetTeam() == kAlliedTeam )
			{
				fMission.mDisplayMessage( kBaseWhy_msg, kMissionScriptMessage );
				fPlayerShip->mSetShipCallbackEvents( kNotifyBeginTractor );
			}
			else
			{
				fMission.mDisplayMessage( kSuluT5WeaponsDisabled_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndSuluT5WeaponsDisabled);

				Ship = PlayerTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );
				Ship->mDamageSystem( kDamageTransporterSystem, 0.0 );

				Ship = EnemyTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageMaxShields, 100.0 );
				fMission.mSetTimedInterval ( kTimerLimitSet_3, 2 );  // keep AI shields down
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyDamage:
			{
				Ship = EnemyTeam->mGetFirstShip();
				WeaponsRemaining = Ship->mAccessSupplies( kGetHeavyWeaponCount, kDamageMaxWeapons );
				// if the weapon is destroyed move on in the tutorial
				if (( WeaponsRemaining == 0 ) && ( marinesSent == false ))
				{
					TutorialSegment++;
					fMission.mSetTimedInterval ( kTimerLimitSet, 2);
					fMission.mDisplayMessage( kSuluT5Excellent_msg, kScienceScriptMessage );
					marinesSent = true;
				}
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				Ship->mClearCommands();
				fMission.mDisplayMessage( kSuluT5CaptureComplete_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndSuluT5CaptureComplete);
				fMission.mSetTimedInterval( kTimerLimitSet_5, 8 );
				EnemyCaptured = true;
			}
			else
			{
				BadStuff = true;
				return kPlayerLoses;
			}
			break;

		case kNotifyAtDeath:
			if(( Ship->mGetOriginalTeam() == kEnemyTeam ) && ( EnemyCaptured == true ))
			{
				fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				fMission.mDisplayMessage( kSuluT5GoodWork_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndSuluT5GoodWork);
				return kPlayerWins;
			}
			else
			{
				BadStuff = true;
				return kPlayerLoses;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus( void )
{
	return( kAstoundingVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if(( PlayerTeam->mGetShipsInPlayCount() == 0 ) || ( BadStuff == true ))
	{
		fMission.mAddToDebriefingList( kBadDebrief_msg );
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList( kGoodDebrief_msg );
	}

	return;
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel(tDynaverseScriptInfo& infovoid )
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
