#include "scriptinterface.h"

#include "Tut_Fed_6MissionText.h"
#include "Tut_Fed_6MissionMaps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool SuluAlive = true;
bool TutorialPassed = false;

enum eMode
{
	kStarting = 0,
	kSkirmish,
	kKlingonAttack,
	kWinner,
};

eMode Mode;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kSuluTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 5;					// sixth choice in the menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, kFederation);
	fMission.mSetTeamRace (kEnemyTeam, kKlingon);
	fMission.mSetTeamRace (kSuluTeam, kFederation);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kSuluTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kSuluTeam, kTeamAtWar, kGoalDestroy, 1.25, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* SuluTeam = fMission.mGetTeamHandle (kSuluTeam);
	tShipInfo* Ship;

	int32 x, y;

	if (Mode == kStarting)
	{
		PlayerTeam->mCreateShip( "F-CC+", kStartPosition_G );
		SuluTeam->mCreateShip( "F-BCJ", kStartPosition_S );
	}

	if (Mode == kKlingonAttack)
	{
		Ship = PlayerTeam->mGetFirstShip ();
		Ship->mGetGridPosition( &x, &y );
		EnemyTeam->mCreateShip( "K-D7", kStartPosition_Specified, x+3, y+3 );
		EnemyTeam->mCreateShip( "K-D7C", kStartPosition_Specified, x+3, y+3 );
		EnemyTeam->mCreateShip( "K-E4Y", kStartPosition_Specified, x-5, y-5 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* SuluTeam = fMission.mGetTeamHandle (kSuluTeam);
	tShipInfo* Ship;

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kKongo_msg );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 4 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 2 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 2 );
	Ship->mSetShipCallbackEvents( kNotifyBeginTractor );

	Ship = SuluTeam->mGetFirstShip();
	Ship->mChangeName( kExcelsior_msg );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyWildWeasel, 0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyShuttles, 0 );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
	Ship->mDamageSystem( kDamageInternalsWithPoints, 15 );
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* SuluTeam = fMission.mGetTeamHandle (kSuluTeam);
	tShipInfo* target = fMission.mGetShipTarget( fPlayerShip );

	switch( Event )
	{
		case kNotifyBeginTractor:
			if( Mode == kStarting )
			{
				if( target == SuluTeam->mGetFirstShip() )
				{
					fMission.mDisplayMessage( kSuluT6NoFair_msg, kMissionScriptMessage );
					fMission.mPlayVoiceSample( kSndSuluT6NoFair );
					fMission.mSetTeamRelations( kPlayerTeam, kSuluTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					Mode = kSkirmish;
				}
				else
				{
					fPlayerShip->mSetShipCallbackEvents( kNotifyBeginTractor );
				}
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtTimedInterval:
		{
			switch( fEventID )
			{
				case kTimerLimitSet:
				{
					switch( TutorialSegment )
					{
						case 0:
							fMission.mDisplayMessage( kSuluT6Where_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage (kSuluT6Where2_msg, kNonCommScriptMessage);
							TutorialSegment++;
							fMission.mSetTimedInterval (kTimerLimitSet, 1);
							break;

						case 2:
							fMission.mDisplayMessage (kSuluT6Where3_msg, kNonCommScriptMessage);
							TutorialSegment++;
							fMission.mSetTimedInterval (kTimerLimitSet, 1);
							break;

						case 3:
							fMission.mDisplayMessage( kSuluT6Welcome_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndSuluT6Welcome );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 4:
							fMission.mDisplayMessage( kSuluT6Purpose_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndSuluT6Purpose );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 5: //** COMM Button **
							fMission.mDisplayMessage( kSuluT6Communications_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndSuluT6Communications );
							fMission.mStartFlashingButton( "A3_OFFICER_COMM" );
							break;

						case 6:
							fMission.mDisplayMessage( kSuluT6HailMe_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndSuluT6HailMe );
							TutorialSegment++;
							SuluTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
							break;

						default:
							return kPlayerWins;
					}
				}
				break;

				case kTimerLimitSet_2:	// Loop until voice is done playing...
					if( fMission.mVoiceSampleStillPlaying() != 0 )
					{
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					else
					{
						TutorialSegment++;
						fMission.mSetTimedInterval( kTimerLimitSet, 1 );
					}
					break;

				case kTimerLimitSet_3:
					fMission.mDisplayMessage( kSuluT6Klingons_msg, kMissionScriptMessage );
					fMission.mPlayVoiceSample( kSndSuluT6Klingons );
					mCreateMissionShips( kKlingonAttack );
					break;

				case kTimerLimitSet_4:
					fMission.mDisplayMessage( kSuluT6End_msg, kMissionScriptMessage );
					fMission.mPlayVoiceSample( kSndSuluT6End );

					Ship = PlayerTeam->mGetFirstShip();
					Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0.0);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 4);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, 2);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 2);

					Ship = SuluTeam->mGetFirstShip();
					Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0.0);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 4);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, 2);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 2);

					fMission.mSetTimedInterval( kTimerLimitSet_3, 10 );
					break;

				case kTimerLimitSet_5:
					TutorialPassed = true;

					if( SuluAlive == false )
					{
						fMission.mDisplayMessage( kSuluT6ILived_msg, kMissionScriptMessage );
						fMission.mPlayVoiceSample( kSndSuluT6ILived );
						SuluAlive = true;
						fMission.mSetTimedInterval( kTimerLimitSet_5, 6 );
					}
					else
					{
						return kPlayerWins;
					}
					break;

				case kTimerTurnsSet:
					if( Mode != kSkirmish )
					{
						fMission.mDisplayMessage( kSuluT6Begin_msg, kMissionScriptMessage );
						fMission.mPlayVoiceSample( kSndSuluT6Begin );
						Ship = SuluTeam->mGetFirstShip();
						Ship->mClearCommButton( 1 );
						Ship->mClearCommButton( 2 );
						fMission.mSetTeamRelations( kPlayerTeam, kSuluTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					}
					break;
			}
		}
		break;

		case kNotifyDamage:
			if( Mode == kStarting )
			{
				fMission.mDisplayMessage( kSuluT6NoFair_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample( kSndSuluT6NoFair );
				fMission.mSetTeamRelations( kPlayerTeam, kSuluTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
				Mode = kSkirmish;
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			else if( Mode == kSkirmish )
			{
				// if the Excelsior is too damaged, stop the sim and start the Klingon attack
				Ship = SuluTeam->mGetFirstShip();
				if( Ship->mGetDamagePercent() >= 33 )
				{
					fMission.mDisplayMessage (kSuluT6Excellent_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndSuluT6Excellent);
					Mode = kKlingonAttack;
				}

				// if the Player's ship is too damaged, stop the sim and start the Klingon attack
				Ship = PlayerTeam->mGetFirstShip();

				if( Ship->mGetDamagePercent() >= 50 )
				{
					fMission.mDisplayMessage( kSuluT6Bummer_msg, kMissionScriptMessage );
					fMission.mPlayVoiceSample( kSndSuluT6Bummer );
					Mode = kKlingonAttack;
				}

				// if the Klingon attack has started
				if( Mode == kKlingonAttack )
				{
					fMission.mSetTimedInterval (kTimerLimitSet_4, 8);
					fMission.mSetTeamRelations( kPlayerTeam, kSuluTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
				}
				else
				{
					PlayerTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
					SuluTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			else if( Ship->mGetOriginalTeam() == kSuluTeam )
			{
				fMission.mDisplayMessage( kSuluT6SuluDead_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample( kSndSuluT6SuluDead );
				SuluAlive = false;
			}
			else if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					Mode = kWinner;
					fMission.mDisplayMessage( kSuluT6GotThem_msg, kMissionScriptMessage );
					fMission.mPlayVoiceSample( kSndSuluT6GotThem );
					fMission.mSetTimedInterval( kTimerLimitSet_5, 8 );
				}
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					fMission.mDisplayMessage( kSuluT6Hailed_msg, kMissionScriptMessage );
					fMission.mPlayVoiceSample( kSndSuluT6Hailed );
					Ship->mSetCommButton( 1, kButtonAccept_text );
					Ship->mSetCommButton( 2, kButtonDecline_text );
					fMission.mSetTimedInterval( kTimerTurnsSet, 2 );
					break;

				case kButtonAccept_text:
					fMission.mDisplayMessage (kSuluT6Accepted_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndSuluT6Accepted);
					Mode = kSkirmish;
					break;

				case kButtonDecline_text:
					fMission.mDisplayMessage (kSuluT6Declined_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndSuluT6Declined);
					Mode = kSkirmish;
					break;
			}

			if( Mode == kSkirmish )
			{
				Ship = SuluTeam->mGetFirstShip();
				Ship->mClearCommButton( 1 );
				Ship->mClearCommButton( 2 );
				fMission.mSetTeamRelations( kPlayerTeam, kSuluTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
				fMission.mSetTimedInterval( kTimerTurnsSet, 0 );
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( TutorialPassed == true )
	{
		fMission.mAddToDebriefingList( kGoodDebrief_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kBadDebrief_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}
	return 0;
}
