#include "scriptinterface.h"

#include "Tut_Gorn1Text.h"
#include "Tut_Gorn1Maps.h"

int32 MoveX = 0;
int32 MoveY = 0;
int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool TutorialPassed = false;

enum eCurrentRound
{
	FirstRound = 1,
	SecondRound,
	ThirdRound,
};

eCurrentRound CurrentRound = FirstRound;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, kGorn );
	fMission.mSetTeamRace( kEnemyTeam, kRomulan );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tShipInfo* Ship;

	int32 x, y;

	// Create a ship for the player to use
	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( "G-BC", kStartPosition_G );
	}
	else if( Mode == FirstRound )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &MoveX, &MoveY );
		EnemyTeam->mCreateShip( "R-FSPh", kStartPosition_Specified, MoveX, MoveY+1 );
	}
	else if( Mode == SecondRound )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &MoveX, &MoveY );
		EnemyTeam->mCreateFleet( 3, kMaxBPV, kClassFreighter, kClassFreighter, 1, kStartPosition_Specified | kGroupAround, MoveX+1, MoveY+1 );
	}
	else if( Mode == ThirdRound )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &x, &y );
		EnemyTeam->mCreateShip( "R-WER", kStartPosition_Specified, x-2, y+4 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	// configure the player's ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 0 );
	Ship->mChangeName( kEggtutor_name );
	Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if( CurrentRound == ThirdRound )
	{
		fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		Ship->mChangeAIOfficerState( kCloakOfficer, kFalse );
	}
	else
	{
		Ship->mDamageSystem( kDamageWeaponsAndSystems, 75.0 );
		Ship->mSetFinalDestination( MoveX, MoveY );
		Ship->mSetShipMaxSpeed( 6 );
	}

	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					switch( TutorialSegment )
					{
						case 0:
							fMission.mDisplayMessage( kWhere_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kWhere2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kWhere3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kGornWelcome_msg );
							fMission.mPlayVoiceSample( kSndGornWelcome );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 4:
							fMission.mDisplayMessage( kGornCharging_msg );
							fMission.mPlayVoiceSample( kSndGornCharging );

							if( fPlayerShip->mGetAlertStatus() == kAlertRed )
							{
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							break;

						case 5:
							fMission.mStartFlashingButton( "PlayerSchem.Weapon3" );	// pl-S RP
							fMission.mDisplayMessage( kGornClickPlasma_msg );
							fMission.mPlayVoiceSample( kSndGornClickPlasma );
							break;

						case 6:
							fMission.mDisplayMessage( kGornPlasmaPanel_msg );
							fMission.mPlayVoiceSample( kSndGornPlasmaPanel );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 7:
							fMission.mDisplayMessage( kGornPanelDescription_msg );
							fMission.mPlayVoiceSample( kSndGornPanelDescription );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 8:
							fMission.mDisplayMessage( kGornPlasmaSize_msg );
							fMission.mPlayVoiceSample( kSndGornPlasmaSize );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 9:
							mCreateMissionShips( FirstRound );
							fMission.mDisplayMessage( kGornFireOne_msg );
							fMission.mPlayVoiceSample( kSndGornFireOne );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 10:
							fMission.mDisplayMessage( kGornPlasmaDamage_msg );
							fMission.mPlayVoiceSample( kSndGornPlasmaDamage );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 11:
							fMission.mDisplayMessage( kGornPseudo_msg );
							fMission.mPlayVoiceSample( kSndGornPseudo );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 12:
							fMission.mDisplayMessage( kGornEnveloping_msg );
							fMission.mPlayVoiceSample( kSndGornEnveloping );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 13:
							if( EnemyTeam->mGetShipsInPlayCount() > 0 )
							{
								fMission.mDisplayMessage( kGornDestroy_msg );
								fMission.mPlayVoiceSample( kSndGornDestroy );
							}
							else
							{
								TutorialSegment = 14;
								fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							}
							break;

						case 14:
							fMission.mDisplayMessage( kGornRomulanDestroyed_msg );
							fMission.mPlayVoiceSample( kSndGornRomulanDestroyed );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 15:
							mCreateMissionShips( SecondRound );
							fMission.mStartFlashingButton( "DefensivePlasma" );
							fMission.mDisplayMessage( kGornClickShotgun_msg );
							fMission.mPlayVoiceSample( kSndGornClickShotgun );
							break;

						case 16:
							fMission.mDisplayMessage( kGornShotgun_msg );
							fMission.mPlayVoiceSample( kSndGornShotgun );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 17:
							fMission.mDisplayMessage( kGornDestroyAll_msg );
							fMission.mPlayVoiceSample( kSndGornDestroyAll );
							break;

						case 18:
							if( EnemyTeam->mGetShipsInPlayCount() == 0 )
							{
								mCreateMissionShips( ThirdRound );
								fMission.mDisplayMessage( kGornFightRomulan_msg );
								fMission.mPlayVoiceSample( kSndGornFightRomulan );
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							break;

						case 19:
							fMission.mDisplayMessage( kGornDefenses_msg );
							fMission.mPlayVoiceSample( kSndGornDefenses );
							break;
					}
					break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;
			}
			break;

		case kNotifyAlertStatusChange:
			if(( Ship->mGetAlertStatus() == kAlertRed ) && ( TutorialSegment >= 4 ))
			{
				TutorialSegment = 5;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			else
			{
				Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );
			}
			break;

		case kNotifyAtDestination:
			Ship->mSetShipMaxSpeed( 0 );
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			else
			{
				if( CurrentRound == FirstRound )
				{
					CurrentRound = SecondRound;
					TutorialSegment = 14;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				else if( CurrentRound == SecondRound )
				{
					if( EnemyTeam->mGetShipsInPlayCount() == 0 )
					{
						CurrentRound = ThirdRound;
						TutorialSegment = 18;
						fMission.mSetTimedInterval( kTimerLimitSet, 1 );
					}
				}
				else if( CurrentRound == ThirdRound )
				{
					fMission.mDisplayMessage( kGornKilledRomulan_msg );
					fMission.mPlayVoiceSample( kSndGornKilledRomulan );
					return kPlayerWins;
				}
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if( PlayerTeam->mGetShipsInPlayCount() == 0 )
	{
		fMission.mAddToDebriefingList (kGornDied_msg);
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList (kDebrief_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
