#include "scriptinterface.h"

#include "Tut_Hydran1Text.h"
#include "Tut_Hydran1Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool KlingonsDead = false;
bool TutorialPassed = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kD6Team,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kKlingon );
	fMission.mSetTeamRace( kD6Team, kKlingon );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kD6Team, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kD6Team, kTeamAllied, kGoalDestroy, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* D6Team = fMission.mGetTeamHandle( kD6Team );
	tShipInfo* Ship;

	int32 x, y;

	// Create a ship for the player to use
	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( "H-LM", kStartPosition_G );
	}

	if( Mode == 1 )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &x, &y );
		EnemyTeam->mCreateShip( "K-FSPh", kStartPosition_Specified, x, y+1 );
	}

	if( Mode == 2 )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &x, &y );
		D6Team->mCreateShip( "K-D6", kStartPosition_Specified, x, y+2 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* D6Team = fMission.mGetTeamHandle( kD6Team );
	tShipInfo* Ship;

	// configure the player's ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 0 );
	Ship->mChangeName( kHeadmaster_name );
	Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if( Ship->mGetTeam() == kEnemyTeam )
	{
		Ship->mDamageSystem( kDamageLeftWarpEngine, 50.0 );
		Ship->mDamageSystem( kDamageRightWarpEngine, 50.0 );
		Ship->mDamageSystem( kDamageImpulseEngine, 100.0 );
		Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );
		Ship->mDamageSystem( kDamageMaxShields, 75.0 );
		Ship->mSetShipMaxSpeed( 2 );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}
	else					// Klingon D6
	{
		Ship->mDamageSystem( kDamageShield_4_Aft, 100.0 );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyMissileSpeed, 0 );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyNumberOfReloads, 1 );
	}

	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* D6Team = fMission.mGetTeamHandle( kD6Team );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					switch( TutorialSegment )
					{
						case 0:
							fMission.mDisplayMessage( kHydran1Where_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kHydran1Where2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kHydran1Where3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kHydran1Welcome_msg );
							fMission.mPlayVoiceSample( kSndHydran1Welcome );

							if( fPlayerShip->mGetAlertStatus() == kAlertRed )
							{
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							break;

						case 4:
							fMission.mStartFlashingButton( "PlayerSchem.Weapon7" );
							fMission.mDisplayMessage( kHydran1ClickFusion_msg );
							fMission.mPlayVoiceSample( kSndHydran1ClickFusion );
							break;

						case 5:
							fMission.mDisplayMessage( kHydran1FusionPanel_msg );
							fMission.mPlayVoiceSample( kSndHydran1FusionPanel );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 6:
							mCreateMissionShips( 1 );
							fMission.mDisplayMessage( kHydran1UseFusions_msg );
							fMission.mPlayVoiceSample( kSndHydran1UseFusions );
							// continues when target ship is 75% damaged...
							break;

						case 7:
							fMission.mStartFlashingButton( "SuicideFusion" );
							fMission.mDisplayMessage( kHydran1ClickSuicide_msg );
							fMission.mPlayVoiceSample( kSndHydran1ClickSuicide );
							break;

						case 8:
							fMission.mDisplayMessage( kHydran1SuicideOL_msg );
							fMission.mPlayVoiceSample( kSndHydran1SuicideOL );
							// continues when freighter is destroyed...
							break;

						case 9:
							Ship = PlayerTeam->mGetFirstShip();
							Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 0.0 );
							Ship->mAccessSupplies( kResetAllItems, 0, 0 );
							fMission.mDisplayMessage( kHydran1FreighterDestroyed_msg );
							fMission.mPlayVoiceSample( kSndHydran1FreighterDestroyed );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							fMission.mDisplayMessage( kShipRepaired_msg, kNonCommScriptMessage );
							break;

						case 10:
							fMission.mDisplayMessage( kHydran1FusionAndFighters_msg );
							fMission.mPlayVoiceSample( kSndHydran1FusionAndFighters );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 11:
							fMission.mStartFlashingButton( "PlayerSchem.Weapon1" );
							fMission.mDisplayMessage( kHydran1ClickHellbore_msg );
							fMission.mPlayVoiceSample( kSndHydran1ClickHellbore );
							break;

						case 12:
							fMission.mDisplayMessage( kHydran1HellborePanel_msg );
							fMission.mPlayVoiceSample( kSndHydran1HellborePanel );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 13:
							fMission.mDisplayMessage( kHydran1HellboreNature_msg );
							fMission.mPlayVoiceSample( kSndHydran1HellboreNature );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 14:
							mCreateMissionShips( 2 );
							fMission.mDisplayMessage( kHydran1Klingon_msg );
							fMission.mPlayVoiceSample( kSndHydran1Klingon );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
// CNF: had to comment this line to avoid calling the shield removal over and over which causes a missile bug
//							fMission.mSetTimedInterval( kTimerLimitSet_3, 2 );
							break;

						case 15:
							fMission.mDisplayMessage( kHydran1GoodLuck_msg );
							fMission.mPlayVoiceSample( kSndHydran1GoodLuck );
							break;
					}
					break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;

			case kTimerLimitSet_3:
				if( D6Team->mGetShipsInPlayCount() > 0 )
				{
					Ship = D6Team->mGetFirstShipInPlay();

					if( Ship != NULL )
					{
						Ship->mDamageSystem( kDamageShield_4_Aft, 100.0 );
						fMission.mSetTimedInterval( kTimerLimitSet_3, 2 );
					}
				}
				break;
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAlertStatusChange:
			if(( Ship->mGetAlertStatus() == kAlertRed ) && ( TutorialSegment <= 3 ))
			{
				TutorialSegment = 4;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( Ship->mGetDamagePercent() >= 75.0 )
				{
					TutorialSegment = 7;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				else
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}

			if( Ship->mGetTeam() == kEnemyTeam )
			{
				TutorialSegment = 9;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}

			if( Ship->mGetTeam() == kD6Team )
			{
				fMission.mDisplayMessage( kHydran1Congrats_msg );
				fMission.mPlayVoiceSample( kSndHydran1Congrats );
// CNF: had to comment this line to avoid calling the shield removal over and over which causes a missile bug
//				fMission.mSetTimedInterval( kTimerLimitSet_3, 0 );
				fMission.mSetTimedInterval( kTimerEndGamePause, 8 );

				return kPlayerWins;
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if( PlayerTeam->mGetShipsInPlayCount() == 0 )
	{
		fMission.mAddToDebriefingList( kDied_msg );
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList( kDebrief_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
