#include "scriptinterface.h"

#include "Tut_Klingon1Text.h"
#include "Tut_Klingon1Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 ResponseToPlayerAction = 0; // a player state to select between doing TutorialSegment Sounds and other sounds
int32 DelayReallyShort = 1;
int32 DelayQuiteShort = 4;
int32 DelayShort = 7;
int32 DelayLong = 12;
int32 DelayReallyLong = 18;
int32 EnemyDestroyed = 0;
int32 extraTimer = 0; // add seconds to wait for Mission text to scroll off screen
bool BlinkingButtonBlinking = false;
bool TutorialPassed = false;

const char* kA4_SPEED_BAR	=	"A4_SPEED_BAR";
const char* lastBlinkingButton	=	NULL;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a frigate for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip ("K-E4", kStartPosition_G);
	}

	if (Mode == 1)
	{
		Ship = PlayerTeam->mGetFirstShip ();
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("K-FSPh", kStartPosition_Specified, x+2, y);
		EnemyTeam->mCreateShip ("K-FSPh", kStartPosition_Specified, x, y+2);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	// configure the player's frigate
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kGhostship_name );
	Ship->mSetShipThrust (0);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 4 );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 45 );
	Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 100 );
	Ship->mDamageSystem( kDamageMaxShields, 100 );
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	Ship->mSetShipMaxSpeed( 8 );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch (Event)
	{
		case kNotifyAtDistance:
			break;

		case kNotifyFlashingButtonPressed:
			{
				extraTimer = 0;
				BlinkingButtonBlinking = false;
				fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch (fEventID)
				{
					case kTimerLimitSet:
						{
							if ( ResponseToPlayerAction == 1 )
							{
								if ( EnemyDestroyed == 2 )
								{
									fMission.mDisplayMessage (kKetrick_Excellent_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick_Excellent);
									ResponseToPlayerAction = 0;
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									//return kPlayerWins;
								}
								else
								{
									fMission.mDisplayMessage (kKetrick_OneMore_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick_OneMore);
									//fMission.mSetTimedInterval (kTimerLimitSet, DelayQuiteShort);
									//ResponseToPlayerAction = 0;
								}
							}
							else if ( ResponseToPlayerAction == 0 )
							{
								switch (TutorialSegment)
								{
									case 0:
										fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);				// startup message
										TutorialSegment++;
										fMission.mSetTimedInterval (kTimerLimitSet, 1);
										break;
									case 1:
										fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
										TutorialSegment++;
										fMission.mSetTimedInterval (kTimerLimitSet, 1);
										break;
									case 2:
										fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
										TutorialSegment++;
										fMission.mSetTimedInterval (kTimerLimitSet, 1);
										break;
									case 3:
										fMission.mDisplayMessage (kKetrick_Welcome_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Welcome);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 4:
										fMission.mDisplayMessage (kKetrick_IHaveBeenAsked_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_IHaveBeenAsked);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
										
									case 5:
										fMission.mDisplayMessage (kKetrick_QuickReview_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_QuickReview);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 6:
										fMission.mDisplayMessage (kKetrick_ToggleD_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_ToggleD);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 7:
										fMission.mDisplayMessage (kKetrick_BeautySpace_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_BeautySpace);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;					
									case 8:
										fMission.mDisplayMessage (kKetrick_SpaceDangerous_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_SpaceDangerous);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 9:
										fMission.mDisplayMessage (kKetrick_RedAlert_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_RedAlert);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 10: // ** blinking speed bar **
										// LNF blinking speed 
										fMission.mDisplayMessage (kKetrick_Speed_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Speed);
										fMission.mStartFlashingButton( kA4_SPEED_BAR );
										lastBlinkingButton = kA4_SPEED_BAR;
										BlinkingButtonBlinking = true;
										fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
										break;
									case 11:
										fMission.mDisplayMessage (kKetrick_Turn_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Turn);
										// LNF wait a few turns/seconds for the player to click around
										extraTimer = 6;
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 12:
										fMission.mDisplayMessage (kKetrick_Freighters_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Freighters);
										mCreateMissionShips ( 1 );
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 13:
										fMission.mDisplayMessage (kKetrick_TKey_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_TKey);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 14:
										fMission.mDisplayMessage (kKetrick_Good_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Good);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 15:
										fMission.mDisplayMessage (kKetrick_Fire_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Fire);
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									case 16:
										fMission.mDisplayMessage (kKetrick_Destroy_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick_Destroy);
										//TutorialSegment++;
										//fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
										break;
									case 17:
										return kPlayerWins;
										break;
									}
							}
						}
						break; // TimerLimitSet	

					case kTimerLimitSet_2:
						if ( fMission.mVoiceSampleStillPlaying() == 0 ) // voice stopped
						{
							if ( BlinkingButtonBlinking == false ) // the player already pushed the button
							{
								fMission.mSetTimedInterval (kTimerLimitSet_9, extraTimer);					
							}
							else // button not pressed
							{
								fMission.mSetTimedInterval( kTimerCountdownSet, 1 );
							}
						}
						else
						{
							fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
						}
						break;

					case kTimerLimitSet_9: // extra timer to wait for text to scroll
						TutorialSegment++;
						extraTimer = 0; // maybe wait a second or so instead of 0 to smooth out waiting
						fMission.mSetTimedInterval (kTimerLimitSet, 0);
						break;
 
					case kTimerLimitSet_10:
						if ( fMission.mVoiceSampleStillPlaying() == 0 )
						{
							fMission.mSetTimedInterval (kTimerLimitSet_9, extraTimer);						}
						else
						{
							fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
						}
						break;

					case kTimerCountdownSet:
						// next tutorial segment in 1 turn automatically
						// LNF Need to stop the button from blinking and make sure it doesn't 
						// accept clicks or it will advance the tutorial
						fMission.mDisplayMessage( kKetrick_MovingOn_msg, kMissionScriptMessage );
						TutorialSegment++;
						extraTimer = 0;
						BlinkingButtonBlinking = false;
						fMission.mStopFlashingButton( lastBlinkingButton );
						fMission.mSetTimedInterval ( kTimerLimitSet, 0 );
						break;

				}
			}
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;
				ResponseToPlayerAction = 1;
				fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
			}
			else if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 50);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
	//return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}

	fMission.mAddToDebriefingList (kDebrief_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
