#include "scriptinterface.h"

#include "Tut_Klingon2Text.h"
#include "Tut_Klingon2Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 ResponseToPlayerAction = 0; // a player state to select between doing TutorialSegment Sounds and other sounds
int32 DelayReallyShort = 1;
int32 DelayQuiteShort = 4;
int32 DelayShort = 7;
int32 DelayLong = 12;
int32 DelayReallyLong = 18;
int32 EnemyDestroyed = 0;
int32 BlinkingButtonBlinking = 0;
int32 Timer2 = 0; // kNotSet
int32 Timer3 = 0; // kNotSet
int32 extraTimer = 0; // add seconds to wait for Mission text to scroll off screen
int32 MissionMode = 0;
bool TutorialPassed = true;

enum eTimer2Expired
{
	kNotSet = 0,
	kExpired = 1
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 1;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a frigate for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip ("K-E4", kStartPosition_G);
	}

	if (Mode == 1) // ENF tune the exact freighter strengths
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("K-FSPh", kStartPosition_Specified, x+2, y);
		MissionMode = 1;
	}

	if (Mode == 2)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("K-FSPh", kStartPosition_Specified, x+2, y);
		MissionMode = 2;
	}

	if (Mode == 3)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("K-FSPh", kStartPosition_Specified, x+2, y);
		MissionMode = 3;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	// configure the player's frigate
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust (0);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
	Ship->mChangeName( kGhostship_name );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 50 );
	Ship->mDamageSystem( kDamageMaxWeapons, 0);
	Ship->mDamageSystem(kDamageShuttleBaySystem, 100);
	Ship->mDamageSystem( kDamageMaxShields, 100 );

	if ( MissionMode == 1 )
	{
		Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 100 );
		
	}
	else if ( MissionMode == 2 )
	{
		Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 75 );
		
	}
	else if ( MissionMode == 3 )
	{
		Ship->mDamageSystem( kDamageInternalsWithPointsSilently, 60 );
		
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* PlayerShip;

	switch (Event)
	{
		case kNotifyAtDistance:
			break;

		case kNotifyFlashingButtonPressed:
			{
				// increment the tutorial segment
				TutorialSegment++;
				BlinkingButtonBlinking = 0;
				// if the Ketrick voice is already finished, move on immediately
				// otherwise, wait for Timer_2 to expire
				if (Timer2 == kExpired)
				{
					Timer2 = kNotSet;
					Timer3 = kExpired;
					fMission.mSetTimedInterval( kTimerLimitSet_3, 0 );
					fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				}
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch (fEventID)
				{
					case kTimerLimitSet:
						if ( ResponseToPlayerAction == 1 )
						{
							switch ( EnemyDestroyed )
							{
							case 1:
								{
									fMission.mDisplayMessage (kKetrick2_WellDone_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_WellDone);
									ResponseToPlayerAction = 0;
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								}
							case 2:
								{
									fMission.mDisplayMessage (kKetrick2_FineJob_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_FineJob);
									ResponseToPlayerAction = 0;
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								}
							case 3:
								{
									fMission.mDisplayMessage (kKetrick2_ExcellentCommander_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_ExcellentCommander);
									ResponseToPlayerAction = 0;
									//fMission.mSetTimedInterval (kTimerLimitSet, 4);
									return kPlayerWins;
									break;
								}
							}
						}
						else if ( ResponseToPlayerAction == 0 )
						{
							switch (TutorialSegment)
							{
								case 0:
									fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 1:
									fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 2:
									fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 3:
									fMission.mDisplayMessage (kKetrick2_Anxious_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_Anxious);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 4:
									{
										fMission.mDisplayMessage (kKetrick2_CrippledPhasers_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick2_CrippledPhasers);
										// This loop will disable all of a ship's phasers
										PlayerShip = PlayerTeam->mGetFirstShip();
										for ( int32 i = kDamageWeapon11; i < kDamageWeapon25; i++ )
										{
											PlayerShip->mDamageSystem( (eDamageSystems) i, 100 );
										}
										PlayerShip->mDamageSystem( kDamageHeavyWeapon7, 100 );
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									}
									
								case 5:
									fMission.mDisplayMessage (kKetrick2_LearnDisruptor_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_LearnDisruptor);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 6:
									fMission.mDisplayMessage (kKetrick2_DisruptorPowerful_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_DisruptorPowerful);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 7:
									fMission.mDisplayMessage (kKetrick2_DisruptorDrawback_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_DisruptorDrawback);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;					
								case 8: // **blinking weapon 1**
									fMission.mDisplayMessage (kKetrick2_ClickOnDisruptorBlinking_msg, kScienceScriptMessage);
									fMission.mStartFlashingButton( "PlayerSchem.Weapon3" );
									fMission.mPlayVoiceSample (kSndKetrick2_ClickOnDisruptorBlinking);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 9:
									fMission.mDisplayMessage (kKetrick2_PieChart_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_PieChart);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 10: // **blinking All Weapons**
									fMission.mDisplayMessage (kKetrick2_ClickAllWeapons_msg, kScienceScriptMessage);
									fMission.mStartFlashingButton( "A5_WEAPONS_SELECT" );
									fMission.mPlayVoiceSample (kSndKetrick2_ClickAllWeapons);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 11: // **First Freighter Created here**
									fMission.mDisplayMessage (kKetrick2_DestroyFreighterDisruptor_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_DestroyFreighterDisruptor);
									mCreateMissionShips ( 1 );
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 12:
									fMission.mDisplayMessage (kKetrick2_ManeuverFrigate_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_ManeuverFrigate);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 13:
									fMission.mDisplayMessage (kKetrick2_GainFeel_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_GainFeel);
									TutorialSegment = 16;	// HACK!!!!
									//fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									// The player will need to destroy the frieghter to advance the script.
									break;
/*								case 14:
									fMission.mDisplayMessage (kKetrick2_OneMoreThing_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_OneMoreThing);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 15:
									fMission.mDisplayMessage (kKetrick2_TorpedoesCannotLaunch_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_TorpedoesCannotLaunch);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 16: // **wait for FIRST FREIGHTER**
									fMission.mDisplayMessage (kKetrick2_DistanceDisplay1_msg, kScienceScriptMessage);
									fMission.mDisplayMessage (kKetrick2_DistanceDisplay2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_DistanceDisplay1);
									//fMission.mPlayVoiceSample (kSndKetrick2_DistanceDisplay2);
									// The player will need to destroy the frieghter to advance the script.
									break;
*/								case 17:
									{
										// We should get into here from the destruction of a freighter.
										fMission.mDisplayMessage (kKetrick2_DireCircum_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick2_DireCircum);
										// Restore their Disruptor torpedoes if they were damaged.
										PlayerShip = PlayerTeam->mGetFirstShip();
										for ( int32 i = kDamageHeavyWeapon1; i < kDamageHeavyWeapon10; i++ )
										{
											PlayerShip->mDamageSystem( (eDamageSystems) i, 0 );
										}
										PlayerShip->mDamageSystem( kDamageHeavyWeapon7, 100 );
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									}
								case 18: // **blinking Weapon 1**
									fMission.mDisplayMessage (kKetrick2_ClickOnDisruptorBlinking_msg, kScienceScriptMessage);
									fMission.mStartFlashingButton( "PlayerSchem.Weapon3" );
									fMission.mPlayVoiceSample (kSndKetrick2_ClickOnDisruptorBlinking);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;

								case 19: // **blinking Overloaded**
									fMission.mDisplayMessage (kKetrick2_ThatIsOverloaded_msg, kScienceScriptMessage);
									fMission.mStartFlashingButton( "A6_DISRUPTOR_OVERLOAD" );
									fMission.mPlayVoiceSample (kSndKetrick2_ThatIsOverloaded);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 7);
									break;
								case 20:
									fMission.mDisplayMessage (kKetrick2_OverloadedOnlyShort_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_OverloadedOnlyShort);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 21:
									fMission.mDisplayMessage (kKetrick2_OverloadedCostMoreEnergy_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_OverloadedCostMoreEnergy);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 22: // **blinking All Weapons**
									fMission.mDisplayMessage (kKetrick2_ClickAllWeapons_msg, kScienceScriptMessage);
									// ENF need to change to all weapons
									fMission.mStartFlashingButton( "A5_WEAPONS_SELECT" );
									fMission.mPlayVoiceSample (kSndKetrick2_ClickAllWeapons);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 23: // ** SECOND FREIGHTER**
									fMission.mDisplayMessage (kKetrick2_DestroyFreighterWithDisruptor_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_DestroyFreighterWithDisruptor);
									mCreateMissionShips ( 2 );
									// The player will need to destroy the frieghter to advance the script.
									break;
								case 24: // **blinking All Weapons**
									fMission.mDisplayMessage (kKetrick2_ClickAllWeapons_msg, kScienceScriptMessage);
									// ENF need to change to all weapons
									fMission.mStartFlashingButton( "A5_WEAPONS_SELECT" );
									fMission.mPlayVoiceSample (kSndKetrick2_ClickAllWeapons);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 25:
									fMission.mDisplayMessage (kKetrick2_HoldCtrlKey_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_HoldCtrlKey);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 26:
									fMission.mDisplayMessage (kKetrick2_SetAllToWeapon1_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_SetAllToWeapon1);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 27: // **blinking Weapons 1**
									fMission.mDisplayMessage (kKetrick2_ClickOnDisruptorBlinking_msg, kScienceScriptMessage);
									fMission.mStartFlashingButton( "PlayerSchem.Weapon3" );
									fMission.mPlayVoiceSample (kSndKetrick2_ClickOnDisruptorBlinking);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 28: // **blinking Weapons 2**
									fMission.mDisplayMessage (kKetrick2_ShiftClick_msg, kScienceScriptMessage);
									fMission.mStartFlashingButton( "PlayerSchem.Weapon4" );
									fMission.mPlayVoiceSample (kSndKetrick2_ShiftClick);
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 7);
									break;
								case 29:
									fMission.mDisplayMessage (kKetrick2_SelectedTwoDisruptor_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_SelectedTwoDisruptor);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 30:
									fMission.mDisplayMessage (kKetrick2_AgainCtrlKey2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_AgainCtrlKey2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 31:
									fMission.mDisplayMessage (kKetrick2_WeaponGroup2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_WeaponGroup2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 32:
									fMission.mDisplayMessage (kKetrick2_PressBackAndForth1n2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_PressBackAndForth1n2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 33:
									fMission.mDisplayMessage (kKetrick2_NoticeSelection_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_NoticeSelection);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 34:
									fMission.mDisplayMessage (kKetrick2_FireDisruptorsTap2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick2_FireDisruptorsTap);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 35:
									{
										fMission.mDisplayMessage (kKetrick2_GoAheadDestroy_msg, kScienceScriptMessage);
										fMission.mPlayVoiceSample (kSndKetrick2_GoAheadDestroy);
										// Restore the player's ship if it has been damaged.
										PlayerShip = fPlayerShip;
										PlayerShip->mDamageSystem( kDamageWeaponsAndSystems, 0 );
										fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
										break;
									}
								case 36:
									// LNF need to change this text message
									fMission.mDisplayMessage (kKetrick2_BasicTactic_msg, kScienceScriptMessage);
									mCreateMissionShips ( 3 );
									fMission.mPlayVoiceSample (kSndKetrick2_BasicTactic);
									// The player will need to destroy the frieghter to advance the script.
									break;
							}
						}
						break;

					case kTimerLimitSet_2: // waits for the Ketrick voice to finish before the script continues
						// if the buttons was already pressed, move on when the voice finishes
						if (BlinkingButtonBlinking == 0)
						{
							fMission.mSetTimedInterval (kTimerLimitSet, 0);
						}
						else // set Timer2 flag to notify that the Ketrick voice is already finished
							// and the script can move on once the button is pressed
							// or when timer3 expires
						{
							Timer2 = kExpired;
							// Timer 3 moves on automatically if the player doesn't/can't press a button
							fMission.mSetTimedInterval( kTimerLimitSet_3, 15 );
						}
						break;

					case kTimerLimitSet_3: // next tutorial segment in 15 seconds automatically
						// LNF Need to stop the button from blinking and make sure it doesn't 
						// accept clicks or it will advance the tutorial
						if ( Timer3 == kNotSet )
						{
							fMission.mDisplayMessage( kKetrick2_MovingOn_msg, kMissionScriptMessage );
							TutorialSegment++;
							BlinkingButtonBlinking = 0;
							Timer2 = kNotSet;
							fMission.mSetTimedInterval ( kTimerLimitSet, 0 );
						}

						Timer3 = kNotSet;

						break;

					case kTimerLimitSet_9: // extra timer to wait for text to scroll
						TutorialSegment++;
						extraTimer = 0; // maybe wait a second or so instead of 0 to smooth out waiting
						fMission.mSetTimedInterval (kTimerLimitSet, 0);
						break;

					case kTimerLimitSet_10: // wait for sound to finish
						if ( fMission.mVoiceSampleStillPlaying() == 0 )
						{
							fMission.mSetTimedInterval (kTimerLimitSet_9, extraTimer);
						}
						else
						{
							fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;
				ResponseToPlayerAction = 1;
				fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
//				fMission.mDisplayMessage (kKetrick2_Sigh_msg, kScienceScriptMessage);
				fMission.mPlayVoiceSample (kSndKetrick2_Sigh);
				return kPlayerLoses;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 15);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
	//return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}
	fMission.mAddToDebriefingList (kDebrief_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
