#include "scriptinterface.h"

#include "Tut_Klingon3Text.h"
#include "Tut_Klingon3Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 ResponseToPlayerAction = 0; // a player state to select between doing TutorialSegment Sounds and other sounds
int32 DelayReallyShort = 1;
int32 DelayQuiteShort = 4;
int32 DelayShort = 7;
int32 DelayLong = 12;
int32 DelayReallyLong = 18;
int32 EnemyDestroyed = 0;
int32 Mode = 0;
int32 BlinkingButtonBlinking = 0;
int32 Timer2 = 0;
int32 Timer3 = 0;
int32 extraTimer = 0; // add seconds to wait for Mission text to scroll off screen
bool TutorialPassed = false;

enum eTimer2Expired
{
	kNotSet = 0,
	kExpired = 1
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBaseTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 2;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a ship with mines for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip ("K-D7", kStartPosition_G);
	}

	if (Mode == 1)
	{
		Ship = PlayerTeam->mGetFirstShip ();
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("K-FSPh", kStartPosition_Specified, x+2, y);
	}

	if (Mode == 2)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_Specified, x+1, y);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_Specified, x-1, y);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_Specified, x-1, y-1);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_Specified, x-2, y-1);
		BaseTeam->mCreateShip (kMaxBPV, kClassStarBase, kClassStarBase, 30, kStartPosition_Specified, x-6, y-6);
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* Ship;

	// configure the player's frigate
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kGhostship_name );
	Ship->mSetShipThrust( 8 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, 8 );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch ( Mode )
	{
		case 1:
			Ship->mDamageSystem( kDamageWeaponsAndSystems, 80 );
			Ship->mDamageSystem( kDamageLeftWarpEngine, 0 );
			Ship->mDamageSystem( kDamageRightWarpEngine, 0 );
			Ship->mSetShipThrust(10);
			Ship->mDamageSystem(kDamageShuttleBaySystem, 100);
			break;

		case 2:
			Ship->mSetShipThrust(0);
			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 30 );
			}
			break;
	}

	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tShipInfo* PlayerShip;

	switch (Event)
	{
		case kNotifyFlashingButtonPressed:
			{
				// increment the tutorial segment
				TutorialSegment++;
				BlinkingButtonBlinking = 0;
				// if the Sulu voice is already finished, move on immediately
				// otherwise, wait for Timer_2 to expire
				if (Timer2 == kExpired)
				{
					Timer2 = kNotSet;
					Timer3 = kExpired;
					fMission.mSetTimedInterval( kTimerLimitSet_3, 0 );
					fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				}
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch (fEventID)
				{
					case kTimerLimitSet:
						if ( ResponseToPlayerAction == 1 )
						{
							if ( EnemyDestroyed == 1 )
							{
								ResponseToPlayerAction = 0;
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 0);
								break;
							}
							else if ( EnemyDestroyed == 5 )
							{
								//kKetrick3_IAmImpressed_msg
								// ENF how do we distiguish if the player has destroyed the Frigates alone?
								// LNF can't do that yet...
								// SpecialBadge
								fMission.mDisplayMessage (kKetrick3_StarbasesAreImpressive_msg, kScienceScriptMessage);
								fMission.mPlayVoiceSample (kSndKetrick3_StarbasesAreImpressive);
								ResponseToPlayerAction = 0;
								return kPlayerWins;
								break;
							}
						}
						else if ( ResponseToPlayerAction == 0 )
						{
							switch (TutorialSegment)
							{
								case 0:
									fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 1:
									fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 2:
									fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 3:
									fMission.mDisplayMessage (kKetrick3_DiscussMines_msg, kScienceScriptMessage);
									fMission.mDisplayMessage (kKetrick3_SetRedAlert_msg, kNonCommScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_DiscussMines);
									//fMission.mPlayVoiceSample (kSndKetrick3_SetRedAlert);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 4: // **blinking Weapons Officer button**
									fMission.mDisplayMessage (kKetrick3_OfficerPanel_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_OfficerPanel);
									fMission.mStartFlashingButton( "A3_OFFICER_WEAPONS" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
									break;							
								case 5: // **blinking mine button**
									fMission.mDisplayMessage (kKetrick3_MineDeploy_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_MineDeploy);
									fMission.mStartFlashingButton( "A3_WEAPONS_LAYMINE_BOX" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 6);
									break;
								case 6:
									fMission.mDisplayMessage (kKetrick3_GreenRings1_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_GreenRings1);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 7:
									fMission.mDisplayMessage (kKetrick3_GreenRings2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_GreenRings2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 8:
									fMission.mDisplayMessage (kKetrick3_PlacedInSpace_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_PlacedInSpace);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;					
								case 9: // **blinking Transporter button**
									fMission.mDisplayMessage (kKetrick3_TransporterButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_TransporterButton);
									Ship = PlayerTeam->mGetFirstShip();
									fMission.mStartFlashingButton( "PlayerSchem.Transporter" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 10: // **Blinking Transporter Mine button**
									fMission.mDisplayMessage (kKetrick3_ActivateMine1_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_ActivateMine1);
									fMission.mStartFlashingButton( "TRANSPORT_BOMB" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 23);
									break;
								case 11:
									fMission.mDisplayMessage (kKetrick3_ActivateMine2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_ActivateMine2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 12:
									fMission.mDisplayMessage (kKetrick3_NoticeWeaponOfficer_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_NoticeWeaponOfficer);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 13:
									fMission.mDisplayMessage (kKetrick3_CrippledFrigate_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_CrippledFrigate);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 14:
									fMission.mDisplayMessage (kKetrick3_PhasersPhotonsCrippled_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_PhasersPhotonsCrippled);
									Mode = 1;
									// Create first wave of ships
									mCreateMissionShips ( Mode );
									PlayerShip = PlayerTeam->mGetFirstShip();
									PlayerShip->mDamageSystem(kDamageMaxWeapons, 100);
									break;
								case 15:
									fMission.mDisplayMessage (kKetrick3_WellDoneAdvancedHelm_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_WellDoneAdvancedHelm);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 16:
									fMission.mDisplayMessage (kKetrick3_Speed15_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_Speed15);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 17: // **blinking Helm Officer button**
									fMission.mDisplayMessage (kKetrick3_HelmOfficer_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_HelmOfficer);
									fMission.mStartFlashingButton( "A3_OFFICER_HELM" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 18: // **blinking HET reverse button**
									fMission.mDisplayMessage (kKetrick3_HETButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_HETButton);
									fMission.mStartFlashingButton( "A3_HELM_180_HET" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 10);
									break;
								case 19:
									fMission.mDisplayMessage (kKetrick3_ManeuverPowerful_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_ManeuverPowerful);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 20:
									fMission.mDisplayMessage (kKetrick3_SimulatedRogue_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_SimulatedRogue);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 21: // Final wave of ships
									fMission.mDisplayMessage (kKetrick3_LureFrigates_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_LureFrigates);
									Mode = 2;
									mCreateMissionShips ( Mode );
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 22:
									fMission.mDisplayMessage (kKetrick3_UseHET_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick3_UseHET);
									break;
								}
						}
						break;

					case kTimerLimitSet_2: // waits for the voice to finish before the script continues
						// if the buttons was already pressed, move on when the voice finishes
						if (BlinkingButtonBlinking == 0)
						{
							fMission.mSetTimedInterval (kTimerLimitSet, 0);
						}
						else // set Timer2 flag to notify that the voice is already finished
							// and the script can move on once the button is pressed
							// or when timer3 expires
						{
							Timer2 = kExpired;
							// Timer 3 moves on automatically if the player doesn't/can't press a button
							fMission.mSetTimedInterval( kTimerLimitSet_3, 15 );
						}
						break;

					case kTimerLimitSet_3: // next tutorial segment in 15 seconds automatically
						// LNF Need to stop the button from blinking and make sure it doesn't 
						// accept clicks or it will advance the tutorial
						if ( Timer3 == kNotSet )
						{
							fMission.mDisplayMessage( kKetrick3_MovingOn_msg, kMissionScriptMessage );
							TutorialSegment++;
							BlinkingButtonBlinking = 0;
							Timer2 = kNotSet;
							fMission.mSetTimedInterval ( kTimerLimitSet, 0 );
						}

						Timer3 = kNotSet;

						break;

					case kTimerLimitSet_9: // extra timer to wait for text to scroll
						TutorialSegment++;
						extraTimer = 0; // maybe wait a second or so instead of 0 to smooth out waiting
						fMission.mSetTimedInterval (kTimerLimitSet, 0);
						break;

					case kTimerLimitSet_10: // wait for sound to finish
						if ( fMission.mVoiceSampleStillPlaying() == 0 )
						{
							fMission.mSetTimedInterval (kTimerLimitSet_9, extraTimer);
						}
						else
						{
							fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			{
				if ( Ship->mGetTeam() == kEnemyTeam )
				{
					EnemyDestroyed++;
					//TutorialSegment++;
					ResponseToPlayerAction = 1;
					fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
				}
				else if (Ship->mGetTeam() == kPlayerTeam)
				{
					fMission.mDisplayMessage (kKetrick3_Sigh_msg, kScienceScriptMessage);
					return kPlayerLoses;
				}

			break;
			}

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 20);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
	//return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}
	fMission.mAddToDebriefingList (kKetrick3_IAmImpressed_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every end of tutorial or technical mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& infovoid)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
