#include "scriptinterface.h"

#include "Tut_Klingon4Text.h"
#include "Tut_Klingon4Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 ResponseToPlayerAction = 0; // a player state to select between doing TutorialSegment Sounds and other sounds
int32 DelayReallyShort = 1;
int32 DelayQuiteShort = 4;
int32 DelayShort = 7;
int32 DelayLong = 12;
int32 DelayReallyLong = 18;
int32 EnemyDestroyed = 0;
int32 NumberToKill = 4;
int32 Mode = 0;
int32 BlinkingButtonBlinking = 0;
int32 Timer2 = 0;
int32 Timer3 = 0;
int32 extraTimer = 0; // add seconds to wait for Mission text to scroll off screen
bool TutorialPassed = false;

enum eTimer2Expired
{
	kNotSet = 0,
	kExpired = 1
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 3;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fPlayerRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	int32 x, y;

	// Create a frigate for the player to use
	if (Mode == 0)
	{
		PlayerTeam->mCreateShip ("K-E4", kStartPosition_G);
	}

	if (Mode == 1)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&x, &y);
		for (int j = 1; j < NumberToKill; j++ )
		{
			EnemyTeam->mCreateShip (kMaxBPV, kClassFreighter, kClassFreighter, 100, kStartPosition_Specified, x+j+1, y+j+1);
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	// configure the player's Ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled ();
	Ship->mSetShipThrust (0);
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 4 );
	Ship->mChangeName( kGhostship_name );

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mDamageSystem( kDamageWeaponsAndSystems, 35 );
	Ship->mDamageSystem( kDamageShuttleBaySystem, 100.0 );
	Ship->mDamageSystem( kDamageMaxShields, 50.0 );
	Ship->mDamageSystem( kDamageExcessDamageSystem, 50.0 );
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	switch (Event)
	{
		case kNotifyAtDistance:
			break;

		case kNotifyFlashingButtonPressed:
			{
				// increment the tutorial segment
				TutorialSegment++;
				BlinkingButtonBlinking = 0;
				// if the Ketrick voice is already finished, move on immediately
				// otherwise, wait for Timer_2 to expire
				if (Timer2 == kExpired)
				{
					Timer2 = kNotSet;
					Timer3 = kExpired;
					fMission.mSetTimedInterval( kTimerLimitSet_3, 0 );
					fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				}
			}
			break;

		case kNotifyAtTimedInterval:
			{
				switch (fEventID)
				{
					case kTimerLimitSet:
						if ( ResponseToPlayerAction == 1 )
						{
							//kKetrick3_IAmImpressed_msg
							// ENF how do we distiguish if the player has destroyed the Frigates alone?
							// SpecialBadge
							fMission.mDisplayMessage (kKetrick4_StrongImprovement_msg, kScienceScriptMessage);
							fMission.mPlayVoiceSample (kSndKetrick4_StrongImprovement);
							ResponseToPlayerAction = 0;
							return kPlayerWins;
							break;
						}
						else if ( ResponseToPlayerAction == 0 )
						{
							switch (TutorialSegment)
							{
								case 0:
									fMission.mDisplayMessage (kWhere_msg, kNonCommScriptMessage);				// startup message
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 1:
									fMission.mDisplayMessage (kWhere2_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 2:
									fMission.mDisplayMessage (kWhere3_msg, kNonCommScriptMessage);
									TutorialSegment++;
									fMission.mSetTimedInterval (kTimerLimitSet, 1);
									break;
								case 3:
									fMission.mDisplayMessage (kKetrick4_ManyDefenses_1_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ManyDefenses_1);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									if( fPlayerShip->mGetAlertStatus() != kAlertRed )
									{
										fMission.mDisplayMessage( kGoToRedAlert_msg, kNonCommScriptMessage );
									}
									break;
								case 4:
									fMission.mDisplayMessage (kKetrick4_ManyDefenses_2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ManyDefenses_2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 5: // **blinking EW**
									fMission.mDisplayMessage (kKetrick4_EWButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_EWButton);
									fMission.mStartFlashingButton( "PlayerSchem.Sensors" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 6:
									fMission.mDisplayMessage (kKetrick4_StarshipEW_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_StarshipEW);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 7:
									fMission.mDisplayMessage (kKetrick4_UseECM_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_UseECM);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 8:
									fMission.mDisplayMessage (kKetrick4_ECCM_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ECCM);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;					
								case 9:
									fMission.mDisplayMessage (kKetrick4_ECCMSlider_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ECCMSlider);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 10:
									fMission.mDisplayMessage (kKetrick4_CurrentECM_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_CurrentECM);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 11:
									fMission.mDisplayMessage (kKetrick4_Speed10_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_Speed10);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 12:
									fMission.mDisplayMessage (kKetrick4_Shields_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_Shields);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 13: // **blinking Shield button**
									fMission.mDisplayMessage (kKetrick4_ShieldButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ShieldButton);
									fMission.mStartFlashingButton( "A5_SHIELD_SELECTER" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 14: // **blinking Center Shield**
									fMission.mDisplayMessage (kKetrick4_CenterShield_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_CenterShield);
									fMission.mStartFlashingButton( "A6_ALL_SHIELDS_SELECT" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
									break;
								case 15:
									fMission.mDisplayMessage (kKetrick4_Reinforce_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_Reinforce);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 16: // **blinking Science Officer**
									fMission.mDisplayMessage (kKetrick4_ScienceButton_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ScienceButton);
									fMission.mStartFlashingButton( "A3_OFFICER_SCIENCE" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 15);
									break;
								case 17:
									fMission.mDisplayMessage (kKetrick4_Science_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_Science);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 18: // **blinking Helm Officer**
									fMission.mDisplayMessage (kKetrick4_FinallyClick_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_FinallyClick);
									fMission.mStartFlashingButton( "A3_OFFICER_HELM" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 4);
									break;
								case 19: // **blinking Erratie Man**
									fMission.mDisplayMessage (kKetrick4_ClickOnErratic1_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ClickOnErratic1);
									fMission.mStartFlashingButton( "A3_HELM_ERRATIC" );
									BlinkingButtonBlinking = 1;
									fMission.mSetTimedInterval (kTimerLimitSet_2, 14);
									break;
								case 20:
									fMission.mDisplayMessage (kKetrick4_ClickOnErratic2_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ClickOnErratic2);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 21:
									fMission.mDisplayMessage (kKetrick4_BetweenEW_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_BetweenEW);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 22:
									fMission.mDisplayMessage (kKetrick4_ScottySays_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_ScottySays);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 23:
									fMission.mDisplayMessage (kKetrick4_QShips_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_QShips);
									fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
									break;
								case 24:
									Mode = 1;	
									mCreateMissionShips ( 1 );
									fMission.mDisplayMessage (kKetrick4_Experiment_msg, kScienceScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrick4_Experiment);
									break;
							}
						}
						break;

					case kTimerLimitSet_2: // waits for the Sulu voice to finish before the script continues
						// if the buttons was already pressed, move on when the voice finishes
						if (BlinkingButtonBlinking == 0)
						{
							fMission.mSetTimedInterval (kTimerLimitSet, 0);
						}
						else // set Timer2 flag to notify that the Sulu voice is already finished
							// and the script can move on once the button is pressed
							// or when timer3 expires
						{
							Timer2 = kExpired;
							// Timer 3 moves on automatically if the player doesn't/can't press a button
							fMission.mSetTimedInterval( kTimerLimitSet_3, 15 );
						}
						break;

					case kTimerLimitSet_3: // next tutorial segment in 15 seconds automatically
						// LNF Need to stop the button from blinking and make sure it doesn't 
						// accept clicks or it will advance the tutorial
						if ( Timer3 == kNotSet )
						{
							fMission.mDisplayMessage( kKetrick4_MovingOn_msg, kMissionScriptMessage );
							TutorialSegment++;
							BlinkingButtonBlinking = 0;
							Timer2 = kNotSet;
							fMission.mSetTimedInterval ( kTimerLimitSet, 0 );
						}

						Timer3 = kNotSet;

						break;

					case kTimerLimitSet_9: // extra timer to wait for text to scroll
						TutorialSegment++;
						extraTimer = 0; // maybe wait a second or so instead of 0 to smooth out waiting
						fMission.mSetTimedInterval (kTimerLimitSet, 0);
						break;

					case kTimerLimitSet_10: // wait for sound to finish
						if ( fMission.mVoiceSampleStillPlaying() == 0 )
						{
							fMission.mSetTimedInterval (kTimerLimitSet_9, extraTimer);
						}
						else
						{
							fMission.mSetTimedInterval (kTimerLimitSet_10, 1);
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			if ( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;
				if ( EnemyDestroyed == 3 )
				{
					ResponseToPlayerAction = 1;
					fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				fMission.mDisplayMessage (kKetrick4_Sigh_msg, kScienceScriptMessage);
				return kPlayerLoses;
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;

	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (EnemyDestroyed * 35);
	
	if (VictoryPoints >= 100)
	{
		TutorialPassed = true;
		MyVictoryLevel = kVictory;
	}
	else
	{
		MyVictoryLevel = kDefeat;
	}

	return (MyVictoryLevel);
	//return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if ( VictoryLevel == kVictory )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}
	fMission.mAddToDebriefingList (kDebrief_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
