#include "scriptinterface.h"

#include "Tut_Klingon5Text.h"
#include "Tut_Klingon5Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32 DelayReallyShort = 1;
int32 DelayQuiteShort = 4;
int32 DelayShort = 7;
int32 DelayLong = 12;
int32 DelayReallyLong = 18;
bool BadStuff = false;
bool EnemyCaptured = false;
bool TutorialPassed = false;
bool marinesSent = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID( void )
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 4;							// fifth choice in the menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart( tDynaverseScriptInfo& info )
{
	fMission.mSetPrestigePoints( 0, 0, 0, 0, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kKlingon );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip( tShipInGame& Ship )
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips( int32 Mode )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	PlayerTeam->mCreateShip( "K-D5G", kStartPosition_G );
	AlliedTeam->mCreateShip( "K-BS", kStartPosition_H );
	EnemyTeam->mCreateShip( "F-FF", kStartPosition_Z );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished( void )
{
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission( void )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tShipInfo* Ship;

	// configure players ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipCallbackEvents( kNotifyBeginTractor );
	Ship->mAccessSupplies (kSetSupplyCount, kSupplySpareParts, 0);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplySuicides, 2);
	Ship->mChangeName( kPlayerShipName_msg );
	Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );

	// configure enemy ship
	Ship = EnemyTeam->mGetFirstShip();
	Ship->mChangeName( kEnemyShipName_msg );
	Ship->mSetFinalDestination();
	Ship->mSetShipMaxSpeed( 9 );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mDamageSystem( kDamageMaxWeapons, 75.0 );
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );

	AlliedTeam->mGetFirstShip()->mDamageSystem( kDamageMaxWeapons, 100 );
	AlliedTeam->mGetFirstShip()->mAccessSupplies (kSetSupplyCount, kSupplySpareParts, 0);

	fMission.mSetTimedInterval( kTimerLimitSet, DelayQuiteShort );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip( tShipInfo* Ship )
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent( tShipInfo* Ship, iNotifyEvents Event )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	int32 BaseX = 0;
	int32 BaseY = 0;

	int32 WeaponsRemaining = 0;

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
				{
					switch( TutorialSegment )
					{
						case 0:		// mission start
							fMission.mDisplayMessage( kWhere_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kWhere2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kWhere3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kKetrickT5Welcome_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndKetrickT5Welcome );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 4:
							fMission.mDisplayMessage( kKetrickT5Object_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndKetrickT5Object );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 5:
							fMission.mDisplayMessage( kKetrickT5Chase_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndKetrickT5Chase );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 6:
							fMission.mDisplayMessage( kKetrickT5Security_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndKetrickT5Security );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 7:
							fMission.mDisplayMessage( kKetrickT5HitAndRunRaids_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample( kSndKetrickT5HitAndRunRaids );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 8:		// last message until all weapons on the enemy ship are destroyed
							fMission.mDisplayMessage( kKetrickT5DoHitAndRun_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5DoHitAndRun);
							fMission.mStartFlashingButton( "bA3_TOPBAR_S2" );
							break;

						case 9:		// all enemy ship weapons have been destroyed
							fMission.mDisplayMessage( kKetrickT5Capture_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5Capture);
							fMission.mStartFlashingButton( "A3_CAPTURE_SHIP" );
							break;

						case 10:
							if( EnemyCaptured == false )
							{
								fMission.mDisplayMessage( kKetrickT5CaptureProblems_msg, kMissionScriptMessage );
								fMission.mPlayVoiceSample (kSndKetrickT5CaptureProblems);
							}
							break;

/*						case 11:	// the enemy ship has been captured
							fMission.mDisplayMessage( kKetrickT5TakeControl_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5TakeControl);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 12: // wait here until ship reaches base
							fMission.mDisplayMessage( kKetrickT5BaseTargeted_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5BaseTargeted);
							AlliedTeam->mGetFirstShip()->mGetGridPosition (&BaseX, &BaseY);
							EnemyTeam->mGetFirstShip()->mTrackTacticalDistance (10.0, BaseX, BaseY);	// check when the captured E4 closes on the base
							break;
*/
						case 13:	// the enemy ship has arrived at the base
							fMission.mDisplayMessage( kKetrickT5SuicideShuttle_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5SuicideShuttle);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							Ship = PlayerTeam->mGetFirstShip();
							Ship->mDamageSystem( kDamageShuttleBaySystem, 0.0 );
							Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 4 );
							break;

						case 14:
							fMission.mDisplayMessage( kKetrickT5SuicideLaunch_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5SuicideLaunch);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 15:
							fMission.mDisplayMessage( kKetrickT5RepeatSuicide_msg, kMissionScriptMessage );
							fMission.mPlayVoiceSample (kSndKetrickT5RepeatSuicide);
							PlayerTeam->mGetFirstShip()->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 4 );
							fMission.mSetTimedInterval( kTimerLimitSet_4, 10 );
							break;
					}
				}
				break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;

			case kTimerLimitSet_3:
				// keep the AI ship's shield down
				Ship = EnemyTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageMaxShields, 100.0 );
				fMission.mSetTimedInterval ( kTimerLimitSet_3, 2);
				break;

			case kTimerLimitSet_4:
/*				Ship = PlayerTeam->mGetFirstShip();
				int32 SuicideCount = Ship->mAccessSupplies( kGetSupplyCount, kSupplySuicides );

				if( SuicideCount == 0 )
				{
					fMission.mDisplayMessage( kHaveMoreShuttles_msg, kNonCommScriptMessage );
					Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 2 );
				}

				fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
*/				break;

			case kTimerLimitSet_5:
				fMission.mDisplayMessage( kKetrickT5GoodWork_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndKetrickT5GoodWork);
				return kPlayerWins;
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyBeginTractor:
			if( PlayersTarget->mGetTeam() == kAlliedTeam )
			{
				fMission.mDisplayMessage( kBaseWhy_msg, kMissionScriptMessage );
				PlayerTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyBeginTractor );
			}
			else
			{
				fMission.mDisplayMessage( kKetrickT5WeaponsDisabled_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndKetrickT5WeaponsDisabled);
				Ship = PlayerTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );
				Ship->mDamageSystem( kDamageTransporterSystem, 0.0 );
				Ship = EnemyTeam->mGetFirstShip();
				Ship->mDamageSystem( kDamageMaxShields, 100.0 );
				fMission.mSetTimedInterval ( kTimerLimitSet_3, 2);  // keep AI shields down
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyDamage:
			{
				Ship = EnemyTeam->mGetFirstShip();
				WeaponsRemaining = Ship->mAccessSupplies( kGetHeavyWeaponCount, kDamageMaxWeapons );
				// if the weapon is destroyed move on in the tutorial
				if (( WeaponsRemaining == 0 ) && ( marinesSent == false ))
				{
					TutorialSegment++;
					fMission.mSetTimedInterval ( kTimerLimitSet, 2);
					fMission.mDisplayMessage( kKetrickT5Excellent_msg, kScienceScriptMessage );
					marinesSent = true;
				}
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				Ship->mClearCommands();
				fMission.mDisplayMessage( kKetrickT5CaptureComplete_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndKetrickT5CaptureComplete);
				fMission.mSetTimedInterval( kTimerLimitSet_5, 8 );
				EnemyCaptured = true;
			}
			else
			{
				BadStuff = true;
				return kPlayerLoses;
			}
			break;

		case kNotifyAtDistance:
			fMission.mDisplayMessage( kKetrickT5ShipReturned_msg, kMissionScriptMessage );
			fMission.mPlayVoiceSample (kSndKetrickT5ShipReturned);
			//TutorialSegment = 12;
			TutorialSegment++;
			fMission.mSetTimedInterval( kTimerLimitSet, DelayShort );
			// stop and damage the Fed
			EnemyTeam->mClearCommands();
			EnemyTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 100.0 );
			EnemyTeam->mGetFirstShip()->mDamageSystem( kDamageLeftWarpEngine, 100.0 );
			EnemyTeam->mGetFirstShip()->mDamageSystem( kDamageRightWarpEngine, 100.0 );
			EnemyTeam->mGetFirstShip()->mDamageSystem( kDamageInternalsWithPointsSilently, 180 );
			break;

		case kNotifyAtDeath:
			if(( Ship->mGetOriginalTeam() == kEnemyTeam ) && ( EnemyCaptured == true ))
			{
				fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				fMission.mDisplayMessage( kKetrickT5GoodWork_msg, kMissionScriptMessage );
				fMission.mPlayVoiceSample (kSndKetrickT5GoodWork);
				return kPlayerWins;
			}
			else
			{
				BadStuff = true;
				return kPlayerLoses;
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus( void )
{
	return( kAstoundingVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if(( PlayerTeam->mGetShipsInPlayCount() == 0 ) || ( BadStuff == true ))
	{
		fMission.mAddToDebriefingList( kBadDebrief_msg );
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList( kGoodDebrief_msg );
	}

	return;
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel(tDynaverseScriptInfo& infovoid )
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
