#include "scriptinterface.h"

#include "Tut_Klingon6Text.h"
#include "Tut_Klingon6Maps.h"

int32	TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
int32	DelayReallyShort = 1;
int32	DelayQuiteShort = 4;
int32	DelayShort = 7;
int32	DelayLong = 12;
int32	DelayReallyLong = 18;
bool	SuluAlive = true;
bool	TutorialPassed = false;

enum	eMode
{
	kStarting = 0,
	kSkirmish,
	kKlingonAttack,
	kWinner,
};
eMode	Mode;

int32 BlinkingButtonBlinking = 0;
int32 Timer2 = 0;
int32 Timer3 = 0;

enum eTimer2Expired
{
	kNotSet = 0,
	kExpired = 1
};

int32 CurrentMissionState = 0;
enum eTimer3MissionState
{
	kNormal = 0,
	kHailKetrick,
	kChallenge
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kKetrickTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 5;					// sixth choice in the menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, kKlingon);
	fMission.mSetTeamRace (kEnemyTeam, kFederation);
	fMission.mSetTeamRace (kKetrickTeam, kKlingon);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kKetrickTeam, kTeamAtWar, kGoalDestroy, 1.25, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* KetrickTeam = fMission.mGetTeamHandle (kKetrickTeam);
	tShipInfo* Ship;

	int32 x, y;

	if (Mode == kStarting)
	{
		PlayerTeam->mCreateShip ("K-D7", kStartPosition_G);
		KetrickTeam->mCreateShip ("K-D7C", kStartPosition_S);
	}

	if (Mode == kKlingonAttack)
	{
		Ship = PlayerTeam->mGetFirstShip ();
		Ship->mGetGridPosition (&x, &y);
		EnemyTeam->mCreateShip ("F-CA", kStartPosition_Specified, x+3, y+3);
		EnemyTeam->mCreateShip ("F-CC", kStartPosition_Specified, x+3, y+3);
		EnemyTeam->mCreateShip ("F-FFD+", kStartPosition_Specified, x-5, y-5);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* KetrickTeam = fMission.mGetTeamHandle (kKetrickTeam);
	tShipInfo* Ship;

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName (kAnarchy_msg);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 4);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, 2);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 2);
	Ship->mSetShipCallbackEvents(kNotifyBeginTractor);

	Ship = KetrickTeam->mGetFirstShip();
	Ship->mChangeName (kKole_msg);
	Ship->mSetShipCallbackEvents (kNotifyDamage);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 0);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, 0);
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 0);

	fMission.mSetTimedInterval (kTimerLimitSet, 1);	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 0);
	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* KetrickTeam = fMission.mGetTeamHandle (kKetrickTeam);
	tShipInfo* target = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyBeginTractor:
			if( target == KetrickTeam->mGetFirstShip() )
			{
				if (Mode == kStarting)
				{
					fMission.mDisplayMessage (kKetrickT6NoFair_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6NoFair);
					fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					Mode = kSkirmish;
				}
			}
			else
			{
				fPlayerShip->mSetShipCallbackEvents(kNotifyBeginTractor);
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				// increment the tutorial segment
				TutorialSegment++;
				BlinkingButtonBlinking = 0;
				// if the Sulu voice is already finished, move on immediately
				// otherwise, wait for Timer_2 to expire
				if (Timer2 == kExpired)
				{
					Timer2 = kNotSet;
					Timer3 = kExpired;
					fMission.mSetTimedInterval( kTimerLimitSet_6, 0 );
					fMission.mSetTimedInterval( kTimerLimitSet, 0 );
				}
			}
			break;

		case kNotifyAtTimedInterval:
		{
			switch (fEventID)
			{
				case kTimerLimitSet:
				{
					if (Mode == kStarting)
					{
						switch (TutorialSegment)
						{
							case 0:
								fMission.mDisplayMessage (kKetrickT6Where_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;

							case 1:
								fMission.mDisplayMessage (kKetrickT6Where2_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;

							case 2:
								fMission.mDisplayMessage (kKetrickT6Where3_msg, kNonCommScriptMessage);
								TutorialSegment++;
								fMission.mSetTimedInterval (kTimerLimitSet, 1);
								break;

							case 3:
								fMission.mDisplayMessage (kKetrickT6Welcome_msg, kMissionScriptMessage);
								fMission.mPlayVoiceSample (kSndKetrickT6Welcome);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;

							case 4:
								fMission.mDisplayMessage (kKetrickT6Purpose_msg, kMissionScriptMessage);
								fMission.mPlayVoiceSample (kSndKetrickT6Purpose);
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;

							case 5: //** COMM Button **
								fMission.mDisplayMessage (kKetrickT6Communications_msg, kMissionScriptMessage);
								fMission.mPlayVoiceSample (kSndKetrickT6Communications);
								fMission.mStartFlashingButton( "A3_OFFICER_COMM" );
								BlinkingButtonBlinking = 1;
								fMission.mSetTimedInterval (kTimerLimitSet_2, 1);
								break;

							case 6:
								fMission.mDisplayMessage (kKetrickT6HailMe_msg, kMissionScriptMessage);
								fMission.mPlayVoiceSample (kSndKetrickT6HailMe);
								CurrentMissionState = kHailKetrick;
								fMission.mSetTimedInterval (kTimerLimitSet_3, 1);
								break;

							case 7:
								if (Mode != kSkirmish)
								{
									fMission.mDisplayMessage (kKetrickT6Begin_msg, kMissionScriptMessage);
									fMission.mPlayVoiceSample (kSndKetrickT6Begin);
									Ship = KetrickTeam->mGetFirstShip();
									Ship->mClearCommButton (1);
									Ship->mClearCommButton (2);
									fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
									Mode = kSkirmish;
								}
								break;
						}
					}
					else if (Mode == kKlingonAttack)
					{
						fMission.mDisplayMessage (kKetrickT6Klingons_msg, kMissionScriptMessage);
						fMission.mPlayVoiceSample (kSndKetrickT6Klingons);
						mCreateMissionShips (kKlingonAttack);
					}
					else if (Mode == kWinner)
					{
						if ((SuluAlive == false) && (TutorialSegment != 9999))
						{
							fMission.mDisplayMessage (kKetrickT6ILived_msg, kMissionScriptMessage);
							fMission.mPlayVoiceSample (kSndKetrickT6ILived);
							TutorialSegment = 9999;
							fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
						}
						else
						{
							return kPlayerWins;
						}
					}
				}
				break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;


				case kTimerLimitSet_3:
					switch (CurrentMissionState)
					{
						case kChallenge:
							Ship = KetrickTeam->mGetFirstShip();
							Ship->mSetCommButton (1, kButtonAccept_text);
							Ship->mSetCommButton (2, kButtonDecline_text);
							CurrentMissionState = kNormal;
							// time limit for player to accept/reject challenge
							fMission.mSetTimedInterval ( kTimerTurnsSet, 2 );
							break;

						case kHailKetrick:
							Ship = KetrickTeam->mGetFirstShip();
							Ship->mSetCommButton (1, kButtonHail_text);
							CurrentMissionState = kNormal;
							// time limit for the player to hail sulu
							fMission.mSetTimedInterval ( kTimerTurnsSet_2, 2 );
							break;
					}
					break;

				case kTimerLimitSet_4:
					fMission.mDisplayMessage (kKetrickT6End_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6End);
					fMission.mSetTimedInterval (kTimerLimitSet, DelayShort);
					fMission.mSetTimedInterval ( kTimerLimitSet_5, 6 );
					break;

				case kTimerLimitSet_5:
					Ship = PlayerTeam->mGetFirstShip();
					Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0.0);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 4);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, 2);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 2);

					Ship = KetrickTeam->mGetFirstShip();
					Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0.0);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, 4);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, 2);
					Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 2);
					break;

				case kTimerLimitSet_6: // next tutorial segment in 15 seconds automatically
					// LNF Need to stop the button from blinking and make sure it doesn't 
					// accept clicks or it will advance the tutorial
					if ( Timer3 == kNotSet )
					{
						fMission.mDisplayMessage( kKetrickT6MovingOn_msg, kMissionScriptMessage );
						TutorialSegment++;
						BlinkingButtonBlinking = 0;
						Timer2 = kNotSet;
						fMission.mSetTimedInterval ( kTimerLimitSet, 0 );
					}

					Timer3 = kNotSet;

					break;

				case kTimerTurnsSet:
					if (Mode != kSkirmish)
					{
						fMission.mDisplayMessage (kKetrickT6Begin_msg, kMissionScriptMessage);
						//TutorialSegment++;
						fMission.mPlayVoiceSample (kSndKetrickT6Begin);
						Ship = KetrickTeam->mGetFirstShip();
						Ship->mClearCommButton (1);
						Ship->mClearCommButton (2);
						fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						Mode = kSkirmish;
					}
					break;

				case kTimerTurnsSet_2:
					// if the player doesn't push the hail button
					if (CurrentMissionState != kChallenge)
					{
						fMission.mDisplayMessage (kKetrickT6Hailed_msg, kMissionScriptMessage);
						fMission.mPlayVoiceSample (kSndKetrickT6Hailed);
						CurrentMissionState = kChallenge;
						fMission.mSetTimedInterval (kTimerLimitSet_3, 6);
					}
					break;
			}
		}
		break;

		case kNotifyDamage:
			if (Mode == kStarting)
			{
				fMission.mDisplayMessage (kKetrickT6NoFair_msg, kMissionScriptMessage);
				fMission.mPlayVoiceSample (kSndKetrickT6NoFair);
				fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				Mode = kSkirmish;
				Ship->mSetShipCallbackEvents (kNotifyDamage);
			}
			else if (Mode == kSkirmish)
			{
				// if the Kole is too damaged, stop the sim and start the Federation attack
				Ship = KetrickTeam->mGetFirstShip();
				if (Ship->mGetDamagePercent() >= 33)
				{
					fMission.mDisplayMessage (kKetrickT6Excellent_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6Excellent);
					fMission.mSetTimedInterval (kTimerLimitSet_4, 5);
					Mode = kKlingonAttack;
				}

				// if the Player's ship is too damaged, stop the sim and start the Federation attack
				Ship = PlayerTeam->mGetFirstShip();
				if (Ship->mGetDamagePercent() >= 33)
				{
					fMission.mDisplayMessage (kKetrickT6Bummer_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6Bummer);
					fMission.mSetTimedInterval (kTimerLimitSet_4, 8);
					Mode = kKlingonAttack;
				}

				// if the Federation attack has started
				// repair the ships in kTimerLimitSet_5
				if (Mode == kKlingonAttack)
				{
					fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
				}
				else
				{
					PlayerTeam->mGetFirstShip()->mSetShipCallbackEvents (kNotifyDamage);
					KetrickTeam->mGetFirstShip()->mSetShipCallbackEvents (kNotifyDamage);
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				return kPlayerLoses;
			}
			else if (Ship->mGetTeam() == kKetrickTeam)
			{
				fMission.mDisplayMessage (kKetrickT6KetrickDead_msg, kMissionScriptMessage);
				fMission.mPlayVoiceSample (kSndKetrickT6KetrickDead);
				SuluAlive = false;
			}
			else if (Ship->mGetTeam() == kEnemyTeam)
			{
				if (EnemyTeam->mGetShipsInPlayCount() == 0)
				{
					Mode = kWinner;
					fMission.mDisplayMessage (kKetrickT6GotThem_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6GotThem);
					fMission.mSetTimedInterval (kTimerLimitSet, DelayLong);
				}
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kButtonHail_text:
					fMission.mDisplayMessage (kKetrickT6Hailed_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6Hailed);
					CurrentMissionState = kChallenge;
					// expire timer waiting for player to push hail
					fMission.mSetTimedInterval (kTimerTurnsSet_2, 0);
					fMission.mSetTimedInterval (kTimerLimitSet_3, 6);
					break;

				case kButtonAccept_text:
					fMission.mDisplayMessage (kKetrickT6Accepted_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6Accepted);
					CurrentMissionState = kNormal;
					Mode = kSkirmish;
					break;

				case kButtonDecline_text:
					fMission.mDisplayMessage (kKetrickT6Declined_msg, kMissionScriptMessage);
					fMission.mPlayVoiceSample (kSndKetrickT6Declined);
					CurrentMissionState = kNormal;
					Mode = kSkirmish;
					break;
			}

			if (Mode == kSkirmish)
			{
				Ship = KetrickTeam->mGetFirstShip();
				Ship->mClearCommButton (1);
				Ship->mClearCommButton (2);
				fMission.mSetTeamRelations (kPlayerTeam, kKetrickTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
				fMission.mSetTimedInterval (kTimerTurnsSet, 0);
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (Mode == kWinner)
	{
		fMission.mAddToDebriefingList (kGoodDebrief_msg);
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList (kBadDebrief_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
