#include "scriptinterface.h"

#include "Tut_Lyran1Text.h"
#include "Tut_Lyran1Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool FightersDead = false;
bool TutorialPassed = false;

enum eCurrentRound
{
	Starting = 0,
	FirstRound,
	SecondRound,
};

eCurrentRound CurrentRound = Starting;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kFighterTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kLyranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;							// first choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, kLyran );
	fMission.mSetTeamRace( kEnemyTeam, kFederation );
	fMission.mSetTeamRace( kFighterTeam, kHydran );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kFighterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kFighterTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* FighterTeam = fMission.mGetTeamHandle( kFighterTeam );
	tShipInfo* Ship;

	int32 x, y;

	// Create a ship for the player to use
	if( CurrentRound == Starting )
	{
		PlayerTeam->mCreateShip( "L-CL", kStartPosition_G );
	}

	if( CurrentRound == FirstRound )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &x, &y );
		EnemyTeam->mCreateShip( "F-NCD", kStartPosition_Specified, x, y+2 );
	}

	if( CurrentRound == SecondRound )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &x, &y );
		FighterTeam->mCreateShip( "H-ST12", kStartPosition_Specified, x, y+2 );
		FighterTeam->mCreateShip( "H-ST12", kStartPosition_Specified, x-1, y+2 );
		FighterTeam->mCreateShip( "H-ST12", kStartPosition_Specified, x+1, y+2 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* FighterTeam = fMission.mGetTeamHandle( kFighterTeam );
	tShipInfo* Ship;

	// configure the player's ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kInstructor_name );

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if( CurrentRound == FirstRound )
	{
		Ship->mSetShipMaxSpeed( 0 );
		Ship->mDamageSystem( kDamageMaxShields, 100.0 );
		Ship->mDamageSystem( kDamageWeapon11, 100.0 );
		Ship->mDamageSystem( kDamageWeapon12, 100.0 );
		Ship->mDamageSystem( kDamageWeapon13, 100.0 );
		Ship->mDamageSystem( kDamageMaxShields, 100.0 );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 0 );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
		Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
		Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
		Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
		Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
		Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
		Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	}
	else
	{
		Ship->mDamageSystem( kDamageHeavyWeapon1, 100.0 );		// Fusion-less fighters
		Ship->mDamageSystem( kDamageHeavyWeapon2, 100.0 );
		Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
		Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* FighterTeam = fMission.mGetTeamHandle( kFighterTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					switch( TutorialSegment )
					{
						case 0:
							fMission.mDisplayMessage( kWhere_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kWhere2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kWhere3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kWelcome_msg );
							fMission.mPlayVoiceSample( kSndWelcome );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 4:
							fMission.mDisplayMessage( kBasicESG_msg );
							fMission.mPlayVoiceSample( kSndBasicESG );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 5:
							if( fPlayerShip->mGetAlertStatus() == kAlertRed )
							{
								TutorialSegment++;
								fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							}
							else
							{
								fMission.mDisplayMessage( kRedAlert_msg );
								fMission.mPlayVoiceSample( kSndVoiceRedAlert );
							}
							break;

						case 6:
							fMission.mStartFlashingButton( "PlayerSchem.Weapon6" );
							fMission.mDisplayMessage( kClickESG_msg );
							fMission.mPlayVoiceSample( kSndClickESG );
							break;

						case 7:
							fMission.mDisplayMessage( kESGPanel_msg );
							fMission.mPlayVoiceSample( kSndESGPanel );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 8:
							fMission.mDisplayMessage( kCapacitor_msg );
							fMission.mPlayVoiceSample( kSndCapacitor );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 9:
							fMission.mDisplayMessage( kRadius_msg );
							fMission.mPlayVoiceSample( kSndRadius );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 10:
							fMission.mDisplayMessage( kNotice_msg );
							fMission.mPlayVoiceSample( kSndNotice );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 11:
							fMission.mDisplayMessage( kActivation_msg );
							fMission.mPlayVoiceSample( kSndActivation );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 12:
							fMission.mDisplayMessage( kReduction_msg );
							fMission.mPlayVoiceSample( kSndReduction );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 13:
							CurrentRound = FirstRound;
							mCreateMissionShips( CurrentRound );
							fMission.mDisplayMessage( kFederation_msg );
							fMission.mPlayVoiceSample( kSndFederation );
							fMission.mSetTimedInterval( kTimerTurnsSet, 6 );
							break;

						case 14:
							fMission.mDisplayMessage( kFedDead_msg );
							fMission.mPlayVoiceSample( kSndFedDead );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 15:
							CurrentRound = SecondRound;
							mCreateMissionShips( CurrentRound );
							fMission.mDisplayMessage( kHydranFighters_msg );
							fMission.mPlayVoiceSample( kSndHydranFighters );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 16:
							fMission.mDisplayMessage( kESGDamage_msg );
							fMission.mPlayVoiceSample( kSndESGDamage );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 17:
							fMission.mDisplayMessage( kESGDamagePart2_msg );
							fMission.mPlayVoiceSample( kSndESGDamagePart2 );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 18:
							fMission.mDisplayMessage( kWeaponGrouping_msg );
							fMission.mPlayVoiceSample( kSndWeaponGrouping );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 19:
							if( FightersDead == true )
							{
								fMission.mDisplayMessage( kFightersDead_msg );
								fMission.mPlayVoiceSample( kSndFightersDead );
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							else
							{
								fMission.mSetTimedInterval( kTimerLimitSet, 3 );
							}
							break;

						case 20:
							return kPlayerWins;
					}
					break;

					
			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;

			case kTimerTurnsSet:
				if( EnemyTeam->mGetShipsInPlayCount() == 1 )
				{
					fMission.mDisplayMessage( kKillFed_msg );
					fMission.mPlayVoiceSample( kSndKillFed );
					Ship = EnemyTeam->mGetFirstShip();
					Ship->mDamageSystem( kDamageWeaponsAndSystems, 80.0 );
					Ship->mDamageSystem( kDamageHeavyWeapon1, 100.0 );
					Ship->mDamageSystem( kDamageHeavyWeapon2, 100.0 );
					Ship->mDamageSystem( kDamageHeavyWeapon7, 100.0 );
				}
				break;
			}
			break;

		case kNotifyAlertStatusChange:
			if(( Ship->mGetAlertStatus() == kAlertRed ) && ( TutorialSegment >= 5 ))
			{
				TutorialSegment = 6;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			else
			{
				Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			else if( Ship->mGetTeam() == kEnemyTeam )
			{
				TutorialSegment = 14;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			else if( Ship->mGetTeam() == kFighterTeam )
			{
				if( FighterTeam->mGetShipsInPlayCount() == 0 )
				{
					FightersDead = true;
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return( kAstoundingVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if( PlayerTeam->mGetShipsInPlayCount() == 0 )
	{
		fMission.mAddToDebriefingList( kDied_msg );
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList( kDebrief_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
