#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 gScenarioAct = 0;
int32 gBaseDefenseKilled = 0;
int32 gBaseDefenseCaptured = 0;
int32 gNumShipsAddedToCommand = 0;
iTruth gBaseCaptured = kFalse; 
iTruth PlayerDisengaged = kFalse;
int32 PiratesDestroyed = 0;
int32 EnemyDestroyed = 0;
int32 PiratesCaptured = 0;
int32 EnemyCaptured = 0;
float BaseHealth = 0.0;
float DefenderHealth = 0.0;
float PiratesHealth = 0.0;
float EnemyRaceHealth = 0.0;
int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
eVictoryLevels MyVicLevel = kDevastatingDefeat;


enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kBaseTeam,
	kDefenderTeam,
	kPiratesTeam,
	kEnemyRaceTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kBaseTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kWarzone;
	Specs->fPlayerRace = kLyranMission | kKlingMission | kRomulanMission | kGornMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 10;
	Specs->fMaxDate = 40;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (360, 240, 100, 50, 00);

		// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kBaseTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kDefenderTeam, info.fPlayerRace);
	

	fMission.mSetTeamRace (kEnemyRaceTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kPiratesTeam, kOrion);

	// setup team relations and target goals


	fMission.mSetTeamRelations (kPlayerTeam, kEnemyRaceTeam, kTeamAtWar, kGoalCapture, 1.0);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalCapture, 1.0);
	fMission.mSetTeamRelations (kPlayerTeam, kPiratesTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	fMission.mSetTeamRelations (kEnemyRaceTeam, kDefenderTeam,  kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyRaceTeam, kBaseTeam,  kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyRaceTeam, kPlayerTeam,  kTeamAtWar, kGoalDestroy, 1.0);
	
	fMission.mSetTeamRelations (kPiratesTeam, kBaseTeam, kTeamHostile, kGoalCapture, 1.0, kTrue);
	fMission.mSetTeamRelations (kPiratesTeam, kDefenderTeam, kTeamHostile, kGoalCapture, 1.0, kTrue);
	fMission.mSetTeamRelations (kPiratesTeam, kEnemyRaceTeam, kTeamHostile, kGoalCapture, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kDefenderTeam, kPlayerTeam,  kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kDefenderTeam, kBaseTeam,  kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kBaseTeam, kPlayerTeam,  kTeamAtWar, kGoalDestroy, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* DefenderTeam = fMission.mGetTeamHandle (kDefenderTeam);
	tTeamInfo* EnemyRaceTeam = fMission.mGetTeamHandle (kEnemyRaceTeam);
	tTeamInfo* PiratesTeam = fMission.mGetTeamHandle (kPiratesTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	int32 NumPlayerShips = PlayerTeam->mGetShipsInPlayCount();
	int32 PlayerCombat = PlayerTeam->mGetCombatBPV();

	int32 LowBPV = (float) PlayerCombat / (float) NumPlayerShips * 0.8f;
	int32 HighBPV = (float) PlayerCombat / (float) NumPlayerShips * 1.3f;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	int32 EstimatedEnemyBPV = 5 * TotalBPV + 200;  // 5 enemy ships and the bats
	int32 OutGunned = EstimatedEnemyBPV - PlayerCombat;

	int32 DifLev = fMission.mGetDifficultyLevel();
	if (Mode == 0)
	{
		// small escorts....
		DefenderTeam->mCreateShip ( LowBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Y);
		DefenderTeam->mCreateShip ( LowBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Z);

		// I want the player to have 6 ships under their command.
		gNumShipsAddedToCommand = 6 - PlayerTeam->mGetShipsInPlayCount();
		int32 EscortBPV = 0.65f * (float) OutGunned / (float) gNumShipsAddedToCommand;

		switch ( DifLev )
		{
		case 0:
			{
				EscortBPV *= 1.35;
				PlayerCombat *= 0.5;
			}
			break;
		case 1:
			{
				PlayerCombat *= 0.75;
			}
			break;
		case 2:
			{
				EscortBPV *= 0.75;
				PlayerCombat *= 1.0;
			}
			break;
		}

		// create research base
		BaseTeam->mCreateShip ( PlayerCombat, kClassBaseStation, kClassBattleStation, 0, kStartPosition_X);

		for ( int32 j = 0; j< gNumShipsAddedToCommand; j++ )
		{
			PlayerTeam->mCreateShip (EscortBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_H + j);
		}

	}

	// pirates are created after the first E4 takes damage
	if ( Mode == 1 )
	{
		PiratesTeam->mCreateShip ( LowBPV, kClassFrigate, kClassFrigate, 50, kStartPosition_Z);
		if ( DifLev > 0 )
		{
			PiratesTeam->mCreateShip ( TotalBPV, kClassDestroyer, kClassHeavyCruiser, 0, kStartPosition_Z);
		}
		if ( DifLev > 1 )
		{
			PiratesTeam->mCreateShip ( HighBPV, kClassLightCruiser, kClassHeavyCruiser, 50, kStartPosition_Z);
		}
		return;
	}

	if ( Mode == 2 )
	{

		EnemyRaceTeam->mCreateShip (LowBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_T);
		if (DifLev > 0 )
		{
			EnemyRaceTeam->mCreateShip (HighBPV, kClassFrigate, kClassBattleship, 0, kStartPosition_U);
		}
		return;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
//	char msg[200];
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* DefenderTeam = fMission.mGetTeamHandle( kDefenderTeam );
	tShipInfo* Ship;

	fMission.mDisplayMessage( kReportingForDuty_msg, kFleetScriptMessage );
	fMission.mDisplayMessage( kDebugBrief_msg, kNonCommScriptMessage );

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipThrust (15);
		Ship->mTrackTacticalDistance (50.0, 10, 30);			// setup notice at half-way point
	}

	for (Ship = BaseTeam->mGetFirstShip (); Ship != NULL; Ship = BaseTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget (PlayerTeam->mGetFirstShip(), kVeryHighPriority);
	}

	for (Ship = DefenderTeam->mGetFirstShip (); Ship != NULL; Ship = DefenderTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget (PlayerTeam->mGetFirstShip(), kVeryHighPriority);
	}

	//fMission.mSetTimedInterval (kTimerLimitSet, 300); // bring in the pirates after 5 minutes
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* EnemyRaceTeam = fMission.mGetTeamHandle (kEnemyRaceTeam);
	tTeamInfo* PiratesTeam = fMission.mGetTeamHandle (kPiratesTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	tShipInfo* ShipIter;

	switch ( gScenarioAct )
	{
		case 1:
			{
				for (ShipIter = PiratesTeam->mGetFirstShip (); ShipIter != NULL; ShipIter = PiratesTeam->mGetNextShip ())
				{
					ShipIter->mSetShipTarget (PlayerTeam->mGetFirstShip(), kVeryHighPriority);
					ShipIter->mSetShipThrust (15);
				}
				break;
			}
		case 2:
			{
				for (ShipIter = EnemyRaceTeam->mGetFirstShip (); ShipIter != NULL; ShipIter = EnemyRaceTeam->mGetNextShip ())
				{
					ShipIter->mSetShipTarget (PlayerTeam->mGetFirstShip(), kVeryHighPriority);
					ShipIter->mSetShipThrust (15);
				}
				break;
			}
	}


	return;
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* DefenderTeam = fMission.mGetTeamHandle (kDefenderTeam);
	tTeamInfo* PiratesTeam = fMission.mGetTeamHandle (kPiratesTeam);
	tTeamInfo* EnemyRaceTeam = fMission.mGetTeamHandle (kEnemyRaceTeam);

	BaseHealth = 100.0f - BaseTeam->mGetTotalDamagePercent();
	DefenderHealth = 100.0f - DefenderTeam->mGetTotalDamagePercent();
	PiratesHealth = 100.0f - PiratesTeam->mGetTotalDamagePercent();
	EnemyRaceHealth = 100.0f - EnemyRaceTeam->mGetTotalDamagePercent();

	int32 Pirates = PiratesTeam->mGetShipCount( kShipCreated ) - PiratesTeam->mGetShipCount( kShipDestroyed );
	Pirates -= PiratesTeam->mGetShipCount( kShipCaptured );

	int32 Enemy = EnemyRaceTeam->mGetShipCount( kShipCreated ) - EnemyRaceTeam->mGetShipCount( kShipDestroyed );
	Enemy -= EnemyRaceTeam->mGetShipCount( kShipCaptured );

	int32 Defenders = DefenderTeam->mGetShipCount( kShipCreated ) - DefenderTeam->mGetShipCount( kShipDestroyed );
	Defenders -= DefenderTeam->mGetShipCount( kShipCaptured );

	int32 PlayerFleet = PlayerTeam->mGetShipsInPlayCount();

	switch (Event)
	{
		case kNotifyDisengaged:
		{
			if (( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				return kPlayerWins;
			}
		}

		case kNotifyOnCapture:
			{
				if (Ship->mGetOriginalTeam () == kPlayerTeam)
				{
					PlayerCaptured++;
					// if the player has lost all of their ships, then they lose
					tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );				
					int32 shipsLost = PlayerTeam->mGetShipCount( kShipDestroyed );
					shipsLost += PlayerTeam->mGetShipCount( kShipCaptured );
					int32 shipsCreated = PlayerTeam->mGetShipCount( kShipCreated );
					if( PlayerTeam->mGetShipsInPlayCount() == 0 )
					{
						return kPlayerCaptured;
					}
				}

				if (Ship->mGetOriginalTeam () == kBaseTeam)
				{
					fMission.mDisplayMessage ( kTheBaseHasBeenCaptured );
					gBaseCaptured = true;
					
					if ( PiratesHealth > 0.0 )
					{
						fMission.mDisplayMessage( kMustDestroyPirates_msg );
					}

					if ( EnemyRaceHealth > 0.0 )
					{
						fMission.mDisplayMessage( kMustDestroyEnemy_msg );
					}

				}

				if (Ship->mGetOriginalTeam () == kDefenderTeam)
				{
					gBaseDefenseCaptured++;
					Defenders--;
					// the player has killed a base escort faster than the time limit... bring in the pirates
					if ( gScenarioAct == 0 )  
					{
						fMission.mDisplayMessage ( kPirates_msg );
						mCreateMissionShips (1);
						gScenarioAct = 1;
					}
				}

				if ( Ship->mGetOriginalTeam () == kPiratesTeam )
				{
					PiratesCaptured++;
					Pirates--;
					if ( gScenarioAct == 1 )
					{
						fMission.mDisplayMessage (kEnemy_msg);
						mCreateMissionShips (2);
						gScenarioAct = 2;
					}
				}

 				if ( Ship->mGetOriginalTeam () == kEnemyRaceTeam )
				{
					EnemyCaptured++;
					Enemy--;
				}

				if ( ( Enemy < 1 ) && ( Pirates < 1 ) && ( Defenders < 1 ) && gBaseCaptured && ( gScenarioAct > 0 ) )
				{
					return kPlayerWins;
				}
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam () == kPlayerTeam)
				{
					PlayerDestroyed++;
					if ( PlayerFleet ==  0 )
					{
						return kPlayerDestroyed;
					}
				}

				if (Ship->mGetTeam () == kBaseTeam)
				{		
					// if the base is destroyed for any reason they lose
					fMission.mDisplayMessage ( kLostBase_msg );
					return kPlayerLoses;
				}

				if (Ship->mGetTeam () == kDefenderTeam)
				{		

					gBaseDefenseKilled++;
					if ( gBaseDefenseCaptured > 0 )
					{
						gBaseDefenseCaptured--;
					}
					fMission.mDisplayMessage ( kCaptureNotDestory_msg );
					// the player has killed a base escort faster than the time limit... bring in the pirates
					if ( gScenarioAct == 0 )  
					{
						fMission.mDisplayMessage ( kPirates_msg );
						mCreateMissionShips (1);
						gScenarioAct = 1;
					}
				}

				if ( Ship->mGetTeam () == kPiratesTeam )
				{
					PiratesDestroyed++;
					if ( gScenarioAct == 1 )
					{
						fMission.mDisplayMessage (kEnemy_msg);
						mCreateMissionShips (2);
						gScenarioAct = 2;
					}
				}

				if ( Ship->mGetTeam () == kEnemyRaceTeam )
				{
					EnemyDestroyed++;
					if ( gScenarioAct == 1 )
					{
						fMission.mDisplayMessage (kEnemy_msg);
					}
				}

				if ( ( Enemy < 1 ) && ( Pirates < 1 ) && ( Defenders < 1 ) && gBaseCaptured && ( gScenarioAct > 0 ) )
				{
					return kPlayerWins;
				}

			break;
			}

		case kNotifyAtTimedInterval:
			{
				if ( gScenarioAct == 0 )
				{
					fMission.mDisplayMessage (kPirates_msg);
					fMission.mSetTimedInterval (kTimerLimitSet, 240); // bring in the enemy race in 4 minutes
					mCreateMissionShips (1);
					gScenarioAct = 1;

				}
				else if ( gScenarioAct == 1 )
				{
					fMission.mDisplayMessage (kEnemy_msg);
					mCreateMissionShips (2);
					gScenarioAct = 2;
				}
				break;
			}
	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	// ENF possible tweak:  use the health levesl for victory calculation
	if ( PlayerDisengaged )
	{
		if ( gBaseCaptured )
		{
			return kDefeat;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}

	if ( gBaseCaptured )
	{
		if ( ( ( EnemyCaptured + PiratesCaptured ) > 1 ) && ( gBaseDefenseCaptured > 1 ) )
		{
			return kAstoundingVictory;
		}
		else if ( ( gBaseDefenseCaptured > 0 ) )
		{
			return kVictory;
		}
		else
		{
			return kDraw;
		}
	}
	else
	{
		return kDevastatingDefeat;
	}
	///////

	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_1;
	}
	else
	{
		return 0;
	}
}
