#include "scriptinterface.h"

#include "Uni_BlackHoleManiaText.h"
#include "Uni_BlackHoleManiaMaps.h"

int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
iTruth PlayerDisengaged = kFalse;
int32 TotalBlackHoles = 0;
bool NewBlackHole = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 20;
	Specs->fMaxDate = 40;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 400, 225, 100, 25, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = fMission.mRandomInt32( kStartPosition_G, kStartPosition_I );
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tShipInfo* Ship;

	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;
	int32 RandomPosition = 0;
	int32 TotalShips = 0;

	// Create the enemy ship at the beginning of the mission
	if( Mode == 0 )
	{
		// Get the total BPV of the player and modify by 80% to 130%
		int32 LowBPV = PlayerTeam->mGetCombatBPV() * .8;
		int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.25;
		int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

		// pick a random number of ships
		int32 NumberEnemyShips = fMission.mRandomInt32 (2, PlayerTeam->mGetShipsInPlayCount());

		int32 DifLev = fMission.mGetDifficultyLevel();

		switch ( DifLev )
		{
		case 0:
			{
				TotalBPV *= 0.70f;
			}
			break;
		case 1:
			{
				// do nothing
			}
			break;
		case 2:
			{
				
				//NumberEnemyShips++;
				TotalBPV *= 1.5f;
			}
			break;
		}


		// Any size ship within the appropriate BPV and a 33% chance of a variant
		EnemyTeam->mCreateFleet(NumberEnemyShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 33, kStartPosition_T);
	}

	// Let's make a new black hole!  Yea!
	if( Mode == 1 )
	{
		NewBlackHole = true;
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&OldX, &OldY);
		NewX = OldX + fMission.mRandomInt32( -3, 3 );
		NewY = OldY + fMission.mRandomInt32( -3, 3 );

		if(( NewX <= 1 ) || ( NewX >= 40 ))
		{
			NewBlackHole = false;
		}

		if(( NewY <= 1 ) || ( NewY >= 40 ))
		{
			NewBlackHole = false;
		}

		if(( NewX == OldX ) && ( NewY == OldY ))
		{
			NewBlackHole = false;
		}
		
		if( NewBlackHole == true )
		{
			fMission.mCreateMapObject( 'B', NewX, NewY );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if( EnemyTeam->mGetShipsInPlayCount() > 1 )
	{
		return( kBriefingMultiple_msg );
	}
	else
	{
		return( kBriefing_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	int32 RandomTurns = 0;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;


	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for( Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip () )
	{
		Ship->mSetShipThrust( 5 );
	}

	for( Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip () )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	// Start the initial timer
	RandomTurns = fMission.mRandomInt32( 1, 2 );
	fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	int32 RandomTurns = 0;
	int32 RandomMessage = 0;

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			if( TotalBlackHoles == 0 )
			{
				fMission.mDisplayMessage( kScienceStarting_msg, kScienceScriptMessage );
				RandomTurns = fMission.mRandomInt32( 1, 2 );
				fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
				TotalBlackHoles++;
			}
			else if( TotalBlackHoles < 15 )
			{
				mCreateMissionShips( 1 );

				if( NewBlackHole == true )
				{
					RandomMessage = fMission.mRandomInt32( kNewSingularity1_msg, kNewSingularity3_msg );
					fMission.mDisplayMessage( RandomMessage, kScienceScriptMessage );
					TotalBlackHoles++;
				}

				RandomTurns = fMission.mRandomInt32( 1, 4 );
				fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					fMission.mDisplayMessage( kEnemyNoResponse_msg, kMissionScriptMessage );
					break;
			}
			break;
			
		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				return kPlayerWins;
			}
		break;

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerCaptured++;
			}
			else
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;
				
				if ( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam () == kPlayerTeam)
			{
				PlayerDestroyed++;
			}
			else
			{
				// see if this is all over.
				if ( Enemies == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			break;
	}
	
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if ( PlayerDisengaged )
	{
		if ( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if ( PlayerHealth < ( EnemyHealth / 2 ) )
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}


	if ( ( PlayerHealth > 90.0 ) || ( EnemyCaptured ) )
	{
		return kAstoundingVictory;
	}


	if ( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
