#include "scriptinterface.h"

#include "Uni_BullyText.h"
#include "Uni_BullyMaps.h"

int32 Proof = 0;
bool Killer = false;
bool PlayerLeft = false;
bool BullyCaptured = false;
bool WorkerCaptured = false;
bool BullyStopped = false;
bool WorkerStopped = false;
bool BullyProbed = false;
bool WorkerProbed = false;
bool Scanned = false;
bool ShipThreatened = false;
tShipInfo* WorkerID;
tShipInfo* BullyID;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kBullyTeam, kWorkerTeam, kAlliedTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kPlayerCapital;
	Specs->fPlayerRace = kFedMission | kGornMission | kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = 150;
	Specs->fMinDate = 4;
	Specs->fMaxDate = 12;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (100, 60, 30, 0, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kBullyTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kWorkerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kBullyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kWorkerTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kBullyTeam, kWorkerTeam, kTeamAtWar, kGoalCripple, 1.0 );
	fMission.mSetTeamRelations( kWorkerTeam, kBullyTeam, kTeamAtWar, kGoalCripple, 1.0 );
	fMission.mSetTeamRelations( kBullyTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kWorkerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BullyTeam = fMission.mGetTeamHandle( kBullyTeam );
	tTeamInfo* WorkerTeam = fMission.mGetTeamHandle( kWorkerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	// the base & allied freighters
	AlliedTeam->mCreateShip( kMaxBPV, kClassBaseStation, kClassBaseStation, 0, kStartPosition_I );
	AlliedTeam->mCreateFleet( 3, kMaxBPV, kClassFreighter, kClassFreighter, 1, kStartPosition_L );

	// the bully freighter
	BullyTeam->mCreateShip( kMaxBPV, kClassFreighter, kClassFreighter, 1, kStartPosition_K );

	// the other freighter
	WorkerTeam->mCreateShip( kMaxBPV, kClassFreighter, kClassFreighter, 1, kStartPosition_J );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BullyTeam = fMission.mGetTeamHandle( kBullyTeam );
	tTeamInfo* WorkerTeam = fMission.mGetTeamHandle( kWorkerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tShipInfo* Ship;

	int32 BaseX = 0;
	int32 BaseY = 0;

	// configure player ship
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure NPC ships
	AlliedTeam->mGetFirstShip()->mGetGridPosition( &BaseX, &BaseY );	// find the base coordinates

	for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
		if( Ship->mGetClassType() == kClassBaseStation )
		{
			Ship->mSetCommButton( 2, kButtonRequestHelp_text );
		}
		else
		{
			Ship->mSetShipAIControlled( kFalse );
		}
	}

	Ship = BullyTeam->mGetFirstShip();
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mDamageSystem( kDamageInternalsWithPoints, 20 );
	Ship->mTrackTacticalDistance( 10.0, BaseX, BaseY );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mSetOfficerRank( kWeaponsOfficer, kRookie );
	Ship->mSetOfficerRank( kHelmOfficer, kRookie );
	Ship->mSetOfficerRank( kScienceOfficer, kRookie );
	Ship->mSetShipMaxSpeed( 8 );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	Ship->mChangeAIOfficerState( kFighterOfficer, kFalse );
	BullyID = Ship;

	Ship = WorkerTeam->mGetFirstShip();
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
	Ship->mTrackTacticalDistance( 10.0, BaseX, BaseY );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mSetOfficerRank( kWeaponsOfficer, kRookie );
	Ship->mSetOfficerRank( kHelmOfficer, kRookie );
	Ship->mSetOfficerRank( kScienceOfficer, kRookie );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
	Ship->mChangeAIOfficerState( kFighterOfficer, kFalse );
	Ship->mSetShipMaxSpeed( 6 );
	WorkerID = Ship;

	// Start the initial timers
	fMission.mSetTimedInterval( kTimerLimitSet, 3 );
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BullyTeam = fMission.mGetTeamHandle( kBullyTeam );
	tTeamInfo* WorkerTeam = fMission.mGetTeamHandle( kWorkerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kBullyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kWorkerTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kBullyTeam, kWorkerTeam, kTeamAtWar, kGoalCripple, 2.0 );
					fMission.mSetTeamRelations( kWorkerTeam, kBullyTeam, kTeamAtWar, kGoalCripple, 1.0 );
					fMission.mSetTeamRelations( kBullyTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kWorkerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;

				case kTimerLimitSet_2:
					Ship = BullyTeam->mGetFirstShip();
					if( Ship != NULL )
					{
						Ship->mDamageSystem( kDamageMaxShields, 100.0 );
						fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
					}
					break;

				case kTimerLimitSet_3:
					Ship = WorkerTeam->mGetFirstShip();
					if( Ship != NULL )
					{
						Ship->mDamageSystem( kDamageMaxShields, 100.0 );
						fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
					}
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kBaseOrders_msg, kFleetScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
					break;

				case kTimerTurnsSet_2:
					if( Proof == 0 )
					{
						fMission.mDisplayMessage( kPissedGuys_msg, kScienceScriptMessage );
					}
					break;

				case kTimerTurnsSet_3:
					return kPlayerWins;
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					switch( Ship->mGetTeam() )
					{
						case kBullyTeam:
							fMission.mDisplayMessage( kBullyPissed_msg, kMissionScriptMessage, Ship );
							Ship->mSetCommButton( 1, kButtonLower_text );
							Ship->mSetCommButton( 2, kButtonStop_text );
							break;

						case kWorkerTeam:
							fMission.mDisplayMessage( kWorkerPissed_msg, kMissionScriptMessage, Ship );
							Ship->mSetCommButton( 1, kButtonLower_text );
							Ship->mSetCommButton( 2, kButtonStop_text );
							break;

						case kAlliedTeam:
							if( Ship->mGetClassType() == kClassBaseStation )
							{
								fMission.mDisplayMessage( kAlliedBase_msg, kFleetScriptMessage );
							}
							else
							{
								fMission.mDisplayMessage( fMission.mRandomInt32( kAlliedShipA_msg, kAlliedShipC_msg ), kFleetScriptMessage, Ship );
							}
							break;
					}
					break;

				case kButtonLower_text:
					if( Ship->mGetTeam() == kBullyTeam )
					{
						if( Ship->mGetDamagePercent() >= 65.0 )
						{
							fMission.mDisplayMessage( kBullyLowerYes_msg, kMissionScriptMessage, Ship );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 3 );
						}
						else
						{
							fMission.mDisplayMessage( kBullyLowerNo_msg, kMissionScriptMessage, Ship );
							Ship->mSetCommButton( 1, kButtonLower_text );
						}
					}
					else
					{
						if(( Ship->mGetDamagePercent() >= 65.0 ) || ( BullyStopped == true ))
						{
							fMission.mDisplayMessage( kWorkerLowerYes_msg, kMissionScriptMessage, Ship );
							fMission.mSetTimedInterval( kTimerLimitSet_3, 3 );
						}
						else
						{
							fMission.mDisplayMessage( kWorkerLowerNo_msg, kMissionScriptMessage, Ship );
							Ship->mSetCommButton( 1, kButtonLower_text );
						}
					}
					break;

				case kButtonStop_text:
					if( Ship->mGetTeam() == kBullyTeam )
					{
						if(( Ship->mGetDamagePercent() >= 90.0 ) && ( WorkerStopped == true ))
						{
							fMission.mDisplayMessage( kBullyStopYes_msg, kMissionScriptMessage, Ship );
							fMission.mSetTeamRelations( kBullyTeam, kWorkerTeam, kTeamNeutral, kGoalIgnore, 1.0 );
							BullyStopped = true;

							if(( WorkerStopped == true ) || ( WorkerCaptured == true ))
							{
								fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
							}
						}
						else
						{
							fMission.mDisplayMessage( kBullyStopNo_msg, kMissionScriptMessage, Ship );
							Ship->mSetCommButton( 2, kButtonStop_text );
						}
					}
					else
					{
						if(( Ship->mGetDamagePercent() >= 50.0 ) || ( BullyStopped == true ))
						{
							fMission.mDisplayMessage( kWorkerStopYes_msg, kMissionScriptMessage, Ship );
							fMission.mSetTeamRelations( kWorkerTeam, kBullyTeam, kTeamNeutral, kGoalIgnore, 1.0 );
							WorkerStopped = true;

							if(( BullyStopped == true ) || ( BullyCaptured ))
							{
								fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
							}
						}
						else
						{
							fMission.mDisplayMessage( kWorkerStopNo_msg, kMissionScriptMessage, Ship );
							Ship->mSetCommButton( 2, kButtonStop_text );
						}
					}

					ShipThreatened = true;

					break;

				case kButtonRequestHelp_text:
					fMission.mDisplayMessage( kBaseHelp_msg, kFleetScriptMessage );
					break;

				case kButtonTransmitLogs_text:
					if( Ship->mGetTeam() == kBullyTeam )
					{
						fMission.mDisplayMessage( kBullyLogs_msg, kMissionScriptMessage, Ship );
						Proof++;
					}
					else if( Ship->mGetTeam() == kWorkerTeam )
					{
						fMission.mDisplayMessage( kWorkerLogs_msg, kMissionScriptMessage, Ship );
						Proof++;
					}
					break;
			}
			break;
			
		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kBullyTeam )
			{
				BullyCaptured = true;
				fMission.mDisplayMessage( kGotBully_msg, kMissionScriptMessage, Ship );
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetCommButton( 1, kButtonTransmitLogs_text );
				Ship->mClearCommButton( 2 );

				if(( WorkerCaptured == true ) || ( WorkerStopped == true ))
				{
					fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
				}
			}
			else if( Ship->mGetOriginalTeam() == kWorkerTeam )
			{
				WorkerCaptured = true;
				fMission.mDisplayMessage( kGotWorker_msg, kMissionScriptMessage, Ship );
				Ship->mSetShipAIControlled( kFalse );
				Ship->mSetCommButton( 1, kButtonTransmitLogs_text );
				Ship->mClearCommButton( 2 );

				if(( BullyCaptured == true ) || ( BullyStopped == true ))
				{
					fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else
			{
				Killer = true;
				return kPlayerLoses;
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerLeft = true;
				return kPlayerDisengaged;
			}
			break;

		case kNotifyAtDistance:
			if( Ship->mGetTeam() == kBullyTeam )
			{
				if(( BullyStopped == false ) && ( BullyCaptured == false ))
				{
					BullyStopped = true;
					fMission.mDisplayMessage( kBullyAtBase_msg, kMissionScriptMessage, Ship );
					Ship->mSetShipAIControlled( kFalse );
					Ship->mSetCommButton( 1, kButtonTransmitLogs_text );
					Ship->mClearCommButton( 2 );

					if(( WorkerCaptured == true ) || ( WorkerStopped == true ))
					{
						fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
					}
				}
			}
			else if( Ship->mGetTeam() == kWorkerTeam )
			{
				if(( WorkerStopped == false ) && ( WorkerCaptured == false ))
				{
					WorkerStopped = true;
					fMission.mDisplayMessage( kWorkerAtBase_msg, kMissionScriptMessage, Ship );
					Ship->mSetShipAIControlled( kFalse );
					Ship->mSetCommButton( 1, kButtonTransmitLogs_text );
					Ship->mClearCommButton( 2 );

					if(( BullyCaptured == true ) || ( BullyStopped == true ))
					{
						fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
					}
				}
			}
			break;

		case kNotifyDamage:
			if(( Ship->mGetDamagePercent() >= 90.0 ) && ( ShipThreatened == true ))
			{
				if( Ship->mGetTeam() == kBullyTeam )
				{
					if(( BullyStopped == false ) && ( BullyCaptured == false ))
					{
						BullyStopped = true;
						fMission.mDisplayMessage( kBullyGiveUp_msg, kMissionScriptMessage, Ship );
						Ship->mSetShipAIControlled( kFalse );

						if(( WorkerCaptured == true ) || ( WorkerStopped == true ))
						{
							fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
						}
					}
				}
				else
				{
					if(( WorkerStopped == false ) && ( WorkerCaptured == false ))
					{
						WorkerStopped = true;
						fMission.mDisplayMessage( kWorkerGiveUp_msg, kMissionScriptMessage, Ship );
						Ship->mSetShipAIControlled( kFalse );

						if(( BullyCaptured == true ) || ( BullyStopped == true ))
						{
							fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
						}
					}
				}
			}
			else
			{
				Ship->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;

		case kNotifyDeepScan:
			if(( fIsDeepScanOn == kTrue ) && ( Scanned == false ))
			{
				if(( Ship->mGetTacticalDistance( BullyID ) < 30 ) && ( Ship->mGetTacticalDistance( WorkerID ) < 30 ))
				{
					fMission.mDisplayMessage( kDeepScan_msg, kScienceScriptMessage, Ship );
					Scanned = true;
					Proof++;
				}
				else
				{
					fMission.mDisplayMessage( kDeepScanFail_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyProbeLaunch:
			if( fMission.mGetShipTarget( fPlayerShip ) == BullyID ) 
			{
				if( BullyProbed == false )
				{
					BullyProbed = true;
					Proof++;
					fMission.mDisplayMessage( kProbeBully_msg, kScienceScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kProbeFail_msg, kScienceScriptMessage );
				}
			}
			else if( fMission.mGetShipTarget( fPlayerShip ) == WorkerID ) 
			{
				if( WorkerProbed == false )
				{
					WorkerProbed = true;
					Proof++;
					Ship = WorkerTeam->mGetFirstShipInPlay();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kProbeWorker_msg, kScienceScriptMessage, Ship );
					}
				}
				else
				{
					fMission.mDisplayMessage( kProbeFail_msg, kScienceScriptMessage );
				}
			}
			break;
	}
	
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( Killer == true )
	{
		return( kDevastatingDefeat );
	}

	if( PlayerLeft == true )
	{
		return( kDefeat );
	}

	if( Proof >= 3 )
	{
		if(( WorkerCaptured == false ) && ( WorkerStopped == false ))
		{
			return( kAstoundingVictory );
		}
		else
		{
			return( kVictory );
		}
	}

	return( kDraw );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( Killer == true )
	{
		fMission.mAddToDebriefingList( kKiller_msg );
		return;
	}

	if( PlayerLeft == true )
	{
		fMission.mAddToDebriefingList( kDisengaged_msg );
		return;
	}

	fMission.mAddToDebriefingList( kSuccessful_msg );

	if( BullyCaptured == true )
	{
		fMission.mAddToDebriefingList( kBullyCaptured_msg );
	}

	if( WorkerCaptured == true )
	{
		fMission.mAddToDebriefingList( kWorkerCaptured_msg );
	}

	if( Proof <= 2 )
	{
		fMission.mAddToDebriefingList( kNoProof_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
