#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 EnemyCaptured = 0;
int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 PlayerReinforcements = 0;	// player can call one set of reinforcements
int32 HydranWarning = 0;
int32 PirateAlert = 0;
bool PlayerDisengaged = false;
eVictoryLevels MyVicLevel = kDevastatingDefeat;

enum eMissionModes
{
	kStartMode				= 0,
	kOrionPirateAttack,
	kPlayerReinforcements,
	kEnemyAttack,
};

enum eMissionActs
{
	kAct_Monster			= 0,
	kAct_Pirates,
	kAct_EnemyAttack,
};
eMissionActs MissionAct = kAct_Monster;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kPirateTeam, kMonsterTeam, kAlliedTeam, kHydranTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kEnemyBorder | kWarzone | kEnemyColonial | kEnemyCapital;
	Specs->fPlayerRace = kFedMission | kGornMission;
	Specs->fEnemyRace = kLyranMission | kFedMission | kKlingMission | kRomulanMission | kGornMission | kOrionMission;
	Specs->fNumShips = MissSpecTwoShip | MissSpecThreeShip;
	Specs->fMinBPV = 225;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 10;
	Specs->fMaxDate = 20;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 200, 150, 75, 25, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kLyran;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kPirateTeam, kOrion );
	fMission.mSetTeamRace( kMonsterTeam, kMonster );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kHydranTeam, kHydran );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kMonsterTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kHydranTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kPirateTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 2.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 2.0, kTrue );
	fMission.mSetTeamRelations( kPirateTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPirateTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPirateTeam, kHydranTeam, kTeamAtWar, kGoalCapture, 2.0 );
	fMission.mSetTeamRelations( kMonsterTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kMonsterTeam, kHydranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kAlliedTeam, kHydranTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kHydranTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tShipInfo* Ship;

	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	// Create ships at the beginning of the mission
	if (Mode == kStartMode)
	{
		// allied ships
		AlliedTeam->mCreateShip (kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_H);
		AlliedTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_I);

		// allied starbase
		AlliedTeam->mCreateShip (kMaxBPV, kClassStarBase, kClassStarBase, 0, kStartPosition_O);

		// monster!
		MonsterTeam->mCreateShip ("M-SGM", kStartPosition_N);
	}

	// Orion Pirate Attack
	if (Mode == kOrionPirateAttack)
	{
		// hydran force
		HydranTeam->mCreateShip (kMaxBPV, kClassDreadnought, kClassDreadnought, 0, kStartPosition_J);
		HydranTeam->mCreateShip (kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_K);
		HydranTeam->mCreateShip (kMaxBPV, kClassLightCruiser, kClassLightCruiser, 0, kStartPosition_L);
		HydranTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_M);

		// pirates!
		PirateTeam->mCreateFleet (1, kMaxBPV, kClassLightCruiser, kClassLightCruiser, 1, kStartPosition_T);
		PirateTeam->mCreateFleet (2, kMaxBPV, kClassWarDestroyer, kClassWarDestroyer, 1, kStartPosition_V);
		PirateTeam->mCreateFleet (2, kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_X);
	}

	// Player Reinforcements
	if (Mode == kPlayerReinforcements)
	{
		AlliedTeam->mCreateShip (kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 1, kStartPosition_Q); 
		AlliedTeam->mCreateShip (kMaxBPV, kClassWarDestroyer, kClassWarDestroyer, 1, kStartPosition_R);
	}

	// Enemy Attack
	if (Mode == kEnemyAttack)
	{
		Ship = fPlayerShip;
		Ship->mGetGridPosition (&OldX, &OldY);
		NewX = OldX + fMission.mRandomInt32 (1, 2);
		NewY = OldY + fMission.mRandomInt32 (5, 10);
		EnemyTeam->mCreateShip (kMaxBPV, kClassDreadnought, kClassDreadnought, 1, kStartPosition_Specified, NewX-2, NewY);
		EnemyTeam->mCreateShip (kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 1, kStartPosition_Specified, NewX-1, NewY);
		EnemyTeam->mCreateShip (kMaxBPV, kClassLightCruiser, kClassLightCruiser, 1, kStartPosition_Specified, NewX, NewY);
		EnemyTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_Specified, NewX+1, NewY);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);
	tShipInfo* Ship;

	// configure player ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();

	Ship = MonsterTeam->mGetFirstShip();
	Ship->mSetCommButton (1, kButtonMonsterHail_text);

	for (Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);
	fMission.mDisplayMessage (kMonsterWarning_msg, kScienceScriptMessage);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mSetCommButton (1, kButtonHail_text);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle (kPirateTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle (kAlliedTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			switch (MissionAct)
			{
				case kAct_Pirates:
					if (PirateAlert == 0)
					{
						mCreateMissionShips (kOrionPirateAttack);	// pirate attack!
						fMission.mDisplayMessage (kPirateWarning_msg, kScienceScriptMessage);
						PirateAlert = 1;
						fMission.mSetTimedInterval (kTimerTurnsSet, 1);
					}
					else if (HydranWarning == 0)
					{
						HydranWarning = 1;
						fMission.mDisplayMessage (kHydranUnhappyHail_msg, kMissionScriptMessage);
						for (Ship = HydranTeam->mGetFirstShip (); Ship != NULL; Ship = HydranTeam->mGetNextShip ())
						{
							Ship->mSetCommButton (1, kButtonHydranProof_text);
						}
						fMission.mSetTimedInterval (kTimerTurnsSet, 1);
					}
					break;

				case kAct_EnemyAttack:
					fMission.mDisplayMessage (kEnemyAttack_msg, kMissionScriptMessage);
					mCreateMissionShips (kEnemyAttack);
					break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kButtonMonsterHail_text:
					fMission.mDisplayMessage (kMonsterHail_msg, kScienceScriptMessage);
					break;

				case kButtonHail_text:
					switch (Ship->mGetTeam())
					{
						case kAlliedTeam:
							fMission.mDisplayMessage (kBaseMessage_msg, kMissionScriptMessage);
							Ship->mSetCommButton (1, kButtonHelp_text);
							break;

						case kEnemyTeam:
							fMission.mDisplayMessage (kEnemyMessage_msg, kMissionScriptMessage);
							break;

						case kHydranTeam:
							if (MissionAct == kAct_Monster)
							{
								fMission.mDisplayMessage (kHydranHappyHail_msg, kMissionScriptMessage);
							}
							else if (MissionAct == kAct_Pirates)
							{
								fMission.mDisplayMessage (kHydranUnhappyHail_msg, kMissionScriptMessage);
								HydranWarning = 1;
								for (Ship = HydranTeam->mGetFirstShip (); Ship != NULL; Ship = HydranTeam->mGetNextShip ())
								{
									Ship->mSetCommButton (1, kButtonHydranProof_text);
								}
							}
							else
							{
								fMission.mDisplayMessage (kHydranHappyHail_msg, kMissionScriptMessage);
							}
							break;

						case kPirateTeam:
							fMission.mDisplayMessage (kPirateMessage_msg, kMissionScriptMessage);
							break;
					}
					break;

					case kButtonHelp_text:
						fMission.mDisplayMessage (kHelpDispatched_msg, kMissionScriptMessage);
						mCreateMissionShips (kPlayerReinforcements);
						PlayerReinforcements = 1;
						break;

					case kButtonHydranProof_text:
						fMission.mDisplayMessage (kWhatProof_msg, kMissionScriptMessage);
						break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kMonsterTeam )
			{
				MissionAct = kAct_Pirates;
				fMission.mSetTimedInterval (kTimerTurnsSet, 2);
			}

			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerDestroyed++;
			}

			if( HydranTeam->mGetShipsInPlayCount() == 0 )
			{
				return kAllyDestroyed;
			}

			if( ( MissionAct == kAct_EnemyAttack ) && ( EnemyTeam->mGetShipsInPlayCount() == 0 ) )
			{
				fMission.mDisplayMessage( kCongratulations_msg, kMissionScriptMessage );
				return kPlayerWins;
			}

			if( ( MissionAct == kAct_Pirates ) && ( PirateTeam->mGetShipsInPlayCount() == 0 ) )
			{
				fMission.mDisplayMessage( kPiratesDead_msg, kMissionScriptMessage );
				MissionAct = kAct_EnemyAttack;

				for( Ship = HydranTeam->mGetFirstShip(); Ship != NULL; Ship = HydranTeam->mGetNextShip() )
				{
					Ship->mSetFinalDestination( 25, 36 );
				}

				fMission.mSetTimedInterval( kTimerTurnsSet, 3 );
			}

			// if the player is down to one ship, dispatch the
			// reinforcements (if they haven't already been called)
			if( ( PlayerTeam->mGetShipsInPlayCount() == 1 ) && ( PlayerReinforcements == 0 ) )
			{
				fMission.mDisplayMessage( kHelpDispatched_msg, kMissionScriptMessage );
				PlayerReinforcements = 1;
				mCreateMissionShips( kPlayerReinforcements );
				Ship = AlliedTeam->mGetFirstShip();
				Ship->mClearCommButton( 1 );
			}
			break;

			if( PlayerTeam->mGetShipsInPlayCount() == 0 )
			{
				return kPlayerDestroyed;
			}
			
		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerCaptured++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerCaptured;
				}
			}
			
			if( ( Ship->mGetTeam() == kAlliedTeam ) || ( Ship->mGetTeam() == kHydranTeam ) )
			{
				fMission.mAddToBonusPrestige( -10 );
			}
			
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyCaptured++;
				fMission.mAddToBonusPrestige( kCaptureBonus );

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					return kEnemyCaptured;
				}
			}

			if( Ship->mGetTeam() == kPirateTeam )
			{
				fMission.mDisplayMessage( kSelfDestruction_msg, kScienceScriptMessage );
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					PlayerDisengaged = true;
					return kPlayerLoses;
				}
			}
			break;
	}
	
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HydranTeam = fMission.mGetTeamHandle (kHydranTeam);

	if (HydranTeam->mGetShipsInPlayCount() == 0)
	{
		fMission.mAdjustDynaverse( kAdjustForeignRelation, -10, kHydran );
		return kDevastatingDefeat;
	}

	if (HydranTeam->mGetShipsInPlayCount() == 1)
	{
		return kDefeat;
	}

	if( PlayerDisengaged == true )
	{
		return kDefeat;
	}

	if( PlayerDestroyed >= 2 )
	{
		return kDraw;
	}

	if( PlayerDestroyed == 1 )
	{
		fMission.mAdjustDynaverse( kAdjustForeignRelation, +4, kHydran );
		return kVictory;
	}

	return kAstoundingVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if( EnemyCaptured > 0 )
	{
		fMission.mAddToDebriefingList (kEnemyCaptured_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_1;
	}
	else
	{
		return 0;
	}
}
