#include "scriptinterface.h"

#include "Uni_InvestigationAmbushText.h"
#include "Uni_InvestigationAmbushMaps.h"

int32 PlayerCaptured = 0;
int32 EnemyCaptured = 0;
int32 EnemyDestroyed = 0;
int32 TimerState = 0;		// the number of times [PLAYER_RACE_COMMAND] has bugged the player
int32 StartDistance = 0;	// the starting distance from the player to the base
int32 DistanceReports = 0;	// number of reports displayed as the player gets closer
int32 BaseX = 0;
int32 BaseY = 0;
int32 BonusBalancePoints = 0;
bool BaseDestroyed = false;
bool EnemyHailed = false;
bool BaseScanned = false;
bool PlanetScanned = false;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kBaseTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kWarzone | kAlliedBorder;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kKlingMission | kRomulanMission | kLyranMission | kHydranMission | kGornMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 10;
	Specs->fMaxDate = 30;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 150, 100, 50, 0, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kBaseTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	if( Mode == 0 )
	{
		BaseTeam->mCreateShip( kMaxBPV, kClassBaseStation, 0, kStartPosition_M );
	}

	// The ambushing enemy ships...
	if( Mode == 1 )
	{
		int32 TotalBPV = ( PlayerTeam->mGetCombatBPV() * ( 1 + fMission.mGetDifficultyLevel() ));

		EnemyTeam->mCreateFleet( 2, TotalBPV, kClassWarDestroyer, kClassHeavyCruiser, 20, kStartPosition_X );
	}

	return;
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	BaseTeam->mGetFirstShip()->mGetGridPosition( &BaseX, &BaseY );

	// configure player ship
	Ship = PlayerTeam->mGetFirstShip();
	StartDistance = Ship->mGetTacticalDistance( 'M' );	// distance to base
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 10 );
	Ship->mTrackTacticalDistance( StartDistance * .9, BaseX, BaseY );

	// configure base
	Ship = BaseTeam->mGetFirstShip();
	Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 0 );
	Ship->mSetShipAIControlled( kFalse );
	Ship->mSetCommButton( 1, kButtonHail_text );

	fMission.mSetTimedInterval( kTimerTurnsSet, 3 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mSetCommButton( 1, kButtonHail_text );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyCommButtonPress:
			if( Ship->mGetTeam() == kBaseTeam )
			{
				fMission.mDisplayMessage( kNoResponse_msg );
			}
			else
			{
				fMission.mDisplayMessage( kEnemyReason_msg, kFleetScriptMessage, Ship );
				EnemyHailed = true;

				for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
				{
					Ship->mClearCommButton( 1 );
				}
			}
			break;

		case kNotifyAtDistance:
			switch( DistanceReports )
			{
				case 0:
					if( BaseScanned == false )
					{
						fMission.mDisplayMessage( kFirstReport_msg, kScienceScriptMessage );
					}

					Ship->mTrackTacticalDistance( StartDistance * .5, BaseX, BaseY );
					DistanceReports++;
					break;

				case 1:
					if( BaseScanned == false )
					{
						fMission.mDisplayMessage( kSecondReport_msg, kScienceScriptMessage );
					}

					Ship->mTrackTacticalDistance( 15, BaseX, BaseY );
					DistanceReports++;
					break;

				case 2:
					fMission.mDisplayMessage( kThirdReport_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet, 10 );	// prepare the ambush
					DistanceReports = 99;	// arrived at the base
					break;
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					if( TimerState == 0 )
					{
						if(( BaseScanned == false ) && ( DistanceReports != 99 ))
						{
							fMission.mDisplayMessage( kMoveFast_msg );
							fMission.mSetTimedInterval( kTimerTurnsSet, 4 );
							TimerState = 1;
						}
					}
					else
					{
						if(( BaseScanned == false ) && ( DistanceReports != 99 ))
						{
							fMission.mDisplayMessage( kMoveFaster_msg );
						}
					}
					break;

				case kTimerTurnsSet_2:
					if( BaseScanned == false )
					{
						fMission.mDisplayMessage( kBaseScanned_msg, kScienceScriptMessage );
						BaseScanned = true;
					}
					break;

				case kTimerTurnsSet_3:
					Ship = BaseTeam->mGetFirstShip();
					if( Ship != NULL )
					{
						fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
						Ship->mSetShipAIControlled( kTrue );
						Ship->mClearCommands();
					}
					break;

				case kTimerLimitSet:
					mCreateMissionShips( 1 );					// create the ambushing ship(s)
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue );

					if( EnemyTeam->mGetShipsInPlayCount() == 1 )
					{
						fMission.mDisplayMessage( kAmbushed_msg, kScienceScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage( kAmbushedMultiple_msg, kScienceScriptMessage );
					}
					break;
			}
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				Ship = BaseTeam->mGetFirstShip();

				if( Ship!= NULL )
				{
					if(( fMission.mGetShipTarget( fPlayerShip ) != Ship ) && ( PlanetScanned == false ))
					{
						fMission.mDisplayMessage( kScanPlanet_msg, kScienceScriptMessage );
						PlanetScanned = true;
					}
					else if( BaseScanned == false )				
					{
						if( fPlayerShip->mGetTacticalDistance( 'M' ) > 5 )
						{
							fMission.mDisplayMessage( kTooFarToScan_msg, kScienceScriptMessage );
						}
						else
						{
							fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
						}
					}
				}
			}
			break;

		case kNotifyProbeLaunch:
			Ship = BaseTeam->mGetFirstShip();

			if( Ship != NULL )
			{
				if(( BaseScanned == false ) && ( fMission.mGetShipTarget( fPlayerShip ) == Ship ))
				{
					fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				EnemyCaptured++;
				fMission.mAddToBonusPrestige( kCaptureBonus );

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kEnemyCaptured;
				}
			}
			else if( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				if( BaseScanned == true )
				{
					fMission.mDisplayMessage( kControlBase_msg );
				}
				else
				{
					fMission.mDisplayMessage( kControlBaseScanned_msg );
					BaseScanned = true;
				}

				fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}
			else if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyDestroyed++;

				if ( Enemies == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kEnemyDestroyed;
				}
			}
			else if( Ship->mGetTeam() == kBaseTeam )
			{
				BaseDestroyed = true;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	int32 VictoryPoints = EnemyDestroyed;	// starts based on number of enemy kills

	if( BaseScanned == true )
	{
		VictoryPoints++;
	}

	if( EnemyHailed == true )
	{
		VictoryPoints++;
	}

	if( BaseDestroyed == true )
	{
		VictoryPoints = VictoryPoints - 2;
	}

	if( VictoryPoints < 0 )
	{
		VictoryPoints = 0;
	}

	switch( VictoryPoints )
	{
		case 0:
			return( kDevastatingDefeat );

		case 1:
			return( kDefeat );

		case 2:
			return( kDraw );

		case 3:
			return( kVictory );
	}

	return( kAstoundingVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );

	if( BaseScanned == true )
	{
		fMission.mAddToDebriefingList( kDebriefBaseScanned_msg );

		if( EnemyHailed == true )
		{
			fMission.mAddToDebriefingList( kDebriefEnemyHailed_msg );
		}
	}
	else
	{
		fMission.mAddToDebriefingList( kDebriefBaseNotScanned_msg );
	}

	if( EnemyCaptured == 1 )
	{
		fMission.mAddToDebriefingList( kCapturedSingleEnemy_msg );
	}

	if( EnemyCaptured >= 2 )
	{
		fMission.mAddToDebriefingList( kCapturedMultipleEnemies_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
