#include "scriptinterface.h"

#include "Uni_RavagePiratesText.h"
#include "Uni_RavagePiratesMaps.h"

int32 PlayerCaptured = 0;
int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 BaseCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
float BaseHealth = 0.0;
iTruth PlayerDisengaged = kFalse;
bool Reinforcements = false;
int32 Act = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
	kBaseTeam,
	kBaseDefenseTeam,   // the defense platforms and asteroids
	kRomFreighterTeam,  // has the mines
	kFedFreighterTeam,  // has wild weasels
	kHydFreighterTeam,  // has suicide shuttles
	kKliFreighterTeam,  // has scatter packs
	kLyrFreighterTeam,  // has spare parts


};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kWarzone;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 17;
	Specs->fMaxDate = 30;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 150, 50, 0, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, kOrion);
	fMission.mSetTeamRace (kBaseTeam, kOrion);
	fMission.mSetTeamRace (kFedFreighterTeam, kFederation);
	fMission.mSetTeamRace (kKliFreighterTeam, kKlingon);
	fMission.mSetTeamRace (kRomFreighterTeam, kRomulan);
	fMission.mSetTeamRace (kHydFreighterTeam, kHydran);
	fMission.mSetTeamRace (kLyrFreighterTeam, kLyran);
	
	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kBaseDefenseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kEnemyTeam, kBaseDefenseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenseTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlayerTeam, kFedFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kKliFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kRomFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kHydFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kLyrFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kEnemyTeam, kFedFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kKliFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kRomFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kHydFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kLyrFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kBaseDefenseTeam, kFedFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenseTeam, kKliFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenseTeam, kRomFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenseTeam, kHydFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseDefenseTeam, kLyrFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kBaseTeam, kFedFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kKliFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kRomFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kHydFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kBaseTeam, kLyrFreighterTeam, kTeamFriendly, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kFedFreighterTeam, kKliFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedFreighterTeam, kRomFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedFreighterTeam, kLyrFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kFedFreighterTeam, kHydFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kKliFreighterTeam, kRomFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKliFreighterTeam, kLyrFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kKliFreighterTeam, kHydFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kRomFreighterTeam, kLyrFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kRomFreighterTeam, kHydFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kLyrFreighterTeam, kHydFreighterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	int32 map = 1;
	// pick a random map
	int32 DifLev = fMission.mGetDifficultyLevel();
	switch ( DifLev )
	{
	case 0:
		{
		}
		break;
	case 1:
		{
			map = 2;
		}
		break;
	case 2:
		{
			map = 3;
		}
		break;
	}

	return (map);
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = kStartPosition_G;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle (kBaseTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	tTeamInfo* BaseDefenseTeam = fMission.mGetTeamHandle (kBaseDefenseTeam);

	tTeamInfo* RomFreighterTeam = fMission.mGetTeamHandle (kRomFreighterTeam);
	tTeamInfo* FedFreighterTeam = fMission.mGetTeamHandle (kFedFreighterTeam);
	tTeamInfo* HydFreighterTeam = fMission.mGetTeamHandle (kHydFreighterTeam);
	tTeamInfo* KliFreighterTeam = fMission.mGetTeamHandle (kKliFreighterTeam);
	tTeamInfo* LyrFreighterTeam = fMission.mGetTeamHandle (kLyrFreighterTeam);

	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .7;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.2;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	eClassTypes BaseType = kClassBaseStation;
	int32 NumEscorts = PlayerTeam->mGetShipsInPlayCount();
	int32 FreighterBPV = 35;

	int32 DifLev = fMission.mGetDifficultyLevel();
	int32 NumPlayerShips = 5;

	switch ( DifLev )
	{
	case 0:
		{
			if ( NumEscorts > 3 )
			{
				NumEscorts--;
			}
			TotalBPV *= 0.6;
			NumPlayerShips++;
		}
		break;
	case 1:
		{
			BaseType = kClassBattleStation;
			FreighterBPV = 45;
		}
		break;
	case 2:
		{
			BaseType = kClassBattleStation;
			NumEscorts++;
			TotalBPV *= 1.2;
			FreighterBPV = 70;
			NumPlayerShips--;
		}
		break;
	}


	if ( Mode == 0 )
	{
		BaseTeam->mCreateShip(TotalBPV, kClassBaseStation, kClassBattleStation, 20, kStartPosition_Z);
		EnemyTeam->mCreateFleet( NumEscorts, TotalBPV, kClassFrigate, kClassBattleship, 20, kStartPosition_Y | kGroupAround );

		int32 PBPV = PlayerTeam->mGetCombatBPV();
		int32 EBPV = EnemyTeam->mGetCombatBPV();
		int32 BBPV = BaseTeam->mGetCombatBPV();
		BonusBalancePoints = ( 20 * (EBPV+BBPV) / PBPV ) - 20;

		int32 OutGunned = BaseTeam->mGetCombatBPV() + EnemyTeam->mGetCombatBPV() - PlayerTeam->mGetCombatBPV();
		if ( OutGunned > 0 )
		{
			int32 PlayerWingmanBPV = 0.65f * (float) OutGunned / (float) NumEscorts - PlayerTeam->mGetShipsInPlayCount();
			PlayerTeam->mCreateFleet( NumPlayerShips, PlayerWingmanBPV, kClassFrigate, kClassBattleship, 20, kStartPosition_G | kGroupAround ); 
		}

		FedFreighterTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 50, kStartPosition_X );
		KliFreighterTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 50, kStartPosition_W );
		RomFreighterTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 50, kStartPosition_V );
		HydFreighterTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 50, kStartPosition_U );
		LyrFreighterTeam->mCreateShip( FreighterBPV, kClassFreighter, kClassFreighter, 50, kStartPosition_T );

		BaseDefenseTeam->mCreateShip( "N-ABDr", kStartPosition_I );
		BaseDefenseTeam->mCreateShip( "N-ABPL", kStartPosition_J );
		BaseDefenseTeam->mCreateShip( "N-ABPh", kStartPosition_K );
		BaseDefenseTeam->mCreateShip( "N-ABDs", kStartPosition_L );
		BaseDefenseTeam->mCreateShip( "N-ABHB", kStartPosition_M );

	}

	if ( Mode == 1 )
	{
		// 5 reinforcements
		EnemyTeam->mCreateFleet( 5, TotalBPV, kClassFrigate, kClassBattleship, 20, kStartPosition_G | kGroupAround );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return kBriefingA_msg;
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip (); Ship != NULL; Ship = PlayerTeam->mGetNextShip ())
	{
		Ship->mSetShipThrust (5);
	}

	fMission.mSetTimedInterval (kTimerTurnsSet, 20);  // summons reinforcements	
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tShipInfo* WinnerShip;

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();
	int32 PlayerFleet = PlayerTeam->mGetShipsInPlayCount();


	switch (Event)
	{

		case kNotifyAtTimedInterval:
			{
				if ( Act == 0 )
				{
					Act++;
					fMission.mDisplayMessage ( kReinforcements_msg );
					mCreateMissionShips( Act );
				}
			}
			break;

		case kNotifyDisengaged:
			{
				if (( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
				{
					PlayerDisengaged = kTrue;
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
				}
		break;

		case kNotifyOnCapture:
			{
				WinnerShip = fMission.mFindShipID( fEventID );
				if (Ship->mGetOriginalTeam () == kPlayerTeam)
				{
					PlayerCaptured++;
				}
				else if (Ship->mGetOriginalTeam () == kBaseTeam)
				{
					BaseCaptured++;
					if ( Act == 0 )
					{
						Act++;
						fMission.mDisplayMessage ( kReinforcements_msg );
						mCreateMissionShips( Act );
					}
					fMission.mDisplayMessage ( kBaseCaptured_msg );
				}
				else if (Ship->mGetOriginalTeam () == kFedFreighterTeam)
				{
					int32 stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplyWildWeasel );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyWildWeasel, stores + 6 );
					stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplySpareParts );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 2 );
					fMission.mDisplayMessage ( kFoundSuppliesSpareParts_msg );
					fMission.mDisplayMessage ( kFoundSuppliesWildWeasel_msg );
				}
				else if (Ship->mGetOriginalTeam () == kKliFreighterTeam)
				{
					int32 stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplyScatterPack );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 4 );
					stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplySpareParts );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 2 );
					fMission.mDisplayMessage ( kFoundSuppliesSpareParts_msg );
					fMission.mDisplayMessage ( kFoundSuppliesScatterPack_msg );
				}
				else if (Ship->mGetOriginalTeam () == kRomFreighterTeam)
				{
					int32 stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplyMines );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 10 );
					stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplySpareParts );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 2 );
					fMission.mDisplayMessage ( kFoundSuppliesSpareParts_msg );
					fMission.mDisplayMessage ( kFoundSuppliesMines_msg );
				}
				else if (Ship->mGetOriginalTeam () == kLyrFreighterTeam)
				{
					int32 stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplyMines );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 4 );
					stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplySpareParts );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 12 );
					fMission.mDisplayMessage ( kFoundSuppliesSpareParts_msg );
					fMission.mDisplayMessage ( kFoundSuppliesMines_msg );
				}
				else if (Ship->mGetOriginalTeam () == kHydFreighterTeam)
				{
					int32 stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplySuicides );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 8 );
					stores = WinnerShip->mAccessSupplies (kGetSupplyCount, kSupplySpareParts );
					WinnerShip->mAccessSupplies (kSetSupplyCount, kSupplyScatterPack, stores + 2 );
					fMission.mDisplayMessage ( kFoundSuppliesSpareParts_msg );
					fMission.mDisplayMessage ( kFoundSuppliesSuicide_msg );
				}
				else if (Ship->mGetOriginalTeam () == kEnemyTeam)
				{
					Enemies--;
					EnemyCaptured++;

					if ( Enemies == 0 )
					{
						if ( Act == 0 )
						{
							Act++;
							fMission.mDisplayMessage ( kReinforcements_msg );
							mCreateMissionShips( Act );
						}
					}
				}
				return kGameInProgress;
			}
			break;
		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam () == kPlayerTeam)
				{
					PlayerDestroyed++;

					if ( PlayerFleet ==  0 )
					{
						return kPlayerDestroyed;
					}
				}
				if (Ship->mGetTeam () == kBaseTeam)
				{
					fMission.mDisplayMessage ( kBaseDestroyed_msg );
					if ( Act == 0 )
					{
						Act++;
						fMission.mDisplayMessage ( kReinforcements_msg );
						mCreateMissionShips( Act );
					}
				}
				if (Ship->mGetTeam () == kEnemyTeam)
				{
					if ( EnemyHealth == 0.0 )
					{
						if ( Act == 0 )
						{
							Act++;
							fMission.mDisplayMessage ( kReinforcements_msg );
							mCreateMissionShips( Act );
						}
					}
				}
				return kGameInProgress;
			}		
			break;
	}

	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	if ( PlayerDisengaged )
	{
		if ( ( BaseCaptured ) && ( PlayerHealth > 0.50 ) )
		{
			return kAstoundingVictory;
		}
		else if ( ( BaseHealth == 0.0 ) && ( PlayerHealth > 0.30 ) )
		{
			return kVictory;
		}
		else if ( BaseHealth == 0.0 ) 
		{
			return kDraw;
		}
		else if ( PlayerHealth > 0.40 )
		{
			return kDefeat;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}

	// By default // Shoulde never get here
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
