#include "scriptinterface.h"

#include "Uni_RunawayText.h"
#include "Uni_RunawayMaps.h"

int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float PirateHealth = 0.0;
bool PlayerDisengaged = false;
bool RunawayScreamed = false;
bool RunawaySaved = false;
bool AllyHailed = false;
bool AllyBitched = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kPirateTeam, kRunawayTeam, kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerCapital | kPlayerColonial | kWarzone;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kOrionMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 6;
	Specs->fMaxDate = 17;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 150, 100, 50, 25, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kPirateTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kRunawayTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kPlayerTeam, kRunawayTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPirateTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations( kPirateTeam, kRunawayTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPirateTeam, kAlliedTeam, kTeamAtWar, kGoalCapture, 2.0 );
	fMission.mSetTeamRelations( kRunawayTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kAlliedTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0 );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle( kPirateTeam );
	tTeamInfo* RunawayTeam = fMission.mGetTeamHandle( kRunawayTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	int32 TotalBPV = PlayerTeam->mGetCombatBPV();
	int32 NumShips = 0;

	switch( fMission.mGetDifficultyLevel() )
	{
		case 0:
			TotalBPV = TotalBPV * .75;
			break;

		case 1:
			TotalBPV = TotalBPV * 1;
			NumShips = 1;
			break;

		case 2:
			TotalBPV = TotalBPV * 1.25;
			NumShips = 2;
			break;
	}

	// create pirate team
	PirateTeam->mCreateShip( TotalBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_X );
	PirateTeam->mCreateFleet( NumShips, kMaxBPV, kClassFrigate, kClassFrigate, 100, kStartPosition_Y );

	// create runaway team
	RunawayTeam->mCreateShip( kMaxBPV, kClassFreighter, kClassFreighter, 0, kStartPosition_R );

	// create allied team
	AlliedTeam->mCreateFleet( 5, kMaxBPV, kClassFreighter, kClassFreighter, 1, kStartPosition_S );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle( kPirateTeam );
	tTeamInfo* RunawayTeam = fMission.mGetTeamHandle( kRunawayTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = PirateTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;
	fMission.mAddToBonusPrestige( BonusBalancePoints );

	int32 SunX = 0;
	int32 SunY = 0;
	fMission.mGetGridPosition( '7', &SunX, &SunY );

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipCallbackEvents( kNotifyBeginTractor );

	for( Ship = PirateTeam->mGetFirstShip(); Ship != NULL; Ship = PirateTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	Ship = RunawayTeam->mGetFirstShip();
	Ship->mDamageSystem( kDamageInternalsWithPoints, 30 );
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mSetFinalDestination( SunX, SunY );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mSetShipMaxSpeed( 12 );
	Ship->mDamageSystem( kDamageMaxWeapons, 100.0 );
	int32 DistanceFromSun = Ship->mGetTacticalDistance( SunX, SunY );
	Ship->mTrackTacticalDistance(( DistanceFromSun + 10 ), SunX, SunY, kTrue );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );

	for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetShipMaxSpeed( 8 );
		Ship->mSetFinalDestination();
		Ship->mDamageSystem( kDamageMaxWeapons, 50.0 );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// starting
	fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );	// runaway scream
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );	// ally scream
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle( kPirateTeam );
	tTeamInfo* RunawayTeam = fMission.mGetTeamHandle( kRunawayTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	PirateHealth = 100.0f - PirateTeam->mGetTotalDamagePercent();

	int32 Pirates = PirateTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0 );
					fMission.mSetTeamRelations( kPlayerTeam, kRunawayTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPirateTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
					fMission.mSetTeamRelations( kPirateTeam, kRunawayTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPirateTeam, kAlliedTeam, kTeamAtWar, kGoalCapture, 2.0 );
					fMission.mSetTeamRelations( kRunawayTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kAlliedTeam, kPirateTeam, kTeamAtWar, kGoalDestroy, 1.0 );
					break;

				case kTimerLimitSet_2:
					Ship = RunawayTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kRunawayHelp_msg, kFleetScriptMessage, Ship );
						RunawayScreamed = true;
					}
					break;

				case kTimerTurnsSet:
					if( AllyHailed == false )
					{
						Ship = AlliedTeam->mGetFirstShipInPlay();

						if( Ship != NULL )
						{
							fMission.mDisplayMessage( kAllyScream_msg, kFleetScriptMessage, Ship );
						}
					}
					break;

				case kTimerTurnsSet_2:
					Ship = AlliedTeam->mGetFirstShipInPlay();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kAllyThreat_msg, kFleetScriptMessage, Ship );
						AllyBitched = true;
					}
					break;
			}
			break;

		case kNotifyAtDistance:
			if( RunawaySaved == false )
			{
				RunawaySaved = true;
				Ship = RunawayTeam->mGetFirstShip();

				if( Ship != NULL )
				{
					fMission.mDisplayMessage( kRunawayThanks_msg, kFleetScriptMessage, Ship );
					Ship->mSetFinalDestination( 10, 12 );
					Ship->mSetShipMaxSpeed( 12 );
				}
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kRunawayTeam )
			{
				if( Ship != NULL )
				{
					if( RunawayScreamed == false )
					{
						fMission.mDisplayMessage( kRunawayHelp_msg, kFleetScriptMessage, Ship );
						RunawayScreamed = true;
					}
					else
					{
						if( Ship->mGetDamagePercent() > 75 )
						{
							fMission.mDisplayMessage( kRunawayDying_msg, kFleetScriptMessage, Ship );
						}
						else
						{
							Ship->mSetShipCallbackEvents( kNotifyDamage );
						}
					}
				}
			}
			else if( Ship != NULL )
			{
				if( Ship->mGetDamagePercent() > 60 )
				{
					Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					fMission.mDisplayMessage( kPirateRunaway_msg, kScienceScriptMessage, Ship );
				}
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}

			if( Ship->mGetOriginalTeam() == kPirateTeam )
			{
				if(( Pirates == 0 ) && ( RunawaySaved == true ))
				{
					return kPlayerWins;
				}

				if( Pirates == 0 )
				{
					for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
					{
						Ship->mSetCommButton( 1, kButtonHail_text );
					}

					break;
				}
			}

			if( Ship->mGetOriginalTeam() == kAlliedTeam )
			{
				fMission.mAddToBonusPrestige( -15 );
			}

			if( Ship->mGetOriginalTeam() == kRunawayTeam )
			{
				if( Pirates == 0 )
				{
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kPirateTeam )
			{
				if( Ship != NULL )
				{
					Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
					Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
					fMission.mDisplayMessage( kPirateSelfDestruct_msg, kScienceScriptMessage, Ship );
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					PlayerDisengaged = true;

					if( RunawaySaved == true )
					{
						return kPlayerWins;
					}
					else
					{
						return kPlayerLoses;
					}
				}
			}

			if( Pirates == 0 )
			{
				if( RunawayTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerLoses;
				}

				if( RunawaySaved == true )
				{
					return kPlayerWins;
				}
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					switch( Ship->mGetTeam() )
					{
						case kRunawayTeam:
							fMission.mDisplayMessage( kRunawayHail_msg, kFleetScriptMessage, Ship );
							Ship->mSetCommButton( 1, kButtonHalt_text );
							Ship->mSetCommButton( 2, kButtonWait_text );
							break;

						case kPirateTeam:
							fMission.mDisplayMessage( kPirateHail_msg, kFleetScriptMessage, Ship );

							for( Ship = PirateTeam->mGetFirstShip(); Ship != NULL; Ship = PirateTeam->mGetNextShip() )
							{
								Ship->mSetCommButton( 1, kButtonThreaten_text );
							}
							break;

						case kAlliedTeam:
							if( Pirates == 0 )
							{
								fMission.mDisplayMessage( kAllyThanks_msg, kFleetScriptMessage, Ship );
							}
							else
							{
								fMission.mDisplayMessage( kAllyHail_msg, kFleetScriptMessage, Ship );
								AllyHailed = true;

								for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
								{
									Ship->mSetCommButton( 1, kButtonWait_text );
								}
							}
							break;
					}
					break;

				case kButtonHalt_text:
					fMission.mDisplayMessage( kRunawayHalt_msg, kFleetScriptMessage, Ship );
					break;

				case kButtonThreaten_text:
					if( Ship->mGetDamagePercent() > 50 )
					{
						fMission.mDisplayMessage( kPirateRunaway_msg, kScienceScriptMessage, Ship );
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					}
					else
					{
						fMission.mDisplayMessage( fMission.mRandomInt32( kPirateThreat1_msg, kPirateThreat3_msg ), kFleetScriptMessage, Ship );
					}
					break;

				case kButtonWait_text:
					switch( Ship->mGetTeam() )
					{
						case kRunawayTeam:
							fMission.mDisplayMessage( kRunawayWait_msg, kFleetScriptMessage, Ship );
							break;

						case kAlliedTeam:
							fMission.mDisplayMessage( kAllyHelp_msg, kFleetScriptMessage, Ship );
							fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );

							for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
							{
								Ship->mClearCommButton( 1 );
							}
							break;
					}
					break;
			}
			break;

		case kNotifyBeginTractor:
			tShipInfo* Tractor = fMission.mGetShipTarget( fPlayerShip );

			if( Tractor != NULL )
			{
				if( Tractor->mGetTeam() == kRunawayTeam )
				{
					Ship = RunawayTeam->mGetFirstShipInPlay();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kRunawayTractor_msg, kFleetScriptMessage, Ship );
						Ship->mClearCommButton( 1 );
						Ship->mClearCommButton( 2 );
					}
				}
				else
				{
					fPlayerShip->mSetShipCallbackEvents( kNotifyBeginTractor );
				}
			}
			break;
	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( RunawaySaved == false )
	{
		if( PlayerHealth > PirateHealth )
		{
			return kVictory;
		}
		else if( PlayerHealth < ( PirateHealth / 2 ))
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}

	if( PlayerHealth > 90.0 )
	{
		return kAstoundingVictory;
	}

	if( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PirateTeam = fMission.mGetTeamHandle( kPirateTeam );

	if( RunawaySaved == false )
	{
		fMission.mAddToDebriefingList( kFailed_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kSavedRunaway_msg );
	}

	if( PirateTeam->mGetShipsInPlayCount() == 0 )
	{
		fMission.mAddToDebriefingList( kKilledPirates_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kPiratesGotAway_msg );
	}

	if( AllyBitched == true )
	{
		fMission.mAddToDebriefingList( kAlliesBitched_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
