#include "scriptinterface.h"

#include "Uni_StarbaseAssaultText.h"
#include "Uni_StarbaseAssaultMaps.h"

int32 DiffLevel = 0;
int32 PlayerKilled = 0;
int32 EnemyKilled = 0;
int32 EnemyCaptured = 0;
bool PlayerReinforced = false;
bool EnemyReinforced = false;
bool BaseDestroyed = false;
bool AllyDestroyed = false;
bool GameOver = false;
int32 MyVicLevel = kDefeat;

enum eMissionModes
{
	kStartMode				= 0,
	kEnemyReinforcements,
	kPlayerReinforcements,
};

eMissionModes Mode = kStartMode;
enum eMissionActs
{
	kCallingForHelp			= 0,
	kHelpArrives,
	kEnemyDisengaging,
};
eMissionActs MissionAct = kCallingForHelp;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kBaseTeam, kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 35;
	Specs->fMaxDate = 9999;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 500, 250, 100, 50, 0 );

	DiffLevel = fMission.mGetDifficultyLevel();	// 0 = easy, 1 = medium, 2 = hard

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kBaseTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kAlliedTeam, info.fAlliedRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalCripple, 1.0, kTrue );
	fMission.mSetTeamRelations( kAlliedTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 TotalShips = PlayerTeam->mGetShipsInPlayCount();

	if( Mode == kStartMode )
	{
		// create enemy forces
		if( DiffLevel == 0 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, 20, kStartPosition_X );
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 20, kStartPosition_Y );
			EnemyTeam->mCreateShip( kMaxBPV, kClassWarDestroyer, 20, kStartPosition_Z );
		}
		else if( DiffLevel == 1 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 20, kStartPosition_Y );
			EnemyTeam->mCreateShip( kMaxBPV, kClassWarDestroyer, 20, kStartPosition_Z );
			EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 20, kStartPosition_X );
		}
		else if( DiffLevel == 2 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassWarDestroyer, 20, kStartPosition_Z );
			EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 20, kStartPosition_X );
			EnemyTeam->mCreateShip( kMaxBPV, kClassNewHeavyCruiser, 20, kStartPosition_Y );
		}

		BaseTeam->mCreateShip( kMaxBPV, kClassStarBase, 100, kStartPosition_M );

		// create additional player ships
		int32 AlliedBPV = EnemyTeam->mGetCombatBPV() + BaseTeam->mGetCombatBPV() - PlayerBPV;
		PlayerTeam->mCreateFleet(( 6 - TotalShips ), AlliedBPV, kClassFrigate, kClassHeavyCruiser, 20, kStartPosition_J | kGroupAround );

		// create ally ships
		AlliedBPV = EnemyTeam->mGetCombatBPV() + BaseTeam->mGetCombatBPV() - PlayerBPV;
		PlayerTeam->mCreateFleet( 3, AlliedBPV, kClassFrigate, kClassHeavyCruiser, 20, kStartPosition_K | kGroupAround );

		// based on difficulty level, finish the enemy fleet
		if( DiffLevel >= 1 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 1, kStartPosition_V );
		}

		if( DiffLevel == 2 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_W );
		}
	}
	
	if( Mode == kEnemyReinforcements )
	{
		EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyBattlecruiser, 1, kStartPosition_R );
		EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 1, kStartPosition_S );
		EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_T );

		if( DiffLevel == 2 )
		{
			EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 1, kStartPosition_U );
		}
	}

	if( Mode == kPlayerReinforcements )
	{
		int32 NewX = 0;
		int32 NewY = 0;
		fPlayerShip->mGetGridPosition( &NewX, &NewY );
		NewX = NewX - 8;
		NewY = NewY + 8;
		PlayerTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 100, kStartPosition_Specified, NewX, NewY );
		PlayerTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 100, kStartPosition_Specified, NewX+1, NewY );
		PlayerTeam->mCreateShip( kMaxBPV, kClassDestroyer, 100, kStartPosition_Specified, NewX, NewY+1 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tShipInfo* Ship;

	int32 DifLev = fMission.mGetDifficultyLevel();

	// configure player ships
	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 10 );
	}

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// configure enemy ships
	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mDefendEntity( BaseTeam->mGetFirstShip(), 5, 20, kAbsolutePriority );
	}

	// configure the base
	Ship = BaseTeam->mGetFirstShip();
	Ship->mSetCommButton( 1, kButtonHail_text );

	// configure allied ships
	for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	if( Mode == kEnemyReinforcements )
	{
		int32 DifLev = fMission.mGetDifficultyLevel();
		Ship->mDefendEntity( BaseTeam->mGetFirstShip(), 5, 20, kAbsolutePriority );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					if( MissionAct == kCallingForHelp )
					{
						MissionAct = kHelpArrives;
						fMission.mDisplayMessage( kBaseScream_msg, kScienceScriptMessage );
						int32 RandomTurns = fMission.mRandomInt32( 10 - DiffLevel, 20 - DiffLevel );
						fMission.mSetTimedInterval( kTimerTurnsSet, RandomTurns );
					}
					else if( MissionAct == kHelpArrives )
					{
						if( EnemyReinforced == false )
						{
							fMission.mDisplayMessage( kEnemyReinforcements_msg, kScienceScriptMessage );
							mCreateMissionShips( kEnemyReinforcements );
							EnemyReinforced = true;
						}
					}
					break;

				case kTimerTurnsSet_2:
					fMission.mDisplayMessage( kPlayerReinforcements_msg, kScienceScriptMessage );
					mCreateMissionShips( kPlayerReinforcements );
					PlayerReinforced = true;
					break;

				case kTimerLimitSet:
					Ship = BaseTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage, Ship );
					}

					fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalCripple, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kAlliedTeam, kEnemyTeam, kTeamAtWar, kGoalCripple, 1.0, kTrue );
					fMission.mSetTeamRelations( kAlliedTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				PlayerKilled++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}

				if(( PlayerTeam->mGetShipsInPlayCount() == 3 ) && ( PlayerReinforced == false ))
				{
					fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
				}
			}

			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyKilled++;

				if(( EnemyTeam->mGetShipsInPlayCount() == 0 ) && ( BaseDestroyed == true ))
				{
					if( fMission.mGetEliteOrganization() < 1 )
					{
						GameOver = true;
						return eGameStatus( kPlayerWins | kCampaignCompleted );
					}
					else
					{
						return kPlayerWins;
					}
				}
			}

			if( Ship->mGetTeam() == kBaseTeam )
			{
				BaseDestroyed = true;

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					if( fMission.mGetEliteOrganization() < 1 )
					{
						GameOver = true;
						return eGameStatus( kPlayerWins | kCampaignCompleted );
					}
					else
					{
						return kPlayerWins;
					}
				}

				if( EnemyTeam->mGetShipsInPlayCount() >= 2 )
				{
					fMission.mDisplayMessage( kMultipleDisengaging_msg, kScienceScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kSingleDisengaging_msg, kScienceScriptMessage );
				}

				EnemyTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
				MissionAct = kEnemyDisengaging;
			}

			if( Ship->mGetTeam() == kAlliedTeam )
			{
				if( AlliedTeam->mGetShipsInPlayCount() == 0 )
				{
					AllyDestroyed = true;
				}
			}
			break;

		case kNotifyCommButtonPress:
			if( Ship->mGetTeam() == kBaseTeam )
			{
				fMission.mDisplayMessage( kBaseResponse_msg, kMissionScriptMessage );
			}
				
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( fMission.mRandomInt32( kEnemyNoResponse1_msg, kEnemyNoResponse3_msg) , kMissionScriptMessage, Ship );
			}
			break;
			
		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				EnemyCaptured++;
			}

			if( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				fMission.mAddToBonusPrestige( 200 );

				if( EnemyTeam->mGetShipsInPlayCount() >= 2 )
				{
					fMission.mDisplayMessage( kMultipleDisengaging_msg, kScienceScriptMessage );
				}
				else
				{
					fMission.mDisplayMessage( kSingleDisengaging_msg, kScienceScriptMessage );
				}

				EnemyTeam->mDisengageShips( kByMapEdge, kAbsolutePriority );
				MissionAct = kEnemyDisengaging;
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				if( fMission.mGetEliteOrganization() < 1 )
				{
					GameOver = true;

					if( BaseDestroyed == true )
					{
						return eGameStatus( kPlayerWins | kCampaignCompleted );
					}
					else
					{
						return eGameStatus( kPlayerLoses | kCampaignCompleted );
					}
				}
				else
				{
					return kPlayerDisengaged;
				}
			}

			if(( Ship->mGetTeam() == kEnemyTeam ) && ( EnemyTeam->mGetShipsInPlayCount() == 0 ))
			{
				if( fMission.mGetEliteOrganization() < 1 )
				{
					GameOver = true;

					if( BaseDestroyed == true )
					{
						return eGameStatus( kPlayerWins | kCampaignCompleted );
					}
					else
					{
						return eGameStatus( kPlayerLoses | kCampaignCompleted );
					}
				}
				else
				{
					return kPlayerWins;
				}
			}
			break;
	}

	// test to end the campaign
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		status = eGameStatus(status | kCampaignCompleted);
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	MyVicLevel = kDefeat;

	if( BaseDestroyed == true )
	{
		MyVicLevel = kVictory;
	}
 
	if( EnemyKilled >= 2 )
	{
		MyVicLevel--;
	}

	if(( PlayerKilled >= 4 ) || ( AllyDestroyed == true ))
	{
		MyVicLevel++;
	}

	if( EnemyCaptured >= 1 )
	{
		MyVicLevel--;
	}

	if( MyVicLevel <= kAstoundingVictory )
	{
		MyVicLevel = kAstoundingVictory;
	}

	if( MyVicLevel >= kDevastatingDefeat )
	{
		MyVicLevel = kDevastatingDefeat;
	}

	return( eVictoryLevels( MyVicLevel ));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	MyVicLevel = VictoryLevel;

	if( GameOver == false )
	{
		if( BaseDestroyed == false )
		{
			fMission.mAddToDebriefingList( kBaseNotDestroyed_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kBaseDestroyed_msg );
		}

		if( EnemyCaptured == 1 )
		{
			fMission.mAddToDebriefingList( kEnemyCapturedSingle_msg );
		}
		else if( EnemyCaptured >= 2 )
		{
			fMission.mAddToDebriefingList( kEnemyCapturedMultiple_msg );
		}

		fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
	}
	else
	{
		if( BaseDestroyed == false )
		{
			fMission.mAddToDebriefingList( kUltimateDefeat_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kUltimateVictory_msg );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
// to award Mission or Special medals in a regular campaign game:
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if (MyVicLevel <= kVictory)
	{
		return kSpecialMedal_3;
	}
	else
	{
		return 0;
	}
}
