#include "scriptinterface.h"

#include "Uni_SurpriseReversedText.h"
#include "Uni_SurpriseReversedMaps.h"

int32 EnemyDestroyed = 0;
int32 WakeUp = 0;
bool EnemyShipActive[5] = {false, false, false, false, false};
bool EnemyAlive = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kWarzone;
	Specs->fPlayerRace = kKlingMission | kRomulanMission | kFedMission | kGornMission | kLyranMission | kHydranMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 5;
	Specs->fMaxDate = 25;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 250, 125, 75, 50, -100 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// determine the delay between wake up attempts
	WakeUp = ( 3 - fMission.mGetDifficultyLevel() );	// 0 = Easy, 1 = Medium, 2 = Hard

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	if( EnemyTeam->mGetRace() == kRomulan )
	{
		EnemyTeam->mCreateShip ("R-KR", kStartPosition_V);
		EnemyTeam->mCreateShip ("R-WE", kStartPosition_W);
		EnemyTeam->mCreateShip ("R-K5R", kStartPosition_X);
		EnemyTeam->mCreateShip ("R-SNB", kStartPosition_Y);
		EnemyTeam->mCreateShip ("R-SNB", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kKlingon )
	{
		EnemyTeam->mCreateShip ("K-D7", kStartPosition_V);
		EnemyTeam->mCreateShip ("K-D6", kStartPosition_W);
		EnemyTeam->mCreateShip ("K-D6", kStartPosition_X);
		EnemyTeam->mCreateShip ("K-F5", kStartPosition_Y);
		EnemyTeam->mCreateShip ("K-F5", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kFederation )
	{
		EnemyTeam->mCreateShip ("F-DN", kStartPosition_V);
		EnemyTeam->mCreateShip ("F-CA", kStartPosition_W);
		EnemyTeam->mCreateShip ("F-NCL", kStartPosition_X);
		EnemyTeam->mCreateShip ("F-DD", kStartPosition_Y);
		EnemyTeam->mCreateShip ("F-FF", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kGorn )
	{
		EnemyTeam->mCreateShip ("G-CA", kStartPosition_V);
		EnemyTeam->mCreateShip ("G-CL", kStartPosition_W);
		EnemyTeam->mCreateShip ("G-CL", kStartPosition_X);
		EnemyTeam->mCreateShip ("G-DD", kStartPosition_Y);
		EnemyTeam->mCreateShip ("G-DD", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kLyran )
	{
		EnemyTeam->mCreateShip ("L-DN", kStartPosition_V);
		EnemyTeam->mCreateShip ("L-CA", kStartPosition_W);
		EnemyTeam->mCreateShip ("L-CW", kStartPosition_X);
		EnemyTeam->mCreateShip ("L-DD", kStartPosition_Y);
		EnemyTeam->mCreateShip ("L-DD", kStartPosition_Z);
	}
	else if( EnemyTeam->mGetRace() == kHydran )
	{
		EnemyTeam->mCreateShip ("H-PAL", kStartPosition_V);
		EnemyTeam->mCreateShip ("H-RN", kStartPosition_W);
		EnemyTeam->mCreateShip ("H-LN", kStartPosition_X);
		EnemyTeam->mCreateShip ("H-HN", kStartPosition_Y);
		EnemyTeam->mCreateShip ("H-HN", kStartPosition_Z);
	}

	if( PlayerTeam->mGetRace() == kFederation )
	{
		PlayerTeam->mCreateShip ("F-CC+", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kKlingon )
	{
		PlayerTeam->mCreateShip ("K-D7K", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kRomulan )
	{
		PlayerTeam->mCreateShip ("R-K7RB", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kGorn )
	{
		PlayerTeam->mCreateShip ("G-CC+", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kLyran )
	{
		PlayerTeam->mCreateShip ("L-CC+", kStartPosition_G);
	}
	else if( PlayerTeam->mGetRace() == kHydran )
	{
		PlayerTeam->mCreateShip ("H-LM", kStartPosition_G);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	PlayerTeam->mGetFirstShip()->mTrackTacticalDistance( 40.0, 21, 19 );

	for (Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip())
	{
		Ship->mSetShipAIControlled( kFalse );
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	int32 ShipCounter = 0;
	int32 ActiveShipsThisTurn = 0;

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			for( ShipCounter = 1; ShipCounter <= 5; ShipCounter++ )
			{
				if( ( fMission.mRandomInt32( 1, 3 ) == 1 ) && ( EnemyShipActive[ ShipCounter - 1 ] == false ) )
				{
					EnemyShipActive[ ShipCounter - 1 ] = true;
					Ship = EnemyTeam->mGetThisShip( ShipCounter );
					if( Ship->mTestShipStatus( kShipDestroyed ) == kFalse )
					{
						ActiveShipsThisTurn++;
						Ship->mSetShipAIControlled( kTrue );
					}
				}
			}

			if( ActiveShipsThisTurn > 0 )
			{
				fMission.mDisplayMessage( kDamageAlive_msg + ActiveShipsThisTurn, kScienceScriptMessage, Ship );
				ActiveShipsThisTurn = 0;
			}

			fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 1, ( 2 + WakeUp )));
			break;

		case kNotifyAtDistance:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage (kCrossed_msg, kScienceScriptMessage);
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, ( 2 + WakeUp )));
				EnemyAlive = true;
			}
			break;

		case kNotifyDamage:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage (kDamageAlive_msg, kScienceScriptMessage);
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, ( 2 + WakeUp )));
				EnemyAlive = true;
			}
			break;

		case kNotifyAtDeath:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage (kDamageAlive_msg, kScienceScriptMessage);
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, ( 2 + WakeUp )));
				EnemyAlive = true;
			}

			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else
			{
				EnemyDestroyed++;
			}

			if( EnemyTeam->mGetShipsInPlayCount() == 0 )
			{
				return kPlayerWins;
			}
			break;

		case kNotifyOnCapture:
			fMission.mDisplayMessage( kEnemySelfDestruct_msg, kScienceScriptMessage, Ship );
			Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
			Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
			break;

		case kNotifyProbeLaunch:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage( kProbing_msg, kScienceScriptMessage );
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, ( 2 + WakeUp )));
				EnemyAlive = true;
			}
			break;

		case kNotifyDeepScan:
			if(( fIsDeepScanOn == kTrue ) && ( EnemyAlive == false ))
			{
				if( fMission.mRandomInt32( 1, 2 ) == 2 )
				{
					fMission.mDisplayMessage( kDeepScanning_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, ( 2 + WakeUp )));
					EnemyAlive = true;
				}
			}
			break;

		case kNotifyCommButtonPress:
			if( EnemyAlive == false )
			{
				fMission.mDisplayMessage( kHailing_msg, kMissionScriptMessage );
				fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 2, ( 2 + WakeUp )));
				EnemyAlive = true;
			}
			else
			{
				fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				return kPlayerWins;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if( EnemyDestroyed == 0 )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, -4, EnemyTeam->mGetRace() );
		return( kDevastatingDefeat );
	}

	if( EnemyDestroyed == 1 )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, -2, EnemyTeam->mGetRace() );
		return( kDefeat );
	}

	if( EnemyDestroyed == 2 )
	{
		return( kDraw );
	}

	if( EnemyDestroyed == 3 )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
		return( kVictory );
	}

	fMission.mAdjustDynaverse( kAdjustWarProgression, +4, EnemyTeam->mGetRace() );
	return( kAstoundingVictory );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
