//	<Name>	MissionInfo.cpp
//
//	<Legal>	Copyright (c) 1997-1998 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	DESCRIPTION
//

#include "MissionInfo.h"

extern tString Messages[];
extern tScriptInterface* gScriptPtr;

// ************************************************************************************************
//					mission support routines
// ************************************************************************************************

//----------------------------------------------------------------------
//
tMissionInfo::tMissionInfo (void)
{
	int32 index = kMaxTeams;
	while (index--)
	{
		fTeams[index] = NULL;
		fRaces[index] = kUndefinedRace;
	}
	fEndGameLockout = kFalse;
	fLockOutStatus = kGameInProgress;
	fDebriefingTotal = 0;
	fBonusPrestige = 0;
	fTotalAlliedHumanTeams = 0;
	fTotalEnemyHumanTeams = 0;
	fTotalAlliedCPUTeams = 0;
	fTotalEnemyCPUTeams = 0;
}

//----------------------------------------------------------------------
//
tMissionInfo::~tMissionInfo (void)
{
	int32 index = kMaxTeams;
	while (index--)
	{
		if (fTeams[index] != NULL)
		{
			delete fTeams[index];
			fTeams[index] = NULL;
		}
	}
}

//----------------------------------------------------------------------
// file name of mission to schedule (including extension)
void tMissionInfo::mScheduleMission (int32 MsgID)
{
	tShipSystemsInfo info;

	info.fAction = kScheduleMission;
	info.fText = Messages[MsgID];

	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mSetDate (tDynaverseScriptInfo::tMissionDate& Date)
{
	fDate = Date;
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mSetLocation (eSectorName Location)
{
	fLocation = Location;
}


//----------------------------------------------------------------------
//
void tMissionInfo::mSetSystemName (eSystems SystemName)
{
	fSystemName = SystemName;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTeamRace (int32 Team, eRaceName Race)
{
	fRaces[Team] = Race;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetMultiplayerTeams (tDynaverseScriptInfo& info)
{
	// WARNING!! DO NOT USE kPlayerTeam in here. Use info.fPlayerTeam instead!

	for (int32 MemberIndex = 0; MemberIndex < tEntityMgr::kMaxNumberofPlayers; MemberIndex++)
	{
		if (info.fMembersInGame[MemberIndex] != kPlayerTypeNoOne)
		{
			eTeamID team	= info.fMemberTeams[MemberIndex];
			eRaceName race	= info.fMemberRaces[MemberIndex];

			// handle undefined races
			if (race == kUndefinedRace)
			{
				race = info.fEnemyRace;		// assume the enemy race

				// if this member is on the same team as any other human member
				// then use that human members race
				for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
				{
					if (team == info.fMemberTeams[i] && info.fMembersInGame[i] == kPlayerTypeHuman)
					{
						race = info.fMemberRaces[i];
						break;
					}
				}
			}

			// instead of using the team number as passed in use the players position in the menu 
			// as his team number. This will keep everyones ships on their own team.
			mSetTeamRace (MemberIndex, race);
			mGetTeamHandle (MemberIndex)->mSetMultiplayerTeam (team);

			// now set team relationships
			for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
			{
				if (info.fMembersInGame[i] != kPlayerTypeNoOne)
				{
					if (team != info.fMemberTeams[i])
					{
						mSetTeamRelations (MemberIndex, i, kTeamAtWar, kGoalDestroy, 1.0);
					}
					else if (i != MemberIndex)
					{
						mSetTeamRelations (MemberIndex, i, kTeamAllied, kGoalIgnore, 1.0);
					}
				}
			}

			// keep track of the human teams
			if (info.fMembersInGame[MemberIndex] == kPlayerTypeHuman)
			{
				// if this is a local human then set players team
				if (info.fMemberIsLocal[MemberIndex])
				{
					gScriptPtr->kPlayerTeam = eTeamID(MemberIndex);
				}
				else	// keep track of remote humans (friend or foe)
				{
					eMulti_TeamTypes FoF;
					int32 index;
					if (team == info.fPlayerTeam)
					{
						FoF = kHuman_AlliedTeams;
						index = fTotalAlliedHumanTeams++;
					}
					else
					{
						FoF = kHuman_EnemyTeams;
						index = fTotalEnemyHumanTeams++;
					}
					fMulti_Races[FoF][index]			= race;
					fMulti_TeamNumbers[FoF][index]		= MemberIndex;
				}
			}

			// keep track of the AI teams
			if (info.fMembersInGame[MemberIndex] == kPlayerTypeAI)
			{
				eMulti_TeamTypes FoF;
				int32 index;
				if (team == info.fPlayerTeam)
				{
					FoF = kCPU_AlliedTeams;
					index = fTotalAlliedCPUTeams++;
				}
				else
				{
					FoF = kCPU_EnemyTeams;
					index = fTotalEnemyCPUTeams++;
				}
				fMulti_Races[FoF][index]			= race;
				fMulti_TeamNumbers[FoF][index]		= MemberIndex;
				fMulti_ShipCounts[FoF][index]		= info.fMemberShipCounts[MemberIndex];
				fMulti_BPVLimits[FoF][index]		= info.fMemberBPVLimits[MemberIndex];
				fMulti_BPVTeamLimits[FoF][index]	= info.fMemberBPVTeamLimits[MemberIndex];
				fMulti_SmallHulls[FoF][index]		= info.fMemberSmallHulls[MemberIndex];
				fMulti_LargeHulls[FoF][index]		= info.fMemberLargeHulls[MemberIndex];
				fMulti_StartPositions[FoF][index]	= info.fMemberStartPositions[MemberIndex];

				for (int32 k = 0; k < fMulti_ShipCounts[FoF][index]; k++)
				{
					fMulti_FleetNames[FoF][index][k] = info.fMemberFleetNames[MemberIndex][k];
					fMulti_FleetCallNames[FoF][index][k] = info.fMemberFleetCallNames[MemberIndex][k];
				}

				mGetTeamHandle (MemberIndex)->mSetAsCPUTeam (FoF, index);
			}
		}
	}

	// update struct in case players team changed during setup
	info.fPlayerTeam = gScriptPtr->kPlayerTeam;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetSkirmishTeams (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	mSetTeamRace (info.fPlayerTeam, info.fPlayerRace);

	for (int32 i = 1; i <= kMaxCPUTeams; i++)
	{
		if (info.fMembersInGame[i] == kPlayerTypeAI)
		{
			eTeamID team	= info.fMemberTeams[i];

			// determine if this race is friend or foe
			eMulti_TeamTypes FoF;
			int32 index;
			if (team == info.fPlayerTeam)
			{
				FoF = kCPU_AlliedTeams;
				index = fTotalAlliedCPUTeams++;
			}
			else
			{
				FoF = kCPU_EnemyTeams;
				index = fTotalEnemyCPUTeams++;
			}

			// instead of using the team number as passed in use the players position in the menu 
			// as his team number. This will keep everyones ships on their own team.
			mSetTeamRace (i, info.fMemberRaces[i]);
			mGetTeamHandle (i)->mSetMultiplayerTeam (team);
			mGetTeamHandle (i)->mSetAsCPUTeam (FoF, index);

			fMulti_Races[FoF][index]			= info.fMemberRaces[i];
			fMulti_TeamNumbers[FoF][index]		= i;
			fMulti_ShipCounts[FoF][index]		= info.fMemberShipCounts[i];
			fMulti_BPVLimits[FoF][index]		= info.fMemberBPVLimits[i];
			fMulti_BPVTeamLimits[FoF][index]	= info.fMemberBPVTeamLimits[i];
			fMulti_SmallHulls[FoF][index]		= info.fMemberSmallHulls[i];
			fMulti_LargeHulls[FoF][index]		= info.fMemberLargeHulls[i];
			fMulti_StartPositions[FoF][index]	= info.fMemberStartPositions[i];

			for (int32 k = 0; k < fMulti_ShipCounts[FoF][index]; k++)
			{
				fMulti_FleetNames[FoF][index][k] = info.fMemberFleetNames[i][k];
				fMulti_FleetCallNames[FoF][index][k] = info.fMemberFleetCallNames[i][k];
			}
		}
	}

	// setup team relations and target goals
	for (i = 0; i < fTotalAlliedCPUTeams; i++)
	{
		int32 Iteam = fMulti_TeamNumbers[kCPU_AlliedTeams][i];

		// set as ally to player team
		mSetTeamRelations (info.fPlayerTeam, Iteam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		// set as ally to other allies
		for (int32 j = 0; j < fTotalAlliedCPUTeams; j++)
		{
			if (j != i)
			{
				mSetTeamRelations (Iteam, fMulti_TeamNumbers[kCPU_AlliedTeams][j], 
								   kTeamAllied, kGoalIgnore, 1.0, kTrue);
			}
		}

		// set at war to enemy teams
		for (j = 0; j < fTotalEnemyCPUTeams; j++)
		{
			mSetTeamRelations (Iteam, fMulti_TeamNumbers[kCPU_EnemyTeams][j], 
							   kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		}
	}
	for (i = 0; i < fTotalEnemyCPUTeams; i++)
	{
		// set at war to player team
		int32 Iteam = fMulti_TeamNumbers[kCPU_EnemyTeams][i];
		mSetTeamRelations (info.fPlayerTeam, Iteam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

		// set at war with other enemy teams (unless same team)
		for (int32 j = 0; j < fTotalEnemyCPUTeams; j++)
		{
			int32 Jteam = fMulti_TeamNumbers[kCPU_EnemyTeams][j];
			if (info.fMemberTeams[Iteam] != info.fMemberTeams[Jteam])
			{
				mSetTeamRelations (Iteam, Jteam, kTeamAtWar, kGoalDestroy, 1.0);
			}
			else if (i != j)
			{
				mSetTeamRelations (Iteam, Jteam, kTeamAllied, kGoalIgnore, 1.0);
			}
		}
	}
}

//----------------------------------------------------------------------
//
tTeamInfo* tMissionInfo::mGetTeamHandle (int32 Team)
{
	if (fTeams[Team] == NULL)
	{
		fTeams[Team] = new tTeamInfo (eTeamID(Team), fRaces[Team]);
	}
	return fTeams[Team];
}

//----------------------------------------------------------------------
//
tTeamInfo* tMissionInfo::mGetTeamHandle (eMulti_TeamTypes Type, int32 Index)
{
	return (mGetTeamHandle (fMulti_TeamNumbers[Type][Index]));
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetPrestigePoints (int32 AstoundingVictory, int32 Victory, int32 Draw, 
									   int32 Defeat, int32 DevastatingDefeat)
{
	fPrestigePoints[kAstoundingVictory] = AstoundingVictory;
	fPrestigePoints[kVictory] = Victory;
	fPrestigePoints[kDraw] = Draw;
	fPrestigePoints[kDefeat] = Defeat;
	fPrestigePoints[kDevastatingDefeat] = DevastatingDefeat;
}

//----------------------------------------------------------------------
// end the game if all ships of the player or enemy team
// have been destroyed, captured, or disengaged
eGameStatus tMissionInfo::mGetGameStatus (void)
{
	// if the ship that the player is controlling has disengaged then end the game
	eTeamID Team = gScriptPtr->kPlayerTeam;
	tTeamInfo* fTeam = mGetTeamHandle (Team);
	if (gScriptPtr->fPlayerShip->mTestShipStatus (kShipDisengaged) == kFalse)
	{
		// test the player and enemy team to see if they are both still alive
		if (fTeam->mGetShipsInPlayCount() > 0)
		{
			Team = eTeamID(gScriptPtr->mGetEnemyTeamID());
			fTeam = mGetTeamHandle (Team);
			if (fTeam->mGetShipsInPlayCount() > 0)
			{
				return kGameInProgress;
			}
		}
	}
	iNotifyEvents Event = fTeam->mGetTeamStatus();

	// decide which end game status to return
	// - Team has the team that is dead
	// - Event has the reason why
	eGameStatus status;
	if (Team == gScriptPtr->kPlayerTeam)
	{
		switch (Event)
		{
			case kNotifyAtDeath:
				status = kPlayerDestroyed;
				break;

			case kNotifyOnCapture:
				status = kPlayerCaptured;
				break;

			case kNotifyDisengaged:
				status = kPlayerDisengaged;
				break;

			default:
				status = kPlayerLoses;
				break;
		}
	}
	else // must be enemy team
	{
		switch (Event)
		{
			case kNotifyAtDeath:
				status = kEnemyDestroyed;
				break;

			case kNotifyOnCapture:
				status = kEnemyCaptured;
				break;

			case kNotifyDisengaged:
				status = kEnemyDisengaged;
				break;

			default:
				status = kPlayerWins;
				break;
		}
	}
	return status;
}

//----------------------------------------------------------------------
// return kTrue if all ships on all teams have been initialized
iTruth tMissionInfo::mAllTeamsReady (void)
{
	int32 index = kMaxTeams;
	while (index--)
	{
		if (fTeams[index] != NULL)
		{
			if (fTeams[index]->mAllShipsReady() == kFalse)
			{
				return kFalse;
			}
		}
	}
	return kTrue;
}

//----------------------------------------------------------------------
// returns the players victory level based on BPV
eVictoryLevels tMissionInfo::mGetVictoryLevel (int32 PlayerBPV, int32 EnemyBPV)
{
	if (PlayerBPV < 1)
	{
		PlayerBPV = 1;
	}
	if (EnemyBPV < 1)
	{
		EnemyBPV = 1;
	}

	int32 level = (PlayerBPV * 100) / EnemyBPV;
	if (level >= 200)
	{
		return kAstoundingVictory;		// 200%+
	}
	else if (level >= 110)
	{
		return kVictory;				// 110%-199%
	}
	else if (level >= 91)
	{
		return kDraw;					// 91%-109%
	}
	else if (level >= 30)
	{
		return kDefeat;					// 31%-90%
	}

	return kDevastatingDefeat;			// 30%
}

int32 tMissionInfo::mRandomInt32 (int32 lo, int32 hi)
{
	return (gScriptPtr->mRandomInt (lo, hi));
}

//----------------------------------------------------------------------
//
void tMissionInfo::mDisplayMessage (int32 MsgID, eScriptMessageTypes Type, tShipInfo* Ship)
{
	int32 ShipID = 0;
	if (Ship)
	{
		ShipID = Ship->mGetShipID();
	}
	gScriptPtr->mPopupMessage (Messages[MsgID], Type, ShipID);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mDebugMessage (const char* message)
{
	tString msg = message;
	gScriptPtr->mPopupMessage (msg, kNonCommScriptMessage, 0);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mAddToDebriefingList (int32 DebriefingIndex)
{
	if (fDebriefingTotal < kMaxInList)
	{
		fDebriefingArray[fDebriefingTotal] = DebriefingIndex;
		fDebriefingTotal++;
	}
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mResetDebriefingCount (void)
{
	fDebriefingTotal = 0;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetDebriefingTotal (void)
{
	return fDebriefingTotal;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetDebriefingMessage (int32 Index)
{
	if (Index < kMaxInList)
	{
		return fDebriefingArray[Index];
	}
	return 0;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetPrestigePoints (eVictoryLevels Level)
{
	if (Level == kBonusPrestige)
	{
		return fBonusPrestige;
	}
	if (Level < kMaxVictoryLevels)
	{
		return fPrestigePoints[Level];
	}
	return 0;
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mAddToBonusPrestige (int32 Amount)
{
	fBonusPrestige += Amount;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetBonusPrestige (void)
{
	return fBonusPrestige;
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mGetGridPosition (unsigned char MapObject, int32* GridX, int32* GridY)
{
	tShipSystemsInfo info;

	info.fAction = kGetMapEntryPosition;
	info.fInt32[0] = MapObject;
	gScriptPtr->mEntityMgr (info);

	*GridX = info.fGridX + 1;
	*GridY = info.fGridY + 1;
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tMissionInfo::mGetTacticalDistance (unsigned char MapObject1, unsigned char MapObject2)
{
	tShipSystemsInfo info;
	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fInt32[1] = MapObject1;
	info.fInt32[2] = MapObject2;
	info.fShipEntityID = 0;				// == 0 indicates that distance is between two map objects
	gScriptPtr->mEntityMgr (info);

	return (info.fFloat[0]);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mCreateMapObject (unsigned char MapObject, int32 GridX, int32 GridY)
{
	tShipSystemsInfo info;

	info.fAction = kCreateMapEntry;
	info.fInt32[0] = MapObject;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;

	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetPlayerRank (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScriptPtr->mEntityMgr (info);

	return info.fInt32[0];
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetEliteOrganization (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScriptPtr->mEntityMgr (info);

	return info.fInt32[1];
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetDifficultyLevel (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScriptPtr->mEntityMgr (info);

	return info.fInt32[2];
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mPlaySound (int32 SoundID)
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = SoundID;
	info.fInt32[1] = 0;		// indicate its a non-voice sound
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mPlayVoiceSample (int32 VoiceID, iTruth StopPreviousVoice)
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = VoiceID;
	info.fInt32[1] = 1;			// indicate its a voice sound
	info.fInt32[2] = StopPreviousVoice;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mVoiceSampleStillPlaying (void)
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = 0;
	info.fInt32[1] = 2;			// indicate return status request
	gScriptPtr->mEntityMgr (info);
	return info.fInt32[0];
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTimedInterval (eTimerIntervalTypes Type, int32 Interval)
{
	tShipSystemsInfo info;

	info.fAction = kSetTimedInterval;
	info.fInt32[0] = Interval;
	info.fInt32[1] = Type;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mAddToCareerTotals (eClassTypes Type, eCareerEvents Event, int32 Amount)
{
	tShipSystemsInfo info;

	info.fAction = kAddToCareerTotal;
	info.fInt32[0] = Amount;
	info.fInt32[1] = Event;
	info.fInt32[2] = Type;

	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mRemoveShipAsTarget (tShipInfo* TargetShip)
{
	if (TargetShip)
	{
		eTeamID target_team = TargetShip->mGetTeam();

		// remove this ship as a target from all teams
		for (int32 i = 0; i < kMaxTeams; i++)
		{
			tTeamInfo* Team = fTeams[i];
			if (Team && i != target_team)
			{
				Team->mRemoveShipAsTarget (TargetShip);
			}
		}
	}
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTeamRelations (int32 Team1, int32 Team2, eTeamRelationship Relations,
									   eTargetGoals Goals, float EPVFactor, iTruth MutualRelations)
{
	if (Team1 > kNoTeam && Team1 < kMaxTeams
		&& Team2 > kNoTeam && Team2 < kMaxTeams)
	{
		// set Team1 relationship with Team2
		tTeamInfo* team = mGetTeamHandle (Team1);
		team->mSetRelations (Team2, Relations, Goals, EPVFactor);

		if (MutualRelations)
		{
			// set Team2 relationship with Team1
			team = mGetTeamHandle (Team2);
			team->mSetRelations (Team1, Relations, Goals, EPVFactor);
		}
	}
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTeamRelations (eMulti_TeamTypes Team1, int32 Team2, eTeamRelationship Relations,
									  eTargetGoals Goals, float EPVFactor, iTruth MutualRelations)
{
	int32 team_count = 0;
	if (Team1 == kHuman_AlliedTeams) team_count = fTotalAlliedHumanTeams;
	if (Team1 == kHuman_EnemyTeams) team_count = fTotalEnemyHumanTeams;
	if (Team1 == kCPU_AlliedTeams) team_count = fTotalAlliedCPUTeams;
	if (Team1 == kCPU_EnemyTeams) team_count = fTotalEnemyCPUTeams;

	while (team_count--)
	{
		mSetTeamRelations (fMulti_TeamNumbers[Team1][team_count], Team2, Relations, Goals, EPVFactor, MutualRelations);
	}
}

//------------------------------------------------------------------------------------------------
//
eTeamRelationship tMissionInfo::mGetTeamRelations (eTeamID Team1, eTeamID Team2)
{
	if (Team1 > kNoTeam && Team1 < kMaxTeams)
	{
		tTeamInfo* team = fTeams[Team1];
		if (team != NULL)
		{
			return team->mGetRelations (Team2);
		}
	}
	return kUnknownRelationship;
}

//----------------------------------------------------------------------
// return pointer to ship that matches given ID
tShipInfo* tMissionInfo::mFindShipID (int32 ShipID)
{
	for (int32 i = 0; i < kMaxTeams; i++)
	{
		tTeamInfo* Team = fTeams[i];
		if (Team)
		{
			for (tShipInfo* ship = Team->mGetFirstShip (); ship != NULL; ship = Team->mGetNextShip())
			{
				if (ship->mGetShipID() == ShipID)
				{
					return ship;
				}
			}
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
//
ePoliticalRelation tMissionInfo::mGetRaceRelationship (eRaceName Race1, eRaceName Race2)
{
	tShipSystemsInfo info;

	info.fAction = kGetRaceRelationship;
	info.fInt32[0] = Race1;
	info.fInt32[1] = Race2;
	gScriptPtr->mEntityMgr (info);

	return (ePoliticalRelation(info.fInt32[2]));
}

//----------------------------------------------------------------------
//
tShipInfo* tMissionInfo::mGetShipTarget (tShipInfo* Ship)
{
	if (Ship)
	{
		return (mFindShipID (Ship->mGetShipTargetID()));
	}
	return NULL;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mStartFlashingButton (const char* ButtonID)
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fTextPtr = ButtonID;
	info.fInt32[0] = kTrue;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mStopFlashingButton (const char* ButtonID)
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fTextPtr = ButtonID;
	info.fInt32[0] = kFalse;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
int32 tMissionInfo::mGetINIFileInt (int32 HeaderID, int32 LabelID)
{
	tShipSystemsInfo info;
	info.fAction = kGetINIFileEntry;
	info.fText = Messages[HeaderID];
	info.fText += "^";
	info.fText += Messages[LabelID];
	gScriptPtr->mEntityMgr (info);

	return info.fInt32[0];
}

//----------------------------------------------------------------------
// Grid X/Y are 1 based
void tMissionInfo::mSetGoalIndicator (iTruth IndicatorOn, int32 GridX, int32 GridY)
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fInt32[0] = IndicatorOn;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;
	info.fShipEntityID = -1;
	info.fTextPtr = NULL;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetGoalIndicator (tShipInfo* TargetShip)
{
	if (TargetShip)
	{
		tShipSystemsInfo info;
		info.fAction = kStartFlashingButton;
		info.fInt32[0] = kTrue;
		info.fShipEntityID = TargetShip->mGetShipID();
		info.fTextPtr = NULL;
		gScriptPtr->mEntityMgr (info);
	}
}

//----------------------------------------------------------------------
//
iTruth tMissionInfo::mIsTeamLeaving (eTeamID Team)
{
	tTeamInfo* team = mGetTeamHandle (Team);
	return team->mIsTeamLeaving();
}

//----------------------------------------------------------------------
//
void tMissionInfo::mAdjustDynaverse (eDynaverseTableType Type, int32 Modifier, eRaceName Race)
{
	tShipSystemsInfo info;
	info.fAction = kDynaverseAdjustments;
	info.fInt32[0] = Type;
	info.fInt32[1] = Modifier;
	info.fInt32[2] = Race;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mGetMapDimensions (int32* GridX, int32* GridY)
{
	gScriptPtr->mGetMapDimensions (GridX, GridY);
}


// 4Nov99	zdrummond	Added Comment Header and Footer's