//	<Name>	MissionInfo.h
//
//	<Legal>	Copyright (c) 1997-1998 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	DESCRIPTION
//

#if !defined( MissionInfo_h )
#define MissionInfo_h

enum eMulti_TeamTypes
{
	kHuman_AlliedTeams,
	kHuman_EnemyTeams,
	kCPU_AlliedTeams,
	kCPU_EnemyTeams,
	kMaxMultiTeams
};

class tMissionInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	void			mSetTeamRace (int32 Team, eRaceName Race);
	void			mSetTeamRelations (int32 Team1, int32 Team2, eTeamRelationship Relations,
									   eTargetGoals Goals, float EPVFactor, iTruth MutualRelations = kFalse);
	void			mSetTeamRelations (eMulti_TeamTypes Team1, int32 Team2, eTeamRelationship Relations,
									   eTargetGoals Goals, float EPVFactor, iTruth MutualRelations = kFalse);
	void			mSetMultiplayerTeams (tDynaverseScriptInfo& info);
	void			mSetSkirmishTeams (tDynaverseScriptInfo& info);
	void			mSetPrestigePoints (int32 AstoundingVictory, int32 Victory, int32 Draw,
										int32 Defeat, int32 DevastatingDefeat);
	void			mSetTimedInterval (eTimerIntervalTypes Type, int32 Interval);
	void			mSetGoalIndicator (tShipInfo* TargetShip);
	void			mSetGoalIndicator (iTruth IndicatorOn, int32 GridX, int32 GridY);		// Grid X/Y are 1 based

	tTeamInfo*		mGetTeamHandle (int32 Team);
	tTeamInfo*		mGetTeamHandle (eMulti_TeamTypes Type, int32 Index);
	int32			mGetBonusPrestige (void);
	eVictoryLevels	mGetVictoryLevel (int32 PlayerBPV, int32 EnemyBPV);
	void			mGetGridPosition (unsigned char MapObject, int32* GridX, int32* GridY);		// Grid XY are 1 based
	float			mGetTacticalDistance (unsigned char MapObject1, unsigned char MapObject2);
	int32			mGetPlayerRank (void);
	int32			mGetEliteOrganization (void);
	int32			mGetDifficultyLevel (void);
	ePoliticalRelation	mGetRaceRelationship (eRaceName Race1, eRaceName Race2);
	tShipInfo*		mGetShipTarget (tShipInfo* Ship);
	int32			mGetINIFileInt (int32 HeaderID, int32 LabelID);
	void			mGetMapDimensions (int32* GridX, int32* GridY);

	void			mAddToBonusPrestige (int32 Amount);
	int32			mRandomInt32 (int32 lo, int32 hi);
	void			mDisplayMessage (int32 MsgID, eScriptMessageTypes Type = kMissionScriptMessage, tShipInfo* Ship = NULL);
	void			mAddToDebriefingList (int32 DebriefingIndex);
	void			mCreateMapObject (unsigned char MapObject, int32 GridX, int32 GridY);		// Grid XY are 1 based
	void			mPlaySound (int32 SoundID);
	void			mPlayVoiceSample (int32 VoiceID, iTruth StopPreviousVoice = kTrue);
	int32			mVoiceSampleStillPlaying (void);
	void			mAddToCareerTotals (eClassTypes Type, eCareerEvents Event, int32 Amount);
	tShipInfo*		mFindShipID (int32 ShipID);
	void			mScheduleMission (int32 MsgID);			// file name of mission to schedule (including extension)
	void			mStartFlashingButton (const char* ButtonID);
	void			mStopFlashingButton (const char* ButtonID);

	void			mAdjustDynaverse (eDynaverseTableType Type, int32 Modifier, eRaceName Race);

	int32			fTotalAlliedHumanTeams;
	int32			fTotalEnemyHumanTeams;
	int32			fTotalAlliedCPUTeams;
	int32			fTotalEnemyCPUTeams;

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:
	void		mDebugMessage (const char* message);
	void		mSetDate (tDynaverseScriptInfo::tMissionDate& Date);
	void		mSetLocation (eSectorName Location);
	void		mSetSystemName (eSystems SystemName);
	iTruth		mAllTeamsReady (void);
	eGameStatus	mGetGameStatus (void);
	int32		mGetDebriefingTotal (void);
	int32		mGetDebriefingMessage (int32 Index);
	int32		mGetPrestigePoints (eVictoryLevels Level);
	void		mRemoveShipAsTarget (tShipInfo* TargetShip);
	eTeamRelationship mGetTeamRelations (eTeamID Team1, eTeamID Team2);
	void		mResetDebriefingCount (void);
	iTruth		mIsTeamLeaving (eTeamID Team);

	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tMissionInfo (void);
	~tMissionInfo (void);

	eRaceName		fMulti_Races[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_TeamNumbers[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_ShipCounts[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_BPVLimits[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_BPVTeamLimits[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	eClassTypes		fMulti_SmallHulls[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	eClassTypes		fMulti_LargeHulls[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_StartPositions[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	tString			fMulti_FleetNames[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers][3];
	tString			fMulti_FleetCallNames[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers][3];

	iTruth			fEndGameLockout;
	eGameStatus		fLockOutStatus;

protected:
	int32		fPrestigePoints[kMaxVictoryLevels];
	int32		fBonusPrestige;

	tTeamInfo*	fTeams[kMaxTeams];
	eRaceName	fRaces[kMaxTeams];

	eSectorName		fLocation;
	eSystems		fSystemName;

	tDynaverseScriptInfo::tMissionDate	fDate;

	enum { kMaxInList = 10 };
	int32			fDebriefingArray[kMaxInList];
	int32			fDebriefingTotal;
};


#endif


// 4Nov99	zdrummond	Added Comment Header and Footer's