//	<Name>	ShipInfo.cpp
//
//	<Legal>	Copyright (c) 1997-1998 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	DESCRIPTION
//

#include "ShipInfo.h"

extern tString Messages[];
extern tScriptInterface* gScriptPtr;

// ************************************************************************************************
//					ship support routines
// ************************************************************************************************

tShipInfo::tShipInfo (eTeamID Team, eRaceName Race, int32 CallbackID,
					  int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	fTeam = Team;
	fOriginalTeam = Team;
	fRace = Race;
	fStatus = kUndefined;
	fShipBPV = 0;
	fCallbackID = CallbackID;
	fStartPositionIndex = Position;
	fStartX = StartX;
	fStartY = StartY;
	fEndX = EndX;
	fEndY = EndY;
}

tShipInfo::~tShipInfo (void)
{
}

//----------------------------------------------------------------------
//
void tShipInfo::mClearCommands (void)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIClearCommands;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipTarget (tShipInfo* TargetShip, eAIPriority Priority)
{
	if (TargetShip)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAITargetObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = TargetShip->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptPtr->mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mDisengage (eDisengageMethods Method, eAIPriority Priority)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIDisengage;
	info.fInt32[1] = Priority;
	info.fInt32[2] = Method;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	mSetShipStatus (kShipDisengaged);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mTractorEntity (tShipInfo* Entity, int32 EndX, int32 EndY, eAIPriority Priority)
{
	if (Entity)
	{
		fShipEvents |= kNotifyEndTractor | kNotifyBeginTractor;

		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIMoveObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fInt32[3] = fShipEvents;
		info.fGridX = EndX - 1;
		info.fGridY = EndY - 1;
		info.fShipEntityID = fShipID;
		gScriptPtr->mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
// ThreatRadius and FollowRadius are based on tactical map distance
void tShipInfo::mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority)
{
	if (Entity)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIDefendObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fFloat[0] = ThreatRadius;
		info.fFloat[1] = FollowRadius;
		info.fShipEntityID = fShipID;
		gScriptPtr->mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
//
void tShipInfo::mFollowEntity (tShipInfo* Entity, eAIPriority Priority)
{
	if (Entity)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIFollowObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptPtr->mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
//
void tShipInfo::mFireProbe (tShipInfo* Target, eAIPriority Priority)
{
	if (Target)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIFireProbe;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Target->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptPtr->mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
// DetectionRange is based on tactical map distance
void tShipInfo::mPatrol (float DetectionRange, int32 PatrolPointCount, tMapPoint* PatrolPoints, eAIPriority Priority)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIPatrol;
	info.fInt32[1] = Priority;
	info.fInt32[2] = PatrolPointCount;
	info.fFloat[0] = DetectionRange;
	info.fTextPtr = (const char*)PatrolPoints;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// sets state of officers on AI ships
void tShipInfo::mChangeAIOfficerState (eAIOfficers Officer, iTruth State)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIChangeOfficerState;
	info.fInt32[1] = Officer;
	info.fInt32[2] = State;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// sets cloaking state of ship - NOTE: kCloakOfficer MUST be turned off first
// with mChangeAIOfficerState function before using this function
void tShipInfo::mCloak (iTruth State)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAICloakShip;
	info.fInt32[1] = State;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipThrust (int32 NewThrust)
{
	tShipSystemsInfo info;

	info.fAction = kSetNewThrust;
	info.fFloat[0] = NewThrust;
	info.fInt32[0] = 0;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipMaxSpeed (int32 MaxSpeed)
{
	tShipSystemsInfo info;

	info.fAction = kSetNewThrust;
	info.fFloat[0] = MaxSpeed;
	info.fInt32[0] = -1;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipPlayerControlled (void)
{
	tShipSystemsInfo info;

	info.fAction = kSetPlayerControlledShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = -1;
	gScriptPtr->mEntityMgr (info);

	gScriptPtr->fPlayerShip = this;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipAIControlled (iTruth AIControlled)
{
	tShipSystemsInfo info;

	info.fAction = kSetPlayerControlledShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = AIControlled;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipCallbackEvents (iNotifyEvents NewEvents)
{
	mReplaceShipCallbackEvents (fShipEvents | NewEvents);
}

//----------------------------------------------------------------------
//
void tShipInfo::mReplaceShipCallbackEvents (iNotifyEvents NewEvents)
{
	tShipSystemsInfo info;

	info.fAction = kSetNewEvents;
	info.fInt32[0] = NewEvents;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	fShipEvents = NewEvents;
}

//----------------------------------------------------------------------
//
iNotifyEvents tShipInfo::mGetShipCallbackEvents (void)
{
	return fShipEvents;
}

//----------------------------------------------------------------------
// End X/Y are 1 based
void tShipInfo::mSetFinalDestination (int32 EndX, int32 EndY)
{
	fShipEvents |= kNotifyAtDestination;

	tShipSystemsInfo info;

	info.fAction = kSetFinalDestination;
	info.fInt32[0] = mGetStartPositionIndex() - kStartPosition_G;
	info.fInt32[1] = fShipEvents;
	info.fGridX = EndX - 1;
	info.fGridY = EndY - 1;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
// Grid X/Y are 1 based - Distance is based on tactical map distance
void tShipInfo::mTrackTacticalDistance (float Distance, int32 GridX, int32 GridY, iTruth NotifyInOut)
{
	fShipEvents |= kNotifyAtDistance;
	fTrackingDistance = Distance;
	if (NotifyInOut)
	{
		Distance = -Distance;	// a neg. dist means the game should detect boundry crossings both ways
	}

	tShipSystemsInfo info;

	info.fAction = kTrackDistanceFromTarget;
	info.fInt32[0] = fShipEvents;
	info.fFloat[0] = Distance;
	info.fGridX = GridX - 1;
	info.fGridY	= GridY - 1;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
// ButtonNumber is 1 based
void tShipInfo::mSetCommButton (int32 ButtonNumber, int32 MsgID, iTruth SetAsTarget)
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = MsgID;
	info.fInt32[1] = ButtonNumber;
	info.fInt32[2] = SetAsTarget;
	info.fText = Messages[MsgID];
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
// ButtonNumber is 1 based
void tShipInfo::mClearCommButton (int32 ButtonNumber)
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = -1;
	info.fInt32[1] = ButtonNumber;
	info.fInt32[2] = kFalse;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//----------------------------------------------------------------------
// sets/reset status of ship
void tShipInfo::mSetShipStatus (eShipStatus Status, iTruth SetState)
{
	if (SetState)
	{
		fStatus |= Status;
	}
	else
	{
		fStatus &= ~Status;
	}
}

//----------------------------------------------------------------------
// returns status of ship
iTruth tShipInfo::mTestShipStatus (int32u Status)
{
	if (fStatus & Status)
	{
		return kTrue;
	}
	return kFalse;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipID (int32 ShipID)
{
	fShipID = ShipID;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetTeam (eTeamID NewTeam)
{
	fTeam = NewTeam;
}

//----------------------------------------------------------------------
//
eTeamID tShipInfo::mGetTeam (void)
{
	return fTeam;
}

//----------------------------------------------------------------------
//
eTeamID tShipInfo::mGetOriginalTeam (void)
{
	return fOriginalTeam;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetShipID (void)
{
	return fShipID;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetClassType (eClassTypes ClassType)
{
	fClassType = ClassType;
}

//----------------------------------------------------------------------
//
eClassTypes tShipInfo::mGetClassType (void)
{
	return fClassType;
}

//----------------------------------------------------------------------
//
const char*	tShipInfo::mGetClassName (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipClassName;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	return (info.fTextPtr);
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetShipTargetID (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipTargetID;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	return (info.fInt32[0]);
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetStartPositionIndex (void)
{
	return fStartPositionIndex;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetStartX (void)
{
	return fStartX + 1;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetStartY (void)
{
	return fStartY + 1;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetEndX (void)
{
	return fEndX + 1;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetEndY (void)
{
	return fEndY + 1;
}

//----------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTrackingDistance (void)
{
	return fTrackingDistance;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipBPV (int32 ShipBPV)
{
	fShipBPV = ShipBPV;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetShipBPV (void)
{
	return fShipBPV;
}

//----------------------------------------------------------------------
// return kTrue if ship is alive and has not been captured or disengaged
iTruth tShipInfo::mInPlay (void)
{
	if (gScriptPtr->fMission.mIsTeamLeaving(fTeam))
	{
		return kFalse;
	}

	if (mTestShipStatus (kShipCaptured | kShipDisengaged | kShipDestroyed))
	{
		return kFalse;
	}
	return kTrue;
}

//------------------------------------------------------------------------------------------------
// return total BPV damage to this ship
int32 tShipInfo::mAssessDamage (eRaceName EnemyRace)
{
	int32 DamageBPV = 0;

	if (fStatus & kShipCaptured)
	{
		if (fClassType == kClassShuttle)
		{
			DamageBPV = fShipBPV;
		}
		else if (EnemyRace == kOrion)
		{
			DamageBPV = fShipBPV * 5;
		}
		else
		{
			DamageBPV = fShipBPV * 2;
		}
	}
	else if (fStatus & kShipDestroyed)
	{
		DamageBPV = fShipBPV;
	}
	else if (fStatus & kShipDisengaged)
	{
		if (EnemyRace != kOrion)
		{
			DamageBPV = float(fShipBPV * .5);
		}
	}

	return DamageBPV;
}

//------------------------------------------------------------------------------------------------
// 
int32 tShipInfo::mGetCallbackID (void)
{
	return fCallbackID;
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mGetGridPosition (int32* GridX, int32* GridY)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipPosition;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	*GridX = info.fGridX + 1;
	*GridY = info.fGridY + 1;
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTacticalDistance (tShipInfo* TargetShip)
{
	if (TargetShip)
	{
		tShipSystemsInfo info;

		info.fAction = kGetTacticalDistance;
		info.fInt32[0] = kGetShipPosition;
		info.fInt32[1] = TargetShip->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptPtr->mEntityMgr (info);

		return (info.fFloat[0]);
	}
	return 0.0;
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTacticalDistance (unsigned char MapObject)
{
	tShipSystemsInfo info;

	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fInt32[1] = MapObject;
	info.fGridX = -1;					// == -1 indicates that distance is between ship and map object
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	return (info.fFloat[0]);
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTacticalDistance (int32 GridX, int32 GridY)
{
	tShipSystemsInfo info;

	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	return (info.fFloat[0]);
}

//------------------------------------------------------------------------------------------------
// 
eAlertLevels tShipInfo::mGetAlertStatus (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetAlertStatus;
	info.fInt32[0] = -1;
	gScriptPtr->mEntityMgr (info);

	return (eAlertLevels(info.fInt32[0]));
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetAlertStatus (eAlertLevels Status)
{
	tShipSystemsInfo info;

	info.fAction = kGetAlertStatus;
	info.fInt32[0] = Status;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mDamageSystem (eDamageSystems System, float Percent)
{
	tShipSystemsInfo info;

	info.fAction = kDamageShipSystem;
	info.fInt32[0] = System;
	info.fFloat[0] = Percent;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
int32 tShipInfo::mAccessSupplies (eShipSupplyAccess Access, int32 SupplyItem, int32 NewValue)
{
	tShipSystemsInfo info;

	info.fAction = kAccessShipSupply;
	info.fInt32[0] = Access;
	info.fInt32[1] = SupplyItem;
	info.fInt32[2] = NewValue;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	return info.fInt32[2];
}

//------------------------------------------------------------------------------------------------
// 
float tShipInfo::mGetDamagePercent (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipDamagePercent;
	info.fShipEntityID = fShipID;
	info.fFloat[0] = 100.0;				// assume 100% damage if no ship
	gScriptPtr->mEntityMgr (info);

	return info.fFloat[0];
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mChangeName (int32 MsgID)
{
	tShipSystemsInfo info;

	info.fAction = kSetObjectName;
	info.fTextPtr = Messages[MsgID].mCharStr();
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
eOfficerRank tShipInfo::mGetOfficerRank (eOfficerTypes Officer)
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = kUnknownRank;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	return ((eOfficerRank)info.fInt32[2]);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetOfficerRank (eOfficerTypes Officer, eOfficerRank Rank)
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = Rank;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetOfficerName (eOfficerTypes Officer, int32 MsgID)
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = kMaxOfficerRank;
	info.fShipEntityID = fShipID;
	info.fText = Messages[MsgID];
	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetTargetable (iTruth State)
{
	tShipSystemsInfo info;

	info.fAction = kMakeShipTargetable;
	info.fInt32[0] = State;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);

	info.fAction = kMakeShipAbleToTarget;
	info.fInt32[0] = State;
	info.fShipEntityID = fShipID;
	gScriptPtr->mEntityMgr (info);
}


// 4Nov99	zdrummond	Added Comment Header and Footer's