//	<Name>	ShipInfo.h
//
//	<Legal>	Copyright (c) 1997-1998 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	DESCRIPTION
//

#if !defined( ShipInfo_h )
#define ShipInfo_h

//-------------------------------------------------------
// values to use with mSetShipStatus and mTestShipStatus
enum eShipStatus
{
	kUndefined					= 0x00,
	kShipCreated				= 0x01,
	kShipAtFinalDestination		= 0x02,
	kShipCaptured				= 0x04,
	kShipDisengaged				= 0x08,
	kShipDestroyed				= 0x10,
};

// must match enum eDisengageType in aiShipCommands.h
enum eDisengageMethods
{
	kByAcceleration,
	kByMapEdge,
};


class tShipInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	void		mSetShipCallbackEvents (iNotifyEvents NewEvents);
	void		mSetShipThrust (int32 NewThrust);
	void		mSetShipMaxSpeed (int32 MaxSpeed);
	void		mSetShipPlayerControlled (void);
	void		mSetShipAIControlled (iTruth AIControlled);
	void		mSetFinalDestination (int32 EndX = -1, int32 EndY = -1);
	void		mTrackTacticalDistance (float Distance, int32 GridX, int32 GridY, iTruth NotifyInOut = kFalse);	// Grid XY are 1 based
	void		mDamageSystem (eDamageSystems System, float Percent);
	void		mChangeName (int32 MsgID);
	int32		mAccessSupplies (eShipSupplyAccess Access, int32 SupplyItem, int32 NewValue = 0);


	eClassTypes		mGetClassType (void);
	const char*		mGetClassName (void);
	int32			mGetStartPositionIndex (void);
	int32			mGetStartX (void);
	int32			mGetStartY (void);
	int32			mGetEndX (void);
	int32			mGetEndY (void);
	float			mGetTrackingDistance (void);
	int32			mAssessDamage (eRaceName EnemyRace);
	void			mGetGridPosition (int32* GridX, int32* GridY);		// Grid XY are 1 based
	float			mGetTacticalDistance (tShipInfo* TargetShip);
	float			mGetTacticalDistance (unsigned char MapObject);
	float			mGetTacticalDistance (int32 GridX, int32 GridY);	// Grid XY are 1 based
	eAlertLevels	mGetAlertStatus (void);
	void			mSetAlertStatus (eAlertLevels Status);
	float			mGetDamagePercent (void);
	eOfficerRank	mGetOfficerRank (eOfficerTypes Officer);
	void			mSetOfficerRank (eOfficerTypes Officer, eOfficerRank Rank);
	void			mSetOfficerName (eOfficerTypes Officer, int32 MsgID);
	void			mSetTargetable (iTruth State);

	// AI specific commands
	void			mSetShipTarget (tShipInfo* TargetShip, eAIPriority TargetPriority);
	void			mClearCommands (void);
	void			mDisengage (eDisengageMethods Method, eAIPriority Priority);
	void			mPatrol (float DetectionRange, int32 PatrolPointCount, tMapPoint* PatrolPoints, eAIPriority Priority);
	void			mChangeAIOfficerState (eAIOfficers Officer, iTruth State);
	void			mCloak (iTruth State);
	void			mTractorEntity (tShipInfo* Entity, int32 EndX, int32 EndY, eAIPriority Priority); // End XY are 1 based
	void			mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority);
	void			mFollowEntity (tShipInfo* Entity, eAIPriority Priority);
	void			mFireProbe (tShipInfo* Target, eAIPriority Priority);

	void		mSetCommButton (int32 ButtonNumber, int32 MsgID, iTruth SetAsTarget = kFalse);	// ButtonNumber is 1 based
	void		mClearCommButton (int32 ButtonNumber);											// ButtonNumber is 1 based

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:
	void		mSetShipStatus (eShipStatus Status, iTruth SetState = kTrue);
	void		mSetShipID (int32 ShipID);
	void		mSetShipBPV (int32 ShipBPV);
	void		mSetClassType (eClassTypes ClassType);
	int32		mGetShipTargetID (void);
	void		mReplaceShipCallbackEvents (iNotifyEvents NewEvents);

	eTeamID			mGetTeam (void);
	eTeamID			mGetOriginalTeam (void);
	int32			mGetShipID (void);
	iTruth			mInPlay (void);
	iTruth			mTestShipStatus (int32u Status);
	int32			mGetShipBPV (void);
	iNotifyEvents	mGetShipCallbackEvents (void);
	int32			mGetCallbackID (void);
	void			mSetTeam (eTeamID NewTeam);

	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tShipInfo (eTeamID Team, eRaceName Race, int32 CallbackID, int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY);
	~tShipInfo (void);

protected:
	eTeamID			fTeam;
	eTeamID			fOriginalTeam;
	eRaceName		fRace;
	int32			fShipID;
	eClassTypes		fClassType;
	int32			fStartPositionIndex;
	int32			fStartX;
	int32			fStartY;
	int32			fEndX;
	int32			fEndY;
	float			fTrackingDistance;
	int32u			fStatus;
	int32			fShipBPV;
	iNotifyEvents	fShipEvents;
	int32			fCallbackID;
};


#endif


// 4Nov99	zdrummond	Added Comment Header and Footer's