//	<Name>	TeamInfo.cpp
//
//	<Legal>	Copyright (c) 1997-1998 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	DESCRIPTION
//

#include "TeamInfo.h"

extern tScriptInterface* gScriptPtr;

// ************************************************************************************************
//					team support routines
// ************************************************************************************************

//----------------------------------------------------------------------
//
tTeamInfo::tTeamInfo (eTeamID Team, eRaceName Race)
{
	fTeam = Team;
	fMultiplayerTeam = Team;
	fRace = Race;
	fCombatBPV = 0;
	fNextCallbackID = (Team * (0x7fffffff / kMaxTeams)) + 1;
	fIsCPUTeam = kFalse;
	fTeamLeaving = kFalse;

	// setup AI info
	int32 i = kMaxTeams;
	while (i--)
	{
		fAITargets.fRelationships[i] = kUnknownRelationship;
		fAITargets.fTargetGoals[i] = kGoalIgnore;
		fAITargets.fEPVFactors[i] = 1.0;
	}
	fAITargets.fRelationships[Team] = kTeamSelfAware;
}

//----------------------------------------------------------------------
//
tTeamInfo::~tTeamInfo (void)
{
	while (fShipList.size())
	{
		tShipInfo* Ship = fShipList.back();
		delete Ship;
		Ship = NULL;
		fShipList.pop_back();
	}
	fShipList.erase(fShipList.begin(), fShipList.end());
}

//----------------------------------------------------------------------
//
eTeamID tTeamInfo::mGetTeam (void)
{
	return fTeam;
}

//----------------------------------------------------------------------
//
eTeamID tTeamInfo::mGetMultiplayerTeam (void)
{
	return fMultiplayerTeam;
}

//----------------------------------------------------------------------
//
void tTeamInfo::mSetMultiplayerTeam (eTeamID Team)
{
	fMultiplayerTeam = Team;
}

//----------------------------------------------------------------------
//
eRaceName tTeamInfo::mGetRace (void)
{
	return fRace;
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetFirstShip (void)
{
	fShipIndex = 0;
	return mGetNextShip();
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetNextShip (void)
{
	if (fShipIndex >= fShipList.size())
	{
		return NULL;
	}

	tShipInfo* Ship = fShipList[fShipIndex++];
	return Ship;
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetFirstShipInPlay (void)
{
	fShipIndex = 0;
	return mGetNextShipInPlay();
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetNextShipInPlay (void)
{
	while (fShipIndex < fShipList.size())
	{
		tShipInfo* Ship = fShipList[fShipIndex++];
		if (Ship && Ship->mInPlay())
		{
			return Ship;
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
// return ship at given index - index is 1 based
tShipInfo* tTeamInfo::mGetThisShip (int32 ShipIndex)
{
	if (ShipIndex > fShipList.size ())
	{
		return NULL;
	}

	ShipIndex--;
	if (ShipIndex < 0)
	{
		ShipIndex = 0;
	}

	return (fShipList[ShipIndex]);
}

//----------------------------------------------------------------------
// return pointer to ship that matches given position index
tShipInfo* tTeamInfo::mFindShipIndex (int32 CallbackID)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mGetCallbackID() == CallbackID)
		{
			return ship;
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
// return pointer to ship that matches given ID
tShipInfo* tTeamInfo::mFindShipID (int32 ShipID)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mGetShipID() == ShipID)
		{
			return ship;
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
//
iTruth tTeamInfo::mAddShipToTeamList (tCreateShipInfo& info)
{
	// call back into the game to create the ship
	gScriptPtr->mCreateSpecShip (info);

	if (info.fSelectedClassType == kNoClassType)
	{
		return kFalse;
	}

	// add this ship to the teams list if it was created
	tShipInfo* Ship = new tShipInfo (fTeam, fRace, info.fShipData.fScriptCallbackID, info.fShipData.fShipPosition, 
									 info.fShipData.fStartX, info.fShipData.fStartY, info.fShipData.fEndX, info.fShipData.fEndY);
	fShipList.push_back (Ship);
	Ship->mSetClassType (info.fSelectedClassType);

	Ship->mSetShipBPV (info.fShipData.fShipBPV);
	mAddToCombatBPV (info.fShipData.fShipBPV);
	return kTrue;
}

//----------------------------------------------------------------------
// call back into game engine to create a ship
// Start XY/End XY are 1 based
void tTeamInfo::mCreateTeamShip (tCreateShipInfo& info, int32 Position,
								 int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	Position &= ~kGroupAround;

	if (Position == kStartPosition_Specified)
	{
		if (StartX < 1)
		{
			StartX = 1;
		}
		if (StartY < 1)
		{
			StartY = 1;
		}
		if (EndX < 1)
		{
			EndX = StartX;
		}
		if (EndY < 1)
		{
			EndY = StartY;
		}
	}

	info.fShipRace = fRace;
	info.fShipData.fTeam = fTeam;
	info.fShipData.fShipPosition = Position;
	info.fShipData.fStartX = StartX - 1;
	info.fShipData.fStartY = StartY - 1;
	info.fShipData.fEndX = EndX - 1;
	info.fShipData.fEndY = EndY - 1;
	info.fShipData.fScriptCallbackID = fNextCallbackID;
	info.fShipData.fCreateShadowShip = kFalse;

	if (mAddShipToTeamList (info))
	{
		fNextCallbackID++;
	}
}

//----------------------------------------------------------------------
// create ship from class name only
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (int32 PlayerShipIndex, const char* ClassName, int32 Position,
							 int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ClassName;
	info.fShipData.fPlayerShipIndex = PlayerShipIndex;
	info.fShipData.fCustomName = CustomName;
	mCreateTeamShip (info, Position, StartX, StartY, EndX, EndY);
}

//----------------------------------------------------------------------
// create ship from supplied ship data
void tTeamInfo::mCreateShip (int32 PlayerShipIndex, tShipInGame& ShipData)
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ShipData.fClassName;
	info.fShipData.fPlayerShipIndex = PlayerShipIndex;
	info.fShipData.fCustomName = NULL;
	mCreateTeamShip (info, ShipData.fShipPosition, ShipData.fStartX, ShipData.fStartY, ShipData.fEndX, ShipData.fEndY);
}

//----------------------------------------------------------------------
// create ship from class name only
// Start XY/End XY are 1 based
void tTeamInfo::mDuplicateShip (tShipInfo* Ship, int32 Position,
								int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	if (Ship)
	{
		tCreateShipInfo info;

		info.fShipData.fClassType = kNoClassType;
		info.fShipData.fClassName = Ship->mGetClassName();
		info.fShipData.fPlayerShipIndex = -1;
		info.fShipData.fCustomName = NULL;
		mCreateTeamShip (info, Position, StartX, StartY, EndX, EndY);
	}
}

//----------------------------------------------------------------------
// create ship from class name only
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (const char* ClassName, int32 Position,
							 int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ClassName;
	info.fShipData.fPlayerShipIndex = -1;
	info.fShipData.fCustomName = CustomName;
	mCreateTeamShip (info, Position, StartX, StartY, EndX, EndY);
}

//----------------------------------------------------------------------
// create group of ships
// Start XY/End XY are 1 based
void tTeamInfo::mCreateFleet (int32 TotalShips, int32 TotalBPV, eClassTypes SmallestHullSize, 
							  eClassTypes LargestHullSize, int32 VariantPercent, int32 Position,
							  int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	if (Position & kGroupAround)
	{
		Position &= ~kGroupAround;
		if (Position != kStartPosition_Specified)
		{
			unsigned char MapObject = 'G' + Position - kStartPosition_G;
			gScriptPtr->mFindMapObject (MapObject, &StartX, &StartY);
			MapObject += 'g' - 'G';
			gScriptPtr->mFindMapObject (MapObject, &EndX, &EndY);
			Position = kStartPosition_Specified;
		}
	}

	tCreateShipInfo info;
	int32	sX = StartX;
	int32	sY = StartY;
	int32	Edge = 0;		// 0=top, 1=right, 2=bottom, 3=left
	int32	EdgeStep = 2;
	int32	EdgeCount = 0;
	int32	Xstep;
	int32	Ystep;

	info.fShipData.fClassType = LargestHullSize;
	info.fTotalBPV = TotalBPV;
	info.fVariant = VariantPercent;
	info.fClassTypeLast = SmallestHullSize;
	info.fShipData.fPlayerShipIndex = -1;
	info.fShipData.fCustomName = CustomName;
	while (TotalShips--)
	{
		mCreateTeamShip (info, Position, sX, sY, EndX, EndY);
		// test to step the next start position out in a spiral pattern
		if (Position == kStartPosition_Specified)
		{
			// test to start a new edge
			if (EdgeCount == 0)
			{
				// test to reset edge back to top edge and lengthen edge step
				if (Edge > 3)
				{
					Edge = 0;				// start top edge
					EdgeStep += 2;
				}
				switch (Edge)
				{
					case 0:					// starting top edge
					case 3:					// starting left edge
						Xstep = 0;
						Ystep = -1;
						break;
					case 1:					// starting right edge
						Xstep = 0;
						Ystep = 1;
						break;
					case 2:					// starting bottom edge
						Xstep = -1;
						Ystep = 0;
						break;
				}
				EdgeCount = EdgeStep;
			}

			// move to new position
			sX += Xstep;
			sY += Ystep;

			// setup for next edge
			if (EdgeCount == EdgeStep)
			{
				if (Edge == 0)
				{
					Xstep = 1;
					Ystep = 0;
				}
				Edge++;
			}
			EdgeCount--;
		}
		else
		{
			Position++;
		}
	}
}

//----------------------------------------------------------------------
// create single ship from single class type
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (int32 TotalBPV, eClassTypes ShipType, int32 VariantPercent,
							 int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	mCreateFleet (1, TotalBPV, ShipType, ShipType, VariantPercent, 
				  Position, StartX, StartY, EndX, EndY, CustomName);
}

//----------------------------------------------------------------------
// create single ship from random class type
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (int32 TotalBPV, eClassTypes SmallestHullSize, eClassTypes LargestHullSize, 
							 int32 VariantPercent, int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY,
							 const char* CustomName)
{
	mCreateFleet (1, TotalBPV, SmallestHullSize, LargestHullSize, VariantPercent,
				  Position, StartX, StartY, EndX, EndY, CustomName);
}

//----------------------------------------------------------------------
// Start XY/End XY are 1 based
void tTeamInfo::mCreateCPUFleet (int32 VariantPercent, int32 Position,
								 int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	if (fIsCPUTeam)
	{
		if (gScriptPtr->fMission.fMulti_FleetNames[fCPUType][fCPUIndex][0].size() == 0)
		{
			mCreateFleet (gScriptPtr->fMission.fMulti_ShipCounts[fCPUType][fCPUIndex], 
						  gScriptPtr->fMission.fMulti_BPVLimits[fCPUType][fCPUIndex], 
						  gScriptPtr->fMission.fMulti_SmallHulls[fCPUType][fCPUIndex],
						  gScriptPtr->fMission.fMulti_LargeHulls[fCPUType][fCPUIndex], 
						  VariantPercent, Position, StartX, StartY, EndX, EndY, NULL);
		}
		else for (int32 i = 0; i < gScriptPtr->fMission.fMulti_ShipCounts[fCPUType][fCPUIndex]; i++)
		{
			const char* CustomName = NULL;
			if (gScriptPtr->fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][i].size() > 0)
			{
				CustomName = gScriptPtr->fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][i].mCharStr();
			}

			// to avoid ships starting on top of each other we offset from the
			// center point up or down based on their position in the menu
			// (evens up, odds down)
			int32 new_position = Position;
			int32 new_startX = StartX;
			int32 new_startY = StartY;
			int32 new_endX = EndX;
			int32 new_endY = EndY;
			if (new_position & kGroupAround)
			{
				new_position &= ~kGroupAround;
				if (new_position != kStartPosition_Specified)
				{
					unsigned char MapObject = 'G' + new_position - kStartPosition_G;
					gScriptPtr->mFindMapObject (MapObject, &new_startX, &new_startY);
					MapObject += 'g' - 'G';
					gScriptPtr->mFindMapObject (MapObject, &new_endX, &new_endY);
					new_position = kStartPosition_Specified;
				}

				int32 side = fTeam;
				if (side == 2) side = -1;
				else if (side == 3) side = 2;
				else if (side == 4) side = -2;
				else if (side == 5) side = 3;
				new_startY += side;

				if (i == 1) new_startX--;		// move second ship to the left
				if (i == 2) new_startX++;		// move third ship to the right
			}

			mCreateShip (gScriptPtr->fMission.fMulti_FleetNames[fCPUType][fCPUIndex][i].mCharStr(),
						 new_position, new_startX, new_startY, new_endX, new_endY, CustomName);
		}
	}
}

//------------------------------------------------------------------------------------------------
// returns number of ships in play
int32 tTeamInfo::mGetShipsInPlayCount (void)
{
	tShipInfo* ship = mGetFirstShip ();
	if (ship == NULL)
	{
		return -1;			// no ships created yet for this team
	}

	int32 count = 0;
	for (; ship != NULL; ship = mGetNextShip())
	{
		if (ship->mInPlay())
		{
			count++;
		}
	}
	return count;
}

//------------------------------------------------------------------------------------------------
// return kTrue if all ships have been initialized
iTruth tTeamInfo::mAllShipsReady (void)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mTestShipStatus (kShipCreated) == kFalse)
		{
			return kFalse;
		}
	}
	return kTrue;
}

//------------------------------------------------------------------------------------------------
// return kTrue if all ships have reached their final destination
iTruth tTeamInfo::mAllInPosition (void)
{
	for (tShipInfo* Ship = mGetFirstShip (); Ship != NULL; Ship = mGetNextShip())
	{
		if (Ship->mInPlay()
			&& (Ship->mGetShipCallbackEvents() & kNotifyAtDestination))
		{
			return kFalse;
		}
	}
	return kTrue;
}

//------------------------------------------------------------------------------------------------
// order all ships to disengage now
void tTeamInfo::mDisengageShips (eDisengageMethods Method, eAIPriority Priority)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mDisengage(Method, Priority);
	}
}

//------------------------------------------------------------------------------------------------
// order all ships to defend entity
void tTeamInfo::mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mDefendEntity (Entity, ThreatRadius, FollowRadius, Priority);
	}
}

//------------------------------------------------------------------------------------------------
//
void tTeamInfo::mClearCommands (void)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mClearCommands();
	}
}

//------------------------------------------------------------------------------------------------
//
int32 tTeamInfo::mGetCombatBPV (void)
{
	return fCombatBPV;
}

//------------------------------------------------------------------------------------------------
// compute teams combat BPV
void tTeamInfo::mAddToCombatBPV (int32 ShipBPV)
{
	fCombatBPV += ShipBPV;
}

//------------------------------------------------------------------------------------------------
// returns total BPV damage done to fleet
int32 tTeamInfo::mAssessDamages (tTeamInfo* EnemyTeam)
{
	if (EnemyTeam)
	{
		eRaceName EnemyRace = EnemyTeam->mGetRace();
		float EPVFactor = EnemyTeam->mGetEPV (fTeam);

		if (EnemyTeam->mIsTeamLeaving())
		{
			return 0;
		}
		if (mIsTeamLeaving())
		{
			return mGetCombatBPV() * EPVFactor;
		}

		int32 DamageBPV = 0;
		for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
		{
			DamageBPV += ship->mAssessDamage (EnemyRace) * EPVFactor;
		}
		return DamageBPV;
	}
	return 0;
}

//------------------------------------------------------------------------------------------------
// 
void tTeamInfo::mRemoveShipAsTarget (tShipInfo* TargetShip)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mSetShipTarget (TargetShip, kNeverPriority);
	}
}

//------------------------------------------------------------------------------------------------
// returns reason why team is no longer in play 
iNotifyEvents tTeamInfo::mGetTeamStatus (void)
{
	if (mIsTeamLeaving())
	{
		return kNotifyPlayerLeaving;
	}

	int32 disengaged_count = 0;
	int32 captured_count = 0;
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mTestShipStatus (kShipDisengaged))
		{
			disengaged_count++;
		}
		else if (ship->mTestShipStatus (kShipCaptured))
		{
			captured_count++;
		}
	}

	if (disengaged_count)
	{
		return kNotifyDisengaged;
	}
	else if (captured_count)
	{
		return kNotifyOnCapture;
	}

	return kNotifyAtDeath;
}

//------------------------------------------------------------------------------------------------
// sets *this teams relationship to Team (NOT Teams relationship to *this team)
void tTeamInfo::mSetRelations (int32 Team, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor)
{
	fAITargets.fRelationships[Team] = Relations;
	fAITargets.fTargetGoals[Team] = Goals;
	fAITargets.fEPVFactors[Team] = EPVFactor;

	// let the game know of this change
	tShipSystemsInfo info;

	info.fAction = kSetTeamRelationship;
	info.fInt32[0] = Relations;
	info.fInt32[1] = fTeam;					// *this team
	info.fInt32[2] = Team;
	info.fInt32[3] = Goals;
	info.fFloat[0] = EPVFactor;

	gScriptPtr->mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
eTeamRelationship tTeamInfo::mGetRelations (eTeamID Team)
{
	if (Team > kNoTeam && Team < kMaxTeams)
	{
		return (fAITargets.fRelationships[Team]);
	}
	return kUnknownRelationship;
}

//------------------------------------------------------------------------------------------------
//
eTargetGoals tTeamInfo::mGetGoal (eTeamID Team)
{
	if (Team > kNoTeam && Team < kMaxTeams)
	{
		return (fAITargets.fTargetGoals[Team]);
	}
	return kGoalIgnore;
}

//------------------------------------------------------------------------------------------------
//
float tTeamInfo::mGetEPV (eTeamID Team)
{
	if (Team > kNoTeam && Team < kMaxTeams)
	{
		return (fAITargets.fEPVFactors[Team]);
	}
	return 0.0;
}

//------------------------------------------------------------------------------------------------
//
int32 tTeamInfo::mGetShipCount (eShipStatus Status)
{
	int32 count = 0;
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mTestShipStatus (Status))
		{
			count++;
		}
	}
	return count;
}

//------------------------------------------------------------------------------------------------
//
void tTeamInfo::mSetAsCPUTeam (eMulti_TeamTypes Type, int32 Index)
{
	fIsCPUTeam = kTrue;
	fCPUType = Type;
	fCPUIndex = Index;
}

//------------------------------------------------------------------------------------------------
//
iTruth tTeamInfo::mIsCPUTeam (void)
{
	return fIsCPUTeam;
}

//------------------------------------------------------------------------------------------------
//
const char* tTeamInfo::mGetCPUCallName (tShipInfo& Ship)
{
	const char* call_name = NULL;
	if (fIsCPUTeam)
	{
		int32 index = 0;
		for (tShipInfo* test_ship = mGetFirstShip (); test_ship != NULL; test_ship = mGetNextShip())
		{
			if (test_ship->mGetShipID() == Ship.mGetShipID())
			{
				break;
			}
			index++;
		}

		if (index < 3)
		{
			if (gScriptPtr->fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][index].size() != 0)
			{
				call_name = gScriptPtr->fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][index].mCharStr();
			}
		}
	}
	return call_name;
}

//------------------------------------------------------------------------------------------------
//
float tTeamInfo::mGetTotalDamagePercent (void)
{
	int32 count = 0;
	float total_damage = 0.0;
	for (tShipInfo* Ship = mGetFirstShip (); Ship != NULL; Ship = mGetNextShip())
	{
		count++;
		if (Ship->mTestShipStatus (kShipCaptured) == kTrue)
		{
			total_damage += 100.0;
		}
		else
		{
			total_damage += Ship->mGetDamagePercent();
		}
	}
	if (count)
	{
		return total_damage / count;
	}
	return 100.0;
}

//------------------------------------------------------------------------------------------------
//
void tTeamInfo::mSetTeamLeaving (void)
{
	fTeamLeaving = kTrue;
}

//------------------------------------------------------------------------------------------------
//
iTruth tTeamInfo::mIsTeamLeaving (void)
{
	return fTeamLeaving;
}

//------------------------------------------------------------------------------------------------
//
eClassTypes tTeamInfo::mGetMaxClassType (void)
{
	eClassTypes Max = kNoClassType;
	for (tShipInfo* Ship = mGetFirstShip (); Ship != NULL; Ship = mGetNextShip())
	{
		eClassTypes type = Ship->mGetClassType();
		if (type > Max)
		{
			Max = type;
		}
	}
	return Max;
}


// 4Nov99	zdrummond	Added Comment Header and Footer's