//	<Name>	TeamInfo.h
//
//	<Legal>	Copyright (c) 1997-1998 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	DESCRIPTION
//

#if !defined( TeamInfo_h )
#define TeamInfo_h

#include "ShipInfo.h"

struct tAITargetInfo
{
	eTeamRelationship	fRelationships[kMaxTeams];
	eTargetGoals		fTargetGoals[kMaxTeams];
	float				fEPVFactors[kMaxTeams];
};

class tTeamInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	eTeamID		mGetTeam (void);
	eRaceName	mGetRace (void);
	int32		mGetShipsInPlayCount (void);
	eTeamID		mGetMultiplayerTeam (void);

	// AI specific commands
	void		mClearCommands (void);
	void		mDisengageShips (eDisengageMethods Method, eAIPriority Priority);
	void		mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority);

	// Start XY/End XY are 1 based
	void		mCreateShip (int32 PlayerShipIndex, tShipInGame& ShipData);
	void		mCreateShip (int32 PlayerShipIndex, const char* ClassName, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateShip (const char* ClassName, int32 Position, 
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateShip (int32 TotalBPV, eClassTypes ShipType, int32 VariantPercent, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateShip (int32 TotalBPV, eClassTypes SmallestHullSize, eClassTypes LargestHullSize, 
							 int32 VariantPercent, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateFleet (int32 TotalShips, int32 TotalBPV, eClassTypes SmallestHullSize, 
							  eClassTypes LargestHullSize, int32 VariantPercent, int32 Position,
							  int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1, 
							  const char* CustomName = NULL);
	void		mDuplicateShip (tShipInfo* Ship, int32 Position,
								int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1);
	void		mCreateCPUFleet (int32 VariantPercent, int32 Position,
								 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1);

	tShipInfo*	mGetFirstShip (void);
	tShipInfo*	mGetNextShip (void);
	tShipInfo*	mGetFirstShipInPlay (void);
	tShipInfo*	mGetNextShipInPlay (void);
	tShipInfo*	mGetThisShip (int32 ShipIndex);		// return ship at given index - index is 1 based
	iTruth		mAllInPosition (void);
	int32		mGetCombatBPV (void);
	int32		mAssessDamages (tTeamInfo* EnemyTeam);
	float		mGetTotalDamagePercent (void);		// must be called before ending mission!!!
	eClassTypes mGetMaxClassType (void);

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:
	tShipInfo*		mFindShipIndex (int32 CallbackID);
	tShipInfo*		mFindShipID (int32 ShipID);
	iTruth			mAllShipsReady (void);
	void			mAddToCombatBPV (int32 ShipBPV);
	void			mRemoveShipAsTarget (tShipInfo* TargetShip);
	iNotifyEvents	mGetTeamStatus (void);
	void			mSetRelations (int32 Team, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor);
	eTeamRelationship mGetRelations (eTeamID Team);
	eTargetGoals	  mGetGoal (eTeamID Team);
	float			  mGetEPV (eTeamID Team);
	int32			mGetShipCount (eShipStatus Status);
	iTruth			mAddShipToTeamList (tCreateShipInfo& info);
	void			mSetAsCPUTeam (eMulti_TeamTypes Type, int32 Index);
	iTruth			mIsCPUTeam (void);
	const char*		mGetCPUCallName (tShipInfo& Ship);
	void			mSetMultiplayerTeam (eTeamID Team);
	void			mSetTeamLeaving (void);
	iTruth			mIsTeamLeaving (void);


	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tTeamInfo (eTeamID Team, eRaceName Race);
	~tTeamInfo (void);

protected:
	void	mCreateTeamShip (tCreateShipInfo& info, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1);

	eTeamID			fTeam;
	eTeamID			fMultiplayerTeam;
	eRaceName		fRace;
	int32			fNextCallbackID;

	vector<tShipInfo*>	fShipList;
	int32				fShipIndex;
	int32				fCombatBPV;

	tAITargetInfo		fAITargets;

	iTruth				fIsCPUTeam;
	eMulti_TeamTypes	fCPUType;
	int32				fCPUIndex;
	iTruth				fTeamLeaving;
};


#endif


// 4Nov99	zdrummond	Added Comment Header and Footer's