#include "scriptinterface.h"

tScriptInterface gScript;

extern tString Messages[];
extern int32 TotalMaps;
extern const char* MapTypes[];
extern const char* MapLines[];

// ************************************************************************************************
//					ship support routines
// ************************************************************************************************

tShipInfo::tShipInfo (eTeamID Team, eRaceName Race, int32 CallbackID,
					  int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	fTeam = Team;
	fOriginalTeam = Team;
	fRace = Race;
	fStatus = kUndefined;
	fShipBPV = 0;
	fCallbackID = CallbackID;
	fStartPositionIndex = Position;
	fStartX = StartX;
	fStartY = StartY;
	fEndX = EndX;
	fEndY = EndY;
}

tShipInfo::~tShipInfo (void)
{
}

//----------------------------------------------------------------------
//
void tShipInfo::mClearCommands (void)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIClearCommands;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipTarget (tShipInfo* TargetShip, eAIPriority Priority)
{
	if (TargetShip)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAITargetObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = TargetShip->mGetShipID();
		info.fShipEntityID = fShipID;
		gScript.mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mDisengage (eDisengageMethods Method, eAIPriority Priority)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIDisengage;
	info.fInt32[1] = Priority;
	info.fInt32[2] = Method;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	mSetShipStatus (kShipDisengaged);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mTractorEntity (tShipInfo* Entity, int32 EndX, int32 EndY, eAIPriority Priority)
{
	if (Entity)
	{
		fShipEvents |= kNotifyEndTractor | kNotifyBeginTractor;

		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIMoveObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fInt32[3] = fShipEvents;
		info.fGridX = EndX - 1;
		info.fGridY = EndY - 1;
		info.fShipEntityID = fShipID;
		gScript.mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
// ThreatRadius and FollowRadius are based on tactical map distance
void tShipInfo::mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority)
{
	if (Entity)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIDefendObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fFloat[0] = ThreatRadius;
		info.fFloat[1] = FollowRadius;
		info.fShipEntityID = fShipID;
		gScript.mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
//
void tShipInfo::mFollowEntity (tShipInfo* Entity, eAIPriority Priority)
{
	if (Entity)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIFollowObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fShipEntityID = fShipID;
		gScript.mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
//
void tShipInfo::mFireProbe (tShipInfo* Target, eAIPriority Priority)
{
	if (Target)
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIFireProbe;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Target->mGetShipID();
		info.fShipEntityID = fShipID;
		gScript.mEntityMgr (info);
	}
}

//------------------------------------------------------------------------------------------------
// DetectionRange is based on tactical map distance
void tShipInfo::mPatrol (float DetectionRange, int32 PatrolPointCount, tMapPoint* PatrolPoints, eAIPriority Priority)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIPatrol;
	info.fInt32[1] = Priority;
	info.fInt32[2] = PatrolPointCount;
	info.fFloat[0] = DetectionRange;
	info.fTextPtr = (const char*)PatrolPoints;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// sets state of officers on AI ships
void tShipInfo::mChangeAIOfficerState (eAIOfficers Officer, iTruth State)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIChangeOfficerState;
	info.fInt32[1] = Officer;
	info.fInt32[2] = State;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// sets cloaking state of ship - NOTE: kCloakOfficer MUST be turned off first
// with mChangeAIOfficerState function before using this function
void tShipInfo::mCloak (iTruth State)
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAICloakShip;
	info.fInt32[1] = State;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipThrust (int32 NewThrust)
{
	tShipSystemsInfo info;

	info.fAction = kSetNewThrust;
	info.fFloat[0] = NewThrust;
	info.fInt32[0] = 0;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipMaxSpeed (int32 MaxSpeed)
{
	tShipSystemsInfo info;

	info.fAction = kSetNewThrust;
	info.fFloat[0] = MaxSpeed;
	info.fInt32[0] = -1;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipPlayerControlled (void)
{
	tShipSystemsInfo info;

	info.fAction = kSetPlayerControlledShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = -1;
	gScript.mEntityMgr (info);

	gScript.fPlayerShip = this;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipAIControlled (iTruth AIControlled)
{
	tShipSystemsInfo info;

	info.fAction = kSetPlayerControlledShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = AIControlled;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipCallbackEvents (iNotifyEvents NewEvents)
{
	mReplaceShipCallbackEvents (fShipEvents | NewEvents);
}

//----------------------------------------------------------------------
//
void tShipInfo::mReplaceShipCallbackEvents (iNotifyEvents NewEvents)
{
	tShipSystemsInfo info;

	info.fAction = kSetNewEvents;
	info.fInt32[0] = NewEvents;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	fShipEvents = NewEvents;
}

//----------------------------------------------------------------------
//
iNotifyEvents tShipInfo::mGetShipCallbackEvents (void)
{
	return fShipEvents;
}

//----------------------------------------------------------------------
// End X/Y are 1 based
void tShipInfo::mSetFinalDestination (int32 EndX, int32 EndY)
{
	fShipEvents |= kNotifyAtDestination;

	tShipSystemsInfo info;

	info.fAction = kSetFinalDestination;
	info.fInt32[0] = mGetStartPositionIndex() - kStartPosition_G;
	info.fInt32[1] = fShipEvents;
	info.fGridX = EndX - 1;
	info.fGridY = EndY - 1;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
// Grid X/Y are 1 based - Distance is based on tactical map distance
void tShipInfo::mTrackTacticalDistance (float Distance, int32 GridX, int32 GridY, iTruth NotifyInOut)
{
	fShipEvents |= kNotifyAtDistance;
	fTrackingDistance = Distance;
	if (NotifyInOut)
	{
		Distance = -Distance;	// a neg. dist means the game should detect boundry crossings both ways
	}

	tShipSystemsInfo info;

	info.fAction = kTrackDistanceFromTarget;
	info.fInt32[0] = fShipEvents;
	info.fFloat[0] = Distance;
	info.fGridX = GridX - 1;
	info.fGridY	= GridY - 1;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
// ButtonNumber is 1 based
void tShipInfo::mSetCommButton (int32 ButtonNumber, int32 MsgID, iTruth SetAsTarget)
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = MsgID;
	info.fInt32[1] = ButtonNumber;
	info.fInt32[2] = SetAsTarget;
	info.fText = Messages[MsgID];
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
// ButtonNumber is 1 based
void tShipInfo::mClearCommButton (int32 ButtonNumber)
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = -1;
	info.fInt32[1] = ButtonNumber;
	info.fInt32[2] = kFalse;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
// sets/reset status of ship
void tShipInfo::mSetShipStatus (eShipStatus Status, iTruth SetState)
{
	if (SetState)
	{
		fStatus |= Status;
	}
	else
	{
		fStatus &= ~Status;
	}
}

//----------------------------------------------------------------------
// returns status of ship
iTruth tShipInfo::mTestShipStatus (int32u Status)
{
	if (fStatus & Status)
	{
		return kTrue;
	}
	return kFalse;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipID (int32 ShipID)
{
	fShipID = ShipID;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetTeam (eTeamID NewTeam)
{
	fTeam = NewTeam;
}

//----------------------------------------------------------------------
//
eTeamID tShipInfo::mGetTeam (void)
{
	return fTeam;
}

//----------------------------------------------------------------------
//
eTeamID tShipInfo::mGetOriginalTeam (void)
{
	return fOriginalTeam;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetShipID (void)
{
	return fShipID;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetClassType (eClassTypes ClassType)
{
	fClassType = ClassType;
}

//----------------------------------------------------------------------
//
eClassTypes tShipInfo::mGetClassType (void)
{
	return fClassType;
}

//----------------------------------------------------------------------
//
const char*	tShipInfo::mGetClassName (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipClassName;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	return (info.fTextPtr);
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetShipTargetID (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipTargetID;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	return (info.fInt32[0]);
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetStartPositionIndex (void)
{
	return fStartPositionIndex;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetStartX (void)
{
	return fStartX + 1;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetStartY (void)
{
	return fStartY + 1;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetEndX (void)
{
	return fEndX + 1;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetEndY (void)
{
	return fEndY + 1;
}

//----------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTrackingDistance (void)
{
	return fTrackingDistance;
}

//----------------------------------------------------------------------
//
void tShipInfo::mSetShipBPV (int32 ShipBPV)
{
	fShipBPV = ShipBPV;
}

//----------------------------------------------------------------------
//
int32 tShipInfo::mGetShipBPV (void)
{
	return fShipBPV;
}

//----------------------------------------------------------------------
// return kTrue if ship is alive and has not been captured or disengaged
iTruth tShipInfo::mInPlay (void)
{
	if (gScript.fMission.mIsTeamLeaving(fTeam))
	{
		return kFalse;
	}

	if (mTestShipStatus (kShipCaptured | kShipDisengaged | kShipDestroyed))
	{
		return kFalse;
	}
	return kTrue;
}

//------------------------------------------------------------------------------------------------
// return total BPV damage to this ship
int32 tShipInfo::mAssessDamage (eRaceName EnemyRace)
{
	int32 DamageBPV = 0;

	if (fStatus & kShipCaptured)
	{
		if (fClassType == kClassShuttle)
		{
			DamageBPV = fShipBPV;
		}
		else if (EnemyRace == kOrion)
		{
			DamageBPV = fShipBPV * 5;
		}
		else
		{
			DamageBPV = fShipBPV * 2;
		}
	}
	else if (fStatus & kShipDestroyed)
	{
		DamageBPV = fShipBPV;
	}
	else if (fStatus & kShipDisengaged)
	{
		if (EnemyRace != kOrion)
		{
			DamageBPV = float(fShipBPV * .5);
		}
	}

	return DamageBPV;
}

//------------------------------------------------------------------------------------------------
// 
int32 tShipInfo::mGetCallbackID (void)
{
	return fCallbackID;
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mGetGridPosition (int32* GridX, int32* GridY)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipPosition;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	*GridX = info.fGridX + 1;
	*GridY = info.fGridY + 1;
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTacticalDistance (tShipInfo* TargetShip)
{
	if (TargetShip)
	{
		tShipSystemsInfo info;

		info.fAction = kGetTacticalDistance;
		info.fInt32[0] = kGetShipPosition;
		info.fInt32[1] = TargetShip->mGetShipID();
		info.fShipEntityID = fShipID;
		gScript.mEntityMgr (info);

		return (info.fFloat[0]);
	}
	return 0.0;
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTacticalDistance (unsigned char MapObject)
{
	tShipSystemsInfo info;

	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fInt32[1] = MapObject;
	info.fGridX = -1;					// == -1 indicates that distance is between ship and map object
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	return (info.fFloat[0]);
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tShipInfo::mGetTacticalDistance (int32 GridX, int32 GridY)
{
	tShipSystemsInfo info;

	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	return (info.fFloat[0]);
}

//------------------------------------------------------------------------------------------------
// 
eAlertLevels tShipInfo::mGetAlertStatus (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetAlertStatus;
	info.fInt32[0] = -1;
	gScript.mEntityMgr (info);

	return (eAlertLevels(info.fInt32[0]));
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetAlertStatus (eAlertLevels Status)
{
	tShipSystemsInfo info;

	info.fAction = kGetAlertStatus;
	info.fInt32[0] = Status;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mDamageSystem (eDamageSystems System, float Percent)
{
	tShipSystemsInfo info;

	info.fAction = kDamageShipSystem;
	info.fInt32[0] = System;
	info.fFloat[0] = Percent;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
int32 tShipInfo::mAccessSupplies (eShipSupplyAccess Access, int32 SupplyItem, int32 NewValue)
{
	tShipSystemsInfo info;

	info.fAction = kAccessShipSupply;
	info.fInt32[0] = Access;
	info.fInt32[1] = SupplyItem;
	info.fInt32[2] = NewValue;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	return info.fInt32[2];
}

//------------------------------------------------------------------------------------------------
// 
float tShipInfo::mGetDamagePercent (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetShipDamagePercent;
	info.fShipEntityID = fShipID;
	info.fFloat[0] = 100.0;				// assume 100% damage if no ship
	gScript.mEntityMgr (info);

	return info.fFloat[0];
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mChangeName (int32 MsgID)
{
	tShipSystemsInfo info;

	info.fAction = kSetObjectName;
	info.fTextPtr = Messages[MsgID].mCharStr();
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
eOfficerRank tShipInfo::mGetOfficerRank (eOfficerTypes Officer)
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = kUnknownRank;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	return ((eOfficerRank)info.fInt32[2]);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetOfficerRank (eOfficerTypes Officer, eOfficerRank Rank)
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = Rank;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetOfficerName (eOfficerTypes Officer, int32 MsgID)
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = kMaxOfficerRank;
	info.fShipEntityID = fShipID;
	info.fText = Messages[MsgID];
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
// 
void tShipInfo::mSetTargetable (iTruth State)
{
	tShipSystemsInfo info;

	info.fAction = kMakeShipTargetable;
	info.fInt32[0] = State;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);

	info.fAction = kMakeShipAbleToTarget;
	info.fInt32[0] = State;
	info.fShipEntityID = fShipID;
	gScript.mEntityMgr (info);
}

// ************************************************************************************************
//					team support routines
// ************************************************************************************************

//----------------------------------------------------------------------
//
tTeamInfo::tTeamInfo (eTeamID Team, eRaceName Race)
{
	fTeam = Team;
	fMultiplayerTeam = Team;
	fRace = Race;
	fCombatBPV = 0;
	fNextCallbackID = (Team * (0x7fffffff / kMaxTeams)) + 1;
	fIsCPUTeam = kFalse;
	fTeamLeaving = kFalse;

	// setup AI info
	int32 i = kMaxTeams;
	while (i--)
	{
		fAITargets.fRelationships[i] = kUnknownRelationship;
		fAITargets.fTargetGoals[i] = kGoalIgnore;
		fAITargets.fEPVFactors[i] = 1.0;
	}
	fAITargets.fRelationships[Team] = kTeamSelfAware;
}

//----------------------------------------------------------------------
//
tTeamInfo::~tTeamInfo (void)
{
	while (fShipList.size())
	{
		tShipInfo* Ship = fShipList.back();
		delete Ship;
		Ship = NULL;
		fShipList.pop_back();
	}
	fShipList.erase(fShipList.begin(), fShipList.end());
}

//----------------------------------------------------------------------
//
eTeamID tTeamInfo::mGetTeam (void)
{
	return fTeam;
}

//----------------------------------------------------------------------
//
eTeamID tTeamInfo::mGetMultiplayerTeam (void)
{
	return fMultiplayerTeam;
}

//----------------------------------------------------------------------
//
void tTeamInfo::mSetMultiplayerTeam (eTeamID Team)
{
	fMultiplayerTeam = Team;
}

//----------------------------------------------------------------------
//
eRaceName tTeamInfo::mGetRace (void)
{
	return fRace;
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetFirstShip (void)
{
	fShipIndex = 0;
	return mGetNextShip();
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetNextShip (void)
{
	if (fShipIndex >= fShipList.size())
	{
		return NULL;
	}

	tShipInfo* Ship = fShipList[fShipIndex++];
	return Ship;
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetFirstShipInPlay (void)
{
	fShipIndex = 0;
	return mGetNextShipInPlay();
}

//----------------------------------------------------------------------
//
tShipInfo* tTeamInfo::mGetNextShipInPlay (void)
{
	while (fShipIndex < fShipList.size())
	{
		tShipInfo* Ship = fShipList[fShipIndex++];
		if (Ship && Ship->mInPlay())
		{
			return Ship;
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
// return ship at given index - index is 1 based
tShipInfo* tTeamInfo::mGetThisShip (int32 ShipIndex)
{
	if (ShipIndex > fShipList.size ())
	{
		return NULL;
	}

	ShipIndex--;
	if (ShipIndex < 0)
	{
		ShipIndex = 0;
	}

	return (fShipList[ShipIndex]);
}

//----------------------------------------------------------------------
// return pointer to ship that matches given position index
tShipInfo* tTeamInfo::mFindShipIndex (int32 CallbackID)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mGetCallbackID() == CallbackID)
		{
			return ship;
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
// return pointer to ship that matches given ID
tShipInfo* tTeamInfo::mFindShipID (int32 ShipID)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mGetShipID() == ShipID)
		{
			return ship;
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
//
iTruth tTeamInfo::mAddShipToTeamList (tCreateShipInfo& info)
{
	// call back into the game to create the ship
	gScript.mCreateSpecShip (info);

	if (info.fSelectedClassType == kNoClassType)
	{
		return kFalse;
	}

	// add this ship to the teams list if it was created
	tShipInfo* Ship = new tShipInfo (fTeam, fRace, info.fShipData.fScriptCallbackID, info.fShipData.fShipPosition, 
									 info.fShipData.fStartX, info.fShipData.fStartY, info.fShipData.fEndX, info.fShipData.fEndY);
	fShipList.push_back (Ship);
	Ship->mSetClassType (info.fSelectedClassType);

	Ship->mSetShipBPV (info.fShipData.fShipBPV);
	mAddToCombatBPV (info.fShipData.fShipBPV);
	return kTrue;
}

//----------------------------------------------------------------------
// call back into game engine to create a ship
// Start XY/End XY are 1 based
void tTeamInfo::mCreateTeamShip (tCreateShipInfo& info, int32 Position,
								 int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	Position &= ~kGroupAround;

	if (Position == kStartPosition_Specified)
	{
		if (StartX < 1)
		{
			StartX = 1;
		}
		if (StartY < 1)
		{
			StartY = 1;
		}
		if (EndX < 1)
		{
			EndX = StartX;
		}
		if (EndY < 1)
		{
			EndY = StartY;
		}
	}

	info.fShipRace = fRace;
	info.fShipData.fTeam = fTeam;
	info.fShipData.fShipPosition = Position;
	info.fShipData.fStartX = StartX - 1;
	info.fShipData.fStartY = StartY - 1;
	info.fShipData.fEndX = EndX - 1;
	info.fShipData.fEndY = EndY - 1;
	info.fShipData.fScriptCallbackID = fNextCallbackID;
	info.fShipData.fCreateShadowShip = kFalse;

	if (mAddShipToTeamList (info))
	{
		fNextCallbackID++;
	}
}

//----------------------------------------------------------------------
// create ship from class name only
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (int32 PlayerShipIndex, const char* ClassName, int32 Position,
							 int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ClassName;
	info.fShipData.fPlayerShipIndex = PlayerShipIndex;
	info.fShipData.fCustomName = CustomName;
	mCreateTeamShip (info, Position, StartX, StartY, EndX, EndY);
}

//----------------------------------------------------------------------
// create ship from supplied ship data
void tTeamInfo::mCreateShip (int32 PlayerShipIndex, tShipInGame& ShipData)
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ShipData.fClassName;
	info.fShipData.fPlayerShipIndex = PlayerShipIndex;
	info.fShipData.fCustomName = NULL;
	mCreateTeamShip (info, ShipData.fShipPosition, ShipData.fStartX, ShipData.fStartY, ShipData.fEndX, ShipData.fEndY);
}

//----------------------------------------------------------------------
// create ship from class name only
// Start XY/End XY are 1 based
void tTeamInfo::mDuplicateShip (tShipInfo* Ship, int32 Position,
								int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	if (Ship)
	{
		tCreateShipInfo info;

		info.fShipData.fClassType = kNoClassType;
		info.fShipData.fClassName = Ship->mGetClassName();
		info.fShipData.fPlayerShipIndex = -1;
		info.fShipData.fCustomName = NULL;
		mCreateTeamShip (info, Position, StartX, StartY, EndX, EndY);
	}
}

//----------------------------------------------------------------------
// create ship from class name only
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (const char* ClassName, int32 Position,
							 int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ClassName;
	info.fShipData.fPlayerShipIndex = -1;
	info.fShipData.fCustomName = CustomName;
	mCreateTeamShip (info, Position, StartX, StartY, EndX, EndY);
}

//----------------------------------------------------------------------
// create group of ships
// Start XY/End XY are 1 based
void tTeamInfo::mCreateFleet (int32 TotalShips, int32 TotalBPV, eClassTypes SmallestHullSize, 
							  eClassTypes LargestHullSize, int32 VariantPercent, int32 Position,
							  int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	if (Position & kGroupAround)
	{
		Position &= ~kGroupAround;
		if (Position != kStartPosition_Specified)
		{
			unsigned char MapObject = 'G' + Position - kStartPosition_G;
			gScript.mFindMapObject (MapObject, &StartX, &StartY);
			MapObject += 'g' - 'G';
			gScript.mFindMapObject (MapObject, &EndX, &EndY);
			Position = kStartPosition_Specified;
		}
	}

	tCreateShipInfo info;
	int32	sX = StartX;
	int32	sY = StartY;
	int32	Edge = 0;		// 0=top, 1=right, 2=bottom, 3=left
	int32	EdgeStep = 2;
	int32	EdgeCount = 0;
	int32	Xstep;
	int32	Ystep;

	info.fShipData.fClassType = LargestHullSize;
	info.fTotalBPV = TotalBPV;
	info.fVariant = VariantPercent;
	info.fClassTypeLast = SmallestHullSize;
	info.fShipData.fPlayerShipIndex = -1;
	info.fShipData.fCustomName = CustomName;
	while (TotalShips--)
	{
		mCreateTeamShip (info, Position, sX, sY, EndX, EndY);
		// test to step the next start position out in a spiral pattern
		if (Position == kStartPosition_Specified)
		{
			// test to start a new edge
			if (EdgeCount == 0)
			{
				// test to reset edge back to top edge and lengthen edge step
				if (Edge > 3)
				{
					Edge = 0;				// start top edge
					EdgeStep += 2;
				}
				switch (Edge)
				{
					case 0:					// starting top edge
					case 3:					// starting left edge
						Xstep = 0;
						Ystep = -1;
						break;
					case 1:					// starting right edge
						Xstep = 0;
						Ystep = 1;
						break;
					case 2:					// starting bottom edge
						Xstep = -1;
						Ystep = 0;
						break;
				}
				EdgeCount = EdgeStep;
			}

			// move to new position
			sX += Xstep;
			sY += Ystep;

			// setup for next edge
			if (EdgeCount == EdgeStep)
			{
				if (Edge == 0)
				{
					Xstep = 1;
					Ystep = 0;
				}
				Edge++;
			}
			EdgeCount--;
		}
		else
		{
			Position++;
		}
	}
}

//----------------------------------------------------------------------
// create single ship from single class type
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (int32 TotalBPV, eClassTypes ShipType, int32 VariantPercent,
							 int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName)
{
	mCreateFleet (1, TotalBPV, ShipType, ShipType, VariantPercent, 
				  Position, StartX, StartY, EndX, EndY, CustomName);
}

//----------------------------------------------------------------------
// create single ship from random class type
// Start XY/End XY are 1 based
void tTeamInfo::mCreateShip (int32 TotalBPV, eClassTypes SmallestHullSize, eClassTypes LargestHullSize, 
							 int32 VariantPercent, int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY,
							 const char* CustomName)
{
	mCreateFleet (1, TotalBPV, SmallestHullSize, LargestHullSize, VariantPercent,
				  Position, StartX, StartY, EndX, EndY, CustomName);
}

//----------------------------------------------------------------------
// Start XY/End XY are 1 based
void tTeamInfo::mCreateCPUFleet (int32 VariantPercent, int32 Position,
								 int32 StartX, int32 StartY, int32 EndX, int32 EndY)
{
	if (fIsCPUTeam)
	{
		if (gScript.fMission.fMulti_FleetNames[fCPUType][fCPUIndex][0].size() == 0)
		{
			mCreateFleet (gScript.fMission.fMulti_ShipCounts[fCPUType][fCPUIndex], 
						  gScript.fMission.fMulti_BPVLimits[fCPUType][fCPUIndex], 
						  gScript.fMission.fMulti_SmallHulls[fCPUType][fCPUIndex],
						  gScript.fMission.fMulti_LargeHulls[fCPUType][fCPUIndex], 
						  VariantPercent, Position, StartX, StartY, EndX, EndY, NULL);
		}
		else for (int32 i = 0; i < gScript.fMission.fMulti_ShipCounts[fCPUType][fCPUIndex]; i++)
		{
			const char* CustomName = NULL;
			if (gScript.fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][i].size() > 0)
			{
				CustomName = gScript.fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][i].mCharStr();
			}

			// to avoid ships starting on top of each other we offset from the
			// center point up or down based on their position in the menu
			// (evens up, odds down)
			int32 new_position = Position;
			int32 new_startX = StartX;
			int32 new_startY = StartY;
			int32 new_endX = EndX;
			int32 new_endY = EndY;
			if (new_position & kGroupAround)
			{
				new_position &= ~kGroupAround;
				if (new_position != kStartPosition_Specified)
				{
					unsigned char MapObject = 'G' + new_position - kStartPosition_G;
					gScript.mFindMapObject (MapObject, &new_startX, &new_startY);
					MapObject += 'g' - 'G';
					gScript.mFindMapObject (MapObject, &new_endX, &new_endY);
					new_position = kStartPosition_Specified;
				}

				int32 side = fTeam;
				if (side == 2) side = -1;
				else if (side == 3) side = 2;
				else if (side == 4) side = -2;
				else if (side == 5) side = 3;
				new_startY += side;

				if (i == 1) new_startX--;		// move second ship to the left
				if (i == 2) new_startX++;		// move third ship to the right
			}

			mCreateShip (gScript.fMission.fMulti_FleetNames[fCPUType][fCPUIndex][i].mCharStr(),
						 new_position, new_startX, new_startY, new_endX, new_endY, CustomName);
		}
	}
}

//------------------------------------------------------------------------------------------------
// returns number of ships in play
int32 tTeamInfo::mGetShipsInPlayCount (void)
{
	tShipInfo* ship = mGetFirstShip ();
	if (ship == NULL)
	{
		return -1;			// no ships created yet for this team
	}

	int32 count = 0;
	for (; ship != NULL; ship = mGetNextShip())
	{
		if (ship->mInPlay())
		{
			count++;
		}
	}
	return count;
}

//------------------------------------------------------------------------------------------------
// return kTrue if all ships have been initialized
iTruth tTeamInfo::mAllShipsReady (void)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mTestShipStatus (kShipCreated) == kFalse)
		{
			return kFalse;
		}
	}
	return kTrue;
}

//------------------------------------------------------------------------------------------------
// return kTrue if all ships have reached their final destination
iTruth tTeamInfo::mAllInPosition (void)
{
	for (tShipInfo* Ship = mGetFirstShip (); Ship != NULL; Ship = mGetNextShip())
	{
		if (Ship->mInPlay()
			&& (Ship->mGetShipCallbackEvents() & kNotifyAtDestination))
		{
			return kFalse;
		}
	}
	return kTrue;
}

//------------------------------------------------------------------------------------------------
// order all ships to disengage now
void tTeamInfo::mDisengageShips (eDisengageMethods Method, eAIPriority Priority)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mDisengage(Method, Priority);
	}
}

//------------------------------------------------------------------------------------------------
// order all ships to defend entity
void tTeamInfo::mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mDefendEntity (Entity, ThreatRadius, FollowRadius, Priority);
	}
}

//------------------------------------------------------------------------------------------------
//
void tTeamInfo::mClearCommands (void)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mClearCommands();
	}
}

//------------------------------------------------------------------------------------------------
//
int32 tTeamInfo::mGetCombatBPV (void)
{
	return fCombatBPV;
}

//------------------------------------------------------------------------------------------------
// compute teams combat BPV
void tTeamInfo::mAddToCombatBPV (int32 ShipBPV)
{
	fCombatBPV += ShipBPV;
}

//------------------------------------------------------------------------------------------------
// returns total BPV damage done to fleet
int32 tTeamInfo::mAssessDamages (tTeamInfo* EnemyTeam)
{
	if (EnemyTeam)
	{
		eRaceName EnemyRace = EnemyTeam->mGetRace();
		float EPVFactor = EnemyTeam->mGetEPV (fTeam);

		if (EnemyTeam->mIsTeamLeaving())
		{
			return 0;
		}
		if (mIsTeamLeaving())
		{
			return mGetCombatBPV() * EPVFactor;
		}

		int32 DamageBPV = 0;
		for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
		{
			DamageBPV += ship->mAssessDamage (EnemyRace) * EPVFactor;
		}
		return DamageBPV;
	}
	return 0;
}

//------------------------------------------------------------------------------------------------
// 
void tTeamInfo::mRemoveShipAsTarget (tShipInfo* TargetShip)
{
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		ship->mSetShipTarget (TargetShip, kNeverPriority);
	}
}

//------------------------------------------------------------------------------------------------
// returns reason why team is no longer in play 
iNotifyEvents tTeamInfo::mGetTeamStatus (void)
{
	if (mIsTeamLeaving())
	{
		return kNotifyPlayerLeaving;
	}

	int32 disengaged_count = 0;
	int32 captured_count = 0;
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mTestShipStatus (kShipDisengaged))
		{
			disengaged_count++;
		}
		else if (ship->mTestShipStatus (kShipCaptured))
		{
			captured_count++;
		}
	}

	if (disengaged_count)
	{
		return kNotifyDisengaged;
	}
	else if (captured_count)
	{
		return kNotifyOnCapture;
	}

	return kNotifyAtDeath;
}

//------------------------------------------------------------------------------------------------
// sets *this teams relationship to Team (NOT Teams relationship to *this team)
void tTeamInfo::mSetRelations (int32 Team, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor)
{
	fAITargets.fRelationships[Team] = Relations;
	fAITargets.fTargetGoals[Team] = Goals;
	fAITargets.fEPVFactors[Team] = EPVFactor;

	// let the game know of this change
	tShipSystemsInfo info;

	info.fAction = kSetTeamRelationship;
	info.fInt32[0] = Relations;
	info.fInt32[1] = fTeam;					// *this team
	info.fInt32[2] = Team;
	info.fInt32[3] = Goals;
	info.fFloat[0] = EPVFactor;

	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
eTeamRelationship tTeamInfo::mGetRelations (eTeamID Team)
{
	if (Team > kNoTeam && Team < kMaxTeams)
	{
		return (fAITargets.fRelationships[Team]);
	}
	return kUnknownRelationship;
}

//------------------------------------------------------------------------------------------------
//
eTargetGoals tTeamInfo::mGetGoal (eTeamID Team)
{
	if (Team > kNoTeam && Team < kMaxTeams)
	{
		return (fAITargets.fTargetGoals[Team]);
	}
	return kGoalIgnore;
}

//------------------------------------------------------------------------------------------------
//
float tTeamInfo::mGetEPV (eTeamID Team)
{
	if (Team > kNoTeam && Team < kMaxTeams)
	{
		return (fAITargets.fEPVFactors[Team]);
	}
	return 0.0;
}

//------------------------------------------------------------------------------------------------
//
int32 tTeamInfo::mGetShipCount (eShipStatus Status)
{
	int32 count = 0;
	for (tShipInfo* ship = mGetFirstShip (); ship != NULL; ship = mGetNextShip())
	{
		if (ship->mTestShipStatus (Status))
		{
			count++;
		}
	}
	return count;
}

//------------------------------------------------------------------------------------------------
//
void tTeamInfo::mSetAsCPUTeam (eMulti_TeamTypes Type, int32 Index)
{
	fIsCPUTeam = kTrue;
	fCPUType = Type;
	fCPUIndex = Index;
}

//------------------------------------------------------------------------------------------------
//
iTruth tTeamInfo::mIsCPUTeam (void)
{
	return fIsCPUTeam;
}

//------------------------------------------------------------------------------------------------
//
const char* tTeamInfo::mGetCPUCallName (tShipInfo& Ship)
{
	const char* call_name = NULL;
	if (fIsCPUTeam)
	{
		int32 index = 0;
		for (tShipInfo* test_ship = mGetFirstShip (); test_ship != NULL; test_ship = mGetNextShip())
		{
			if (test_ship->mGetShipID() == Ship.mGetShipID())
			{
				break;
			}
			index++;
		}

		if (index < 3)
		{
			if (gScript.fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][index].size() != 0)
			{
				call_name = gScript.fMission.fMulti_FleetCallNames[fCPUType][fCPUIndex][index].mCharStr();
			}
		}
	}
	return call_name;
}

//------------------------------------------------------------------------------------------------
//
float tTeamInfo::mGetTotalDamagePercent (void)
{
	int32 count = 0;
	float total_damage = 0.0;
	for (tShipInfo* Ship = mGetFirstShip (); Ship != NULL; Ship = mGetNextShip())
	{
		count++;
		if (Ship->mTestShipStatus (kShipCaptured) == kTrue)
		{
			total_damage += 100.0;
		}
		else
		{
			total_damage += Ship->mGetDamagePercent();
		}
	}
	if (count)
	{
		return total_damage / count;
	}
	return 100.0;
}

//------------------------------------------------------------------------------------------------
//
void tTeamInfo::mSetTeamLeaving (void)
{
	fTeamLeaving = kTrue;
}

//------------------------------------------------------------------------------------------------
//
iTruth tTeamInfo::mIsTeamLeaving (void)
{
	return fTeamLeaving;
}

//------------------------------------------------------------------------------------------------
//
eClassTypes tTeamInfo::mGetMaxClassType (void)
{
	eClassTypes Max = kNoClassType;
	for (tShipInfo* Ship = mGetFirstShip (); Ship != NULL; Ship = mGetNextShip())
	{
		eClassTypes type = Ship->mGetClassType();
		if (type > Max)
		{
			Max = type;
		}
	}
	return Max;
}

// ************************************************************************************************
//					mission support routines
// ************************************************************************************************



//----------------------------------------------------------------------
//
tMissionInfo::tMissionInfo (void)
{
	int32 index = kMaxTeams;
	while (index--)
	{
		fTeams[index] = NULL;
		fRaces[index] = kUndefinedRace;
	}
	fEndGameLockout = kFalse;
	fLockOutStatus = kGameInProgress;
	fDebriefingTotal = 0;
	fBonusPrestige = 0;
	fTotalAlliedHumanTeams = 0;
	fTotalEnemyHumanTeams = 0;
	fTotalAlliedCPUTeams = 0;
	fTotalEnemyCPUTeams = 0;
}

//----------------------------------------------------------------------
//
tMissionInfo::~tMissionInfo (void)
{
	int32 index = kMaxTeams;
	while (index--)
	{
		if (fTeams[index] != NULL)
		{
			delete fTeams[index];
			fTeams[index] = NULL;
		}
	}
}

//----------------------------------------------------------------------
// file name of mission to schedule (including extension)
void tMissionInfo::mScheduleMission (int32 MsgID)
{
	tShipSystemsInfo info;

	info.fAction = kScheduleMission;
	info.fText = Messages[MsgID];

	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mSetDate (tDynaverseScriptInfo::tMissionDate& Date)
{
	fDate = Date;
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mSetLocation (eSectorName Location)
{
	fLocation = Location;
}


//----------------------------------------------------------------------
//
void tMissionInfo::mSetSystemName (eSystems SystemName)
{
	fSystemName = SystemName;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTeamRace (int32 Team, eRaceName Race)
{
	fRaces[Team] = Race;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetMultiplayerTeams (tDynaverseScriptInfo& info)
{
	// WARNING!! DO NOT USE kPlayerTeam in here. Use info.fPlayerTeam instead!

	for (int32 MemberIndex = 0; MemberIndex < tEntityMgr::kMaxNumberofPlayers; MemberIndex++)
	{
		if (info.fMembersInGame[MemberIndex] != kPlayerTypeNoOne)
		{
			eTeamID team	= info.fMemberTeams[MemberIndex];
			eRaceName race	= info.fMemberRaces[MemberIndex];

			// handle undefined races
			if (race == kUndefinedRace)
			{
				race = info.fEnemyRace;		// assume the enemy race

				// if this member is on the same team as any other human member
				// then use that human members race
				for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
				{
					if (team == info.fMemberTeams[i] && info.fMembersInGame[i] == kPlayerTypeHuman)
					{
						race = info.fMemberRaces[i];
						break;
					}
				}
			}

			// instead of using the team number as passed in use the players position in the menu 
			// as his team number. This will keep everyones ships on their own team.
			mSetTeamRace (MemberIndex, race);
			mGetTeamHandle (MemberIndex)->mSetMultiplayerTeam (team);

			// now set team relationships
			for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
			{
				if (info.fMembersInGame[i] != kPlayerTypeNoOne)
				{
					if (team != info.fMemberTeams[i])
					{
						mSetTeamRelations (MemberIndex, i, kTeamAtWar, kGoalDestroy, 1.0);
					}
					else if (i != MemberIndex)
					{
						mSetTeamRelations (MemberIndex, i, kTeamAllied, kGoalIgnore, 1.0);
					}
				}
			}

			// keep track of the human teams
			if (info.fMembersInGame[MemberIndex] == kPlayerTypeHuman)
			{
				// if this is a local human then set players team
				if (info.fMemberIsLocal[MemberIndex])
				{
					gScript.kPlayerTeam = eTeamID(MemberIndex);
				}
				else	// keep track of remote humans (friend or foe)
				{
					eMulti_TeamTypes FoF;
					int32 index;
					if (team == info.fPlayerTeam)
					{
						FoF = kHuman_AlliedTeams;
						index = fTotalAlliedHumanTeams++;
					}
					else
					{
						FoF = kHuman_EnemyTeams;
						index = fTotalEnemyHumanTeams++;
					}
					fMulti_Races[FoF][index]			= race;
					fMulti_TeamNumbers[FoF][index]		= MemberIndex;
				}
			}

			// keep track of the AI teams
			if (info.fMembersInGame[MemberIndex] == kPlayerTypeAI)
			{
				eMulti_TeamTypes FoF;
				int32 index;
				if (team == info.fPlayerTeam)
				{
					FoF = kCPU_AlliedTeams;
					index = fTotalAlliedCPUTeams++;
				}
				else
				{
					FoF = kCPU_EnemyTeams;
					index = fTotalEnemyCPUTeams++;
				}
				fMulti_Races[FoF][index]			= race;
				fMulti_TeamNumbers[FoF][index]		= MemberIndex;
				fMulti_ShipCounts[FoF][index]		= info.fMemberShipCounts[MemberIndex];
				fMulti_BPVLimits[FoF][index]		= info.fMemberBPVLimits[MemberIndex];
				fMulti_BPVTeamLimits[FoF][index]	= info.fMemberBPVTeamLimits[MemberIndex];
				fMulti_SmallHulls[FoF][index]		= info.fMemberSmallHulls[MemberIndex];
				fMulti_LargeHulls[FoF][index]		= info.fMemberLargeHulls[MemberIndex];
				fMulti_StartPositions[FoF][index]	= info.fMemberStartPositions[MemberIndex];

				for (int32 k = 0; k < fMulti_ShipCounts[FoF][index]; k++)
				{
					fMulti_FleetNames[FoF][index][k] = info.fMemberFleetNames[MemberIndex][k];
					fMulti_FleetCallNames[FoF][index][k] = info.fMemberFleetCallNames[MemberIndex][k];
				}

				mGetTeamHandle (MemberIndex)->mSetAsCPUTeam (FoF, index);
			}
		}
	}

	// update struct in case players team changed during setup
	info.fPlayerTeam = gScript.kPlayerTeam;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetSkirmishTeams (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	mSetTeamRace (info.fPlayerTeam, info.fPlayerRace);

	for (int32 i = 1; i <= kMaxCPUTeams; i++)
	{
		if (info.fMembersInGame[i] == kPlayerTypeAI)
		{
			eTeamID team	= info.fMemberTeams[i];

			// determine if this race is friend or foe
			eMulti_TeamTypes FoF;
			int32 index;
			if (team == info.fPlayerTeam)
			{
				FoF = kCPU_AlliedTeams;
				index = fTotalAlliedCPUTeams++;
			}
			else
			{
				FoF = kCPU_EnemyTeams;
				index = fTotalEnemyCPUTeams++;
			}

			// instead of using the team number as passed in use the players position in the menu 
			// as his team number. This will keep everyones ships on their own team.
			mSetTeamRace (i, info.fMemberRaces[i]);
			mGetTeamHandle (i)->mSetMultiplayerTeam (team);
			mGetTeamHandle (i)->mSetAsCPUTeam (FoF, index);

			fMulti_Races[FoF][index]			= info.fMemberRaces[i];
			fMulti_TeamNumbers[FoF][index]		= i;
			fMulti_ShipCounts[FoF][index]		= info.fMemberShipCounts[i];
			fMulti_BPVLimits[FoF][index]		= info.fMemberBPVLimits[i];
			fMulti_BPVTeamLimits[FoF][index]	= info.fMemberBPVTeamLimits[i];
			fMulti_SmallHulls[FoF][index]		= info.fMemberSmallHulls[i];
			fMulti_LargeHulls[FoF][index]		= info.fMemberLargeHulls[i];
			fMulti_StartPositions[FoF][index]	= info.fMemberStartPositions[i];

			for (int32 k = 0; k < fMulti_ShipCounts[FoF][index]; k++)
			{
				fMulti_FleetNames[FoF][index][k] = info.fMemberFleetNames[i][k];
				fMulti_FleetCallNames[FoF][index][k] = info.fMemberFleetCallNames[i][k];
			}
		}
	}

	// setup team relations and target goals
	for (i = 0; i < fTotalAlliedCPUTeams; i++)
	{
		int32 Iteam = fMulti_TeamNumbers[kCPU_AlliedTeams][i];

		// set as ally to player team
		mSetTeamRelations (info.fPlayerTeam, Iteam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

		// set as ally to other allies
		for (int32 j = 0; j < fTotalAlliedCPUTeams; j++)
		{
			if (j != i)
			{
				mSetTeamRelations (Iteam, fMulti_TeamNumbers[kCPU_AlliedTeams][j], 
								   kTeamAllied, kGoalIgnore, 1.0, kTrue);
			}
		}

		// set at war to enemy teams
		for (j = 0; j < fTotalEnemyCPUTeams; j++)
		{
			mSetTeamRelations (Iteam, fMulti_TeamNumbers[kCPU_EnemyTeams][j], 
							   kTeamAtWar, kGoalDestroy, 1.0, kTrue);
		}
	}
	for (i = 0; i < fTotalEnemyCPUTeams; i++)
	{
		// set at war to player team
		int32 Iteam = fMulti_TeamNumbers[kCPU_EnemyTeams][i];
		mSetTeamRelations (info.fPlayerTeam, Iteam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

		// set at war with other enemy teams (unless same team)
		for (int32 j = 0; j < fTotalEnemyCPUTeams; j++)
		{
			int32 Jteam = fMulti_TeamNumbers[kCPU_EnemyTeams][j];
			if (info.fMemberTeams[Iteam] != info.fMemberTeams[Jteam])
			{
				mSetTeamRelations (Iteam, Jteam, kTeamAtWar, kGoalDestroy, 1.0);
			}
			else if (i != j)
			{
				mSetTeamRelations (Iteam, Jteam, kTeamAllied, kGoalIgnore, 1.0);
			}
		}
	}
}

//----------------------------------------------------------------------
//
tTeamInfo* tMissionInfo::mGetTeamHandle (int32 Team)
{
	if (fTeams[Team] == NULL)
	{
		fTeams[Team] = new tTeamInfo (eTeamID(Team), fRaces[Team]);
	}
	return fTeams[Team];
}

//----------------------------------------------------------------------
//
tTeamInfo* tMissionInfo::mGetTeamHandle (eMulti_TeamTypes Type, int32 Index)
{
	return (mGetTeamHandle (fMulti_TeamNumbers[Type][Index]));
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetPrestigePoints (int32 AstoundingVictory, int32 Victory, int32 Draw, 
									   int32 Defeat, int32 DevastatingDefeat)
{
	fPrestigePoints[kAstoundingVictory] = AstoundingVictory;
	fPrestigePoints[kVictory] = Victory;
	fPrestigePoints[kDraw] = Draw;
	fPrestigePoints[kDefeat] = Defeat;
	fPrestigePoints[kDevastatingDefeat] = DevastatingDefeat;
}

//----------------------------------------------------------------------
// end the game if all ships of the player or enemy team
// have been destroyed, captured, or disengaged
eGameStatus tMissionInfo::mGetGameStatus (void)
{
	// if the ship that the player is controlling has disengaged then end the game
	eTeamID Team = gScript.kPlayerTeam;
	tTeamInfo* fTeam = mGetTeamHandle (Team);
	if (gScript.fPlayerShip->mTestShipStatus (kShipDisengaged) == kFalse)
	{
		// test the player and enemy team to see if they are both still alive
		if (fTeam->mGetShipsInPlayCount() > 0)
		{
			Team = eTeamID(gScript.mGetEnemyTeamID());
			fTeam = mGetTeamHandle (Team);
			if (fTeam->mGetShipsInPlayCount() > 0)
			{
				return kGameInProgress;
			}
		}
	}
	iNotifyEvents Event = fTeam->mGetTeamStatus();

	// decide which end game status to return
	// - Team has the team that is dead
	// - Event has the reason why
	eGameStatus status;
	if (Team == gScript.kPlayerTeam)
	{
		switch (Event)
		{
			case kNotifyAtDeath:
				status = kPlayerDestroyed;
				break;

			case kNotifyOnCapture:
				status = kPlayerCaptured;
				break;

			case kNotifyDisengaged:
				status = kPlayerDisengaged;
				break;

			default:
				status = kPlayerLoses;
				break;
		}
	}
	else // must be enemy team
	{
		switch (Event)
		{
			case kNotifyAtDeath:
				status = kEnemyDestroyed;
				break;

			case kNotifyOnCapture:
				status = kEnemyCaptured;
				break;

			case kNotifyDisengaged:
				status = kEnemyDisengaged;
				break;

			default:
				status = kPlayerWins;
				break;
		}
	}
	return status;
}

//----------------------------------------------------------------------
// return kTrue if all ships on all teams have been initialized
iTruth tMissionInfo::mAllTeamsReady (void)
{
	int32 index = kMaxTeams;
	while (index--)
	{
		if (fTeams[index] != NULL)
		{
			if (fTeams[index]->mAllShipsReady() == kFalse)
			{
				return kFalse;
			}
		}
	}
	return kTrue;
}

//----------------------------------------------------------------------
// returns the players victory level based on BPV
eVictoryLevels tMissionInfo::mGetVictoryLevel (int32 PlayerBPV, int32 EnemyBPV)
{
	if (PlayerBPV < 1)
	{
		PlayerBPV = 1;
	}
	if (EnemyBPV < 1)
	{
		EnemyBPV = 1;
	}

	int32 level = (PlayerBPV * 100) / EnemyBPV;
	if (level >= 200)
	{
		return kAstoundingVictory;		// 200%+
	}
	else if (level >= 110)
	{
		return kVictory;				// 110%-199%
	}
	else if (level >= 91)
	{
		return kDraw;					// 91%-109%
	}
	else if (level >= 30)
	{
		return kDefeat;					// 31%-90%
	}

	return kDevastatingDefeat;			// 30%
}

int32 tMissionInfo::mRandomInt32 (int32 lo, int32 hi)
{
	return (gScript.mRandomInt (lo, hi));
}

//----------------------------------------------------------------------
//
void tMissionInfo::mDisplayMessage (int32 MsgID, eScriptMessageTypes Type, tShipInfo* Ship)
{
	int32 ShipID = 0;
	if (Ship)
	{
		ShipID = Ship->mGetShipID();
	}
	gScript.mPopupMessage (Messages[MsgID], Type, ShipID);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mDebugMessage (const char* message)
{
	tString msg = message;
	gScript.mPopupMessage (msg, kNonCommScriptMessage, 0);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mAddToDebriefingList (int32 DebriefingIndex)
{
	if (fDebriefingTotal < kMaxInList)
	{
		fDebriefingArray[fDebriefingTotal] = DebriefingIndex;
		fDebriefingTotal++;
	}
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mResetDebriefingCount (void)
{
	fDebriefingTotal = 0;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetDebriefingTotal (void)
{
	return fDebriefingTotal;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetDebriefingMessage (int32 Index)
{
	if (Index < kMaxInList)
	{
		return fDebriefingArray[Index];
	}
	return 0;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetPrestigePoints (eVictoryLevels Level)
{
	if (Level == kBonusPrestige)
	{
		return fBonusPrestige;
	}
	if (Level < kMaxVictoryLevels)
	{
		return fPrestigePoints[Level];
	}
	return 0;
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mAddToBonusPrestige (int32 Amount)
{
	fBonusPrestige += Amount;
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetBonusPrestige (void)
{
	return fBonusPrestige;
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mGetGridPosition (unsigned char MapObject, int32* GridX, int32* GridY)
{
	tShipSystemsInfo info;

	info.fAction = kGetMapEntryPosition;
	info.fInt32[0] = MapObject;
	gScript.mEntityMgr (info);

	*GridX = info.fGridX + 1;
	*GridY = info.fGridY + 1;
}

//------------------------------------------------------------------------------------------------
// Distance is based on tactical map distance
float tMissionInfo::mGetTacticalDistance (unsigned char MapObject1, unsigned char MapObject2)
{
	tShipSystemsInfo info;
	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fInt32[1] = MapObject1;
	info.fInt32[2] = MapObject2;
	info.fShipEntityID = 0;				// == 0 indicates that distance is between two map objects
	gScript.mEntityMgr (info);

	return (info.fFloat[0]);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mCreateMapObject (unsigned char MapObject, int32 GridX, int32 GridY)
{
	tShipSystemsInfo info;

	info.fAction = kCreateMapEntry;
	info.fInt32[0] = MapObject;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;

	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetPlayerRank (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScript.mEntityMgr (info);

	return info.fInt32[0];
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetEliteOrganization (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScript.mEntityMgr (info);

	return info.fInt32[1];
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mGetDifficultyLevel (void)
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScript.mEntityMgr (info);

	return info.fInt32[2];
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mPlaySound (int32 SoundID)
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = SoundID;
	info.fInt32[1] = 0;		// indicate its a non-voice sound
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
void tMissionInfo::mPlayVoiceSample (int32 VoiceID, iTruth StopPreviousVoice)
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = VoiceID;
	info.fInt32[1] = 1;			// indicate its a voice sound
	info.fInt32[2] = StopPreviousVoice;
	gScript.mEntityMgr (info);
}

//------------------------------------------------------------------------------------------------
//
int32 tMissionInfo::mVoiceSampleStillPlaying (void)
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = 0;
	info.fInt32[1] = 2;			// indicate return status request
	gScript.mEntityMgr (info);
	return info.fInt32[0];
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTimedInterval (eTimerIntervalTypes Type, int32 Interval)
{
	tShipSystemsInfo info;

	info.fAction = kSetTimedInterval;
	info.fInt32[0] = Interval;
	info.fInt32[1] = Type;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mAddToCareerTotals (eClassTypes Type, eCareerEvents Event, int32 Amount)
{
	tShipSystemsInfo info;

	info.fAction = kAddToCareerTotal;
	info.fInt32[0] = Amount;
	info.fInt32[1] = Event;
	info.fInt32[2] = Type;

	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mRemoveShipAsTarget (tShipInfo* TargetShip)
{
	if (TargetShip)
	{
		eTeamID target_team = TargetShip->mGetTeam();

		// remove this ship as a target from all teams
		for (int32 i = 0; i < kMaxTeams; i++)
		{
			tTeamInfo* Team = fTeams[i];
			if (Team && i != target_team)
			{
				Team->mRemoveShipAsTarget (TargetShip);
			}
		}
	}
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTeamRelations (int32 Team1, int32 Team2, eTeamRelationship Relations,
									   eTargetGoals Goals, float EPVFactor, iTruth MutualRelations)
{
	if (Team1 > kNoTeam && Team1 < kMaxTeams
		&& Team2 > kNoTeam && Team2 < kMaxTeams)
	{
		// set Team1 relationship with Team2
		tTeamInfo* team = mGetTeamHandle (Team1);
		team->mSetRelations (Team2, Relations, Goals, EPVFactor);

		if (MutualRelations)
		{
			// set Team2 relationship with Team1
			team = mGetTeamHandle (Team2);
			team->mSetRelations (Team1, Relations, Goals, EPVFactor);
		}
	}
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetTeamRelations (eMulti_TeamTypes Team1, int32 Team2, eTeamRelationship Relations,
									  eTargetGoals Goals, float EPVFactor, iTruth MutualRelations)
{
	int32 team_count = 0;
	if (Team1 == kHuman_AlliedTeams) team_count = fTotalAlliedHumanTeams;
	if (Team1 == kHuman_EnemyTeams) team_count = fTotalEnemyHumanTeams;
	if (Team1 == kCPU_AlliedTeams) team_count = fTotalAlliedCPUTeams;
	if (Team1 == kCPU_EnemyTeams) team_count = fTotalEnemyCPUTeams;

	while (team_count--)
	{
		mSetTeamRelations (fMulti_TeamNumbers[Team1][team_count], Team2, Relations, Goals, EPVFactor, MutualRelations);
	}
}

//------------------------------------------------------------------------------------------------
//
eTeamRelationship tMissionInfo::mGetTeamRelations (eTeamID Team1, eTeamID Team2)
{
	if (Team1 > kNoTeam && Team1 < kMaxTeams)
	{
		tTeamInfo* team = fTeams[Team1];
		if (team != NULL)
		{
			return team->mGetRelations (Team2);
		}
	}
	return kUnknownRelationship;
}

//----------------------------------------------------------------------
// return pointer to ship that matches given ID
tShipInfo* tMissionInfo::mFindShipID (int32 ShipID)
{
	for (int32 i = 0; i < kMaxTeams; i++)
	{
		tTeamInfo* Team = fTeams[i];
		if (Team)
		{
			for (tShipInfo* ship = Team->mGetFirstShip (); ship != NULL; ship = Team->mGetNextShip())
			{
				if (ship->mGetShipID() == ShipID)
				{
					return ship;
				}
			}
		}
	}
	return NULL;
}

//----------------------------------------------------------------------
//
ePoliticalRelation tMissionInfo::mGetRaceRelationship (eRaceName Race1, eRaceName Race2)
{
	tShipSystemsInfo info;

	info.fAction = kGetRaceRelationship;
	info.fInt32[0] = Race1;
	info.fInt32[1] = Race2;
	gScript.mEntityMgr (info);

	return (ePoliticalRelation(info.fInt32[2]));
}

//----------------------------------------------------------------------
//
tShipInfo* tMissionInfo::mGetShipTarget (tShipInfo* Ship)
{
	if (Ship)
	{
		return (mFindShipID (Ship->mGetShipTargetID()));
	}
	return NULL;
}

//----------------------------------------------------------------------
//
void tMissionInfo::mStartFlashingButton (const char* ButtonID)
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fTextPtr = ButtonID;
	info.fInt32[0] = kTrue;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mStopFlashingButton (const char* ButtonID)
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fTextPtr = ButtonID;
	info.fInt32[0] = kFalse;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
int32 tMissionInfo::mGetINIFileInt (int32 HeaderID, int32 LabelID)
{
	tShipSystemsInfo info;
	info.fAction = kGetINIFileEntry;
	info.fText = Messages[HeaderID];
	info.fText += "^";
	info.fText += Messages[LabelID];
	gScript.mEntityMgr (info);

	return info.fInt32[0];
}

//----------------------------------------------------------------------
// Grid X/Y are 1 based
void tMissionInfo::mSetGoalIndicator (iTruth IndicatorOn, int32 GridX, int32 GridY)
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fInt32[0] = IndicatorOn;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;
	info.fShipEntityID = -1;
	info.fTextPtr = NULL;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mSetGoalIndicator (tShipInfo* TargetShip)
{
	if (TargetShip)
	{
		tShipSystemsInfo info;
		info.fAction = kStartFlashingButton;
		info.fInt32[0] = kTrue;
		info.fShipEntityID = TargetShip->mGetShipID();
		info.fTextPtr = NULL;
		gScript.mEntityMgr (info);
	}
}

//----------------------------------------------------------------------
//
iTruth tMissionInfo::mIsTeamLeaving (eTeamID Team)
{
	tTeamInfo* team = mGetTeamHandle (Team);
	return team->mIsTeamLeaving();
}

//----------------------------------------------------------------------
//
void tMissionInfo::mAdjustDynaverse (eDynaverseTableType Type, int32 Modifier, eRaceName Race)
{
	tShipSystemsInfo info;
	info.fAction = kDynaverseAdjustments;
	info.fInt32[0] = Type;
	info.fInt32[1] = Modifier;
	info.fInt32[2] = Race;
	gScript.mEntityMgr (info);
}

//----------------------------------------------------------------------
//
void tMissionInfo::mGetMapDimensions (int32* GridX, int32* GridY)
{
	gScript.mGetMapDimensions (GridX, GridY);
}


// ************************************************************************************************
//					Script interface routines
// ************************************************************************************************

//------------------------------------------------------------------------------------------------
//
tScriptInterface::tScriptInterface (void)
{
	fMissionBegun = kFalse;
	fIsDeepScanOn = kFalse;
}

//------------------------------------------------------------------------------------------------
//
tScriptInterface::~tScriptInterface (void)
{
}

//------------------------------------------------------------------------------------------------
//
int32 tScriptInterface::mDLLVersion (void)
{
	return ScriptVersion;
}

//------------------------------------------------------------------------------------------------
// save callback into game engine
void tScriptInterface::mCreateShipCallback (cbCreateShip* func)
{
	mCreateSpecShip = func;
}

//------------------------------------------------------------------------------------------------
// save callback into game engine
void tScriptInterface::mEntityMgrCallback (cbEntityMgr* func)
{
	mEntityMgr = func;
}

//------------------------------------------------------------------------------------------------
// save callback into game engine
void tScriptInterface::mMessageCallback (cbMessages* func)
{
	mPopupMessage = func;
}
	
//------------------------------------------------------------------------------------------------
// save callback into game engine
void tScriptInterface::mRandomCallback (cbRandom* func)
{
	mRandomInt = func;
}
	
//------------------------------------------------------------------------------------------------
// begin mission
void tScriptInterface::mInitialize (tDynaverseScriptInfo& info)
{
	tMissionSpec Specs;
	mScriptSpecs (&Specs);

	// save player team
	kPlayerTeam = info.fPlayerTeam;

	// save mission date
	fMission.mSetDate (info.fMissionDate);

	// save mission location
	fMission.mSetLocation (info.fMissionLocation);

	// pick allied race
	info.fAlliedRace = mChooseRace (info.fAlliedRaces, kAllRaceMission);

	// pick enemy race
	info.fEnemyRace = mChooseRace (info.fEnemyRaces, Specs.fEnemyRace);

	// pick system
	info.fSystemName = mChooseSystem (info.fSystemNames);

	// set title of mission
	info.fMissionTitle = Specs.fMissionTitle;
	info.fEnableMenuButtons = Specs.fEnableSupplyDock;
	fThisScriptIsTheHost = info.fHosting;

	// let the script change things if it needs to
	info.fMapPicked = mInitializeStart (info) - 1;

	if (info.fPreferedMap > -1)
	{
		info.fMapPicked = info.fPreferedMap;
		if (info.fMapPicked >= TotalMaps)
		{
			info.fMapPicked = 0;
		}
	}
	info.fMapName = MapTypes[info.fMapPicked];
	fMapPicked = info.fMapPicked;
	mInitMapForRead();

	fMission.mSetSystemName (info.fSystemName);

	fVictoryLevel = kMaxVictoryLevels;

	// now create the ships used in this mission
	mSetupShips (info);

	// let the script finish things up
	fBriefingIndex = mInitializeFinished ();
}

//------------------------------------------------------------------------------------------------
// 50% chance of selecting a random race
eRaceName tScriptInterface::mChooseRace (eRaceName RaceArray[], int32 ValidRaces)
{
	// make certain one of the valid races exists in the array - if not then
	// just return the first one
	iTruth valid = kFalse;
	int32 index = 0;
	while (RaceArray[index] != kNoRace)
	{
		if ((1 << RaceArray[index]) & ValidRaces)
		{
			valid = kTrue;
			break;
		}
		index++;
	}

	index = 0;		// default to first race
	while (valid)
	{
		if (mRandomInt (1, 100) < 10)
		{
			// count number of races in array
			while (RaceArray[index] != kNoRace)
			{
				index++;
			}
			// pick a random race
			index = mRandomInt (0, index - 1);
		}
		if ((1 << RaceArray[index]) & ValidRaces)
		{
			break;			// found one - we're done
		}
		index = 0;		// default to first race again
	}
	return RaceArray[index];
}

//------------------------------------------------------------------------------------------------
// 75% chance of selecting a random system
eSystems tScriptInterface::mChooseSystem (eSystems SystemsArray[])
{
	int32 index = 0;	// default to first system
	if (mRandomInt (1, 100) < 75)
	{
		// count number of systems in array
		while (SystemsArray[index] != kMaxSystems)
		{
			index++;
		}
		// pick a random system
		index = mRandomInt (0, index - 1);
	}
	return SystemsArray[index];
}

//------------------------------------------------------------------------------------------------
// create ships for the mission
void tScriptInterface::mSetupShips (tDynaverseScriptInfo& info)
{
	// create the players ships first
	int32 Position = kStartPosition_G;
	if (info.fMissionType == kMultiplayerMission)
	{
		for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
		{
			if (info.fMembersInGame[i] == kPlayerTypeHuman && info.fMemberIsLocal[i])
			{
				Position += i;
				break;
			}
		}
	}
	for (int32 index = 0; index < info.fPlayerFleetSize; index++)
	{
		tShipInGame& ship_info = info.fPlayerFleet[index];
		ship_info.fShipPosition = Position;
		ship_info.fEndX = -1;
		ship_info.fEndY = -1;
		ship_info.fTeam = info.fPlayerTeam;
		if (mUsePlayerShip (ship_info))
		{
			if ((ship_info.fShipPosition & kGroupAround)
				|| ship_info.fShipPosition == kStartPosition_Specified)
			{
				int32 grouping = (ship_info.fShipPosition & kGroupAround);
				ship_info.fShipPosition &= ~kGroupAround;
				if (ship_info.fShipPosition != kStartPosition_Specified)
				{
					unsigned char MapObject = 'G' + ship_info.fShipPosition - kStartPosition_G;
					mFindMapObject (MapObject, &ship_info.fStartX, &ship_info.fStartY);
					MapObject += 'g' - 'G';
					mFindMapObject (MapObject, &ship_info.fEndX, &ship_info.fEndY);
					ship_info.fShipPosition = kStartPosition_Specified;
				}
				// to avoid ships starting on top of each other we offset from the
				// center point up or down based on their position in the menu
				// (evens up, odds down)
				if (info.fMissionType == kMultiplayerMission && grouping)
				{
					int32 side = Position - kStartPosition_G;
					if (side == 2) side = -1;
					else if (side == 3) side = 2;
					else if (side == 4) side = -2;
					else if (side == 5) side = 3;
					ship_info.fStartY += side;
				}
				if (index == 1) ship_info.fStartX--;		// move second ship to the left
				if (index == 2) ship_info.fStartX++;		// move third ship to the right
			}
			else
			{
				Position++;
			}
			fMission.mGetTeamHandle (info.fPlayerTeam)->mCreateShip (index, ship_info);
		}
	}

	// create the rest of the ships
	mCreateMissionShips (0);
}

//------------------------------------------------------------------------------------------------
// ship just created
iTruth tScriptInterface::mSaveEntityID (int32 Team, int32 CallbackID, int32 EntityID)
{
	tTeamInfo* fTeam = fMission.mGetTeamHandle (eTeamID(Team));

	tShipInfo* Ship = fTeam->mFindShipIndex (CallbackID);
	if (Ship == NULL)
	{
		// a ship has been created that did not come from this script (most likely 
		// it came from a remote script). we need to create a shadow version of it
		// in this script.

		tCreateShipInfo info;
		info.fShipData.fTeam				= eTeamID(Team);
		info.fShipData.fScriptCallbackID	= CallbackID;
		info.fShipData.fShipPosition		= kStartPosition_Specified;
		info.fShipData.fPlayerShipIndex		= EntityID;
		info.fShipData.fCreateShadowShip	= kTrue;
		info.fShipData.fCustomName			= NULL;
		if (fTeam->mAddShipToTeamList (info))
		{
			Ship = fTeam->mFindShipIndex (CallbackID);
		}
	}

	if (Ship)
	{
		Ship->mSetShipID (EntityID);
		Ship->mSetShipStatus (kShipCreated);
		Ship->mReplaceShipCallbackEvents (kNotifyDefaultEvents);

		// if this is a CPU ship and we're the host then we need to test to set its callname
		if (fThisScriptIsTheHost && fTeam->mIsCPUTeam())
		{
			tShipSystemsInfo info;
			info.fTextPtr = fTeam->mGetCPUCallName (*Ship);
			if (info.fTextPtr)
			{
				info.fAction = kSetObjectName;
				info.fShipEntityID = Ship->mGetShipID();
				mEntityMgr (info);
			}
		}

		// configure the ship now if mission already started
		if (fMissionBegun)
		{
			mNewMissionShip (Ship);
		}
		// start the mission once all the ships have been created
		else if (fMission.mAllTeamsReady ())
		{
			mStartMission();
			fMissionBegun = kTrue;
		}
	}

	return fMissionBegun;
}

//------------------------------------------------------------------------------------------------
// an in-game event has occured
eGameStatus tScriptInterface::mEntityEvent (tShipSystemsInfo& info)
{
	tShipInfo* Ship = NULL;
	eTeamID team = eTeamID (info.fTeam);
	if (team < kMaxTeams)
	{
		tTeamInfo* fTeam = fMission.mGetTeamHandle (team);
		Ship = fTeam->mFindShipID (info.fShipEntityID);
		if (Ship == NULL)
		{
			Ship = fMission.mFindShipID (info.fShipEntityID);
			if (Ship == NULL)
			{
				return kGameInProgress;
			}
		}
	}

	switch (info.fAction)
	{
		case kNotifyOnCapture:
			Ship->mSetShipStatus (kShipCaptured, (info.fInt32[0] != Ship->mGetOriginalTeam()));
			break;

		case kNotifyAtDeath:
			Ship->mSetShipStatus (kShipDestroyed);
			Ship->mClearCommButton (1);
			Ship->mClearCommButton (2);
			break;

		case kNotifyDisengaged:
			Ship->mSetShipStatus (kShipDisengaged);
			break;

		case kNotifyAtDestination:
			Ship->mSetShipStatus (kShipAtFinalDestination);
			break;

		case kNotifyChangingShip:
			fPlayerShip = fMission.mFindShipID (info.fInt32[0]);
			break;

		case kNotifyDeepScan:
			fIsDeepScanOn = info.fInt32[0];	// set the state of the deep scan button (kTrue == on)
			break;

		case kNotifyPlayerLeaving:
			tTeamInfo* fTeam = fMission.mGetTeamHandle (info.fShipEntityID);
			fTeam->mSetTeamLeaving();
			break;
	}

	// remove this action from this ships callback events
	if (Ship != NULL)
	{
		iNotifyEvents old_events = Ship->mGetShipCallbackEvents();
		iNotifyEvents new_events = old_events & ~info.fAction;
		if (new_events != old_events)
		{
			Ship->mReplaceShipCallbackEvents (new_events);
		}
	}

	// now let the script handle this event
	fEventID = info.fShipEntityID;
	fEventItem0 = info.fInt32[0];
	fEventItem1 = info.fInt32[1];
	fEventItem2 = info.fInt32[2];
	fEventItem3 = info.fInt32[3];
	if (info.fAction == kNotifyCommButtonPress
		|| info.fAction == kNotifyAlertStatusChange
		|| info.fAction == kNotifyBeginOrbit
		|| info.fAction == kNotifyEndOrbit
		|| info.fAction == kNotifyBeginTractor
		|| info.fAction == kNotifyEndTractor
		|| info.fAction == kNotifyOnCapture
		|| info.fAction == kNotifyDamage
		|| info.fAction == kNotifyProbeLaunch
		|| info.fAction == kNotifyTransportedItemTo
		|| info.fAction == kNotifyTransportedItemFrom
		|| info.fAction == kNotifyChangingShip
		|| info.fAction == kNotifyAtDistance)
	{
		fEventID = info.fInt32[0];			// pass in the extra value instead
	}

	// the only events we handle AFTER the game has ended are death events
	eGameStatus status;
	if (fMission.fEndGameLockout == kFalse
		|| info.fAction == kNotifyAtDeath)
	{
		status = mHandleShipEvent (Ship, info.fAction);
	}
	else	// game has ended before this event - throw it away
	{
		status = kGameInProgress;
	}

	// if a ship has been captured switch it to its new team
	if (info.fAction == kNotifyOnCapture)
	{
		Ship->mSetTeam (eTeamID(info.fInt32[0]));
		Ship->mSetShipCallbackEvents (kNotifyOnCapture);
	}

	// handle the player leaving if the script did not
	if (status == kGameInProgress
		&& fMission.mIsTeamLeaving (kPlayerTeam))
	{
		status = kPlayerLoses;
	}

	if (status != kGameInProgress)
	{
		// the players status can change after the game has ended, but ONLY to a losing status.
		if (fMission.fEndGameLockout)
		{
			// reset lockout if new status is a losing status
			if (((status & (kEnemyCaptured | kEnemyDestroyed | kEnemyDisengaged | kPlayerWins)) == 0)
				&& (fMission.fLockOutStatus & (kEnemyCaptured | kEnemyDestroyed | kEnemyDisengaged | kPlayerWins)))
			{
				fMission.fEndGameLockout = kFalse;
			}
			else
			{
				status = kGameInProgress;
			}
		}

		if (fMission.fEndGameLockout == kFalse)
		{
			fMission.mResetDebriefingCount();
			// get victory level if player is still alive
			if ((status & (kMissionFailed | kPlayerDestroyed | kPlayerCaptured)) == 0)
			{
				if (fMission.mIsTeamLeaving (kPlayerTeam))
				{
					fVictoryLevel = kDevastatingDefeat;
				}
				else
				{
					fVictoryLevel = mCalcVictoryStatus();
					// change a disengage status to player wins if victory level is victory or better
					int32 i = status & ~(kExpellFromElite | kCampaignCompleted);
					if (i == kPlayerDisengaged && fVictoryLevel <= kVictory)
					{
						status = eGameStatus((status & kCampaignCompleted) | kPlayerWins);
					}
				}
				mGetDebriefingList(fVictoryLevel);
			}	
			fMission.fLockOutStatus = status;
			fMission.fEndGameLockout = kTrue;
		}
		fMission.mSetGoalIndicator (kFalse, 0, 0);
	}
	return status;
}

//------------------------------------------------------------------------------------------------
//
void tScriptInterface::mInitMapForRead (void)
{
	fMapLineIndex = 0;
}

//------------------------------------------------------------------------------------------------
//
iTruth tScriptInterface::mInitMapForRead (int32 Map)
{
	mInitMapForRead();
	const char* MapName = MapTypes[Map];

	for (;;)
	{
		const char* mapLine = mGetNextMapLine();
		if (mapLine == NULL)
		{
			break;		// No matching map found
		}
		tString CurLine = mapLine;
		if (CurLine.substr(0,2) == "//")
		{
			if (CurLine.find (MapName) >= 0)
			{
				return kTrue;
			}
		}
	}
	return kFalse;
}

//------------------------------------------------------------------------------------------------
//
const char* tScriptInterface::mGetNextMapLine (void)
{
	if (MapLines[fMapLineIndex] == 0)
	{
		return NULL;
	}
	return MapLines[fMapLineIndex++];
}

//------------------------------------------------------------------------------------------------
//
tString& tScriptInterface::mBriefingText (void)
{
	return (Messages[fBriefingIndex]);
}

//------------------------------------------------------------------------------------------------
//
int32 tScriptInterface::mGetDebriefingTotal (void)
{
	return (fMission.mGetDebriefingTotal());
}

//------------------------------------------------------------------------------------------------
//
tString& tScriptInterface::mDebriefingText (int32 Index)
{
	int32 fDebriefingIndex = fMission.mGetDebriefingMessage (Index);
	return (Messages[fDebriefingIndex]);
}

//------------------------------------------------------------------------------------------------
//
int32 tScriptInterface::mGetPrestigePoints (eVictoryLevels Level)
{
	if (Level == kMaxVictoryLevels)
	{
		// return victory level of just completed campaign
		return fVictoryLevel;
	}
	return fMission.mGetPrestigePoints(Level);
}

//------------------------------------------------------------------------------------------------
//
iTruth tScriptInterface::mGetScriptSpecs (tMissionSpec* Specs)
{
	if (Specs->fVersion == tMissionSpecVersion)
	{
		mScriptSpecs (Specs);
		return kTrue;
	}
	return kFalse;
}

//------------------------------------------------------------------------------------------------
//
int32 tScriptInterface::mGetTechnicalSuccess (tDynaverseScriptInfo& info)
{
	return (mGetTechnicalSuccessLevel(info));
}

//------------------------------------------------------------------------------------------------
//
eTeamRelationship tScriptInterface::mGetTeamRelations (eTeamID Team1, eTeamID Team2)
{
	return (fMission.mGetTeamRelations (Team1, Team2));
}

// GridX & GridY are 1 based
void tScriptInterface::mFindMapObject (unsigned char MapObject, int32* GridX, int32* GridY)
{
	// find the start of the map
	if (mInitMapForRead (fMapPicked))
	{
		*GridY = 1;
		for (;;)
		{
			const char* mapLine = mGetNextMapLine();

			if (mapLine == NULL) break;			// Map too short
			if ( *mapLine == '/' ) break;		// Map too short
			while ( *mapLine == ' ' ) mapLine++;

			if ( *mapLine != '+' )
			{
				while ( *mapLine != '|' )
				{
					if ( *mapLine == '\0' ) break;		// Bad map line - missing '|'
					mapLine++;
				}
				if ( *mapLine != '|' ) break;		// Bad map line - missing '|'
				*GridX = 1;
				for (;;)
				{
					mapLine++;
					if (*mapLine == MapObject) return;	// object found
					if (*mapLine == '|') break;			// end of line
					if (*mapLine == '\0') break;		// Bad map line - missing '|'
					*GridX += 1;
				}
				*GridY += 1;
			}
		}
	}

	// didn't find object
	*GridX = -1;
	*GridY = -1;
}

// GridX & GridY are 1 based
void tScriptInterface::mGetMapDimensions (int32* GridX, int32* GridY)
{
	// find the start of the map
	if (mInitMapForRead (fMapPicked))
	{
		*GridY = 0;
		for (;;)
		{
			const char* mapLine = mGetNextMapLine();

			if (mapLine == NULL) break;			// Map too short
			if ( *mapLine == '/' ) break;		// Map too short
			while ( *mapLine == ' ' ) mapLine++;

			if ( *mapLine != '+' )
			{
				while ( *mapLine != '|' )
				{
					if ( *mapLine == '\0' ) break;		// Bad map line - missing '|'
					mapLine++;
				}
				if ( *mapLine != '|' ) break;		// Bad map line - missing '|'
				*GridX = 0;
				for (;;)
				{
					mapLine++;
					if (*mapLine == '|') break;			// end of line
					if (*mapLine == '\0') break;		// Bad map line - missing '|'
					*GridX += 1;
				}
				*GridY += 1;
			}
		}
	}
	else	// didn't find map
	{
		*GridX = 0;
		*GridY = 0;
	}
}

// ************************************************************************************************
//					public DLL routines
// ************************************************************************************************

extern "C" __declspec( dllexport ) tScriptEntry* ScriptEntry(void)
{
	return &gScript;
}

