#ifndef scriptinterface_h
#define scriptinterface_h

#include "windows.h"
#include "math.h"
#include "tbasic.h"
#include "soundnums.h"
#include "scriptstructs.h"
#include "scriptentry.h"
#include "tmissionmap.h"

//-------------------------------------------------------
// max range constants
const int32 kUniqueMission = 32767;
const int32 kCaptureBonus = 30;		// prestige bonus for capturing ships
const int32 kTechnicalBonus = 40;	// 1-time prestige bonus for completing technical

//-------------------------------------------------------
// values to use with mSetShipStatus and mTestShipStatus
enum eShipStatus
{
	kUndefined					= 0x00,
	kShipCreated				= 0x01,
	kShipAtFinalDestination		= 0x02,
	kShipCaptured				= 0x04,
	kShipDisengaged				= 0x08,
	kShipDestroyed				= 0x10,
};

enum eMulti_TeamTypes
{
	kHuman_AlliedTeams,
	kHuman_EnemyTeams,
	kCPU_AlliedTeams,
	kCPU_EnemyTeams,
	kMaxMultiTeams
};

struct tAITargetInfo
{
	eTeamRelationship	fRelationships[kMaxTeams];
	eTargetGoals		fTargetGoals[kMaxTeams];
	float				fEPVFactors[kMaxTeams];
};

// must match enum eDisengageType in aiShipCommands.h
enum eDisengageMethods
{
	kByAcceleration,
	kByMapEdge,
};

enum eMedalsAwarded
{
	kMissionMedal_1 = 1,
	kMissionMedal_2,
	kMissionMedal_3,
	kMissionMedal_4,
	kSpecialMedal_1,
	kSpecialMedal_2,
	kSpecialMedal_3,
	kSpecialMedal_4,
};

//-------------------------------------------------------
// ship specific routines
class tShipInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	void		mSetShipCallbackEvents (iNotifyEvents NewEvents);
	void		mSetShipThrust (int32 NewThrust);
	void		mSetShipMaxSpeed (int32 MaxSpeed);
	void		mSetShipPlayerControlled (void);
	void		mSetShipAIControlled (iTruth AIControlled);
	void		mSetFinalDestination (int32 EndX = -1, int32 EndY = -1);
	void		mTrackTacticalDistance (float Distance, int32 GridX, int32 GridY, iTruth NotifyInOut = kFalse);	// Grid XY are 1 based
	void		mDamageSystem (eDamageSystems System, float Percent);
	void		mChangeName (int32 MsgID);
	int32		mAccessSupplies (eShipSupplyAccess Access, int32 SupplyItem, int32 NewValue = 0);


	eClassTypes		mGetClassType (void);
	const char*		mGetClassName (void);
	int32			mGetStartPositionIndex (void);
	int32			mGetStartX (void);
	int32			mGetStartY (void);
	int32			mGetEndX (void);
	int32			mGetEndY (void);
	float			mGetTrackingDistance (void);
	int32			mAssessDamage (eRaceName EnemyRace);
	void			mGetGridPosition (int32* GridX, int32* GridY);		// Grid XY are 1 based
	float			mGetTacticalDistance (tShipInfo* TargetShip);
	float			mGetTacticalDistance (unsigned char MapObject);
	float			mGetTacticalDistance (int32 GridX, int32 GridY);	// Grid XY are 1 based
	eAlertLevels	mGetAlertStatus (void);
	void			mSetAlertStatus (eAlertLevels Status);
	float			mGetDamagePercent (void);
	eOfficerRank	mGetOfficerRank (eOfficerTypes Officer);
	void			mSetOfficerRank (eOfficerTypes Officer, eOfficerRank Rank);
	void			mSetOfficerName (eOfficerTypes Officer, int32 MsgID);
	void			mSetTargetable (iTruth State);

	// AI specific commands
	void			mSetShipTarget (tShipInfo* TargetShip, eAIPriority TargetPriority);
	void			mClearCommands (void);
	void			mDisengage (eDisengageMethods Method, eAIPriority Priority);
	void			mPatrol (float DetectionRange, int32 PatrolPointCount, tMapPoint* PatrolPoints, eAIPriority Priority);
	void			mChangeAIOfficerState (eAIOfficers Officer, iTruth State);
	void			mCloak (iTruth State);
	void			mTractorEntity (tShipInfo* Entity, int32 EndX, int32 EndY, eAIPriority Priority); // End XY are 1 based
	void			mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority);
	void			mFollowEntity (tShipInfo* Entity, eAIPriority Priority);
	void			mFireProbe (tShipInfo* Target, eAIPriority Priority);

	void		mSetCommButton (int32 ButtonNumber, int32 MsgID, iTruth SetAsTarget = kFalse);	// ButtonNumber is 1 based
	void		mClearCommButton (int32 ButtonNumber);											// ButtonNumber is 1 based

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:
	void		mSetShipStatus (eShipStatus Status, iTruth SetState = kTrue);
	void		mSetShipID (int32 ShipID);
	void		mSetShipBPV (int32 ShipBPV);
	void		mSetClassType (eClassTypes ClassType);
	int32		mGetShipTargetID (void);
	void		mReplaceShipCallbackEvents (iNotifyEvents NewEvents);

	eTeamID			mGetTeam (void);
	eTeamID			mGetOriginalTeam (void);
	int32			mGetShipID (void);
	iTruth			mInPlay (void);
	iTruth			mTestShipStatus (int32u Status);
	int32			mGetShipBPV (void);
	iNotifyEvents	mGetShipCallbackEvents (void);
	int32			mGetCallbackID (void);
	void			mSetTeam (eTeamID NewTeam);

	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tShipInfo (eTeamID Team, eRaceName Race, int32 CallbackID, int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY);
	~tShipInfo (void);

protected:
	eTeamID			fTeam;
	eTeamID			fOriginalTeam;
	eRaceName		fRace;
	int32			fShipID;
	eClassTypes		fClassType;
	int32			fStartPositionIndex;
	int32			fStartX;
	int32			fStartY;
	int32			fEndX;
	int32			fEndY;
	float			fTrackingDistance;
	int32u			fStatus;
	int32			fShipBPV;
	iNotifyEvents	fShipEvents;
	int32			fCallbackID;
};

//-------------------------------------------------------
// team specific routines
class tTeamInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	eTeamID		mGetTeam (void);
	eRaceName	mGetRace (void);
	int32		mGetShipsInPlayCount (void);
	eTeamID		mGetMultiplayerTeam (void);

	// AI specific commands
	void		mClearCommands (void);
	void		mDisengageShips (eDisengageMethods Method, eAIPriority Priority);
	void		mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority);

	// Start XY/End XY are 1 based
	void		mCreateShip (int32 PlayerShipIndex, tShipInGame& ShipData);
	void		mCreateShip (int32 PlayerShipIndex, const char* ClassName, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateShip (const char* ClassName, int32 Position, 
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateShip (int32 TotalBPV, eClassTypes ShipType, int32 VariantPercent, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateShip (int32 TotalBPV, eClassTypes SmallestHullSize, eClassTypes LargestHullSize, 
							 int32 VariantPercent, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1,
							 const char* CustomName = NULL);
	void		mCreateFleet (int32 TotalShips, int32 TotalBPV, eClassTypes SmallestHullSize, 
							  eClassTypes LargestHullSize, int32 VariantPercent, int32 Position,
							  int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1, 
							  const char* CustomName = NULL);
	void		mDuplicateShip (tShipInfo* Ship, int32 Position,
								int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1);
	void		mCreateCPUFleet (int32 VariantPercent, int32 Position,
								 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1);

	tShipInfo*	mGetFirstShip (void);
	tShipInfo*	mGetNextShip (void);
	tShipInfo*	mGetFirstShipInPlay (void);
	tShipInfo*	mGetNextShipInPlay (void);
	tShipInfo*	mGetThisShip (int32 ShipIndex);		// return ship at given index - index is 1 based
	iTruth		mAllInPosition (void);
	int32		mGetCombatBPV (void);
	int32		mAssessDamages (tTeamInfo* EnemyTeam);
	float		mGetTotalDamagePercent (void);		// must be called before ending mission!!!
	eClassTypes mGetMaxClassType (void);

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:
	tShipInfo*		mFindShipIndex (int32 CallbackID);
	tShipInfo*		mFindShipID (int32 ShipID);
	iTruth			mAllShipsReady (void);
	void			mAddToCombatBPV (int32 ShipBPV);
	void			mRemoveShipAsTarget (tShipInfo* TargetShip);
	iNotifyEvents	mGetTeamStatus (void);
	void			mSetRelations (int32 Team, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor);
	eTeamRelationship mGetRelations (eTeamID Team);
	eTargetGoals	  mGetGoal (eTeamID Team);
	float			  mGetEPV (eTeamID Team);
	int32			mGetShipCount (eShipStatus Status);
	iTruth			mAddShipToTeamList (tCreateShipInfo& info);
	void			mSetAsCPUTeam (eMulti_TeamTypes Type, int32 Index);
	iTruth			mIsCPUTeam (void);
	const char*		mGetCPUCallName (tShipInfo& Ship);
	void			mSetMultiplayerTeam (eTeamID Team);
	void			mSetTeamLeaving (void);
	iTruth			mIsTeamLeaving (void);


	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tTeamInfo (eTeamID Team, eRaceName Race);
	~tTeamInfo (void);

protected:
	void	mCreateTeamShip (tCreateShipInfo& info, int32 Position,
							 int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1);

	eTeamID			fTeam;
	eTeamID			fMultiplayerTeam;
	eRaceName		fRace;
	int32			fNextCallbackID;

	vector<tShipInfo*>	fShipList;
	int32				fShipIndex;
	int32				fCombatBPV;

	tAITargetInfo		fAITargets;

	iTruth				fIsCPUTeam;
	eMulti_TeamTypes	fCPUType;
	int32				fCPUIndex;
	iTruth				fTeamLeaving;
};

//-------------------------------------------------------
// mission specific routines
class tMissionInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	void			mSetTeamRace (int32 Team, eRaceName Race);
	void			mSetTeamRelations (int32 Team1, int32 Team2, eTeamRelationship Relations,
									   eTargetGoals Goals, float EPVFactor, iTruth MutualRelations = kFalse);
	void			mSetTeamRelations (eMulti_TeamTypes Team1, int32 Team2, eTeamRelationship Relations,
									   eTargetGoals Goals, float EPVFactor, iTruth MutualRelations = kFalse);
	void			mSetMultiplayerTeams (tDynaverseScriptInfo& info);
	void			mSetSkirmishTeams (tDynaverseScriptInfo& info);
	void			mSetPrestigePoints (int32 AstoundingVictory, int32 Victory, int32 Draw,
										int32 Defeat, int32 DevastatingDefeat);
	void			mSetTimedInterval (eTimerIntervalTypes Type, int32 Interval);
	void			mSetGoalIndicator (tShipInfo* TargetShip);
	void			mSetGoalIndicator (iTruth IndicatorOn, int32 GridX, int32 GridY);		// Grid X/Y are 1 based

	tTeamInfo*		mGetTeamHandle (int32 Team);
	tTeamInfo*		mGetTeamHandle (eMulti_TeamTypes Type, int32 Index);
	int32			mGetBonusPrestige (void);
	eVictoryLevels	mGetVictoryLevel (int32 PlayerBPV, int32 EnemyBPV);
	void			mGetGridPosition (unsigned char MapObject, int32* GridX, int32* GridY);		// Grid XY are 1 based
	float			mGetTacticalDistance (unsigned char MapObject1, unsigned char MapObject2);
	int32			mGetPlayerRank (void);
	int32			mGetEliteOrganization (void);
	int32			mGetDifficultyLevel (void);
	ePoliticalRelation	mGetRaceRelationship (eRaceName Race1, eRaceName Race2);
	tShipInfo*		mGetShipTarget (tShipInfo* Ship);
	int32			mGetINIFileInt (int32 HeaderID, int32 LabelID);
	void			mGetMapDimensions (int32* GridX, int32* GridY);

	void			mAddToBonusPrestige (int32 Amount);
	int32			mRandomInt32 (int32 lo, int32 hi);
	void			mDisplayMessage (int32 MsgID, eScriptMessageTypes Type = kMissionScriptMessage, tShipInfo* Ship = NULL);
	void			mAddToDebriefingList (int32 DebriefingIndex);
	void			mCreateMapObject (unsigned char MapObject, int32 GridX, int32 GridY);		// Grid XY are 1 based
	void			mPlaySound (int32 SoundID);
	void			mPlayVoiceSample (int32 VoiceID, iTruth StopPreviousVoice = kTrue);
	int32			mVoiceSampleStillPlaying (void);
	void			mAddToCareerTotals (eClassTypes Type, eCareerEvents Event, int32 Amount);
	tShipInfo*		mFindShipID (int32 ShipID);
	void			mScheduleMission (int32 MsgID);			// file name of mission to schedule (including extension)
	void			mStartFlashingButton (const char* ButtonID);
	void			mStopFlashingButton (const char* ButtonID);

	void			mAdjustDynaverse (eDynaverseTableType Type, int32 Modifier, eRaceName Race);

	int32			fTotalAlliedHumanTeams;
	int32			fTotalEnemyHumanTeams;
	int32			fTotalAlliedCPUTeams;
	int32			fTotalEnemyCPUTeams;

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:
	void		mDebugMessage (const char* message);
	void		mSetDate (tDynaverseScriptInfo::tMissionDate& Date);
	void		mSetLocation (eSectorName Location);
	void		mSetSystemName (eSystems SystemName);
	iTruth		mAllTeamsReady (void);
	eGameStatus	mGetGameStatus (void);
	int32		mGetDebriefingTotal (void);
	int32		mGetDebriefingMessage (int32 Index);
	int32		mGetPrestigePoints (eVictoryLevels Level);
	void		mRemoveShipAsTarget (tShipInfo* TargetShip);
	eTeamRelationship mGetTeamRelations (eTeamID Team1, eTeamID Team2);
	void		mResetDebriefingCount (void);
	iTruth		mIsTeamLeaving (eTeamID Team);

	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tMissionInfo (void);
	~tMissionInfo (void);

	eRaceName		fMulti_Races[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_TeamNumbers[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_ShipCounts[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_BPVLimits[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_BPVTeamLimits[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	eClassTypes		fMulti_SmallHulls[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	eClassTypes		fMulti_LargeHulls[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	int32			fMulti_StartPositions[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers];
	tString			fMulti_FleetNames[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers][3];
	tString			fMulti_FleetCallNames[kMaxMultiTeams][tEntityMgr::kMaxNumberofPlayers][3];

	iTruth			fEndGameLockout;
	eGameStatus		fLockOutStatus;

protected:
	int32		fPrestigePoints[kMaxVictoryLevels];
	int32		fBonusPrestige;

	tTeamInfo*	fTeams[kMaxTeams];
	eRaceName	fRaces[kMaxTeams];

	eSectorName		fLocation;
	eSystems		fSystemName;

	tDynaverseScriptInfo::tMissionDate	fDate;

	enum { kMaxInList = 10 };
	int32			fDebriefingArray[kMaxInList];
	int32			fDebriefingTotal;
};

//-------------------------------------------------------
// script specific routines
class tScript
{
	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	int32	mInitializeStart (tDynaverseScriptInfo& info);
	iTruth	mUsePlayerShip (tShipInGame& Ship);
	void	mCreateMissionShips (int32 Mode);
	int32	mInitializeFinished (void);
	void	mStartMission (void);
	void	mNewMissionShip (tShipInfo* Ship);
	void	mGetDebriefingList (eVictoryLevels VictoryLevel);
	void	mScriptSpecs (tMissionSpec* Specs);
	eGameStatus	   mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event);
	eVictoryLevels mCalcVictoryStatus (void);
	int32	mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info);
	int32	mGetEnemyTeamID (void);

	eTeamID			kPlayerTeam;
	tShipInfo*		fPlayerShip;
	iTruth			fIsDeepScanOn;
	iTruth			fThisScriptIsTheHost;

public:
	tMissionInfo			fMission;
	int32					fEventID;
	int32					fEventItem0;
	int32					fEventItem1;
	int32					fEventItem2;
	int32					fEventItem3;
};


class tScriptInterface : public tScript, public tScriptEntry
{
	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	tScriptInterface (void);
	~tScriptInterface (void);

	virtual int32		mDLLVersion (void);
	virtual void		mInitialize (tDynaverseScriptInfo& info);
	virtual iTruth		mSaveEntityID (int32 Team, int32 CallbackID, int32 EntityID);
	virtual eGameStatus	mEntityEvent (tShipSystemsInfo& info);
	virtual void		mInitMapForRead (void);
	virtual const char* mGetNextMapLine (void);
	virtual tString&	mBriefingText (void);
	virtual	int32		mGetDebriefingTotal (void);
	virtual	tString&	mDebriefingText (int32 Index);
	virtual int32		mGetPrestigePoints (eVictoryLevels Level);
	virtual iTruth		mGetScriptSpecs (tMissionSpec* Specs);
	virtual int32		mGetTechnicalSuccess (tDynaverseScriptInfo& info);
	virtual eTeamRelationship mGetTeamRelations (eTeamID Team1, eTeamID Team2);

	virtual void		mEntityMgrCallback (cbEntityMgr* func);
	virtual void		mCreateShipCallback (cbCreateShip* func);
	virtual void		mMessageCallback (cbMessages* func);
	virtual void		mRandomCallback (cbRandom* func);

public:
	cbCreateShip*	mCreateSpecShip;
	cbEntityMgr*	mEntityMgr;
	cbMessages*		mPopupMessage;
	cbRandom*		mRandomInt;
	void			mFindMapObject (unsigned char MapObject, int32* GridX, int32* GridY);
	void			mGetMapDimensions (int32* GridX, int32* GridY);

	iTruth		fMissionBegun;

protected:
	void		mSetupShips (tDynaverseScriptInfo& info);
	eRaceName	mChooseRace (eRaceName RaceArray[], int32 ValidRaces);
	eSystems	mChooseSystem (eSystems SystemsArray[]);
	iTruth		mInitMapForRead (int32 Map);

	int32			fBriefingIndex;
	int32			fMapLineIndex;
	eVictoryLevels	fVictoryLevel;
	int32			fMapPicked;
};

#endif		// #ifndef scriptinterface_h
